@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -126,12 +125,12 @@ int64 Mysql_query_runner::run_query_unguarded(string query) {
 
   MYSQL_RES *results = mysql_use_result(m_connection);
 
-  if (results != nullptr) {
+  if (results != NULL) {
     for (;;) {
       // Feed result callbacks with results.
       MYSQL_ROW row = mysql_fetch_row(results);
 
-      if (row == nullptr) {
+      if (row == NULL) {
         // NULL row indicates end of rows or error
         if (mysql_errno(m_connection) == 0)
           break;
@@ -146,7 +145,7 @@ int64 Mysql_query_runner::run_query_unguarded(string query) {
 
       vector<std::function<int64(const Row &)> *>::reverse_iterator it;
       for (it = m_result_callbacks.rbegin(); it != m_result_callbacks.rend();
-           ++it) {
+           it++) {
         int64 callback_result = (**it)(*processed_row);
         if (callback_result != 0) {
           mysql_free_result(results);
@@ -161,7 +160,7 @@ int64 Mysql_query_runner::run_query_unguarded(string query) {
 
   // Get all notes, warnings and errors of last query.
   if (0 != mysql_query(m_connection, "SHOW WARNINGS") ||
-      nullptr == (results = mysql_use_result(m_connection))) {
+      NULL == (results = mysql_use_result(m_connection))) {
     this->report_mysql_error();
     return 0;
   }
@@ -171,7 +170,7 @@ int64 Mysql_query_runner::run_query_unguarded(string query) {
     // Feed message callbacks with results.
     MYSQL_ROW row = mysql_fetch_row(results);
 
-    if (row == nullptr) {
+    if (row == NULL) {
       // End of rows or an error.
       if (mysql_errno(m_connection) != 0) this->report_mysql_error();
       break;
@@ -290,7 +289,7 @@ void Mysql_query_runner::cleanup_result(
     std::vector<const Mysql_query_runner::Row *> *result) {
   for (vector<const Mysql_query_runner::Row *>::const_iterator it =
            result->begin();
-       it != result->end(); ++it) {
+       it != result->end(); it++) {
     delete *it;
   }
   result->clear();
@@ -298,7 +297,7 @@ void Mysql_query_runner::cleanup_result(
 
 Mysql_query_runner::Row::Row(MYSQL_RES *mysql_result_info,
                              unsigned int column_count, MYSQL_ROW row)
-    : m_buffer(nullptr),
+    : m_buffer(NULL),
       m_buffer_capacity(0),
       m_buffer_size(0),
       m_mysql_result_info(mysql_result_info) {
@@ -316,7 +315,7 @@ Mysql_query_runner::Row::Row(MYSQL_RES *mysql_result_info,
 }
 
 Mysql_query_runner::Row::~Row() {
-  if (m_buffer != nullptr) free(m_buffer);
+  if (m_buffer != NULL) free(m_buffer);
 }
 
 std::string Mysql_query_runner::Row::operator[](std::size_t index) const {
@@ -326,7 +325,7 @@ std::string Mysql_query_runner::Row::operator[](std::size_t index) const {
 }
 
 void Mysql_query_runner::Row::push_back(char *buff, std::size_t length) {
-  if (buff != nullptr) {
+  if (buff != NULL) {
     // Prepare buffer to be able to contain new data.
     this->reserve(m_buffer_starts.size(), m_buffer_size + length + 1);
     // Copy data.
