@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,7 +51,7 @@ class Number_option
 };
 
 template <typename T_value>
-Number_option<T_value>::Number_option() = default;
+Number_option<T_value>::Number_option() {}
 
 /**
   32-bit signed number option.
@@ -153,7 +152,7 @@ class Number_option<double>
     Sets minimum value boundary for option value. Smaller values passed as
     option value will be changed to this minimum value.
    */
-  Number_option<double> *set_minimum_value(double minimum) override {
+  virtual Number_option<double> *set_minimum_value(double minimum) {
     this->m_option_structure.min_value = getopt_double2ulonglong(minimum);
     return this;
   }
@@ -162,7 +161,7 @@ class Number_option<double>
     Sets maximum value boundary for option value. Greater values passed as
     option value will be changed to this maximum value.
    */
-  Number_option<double> *set_maximum_value(double maximum) override {
+  virtual Number_option<double> *set_maximum_value(double maximum) {
     this->m_option_structure.max_value = getopt_double2ulonglong(maximum);
     return this;
   }
