@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2001, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -53,23 +52,22 @@ using std::vector;
 #define EX_USAGE 1
 #define EX_MYSQLERR 2
 
-static MYSQL mysql_connection, *sock = nullptr;
-static bool opt_alldbs = false, opt_check_only_changed = false,
-            opt_extended = false, opt_compress = false, opt_databases = false,
-            opt_fast = false, opt_medium_check = false, opt_quick = false,
-            opt_all_in_1 = false, opt_silent = false, opt_auto_repair = false,
-            ignore_errors = false, opt_frm = false, debug_info_flag = false,
-            debug_check_flag = false, opt_fix_table_names = false,
-            opt_fix_db_names = false, opt_upgrade = false,
-            opt_write_binlog = true;
+static MYSQL mysql_connection, *sock = 0;
+static bool opt_alldbs = 0, opt_check_only_changed = 0, opt_extended = 0,
+            opt_compress = 0, opt_databases = 0, opt_fast = 0,
+            opt_medium_check = 0, opt_quick = 0, opt_all_in_1 = 0,
+            opt_silent = 0, opt_auto_repair = 0, ignore_errors = 0,
+            tty_password = 0, opt_frm = 0, debug_info_flag = 0,
+            debug_check_flag = 0, opt_fix_table_names = 0, opt_fix_db_names = 0,
+            opt_upgrade = 0, opt_write_binlog = 1;
 static uint verbose = 0, opt_mysql_port = 0;
 static uint opt_enable_cleartext_plugin = 0;
-static bool using_opt_enable_cleartext_plugin = false;
+static bool using_opt_enable_cleartext_plugin = 0;
 static int my_end_arg;
-static char *opt_mysql_unix_port = nullptr;
-static char *current_user = nullptr, *current_host = nullptr;
+static char *opt_mysql_unix_port = 0;
+static char *opt_password = 0, *current_user = 0, *current_host = 0;
 static const char *default_charset = nullptr;
-static char *opt_plugin_dir = nullptr, *opt_default_auth = nullptr;
+static char *opt_plugin_dir = 0, *opt_default_auth = 0;
 static int first_error = 0;
 static const char *opt_skip_database = "";
 static uint opt_zstd_compress_level = default_zstd_compression_level;
@@ -78,121 +76,108 @@ static char *opt_compress_algorithm = nullptr;
 static char *shared_memory_base_name = 0;
 #endif
 static uint opt_protocol = 0;
-static char *opt_bind_addr = nullptr;
-
-#include "multi_factor_passwordopt-vars.h"
+static char *opt_bind_addr = NULL;
 
 static struct my_option my_long_options[] = {
     {"all-databases", 'A',
      "Check all the databases. This is the same as --databases with all "
      "databases selected.",
-     &opt_alldbs, &opt_alldbs, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
-    {"analyze", 'a', "Analyze given tables.", nullptr, nullptr, nullptr,
-     GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &opt_alldbs, &opt_alldbs, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"analyze", 'a', "Analyze given tables.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0,
+     0, 0, 0, 0},
     {"all-in-1", '1',
      "Instead of issuing one query for each table, use one query per database, "
      "naming all tables in the database in a comma-separated list.",
-     &opt_all_in_1, &opt_all_in_1, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr,
-     0, nullptr},
+     &opt_all_in_1, &opt_all_in_1, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"auto-repair", OPT_AUTO_REPAIR,
      "If a checked table is corrupted, automatically fix it. Repairing will be "
      "done after all tables have been checked, if corrupted ones were found.",
-     &opt_auto_repair, &opt_auto_repair, nullptr, GET_BOOL, NO_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &opt_auto_repair, &opt_auto_repair, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"bind-address", 0, "IP address to bind to.", (uchar **)&opt_bind_addr,
-     (uchar **)&opt_bind_addr, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr,
-     0, nullptr},
+     (uchar **)&opt_bind_addr, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"character-sets-dir", OPT_CHARSETS_DIR,
-     "Directory for character set files.", &charsets_dir, &charsets_dir,
-     nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
-    {"check", 'c', "Check table for errors.", nullptr, nullptr, nullptr,
-     GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     "Directory for character set files.", &charsets_dir, &charsets_dir, 0,
+     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+    {"check", 'c', "Check table for errors.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0,
+     0, 0, 0, 0},
     {"check-only-changed", 'C',
      "Check only tables that have changed since last check or haven't been "
      "closed properly.",
-     nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"check-upgrade", 'g',
      "Check tables for version-dependent changes. May be used with "
      "--auto-repair to correct tables requiring version-dependent updates.",
-     nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"compress", OPT_COMPRESS, "Use compression in server/client protocol.",
-     &opt_compress, &opt_compress, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr,
-     0, nullptr},
+     &opt_compress, &opt_compress, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"databases", 'B',
      "Check several databases. Note the difference in usage; in this case no "
      "tables are given. All name arguments are regarded as database names.",
-     &opt_databases, &opt_databases, nullptr, GET_BOOL, NO_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
-#ifdef NDEBUG
-    {"debug", '#', "This is a non-debug version. Catch this and exit.", nullptr,
-     nullptr, nullptr, GET_DISABLED, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &opt_databases, &opt_databases, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+#ifdef DBUG_OFF
+    {"debug", '#', "This is a non-debug version. Catch this and exit.", 0, 0, 0,
+     GET_DISABLED, OPT_ARG, 0, 0, 0, 0, 0, 0},
     {"debug-check", OPT_DEBUG_CHECK,
-     "This is a non-debug version. Catch this and exit.", nullptr, nullptr,
-     nullptr, GET_DISABLED, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     "This is a non-debug version. Catch this and exit.", 0, 0, 0, GET_DISABLED,
+     NO_ARG, 0, 0, 0, 0, 0, 0},
     {"debug-info", OPT_DEBUG_INFO,
-     "This is a non-debug version. Catch this and exit.", nullptr, nullptr,
-     nullptr, GET_DISABLED, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     "This is a non-debug version. Catch this and exit.", 0, 0, 0, GET_DISABLED,
+     NO_ARG, 0, 0, 0, 0, 0, 0},
 #else
-    {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.", nullptr,
-     nullptr, nullptr, GET_STR, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
+    {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.", 0, 0, 0,
+     GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
     {"debug-check", OPT_DEBUG_CHECK,
      "Check memory and open file usage at exit.", &debug_check_flag,
-     &debug_check_flag, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     &debug_check_flag, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"debug-info", OPT_DEBUG_INFO, "Print some debug info at exit.",
-     &debug_info_flag, &debug_info_flag, nullptr, GET_BOOL, NO_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &debug_info_flag, &debug_info_flag, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 #endif
     {"default-character-set", OPT_DEFAULT_CHARSET,
-     "Set the default character set.", &default_charset, &default_charset,
-     nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     "Set the default character set.", &default_charset, &default_charset, 0,
+     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"default_auth", OPT_DEFAULT_AUTH,
      "Default authentication client-side plugin to use.", &opt_default_auth,
-     &opt_default_auth, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     &opt_default_auth, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"enable_cleartext_plugin", OPT_ENABLE_CLEARTEXT_PLUGIN,
      "Enable/disable the clear text authentication plugin.",
-     &opt_enable_cleartext_plugin, &opt_enable_cleartext_plugin, nullptr,
-     GET_BOOL, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &opt_enable_cleartext_plugin, &opt_enable_cleartext_plugin, 0, GET_BOOL,
+     OPT_ARG, 0, 0, 0, 0, 0, 0},
     {"fast", 'F', "Check only tables that haven't been closed properly.",
-     &opt_fast, &opt_fast, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     &opt_fast, &opt_fast, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"force", 'f', "Continue even if we get an SQL error.", &ignore_errors,
-     &ignore_errors, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &ignore_errors, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"extended", 'e',
      "If you are using this option with CHECK TABLE, it will ensure that the "
      "table is 100 percent consistent, but will take a long time. If you are "
      "using this option with REPAIR TABLE, it will force using old slow repair "
      "with keycache method, instead of much faster repair by sorting.",
-     &opt_extended, &opt_extended, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr,
-     0, nullptr},
-    {"help", '?', "Display this help message and exit.", nullptr, nullptr,
-     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
-    {"host", 'h', "Connect to host.", &current_host, &current_host, nullptr,
-     GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &opt_extended, &opt_extended, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"help", '?', "Display this help message and exit.", 0, 0, 0, GET_NO_ARG,
+     NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"host", 'h', "Connect to host.", &current_host, &current_host, 0,
+     GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"medium-check", 'm',
      "Faster than extended-check, but only finds 99.99 percent of all errors. "
      "Should be good enough for most cases.",
-     nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"write-binlog", OPT_WRITE_BINLOG,
-     "Write ANALYZE, OPTIMIZE and REPAIR TABLE commands to the binary log. "
-     "Use --skip-write-binlog to omit this.",
-     &opt_write_binlog, &opt_write_binlog, nullptr, GET_BOOL, NO_ARG, 1, 0, 0,
-     nullptr, 0, nullptr},
-    {"optimize", 'o', "Optimize table.", nullptr, nullptr, nullptr, GET_NO_ARG,
-     NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
-#include "multi_factor_passwordopt-longopts.h"
+     "Log ANALYZE, OPTIMIZE and REPAIR TABLE commands. Use --skip-write-binlog "
+     "when commands should not be sent to replication slaves.",
+     &opt_write_binlog, &opt_write_binlog, 0, GET_BOOL, NO_ARG, 1, 0, 0, 0, 0,
+     0},
+    {"optimize", 'o', "Optimize table.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0,
+     0, 0, 0},
+    {"password", 'p',
+     "Password to use when connecting to server. If password is not given, "
+     "it's solicited on the tty.",
+     0, 0, 0, GET_PASSWORD, OPT_ARG, 0, 0, 0, 0, 0, 0},
 #ifdef _WIN32
     {"pipe", 'W', "Use named pipes to connect to server.", 0, 0, 0, GET_NO_ARG,
      NO_ARG, 0, 0, 0, 0, 0, 0},
 #endif
     {"plugin_dir", OPT_PLUGIN_DIR, "Directory for client-side plugins.",
-     &opt_plugin_dir, &opt_plugin_dir, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &opt_plugin_dir, &opt_plugin_dir, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0,
+     0},
     {"port", 'P',
      "Port number to use for connection or 0 for default to, in "
      "order of preference, my.cnf, $MYSQL_TCP_PORT, "
@@ -200,74 +185,70 @@ static struct my_option my_long_options[] = {
      "/etc/services, "
 #endif
      "built-in default (" STRINGIFY_ARG(MYSQL_PORT) ").",
-     &opt_mysql_port, &opt_mysql_port, nullptr, GET_UINT, REQUIRED_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &opt_mysql_port, &opt_mysql_port, 0, GET_UINT, REQUIRED_ARG, 0, 0, 0, 0, 0,
+     0},
     {"protocol", OPT_MYSQL_PROTOCOL,
-     "The protocol to use for connection (tcp, socket, pipe, memory).", nullptr,
-     nullptr, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     "The protocol to use for connection (tcp, socket, pipe, memory).", 0, 0, 0,
+     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"quick", 'q',
      "If you are using this option with CHECK TABLE, it prevents the check "
      "from scanning the rows to check for wrong links. This is the fastest "
      "check. If you are using this option with REPAIR TABLE, it will try to "
      "repair only the index tree. This is the fastest repair method for a "
      "table.",
-     &opt_quick, &opt_quick, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     &opt_quick, &opt_quick, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"repair", 'r',
-     "Can fix almost anything except unique keys that aren't unique.", nullptr,
-     nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     "Can fix almost anything except unique keys that aren't unique.", 0, 0, 0,
+     GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
 #if defined(_WIN32)
     {"shared-memory-base-name", OPT_SHARED_MEMORY_BASE_NAME,
      "Base name of shared memory.", &shared_memory_base_name,
      &shared_memory_base_name, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0,
      0},
 #endif
-    {"silent", 's', "Print only error messages.", &opt_silent, &opt_silent,
-     nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+    {"silent", 's', "Print only error messages.", &opt_silent, &opt_silent, 0,
+     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"skip_database", 0, "Don't process the database specified as argument",
-     &opt_skip_database, &opt_skip_database, nullptr, GET_STR, REQUIRED_ARG, 0,
-     0, 0, nullptr, 0, nullptr},
+     &opt_skip_database, &opt_skip_database, 0, GET_STR, REQUIRED_ARG, 0, 0, 0,
+     0, 0, 0},
     {"socket", 'S', "The socket file to use for connection.",
-     &opt_mysql_unix_port, &opt_mysql_unix_port, nullptr, GET_STR, REQUIRED_ARG,
-     0, 0, 0, nullptr, 0, nullptr},
+     &opt_mysql_unix_port, &opt_mysql_unix_port, 0, GET_STR, REQUIRED_ARG, 0, 0,
+     0, 0, 0, 0},
 #include "caching_sha2_passwordopt-longopts.h"
 #include "sslopt-longopts.h"
 
-    {"tables", OPT_TABLES, "Overrides option --databases (-B).", nullptr,
-     nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+    {"tables", OPT_TABLES, "Overrides option --databases (-B).", 0, 0, 0,
+     GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"use-frm", OPT_FRM,
      "When used with REPAIR, get table structure from .frm file, so the table "
      "can be repaired even if .MYI header is corrupted.",
-     &opt_frm, &opt_frm, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     &opt_frm, &opt_frm, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"user", 'u', "User for login if not current user.", &current_user,
-     &current_user, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
-    {"verbose", 'v', "Print info about the various stages.", nullptr, nullptr,
-     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
-    {"version", 'V', "Output version information and exit.", nullptr, nullptr,
-     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &current_user, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+    {"verbose", 'v', "Print info about the various stages.", 0, 0, 0,
+     GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"version", 'V', "Output version information and exit.", 0, 0, 0,
+     GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"compression-algorithms", 0,
      "Use compression algorithm in server/client protocol. Valid values "
      "are any combination of 'zstd','zlib','uncompressed'.",
-     &opt_compress_algorithm, &opt_compress_algorithm, nullptr, GET_STR,
-     REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &opt_compress_algorithm, &opt_compress_algorithm, 0, GET_STR, REQUIRED_ARG,
+     0, 0, 0, 0, 0, 0},
     {"zstd-compression-level", 0,
      "Use this compression level in the client/server protocol, in case "
      "--compression-algorithms=zstd. Valid range is between 1 and 22, "
      "inclusive. Default is 3.",
-     &opt_zstd_compress_level, &opt_zstd_compress_level, nullptr, GET_UINT,
-     REQUIRED_ARG, 3, 1, 22, nullptr, 0, nullptr},
-    {nullptr, 0, nullptr, nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0,
-     0, nullptr, 0, nullptr}};
+     &opt_zstd_compress_level, &opt_zstd_compress_level, 0, GET_UINT,
+     REQUIRED_ARG, 3, 1, 22, 0, 0, 0},
+    {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
 
-static const char *load_default_groups[] = {"mysqlcheck", "client", nullptr};
+static const char *load_default_groups[] = {"mysqlcheck", "client", 0};
 
 static void usage(void);
 static int get_options(int *argc, char ***argv, MEM_ROOT *alloc);
-static int dbConnect(char *host, char *user);
+static int dbConnect(char *host, char *user, char *passwd);
 static void dbDisconnect(char *host);
-static void DBerror(MYSQL *mysql, const string &when);
+static void DBerror(MYSQL *mysql, string when);
 static void safe_exit(int error);
 
 static int what_to_do = 0;
@@ -315,27 +296,43 @@ static bool get_one_option(int optid, const struct my_option *opt,
       break;
     case 'C':
       what_to_do = DO_CHECK;
-      opt_check_only_changed = true;
+      opt_check_only_changed = 1;
       break;
-    case 'I':
-      [[fallthrough]];
+    case 'I': /* Fall through */
     case '?':
       usage();
       exit(0);
     case 'm':
       what_to_do = DO_CHECK;
-      opt_medium_check = true;
+      opt_medium_check = 1;
       break;
     case 'o':
       what_to_do = DO_OPTIMIZE;
       break;
-      PARSE_COMMAND_LINE_PASSWORD_OPTION;
+    case 'p':
+      if (argument == disabled_my_option) {
+        // Don't require password
+        static char empty_password[] = {'\0'};
+        DBUG_ASSERT(empty_password[0] ==
+                    '\0');  // Check that it has not been overwritten
+        argument = empty_password;
+      }
+      if (argument) {
+        char *start = argument;
+        my_free(opt_password);
+        opt_password = my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
+        while (*argument) *argument++ = 'x'; /* Destroy argument */
+        if (*start) start[1] = 0;            /* Cut length of argument */
+        tty_password = 0;
+      } else
+        tty_password = 1;
+      break;
     case 'r':
       what_to_do = DO_REPAIR;
       break;
     case 'g':
       what_to_do = DO_CHECK;
-      opt_upgrade = true;
+      opt_upgrade = 1;
       break;
     case 'W':
 #ifdef _WIN32
@@ -344,12 +341,12 @@ static bool get_one_option(int optid, const struct my_option *opt,
       break;
     case '#':
       DBUG_PUSH(argument ? argument : "d:t:o");
-      debug_check_flag = true;
+      debug_check_flag = 1;
       break;
 #include "sslopt-case.h"
 
     case OPT_TABLES:
-      opt_databases = false;
+      opt_databases = 0;
       break;
     case 'v':
       verbose++;
@@ -364,18 +361,15 @@ static bool get_one_option(int optid, const struct my_option *opt,
       opt_protocol =
           find_type_or_exit(argument, &sql_protocol_typelib, opt->name);
       break;
-    case OPT_COMPRESS:
-      CLIENT_WARN_DEPRECATED("--compress", "--compression-algorithms");
-      break;
   }
 
   if (orig_what_to_do && (what_to_do != orig_what_to_do)) {
     fprintf(stderr,
             "Error:  %s doesn't support multiple contradicting commands.\n",
             my_progname);
-    return true;
+    return 1;
   }
-  return false;
+  return 0;
 }
 }
 
@@ -438,12 +432,13 @@ static int get_options(int *argc, char ***argv, MEM_ROOT *alloc) {
     printf("for more information.\n");
     return 1;
   }
+  if (tty_password) opt_password = get_tty_password(NullS);
   if (debug_info_flag) my_end_arg = MY_CHECK_ERROR | MY_GIVE_INFO;
   if (debug_check_flag) my_end_arg = MY_CHECK_ERROR;
   return (0);
 } /* get_options */
 
-static int dbConnect(char *host, char *user) {
+static int dbConnect(char *host, char *user, char *passwd) {
   DBUG_TRACE;
   if (verbose) {
     fprintf(stderr, "# Connecting to %s...\n", host ? host : "localhost");
@@ -482,19 +477,17 @@ static int dbConnect(char *host, char *user) {
                   (char *)&opt_enable_cleartext_plugin);
 
   mysql_options(&mysql_connection, MYSQL_SET_CHARSET_NAME, default_charset);
-  mysql_options(&mysql_connection, MYSQL_OPT_CONNECT_ATTR_RESET, nullptr);
+  mysql_options(&mysql_connection, MYSQL_OPT_CONNECT_ATTR_RESET, 0);
   mysql_options4(&mysql_connection, MYSQL_OPT_CONNECT_ATTR_ADD, "program_name",
                  "mysqlcheck");
   set_server_public_key(&mysql_connection);
   set_get_server_public_key_option(&mysql_connection);
-  set_password_options(&mysql_connection);
-  if (!(sock =
-            mysql_real_connect(&mysql_connection, host, user, nullptr, nullptr,
-                               opt_mysql_port, opt_mysql_unix_port, 0))) {
+  if (!(sock = mysql_real_connect(&mysql_connection, host, user, passwd, NULL,
+                                  opt_mysql_port, opt_mysql_unix_port, 0))) {
     DBerror(&mysql_connection, "when trying to connect");
     return 1;
   }
-  mysql_connection.reconnect = true;
+  mysql_connection.reconnect = 1;
   return 0;
 } /* dbConnect */
 
@@ -504,7 +497,7 @@ static void dbDisconnect(char *host) {
   mysql_close(sock);
 } /* dbDisconnect */
 
-static void DBerror(MYSQL *mysql, const string &when) {
+static void DBerror(MYSQL *mysql, string when) {
   DBUG_TRACE;
   my_printf_error(0, "Got error: %d: %s %s", MYF(0), mysql_errno(mysql),
                   mysql_error(mysql), when.c_str());
@@ -528,7 +521,7 @@ int main(int argc, char **argv) {
     my_end(my_end_arg);
     exit(EX_USAGE);
   }
-  if (dbConnect(current_host, current_user)) exit(EX_MYSQLERR);
+  if (dbConnect(current_host, current_user, opt_password)) exit(EX_MYSQLERR);
 
   // Sun Studio does not work with range constructor from char** to string.
   vector<string> conv;
@@ -543,11 +536,11 @@ int main(int argc, char **argv) {
               DBerror);
 
   dbDisconnect(current_host);
-  free_passwords();
+  my_free(opt_password);
 #if defined(_WIN32)
   my_free(shared_memory_base_name);
 #endif
-  alloc.Clear();
+  free_root(&alloc, MYF(0));
   my_end(my_end_arg);
   return (first_error != 0);
 } /* main */
