@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +26,6 @@
 
 #include <stdlib.h>
 #include <functional>
-#include <optional>
 #include <string>
 #include <vector>
 
@@ -55,12 +53,10 @@ Mysql_crawler::Mysql_crawler(
         *message_handler,
     Simple_id_generator *object_id_generator,
     Mysql_chain_element_options *options,
-    Mysqldump_tool_chain_maker_options *mysqldump_tool_cmaker_options,
     Mysql::Tools::Base::Abstract_program *program)
     : Abstract_crawler(message_handler, object_id_generator, program),
       Abstract_mysql_chain_element_extension(connection_provider,
-                                             message_handler, options),
-      m_mysqldump_tool_cmaker_options(mysqldump_tool_cmaker_options) {}
+                                             message_handler, options) {}
 
 void Mysql_crawler::enumerate_objects() {
   Mysql::Tools::Base::Mysql_query_runner *runner = this->get_runner();
@@ -98,7 +94,6 @@ void Mysql_crawler::enumerate_objects() {
   m_dump_end_task = new Dump_end_dump_task();
   m_tables_definition_ready_dump_task = new Tables_definition_ready_dump_task();
 
-  m_dump_end_task->add_dependency(m_dump_start_task);
   this->process_dump_task(m_dump_start_task);
 
   std::vector<const Mysql::Tools::Base::Mysql_query_runner::Row *> databases;
@@ -112,37 +107,23 @@ void Mysql_crawler::enumerate_objects() {
        it != databases.end(); ++it) {
     std::string db_name = (**it)[0];
 
-    std::optional<std::string> stmt = this->get_create_statement(
-        runner, "", db_name, "DATABASE IF NOT EXISTS");
-    if (!stmt.has_value()) continue;  // some error occurred
-
     Database *database =
-        new Database(this->generate_new_object_id(), db_name, stmt.value());
-    m_current_database_start_dump_task = new Database_start_dump_task(database);
-    Abstract_data_object *db_object = dynamic_cast<Abstract_data_object *>(
-        m_current_database_start_dump_task->get_related_db_object());
+        new Database(this->generate_new_object_id(), db_name,
+                     this->get_create_statement(runner, "", db_name,
+                                                "DATABASE IF NOT EXISTS")
+                         .value());
 
-    /*
-     This check is introduced so that only the database objects that are
-     included in the dump are validated.
-    */
-    if (!m_mysqldump_tool_cmaker_options->is_object_included_in_dump(
-            db_object)) {
-      delete m_current_database_start_dump_task;
-      delete database;
-      continue;
-    }
     db_list.push_back(database);
+    m_current_database_start_dump_task = new Database_start_dump_task(database);
     m_current_database_end_dump_task = new Database_end_dump_task(database);
     db_end_task_list.push_back(m_current_database_end_dump_task);
 
     m_current_database_start_dump_task->add_dependency(m_dump_start_task);
-    m_dump_end_task->add_dependency(m_current_database_start_dump_task);
     m_dump_end_task->add_dependency(m_current_database_end_dump_task);
 
     this->process_dump_task(m_current_database_start_dump_task);
     this->enumerate_database_objects(*database);
-    m_current_database_start_dump_task = nullptr;
+    m_current_database_start_dump_task = NULL;
   }
 
   m_dump_end_task->add_dependency(m_tables_definition_ready_dump_task);
@@ -159,7 +140,7 @@ void Mysql_crawler::enumerate_objects() {
     m_current_database_end_dump_task = *it_end;
     this->enumerate_views(**it);
     this->process_dump_task(m_current_database_end_dump_task);
-    m_current_database_end_dump_task = nullptr;
+    m_current_database_end_dump_task = NULL;
   }
 
   Mysql::Tools::Base::Mysql_query_runner::cleanup_result(&databases);
@@ -203,12 +184,6 @@ void Mysql_crawler::enumerate_tables(const Database &db) {
   Mysql::Tools::Base::Mysql_query_runner *runner = this->get_runner();
 
   if (!runner) return;
-
-  if (m_mysqldump_tool_cmaker_options->m_object_reader_options->m_skip_gipk)
-    runner->run_query(
-        "/*!80030 SET SESSION "
-        "show_gipk_in_create_table_and_information_schema = OFF */");
-
   /*
     Get statistics from SE by setting information_schema_stats_expiry=0.
     This makes the queries IS queries retrieve latest
@@ -280,13 +255,12 @@ void Mysql_crawler::enumerate_tables(const Database &db) {
                       ? 0
                       : atoll(table_data[4].c_str());  // "Rows"
     bool isInnoDB = table_data[1] == "InnoDB";         // "Engine"
-    std::optional<std::string> stmt =
-        this->get_create_statement(runner, db.get_name(), table_name, "TABLE");
-    if (!stmt.has_value()) continue;  // some error occurred
-    Table *table = new Table(this->generate_new_object_id(), table_name,
-                             db.get_name(), stmt.value(), fields, table_data[1],
-                             rows, (uint64)(rows * (isInnoDB ? 1.5 : 1)),
-                             atoll(table_data[6].c_str())  // "Data_length"
+    Table *table = new Table(
+        this->generate_new_object_id(), table_name, db.get_name(),
+        this->get_create_statement(runner, db.get_name(), table_name, "TABLE")
+            .value(),
+        fields, table_data[1], rows, (uint64)(rows * (isInnoDB ? 1.5 : 1)),
+        atoll(table_data[6].c_str())  // "Data_length"
     );
 
     Table_definition_dump_task *ddl_task =
@@ -313,11 +287,6 @@ void Mysql_crawler::enumerate_tables(const Database &db) {
   Mysql::Tools::Base::Mysql_query_runner::cleanup_result(&tables);
   runner->run_query_store(
       "/*!80000 SET SESSION information_schema_stats_expiry=default */", &t);
-
-  if (m_mysqldump_tool_cmaker_options->m_object_reader_options->m_skip_gipk)
-    runner->run_query(
-        "/*!80030 SET SESSION "
-        "show_gipk_in_create_table_and_information_schema = default */");
   delete runner;
 }
 
@@ -353,25 +322,15 @@ void Mysql_crawler::enumerate_views(const Database &db) {
         if (runner->run_query(
                 std::string("LOCK TABLES ") +
                 this->get_quoted_object_full_name(db.get_name(), table_name) +
-                " READ") != 0) {
-          Mysql::Tools::Base::Mysql_query_runner::cleanup_result(&check_view);
-          Mysql::Tools::Base::Mysql_query_runner::cleanup_result(&tables);
-          delete runner;
+                " READ") != 0)
           return;
-        } else
+        else
           runner->run_query(std::string("UNLOCK TABLES"));
-        std::optional<std::string> stmt = this->get_create_statement(
-            runner, db.get_name(), table_name, "TABLE");
-
-        if (!stmt.has_value()) {
-          Mysql::Tools::Base::Mysql_query_runner::cleanup_result(&check_view);
-          Mysql::Tools::Base::Mysql_query_runner::cleanup_result(&tables);
-          delete runner;
-          return;
-        }
-
-        View *view = new View(this->generate_new_object_id(), table_name,
-                              db.get_name(), stmt.value());
+        View *view =
+            new View(this->generate_new_object_id(), table_name, db.get_name(),
+                     this->get_create_statement(runner, db.get_name(),
+                                                table_name, "TABLE")
+                         .value());
         m_current_database_end_dump_task->add_dependency(view);
         view->add_dependency(m_tables_definition_ready_dump_task);
         this->process_dump_task(view);
@@ -400,14 +359,13 @@ void Mysql_crawler::enumerate_functions(const Database &db, std::string type) {
        it != functions.end(); ++it) {
     const Mysql::Tools::Base::Mysql_query_runner::Row &function_row = **it;
 
-    std::optional<std::string> stmt = this->get_create_statement(
-        runner, db.get_name(), function_row[1], type, 2);
-    if (!stmt.has_value())  // some error occurred
-      break;
-
     TObject *function = new TObject(
         this->generate_new_object_id(), function_row[1], db.get_name(),
-        "DELIMITER //\n" + stmt.value() + "//\n" + "DELIMITER ;\n");
+        "DELIMITER //\n" +
+            this->get_create_statement(runner, db.get_name(), function_row[1],
+                                       type, 2)
+                .value() +
+            "//\n" + "DELIMITER ;\n");
 
     function->add_dependency(m_current_database_start_dump_task);
     m_current_database_end_dump_task->add_dependency(function);
@@ -440,15 +398,13 @@ void Mysql_crawler::enumerate_event_scheduler_events(const Database &db) {
        it != events.end(); ++it) {
     const Mysql::Tools::Base::Mysql_query_runner::Row &event_row = **it;
 
-    std::optional<std::string> stmt = this->get_create_statement(
-        runner, db.get_name(), event_row[1], "EVENT", 3);
-
-    if (!stmt.has_value())  // some error occurred
-      break;
-
     Event_scheduler_event *event = new Event_scheduler_event(
         this->generate_new_object_id(), event_row[1], db.get_name(),
-        "DELIMITER //\n" + stmt.value() + "//\n" + "DELIMITER ;\n");
+        "DELIMITER //\n" +
+            this->get_create_statement(runner, db.get_name(), event_row[1],
+                                       "EVENT", 3)
+                .value() +
+            "//\n" + "DELIMITER ;\n");
 
     event->add_dependency(m_current_database_start_dump_task);
     m_current_database_end_dump_task->add_dependency(event);
@@ -501,7 +457,6 @@ void Mysql_crawler::enumerate_users() {
         new Privilege(this->generate_new_object_id(), user_row[0], user);
 
     grant->add_dependency(previous_grant);
-    grant->add_dependency(m_tables_definition_ready_dump_task);
     m_dump_end_task->add_dependency(grant);
     this->process_dump_task(grant);
     previous_grant = grant;
@@ -533,15 +488,14 @@ void Mysql_crawler::enumerate_table_triggers(const Table &table,
            triggers.begin();
        it != triggers.end(); ++it) {
     const Mysql::Tools::Base::Mysql_query_runner::Row &trigger_row = **it;
-    std::optional<std::string> stmt = this->get_create_statement(
-        runner, table.get_schema(), trigger_row[0], "TRIGGER", 2);
-    if (!stmt.has_value())  // some error occurred
-      break;
     Trigger *trigger = new Trigger(
         this->generate_new_object_id(), trigger_row[0], table.get_schema(),
         "DELIMITER //\n" +
-            this->get_version_specific_statement(stmt.value(), "TRIGGER",
-                                                 "50017", "50003") +
+            this->get_version_specific_statement(
+                this->get_create_statement(runner, table.get_schema(),
+                                           trigger_row[0], "TRIGGER", 2)
+                    .value(),
+                "TRIGGER", "50017", "50003") +
             "\n//\n" + "DELIMITER ;\n",
         &table);
 
