@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,14 +43,13 @@ class Row_group_dump_task : public Abstract_simple_dump_task {
  public:
   Row_group_dump_task(Table *source_table,
                       const std::vector<Mysql_field> &fields,
-                      const bool has_generated_column,
-                      const bool has_invisible_columns);
+                      const bool has_generated_column);
 
-  I_data_object *get_related_db_object() const override;
+  virtual I_data_object *get_related_db_object() const;
 
-  bool can_be_executed() const override;
+  bool can_be_executed() const;
 
-  void set_completed() override;
+  void set_completed();
 
   /**
     Returns a table the rows are contained in.
@@ -69,10 +67,6 @@ class Row_group_dump_task : public Abstract_simple_dump_task {
     Contains generated/virtual fields.
   */
   const bool m_has_generated_columns;
-  /**
-    Contains invisible columns.
-  */
-  const bool m_has_invisible_columns;
 };
 
 }  // namespace Dump
