@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,8 +40,7 @@ class Standard_progress_watcher : public Abstract_progress_watcher {
           *message_handler,
       Simple_id_generator *object_id_generator);
 
-  void process_progress_step(
-      Abstract_progress_watcher::Progress_data &change) override;
+  void process_progress_step(Abstract_progress_watcher::Progress_data &change);
 };
 
 }  // namespace Dump
