@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,7 +29,6 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <algorithm>
-#include <array>
 #include <cstdio>
 #include <cstdlib>
 #include <fstream>
@@ -37,7 +36,6 @@
 #include <sstream>
 #include <string>
 
-#include "client/client_priv.h"
 #include "client/logger.h"
 #include "client/path.h"
 #ifdef _WIN32
@@ -57,12 +55,10 @@
 #include "print_version.h"
 #include "welcome_copyright_notice.h" /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
-#ifdef HAVE_CHOWN
+#if HAVE_CHOWN
 #include <pwd.h>
 #endif
 
-#include <openssl/ssl.h>
-
 /* Forward declarations */
 
 using namespace std;
@@ -88,8 +84,6 @@ enum certs {
 
 enum extfiles { CAV3_EXT = 0, CERTV3_EXT };
 
-constexpr const std::array rsa_key_sizes{2048, 2048, 2048, 3072, 7680, 15360};
-
 Sql_string_t cert_files[] = {
     create_string("ca.pem"),          create_string("ca-key.pem"),
     create_string("ca-req.pem"),      create_string("server-cert.pem"),
@@ -104,7 +98,7 @@ Sql_string_t ext_files[] = {create_string("cav3.ext"),
 #define MAX_PATH_LEN \
   (FN_REFLEN - strlen(FN_DIRSEP) - cert_files[SERVER_CERT].length() - 1)
 /*
-  Highest number of fixed characters in subject line is 47:
+  Higest number of fixed characters in subject line is 47:
   MySQL_SERVER_<suffix>_Auto_Generated_Server_Certificate
   Maximum size of subject is 64. So suffix can't be longer
   than 17 characters.
@@ -114,77 +108,44 @@ Sql_string_t ext_files[] = {create_string("cav3.ext"),
 Log info(cout, "NOTE");
 Log error(cerr, "ERROR");
 
-char **defaults_argv = nullptr;
-static char *opt_datadir = nullptr;
+char **defaults_argv = 0;
+static char *opt_datadir = 0;
 static char default_data_dir[] = MYSQL_DATADIR;
-static char *opt_suffix = nullptr;
+static char *opt_suffix = 0;
 static char default_suffix[] = MYSQL_SERVER_VERSION;
-#ifdef HAVE_CHOWN
-static char *opt_userid = nullptr;
-struct passwd *user_info = nullptr;
+#if HAVE_CHOWN
+static char *opt_userid = 0;
+struct passwd *user_info = 0;
 #endif /* HAVE_CHOWN */
 Path dir_string;
 Sql_string_t suffix_string;
 bool opt_verbose;
 
-static const char *load_default_groups[] = {"mysql_ssl_rsa_setup", "mysqld",
-                                            nullptr};
+static const char *load_default_groups[] = {"mysql_ssl_rsa_setup", "mysqld", 0};
 
 static struct my_option my_options[] = {
-    {"help", '?', "Display this help and exit.", nullptr, nullptr, nullptr,
-     GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
-    {"verbose", 'v', "Be more verbose when running program", &opt_verbose,
-     nullptr, nullptr, GET_BOOL, NO_ARG, false, 0, 0, nullptr, 0, nullptr},
-    {"version", 'V', "Print program version and exit", nullptr, nullptr,
-     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+    {"help", '?', "Display this help and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0,
+     0, 0, 0, 0, 0},
+    {"verbose", 'v', "Be more verbose when running program", &opt_verbose, 0, 0,
+     GET_BOOL, NO_ARG, false, 0, 0, 0, 0, 0},
+    {"version", 'V', "Print program version and exit", 0, 0, 0, GET_NO_ARG,
+     NO_ARG, 0, 0, 0, 0, 0, 0},
     {"datadir", 'd', "Directory to store generated files.", &opt_datadir,
-     &opt_datadir, nullptr, GET_STR_ALLOC, REQUIRED_ARG,
-     (longlong)&default_data_dir, 0, 0, nullptr, 0, nullptr},
+     &opt_datadir, 0, GET_STR_ALLOC, REQUIRED_ARG, (longlong)&default_data_dir,
+     0, 0, 0, 0, 0},
     {"suffix", 's', "Suffix to be added in certificate subject line",
-     &opt_suffix, &opt_suffix, nullptr, GET_STR_ALLOC, REQUIRED_ARG,
-     (longlong)&default_suffix, 0, 0, nullptr, 0, nullptr},
-#ifdef HAVE_CHOWN
+     &opt_suffix, &opt_suffix, 0, GET_STR_ALLOC, REQUIRED_ARG,
+     (longlong)&default_suffix, 0, 0, 0, 0, 0},
+#if HAVE_CHOWN
     {"uid", 0, "The effective user id to be used for file permission",
-     &opt_userid, &opt_userid, nullptr, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &opt_userid, &opt_userid, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0,
+     0},
 #endif /* HAVE_CHOWN */
     /* END TOKEN */
-    {nullptr, 0, nullptr, nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0,
-     0, nullptr, 0, nullptr}};
+    {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
 
 /* Helper Functions */
 
-/*
-  Fetch the SSL security level
-*/
-int security_level(void) {
-  int current_sec_level = 2;
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-  /*
-    create a temporary SSL_CTX, we're going to use it to fetch
-    the current OpenSSL security level. So that we can generate
-    keys accordingly.
-  */
-  SSL_CTX *temp_ssl_ctx = SSL_CTX_new(TLS_server_method());
-
-  /* get the current security level */
-  current_sec_level = SSL_CTX_get_security_level(temp_ssl_ctx);
-
-  assert(current_sec_level <= 5);
-
-  /* current range for security level is [1,5] */
-  if (current_sec_level > 5)
-    current_sec_level = 5;
-  else if (current_sec_level <= 1)
-    current_sec_level = 2;
-
-  /* get rid of temp_ssl_ctx, we're done with it */
-  SSL_CTX_free(temp_ssl_ctx);
-#endif
-
-  return current_sec_level;
-}
-
 /**
   The string class will break if constructed with a NULL pointer. This wrapper
   provides a systematic protection when importing char pointers.
@@ -224,7 +185,7 @@ static int set_file_pair_permission(const Sql_string_t &priv,
           << pub.c_str() << endl;
     return 1;
   }
-#ifdef HAVE_CHOWN
+#if HAVE_CHOWN
   if (user_info) {
     if (chown(priv.c_str(), user_info->pw_uid, user_info->pw_gid) ||
         chown(pub.c_str(), user_info->pw_uid, user_info->pw_gid)) {
@@ -238,7 +199,7 @@ static int set_file_pair_permission(const Sql_string_t &priv,
 
 static bool file_exists(const Sql_string_t &filename) {
   MY_STAT file_stat;
-  if (my_stat(filename.c_str(), &file_stat, MYF(0)) == nullptr) return false;
+  if (my_stat(filename.c_str(), &file_stat, MYF(0)) == NULL) return false;
 
   return true;
 }
@@ -254,16 +215,16 @@ static int remove_file(const Sql_string_t &filename, bool report_error = true) {
 static void free_resources() {
   if (opt_datadir) my_free(opt_datadir);
   if (opt_suffix) my_free(opt_suffix);
-#ifdef HAVE_CHOWN
+#if HAVE_CHOWN
   if (opt_userid) my_free(opt_userid);
 #endif
 }
 
 class RSA_priv {
  public:
-  explicit RSA_priv(uint32_t key_size) : m_key_size(key_size) {}
+  RSA_priv(uint32_t key_size = 2048) : m_key_size(key_size) {}
 
-  ~RSA_priv() = default;
+  ~RSA_priv() {}
 
   Sql_string_t operator()(const Sql_string_t &key_file) {
     stringstream command;
@@ -290,16 +251,15 @@ class RSA_pub {
 
 class X509_key {
  public:
-  explicit X509_key(const Sql_string_t &version, uint32_t validity = 10 * 365L)
+  X509_key(const Sql_string_t &version, uint32_t validity = 10 * 365L)
       : m_validity(validity) {
     m_subj_prefix << "-subj /CN=MySQL_Server_" << version;
   }
 
   Sql_string_t operator()(Sql_string_t suffix, const Sql_string_t &key_file,
-                          const Sql_string_t &req_file,
-                          const Sql_string_t &key_size) {
+                          const Sql_string_t &req_file) {
     stringstream command;
-    command << "openssl req -newkey rsa:" << key_size << " -days " << m_validity
+    command << "openssl req -newkey rsa:2048 -days " << m_validity
             << " -nodes -keyout " << key_file << " " << m_subj_prefix.str()
             << suffix << " -out " << req_file << " && openssl rsa -in "
             << key_file << " -out " << key_file;
@@ -319,7 +279,7 @@ class X509v3_ext_writer {
 
     m_certv3_ext_options << "basicConstraints=CA:FALSE" << std::endl;
   }
-  ~X509v3_ext_writer() = default;
+  ~X509v3_ext_writer() {}
 
   bool operator()(const Sql_string_t &cav3_ext_file,
                   const Sql_string_t &certv3_ext_file) {
@@ -350,9 +310,9 @@ class X509v3_ext_writer {
 
 class X509_cert {
  public:
-  explicit X509_cert(uint32_t validity = 10 * 365L) : m_validity(validity) {}
+  X509_cert(uint32_t validity = 10 * 365L) : m_validity(validity) {}
 
-  ~X509_cert() = default;
+  ~X509_cert() {}
 
   Sql_string_t operator()(const Sql_string_t &req_file,
                           const Sql_string_t &cert_file, uint32_t serial,
@@ -399,7 +359,7 @@ static bool my_arguments_get_one_option(int optid, const struct my_option *,
       free_resources();
       exit(0);
   }
-  return false;
+  return 0;
 }
 }
 
@@ -415,7 +375,6 @@ static bool check_suffix() {
 
 int main(int argc, char *argv[]) {
   int ret_val = 0;
-  int sec_level = security_level();
   Sql_string_t openssl_check("openssl version");
   bool save_skip_unknown = my_getopt_skip_unknown;
   MEM_ROOT alloc{PSI_NOT_INSTRUMENTED, 512};
@@ -424,8 +383,6 @@ int main(int argc, char *argv[]) {
   DBUG_TRACE;
   DBUG_PROCESS(argv[0]);
 
-  CLIENT_WARN_DEPRECATED("mysql_ssl_rsa_setup", "the mysqld server");
-
   /* Parse options : Command Line/Config file */
 
 #ifdef _WIN32
@@ -514,7 +471,7 @@ int main(int argc, char *argv[]) {
       ret_val = 1;
       goto end;
     }
-#ifdef HAVE_CHOWN
+#if HAVE_CHOWN
     if (opt_userid && geteuid() == 0) {
       user_info = getpwnam(opt_userid);
       if (!user_info) {
@@ -569,8 +526,7 @@ int main(int argc, char *argv[]) {
       /* Generate CA Key and Certificate */
       if ((ret_val =
                execute_command(x509_key("_Auto_Generated_CA_Certificate",
-                                        cert_files[CA_KEY], cert_files[CA_REQ],
-                                        to_string(rsa_key_sizes[sec_level])),
+                                        cert_files[CA_KEY], cert_files[CA_REQ]),
                                "Error generating ca_key.pem and ca_req.pem")))
         goto end;
 
@@ -583,8 +539,7 @@ int main(int argc, char *argv[]) {
       /* Generate Server Key and Certificate */
       if ((ret_val = execute_command(
                x509_key("_Auto_Generated_Server_Certificate",
-                        cert_files[SERVER_KEY], cert_files[SERVER_REQ],
-                        to_string(rsa_key_sizes[sec_level])),
+                        cert_files[SERVER_KEY], cert_files[SERVER_REQ]),
                "Error generating server_key.pem and server_req.pem")))
         goto end;
 
@@ -598,8 +553,7 @@ int main(int argc, char *argv[]) {
       /* Generate Client Key and Certificate */
       if ((ret_val = execute_command(
                x509_key("_Auto_Generated_Client_Certificate",
-                        cert_files[CLIENT_KEY], cert_files[CLIENT_REQ],
-                        to_string(rsa_key_sizes[sec_level])),
+                        cert_files[CLIENT_KEY], cert_files[CLIENT_REQ]),
                "Error generating client_key.pem and client_req.pem")))
         goto end;
 
@@ -658,7 +612,7 @@ int main(int argc, char *argv[]) {
       info << "RSA key files are present in given dir. Skipping generation."
            << endl;
     } else {
-      RSA_priv rsa_priv(rsa_key_sizes[sec_level]);
+      RSA_priv rsa_priv;
       RSA_pub rsa_pub;
 
       /* Remove existing file if any */
