@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,6 +33,7 @@
 #include <time.h>
 
 #include "client/client_priv.h"
+#include "compression.h"
 #include "my_alloc.h"
 #include "my_dbug.h"
 #include "my_default.h"
@@ -67,8 +68,8 @@ static bool debug_info_flag = 0, debug_check_flag = 0;
 static uint opt_use_threads = 0, opt_local_file = 0, my_end_arg = 0;
 static char *opt_password = 0, *current_user = 0, *current_host = 0,
             *current_db = 0, *fields_terminated = 0, *lines_terminated = 0,
-            *enclosed = 0, *opt_enclosed = 0, *escaped = 0, *opt_columns = 0,
-            *default_charset = (char *)MYSQL_AUTODETECT_CHARSET_NAME;
+            *enclosed = 0, *opt_enclosed = 0, *escaped = 0, *opt_columns = 0;
+static const char *default_charset = MYSQL_AUTODETECT_CHARSET_NAME;
 static uint opt_enable_cleartext_plugin = 0;
 static bool using_opt_enable_cleartext_plugin = 0;
 static uint opt_mysql_port = 0, opt_protocol = 0;
@@ -76,6 +77,9 @@ static char *opt_bind_addr = NULL;
 static char *opt_mysql_unix_port = 0;
 static char *opt_plugin_dir = 0, *opt_default_auth = 0;
 static longlong opt_ignore_lines = -1;
+static uint opt_zstd_compress_level = default_zstd_compression_level;
+static char *opt_compress_algorithm = nullptr;
+
 #include "caching_sha2_passwordopt-vars.h"
 #include "sslopt-vars.h"
 
@@ -211,6 +215,17 @@ static struct my_option my_long_options[] = {
      0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"version", 'V', "Output version information and exit.", 0, 0, 0,
      GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"compression-algorithms", 0,
+     "Use compression algorithm in server/client protocol. Valid values "
+     "are any combination of 'zstd','zlib','uncompressed'.",
+     &opt_compress_algorithm, &opt_compress_algorithm, 0, GET_STR, REQUIRED_ARG,
+     0, 0, 0, 0, 0, 0},
+    {"zstd-compression-level", 0,
+     "Use this compression level in the client/server protocol, in case "
+     "--compression-algorithms=zstd. Valid range is between 1 and 22, "
+     "inclusive. Default is 3.",
+     &opt_zstd_compress_level, &opt_zstd_compress_level, 0, GET_UINT,
+     REQUIRED_ARG, 3, 1, 22, 0, 0, 0},
     {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
 
 static const char *load_default_groups[] = {"mysqlimport", "client", 0};
@@ -237,8 +252,13 @@ static bool get_one_option(int optid, const struct my_option *opt,
                            char *argument) {
   switch (optid) {
     case 'p':
-      if (argument == disabled_my_option)
-        argument = (char *)""; /* Don't require password */
+      if (argument == disabled_my_option) {
+        // Don't require password
+        static char empty_password[] = {'\0'};
+        DBUG_ASSERT(empty_password[0] ==
+                    '\0');  // Check that it has not been overwritten
+        argument = empty_password;
+      }
       if (argument) {
         char *start = argument;
         my_free(opt_password);
@@ -314,7 +334,7 @@ static int write_to_table(char *filename, MYSQL *mysql) {
   char tablename[FN_REFLEN], hard_path[FN_REFLEN],
       escaped_name[FN_REFLEN * 2 + 1], sql_statement[FN_REFLEN * 16 + 256],
       *end, *pos;
-  DBUG_ENTER("write_to_table");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("filename: %s", filename));
 
   fn_format(tablename, filename, "", "", 1 | 2); /* removes path & ext. */
@@ -329,7 +349,7 @@ static int write_to_table(char *filename, MYSQL *mysql) {
     snprintf(sql_statement, FN_REFLEN * 16 + 256, "DELETE FROM %s", tablename);
     if (mysql_query(mysql, sql_statement)) {
       db_error_with_table(mysql, tablename);
-      DBUG_RETURN(1);
+      return 1;
     }
   }
   to_unix_path(hard_path);
@@ -374,7 +394,7 @@ static int write_to_table(char *filename, MYSQL *mysql) {
 
   if (mysql_query(mysql, sql_statement)) {
     db_error_with_table(mysql, tablename);
-    DBUG_RETURN(1);
+    return 1;
   }
   if (!silent) {
     if (mysql_info(mysql)) /* If NULL-pointer, print nothing */
@@ -382,7 +402,7 @@ static int write_to_table(char *filename, MYSQL *mysql) {
       fprintf(stdout, "%s.%s: %s\n", current_db, tablename, mysql_info(mysql));
     }
   }
-  DBUG_RETURN(0);
+  return 0;
 }
 
 static void lock_table(MYSQL *mysql, int tablecount, char **raw_tablename) {
@@ -414,6 +434,14 @@ static MYSQL *db_connect(char *host, char *database, char *user, char *passwd) {
   } else if (!(mysql = mysql_init(NULL)))
     return 0;
   if (opt_compress) mysql_options(mysql, MYSQL_OPT_COMPRESS, NullS);
+
+  if (opt_compress_algorithm)
+    mysql_options(mysql, MYSQL_OPT_COMPRESSION_ALGORITHMS,
+                  opt_compress_algorithm);
+
+  mysql_options(mysql, MYSQL_OPT_ZSTD_COMPRESSION_LEVEL,
+                &opt_zstd_compress_level);
+
   if (opt_local_file)
     mysql_options(mysql, MYSQL_OPT_LOCAL_INFILE, (char *)&opt_local_file);
   if (SSL_SET_OPTIONS(mysql)) {
