@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -50,28 +49,25 @@
 #include "typelib.h"
 #include "welcome_copyright_notice.h" /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
-static char *host = nullptr, *user = nullptr;
-static bool opt_show_keys = false, opt_compress = false, opt_count = false,
-            opt_status = false;
-static bool opt_table_type = false;
-static bool debug_info_flag = false, debug_check_flag = false;
+static char *host = 0, *opt_password = 0, *user = 0;
+static bool opt_show_keys = 0, opt_compress = 0, opt_count = 0, opt_status = 0;
+static bool tty_password = 0, opt_table_type = 0;
+static bool debug_info_flag = 0, debug_check_flag = 0;
 static uint my_end_arg = 0;
 static uint opt_verbose = 0;
 static const char *default_charset = MYSQL_AUTODETECT_CHARSET_NAME;
-static char *opt_plugin_dir = nullptr, *opt_default_auth = nullptr;
+static char *opt_plugin_dir = 0, *opt_default_auth = 0;
 static uint opt_enable_cleartext_plugin = 0;
-static bool using_opt_enable_cleartext_plugin = false;
+static bool using_opt_enable_cleartext_plugin = 0;
 
 static uint opt_zstd_compress_level = default_zstd_compression_level;
 static char *opt_compress_algorithm = nullptr;
 
-#include "multi_factor_passwordopt-vars.h"
-
 #if defined(_WIN32)
 static char *shared_memory_base_name = 0;
 #endif
 static uint opt_protocol = 0;
-static char *opt_bind_addr = nullptr;
+static char *opt_bind_addr = NULL;
 
 static void get_options(int *argc, char ***argv);
 static uint opt_mysql_port = 0;
@@ -87,12 +83,12 @@ static void print_res_header(MYSQL_RES *result);
 static void print_res_top(MYSQL_RES *result);
 static void print_res_row(MYSQL_RES *result, MYSQL_ROW cur);
 
-static const char *load_default_groups[] = {"mysqlshow", "client", nullptr};
-static char *opt_mysql_unix_port = nullptr;
+static const char *load_default_groups[] = {"mysqlshow", "client", 0};
+static char *opt_mysql_unix_port = 0;
 
 int main(int argc, char **argv) {
   int error;
-  bool first_argument_uses_wildcards = false;
+  bool first_argument_uses_wildcards = 0;
   char *wild;
   MYSQL mysql;
   MY_INIT(argv[0]);
@@ -104,22 +100,22 @@ int main(int argc, char **argv) {
 
   get_options(&argc, &argv);
 
-  wild = nullptr;
+  wild = 0;
   if (argc) {
     char *pos = argv[argc - 1], *to;
     for (to = pos; *pos; pos++, to++) {
       switch (*pos) {
         case '*':
           *pos = '%';
-          first_argument_uses_wildcards = true;
+          first_argument_uses_wildcards = 1;
           break;
         case '?':
           *pos = '_';
-          first_argument_uses_wildcards = true;
+          first_argument_uses_wildcards = 1;
           break;
         case '%':
         case '_':
-          first_argument_uses_wildcards = true;
+          first_argument_uses_wildcards = 1;
           break;
         case '\\':
           pos++;
@@ -172,27 +168,18 @@ int main(int argc, char **argv) {
     mysql_options(&mysql, MYSQL_ENABLE_CLEARTEXT_PLUGIN,
                   (char *)&opt_enable_cleartext_plugin);
 
-  mysql_options(&mysql, MYSQL_OPT_CONNECT_ATTR_RESET, nullptr);
+  mysql_options(&mysql, MYSQL_OPT_CONNECT_ATTR_RESET, 0);
   mysql_options4(&mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "program_name",
                  "mysqlshow");
   set_server_public_key(&mysql);
   set_get_server_public_key_option(&mysql);
-  set_password_options(&mysql);
-  if (!(mysql_real_connect(&mysql, host, user, nullptr,
+  if (!(mysql_real_connect(&mysql, host, user, opt_password,
                            (first_argument_uses_wildcards) ? "" : argv[0],
                            opt_mysql_port, opt_mysql_unix_port, 0))) {
     fprintf(stderr, "%s: %s\n", my_progname, mysql_error(&mysql));
     exit(1);
   }
-  if (ssl_client_check_post_connect_ssl_setup(
-          &mysql, [](const char *err) { fprintf(stderr, "%s\n", err); })) {
-    mysql_close(&mysql);
-    free_passwords();
-    mysql_server_end();
-    my_end(my_end_arg);
-    exit(1);
-  }
-  mysql.reconnect = true;
+  mysql.reconnect = 1;
 
   switch (argc) {
     case 0:
@@ -212,7 +199,7 @@ int main(int argc, char **argv) {
       break;
   }
   mysql_close(&mysql); /* Close & free connection */
-  free_passwords();
+  my_free(opt_password);
 #if defined(_WIN32)
   my_free(shared_memory_base_name);
 #endif
@@ -223,51 +210,47 @@ int main(int argc, char **argv) {
 
 static struct my_option my_long_options[] = {
     {"bind-address", 0, "IP address to bind to.", (uchar **)&opt_bind_addr,
-     (uchar **)&opt_bind_addr, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr,
-     0, nullptr},
+     (uchar **)&opt_bind_addr, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"character-sets-dir", 'c', "Directory for character set files.",
-     &charsets_dir, &charsets_dir, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &charsets_dir, &charsets_dir, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"default-character-set", OPT_DEFAULT_CHARSET,
-     "Set the default character set.", &default_charset, &default_charset,
-     nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     "Set the default character set.", &default_charset, &default_charset, 0,
+     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"count", OPT_COUNT,
      "Show number of rows per table (may be slow for non-MyISAM tables).",
-     &opt_count, &opt_count, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     &opt_count, &opt_count, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"compress", 'C', "Use compression in server/client protocol.",
-     &opt_compress, &opt_compress, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr,
-     0, nullptr},
-    {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.", nullptr,
-     nullptr, nullptr, GET_STR, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &opt_compress, &opt_compress, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"debug", '#', "Output debug log. Often this is 'd:t:o,filename'.", 0, 0, 0,
+     GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
     {"debug-check", OPT_DEBUG_CHECK,
      "Check memory and open file usage at exit.", &debug_check_flag,
-     &debug_check_flag, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     &debug_check_flag, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"debug-info", OPT_DEBUG_INFO, "Print some debug info at exit.",
-     &debug_info_flag, &debug_info_flag, nullptr, GET_BOOL, NO_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &debug_info_flag, &debug_info_flag, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"default_auth", OPT_DEFAULT_AUTH,
      "Default authentication client-side plugin to use.", &opt_default_auth,
-     &opt_default_auth, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     &opt_default_auth, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"enable_cleartext_plugin", OPT_ENABLE_CLEARTEXT_PLUGIN,
      "Enable/disable the clear text authentication plugin.",
-     &opt_enable_cleartext_plugin, &opt_enable_cleartext_plugin, nullptr,
-     GET_BOOL, OPT_ARG, 0, 0, 0, nullptr, 0, nullptr},
-    {"help", '?', "Display this help and exit.", nullptr, nullptr, nullptr,
-     GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
-    {"host", 'h', "Connect to host.", &host, &host, nullptr, GET_STR,
-     REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &opt_enable_cleartext_plugin, &opt_enable_cleartext_plugin, 0, GET_BOOL,
+     OPT_ARG, 0, 0, 0, 0, 0, 0},
+    {"help", '?', "Display this help and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0,
+     0, 0, 0, 0, 0},
+    {"host", 'h', "Connect to host.", &host, &host, 0, GET_STR, REQUIRED_ARG, 0,
+     0, 0, 0, 0, 0},
     {"status", 'i', "Shows a lot of extra information about each table.",
-     &opt_status, &opt_status, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
-    {"keys", 'k', "Show keys for table.", &opt_show_keys, &opt_show_keys,
-     nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
-#include "multi_factor_passwordopt-longopts.h"
+     &opt_status, &opt_status, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"keys", 'k', "Show keys for table.", &opt_show_keys, &opt_show_keys, 0,
+     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"password", 'p',
+     "Password to use when connecting to server. If password is not given, "
+     "it's "
+     "solicited on the tty.",
+     0, 0, 0, GET_PASSWORD, OPT_ARG, 0, 0, 0, 0, 0, 0},
     {"plugin_dir", OPT_PLUGIN_DIR, "Directory for client-side plugins.",
-     &opt_plugin_dir, &opt_plugin_dir, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &opt_plugin_dir, &opt_plugin_dir, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0,
+     0},
     {"port", 'P',
      "Port number to use for connection or 0 for default to, in "
      "order of preference, my.cnf, $MYSQL_TCP_PORT, "
@@ -275,15 +258,15 @@ static struct my_option my_long_options[] = {
      "/etc/services, "
 #endif
      "built-in default (" STRINGIFY_ARG(MYSQL_PORT) ").",
-     &opt_mysql_port, &opt_mysql_port, nullptr, GET_UINT, REQUIRED_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &opt_mysql_port, &opt_mysql_port, 0, GET_UINT, REQUIRED_ARG, 0, 0, 0, 0, 0,
+     0},
 #ifdef _WIN32
     {"pipe", 'W', "Use named pipes to connect to server.", 0, 0, 0, GET_NO_ARG,
      NO_ARG, 0, 0, 0, 0, 0, 0},
 #endif
     {"protocol", OPT_MYSQL_PROTOCOL,
-     "The protocol to use for connection (tcp, socket, pipe, memory).", nullptr,
-     nullptr, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     "The protocol to use for connection (tcp, socket, pipe, memory).", 0, 0, 0,
+     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 #if defined(_WIN32)
     {"shared-memory-base-name", OPT_SHARED_MEMORY_BASE_NAME,
      "Base name of shared memory.", &shared_memory_base_name,
@@ -291,35 +274,33 @@ static struct my_option my_long_options[] = {
      0},
 #endif
     {"show-table-type", 't', "Show table type column.", &opt_table_type,
-     &opt_table_type, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &opt_table_type, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"socket", 'S', "The socket file to use for connection.",
-     &opt_mysql_unix_port, &opt_mysql_unix_port, nullptr, GET_STR, REQUIRED_ARG,
-     0, 0, 0, nullptr, 0, nullptr},
+     &opt_mysql_unix_port, &opt_mysql_unix_port, 0, GET_STR, REQUIRED_ARG, 0, 0,
+     0, 0, 0, 0},
 #include "caching_sha2_passwordopt-longopts.h"
 #include "sslopt-longopts.h"
 
-    {"user", 'u', "User for login if not current user.", &user, &user, nullptr,
-     GET_STR, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+    {"user", 'u', "User for login if not current user.", &user, &user, 0,
+     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"verbose", 'v',
      "More verbose output; you can use this multiple times to get even more "
      "verbose output.",
-     nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
-    {"version", 'V', "Output version information and exit.", nullptr, nullptr,
-     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
+    {"version", 'V', "Output version information and exit.", 0, 0, 0,
+     GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"compression-algorithms", 0,
      "Use compression algorithm in server/client protocol. Valid values "
      "are any combination of 'zstd','zlib','uncompressed'.",
-     &opt_compress_algorithm, &opt_compress_algorithm, nullptr, GET_STR,
-     REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     &opt_compress_algorithm, &opt_compress_algorithm, 0, GET_STR, REQUIRED_ARG,
+     0, 0, 0, 0, 0, 0},
     {"zstd-compression-level", 0,
      "Use this compression level in the client/server protocol, in case "
      "--compression-algorithms=zstd. Valid range is between 1 and 22, "
      "inclusive. Default is 3.",
-     &opt_zstd_compress_level, &opt_zstd_compress_level, nullptr, GET_UINT,
-     REQUIRED_ARG, 3, 1, 22, nullptr, 0, nullptr},
-    {nullptr, 0, nullptr, nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0,
-     0, nullptr, 0, nullptr}};
+     &opt_zstd_compress_level, &opt_zstd_compress_level, 0, GET_UINT,
+     REQUIRED_ARG, 3, 1, 22, 0, 0, 0},
+    {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
 
 static void usage(void) {
   print_version();
@@ -348,7 +329,24 @@ static bool get_one_option(int optid, const struct my_option *opt,
     case 'v':
       opt_verbose++;
       break;
-      PARSE_COMMAND_LINE_PASSWORD_OPTION;
+    case 'p':
+      if (argument == disabled_my_option) {
+        // Don't require password
+        static char empty_password[] = {'\0'};
+        DBUG_ASSERT(empty_password[0] ==
+                    '\0');  // Check that it has not been overwritten
+        argument = empty_password;
+      }
+      if (argument) {
+        char *start = argument;
+        my_free(opt_password);
+        opt_password = my_strdup(PSI_NOT_INSTRUMENTED, argument, MYF(MY_FAE));
+        while (*argument) *argument++ = 'x'; /* Destroy argument */
+        if (*start) start[1] = 0;            /* Cut length of argument */
+        tty_password = 0;
+      } else
+        tty_password = 1;
+      break;
     case 'W':
 #ifdef _WIN32
       opt_protocol = MYSQL_PROTOCOL_PIPE;
@@ -363,7 +361,7 @@ static bool get_one_option(int optid, const struct my_option *opt,
       break;
     case '#':
       DBUG_PUSH(argument ? argument : "d:t:o");
-      debug_check_flag = true;
+      debug_check_flag = 1;
       break;
 #include "sslopt-case.h"
 
@@ -374,11 +372,8 @@ static bool get_one_option(int optid, const struct my_option *opt,
     case 'I': /* Info */
       usage();
       exit(0);
-    case 'C':
-      CLIENT_WARN_DEPRECATED("--compress", "--compression-algorithms");
-      break;
   }
-  return false;
+  return 0;
 }
 }  // extern "C"
 
@@ -388,6 +383,7 @@ static void get_options(int *argc, char ***argv) {
   if ((ho_error = handle_options(argc, argv, my_long_options, get_one_option)))
     exit(ho_error);
 
+  if (tty_password) opt_password = get_tty_password(NullS);
   if (opt_count) {
     /*
       We need to set verbose to 2 as we need to change the output to include
@@ -409,7 +405,7 @@ static int list_dbs(MYSQL *mysql, const char *wild) {
   char query[NAME_LEN + 100];
   MYSQL_FIELD *field;
   MYSQL_RES *result;
-  MYSQL_ROW row = nullptr, rrow;
+  MYSQL_ROW row = NULL, rrow;
 
   if (!(result = mysql_list_dbs(mysql, wild))) {
     fprintf(stderr, "%s: Cannot list databases: %s\n", my_progname,
@@ -428,9 +424,9 @@ static int list_dbs(MYSQL *mysql, const char *wild) {
     if (!my_strcasecmp(&my_charset_latin1, row[0], wild)) {
       mysql_free_result(result);
       if (opt_status)
-        return list_table_status(mysql, wild, nullptr);
+        return list_table_status(mysql, wild, NULL);
       else
-        return list_tables(mysql, wild, nullptr);
+        return list_tables(mysql, wild, NULL);
     }
   }
 
@@ -454,7 +450,7 @@ static int list_dbs(MYSQL *mysql, const char *wild) {
 
     if (opt_verbose) {
       if (!(mysql_select_db(mysql, row[0]))) {
-        MYSQL_RES *tresult = mysql_list_tables(mysql, (char *)nullptr);
+        MYSQL_RES *tresult = mysql_list_tables(mysql, (char *)NULL);
         if (mysql_affected_rows(mysql) > 0) {
           sprintf(tables, "%6lu", (ulong)mysql_affected_rows(mysql));
           rowcount = 0;
@@ -468,7 +464,7 @@ static int list_dbs(MYSQL *mysql, const char *wild) {
                 MYSQL_RES *rresult;
                 if ((rresult = mysql_store_result(mysql))) {
                   rrow = mysql_fetch_row(rresult);
-                  rowcount += (ulong)my_strtoull(rrow[0], (char **)nullptr, 10);
+                  rowcount += (ulong)my_strtoull(rrow[0], (char **)0, 10);
                   mysql_free_result(rresult);
                 }
               }
@@ -493,7 +489,7 @@ static int list_dbs(MYSQL *mysql, const char *wild) {
     else
       print_row(row[0], length, tables, 6, rows, 12, NullS);
 
-    row = nullptr;
+    row = NULL;
   }
 
   print_trailer(length, (opt_verbose > 0 ? 6 : 0), (opt_verbose > 1 ? 12 : 0),
@@ -567,7 +563,7 @@ static int list_tables(MYSQL *mysql, const char *db, const char *table) {
     counter++;
     if (opt_verbose > 0) {
       if (!(mysql_select_db(mysql, db))) {
-        MYSQL_RES *rresult = mysql_list_fields(mysql, row[0], nullptr);
+        MYSQL_RES *rresult = mysql_list_fields(mysql, row[0], NULL);
         ulong rowcount = 0L;
         if (!rresult) {
           my_stpcpy(fields, "N/A");
@@ -582,8 +578,7 @@ static int list_tables(MYSQL *mysql, const char *db, const char *table) {
             if (!(mysql_query(mysql, query))) {
               if ((rresult = mysql_store_result(mysql))) {
                 rrow = mysql_fetch_row(rresult);
-                rowcount +=
-                    (unsigned long)my_strtoull(rrow[0], (char **)nullptr, 10);
+                rowcount += (unsigned long)my_strtoull(rrow[0], (char **)0, 10);
                 mysql_free_result(rresult);
               }
               sprintf(rows, "%10lu", rowcount);
@@ -614,15 +609,9 @@ static int list_tables(MYSQL *mysql, const char *db, const char *table) {
   }
 
   print_trailer(
-      head_length,
-      (opt_table_type    ? 10
-       : opt_verbose > 0 ? 8
-                         : 0),
+      head_length, (opt_table_type ? 10 : opt_verbose > 0 ? 8 : 0),
       (opt_table_type ? (opt_verbose > 0 ? 8 : 0) : (opt_verbose > 1 ? 10 : 0)),
-      !opt_table_type   ? 0
-      : opt_verbose > 1 ? 10
-                        : 0,
-      0);
+      !opt_table_type ? 0 : opt_verbose > 1 ? 10 : 0, 0);
 
   if (counter && opt_verbose)
     printf("%u row%s in set.\n\n", counter, (counter > 1) ? "s" : "");
@@ -689,7 +678,7 @@ static int list_fields(MYSQL *mysql, const char *db, const char *table,
       return 1;
     }
     row = mysql_fetch_row(result);
-    rows = (ulong)my_strtoull(row[0], (char **)nullptr, 10);
+    rows = (ulong)my_strtoull(row[0], (char **)0, 10);
     mysql_free_result(result);
   }
 
