@@ -1,16 +1,15 @@
-// Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+// Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0,
 // as published by the Free Software Foundation.
 //
-// This program is designed to work with certain software (including
+// This program is also distributed with certain software (including
 // but not limited to OpenSSL) that is licensed under separate terms,
 // as designated in a particular file or component or in included license
 // documentation.  The authors of MySQL hereby grant you an additional
 // permission to link the program and your derivative works with the
-// separately licensed software that they have either included with
-// the program or referenced in the documentation.
+// separately licensed software that they have included with MySQL.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,7 +37,7 @@
 /// @retval True if fails to get the value, true otherwise.
 bool query_get_string(MYSQL *mysql, const char *query, int column,
                       std::string *str) {
-  MYSQL_RES *res = nullptr;
+  MYSQL_RES *res = NULL;
   MYSQL_ROW row;
 
   if (mysql_query(mysql, query)) {
@@ -46,10 +45,10 @@ bool query_get_string(MYSQL *mysql, const char *query, int column,
     return true;
   }
 
-  if (((res = mysql_store_result(mysql)) == nullptr) ||
-      ((row = mysql_fetch_row(res)) == nullptr)) {
+  if (((res = mysql_store_result(mysql)) == NULL) ||
+      ((row = mysql_fetch_row(res)) == NULL)) {
     mysql_free_result(res);
-    str = nullptr;
+    str = 0;
     return true;
   }
 
