@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,26 +51,25 @@ LINE_BUFFER *batch_readline_init(ulong max_size, FILE *file) {
   if (my_fstat(fileno(file), &input_file_stat) ||
       MY_S_ISDIR(input_file_stat.st_mode) ||
       MY_S_ISBLK(input_file_stat.st_mode))
-    return nullptr;
+    return 0;
 #endif
 
   if (!(line_buff =
             (LINE_BUFFER *)my_malloc(PSI_NOT_INSTRUMENTED, sizeof(*line_buff),
                                      MYF(MY_WME | MY_ZEROFILL))))
-    return nullptr;
+    return 0;
   if (init_line_buffer(line_buff, my_fileno(file), batch_io_size, max_size)) {
     my_free(line_buff);
-    return nullptr;
+    return 0;
   }
   return line_buff;
 }
 
-char *batch_readline(LINE_BUFFER *line_buff,
-                     bool binary_mode [[maybe_unused]]) {
+char *batch_readline(LINE_BUFFER *line_buff, bool binary_mode) {
   char *pos;
   ulong out_length;
 
-  if (!(pos = intern_read_line(line_buff, &out_length))) return nullptr;
+  if (!(pos = intern_read_line(line_buff, &out_length))) return 0;
   if (out_length && pos[out_length - 1] == '\n') {
 #if defined(_WIN32)
     /*
@@ -112,10 +110,10 @@ LINE_BUFFER *batch_readline_command(LINE_BUFFER *line_buff, char *str) {
     if (!(line_buff =
               (LINE_BUFFER *)my_malloc(PSI_NOT_INSTRUMENTED, sizeof(*line_buff),
                                        MYF(MY_WME | MY_ZEROFILL))))
-      return nullptr;
+      return 0;
   if (init_line_buffer_from_string(line_buff, str)) {
     my_free(line_buff);
-    return nullptr;
+    return 0;
   }
   return line_buff;
 }
@@ -131,10 +129,10 @@ static bool init_line_buffer(LINE_BUFFER *buffer, File file, ulong size,
   buffer->max_size = max_buffer;
   if (!(buffer->buffer = (char *)my_malloc(
             PSI_NOT_INSTRUMENTED, buffer->bufread + 1, MYF(MY_WME | MY_FAE))))
-    return true;
+    return 1;
   buffer->end_of_line = buffer->end = buffer->buffer;
   buffer->buffer[0] = 0; /* For easy start test */
-  return false;
+  return 0;
 }
 
 /*
@@ -149,7 +147,7 @@ static bool init_line_buffer_from_string(LINE_BUFFER *buffer, char *str) {
             (char *)my_realloc(PSI_NOT_INSTRUMENTED, (uchar *)buffer->buffer,
                                old_length + length + 2,
                                MYF(MY_FAE | MY_ALLOW_ZERO_PTR))))
-    return true;
+    return 1;
   buffer->end = buffer->buffer + old_length;
   if (old_length) buffer->end[-1] = ' ';
   memcpy(buffer->end, str, length);
@@ -158,7 +156,7 @@ static bool init_line_buffer_from_string(LINE_BUFFER *buffer, char *str) {
   buffer->end += length + 1;
   buffer->eof = 1;
   buffer->max_size = 1;
-  return false;
+  return 0;
 }
 
 /*
@@ -243,15 +241,15 @@ char *intern_read_line(LINE_BUFFER *buffer, ulong *out_length) {
         string truncation.
       */
       if (!(length = fill_buffer(buffer))) {
-        if (buffer->eof) return nullptr;
+        if (buffer->eof) return 0;
       } else if (length == (size_t)-1)
-        return nullptr;
+        return NULL;
       else
         continue;
       pos--; /* break line here */
-      buffer->truncated = true;
+      buffer->truncated = 1;
     } else
-      buffer->truncated = false;
+      buffer->truncated = 0;
     buffer->end_of_line = pos + 1;
     *out_length = (ulong)(pos + 1 - buffer->eof - buffer->start_of_line);
 
