@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,6 +40,7 @@
   } while (0)
 #endif /* HAVE_VALGRIND */
 
+#define MY_ZEROFILL 32 /* fill array with zero */
 #define HEADER_SIZE 32
 #define MAGIC 1234
 #define USER_TO_HEADER(P) ((my_memory_header *)(((char *)P) - HEADER_SIZE))
@@ -66,7 +66,7 @@ extern "C" void *my_malloc(PSI_memory_key key, size_t size, int flags) {
   else
     mh = (my_memory_header *)malloc(raw_size);
 
-  if (mh != nullptr) {
+  if (mh != NULL) {
     void *user_ptr;
     mh->m_magic = MAGIC;
     mh->m_size = size;
@@ -75,13 +75,13 @@ extern "C" void *my_malloc(PSI_memory_key key, size_t size, int flags) {
     MEM_MALLOCLIKE_BLOCK(user_ptr, size, 0, (flags & MY_ZEROFILL));
     return user_ptr;
   }
-  return nullptr;
+  return NULL;
 }
 
 extern "C" void my_free(void *ptr) {
   my_memory_header *mh;
 
-  if (ptr == nullptr) return;
+  if (ptr == NULL) return;
 
   mh = USER_TO_HEADER(ptr);
   assert(mh->m_magic == MAGIC);
