@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,6 +26,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include <mysql/components/services/component_sys_var_service.h>
 #include <mysql/plugin.h>
 
+#include "../../components/mysql_server/component_sys_var_service.h"
 #include "my_macros.h"
 #include "typelib.h"
 
@@ -37,11 +37,9 @@ char log_text[MAX_BUFFER_LENGTH];
 FILE *outfile;
 const char *filename = "test_component_sys_var_service_int.log";
 
-#define WRITE_LOG(format, lit_log_text)                                 \
-  log_text_len = sprintf(log_text, format, lit_log_text);               \
-  if (fwrite((uchar *)log_text, sizeof(char), log_text_len, outfile) != \
-      static_cast<size_t>(log_text_len))                                \
-    return true;
+#define WRITE_LOG(format, lit_log_text)                   \
+  log_text_len = sprintf(log_text, format, lit_log_text); \
+  fwrite((uchar *)log_text, sizeof(char), log_text_len, outfile)
 
 REQUIRES_SERVICE_PLACEHOLDER(component_sys_variable_register);
 REQUIRES_SERVICE_PLACEHOLDER(component_sys_variable_unregister);
@@ -68,7 +66,7 @@ static mysql_service_status_t test_component_sys_var_service_int_init() {
   int_arg.blk_sz = 0;
   if (mysql_service_component_sys_variable_register->register_variable(
           "test_component_int", "int_sys_var", PLUGIN_VAR_INT,
-          "Registering int sys_variable", nullptr, nullptr, (void *)&int_arg,
+          "Registering int sys_variable", NULL, NULL, (void *)&int_arg,
           (void *)&int_variable_value)) {
     WRITE_LOG("%s\n", "int register_variable failed.");
   }
@@ -79,7 +77,7 @@ static mysql_service_status_t test_component_sys_var_service_int_init() {
   int_arg_2.blk_sz = 0;
   if (mysql_service_component_sys_variable_register->register_variable(
           "test_component_int", "int_sys_var_2", PLUGIN_VAR_INT,
-          "Registering int sys_variable", nullptr, nullptr, (void *)&int_arg_2,
+          "Registering int sys_variable", NULL, NULL, (void *)&int_arg_2,
           (void *)&int_var_2_value)) {
     WRITE_LOG("%s\n", "uint register_variable failed.");
   }
@@ -92,7 +90,7 @@ static mysql_service_status_t test_component_sys_var_service_int_init() {
   if (mysql_service_component_sys_variable_register->register_variable(
           "test_component_int", "uint_sys_var",
           PLUGIN_VAR_INT | PLUGIN_VAR_UNSIGNED, "Registering uint sys_variable",
-          nullptr, nullptr, (void *)&uint_arg, (void *)&uint_variable_value)) {
+          NULL, NULL, (void *)&uint_arg, (void *)&uint_variable_value)) {
     WRITE_LOG("%s\n", "uint register_variable failed.");
   }
 
