@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -46,8 +46,8 @@ class udf_list {
  public:
   ~udf_list() { unregister(); }
   bool add_scalar(const char *func_name, enum Item_result return_type,
-                  Udf_func_any func, Udf_func_init init_func = nullptr,
-                  Udf_func_deinit deinit_func = nullptr) {
+                  Udf_func_any func, Udf_func_init init_func = NULL,
+                  Udf_func_deinit deinit_func = NULL) {
     if (!mysql_service_udf_registration->udf_register(
             func_name, return_type, func, init_func, deinit_func)) {
       set.push_back(func_name);
@@ -57,10 +57,10 @@ class udf_list {
   }
 
   bool add_aggregate(const char *func_name, enum Item_result return_type,
-                     Udf_func_any func, Udf_func_add add_func = nullptr,
-                     Udf_func_clear clear_func = nullptr,
-                     Udf_func_init init_func = nullptr,
-                     Udf_func_deinit deinit_func = nullptr) {
+                     Udf_func_any func, Udf_func_add add_func = NULL,
+                     Udf_func_clear clear_func = NULL,
+                     Udf_func_init init_func = NULL,
+                     Udf_func_deinit deinit_func = NULL) {
     if (!mysql_service_udf_registration_aggregate->udf_register(
             func_name, return_type, func, init_func, deinit_func, add_func,
             clear_func)) {
@@ -102,10 +102,10 @@ const char *test_init = "test_init", *test_udf = "test_udf",
 
 static bool dynamic_udf_init(UDF_INIT *initid, UDF_ARGS *, char *) {
   initid->ptr = const_cast<char *>(test_init);
-  return false;
+  return 0;
 }
 
-static void dynamic_udf_deinit(UDF_INIT *initid [[maybe_unused]]) {
+static void dynamic_udf_deinit(UDF_INIT *initid MY_ATTRIBUTE((unused))) {
   assert(initid->ptr == test_init || initid->ptr == test_udf);
 }
 
@@ -120,7 +120,7 @@ static long long dynamic_udf(UDF_INIT *initid, UDF_ARGS *,
   return 42;
 }
 
-static void dynamic_agg_deinit(UDF_INIT *initid [[maybe_unused]]) {
+static void dynamic_agg_deinit(UDF_INIT *initid MY_ATTRIBUTE((unused))) {
   assert(initid->ptr == test_init || initid->ptr == test_udf ||
          initid->ptr == test_udf_clear || initid->ptr == test_udf_add);
 }
