@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,17 +57,17 @@ bool myfunc_double_init(UDF_INIT *initid, UDF_ARGS *args, char *message) {
 
   if (!args->arg_count) {
     strcpy(message, "myfunc_double must have at least one argument");
-    return true;
+    return 1;
   }
   /*
   ** As this function wants to have everything as strings, force all arguments
   ** to strings.
   */
   for (i = 0; i < args->arg_count; i++) args->arg_type[i] = STRING_RESULT;
-  initid->maybe_null = true; /* The result may be null */
-  initid->decimals = 2;      /* We want 2 decimals in the result */
-  initid->max_length = 6;    /* 3 digits + . + 2 decimals */
-  return false;
+  initid->maybe_null = 1; /* The result may be null */
+  initid->decimals = 2;   /* We want 2 decimals in the result */
+  initid->max_length = 6; /* 3 digits + . + 2 decimals */
+  return 0;
 }
 
 double myfunc_double(UDF_INIT *, UDF_ARGS *args, unsigned char *is_null,
@@ -78,7 +77,7 @@ double myfunc_double(UDF_INIT *, UDF_ARGS *args, unsigned char *is_null,
   unsigned i, j;
 
   for (i = 0; i < args->arg_count; i++) {
-    if (args->args[i] == nullptr) continue;
+    if (args->args[i] == NULL) continue;
     val += args->lengths[i];
     for (j = args->lengths[i]; j-- > 0;) v += args->args[i][j];
   }
@@ -93,7 +92,7 @@ static mysql_service_status_t init() {
   bool ret_double = false;
   ret_double = mysql_service_udf_registration->udf_register(
       "myfunc_double", REAL_RESULT, (Udf_func_any)myfunc_double,
-      myfunc_double_init, nullptr);
+      myfunc_double_init, NULL);
   return ret_double;
 }
 
