@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,13 +30,10 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include <iomanip>
 #include <set>  // std::set
 #include <sstream>
-#include <unordered_set>
 
 #include "mysql/components/library_mysys/my_memory.h"
 #include "mysql/components/services/mysql_rwlock.h"
-#include "mysql/components/services/psi_memory.h"
 #include "mysqld_error.h"
-#include "scope_guard.h"
 
 #define PSI_NOT_INSTRUMENTED 0
 const int MAX_DICTIONARY_FILE_LENGTH = (1024 * 1024);
@@ -74,15 +70,15 @@ enum password_policy_enum {
   PASSWORD_POLICY_STRONG
 };
 
-static const char *policy_names[] = {"LOW", "MEDIUM", "STRONG", nullptr};
+static const char *policy_names[] = {"LOW", "MEDIUM", "STRONG", NULL};
 
 static TYPE_LIB password_policy_typelib_t = {array_elements(policy_names) - 1,
                                              "password_policy_typelib_t",
-                                             policy_names, nullptr};
+                                             policy_names, NULL};
 
 typedef std::string string_type;
 typedef std::set<string_type> set_type;
-static set_type *dictionary_words{nullptr};
+static set_type dictionary_words;
 
 static int validate_password_length;
 static int validate_password_number_count;
@@ -90,10 +86,9 @@ static int validate_password_mixed_case_count;
 static int validate_password_special_char_count;
 static ulong validate_password_policy;
 static char *validate_password_dictionary_file;
-static char *validate_password_dictionary_file_last_parsed = nullptr;
+static char *validate_password_dictionary_file_last_parsed = NULL;
 static long long validate_password_dictionary_file_words_count = 0;
 static bool check_user_name;
-static int validate_password_changed_characters_percentage = 0;
 /*
   This variable is used, to make sure the use of component services
   after the component load/initialization is done.
@@ -107,7 +102,7 @@ static SHOW_VAR validate_password_status_variables[] = {
     {"validate_password.dictionary_file_words_count",
      (char *)&validate_password_dictionary_file_words_count, SHOW_LONGLONG,
      SHOW_SCOPE_GLOBAL},
-    {nullptr, nullptr, SHOW_LONG, SHOW_SCOPE_GLOBAL}};
+    {NULL, NULL, SHOW_LONG, SHOW_SCOPE_GLOBAL}};
 
 /**
   Activate the new dictionary
@@ -124,7 +119,7 @@ static void dictionary_activate(set_type *dict_words) {
   std::stringstream ss; /* To store date & time in "YYYY-MM-DD HH:MM:SS" */
 
   /* fetch the start time */
-  start_time = time(nullptr);
+  start_time = time(0);
   localtime_r(&start_time, &tm);
 
   ss << std::setfill('0') << std::setw(4) << tm.tm_year + 1900 << "-"
@@ -135,15 +130,15 @@ static void dictionary_activate(set_type *dict_words) {
      << std::setfill('0') << std::setw(2) << tm.tm_sec;
 
   mysql_rwlock_wrlock(&LOCK_dict_file);
-  std::swap(*dictionary_words, *dict_words);
-  validate_password_dictionary_file_words_count = dictionary_words->size();
+  std::swap(dictionary_words, *dict_words);
+  validate_password_dictionary_file_words_count = dictionary_words.size();
   /*
     We are re-using 'validate_password_dictionary_file_last_parsed'
     so, we need to make sure to free the previously allocated memory.
   */
   if (validate_password_dictionary_file_last_parsed) {
     my_free(validate_password_dictionary_file_last_parsed);
-    validate_password_dictionary_file_last_parsed = nullptr;
+    validate_password_dictionary_file_last_parsed = NULL;
   }
   validate_password_dictionary_file_last_parsed =
       (char *)my_malloc(PSI_NOT_INSTRUMENTED, ss.str().length() + 1, MYF(0));
@@ -161,7 +156,7 @@ static void read_dictionary_file() {
   set_type dict_words;
   std::streamoff file_length;
 
-  if (validate_password_dictionary_file == nullptr) {
+  if (validate_password_dictionary_file == NULL) {
     if (validate_password_policy == PASSWORD_POLICY_STRONG)
       LogEvent()
           .type(LOG_TYPE_ERROR)
@@ -208,10 +203,10 @@ static void read_dictionary_file() {
 /* Clear words from std::set */
 static void free_dictionary_file() {
   mysql_rwlock_wrlock(&LOCK_dict_file);
-  if (!dictionary_words->empty()) dictionary_words->clear();
+  if (!dictionary_words.empty()) dictionary_words.clear();
   if (validate_password_dictionary_file_last_parsed) {
     my_free(validate_password_dictionary_file_last_parsed);
-    validate_password_dictionary_file_last_parsed = nullptr;
+    validate_password_dictionary_file_last_parsed = NULL;
   }
   mysql_rwlock_unlock(&LOCK_dict_file);
 }
@@ -225,7 +220,7 @@ static int validate_dictionary_check(my_h_string password) {
   char *buffer;
   my_h_string lower_string_handle;
 
-  if (dictionary_words->empty()) return (1);
+  if (dictionary_words.empty()) return (1);
 
   /* New String is allocated */
   if (mysql_service_mysql_string_factory->create(&lower_string_handle)) {
@@ -248,7 +243,7 @@ static int validate_dictionary_check(my_h_string password) {
     return (0);
 
   if (mysql_service_mysql_string_converter->convert_to_buffer(
-          lower_string_handle, buffer, MAX_PASSWORD_LENGTH, "utf8mb3")) {
+          lower_string_handle, buffer, MAX_PASSWORD_LENGTH, "utf8")) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
         .prio(ERROR_LEVEL)
@@ -272,8 +267,8 @@ static int validate_dictionary_check(my_h_string password) {
     substr_pos = 0;
     while (substr_pos + substr_length <= length) {
       password_substr = password_str.substr(substr_pos, substr_length);
-      itr = dictionary_words->find(password_substr);
-      if (itr != dictionary_words->end()) {
+      itr = dictionary_words.find(password_substr);
+      if (itr != dictionary_words.end()) {
         mysql_rwlock_unlock(&LOCK_dict_file);
         my_free(buffer);
         return (0);
@@ -330,7 +325,7 @@ static bool my_memcmp_reverse(const char *a, size_t a_len, const char *b,
 */
 static bool is_valid_user(Security_context_handle ctx, const char *buffer,
                           int length, const char *field_name) {
-  MYSQL_LEX_CSTRING user = {nullptr, 0};
+  MYSQL_LEX_CSTRING user = {NULL, 0};
 
   if (mysql_service_mysql_security_context_options->get(ctx, field_name,
                                                         &user)) {
@@ -365,7 +360,7 @@ static bool is_valid_user(Security_context_handle ctx, const char *buffer,
 static bool is_valid_password_by_user_name(void *thd, my_h_string password) {
   char buffer[MAX_PASSWORD_LENGTH];
   int length;
-  Security_context_handle ctx = nullptr;
+  Security_context_handle ctx = NULL;
 
   if (!check_user_name) return true;
 
@@ -378,7 +373,7 @@ static bool is_valid_password_by_user_name(void *thd, my_h_string password) {
   }
 
   if (mysql_service_mysql_string_converter->convert_to_buffer(
-          password, buffer, MAX_PASSWORD_LENGTH, "utf8mb3")) {
+          password, buffer, MAX_PASSWORD_LENGTH, "utf8")) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
         .prio(WARNING_LEVEL)
@@ -473,7 +468,7 @@ static int validate_password_policy_strength(void *thd, my_h_string password,
   int has_upper = 0;
   int has_special_chars = 0;
   int n_chars = 0;
-  my_h_string_iterator iter = nullptr;
+  my_h_string_iterator iter = NULL;
   int out_iter_char;
   bool out = false;
 
@@ -485,7 +480,7 @@ static int validate_password_policy_strength(void *thd, my_h_string password,
     return (0);
   }
   while (mysql_service_mysql_string_iterator->iterator_get_next(
-             iter, &out_iter_char) == 0) {
+             iter, &out_iter_char) != true) {
     n_chars++;
     if (policy > PASSWORD_POLICY_LOW) {
       if (!mysql_service_mysql_string_ctype->is_lower(iter, &out) && out)
@@ -533,7 +528,7 @@ DEFINE_BOOL_METHOD(validate_password_imp::get_strength,
   int policy = 0;
   int n_chars = 0;
   int out_iter_char;
-  my_h_string_iterator iter = nullptr;
+  my_h_string_iterator iter = NULL;
 
   *strength = 0;
 
@@ -555,7 +550,7 @@ DEFINE_BOOL_METHOD(validate_password_imp::get_strength,
     return true;
   }
   while (mysql_service_mysql_string_iterator->iterator_get_next(
-             iter, &out_iter_char) == 0)
+             iter, &out_iter_char) != true)
     n_chars++;
 
   mysql_service_mysql_string_iterator->iterator_destroy(iter);
@@ -598,114 +593,6 @@ DEFINE_BOOL_METHOD(validate_password_imp::validate,
                                             validate_password_policy) == 0);
 }
 
-/**
-  Validate if number of changed characters matches the pre-configured
-  criteria
-
-  @param [in]  current_password Current password
-  @param [in]  new_password     New password
-  @param [out] minimum_required Minimum required number of changed characters
-  @param [out] changed          Actual number of changed characters
-
-  @returns Result of validation
-    @retval false Success
-    @retval true  Error
-*/
-DEFINE_BOOL_METHOD(validate_password_changed_characters_imp::validate,
-                   (my_h_string current_password, my_h_string new_password,
-                    uint *minimum_required, uint *changed)) {
-  try {
-    uint current_length = 0, new_length = 0;
-    if (changed) *changed = 0;
-
-    /* quick exit if restriction is not imposed */
-    if (validate_password_changed_characters_percentage == 0) return false;
-
-    /* Convert passwords to lowercase before comparison */
-    my_h_string current_password_lc, new_password_lc;
-    if (mysql_service_mysql_string_factory->create(&current_password_lc) ||
-        mysql_service_mysql_string_factory->create(&new_password_lc)) {
-      LogEvent()
-          .type(LOG_TYPE_ERROR)
-          .prio(ERROR_LEVEL)
-          .lookup(ER_VALIDATE_PWD_STRING_HANDLER_MEM_ALLOCATION_FAILED);
-      return true;
-    }
-
-    auto cleanup_guard = create_scope_guard([&] {
-      mysql_service_mysql_string_factory->destroy(current_password_lc);
-      mysql_service_mysql_string_factory->destroy(new_password_lc);
-    });
-
-    if (mysql_service_mysql_string_case->tolower(&current_password_lc,
-                                                 current_password) ||
-        mysql_service_mysql_string_case->tolower(&new_password_lc,
-                                                 new_password)) {
-      LogEvent()
-          .type(LOG_TYPE_ERROR)
-          .prio(ERROR_LEVEL)
-          .lookup(ER_VALIDATE_PWD_STRING_CONV_TO_LOWERCASE_FAILED);
-      return true;
-    }
-
-    if (mysql_service_mysql_string_character_access->get_char_length(
-            current_password_lc, &current_length) ||
-        mysql_service_mysql_string_character_access->get_char_length(
-            new_password_lc, &new_length)) {
-      return true;
-    }
-
-    /* Determine number of characters required to be changed */
-    uint number_of_characters_to_be_changed =
-        (std::max(static_cast<uint>(validate_password_length), current_length) *
-         (static_cast<uint>(validate_password_changed_characters_percentage)) /
-         100);
-
-    if (minimum_required)
-      *minimum_required = number_of_characters_to_be_changed;
-
-    std::unordered_set<long> characters;
-    auto process_password = [&characters](my_h_string password,
-                                          bool add) -> bool {
-      int pos = 0;
-      ulong character = 0;
-      my_h_string_iterator password_iterator{nullptr};
-
-      if (mysql_service_mysql_string_iterator->iterator_create(
-              password, &password_iterator))
-        return true;
-
-      auto iterator_cleanup = create_scope_guard([&] {
-        mysql_service_mysql_string_iterator->iterator_destroy(
-            password_iterator);
-      });
-
-      while (!mysql_service_mysql_string_iterator->iterator_get_next(
-          password_iterator, &pos)) {
-        if (mysql_service_mysql_string_value->get(password_iterator,
-                                                  &character))
-          return true;
-
-        if (add)
-          (void)characters.insert(character);
-        else
-          (void)characters.erase(character);
-      }
-      return false;
-    };
-
-    if (process_password(new_password_lc, true)) return true;
-
-    if (process_password(current_password_lc, false)) return true;
-
-    if (changed) *changed = characters.size();
-
-    return (characters.size() < number_of_characters_to_be_changed);
-  } catch (...) {
-    return true;
-  }
-}
-
 int register_status_variables() {
   if (mysql_service_status_variable_registration->register_variable(
           (SHOW_VAR *)&validate_password_status_variables)) {
@@ -719,19 +606,15 @@ int register_status_variables() {
 }
 
 int register_system_variables() {
-  INTEGRAL_CHECK_ARG(int)
-  length, num_count, mixed_case_count, spl_char_count,
-      changed_characters_percentage;
-
+  INTEGRAL_CHECK_ARG(int) length, num_count, mixed_case_count, spl_char_count;
   length.def_val = 8;
   length.min_val = 0;
   length.max_val = 0;
   length.blk_sz = 0;
   if (mysql_service_component_sys_variable_register->register_variable(
           "validate_password", "length", PLUGIN_VAR_INT | PLUGIN_VAR_RQCMDARG,
-          "Password validate length to check for minimum password_length",
-          nullptr, length_update, (void *)&length,
-          (void *)&validate_password_length)) {
+          "Password validate length to check for minimum password_length", NULL,
+          length_update, (void *)&length, (void *)&validate_password_length)) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
         .prio(ERROR_LEVEL)
@@ -749,7 +632,7 @@ int register_system_variables() {
           PLUGIN_VAR_INT | PLUGIN_VAR_RQCMDARG,
           "password validate digit to ensure minimum numeric character in "
           "password",
-          nullptr, length_update, (void *)&num_count,
+          NULL, length_update, (void *)&num_count,
           (void *)&validate_password_number_count)) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
@@ -768,7 +651,7 @@ int register_system_variables() {
           PLUGIN_VAR_INT | PLUGIN_VAR_RQCMDARG,
           "Password validate mixed case to ensure minimum upper/lower case "
           "in password",
-          nullptr, length_update, (void *)&mixed_case_count,
+          NULL, length_update, (void *)&mixed_case_count,
           (void *)&validate_password_mixed_case_count)) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
@@ -787,7 +670,7 @@ int register_system_variables() {
           PLUGIN_VAR_INT | PLUGIN_VAR_RQCMDARG,
           "password validate special to ensure minimum special character "
           "in password",
-          nullptr, length_update, (void *)&spl_char_count,
+          NULL, length_update, (void *)&spl_char_count,
           (void *)&validate_password_special_char_count)) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
@@ -804,8 +687,7 @@ int register_system_variables() {
           "validate_password", "policy", PLUGIN_VAR_ENUM | PLUGIN_VAR_RQCMDARG,
           "password_validate_policy choosen policy to validate password "
           "possible values are LOW MEDIUM (default), STRONG",
-          nullptr, nullptr, (void *)&enum_arg,
-          (void *)&validate_password_policy)) {
+          NULL, NULL, (void *)&enum_arg, (void *)&validate_password_policy)) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
         .prio(ERROR_LEVEL)
@@ -815,13 +697,13 @@ int register_system_variables() {
   }
 
   STR_CHECK_ARG(str) str_arg;
-  str_arg.def_val = nullptr;
+  str_arg.def_val = NULL;
   if (mysql_service_component_sys_variable_register->register_variable(
           "validate_password", "dictionary_file",
           PLUGIN_VAR_STR | PLUGIN_VAR_MEMALLOC | PLUGIN_VAR_RQCMDARG,
           "password_validate_dictionary file to be loaded and check for "
           "password",
-          nullptr, dictionary_update, (void *)&str_arg,
+          NULL, dictionary_update, (void *)&str_arg,
           (void *)&validate_password_dictionary_file)) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
@@ -838,7 +720,7 @@ int register_system_variables() {
           PLUGIN_VAR_BOOL | PLUGIN_VAR_RQCMDARG,
           "Check if the password matches the login or the effective "
           "user names or the reverse of them",
-          nullptr, nullptr, (void *)&bool_arg, (void *)&check_user_name)) {
+          NULL, NULL, (void *)&bool_arg, (void *)&check_user_name)) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
         .prio(ERROR_LEVEL)
@@ -846,30 +728,7 @@ int register_system_variables() {
                 "validate_password.check_user_name");
     goto check_user_name;
   }
-
-  changed_characters_percentage.def_val = 0;
-  changed_characters_percentage.min_val = 0;
-  changed_characters_percentage.max_val = 100;
-  changed_characters_percentage.blk_sz = 0;
-  if (mysql_service_component_sys_variable_register->register_variable(
-          "validate_password", "changed_characters_percentage",
-          PLUGIN_VAR_INT | PLUGIN_VAR_RQCMDARG,
-          "password validate percentage of changed characters required in new "
-          "password. Valid values between 0 and 100.",
-          nullptr, length_update, (void *)&changed_characters_percentage,
-          (void *)&validate_password_changed_characters_percentage)) {
-    LogEvent()
-        .type(LOG_TYPE_ERROR)
-        .prio(ERROR_LEVEL)
-        .lookup(ER_VALIDATE_PWD_VARIABLE_REGISTRATION_FAILED,
-                "validate_password.changed_characters_percentage");
-    goto changed_characters_percentage;
-  }
   return 0; /* All system variables registered successfully */
-
-changed_characters_percentage:
-  mysql_service_component_sys_variable_unregister->unregister_variable(
-      "validate_password", "check_user_name");
 check_user_name:
   mysql_service_component_sys_variable_unregister->unregister_variable(
       "validate_password", "dictionary_file");
@@ -966,15 +825,6 @@ int unregister_system_variables() {
         .lookup(ER_VALIDATE_PWD_VARIABLE_UNREGISTRATION_FAILED,
                 "validate_password.check_user_name");
   }
-
-  if (mysql_service_component_sys_variable_unregister->unregister_variable(
-          "validate_password", "changed_characters_percentage")) {
-    LogEvent()
-        .type(LOG_TYPE_ERROR)
-        .prio(ERROR_LEVEL)
-        .lookup(ER_VALIDATE_PWD_VARIABLE_UNREGISTRATION_FAILED,
-                "validate_password.changed_characters_percentage");
-  }
   return 0;
 }
 
@@ -1014,30 +864,11 @@ bool log_service_deinit() { return false; }
   @retval true failure
 */
 static mysql_service_status_t validate_password_init() {
-  dictionary_words = new set_type();
   init_validate_password_psi_keys();
   mysql_rwlock_init(key_validate_password_LOCK_dict_file, &LOCK_dict_file);
-  if (log_service_init()) {
-    delete dictionary_words;
-    dictionary_words = nullptr;
-    mysql_rwlock_destroy(&LOCK_dict_file);
+  if (log_service_init() || register_system_variables() ||
+      register_status_variables())
     return true;
-  }
-  if (register_system_variables()) {
-    log_service_deinit();
-    delete dictionary_words;
-    dictionary_words = nullptr;
-    mysql_rwlock_destroy(&LOCK_dict_file);
-    return true;
-  }
-  if (register_status_variables()) {
-    unregister_system_variables();
-    log_service_deinit();
-    delete dictionary_words;
-    dictionary_words = nullptr;
-    mysql_rwlock_destroy(&LOCK_dict_file);
-    return true;
-  }
   read_dictionary_file();
   /* Check if validate_password_length needs readjustment */
   readjust_validate_password_length();
@@ -1055,8 +886,6 @@ static mysql_service_status_t validate_password_init() {
 static mysql_service_status_t validate_password_deinit() {
   free_dictionary_file();
   mysql_rwlock_destroy(&LOCK_dict_file);
-  delete dictionary_words;
-  dictionary_words = nullptr;
   if (unregister_system_variables() || unregister_status_variables() ||
       log_service_deinit())
     return true;
@@ -1068,14 +897,9 @@ BEGIN_SERVICE_IMPLEMENTATION(validate_password, validate_password)
 validate_password_imp::validate,
     validate_password_imp::get_strength END_SERVICE_IMPLEMENTATION();
 
-BEGIN_SERVICE_IMPLEMENTATION(validate_password,
-                             validate_password_changed_characters)
-validate_password_changed_characters_imp::validate END_SERVICE_IMPLEMENTATION();
-
 /* component provides: the validate_password service */
 BEGIN_COMPONENT_PROVIDES(validate_password)
 PROVIDES_SERVICE(validate_password, validate_password),
-    PROVIDES_SERVICE(validate_password, validate_password_changed_characters),
     END_COMPONENT_PROVIDES();
 
 /* A block for specifying dependencies of this Component. Note that for each
@@ -1083,32 +907,28 @@ PROVIDES_SERVICE(validate_password, validate_password),
   header file of the Component, and an entry on requires list below. */
 REQUIRES_SERVICE_PLACEHOLDER(log_builtins);
 REQUIRES_SERVICE_PLACEHOLDER(log_builtins_string);
-REQUIRES_SERVICE_PLACEHOLDER(mysql_string_character_access);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_string_factory);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_string_case);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_string_converter);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_string_iterator);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_string_ctype);
-REQUIRES_SERVICE_PLACEHOLDER(mysql_string_value);
 REQUIRES_SERVICE_PLACEHOLDER(component_sys_variable_register);
 REQUIRES_SERVICE_PLACEHOLDER(component_sys_variable_unregister);
 REQUIRES_SERVICE_PLACEHOLDER(status_variable_registration);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_thd_security_context);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_security_context_options);
-REQUIRES_PSI_MEMORY_SERVICE_PLACEHOLDER;
-REQUIRES_MYSQL_RWLOCK_SERVICE_PLACEHOLDER;
 
 /* A list of dependencies.
    The dynamic_loader fetches the references for the below services at the
    component load time and disposes off them at unload.
 */
 BEGIN_COMPONENT_REQUIRES(validate_password)
-REQUIRES_SERVICE(log_builtins), REQUIRES_SERVICE(log_builtins_string),
-    REQUIRES_SERVICE(mysql_string_character_access),
+REQUIRES_SERVICE(registry), REQUIRES_SERVICE(log_builtins),
+    REQUIRES_SERVICE(log_builtins_string),
     REQUIRES_SERVICE(mysql_string_factory), REQUIRES_SERVICE(mysql_string_case),
     REQUIRES_SERVICE(mysql_string_converter),
     REQUIRES_SERVICE(mysql_string_iterator),
-    REQUIRES_SERVICE(mysql_string_ctype), REQUIRES_SERVICE(mysql_string_value),
+    REQUIRES_SERVICE(mysql_string_ctype),
     REQUIRES_SERVICE(component_sys_variable_register),
     REQUIRES_SERVICE(component_sys_variable_unregister),
     REQUIRES_SERVICE(status_variable_registration),
