@@ -2,8 +2,9 @@
 
 // Copyright (c) 2007-2015 Barend Gehrels, Amsterdam, the Netherlands.
 
-// This file was modified by Oracle on 2013-2021.
-// Modifications Copyright (c) 2013, 2025, Oracle and/or its affiliates.
+// This file was modified by Oracle on 2013, 2014, 2015, 2017.
+// Modifications copyright (c) 2013-2017 Oracle and/or its affiliates.
+
 // Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 // Contributed and/or modified by Menelaos Karavelas, on behalf of Oracle
 
@@ -14,18 +15,15 @@
 #ifndef BOOST_GEOMETRY_ALGORITHMS_DETAIL_RELATE_TURNS_HPP
 #define BOOST_GEOMETRY_ALGORITHMS_DETAIL_RELATE_TURNS_HPP
 
-
+#include <boost/geometry/strategies/distance.hpp>
 #include <boost/geometry/algorithms/detail/overlay/do_reverse.hpp>
 #include <boost/geometry/algorithms/detail/overlay/get_turns.hpp>
 #include <boost/geometry/algorithms/detail/overlay/get_turn_info.hpp>
 
 #include <boost/geometry/policies/robustness/get_rescale_policy.hpp>
 #include <boost/geometry/policies/robustness/no_rescale_policy.hpp>
-#include <boost/geometry/policies/robustness/segment_ratio_type.hpp>
 
-#include <boost/geometry/strategies/cartesian/point_in_point.hpp>
-#include <boost/geometry/strategies/spherical/point_in_point.hpp>
-#include <boost/geometry/strategies/distance.hpp>
+#include <boost/type_traits/is_base_of.hpp>
 
 
 namespace boost { namespace geometry {
@@ -49,65 +47,70 @@ template
     typename GetTurnPolicy = detail::get_turns::get_turn_info_type
         <
             Geometry1, Geometry2, assign_policy<>
-        >
+        >,
+    typename RobustPolicy = detail::no_rescale_policy
+    /*
+    typename RobustPolicy = typename geometry::rescale_overlay_policy_type
+                                <
+                                    Geometry1,
+                                    Geometry2
+                                >::type
+    */
 >
 struct get_turns
 {
     typedef typename geometry::point_type<Geometry1>::type point1_type;
 
-    template <typename Strategy>
-    struct robust_policy_type
-        /*: geometry::rescale_overlay_policy_type
-            <
-                Geometry1,
-                Geometry2,
-                typename Strategy::cs_tag
-            >*/
-    {
-        typedef detail::no_rescale_policy type;
-    };
-
-    template
-    <
-        typename Strategy,
-        typename RobustPolicy = typename robust_policy_type<Strategy>::type
-    >
-    struct turn_info_type
-    {
-        typedef typename segment_ratio_type<point1_type, RobustPolicy>::type ratio_type;
-        typedef overlay::turn_info
+    typedef overlay::turn_info
             <
                 point1_type,
-                ratio_type,
+                typename segment_ratio_type<point1_type, RobustPolicy>::type,
                 typename detail::get_turns::turn_operation_type
                     <
-                        Geometry1, Geometry2, ratio_type
+                        Geometry1, Geometry2,
+                        typename segment_ratio_type
+                            <
+                                point1_type, RobustPolicy
+                            >::type
                     >::type
-            > type;
-    };
+            > turn_info;
 
-    template <typename Turns, typename InterruptPolicy, typename Strategy>
+    template <typename Turns>
     static inline void apply(Turns & turns,
                              Geometry1 const& geometry1,
-                             Geometry2 const& geometry2,
-                             InterruptPolicy & interrupt_policy,
-                             Strategy const& strategy)
+                             Geometry2 const& geometry2)
     {
-        typedef typename robust_policy_type<Strategy>::type robust_policy_t;
+        detail::get_turns::no_interrupt_policy interrupt_policy;
 
-        robust_policy_t robust_policy
-                = geometry::get_rescale_policy<robust_policy_t>(
-                    geometry1, geometry2, strategy);
+        typename strategy::intersection::services::default_strategy
+            <
+                typename cs_tag<Geometry1>::type
+            >::type intersection_strategy;
 
-        apply(turns, geometry1, geometry2, interrupt_policy, strategy, robust_policy);
+        apply(turns, geometry1, geometry2, interrupt_policy, intersection_strategy);
     }
 
-    template <typename Turns, typename InterruptPolicy, typename Strategy, typename RobustPolicy>
+    template <typename Turns, typename InterruptPolicy, typename IntersectionStrategy>
     static inline void apply(Turns & turns,
                              Geometry1 const& geometry1,
                              Geometry2 const& geometry2,
                              InterruptPolicy & interrupt_policy,
-                             Strategy const& strategy,
+                             IntersectionStrategy const& intersection_strategy)
+    {
+        RobustPolicy robust_policy = geometry::get_rescale_policy
+            <
+                RobustPolicy
+            >(geometry1, geometry2);
+
+        apply(turns, geometry1, geometry2, interrupt_policy, intersection_strategy, robust_policy);
+    }
+
+    template <typename Turns, typename InterruptPolicy, typename IntersectionStrategy>
+    static inline void apply(Turns & turns,
+                             Geometry1 const& geometry1,
+                             Geometry2 const& geometry2,
+                             InterruptPolicy & interrupt_policy,
+                             IntersectionStrategy const& intersection_strategy,
                              RobustPolicy const& robust_policy)
     {
         static const bool reverse1 = detail::overlay::do_reverse
@@ -130,7 +133,7 @@ struct get_turns
                 reverse2,
                 GetTurnPolicy
             >::apply(0, geometry1, 1, geometry2,
-                     strategy, robust_policy,
+                     intersection_strategy, robust_policy,
                      turns, interrupt_policy);
     }
 };
@@ -271,8 +274,9 @@ struct less_other_multi_index
 };
 
 // sort turns by G1 - source_index == 0 by:
-// seg_id -> distance and coordinates -> operation
-template <std::size_t OpId, typename LessOp, typename CSTag>
+// seg_id -> distance -> operation
+template <std::size_t OpId = 0,
+          typename LessOp = less_op_xxx_linear< OpId, op_to_int<> > >
 struct less
 {
     BOOST_STATIC_ASSERT(OpId < 2);
@@ -280,27 +284,16 @@ struct less
     template <typename Turn>
     static inline bool use_fraction(Turn const& left, Turn const& right)
     {
-        typedef typename geometry::strategy::within::services::default_strategy
-            <
-                typename Turn::point_type, typename Turn::point_type,
-                point_tag, point_tag,
-                pointlike_tag, pointlike_tag,
-                typename tag_cast<CSTag, spherical_tag>::type,
-                typename tag_cast<CSTag, spherical_tag>::type
-            >::type eq_pp_strategy_type;
-
         static LessOp less_op;
 
-        // NOTE: Assuming fraction is more permissive and faster than
-        //       comparison of points with strategy.
-        return geometry::math::equals(left.operations[OpId].fraction,
-                                      right.operations[OpId].fraction)
-                && eq_pp_strategy_type::apply(left.point, right.point)
-             ?
-             less_op(left, right)
-             :
-             (left.operations[OpId].fraction < right.operations[OpId].fraction)
-             ;
+        return
+            geometry::math::equals(left.operations[OpId].fraction,
+                                   right.operations[OpId].fraction)
+            ?
+            less_op(left, right)
+            :
+            (left.operations[OpId].fraction < right.operations[OpId].fraction)
+            ;
     }
 
     template <typename Turn>
