@@ -2,8 +2,8 @@
 
 // Copyright (c) 2013 Barend Gehrels, Amsterdam, the Netherlands.
 
-// This file was modified by Oracle on 2016-2021.
-// Modifications Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+// This file was modified by Oracle on 2016-2018.
+// Modifications copyright (c) 2016, 2018 Oracle and/or its affiliates.
 // Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 
 // Use, modification and distribution is subject to the Boost Software License,
@@ -13,8 +13,6 @@
 #ifndef BOOST_GEOMETRY_POLICIES_ROBUSTNESS_SEGMENT_RATIO_HPP
 #define BOOST_GEOMETRY_POLICIES_ROBUSTNESS_SEGMENT_RATIO_HPP
 
-#include <type_traits>
-
 #include <boost/config.hpp>
 #include <boost/rational.hpp>
 
@@ -22,7 +20,7 @@
 #include <boost/geometry/util/math.hpp>
 #include <boost/geometry/util/promote_floating_point.hpp>
 
-#include <boost/geometry/policies/robustness/robust_type.hpp>
+#include <boost/multiprecision/cpp_int.hpp>
 
 namespace boost { namespace geometry
 {
@@ -31,21 +29,11 @@ namespace boost { namespace geometry
 namespace detail { namespace segment_ratio
 {
 
-
-// It seems boost::is_integral is not specialized for boost::multiprecision::number
-template <typename Type>
-struct use_rational
-{
-    static const bool value = (std::is_integral<Type>::type::value
-                                && std::numeric_limits<Type>::is_specialized)
-                            || std::is_same<Type, robust_signed_integral_type>::value;
-};
-
-
 template
 <
     typename Type,
-    bool UseRational = use_rational<Type>::value
+    bool UseRational = boost::is_integral<Type>::type::value
+                    && std::numeric_limits<Type>::is_specialized
 >
 struct less {};
 
@@ -68,17 +56,16 @@ struct less<Type, false>
     {
         BOOST_GEOMETRY_ASSERT(lhs.denominator() != 0);
         BOOST_GEOMETRY_ASSERT(rhs.denominator() != 0);
-        Type const a = lhs.numerator() / lhs.denominator();
-        Type const b = rhs.numerator() / rhs.denominator();
-        return ! geometry::math::equals(a, b)
-            && a < b;
+        return lhs.numerator() * rhs.denominator()
+             < rhs.numerator() * lhs.denominator();
     }
 };
 
 template
 <
     typename Type,
-    bool UseRational = use_rational<Type>::value
+    bool UseRational = boost::is_integral<Type>::type::value
+                    && std::numeric_limits<Type>::is_specialized
 >
 struct equal {};
 
@@ -101,12 +88,15 @@ struct equal<Type, false>
     {
         BOOST_GEOMETRY_ASSERT(lhs.denominator() != 0);
         BOOST_GEOMETRY_ASSERT(rhs.denominator() != 0);
-        Type const a = lhs.numerator() / lhs.denominator();
-        Type const b = rhs.numerator() / rhs.denominator();
-        return geometry::math::equals(a, b);
+        return geometry::math::equals
+            (
+                lhs.numerator() * rhs.denominator(),
+                rhs.numerator() * lhs.denominator()
+            );
     }
 };
 
+
 template <typename T, typename F>
 struct mul_fp_impl
 {
@@ -117,7 +107,7 @@ struct mul_fp_impl
     }
 };
 
-
+#if !defined(BOOST_HAS_INT128) || defined(BOOST_GEOMETRY_DISABLE_INT128_TYPE)
 template <typename Backend, typename F>
 struct mul_fp_impl<boost::multiprecision::number<Backend>, F>
 {
@@ -129,35 +119,8 @@ struct mul_fp_impl<boost::multiprecision::number<Backend>, F>
                 / denominator.template convert_to<F>();
     }
 };
+#endif
 
-template
-<
-    typename Type,
-    bool IsFloatingPoint = std::is_floating_point<Type>::type::value
->
-struct possibly_collinear {};
-
-template <typename Type>
-struct possibly_collinear<Type, true>
-{
-    template <typename Ratio, typename Threshold>
-    static inline bool apply(Ratio const& ratio, Threshold threshold)
-    {
-        return std::abs(ratio.denominator()) < threshold;
-    }
-};
-
-// Any ratio based on non-floating point (or user defined floating point)
-// is collinear if the denominator is exactly zero
-template <typename Type>
-struct possibly_collinear<Type, false>
-{
-    template <typename Ratio, typename Threshold>
-    static inline bool apply(Ratio const& ratio, Threshold)
-    {
-        return ratio.denominator() == 0;
-    }
-};
 
 }}
 
@@ -183,63 +146,19 @@ public :
         , m_approximation(0)
     {}
 
-    inline segment_ratio(const Type& numerator, const Type& denominator)
-        : m_numerator(numerator)
+    inline segment_ratio(const Type& nominator, const Type& denominator)
+        : m_numerator(nominator)
         , m_denominator(denominator)
     {
         initialize();
     }
 
-    segment_ratio(segment_ratio const&) = default;
-    segment_ratio& operator=(segment_ratio const&) = default;
-    segment_ratio(segment_ratio&&) = default;
-    segment_ratio& operator=(segment_ratio&&) = default;
-    
-    // These are needed because in intersection strategies ratios are assigned
-    // in fractions and if a user passes CalculationType then ratio Type in
-    // turns is taken from geometry coordinate_type and the one used in
-    // a strategy uses Type selected using CalculationType.
-    // See: detail::overlay::intersection_info_base
-    // and  policies::relate::segments_intersection_points
-    //      in particular segments_collinear() where ratios are assigned.
-    template<typename T> friend class segment_ratio;
-    template <typename T>
-    segment_ratio(segment_ratio<T> const& r)
-        : m_numerator(r.m_numerator)
-        , m_denominator(r.m_denominator)
-    {
-        initialize();
-    }
-    template <typename T>
-    segment_ratio& operator=(segment_ratio<T> const& r)
-    {
-        m_numerator = r.m_numerator;
-        m_denominator = r.m_denominator;
-        initialize();
-        return *this;
-    }
-    template <typename T>
-    segment_ratio(segment_ratio<T> && r)
-        : m_numerator(std::move(r.m_numerator))
-        , m_denominator(std::move(r.m_denominator))
-    {
-        initialize();
-    }
-    template <typename T>
-    segment_ratio& operator=(segment_ratio<T> && r)
-    {
-        m_numerator = std::move(r.m_numerator);
-        m_denominator = std::move(r.m_denominator);
-        initialize();
-        return *this;
-    }
-
     inline Type const& numerator() const { return m_numerator; }
     inline Type const& denominator() const { return m_denominator; }
 
-    inline void assign(const Type& numerator, const Type& denominator)
+    inline void assign(const Type& nominator, const Type& denominator)
     {
-        m_numerator = numerator;
+        m_numerator = nominator;
         m_denominator = denominator;
         initialize();
     }
@@ -304,12 +223,6 @@ public :
         return geometry::math::abs(m_approximation - other.m_approximation) < 50;
     }
 
-    template <typename Threshold>
-    inline bool possibly_collinear(Threshold threshold) const
-    {
-        return detail::segment_ratio::possibly_collinear<Type>::apply(*this, threshold);
-    }
-
     inline bool operator< (thistype const& other) const
     {
         return close_to(other)
@@ -361,12 +274,10 @@ private :
     // if Type is non-fundamental type
     //typedef typename promote_floating_point<Type>::type fp_type;
 
-    // TODO: What with user-defined numeric types?
-    //       Shouldn't here is_integral be checked?
-    typedef std::conditional_t
+    typedef typename boost::mpl::if_c
         <
-            std::is_floating_point<Type>::value, Type, double
-        > fp_type;
+            boost::is_float<Type>::value, Type, double
+        >::type fp_type;
 
     Type m_numerator;
     Type m_denominator;
