@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,8 +21,8 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#ifndef M_STRING_INCLUDED
-#define M_STRING_INCLUDED
+#ifndef _m_string_h
+#define _m_string_h
 
 /**
   @file include/m_string.h
@@ -37,8 +36,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <algorithm>
-
 #include "decimal.h"
 #include "lex_string.h"
 #include "my_config.h"
@@ -61,9 +58,9 @@ extern void *(*my_str_malloc)(size_t);
 extern void *(*my_str_realloc)(void *, size_t);
 extern void (*my_str_free)(void *);
 
-/* Declared in int2str.cc. */
-extern const char _dig_vec_upper[];
-extern const char _dig_vec_lower[];
+/* Declared in int2str() */
+extern char _dig_vec_upper[];
+extern char _dig_vec_lower[];
 
 /* Prototypes for string functions */
 
@@ -90,14 +87,14 @@ static inline void bchange(uchar *dst, size_t old_length, const uchar *src,
   be used instead, but this is clearer and faster.
 */
 static inline const char *strend(const char *s) {
-  while (*s++) {
-  }
+  while (*s++)
+    ;
   return s - 1;
 }
 
 static inline char *strend(char *s) {
-  while (*s++) {
-  }
+  while (*s++)
+    ;
   return s - 1;
 }
 
@@ -114,7 +111,7 @@ static inline const char *strcend(const char *s, char c) {
 
 /*
   strfill(dest, len, fill) makes a string of fill-characters. The result
-  string is of length == len. The des+len character is always set to NULL.
+  string is of length == len. The des+len character is allways set to NULL.
   strfill() returns pointer to dest+len;
 */
 static inline char *strfill(char *s, size_t len, char fill) {
@@ -131,8 +128,8 @@ static inline char *strfill(char *s, size_t len, char fill) {
   into dst, which seems useful.
 */
 static inline char *my_stpmov(char *dst, const char *src) {
-  while ((*dst++ = *src++)) {
-  }
+  while ((*dst++ = *src++))
+    ;
   return dst - 1;
 }
 
@@ -164,13 +161,6 @@ static inline char *my_stpnmov(char *dst, const char *src, size_t n) {
 */
 static inline char *my_stpcpy(char *dst, const char *src) {
 #if defined(HAVE_BUILTIN_STPCPY)
-  /*
-    If __builtin_stpcpy() is available, use it instead of stpcpy(), since GCC in
-    some situations is able to transform __builtin_stpcpy() into more efficient
-    strcpy() or memcpy() calls. It does not perform these transformations for a
-    plain call to stpcpy() when the compiler runs in strict mode. See GCC bug
-    82429.
-  */
   return __builtin_stpcpy(dst, src);
 #elif defined(HAVE_STPCPY)
   return stpcpy(dst, src);
@@ -257,6 +247,7 @@ static inline int is_prefix(const char *s, const char *t) {
 typedef enum { MY_GCVT_ARG_FLOAT, MY_GCVT_ARG_DOUBLE } my_gcvt_arg_type;
 
 double my_strtod(const char *str, const char **end, int *error);
+double my_atof(const char *nptr);
 size_t my_fcvt(double x, int precision, char *to, bool *error);
 size_t my_fcvt_compact(double x, char *to, bool *error);
 size_t my_gcvt(double x, my_gcvt_arg_type type, int width, char *to,
@@ -286,17 +277,26 @@ static constexpr int FLOATING_POINT_BUFFER{311 + DECIMAL_NOT_SPECIFIED};
   MAX_DECPT_FOR_F_FORMAT zeros for cases when |x|<1 and the 'f' format is used).
 */
 #define MY_GCVT_MAX_FIELD_WIDTH \
-  (DBL_DIG + 4 + std::max(5, MAX_DECPT_FOR_F_FORMAT))
+  (DBL_DIG + 4 + MY_MAX(5, MAX_DECPT_FOR_F_FORMAT))
 
-const char *str2int(const char *src, int radix, long lower, long upper,
-                    long *val);
+extern char *int2str(long val, char *dst, int radix, int upcase);
+C_MODE_START
+extern char *int10_to_str(long val, char *dst, int radix);
+C_MODE_END
+extern const char *str2int(const char *src, int radix, long lower, long upper,
+                           long *val);
 longlong my_strtoll10(const char *nptr, const char **endptr, int *error);
-char *ll2str(int64_t val, char *dst, int radix, bool upcase);
-char *longlong10_to_str(int64_t val, char *dst, int radix);
-
-inline char *longlong2str(int64_t val, char *dst, int radix) {
-  return ll2str(val, dst, radix, true);
-}
+#if SIZEOF_LONG == SIZEOF_LONG_LONG
+#define ll2str(A, B, C, D) int2str((A), (B), (C), (D))
+#define longlong10_to_str(A, B, C) int10_to_str((A), (B), (C))
+#undef strtoll
+#define strtoll(A, B, C) strtol((A), (B), (C))
+#define strtoull(A, B, C) strtoul((A), (B), (C))
+#else
+extern char *ll2str(longlong val, char *dst, int radix, int upcase);
+extern char *longlong10_to_str(longlong val, char *dst, int radix);
+#endif
+#define longlong2str(A, B, C) ll2str((A), (B), (C), 1)
 
 /*
   This function saves a longlong value in a buffer and returns the pointer to
@@ -351,9 +351,7 @@ static inline void human_readable_num_bytes(char *buf, int buf_len,
   for (i = 0; dbl_val > 1024 && i < sizeof(size) - 1; i++) dbl_val /= 1024;
   const char mult = size[i];
   // 18446744073709551615 Yottabytes should be enough for most ...
-  // ULLONG_MAX is not exactly representable as a double. This is the largest
-  // double that is still below ULLONG_MAX.
-  if (dbl_val > 18446744073709549568.0)
+  if (dbl_val > ULLONG_MAX)
     snprintf(buf, buf_len, "+INF");
   else
     snprintf(buf, buf_len, "%llu%c", (unsigned long long)dbl_val, mult);
@@ -369,4 +367,4 @@ static inline void lex_cstring_set(LEX_CSTRING *lex_str, const char *c_str) {
   lex_str->length = strlen(c_str);
 }
 
-#endif  // M_STRING_INCLUDED
+#endif
