@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -51,25 +51,20 @@
 #include <sys/types.h>
 #include <time.h>
 
-#include <atomic>  // error_handler_hook
-
 #include "m_string.h" /* IWYU pragma: keep */
 #include "my_compiler.h"
 #include "my_compress.h"
 #include "my_inttypes.h"
 #include "my_loglevel.h"
-
-/* HAVE_PSI_*_INTERFACE */
 #include "my_psi_config.h" /* IWYU pragma: keep */
-
 #include "my_sharedlib.h"
-#include "mysql/components/services/bits/my_io_bits.h"
-#include "mysql/components/services/bits/mysql_cond_bits.h"
-#include "mysql/components/services/bits/mysql_mutex_bits.h"
-#include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/components/services/bits/psi_file_bits.h"
-#include "mysql/components/services/bits/psi_memory_bits.h"
-#include "mysql/components/services/bits/psi_stage_bits.h"
+#include "mysql/components/services/my_io_bits.h"
+#include "mysql/components/services/mysql_cond_bits.h"
+#include "mysql/components/services/mysql_mutex_bits.h"
+#include "mysql/components/services/psi_file_bits.h"
+#include "mysql/components/services/psi_memory_bits.h"
+#include "mysql/components/services/psi_stage_bits.h"
+#include "mysql/psi/psi_base.h"
 #include "sql/stream_cipher.h"
 
 struct CHARSET_INFO;
@@ -91,7 +86,6 @@ struct PSI_system_bootstrap;
 struct PSI_table_bootstrap;
 struct PSI_thread_bootstrap;
 struct PSI_transaction_bootstrap;
-struct PSI_tls_channel_bootstrap;
 struct MEM_ROOT;
 
 #define MY_INIT(name)   \
@@ -115,18 +109,19 @@ struct MEM_ROOT;
 #define MY_FILE_ERROR ((size_t)-1)
 
 /* General bitmaps for my_func's */
-// 1 used to be MY_FFNF which has been removed
-#define MY_FNABP 2         /* Fatal if not all bytes read/written */
-#define MY_NABP 4          /* Error if not all bytes read/written */
+#define MY_FFNF 1          /* Fatal if file not found */
+#define MY_FNABP 2         /* Fatal if not all bytes read/writen */
+#define MY_NABP 4          /* Error if not all bytes read/writen */
 #define MY_FAE 8           /* Fatal if any error */
 #define MY_WME 16          /* Write message on error */
 #define MY_WAIT_IF_FULL 32 /* Wait and try again if disk full error */
 #define MY_IGNORE_BADFD 32 /* my_sync: ignore 'bad descriptor' errors */
+#define MY_SYNC_DIR 8192   /* my_create/delete/rename: sync directory */
 #define MY_REPORT_WAITING_IF_FULL 64 /* my_write: set status as waiting */
 #define MY_FULL_IO 512 /* For my_read - loop intil I/O is complete */
 #define MY_DONT_CHECK_FILESIZE 128  /* Option to init_io_cache() */
 #define MY_LINK_WARNING 32          /* my_redel() gives warning if links */
-#define MY_COPYTIME 64              /* my_redel() copies time */
+#define MY_COPYTIME 64              /* my_redel() copys time */
 #define MY_DELETE_OLD 256           /* my_create_with_symlink() */
 #define MY_RESOLVE_LINK 128         /* my_realpath(); Only resolve links */
 #define MY_HOLD_ORIGINAL_MODES 128  /* my_copy() holds to file modes */
@@ -137,6 +132,7 @@ struct MEM_ROOT;
 #define MY_FREE_ON_ERROR 128        /* my_realloc() ; Free old ptr on error */
 #define MY_HOLD_ON_ERROR 256        /* my_realloc() ; Return old ptr on error */
 #define MY_DONT_OVERWRITE_FILE 1024 /* my_copy: Don't overwrite file */
+#define MY_THREADSAFE 2048          /* my_seek(): lock fd mutex */
 #define MY_SYNC 4096                /* my_copy(): sync dst file */
 
 #define MYF_RW MYF(MY_WME + MY_NABP) /* For my_read & my_write */
@@ -172,6 +168,10 @@ struct MEM_ROOT;
 #define MIN_COMPRESS_LENGTH 50           /* Don't compress small bl. */
 #define DFLT_INIT_HITS 3
 
+/* root_alloc flags */
+#define MY_KEEP_PREALLOC 1
+#define MY_MARK_BLOCKS_FREE 2 /* move used to free list and reuse them */
+
 /* Internal error numbers (for assembler functions) */
 #define MY_ERRNO_EDOM 33
 #define MY_ERRNO_ERANGE 34
@@ -191,9 +191,7 @@ extern PSI_memory_key key_memory_max_alloca;
   if (size > max_alloca_sz) my_free(ptr)
 
 #if defined(ENABLED_DEBUG_SYNC)
-using DebugSyncCallbackFp = void (*)(const char *, size_t);
-extern DebugSyncCallbackFp debug_sync_C_callback_ptr;
-
+extern "C" void (*debug_sync_C_callback_ptr)(const char *, size_t);
 #define DEBUG_SYNC_C(_sync_point_name_)                                 \
   do {                                                                  \
     if (debug_sync_C_callback_ptr != NULL)                              \
@@ -219,11 +217,11 @@ extern uint my_get_large_page_size(void);
 
 extern char *home_dir;          /* Home directory for user */
 extern const char *my_progname; /* program-name (printed in errors) */
-
-using ErrorHandlerFunctionPointer = void (*)(uint, const char *, myf);
-extern std::atomic<ErrorHandlerFunctionPointer> error_handler_hook;
+extern void (*error_handler_hook)(uint my_err, const char *str, myf MyFlags);
+extern void (*fatal_error_handler_hook)(uint my_err, const char *str,
+                                        myf MyFlags);
 extern void (*local_message_hook)(enum loglevel ll, uint ecode, va_list args);
-
+extern uint my_file_limit;
 extern MYSQL_PLUGIN_IMPORT ulong my_thread_stack_size;
 
 /*
@@ -264,9 +262,7 @@ extern MYSQL_PLUGIN_IMPORT CHARSET_INFO *all_charsets[MY_ALL_CHARSETS_SIZE];
 extern CHARSET_INFO compiled_charsets[];
 
 /* statistics */
-extern ulong my_tmp_file_created;
-extern ulong my_file_opened;
-extern ulong my_stream_opened;
+extern ulong my_file_opened, my_stream_opened, my_tmp_file_created;
 extern ulong my_file_total_opened;
 extern bool my_init_done;
 
@@ -299,6 +295,31 @@ enum flush_type {
   FLUSH_FORCE_WRITE
 };
 
+/*
+ How was this file opened (for debugging purposes).
+ The important part is whether it is UNOPEN or not.
+ */
+enum file_type {
+  UNOPEN = 0,
+  FILE_BY_OPEN,
+  FILE_BY_CREATE,
+  STREAM_BY_FOPEN,
+  STREAM_BY_FDOPEN,
+  FILE_BY_MKSTEMP,
+  FILE_BY_O_TMPFILE
+};
+
+struct st_my_file_info {
+  char *name;
+#ifdef _WIN32
+  HANDLE fhandle; /* win32 file handle */
+  int oflag;      /* open flags, e.g O_APPEND */
+#endif
+  enum file_type type;
+};
+
+extern struct st_my_file_info *my_file_info;
+
 struct DYNAMIC_ARRAY {
   uchar *buffer{nullptr};
   uint elements{0}, max_element{0};
@@ -315,7 +336,7 @@ struct MY_TMPDIR {
 
 struct DYNAMIC_STRING {
   char *str;
-  size_t length, max_length;
+  size_t length, max_length, alloc_increment;
 };
 
 struct IO_CACHE;
@@ -336,7 +357,7 @@ struct IO_CACHE_SHARE {
   int error;           /* Last error. */
 };
 
-struct IO_CACHE /* Used when caching files */
+struct IO_CACHE /* Used when cacheing files */
 {
   /* Offset in file corresponding to the first byte of uchar* buffer. */
   my_off_t pos_in_file{0};
@@ -406,7 +427,7 @@ struct IO_CACHE /* Used when caching files */
   int (*write_function)(IO_CACHE *, const uchar *, size_t){nullptr};
   /*
     Specifies the type of the cache. Depending on the type of the cache
-    certain operations might not be available and yield unpredictable
+    certain operations might not be available and yield unpredicatable
     results. Details to be documented later
   */
   cache_type type{TYPE_NOT_SET};
@@ -434,7 +455,7 @@ struct IO_CACHE /* Used when caching files */
 
   /*
     seek_not_done is set by my_b_seek() to inform the upcoming read/write
-    operation that a seek needs to be performed prior to the actual I/O
+    operation that a seek needs to be preformed prior to the actual I/O
     error is 0 if the cache operation was successful, -1 if there was a
     "hard" error, and the actual number of I/O-ed bytes if the read/write was
     partial.
@@ -457,11 +478,6 @@ struct IO_CACHE /* Used when caching files */
   Stream_cipher *m_encryptor = nullptr;
   // This is a decryptor for decrypting the temporary file of the IO cache.
   Stream_cipher *m_decryptor = nullptr;
-  // Synchronize flushed buffer with disk.
-  bool disk_sync{false};
-  // Delay in milliseconds after disk synchronization of the flushed buffer.
-  // Requires disk_sync = true.
-  uint disk_sync_delay{0};
 };
 
 typedef int (*qsort2_cmp)(const void *, const void *, const void *);
@@ -542,6 +558,8 @@ inline size_t my_b_bytes_in_cache(const IO_CACHE *info) {
   return *info->current_end - *info->current_pos;
 }
 
+typedef uint32 ha_checksum;
+
 /*
   How much overhead does malloc have. The code often allocates
   something like 1024-MALLOC_OVERHEAD bytes
@@ -568,11 +586,13 @@ extern void *my_once_alloc(size_t Size, myf MyFlags);
 extern void my_once_free(void);
 extern char *my_once_strdup(const char *src, myf myflags);
 extern void *my_once_memdup(const void *src, size_t len, myf myflags);
-extern File my_open(const char *filename, int Flags, myf MyFlags);
-
+extern File my_open(const char *FileName, int Flags, myf MyFlags);
+extern File my_register_filename(File fd, const char *FileName,
+                                 enum file_type type_of_file,
+                                 uint error_message_number, myf MyFlags);
 extern File my_create(const char *FileName, int CreateFlags, int AccessFlags,
                       myf MyFlags);
-extern int my_close(File fd, myf MyFlags);
+extern int my_close(File Filedes, myf MyFlags);
 extern int my_mkdir(const char *dir, int Flags, myf MyFlags);
 extern int my_readlink(char *to, const char *filename, myf MyFlags);
 extern int my_is_symlink(const char *filename, ST_FILE_ID *file_id);
@@ -647,16 +667,18 @@ extern void my_osmaperr(unsigned long last_error);
 
 extern const char *get_global_errmsg(int nr);
 extern void wait_for_free_space(const char *filename, int errors);
-extern FILE *my_fopen(const char *filename, int Flags, myf MyFlags);
-extern FILE *my_fdopen(File fd, const char *filename, int Flags, myf MyFlags);
-extern FILE *my_freopen(const char *filename, const char *mode, FILE *stream);
-extern int my_fclose(FILE *stream, myf MyFlags);
-extern File my_fileno(FILE *stream);
+extern FILE *my_fopen(const char *FileName, int Flags, myf MyFlags);
+extern FILE *my_fdopen(File Filedes, const char *name, int Flags, myf MyFlags);
+extern FILE *my_freopen(const char *path, const char *mode, FILE *stream);
+extern int my_fclose(FILE *fd, myf MyFlags);
+extern File my_fileno(FILE *fd);
 extern int my_chsize(File fd, my_off_t newlength, int filler, myf MyFlags);
 extern int my_fallocator(File fd, my_off_t newlength, int filler, myf MyFlags);
 extern void thr_set_sync_wait_callback(void (*before_sync)(void),
                                        void (*after_sync)(void));
 extern int my_sync(File fd, myf my_flags);
+extern int my_sync_dir(const char *dir_name, myf my_flags);
+extern int my_sync_dir_by_file(const char *file_name, myf my_flags);
 extern char *my_strerror(char *buf, size_t len, int errnum);
 extern const char *my_get_err_msg(int nr);
 extern void my_error(int nr, myf MyFlags, ...);
@@ -671,31 +693,18 @@ extern void my_message(uint my_err, const char *str, myf MyFlags);
 extern void my_message_stderr(uint my_err, const char *str, myf MyFlags);
 void my_message_local_stderr(enum loglevel, uint ecode, va_list args);
 extern void my_message_local(enum loglevel ll, uint ecode, ...);
-
-/**
-  Convenience wrapper for OS error messages which report
-  errno/my_errno with %d followed by strerror as %s, as the last
-  conversions in the error message.
-
-  The OS error message (my_errno) is formatted in a stack buffer and
-  the errno value and a pointer to the buffer is added to the end of
-  the parameter pack passed to my_error().
-
-  @param errno_val errno/my_errno number.
-  @param ppck parameter pack of additional arguments to pass to my_error().
- */
-template <class... Ts>
-inline void MyOsError(int errno_val, Ts... ppck) {
-  char errbuf[MYSYS_STRERROR_SIZE];
-  my_error(ppck..., errno_val, my_strerror(errbuf, sizeof(errbuf), errno_val));
-}
-
 extern bool my_init(void);
 extern void my_end(int infoflag);
 extern const char *my_filename(File fd);
 extern MY_MODE get_file_perm(ulong perm_flags);
 extern bool my_chmod(const char *filename, ulong perm_flags, myf my_flags);
 
+#ifdef EXTRA_DEBUG
+void my_print_open_files(void);
+#else
+#define my_print_open_files()
+#endif
+
 extern bool init_tmpdir(MY_TMPDIR *tmpdir, const char *pathlist);
 extern char *my_tmpdir(MY_TMPDIR *tmpdir);
 extern void free_tmpdir(MY_TMPDIR *tmpdir);
@@ -771,11 +780,6 @@ extern bool real_open_cached_file(IO_CACHE *cache);
 extern void close_cached_file(IO_CACHE *cache);
 
 enum UnlinkOrKeepFile { UNLINK_FILE, KEEP_FILE };
-#ifdef WIN32
-// Maximum temporary filename length is 3 chars for prefix + 16 chars for base
-// 32 encoded UUID (excluding MAC address)
-const size_t MY_MAX_TEMP_FILENAME_LEN = 19;
-#endif
 File create_temp_file(char *to, const char *dir, const char *pfx, int mode,
                       UnlinkOrKeepFile unlink_or_keep, myf MyFlags);
 
@@ -791,16 +795,18 @@ extern bool my_init_dynamic_array(DYNAMIC_ARRAY *array, PSI_memory_key key,
 /* Some functions are still in use in C++, because HASH uses DYNAMIC_ARRAY */
 extern bool insert_dynamic(DYNAMIC_ARRAY *array, const void *element);
 extern void *alloc_dynamic(DYNAMIC_ARRAY *array);
+extern void *pop_dynamic(DYNAMIC_ARRAY *);
+extern void claim_dynamic(DYNAMIC_ARRAY *array);
 extern void delete_dynamic(DYNAMIC_ARRAY *array);
+extern void freeze_size(DYNAMIC_ARRAY *array);
+static inline void reset_dynamic(DYNAMIC_ARRAY *array) { array->elements = 0; }
 
 extern bool init_dynamic_string(DYNAMIC_STRING *str, const char *init_str,
-                                size_t init_alloc);
+                                size_t init_alloc, size_t alloc_increment);
 extern bool dynstr_append(DYNAMIC_STRING *str, const char *append);
 bool dynstr_append_mem(DYNAMIC_STRING *str, const char *append, size_t length);
 extern bool dynstr_append_os_quoted(DYNAMIC_STRING *str, const char *append,
                                     ...);
-extern bool dynstr_append_quoted(DYNAMIC_STRING *str, const char *quote_str,
-                                 const uint quote_len, const char *append, ...);
 extern bool dynstr_set(DYNAMIC_STRING *str, const char *init_str);
 extern bool dynstr_realloc(DYNAMIC_STRING *str, size_t additional_size);
 extern bool dynstr_trunc(DYNAMIC_STRING *str, size_t n);
@@ -816,8 +822,10 @@ extern bool my_uncompress(mysql_compress_context *, uchar *, size_t, size_t *);
 extern uchar *my_compress_alloc(mysql_compress_context *comp_ctx,
                                 const uchar *packet, size_t *len,
                                 size_t *complen);
+extern ha_checksum my_checksum(ha_checksum crc, const uchar *mem, size_t count);
 
 extern uint my_set_max_open_files(uint files);
+void my_free_open_file_info(void);
 
 extern bool my_gethwaddr(uchar *to);
 
@@ -872,9 +880,10 @@ static inline int my_getpagesize() {
 int my_msync(int, void *, size_t, int);
 
 /* character sets */
+extern void my_charset_loader_init_mysys(MY_CHARSET_LOADER *loader);
 extern uint get_charset_number(const char *cs_name, uint cs_flags);
 extern uint get_collation_number(const char *name);
-extern const char *get_collation_name(uint cs_number);
+extern const char *get_charset_name(uint cs_number);
 
 extern CHARSET_INFO *get_charset(uint cs_number, myf flags);
 extern CHARSET_INFO *get_charset_by_name(const char *cs_name, myf flags);
@@ -891,6 +900,7 @@ extern bool resolve_collation(const char *cl_name,
                               const CHARSET_INFO *default_cl,
                               const CHARSET_INFO **cl);
 extern char *get_charsets_dir(char *buf);
+extern bool my_charset_same(const CHARSET_INFO *cs1, const CHARSET_INFO *cs2);
 extern bool init_compiled_charsets(myf flags);
 extern void add_compiled_collation(CHARSET_INFO *cs);
 extern size_t escape_string_for_mysql(const CHARSET_INFO *charset_info,
@@ -963,16 +973,17 @@ extern MYSQL_PLUGIN_IMPORT PSI_thread_bootstrap *psi_thread_hook;
 extern void set_psi_thread_service(void *psi);
 extern MYSQL_PLUGIN_IMPORT PSI_transaction_bootstrap *psi_transaction_hook;
 extern void set_psi_transaction_service(void *psi);
-extern MYSQL_PLUGIN_IMPORT PSI_tls_channel_bootstrap *psi_tls_channel_hook;
-extern void set_psi_tls_channel_service(void *psi);
 #endif /* HAVE_PSI_INTERFACE */
 
+struct MYSQL_FILE;
+extern MYSQL_FILE *mysql_stdin;
+
 /**
   @} (end of group MYSYS)
 */
 
 // True if the temporary file of binlog cache is encrypted.
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 extern bool binlog_cache_temporary_file_is_encrypted;
 #endif
 
@@ -1022,7 +1033,17 @@ size_t mysql_encryption_file_read(IO_CACHE *cache, uchar *buffer, size_t count,
                 MY_WME | MY_FAE | MY_NABP | MY_FNABP |
                 MY_DONT_CHECK_FILESIZE and so on
 
-   @return The number of bytes written
+   if (flags & (MY_NABP | MY_FNABP)) {
+     @retval 0 if count == 0
+     @retval 0 success
+     @retval MY_FILE_ERROR error
+   } else {
+     @retval 0 if count == 0
+     @retval The number of bytes written on success.
+     @retval MY_FILE_ERROR error
+     @retval The actual number of bytes written on partial success (if
+             less than count bytes were written).
+   }
 */
 size_t mysql_encryption_file_write(IO_CACHE *cache, const uchar *buffer,
                                    size_t count, myf flags);
