@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -79,8 +78,8 @@ struct TREE {
 };
 
 /* Functions on whole tree */
-void init_tree(TREE *tree, ulong memory_limit, int element_size,
-               qsort2_cmp compare, bool with_delete,
+void init_tree(TREE *tree, size_t default_alloc_size, ulong memory_limit,
+               int size, qsort2_cmp compare, bool with_delete,
                tree_element_free free_element, const void *custom_arg);
 void delete_tree(TREE *);
 void reset_tree(TREE *);
