@@ -1,19 +1,18 @@
 #ifndef MYISAMPACK_INCLUDED
 #define MYISAMPACK_INCLUDED
 
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,6 +33,9 @@
 
 #include "my_config.h"
 
+#ifdef HAVE_ENDIAN_H
+#include <endian.h>
+#endif
 #include <sys/types.h>
 
 #include "my_inttypes.h"
@@ -286,7 +288,19 @@ static inline double mi_float8get(const uchar *M) {
 #define mi_rowstore(T, A) mi_int8store(T, A)
 #define mi_rowkorr(T) mi_uint8korr(T)
 
+#if SIZEOF_OFF_T > 4
 #define mi_sizestore(T, A) mi_int8store(T, A)
 #define mi_sizekorr(T) mi_uint8korr(T)
-
+#else
+#define mi_sizestore(T, A)        \
+  {                               \
+    if ((A) == HA_OFFSET_ERROR)   \
+      memset((T), 255, 8);        \
+    else {                        \
+      mi_int4store((T), 0);       \
+      mi_int4store(((T) + 4), A); \
+    }                             \
+  }
+#define mi_sizekorr(T) mi_uint4korr((uchar *)(T) + 4)
+#endif
 #endif /* MYISAMPACK_INCLUDED */
