@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2020, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -23,8 +23,6 @@
 #ifndef COMPONENTS_SERVICES_PSI_RWLOCK_BITS_H
 #define COMPONENTS_SERVICES_PSI_RWLOCK_BITS_H
 
-#include <mysql/components/services/bits/psi_bits.h>
-
 /**
   @file
   Performance schema instrumentation interface.
@@ -67,7 +65,7 @@ typedef unsigned int PSI_rwlock_key;
   Interface for an instrumented rwlock.
   This is an opaque structure.
 */
-struct PSI_rwlock : PSI_instr {};
+struct PSI_rwlock;
 typedef struct PSI_rwlock PSI_rwlock;
 
 /**
@@ -200,11 +198,11 @@ struct PSI_rwlock_locker_state_v1 {
   /** Current thread. */
   struct PSI_thread *m_thread;
   /** Timer start. */
-  unsigned long long m_timer_start{0};
+  unsigned long long m_timer_start;
   /** Timer function. */
   unsigned long long (*m_timer)(void);
   /** Internal data. */
-  void *m_wait{nullptr};
+  void *m_wait;
 };
 typedef struct PSI_rwlock_locker_state_v1 PSI_rwlock_locker_state_v1;
 
