@@ -1,16 +1,15 @@
-/*  Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/*  Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License, version 2.0,
     as published by the Free Software Foundation.
 
-    This program is designed to work with certain software (including
+    This program is also distributed with certain software (including
     but not limited to OpenSSL) that is licensed under separate terms,
     as designated in a particular file or component or in included license
     documentation.  The authors of MySQL hereby grant you an additional
     permission to link the program and your derivative works with the
-    separately licensed software that they have either included with
-    the program or referenced in the documentation.
+    separately licensed software that they have included with MySQL.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,17 +27,23 @@
   Header file for the Server session service. This service is to provide
   of creating sessions with the server. These sessions can be furtherly used
   together with the Command service to execute commands in the server.
-
-  @note Session should not be used after being closed, unless MYSQL_SESSION
-        handle is set to NULL.
 */
 
-#include "mysql/service_srv_session_bits.h" /* MYSQL_SESSION, srv_session_error_cb */
+#ifdef __cplusplus
+class Srv_session;
+typedef class Srv_session *MYSQL_SESSION;
+#else
+struct Srv_session;
+typedef struct Srv_session *MYSQL_SESSION;
+#endif
 
 #ifndef MYSQL_ABI_CHECK
 #include "mysql/plugin.h" /* MYSQL_THD */
 #endif
 
+typedef void (*srv_session_error_cb)(void *ctx, unsigned int sql_errno,
+                                     const char *err_msg);
+
 extern "C" struct srv_session_service_st {
   int (*init_session_thread)(const void *plugin);
 
@@ -54,7 +59,7 @@ extern "C" struct srv_session_service_st {
   int (*server_is_available)();
 
   int (*attach_session)(MYSQL_SESSION session, MYSQL_THD *ret_previous_thd);
-} *srv_session_service;
+} * srv_session_service;
 
 #ifdef MYSQL_DYNAMIC_PLUGIN
 
@@ -138,10 +143,6 @@ int srv_session_detach(MYSQL_SESSION session);
 
   @param session  Session to close
 
-  @note This method close the session but session handle is not set to NULL.
-        Session handle should be set to NULL explicitly after calling this
-        method. Session should not be used otherwise.
-
   @return
     0  success
     1  failure
@@ -164,7 +165,6 @@ int srv_session_server_is_available();
   THD associated with session is attached.
 
   @param session  Session to attach
-  @param ret_previous_thd Previously attached THD
 
   @returns
     0  success
