@@ -35,9 +35,11 @@
 
 #ifndef MYSQL_ABI_CHECK
 #include <stdbool.h>
+#include <stdint.h>
 #endif
 
 #include "my_command.h"
+#include "my_compress.h"
 
 /*
   We need a definition for my_socket. On the client, <mysql.h> already provides
@@ -45,13 +47,13 @@
 */
 #ifndef my_socket_defined
 #include "my_io.h"
+#include "mysql/components/services/my_io_bits.h"
 #endif
 
 #ifndef MYSQL_ABI_CHECK
 #include <stdbool.h>
 #endif
 
-#define HOSTNAME_LENGTH 60
 #define SYSTEM_CHARSET_MBMAXLEN 3
 #define FILENAME_CHARSET_MBMAXLEN 5
 #define NAME_CHAR_LEN 64 /**< Field/table name length */
@@ -93,14 +95,6 @@
 */
 #define MAX_PACKET_LENGTH (256L * 256L * 256L - 1)
 
-/**
-  Length of string buffer, that is enough to contain
-  username and hostname parts of the user identifier with trailing zero in
-  MySQL standard format:
-  user_name_part\@host_name_part\\0
-*/
-#define USER_HOST_BUFF_SIZE HOSTNAME_LENGTH + USERNAME_LENGTH + 2
-
 #define LOCAL_HOST "localhost"
 #define LOCAL_HOST_NAMEDPIPE "."
 
@@ -690,6 +684,30 @@
 */
 #define CLIENT_OPTIONAL_RESULTSET_METADATA (1UL << 25)
 
+/**
+  Compression protocol extended to support zstd compression method
+
+  This capability flag is used to send zstd compression level between
+  client and server provided both client and server are enabled with
+  this flag.
+
+  Server
+  ------
+  Server sets this flag when global variable protocol-compression-algorithms
+  has zstd in its list of supported values.
+
+  Client
+  ------
+  Client sets this flag when it is configured to use zstd compression method.
+
+*/
+#define CLIENT_ZSTD_COMPRESSION_ALGORITHM (1UL << 26)
+
+/**
+  This flag will be reserved to extend the 32bit capabilities structure to
+  64bits.
+*/
+#define CLIENT_CAPABILITY_EXTENSION (1UL << 29)
 /**
   Don't reset the options after an unsuccessful connect
 
@@ -717,7 +735,8 @@
    CLIENT_PLUGIN_AUTH | CLIENT_CONNECT_ATTRS |                                 \
    CLIENT_PLUGIN_AUTH_LENENC_CLIENT_DATA |                                     \
    CLIENT_CAN_HANDLE_EXPIRED_PASSWORDS | CLIENT_SESSION_TRACK |                \
-   CLIENT_DEPRECATE_EOF | CLIENT_OPTIONAL_RESULTSET_METADATA)
+   CLIENT_DEPRECATE_EOF | CLIENT_OPTIONAL_RESULTSET_METADATA |                 \
+   CLIENT_ZSTD_COMPRESSION_ALGORITHM)
 
 /**
   Switch off from ::CLIENT_ALL_FLAGS the flags that are optional and
@@ -725,9 +744,10 @@
   If any of the optional flags is supported by the build it will be switched
   on before sending to the client during the connection handshake.
 */
-#define CLIENT_BASIC_FLAGS                                 \
-  (((CLIENT_ALL_FLAGS & ~CLIENT_SSL) & ~CLIENT_COMPRESS) & \
-   ~CLIENT_SSL_VERIFY_SERVER_CERT)
+#define CLIENT_BASIC_FLAGS                                          \
+  (CLIENT_ALL_FLAGS &                                               \
+   ~(CLIENT_SSL | CLIENT_COMPRESS | CLIENT_SSL_VERIFY_SERVER_CERT | \
+     CLIENT_ZSTD_COMPRESSION_ALGORITHM))
 
 /** The status flags are a bit-field */
 enum SERVER_STATUS_flags_enum {
@@ -1085,7 +1105,7 @@ unsigned long STDCALL net_field_length(unsigned char **packet);
 unsigned long STDCALL net_field_length_checked(unsigned char **packet,
                                                unsigned long max_length);
 #endif
-unsigned long long net_field_length_ll(unsigned char **packet);
+uint64_t net_field_length_ll(unsigned char **packet);
 unsigned char *net_store_length(unsigned char *pkg, unsigned long long length);
 unsigned int net_length_size(unsigned long long num);
 unsigned int net_field_length_size(const unsigned char *pos);
@@ -1093,6 +1113,4 @@ unsigned int net_field_length_size(const unsigned char *pos);
 #define NULL_LENGTH ((unsigned long)~0) /**< For ::net_store_length() */
 #define MYSQL_STMT_HEADER 4
 #define MYSQL_LONG_DATA_HEADER 6
-
-#define NOT_FIXED_DEC 31
 #endif
