@@ -1,19 +1,18 @@
 #ifndef SQL_STRING_INCLUDED
 #define SQL_STRING_INCLUDED
 
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,13 +29,10 @@
   See in particular the comment on String before you use anything from here.
 */
 
-#include <assert.h>
-#include <stdint.h>
 #include <string.h>
 #include <sys/types.h>
 #include <new>
 #include <string>
-#include <string_view>
 
 #include "lex_string.h"
 #include "m_ctype.h"   // my_convert
@@ -44,11 +40,10 @@
 #include "memory_debugging.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
-#include "my_sys.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/mysql_lex_string.h"  // LEX_STRING
+#include "mysql/psi/psi_base.h"
 #include "mysql/psi/psi_memory.h"
 #include "mysql/service_mysql_alloc.h"  // my_free
 
@@ -70,7 +65,7 @@ extern PSI_memory_key key_memory_String_value;
   pass it and its descendants (such as Name_string) into functions
   using call-by-value evaluation.
 
-  Don't add new members or virtual methods to this class!
+  Don't add new members or virual methods into this class!
 */
 class Simple_cstring {
  private:
@@ -83,19 +78,19 @@ class Simple_cstring {
   */
   void set(const char *str_arg, size_t length_arg) {
     // NULL is allowed only with length==0
-    assert(str_arg || length_arg == 0);
+    DBUG_ASSERT(str_arg || length_arg == 0);
     // For non-NULL, make sure length_arg is in sync with '\0' terminator.
-    assert(!str_arg || str_arg[length_arg] == '\0');
+    DBUG_ASSERT(!str_arg || str_arg[length_arg] == '\0');
     m_str = str_arg;
     m_length = length_arg;
   }
-  Simple_cstring() { set(nullptr, 0); }
+  Simple_cstring() { set(NULL, 0); }
   Simple_cstring(const char *str_arg, size_t length_arg) {
     set(str_arg, length_arg);
   }
   Simple_cstring(const LEX_STRING arg) { set(arg.str, arg.length); }
   Simple_cstring(const LEX_CSTRING arg) { set(arg.str, arg.length); }
-  void reset() { set(nullptr, 0); }
+  void reset() { set(NULL, 0); }
   /**
     Set to a null-terminated string.
   */
@@ -107,7 +102,7 @@ class Simple_cstring {
   /**
     Check if m_ptr is set.
   */
-  bool is_set() const { return m_str != nullptr; }
+  bool is_set() const { return m_str != NULL; }
   /**
     Return name length.
   */
@@ -133,8 +128,6 @@ class String;
 struct CHARSET_INFO;
 struct IO_CACHE;
 
-bool validate_string(const CHARSET_INFO *cs, const char *str, size_t length,
-                     size_t *valid_length, bool *length_error);
 int sortcmp(const String *a, const String *b, const CHARSET_INFO *cs);
 String *copy_if_not_alloced(String *to, String *from, size_t from_length);
 inline size_t copy_and_convert(char *to, size_t to_length,
@@ -175,13 +168,13 @@ class String {
 
  public:
   String()
-      : m_ptr(nullptr),
+      : m_ptr(NULL),
         m_length(0),
         m_charset(&my_charset_bin),
         m_alloced_length(0),
         m_is_alloced(false) {}
   explicit String(size_t length_arg)
-      : m_ptr(nullptr),
+      : m_ptr(NULL),
         m_length(0),
         m_charset(&my_charset_bin),
         m_alloced_length(0),
@@ -221,8 +214,8 @@ class String {
     str.m_is_alloced = false;
   }
   static void *operator new(size_t size, MEM_ROOT *mem_root,
-                            const std::nothrow_t &arg
-                            [[maybe_unused]] = std::nothrow) noexcept {
+                            const std::nothrow_t &arg MY_ATTRIBUTE((unused)) =
+                                std::nothrow) noexcept {
     return mem_root->Alloc(size);
   }
   static void operator delete(void *ptr_arg, size_t size) {
@@ -232,7 +225,7 @@ class String {
   }
 
   static void operator delete(
-      void *, MEM_ROOT *, const std::nothrow_t &) noexcept { /* never called */
+      void *, MEM_ROOT *, const std::nothrow_t &)noexcept { /* never called */
   }
 
   ~String() { mem_free(); }
@@ -250,8 +243,8 @@ class String {
   const char *ptr() const { return m_ptr; }
   char *ptr() { return m_ptr; }
   char *c_ptr() {
-    assert(!m_is_alloced || !m_ptr || !m_alloced_length ||
-           (m_alloced_length >= (m_length + 1)));
+    DBUG_ASSERT(!m_is_alloced || !m_ptr || !m_alloced_length ||
+                (m_alloced_length >= (m_length + 1)));
 
     /*
       Should be safe, but in case valgrind complains on this line, it means
@@ -279,7 +272,7 @@ class String {
   }
 
   void set(String &str, size_t offset, size_t arg_length) {
-    assert(&str != this);
+    DBUG_ASSERT(&str != this);
     mem_free();
     m_ptr = str.ptr() + offset;
     m_length = arg_length;
@@ -370,9 +363,9 @@ class String {
     m_ptr[m_length] = '\0';
   }
 
-  void mem_claim(bool claim) {
+  void mem_claim() {
     if (m_is_alloced) {
-      my_claim(m_ptr, claim);
+      my_claim(m_ptr);
     }
   }
 
@@ -381,7 +374,7 @@ class String {
       m_is_alloced = false;
       m_alloced_length = 0;
       my_free(m_ptr);
-      m_ptr = nullptr;
+      m_ptr = NULL;
       m_length = 0; /* Safety */
     }
   }
@@ -420,7 +413,7 @@ class String {
         It is forbidden to do assignments like
         some_string = substring_of_that_string
        */
-      assert(!s.uses_buffer_owned_by(this));
+      DBUG_ASSERT(!s.uses_buffer_owned_by(this));
       mem_free();
       m_ptr = s.m_ptr;
       m_length = s.m_length;
@@ -436,7 +429,7 @@ class String {
         It is forbidden to do assignments like
         some_string = substring_of_that_string
        */
-      assert(!s.uses_buffer_owned_by(this));
+      DBUG_ASSERT(!s.uses_buffer_owned_by(this));
       mem_free();
       m_ptr = s.m_ptr;
       m_length = s.m_length;
@@ -450,44 +443,34 @@ class String {
   }
   /**
     Takeover the buffer owned by another string.
-    "this" becomes the owner of the buffer and
+    "this" becames the owner of the buffer and
     is further responsible to free it.
-    The string "s" is detached from the buffer (cleared).
+    The string "s" is detouched from the buffer (cleared).
 
     @param s - a String object to steal buffer from.
   */
   void takeover(String &s) {
-    assert(this != &s);
+    DBUG_ASSERT(this != &s);
     // Make sure buffers of the two Strings do not overlap
-    assert(!s.uses_buffer_owned_by(this));
+    DBUG_ASSERT(!s.uses_buffer_owned_by(this));
     mem_free();
     m_ptr = s.m_ptr;
     m_length = s.m_length;
     m_alloced_length = s.m_alloced_length;
     m_is_alloced = s.m_is_alloced;
     m_charset = s.m_charset;
-    s.m_ptr = nullptr;
+    s.m_ptr = NULL;
     s.m_alloced_length = 0;
     s.m_length = 0;
     s.m_is_alloced = false;
   }
 
-  bool copy();                 // Alloc string if not allocated
+  bool copy();                 // Alloc string if not alloced
   bool copy(const String &s);  // Allocate new string
   // Allocate new string
   bool copy(const char *s, size_t arg_length, const CHARSET_INFO *cs);
   static bool needs_conversion(size_t arg_length, const CHARSET_INFO *cs_from,
                                const CHARSET_INFO *cs_to, size_t *offset);
-  bool needs_conversion(const CHARSET_INFO *cs_to) const {
-    size_t offset;
-    return needs_conversion(length(), charset(), cs_to, &offset);
-  }
-  bool is_valid_string(const CHARSET_INFO *cs_to) const {
-    size_t valid_length;
-    bool length_error;
-    return !validate_string(cs_to, ptr(), length(), &valid_length,
-                            &length_error);
-  }
   static bool needs_conversion_on_storage(size_t arg_length,
                                           const CHARSET_INFO *cs_from,
                                           const CHARSET_INFO *cs_to);
@@ -498,16 +481,17 @@ class String {
   bool copy(const char *s, size_t arg_length, const CHARSET_INFO *csfrom,
             const CHARSET_INFO *csto, uint *errors);
   bool append(const String &s);
-  bool append(std::string_view s) { return append(s.data(), s.size()); }
+  bool append(const char *s);
   bool append(LEX_STRING *ls) { return append(ls->str, ls->length); }
   bool append(Simple_cstring str) { return append(str.ptr(), str.length()); }
   bool append(const char *s, size_t arg_length);
   bool append(const char *s, size_t arg_length, const CHARSET_INFO *cs);
   bool append_ulonglong(ulonglong val);
   bool append_longlong(longlong val);
+  bool append(IO_CACHE *file, size_t arg_length);
   bool append_with_prefill(const char *s, size_t arg_length, size_t full_length,
                            char fill_char);
-  bool append_parenthesized(int64_t nr);
+  bool append_parenthesized(long nr, int radix = 10);
   /**
     Search for a substring.
 
@@ -529,7 +513,7 @@ class String {
   */
   int strrstr(const String &search, size_t offset = 0) const;
   /**
-   * Returns substring of given characters length, starting at given character
+   * Returns substring of given characters lenght, starting at given character
    * offset. Note that parameter indexes are character indexes and not byte
    * indexes.
    */
@@ -541,12 +525,13 @@ class String {
     if (m_length < m_alloced_length) {
       m_ptr[m_length++] = chr;
     } else {
-      if (mem_realloc_exp(m_length + 1)) return true;
+      if (mem_realloc_exp(m_length + 1)) return 1;
       m_ptr[m_length++] = chr;
     }
-    return false;
+    return 0;
   }
   bool fill(size_t max_length, char fill);
+  void strip_sp();
   friend int sortcmp(const String *a, const String *b, const CHARSET_INFO *cs);
   friend int stringcmp(const String *a, const String *b);
   friend String *copy_if_not_alloced(String *to, String *from,
@@ -555,10 +540,7 @@ class String {
   size_t charpos(size_t i, size_t offset = 0) const;
 
   bool reserve(size_t space_needed) {
-    if (m_alloced_length < m_length + space_needed) {
-      return mem_realloc(m_length + space_needed);
-    }
-    return false;
+    return mem_realloc(m_length + space_needed);
   }
   bool reserve(size_t space_needed, size_t grow_by);
 
@@ -567,7 +549,7 @@ class String {
   char *prep_append(size_t arg_length, size_t step_alloc) {
     size_t new_length = arg_length + m_length;
     if (new_length > m_alloced_length) {
-      if (mem_realloc(new_length + step_alloc)) return nullptr;
+      if (mem_realloc(new_length + step_alloc)) return NULL;
     }
     size_t old_length = m_length;
     m_length += arg_length;
@@ -610,35 +592,6 @@ class String {
   char *dup(MEM_ROOT *root) const;
 };
 
-/**
-  Checks that the source string can be just copied to the destination string
-  without conversion.
-
-  @param arg_length     Length of string to copy.
-  @param from_cs        Character set to copy from
-  @param to_cs          Character set to copy to
-  @param *offset	Returns number of unaligned characters.
-
-  @returns true if conversion is required, false otherwise.
-
-  @note
-  to_cs may be nullptr for "no conversion" if the system variable
-  character_set_results is NULL.
-*/
-
-inline bool String::needs_conversion(size_t arg_length,
-                                     const CHARSET_INFO *from_cs,
-                                     const CHARSET_INFO *to_cs,
-                                     size_t *offset) {
-  *offset = 0;
-  if (to_cs == nullptr || (to_cs == &my_charset_bin) || from_cs == to_cs ||
-      my_charset_same(from_cs, to_cs) ||
-      ((from_cs == &my_charset_bin) &&
-       (0 == (*offset = (arg_length % to_cs->mbminlen)))))
-    return false;
-  return true;
-}
-
 static inline void swap(String &a, String &b) noexcept { a.swap(b); }
 
 static inline std::string to_string(const String &str) {
@@ -687,10 +640,13 @@ inline LEX_STRING to_lex_string(const LEX_CSTRING &s) {
 }
 
 inline LEX_CSTRING to_lex_cstring(const char *s) {
-  LEX_CSTRING cstr = {s, s != nullptr ? strlen(s) : 0};
+  LEX_CSTRING cstr = {s, s != NULL ? strlen(s) : 0};
   return cstr;
 }
 
+bool validate_string(const CHARSET_INFO *cs, const char *str, size_t length,
+                     size_t *valid_length, bool *length_error);
+
 bool append_escaped(String *to_str, const String *from_str);
 
 #endif /* SQL_STRING_INCLUDED */
