@@ -1,7 +1,7 @@
 #ifndef THR_MUTEX_INCLUDED
 #define THR_MUTEX_INCLUDED
 
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -40,11 +40,10 @@
        See include/mysql/psi/mysql_thread.h
 */
 
-#include <assert.h>
 #include <stddef.h>
-#include <stdlib.h>
 #include <sys/types.h>
 
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "my_thread.h"
@@ -54,7 +53,7 @@
   - native_mutex_t
   - my_mutex_t
 */
-#include "mysql/components/services/bits/thr_mutex_bits.h"
+#include "mysql/components/services/thr_mutex_bits.h"
 
 /* Define mutex types, see my_thr_init.c */
 #define MY_MUTEX_INIT_SLOW NULL
@@ -77,7 +76,7 @@ extern native_mutexattr_t my_errorcheck_mutexattr;
 
 static inline int native_mutex_init(native_mutex_t *mutex,
                                     const native_mutexattr_t *attr
-                                    [[maybe_unused]]) {
+                                        MY_ATTRIBUTE((unused))) {
 #ifdef _WIN32
   InitializeCriticalSection(mutex);
   return 0;
@@ -147,16 +146,16 @@ int safe_mutex_unlock(safe_mutex_t *mp, const char *file, uint line);
 int safe_mutex_destroy(safe_mutex_t *mp, const char *file, uint line);
 
 static inline void safe_mutex_assert_owner(safe_mutex_t *mp) {
-  assert(mp != nullptr);
+  DBUG_ASSERT(mp != NULL);
   native_mutex_lock(&mp->global);
-  assert(mp->count > 0 && my_thread_equal(my_thread_self(), mp->thread));
+  DBUG_ASSERT(mp->count > 0 && my_thread_equal(my_thread_self(), mp->thread));
   native_mutex_unlock(&mp->global);
 }
 
 static inline void safe_mutex_assert_not_owner(safe_mutex_t *mp) {
-  assert(mp != nullptr);
+  DBUG_ASSERT(mp != NULL);
   native_mutex_lock(&mp->global);
-  assert(!mp->count || !my_thread_equal(my_thread_self(), mp->thread));
+  DBUG_ASSERT(!mp->count || !my_thread_equal(my_thread_self(), mp->thread));
   native_mutex_unlock(&mp->global);
 }
 #endif /* SAFE_MUTEX */
@@ -168,7 +167,7 @@ static inline int my_mutex_init(my_mutex_t *mp, const native_mutexattr_t *attr
 #endif
 ) {
 #ifdef SAFE_MUTEX
-  assert(mp != nullptr);
+  DBUG_ASSERT(mp != NULL);
   mp->m_u.m_safe_ptr = (safe_mutex_t *)malloc(sizeof(safe_mutex_t));
   return safe_mutex_init(mp->m_u.m_safe_ptr, attr, file, line);
 #else
@@ -183,8 +182,8 @@ static inline int my_mutex_lock(my_mutex_t *mp
 #endif
 ) {
 #ifdef SAFE_MUTEX
-  assert(mp != nullptr);
-  assert(mp->m_u.m_safe_ptr != nullptr);
+  DBUG_ASSERT(mp != NULL);
+  DBUG_ASSERT(mp->m_u.m_safe_ptr != NULL);
   return safe_mutex_lock(mp->m_u.m_safe_ptr, false, file, line);
 #else
   return native_mutex_lock(&mp->m_u.m_native);
@@ -198,8 +197,8 @@ static inline int my_mutex_trylock(my_mutex_t *mp
 #endif
 ) {
 #ifdef SAFE_MUTEX
-  assert(mp != nullptr);
-  assert(mp->m_u.m_safe_ptr != nullptr);
+  DBUG_ASSERT(mp != NULL);
+  DBUG_ASSERT(mp->m_u.m_safe_ptr != NULL);
   return safe_mutex_lock(mp->m_u.m_safe_ptr, true, file, line);
 #else
   return native_mutex_trylock(&mp->m_u.m_native);
@@ -213,8 +212,8 @@ static inline int my_mutex_unlock(my_mutex_t *mp
 #endif
 ) {
 #ifdef SAFE_MUTEX
-  assert(mp != nullptr);
-  assert(mp->m_u.m_safe_ptr != nullptr);
+  DBUG_ASSERT(mp != NULL);
+  DBUG_ASSERT(mp->m_u.m_safe_ptr != NULL);
   return safe_mutex_unlock(mp->m_u.m_safe_ptr, file, line);
 #else
   return native_mutex_unlock(&mp->m_u.m_native);
@@ -228,11 +227,11 @@ static inline int my_mutex_destroy(my_mutex_t *mp
 #endif
 ) {
 #ifdef SAFE_MUTEX
-  assert(mp != nullptr);
-  assert(mp->m_u.m_safe_ptr != nullptr);
+  DBUG_ASSERT(mp != NULL);
+  DBUG_ASSERT(mp->m_u.m_safe_ptr != NULL);
   int rc = safe_mutex_destroy(mp->m_u.m_safe_ptr, file, line);
   free(mp->m_u.m_safe_ptr);
-  mp->m_u.m_safe_ptr = nullptr;
+  mp->m_u.m_safe_ptr = NULL;
   return rc;
 #else
   return native_mutex_destroy(&mp->m_u.m_native);
