@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -42,24 +41,21 @@
 
 #include "my_inttypes.h"
 #include "my_psi_config.h"  // IWYU pragma: keep
-#include "mysql/components/services/bits/my_io_bits.h"
-#include "mysql/components/services/bits/my_thread_bits.h"
-#include "mysql/components/services/bits/mysql_socket_bits.h"
-
-#include "mysql/psi/mysql_socket.h"
+#include "mysql/components/services/my_io_bits.h"
+#include "mysql/components/services/my_thread_bits.h"
+#include "mysql/components/services/mysql_socket_bits.h"
 
 struct Vio;
 
 /* Simple vio interface in C;  The functions are implemented in violite.c */
 
-#if !defined(_WIN32) && !defined(HAVE_KQUEUE)
+#if !defined(_WIN32) && !defined(HAVE_KQUEUE) && !defined(__SUNPRO_CC)
 #define USE_PPOLL_IN_VIO
 #endif
 
 #if defined(__cplusplus) && defined(USE_PPOLL_IN_VIO)
 #include <signal.h>
 #include <atomic>
-#include <optional>
 #elif defined(__cplusplus) && defined(HAVE_KQUEUE)
 #include <sys/event.h>
 #include <atomic>
@@ -142,6 +138,7 @@ enum enum_vio_io_event {
 #define VIO_LOCALHOST 1            /* a localhost connection */
 #define VIO_BUFFERED_READ 2        /* use buffered read */
 #define VIO_READ_BUFFER_SIZE 16384 /* size of read buffer */
+#define OPENSSL_ERROR_LENGTH 512   /* Openssl error code max length */
 
 MYSQL_VIO vio_new(my_socket sd, enum enum_vio_type type, uint flags);
 MYSQL_VIO mysql_socket_vio_new(MYSQL_SOCKET mysql_socket,
@@ -177,11 +174,11 @@ int vio_keepalive(MYSQL_VIO vio, bool onoff);
 bool vio_should_retry(MYSQL_VIO vio);
 /* Check that operation was timed out */
 bool vio_was_timeout(MYSQL_VIO vio);
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 /* Short text description of the socket for those, who are curious.. */
 #define VIO_DESCRIPTION_SIZE 30 /* size of description */
 void vio_description(MYSQL_VIO vio, char *buf);
-#endif  // NDEBUG
+#endif  // DBUG_OFF
 /* Return the type of the connection */
 enum enum_vio_type vio_type(const MYSQL_VIO vio);
 /* Return last error number */
@@ -193,15 +190,14 @@ bool vio_peer_addr(MYSQL_VIO vio, char *buf, uint16 *port, size_t buflen);
 /* Wait for an I/O event notification. */
 int vio_io_wait(MYSQL_VIO vio, enum enum_vio_io_event event, int timeout);
 bool vio_is_connected(MYSQL_VIO vio);
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 ssize_t vio_pending(MYSQL_VIO vio);
 #endif
 /* Set timeout for a network operation. */
 int vio_timeout(MYSQL_VIO vio, uint which, int timeout_sec);
 /* Connect to a peer. */
 bool vio_socket_connect(MYSQL_VIO vio, struct sockaddr *addr, socklen_t len,
-                        bool nonblocking, int timeout,
-                        bool *connect_done = nullptr);
+                        bool nonblocking, int timeout);
 
 bool vio_get_normalized_ip_string(const struct sockaddr *addr,
                                   size_t addr_length, char *ip_string,
@@ -213,6 +209,7 @@ int vio_getnameinfo(const struct sockaddr *sa, char *hostname,
                     size_t hostname_size, char *port, size_t port_size,
                     int flags);
 
+#if defined(HAVE_OPENSSL)
 extern "C" {
 #include <openssl/opensslv.h>
 }
@@ -228,6 +225,11 @@ extern "C" {
 #define HAVE_OPENSSL11 1
 #endif  // OPENSSL_VERSION_NUMBER
 
+/* apple deprecated openssl in MacOSX Lion */
+#ifdef __APPLE__
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
 #define HEADER_DES_LOCL_H dummy_something
 
 #include <openssl/err.h>
@@ -246,8 +248,6 @@ enum enum_ssl_init_error {
   SSL_TLS_VERSION_INVALID,
   SSL_FIPS_MODE_INVALID,
   SSL_FIPS_MODE_FAILED,
-  SSL_INITERR_ECDHFAIL,
-  SSL_INITERR_X509_VERIFY_PARAM,
   SSL_INITERR_LASTERR
 };
 const char *sslGetErrString(enum enum_ssl_init_error err);
@@ -259,16 +259,20 @@ struct st_VioSSLFd {
 int sslaccept(struct st_VioSSLFd *, MYSQL_VIO, long timeout,
               unsigned long *errptr);
 int sslconnect(struct st_VioSSLFd *, MYSQL_VIO, long timeout,
-               SSL_SESSION *session, unsigned long *errptr, SSL **ssl);
+               unsigned long *errptr, SSL **ssl);
 
 struct st_VioSSLFd *new_VioSSLConnectorFd(
     const char *key_file, const char *cert_file, const char *ca_file,
     const char *ca_path, const char *cipher, const char *ciphersuites,
     enum enum_ssl_init_error *error, const char *crl_file, const char *crl_path,
-    const long ssl_ctx_flags, const char *server_host);
+    const long ssl_ctx_flags);
 
 long process_tls_version(const char *tls_version);
 
+int set_fips_mode(const uint fips_mode, char *err_string);
+
+uint get_fips_mode();
+
 struct st_VioSSLFd *new_VioSSLAcceptorFd(
     const char *key_file, const char *cert_file, const char *ca_file,
     const char *ca_path, const char *cipher, const char *ciphersuites,
@@ -278,6 +282,8 @@ void free_vio_ssl_acceptor_fd(struct st_VioSSLFd *fd);
 
 void vio_ssl_end();
 
+#endif /* HAVE_OPENSSL */
+
 void ssl_start(void);
 void vio_end(void);
 
@@ -334,18 +340,8 @@ struct Vio {
   char *read_end = {nullptr};     /* end of unfetched data */
 
 #ifdef USE_PPOLL_IN_VIO
-  /** Thread PID which is to be sent SIGALRM to terminate ppoll
-    wait when shutting down vio. It is made an std::optional so
-    that server code has the ability to set this to an illegal value
-    and thereby ensure that it is set before shutting down vio. In the server
-    the THD and thereby the Vio can switch between OS threads, so it does not
-    make sense to assign the thread id when creating the THD/Vio.
-
-    It is initialized to 0 here, meaning don't attempt to send a signal, to
-    keep non-server code unaffected.
-  */
-  std::optional<my_thread_t> thread_id = 0;
-  sigset_t signal_mask;  // Signal mask
+  my_thread_t thread_id = {0};  // Thread PID
+  sigset_t signal_mask;         // Signal mask
   /*
     Flag to indicate whether we are in poll or shutdown.
     A true value of flag indicates either the socket
@@ -411,11 +407,9 @@ struct Vio {
 #endif
   HANDLE hPipe{nullptr};
 #endif
+#ifdef HAVE_OPENSSL
   void *ssl_arg = {nullptr};
-  struct PSI_socket_locker *m_psi_read_locker = {nullptr};
-  PSI_socket_locker_state m_psi_read_state;
-  struct PSI_socket_locker *m_psi_write_locker = {nullptr};
-  PSI_socket_locker_state m_psi_write_state;
+#endif
 #if defined(_WIN32)
   HANDLE handle_file_map = {nullptr};
   char *handle_map = {nullptr};
@@ -449,6 +443,10 @@ struct Vio {
   Vio &operator=(Vio &&vio);
 };
 
+#ifdef HAVE_OPENSSL
 #define SSL_handle SSL *
+#else
+#define SSL_handle void *
+#endif
 
 #endif /* vio_violite_h_ */
