@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -689,11 +688,11 @@ class Query_event : public Binary_log_event {
     to the log.
   */
   Query_event(Log_event_type type_arg = QUERY_EVENT);
-  ~Query_event() override = default;
+  virtual ~Query_event() {}
 
 #ifndef HAVE_MYSYS
-  void print_event_info(std::ostream &info) override;
-  void print_long_info(std::ostream &info) override;
+  void print_event_info(std::ostream &info);
+  void print_long_info(std::ostream &info);
 #endif
 };
 
@@ -849,7 +848,7 @@ class User_var_event : public Binary_log_event {
     @param fde  An FDE event (see Rotate_event constructor for more info).
   */
   User_var_event(const char *buf, const Format_description_event *fde);
-  ~User_var_event() override;
+  virtual ~User_var_event();
   const char *name;
   unsigned int name_len;
   char *val;
@@ -859,8 +858,8 @@ class User_var_event : public Binary_log_event {
   bool is_null;
   unsigned char flags;
 #ifndef HAVE_MYSYS
-  void print_event_info(std::ostream &info) override;
-  void print_long_info(std::ostream &info) override;
+  void print_event_info(std::ostream &info);
+  void print_long_info(std::ostream &info);
   const char *get_value_type_string(Value_type type_arg) const {
     switch (type_arg) {
       case STRING_RESULT:
@@ -984,11 +983,11 @@ class Intvar_event : public Binary_log_event {
   Intvar_event(uint8_t type_arg, uint64_t val_arg)
       : Binary_log_event(INTVAR_EVENT), type(type_arg), val(val_arg) {}
 
-  ~Intvar_event() override = default;
+  ~Intvar_event() {}
 
 #ifndef HAVE_MYSYS
-  void print_event_info(std::ostream &info) override;
-  void print_long_info(std::ostream &info) override;
+  void print_event_info(std::ostream &info);
+  void print_long_info(std::ostream &info);
 #endif
 };
 
@@ -1064,8 +1063,8 @@ type_code as RAND_EVENT in the header object in Binary_log_event
   */
   Rand_event(const char *buf, const Format_description_event *fde);
 #ifndef HAVE_MYSYS
-  void print_event_info(std::ostream &info) override;
-  void print_long_info(std::ostream &info) override;
+  void print_event_info(std::ostream &info);
+  void print_long_info(std::ostream &info);
 #endif
 };
 }  // end namespace binary_log
