@@ -1,17 +1,16 @@
 # -*- cperl -*-
-# Copyright (c) 2007, 2025, Oracle and/or its affiliates.
+# Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0,
 # as published by the Free Software Foundation.
 #
-# This program is designed to work with certain software (including
+# This program is also distributed with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
 # documentation.  The authors of MySQL hereby grant you an additional
 # permission to link the program and your derivative works with the
-# separately licensed software that they have either included with
-# the program or referenced in the documentation.
+# separately licensed software that they have included with MySQL.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,7 +34,7 @@ use My::Find;
 use My::Platform;
 
 # Rules to run first of all
-my @pre_rules = (\&pre_check_no_hosts_in_cluster_config);
+my @pre_rules = ();
 
 sub get_basedir {
   my ($self, $group) = @_;
@@ -51,7 +50,7 @@ sub get_testdir {
   return $testdir;
 }
 
-# Retrieve build directory (which is different from basedir
+# Retrive build directory (which is different from basedir
 # in out-of-source build).
 sub get_bindir {
   if (defined $ENV{MTR_BINDIR}) {
@@ -67,6 +66,11 @@ sub fix_charset_dir {
                      "charsets");
 }
 
+sub fix_language {
+  my ($self, $config, $group_name, $group) = @_;
+  return my_find_dir($self->get_bindir($group), \@::share_locations);
+}
+
 sub fix_datadir {
   my ($self, $config, $group_name) = @_;
   my $vardir = $self->{ARGS}->{vardir};
@@ -101,12 +105,6 @@ sub fix_port {
   return $self->{HOSTS}->{$hostname}++;
 }
 
-sub fix_admin_port {
-  my ($self, $config, $group_name, $group) = @_;
-  my $hostname = $group->value('#host');
-  return $self->{HOSTS}->{$hostname}++;
-}
-
 sub fix_x_port {
   my ($self, $config, $group_name, $group) = @_;
   return $self->{ARGS}->{mysqlxbaseport}++;
@@ -120,23 +118,6 @@ sub fix_host {
   return $hosts[$host_no];
 }
 
-sub fix_cpubind {
-  my ($self, $config, $group_name, $group) = @_;
-  # Replace cpubind with #cpubind
-  my $value = $group->if_exist('cpubind');
-  if (defined $value) {
-    $group->remove('cpubind');
-  }
-  return $value;
-}
-
-sub fix_cluster_config_suffix {
-  my ($self, $config, $group_name, $group) = @_;
-
-  my ($process_type, $idx, $suffix) = split(/\./, $group_name);
-  return ".$suffix";
-}
-
 sub is_unique {
   my ($config, $name, $value) = @_;
 
@@ -188,7 +169,7 @@ sub fix_socket {
 
   # Make sure the socket path does not become longer then the path
   # which mtr uses to test if a new tmpdir should be created.
-  if (length($socket) > length("$dir/mysql*.NN.sock")) {
+  if (length($socket) > length("$dir/mysql_testsocket.sock")) {
     # Too long socket path, generate shorter based on port
     my $port = $group->value('port');
     my $group_prefix = substr($group_name, 0, index($group_name, '.'));
@@ -220,26 +201,6 @@ sub fix_log {
   return "$dir/mysqld.log";
 }
 
-sub fix_group_setting {
-  my ($self, $config, $group_name, $group, $option_name) = @_;
-  # Use value from [mysqld.X], [mysqld](if group exists) or 1
-  return $group->if_exist($option_name) ||
-         ($config->group_exists('mysqld') &&
-          $config->group('mysqld')->if_exist($option_name)) || 1;
-}
-
-sub fix_slow_query_log {
-  my ($self, $config, $group_name, $group) = @_;
-  return fix_group_setting($self, $config, $group_name,
-                           $group, 'slow_query_log');
-}
-
-sub fix_general_log {
-  my ($self, $config, $group_name, $group) = @_;
-  return fix_group_setting($self, $config, $group_name,
-                           $group, 'general_log');
-}
-
 sub fix_log_slow_queries {
   my ($self, $config, $group_name, $group) = @_;
   my $dir = dirname($group->value('datadir'));
@@ -293,21 +254,11 @@ sub fix_rsa_public_key {
   return "$std_data/rsa_public_key.pem";
 }
 
-sub fix_bind_address {
-  my ($self) = @_;
-  if ($self->{ARGS}->{bind_local}) {
-    # Listen only for incoming network connections from local machine
-    return "localhost";
-  }
-  return undef;
-}
-
 # Rules to run for each mysqld in the config
 #  - some rules depend on each other and thus need to be run
 #    in the order listed here
 my @mysqld_rules = (
   { '#abs_datadir'                                 => \&fix_abs_datadir },
-  { '#cpubind'                                     => \&fix_cpubind },
   { '#host'                                        => \&fix_host },
   { '#log-error'                                   => \&fix_log_error },
   { 'caching_sha2_password_private_key_path'       => \&fix_rsa_private_key },
@@ -317,12 +268,9 @@ my @mysqld_rules = (
   { 'character-sets-dir'                           => \&fix_charset_dir },
   { 'datadir'                                      => \&fix_datadir },
   { 'port'                                         => \&fix_port },
-  { 'admin-port'                                   => \&fix_admin_port },
-  { 'bind-address'                                 => \&fix_bind_address },
-  { 'general_log'                                  => \&fix_general_log },
+  { 'general_log'                                  => 1 },
   { 'general_log_file'                             => \&fix_log },
   { 'loose-mysqlx-port'                            => \&fix_x_port },
-  { 'loose-mysqlx-bind-address'                    => \&fix_bind_address },
   { 'loose-mysqlx-socket'                          => \&fix_x_socket },
   { 'loose-mysqlx-ssl'                             => \&fix_ssl_disabled },
   { 'loose-mysqlx-ssl-ca'                          => "" },
@@ -330,7 +278,7 @@ my @mysqld_rules = (
   { 'loose-mysqlx-ssl-key'                         => "" },
   { 'pid-file'                                     => \&fix_pidfile },
   { 'server-id'                                    => \&fix_server_id, },
-  { 'slow_query_log'                               => \&fix_slow_query_log },
+  { 'slow_query_log'                               => 1 },
   { 'slow_query_log_file'                          => \&fix_log_slow_queries },
   { 'socket'                                       => \&fix_socket },
   { 'ssl-ca'                                       => \&fix_ssl_ca },
@@ -383,15 +331,13 @@ sub fix_cluster_backup_dir {
 #  - will be run in order listed here
 my @ndb_mgmd_rules =
   ({ 'DataDir' => \&fix_cluster_dir },
-   { 'PortNumber' => \&fix_ndb_mgmd_port },
-   { '#cpubind'   => \&fix_cpubind },);
+   { 'PortNumber' => \&fix_ndb_mgmd_port },);
 
 # Rules to run for each ndbd in the config
 #  - will be run in order listed here
 my @ndbd_rules = ({ 'BackupDataDir' => \&fix_cluster_backup_dir },
                   { 'DataDir'       => \&fix_cluster_dir },
-                  { 'HostName'      => \&fix_host },
-                  { '#cpubind'      => \&fix_cpubind },);
+                  { 'HostName'      => \&fix_host },);
 
 # Rules to run for each memcached in the config
 #  - will be run in order listed here
@@ -411,8 +357,7 @@ my @client_rules = ();
 # Rules to run for [mysqltest] section
 #  - will be run in order listed here
 my @mysqltest_rules = ({ 'server-public-key-path' => \&fix_rsa_public_key },
-                       { 'ssl-mode'               => \&fix_ssl_disabled },
-                       { '#cpubind'               => \&fix_cpubind },);
+                       { 'ssl-mode'               => \&fix_ssl_disabled },);
 
 # Rules to run for [mysqlbinlog] section
 #  - will be run in order listed here
@@ -425,45 +370,6 @@ my @mysqlbinlog_rules = (
 #  - will be run in order listed here
 my @mysql_upgrade_rules = ();
 
-# For [cluster_config] section there should be no ndbd, ndb_mgmd, mysqld,
-# ndbapi options set.  These should only be set in cluster instance specific
-# sections, [cluster_config.X].  This is crucial when determining in the order
-# ndb_mgmd will enumerate node ids for nodes without explicit node id.
-sub pre_check_no_hosts_in_cluster_config {
-  my ($self, $config) = @_;
-
-  my $group = $config->group('cluster_config');
-  if (defined $group) {
-    if (defined $group->if_exist('ndbd') ||
-        defined $group->if_exist('ndb_mgmd') ||
-        defined $group->if_exist('mysqld') ||
-        defined $group->if_exist('ndbapi')) {
-      croak "Configuration error: Do not set ndbd, ndb_mgmd, mysqld, ndbapi ".
-            "in [cluster_config] section.  Use cluster specific sections ".
-            "[cluster_config.X] instead.";
-    }
-  }
-}
-
-# For each [cluster_config.X] record all allocated node ids (set using NodeId
-# property).  This will be used when determining what node id nodes with no
-# configured node id will get.
-sub track_allocated_nodeid {
-  my ($self, $config, $group_name, $group) = @_;
-
-  my (undef, $process_type, $idx, $suffix) = split(/\./, $group_name);
-
-  if (my $nodeid = $group->if_exist('NodeId')) {
-    if (!defined $suffix) {
-      croak "Configuration error: Do not set NodeId in general group ".
-            "[$group_name] but use instance specific group like ".
-            "[cluster_config.$process_type.1.cluster] instead.";
-    }
-    $config->{"cluster_config.$suffix"}->{"ALLOCATED NODEIDS"}->{$nodeid} = 1;
-  }
-  return;
-}
-
 # Generate a [client.<suffix>] group to be used for
 # connecting to [mysqld.<suffix>].
 sub post_check_client_group {
@@ -564,28 +470,8 @@ sub post_fix_mysql_cluster_section {
     $config->insert('mysql_cluster' . $group->suffix(),
                     'ndb_connectstring', $ndb_connectstring);
 
-    # Add ndb_connectstring to each ndbd connected to this
-    # cluster.
-    foreach my $ndbd ($config->like('cluster_config.ndbd.')) {
-      if ($ndbd->suffix() eq $group->suffix()) {
-        my $after = $ndbd->after('cluster_config.ndbd');
-        $config->insert("ndbd$after",
-                        'ndb_connectstring', $ndb_connectstring);
-      }
-    }
-
-    # Add ndb_connectstring to each ndb_mgmd connected to this
-    # cluster.
-    foreach my $ndb_mgmd ($config->like('cluster_config.ndb_mgmd.')) {
-      if ($ndb_mgmd->suffix() eq $group->suffix()) {
-        my $after = $ndb_mgmd->after('cluster_config.ndb_mgmd');
-        $config->insert("ndb_mgmd$after",
-                        'ndb_connectstring', $ndb_connectstring);
-      }
-    }
-
     # Add ndb_connectstring to each mysqld connected to this
-    # cluster.
+    # cluster
     foreach my $mysqld ($config->like('cluster_config.mysqld.')) {
       if ($mysqld->suffix() eq $group->suffix()) {
         my $after = $mysqld->after('cluster_config.mysqld');
@@ -635,23 +521,10 @@ sub run_section_rules {
 sub run_generate_sections_from_cluster_config {
   my ($self, $config) = @_;
 
+  my @options = ('ndb_mgmd', 'ndbd', 'mysqld', 'ndbapi');
+
   foreach my $group ($config->like('cluster_config\.\w*$')) {
 
-    # @options will be a list of ndbd, ndb_mgmd, mysqld, ndbapi ordered in the
-    # way that ndb_mgmd will assign node ids to nodes without id.
-    my @options;
-
-    foreach my $option ($group->options()) {
-      my $option_name = $option->name();
-      if ($option_name =~ /^(ndbd|ndb_mgmd|mysqld|ndbapi)$/)
-      {
-        push @options, $option_name;
-      }
-    }
-
-    # Keep track of next node id to use if not explicitly set.
-    my $next_nodeid = 1;
-
     # Keep track of current index per process type
     my %idxes;
     map { $idxes{$_} = 1; } @options;
@@ -670,7 +543,6 @@ sub run_generate_sections_from_cluster_config {
       $group->insert($option_name, join(",", @hosts));
 
       # Generate sections for each host
-      my $instances = @hosts;
       foreach my $host (@hosts) {
         my $idx    = $idxes{$option_name}++;
         my $suffix = $group->suffix();
@@ -679,38 +551,6 @@ sub run_generate_sections_from_cluster_config {
         $config->insert("cluster_config.$option_name.$idx$suffix",
                         "HostName", $host);
 
-        # Predict how ndbd_mgm will assign node id for current node.
-        my $node = $config->group("cluster_config.$option_name.$idx$suffix");
-        my $nodeid = $node->if_exist('NodeId');
-        if (!$nodeid) {
-          $nodeid = $next_nodeid;
-          while ($config->{"cluster_config$suffix"}->
-                          {"ALLOCATED NODEIDS"}->
-                          {$nodeid}) {
-            $nodeid++;
-          }
-        }
-        $next_nodeid = $nodeid + 1;
-
-        # For data node set matching command line option --ndb-nodeid.
-        # If prediction of node id is wrong, and node id for another node type
-        # is used, that will cause testcase to fail during setup.
-        if ($option_name eq 'ndbd') {
-          if ($instances > 1) {
-            $config->insert("$option_name.$idx$suffix",
-                            'ndb-nodeid', $nodeid);
-          }
-        }
-
-        if ($option_name eq 'ndb_mgmd') {
-          if ($instances > 1) {
-            $config->insert("$option_name.$idx$suffix",
-                            'ndb-nodeid', $nodeid);
-          }
-          $config->insert("$option_name.$idx$suffix",
-                          'cluster-config-suffix', $suffix);
-        }
-
         if ($option_name eq 'mysqld') {
           my $datadir =
             $self->fix_cluster_dir($config, "cluster_config.mysqld.$idx$suffix",
@@ -725,7 +565,7 @@ sub run_generate_sections_from_cluster_config {
 sub new_config {
   my ($class, $args) = @_;
 
-  my @required_args = ('basedir', 'baseport', 'vardir', 'template_path', 'testdir', 'tmpdir');
+  my @required_args = ('basedir', 'baseport', 'vardir', 'template_path');
 
   foreach my $required (@required_args) {
     croak "you must pass '$required'" unless defined $args->{$required};
@@ -763,17 +603,9 @@ sub new_config {
 
   $self->run_generate_sections_from_cluster_config($config);
 
-  $self->run_section_rules($config, 'cluster_config\.\w',
-                           ({ 'CODE' => \&track_allocated_nodeid }));
-
   $self->run_section_rules($config, 'cluster_config.ndb_mgmd.',
                            @ndb_mgmd_rules);
 
-  $self->run_section_rules($config, 'ndb_mgmd.',
-    ({ 'cluster-config-suffix' => \&fix_cluster_config_suffix },
-     { 'bind-address' => \&fix_bind_address },
-    ));
-
   $self->run_section_rules($config, 'cluster_config.ndbd', @ndbd_rules);
 
   $self->run_section_rules($config, 'mysqld.', @mysqld_rules);
