@@ -58,7 +58,7 @@
  *
  *      Michael Widenius:
  *        DBUG_DUMP       - To dump a block of memory.
- *        PUSH_FLAG "O"   - To be used instead of "o" if we
+ *        PUSH_FLAG "O"   - To be used insted of "o" if we
  *                          want flushing after each write
  *        PUSH_FLAG "A"   - as 'O', but we will append to the out file instead
  *                          of creating a new one.
@@ -90,8 +90,6 @@
 #include <sys/types.h>
 #include <time.h>
 
-#include <algorithm>
-
 #include "m_string.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
@@ -118,7 +116,7 @@
 #include <signal.h>
 #endif
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 
 /*
  *            Manifest constants which may be "tuned" if desired.
@@ -212,7 +210,7 @@ static bool init_done = false; /* Set to true when initialization done */
   Protected by @c THR_LOCK_init_settings.
 */
 static struct settings init_settings;
-static const char *db_process = nullptr; /* Pointer to process name; argv[0] */
+static const char *db_process = 0; /* Pointer to process name; argv[0] */
 
 struct CODE_STATE {
   const char *process; /* Pointer to process name; usually argv[0] */
@@ -283,7 +281,7 @@ static char *DbugMalloc(size_t size);
 static const char *BaseName(const char *pathname);
 static void Indent(CODE_STATE *cs, int indent);
 static void DbugFlush(CODE_STATE *);
-[[noreturn]] static void DbugExit(const char *why);
+static void DbugExit(const char *why) MY_ATTRIBUTE((noreturn));
 static const char *DbugStrTok(const char *s);
 static void DbugVfprintf(FILE *stream, const char *format, va_list args);
 
@@ -323,16 +321,15 @@ static CODE_STATE *code_state(void) {
 
   if (!init_done) {
     init_done = true;
-    native_mutex_init(&THR_LOCK_dbug, nullptr);
-    native_mutex_init(&THR_LOCK_gcov, nullptr);
+    native_mutex_init(&THR_LOCK_dbug, NULL);
+    native_mutex_init(&THR_LOCK_gcov, NULL);
     native_rw_init(&THR_LOCK_init_settings);
     memset(&init_settings, 0, sizeof(init_settings));
     init_settings.out_file = stderr;
     init_settings.flags = OPEN_APPEND;
   }
 
-  if (!(cs_ptr = my_thread_var_dbug()))
-    return nullptr; /* Thread not initialised */
+  if (!(cs_ptr = my_thread_var_dbug())) return 0; /* Thread not initialised */
   if (!(cs = *cs_ptr)) {
     cs = (CODE_STATE *)DbugMalloc(sizeof(*cs));
     memset(cs, 0, sizeof(*cs));
@@ -467,11 +464,11 @@ static int DbugParse(CODE_STATE *cs, const char *control) {
     stack->maxdepth = 0;
     stack->sub_level = 0;
     stack->out_file = stderr;
-    stack->prof_file = nullptr;
-    stack->functions = nullptr;
-    stack->p_functions = nullptr;
-    stack->keywords = nullptr;
-    stack->processes = nullptr;
+    stack->prof_file = NULL;
+    stack->functions = NULL;
+    stack->p_functions = NULL;
+    stack->keywords = NULL;
+    stack->processes = NULL;
   } else if (!stack->out_file) {
     stack->flags = stack->next->flags;
     stack->delay = stack->next->delay;
@@ -516,7 +513,7 @@ static int DbugParse(CODE_STATE *cs, const char *control) {
       case 'd':
         if (sign < 0 && control == end) {
           if (!is_shared(stack, keywords)) FreeList(stack->keywords);
-          stack->keywords = nullptr;
+          stack->keywords = NULL;
           stack->flags &= ~DEBUG_ON;
           break;
         }
@@ -526,13 +523,13 @@ static int DbugParse(CODE_STATE *cs, const char *control) {
           if (DEBUGGING) {
             stack->keywords = ListDel(stack->keywords, control, end);
             /* Turn off DEBUG_ON if it is last keyword to be removed. */
-            if (stack->keywords == nullptr) stack->flags &= ~DEBUG_ON;
+            if (stack->keywords == NULL) stack->flags &= ~DEBUG_ON;
           }
           break;
         }
 
         /* Do not add keyword if debugging all is enabled. */
-        if (!(DEBUGGING && stack->keywords == nullptr)) {
+        if (!(DEBUGGING && stack->keywords == NULL)) {
           stack->keywords = ListAdd(stack->keywords, control, end);
           stack->flags |= DEBUG_ON;
         }
@@ -540,7 +537,7 @@ static int DbugParse(CODE_STATE *cs, const char *control) {
         /* If debug all is enabled, make the keyword list empty. */
         if (sign == 1 && control == end) {
           FreeList(stack->keywords);
-          stack->keywords = nullptr;
+          stack->keywords = NULL;
         }
 
         break;
@@ -551,7 +548,7 @@ static int DbugParse(CODE_STATE *cs, const char *control) {
         f_used = 1;
         if (sign < 0 && control == end) {
           if (!is_shared(stack, functions)) FreeList(stack->functions);
-          stack->functions = nullptr;
+          stack->functions = NULL;
           break;
         }
         if (rel && is_shared(stack, functions))
@@ -594,7 +591,7 @@ static int DbugParse(CODE_STATE *cs, const char *control) {
       case 'A':
       case 'O':
         stack->flags |= FLUSH_ON_WRITE;
-        [[fallthrough]];
+        /* fall through */
       case 'a':
       case 'o':
         /* In case we already have an open file. */
@@ -611,12 +608,12 @@ static int DbugParse(CODE_STATE *cs, const char *control) {
         if (control != end)
           DBUGOpenFile(cs, control, end, stack->flags & OPEN_APPEND);
         else
-          DBUGOpenFile(cs, "-", nullptr, 0);
+          DBUGOpenFile(cs, "-", 0, 0);
         break;
       case 'p':
         if (sign < 0 && control == end) {
           if (!is_shared(stack, processes)) FreeList(stack->processes);
-          stack->processes = nullptr;
+          stack->processes = NULL;
           break;
         }
         if (rel && is_shared(stack, processes))
@@ -668,13 +665,13 @@ static int DbugParse(CODE_STATE *cs, const char *control) {
     /*
       Enforce nothing is shared with the global init_settings
     */
-    assert((stack->functions == nullptr) ||
+    assert((stack->functions == NULL) ||
            (stack->functions != init_settings.functions));
-    assert((stack->p_functions == nullptr) ||
+    assert((stack->p_functions == NULL) ||
            (stack->p_functions != init_settings.p_functions));
-    assert((stack->keywords == nullptr) ||
+    assert((stack->keywords == NULL) ||
            (stack->keywords != init_settings.keywords));
-    assert((stack->processes == nullptr) ||
+    assert((stack->processes == NULL) ||
            (stack->processes != init_settings.processes));
   }
 
@@ -683,10 +680,9 @@ static int DbugParse(CODE_STATE *cs, const char *control) {
   return !rel || f_used;
 }
 
-#define framep_trace_flag(cs, frp)                                       \
-  (frp                                           ? frp->level & TRACE_ON \
-   : (ListFlags(cs->stack->functions) & INCLUDE) ? 0                     \
-                                                 : (uint)TRACE_ON)
+#define framep_trace_flag(cs, frp) \
+  (frp ? frp->level & TRACE_ON     \
+       : (ListFlags(cs->stack->functions) & INCLUDE) ? 0 : (uint)TRACE_ON)
 
 static void FixTraceFlags_helper(CODE_STATE *cs, const char *func,
                                  struct _db_stack_frame_ *framep) {
@@ -723,7 +719,7 @@ static void FixTraceFlags(uint old_fflags, CODE_STATE *cs) {
     if we haven't started tracing yet, no call stack at all - we're safe.
   */
   framep = cs->framep;
-  if (framep == nullptr) return;
+  if (framep == 0) return;
 
   /*
     Ok, the tracing has started, call stack isn't empty.
@@ -857,7 +853,7 @@ void _db_push_(const char *control) {
 */
 
 int _db_is_pushed_() {
-  CODE_STATE *cs = nullptr;
+  CODE_STATE *cs = NULL;
   get_code_state_or_return false;
   return (cs->stack != &init_settings);
 }
@@ -958,11 +954,11 @@ void _db_pop_() {
       listp = listp->next_link;                      \
     }                                                \
   } while (0)
-#define int_to_buf(i)              \
-  do {                             \
-    char b[50];                    \
-    longlong10_to_str((i), b, 10); \
-    str_to_buf(b);                 \
+#define int_to_buf(i)         \
+  do {                        \
+    char b[50];               \
+    int10_to_str((i), b, 10); \
+    str_to_buf(b);            \
   } while (0)
 #define colon_to_buf                    \
   do {                                  \
@@ -1125,7 +1121,7 @@ void _db_enter_(const char *_func_, int func_len, const char *_file_,
   if (!((cs = code_state()))) {
     _stack_frame_->level =
         0; /* Set to avoid valgrind warnings if dbug is enabled later */
-    _stack_frame_->prev = nullptr;
+    _stack_frame_->prev = 0;
     return;
   }
   save_errno = errno;
@@ -1146,7 +1142,7 @@ void _db_enter_(const char *_func_, int func_len, const char *_file_,
     case ENABLE_TRACE:
       cs->framep->level |= TRACE_ON;
       if (!TRACING) break;
-      [[fallthrough]];
+      /* fall through */
     case DO_TRACE:
       if (TRACING) {
         if (!cs->locked) native_mutex_lock(&THR_LOCK_dbug);
@@ -1158,7 +1154,7 @@ void _db_enter_(const char *_func_, int func_len, const char *_file_,
       break;
     case DISABLE_TRACE:
       cs->framep->level &= ~TRACE_ON;
-      [[fallthrough]];
+      /* fall through */
     case DONT_TRACE:
       break;
   }
@@ -1223,7 +1219,7 @@ void _db_return_(uint _line_, struct _db_stack_frame_ *_stack_frame_) {
   cs->func = _stack_frame_->func;
   cs->func_len = _stack_frame_->func_len;
   cs->file = _stack_frame_->file;
-  if (cs->framep != nullptr) cs->framep = cs->framep->prev;
+  if (cs->framep != NULL) cs->framep = cs->framep->prev;
   errno = save_errno;
 
   unlock_stack(cs);
@@ -1383,6 +1379,7 @@ void _db_dump_(uint _line_, const char *keyword, const unsigned char *memory,
     DoPrefix(cs, _line_);
     if (TRACING) {
       Indent(cs, cs->level + 1);
+      pos = MY_MIN(MY_MAX(cs->level - cs->stack->sub_level, 0) * INDENT, 80);
     } else {
       fprintf(cs->stack->out_file, "%.*s: ", cs->func_len, cs->func);
     }
@@ -1484,7 +1481,7 @@ next:
     memcpy((*cur)->str, start, len);
     (*cur)->str[len] = 0;
     (*cur)->flags = todo | subdir;
-    (*cur)->next_link = nullptr;
+    (*cur)->next_link = 0;
   }
   return head;
 }
@@ -1517,8 +1514,8 @@ static struct link *ListCopy(struct link *orig) {
   struct link *head;
   size_t len;
 
-  head = nullptr;
-  while (orig != nullptr) {
+  head = NULL;
+  while (orig != NULL) {
     len = strlen(orig->str);
     new_malloc = (struct link *)DbugMalloc(sizeof(struct link) + len);
     memcpy(new_malloc->str, orig->str, len);
@@ -1555,7 +1552,7 @@ static struct link *ListCopy(struct link *orig) {
 static int InList(struct link *linkp, const char *cp) {
   int result;
 
-  for (result = MATCHED; linkp != nullptr; linkp = linkp->next_link) {
+  for (result = MATCHED; linkp != NULL; linkp = linkp->next_link) {
     if (!fnmatch(linkp->str, cp, 0)) return linkp->flags;
     if (!(linkp->flags & EXCLUDE)) result = NOT_MATCHED;
     if (linkp->flags & SUBDIR) result |= SUBDIR;
@@ -1572,7 +1569,7 @@ static int InList(struct link *linkp, const char *cp) {
 
 static uint ListFlags(struct link *linkp) {
   uint f;
-  for (f = 0; linkp != nullptr; linkp = linkp->next_link) f |= linkp->flags;
+  for (f = 0; linkp != NULL; linkp = linkp->next_link) f |= linkp->flags;
   return f;
 }
 
@@ -1688,10 +1685,10 @@ void _db_end_() {
   init_settings.maxdepth = 0;
   init_settings.delay = 0;
   init_settings.sub_level = 0;
-  init_settings.functions = nullptr;
-  init_settings.p_functions = nullptr;
-  init_settings.keywords = nullptr;
-  init_settings.processes = nullptr;
+  init_settings.functions = 0;
+  init_settings.p_functions = 0;
+  init_settings.keywords = 0;
+  init_settings.processes = 0;
   native_rw_unlock(&THR_LOCK_init_settings);
   FreeState(cs, &tmp, 0);
 }
@@ -1732,7 +1729,7 @@ static int DoTrace(CODE_STATE *cs) {
 
 FILE *_db_fp_(void) {
   CODE_STATE *cs;
-  get_code_state_or_return nullptr;
+  get_code_state_or_return NULL;
   return cs->stack->out_file;
 }
 
@@ -1798,7 +1795,7 @@ int _db_keyword_(CODE_STATE *cs, const char *keyword, int strict) {
 static void Indent(CODE_STATE *cs, int indent) {
   int count;
 
-  indent = std::max<int>(indent - 1 - cs->stack->sub_level, 0) * INDENT;
+  indent = MY_MAX(indent - 1 - cs->stack->sub_level, 0) * INDENT;
   for (count = 0; count < indent; count++) {
     if ((count % INDENT) == 0)
       fputc('|', cs->stack->out_file);
@@ -1827,7 +1824,7 @@ static void Indent(CODE_STATE *cs, int indent) {
 static void FreeList(struct link *linkp) {
   struct link *old;
 
-  while (linkp != nullptr) {
+  while (linkp != NULL) {
     old = linkp;
     linkp = linkp->next_link;
     free((void *)old);
@@ -1875,7 +1872,7 @@ static void DoPrefix(CODE_STATE *cs, uint _line_) {
 #else
     struct timeval tv;
     struct tm *tm_p;
-    if (gettimeofday(&tv, nullptr) != -1) {
+    if (gettimeofday(&tv, NULL) != -1) {
       if ((tm_p = localtime((const time_t *)&tv.tv_sec))) {
         (void)fprintf(cs->stack->out_file,
                       /* "%04d-%02d-%02d " */
@@ -1918,7 +1915,7 @@ static void DBUGOpenFile(CODE_STATE *cs, const char *name, const char *end,
                          int append) {
   FILE *fp;
 
-  if (name != nullptr) {
+  if (name != NULL) {
     if (end) {
       size_t len = end - name;
       memcpy(cs->stack->name, name, len);
@@ -1966,7 +1963,7 @@ static void DBUGOpenFile(CODE_STATE *cs, const char *name, const char *end,
  */
 
 static void DBUGCloseFile(CODE_STATE *cs, FILE *fp) {
-  if (fp != nullptr && fp != stderr && fp != stdout && fclose(fp) == EOF) {
+  if (fp != NULL && fp != stderr && fp != stdout && fclose(fp) == EOF) {
     native_mutex_lock(&THR_LOCK_dbug);
     (void)fprintf(cs->stack->out_file, ERR_CLOSE, cs->process);
     perror("");
@@ -2097,18 +2094,18 @@ static bool Writable(const char *pathname) {
     if (my_access(pathname, W_OK) == 0) granted = true;
   } else {
     lastslash = strrchr(const_cast<char *>(pathname), '/');
-    if (lastslash != nullptr)
+    if (lastslash != NULL)
       *lastslash = '\0';
     else
       pathname = ".";
     if (my_access(pathname, W_OK) == 0) granted = true;
-    if (lastslash != nullptr) *lastslash = '/';
+    if (lastslash != NULL) *lastslash = '/';
   }
   return granted;
 }
 
 /* flush dbug-stream, free mutex lock & wait delay */
-/* This is because some systems (MSDOS!!) don't flush the file header */
+/* This is because some systems (MSDOS!!) dosn't flush fileheader */
 /* and dbug-file isn't readable after a system crash !! */
 
 static void DbugFlush(CODE_STATE *cs) {
@@ -2122,7 +2119,7 @@ static void DbugFlush(CODE_STATE *cs) {
 /* For debugging */
 
 void _db_flush_() {
-  CODE_STATE *cs = nullptr;
+  CODE_STATE *cs = NULL;
   get_code_state_or_return;
   (void)fflush(cs->stack->out_file);
 }
@@ -2130,20 +2127,14 @@ void _db_flush_() {
 #ifndef _WIN32
 
 #ifdef HAVE_GCOV
-#if defined(__GNUC__) && __GNUC__ >= 11
-extern "C" void __gcov_dump();
-static void dump_gcov_data() { __gcov_dump(); }
-#else
 extern "C" void __gcov_flush();
-static void dump_gcov_data() { __gcov_flush(); }
-#endif
 #endif
 
 void _db_flush_gcov_() {
 #ifdef HAVE_GCOV
   // Gcov will assert() if we try to flush in parallel.
   native_mutex_lock(&THR_LOCK_gcov);
-  dump_gcov_data();
+  __gcov_flush();
   native_mutex_unlock(&THR_LOCK_gcov);
 #endif
 }
