@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -88,7 +88,7 @@ MY_MODE get_file_perm(ulong perm_flags) {
 bool my_chmod(const char *filename, ulong perm_flags, myf my_flags) {
   int ret_val;
   MY_MODE file_perm;
-  DBUG_ENTER("my_chmod");
+  DBUG_TRACE;
   DBUG_ASSERT(filename && filename[0]);
 
   file_perm = get_file_perm(perm_flags);
@@ -105,5 +105,5 @@ bool my_chmod(const char *filename, ulong perm_flags, myf my_flags) {
              my_strerror(errbuf, sizeof(errbuf), errno));
   }
 
-  DBUG_RETURN(ret_val ? true : false);
+  return ret_val ? true : false;
 }
