@@ -1,16 +1,15 @@
-/* Copyright (c) 2004, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2004, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -100,21 +99,21 @@ bool my_gethwaddr(uchar *to) {
   int res = 1;
   struct ifreq ifr;
   struct ifreq ifs[MAX_IFS];
-  struct ifreq *ifri = nullptr;
-  struct ifreq *ifend = nullptr;
+  struct ifreq *ifri = NULL;
+  struct ifreq *ifend = NULL;
 
   char zero_array[ETHER_ADDR_LEN] = {0};
   struct ifconf ifc;
 
   fd = socket(AF_INET, SOCK_DGRAM, 0);
-  if (fd < 0) return true;
+  if (fd < 0) return 1;
 
   /* Retrieve interfaces */
   ifc.ifc_len = sizeof(ifs);
   ifc.ifc_req = ifs;
   if (ioctl(fd, SIOCGIFCONF, &ifc) < 0) {
     close(fd);
-    return true;
+    return 1;
   }
 
   /* Initialize out parameter */
@@ -235,11 +234,11 @@ bool my_gethwaddr(uchar *to) {
 
 #else /* __FreeBSD__ || __linux__ || _WIN32 */
 /* just fail */
-bool my_gethwaddr(uchar *to [[maybe_unused]]) { return 1; }
+bool my_gethwaddr(uchar *to MY_ATTRIBUTE((unused))) { return 1; }
 #endif
 
 #else /* MAIN */
-int main(int argc [[maybe_unused]], char **argv) {
+int main(int argc MY_ATTRIBUTE((unused)), char **argv) {
   uchar mac[6];
   uint i;
   MY_INIT(argv[0]);
