@@ -1,16 +1,15 @@
-/* Copyright (c) 2002, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -35,7 +34,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
-#include <algorithm>
 #include <array>
 #include <bitset>
 #include <type_traits>
@@ -62,23 +60,25 @@ typedef void (*init_func_p)(const struct my_option *option, void *variable,
 my_error_reporter my_getopt_error_reporter = &my_message_local;
 
 static bool getopt_compare_strings(const char *, const char *, uint);
-static longlong getopt_ll(const char *, bool, const my_option *, int *);
-static ulonglong getopt_ull(const char *, bool, const my_option *, int *);
-static double getopt_double(const char *, bool, const my_option *, int *);
+static longlong getopt_ll(const char *arg, const struct my_option *optp,
+                          int *err);
+static ulonglong getopt_ull(const char *, const struct my_option *, int *);
+static double getopt_double(const char *arg, const struct my_option *optp,
+                            int *err);
 static void init_variables(const struct my_option *, init_func_p);
 static void init_one_value(const struct my_option *, void *, longlong);
 static void fini_one_value(const struct my_option *, void *, longlong);
-static int setval(const struct my_option *, void *, const char *, bool, bool);
+static int setval(const struct my_option *, void *, const char *, bool);
 static void setval_source(const struct my_option *, void *);
 static char *check_struct_option(char *cur_arg, char *key_name);
-static bool get_bool_int_argument(const char *argument, bool *error);
+static bool get_bool_argument(const char *argument, bool *error);
 
 /*
   The following three variables belong to same group and the number and
   order of their arguments must correspond to each other.
 */
 static const char *special_opt_prefix[] = {"skip",    "disable", "enable",
-                                           "maximum", "loose",   nullptr};
+                                           "maximum", "loose",   0};
 static const uint special_opt_prefix_lengths[] = {4, 7, 6, 7, 5, 0};
 enum enum_special_opt {
   OPT_SKIP,
@@ -93,19 +93,19 @@ static char enabled_my_option[] = "1";
 static char space_char[] = " ";
 
 /*
-   This is a flag that can be set in client programs. false means that
+   This is a flag that can be set in client programs. 0 means that
    my_getopt will not print error messages, but the client should do
    it by itself
 */
 
-bool my_getopt_print_errors = true;
+bool my_getopt_print_errors = 1;
 
 /*
-   This is a flag that can be set in client programs. true means that
+   This is a flag that can be set in client programs. 1 means that
    my_getopt will skip over options it does not know how to handle.
 */
 
-bool my_getopt_skip_unknown = false;
+bool my_getopt_skip_unknown = 0;
 
 static my_getopt_value getopt_get_addr;
 
@@ -113,16 +113,13 @@ void my_getopt_register_get_addr(my_getopt_value func_addr) {
   getopt_get_addr = func_addr;
 }
 
-bool is_key_cache_variable_suffix(std::string_view suffix) {
-  constexpr static std::array<std::string_view, 4> key_cache_components = {
+bool is_key_cache_variable_suffix(const char *suffix) {
+  static std::array<const char *, 4> key_cache_components = {
       {"key_buffer_size", "key_cache_block_size", "key_cache_division_limit",
        "key_cache_age_threshold"}};
 
-  for (auto component : key_cache_components) {
-    if (suffix.size() == component.size() &&
-        !native_strncasecmp(suffix.data(), component.data(), suffix.size()))
-      return true;
-  }
+  for (auto component : key_cache_components)
+    if (!my_strcasecmp(&my_charset_latin1, component, suffix)) return true;
 
   return false;
 }
@@ -141,8 +138,7 @@ bool is_key_cache_variable_suffix(std::string_view suffix) {
 */
 int handle_options(int *argc, char ***argv, const struct my_option *longopts,
                    my_get_one_option get_one_option) {
-  return my_handle_options(argc, argv, longopts, get_one_option, nullptr,
-                           false);
+  return my_handle_options(argc, argv, longopts, get_one_option, NULL, false);
 }
 
 union ull_dbl {
@@ -237,44 +233,26 @@ double getopt_ulonglong2double(ulonglong v) {
   @param [in] ignore_unknown_option When set to true, options are continued to
                                     be read even when unknown options are
                                     encountered.
-  @param [in] boolean_as_int        Parse boolean as integer value.
-                                    Mimic the logic of parsing booleans at
-  runtime: Instead of parsing
-  @code
-  bool_val = (str_val == '1' || str_val = 'ON') ?
-                true :
-                (str_val == '0' || str_val = 'OFF') ?
-                  false :
-                  false;
-  @endcode
-  do:
-  @code
-  bool_val == (str_val == 'OFF') ?
-                false :
-                ((str_val == 'ON') ?
-                  true :
-                  atoi(str_val) != 0)
-  @endcode
+
   @return error in case of ambiguous or unknown options,
           0 on success.
 */
-int my_handle_options2(int *argc, char ***argv,
-                       const struct my_option *longopts,
-                       my_get_one_option get_one_option,
-                       const char **command_list, bool ignore_unknown_option,
-                       bool boolean_as_int) {
+int my_handle_options(int *argc, char ***argv, const struct my_option *longopts,
+                      my_get_one_option get_one_option,
+                      const char **command_list, bool ignore_unknown_option) {
   uint argvpos = 0, length;
-  bool end_of_options = false, must_be_var, set_maximum_value, option_is_loose;
+  bool end_of_options = 0, must_be_var, set_maximum_value, option_is_loose;
   char **pos, **pos_end, *optend, *opt_str, key_name[FN_REFLEN];
-  char **arg_sep = nullptr, **persist_arg_sep = nullptr;
+  char **arg_sep = NULL, **persist_arg_sep = NULL;
   const struct my_option *optp;
   void *value;
-  bool is_cmdline_arg = true, is_persist_arg = true;
+  int error, i;
+  bool is_cmdline_arg = 1, is_persist_arg = 1;
   int opt_found;
 
   /* handle_options() assumes arg0 (program name) always exists */
-  assert(argc && *argc >= 1);
-  assert(argv && *argv);
+  DBUG_ASSERT(argc && *argc >= 1);
+  DBUG_ASSERT(argv && *argv);
   (*argc)--; /* Skip the program name */
   (*argv)++; /*      --- || ----      */
   init_variables(longopts, init_one_value);
@@ -286,7 +264,7 @@ int my_handle_options2(int *argc, char ***argv,
   for (pos = *argv, pos_end = pos + *argc; pos != pos_end; pos++) {
     if (my_getopt_is_args_separator(*pos)) {
       arg_sep = pos;
-      is_cmdline_arg = false;
+      is_cmdline_arg = 0;
       break;
     }
   }
@@ -295,7 +273,7 @@ int my_handle_options2(int *argc, char ***argv,
     for (pos = arg_sep, pos_end = (*argv + *argc); pos != pos_end; pos++) {
       if (my_getopt_is_ro_persist_args_separator(*pos)) {
         persist_arg_sep = pos;
-        is_persist_arg = false;
+        is_persist_arg = 0;
         break;
       }
     }
@@ -309,7 +287,7 @@ int my_handle_options2(int *argc, char ***argv,
     */
     pos = arg_sep + 1;
     while (*pos && pos != persist_arg_sep) {
-      update_variable_source((const char *)*pos, nullptr);
+      update_variable_source((const char *)*pos, NULL);
       ++pos;
     }
   }
@@ -333,9 +311,9 @@ int my_handle_options2(int *argc, char ***argv,
     char *cur_arg = *pos;
     opt_found = false;
     if (!is_cmdline_arg && (my_getopt_is_args_separator(cur_arg))) {
-      is_cmdline_arg = true;
+      is_cmdline_arg = 1;
 
-      /* save the separator too if skip unknown options  */
+      /* save the separator too if skip unkown options  */
       if (my_getopt_skip_unknown)
         (*argv)[argvpos++] = cur_arg;
       else
@@ -344,7 +322,7 @@ int my_handle_options2(int *argc, char ***argv,
     }
     /* skip persist args separator */
     if (!is_persist_arg && my_getopt_is_ro_persist_args_separator(cur_arg)) {
-      is_persist_arg = true;
+      is_persist_arg = 1;
       if (my_getopt_skip_unknown)
         (*argv)[argvpos++] = cur_arg;
       else
@@ -353,10 +331,10 @@ int my_handle_options2(int *argc, char ***argv,
     }
     if (cur_arg[0] == '-' && cur_arg[1] && !end_of_options) /* must be opt */
     {
-      char *argument = nullptr;
-      must_be_var = false;
-      set_maximum_value = false;
-      option_is_loose = false;
+      char *argument = 0;
+      must_be_var = 0;
+      set_maximum_value = 0;
+      option_is_loose = 0;
 
       cur_arg++;           /* skip '-' */
       if (*cur_arg == '-') /* check for long option, */
@@ -364,7 +342,7 @@ int my_handle_options2(int *argc, char ***argv,
         if (!*++cur_arg) /* skip the double dash */
         {
           /* '--' means end of options, look no further */
-          end_of_options = true;
+          end_of_options = 1;
           (*argc)--;
           continue;
         }
@@ -374,7 +352,7 @@ int my_handle_options2(int *argc, char ***argv,
         if (*optend == '=')
           optend++;
         else
-          optend = nullptr;
+          optend = 0;
 
         /*
          * For component system variables key_name is the component name and
@@ -387,10 +365,7 @@ int my_handle_options2(int *argc, char ***argv,
 
           if (!is_key_cache_variable_suffix(tmp_name.c_str())) {
             opt_str = cur_arg;
-            if (optend)
-              length = (uint)((optend - opt_str) - 1);
-            else
-              length = strlen(opt_str);
+            length = (uint)((optend - opt_str) - 1);
           }
         }
         /*
@@ -404,9 +379,8 @@ int my_handle_options2(int *argc, char ***argv,
             option with a special option prefix
           */
           if (!must_be_var) {
-            if (optend)
-              must_be_var = true; /* option is followed by an argument */
-            for (int i = 0; special_opt_prefix[i]; i++) {
+            if (optend) must_be_var = 1; /* option is followed by an argument */
+            for (i = 0; special_opt_prefix[i]; i++) {
               if (!getopt_compare_strings(special_opt_prefix[i], opt_str,
                                           special_opt_prefix_lengths[i]) &&
                   (opt_str[special_opt_prefix_lengths[i]] == '-' ||
@@ -416,11 +390,11 @@ int my_handle_options2(int *argc, char ***argv,
                 */
                 opt_str += special_opt_prefix_lengths[i] + 1;
                 length -= special_opt_prefix_lengths[i] + 1;
-                if (i == OPT_LOOSE) option_is_loose = true;
+                if (i == OPT_LOOSE) option_is_loose = 1;
                 if ((opt_found = findopt(opt_str, length, &optp))) {
                   switch (i) {
                     case OPT_SKIP:
-                    case OPT_DISABLE:
+                    case OPT_DISABLE: /* fall through */
                       /*
                         double negation is actually enable again,
                         for example: --skip-option=0 -> option = true
@@ -435,8 +409,8 @@ int my_handle_options2(int *argc, char ***argv,
                                    : enabled_my_option;
                       break;
                     case OPT_MAXIMUM:
-                      set_maximum_value = true;
-                      must_be_var = true;
+                      set_maximum_value = 1;
+                      must_be_var = 1;
                       break;
                   }
                   break; /* break from the inner loop, main loop continues */
@@ -483,14 +457,13 @@ int my_handle_options2(int *argc, char ***argv,
           }
           return EXIT_OPTION_DISABLED;
         }
-        {
-          int error = 0;
-          value =
-              optp->var_type & GET_ASK_ADDR
-                  ? (*getopt_get_addr)(key_name, strlen(key_name), optp, &error)
-                  : optp->value;
-          if (error) return error;
-        }
+        error = 0;
+        value =
+            optp->var_type & GET_ASK_ADDR
+                ? (*getopt_get_addr)(key_name, strlen(key_name), optp, &error)
+                : optp->value;
+        if (error) return error;
+
         if (optp->arg_type == NO_ARG) {
           /*
             Due to historical reasons GET_BOOL var_types still accepts arguments
@@ -505,7 +478,7 @@ int my_handle_options2(int *argc, char ***argv,
           }
           if ((optp->var_type & GET_TYPE_MASK) == GET_BOOL) {
             /*
-              Set bool to true if no argument or if the user has used
+              Set bool to 1 if no argument or if the user has used
               --enable-'option-name'.
               *optend was set to '0' if one used --disable-option
             */
@@ -513,10 +486,9 @@ int my_handle_options2(int *argc, char ***argv,
             if (!optend)
               *((bool *)value) = true;
             else {
-              bool ret = false;
-              bool error = false;
-              ret = boolean_as_int ? get_bool_int_argument(optend, &error)
-                                   : get_bool_argument(optend, &error);
+              bool ret = 0;
+              bool error = 0;
+              ret = get_bool_argument(optend, &error);
               if (error) {
                 my_getopt_error_reporter(WARNING_LEVEL,
                                          EE_OPTION_IGNORED_DUE_TO_INVALID_VALUE,
@@ -536,11 +508,10 @@ int my_handle_options2(int *argc, char ***argv,
           }
           argument = optend;
         } else if (optp->arg_type == REQUIRED_ARG && !optend) {
-          /*
-             Check if there are more arguments after this one,
-             Note: options loaded from config file that requires value
-             should always be in the form '--option=value'.
-           */
+          /* Check if there are more arguments after this one,
+       Note: options loaded from config file that requires value
+       should always be in the form '--option=value'.
+    */
           if (!is_cmdline_arg || !*++pos) {
             if (my_getopt_print_errors)
               my_getopt_error_reporter(ERROR_LEVEL, EE_OPTION_REQUIRES_ARGUMENT,
@@ -605,9 +576,8 @@ int my_handle_options2(int *argc, char ***argv,
                   /* the other loop will break, because *optend + 1 == 0 */
                 }
               }
-              int error;
-              if ((error = setval(optp, optp->value, argument,
-                                  set_maximum_value, boolean_as_int)))
+              if ((error =
+                       setval(optp, optp->value, argument, set_maximum_value)))
                 return error;
               if (get_one_option && get_one_option(optp->id, optp, argument))
                 return EXIT_UNSPECIFIED_ERROR;
@@ -623,10 +593,10 @@ int my_handle_options2(int *argc, char ***argv,
                 Hack the string "-XYZ" to make a "-YZ" substring in it,
                 and push that to the output as an unrecognized parameter.
               */
-              assert(optend > *pos);
-              assert(optend >= cur_arg);
-              assert(optend <= *pos + strlen(*pos));
-              assert(*optend);
+              DBUG_ASSERT(optend > *pos);
+              DBUG_ASSERT(optend >= cur_arg);
+              DBUG_ASSERT(optend <= *pos + strlen(*pos));
+              DBUG_ASSERT(*optend);
               optend--;
               optend[0] = '-'; /* replace 'X' or '-' by '-' */
               (*argv)[argvpos++] = optend;
@@ -647,9 +617,7 @@ int my_handle_options2(int *argc, char ***argv,
           (*argc)--; /* option handled (short), decrease argument count */
         continue;
       }
-      int error;
-      if ((error = setval(optp, value, argument, set_maximum_value,
-                          boolean_as_int)))
+      if ((error = setval(optp, value, argument, set_maximum_value)))
         return error;
       if (get_one_option && get_one_option(optp->id, optp, argument))
         return EXIT_UNSPECIFIED_ERROR;
@@ -681,17 +649,10 @@ done:
     Items in argv, before the destroyed one, are all non-option -arguments
     to the program, yet to be (possibly) handled.
   */
-  (*argv)[argvpos] = nullptr;
+  (*argv)[argvpos] = 0;
   return 0;
 }
 
-int my_handle_options(int *argc, char ***argv, const struct my_option *longopts,
-                      my_get_one_option get_one_option,
-                      const char **command_list, bool ignore_unknown_option) {
-  return my_handle_options2(argc, argv, longopts, get_one_option, command_list,
-                            ignore_unknown_option, false);
-}
-
 /**
  * This function should be called to print a warning message
  * if password string is specified on the command line.
@@ -738,7 +699,8 @@ static char *check_struct_option(char *cur_arg, char *key_name) {
      dot found, the option is not a struct option.
   */
   if ((equal_pos > dot_pos) && (space_pos > dot_pos)) {
-    size_t len = std::min(size_t(dot_pos - cur_arg), size_t(FN_REFLEN - 1));
+    size_t len = dot_pos - cur_arg;
+    set_if_smaller(len, FN_REFLEN - 1);
     strmake(key_name, cur_arg, len);
     return ++dot_pos;
   } else {
@@ -757,48 +719,19 @@ static char *check_struct_option(char *cur_arg, char *key_name) {
    @param [out] error Error indicator
    @return boolean value
 */
-bool get_bool_argument(const char *argument, bool *error) {
+static bool get_bool_argument(const char *argument, bool *error) {
   if (!my_strcasecmp(&my_charset_latin1, argument, "true") ||
       !my_strcasecmp(&my_charset_latin1, argument, "on") ||
       !my_strcasecmp(&my_charset_latin1, argument, "1"))
-    return true;
+    return 1;
 
   if (!my_strcasecmp(&my_charset_latin1, argument, "false") ||
       !my_strcasecmp(&my_charset_latin1, argument, "off") ||
       !my_strcasecmp(&my_charset_latin1, argument, "0"))
-    return false;
+    return 0;
 
   *error = true;
-  return false;
-}
-
-/**
-   Parse a boolean command line argument as the SQL interpreter does
-
-   "ON" and "TRUE" will return true,
-   "OFF" and FALSE" will return false;
-
-   Non-zero numeric values will return true, zero will return false.
-
-   @param argument The value argument
-   @param [out] error Error indicator
-   @return boolean value
-*/
-
-static bool get_bool_int_argument(const char *argument, bool *error) {
-  if (!my_strcasecmp(&my_charset_latin1, argument, "true") ||
-      !my_strcasecmp(&my_charset_latin1, argument, "on"))
-    return true;
-
-  if (!my_strcasecmp(&my_charset_latin1, argument, "false") ||
-      !my_strcasecmp(&my_charset_latin1, argument, "off"))
-    return false;
-
-  if (!strchr("0123456789+-", argument[0])) {
-    *error = true;
-    return false;
-  }
-  return atoi(argument) != 0;
+  return 0;
 }
 
 /**
@@ -816,9 +749,9 @@ static void setval_source(const struct my_option *opts, void *value) {
 */
 
 static int setval(const struct my_option *opts, void *value,
-                  const char *argument, bool set_maximum_value,
-                  bool boolean_as_int) {
+                  const char *argument, bool set_maximum_value) {
   int err = 0, res = 0;
+  bool error = 0;
   ulong var_type = opts->var_type & GET_TYPE_MASK;
 
   if (!argument) argument = enabled_my_option;
@@ -851,45 +784,35 @@ static int setval(const struct my_option *opts, void *value,
       return EXIT_NO_PTR_TO_VARIABLE;
     }
 
-    bool error = false;
     switch (var_type) {
       case GET_BOOL: /* If argument differs from 0, enable option, else disable
                       */
-        *((bool *)value) = boolean_as_int
-                               ? get_bool_int_argument(argument, &error)
-                               : get_bool_argument(argument, &error);
+        *((bool *)value) = get_bool_argument(argument, &error);
         if (error)
           my_getopt_error_reporter(WARNING_LEVEL,
                                    EE_INCORRECT_BOOLEAN_VALUE_FOR_OPTION,
                                    opts->name, argument);
         break;
       case GET_INT:
-        *((int *)value) =
-            (int)getopt_ll(argument, set_maximum_value, opts, &err);
+        *((int *)value) = (int)getopt_ll(argument, opts, &err);
         break;
       case GET_UINT:
-        *((uint *)value) =
-            (uint)getopt_ull(argument, set_maximum_value, opts, &err);
+        *((uint *)value) = (uint)getopt_ull(argument, opts, &err);
         break;
       case GET_LONG:
-        *((long *)value) =
-            (long)getopt_ll(argument, set_maximum_value, opts, &err);
+        *((long *)value) = (long)getopt_ll(argument, opts, &err);
         break;
       case GET_ULONG:
-        *((long *)value) =
-            (long)getopt_ull(argument, set_maximum_value, opts, &err);
+        *((long *)value) = (long)getopt_ull(argument, opts, &err);
         break;
       case GET_LL:
-        *((longlong *)value) =
-            getopt_ll(argument, set_maximum_value, opts, &err);
+        *((longlong *)value) = getopt_ll(argument, opts, &err);
         break;
       case GET_ULL:
-        *((ulonglong *)value) =
-            getopt_ull(argument, set_maximum_value, opts, &err);
+        *((ulonglong *)value) = getopt_ull(argument, opts, &err);
         break;
       case GET_DOUBLE:
-        *((double *)value) =
-            getopt_double(argument, set_maximum_value, opts, &err);
+        *((double *)value) = getopt_double(argument, opts, &err);
         break;
       case GET_STR:
       case GET_PASSWORD:
@@ -927,8 +850,7 @@ static int setval(const struct my_option *opts, void *value,
           *(ulong *)value = type - 1;
       } break;
       case GET_SET:
-        *(static_cast<ulonglong *>(value)) =
-            find_typeset(argument, opts->typelib, &err);
+        *((ulonglong *)value) = find_typeset(argument, opts->typelib, &err);
         if (err) {
           /* Accept an integer representation of the set */
           char *endptr;
@@ -937,19 +859,19 @@ static int setval(const struct my_option *opts, void *value,
             res = EXIT_ARGUMENT_INVALID;
             goto ret;
           };
-          *static_cast<ulonglong *>(value) = arg;
+          *(ulonglong *)value = arg;
           err = 0;
         }
         break;
       case GET_FLAGSET: {
-        const char *flag_error;
+        const char *error;
         uint error_len;
 
-        *(static_cast<ulonglong *>(value)) = find_set_from_flags(
-            opts->typelib, opts->typelib->count,
-            *static_cast<ulonglong *>(value), opts->def_value, argument,
-            strlen(argument), &flag_error, &error_len);
-        if (flag_error) {
+        *((ulonglong *)value) =
+            find_set_from_flags(opts->typelib, opts->typelib->count,
+                                *(ulonglong *)value, opts->def_value, argument,
+                                (uint)strlen(argument), &error, &error_len);
+        if (error) {
           res = EXIT_ARGUMENT_INVALID;
           goto ret;
         };
@@ -1001,15 +923,15 @@ int findopt(const char *optpat, uint length, const struct my_option **opt_res) {
   function: compare_strings
 
   Works like strncmp, other than 1.) considers '-' and '_' the same.
-  2.) Returns true if strings differ, false if they are equal
+  2.) Returns -1 if strings differ, 0 if they are equal
 */
 
 bool getopt_compare_strings(const char *s, const char *t, uint length) {
   char const *end = s + length;
   for (; s != end; s++, t++) {
-    if ((*s != '-' ? *s : '_') != (*t != '-' ? *t : '_')) return true;
+    if ((*s != '-' ? *s : '_') != (*t != '-' ? *t : '_')) return 1;
   }
-  return false;
+  return 0;
 }
 
 /*
@@ -1055,7 +977,7 @@ LLorULL eval_num_suffix(const char *argument, int *error,
     if (static_cast<long long>(num) == LLONG_MIN)
       errno = ERANGE;  // This will overflow
     else
-      num = -1 * num;
+      num = -num;
   }
 
   unsigned long long ull_num = num;
@@ -1121,7 +1043,7 @@ LLorULL eval_num_suffix(const char *argument, int *error,
     *error = 1;
     return 0;
   }
-  if (is_negative) return -1 * result;
+  if (is_negative) return -result;
   return result;
 }
 
@@ -1142,14 +1064,10 @@ template ulonglong eval_num_suffix<ulonglong>(const char *, int *,
   In case of an error, set error value in *err.
 */
 
-static longlong getopt_ll(const char *arg, bool set_maximum_value,
-                          const my_option *optp, int *err) {
+static longlong getopt_ll(const char *arg, const struct my_option *optp,
+                          int *err) {
   longlong num = eval_num_suffix<longlong>(arg, err, optp->name);
-  if (set_maximum_value && *err == 0 &&
-      *static_cast<longlong *>(optp->value) > num) {
-    *static_cast<longlong *>(optp->value) = num;
-  }
-  return getopt_ll_limit_value(num, optp, nullptr);
+  return getopt_ll_limit_value(num, optp, NULL);
 }
 
 /**
@@ -1172,7 +1090,7 @@ ulonglong max_of_int_range(int var_type) {
     case GET_ULL:
       return ULLONG_MAX;
     default:
-      assert(0);
+      DBUG_ASSERT(0);
       return 0;
   }
 }
@@ -1234,13 +1152,13 @@ static inline bool is_negative_num(const char *num) {
   values.
 */
 
-static ulonglong getopt_ull(const char *arg, bool set_maximum_value,
-                            const my_option *optp, int *err) {
+static ulonglong getopt_ull(const char *arg, const struct my_option *optp,
+                            int *err) {
   char buf[255];
   ulonglong num;
 
   /* If a negative number is specified as a value for the option. */
-  if (arg == nullptr || is_negative_num(arg) == true) {
+  if (arg == NULL || is_negative_num(arg) == true) {
     num = (ulonglong)optp->min_value;
     my_getopt_error_reporter(WARNING_LEVEL,
                              EE_ADJUSTED_ULONGLONG_VALUE_FOR_OPTION, optp->name,
@@ -1248,11 +1166,7 @@ static ulonglong getopt_ull(const char *arg, bool set_maximum_value,
   } else
     num = eval_num_suffix<ulonglong>(arg, err, optp->name);
 
-  if (set_maximum_value && *err == 0 &&
-      *static_cast<ulonglong *>(optp->value) > num) {
-    *static_cast<ulonglong *>(optp->value) = num;
-  }
-  return getopt_ull_limit_value(num, optp, nullptr);
+  return getopt_ull_limit_value(num, optp, NULL);
 }
 
 ulonglong getopt_ull_limit_value(ulonglong num, const struct my_option *optp,
@@ -1320,7 +1234,7 @@ double getopt_double_limit_value(double num, const struct my_option *optp,
 }
 
 /*
-  Get double value within ranges
+  Get double value withing ranges
 
   Evaluates and returns the value that user gave as an argument to a variable.
 
@@ -1328,11 +1242,11 @@ double getopt_double_limit_value(double num, const struct my_option *optp,
     decimal value of arg
 
     In case of an error, prints an error message and sets *err to
-    EXIT_ARGUMENT_INVALID.  Otherwise err is not touched.
+    EXIT_ARGUMENT_INVALID.  Otherwise err is not touched
 */
 
-static double getopt_double(const char *arg, bool set_maximum_value,
-                            const my_option *optp, int *err) {
+static double getopt_double(const char *arg, const struct my_option *optp,
+                            int *err) {
   double num;
   int error;
   const char *end = arg + 1000; /* Big enough as *arg is \0 terminated */
@@ -1343,11 +1257,7 @@ static double getopt_double(const char *arg, bool set_maximum_value,
     *err = EXIT_ARGUMENT_INVALID;
     return 0.0;
   }
-  if (set_maximum_value && *err == 0 &&
-      *static_cast<double *>(optp->value) > num) {
-    *static_cast<double *>(optp->value) = num;
-  }
-  return getopt_double_limit_value(num, optp, nullptr);
+  return getopt_double_limit_value(num, optp, NULL);
 }
 
 /*
@@ -1367,30 +1277,29 @@ static void init_one_value(const struct my_option *option, void *variable,
       *((bool *)variable) = (bool)value;
       break;
     case GET_INT:
-      *((int *)variable) =
-          (int)getopt_ll_limit_value((int)value, option, nullptr);
+      *((int *)variable) = (int)getopt_ll_limit_value((int)value, option, NULL);
       break;
     case GET_ENUM:
       *((ulong *)variable) = (ulong)value;
       break;
     case GET_UINT:
       *((uint *)variable) =
-          (uint)getopt_ull_limit_value((uint)value, option, nullptr);
+          (uint)getopt_ull_limit_value((uint)value, option, NULL);
       break;
     case GET_LONG:
       *((long *)variable) =
-          (long)getopt_ll_limit_value((long)value, option, nullptr);
+          (long)getopt_ll_limit_value((long)value, option, NULL);
       break;
     case GET_ULONG:
       *((ulong *)variable) =
-          (ulong)getopt_ull_limit_value((ulong)value, option, nullptr);
+          (ulong)getopt_ull_limit_value((ulong)value, option, NULL);
       break;
     case GET_LL:
-      *((longlong *)variable) = getopt_ll_limit_value(value, option, nullptr);
+      *((longlong *)variable) = getopt_ll_limit_value(value, option, NULL);
       break;
     case GET_ULL:
       *((ulonglong *)variable) =
-          getopt_ull_limit_value((ulonglong)value, option, nullptr);
+          getopt_ull_limit_value((ulonglong)value, option, NULL);
       break;
     case GET_SET:
     case GET_FLAGSET:
@@ -1438,12 +1347,12 @@ static void init_one_value(const struct my_option *option, void *variable,
 */
 
 static void fini_one_value(const struct my_option *option, void *variable,
-                           longlong value [[maybe_unused]]) {
+                           longlong value MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   switch ((option->var_type & GET_TYPE_MASK)) {
     case GET_STR_ALLOC:
       my_free(*((char **)variable));
-      *((char **)variable) = nullptr;
+      *((char **)variable) = NULL;
       break;
     default: /* dummy default to avoid compiler warnings */
       break;
@@ -1481,7 +1390,7 @@ static void init_variables(const struct my_option *options,
     if (options->u_max_value)
       init_one_value(options, options->u_max_value, options->max_value);
     value = (options->var_type & GET_ASK_ADDR
-                 ? (*getopt_get_addr)("", 0, options, nullptr)
+                 ? (*getopt_get_addr)("", 0, options, 0)
                  : options->value);
     if (value) init_one_value(options, value, options->def_value);
   }
@@ -1550,20 +1459,10 @@ void my_print_help(const struct my_option *options) {
       const char *comment = optp->comment, *end = strend(comment);
 
       while ((uint)(end - comment) > comment_space) {
-        bool had_space;
-        // find the last space before the limit to break the comment on
-        for (line_end = comment + comment_space;
-             line_end > comment && *line_end != ' '; line_end--) {
+        for (line_end = comment + comment_space; *line_end != ' '; line_end--) {
         }
-        // if no space is found break at the limit - 1 (for the new line)
-        if (line_end == comment && *comment != ' ') {
-          line_end = comment + comment_space - 1;
-          had_space = false;
-        } else
-          had_space = true;
         for (; comment != line_end; comment++) putchar(*comment);
-        if (had_space)
-          comment++; /* skip the space, as a newline will take it's place now */
+        comment++; /* skip the space, as a newline will take it's place now */
         putchar('\n');
         for (col = 0; col < name_space; col++) putchar(' ');
       }
@@ -1616,15 +1515,15 @@ void my_print_variables_ex(const struct my_option *options, FILE *file) {
   putc('\n', file);
 
   for (optp = options; optp->name; optp++) {
-    void *value = (optp->var_type & GET_ASK_ADDR
-                       ? (*getopt_get_addr)("", 0, optp, nullptr)
-                       : optp->value);
+    void *value =
+        (optp->var_type & GET_ASK_ADDR ? (*getopt_get_addr)("", 0, optp, 0)
+                                       : optp->value);
     if (value) {
       length = print_name(optp, file);
       for (; length < name_space; length++) putc(' ', file);
       switch ((optp->var_type & GET_TYPE_MASK)) {
         case GET_SET:
-          if (!(llvalue = *static_cast<ulonglong *>(value)))
+          if (!(llvalue = *(ulonglong *)value))
             fprintf(file, "%s\n", "");
           else
             for (nr = 0; llvalue && nr < optp->typelib->count;
@@ -1635,7 +1534,7 @@ void my_print_variables_ex(const struct my_option *options, FILE *file) {
             }
           break;
         case GET_FLAGSET:
-          llvalue = *static_cast<ulonglong *>(value);
+          llvalue = *(ulonglong *)value;
           for (nr = 0; llvalue && nr < optp->typelib->count;
                nr++, llvalue >>= 1) {
             fprintf(file, "%s%s=", (nr ? "," : ""),
@@ -1649,7 +1548,7 @@ void my_print_variables_ex(const struct my_option *options, FILE *file) {
           break;
         case GET_STR:
         case GET_PASSWORD:
-        case GET_STR_ALLOC:
+        case GET_STR_ALLOC: /* fall through */
           fprintf(file, "%s\n",
                   *((char **)value) ? *((char **)value) : "(No default value)");
           break;
@@ -1672,7 +1571,7 @@ void my_print_variables_ex(const struct my_option *options, FILE *file) {
           fprintf(file, "%s\n", llstr(*((longlong *)value), buff));
           break;
         case GET_ULL:
-          ullstr(*(static_cast<ulonglong *>(value)), buff);
+          longlong2str(*((ulonglong *)value), buff, 10);
           fprintf(file, "%s\n", buff);
           break;
         case GET_DOUBLE:
