@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -31,7 +30,7 @@
   @file mysys/my_lib.cc
 */
 
-/* TODO: check for overrun of memory for names. */
+/* TODO: check for overun of memory for names. */
 
 #include <errno.h>
 #include <string.h>
@@ -72,8 +71,9 @@ void my_dirend(MY_DIR *buffer) {
     Entries_array *array = pointer_cast<Entries_array *>(
         (char *)buffer + ALIGN_SIZE(sizeof(MY_DIR)));
     array->~Entries_array();
-    destroy((MEM_ROOT *)((char *)buffer + ALIGN_SIZE(sizeof(MY_DIR)) +
-                         ALIGN_SIZE(sizeof(Entries_array))));
+    free_root((MEM_ROOT *)((char *)buffer + ALIGN_SIZE(sizeof(MY_DIR)) +
+                           ALIGN_SIZE(sizeof(Entries_array))),
+              MYF(0));
     my_free(buffer);
   }
 } /* my_dirend */
@@ -84,19 +84,19 @@ static char *directory_file_name(char *dst, const char *src);
 
 MY_DIR *my_dir(const char *path, myf MyFlags) {
   char *buffer;
-  MY_DIR *result = nullptr;
+  MY_DIR *result = 0;
   FILEINFO finfo;
   Entries_array *dir_entries_storage;
   MEM_ROOT *names_storage;
   DIR *dirp;
   char tmp_path[FN_REFLEN + 2], *tmp_file;
-  void *rawmem = nullptr;
+  void *rawmem = NULL;
 
   DBUG_TRACE;
   DBUG_PRINT("my", ("path: '%s' MyFlags: %d", path, MyFlags));
 
   dirp = opendir(directory_file_name(tmp_path, path));
-  if (dirp == nullptr ||
+  if (dirp == NULL ||
       !(buffer = static_cast<char *>(
             my_malloc(key_memory_MY_DIR,
                       ALIGN_SIZE(sizeof(MY_DIR)) +
@@ -106,11 +106,13 @@ MY_DIR *my_dir(const char *path, myf MyFlags) {
 
   rawmem = pointer_cast<Entries_array *>(buffer + ALIGN_SIZE(sizeof(MY_DIR)));
   dir_entries_storage = new (rawmem) Entries_array(key_memory_MY_DIR);
-  names_storage = new (buffer + ALIGN_SIZE(sizeof(MY_DIR)) +
-                       ALIGN_SIZE(sizeof(Entries_array)))
-      MEM_ROOT(key_memory_MY_DIR, NAMES_START_SIZE);
-  ;
-  /* MY_DIR structure is allocated and completely initialized at this point */
+  names_storage = (MEM_ROOT *)(buffer + ALIGN_SIZE(sizeof(MY_DIR)) +
+                               ALIGN_SIZE(sizeof(Entries_array)));
+
+  init_alloc_root(key_memory_MY_DIR, names_storage, NAMES_START_SIZE,
+                  NAMES_START_SIZE);
+
+  /* MY_DIR structure is allocated and completly initialized at this point */
   result = (MY_DIR *)buffer;
 
   tmp_file = strend(tmp_path);
@@ -127,7 +129,7 @@ MY_DIR *my_dir(const char *path, myf MyFlags) {
       (void)my_stat(tmp_path, finfo.mystat, MyFlags);
       if (!(finfo.mystat->st_mode & MY_S_IREAD)) continue;
     } else
-      finfo.mystat = nullptr;
+      finfo.mystat = NULL;
 
     if (dir_entries_storage->push_back(finfo)) goto error;
   }
@@ -149,10 +151,12 @@ error:
   if (dirp) (void)closedir(dirp);
   my_dirend(result);
   if (MyFlags & (MY_FAE | MY_WME)) {
-    MyOsError(my_errno(), EE_DIR, MYF(0), path);
+    char errbuf[MYSYS_STRERROR_SIZE];
+    my_error(EE_DIR, MYF(0), path, my_errno(),
+             my_strerror(errbuf, sizeof(errbuf), my_errno()));
   }
-  return nullptr;
-}
+  return (MY_DIR *)NULL;
+} /* my_dir */
 
 /*
  * Convert from directory name to filename.
@@ -164,7 +168,7 @@ error:
 static char *directory_file_name(char *dst, const char *src) {
   /* Process as Unix format: just remove test the final slash. */
   char *end;
-  assert(strlen(src) < (FN_REFLEN + 1));
+  DBUG_ASSERT(strlen(src) < (FN_REFLEN + 1));
 
   if (src[0] == 0) src = "."; /* Use empty as current */
   end = my_stpnmov(dst, src, FN_REFLEN + 1);
@@ -192,7 +196,7 @@ MY_DIR *my_dir(const char *path, myf MyFlags) {
   struct _finddata_t find;
   ushort mode;
   char tmp_path[FN_REFLEN], *tmp_file, attrib;
-  __int64 handle = -1;
+  __int64 handle;
   void *rawmem = NULL;
 
   DBUG_TRACE;
@@ -218,11 +222,13 @@ MY_DIR *my_dir(const char *path, myf MyFlags) {
 
   rawmem = buffer + ALIGN_SIZE(sizeof(MY_DIR));
   dir_entries_storage = new (rawmem) Entries_array(key_memory_MY_DIR);
-  names_storage = new (buffer + ALIGN_SIZE(sizeof(MY_DIR)) +
-                       ALIGN_SIZE(sizeof(Entries_array)))
-      MEM_ROOT(key_memory_MY_DIR, NAMES_START_SIZE);
+  names_storage = pointer_cast<MEM_ROOT *>(buffer + ALIGN_SIZE(sizeof(MY_DIR)) +
+                                           ALIGN_SIZE(sizeof(Entries_array)));
 
-  /* MY_DIR structure is allocated and completely initialized at this point */
+  init_alloc_root(key_memory_MY_DIR, names_storage, NAMES_START_SIZE,
+                  NAMES_START_SIZE);
+
+  /* MY_DIR structure is allocated and completly initialized at this point */
   result = (MY_DIR *)buffer;
 
   if ((handle = _findfirst(tmp_path, &find)) == -1L) {
@@ -277,11 +283,13 @@ error:
   set_my_errno(errno);
   if (handle != -1) _findclose(handle);
   my_dirend(result);
-  if (MyFlags & (MY_FAE | MY_WME)) {
-    MyOsError(my_errno(), EE_DIR, MYF(0), path);
+  if (MyFlags & MY_FAE + MY_WME) {
+    char errbuf[MYSYS_STRERROR_SIZE];
+    my_error(EE_DIR, MYF(0), path, errno,
+             my_strerror(errbuf, sizeof(errbuf), errno));
   }
-  return nullptr;
-}
+  return (MY_DIR *)NULL;
+} /* my_dir */
 
 #endif /* _WIN32 */
 
@@ -300,11 +308,11 @@ int my_fstat(File Filedes, MY_STAT *stat_area) {
 #endif
 }
 
-MY_STAT *my_stat(const char *path, MY_STAT *stat_area, myf MyFlags) {
+MY_STAT *my_stat(const char *path, MY_STAT *stat_area, myf my_flags) {
   DBUG_TRACE;
-  assert(stat_area != nullptr);
+  DBUG_ASSERT(stat_area != nullptr);
   DBUG_PRINT("my", ("path: '%s'  stat_area: %p  MyFlags: %d", path, stat_area,
-                    MyFlags));
+                    my_flags));
 
 #ifndef _WIN32
   if (!stat(path, stat_area)) return stat_area;
@@ -315,8 +323,10 @@ MY_STAT *my_stat(const char *path, MY_STAT *stat_area, myf MyFlags) {
   DBUG_PRINT("error", ("Got errno: %d from stat", errno));
   set_my_errno(errno);
 
-  if (MyFlags & (MY_FAE | MY_WME)) {
-    MyOsError(my_errno(), EE_STAT, MYF(0), path);
+  if (my_flags & (MY_FAE + MY_WME)) {
+    char errbuf[MYSYS_STRERROR_SIZE];
+    my_error(EE_STAT, MYF(0), path, my_errno(),
+             my_strerror(errbuf, sizeof(errbuf), my_errno()));
   }
   return nullptr;
-}
+} /* my_stat */
