@@ -57,9 +57,16 @@ int my_rename(const char *from, const char *to, myf MyFlags) {
 #endif
     set_my_errno(errno);
     error = -1;
-    if (MyFlags & (MY_FAE | MY_WME)) {
-      MyOsError(my_errno(), EE_LINK, MYF(0), from, to);
+    if (MyFlags & (MY_FAE + MY_WME)) {
+      char errbuf[MYSYS_STRERROR_SIZE];
+      my_error(EE_LINK, MYF(0), from, to, my_errno(),
+               my_strerror(errbuf, sizeof(errbuf), my_errno()));
     }
+  } else if (MyFlags & MY_SYNC_DIR) {
+    /* do only the needed amount of syncs: */
+    if (my_sync_dir_by_file(from, MyFlags) ||
+        (strcmp(from, to) && my_sync_dir_by_file(to, MyFlags)))
+      error = -1;
   }
   return error;
-}
+} /* my_rename */
