@@ -1,16 +1,15 @@
-/* Copyright (c) 2001, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -39,23 +38,14 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
-#include <cstdint>
-#include <string_view>
-#if defined(__linux__) || defined(__sun) || defined(__FreeBSD__)
-#include <sys/syscall.h>
-#ifdef HAVE_EXT_BACKTRACE
-#include <backtrace/stacktrace.hpp>
-#endif
+#ifdef __linux__
+#include <syscall.h>
 #endif
 #include <time.h>
 
-#include <algorithm>
-#include <cinttypes>
-
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "my_stacktrace.h"
-#include "template_utils.h"
 
 #ifndef _WIN32
 #include <signal.h>
@@ -86,8 +76,8 @@ static const char *heap_start;
 extern char *__bss_start;
 #endif /* __linux */
 
-static inline bool ptr_sane(const char *p [[maybe_unused]],
-                            const char *heap_end [[maybe_unused]]) {
+static inline bool ptr_sane(const char *p MY_ATTRIBUTE((unused)),
+                            const char *heap_end MY_ATTRIBUTE((unused))) {
 #ifdef __linux__
   return p && p >= heap_start && p <= heap_end;
 #else
@@ -142,7 +132,7 @@ static int safe_print_str(const char *addr, int max_len) {
 
   /* Read up to the maximum number of bytes. */
   while (total) {
-    count = std::min(sizeof(buf), total);
+    count = MY_MIN(sizeof(buf), total);
 
     if ((nbytes = pread(fd, buf, count, offset)) < 0) {
       /* Just in case... */
@@ -195,63 +185,18 @@ void my_safe_puts_stderr(const char *val, size_t max_len) {
   my_safe_printf_stderr("%s", "\n");
 }
 
-#ifdef HAVE_EXT_BACKTRACE
-static size_t my_write_stderr(const std::string_view &sv) {
-  return my_write_stderr(std::data(sv), std::size(sv));
-}
-
-void my_print_stacktrace(const uchar *stack_bottom, ulong thread_stack) {
-  my_safe_printf_stderr("stack_bottom = %p thread_stack 0x%lx\n", stack_bottom,
-                        thread_stack);
-
-  struct cookie_t {
-    int index;
-  };
-  auto print_callback = [](void *cookie, uintptr_t pc, const char *filename,
-                           int lineno, const char *function) {
-    auto idx = static_cast<cookie_t *>(cookie)->index++;
-    my_safe_printf_stderr(" #%d 0x%lx ", idx, (unsigned long)pc);
-    my_write_stderr(function == nullptr ? "<unknown>" : function);
-    if (filename != nullptr) {
-      std::string_view filename_sv{filename};
-      constexpr std::string_view parent_dir{"../"};
-      if (auto pos = filename_sv.rfind(parent_dir);
-          pos != std::string_view::npos) {
-        filename_sv.remove_prefix(pos + std::size(parent_dir));
-      }
-      constexpr std::string_view mysql_dir{"mysql/"};
-      if (auto pos = filename_sv.find(mysql_dir);
-          pos != std::string_view::npos) {
-        filename_sv.remove_prefix(pos + std::size(mysql_dir));
-      }
-      my_write_stderr(" at ");
-      my_write_stderr(filename_sv);
-      my_safe_printf_stderr(":%d", lineno);
-    }
-    my_write_stderr("\n");
-
-    return 0;
-  };
-  auto error_callback = [](void *, const char *msg, int errnum) {
-    my_safe_printf_stderr("libbacktrace: %s", msg);
-    if (errnum > 0) my_safe_printf_stderr(": %d", errnum);
-    my_safe_printf_stderr("\n");
-  };
-  cookie_t cookie{.index = 0};
-  stacktrace::full(1, print_callback, error_callback, &cookie);
-}
-#elif defined(HAVE_BACKTRACE)
+#if defined(HAVE_BACKTRACE)
 
 #ifdef HAVE_ABI_CXA_DEMANGLE
 
 #include <cxxabi.h>
 
 static char *my_demangle(const char *mangled_name, int *status) {
-  return abi::__cxa_demangle(mangled_name, nullptr, nullptr, status);
+  return abi::__cxa_demangle(mangled_name, NULL, NULL, status);
 }
 
 static bool my_demangle_symbol(char *line) {
-  char *demangled = nullptr;
+  char *demangled = NULL;
 #ifdef __APPLE__  // OS X formatting of stacktraces is different from Linux
   char *begin = strstr(line, "_Z");
   char *end = begin ? strchr(begin, ' ') : NULL;
@@ -268,23 +213,37 @@ static bool my_demangle_symbol(char *line) {
     }
   }
   if (demangled) my_safe_printf_stderr("%s %s %s\n", line, demangled, end + 1);
-#else             // !__APPLE__
+#elif defined(__SUNPRO_CC)  // Solaris has different formatting .....
+  char *begin = strchr(line, '\'');
+  char *end = begin ? strchr(begin, '+') : NULL;
+  if (begin && end) {
+    *begin++ = *end++ = '\0';
+    int status = 0;
+    demangled = my_demangle(begin, &status);
+    if (!demangled || status) {
+      demangled = NULL;
+      begin[-1] = ' ';
+      end[-1] = '+';
+    }
+  }
+  if (demangled) my_safe_printf_stderr("%s %s+%s\n", line, demangled, end);
+#else                       // !__APPLE__ and !__SUNPRO_CC
   char *begin = strchr(line, '(');
-  char *end = begin ? strchr(begin, '+') : nullptr;
+  char *end = begin ? strchr(begin, '+') : NULL;
 
   if (begin && end) {
     *begin++ = *end++ = '\0';
     int status;
     demangled = my_demangle(begin, &status);
     if (!demangled || status) {
-      demangled = nullptr;
+      demangled = NULL;
       begin[-1] = '(';
       end[-1] = '+';
     }
   }
   if (demangled) my_safe_printf_stderr("%s(%s+%s\n", line, demangled, end);
 #endif
-  bool ret = (demangled == nullptr);
+  bool ret = (demangled == NULL);
   free(demangled);
   return (ret);
 }
@@ -320,8 +279,9 @@ void my_print_stacktrace(const uchar *stack_bottom, ulong thread_stack) {
     if (demangled) free(demangled);
   }
 #endif
+
   void *addrs[128];
-  char **strings = nullptr;
+  char **strings = NULL;
   int n = backtrace(addrs, array_elements(addrs));
   my_safe_printf_stderr("stack_bottom = %p thread_stack 0x%lx\n", stack_bottom,
                         thread_stack);
@@ -335,7 +295,8 @@ void my_print_stacktrace(const uchar *stack_bottom, ulong thread_stack) {
     backtrace_symbols_fd(addrs, n, fileno(stderr));
   }
 }
-#endif /* HAVE_EXT_BACKTRACE || HAVE_BACKTRACE */
+
+#endif /* HAVE_BACKTRACE */
 #endif /* HAVE_STACKTRACE */
 
 /* Produce a core for the thread */
@@ -356,7 +317,7 @@ void my_write_core(int sig) {
 #pragma comment(lib, "dbghelp")
 #endif
 
-static thread_local EXCEPTION_POINTERS *exception_ptrs;
+static EXCEPTION_POINTERS *exception_ptrs;
 
 #define MODULE64_SIZE_WINXP 576
 #define STACKWALK_MAX_FRAMES 64
@@ -386,13 +347,13 @@ static void get_symbol_path(char *path, size_t size) {
   HANDLE hSnap;
   char *envvar;
   char *p;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   static char pdb_debug_dir[MAX_PATH + 7];
 #endif
 
   path[0] = '\0';
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   /*
     Add "debug" subdirectory of the application directory, sometimes PDB will
     placed here by installation.
@@ -448,32 +409,22 @@ static void get_symbol_path(char *path, size_t size) {
 #define SYMOPT_NO_PROMPTS 0
 #endif
 
-void my_print_stacktrace(const uchar * /* stack_bottom */,
-                         ulong /* thread_stack */) {
+void my_print_stacktrace(const uchar *unused1, ulong unused2) {
   HANDLE hProcess = GetCurrentProcess();
   HANDLE hThread = GetCurrentThread();
-  static IMAGEHLP_MODULE64 module{};
-  module.SizeOfStruct = sizeof(module);
-
+  static IMAGEHLP_MODULE64 module = {sizeof(module)};
   static IMAGEHLP_SYMBOL64_PACKAGE package;
   DWORD64 addr;
   DWORD machine;
   int i;
   CONTEXT context;
-  STACKFRAME64 frame{};
+  STACKFRAME64 frame = {0};
   static char symbol_path[MAX_SYMBOL_PATH];
 
-  if (exception_ptrs) {
-    /* Copy context, as stackwalking on original will unwind the stack */
-    context = *(exception_ptrs->ContextRecord);
-  } else {
-    /*
-      We are to print stack outside the signal handler, let's just capture the
-      current context.
-    */
-    RtlCaptureContext(&context);
-  }
+  if (!exception_ptrs) return;
 
+  /* Copy context, as stackwalking on original will unwind the stack */
+  context = *(exception_ptrs->ContextRecord);
   /*Initialize symbols.*/
   SymSetOptions(SYMOPT_LOAD_LINES | SYMOPT_NO_PROMPTS | SYMOPT_DEFERRED_LOADS |
                 SYMOPT_DEBUG);
@@ -483,19 +434,13 @@ void my_print_stacktrace(const uchar * /* stack_bottom */,
   /*Prepare stackframe for the first StackWalk64 call*/
   frame.AddrFrame.Mode = frame.AddrPC.Mode = frame.AddrStack.Mode =
       AddrModeFlat;
-#if (defined _M_IX86)
-  machine = IMAGE_FILE_MACHINE_I386;
-  frame.AddrFrame.Offset = context.Ebp;
-  frame.AddrPC.Offset = context.Eip;
-  frame.AddrStack.Offset = context.Esp;
-#elif (defined _M_X64)
+#if (defined _M_X64)
   machine = IMAGE_FILE_MACHINE_AMD64;
   frame.AddrFrame.Offset = context.Rbp;
   frame.AddrPC.Offset = context.Rip;
   frame.AddrStack.Offset = context.Rsp;
 #else
   /*There is currently no need to support IA64*/
-  /* Warning C4068: unknown pragma 'error' */
 #pragma error("unsupported architecture")
 #endif
 
@@ -506,9 +451,7 @@ void my_print_stacktrace(const uchar * /* stack_bottom */,
   for (i = 0; i < STACKWALK_MAX_FRAMES; i++) {
     DWORD64 function_offset = 0;
     DWORD line_offset = 0;
-    IMAGEHLP_LINE64 line{};
-    line.SizeOfStruct = sizeof(line);
-
+    IMAGEHLP_LINE64 line = {sizeof(line)};
     BOOL have_module = false;
     BOOL have_symbol = false;
     BOOL have_source = false;
@@ -522,7 +465,7 @@ void my_print_stacktrace(const uchar * /* stack_bottom */,
         SymGetSymFromAddr64(hProcess, addr, &function_offset, &(package.sym));
     have_source = SymGetLineFromAddr64(hProcess, addr, &line_offset, &line);
 
-    my_safe_printf_stderr("%llx    ", addr);
+    my_safe_printf_stderr("%p    ", addr);
     if (have_module) {
       char *base_image_name = strrchr(module.ImageName, '\\');
       if (base_image_name)
@@ -543,7 +486,7 @@ void my_print_stacktrace(const uchar * /* stack_bottom */,
         base_file_name++;
       else
         base_file_name = line.FileName;
-      my_safe_printf_stderr("[%s:%lu]", base_file_name, line.LineNumber);
+      my_safe_printf_stderr("[%s:%u]", base_file_name, line.LineNumber);
     }
     my_safe_printf_stderr("%s", "\n");
   }
@@ -554,7 +497,7 @@ void my_print_stacktrace(const uchar * /* stack_bottom */,
   file name is constructed from executable name plus
   ".dmp" extension
 */
-void my_write_core(int /* sig */) {
+void my_write_core(int unused) {
   char path[MAX_PATH];
   // See comment below for clarification about size of dump_fname
   char dump_fname[MAX_PATH + 1 + 10 + 4 + 1] = "core.dmp";
@@ -570,7 +513,7 @@ void my_write_core(int /* sig */) {
     // 1 byte for termitated \0. Such size of output buffer guarantees
     // that there is enough space to place a result of string formatting
     // performed by snprintf().
-    snprintf(dump_fname, sizeof(dump_fname), "%s.%lu.dmp", module_name,
+    snprintf(dump_fname, sizeof(dump_fname), "%s.%u.dmp", module_name,
              GetCurrentProcessId());
   }
   my_create_minidump(dump_fname, 0, 0);
@@ -609,12 +552,12 @@ void my_create_minidump(const char *name, HANDLE process, DWORD pid) {
       my_safe_printf_stderr("Minidump written to %s\n",
                             _fullpath(path, name, sizeof(path)) ? path : name);
     } else {
-      my_safe_printf_stderr("MiniDumpWriteDump() failed, last error %lu\n",
+      my_safe_printf_stderr("MiniDumpWriteDump() failed, last error %d\n",
                             GetLastError());
     }
     CloseHandle(hFile);
   } else {
-    my_safe_printf_stderr("CreateFile(%s) failed, last error %lu\n", name,
+    my_safe_printf_stderr("CreateFile(%s) failed, last error %d\n", name,
                           GetLastError());
   }
 }
@@ -846,7 +789,7 @@ void my_safe_print_system_time() {
   secs = utc_time.wSecond;
 #else
   /* Using time() instead of my_time() to avoid looping */
-  const time_t curr_time = time(nullptr);
+  const time_t curr_time = time(NULL);
   /* Calculate time of day */
   const long tmins = curr_time / 60;
   const long thrs = tmins / 60;
