@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,7 +27,7 @@
 
 /**
   @file mysys/thr_lock.cc
-Read and write locks for Posix threads. Every thread must acquire
+Read and write locks for Posix threads. All tread must acquire
 all locks it needs through thr_multi_lock() to avoid dead-locks.
 A lock consists of a master lock (THR_LOCK), and lock instances
 (THR_LOCK_DATA).
@@ -107,7 +107,6 @@ lock at the same time as multiple read locks.
 #include "mysql/psi/psi_stage.h"
 #include "mysql/psi/psi_table.h"
 #include "mysys/mysys_priv.h"
-#include "template_utils.h"
 #include "thr_lock.h"
 #include "thr_mutex.h"
 
@@ -121,8 +120,8 @@ enum thr_lock_type thr_upgraded_concurrent_insert_lock = TL_WRITE;
 LIST *thr_lock_thread_list; /* List of threads in use */
 ulong max_write_lock_count = ~(ulong)0L;
 
-static void (*before_lock_wait)(void) = nullptr;
-static void (*after_lock_wait)(void) = nullptr;
+static void (*before_lock_wait)(void) = 0;
+static void (*after_lock_wait)(void) = 0;
 
 void thr_set_lock_wait_callback(void (*before_wait)(void),
                                 void (*after_wait)(void)) {
@@ -207,7 +206,7 @@ static void check_locks(THR_LOCK *lock, const char *where,
       for (data = lock->read.data; data; data = data->next) {
         if ((int)data->type == (int)TL_READ_NO_INSERT) count++;
         /* Protect against infinite loop. */
-        assert(count <= lock->read_no_write_count);
+        DBUG_ASSERT(count <= lock->read_no_write_count);
       }
       if (count != lock->read_no_write_count) {
         found_errors++;
@@ -337,17 +336,17 @@ void thr_lock_info_init(THR_LOCK_INFO *info, my_thread_id thread_id,
 void thr_lock_data_init(THR_LOCK *lock, THR_LOCK_DATA *data, void *param) {
   data->lock = lock;
   data->type = TL_UNLOCK;
-  data->owner = nullptr; /* no owner yet */
+  data->owner = 0; /* no owner yet */
   data->status_param = param;
-  data->cond = nullptr;
+  data->cond = 0;
 }
 
 static inline bool has_old_lock(THR_LOCK_DATA *data, THR_LOCK_INFO *owner) {
   for (; data; data = data->next) {
     if (thr_lock_owner_equal(data->owner, owner))
-      return true; /* Already locked by thread */
+      return 1; /* Already locked by thread */
   }
-  return false;
+  return 0;
 }
 
 static void wake_up_waiters(THR_LOCK *lock);
@@ -366,8 +365,8 @@ static enum enum_thr_lock_result wait_for_lock(struct st_lock_list *wait,
     One can use this to signal when a thread is going to wait for a lock.
     See debug_sync.cc.
 
-    Beware of waiting for a signal here. The lock has acquired its mutex.
-    While waiting on a signal here, the locking thread could not acquire
+    Beware of waiting for a signal here. The lock has aquired its mutex.
+    While waiting on a signal here, the locking thread could not aquire
     the mutex to release the lock. One could lock up the table
     completely.
 
@@ -376,8 +375,8 @@ static enum enum_thr_lock_result wait_for_lock(struct st_lock_list *wait,
     if not, it calls wait_for_lock(). Here it unlocks the table lock
     while waiting on a condition. The sync point is located before this
     wait for condition. If we have a waiting action here, we hold the
-    table locks mutex all the time. Any attempt to look at the table
-    lock from another thread blocks it immediately on lock->mutex. This
+    the table locks mutex all the time. Any attempt to look at the table
+    lock by another thread blocks it immediately on lock->mutex. This
     can easily become an unexpected and unobvious blockage. So be
     warned: Do not request a WAIT_FOR action for the 'wait_for_lock'
     sync point unless you really know what you do.
@@ -395,7 +394,7 @@ static enum enum_thr_lock_result wait_for_lock(struct st_lock_list *wait,
   /* Set up control struct to allow others to abort locks */
   data->cond = owner->suspend;
 
-  enter_cond_hook(nullptr, data->cond, &data->lock->mutex,
+  enter_cond_hook(NULL, data->cond, &data->lock->mutex,
                   &stage_waiting_for_table_level_lock, &old_stage, __func__,
                   __FILE__, __LINE__);
 
@@ -409,11 +408,11 @@ static enum enum_thr_lock_result wait_for_lock(struct st_lock_list *wait,
     ourselves to call it before_lock_wait once before starting to wait
     and once after the thread has exited the wait loop.
    */
-  if ((!is_killed_hook(nullptr) || in_wait_list) && before_lock_wait)
+  if ((!is_killed_hook(NULL) || in_wait_list) && before_lock_wait)
     (*before_lock_wait)();
 
   set_timespec(&wait_timeout, lock_wait_timeout);
-  while (!is_killed_hook(nullptr) || in_wait_list) {
+  while (!is_killed_hook(NULL) || in_wait_list) {
     int rc =
         mysql_cond_timedwait(data->cond, &data->lock->mutex, &wait_timeout);
     /*
@@ -426,7 +425,7 @@ static enum enum_thr_lock_result wait_for_lock(struct st_lock_list *wait,
       Order of checks below is important to not report about timeout
       if the predicate is true.
     */
-    if (data->cond == nullptr) {
+    if (data->cond == 0) {
       DBUG_PRINT("thr_lock", ("lock granted/aborted"));
       break;
     }
@@ -467,7 +466,7 @@ static enum enum_thr_lock_result wait_for_lock(struct st_lock_list *wait,
   }
   mysql_mutex_unlock(&data->lock->mutex);
 
-  exit_cond_hook(nullptr, &old_stage, __func__, __FILE__, __LINE__);
+  exit_cond_hook(NULL, &old_stage, __func__, __FILE__, __LINE__);
 
   return result;
 }
@@ -481,8 +480,8 @@ enum enum_thr_lock_result thr_lock(THR_LOCK_DATA *data, THR_LOCK_INFO *owner,
   MYSQL_TABLE_WAIT_VARIABLES(locker, state) /* no ';' */
   DBUG_TRACE;
 
-  data->next = nullptr;
-  data->cond = nullptr; /* safety */
+  data->next = 0;
+  data->cond = 0; /* safety */
   data->type = lock_type;
   data->owner = owner; /* Must be reset ! */
 
@@ -516,8 +515,8 @@ enum enum_thr_lock_result thr_lock(THR_LOCK_DATA *data, THR_LOCK_INFO *owner,
            |\  = READ_WITH_SHARED_LOCKS
            \   = READ
 
-        + = Request can be satisfied.
-        - = Request cannot be satisfied.
+        + = Request can be satisified.
+        - = Request cannot be satisified.
 
         READ_NO_INSERT and WRITE_ALLOW_WRITE should in principle
         be incompatible. Before this could have caused starvation of
@@ -615,10 +614,10 @@ enum enum_thr_lock_result thr_lock(THR_LOCK_DATA *data, THR_LOCK_INFO *owner,
         tries to update t1, is an example of statement which requests two
         different types of write lock on the same table).
       */
-      assert(!has_old_lock(lock->write.data, data->owner) ||
-             ((lock_type <= lock->write.data->type ||
-               (lock_type == TL_WRITE &&
-                lock->write.data->type == TL_WRITE_LOW_PRIORITY))));
+      DBUG_ASSERT(!has_old_lock(lock->write.data, data->owner) ||
+                  ((lock_type <= lock->write.data->type ||
+                    (lock_type == TL_WRITE &&
+                     lock->write.data->type == TL_WRITE_LOW_PRIORITY))));
 
       if ((lock_type == TL_WRITE_ALLOW_WRITE && !lock->write_wait.data &&
            lock->write.data->type == TL_WRITE_ALLOW_WRITE) ||
@@ -644,11 +643,11 @@ enum enum_thr_lock_result thr_lock(THR_LOCK_DATA *data, THR_LOCK_INFO *owner,
     } else {
       DBUG_PRINT("info", ("write_wait.data: %p", lock->write_wait.data));
       if (!lock->write_wait.data) { /* no scheduled write locks */
-        bool concurrent_insert = false;
+        bool concurrent_insert = 0;
         if (lock_type == TL_WRITE_CONCURRENT_INSERT) {
-          concurrent_insert = true;
+          concurrent_insert = 1;
           if ((*lock->check_status)(data->status_param)) {
-            concurrent_insert = false;
+            concurrent_insert = 0;
             data->type = lock_type = thr_upgraded_concurrent_insert_lock;
           }
         }
@@ -673,7 +672,7 @@ enum enum_thr_lock_result thr_lock(THR_LOCK_DATA *data, THR_LOCK_INFO *owner,
     wait_queue = &lock->write_wait;
   }
   /* Can't get lock yet;  Wait for it */
-  result = wait_for_lock(wait_queue, data, owner, false, lock_wait_timeout);
+  result = wait_for_lock(wait_queue, data, owner, 0, lock_wait_timeout);
   MYSQL_END_TABLE_LOCK_WAIT(locker);
   return result;
 end:
@@ -719,10 +718,10 @@ static inline void free_all_read_locks(THR_LOCK *lock,
     DBUG_PRINT("lock",
                ("giving read lock to thread: 0x%x", data->owner->thread_id));
     /* purecov: end */
-    data->cond = nullptr; /* Mark thread free */
+    data->cond = 0; /* Mark thread free */
     mysql_cond_signal(cond);
   } while ((data = data->next));
-  *lock->read_wait.last = nullptr;
+  *lock->read_wait.last = 0;
   if (!lock->read_wait.data) lock->write_lock_count = 0;
   check_locks(lock, "after giving read locks", 0);
 }
@@ -787,7 +786,7 @@ static void wake_up_waiters(THR_LOCK *lock) {
             DBUG_PRINT(
                 "info",
                 ("Freeing all read_locks because of max_write_lock_count"));
-            free_all_read_locks(lock, false);
+            free_all_read_locks(lock, 0);
             goto end;
           }
         }
@@ -798,7 +797,7 @@ static void wake_up_waiters(THR_LOCK *lock) {
             lock->write_wait.last = data->prev;
           (*lock->write.last) = data; /* Put in execute list */
           data->prev = lock->write.last;
-          data->next = nullptr;
+          data->next = 0;
           lock->write.last = &data->next;
           if (data->type == TL_WRITE_CONCURRENT_INSERT &&
               (*lock->check_status)(data->status_param))
@@ -809,7 +808,7 @@ static void wake_up_waiters(THR_LOCK *lock) {
           /* purecov: end */
           {
             mysql_cond_t *cond = data->cond;
-            data->cond = nullptr;    /* Mark thread free */
+            data->cond = 0;          /* Mark thread free */
             mysql_cond_signal(cond); /* Start waiting thread */
           }
           if (data->type != TL_WRITE_ALLOW_WRITE || !lock->write_wait.data ||
@@ -838,7 +837,7 @@ static void wake_up_waiters(THR_LOCK *lock) {
       if (lock_type == TL_WRITE_CONCURRENT_INSERT &&
           (*lock->check_status)(data->status_param)) {
         data->type = TL_WRITE; /* Upgrade lock */
-        if (lock->read_wait.data) free_all_read_locks(lock, false);
+        if (lock->read_wait.data) free_all_read_locks(lock, 0);
         goto end;
       }
       do {
@@ -850,8 +849,8 @@ static void wake_up_waiters(THR_LOCK *lock) {
         (*lock->write.last) = data; /* Put in execute list */
         data->prev = lock->write.last;
         lock->write.last = &data->next;
-        data->next = nullptr;    /* Only one write lock */
-        data->cond = nullptr;    /* Mark thread free */
+        data->next = 0;          /* Only one write lock */
+        data->cond = 0;          /* Mark thread free */
         mysql_cond_signal(cond); /* Start waiting thread */
       } while (lock_type == TL_WRITE_ALLOW_WRITE &&
                (data = lock->write_wait.data) &&
@@ -860,7 +859,7 @@ static void wake_up_waiters(THR_LOCK *lock) {
         free_all_read_locks(lock, (lock_type == TL_WRITE_CONCURRENT_INSERT ||
                                    lock_type == TL_WRITE_ALLOW_WRITE));
     } else if (!data && lock->read_wait.data)
-      free_all_read_locks(lock, false);
+      free_all_read_locks(lock, 0);
   }
 end:
   check_locks(lock, "after waking up waiters", 0);
@@ -868,7 +867,7 @@ end:
 
 /*
 ** Get all locks in a specific order to avoid dead-locks
-** Sort according to lock position and put write_locks before read_locks if
+** Sort acording to lock position and put write_locks before read_locks if
 ** lock on same lock.
 */
 
@@ -997,11 +996,8 @@ void thr_multi_unlock(THR_LOCK_DATA **data, uint count) {
     if ((*pos)->type != TL_UNLOCK)
       thr_unlock(*pos);
     else {
-      DBUG_PRINT("lock", ("Free lock: data: %p  lock: %p", *pos, (*pos)->lock));
-      if ((*pos)->owner) {
-        DBUG_PRINT("lock",
-                   ("Free lock: thread: 0x%x", (*pos)->owner->thread_id));
-      }
+      DBUG_PRINT("lock", ("Free lock: data: %p  thread: 0x%x  lock: %p", *pos,
+                          (*pos)->owner->thread_id, (*pos)->lock));
     }
   }
 }
@@ -1024,7 +1020,7 @@ void thr_abort_locks_for_thread(THR_LOCK *lock, my_thread_id thread_id) {
       data->type = TL_UNLOCK; /* Mark killed */
       /* It's safe to signal the cond first: we're still holding the mutex. */
       mysql_cond_signal(data->cond);
-      data->cond = nullptr; /* Removed from list */
+      data->cond = 0; /* Removed from list */
 
       if (((*data->prev) = data->next))
         data->next->prev = data->prev;
@@ -1038,7 +1034,7 @@ void thr_abort_locks_for_thread(THR_LOCK *lock, my_thread_id thread_id) {
       DBUG_PRINT("info", ("Aborting write-wait lock"));
       data->type = TL_UNLOCK;
       mysql_cond_signal(data->cond);
-      data->cond = nullptr;
+      data->cond = 0;
 
       if (((*data->prev) = data->next))
         data->next->prev = data->prev;
@@ -1050,6 +1046,52 @@ void thr_abort_locks_for_thread(THR_LOCK *lock, my_thread_id thread_id) {
   mysql_mutex_unlock(&lock->mutex);
 }
 
+/*
+  Downgrade a WRITE_* to a lower WRITE level
+  SYNOPSIS
+    thr_downgrade_write_lock()
+    in_data                   Lock data of thread downgrading its lock
+    new_lock_type             New write lock type
+  RETURN VALUE
+    NONE
+  DESCRIPTION
+    This can be used to downgrade a lock already owned. When the downgrade
+    occurs also other waiters, both readers and writers can be allowed to
+    start.
+    The previous lock is often TL_WRITE_ONLY but can also be
+    TL_WRITE. The normal downgrade variants are:
+    TL_WRITE_ONLY => TL_WRITE after a short exclusive lock while holding a
+    write table lock
+    TL_WRITE_ONLY => TL_WRITE_ALLOW_WRITE After a short exclusive lock after
+    already earlier having dongraded lock to TL_WRITE_ALLOW_WRITE
+    The implementation is conservative and rather don't start rather than
+    go on unknown paths to start, the common cases are handled.
+
+    NOTE:
+    In its current implementation it is only allowed to downgrade from
+    TL_WRITE_ONLY. In this case there are no waiters. Thus no wake up
+    logic is required.
+*/
+
+void thr_downgrade_write_lock(THR_LOCK_DATA *in_data,
+                              enum thr_lock_type new_lock_type) {
+  THR_LOCK *lock = in_data->lock;
+#ifndef DBUG_OFF
+  enum thr_lock_type old_lock_type = in_data->type;
+#endif
+  DBUG_TRACE;
+
+  mysql_mutex_lock(&lock->mutex);
+  DBUG_ASSERT(old_lock_type == TL_WRITE_ONLY);
+  DBUG_ASSERT(old_lock_type > new_lock_type);
+  in_data->type = new_lock_type;
+  check_locks(lock, "after downgrading lock", 0);
+
+  mysql_mutex_unlock(&lock->mutex);
+}
+
+#include "my_sys.h"
+
 static void thr_print_lock(const char *name, struct st_lock_list *list) {
   THR_LOCK_DATA *data, **prev;
   uint count = 0;
@@ -1174,15 +1216,15 @@ static ulong sum = 0;
 
 /* The following functions is for WRITE_CONCURRENT_INSERT */
 
-static void test_get_status(void *param [[maybe_unused]],
-                            int concurrent_insert [[maybe_unused]]) {}
+static void test_get_status(void *param MY_ATTRIBUTE((unused)),
+                            int concurrent_insert MY_ATTRIBUTE((unused))) {}
 
-static void test_update_status(void *param [[maybe_unused]]) {}
+static void test_update_status(void *param MY_ATTRIBUTE((unused))) {}
 
-static void test_copy_status(void *to [[maybe_unused]],
-                             void *from [[maybe_unused]]) {}
+static void test_copy_status(void *to MY_ATTRIBUTE((unused)),
+                             void *from MY_ATTRIBUTE((unused))) {}
 
-static bool test_check_status(void *param [[maybe_unused]]) { return false; }
+static bool test_check_status(void *param MY_ATTRIBUTE((unused))) { return 0; }
 
 static void *test_thread(void *arg) {
   int i, j, param = *((int *)arg);
@@ -1190,7 +1232,7 @@ static void *test_thread(void *arg) {
   THR_LOCK_INFO lock_info;
   THR_LOCK_DATA *multi_locks[MAX_LOCK_COUNT];
   my_thread_id id;
-  mysql_cond_t COND_thr_lock{};
+  mysql_cond_t COND_thr_lock;
 
   id = param + 1; /* Main thread uses value 0. */
   mysql_cond_init(0, &COND_thr_lock);
@@ -1200,8 +1242,8 @@ static void *test_thread(void *arg) {
 
   thr_lock_info_init(&lock_info, id, &COND_thr_lock);
   for (i = 0; i < lock_counts[param]; i++) {
-    thr_lock_data_init(locks + tests[param][i].lock_nr, data + i, nullptr);
-    data[i].m_psi = nullptr;
+    thr_lock_data_init(locks + tests[param][i].lock_nr, data + i, NULL);
+    data[i].m_psi = NULL;
   }
   for (j = 1; j < 10; j++) /* try locking 10 times */
   {
@@ -1235,7 +1277,7 @@ static void *test_thread(void *arg) {
   mysql_mutex_unlock(&LOCK_thread_count);
   mysql_cond_destroy(&COND_thr_lock);
   free((uchar *)arg);
-  return nullptr;
+  return 0;
 }
 
 int main(int argc, char **argv) {
