@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -32,12 +31,12 @@
 
 #include "thr_mutex.h"
 
-#include <assert.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "my_dbug.h"
 #include "my_thread_local.h"
 
 #if defined(SAFE_MUTEX)
@@ -55,7 +54,7 @@ void safe_mutex_global_init(void) { safe_mutex_inited = true; }
 
 int safe_mutex_init(safe_mutex_t *mp, const native_mutexattr_t *attr,
                     const char *file, uint line) {
-  assert(safe_mutex_inited);
+  DBUG_ASSERT(safe_mutex_inited);
   memset(mp, 0, sizeof(*mp));
   native_mutex_init(&mp->global, MY_MUTEX_INIT_ERRCHK);
   native_mutex_init(&mp->mutex, attr);
@@ -83,7 +82,7 @@ int safe_mutex_lock(safe_mutex_t *mp, bool try_lock, const char *file,
       native_mutex_unlock(&mp->global);
       return EBUSY;
     } else if (my_thread_equal(my_thread_self(), mp->thread)) {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       fprintf(stderr,
               "safe_mutex: Trying to lock mutex at %s, line %d, when the"
               " mutex was already locked at %s, line %d in thread T@%u\n",
@@ -196,7 +195,7 @@ int safe_mutex_destroy(safe_mutex_t *mp, const char *file, uint line) {
   native_mutex_unlock(&mp->global);
   if (native_mutex_destroy(&mp->global)) error = 1;
   if (native_mutex_destroy(&mp->mutex)) error = 1;
-  mp->file = nullptr; /* Mark destroyed */
+  mp->file = 0; /* Mark destroyed */
   return error;
 }
 
