@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -74,7 +73,7 @@ int find_type_or_exit(const char *x, TYPELIB *typelib, const char *option) {
                         FIND_TYPE_ALLOW_NUMBER
                         FIND_TYPE_COMMA_TERM.
                         FIND_TYPE_NO_OVERWRITE can be passed but is
-                        redundant (is always implicitly on).
+                        superfluous (is always implicitely on).
 
   @retval
     -1  Too many matching values
@@ -92,8 +91,8 @@ int find_type(const char *x, const TYPELIB *typelib, uint flags) {
   DBUG_TRACE;
   DBUG_PRINT("enter", ("x: '%s'  lib: %p", x, typelib));
 
-  assert(!(flags & ~(FIND_TYPE_NO_PREFIX | FIND_TYPE_ALLOW_NUMBER |
-                     FIND_TYPE_NO_OVERWRITE | FIND_TYPE_COMMA_TERM)));
+  DBUG_ASSERT(!(flags & ~(FIND_TYPE_NO_PREFIX | FIND_TYPE_ALLOW_NUMBER |
+                          FIND_TYPE_NO_OVERWRITE | FIND_TYPE_COMMA_TERM)));
   if (!typelib->count) {
     DBUG_PRINT("exit", ("no count"));
     return 0;
@@ -131,6 +130,21 @@ int find_type(const char *x, const TYPELIB *typelib, uint flags) {
   return findpos + 1;
 } /* find_type */
 
+/**
+  Get name of type nr
+
+  @note
+  first type is 1, 0 = empty field
+*/
+
+void make_type(char *to, uint nr, TYPELIB *typelib) {
+  DBUG_TRACE;
+  if (!nr)
+    to[0] = 0;
+  else
+    (void)my_stpcpy(to, get_type(typelib, nr - 1));
+} /* make_type */
+
 /**
   Get type
 
@@ -145,7 +159,7 @@ const char *get_type(TYPELIB *typelib, uint nr) {
 }
 
 /**
-  Create an integer value to represent the supplied comma-separated
+  Create an integer value to represent the supplied comma-seperated
   string where each string in the TYPELIB denotes a bit position.
 
   @param x      string to decompose
@@ -199,36 +213,35 @@ TYPELIB *copy_typelib(MEM_ROOT *root, TYPELIB *from) {
   TYPELIB *to;
   uint i;
 
-  if (!from) return nullptr;
+  if (!from) return NULL;
 
-  if (!(to = (TYPELIB *)root->Alloc(sizeof(TYPELIB)))) return nullptr;
+  if (!(to = (TYPELIB *)root->Alloc(sizeof(TYPELIB)))) return NULL;
 
   if (!(to->type_names = (const char **)root->Alloc(
             (sizeof(char *) + sizeof(int)) * (from->count + 1))))
-    return nullptr;
+    return NULL;
   to->type_lengths = (unsigned int *)(to->type_names + from->count + 1);
   to->count = from->count;
   if (from->name) {
-    if (!(to->name = strdup_root(root, from->name))) return nullptr;
+    if (!(to->name = strdup_root(root, from->name))) return NULL;
   } else
-    to->name = nullptr;
+    to->name = NULL;
 
   for (i = 0; i < from->count; i++) {
     if (!(to->type_names[i] =
               strmake_root(root, from->type_names[i], from->type_lengths[i])))
-      return nullptr;
+      return NULL;
     to->type_lengths[i] = from->type_lengths[i];
   }
-  to->type_names[to->count] = nullptr;
+  to->type_names[to->count] = NULL;
   to->type_lengths[to->count] = 0;
 
   return to;
 }
 
-static const char *on_off_default_names[] = {"off", "on", "default", nullptr};
+static const char *on_off_default_names[] = {"off", "on", "default", 0};
 static TYPELIB on_off_default_typelib = {
-    array_elements(on_off_default_names) - 1, "", on_off_default_names,
-    nullptr};
+    array_elements(on_off_default_names) - 1, "", on_off_default_names, 0};
 
 /**
   Parse a TYPELIB name from the buffer
@@ -242,18 +255,16 @@ static TYPELIB on_off_default_typelib = {
   The buffer is assumed to contain one of the names specified in the TYPELIB,
   followed by comma, '=', or end of the buffer.
 
-  @retval
-    -1  Too many matching values
   @retval
     0   No matching name
   @retval
     >0  Offset+1 in typelib for matched name
 */
 
-static int parse_name(const TYPELIB *lib, const char **strpos,
-                      const char *end) {
+static uint parse_name(const TYPELIB *lib, const char **strpos,
+                       const char *end) {
   const char *pos = *strpos;
-  int find = find_type(pos, lib, FIND_TYPE_COMMA_TERM);
+  uint find = find_type(pos, lib, FIND_TYPE_COMMA_TERM);
   for (; pos != end && *pos != '=' && *pos != ','; pos++)
     ;
   *strpos = pos;
@@ -295,23 +306,22 @@ static int parse_name(const TYPELIB *lib, const char **strpos,
     Parsed set value if (*errpos == NULL), otherwise undefined
 */
 
-uint64_t find_set_from_flags(const TYPELIB *lib, int default_name,
+uint64_t find_set_from_flags(const TYPELIB *lib, size_t default_name,
                              uint64_t cur_set, uint64_t default_set,
                              const char *str, uint length, const char **err_pos,
                              uint *err_len) {
   const char *end = str + length;
   uint64_t flags_to_set = 0, flags_to_clear = 0, res;
-  bool set_defaults = false;
+  bool set_defaults = 0;
 
-  *err_pos = nullptr; /* No error yet */
+  *err_pos = 0; /* No error yet */
   if (str != end) {
     const char *start = str;
     for (;;) {
       const char *pos = start;
-      uint value;
+      uint flag_no, value;
 
-      int flag_no = parse_name(lib, &pos, end);
-      if (flag_no <= 0) goto err;
+      if (!(flag_no = parse_name(lib, &pos, end))) goto err;
 
       if (flag_no == default_name) {
         /* Using 'default' twice isn't allowed. */
