@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -41,7 +41,7 @@ SERVICE_TYPE(log_builtins_string) *log_bs = nullptr;
 namespace connection_control {
 class Connection_control_error_handler : public Error_handler {
  public:
-  void handle_error(longlong errcode, ...) override {
+  void handle_error(longlong errcode, ...) {
     va_list vl;
     va_start(vl, errcode);
     LogPluginErrV(ERROR_LEVEL, errcode, vl);
@@ -60,51 +60,8 @@ using connection_control::Error_handler;
 Connection_control_statistics g_statistics;
 Connection_control_variables g_variables;
 
-Connection_event_coordinator *g_connection_event_coordinator = nullptr;
-MYSQL_PLUGIN connection_control_plugin_info = nullptr;
-
-/* Performance Schema instrumentation */
-
-PSI_mutex_key key_connection_delay_mutex = PSI_NOT_INSTRUMENTED;
-
-static PSI_mutex_info all_connection_delay_mutex_info[] = {
-    {&key_connection_delay_mutex, "connection_delay_mutex", 0, 0,
-     PSI_DOCUMENT_ME}};
-
-PSI_rwlock_key key_connection_event_delay_lock;
-
-static PSI_rwlock_info all_connection_delay_rwlock_info[] = {
-    {&key_connection_event_delay_lock, "connection_event_delay_lock",
-     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME}};
-
-PSI_cond_key key_connection_delay_wait = PSI_NOT_INSTRUMENTED;
-
-static PSI_cond_info all_connection_delay_cond_info[] = {
-    {&key_connection_delay_wait, "connection_delay_wait_condition", 0, 0,
-     PSI_DOCUMENT_ME}};
-
-PSI_stage_info stage_waiting_in_connection_control_plugin = {
-    0, "Waiting in connection_control plugin", 0, PSI_DOCUMENT_ME};
-
-static PSI_stage_info *all_connection_delay_stage_info[] = {
-    &stage_waiting_in_connection_control_plugin};
-
-static void init_performance_schema() {
-  const char *category = "conn_delay";
-
-  int count_mutex = array_elements(all_connection_delay_mutex_info);
-  mysql_mutex_register(category, all_connection_delay_mutex_info, count_mutex);
-
-  int count_rwlock = array_elements(all_connection_delay_rwlock_info);
-  mysql_rwlock_register(category, all_connection_delay_rwlock_info,
-                        count_rwlock);
-
-  int count_cond = array_elements(all_connection_delay_cond_info);
-  mysql_cond_register(category, all_connection_delay_cond_info, count_cond);
-
-  int count_stage = array_elements(all_connection_delay_stage_info);
-  mysql_stage_register(category, all_connection_delay_stage_info, count_stage);
-}
+Connection_event_coordinator *g_connection_event_coordinator = 0;
+MYSQL_PLUGIN connection_control_plugin_info = 0;
 
 /**
   event_notify() implementation for connection_control
@@ -149,12 +106,6 @@ static int connection_control_notify(MYSQL_THD thd,
 */
 
 static int connection_control_init(MYSQL_PLUGIN plugin_info) {
-  /*
-    Declare all performance schema instrumentation up front,
-    so it is discoverable.
-  */
-  init_performance_schema();
-
   // Initialize error logging service.
   if (init_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs)) return 1;
 
@@ -186,11 +137,11 @@ static int connection_control_init(MYSQL_PLUGIN plugin_info) {
   @returns success
 */
 
-static int connection_control_deinit(void *arg [[maybe_unused]]) {
+static int connection_control_deinit(void *arg MY_ATTRIBUTE((unused))) {
   delete g_connection_event_coordinator;
-  g_connection_event_coordinator = nullptr;
+  g_connection_event_coordinator = 0;
   connection_control::deinit_connection_delay_event();
-  connection_control_plugin_info = nullptr;
+  connection_control_plugin_info = 0;
 
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
   return 0;
@@ -199,7 +150,7 @@ static int connection_control_deinit(void *arg [[maybe_unused]]) {
 /** Connection_control plugin descriptor */
 static struct st_mysql_audit connection_control_descriptor = {
     MYSQL_AUDIT_INTERFACE_VERSION, /* interface version */
-    nullptr,                       /* release_thd() */
+    NULL,                          /* release_thd() */
     connection_control_notify,     /* event_notify() */
     {
         0, /* MYSQL_AUDIT_GENERAL_CLASS */
@@ -233,10 +184,9 @@ static struct st_mysql_audit connection_control_descriptor = {
     @retval 1 Value is not within valid bounds
 */
 
-static int check_failed_connections_threshold(MYSQL_THD thd [[maybe_unused]],
-                                              SYS_VAR *var [[maybe_unused]],
-                                              void *save [[maybe_unused]],
-                                              struct st_mysql_value *value) {
+static int check_failed_connections_threshold(
+    MYSQL_THD thd MY_ATTRIBUTE((unused)), SYS_VAR *var MY_ATTRIBUTE((unused)),
+    void *save MY_ATTRIBUTE((unused)), struct st_mysql_value *value) {
   longlong new_value;
   if (value->val_int(value, &new_value)) return 1; /* NULL value */
 
@@ -262,10 +212,9 @@ static int check_failed_connections_threshold(MYSQL_THD thd [[maybe_unused]],
   connection_control_failed_connections_threshold
 */
 
-static void update_failed_connections_threshold(MYSQL_THD thd [[maybe_unused]],
-                                                SYS_VAR *var [[maybe_unused]],
-                                                void *var_ptr [[maybe_unused]],
-                                                const void *save) {
+static void update_failed_connections_threshold(
+    MYSQL_THD thd MY_ATTRIBUTE((unused)), SYS_VAR *var MY_ATTRIBUTE((unused)),
+    void *var_ptr MY_ATTRIBUTE((unused)), const void *save) {
   /*
     This won't result in overflow because we have already checked that this is
     within valid bounds.
@@ -302,9 +251,9 @@ static MYSQL_SYSVAR_LONGLONG(
     @retval 1 Value is not within valid bounds
 */
 
-static int check_min_connection_delay(MYSQL_THD thd [[maybe_unused]],
-                                      SYS_VAR *var [[maybe_unused]],
-                                      void *save [[maybe_unused]],
+static int check_min_connection_delay(MYSQL_THD thd MY_ATTRIBUTE((unused)),
+                                      SYS_VAR *var MY_ATTRIBUTE((unused)),
+                                      void *save MY_ATTRIBUTE((unused)),
                                       struct st_mysql_value *value) {
   long long new_value;
   int64 existing_value = g_variables.max_connection_delay;
@@ -331,9 +280,9 @@ static int check_min_connection_delay(MYSQL_THD thd [[maybe_unused]],
   @param save       New value for connection_control_min_connection_delay
 */
 
-static void update_min_connection_delay(MYSQL_THD thd [[maybe_unused]],
-                                        SYS_VAR *var [[maybe_unused]],
-                                        void *var_ptr [[maybe_unused]],
+static void update_min_connection_delay(MYSQL_THD thd MY_ATTRIBUTE((unused)),
+                                        SYS_VAR *var MY_ATTRIBUTE((unused)),
+                                        void *var_ptr MY_ATTRIBUTE((unused)),
                                         const void *save) {
   longlong new_value = *(reinterpret_cast<const longlong *>(save));
   g_variables.min_connection_delay = (int64)new_value;
@@ -366,9 +315,9 @@ static MYSQL_SYSVAR_LONGLONG(
     @retval 1 Value is not within valid bounds
 */
 
-static int check_max_connection_delay(MYSQL_THD thd [[maybe_unused]],
-                                      SYS_VAR *var [[maybe_unused]],
-                                      void *save [[maybe_unused]],
+static int check_max_connection_delay(MYSQL_THD thd MY_ATTRIBUTE((unused)),
+                                      SYS_VAR *var MY_ATTRIBUTE((unused)),
+                                      void *save MY_ATTRIBUTE((unused)),
                                       struct st_mysql_value *value) {
   long long new_value;
   int64 existing_value = g_variables.min_connection_delay;
@@ -395,9 +344,9 @@ static int check_max_connection_delay(MYSQL_THD thd [[maybe_unused]],
   @param save       New value for connection_control_max_connection_delay
 */
 
-static void update_max_connection_delay(MYSQL_THD thd [[maybe_unused]],
-                                        SYS_VAR *var [[maybe_unused]],
-                                        void *var_ptr [[maybe_unused]],
+static void update_max_connection_delay(MYSQL_THD thd MY_ATTRIBUTE((unused)),
+                                        SYS_VAR *var MY_ATTRIBUTE((unused)),
+                                        void *var_ptr MY_ATTRIBUTE((unused)),
                                         const void *save) {
   longlong new_value = *(reinterpret_cast<const longlong *>(save));
   g_variables.max_connection_delay = (int64)new_value;
@@ -419,7 +368,7 @@ static MYSQL_SYSVAR_LONGLONG(
 SYS_VAR *connection_control_system_variables[OPT_LAST + 1] = {
     MYSQL_SYSVAR(failed_connections_threshold),
     MYSQL_SYSVAR(min_connection_delay), MYSQL_SYSVAR(max_connection_delay),
-    nullptr};
+    NULL};
 
 /**
   Function to display value for status variable :
@@ -432,8 +381,7 @@ SYS_VAR *connection_control_system_variables[OPT_LAST + 1] = {
   @returns Always returns success.
 */
 
-static int show_delay_generated(MYSQL_THD thd [[maybe_unused]], SHOW_VAR *var,
-                                char *buff) {
+static int show_delay_generated(MYSQL_THD, SHOW_VAR *var, char *buff) {
   var->type = SHOW_LONGLONG;
   var->value = buff;
   longlong *value = reinterpret_cast<longlong *>(buff);
@@ -448,35 +396,35 @@ SHOW_VAR
 connection_control_status_variables[STAT_LAST + 1] = {
     {"Connection_control_delay_generated", (char *)&show_delay_generated,
      SHOW_FUNC, SHOW_SCOPE_GLOBAL},
-    {nullptr, nullptr, enum_mysql_show_type(0), enum_mysql_show_scope(0)}};
+    {0, 0, enum_mysql_show_type(0), enum_mysql_show_scope(0)}};
 
 mysql_declare_plugin(audit_log){
     MYSQL_AUDIT_PLUGIN,                  /* plugin type                   */
     &connection_control_descriptor,      /* type specific descriptor      */
     "CONNECTION_CONTROL",                /* plugin name                   */
-    PLUGIN_AUTHOR_ORACLE,                /* author                        */
+    "Oracle Inc",                        /* author                        */
     "Connection event processing",       /* description                   */
     PLUGIN_LICENSE_GPL,                  /* license                       */
     connection_control_init,             /* plugin initializer            */
-    nullptr,                             /* plugin check uninstall        */
+    NULL,                                /* plugin check uninstall        */
     connection_control_deinit,           /* plugin deinitializer          */
     0x0100,                              /* version                       */
     connection_control_status_variables, /* status variables              */
     connection_control_system_variables, /* system variables              */
-    nullptr,                             /* reserved                      */
+    NULL,                                /* reserverd                     */
     0                                    /* flags                         */
 },
     {MYSQL_INFORMATION_SCHEMA_PLUGIN,
      &connection_control_failed_attempts_view,
      "CONNECTION_CONTROL_FAILED_LOGIN_ATTEMPTS",
-     PLUGIN_AUTHOR_ORACLE,
+     "Oracle Inc",
      "I_S table providing a view into failed attempts statistics",
      PLUGIN_LICENSE_GPL,
      connection_control_failed_attempts_view_init,
-     nullptr,
-     nullptr,
+     NULL,
+     NULL,
      0x0100,
-     nullptr,
-     nullptr,
-     nullptr,
+     NULL,
+     NULL,
+     NULL,
      0} mysql_declare_plugin_end;
