@@ -1,16 +1,15 @@
-/* Copyright (c) 2006, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -71,10 +70,10 @@ static void *mysql_heartbeat(void *p) {
   time_t result;
   struct tm tm_tmp;
 
-  while (true) {
+  while (1) {
     sleep(5);
 
-    result = time(nullptr);
+    result = time(NULL);
     localtime_r(&result, &tm_tmp);
     snprintf(buffer, sizeof(buffer),
              "Heartbeat at %02d%02d%02d %2d:%02d:%02d\n", tm_tmp.tm_year % 100,
@@ -83,7 +82,7 @@ static void *mysql_heartbeat(void *p) {
     my_write(con->heartbeat_file, (uchar *)buffer, strlen(buffer), MYF(0));
   }
 
-  return nullptr;
+  return 0;
 }
 
 /*
@@ -111,7 +110,7 @@ static int daemon_example_plugin_init(void *p) {
   my_thread_attr_t attr; /* Thread attributes */
   char heartbeat_filename[FN_REFLEN];
   char buffer[HEART_STRING_BUFFER];
-  time_t result = time(nullptr);
+  time_t result = time(NULL);
   struct tm tm_tmp;
 
   struct st_plugin_int *plugin = (struct st_plugin_int *)p;
@@ -168,7 +167,7 @@ static int daemon_example_plugin_deinit(void *p) {
   struct st_plugin_int *plugin = (struct st_plugin_int *)p;
   struct mysql_heartbeat_context *con =
       (struct mysql_heartbeat_context *)plugin->data;
-  time_t result = time(nullptr);
+  time_t result = time(NULL);
   struct tm tm_tmp;
   void *dummy_retval;
 
@@ -204,15 +203,15 @@ mysql_declare_plugin(daemon_example){
     MYSQL_DAEMON_PLUGIN,
     &daemon_example_plugin,
     "daemon_example",
-    PLUGIN_AUTHOR_ORACLE,
+    "Brian Aker",
     "Daemon example, creates a heartbeat beat file in mysql-heartbeat.log",
     PLUGIN_LICENSE_GPL,
     daemon_example_plugin_init,   /* Plugin Init */
-    nullptr,                      /* Plugin Check uninstall */
+    NULL,                         /* Plugin Check uninstall */
     daemon_example_plugin_deinit, /* Plugin Deinit */
     0x0100 /* 1.0 */,
-    nullptr, /* status variables                */
-    nullptr, /* system variables                */
-    nullptr, /* config options                  */
-    0,       /* flags                           */
+    NULL, /* status variables                */
+    NULL, /* system variables                */
+    NULL, /* config options                  */
+    0,    /* flags                           */
 } mysql_declare_plugin_end;
