@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,9 +20,9 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <assert.h>
 #include <stddef.h>
 
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "mysql/plugin.h"
 #include "storage/innobase/include/fts0tokenize.h"
@@ -54,7 +53,7 @@ static int ngram_parse(MYSQL_FTPARSER_PARAM *param, const char *doc, int len,
   int ret = 0;
   bool is_first = true;
 
-  assert(cs->mbminlen == 1);
+  DBUG_ASSERT(cs->mbminlen == 1);
 
   start = const_cast<char *>(doc);
   next = start;
@@ -105,8 +104,8 @@ static int ngram_parse(MYSQL_FTPARSER_PARAM *param, const char *doc, int len,
     case MYSQL_FTPARSER_FULL_BOOLEAN_INFO:
     case MYSQL_FTPARSER_WITH_STOPWORDS:
       if (n_chars > 0 && is_first) {
-        assert(next > start);
-        assert(n_chars < ngram_token_size);
+        DBUG_ASSERT(next > start);
+        DBUG_ASSERT(n_chars < ngram_token_size);
 
         ret = param->mysql_add_word(param, start, next - start, bool_info);
       }
@@ -152,15 +151,15 @@ static int ngram_get_token_size(const CHARSET_INFO *cs, const char *token,
 @retval	1	on failure. */
 static int ngram_term_convert(MYSQL_FTPARSER_PARAM *param, const char *token,
                               int len, MYSQL_FTPARSER_BOOLEAN_INFO *bool_info) {
-  MYSQL_FTPARSER_BOOLEAN_INFO token_info = {FT_TOKEN_WORD, 0,      0, 0, 0, 0,
-                                            ' ',           nullptr};
+  MYSQL_FTPARSER_BOOLEAN_INFO token_info = {FT_TOKEN_WORD, 0, 0, 0, 0, 0,
+                                            ' ',           0};
   const CHARSET_INFO *cs = param->cs;
   int token_size;
   int ret = 0;
 
-  assert(bool_info->type == FT_TOKEN_WORD);
-  assert(bool_info->quot == nullptr);
-  assert(cs->mbminlen == 1);
+  DBUG_ASSERT(bool_info->type == FT_TOKEN_WORD);
+  DBUG_ASSERT(bool_info->quot == NULL);
+  DBUG_ASSERT(cs->mbminlen == 1);
 
   /* Convert rules:
   1. if term with wildcard and term length is less than ngram_token_size,
@@ -176,16 +175,16 @@ static int ngram_term_convert(MYSQL_FTPARSER_PARAM *param, const char *token,
     bool_info->type = FT_TOKEN_LEFT_PAREN;
     bool_info->quot = reinterpret_cast<char *>(1);
 
-    ret = param->mysql_add_word(param, nullptr, 0, bool_info);
+    ret = param->mysql_add_word(param, NULL, 0, bool_info);
     RETURN_IF_ERROR(ret);
 
     ret = ngram_parse(param, token, len, &token_info);
     RETURN_IF_ERROR(ret);
 
     bool_info->type = FT_TOKEN_RIGHT_PAREN;
-    ret = param->mysql_add_word(param, nullptr, 0, bool_info);
+    ret = param->mysql_add_word(param, NULL, 0, bool_info);
 
-    assert(bool_info->quot == nullptr);
+    DBUG_ASSERT(bool_info->quot == NULL);
     bool_info->type = FT_TOKEN_WORD;
   }
 
@@ -197,12 +196,12 @@ static int ngram_term_convert(MYSQL_FTPARSER_PARAM *param, const char *token,
 @retval	0	on success
 @retval	1	on failure. */
 static int ngram_parser_parse(MYSQL_FTPARSER_PARAM *param) {
-  MYSQL_FTPARSER_BOOLEAN_INFO bool_info = {FT_TOKEN_WORD, 0,      0, 0, 0, 0,
-                                           ' ',           nullptr};
+  MYSQL_FTPARSER_BOOLEAN_INFO bool_info = {FT_TOKEN_WORD, 0, 0, 0, 0, 0,
+                                           ' ',           0};
   const CHARSET_INFO *cs = param->cs;
   uchar **start = reinterpret_cast<uchar **>(&param->doc);
   uchar *end = *start + param->length;
-  FT_WORD word = {nullptr, 0, 0};
+  FT_WORD word = {NULL, 0, 0};
   int ret = 0;
 
   switch (param->mode) {
@@ -218,16 +217,16 @@ static int ngram_parser_parse(MYSQL_FTPARSER_PARAM *param) {
       the words into ngram. */
       while (fts_get_word(cs, start, end, &word, &bool_info)) {
         if (bool_info.type == FT_TOKEN_WORD) {
-          if (bool_info.quot != nullptr) {
+          if (bool_info.quot != NULL) {
             /* Phrase search */
             ret = ngram_parse(param, reinterpret_cast<char *>(word.pos),
                               word.len, &bool_info);
           } else {
-            /* Term search */
+            /* Term serach */
             ret = ngram_term_convert(param, reinterpret_cast<char *>(word.pos),
                                      word.len, &bool_info);
-            assert(bool_info.quot == nullptr);
-            assert(bool_info.type == FT_TOKEN_WORD);
+            DBUG_ASSERT(bool_info.quot == NULL);
+            DBUG_ASSERT(bool_info.type == FT_TOKEN_WORD);
           }
         } else {
           ret = param->mysql_add_word(param, reinterpret_cast<char *>(word.pos),
@@ -245,30 +244,30 @@ static int ngram_parser_parse(MYSQL_FTPARSER_PARAM *param) {
 
 /** Fulltext ngram parser */
 static struct st_mysql_ftparser ngram_parser_descriptor = {
-    MYSQL_FTPARSER_INTERFACE_VERSION, ngram_parser_parse, nullptr, nullptr};
+    MYSQL_FTPARSER_INTERFACE_VERSION, ngram_parser_parse, 0, 0};
 
 static MYSQL_SYSVAR_INT(
     token_size, ngram_token_size, PLUGIN_VAR_READONLY,
-    "InnoDB ngram full text plugin parser token size in characters", nullptr,
-    nullptr, 2, 1, 10, 0);
+    "InnoDB ngram full text plugin parser token size in characters", NULL, NULL,
+    2, 1, 10, 0);
 
 /** Ngram plugin system variables */
-static SYS_VAR *ngram_system_variables[] = {MYSQL_SYSVAR(token_size), nullptr};
+static SYS_VAR *ngram_system_variables[] = {MYSQL_SYSVAR(token_size), NULL};
 
 /** Ngram plugin descriptor */
 mysql_declare_plugin(ngram_parser){
     MYSQL_FTPARSER_PLUGIN,    /*!< type	*/
     &ngram_parser_descriptor, /*!< descriptor	*/
     "ngram",                  /*!< name	*/
-    PLUGIN_AUTHOR_ORACLE,     /*!< author	*/
+    "Oracle Corp",            /*!< author	*/
     "Ngram Full-Text Parser", /*!< description*/
     PLUGIN_LICENSE_GPL,
-    nullptr,                /*!< init function (when loaded)*/
-    nullptr,                /*!< check uninstall function*/
-    nullptr,                /*!< deinit function (when unloaded)*/
+    NULL,                   /*!< init function (when loaded)*/
+    NULL,                   /*!< check uninstall function*/
+    NULL,                   /*!< deinit function (when unloaded)*/
     0x0001,                 /*!< version	*/
-    nullptr,                /*!< status variables	*/
+    NULL,                   /*!< status variables	*/
     ngram_system_variables, /*!< system variables	*/
-    nullptr,
+    NULL,
     0,
 } mysql_declare_plugin_end;
