@@ -46,6 +46,7 @@
 #include "plugin/group_replication/include/plugin_server_include.h"
 #include "plugin/group_replication/include/ps_information.h"
 #include "plugin/group_replication/include/recovery.h"
+#include "plugin/group_replication/include/services/message_service/message_service.h"
 #include "plugin/group_replication/include/services/registry.h"
 #include "plugin/group_replication/libmysqlgcs/include/mysql/gcs/gcs_interface.h"
 
@@ -71,9 +72,15 @@ enum enum_channel_observation_manager_position {
   END_CHANNEL_OBSERVATION_MANAGER_POS
 };
 
+/**
+  @enum enum_exit_state_action
+  @brief Action performed when the member leaves the group
+  unexpectedly.
+*/
 enum enum_exit_state_action {
   EXIT_STATE_ACTION_READ_ONLY = 0,
-  EXIT_STATE_ACTION_ABORT_SERVER
+  EXIT_STATE_ACTION_ABORT_SERVER,
+  EXIT_STATE_ACTION_OFFLINE_MODE
 };
 
 /**
@@ -103,6 +110,7 @@ struct gr_modules {
     COMPATIBILITY_MANAGER,
     GCS_EVENTS_HANDLER,
     REMOTE_CLONE_HANDLER,
+    MESSAGE_SERVICE_HANDLER,
     NUM_MODULES
   };
   using mask = std::bitset<NUM_MODULES>;
@@ -136,6 +144,7 @@ extern Group_action_coordinator *group_action_coordinator;
 extern Primary_election_handler *primary_election_handler;
 extern Hold_transactions *hold_transactions;
 extern Autorejoin_thread *autorejoin_module;
+extern Message_service_handler *message_service_handler;
 
 // Auxiliary Functionality
 extern Plugin_gcs_events_handler *events_handler;
@@ -204,6 +213,9 @@ int get_flow_control_member_quota_percent_var();
 int get_flow_control_period_var();
 int get_flow_control_hold_percent_var();
 int get_flow_control_release_percent_var();
+ulong get_components_stop_timeout_var();
+void set_error_state_due_to_error_during_autorejoin();
+bool get_error_state_due_to_error_during_autorejoin();
 
 // Plugin public methods
 int plugin_group_replication_init(MYSQL_PLUGIN plugin_info);
