@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,13 +34,6 @@
 #include "plugin/group_replication/include/plugin_observers/channel_observation_manager.h"
 #include "plugin/group_replication/include/replication_threads_api.h"
 
-typedef enum st_state_transfer_status {
-  STATE_TRANSFER_OK,            // OK
-  STATE_TRANSFER_STOP,          // Fail to stop replica threads
-  STATE_TRANSFER_PURGE,         // Fail to purge replica threads
-  STATE_TRANSFER_NO_CONNECTION  // No connection to donor
-} State_transfer_status;
-
 class Recovery_state_transfer {
  public:
   /**
@@ -97,26 +89,21 @@ class Recovery_state_transfer {
      @param ssl_crl                 SSL revocation list file
      @param ssl_crlpath             path with revocation list files
      @param ssl_verify_server_cert  verify the hostname against the certificate
-     @param tls_version             the list of TLS versions to use
-     @param tls_ciphersuites        the list of TLS ciphersuites to use
   */
   void set_recovery_ssl_options(bool use_ssl, const char *ssl_ca,
                                 const char *ssl_capath, const char *ssl_cert,
                                 const char *ssl_cipher, const char *ssl_key,
                                 const char *ssl_crl, const char *ssl_crlpath,
-                                bool ssl_verify_server_cert, char *tls_version,
-                                char *tls_ciphersuites) {
+                                bool ssl_verify_server_cert) {
     recovery_use_ssl = use_ssl;
-    if (ssl_ca != nullptr) set_recovery_ssl_ca(ssl_ca);
-    if (ssl_capath != nullptr) set_recovery_ssl_capath(ssl_capath);
-    if (ssl_cert != nullptr) set_recovery_ssl_cert(ssl_cert);
-    if (ssl_cipher != nullptr) set_recovery_ssl_cipher(ssl_cipher);
-    if (ssl_key != nullptr) set_recovery_ssl_key(ssl_key);
-    if (ssl_crl != nullptr) set_recovery_ssl_crl(ssl_crl);
-    if (ssl_crlpath != nullptr) set_recovery_ssl_crl(ssl_crlpath);
+    if (ssl_ca != NULL) set_recovery_ssl_ca(ssl_ca);
+    if (ssl_capath != NULL) set_recovery_ssl_capath(ssl_capath);
+    if (ssl_cert != NULL) set_recovery_ssl_cert(ssl_cert);
+    if (ssl_cipher != NULL) set_recovery_ssl_cipher(ssl_cipher);
+    if (ssl_key != NULL) set_recovery_ssl_key(ssl_key);
+    if (ssl_crl != NULL) set_recovery_ssl_crl(ssl_crl);
+    if (ssl_crlpath != NULL) set_recovery_ssl_crl(ssl_crlpath);
     recovery_ssl_verify_server_cert = ssl_verify_server_cert;
-    if (tls_version != nullptr) set_recovery_tls_version(tls_version);
-    set_recovery_tls_ciphersuites(tls_ciphersuites);
   }
 
   /** Set the option that forces the use of SSL on recovery connections */
@@ -162,22 +149,6 @@ class Recovery_state_transfer {
     this->recovery_ssl_verify_server_cert = ssl_verify_server_cert;
   }
 
-  /** Set a TLS versions to be used */
-  void set_recovery_tls_version(const char *tls_version) {
-    memcpy(recovery_tls_version, tls_version, strlen(tls_version) + 1);
-  }
-
-  /** Set a TLS ciphersuites to be used */
-  void set_recovery_tls_ciphersuites(const char *tls_ciphersuites) {
-    if (nullptr == tls_ciphersuites) {
-      recovery_tls_ciphersuites_null = true;
-    } else {
-      recovery_tls_ciphersuites_null = false;
-      memcpy(recovery_tls_ciphersuites, tls_ciphersuites,
-             strlen(tls_ciphersuites) + 1);
-    }
-  }
-
   /**
     @return Is recovery configured to use SSL
   */
@@ -208,7 +179,7 @@ class Recovery_state_transfer {
 
   /** Set a public key file*/
   void set_recovery_public_key_path(const char *public_key_path) {
-    if (public_key_path != nullptr) {
+    if (public_key_path != NULL) {
       memcpy(recovery_public_key_path, public_key_path,
              strlen(public_key_path) + 1);
     }
@@ -294,8 +265,7 @@ class Recovery_state_transfer {
       @retval 0      OK
       @retval !=0    Recovery state transfer failed
    */
-  State_transfer_status state_transfer(
-      Plugin_stage_monitor_handler &stage_handler);
+  int state_transfer(Plugin_stage_monitor_handler &stage_handler);
 
  private:
   /**
@@ -328,20 +298,18 @@ class Recovery_state_transfer {
     Initializes the structures for the donor connection threads.
     Recovery channel is always purged.
 
-    @param hostname hostname of current selected donor
-    @param port port of current selected donor
-
     @return the operation status
       @retval 0      OK
       @retval !=0    Error
   */
-  int initialize_donor_connection(std::string hostname, uint port);
+  int initialize_donor_connection();
 
   /**
     Initializes the connection parameters for the donor connection.
 
-    @retval false Everything OK
-    @retval true  In case of the selected donor is not available
+    @return
+      @retval false Everything OK
+      @retval true  In case of the selected donor is not available
   */
   bool initialize_connection_parameters();
 
@@ -360,11 +328,10 @@ class Recovery_state_transfer {
     @param purge_logs  purge recovery logs
 
     @return the operation status
-      @retval STATE_TRANSFER_OK      OK
-      @retval !=STATE_TRANSFER_OK    Error
+      @retval 0      OK
+      @retval !=0    Error
   */
-  State_transfer_status terminate_recovery_slave_threads(
-      bool purge_logs = true);
+  int terminate_recovery_slave_threads(bool purge_logs = true);
 
   /**
     Purges relay logs and the master info object
@@ -389,16 +356,16 @@ class Recovery_state_transfer {
   /* The selected donor member hostname */
   std::string selected_donor_hostname;
   /* Vector with group members info*/
-  Group_member_info_list *group_members;
+  std::vector<Group_member_info *> *group_members;
   /* Member with suitable donors for use on recovery*/
-  Group_member_info_list suitable_donors;
+  std::vector<Group_member_info *> suitable_donors;
 
   /* Retry count on donor connections*/
   long donor_connection_retry_count;
 
   /* Recovery abort flag */
   bool recovery_aborted;
-  /*  Flag that signals when the donor transferred all its data */
+  /*  Flag that signals when the donor transfered all it's data */
   bool donor_transfer_finished;
   /* Are we successfully connected to a donor*/
   bool connected_to_donor;
@@ -440,11 +407,6 @@ class Recovery_state_transfer {
   bool recovery_ssl_verify_server_cert;
   /** Public key information */
   char recovery_public_key_path[FN_REFLEN];
-  /** Permitted TLS versions. */
-  char recovery_tls_version[FN_REFLEN];
-  /** Permitted TLS 1.3 ciphersuites. */
-  bool recovery_tls_ciphersuites_null;
-  char recovery_tls_ciphersuites[FN_REFLEN];
 
   /* The lock for the recovery wait condition */
   mysql_mutex_t recovery_lock;
