@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,11 +35,13 @@
 
 class Replication_thread_api {
  public:
-  Replication_thread_api(const char *channel_interface);
+  Replication_thread_api(const char *channel_interface)
+      : stop_wait_timeout(LONG_TIMEOUT), interface_channel(channel_interface) {}
 
-  Replication_thread_api();
+  Replication_thread_api()
+      : stop_wait_timeout(LONG_TIMEOUT), interface_channel(NULL) {}
 
-  ~Replication_thread_api() = default;
+  ~Replication_thread_api() {}
 
   /**
     Set the channel name to be used on the interface method invocation.
@@ -75,10 +76,7 @@ class Replication_thread_api {
     @param get_public_key Preference to get public key if unavailable.
     @param compression_algorithm The compression algorithm
     @param zstd_compression_level The compression level
-    @param tls_version   TLS versions
-    @param tls_ciphersuites Permissible ciphersuites for TLS 1.3.
-    @param ignore_ws_mem_limit Shall ignore write set mem limits
-    @param allow_drop_write_set Shall not require write set to be preserved
+
 
     @return the operation status
       @retval 0      OK
@@ -92,9 +90,7 @@ class Replication_thread_api {
                          int retry_count, bool preserve_logs,
                          char *public_key_path, bool get_public_key,
                          char *compression_algorithm,
-                         uint zstd_compression_level, char *tls_version,
-                         char *tls_ciphersuites, bool ignore_ws_mem_limit,
-                         bool allow_drop_write_set);
+                         uint zstd_compression_level);
 
   /**
     Start the Applier/Receiver threads according to the given options.
@@ -254,8 +250,7 @@ class Replication_thread_api {
        @retval true   the id matches a SQL or worker thread
        @retval false  the id doesn't match any thread
    */
-  bool is_own_event_applier(my_thread_id id,
-                            const char *channel_name = nullptr);
+  bool is_own_event_applier(my_thread_id id, const char *channel_name = NULL);
 
   /**
      Checks if the given id matches the receiver thread for
@@ -276,7 +271,8 @@ class Replication_thread_api {
                     last GNO of group's already certified transactions
                     on relay log.
 
-    @retval       GNO value
+    @return
+      @retval       GNO value
   */
   rpl_gno get_last_delivered_gno(rpl_sidno sidno);
 
@@ -293,17 +289,18 @@ class Replication_thread_api {
     @param[out] retrieved_set the set in string format.
     @param channel_name the name of the channel to get the information.
 
-    @retval true there was an error.
-    @retval false the operation has succeeded.
+    @return
+      @retval true there was an error.
+      @retval false the operation has succeeded.
   */
   bool get_retrieved_gtid_set(std::string &retrieved_set,
-                              const char *channel_name = nullptr);
+                              const char *channel_name = NULL);
 
   /**
     Checks if the channel's relay log contains partial transaction.
-
-    @retval true  If relaylog contains partial transaction.
-    @retval false If relaylog does not contain partial transaction.
+    @return
+      @retval true  If relaylog contains partial transaction.
+      @retval false If relaylog does not contain partial transaction.
   */
   bool is_partial_transaction_on_relay_log();
 
@@ -320,28 +317,6 @@ class Replication_thread_api {
   */
   static int rpl_channel_stop_all(int threads_to_stop, long timeout);
 
-  /**
-    Interface to kill binlog dump thread.
-    Kills binlog dump thread thus killing all slave connections.
-    @note binlog dump GTID thread is not killed as of now.
-
-    @return the operation status
-      @retval 0      OK
-  */
-  static int rpl_binlog_dump_thread_kill();
-
-  /**
-    Interface to remove stored credentials from thread api.
-    Removes credentials for the channel from thread api.
-
-    @param channel_name  Credential associated channel name
-
-    @return the operation status
-      @retval 0      OK
-      @retval !=0    Error
-  */
-  static int delete_credential(const char *channel_name);
-
   /**
     Method to get the credentials configured for a channel
 
@@ -354,33 +329,7 @@ class Replication_thread_api {
       @retval true    Error, channel not found
   */
   bool get_channel_credentials(std::string &username, std::string &password,
-                               const char *channel_name = nullptr);
-
-  /**
-      Method to get the network namespace configured for a channel
-
-      @param[out] net_ns      The network namespace to extract
-      @param[in]  channel_name  The name of the channel to get the information.
-
-      @return the operation status
-        @retval false   OK
-        @retval true    Error, channel not found
-    */
-  bool get_channel_network_namespace(std::string &net_ns,
-                                     const char *channel_name = nullptr);
-
-  /**
-    Checks if any channel uses the same UUID for
-    assign_gtids_to_anonymous_transactions as the given uuid
-
-    @param[in]       uuid_param         the group name
-
-    @retval          true               at least one channel has the same uuid
-    @retval          false              none of the the channels have the same
-    uuid
-  */
-  bool is_any_channel_using_uuid_for_assign_gtids_to_anonymous_transaction(
-      const char *uuid_param);
+                               const char *channel_name = NULL);
 
  private:
   ulong stop_wait_timeout;
