@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,9 +38,9 @@ class Sql_service_context_base {
     Sql_service_context_base constructor
     resets all variables
   */
-  Sql_service_context_base() = default;
+  Sql_service_context_base() {}
 
-  virtual ~Sql_service_context_base() = default;
+  virtual ~Sql_service_context_base() {}
 
   /** Getting metadata **/
   /**
@@ -51,8 +50,9 @@ class Sql_service_context_base {
     @param flags    Flags to alter the metadata sending
     @param resultcs Charset of the result set
 
-    @retval 1  Error
-    @retval 0  OK
+    @return
+      @retval 1  Error
+      @retval 0  OK
   */
   virtual int start_result_metadata(uint num_cols, uint flags,
                                     const CHARSET_INFO *resultcs) = 0;
@@ -63,8 +63,9 @@ class Sql_service_context_base {
     @param field   Field's metadata (see field.h)
     @param charset Field's charset
 
-    @retval 1  Error
-    @retval 0  OK
+    @return
+      @retval 1  Error
+      @retval 0  OK
   */
   virtual int field_metadata(struct st_send_field *field,
                              const CHARSET_INFO *charset) = 0;
@@ -75,24 +76,27 @@ class Sql_service_context_base {
     @param server_status   Status of server (see mysql_com.h SERVER_STATUS_*)
     @param warn_count      Number of warnings thrown during execution
 
-    @retval 1  Error
-    @retval 0  OK
+    @return
+      @retval 1  Error
+      @retval 0  OK
   */
   virtual int end_result_metadata(uint server_status, uint warn_count) = 0;
 
   /**
     Indicates the beginning of a new row in the result set/metadata
 
-    @retval 1  Error
-    @retval 0  OK
+    @return
+      @retval 1  Error
+      @retval 0  OK
   */
   virtual int start_row() = 0;
 
   /**
     Indicates end of the row in the result set/metadata
 
-    @retval 1  Error
-    @retval 0  OK
+    @return
+      @retval 1  Error
+      @retval 0  OK
   */
   virtual int end_row() = 0;
 
@@ -246,12 +250,6 @@ class Sql_service_context_base {
   */
   virtual void shutdown(int flag) = 0;
 
-  /**
-   Check if the connection is still alive.
-   It should always return true unless the protocol closed the connection.
-  */
-  virtual bool connection_alive() = 0;
-
  private:
   static int sql_start_result_metadata(void *ctx, uint num_cols, uint flags,
                                        const CHARSET_INFO *resultcs) {
@@ -345,10 +343,6 @@ class Sql_service_context_base {
   static void sql_shutdown(void *ctx, int flag) {
     return ((Sql_service_context_base *)ctx)->shutdown(flag);
   }
-
-  static bool sql_connection_alive(void *ctx) {
-    return ((Sql_service_context_base *)ctx)->connection_alive();
-  }
 };
 
 #endif  // SQL_SERVICE_CONTEXT_BASE_INCLUDE
