@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -73,9 +72,9 @@ static void *suspicions_processing_thread(void *ptr) {
     mgr->process_suspicions();
   }
 
-  My_xp_thread_util::exit(nullptr);
+  My_xp_thread_util::exit(0);
   /* purecov: begin deadcode */
-  return nullptr;
+  return NULL;
   /* purecov: end */
 }
 
@@ -103,9 +102,9 @@ static void *xcom_taskmain_startup(void *ptr) {
   proxy->xcom_init(port);
 
 end:
-  My_xp_thread_util::exit(nullptr);
+  My_xp_thread_util::exit(0);
   /* purecov: begin deadcode */
-  return nullptr;
+  return NULL;
   /* purecov: end */
 }
 
@@ -124,16 +123,14 @@ Gcs_xcom_control::Gcs_xcom_control(
     Gcs_xcom_engine *gcs_engine,
     Gcs_xcom_state_exchange_interface *state_exchange,
     Gcs_xcom_view_change_control_interface *view_control, bool boot,
-    My_xp_socket_util *socket_util,
-    std::unique_ptr<Network_provider_operations_interface>
-        comms_operation_interface)
-    : m_gid(nullptr),
+    My_xp_socket_util *socket_util)
+    : m_gid(NULL),
       m_gid_hash(0),
       m_xcom_proxy(xcom_proxy),
       m_xcom_group_management(xcom_group_management),
       event_listeners(),
-      m_local_node_info(nullptr),
-      m_local_node_address(nullptr),
+      m_local_node_info(NULL),
+      m_local_node_address(NULL),
       m_state_exchange(state_exchange),
       m_xcom_thread(),
       m_socket_util(socket_util),
@@ -142,7 +139,6 @@ Gcs_xcom_control::Gcs_xcom_control(
       m_suspicions_manager(new Gcs_suspicions_manager(xcom_proxy, this)),
       m_suspicions_processing_thread(),
       m_sock_probe_interface(nullptr),
-      m_comms_operation_interface(std::move(comms_operation_interface)),
       m_xcom_running(false),
       m_leave_view_requested(false),
       m_leave_view_delivered(false),
@@ -177,7 +173,7 @@ Gcs_xcom_control::~Gcs_xcom_control() {
   delete m_sock_probe_interface;
 
   set_terminate_suspicion_thread(true);
-  m_suspicions_manager = nullptr;
+  m_suspicions_manager = NULL;
 
   clear_peer_nodes();
 }
@@ -204,7 +200,7 @@ void Gcs_xcom_control::set_boot_node(bool boot) { m_boot = boot; }
 My_xp_socket_util *Gcs_xcom_control::get_socket_util() { return m_socket_util; }
 /* purecov: end */
 
-void Gcs_xcom_control::wait_for_xcom_thread() { m_xcom_thread.join(nullptr); }
+void Gcs_xcom_control::wait_for_xcom_thread() { m_xcom_thread.join(NULL); }
 
 bool Gcs_xcom_control::is_xcom_running() { return m_xcom_running; }
 
@@ -291,18 +287,17 @@ enum_gcs_error Gcs_xcom_control::do_join(bool retry) {
     return GCS_NOK;
   }
 
-  while (ret == GCS_NOK && !m_view_control->is_finalized()) {
+  while (ret == GCS_NOK) {
     ret = retry_do_join();
 
     retry_join_count--;
     if (retry && m_join_attempts != 0 && ret == GCS_NOK &&
         retry_join_count >= 1) {
-      MYSQL_GCS_LOG_INFO("Sleeping for "
-                         << m_join_sleep_time
-                         << " seconds before retrying to join the group. There "
-                            "are "
-                         << retry_join_count
-                         << " more attempt(s) before giving up.");
+      MYSQL_GCS_LOG_DEBUG(
+          "Sleeping for %u seconds before retrying to join the group. There "
+          "are "
+          " %u more attempt(s) before giving up.",
+          m_join_sleep_time, retry_join_count);
       My_xp_util::sleep_seconds(m_join_sleep_time);
     } else {
       break;
@@ -339,35 +334,6 @@ static bool skip_own_peer_address(
   return false;
 }
 
-static constexpr const char *get_signaling_error() {
-  return "The group communication engine could not set up its internal event "
-         "notification mechanism. This may be due to one or more invalid "
-         "configuration settings. Double-check your group replication local "
-         "address, firewall"
-#if !defined(_WIN32)
-         ", SE Linux"
-#endif
-         " and TLS configurations and try "
-         "restarting Group Replication on this server.";
-}
-
-static const std::string get_connection_test_error(const std::string &address,
-                                                   int port) {
-  std::stringstream retval;
-  retval << "The group communication engine failed to test connectivity to the "
-            "local group communication engine on "
-         << address << ":" << port
-         << ". This may be due to one or more invalid configuration settings. "
-            "Double-check your group replication local address, firewall";
-#if !defined(_WIN32)
-  retval << ", SE Linux";
-#endif
-  retval << " and TLS configurations and try restarting Group Replication on "
-            "this server.";
-
-  return retval.str();
-}
-
 enum_gcs_error Gcs_xcom_control::retry_do_join() {
   /* Used to initialize xcom */
   int local_port = m_local_node_address->get_member_port();
@@ -386,12 +352,13 @@ enum_gcs_error Gcs_xcom_control::retry_do_join() {
 
   /* Spawn XCom's main loop thread. */
   if (local_port != 0) {
-    m_xcom_thread.create(key_GCS_THD_Gcs_xcom_control_m_xcom_thread, nullptr,
+    m_xcom_thread.create(key_GCS_THD_Gcs_xcom_control_m_xcom_thread, NULL,
                          xcom_taskmain_startup, (void *)this);
   } else {
     MYSQL_GCS_LOG_ERROR("Error initializing the group communication engine.")
     goto err;
   }
+
   // Wait for XCom comms to become ready
   m_xcom_proxy->xcom_wait_for_xcom_comms_status_change(comm_status);
 
@@ -413,7 +380,8 @@ enum_gcs_error Gcs_xcom_control::retry_do_join() {
      * Tested by the TEST_F(XComControlTest,
      * JoinTestFailedToConnectToXComQueueSignallingMechanism) GCS smoke test.
      */
-    MYSQL_GCS_LOG_ERROR(get_signaling_error());
+    MYSQL_GCS_LOG_ERROR("Error connecting to the local group communication"
+                        << " engine instance.")
     goto err;
     /* purecov: end */
   }
@@ -432,18 +400,12 @@ enum_gcs_error Gcs_xcom_control::retry_do_join() {
       m_local_node_address->get_member_ip(),
       m_local_node_address->get_member_port());
   if (!could_connect_to_local_xcom) {
-    MYSQL_GCS_LOG_ERROR(
-        get_connection_test_error(m_local_node_address->get_member_ip(),
-                                  m_local_node_address->get_member_port())
-            .c_str());
+    MYSQL_GCS_LOG_ERROR("Error connecting to the local group communication"
+                        << " engine instance.")
     goto err;
   }
 
   if (m_boot) {
-    MYSQL_GCS_LOG_INFO(
-        "Booting a group: "
-        << m_local_node_info->get_member_uuid().actual_value.c_str() << ":"
-        << local_port)
     MYSQL_GCS_LOG_TRACE(
         "::join():: I am the boot node. %d - %s. Calling xcom_client_boot.",
         local_port, m_local_node_info->get_member_uuid().actual_value.c_str())
@@ -496,8 +458,8 @@ enum_gcs_error Gcs_xcom_control::retry_do_join() {
   }
 
   m_xcom_running = true;
-  MYSQL_GCS_LOG_INFO(
-      "The member has joined the group. Local port: " << local_port);
+  MYSQL_GCS_LOG_DEBUG("The member has joined the group. Local port: %d",
+                      local_port);
 
   m_suspicions_manager->set_groupid_hash(m_gid_hash);
   m_suspicions_manager->set_my_info(m_local_node_info);
@@ -506,7 +468,7 @@ enum_gcs_error Gcs_xcom_control::retry_do_join() {
 
   // Initialize thread to deal with suspicions
   m_suspicions_processing_thread.create(
-      key_GCS_THD_Gcs_xcom_control_m_suspicions_processing_thread, nullptr,
+      key_GCS_THD_Gcs_xcom_control_m_suspicions_processing_thread, NULL,
       suspicions_processing_thread, (void *)this);
   MYSQL_GCS_LOG_TRACE("Started the suspicions processing thread...");
   m_view_control->end_join();
@@ -523,7 +485,12 @@ err:
         "Killing the group communication engine because the member failed to"
         " join. Local port: %d",
         local_port);
-    if (comm_status != XCOM_COMMS_ERROR) m_xcom_proxy->xcom_exit();
+    if (comm_status != XCOM_COMMS_ERROR &&
+        !m_xcom_proxy->xcom_exit(xcom_input_open)) {
+      MYSQL_GCS_LOG_WARN("Failed to kill the group communication engine "
+                         << "after the member failed to join. Local port: "
+                         << local_port);
+    }
     wait_for_xcom_thread();
   }
 
@@ -542,7 +509,6 @@ bool Gcs_xcom_control::send_add_node_request(
   // Go through the seed list S_CONNECTION_ATTEMPTS times.
   for (int attempt_nr = 0;
        !add_node_accepted && attempt_nr < CONNECTION_ATTEMPTS; attempt_nr++) {
-    if (m_view_control->is_finalized()) break;
     add_node_accepted = try_send_add_node_request_to_seeds(my_addresses);
   }
 
@@ -555,9 +521,7 @@ bool Gcs_xcom_control::try_send_add_node_request_to_seeds(
 
   // Until the add_node is successfully sent, for each peer...
   for (auto it = m_initial_peers.begin();
-       !m_view_control->is_finalized() && !add_node_accepted &&
-       it != m_initial_peers.end();
-       it++) {
+       !add_node_accepted && it != m_initial_peers.end(); it++) {
     // ...try to connect to it and send it the add_node request.
     Gcs_xcom_node_address &peer = **it;
 
@@ -565,13 +529,7 @@ bool Gcs_xcom_control::try_send_add_node_request_to_seeds(
     connection_descriptor *con = nullptr;
     std::tie(connected, con) = connect_to_peer(peer, my_addresses);
 
-    if (bool finalized = m_view_control->is_finalized();
-        !finalized && connected) {
-      MYSQL_GCS_LOG_INFO("Sucessfully connected to peer "
-                         << peer.get_member_ip().c_str() << ":"
-                         << peer.get_member_port()
-                         << ". Sending a request to be added to the group");
-
+    if (connected) {
       MYSQL_GCS_LOG_TRACE(
           "::join():: Calling xcom_client_add_node %d_%s connected to "
           "%s:%d "
@@ -591,14 +549,12 @@ bool Gcs_xcom_control::try_send_add_node_request_to_seeds(
        If not, the peer will not process our request because something
        happened, e.g.:
          - The peer crashed;
-         - The peer killed the connection because we're not allowlist;
+         - The peer killed the connection because we're not whitelisted;
          - The peer rejected our request.
        In this case, we continue the loop and try again using the next peer.
       */
       if (xcom_will_process) add_node_accepted = true;
     }
-
-    free_connection(con);
   }
 
   return add_node_accepted;
@@ -624,11 +580,10 @@ std::pair<bool, connection_descriptor *> Gcs_xcom_control::connect_to_peer(
       m_local_node_address->get_member_port(), addr.c_str(), port);
 
   con = m_xcom_proxy->xcom_client_open_connection(addr, port);
-  if (con->fd == -1) {
+  if (con == nullptr) {
     // Could not connect to the peer.
-    MYSQL_GCS_LOG_ERROR("Error on opening a connection to peer node "
-                        << addr << ":" << port
-                        << " when joining a group. My local port is: "
+    MYSQL_GCS_LOG_ERROR("Error on opening a connection to "
+                        << addr << ":" << port << " on local port: "
                         << m_local_node_address->get_member_port() << ".");
     goto end;
   }
@@ -716,7 +671,15 @@ enum_gcs_error Gcs_xcom_control::do_leave() {
       We have to really kill the XCOM's thread at this point because
       an attempt to make it gracefully exit apparently has failed.
     */
-    m_xcom_proxy->xcom_exit();
+    bool const exit_sent = m_xcom_proxy->xcom_exit(true);
+    if (!exit_sent) {
+      /* purecov: begin deadcode */
+      /* exit_sent will ALWAYS be true. */
+      MYSQL_GCS_LOG_WARN(
+          "Failed to kill the group communication engine "
+          "after the member has failed to leave the group.");
+      /* purecov: end */
+    }
   }
   wait_for_xcom_thread();
 
@@ -726,7 +689,7 @@ enum_gcs_error Gcs_xcom_control::do_leave() {
 
   m_suspicions_manager->wake_suspicions_processing_thread(true);
 
-  m_suspicions_processing_thread.join(nullptr);
+  m_suspicions_processing_thread.join(NULL);
   MYSQL_GCS_LOG_TRACE("The suspicions processing thread has joined.");
   MYSQL_GCS_LOG_DEBUG("The member left the group.")
 
@@ -737,7 +700,7 @@ enum_gcs_error Gcs_xcom_control::do_leave() {
   /*
     Delete current view and set it to NULL.
   */
-  m_view_control->set_current_view(nullptr);
+  m_view_control->set_current_view(NULL);
 
   return GCS_OK;
 }
@@ -749,7 +712,7 @@ void Gcs_xcom_control::do_leave_view() {
   */
   Gcs_view *current_view = m_view_control->get_unsafe_current_view();
 
-  if (current_view != nullptr && !m_leave_view_delivered) {
+  if (current_view != NULL && !m_leave_view_delivered) {
     MYSQL_GCS_LOG_DEBUG("Will install leave view: requested %d, delivered %d",
                         m_leave_view_requested, m_leave_view_delivered);
     install_leave_view(m_leave_view_requested ? Gcs_view::OK
@@ -765,7 +728,7 @@ void Gcs_xcom_control::do_leave_view() {
 
 connection_descriptor *Gcs_xcom_control::get_connection_to_node(
     std::vector<Gcs_xcom_node_address *> *peers_list) {
-  connection_descriptor *con = nullptr;
+  connection_descriptor *con = NULL;
   std::vector<Gcs_xcom_node_address *>::iterator it;
 
   std::map<std::string, int> local_node_info_str_ips;
@@ -779,7 +742,7 @@ connection_descriptor *Gcs_xcom_control::get_connection_to_node(
     return con;
   }
 
-  for (it = peers_list->begin(); (con == nullptr) && it != peers_list->end();
+  for (it = peers_list->begin(); (con == NULL) && it != peers_list->end();
        it++) {
     Gcs_xcom_node_address *peer = *(it);
     std::string peer_rep_ip;
@@ -798,14 +761,13 @@ connection_descriptor *Gcs_xcom_control::get_connection_to_node(
         "get_connection_to_node: xcom_client_open_connection to %s:%d", addr,
         port)
 
-    con = m_xcom_proxy->xcom_client_open_connection(addr, port);
-    if (con->fd == -1) {
+    if ((con = m_xcom_proxy->xcom_client_open_connection(addr, port)) == NULL) {
       MYSQL_GCS_LOG_DEBUG(
           "get_connection_to_node: Error while opening a connection to %s:%d",
           addr, port)
     } else
       MYSQL_GCS_LOG_DEBUG("get_connection_to_node: Opened connection to %s:%d ",
-                          "con is null? %d", addr, port, (con == nullptr))
+                          "con is null? %d", addr, port, (con == NULL))
   }
 
   return con;
@@ -817,7 +779,7 @@ void Gcs_xcom_control::do_remove_node_from_group() {
 
   int local_port = m_local_node_address->get_member_port();
   bool rm_ret = false;
-  connection_descriptor *con = nullptr;
+  connection_descriptor *con = NULL;
 
   MYSQL_GCS_LOG_DEBUG("do_remove_node_from_group started! (%d)", local_port);
 
@@ -830,7 +792,7 @@ void Gcs_xcom_control::do_remove_node_from_group() {
   // VIEW MEMBERS
   Gcs_view *current_view = m_view_control->get_current_view();
 
-  if (current_view != nullptr) {
+  if (current_view != NULL) {
     std::vector<Gcs_member_identifier>::const_iterator it;
     std::vector<Gcs_xcom_node_address *> view_members;
 
@@ -863,21 +825,17 @@ void Gcs_xcom_control::do_remove_node_from_group() {
     delete current_view;
   }
 
-  if (con->fd == -1) {
+  if (!con) {
     // GET INITIAL PEERS
     MYSQL_GCS_LOG_DEBUG(
         "do_remove_node_from_group: (%d) Couldn't get a connection from view! "
         "Using initial peers...",
         local_port)
 
-    free(con);
-    con = nullptr;
-
     con = get_connection_to_node(&m_initial_peers);
   }
 
-  if (con->fd != -1 && !m_leave_view_delivered &&
-      m_view_control->belongs_to_group()) {
+  if (con && !m_leave_view_delivered && m_view_control->belongs_to_group()) {
     MYSQL_GCS_LOG_TRACE(
         "do_remove_node_from_group: (%d) got a connection! "
         "m_leave_view_delivered=%d belongs=%d",
@@ -895,12 +853,10 @@ void Gcs_xcom_control::do_remove_node_from_group() {
         local_port);
   }
 
-  if (con->fd != -1) {
-    m_xcom_proxy->xcom_client_close_connection(con);
+  if (con) {
+    xcom_close_client_connection(con);
   }
 
-  if (con != nullptr) free(con);
-
   /*
     Destroy this node's stored suspicions to avoid them from unnecessary
     processing by the manager.
@@ -993,7 +949,7 @@ void Gcs_xcom_control::build_joined_members(
       considered nodes that are joining.
     */
     bool joined = true;
-    if (current_members != nullptr) {
+    if (current_members != NULL) {
       current_members_it =
           std::find(current_members->begin(), current_members->end(),
                     *(*alive_members_it));
@@ -1020,7 +976,7 @@ void Gcs_xcom_control::build_left_members(
     been installed before and nobody can leave something that does not
     exist.
   */
-  if (current_members == nullptr) return;
+  if (current_members == NULL) return;
 
   for (current_members_it = current_members->begin();
        current_members_it != current_members->end(); current_members_it++) {
@@ -1054,7 +1010,7 @@ void Gcs_xcom_control::build_member_suspect_nodes(
     If there isn't a set of current members, this means that a view hasn't
     been installed before and nobody will be expelled by this node.
   */
-  if ((current_members == nullptr) || current_members->empty() ||
+  if ((current_members == NULL) || current_members->empty() ||
       failed_members.empty())
     return;
 
@@ -1086,7 +1042,7 @@ void Gcs_xcom_control::build_non_member_suspect_nodes(
     If there isn't a set of failed members, this means that there are no
     suspect nodes.
   */
-  if ((current_members == nullptr) || current_members->empty() ||
+  if ((current_members == NULL) || current_members->empty() ||
       failed_members.empty())
     return;
 
@@ -1114,7 +1070,7 @@ bool Gcs_xcom_control::is_killer_node(
     if they are considered faulty is the first one in the list of alive
     members.
   */
-  assert(alive_members.size() != 0 && alive_members[0] != nullptr);
+  assert(alive_members.size() != 0 && alive_members[0] != NULL);
   bool ret = get_local_member_identifier() == *alive_members[0];
   MYSQL_GCS_LOG_DEBUG("The member %s will be responsible for killing: %d",
                       get_local_member_identifier().get_member_id().c_str(),
@@ -1122,8 +1078,7 @@ bool Gcs_xcom_control::is_killer_node(
   return ret;
 }
 
-bool Gcs_xcom_control::xcom_receive_local_view(synode_no const config_id,
-                                               Gcs_xcom_nodes *xcom_nodes,
+bool Gcs_xcom_control::xcom_receive_local_view(Gcs_xcom_nodes *xcom_nodes,
                                                synode_no max_synode) {
   std::map<int, const Gcs_control_event_listener &>::const_iterator callback_it;
   std::vector<Gcs_member_identifier> members;
@@ -1136,7 +1091,7 @@ bool Gcs_xcom_control::xcom_receive_local_view(synode_no const config_id,
   if (xcom_nodes->get_size() <= 0) goto end;
 
   // Ignore view if member has been expelled
-  if (current_view != nullptr &&
+  if (current_view != NULL &&
       !current_view->has_member(
           m_local_node_info->get_member_id().get_member_id())) {
     MYSQL_GCS_LOG_DEBUG(
@@ -1146,7 +1101,7 @@ bool Gcs_xcom_control::xcom_receive_local_view(synode_no const config_id,
   }
 
   // if I am not aware of any view at all
-  if (current_view != nullptr) {
+  if (current_view != NULL) {
     std::vector<Gcs_member_identifier *> alive_members;
     std::vector<Gcs_member_identifier *> failed_members;
     std::vector<Gcs_member_identifier *> left_members;
@@ -1186,7 +1141,7 @@ bool Gcs_xcom_control::xcom_receive_local_view(synode_no const config_id,
       Identify which nodes are alive and which are considered faulty.
 
       Note that there may be new nodes that are marked as faulty because the
-      connections among their peers are still being established.
+      connections among their peers are still beeing established.
     */
     build_total_members(xcom_nodes, alive_members, failed_members);
 
@@ -1228,9 +1183,8 @@ bool Gcs_xcom_control::xcom_receive_local_view(synode_no const config_id,
 
     // Remove and add suspicions
     m_suspicions_manager->process_view(
-        config_id, xcom_nodes, alive_members, left_members,
-        member_suspect_nodes, non_member_suspect_nodes,
-        is_killer_node(alive_members), max_synode);
+        xcom_nodes, alive_members, left_members, member_suspect_nodes,
+        non_member_suspect_nodes, is_killer_node(alive_members), max_synode);
 
     MYSQL_GCS_TRACE_EXECUTE(
         unsigned int node_no = xcom_nodes->get_node_no();
@@ -1334,7 +1288,7 @@ void Gcs_xcom_control::install_leave_view(
   MYSQL_GCS_LOG_DEBUG("Installing leave view.")
 
   Gcs_group_identifier gid(current_view->get_group_id().get_group_id());
-  install_view(new_view_id, gid, nullptr, total, left, joined, error_code);
+  install_view(new_view_id, gid, NULL, total, left, joined, error_code);
 
   set<Gcs_member_identifier *>::iterator total_it;
   for (total_it = total->begin(); total_it != total->end(); total_it++)
@@ -1364,10 +1318,9 @@ bool Gcs_xcom_control::is_this_node_in(
   return is_in_vector;
 }
 
-bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
-                                                synode_no message_id,
+bool Gcs_xcom_control::xcom_receive_global_view(synode_no message_id,
                                                 Gcs_xcom_nodes *xcom_nodes,
-                                                bool do_not_deliver_to_client,
+                                                bool same_view,
                                                 synode_no max_synode) {
   bool ret = false;
   bool free_built_members = false;
@@ -1392,18 +1345,18 @@ bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
     of members.
   */
   Gcs_view *current_view = m_view_control->get_unsafe_current_view();
-  std::vector<Gcs_member_identifier> *current_members = nullptr;
-  if (current_view != nullptr)
+  std::vector<Gcs_member_identifier> *current_members = NULL;
+  if (current_view != NULL)
     current_members = const_cast<std::vector<Gcs_member_identifier> *>(
         &current_view->get_members());
   MYSQL_GCS_LOG_TRACE("::xcom_receive_global_view():: My node_id is %d",
-                      xcom_nodes->get_node_no());
+                      xcom_nodes->get_node_no())
 
   /*
     Identify which nodes are alive and which are considered faulty.
 
     Note that there may be new nodes that are marked as faulty because the
-    connections among their peers are still being established.
+    connections among their peers are still beeing established.
   */
   build_total_members(xcom_nodes, alive_members, failed_members);
 
@@ -1445,7 +1398,7 @@ bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
 
   // Remove and add suspicions
   m_suspicions_manager->process_view(
-      config_id, xcom_nodes, alive_members, left_members, member_suspect_nodes,
+      xcom_nodes, alive_members, left_members, member_suspect_nodes,
       non_member_suspect_nodes, is_killer_node(alive_members), max_synode);
 
   /*
@@ -1493,8 +1446,8 @@ bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
   const Gcs_xcom_node_information *node_info =
       xcom_nodes->get_node(m_local_node_info->get_member_id());
 
-  if ((current_view != nullptr) &&
-      ((nullptr == node_info) || is_this_node_in(&left_members))) {
+  if ((current_view != NULL) &&
+      ((NULL == node_info) || is_this_node_in(&left_members))) {
     MYSQL_GCS_LOG_TRACE(
         "::xcom_receive_global_view()::I'm node %s and I'm not in the "
         "view! "
@@ -1508,10 +1461,10 @@ bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
 
   /*
     If nobody has joined or left the group, there is no need to install any
-    view. This fact is denoted by the do_not_deliver_to_client flag and the
-    execution only gets so far because we may have to kill some faulty members.
-    Note that the code could be optimized but we are focused on correctness for
-    now and any possible optimization will be handled in the future.
+    view. This fact is denoted by the same_view flag and the execution only
+    gets so far because we may have to kill some faulty members. Note that
+    the code could be optimized but we are focused on correctness for now
+    and any possible optimization will be handled in the future.
 
     Views that contain a node marked as faulty are also ignored. This is
     done because XCOM does not deliver messages to or accept messages from
@@ -1527,13 +1480,13 @@ bool Gcs_xcom_control::xcom_receive_global_view(synode_no const config_id,
     If the execution has managed to pass these steps, the node is alive and
     it is time to start building the view.
   */
-  if (do_not_deliver_to_client || failed_members.size() != 0) {
+  if (same_view || failed_members.size() != 0) {
     MYSQL_GCS_LOG_TRACE(
         "(My node_id is %d) ::xcom_receive_global_view():: "
         "Discarding view because nothing has changed. "
-        "Do not deliver to client flag is %d, number of failed nodes is %llu"
+        "Same view flag is %d, number of failed nodes is %llu"
         ", number of joined nodes is %llu, number of left nodes is %llu",
-        xcom_nodes->get_node_no(), do_not_deliver_to_client,
+        xcom_nodes->get_node_no(), same_view,
         static_cast<long long unsigned>(failed_members.size()),
         static_cast<long long unsigned>(joined_members.size()),
         static_cast<long long unsigned>(left_members.size()));
@@ -1628,8 +1581,6 @@ end:
   return ret;
 }
 
-extern uint32_t get_my_xcom_id();
-
 void Gcs_xcom_control::process_control_message(
     Gcs_message *msg, Gcs_protocol_version maximum_supported_protocol_version,
     Gcs_protocol_version used_protocol_version) {
@@ -1646,9 +1597,8 @@ void Gcs_xcom_control::process_control_message(
           msg->get_message_data().get_payload_length()));
 
   MYSQL_GCS_LOG_TRACE(
-      "xcom_id %x ::process_control_message():: From: %s regarding view_id: %s "
-      "in %s",
-      get_my_xcom_id(), msg->get_origin().get_member_id().c_str(),
+      "::process_control_message():: From: %s regarding view_id: %s in %s",
+      msg->get_origin().get_member_id().c_str(),
       ms_info->get_view_id()->get_representation().c_str(),
       get_node_address()->get_member_address().c_str())
 
@@ -1660,21 +1610,18 @@ void Gcs_xcom_control::process_control_message(
   MYSQL_GCS_DEBUG_EXECUTE(
       synode_no configuration_id = ms_info->get_configuration_id();
       if (!m_view_control->is_view_changing()){MYSQL_GCS_LOG_DEBUG(
-          "xcom_id %x There is no state exchange going on. Ignoring "
-          "exchangeable data "
+          "There is no state exchange going on. Ignoring exchangeable data "
           "because its from a previous state exchange phase. Message is "
           "from group_id (%u), msg_no(%llu), node_no(%llu)",
-          get_my_xcom_id(), configuration_id.group_id,
+          configuration_id.group_id,
           static_cast<long long unsigned>(configuration_id.msgno),
           static_cast<long long unsigned>(
               configuration_id
-                  .node))} MYSQL_GCS_LOG_DEBUG("xcom_id %x There is a state "
-                                               "exchange "
+                  .node))} MYSQL_GCS_LOG_DEBUG("There is a state exchange "
                                                "going on. Message is "
                                                "from group_id (%u), "
                                                "msg_no(%llu), "
                                                "node_no(%llu)",
-                                               get_my_xcom_id(),
                                                configuration_id.group_id,
                                                static_cast<long long unsigned>(
                                                    configuration_id.msgno),
@@ -1780,7 +1727,7 @@ void Gcs_xcom_control::install_view(
 
   // Build the exchanged data
   Exchanged_data data_to_deliver;
-  if (states != nullptr) {
+  if (states != NULL) {
     std::map<Gcs_member_identifier, Xcom_member_state *>::iterator states_it;
     for (states_it = states->begin(); states_it != states->end(); states_it++) {
       MYSQL_GCS_LOG_DEBUG(
@@ -1791,9 +1738,9 @@ void Gcs_xcom_control::install_view(
 
       Xcom_member_state *data_exchanged = (*states_it).second;
 
-      Gcs_message_data *data_exchanged_holder = nullptr;
+      Gcs_message_data *data_exchanged_holder = NULL;
 
-      if (data_exchanged != nullptr && data_exchanged->get_data_size() != 0) {
+      if (data_exchanged != NULL && data_exchanged->get_data_size() != 0) {
         data_exchanged_holder =
             new Gcs_message_data(data_exchanged->get_data_size());
         data_exchanged_holder->decode(data_exchanged->get_data(),
@@ -1859,7 +1806,7 @@ void Gcs_xcom_control::build_member_list(
 }
 
 void Gcs_xcom_control::init_me() {
-  assert(m_local_node_info != nullptr);
+  assert(m_local_node_info != NULL);
   m_local_node_info->regenerate_member_uuid();
 }
 
@@ -1874,10 +1821,6 @@ void Gcs_xcom_control::set_node_address(
     init_me() method.
   */
   m_local_node_info = new Gcs_xcom_node_information(address);
-
-  Network_configuration_parameters params;
-  params.port = xcom_node_address->get_member_port();
-  m_comms_operation_interface->configure_active_provider(params);
 }
 
 void Gcs_xcom_control::set_peer_nodes(
@@ -1936,11 +1879,10 @@ Gcs_suspicions_manager::Gcs_suspicions_manager(Gcs_xcom_proxy *proxy,
       m_is_killer_node(false),
       m_cache_last_removed(null_synode) {
   m_suspicions_mutex.init(
-      key_GCS_MUTEX_Gcs_suspicions_manager_m_suspicions_mutex, nullptr);
+      key_GCS_MUTEX_Gcs_suspicions_manager_m_suspicions_mutex, NULL);
   m_suspicions_cond.init(key_GCS_COND_Gcs_suspicions_manager_m_suspicions_cond);
   m_suspicions_parameters_mutex.init(
-      key_GCS_MUTEX_Gcs_suspicions_manager_m_suspicions_parameters_mutex,
-      nullptr);
+      key_GCS_MUTEX_Gcs_suspicions_manager_m_suspicions_parameters_mutex, NULL);
 }
 
 Gcs_suspicions_manager::~Gcs_suspicions_manager() {
@@ -1951,7 +1893,7 @@ Gcs_suspicions_manager::~Gcs_suspicions_manager() {
 
 void Gcs_suspicions_manager::remove_suspicions(
     std::vector<Gcs_member_identifier *> nodes) {
-  const Gcs_xcom_node_information *xcom_node = nullptr;
+  const Gcs_xcom_node_information *xcom_node = NULL;
   std::vector<Gcs_member_identifier *>::iterator non_suspect_it;
 
   // Foreach received node
@@ -1959,7 +1901,7 @@ void Gcs_suspicions_manager::remove_suspicions(
        ++non_suspect_it) {
     const Gcs_xcom_node_information node_to_remove(
         (*non_suspect_it)->get_member_id());
-    if ((xcom_node = m_suspicions.get_node(*(*non_suspect_it))) != nullptr) {
+    if ((xcom_node = m_suspicions.get_node(*(*non_suspect_it))) != NULL) {
       m_suspicions.remove_node(node_to_remove);
       MYSQL_GCS_LOG_DEBUG("Removed suspicion on node %s",
                           (*non_suspect_it)->get_member_id().c_str());
@@ -1979,74 +1921,25 @@ void Gcs_suspicions_manager::clear_suspicions() {
                         (*susp_it).get_member_id().get_member_id().c_str())
     m_suspicions.remove_node(*susp_it);
   }
-
-  m_expels_in_progress = Gcs_xcom_expels_in_progress();
   m_suspicions_mutex.unlock();
 }
 
 void Gcs_suspicions_manager::process_view(
-    synode_no const config_id, Gcs_xcom_nodes *xcom_nodes,
+    Gcs_xcom_nodes *xcom_nodes,
     std::vector<Gcs_member_identifier *> alive_nodes,
     std::vector<Gcs_member_identifier *> left_nodes,
     std::vector<Gcs_member_identifier *> member_suspect_nodes,
     std::vector<Gcs_member_identifier *> non_member_suspect_nodes,
     bool is_killer_node, synode_no max_synode) {
   bool should_wake_up_manager = false;
-  std::vector<Gcs_member_identifier *> back_to_healthy;
 
   m_suspicions_mutex.lock();
 
   m_is_killer_node = is_killer_node;
 
-  m_config_id = config_id;
-
-  /* Remove the information about expels of members left because they
-   * have already taken effect(left). */
-  m_expels_in_progress.forget_expels_that_have_taken_effect(config_id,
-                                                            left_nodes);
-
-  /* Remove the information about expels of members alive because they
-   * have already taken effect(rejoined). */
-  if (m_expels_in_progress.size() && !alive_nodes.empty()) {
-    for (Gcs_member_identifier *const &live : alive_nodes) {
-      if (m_expels_in_progress.contains(*live)) {
-        MYSQL_GCS_LOG_DEBUG(
-            "%s: Expelled node %s, rejoined the group immediately.", __func__,
-            live->get_member_id().c_str());
-        back_to_healthy.push_back(live);
-      }
-    }
-    if (!back_to_healthy.empty())
-      m_expels_in_progress.forget_expels_that_have_taken_effect(
-          config_id, back_to_healthy);
-  }
-
-  MYSQL_GCS_DEBUG_EXECUTE({
-    /* Sanity check: all members in `m_expels_in_progress` must still be in
-       `xcom_nodes` (the XCom view) at this point. Otherwise there is a bug in
-       the logic implemented by `remember_expels_issued` and
-       `forget_expels_that_have_taken_effect` that is creating uncollected
-       garbage in `m_expels_in_progress`.
-     */
-    assert(m_expels_in_progress.all_still_in_view(*xcom_nodes));
-  });
-
-  /* Count any node whose expel we issued, but whose expel has not yet taken
-     effect, as a suspected node for the purposes of deciding whether the view
-     has majority. */
-  auto const total_number_nodes = xcom_nodes->get_nodes().size();
-  auto const number_of_alive_members_expelled_but_not_yet_removed =
-      m_expels_in_progress.number_of_expels_not_about_suspects(
-          member_suspect_nodes, non_member_suspect_nodes);
-  auto const total_number_suspect_nodes =
-      (member_suspect_nodes.size() + non_member_suspect_nodes.size() +
-       number_of_alive_members_expelled_but_not_yet_removed);
-  m_has_majority = (2 * total_number_suspect_nodes < total_number_nodes);
-
-  MYSQL_GCS_LOG_DEBUG(
-      "%s: total_number_nodes=%u total_number_suspect_nodes=%u "
-      "m_has_majority=%d",
-      __func__, total_number_nodes, total_number_suspect_nodes, m_has_majority);
+  m_has_majority =
+      2 * (member_suspect_nodes.size() + non_member_suspect_nodes.size()) <
+      xcom_nodes->get_nodes().size();
 
   /*
     Suspicions are removed for members that are alive. Therefore, the
@@ -2082,7 +1975,7 @@ bool Gcs_suspicions_manager::add_suspicions(
     std::vector<Gcs_member_identifier *> non_member_suspect_nodes,
     std::vector<Gcs_member_identifier *> member_suspect_nodes,
     synode_no max_synode) {
-  const Gcs_xcom_node_information *xcom_node = nullptr;
+  const Gcs_xcom_node_information *xcom_node = NULL;
   std::vector<Gcs_member_identifier *>::iterator susp_it;
   bool member_suspicions_added = false;
 
@@ -2091,7 +1984,7 @@ bool Gcs_suspicions_manager::add_suspicions(
 
   for (susp_it = non_member_suspect_nodes.begin();
        susp_it != non_member_suspect_nodes.end(); ++susp_it) {
-    if ((xcom_node = m_suspicions.get_node(*(*susp_it))) == nullptr) {
+    if ((xcom_node = m_suspicions.get_node(*(*susp_it))) == NULL) {
       MYSQL_GCS_LOG_DEBUG(
           "add_suspicions: Adding non-member expel suspicion for %s",
           (*susp_it)->get_member_id().c_str())
@@ -2111,7 +2004,7 @@ bool Gcs_suspicions_manager::add_suspicions(
 
   for (susp_it = member_suspect_nodes.begin();
        susp_it != member_suspect_nodes.end(); ++susp_it) {
-    if ((xcom_node = m_suspicions.get_node(*(*susp_it))) == nullptr) {
+    if ((xcom_node = m_suspicions.get_node(*(*susp_it))) == NULL) {
       MYSQL_GCS_LOG_DEBUG(
           "add_suspicions: Adding member expel suspicion for %s",
           (*susp_it)->get_member_id().c_str())
@@ -2175,7 +2068,7 @@ void Gcs_suspicions_manager::run_process_suspicions(bool lock) {
   }
 
   // List of nodes to remove
-  Gcs_xcom_nodes nodes_to_remove, nodes_to_remember_expel;
+  Gcs_xcom_nodes nodes_to_remove;
 
   bool force_remove = false;
 
@@ -2215,9 +2108,6 @@ void Gcs_suspicions_manager::run_process_suspicions(bool lock) {
       }
 
       nodes_to_remove.add_node(*susp_it);
-      if ((*susp_it).is_member()) {
-        nodes_to_remember_expel.add_node(*susp_it);
-      }
       m_suspicions.remove_node(*susp_it);
       /* purecov: end */
     } else {
@@ -2238,31 +2128,15 @@ void Gcs_suspicions_manager::run_process_suspicions(bool lock) {
     }
   }
 
-  /* We only act when we have something to do and think we have a majority at
-     the moment. */
-  if (!nodes_to_remove.empty() && m_has_majority) {
-    /* If I am the killer node, I remove the suspects.
-       Otherwise, I proactively remove myself if I suspect myself, i.e.
-       force_remove == true. I might have been partitioned away from the
-       majority of the group for too long, in which case I may not receive the
-       view where I am removed. (Why?) */
-    if (m_is_killer_node) {
-      MYSQL_GCS_LOG_TRACE(
-          "process_suspicions: Expelling suspects that timed out!");
-      bool const removed =
-          m_proxy->xcom_remove_nodes(nodes_to_remove, m_gid_hash);
-      if (removed && !nodes_to_remember_expel.empty()) {
-        m_expels_in_progress.remember_expels_issued(m_config_id,
-                                                    nodes_to_remember_expel);
-      }
-    } else if (force_remove) {
-      assert(!m_is_killer_node);
-      MYSQL_GCS_LOG_TRACE("process_suspicions: Expelling myself!");
-      bool const removed = m_proxy->xcom_remove_node(*m_my_info, m_gid_hash);
-      if (!removed) {
-        // Failed to remove myself from the group so will install leave view
-        m_control_if->install_leave_view(Gcs_view::MEMBER_EXPELLED);
-      }
+  if (((m_is_killer_node && m_has_majority) || force_remove) &&
+      (nodes_to_remove.get_size() > 0)) {
+    MYSQL_GCS_LOG_TRACE(
+        "process_suspicions: Expelling suspects that timed out!");
+    bool const removed =
+        m_proxy->xcom_remove_nodes(nodes_to_remove, m_gid_hash);
+    if (force_remove && !removed) {
+      // Failed to remove myself from the group so will install leave view
+      m_control_if->install_leave_view(Gcs_view::MEMBER_EXPELLED);
     }
   }
 
