@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,9 +38,9 @@
 #include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/sock_probe.h"
 
 #if defined(_WIN32)
-#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/sock_probe_win32.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/sock_probe_win32.c"
 #else
-#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/sock_probe_ix.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/sock_probe_ix.c"
 #endif
 
 /**
@@ -131,7 +130,7 @@ bool get_local_addresses(Gcs_sock_probe_interface &sock_probe_if,
       }
 
       if (ip->sa_family == AF_INET) {
-        struct in_addr *inaddr = nullptr, *inmask = nullptr;
+        struct in_addr *inaddr = NULL, *inmask = NULL;
 
         inaddr = &((struct sockaddr_in *)ip)->sin_addr;
         inmask = &((struct sockaddr_in *)netmask)->sin_addr;
@@ -156,7 +155,7 @@ bool get_local_addresses(Gcs_sock_probe_interface &sock_probe_if,
 
         addr_to_cidr_bits.insert(std::make_pair(sname, prefix.count()));
       } else if (ip->sa_family == AF_INET6) {
-        struct in6_addr *inaddrv6 = nullptr, *inmaskv6 = nullptr;
+        struct in6_addr *inaddrv6 = NULL, *inmaskv6 = NULL;
 
         inaddrv6 = &((struct sockaddr_in6 *)ip)->sin6_addr;
         inmaskv6 = &((struct sockaddr_in6 *)netmask)->sin6_addr;
@@ -260,12 +259,12 @@ bool resolve_ip_addr_from_hostname(std::string name,
   int res = true;
   char cip[INET6_ADDRSTRLEN];
   socklen_t cip_len = static_cast<socklen_t>(sizeof(cip));
-  struct addrinfo *addrinf = nullptr, *addrinf_cycle = nullptr, hints;
-  struct sockaddr *sa = nullptr;
-  void *in_addr = nullptr;
+  struct addrinfo *addrinf = NULL, *addrinf_cycle = NULL, hints;
+  struct sockaddr *sa = NULL;
+  void *in_addr = NULL;
 
   memset(&hints, 0, sizeof(hints));
-  checked_getaddrinfo(name.c_str(), nullptr, &hints, &addrinf);
+  checked_getaddrinfo(name.c_str(), 0, &hints, &addrinf);
   if (!addrinf) goto end;
 
   addrinf_cycle = addrinf;
@@ -302,12 +301,12 @@ bool resolve_all_ip_addr_from_hostname(
   int res = true;
   char cip[INET6_ADDRSTRLEN];
   socklen_t cip_len = static_cast<socklen_t>(sizeof(cip));
-  struct addrinfo *addrinf = nullptr, *addrinfo_list = nullptr, hints;
-  struct sockaddr *sa = nullptr;
-  void *in_addr = nullptr;
+  struct addrinfo *addrinf = NULL, *addrinfo_list = NULL, hints;
+  struct sockaddr *sa = NULL;
+  void *in_addr = NULL;
 
   memset(&hints, 0, sizeof(hints));
-  checked_getaddrinfo(name.c_str(), nullptr, &hints, &addrinf);
+  checked_getaddrinfo(name.c_str(), 0, &hints, &addrinf);
   if (!addrinf) goto end;
 
   addrinfo_list = addrinf;
@@ -480,56 +479,56 @@ bool_t Gcs_sock_probe_interface_impl::is_if_running(sock_probe *s, int count) {
 }
 
 /*
-  The default allowlist contains all locak-link and private address ranges.
+  The default whitelist contains all locak-link and private address ranges.
   Please refer to the documentation of get_local_private_addresses() to a better
   understanding of this concept.
 */
-const std::string Gcs_ip_allowlist::DEFAULT_ALLOWLIST =
+const std::string Gcs_ip_whitelist::DEFAULT_WHITELIST =
     "127.0.0.1/32,10.0.0.0/8,172.16.0.0/12,192.168.0.0/16,::1/128,fe80::/"
     "10,fd00::/8";
 
-Gcs_ip_allowlist_entry::Gcs_ip_allowlist_entry(std::string addr,
+Gcs_ip_whitelist_entry::Gcs_ip_whitelist_entry(std::string addr,
                                                std::string mask)
     : m_addr(addr), m_mask(mask) {}
 
-Gcs_ip_allowlist_entry_ip::Gcs_ip_allowlist_entry_ip(std::string addr,
+Gcs_ip_whitelist_entry_ip::Gcs_ip_whitelist_entry_ip(std::string addr,
                                                      std::string mask)
-    : Gcs_ip_allowlist_entry(addr, mask) {}
+    : Gcs_ip_whitelist_entry(addr, mask) {}
 
-bool Gcs_ip_allowlist_entry_ip::init_value() {
-  bool error = get_address_for_allowlist(get_addr(), get_mask(), m_value);
+bool Gcs_ip_whitelist_entry_ip::init_value() {
+  bool error = get_address_for_whitelist(get_addr(), get_mask(), m_value);
 
   return error;
 }
 
 std::vector<std::pair<std::vector<unsigned char>, std::vector<unsigned char>>>
-    *Gcs_ip_allowlist_entry_ip::get_value() {
+    *Gcs_ip_whitelist_entry_ip::get_value() {
   return new std::vector<
       std::pair<std::vector<unsigned char>, std::vector<unsigned char>>>(
       {m_value});
 }
 
-Gcs_ip_allowlist_entry_hostname::Gcs_ip_allowlist_entry_hostname(
+Gcs_ip_whitelist_entry_hostname::Gcs_ip_whitelist_entry_hostname(
     std::string addr, std::string mask)
-    : Gcs_ip_allowlist_entry(addr, mask) {}
+    : Gcs_ip_whitelist_entry(addr, mask) {}
 
-Gcs_ip_allowlist_entry_hostname::Gcs_ip_allowlist_entry_hostname(
+Gcs_ip_whitelist_entry_hostname::Gcs_ip_whitelist_entry_hostname(
     std::string addr)
-    : Gcs_ip_allowlist_entry(addr, "") {}
+    : Gcs_ip_whitelist_entry(addr, "") {}
 
-bool Gcs_ip_allowlist_entry_hostname::init_value() { return false; }
+bool Gcs_ip_whitelist_entry_hostname::init_value() { return false; }
 
 std::vector<std::pair<std::vector<unsigned char>, std::vector<unsigned char>>>
-    *Gcs_ip_allowlist_entry_hostname::get_value() {
+    *Gcs_ip_whitelist_entry_hostname::get_value() {
   bool error = false;
   std::pair<std::vector<unsigned char>, std::vector<unsigned char>> value;
 
   std::vector<std::pair<sa_family_t, std::string>> ips;
   if (resolve_all_ip_addr_from_hostname(get_addr(), ips)) {
     MYSQL_GCS_LOG_WARN("Hostname "
-                       << get_addr().c_str() << " in Allowlist"
+                       << get_addr().c_str() << " in Whitelist"
                        << " configuration was not resolvable. Please check your"
-                       << " Allowlist configuration.");
+                       << " Whitelist configuration.");
     return nullptr;
   }
 
@@ -556,12 +555,9 @@ std::vector<std::pair<std::vector<unsigned char>, std::vector<unsigned char>>>
         mask.append("128");
     }
 
-    error = get_address_for_allowlist(ip.second, mask, value);
+    error = get_address_for_whitelist(ip.second, mask, value);
 
-    if (error) {
-      delete retval;
-      return nullptr;
-    }
+    if (error) return nullptr;
 
     retval->push_back(make_pair(value.first, value.second));
   }
@@ -570,11 +566,11 @@ std::vector<std::pair<std::vector<unsigned char>, std::vector<unsigned char>>>
 }
 
 /* purecov: begin deadcode */
-std::string Gcs_ip_allowlist::to_string() const {
-  std::set<Gcs_ip_allowlist_entry *>::const_iterator wl_it;
+std::string Gcs_ip_whitelist::to_string() const {
+  std::set<Gcs_ip_whitelist_entry *>::const_iterator wl_it;
   std::stringstream ss;
 
-  for (wl_it = m_ip_allowlist.begin(); wl_it != m_ip_allowlist.end(); wl_it++) {
+  for (wl_it = m_ip_whitelist.begin(); wl_it != m_ip_whitelist.end(); wl_it++) {
     ss << (*wl_it)->get_addr() << "/" << (*wl_it)->get_mask() << ",";
   }
 
@@ -584,18 +580,15 @@ std::string Gcs_ip_allowlist::to_string() const {
 }
 /* purecov: end */
 
-bool Gcs_ip_allowlist::is_valid(const std::string &the_list) {
-  // lock the list
-  Atomic_lock_guard guard{m_atomic_guard};
-
+bool Gcs_ip_whitelist::is_valid(const std::string &the_list) const {
   // copy the string
-  std::string allowlist = the_list;
+  std::string whitelist = the_list;
 
   // remove trailing whitespaces
-  allowlist.erase(std::remove(allowlist.begin(), allowlist.end(), ' '),
-                  allowlist.end());
+  whitelist.erase(std::remove(whitelist.begin(), whitelist.end(), ' '),
+                  whitelist.end());
 
-  std::stringstream list_ss(allowlist);
+  std::stringstream list_ss(whitelist);
   std::string list_entry;
 
   // split list by commas
@@ -628,7 +621,7 @@ bool Gcs_ip_allowlist::is_valid(const std::string &the_list) {
         (sa.ss_family == AF_INET &&
          imask > 32))  // check that IPv4 mask is within range
     {
-      MYSQL_GCS_LOG_ERROR("Invalid IP or subnet mask in the allowlist: "
+      MYSQL_GCS_LOG_ERROR("Invalid IP or subnet mask in the whitelist: "
                           << ip << (mask.empty() ? "" : "/")
                           << (mask.empty() ? "" : mask));
       return false;
@@ -638,22 +631,19 @@ bool Gcs_ip_allowlist::is_valid(const std::string &the_list) {
   return true;
 }
 
-bool Gcs_ip_allowlist::configure(const std::string &the_list) {
-  // lock the list
-  Atomic_lock_guard guard{m_atomic_guard};
-
+bool Gcs_ip_whitelist::configure(const std::string &the_list) {
   // copy the list
-  std::string allowlist = the_list;
-  m_original_list.assign(allowlist);
+  std::string whitelist = the_list;
+  m_original_list.assign(whitelist);
 
   // clear the list
   this->clear();
 
   // remove whitespaces
-  allowlist.erase(std::remove(allowlist.begin(), allowlist.end(), ' '),
-                  allowlist.end());
+  whitelist.erase(std::remove(whitelist.begin(), whitelist.end(), ' '),
+                  whitelist.end());
 
-  std::stringstream list_ss(allowlist);
+  std::stringstream list_ss(whitelist);
   std::string list_entry;
 
   // parse commas
@@ -684,28 +674,28 @@ bool Gcs_ip_allowlist::configure(const std::string &the_list) {
     if (!add_address("127.0.0.1", "32")) {
       MYSQL_GCS_LOG_WARN(
           "Automatically adding IPv4 localhost address to the "
-          "allowlist. It is mandatory that it is added.");
+          "whitelist. It is mandatory that it is added.");
     } else {
       MYSQL_GCS_LOG_ERROR(
           "Error adding IPv4 localhost address automatically"
-          " to the allowlist");
+          " to the whitelist");
     }
 
     if (!add_address("::1", "128")) {
       MYSQL_GCS_LOG_WARN(
           "Automatically adding IPv6 localhost address to the "
-          "allowlist. It is mandatory that it is added.");
+          "whitelist. It is mandatory that it is added.");
     } else {
       MYSQL_GCS_LOG_ERROR(
           "Error adding IPv6 localhost address automatically"
-          " to the allowlist");
+          " to the whitelist");
     }
   }
 
   return false;
 }
 
-bool get_address_for_allowlist(
+bool get_address_for_whitelist(
     std::string addr, std::string mask,
     std::pair<std::vector<unsigned char>, std::vector<unsigned char>>
         &out_pair) {
@@ -758,33 +748,33 @@ bool get_address_for_allowlist(
   return false;
 }
 
-void Gcs_ip_allowlist::clear() {
-  std::set<Gcs_ip_allowlist_entry *>::const_iterator wl_it =
-      m_ip_allowlist.begin();
-  while (wl_it != m_ip_allowlist.end()) {
+void Gcs_ip_whitelist::clear() {
+  std::set<Gcs_ip_whitelist_entry *>::const_iterator wl_it =
+      m_ip_whitelist.begin();
+  while (wl_it != m_ip_whitelist.end()) {
     delete (*wl_it);
-    m_ip_allowlist.erase(wl_it++);
+    m_ip_whitelist.erase(wl_it++);
   }
 }
 
-Gcs_ip_allowlist::~Gcs_ip_allowlist() { this->clear(); }
+Gcs_ip_whitelist::~Gcs_ip_whitelist() { this->clear(); }
 
-bool Gcs_ip_allowlist::add_address(std::string addr, std::string mask) {
-  Gcs_ip_allowlist_entry *addr_for_wl;
+bool Gcs_ip_whitelist::add_address(std::string addr, std::string mask) {
+  Gcs_ip_whitelist_entry *addr_for_wl;
   struct sockaddr_storage sa;
   if (!string_to_sockaddr(addr, &sa)) {
-    addr_for_wl = new Gcs_ip_allowlist_entry_ip(addr, mask);
+    addr_for_wl = new Gcs_ip_whitelist_entry_ip(addr, mask);
   } else {
-    addr_for_wl = new Gcs_ip_allowlist_entry_hostname(addr, mask);
+    addr_for_wl = new Gcs_ip_whitelist_entry_hostname(addr, mask);
   }
   bool error = addr_for_wl->init_value();
 
   if (!error) {
-    std::pair<std::set<Gcs_ip_allowlist_entry *,
-                       Gcs_ip_allowlist_entry_pointer_comparator>::iterator,
+    std::pair<std::set<Gcs_ip_whitelist_entry *,
+                       Gcs_ip_whitelist_entry_pointer_comparator>::iterator,
               bool>
         result;
-    result = m_ip_allowlist.insert(addr_for_wl);
+    result = m_ip_whitelist.insert(addr_for_wl);
 
     error = !result.second;
   }
@@ -792,14 +782,14 @@ bool Gcs_ip_allowlist::add_address(std::string addr, std::string mask) {
   return error;
 }
 
-bool Gcs_ip_allowlist::do_check_block_allowlist(
+bool Gcs_ip_whitelist::do_check_block_whitelist(
     std::vector<unsigned char> const &incoming_octets) const {
   /*
-    Check if the incoming IP matches any IP-mask combination in the allowlist.
+    Check if the incoming IP matches any IP-mask combination in the whitelist.
     The check compares both IPs' bytes (octets) in network byte order.
   */
   bool block = true;
-  for (auto &wl_it : m_ip_allowlist) {
+  for (auto &wl_it : m_ip_whitelist) {
     std::unique_ptr<std::vector<
         std::pair<std::vector<unsigned char>, std::vector<unsigned char>>>>
         wl_value((*wl_it).get_value());
@@ -829,7 +819,7 @@ bool Gcs_ip_allowlist::do_check_block_allowlist(
   return block;
 }
 
-bool Gcs_ip_allowlist::do_check_block_xcom(
+bool Gcs_ip_whitelist::do_check_block_xcom(
     std::vector<unsigned char> const &incoming_octets,
     site_def const *xcom_config) const {
   /*
@@ -841,14 +831,14 @@ bool Gcs_ip_allowlist::do_check_block_xcom(
     Gcs_xcom_node_address xcom_addr(
         std::string(xcom_config->nodes.node_list_val[i].address));
     struct sockaddr_storage xcom_sa;
-    std::unique_ptr<Gcs_ip_allowlist_entry> xcom_addr_wl(nullptr);
+    std::unique_ptr<Gcs_ip_whitelist_entry> xcom_addr_wl(nullptr);
     std::unique_ptr<std::vector<
         std::pair<std::vector<unsigned char>, std::vector<unsigned char>>>>
         wl_value(nullptr);
     std::vector<unsigned char> const *xcom_octets = nullptr;
 
     /*
-      Treat the XCom member as if it is in the allowlist.
+      Treat the XCom member as if it is in the whitelist.
       The XCom member can be an IP or hostname.
       The magic-number "32" for the netmask is tied to IPv4.
 
@@ -857,7 +847,7 @@ bool Gcs_ip_allowlist::do_check_block_xcom(
     bool is_hostname = string_to_sockaddr(xcom_addr.get_member_ip(), &xcom_sa);
     if (is_hostname) {
       xcom_addr_wl.reset(
-          new Gcs_ip_allowlist_entry_hostname(xcom_addr.get_member_ip()));
+          new Gcs_ip_whitelist_entry_hostname(xcom_addr.get_member_ip()));
     } else {
       std::string xcom_entry_netmask;
 
@@ -866,7 +856,7 @@ bool Gcs_ip_allowlist::do_check_block_xcom(
       else
         xcom_entry_netmask.append("128");
 
-      xcom_addr_wl.reset(new Gcs_ip_allowlist_entry_ip(
+      xcom_addr_wl.reset(new Gcs_ip_whitelist_entry_ip(
           xcom_addr.get_member_ip(), xcom_entry_netmask));
     }
 
@@ -897,7 +887,7 @@ bool Gcs_ip_allowlist::do_check_block_xcom(
   return block;
 }
 
-bool Gcs_ip_allowlist::do_check_block(struct sockaddr_storage *sa,
+bool Gcs_ip_whitelist::do_check_block(struct sockaddr_storage *sa,
                                       site_def const *xcom_config) const {
   bool block = true;
   unsigned char *buf;
@@ -944,11 +934,11 @@ bool Gcs_ip_allowlist::do_check_block(struct sockaddr_storage *sa,
     goto end;
 
   /*
-    Allow the incoming IP if it is allowlisted *or* is an XCom member.
+    Allow the incoming IP if it is whitelisted *or* is an XCom member.
     XCom members are authorized by default so that XCom can create its
     all-to-all bidirectional network.
   */
-  if (!m_ip_allowlist.empty()) block = do_check_block_allowlist(ip);
+  if (!m_ip_whitelist.empty()) block = do_check_block_whitelist(ip);
   if (block && xcom_config != nullptr)
     block = do_check_block_xcom(ip, xcom_config);
 
@@ -956,10 +946,7 @@ end:
   return block;
 }
 
-bool Gcs_ip_allowlist::shall_block(int fd, site_def const *xcom_config) {
-  // lock the list
-  Atomic_lock_guard guard{m_atomic_guard};
-
+bool Gcs_ip_whitelist::shall_block(int fd, site_def const *xcom_config) const {
   bool ret = true;
   if (fd > 0) {
     struct sockaddr_storage sa;
@@ -978,16 +965,13 @@ bool Gcs_ip_allowlist::shall_block(int fd, site_def const *xcom_config) {
     MYSQL_GCS_LOG_WARN("Connection attempt from IP address "
                        << addr
                        << " refused. Address is not in the "
-                          "IP allowlist.");
+                          "IP whitelist.");
   }
   return ret;
 }
 
-bool Gcs_ip_allowlist::shall_block(const std::string &ip_addr,
-                                   site_def const *xcom_config) {
-  // lock the list
-  Atomic_lock_guard guard{m_atomic_guard};
-
+bool Gcs_ip_whitelist::shall_block(const std::string &ip_addr,
+                                   site_def const *xcom_config) const {
   bool ret = true;
   if (!ip_addr.empty()) {
     struct sockaddr_storage sa;
@@ -1005,7 +989,7 @@ bool Gcs_ip_allowlist::shall_block(const std::string &ip_addr,
     MYSQL_GCS_LOG_WARN("Connection attempt from IP address "
                        << ip_addr
                        << " refused. Address is not in the "
-                          "IP allowlist.");
+                          "IP whitelist.");
   }
   return ret;
 }
