@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,8 +25,6 @@
 
 #include <string>
 #include <unordered_set>  // std::unordered_set
-#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/network/include/network_provider.h"
-
 #include "plugin/group_replication/libmysqlgcs/include/mysql/gcs/gcs_types.h"
 #include "plugin/group_replication/libmysqlgcs/include/mysql/gcs/xplatform/my_xp_cond.h"
 #include "plugin/group_replication/libmysqlgcs/include/mysql/gcs/xplatform/my_xp_mutex.h"
@@ -51,12 +48,12 @@
 */
 class Gcs_xcom_proxy {
  public:
-  explicit Gcs_xcom_proxy() = default;
+  explicit Gcs_xcom_proxy() {}
 
   /**
     The destructor.
   */
-  virtual ~Gcs_xcom_proxy() = default;
+  virtual ~Gcs_xcom_proxy() {}
 
   /**
     This is an utility member function that is used to call into XCom for
@@ -71,8 +68,7 @@ class Gcs_xcom_proxy {
     @c delete_node_address
   */
 
-  virtual node_address *new_node_address_uuid(unsigned int n,
-                                              char const *names[],
+  virtual node_address *new_node_address_uuid(unsigned int n, char *names[],
                                               blob uuids[]) = 0;
 
   /**
@@ -81,6 +77,9 @@ class Gcs_xcom_proxy {
 
     @param n the length of the list
     @param na the list to delete
+
+    @return false on success, true otherwise. In case of an error, memory may
+    not have been completely freed.
   */
 
   virtual void delete_node_address(unsigned int n, node_address *na) = 0;
@@ -99,7 +98,7 @@ class Gcs_xcom_proxy {
     @param group_id the identifier of the group to which the nodes should
            be added
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of adding a node. Since this is basically an
              asynchronous function, one needs to wait for the actual view change
              to validate that the node was added to the configuration.
@@ -124,7 +123,7 @@ class Gcs_xcom_proxy {
     @param group_id The identifier of the group from which the nodes will
            be removed
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of removing a node. Since this is basically an
              asynchronous function, one needs to wait for the actual view change
              to validate that the nodes were removed from the configuration.
@@ -149,7 +148,7 @@ class Gcs_xcom_proxy {
     @param group_id The identifier of the group from which the nodes will
            be removed
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of removing a node. Since this is basically an
              asynchronous function, one needs to wait for the actual view change
              to validate that the nodes were removed from the configuration.
@@ -189,7 +188,7 @@ class Gcs_xcom_proxy {
     @param group_id The identifier of the group from which the nodes will
            be removed
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of setting the event horizon. Since this is
              basically an asynchronous function, one needs to busy-wait on
              @c xcom_client_get_event_horizon to validate that the event horizon
@@ -198,42 +197,6 @@ class Gcs_xcom_proxy {
   virtual bool xcom_client_set_event_horizon(
       uint32_t group_id, xcom_event_horizon event_horizon) = 0;
 
-  /**
-    This member function is responsible for triggering the reconfiguration of
-    the leaders of the XCom configuration. This function is asynchronous, so you
-    need to poll @c xcom_get_leaders to actually validate that the
-    reconfiguration was successful.
-
-    @param group_id The identifier of the group
-    @param nr_preferred_leaders Number of preferred leaders, i.e. elements in
-                                @c preferred_leaders
-    @param preferred_leaders The "host:port" of the preferred leaders
-    @param max_nr_leaders Maximum number of active leaders
-    @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
-             later in the process of setting the leaders. Since this is
-             basically an asynchronous function, one needs to busy-wait on
-             @c xcom_client_get_leaders to validate that the leaders were
-             modified.
-  */
-  virtual bool xcom_client_set_leaders(uint32_t group_id,
-                                       u_int nr_preferred_leaders,
-                                       char const *preferred_leaders[],
-                                       node_no max_nr_leaders) = 0;
-
-  /**
-    This member function is responsible for retrieving the leaders of the
-    XCom configuration.
-
-    @param[in] group_id The identifier of the group
-    @param[out] leaders A reference to where the group's leaders will be written
-                        to
-    @retval true if successful and @c leaders was written to
-    @retval false otherwise
- */
-  virtual bool xcom_client_get_leaders(uint32_t group_id,
-                                       leader_info_data &leaders) = 0;
-
   /**
    This member function is responsible for retrieving the application payloads
    decided in the synodes in @c synodes, from the XCom instance connected to
@@ -268,7 +231,7 @@ class Gcs_xcom_proxy {
 
     @param size The new value for the maximum size of the XCom cache.
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't.
+             process our request, failure means it wont.
   */
   virtual bool xcom_client_set_cache_size(uint64_t size) = 0;
 
@@ -291,7 +254,7 @@ class Gcs_xcom_proxy {
     @param size the size of the payload
     @param data the payload
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't.
+             process our request, failure means it wont.
   */
 
   virtual bool xcom_client_send_data(unsigned long long size, char *data) = 0;
@@ -311,12 +274,16 @@ class Gcs_xcom_proxy {
     called when the XCOM thread was started but the node has not joined
     a group.
 
-    There could be errors later in the process of exiting XCom. Since this is
-    basically an asynchronous function, one needs to wait for the XCom thread to
-    to ensure that XCom terminated.
+    @param xcom_input_open indicates whether the input channel to the XCom
+                           thread has already been opened.
+    @returns true (false) on success (failure). Success means that XCom will
+             process our request, failure means it wont. There could be errors
+             later in the process of exiting XCom. Since this is basically an
+             asynchronous function, one needs to wait for the XCom thread to
+             to ensure that XCom terminated.
   */
 
-  virtual void xcom_exit() = 0;
+  virtual bool xcom_exit(bool xcom_input_open) = 0;
 
   /*
     Return the operation mode as an integer from an operation mode provided
@@ -381,7 +348,7 @@ class Gcs_xcom_proxy {
   /**
     Initialize the SSL.
 
-    @returns true on error. False otherwise.
+    @returns true (false) on success (failure)
   */
 
   virtual bool xcom_init_ssl() = 0;
@@ -400,6 +367,52 @@ class Gcs_xcom_proxy {
 
   virtual bool xcom_use_ssl() = 0;
 
+  /*
+    Set the necessary SSL parameters before initialization.
+
+    server_key_file  - Path of file that contains the server's X509 key in PEM
+                       format.
+    server_cert_file - Path of file that contains the server's X509 certificate
+                       in PEM format.
+    client_key_file  - Path of file that contains the client's X509 key in PEM
+                       format.
+    client_cert_file - Path of file that contains the client's X509 certificate
+                       in PEM format.
+    ca_file          - Path of file that contains list of trusted SSL CAs.
+    ca_path          - Path of directory that contains trusted SSL CA
+                       certificates in PEM format.
+    crl_file         - Path of file that contains certificate revocation lists.
+    crl_path         - Path of directory that contains certificate revocation
+                       list files.
+    cipher           - List of permitted ciphers to use for connection
+                       encryption.
+    tls_version      - Protocols permitted for secure connections.
+    tls_ciphersuites - List of permitted ciphersuites to use for TLS 1.3
+                       connection encryption.
+
+    Note that only the server_key_file/server_cert_file and the client_key_file/
+    client_cert_file are required and the rest of the pointers can be NULL.
+    If the key is provided along with the certificate, either the key file or
+    the other can be ommited.
+
+    The caller can free the parameters after the SSL is started
+    if this is necessary.
+  */
+  struct ssl_parameters {
+    const char *server_key_file;
+    const char *server_cert_file;
+    const char *client_key_file;
+    const char *client_cert_file;
+    const char *ca_file;
+    const char *ca_path;
+    const char *crl_file;
+    const char *crl_path;
+    const char *cipher;
+  };
+  struct tls_parameters {
+    const char *tls_version;
+    const char *tls_ciphersuites;
+  };
   virtual void xcom_set_ssl_parameters(ssl_parameters ssl,
                                        tls_parameters tls) = 0;
 
@@ -411,7 +424,7 @@ class Gcs_xcom_proxy {
     @param nl List with a single member - the one that boots the group
     @param group_id the Group identifier to which the member belongs to
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of booting. Since this is basically an
              asynchronous function, one needs to wait for XCom to signal it is
              ready to validate whether it booted.
@@ -542,7 +555,7 @@ class Gcs_xcom_proxy {
     @param group_id The identifier of the group from which the nodes will
            belong
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of forcing the configuration. Since this is
              basically an asynchronous function, one needs to wait for the
              actual view change to validate that the configuration was forced.
@@ -555,7 +568,7 @@ class Gcs_xcom_proxy {
     @param node Node information.
     @param group_id_hash Hash of group identifier.
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of booting. Since this is basically an
              asynchronous function, one needs to wait for XCom to signal it is
              ready to validate whether it booted.
@@ -569,7 +582,7 @@ class Gcs_xcom_proxy {
     @param nodes Set of nodes.
     @param group_id_hash Hash of group identifier.
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of removing a node. Since this is basically an
              asynchronous function, one needs to wait for the actual view change
              to validate that the nodes were removed from the configuration.
@@ -585,7 +598,7 @@ class Gcs_xcom_proxy {
     @param nodes Set of nodes to remove.
     @param group_id_hash Hash of group identifier.
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of removing a node. Since this is basically an
              asynchronous function, one needs to wait for the actual view change
              to validate that the nodes were removed from the configuration.
@@ -601,7 +614,7 @@ class Gcs_xcom_proxy {
     @param node Node information.
     @param group_id_hash Hash of group identifier.
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of removing a node. Since this is basically an
              asynchronous function, one needs to wait for the actual view change
              to validate that the node was removed from the configuration.
@@ -617,7 +630,7 @@ class Gcs_xcom_proxy {
     @param nodes Set of nodes.
     @param group_id_hash Hash of group identifier.
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of adding a node. Since this is basically an
              asynchronous function, one needs to wait for the actual view change
              to validate that the nodes were added to the configuration.
@@ -633,7 +646,7 @@ class Gcs_xcom_proxy {
     @param node Node information.
     @param group_id_hash Hash of group identifier.
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of adding a node. Since this is basically an
              asynchronous function, one needs to wait for the actual view change
              to validate that the node was added to the configuration.
@@ -690,14 +703,6 @@ class Gcs_xcom_proxy {
   */
   virtual bool xcom_set_event_horizon(uint32_t group_id_hash,
                                       xcom_event_horizon event_horizon) = 0;
-
-  virtual bool xcom_set_leaders(uint32_t group_id_hash,
-                                u_int nr_preferred_leaders,
-                                char const *preferred_leaders[],
-                                node_no max_nr_leaders) = 0;
-  virtual bool xcom_get_leaders(uint32_t group_id_hash,
-                                leader_info_data &leaders) = 0;
-
   /**
     Function to reconfigure the maximum size of the XCom cache.
 
@@ -713,7 +718,7 @@ class Gcs_xcom_proxy {
     @param nodes Set of nodes.
     @param group_id_hash Hash of group identifier.
     @returns true (false) on success (failure). Success means that XCom will
-             process our request, failure means it won't. There could be errors
+             process our request, failure means it wont. There could be errors
              later in the process of forcing the configuration. Since this is
              basically an asynchronous function, one needs to wait for the
              actual view change to validate that the configuration was forced.
@@ -782,7 +787,7 @@ class Gcs_xcom_proxy {
    * Attempts to retrieve incoming commands. (Called by XCom.)
    *
    * @pre The input channel to XCom is open, i.e. @c xcom_input_connect
-   * @retval app_data_ptr linked list of the queued commands if the queue is
+   * @retval app_data_ptr linked list of the queued comamnds if the queue is
    *                      not empty
    * @retval nullptr if the queue is empty
    */
@@ -796,32 +801,6 @@ class Gcs_xcom_proxy {
    * @returns true if we were able to successfully connect, false otherwise.
    */
   virtual bool test_xcom_tcp_connection(std::string &host, xcom_port port) = 0;
-
-  /**
-   * @brief Initializes XCom's Network Manager. This must be called to ensure
-   *        that we have client connection abilities since the start of GCS.
-   *
-   * @return true in case of error, false otherwise.
-   */
-  virtual bool initialize_network_manager() = 0;
-
-  /**
-   * @brief Finalizes XCom's Network Manager. This cleans up everythins
-   *        regarding network.
-   *
-   * @return true in case of error, false otherwise.
-   */
-  virtual bool finalize_network_manager() = 0;
-
-  /**
-   * @brief Set XCom's network manager active provider
-   *
-   * @param new_value the value of the Communication Stack to use.
-   *
-   * @return true in case of error, false otherwise.
-   */
-  virtual bool set_network_manager_active_provider(
-      enum_transport_protocol new_value) = 0;
 };
 
 /*
@@ -832,44 +811,32 @@ class Gcs_xcom_proxy {
 */
 class Gcs_xcom_proxy_base : public Gcs_xcom_proxy {
  public:
-  explicit Gcs_xcom_proxy_base() = default;
-  ~Gcs_xcom_proxy_base() override = default;
+  explicit Gcs_xcom_proxy_base() {}
+  virtual ~Gcs_xcom_proxy_base() {}
 
-  bool xcom_boot_node(Gcs_xcom_node_information &node,
-                      uint32_t group_id_hash) override;
-  bool xcom_remove_nodes(Gcs_xcom_nodes &nodes,
-                         uint32_t group_id_hash) override;
+  bool xcom_boot_node(Gcs_xcom_node_information &node, uint32_t group_id_hash);
+  bool xcom_remove_nodes(Gcs_xcom_nodes &nodes, uint32_t group_id_hash);
   bool xcom_remove_nodes(connection_descriptor &con, Gcs_xcom_nodes &nodes,
-                         uint32_t group_id_hash) override;
+                         uint32_t group_id_hash);
   bool xcom_remove_node(const Gcs_xcom_node_information &node,
-                        uint32_t group_id_hash) override;
+                        uint32_t group_id_hash);
   bool xcom_add_nodes(connection_descriptor &con, Gcs_xcom_nodes &nodes,
-                      uint32_t group_id_hash) override;
+                      uint32_t group_id_hash);
   bool xcom_add_node(connection_descriptor &con,
                      const Gcs_xcom_node_information &node,
-                     uint32_t group_id_hash) override;
-  xcom_event_horizon xcom_get_minimum_event_horizon() override;
-  xcom_event_horizon xcom_get_maximum_event_horizon() override;
+                     uint32_t group_id_hash);
+  xcom_event_horizon xcom_get_minimum_event_horizon();
+  xcom_event_horizon xcom_get_maximum_event_horizon();
   bool xcom_get_event_horizon(uint32_t group_id_hash,
-                              xcom_event_horizon &event_horizon) override;
+                              xcom_event_horizon &event_horizon);
   bool xcom_set_event_horizon(uint32_t group_id_hash,
-                              xcom_event_horizon event_horizon) override;
-  bool xcom_set_leaders(uint32_t group_id_hash, u_int nr_preferred_leaders,
-                        char const *preferred_leaders[],
-                        node_no max_nr_leaders) override;
-  bool xcom_get_leaders(uint32_t group_id_hash,
-                        leader_info_data &leaders) override;
+                              xcom_event_horizon event_horizon);
   bool xcom_get_synode_app_data(
       Gcs_xcom_node_information const &xcom_instance, uint32_t group_id_hash,
       const std::unordered_set<Gcs_xcom_synode> &synode_set,
-      synode_app_data_array &reply) override;
-  bool xcom_set_cache_size(uint64_t size) override;
-  bool xcom_force_nodes(Gcs_xcom_nodes &nodes, uint32_t group_id_hash) override;
-
-  bool initialize_network_manager() override;
-  bool finalize_network_manager() override;
-  bool set_network_manager_active_provider(
-      enum_transport_protocol new_value) override;
+      synode_app_data_array &reply);
+  bool xcom_set_cache_size(uint64_t size);
+  bool xcom_force_nodes(Gcs_xcom_nodes &nodes, uint32_t group_id_hash);
 
  private:
   /* Serialize information on nodes to be sent to XCOM */
@@ -877,7 +844,7 @@ class Gcs_xcom_proxy_base : public Gcs_xcom_proxy {
 
   /* Free information on nodes sent to XCOM */
   void free_nodes_information(node_list &nl);
-  bool test_xcom_tcp_connection(std::string &host, xcom_port port) override;
+  bool test_xcom_tcp_connection(std::string &host, xcom_port port);
 };
 
 /**
@@ -891,76 +858,70 @@ class Gcs_xcom_proxy_impl : public Gcs_xcom_proxy_base {
  public:
   explicit Gcs_xcom_proxy_impl();
   Gcs_xcom_proxy_impl(unsigned int wt);
-  ~Gcs_xcom_proxy_impl() override;
+  virtual ~Gcs_xcom_proxy_impl();
 
-  node_address *new_node_address_uuid(unsigned int n, char const *names[],
-                                      blob uuids[]) override;
-  void delete_node_address(unsigned int n, node_address *na) override;
+  node_address *new_node_address_uuid(unsigned int n, char *names[],
+                                      blob uuids[]);
+  void delete_node_address(unsigned int n, node_address *na);
   bool xcom_client_add_node(connection_descriptor *fd, node_list *nl,
-                            uint32_t group_id) override;
-  bool xcom_client_remove_node(node_list *nl, uint32_t group_id) override;
+                            uint32_t group_id);
+  bool xcom_client_remove_node(node_list *nl, uint32_t group_id);
   bool xcom_client_remove_node(connection_descriptor *fd, node_list *nl,
-                               uint32_t group_id) override;
-  bool xcom_client_get_event_horizon(
-      uint32_t group_id, xcom_event_horizon &event_horizon) override;
+                               uint32_t group_id);
+  bool xcom_client_get_event_horizon(uint32_t group_id,
+                                     xcom_event_horizon &event_horizon);
   bool xcom_client_set_event_horizon(uint32_t group_id,
-                                     xcom_event_horizon event_horizon) override;
-  bool xcom_client_set_leaders(uint32_t gid, u_int nr_preferred_leaders,
-                               char const *preferred_leaders[],
-                               node_no max_nr_leaders) override;
-  bool xcom_client_get_leaders(uint32_t gid,
-                               leader_info_data &leaders) override;
-
+                                     xcom_event_horizon event_horizon);
   bool xcom_client_get_synode_app_data(connection_descriptor *con,
                                        uint32_t group_id_hash,
                                        synode_no_array &synodes,
-                                       synode_app_data_array &reply) override;
+                                       synode_app_data_array &reply);
 
-  bool xcom_client_set_cache_size(uint64_t size) override;
-  bool xcom_client_boot(node_list *nl, uint32_t group_id) override;
+  bool xcom_client_set_cache_size(uint64_t size);
+  bool xcom_client_boot(node_list *nl, uint32_t group_id);
   connection_descriptor *xcom_client_open_connection(std::string,
-                                                     xcom_port port) override;
-  bool xcom_client_close_connection(connection_descriptor *fd) override;
-  bool xcom_client_send_data(unsigned long long size, char *data) override;
-  void xcom_init(xcom_port listen_port) override;
-  void xcom_exit() override;
-  int xcom_get_ssl_mode(const char *mode) override;
-  int xcom_get_ssl_fips_mode(const char *mode) override;
-  int xcom_set_ssl_fips_mode(int mode) override;
-  int xcom_set_ssl_mode(int mode) override;
-  bool xcom_init_ssl() override;
-  void xcom_destroy_ssl() override;
-  bool xcom_use_ssl() override;
-  void xcom_set_ssl_parameters(ssl_parameters ssl, tls_parameters tls) override;
-  site_def const *find_site_def(synode_no synode) override;
+                                                     xcom_port port);
+  bool xcom_client_close_connection(connection_descriptor *fd);
+  bool xcom_client_send_data(unsigned long long size, char *data);
+  void xcom_init(xcom_port listen_port);
+  bool xcom_exit(bool xcom_input_open);
+  int xcom_get_ssl_mode(const char *mode);
+  int xcom_get_ssl_fips_mode(const char *mode);
+  int xcom_set_ssl_fips_mode(int mode);
+  int xcom_set_ssl_mode(int mode);
+  bool xcom_init_ssl();
+  void xcom_destroy_ssl();
+  bool xcom_use_ssl();
+  void xcom_set_ssl_parameters(ssl_parameters ssl, tls_parameters tls);
+  site_def const *find_site_def(synode_no synode);
 
-  enum_gcs_error xcom_wait_ready() override;
-  bool xcom_is_ready() override;
-  void xcom_set_ready(bool value) override;
-  void xcom_signal_ready() override;
+  enum_gcs_error xcom_wait_ready();
+  bool xcom_is_ready();
+  void xcom_set_ready(bool value);
+  void xcom_signal_ready();
 
-  void xcom_wait_for_xcom_comms_status_change(int &status) override;
-  bool xcom_has_comms_status_changed() override;
-  void xcom_set_comms_status(int status) override;
-  void xcom_signal_comms_status_changed(int status) override;
+  void xcom_wait_for_xcom_comms_status_change(int &status);
+  bool xcom_has_comms_status_changed();
+  void xcom_set_comms_status(int status);
+  void xcom_signal_comms_status_changed(int status);
 
-  enum_gcs_error xcom_wait_exit() override;
-  bool xcom_is_exit() override;
-  void xcom_set_exit(bool value) override;
-  void xcom_signal_exit() override;
+  enum_gcs_error xcom_wait_exit();
+  bool xcom_is_exit();
+  void xcom_set_exit(bool value);
+  void xcom_signal_exit();
 
-  void xcom_set_cleanup() override;
+  void xcom_set_cleanup();
 
-  bool xcom_client_force_config(node_list *nl, uint32_t group_id) override;
-  bool get_should_exit() override;
-  void set_should_exit(bool should_exit) override;
+  bool xcom_client_force_config(node_list *nl, uint32_t group_id);
+  bool get_should_exit();
+  void set_should_exit(bool should_exit);
 
-  bool xcom_input_connect(std::string const &address, xcom_port port) override;
-  void xcom_input_disconnect() override;
-  bool xcom_input_try_push(app_data_ptr data) override;
+  bool xcom_input_connect(std::string const &address, xcom_port port);
+  void xcom_input_disconnect();
+  bool xcom_input_try_push(app_data_ptr data);
   Gcs_xcom_input_queue::future_reply xcom_input_try_push_and_get_reply(
-      app_data_ptr data) override;
-  xcom_input_request_ptr xcom_input_try_pop() override;
+      app_data_ptr data);
+  xcom_input_request_ptr xcom_input_try_pop();
 
  private:
   /*
@@ -985,7 +946,6 @@ class Gcs_xcom_proxy_impl : public Gcs_xcom_proxy_base {
   My_xp_socket_util *m_socket_util;
 
   // Stores SSL parameters
-  int m_ssl_mode;
   const char *m_server_key_file;
   const char *m_server_cert_file;
   const char *m_client_key_file;
@@ -1044,9 +1004,9 @@ class Gcs_xcom_proxy_impl : public Gcs_xcom_proxy_base {
  */
 class Gcs_xcom_app_cfg {
  public:
-  explicit Gcs_xcom_app_cfg() = default;
+  explicit Gcs_xcom_app_cfg() {}
 
-  virtual ~Gcs_xcom_app_cfg() = default;
+  virtual ~Gcs_xcom_app_cfg() {}
 
   /**
     Initializes the data structures to communicate with
@@ -1079,13 +1039,6 @@ class Gcs_xcom_app_cfg {
    */
   bool set_identity(node_address *identity);
 
-  /**
-   * @brief Sets the network namespace manager
-   *
-   * @param ns_mgr a reference to a Network_namespace_manager implementation
-   */
-  void set_network_namespace_manager(Network_namespace_manager *ns_mgr);
-
   /**
     Must be called when XCom is not engaged anymore.
    */
