@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +23,11 @@
 #ifndef GET_SYNODE_APP_DATA_H
 #define GET_SYNODE_APP_DATA_H
 
-#include "xdr_gen/xcom_vp.h"
+#include "plugin/group_replication/libmysqlgcs/xdr_gen/xcom_vp.h"  // synode_no_array, synode_app_data_array
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /*
  Error code for the xcom_get_synode_app_data command.
@@ -54,4 +57,8 @@ typedef enum {
 xcom_get_synode_app_data_result xcom_get_synode_app_data(
     synode_no_array const *const synodes, synode_app_data_array *const reply);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* GET_SYNODE_APP_DATA_H */
