@@ -1113,7 +1113,8 @@ void task_loop() {
   for (;;) {
     // check forced exit callback
     if (get_should_exit()) {
-      terminate_and_exit();
+      xcom_fsm(xa_terminate, int_arg(0));
+      xcom_fsm(xa_exit, int_arg(0));
     }
 
     t = first_runnable();
