@@ -1,4 +1,4 @@
-/* Copyright (c) 2012, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,27 +23,23 @@
 #ifndef XCOM_BASE_H
 #define XCOM_BASE_H
 
+#include <stdbool.h>
 #include <stddef.h>
-#ifdef __APPLE__
-#include <stdio.h>
+
+#ifdef __cplusplus
+extern "C" {
 #endif
 
 #ifndef _WIN32
 #include <netdb.h>
 #endif
 
-#include <memory>
-
-#include "xcom/network/include/network_management_interface.h"
-#include "xcom/network/network_provider_manager.h"
-#include "xcom/site_struct.h"
-#include "xcom/task_arg.h"
-#include "xcom/task_debug.h"
-#include "xcom/x_platform.h"
-#include "xcom/xcom_cache.h"
-#include "xcom/xcom_input_request.h"
-#include "xcom/xcom_os_layer.h"
-#include "xcom/xdr_utils.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/task_debug.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/x_platform.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/xcom_cache.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/xcom_input_request.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/xcom_os_layer.h"
+#include "plugin/group_replication/libmysqlgcs/src/bindings/xcom/xcom/xdr_utils.h"
 
 #define XCOM_THREAD_DEBUG 1
 
@@ -51,26 +47,32 @@ typedef unsigned long long synode_cnt;
 
 #define SET_EXECUTED_MSG(x) \
   do {                      \
-    IFDBG(D_NONE, FN);      \
+    MAY_DBG(FN);            \
     set_executed_msg(x);    \
   } while (0)
 
-/* Constants */
+/* {{{ Constants */
 
 #ifdef XCOM_TRANSACTIONS
 static trans_id const last_trans = {0xffffffff, 0xffffffff};
 
 #endif
 
+/* }}} */
+
 extern int ARBITRATOR_HACK;
 extern task_arg null_arg;
 
 void *xcom_thread_main(void *cp);
 
+synode_no vp_count_to_synode(u_long high, u_long low, node_no nodeid,
+                             uint32_t groupid);
+
 synode_no incr_synode(synode_no synode);
 
 synode_no decr_synode(synode_no synode);
 
+void broadcast_recover_end();
 char *dbg_pax_msg(pax_msg const *p);
 pax_msg *dispatch_op(site_def const *site, pax_msg *p, linkage *reply_queue);
 synode_no set_executed_msg(synode_no msgno);
@@ -80,6 +82,7 @@ void check_tasks();
 int xcom_booted();
 int iamthegreatest(site_def const *s);
 void xcom_send(app_data_ptr a, pax_msg *msg);
+void deliver_view_msg(site_def const *site);
 int reply_handler_task(task_arg arg);
 int acceptor_learner_task(task_arg arg);
 synode_no get_max_synode();
@@ -87,17 +90,23 @@ void xcom_thread_deinit();
 int taskmain(xcom_port listen_port);
 void xcom_thread_init();
 site_def *install_node_group(app_data_ptr a);
+int xcom_taskmain(xcom_port listen_port);
 int xcom_taskmain2(xcom_port listen_port);
 void set_max_synode(synode_no synode);
 synode_no set_current_message(synode_no msgno);
 
+void xcom_send_data(uint32_t size, char *data);
+
+bool_t must_force_recover();
+channel *get_prop_input_queue();
 int is_real_recover(app_data_ptr a);
 
 void init_xcom_base();
+void set_force_recover(bool_t const x);
+void add_to_cache(app_data_ptr a, synode_no synode);
 uint32_t new_id();
 synode_no get_boot_key();
 site_def const *get_executor_site();
-site_def *get_executor_site_rw();
 site_def const *get_proposer_site();
 synode_no get_current_message();
 void start_run_tasks();
@@ -105,41 +114,7 @@ void start_run_tasks();
 int is_node_v4_reachable(char *node_address);
 int is_node_v4_reachable_with_info(struct addrinfo *retrieved_addr_info);
 int are_we_allowed_to_upgrade_to_v6(app_data_ptr a);
-
-/**
- * @brief Process incoming are_you_alive (i.e.: ping) messages and act
- * accordingly
- *
- * GCS/XCom has a full mesh of connections between all nodes. A connects to B
- * and B connects back to A.
- *
- * If we cut out B with, for instance, a firewall, we have the A->B connection
- * silently dead, but we have the B->A connection alive. Since we only do
- * monitoring on one half of the connection (the incoming messages), we will
- * consider that B is alive, although we can't contact it. In the same way, B
- * will consider that A is dead, since it does not receive any message from it.
- *
- * We must be able to break the outgoing connection if we detect that something
- * is wrong, in order to make the bi-directional connection state consistent and
- * report that node as unreachable. That can be done if we start receiving
- * pings from a node that we consider that it is alive. After some pings,
- * we just kill the outgoing connection, thus creating a consistent state.
- *
- * Breaking this connection should only occur if the node has already booted,
- * meaning that the whole joining process is complete and the node is up and
- * running. This is due to the fact that we receive pings as part of the
- * process of joining a group.
- *
- * @param site current site definitions
- * @param pm a possible ping message:
- * @param has_client_already_booted check if this node has already booted
- * @param current_time current XCom time
- *
- * @return int 1 if the node connection is closed. 0, otherwise.
- */
-int pre_process_incoming_ping(site_def const *site, pax_msg const *pm,
-                              int has_client_already_booted,
-                              double current_time);
+struct addrinfo *does_node_have_v4_address(struct addrinfo *retrieved);
 
 #define RESET_CLIENT_MSG              \
   if (ep->client_msg) {               \
@@ -148,6 +123,8 @@ int pre_process_incoming_ping(site_def const *site, pax_msg const *pm,
 
 #define APP ep->client_msg->p->a
 
+#define XAPP ep->p->learner.msg->a
+
 #define FIND_MAX (MIN_LENGTH / 10)
 
 /* Set type and object pointer */
@@ -156,18 +133,30 @@ int pre_process_incoming_ping(site_def const *site, pax_msg const *pm,
   q = (quark);                   \
   object = PLP.ptr
 
-#define x_actions                                                              \
-  X(x_fsm_wait)                                                                \
-  , X(x_fsm_poll), X(x_fsm_init), X(x_fsm_u_boot), X(x_fsm_add),               \
-      X(x_fsm_net_boot), X(x_fsm_force_config), X(x_fsm_snapshot),             \
-      X(x_fsm_local_snapshot), X(x_fsm_snapshot_wait), X(x_fsm_need_snapshot), \
-      X(x_fsm_complete), X(x_fsm_terminate), X(x_fsm_exit), X(x_fsm_timeout)
-#define X(a) a
+#define x_state_list \
+  X(x_start)         \
+  X(x_boot) X(x_recover) X(x_run) X(x_done) X(x_snapshot_wait) X(x_recover_wait)
+#define x_actions     \
+  X(xa_wait)          \
+  X(xa_poll)          \
+  X(xa_init)          \
+  X(xa_u_boot)        \
+  X(xa_add)           \
+  X(xa_net_boot)      \
+  X(xa_force_config)  \
+  X(xa_snapshot)      \
+  X(xa_snapshot_wait) \
+  X(xa_need_snapshot) X(xa_complete) X(xa_terminate) X(xa_exit) X(xa_timeout)
+#define X(a) a,
+enum xcom_state { x_state_list };
+typedef enum xcom_state xcom_state;
 
 enum xcom_actions { x_actions };
 typedef enum xcom_actions xcom_actions;
 #undef X
 
+extern const char *xcom_state_name[];
+
 extern const char *xcom_actions_name[];
 
 struct add_args {
@@ -179,29 +168,16 @@ typedef struct add_args add_args;
 
 synode_no xcom_get_last_removed_from_cache();
 
-char const *xcom_fsm(xcom_actions action, task_arg fsmargs);
+void xcom_add_node(char *addr, xcom_port port, node_list *nl);
+
+xcom_state xcom_fsm(xcom_actions action, task_arg fsmargs);
 void site_post_install_action(site_def *site);
 
 void site_install_action(site_def *site, cargo_type operation);
 void send_client_add_node(char *srv, xcom_port port, node_list *nl);
 void send_client_remove_node(char *srv, xcom_port port, node_list *nl);
 
-typedef void (*xcom_full_data_receiver)(site_def const *site, pax_machine *pma,
-                                        app_data_ptr app,
-                                        delivery_status app_status);
-void set_xcom_full_data_receiver(xcom_full_data_receiver x);
-
-typedef void (*xcom_full_local_view_receiver)(site_def const *site,
-                                              node_set nodes);
-void set_xcom_full_local_view_receiver(xcom_full_local_view_receiver x);
-
-typedef void (*xcom_full_global_view_receiver)(site_def const *site,
-                                               synode_no message_id,
-                                               node_set nodes);
-void set_xcom_full_global_view_receiver(xcom_full_global_view_receiver x);
-
-typedef void (*xcom_data_receiver)(synode_no message_id, synode_no origin,
-                                   site_def const *site, node_set nodes,
+typedef void (*xcom_data_receiver)(synode_no message_id, node_set nodes,
                                    u_int size, synode_no last_removed,
                                    char *data);
 void set_xcom_data_receiver(xcom_data_receiver x);
@@ -214,16 +190,11 @@ typedef void (*xcom_global_view_receiver)(synode_no config_id,
                                           xcom_event_horizon);
 void set_xcom_global_view_receiver(xcom_global_view_receiver x);
 
-typedef void (*xcom_config_receiver)(app_data *a);
-
-void set_xcom_config_receiver(xcom_config_receiver x);
-
 void set_xcom_logger(xcom_logger x);
 void set_xcom_debugger(xcom_debugger x);
 void set_xcom_debugger_check(xcom_debugger_check x);
 
-typedef void (*app_snap_handler)(blob *gcs_snap, synode_no log_start,
-                                 synode_no log_end);
+typedef void (*app_snap_handler)(blob *gcs_snap);
 void set_app_snap_handler(app_snap_handler x);
 
 typedef synode_no (*app_snap_getter)(blob *gcs_snap);
@@ -238,35 +209,22 @@ void set_xcom_expel_cb(xcom_state_change_cb x);
 typedef int (*should_exit_getter)();
 void set_should_exit_getter(should_exit_getter x);
 
-typedef void (*xcom_recovery_cb)();
-
-void set_xcom_recovery_init_cb(xcom_recovery_cb x);
-
-void set_xcom_recovery_restart_cb(xcom_recovery_cb x);
-
-void set_xcom_recovery_begin_cb(xcom_recovery_cb x);
-
-void set_xcom_recovery_end_cb(xcom_recovery_cb x);
-
 app_data_ptr init_config_with_group(app_data *a, node_list *nl, cargo_type type,
                                     uint32_t group_id);
 app_data_ptr init_set_event_horizon_msg(app_data *a, uint32_t group_id,
                                         xcom_event_horizon event_horizon);
-app_data_ptr init_get_leaders_msg(app_data *a, uint32_t group_id);
 app_data_ptr init_set_cache_size_msg(app_data *a, uint64_t cache_limit);
 app_data_ptr init_get_event_horizon_msg(app_data *a, uint32_t group_id);
 app_data_ptr init_app_msg(app_data *a, char *payload, u_int payload_size);
 app_data_ptr init_terminate_command(app_data *a);
 
-void terminate_and_exit();
-
 /* Hook the logic to pop from the input channel. */
 typedef xcom_input_request_ptr (*xcom_input_try_pop_cb)(void);
 void set_xcom_input_try_pop_cb(xcom_input_try_pop_cb pop);
 /* Create a connection to the input channel's signalling socket. */
-bool_t xcom_input_new_signal_connection(char const *address, xcom_port port);
+bool xcom_input_new_signal_connection(char const *address, xcom_port port);
 /* Signal that the input channel has commands. */
-bool_t xcom_input_signal(void);
+bool xcom_input_signal(void);
 /* Destroy the connection to the input channel's signalling socket. */
 void xcom_input_free_signal_connection(void);
 
@@ -277,6 +235,10 @@ void xcom_input_free_signal_connection(void);
 typedef int (*xcom_socket_accept_cb)(int fd, site_def const *config);
 int set_xcom_socket_accept_cb(xcom_socket_accept_cb x);
 
+connection_descriptor *xcom_open_client_connection(const char *server,
+                                                   xcom_port port);
+int xcom_close_client_connection(connection_descriptor *connection);
+
 int xcom_client_disable_arbitrator(connection_descriptor *fd);
 int xcom_client_enable_arbitrator(connection_descriptor *fd);
 int xcom_client_add_node(connection_descriptor *fd, node_list *nl,
@@ -311,11 +273,8 @@ int xcom_client_convert_into_local_server(connection_descriptor *const fd);
 int64_t xcom_send_client_app_data(connection_descriptor *fd, app_data_ptr a,
                                   int force);
 
-std::unique_ptr<Network_provider_management_interface>
-get_network_management_interface();
-
-std::unique_ptr<Network_provider_operations_interface>
-get_network_operations_interface();
+struct pax_machine;
+typedef struct pax_machine pax_machine;
 
 /**
   Copies app data @c source into @c target and checks if the copy
@@ -416,8 +375,9 @@ void init_propose_msg(pax_msg *p);
  */
 pax_msg *handle_simple_accept(pax_machine *p, pax_msg *m, synode_no synode);
 /**
- * Process the incoming acknowledge from an Acceptor to a sent Accept, as in
- * the message for Phase 2 (b) of the Paxos protocol. Executed by Proposers.
+ * Process the incoming acknowledge from an Acceptor to a sent Accept, as in the
+ * message for Phase 2 (b) of the Paxos protocol.
+ * Executed by Proposers.
  *
  * @param site XCom configuration
  * @param p Paxos instance
@@ -453,7 +413,7 @@ void handle_learn(site_def const *site, pax_machine *p, pax_msg *m);
 int pm_finished(pax_machine *p);
 /** @return true if we should process the incoming need_boot_op message passed
  * in parameter p. */
-bool_t should_handle_need_boot(site_def const *site, pax_msg *p);
+bool should_handle_boot(site_def const *site, pax_msg *p);
 /**
  * Initializes the message @c p as a need_boot_op message.
  *
@@ -462,38 +422,6 @@ bool_t should_handle_need_boot(site_def const *site, pax_msg *p);
  */
 void init_need_boot_op(pax_msg *p, node_address *identity);
 
-int xcom_client_set_max_leaders(connection_descriptor *fd, node_no max_leaders,
-                                uint32_t group_id);
-
-void init_set_max_leaders(uint32_t group_id, app_data *a, node_no max_leaders);
-void init_set_leaders(uint32_t group_id, app_data *a,
-                      leader_array const leaders);
-void init_set_leaders(uint32_t group_id, app_data *a, u_int n,
-                      char const *names[]);
-void init_set_leaders(uint32_t group_id, app_data *leader_app,
-                      leader_array const leaders, app_data *max_app,
-                      node_no max_leaders);
-void init_set_leaders(uint32_t group_id, app_data *leader_app, u_int n,
-                      char const *names[], app_data *max_app,
-                      node_no max_leaders);
-
-int xcom_client_set_leaders(connection_descriptor *fd,
-                            leader_array const leaders, uint32_t group_id);
-int xcom_client_set_leaders(connection_descriptor *fd, u_int n,
-                            char const *names[], uint32_t group_id);
-int xcom_client_set_leaders(connection_descriptor *fd,
-                            leader_array const leaders, node_no max_leaders,
-                            uint32_t group_id);
-int xcom_client_set_leaders(connection_descriptor *fd, u_int n,
-                            char const *names[], node_no max_leaders,
-                            uint32_t group_id);
-
-int xcom_client_get_leaders(connection_descriptor *fd, uint32_t group_id,
-                            leader_info_data *leaders);
-
-typedef void (*xcom_election_cb)(leader_array leaders);
-void set_xcom_election_cb(xcom_election_cb x);
-
 static inline char *strerr_msg(char *buf, size_t len, int nr) {
 #if defined(_WIN32)
   strerror_s(buf, len, nr);
@@ -503,30 +431,25 @@ static inline char *strerr_msg(char *buf, size_t len, int nr) {
   return buf;
 }
 
-void xcom_sleep(unsigned int seconds);
-
 #define XCOM_COMMS_ERROR 1
 #define XCOM_COMMS_OTHER 2
 #define XCOM_COMMS_OK 0
 void set_xcom_comms_cb(xcom_state_change_cb x);
 
-extern "C" synode_no get_delivered_msg();
+synode_no get_delivered_msg();
 void set_max_synode_from_unified_boot(synode_no unified_boot_synode);
-void send_x_fsm_complete();
-synode_no get_default_start(app_data_ptr a);
-synode_no get_last_delivered_msg();
-void set_log_end(gcs_snapshot *gcs);
 
-extern "C" void synthesize_leaders(leader_array *leaders);
-
-#define XCOM_FSM(action, arg)                           \
-  do {                                                  \
-    const char *s = xcom_fsm(action, arg);              \
-    G_TRACE("%f %s:%d", seconds(), __FILE__, __LINE__); \
-    G_DEBUG("new state %s", s);                         \
+#define XCOM_FSM(action, arg)                               \
+  do {                                                      \
+    const char *s = xcom_state_name[xcom_fsm(action, arg)]; \
+    G_TRACE("%f %s:%d", seconds(), __FILE__, __LINE__);     \
+    G_DEBUG("new state %s", s);                             \
   } while (0)
 
 int pm_finished(pax_machine *p);
-bool_t handle_max_leaders(app_data_ptr a);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif
