@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,71 +31,19 @@
 
 Group_action_information::Group_action_information(
     bool is_local_arg, Group_action *action,
-    Group_action_diagnostics *message_area,
-    Group_action_message::enum_action_initiator_and_action initiator)
+    Group_action_diagnostics *message_area)
     : is_local(is_local_arg),
       executing_action(action),
       execution_message_area(message_area),
-      action_result(Group_action::GROUP_ACTION_RESULT_END),
-      m_action_initiator(initiator) {
-  assert(m_action_initiator > Group_action_message::ACTION_INITIATOR_UNKNOWN &&
-         m_action_initiator < Group_action_message::ACTION_INITIATOR_END);
-}
+      action_result(Group_action::GROUP_ACTION_RESULT_END) {}
 
-Group_action_information::Group_action_information(
-    Group_action_message::enum_action_initiator_and_action initiator)
+Group_action_information::Group_action_information()
     : is_local(false),
       executing_action(nullptr),
       execution_message_area(new Group_action_diagnostics()),
-      action_result(Group_action::GROUP_ACTION_RESULT_END),
-      m_action_initiator(initiator) {
-  assert(m_action_initiator > Group_action_message::ACTION_INITIATOR_UNKNOWN &&
-         m_action_initiator < Group_action_message::ACTION_INITIATOR_END);
-}
+      action_result(Group_action::GROUP_ACTION_RESULT_END) {}
 
-Group_action_information::~Group_action_information() = default;
-
-const std::pair<std::string, std::string>
-Group_action_information::get_action_name_and_description() {
-  switch (m_action_initiator) {
-      // This type should not be used
-    case Group_action_message::ACTION_INITIATOR_UNKNOWN:
-      assert(false);
-      return std::make_pair("unknown", "unknown");
-      break;
-    case Group_action_message::ACTION_UDF_SWITCH_TO_MULTI_PRIMARY_MODE:
-      return std::make_pair(
-          "SELECT group_replication_switch_to_multi_primary_mode()",
-          "Multi primary mode migration");
-      break;
-    case Group_action_message::ACTION_UDF_SET_PRIMARY:
-      return std::make_pair("SELECT group_replication_set_as_primary()",
-                            "Primary election change");
-      break;
-    case Group_action_message::ACTION_UDF_SWITCH_TO_SINGLE_PRIMARY_MODE:
-      return std::make_pair(
-          "SELECT group_replication_switch_to_single_primary_mode()",
-          "Change to single primary mode");
-      break;
-    case Group_action_message::ACTION_UDF_SWITCH_TO_SINGLE_PRIMARY_MODE_UUID:
-      return std::make_pair(
-          "SELECT group_replication_switch_to_single_primary_mode()",
-          "Change to single primary mode");
-      break;
-    case Group_action_message::ACTION_UDF_COMMUNICATION_PROTOCOL_MESSAGE:
-      return std::make_pair(
-          "SELECT group_replication_set_communication_protocol()",
-          "Set group communication protocol");
-      break;
-    default:
-      /* This is unreachable code since actions can only be triggered from
-         member with the lower version, as such lower member versions do not
-         need to know newer actions. */
-      assert(false);
-      break;
-  }
-  return std::make_pair("unidentified", "unidentified)");
-}
+Group_action_information::~Group_action_information() {}
 
 /**
  The 'action' / 'action information' object life cycle:
@@ -131,7 +78,7 @@ Group_action_information::get_action_name_and_description() {
 
  On critical action errors it can be set to false on awake_coordinator_on_error.
  This usually means the member will leave the group or the coordinator is
- stopping, meaning new actions won't be accepted or processed since the member
+ stopping, meaning new actions wont be accepted or processed since the member
  left.
 
  There is a wait based on this variable in the execute_group_action_handler but
@@ -178,7 +125,7 @@ Group_action_coordinator::Group_action_coordinator(
                    &group_thread_end_lock, MY_MUTEX_INIT_FAST);
   mysql_cond_init(key_GR_COND_group_action_coordinator_thread_end,
                   &group_thread_end_cond);
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   failure_debug_flag = false;
 #endif
 }
@@ -207,18 +154,11 @@ void Group_action_coordinator::set_stop_wait_timeout(ulong timeout) {
 static void *launch_handler_thread(void *arg) {
   Group_action_coordinator *handler = (Group_action_coordinator *)arg;
   handler->execute_group_action_handler();
-  return nullptr;
+  return 0;
 }
 
-bool Group_action_coordinator::is_group_action_running(
-    std::pair<std::string, std::string> &initiator) {
-  bool running = false;
-  mysql_mutex_lock(&coordinator_process_lock);
-  if ((running = action_running.load())) {
-    initiator = current_executing_action->get_action_name_and_description();
-  }
-  mysql_mutex_unlock(&coordinator_process_lock);
-  return running;
+bool Group_action_coordinator::is_group_action_running() {
+  return action_running.load();
 }
 
 int send_message(Group_action_message *message) {
@@ -242,7 +182,7 @@ void Group_action_coordinator::reset_coordinator_process() {
   member_leaving_group = false;
   remote_warnings_reported = false;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   DBUG_EXECUTE_IF("group_replication_group_action_start_msg_error",
                   { failure_debug_flag = true; });
 #endif
@@ -277,13 +217,12 @@ int Group_action_coordinator::stop_coordinator_process(bool coordinator_stop,
 
   if (wait) {
     mysql_mutex_lock(&group_thread_run_lock);
+    // Signal in case the thread is waiting for other members to finish
+    mysql_cond_broadcast(&group_thread_end_cond);
     while (action_handler_thd_state.is_thread_alive()) {
       DBUG_PRINT("sleep",
                  ("Waiting for the group action execution thread to end"));
-      struct timespec abstime;
-      set_timespec(&abstime, 1);
-      mysql_cond_timedwait(&group_thread_run_cond, &group_thread_run_lock,
-                           &abstime);
+      mysql_cond_wait(&group_thread_run_cond, &group_thread_run_lock);
     }
     mysql_mutex_unlock(&group_thread_run_lock);
   }
@@ -292,22 +231,13 @@ int Group_action_coordinator::stop_coordinator_process(bool coordinator_stop,
 }
 
 int Group_action_coordinator::coordinate_action_execution(
-    Group_action *action, Group_action_diagnostics *execution_info,
-    Group_action_message::enum_action_initiator_and_action initiator) {
+    Group_action *action, Group_action_diagnostics *execution_info) {
   mysql_mutex_lock(&coordinator_process_lock);
 
   int error = 0;
   Group_action_message *start_message = nullptr;
   Group_action_information *action_info = nullptr;
 
-#ifndef NDEBUG
-  DBUG_EXECUTE_IF("group_replication_coordinate_action_execution_start", {
-    const char act[] =
-        "now signal signal.start_waiting wait_for signal.start_continue";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
-  });
-#endif
-
   if (action_proposed) {
     execution_info->set_execution_message(
         Group_action_diagnostics::GROUP_ACTION_LOG_ERROR,
@@ -342,14 +272,6 @@ int Group_action_coordinator::coordinate_action_execution(
     goto end;
   }
 
-  if (thread_killed()) {
-    execution_info->set_execution_message(
-        Group_action_diagnostics::GROUP_ACTION_LOG_ERROR,
-        "Thread was killed, action will be terminated.");
-    error = 1;
-    goto end;
-  }
-
   action_proposed = true;
 
   /* Concurrency notes on this flag:
@@ -361,16 +283,13 @@ int Group_action_coordinator::coordinate_action_execution(
 
   local_action_killed = false;
 
-  action_info =
-      new Group_action_information(true, action, execution_info, initiator);
+  action_info = new Group_action_information(true, action, execution_info);
   proposed_action = action_info;
 
   action->get_action_message(&start_message);
   start_message->set_group_action_message_phase(
       Group_action_message::ACTION_START_PHASE);
 
-  start_message->set_action_initiator(initiator);
-
   if (send_message(start_message)) {
     /* purecov: begin inspected */
     execution_info->set_execution_message(
@@ -384,21 +303,10 @@ int Group_action_coordinator::coordinate_action_execution(
     /* purecov: end */
   }
 
-#ifndef NDEBUG
-  DBUG_EXECUTE_IF(
-      "group_replication_coordinate_action_execution_after_sent_to_group", {
-        const char act[] = "now wait_for signal.action_continue";
-        assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
-      });
-#endif
-
   delete start_message;
 
-  // After this thread will start executing it won't rollback or can be killed,
-  // similar behavior to a transaction which also can't be rollback after
-  // commit.
   while (!local_action_terminating && !action_execution_error &&
-         !action_cancelled_on_termination) {
+         !action_cancelled_on_termination && !thread_killed()) {
     struct timespec abstime;
     set_timespec(&abstime, 1);
 
@@ -406,6 +314,26 @@ int Group_action_coordinator::coordinate_action_execution(
                          &coordinator_process_lock, &abstime);
   }
 
+  if (thread_killed()) {
+    local_action_killed = true;
+    // If it is not the local one running the method wont do nothing
+    if (action_running) {
+      action->stop_action_execution(true);
+    }
+    while (!local_action_terminating && !action_execution_error) {
+      mysql_cond_wait(&coordinator_process_condition,
+                      &coordinator_process_lock);
+    }
+
+    if (Group_action::GROUP_ACTION_RESULT_KILLED !=
+            action_info->action_result &&
+        Group_action::GROUP_ACTION_RESULT_ERROR != action_info->action_result &&
+        !action_execution_error) {
+      execution_info->append_execution_message(
+          " Despite being killed, the operation was still completed.");
+    }
+  }
+
   if (action_execution_error &&
       Group_action::GROUP_ACTION_RESULT_ABORTED == action_info->action_result &&
       member_leaving_group) {
@@ -453,7 +381,7 @@ end:
 }
 
 bool Group_action_coordinator::thread_killed() {
-  return current_thd != nullptr && current_thd->is_killed();
+  return current_thd != NULL && current_thd->is_killed();
 }
 
 bool Group_action_coordinator::handle_action_message(
@@ -461,10 +389,10 @@ bool Group_action_coordinator::handle_action_message(
   // If we are not online just ignore it
   Group_member_info::Group_member_status member_status =
       local_member_info->get_recovery_status();
-  if (member_status != Group_member_info::MEMBER_ONLINE) return false;
+  if (member_status != Group_member_info::MEMBER_ONLINE) return 0;
 
   if (coordinator_terminating) {
-    return false; /* purecov: inspected */
+    return 0; /* purecov: inspected */
   }
 
   Group_action_message::enum_action_message_phase message_phase =
@@ -483,19 +411,19 @@ bool Group_action_coordinator::handle_action_message(
       break; /* purecov: inspected */
   }
 
-  return false;
+  return 0;
 }
 
 bool Group_action_coordinator::handle_action_start_message(
     Group_action_message *message, const std::string &message_origin) {
-  Group_member_info_list *all_members_info = nullptr;
+  std::vector<Group_member_info *> *all_members_info = nullptr;
   int error = 0;
   bool is_message_sender = !message_origin.compare(
       local_member_info->get_gcs_member_id().get_member_id());
 
   Group_action_information *action_info = nullptr;
   if (!is_message_sender) {
-    action_info = new Group_action_information(message->get_action_initiator());
+    action_info = new Group_action_information();
   } else {
     action_info = proposed_action;
   }
@@ -559,7 +487,7 @@ bool Group_action_coordinator::handle_action_start_message(
   if (!is_sender) {
     Group_action_message::enum_action_message_type message_type =
         message->get_group_action_message_type();
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     if (failure_debug_flag) {
       message_type = Group_action_message::ACTION_UNKNOWN_MESSAGE;
     }
@@ -673,7 +601,7 @@ bool Group_action_coordinator::handle_action_stop_message(
 }
 
 bool Group_action_coordinator::member_in_recovery(
-    Group_member_info_list *all_members_info) {
+    std::vector<Group_member_info *> *all_members_info) {
   bool member_in_recovery = false;
   for (Group_member_info *member : *all_members_info) {
     if (member->get_recovery_status() ==
@@ -686,7 +614,7 @@ bool Group_action_coordinator::member_in_recovery(
 }
 
 bool Group_action_coordinator::member_from_invalid_version(
-    Group_member_info_list *all_members_info) {
+    std::vector<Group_member_info *> *all_members_info) {
   bool member_from_invalid_version = false;
   for (Group_member_info *member : *all_members_info) {
     if (member->get_member_version().get_version() <
@@ -703,13 +631,12 @@ void Group_action_coordinator::terminate_action() {
 
   signal_and_wait_action_termination(true);
 
-  assert(current_executing_action);
+  DBUG_ASSERT(current_executing_action);
 
   // Log what was the result of the action
   LogPluginErr(
-      SYSTEM_LEVEL, ER_GRP_RPL_CONFIGURATION_ACTION_LOCAL_TERMINATION,
-      current_executing_action->get_action_name_and_description()
-          .second.c_str(),
+      INFORMATION_LEVEL, ER_GRP_RPL_CONFIGURATION_ACTION_LOCAL_TERMINATION,
+      current_executing_action->executing_action->get_action_name(),
       current_executing_action->execution_message_area->get_execution_message()
           .c_str());
 
@@ -743,10 +670,7 @@ void Group_action_coordinator::signal_and_wait_action_termination(bool wait) {
     while (action_handler_thd_state.is_thread_alive()) {
       DBUG_PRINT("sleep",
                  ("Waiting for the group action execution thread to end"));
-      struct timespec abstime;
-      set_timespec(&abstime, 1);
-      mysql_cond_timedwait(&group_thread_run_cond, &group_thread_run_lock,
-                           &abstime);
+      mysql_cond_wait(&group_thread_run_cond, &group_thread_run_lock);
     }
   }
   mysql_mutex_unlock(&group_thread_run_lock);
@@ -809,15 +733,13 @@ int Group_action_coordinator::signal_action_terminated() {
     const char act[] =
         "now signal signal.action_stopping wait_for "
         "signal.action_stop_continue";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
 
-  Group_action_message *end_message = nullptr;
+  Group_action_message *end_message = NULL;
   current_executing_action->executing_action->get_action_message(&end_message);
   end_message->set_group_action_message_phase(
       Group_action_message::ACTION_END_PHASE);
-  end_message->set_action_initiator(
-      current_executing_action->m_action_initiator);
   if (current_executing_action->execution_message_area->has_warning()) {
     end_message->set_return_value(END_ACTION_MESSAGE_WARNING_FLAG);
   }
@@ -856,10 +778,7 @@ int Group_action_coordinator::launch_group_action_handler_thread() {
   while (action_handler_thd_state.is_alive_not_running()) {
     DBUG_PRINT("sleep",
                ("Waiting for the group action execution thread to start"));
-    struct timespec abstime;
-    set_timespec(&abstime, 1);
-    mysql_cond_timedwait(&group_thread_run_cond, &group_thread_run_lock,
-                         &abstime);
+    mysql_cond_wait(&group_thread_run_cond, &group_thread_run_lock);
   }
   mysql_mutex_unlock(&group_thread_run_lock);
 
@@ -870,14 +789,13 @@ int Group_action_coordinator::execute_group_action_handler() {
   DBUG_TRACE;
   int error = 0;
 
-  THD *thd = nullptr;
+  THD *thd = NULL;
   thd = new THD;
   my_thread_init();
   thd->set_new_thread_id();
   thd->thread_stack = (char *)&thd;
   thd->store_globals();
   global_thd_manager_add_thd(thd);
-  Notification_context notification_ctx;
 
   mysql_mutex_lock(&group_thread_run_lock);
   action_handler_thd_state.set_running();
@@ -886,7 +804,7 @@ int Group_action_coordinator::execute_group_action_handler() {
 
   DBUG_EXECUTE_IF("group_replication_block_group_action", {
     const char act[] = "now wait_for signal.action_continue";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
 
   current_executing_action->action_result =
@@ -895,25 +813,17 @@ int Group_action_coordinator::execute_group_action_handler() {
   monitoring_stage_handler.initialize_stage_monitor();
   is_group_action_being_executed = true;
 
-  LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_CONFIGURATION_ACTION_START,
-               current_executing_action->get_action_name_and_description()
-                   .second.c_str());
+  LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_CONFIGURATION_ACTION_START,
+               current_executing_action->executing_action->get_action_name());
   while (Group_action::GROUP_ACTION_RESULT_RESTART ==
          current_executing_action->action_result) {
     current_executing_action->action_result =
         current_executing_action->executing_action->execute_action(
-            is_sender, &monitoring_stage_handler, &notification_ctx);
+            is_sender, &monitoring_stage_handler);
   }
-  Gcs_view *view = gcs_module->get_current_view();
-  if (view != nullptr) {
-    notification_ctx.set_view_id(view->get_view_id().get_representation());
-    delete view;
-  }
-  notify_and_reset_ctx(notification_ctx);
   is_group_action_being_executed = false;
   LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_CONFIGURATION_ACTION_END,
-               current_executing_action->get_action_name_and_description()
-                   .second.c_str());
+               current_executing_action->executing_action->get_action_name());
 
   current_executing_action->execution_message_area->set_execution_info(
       current_executing_action->executing_action->get_execution_info());
@@ -939,7 +849,8 @@ int Group_action_coordinator::execute_group_action_handler() {
       leave_actions.set(leave_group_on_failure::STOP_APPLIER, true);
       leave_actions.set(leave_group_on_failure::HANDLE_EXIT_STATE_ACTION, true);
       leave_group_on_failure::leave(
-          leave_actions, ER_GRP_RPL_CONFIGURATION_ACTION_KILLED_ERROR, nullptr,
+          leave_actions, ER_GRP_RPL_CONFIGURATION_ACTION_KILLED_ERROR,
+          PSESSION_INIT_THREAD, nullptr,
           exit_state_action_abort_log_message.c_str());
 
       awake_coordinator_on_error(current_executing_action, is_sender, true);
@@ -955,7 +866,7 @@ int Group_action_coordinator::execute_group_action_handler() {
     default:
       awake_coordinator_on_error(current_executing_action, is_sender,
                                  true); /* purecov: inspected */
-      assert(0);                        /* purecov: inspected */
+      DBUG_ASSERT(0);                   /* purecov: inspected */
       break;                            /* purecov: inspected */
   }
 
@@ -963,10 +874,7 @@ int Group_action_coordinator::execute_group_action_handler() {
   while (action_running && !coordinator_terminating) {
     DBUG_PRINT("sleep",
                ("Waiting for the group action execution process to terminate"));
-    struct timespec abstime;
-    set_timespec(&abstime, 1);
-    mysql_cond_timedwait(&group_thread_end_cond, &group_thread_end_lock,
-                         &abstime);
+    mysql_cond_wait(&group_thread_end_cond, &group_thread_end_lock);
   }
   mysql_mutex_unlock(&group_thread_end_lock);
 
@@ -1000,7 +908,6 @@ int Group_action_coordinator::execute_group_action_handler() {
   thd->release_resources();
   global_thd_manager_remove_thd(thd);
   delete thd;
-  my_thread_end();
 
   mysql_mutex_lock(&group_thread_run_lock);
   action_handler_thd_state.set_terminated();
@@ -1010,6 +917,8 @@ int Group_action_coordinator::execute_group_action_handler() {
   Gcs_interface_factory::cleanup_thread_communication_resources(
       Gcs_operations::get_gcs_engine());
 
+  my_thread_end();
+
   return error;
 }
 
@@ -1026,7 +935,7 @@ int Group_action_coordinator::after_view_change(
     return 0;
   }
 
-  if (!action_running.load()) return 0;
+  if (!is_group_action_running()) return 0;
 
   for (Gcs_member_identifier leaving_member : leaving) {
     bool found =
@@ -1050,9 +959,9 @@ int Group_action_coordinator::after_view_change(
   return 0;
 }
 
-int Group_action_coordinator::after_primary_election(
-    std::string, enum_primary_election_primary_change_status,
-    enum_primary_election_mode, int) {
+int Group_action_coordinator::after_primary_election(std::string, bool,
+                                                     enum_primary_election_mode,
+                                                     int) {
   return 0;
 }
 
