@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +28,7 @@
 #include "my_dbug.h"
 #include "plugin/group_replication/include/plugin.h"
 
-Applier_handler::Applier_handler() = default;
+Applier_handler::Applier_handler() {}
 
 int Applier_handler::initialize() {
   DBUG_TRACE;
@@ -50,7 +49,7 @@ int Applier_handler::initialize_repositories(bool reset_logs,
   if (reset_logs) {
     LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_PURGE_APPLIER_LOGS);
 
-    if ((error = channel_interface.purge_logs(false))) {
+    if ((error = channel_interface.purge_logs(true))) {
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_RESET_APPLIER_MODULE_LOGS_ERROR);
       return error;
@@ -61,17 +60,9 @@ int Applier_handler::initialize_repositories(bool reset_logs,
   channel_interface.set_stop_wait_timeout(plugin_shutdown_timeout);
 
   error = channel_interface.initialize_channel(
-      /*host*/ const_cast<char *>("<NULL>"), /*port*/ 0, /*user*/ nullptr,
-      /*pass*/ nullptr, /*use_ssl*/ false, /*ssl_ca*/ nullptr,
-      /*ssl_capath*/ nullptr, /*ssl_cert*/ nullptr, /*ssl_cipher*/ nullptr,
-      /*ssl_key*/ nullptr, /*ssl_crl*/ nullptr, /*ssl_crlpath*/ nullptr,
-      /*ssl_verify*/ false,
-      /*priority*/ GROUP_REPLICATION_APPLIER_THREAD_PRIORITY,
-      /*retry_count*/ 0,
-      /*preserve_logs*/ true, /*public_key_path*/ nullptr,
-      /*get_public_key*/ false, /*compression_alg*/ nullptr,
-      /*compression_level*/ 0, /*tls_version*/ nullptr, /*tls_cipher*/ nullptr,
-      /*ignore_ws_mem_limit*/ true, /*allow_drop_write_set*/ true);
+      const_cast<char *>("<NULL>"), 0, NULL, NULL, false, NULL, NULL, NULL,
+      NULL, NULL, NULL, NULL, false, GROUP_REPLICATION_APPLIER_THREAD_PRIORITY,
+      0, true, NULL, false, NULL, 0);
 
   if (error) {
     LogPluginErr(ERROR_LEVEL,
@@ -84,7 +75,7 @@ int Applier_handler::initialize_repositories(bool reset_logs,
 int Applier_handler::start_applier_thread() {
   DBUG_TRACE;
 
-  int error = channel_interface.start_threads(false, true, nullptr, false);
+  int error = channel_interface.start_threads(false, true, NULL, false);
   if (error) {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_APPLIER_THD_START_ERROR);
   }
@@ -111,10 +102,10 @@ int Applier_handler::handle_event(Pipeline_event *event, Continuation *cont) {
   DBUG_TRACE;
   int error = 0;
 
-  Data_packet *p = nullptr;
+  Data_packet *p = NULL;
   error = event->get_Packet(&p);
   DBUG_EXECUTE_IF("applier_handler_force_error_on_pipeline", error = 1;);
-  if (error || (p == nullptr)) {
+  if (error || (p == NULL)) {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_FETCH_TRANS_DATA_FAILED);
     error = 1;
     goto end;
@@ -128,7 +119,9 @@ int Applier_handler::handle_event(Pipeline_event *event, Continuation *cont) {
   if (event->get_event_type() != binary_log::TRANSACTION_CONTEXT_EVENT) {
     error = channel_interface.queue_packet((const char *)p->payload, p->len);
 
-    if (event->get_event_type() == binary_log::GTID_LOG_EVENT) {
+    if (event->get_event_type() == binary_log::GTID_LOG_EVENT &&
+        local_member_info->get_recovery_status() ==
+            Group_member_info::MEMBER_ONLINE) {
       applier_module->get_pipeline_stats_member_collector()
           ->increment_transactions_waiting_apply();
     }
