@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +29,7 @@ static void *launch_handler_thread(void *arg) {
   Primary_election_secondary_process *handler =
       (Primary_election_secondary_process *)arg;
   handler->secondary_election_process_handler();
-  return nullptr;
+  return 0;
 }
 
 Primary_election_secondary_process::Primary_election_secondary_process()
@@ -40,6 +39,8 @@ Primary_election_secondary_process::Primary_election_secondary_process()
       primary_ready(false),
       group_in_read_mode(false),
       is_waiting_on_read_mode_group(false),
+      read_mode_session_id(0),
+      is_read_mode_set(SECONDARY_ELECTION_READ_MODE_NOT_SET),
       number_of_know_members(0) {
   mysql_mutex_init(key_GR_LOCK_primary_election_secondary_process_run,
                    &election_lock, MY_MUTEX_INIT_FAST);
@@ -58,13 +59,13 @@ void Primary_election_secondary_process::set_stop_wait_timeout(ulong timeout) {
 
 int Primary_election_secondary_process::launch_secondary_election_process(
     enum_primary_election_mode mode, std::string &primary_to_elect,
-    Group_member_info_list *group_members_info) {
+    std::vector<Group_member_info *> *group_members_info) {
   DBUG_TRACE;
 
   mysql_mutex_lock(&election_lock);
 
   // Callers should ensure the process is terminated
-  assert(election_process_thd_state.is_thread_dead());
+  DBUG_ASSERT(election_process_thd_state.is_thread_dead());
   if (election_process_thd_state.is_thread_alive()) {
     mysql_mutex_unlock(&election_lock); /* purecov: inspected */
     return 2;                           /* purecov: inspected */
@@ -76,6 +77,8 @@ int Primary_election_secondary_process::launch_secondary_election_process(
   group_in_read_mode = false;
   is_waiting_on_read_mode_group = false;
   election_process_aborted = false;
+  read_mode_session_id = 0;
+  is_read_mode_set = SECONDARY_ELECTION_READ_MODE_NOT_SET;
 
   known_members_addresses.clear();
   for (Group_member_info *member : *group_members_info) {
@@ -121,7 +124,7 @@ int Primary_election_secondary_process::secondary_election_process_handler() {
   int error = 0;
   std::string err_msg;
 
-  THD *thd = nullptr;
+  THD *thd = NULL;
   thd = new THD;
   my_thread_init();
   thd->set_new_thread_id();
@@ -153,16 +156,14 @@ int Primary_election_secondary_process::secondary_election_process_handler() {
       abort_plugin_process(
           "Cannot enable the super read only mode on a secondary member.");
       error = 1;
-      election_process_aborted = true;
+      election_process_aborted = 1;
       goto end;
     }
   }
 
   if (election_mode == DEAD_OLD_PRIMARY) {
-    group_events_observation_manager->after_primary_election(
-        primary_uuid,
-        enum_primary_election_primary_change_status::PRIMARY_DID_CHANGE,
-        election_mode);
+    group_events_observation_manager->after_primary_election(primary_uuid, true,
+                                                             election_mode);
     goto wait_for_queued_message;
   }
 
@@ -215,12 +216,13 @@ end:
   primary_election_handler->set_election_running(false);
 
   if (!election_process_aborted && !error) {
-    Group_member_info primary_member_info;
-    if (!group_member_mgr->get_group_member_info(primary_uuid,
-                                                 primary_member_info)) {
-      LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_SRV_SECONDARY_MEM,
-                   primary_member_info.get_hostname().c_str(),
-                   primary_member_info.get_port());
+    Group_member_info *primary_member_info =
+        group_member_mgr->get_group_member_info(primary_uuid);
+    if (primary_member_info != NULL) {
+      LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_SERVER_WORKING_AS_SECONDARY,
+                   primary_member_info->get_hostname().c_str(),
+                   primary_member_info->get_port());
+      delete primary_member_info;
     }
   }
 
@@ -228,10 +230,7 @@ end:
 
   if (error && !election_process_aborted) {
     group_events_observation_manager->after_primary_election(
-        primary_uuid,
-        enum_primary_election_primary_change_status::
-            PRIMARY_DID_CHANGE_WITH_ERROR,
-        election_mode, error); /* purecov: inspected */
+        primary_uuid, true, election_mode, error); /* purecov: inspected */
     kill_transactions_and_leave_on_election_error(
         err_msg); /* purecov: inspected */
   }
@@ -245,16 +244,16 @@ end:
   global_thd_manager_remove_thd(thd);
   delete thd;
 
-  Gcs_interface_factory::cleanup_thread_communication_resources(
-      Gcs_operations::get_gcs_engine());
-
-  my_thread_end();
-
   mysql_mutex_lock(&election_lock);
   election_process_thd_state.set_terminated();
   mysql_cond_broadcast(&election_cond);
   mysql_mutex_unlock(&election_lock);
 
+  Gcs_interface_factory::cleanup_thread_communication_resources(
+      Gcs_operations::get_gcs_engine());
+
+  my_thread_end();
+
   return error;
 }
 
@@ -267,9 +266,26 @@ bool Primary_election_secondary_process::enable_read_mode_on_server() {
   remote_clone_handler->lock_gr_clone_read_mode_lock();
 
   if (!plugin_is_group_replication_cloning()) {
-    if (!error && !election_process_aborted) {
-      error = enable_server_read_mode();
+    mysql_mutex_lock(&election_lock);
+    Sql_service_command_interface *sql_command_interface =
+        new Sql_service_command_interface();
+    error = sql_command_interface->establish_session_connection(
+        PSESSION_USE_THREAD, GROUPREPL_USER, get_plugin_pointer());
+    if (!error) {
+      read_mode_session_id =
+          sql_command_interface->get_sql_service_interface()->get_session_id();
+      is_read_mode_set = SECONDARY_ELECTION_READ_MODE_BEING_SET;
     }
+    mysql_mutex_unlock(&election_lock);
+
+    if (!error && !election_process_aborted) {
+      error = enable_super_read_only_mode(sql_command_interface);
+    }
+
+    mysql_mutex_lock(&election_lock);
+    delete sql_command_interface;
+    is_read_mode_set = SECONDARY_ELECTION_READ_MODE_IS_SET;
+    mysql_mutex_unlock(&election_lock);
   }
 
   remote_clone_handler->unlock_gr_clone_read_mode_lock();
@@ -277,6 +293,30 @@ bool Primary_election_secondary_process::enable_read_mode_on_server() {
   return error != 0;
 }
 
+bool Primary_election_secondary_process::kill_read_mode_query() {
+  int error = 0;
+
+  mysql_mutex_assert_owner(&election_lock);
+
+  if (is_read_mode_set == SECONDARY_ELECTION_READ_MODE_BEING_SET) {
+    DBUG_ASSERT(read_mode_session_id != 0);
+    Sql_service_command_interface *sql_command_interface =
+        new Sql_service_command_interface();
+    error = sql_command_interface->establish_session_connection(
+        PSESSION_DEDICATED_THREAD, GROUPREPL_USER, get_plugin_pointer());
+    if (!error) {
+      error = sql_command_interface->kill_session(read_mode_session_id);
+      // If the thread is no longer there don't report an warning
+      if (ER_NO_SUCH_THREAD == error) {
+        error = 0; /* purecov: inspected */
+      }
+    }
+    delete sql_command_interface;
+  }
+
+  return error != 0;
+}
+
 bool Primary_election_secondary_process::signal_read_mode_ready() {
   Single_primary_message single_primary_message(
       Single_primary_message::SINGLE_PRIMARY_READ_MODE_SET);
@@ -308,25 +348,14 @@ int Primary_election_secondary_process::after_view_change(
     if (!group_in_read_mode) {
       group_in_read_mode = true;
       mysql_cond_broadcast(&election_cond);
-      /*
-       group_in_read_mode is false so response from some member was still
-       pending. But known_members_addresses is empty so members on which
-       election was waiting have left the group.
-       If primary is part of the group then we can end the election normally.
-       If primary member has left the group then forcefully end the election so
-       that new election can take place.
-      */
-      const enum_primary_election_primary_change_status primary_changed_status =
-          group_member_mgr->is_member_info_present(primary_uuid)
-              ? enum_primary_election_primary_change_status::PRIMARY_DID_CHANGE
-              : enum_primary_election_primary_change_status::
-                    PRIMARY_DID_NOT_CHANGE_PRIMARY_LEFT_FORCE_ELECTION_END;
       group_events_observation_manager->after_primary_election(
-          primary_uuid, primary_changed_status, election_mode);
+          primary_uuid, true, election_mode);
     }
   }
 
-  if (!group_member_mgr->is_member_info_present(primary_uuid)) {
+  Group_member_info *member_info =
+      group_member_mgr->get_group_member_info(primary_uuid);
+  if (member_info == NULL) {
     if (!group_in_read_mode) {
       election_process_aborted = true;
     } else {
@@ -335,14 +364,14 @@ int Primary_election_secondary_process::after_view_change(
     }
     mysql_cond_broadcast(&election_cond);
   }
+  delete member_info;
 
   mysql_mutex_unlock(&election_lock);
   return 0;
 }
 
 int Primary_election_secondary_process::after_primary_election(
-    std::string, enum_primary_election_primary_change_status,
-    enum_primary_election_mode, int) {
+    std::string, bool, enum_primary_election_mode, int) {
   return 0;
 }
 
@@ -391,9 +420,7 @@ int Primary_election_secondary_process::before_message_handling(
           group_in_read_mode = true;
           mysql_cond_broadcast(&election_cond);
           group_events_observation_manager->after_primary_election(
-              primary_uuid,
-              enum_primary_election_primary_change_status::PRIMARY_DID_CHANGE,
-              election_mode);
+              primary_uuid, true, election_mode);
         }
       }
       mysql_mutex_unlock(&election_lock);
@@ -415,6 +442,13 @@ int Primary_election_secondary_process::terminate_election_process(bool wait) {
   // Awake up possible stuck conditions
   mysql_cond_broadcast(&election_cond);
 
+  if (kill_read_mode_query()) {
+    abort_plugin_process(
+        "In the primary election process it was not possible to kill a "
+        "previous query trying to enable the server read "
+        "mode."); /* purecov: inspected */
+  }
+
   if (wait) {
     while (election_process_thd_state.is_thread_alive()) {
       DBUG_PRINT("sleep", ("Waiting for the Primary election secondary process "
@@ -422,7 +456,7 @@ int Primary_election_secondary_process::terminate_election_process(bool wait) {
       mysql_cond_wait(&election_cond, &election_lock);
     }
 
-    assert(election_process_thd_state.is_thread_dead());
+    DBUG_ASSERT(election_process_thd_state.is_thread_dead());
   }
   mysql_mutex_unlock(&election_lock);
 
