@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,9 +26,9 @@
 Transaction_prepared_message::Transaction_prepared_message(const rpl_sid *sid,
                                                            rpl_gno gno)
     : Plugin_gcs_message(CT_TRANSACTION_PREPARED_MESSAGE),
-      m_sid_specified(sid != nullptr ? true : false),
+      m_sid_specified(sid != NULL ? true : false),
       m_gno(gno) {
-  if (sid != nullptr) {
+  if (sid != NULL) {
     m_sid = *sid;
   }
 }
@@ -42,7 +41,7 @@ Transaction_prepared_message::Transaction_prepared_message(
   decode(buf, len);
 }
 
-Transaction_prepared_message::~Transaction_prepared_message() = default;
+Transaction_prepared_message::~Transaction_prepared_message() {}
 
 void Transaction_prepared_message::encode_payload(
     std::vector<unsigned char> *buffer) const {
@@ -86,7 +85,7 @@ void Transaction_prepared_message::decode_payload(const unsigned char *buffer,
 }
 
 const rpl_sid *Transaction_prepared_message::get_sid() {
-  return m_sid_specified ? &m_sid : nullptr;
+  return m_sid_specified ? &m_sid : NULL;
 }
 
 rpl_gno Transaction_prepared_message::get_gno() { return m_gno; }
