@@ -67,6 +67,7 @@ PSI_mutex_key key_GR_LOCK_applier_module_run,
     key_GR_LOCK_recovery_donor_selection,
     key_GR_LOCK_recovery_module_run,
     key_GR_LOCK_server_ongoing_transaction_handler,
+    key_GR_LOCK_message_service_run,
     key_GR_LOCK_session_thread_method_exec,
     key_GR_LOCK_session_thread_run,
     key_GR_LOCK_stage_monitor_handler,
@@ -106,6 +107,7 @@ PSI_cond_key key_GR_COND_applier_module_run,
     key_GR_COND_recovery_module_run,
     key_GR_COND_session_thread_method_exec,
     key_GR_COND_session_thread_run,
+    key_GR_COND_message_service_run,
     key_GR_COND_synchronized_queue,
     key_GR_COND_view_modification_wait,
     key_GR_COND_wait_ticket,
@@ -121,7 +123,8 @@ PSI_thread_key key_GR_THD_applier_module_receiver,
     key_GR_THD_primary_election_primary_process,
     key_GR_THD_primary_election_secondary_process,
     key_GR_THD_group_partition_handler,
-    key_GR_THD_recovery;
+    key_GR_THD_recovery,
+    key_GR_THD_message_service_handler;
 
 PSI_rwlock_key key_GR_RWLOCK_cert_stable_gtid_set,
     key_GR_RWLOCK_channel_observation_list,
@@ -307,6 +310,8 @@ static PSI_mutex_info all_group_replication_psi_mutex_keys[] = {
     {&key_GR_LOCK_server_ongoing_transaction_handler,
      "LOCK_server_ongoing_transaction_handler", PSI_FLAG_SINGLETON, 0,
      PSI_DOCUMENT_ME},
+    {&key_GR_LOCK_message_service_run, "LOCK_message_service_run",
+     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
     {&key_GR_LOCK_session_thread_method_exec, "LOCK_session_thread_method_exec",
      PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
     {&key_GR_LOCK_session_thread_run, "LOCK_session_thread_run",
@@ -361,6 +366,8 @@ static PSI_cond_info all_group_replication_psi_condition_keys[] = {
      PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
     {&key_GR_COND_group_part_handler_abort, "COND_group_part_handler_abort",
      PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
+    {&key_GR_COND_message_service_run, "COND_message_service_run",
+     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
     {&key_GR_COND_multi_primary_action_notification,
      "COND_multi_primary_action_notification", PSI_FLAG_SINGLETON, 0,
      PSI_DOCUMENT_ME},
