@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,11 +27,9 @@
 #include "my_dbug.h"
 #include "my_systime.h"
 #include "mysql/components/services/log_builtins.h"
-#include "plugin/group_replication/include/compatibility_module.h"
 #include "plugin/group_replication/include/plugin.h"
 #include "plugin/group_replication/include/plugin_psi.h"
 #include "plugin/group_replication/include/plugin_server_include.h"
-#include "plugin/group_replication/include/plugin_variables/recovery_endpoints.h"
 #include "plugin/group_replication/include/recovery_channel_state_observer.h"
 #include "plugin/group_replication/include/recovery_state_transfer.h"
 
@@ -41,10 +38,8 @@ using std::string;
 Recovery_state_transfer::Recovery_state_transfer(
     char *recovery_channel_name, const string &member_uuid,
     Channel_observation_manager *channel_obsr_mngr)
-    : selected_donor(nullptr),
-      group_members(nullptr),
-      suitable_donors(
-          Malloc_allocator<Group_member_info *>(key_group_member_info)),
+    : selected_donor(NULL),
+      group_members(NULL),
       donor_connection_retry_count(0),
       recovery_aborted(false),
       donor_transfer_finished(false),
@@ -52,11 +47,10 @@ Recovery_state_transfer::Recovery_state_transfer(
       on_failover(false),
       donor_connection_interface(recovery_channel_name),
       channel_observation_manager(channel_obsr_mngr),
-      recovery_channel_observer(nullptr),
+      recovery_channel_observer(NULL),
       recovery_use_ssl(false),
       recovery_get_public_key(false),
       recovery_ssl_verify_server_cert(false),
-      recovery_tls_ciphersuites_null(true),
       max_connection_attempts_to_donors(0),
       donor_reconnect_interval(0) {
   // set the recovery SSL options to 0
@@ -68,8 +62,6 @@ Recovery_state_transfer::Recovery_state_transfer(
   (void)strncpy(recovery_ssl_crl, "", 1);
   (void)strncpy(recovery_ssl_crlpath, "", 1);
   (void)strncpy(recovery_public_key_path, "", 1);
-  (void)strncpy(recovery_tls_version, "", 1);
-  (void)strncpy(recovery_tls_ciphersuites, "", 1);
 
   this->member_uuid = member_uuid;
 
@@ -82,8 +74,9 @@ Recovery_state_transfer::Recovery_state_transfer(
 }
 
 Recovery_state_transfer::~Recovery_state_transfer() {
-  if (group_members != nullptr) {
-    Group_member_info_list_iterator member_it = group_members->begin();
+  if (group_members != NULL) {
+    std::vector<Group_member_info *>::iterator member_it =
+        group_members->begin();
     while (member_it != group_members->end()) {
       delete (*member_it);
       ++member_it;
@@ -91,8 +84,6 @@ Recovery_state_transfer::~Recovery_state_transfer() {
   }
   delete group_members;
   delete recovery_channel_observer;
-  delete selected_donor;
-  selected_donor = nullptr;
   mysql_mutex_destroy(&recovery_lock);
   mysql_cond_destroy(&recovery_condition);
   mysql_mutex_destroy(&donor_selection_lock);
@@ -167,8 +158,7 @@ void Recovery_state_transfer::inform_of_receiver_stop(my_thread_id thread_id) {
 void Recovery_state_transfer::initialize_group_info() {
   DBUG_TRACE;
 
-  delete selected_donor;
-  selected_donor = nullptr;
+  selected_donor = NULL;
   selected_donor_hostname.clear();
   // Update the group member info
   mysql_mutex_lock(&donor_selection_lock);
@@ -179,18 +169,19 @@ void Recovery_state_transfer::initialize_group_info() {
 void Recovery_state_transfer::update_group_membership(bool update_donor) {
   DBUG_TRACE;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   mysql_mutex_assert_owner(&donor_selection_lock);
 #endif
 
   // if needed update the reference to the donor member
   string donor_uuid;
-  if (selected_donor != nullptr && update_donor) {
+  if (selected_donor != NULL && update_donor) {
     donor_uuid.assign(selected_donor->get_uuid());
   }
 
-  if (group_members != nullptr) {
-    Group_member_info_list_iterator member_it = group_members->begin();
+  if (group_members != NULL) {
+    std::vector<Group_member_info *>::iterator member_it =
+        group_members->begin();
     while (member_it != group_members->end()) {
       delete (*member_it);
       ++member_it;
@@ -239,11 +230,14 @@ int Recovery_state_transfer::update_recovery_process(bool did_members_left) {
     * Was deleted in a previous group updated, but there was no need to
       select a new one since as the data transfer is finished
   */
-  if (selected_donor != nullptr && did_members_left) {
+  if (selected_donor != NULL && did_members_left) {
     current_donor_uuid.assign(selected_donor->get_uuid());
     current_donor_hostname.assign(selected_donor->get_hostname());
     current_donor_port = selected_donor->get_port();
-    donor_left = !group_member_mgr->is_member_info_present(current_donor_uuid);
+    Group_member_info *current_donor =
+        group_member_mgr->get_group_member_info(current_donor_uuid);
+    donor_left = (current_donor == NULL);
+    delete current_donor;
   }
 
   /*
@@ -259,8 +253,7 @@ int Recovery_state_transfer::update_recovery_process(bool did_members_left) {
   */
   if (donor_left) {
     // The selected donor no longer holds a meaning after deleting the group
-    delete selected_donor;
-    selected_donor = nullptr;
+    selected_donor = NULL;
     if (connected_to_donor) {
       /*
        The donor_transfer_finished flag is not lock protected on the recovery
@@ -311,7 +304,7 @@ int Recovery_state_transfer::check_recovery_thread_status() {
   // if some of the threads are running
   if (donor_connection_interface.is_receiver_thread_running() ||
       donor_connection_interface.is_applier_thread_running()) {
-    return terminate_recovery_slave_threads() != STATE_TRANSFER_OK;
+    return terminate_recovery_slave_threads(); /* purecov: inspected */
   }
   return 0;
 }
@@ -323,12 +316,10 @@ bool Recovery_state_transfer::is_own_event_channel(my_thread_id id) {
 
 void Recovery_state_transfer::build_donor_list(string *selected_donor_uuid) {
   DBUG_TRACE;
-  const Member_version local_member_version =
-      local_member_info->get_member_version();
 
   suitable_donors.clear();
 
-  Group_member_info_list_iterator member_it = group_members->begin();
+  std::vector<Group_member_info *>::iterator member_it = group_members->begin();
 
   while (member_it != group_members->end()) {
     Group_member_info *member = *member_it;
@@ -338,18 +329,12 @@ void Recovery_state_transfer::build_donor_list(string *selected_donor_uuid) {
         member->get_recovery_status() == Group_member_info::MEMBER_ONLINE;
     bool not_self = m_uuid.compare(member_uuid);
     bool valid_donor = false;
-    const Member_version member_version = member->get_member_version();
 
     if (is_online && not_self) {
-      if (member_version <= local_member_version) {
+      if (member->get_member_version() <=
+          local_member_info->get_member_version()) {
         suitable_donors.push_back(member);
         valid_donor = true;
-      } else if (Compatibility_module::is_version_8_0_lts(member_version) &&
-                 Compatibility_module::is_version_8_0_lts(
-                     local_member_version)) {
-        suitable_donors.push_back(member);
-        valid_donor = true;
-
       } else if (get_allow_local_lower_version_join()) {
         suitable_donors.push_back(member);
         valid_donor = true;
@@ -358,20 +343,16 @@ void Recovery_state_transfer::build_donor_list(string *selected_donor_uuid) {
 
     // if requested, and if the donor is still in the group, update its
     // reference
-    if (selected_donor_uuid != nullptr &&
-        !m_uuid.compare(*selected_donor_uuid) && valid_donor) {
-      if (selected_donor != nullptr) {
-        selected_donor->update(*member);
-      } else {
-        selected_donor = new Group_member_info(*member);
-      }
+    if (selected_donor_uuid != NULL && !m_uuid.compare(*selected_donor_uuid) &&
+        valid_donor) {
+      selected_donor = member;
     }
 
     ++member_it;
   }
 
   if (suitable_donors.size() > 1) {
-    vector_random_shuffle(&suitable_donors);
+    std::random_shuffle(suitable_donors.begin(), suitable_donors.end());
   }
 
   // no need for errors if no donors exist, we thrown it in the connection
@@ -392,7 +373,7 @@ int Recovery_state_transfer::establish_donor_connection() {
         const char act[] =
             "now signal signal.connection_attempt_3 wait_for "
             "signal.reset_recovery_retry_count_done";
-        assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+        DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
       }
     };);
     // max number of retries reached, abort
@@ -431,7 +412,7 @@ int Recovery_state_transfer::establish_donor_connection() {
 
       mysql_mutex_lock(&donor_selection_lock);
 
-      build_donor_list(nullptr);
+      build_donor_list(NULL);
       if (suitable_donors.empty()) {
         LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_NO_VALID_DONOR);
         donor_connection_retry_count++;
@@ -443,34 +424,18 @@ int Recovery_state_transfer::establish_donor_connection() {
     donor_channel_thread_error = false;
 
     // Get the last element and delete it
-    if (selected_donor != nullptr) {
-      selected_donor->update(*suitable_donors.back());
-    } else {
-      selected_donor = new Group_member_info(*suitable_donors.back());
-    }
+    selected_donor = suitable_donors.back();
     suitable_donors.pop_back();
     // increment the number of tries
     donor_connection_retry_count++;
 
-    Donor_recovery_endpoints donor_endpoints;
-    std::vector<std::pair<std::string, uint>> endpoints;
-    endpoints = donor_endpoints.get_endpoints(selected_donor);
+    if ((error = initialize_donor_connection())) {
+      LogPluginErr(ERROR_LEVEL,
+                   ER_GRP_RPL_CONFIG_RECOVERY); /* purecov: inspected */
+    }
 
-    for (auto endpoint : endpoints) {
-      auto hostname = endpoint.first;
-      uint port = endpoint.second;
-      if ((error = initialize_donor_connection(hostname, port))) {
-        LogPluginErr(ERROR_LEVEL,
-                     ER_GRP_RPL_CONFIG_RECOVERY); /* purecov: inspected */
-      }
-
-      if (!error && !recovery_aborted) {
-        error = start_recovery_donor_threads();
-      }
-
-      if (!error) {
-        break;
-      }
+    if (!error && !recovery_aborted) {
+      error = start_recovery_donor_threads();
     }
 
     if (!error) {
@@ -491,8 +456,7 @@ int Recovery_state_transfer::establish_donor_connection() {
   return error;
 }
 
-int Recovery_state_transfer::initialize_donor_connection(std::string hostname,
-                                                         uint port) {
+int Recovery_state_transfer::initialize_donor_connection() {
   DBUG_TRACE;
 
   int error = 0;
@@ -507,27 +471,25 @@ int Recovery_state_transfer::initialize_donor_connection(std::string hostname,
     attached to this object, more precisely to
     selected_donor_hostname class member.
   */
-
-  selected_donor_hostname.assign(hostname);
+  selected_donor_hostname.assign(selected_donor->get_hostname());
+  char *hostname = const_cast<char *>(selected_donor_hostname.c_str());
+  uint port = selected_donor->get_port();
 
   error = donor_connection_interface.initialize_channel(
-      const_cast<char *>(hostname.c_str()), port, /*user*/ nullptr,
-      /*pass*/ nullptr, recovery_use_ssl, recovery_ssl_ca, recovery_ssl_capath,
-      recovery_ssl_cert, recovery_ssl_cipher, recovery_ssl_key,
-      recovery_ssl_crl, recovery_ssl_crlpath, recovery_ssl_verify_server_cert,
-      DEFAULT_THREAD_PRIORITY, /*retry_count*/ 1, /*preserve_logs*/ false,
+      hostname, port, NULL, NULL, recovery_use_ssl, recovery_ssl_ca,
+      recovery_ssl_capath, recovery_ssl_cert, recovery_ssl_cipher,
+      recovery_ssl_key, recovery_ssl_crl, recovery_ssl_crlpath,
+      recovery_ssl_verify_server_cert, DEFAULT_THREAD_PRIORITY, 1, false,
       recovery_public_key_path, recovery_get_public_key,
-      recovery_compression_algorithm, recovery_zstd_compression_level,
-      recovery_tls_version,
-      recovery_tls_ciphersuites_null ? nullptr : recovery_tls_ciphersuites,
-      /*ignore_ws_mem_limit*/ true, /*allow_drop_write_set*/ true);
+      recovery_compression_algorithm, recovery_zstd_compression_level);
 
   if (!error) {
     LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_ESTABLISHING_CONN_GRP_REC_DONOR,
-                 selected_donor->get_uuid().c_str(), hostname.c_str(), port);
+                 selected_donor->get_uuid().c_str(), hostname, port);
   } else {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_CREATE_GRP_RPL_REC_CHANNEL,
-                 selected_donor->get_uuid().c_str(), hostname.c_str(), port);
+                 selected_donor->get_uuid().c_str(), hostname,
+                 port); /* purecov: inspected */
   }
 
   return error;
@@ -544,13 +506,13 @@ int Recovery_state_transfer::start_recovery_donor_threads() {
       const char act[] =
           "now "
           "WAIT_FOR reached_stopping_io_thread";
-      assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+      DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
     };);
     DBUG_EXECUTE_IF("pause_after_sql_thread_stop_hook", {
       const char act[] =
           "now "
           "WAIT_FOR reached_stopping_sql_thread";
-      assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+      DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
     };);
 
     /*
@@ -586,11 +548,11 @@ int Recovery_state_transfer::start_recovery_donor_threads() {
 
   DBUG_EXECUTE_IF("pause_after_io_thread_stop_hook", {
     const char act[] = "now SIGNAL continue_to_stop_io_thread";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
   DBUG_EXECUTE_IF("pause_after_sql_thread_stop_hook", {
     const char act[] = "now SIGNAL continue_to_stop_sql_thread";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+    DBUG_ASSERT(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   };);
 
   if (error) {
@@ -608,22 +570,21 @@ int Recovery_state_transfer::start_recovery_donor_threads() {
   return error;
 }
 
-State_transfer_status Recovery_state_transfer::terminate_recovery_slave_threads(
-    bool purge_logs) {
+int Recovery_state_transfer::terminate_recovery_slave_threads(bool purge_logs) {
   DBUG_TRACE;
 
   LogPluginErr(INFORMATION_LEVEL, ER_GRP_RPL_DONOR_CONN_TERMINATION);
 
-  State_transfer_status error = STATE_TRANSFER_OK;
+  int error = 0;
 
   // If the threads never started, the method just returns
-  if (donor_connection_interface.stop_threads(true, true)) {
-    error = STATE_TRANSFER_STOP;
-    LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_STOPPING_GRP_REC);
+  if ((error = donor_connection_interface.stop_threads(true, true))) {
+    LogPluginErr(ERROR_LEVEL,
+                 ER_GRP_RPL_STOPPING_GRP_REC); /* purecov: inspected */
   } else {
     if (purge_logs) {
       // If there is no repository in place nothing happens
-      if (purge_recovery_slave_threads_repos()) error = STATE_TRANSFER_PURGE;
+      error = purge_recovery_slave_threads_repos();
     }
   }
 
@@ -641,42 +602,27 @@ int Recovery_state_transfer::purge_recovery_slave_threads_repos() {
     /* purecov: end */
   }
   error = donor_connection_interface.initialize_channel(
-      /*host*/ const_cast<char *>("<NULL>"), /*port*/ 0, /*user*/ nullptr,
-      /*pass*/ nullptr, /*use_ssl*/ false, /*ssl_ca*/ nullptr,
-      /*ssl_capath*/ nullptr, /*ssl_cert*/ nullptr, /*ssl_cipher*/ nullptr,
-      /*ssl_key*/ nullptr, /*ssl_crl*/ nullptr, /*ssl_crlpath*/ nullptr,
-      /*ssl_verify*/ false,
-      /*priority*/ DEFAULT_THREAD_PRIORITY,
-      /*retry_count*/ 1,
-      /*preserve_logs*/ false, /*public_key_path*/ nullptr,
-      /*get_public_key*/ false, /*compression_alg*/ nullptr,
-      /*compression_level*/ 0, /*tls_version*/ nullptr, /*tls_cipher*/ nullptr,
-      /*ignore_ws_mem_limit*/ true, /*allow_drop_write_set*/ true);
+      const_cast<char *>("<NULL>"), 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+      NULL, NULL, NULL, NULL, DEFAULT_THREAD_PRIORITY, 1, false, NULL, false,
+      NULL, 0);
 
   return error;
 }
 
-State_transfer_status Recovery_state_transfer::state_transfer(
+int Recovery_state_transfer::state_transfer(
     Plugin_stage_monitor_handler &stage_handler) {
   DBUG_TRACE;
 
-  State_transfer_status error = STATE_TRANSFER_OK;
+  int error = 0;
 
   while (!donor_transfer_finished && !recovery_aborted) {
-    /*
-      If an applier error happened: stop the slave threads.
-      We do not purge logs or reset channel configuration to
-      preserve the error information on performance schema
-      tables until the next recovery attempt.
-      Recovery_state_transfer::initialize_donor_connection() will
-      take care of that.
-    */
+    // If an applier error happened: stop the receiver thread and purge the logs
     if (donor_channel_thread_error) {
       // Unsubscribe the listener until it connects again.
       channel_observation_manager->unregister_channel_observer(
           recovery_channel_observer);
 
-      if ((error = terminate_recovery_slave_threads(false))) {
+      if ((error = terminate_recovery_slave_threads())) {
         /* purecov: begin inspected */
         LogPluginErr(ERROR_LEVEL,
                      ER_GRP_RPL_UNABLE_TO_KILL_CONN_REC_DONOR_APPLIER);
@@ -693,12 +639,12 @@ State_transfer_status Recovery_state_transfer::state_transfer(
           recovery_channel_observer);
 
       // Stop the threads before reconfiguring the connection
-      if (donor_connection_interface.stop_threads(true, true)) {
+      if ((error = donor_connection_interface.stop_threads(true, true))) {
         /* purecov: begin inspected */
         LogPluginErr(ERROR_LEVEL,
                      ER_GRP_RPL_UNABLE_TO_KILL_CONN_REC_DONOR_FAILOVER);
         // if we can't stop, abort recovery
-        return STATE_TRANSFER_STOP;
+        return error;
         /* purecov: end */
       }
     }
@@ -707,8 +653,7 @@ State_transfer_status Recovery_state_transfer::state_transfer(
                             __FILE__, __LINE__, 0, 0);
     if (!recovery_aborted) {
       // if the connection to the donor failed, abort recovery
-      if (establish_donor_connection()) {
-        error = STATE_TRANSFER_NO_CONNECTION;
+      if ((error = establish_donor_connection())) {
         break;
       }
     }
@@ -738,17 +683,8 @@ State_transfer_status Recovery_state_transfer::state_transfer(
 
   channel_observation_manager->unregister_channel_observer(
       recovery_channel_observer);
-
   // do not purge logs if an error occur, keep the diagnose on SLAVE STATUS
-  bool purge_relay_logs = (error == STATE_TRANSFER_OK);
-  DBUG_EXECUTE_IF("gr_recovery_skip_purge_logs", { purge_relay_logs = false; });
-  State_transfer_status stop_error =
-      terminate_recovery_slave_threads(purge_relay_logs);
-
-  // If a connection error exists, we preserve it
-  if (error == STATE_TRANSFER_OK && stop_error != STATE_TRANSFER_OK)
-    error = stop_error;
-
+  terminate_recovery_slave_threads(!error);
   connected_to_donor = false;
 
   return error;
