@@ -34,8 +34,9 @@ int Replication_thread_api::initialize_channel(
     char *ssl_ca, char *ssl_capath, char *ssl_cert, char *ssl_cipher,
     char *ssl_key, char *ssl_crl, char *ssl_crlpath,
     bool ssl_verify_server_cert, int priority, int retry_count,
-    bool preserve_logs, char *public_key_path, bool get_public_key) {
-  DBUG_ENTER("Replication_thread_api::initialize");
+    bool preserve_logs, char *public_key_path, bool get_public_key,
+    char *compression_algorithm, uint zstd_compression_level) {
+  DBUG_TRACE;
   int error = 0;
 
   Channel_creation_info info;
@@ -63,6 +64,9 @@ int Replication_thread_api::initialize_channel(
 
   info.get_public_key = get_public_key;
 
+  info.compression_algorithm = compression_algorithm;
+  info.zstd_compression_level = zstd_compression_level;
+
   if (use_ssl || ssl_ca != NULL || ssl_capath != NULL || ssl_cert != NULL ||
       ssl_cipher != NULL || ssl_key != NULL || ssl_crl != NULL ||
       ssl_crlpath != NULL || ssl_verify_server_cert) {
@@ -85,13 +89,13 @@ int Replication_thread_api::initialize_channel(
   */
   if (!error) error = channel_flush(interface_channel);
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 int Replication_thread_api::start_threads(bool start_receiver,
                                           bool start_applier, string *view_id,
                                           bool wait_for_connection) {
-  DBUG_ENTER("Replication_thread_api::start_threads");
+  DBUG_TRACE;
 
   Channel_connection_info info;
   initialize_channel_connection_info(&info);
@@ -121,29 +125,29 @@ int Replication_thread_api::start_threads(bool start_receiver,
     delete[] cview_id;
   }
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 int Replication_thread_api::purge_logs(bool reset_all) {
-  DBUG_ENTER("Replication_thread_api::purge_logs");
+  DBUG_TRACE;
 
   // If there is no channel, no point in invoking the method
-  if (!channel_is_active(interface_channel, CHANNEL_NO_THD)) DBUG_RETURN(0);
+  if (!channel_is_active(interface_channel, CHANNEL_NO_THD)) return 0;
 
   int error = channel_purge_queue(interface_channel, reset_all);
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 int Replication_thread_api::stop_threads(bool stop_receiver,
                                          bool stop_applier) {
-  DBUG_ENTER("Replication_thread_api::stop_threads");
+  DBUG_TRACE;
 
   stop_receiver = stop_receiver && is_receiver_thread_running();
   stop_applier = stop_applier && is_applier_thread_running();
 
   // If there is nothing to do, return 0
-  if (!stop_applier && !stop_receiver) DBUG_RETURN(0);
+  if (!stop_applier && !stop_receiver) return 0;
 
   int thread_mask = 0;
   if (stop_applier) {
@@ -155,7 +159,7 @@ int Replication_thread_api::stop_threads(bool stop_receiver,
 
   int error = channel_stop(interface_channel, thread_mask, stop_wait_timeout);
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 bool Replication_thread_api::is_receiver_thread_running() {
@@ -183,7 +187,7 @@ bool Replication_thread_api::is_applier_thread_waiting() {
 }
 
 int Replication_thread_api::wait_for_gtid_execution(double timeout) {
-  DBUG_ENTER("Replication_thread_api::wait_for_gtid_execution");
+  DBUG_TRACE;
 
   int error =
       channel_wait_until_apply_queue_applied(interface_channel, timeout);
@@ -202,33 +206,33 @@ int Replication_thread_api::wait_for_gtid_execution(double timeout) {
       error = REPLICATION_THREAD_WAIT_TIMEOUT_ERROR;
   }
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 int Replication_thread_api::wait_for_gtid_execution(std::string &retrieved_set,
                                                     double timeout,
                                                     bool update_THD_status) {
-  DBUG_ENTER("Replication_thread_api::wait_for_gtid_execution(gtid_set)");
+  DBUG_TRACE;
 
   int error = channel_wait_until_transactions_applied(
       interface_channel, retrieved_set.c_str(), timeout, update_THD_status);
-  DBUG_RETURN(error);
+  return error;
 }
 
 rpl_gno Replication_thread_api::get_last_delivered_gno(rpl_sidno sidno) {
-  DBUG_ENTER("Replication_thread_api::get_last_delivered_gno");
-  DBUG_RETURN(channel_get_last_delivered_gno(interface_channel, sidno));
+  DBUG_TRACE;
+  return channel_get_last_delivered_gno(interface_channel, sidno);
 }
 
 int Replication_thread_api::get_applier_thread_ids(unsigned long **thread_ids) {
-  DBUG_ENTER("Replication_thread_api::get_applier_thread_ids");
-  DBUG_RETURN(channel_get_thread_id(interface_channel, CHANNEL_APPLIER_THREAD,
-                                    thread_ids));
+  DBUG_TRACE;
+  return channel_get_thread_id(interface_channel, CHANNEL_APPLIER_THREAD,
+                               thread_ids);
 }
 
 bool Replication_thread_api::is_own_event_applier(my_thread_id id,
                                                   const char *channel_name) {
-  DBUG_ENTER("Replication_thread_api::is_own_event_applier");
+  DBUG_TRACE;
 
   bool result = false;
   unsigned long *thread_ids = NULL;
@@ -262,11 +266,11 @@ end:
   my_free(thread_ids);
 
   // The given id is not an id of the channel applier threads, return false
-  DBUG_RETURN(result);
+  return result;
 }
 
 bool Replication_thread_api::is_own_event_receiver(my_thread_id id) {
-  DBUG_ENTER("Replication_thread_api::is_own_event_receiver");
+  DBUG_TRACE;
 
   bool result = false;
   unsigned long *thread_id = NULL;
@@ -282,12 +286,12 @@ bool Replication_thread_api::is_own_event_receiver(my_thread_id id) {
   my_free(thread_id);
 
   // The given id is not the id of the channel receiver thread, return false
-  DBUG_RETURN(result);
+  return result;
 }
 
 bool Replication_thread_api::get_retrieved_gtid_set(std::string &retrieved_set,
                                                     const char *channel_name) {
-  DBUG_ENTER("Replication_thread_api::get_retrieved_gtid_set");
+  DBUG_TRACE;
 
   const char *name = channel_name ? channel_name : interface_channel;
   char *receiver_retrieved_gtid_set = NULL;
@@ -298,7 +302,7 @@ bool Replication_thread_api::get_retrieved_gtid_set(std::string &retrieved_set,
 
   my_free(receiver_retrieved_gtid_set);
 
-  DBUG_RETURN((error != 0));
+  return (error != 0);
 }
 
 bool Replication_thread_api::get_channel_credentials(std::string &username,
@@ -328,17 +332,19 @@ bool Replication_thread_api::is_partial_transaction_on_relay_log() {
 }
 
 int Replication_thread_api::rpl_channel_stop_all(int threads_to_stop,
-                                                 long timeout, int ecode) {
+                                                 long timeout) {
   std::string error_message;
   int error = channel_stop_all(threads_to_stop, timeout, &error_message);
   if (error) {
     if (!error_message.empty()) {
-      LogPluginErr(ERROR_LEVEL, ecode, error_message.c_str());
+      LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_ERROR_STOPPING_CHANNELS,
+                   error_message.c_str());
     } else {
       std::stringstream err_msg_ss;
       err_msg_ss << "Got error: " << error
                  << "Please check the error log for more details.";
-      LogPluginErr(ERROR_LEVEL, ecode, err_msg_ss.str().c_str());
+      LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_ERROR_STOPPING_CHANNELS,
+                   err_msg_ss.str().c_str());
     }
   }
   return error;
