@@ -1,18 +1,17 @@
 /***********************************************************************
 
-Copyright (c) 2011, 2025, Oracle and/or its affiliates.
+Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,6 +58,18 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 /** Define also present in daemon/memcached.h */
 #define KEY_MAX_LENGTH 250
 
+/** Time (in seconds) that background thread sleeps before it wakes
+up and commit idle connection transactions */
+#define BK_COMMIT_THREAD_SLEEP_INTERVAL 5
+
+/** Maximum number of connections that background thread processes each
+time */
+#define BK_MAX_PROCESS_COMMIT 5
+
+/** Minimum time (in seconds) that a connection has been idle, that makes
+it candidate for background thread to commit it */
+#define CONN_IDLE_TIME_TO_BK_COMMIT 5
+
 #ifdef UNIV_MEMCACHED_SDI
 static const char SDI_PREFIX[] = "sdi_";
 static const char SDI_CREATE_PREFIX[] = "sdi_create_";
@@ -74,8 +85,8 @@ static bool bk_thd_exited = true;
 
 /** The SDI buffer length for storing list of SDI keys. Example output
 looks like "1:2|2:2|3:4|..". So SDI list of key retrieval has this limit of
-characters from memcached plugin. This is sufficient for testing. */
-const uint32_t SDI_LIST_BUF_MAX_LEN [[maybe_unused]] = 10000;
+characters from memcached plugin. This is sufficent for testing. */
+const uint32_t SDI_LIST_BUF_MAX_LEN MY_ATTRIBUTE((unused)) = 10000;
 
 /** Tells whether all connections need to release MDL locks */
 bool release_mdl_lock = false;
@@ -119,7 +130,8 @@ static inline struct innodb_engine *innodb_handle(
 
 /*******************************************************************/ /**
  Cleanup idle connections if "clear_all" is false, and clean up all
- connections if "clear_all" is true. */
+ connections if "clear_all" is true.
+ @return number of connection cleaned */
 static void innodb_conn_clean_data(
     /*===================*/
     innodb_conn_data_t *conn_data, bool has_lock, bool free_all);
@@ -145,7 +157,7 @@ static ENGINE_ERROR_CODE innodb_arithmetic(
                               or decrement */
     const bool create,        /*!< in: whether to create the key
                               value pair if can't find */
-    const uint64_t delta,     /*!< in: value to add/subtract */
+    const uint64_t delta,     /*!< in: value to add/substract */
     const uint64_t initial,   /*!< in: initial */
     const rel_time_t exptime, /*!< in: expiration time */
     uint64_t *cas,            /*!< out: new cas value */
@@ -153,16 +165,16 @@ static ENGINE_ERROR_CODE innodb_arithmetic(
     uint16_t vbucket);        /*!< in: bucket, used by default
                               engine only */
 
-/**
- Callback function used by Memcached's process_command() function
+/*******************************************************************/ /**
+ Callback functions used by Memcached's process_command() function
  to get the result key/value information
-@param[in]   handle     Engine Handle
-@param[in]   cookie     connection cookie
-@param[in]   item       item in question
-@param[out]  item_info  item info got
-@return true if info fetched */
-static bool innodb_get_item_info(ENGINE_HANDLE *handle, const void *cookie,
-                                 const item *item, item_info *item_info);
+ @return TRUE if info fetched */
+static bool innodb_get_item_info(
+    /*=================*/
+    ENGINE_HANDLE *handle, /*!< in: Engine Handle */
+    const void *cookie,    /*!< in: connection cookie */
+    const item *item,      /*!< in: item in question */
+    item_info *item_info); /*!< out: item info got */
 
 /*******************************************************************/ /**
  Get default Memcached engine handle
@@ -202,7 +214,7 @@ static bool innodb_sdi_get(innodb_conn_data_t *conn_data,
 @param[in,out]	conn_data	innodb connection data
 @param[in,out]	err_ret		error code
 @param[in]	value		memcached value
-@param[in]	val_len		memcached value length
+@param[in]	value_len	memcached value length
 @param[in]	nkey		memcached key length
 @return true if key is SDI key else false */
 static bool innodb_sdi_store(struct innodb_engine *innodb_eng,
@@ -294,22 +306,6 @@ create_instance(
 
   return (ENGINE_SUCCESS);
 }
-static void innodb_close_cursors(innodb_conn_data_t *conn_data) {
-  innodb_cb_cursor_close(conn_data->idx_read_crsr);
-  innodb_cb_cursor_close(conn_data->idx_crsr);
-  innodb_cb_cursor_close(conn_data->read_crsr);
-  innodb_cb_cursor_close(conn_data->crsr);
-}
-static void innodb_commit_and_release_crsr_trx(innodb_conn_data_t *conn_data) {
-  assert(!conn_data->mysql_tbl);
-  innodb_close_cursors(conn_data);
-  innodb_cb_trx_commit(conn_data->crsr_trx);
-  auto err [[maybe_unused]] = ib_cb_trx_release(conn_data->crsr_trx);
-  assert(err == DB_SUCCESS);
-  conn_data->crsr_trx = nullptr;
-}
-
-void innodb_close_mysql_table(innodb_conn_data_t *conn_data);
 
 /*******************************************************************/ /**
  background thread to commit trx.
@@ -319,6 +315,7 @@ static void *innodb_bk_thread(
     void *arg) {
   ENGINE_HANDLE *handle;
   struct innodb_engine *innodb_eng;
+  innodb_conn_data_t *conn_data;
 
   bk_thd_exited = false;
 
@@ -331,8 +328,19 @@ static void *innodb_bk_thread(
   threads, we will "pretend" to be each connection. */
   void *thd = handler_create_thd(innodb_eng->enable_binlog);
 
+  conn_data = UT_LIST_GET_FIRST(innodb_eng->conn_data);
+
   while (!memcached_shutdown) {
-    release_mdl_lock = handler_check_global_read_lock_active();
+    innodb_conn_data_t *next_conn_data;
+    uint64_t time;
+    uint64_t trx_start = 0;
+    uint64_t processed_count = 0;
+
+    if (handler_check_global_read_lock_active()) {
+      release_mdl_lock = true;
+    } else {
+      release_mdl_lock = false;
+    }
 
     /* Do the cleanup every innodb_eng->bk_commit_interval
     seconds. We also check if the plugin is being shutdown
@@ -346,45 +354,122 @@ static void *innodb_bk_thread(
       }
     }
 
-    LOCK_CONN_IF_NOT_LOCKED(false, innodb_eng);
+    time = mci_get_time();
+
+    if (UT_LIST_GET_LEN(innodb_eng->conn_data) == 0) {
+      continue;
+    }
+
+    if (!conn_data) {
+      conn_data = UT_LIST_GET_FIRST(innodb_eng->conn_data);
+    }
+
+    if (conn_data) {
+      next_conn_data = UT_LIST_GET_NEXT(conn_list, conn_data);
+    } else {
+      next_conn_data = NULL;
+    }
 
     /* Set the clean_stale_conn to prevent force clean in
     innodb_conn_clean. */
+    LOCK_CONN_IF_NOT_LOCKED(false, innodb_eng);
     innodb_eng->clean_stale_conn = true;
-    innodb_conn_data_t *next_conn_data = nullptr;
+    UNLOCK_CONN_IF_NOT_LOCKED(false, innodb_eng);
 
-    for (innodb_conn_data_t *conn_data =
-             UT_LIST_GET_FIRST(innodb_eng->conn_data);
-         conn_data; conn_data = next_conn_data) {
-      next_conn_data = UT_LIST_GET_NEXT(conn_list, conn_data);
-      if (conn_data->is_waiting_for_mdl) {
-        continue;
-      }
+    while (conn_data) {
+      if (release_mdl_lock && !conn_data->is_stale) {
+        int err;
 
-      if (LOCK_CURRENT_CONN_TRYLOCK(conn_data) != 0) {
-        continue;
+        if (conn_data->is_waiting_for_mdl) {
+          goto next_item;
+        }
+
+        err = LOCK_CURRENT_CONN_TRYLOCK(conn_data);
+        if (err != 0) {
+          goto next_item;
+        }
+        /* We have got the lock here */
+      } else {
+        LOCK_CURRENT_CONN_IF_NOT_LOCKED(false, conn_data);
       }
 
       if (conn_data->is_stale) {
-        assert(!conn_data->in_use);
+        UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(false, conn_data);
+        LOCK_CONN_IF_NOT_LOCKED(false, innodb_eng);
         UT_LIST_REMOVE(conn_list, innodb_eng->conn_data, conn_data);
-        if (conn_data->thd) {
-          handler_thd_attach(conn_data->thd, nullptr);
-        }
-        innodb_conn_clean_data(conn_data, true, true);
-      } else if (!conn_data->in_use) {
+        UNLOCK_CONN_IF_NOT_LOCKED(false, innodb_eng);
+        innodb_conn_clean_data(conn_data, false, true);
+        goto next_item;
+      }
+
+      if (release_mdl_lock) {
         if (conn_data->thd) {
           handler_thd_attach(conn_data->thd, NULL);
         }
+
+        if (conn_data->in_use) {
+          UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(false, conn_data);
+          goto next_item;
+        }
+
         innodb_reset_conn(conn_data, true, true, innodb_eng->enable_binlog);
-        innodb_close_mysql_table(conn_data);
-        innodb_conn_clean_data(conn_data, true, false);
+        if (conn_data->mysql_tbl) {
+          handler_unlock_table(conn_data->thd, conn_data->mysql_tbl, HDL_READ);
+          conn_data->mysql_tbl = NULL;
+        }
+
+        /*Close the data cursor */
+        if (conn_data->crsr) {
+          innodb_cb_cursor_close(conn_data->crsr);
+          conn_data->crsr = NULL;
+        }
+        if (conn_data->crsr_trx != NULL) {
+          ib_cb_trx_release(conn_data->crsr_trx);
+          conn_data->crsr_trx = NULL;
+        }
+
+        UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(false, conn_data);
+        goto next_item;
+      }
+
+      if (conn_data->crsr_trx) {
+        trx_start = ib_cb_trx_get_start_time(conn_data->crsr_trx);
+      }
+
+      /* Check the trx, if it is qualified for
+      reset and commit */
+      if ((conn_data->n_writes_since_commit > 0 ||
+           conn_data->n_reads_since_commit > 0) &&
+          trx_start && (time - trx_start > CONN_IDLE_TIME_TO_BK_COMMIT) &&
+          !conn_data->in_use) {
+        /* binlog is running, make the thread
+        attach to conn_data->thd for binlog
+        committing */
+        if (thd && conn_data->thd) {
+          handler_thd_attach(conn_data->thd, NULL);
+        }
+
+        innodb_reset_conn(conn_data, true, true, innodb_eng->enable_binlog);
+        processed_count++;
       }
 
       UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(false, conn_data);
+
+    next_item:
+      conn_data = next_conn_data;
+
+      /* Process BK_MAX_PROCESS_COMMIT (5) trx at a time */
+      if (!release_mdl_lock && processed_count > BK_MAX_PROCESS_COMMIT) {
+        break;
+      }
+
+      if (conn_data) {
+        next_conn_data = UT_LIST_GET_NEXT(conn_list, conn_data);
+      }
     }
 
     /* Set the clean_stale_conn back. */
+    LOCK_CONN_IF_NOT_LOCKED(false, innodb_eng);
     innodb_eng->clean_stale_conn = false;
     UNLOCK_CONN_IF_NOT_LOCKED(false, innodb_eng);
   }
@@ -424,7 +509,7 @@ static ENGINE_ERROR_CODE innodb_initialize(
   /* In case you want to make sure that your MTR is robust in case
   of longer plugin initialization, consider adding
      sleep(10);
-  here - I've found lots of errors this way, hopefully all are fixed.*/
+  here - I've found lots of errors this way, hopefuly all are fixed.*/
 
   ENGINE_ERROR_CODE return_status = ENGINE_SUCCESS;
   struct innodb_engine *innodb_eng = innodb_handle(handle);
@@ -468,6 +553,7 @@ static ENGINE_ERROR_CODE innodb_initialize(
   UT_LIST_INIT(innodb_eng->conn_data);
   pthread_mutex_init(&innodb_eng->conn_mutex, NULL);
   pthread_mutex_init(&innodb_eng->cas_mutex, NULL);
+  pthread_mutex_init(&innodb_eng->flush_mutex, NULL);
 
   /* Fetch InnoDB specific settings */
   innodb_eng->meta_info = innodb_config(NULL, 0, &innodb_eng->meta_hash);
@@ -509,31 +595,43 @@ void innodb_close_mysql_table(
 
 #define NUM_MAX_MEM_SLOT 1024
 
-static void innodb_conn_free_used_buffers(innodb_conn_data_t *conn_data) {
-  mem_buf_t *mem_buf = UT_LIST_GET_FIRST(conn_data->mul_used_buf);
-  while (mem_buf) {
-    UT_LIST_REMOVE(mem_list, conn_data->mul_used_buf, mem_buf);
-    free(mem_buf->mem);
-    free(mem_buf);
-    mem_buf = UT_LIST_GET_FIRST(conn_data->mul_used_buf);
-  }
-}
 /*******************************************************************/ /**
  Cleanup idle connections if "clear_all" is false, and clean up all
- connections if "clear_all" is true. */
+ connections if "clear_all" is true.
+ @return number of connection cleaned */
 static void innodb_conn_clean_data(
     /*===================*/
     innodb_conn_data_t *conn_data, bool has_lock, bool free_all) {
+  mem_buf_t *mem_buf;
+
   if (!conn_data) {
     return;
   }
 
   LOCK_CURRENT_CONN_IF_NOT_LOCKED(has_lock, conn_data);
 
-  innodb_close_cursors(conn_data);
+  if (conn_data->idx_crsr) {
+    innodb_cb_cursor_close(conn_data->idx_crsr);
+    conn_data->idx_crsr = NULL;
+  }
+
+  if (conn_data->idx_read_crsr) {
+    innodb_cb_cursor_close(conn_data->idx_read_crsr);
+    conn_data->idx_read_crsr = NULL;
+  }
+
+  if (conn_data->crsr) {
+    innodb_cb_cursor_close(conn_data->crsr);
+    conn_data->crsr = NULL;
+  }
+
+  if (conn_data->read_crsr) {
+    innodb_cb_cursor_close(conn_data->read_crsr);
+    conn_data->read_crsr = NULL;
+  }
 
   if (conn_data->crsr_trx) {
-    ib_err_t err [[maybe_unused]];
+    ib_err_t err MY_ATTRIBUTE((unused));
     innodb_cb_trx_commit(conn_data->crsr_trx);
     err = ib_cb_trx_release(conn_data->crsr_trx);
     assert(err == DB_SUCCESS);
@@ -606,7 +704,13 @@ static void innodb_conn_clean_data(
       conn_data->mul_col_buf_len = 0;
     }
 
-    innodb_conn_free_used_buffers(conn_data);
+    mem_buf = UT_LIST_GET_FIRST(conn_data->mul_used_buf);
+
+    while (mem_buf) {
+      UT_LIST_REMOVE(mem_list, conn_data->mul_used_buf, mem_buf);
+      free(mem_buf->mem);
+      mem_buf = UT_LIST_GET_FIRST(conn_data->mul_used_buf);
+    }
 
     pthread_mutex_destroy(&conn_data->curr_conn_mutex);
     free(conn_data);
@@ -729,6 +833,7 @@ static void innodb_destroy(
 
   pthread_mutex_destroy(&innodb_eng->conn_mutex);
   pthread_mutex_destroy(&innodb_eng->cas_mutex);
+  pthread_mutex_destroy(&innodb_eng->flush_mutex);
 
   if (innodb_eng->default_engine) {
     def_eng->engine.destroy(innodb_eng->default_engine, force);
@@ -809,9 +914,9 @@ static innodb_conn_data_t *innodb_conn_init(
   meta_cfg_info_t *meta_info;
   meta_index_t *meta_index;
   ib_err_t err = DB_SUCCESS;
-  ib_crsr_t crsr = nullptr;
-  ib_crsr_t read_crsr = nullptr;
-  ib_crsr_t idx_crsr = nullptr;
+  ib_crsr_t crsr;
+  ib_crsr_t read_crsr;
+  ib_crsr_t idx_crsr;
   bool trx_updated = false;
 
   /* Get this connection's conn_data */
@@ -856,20 +961,12 @@ static innodb_conn_data_t *innodb_conn_init(
 
     /* FIX_ME: to make this dynamic extensible */
     conn_data->row_buf = (void **)malloc(NUM_MAX_MEM_SLOT * sizeof(void *));
-    if (conn_data->row_buf == NULL) {
-      UNLOCK_CONN_IF_NOT_LOCKED(has_lock, engine);
-      free(conn_data->result);
-      free(conn_data);
-      conn_data = NULL;
-      return (NULL);
-    }
     memset(conn_data->row_buf, 0, NUM_MAX_MEM_SLOT * sizeof(void *));
 
-    conn_data->row_buf[0] = (void *)malloc(REC_BUF_SLOT_SIZE);
+    conn_data->row_buf[0] = (void *)malloc(16384);
 
     if (conn_data->row_buf[0] == NULL) {
       UNLOCK_CONN_IF_NOT_LOCKED(has_lock, engine);
-      free(conn_data->row_buf);
       free(conn_data->result);
       free(conn_data);
       conn_data = NULL;
@@ -881,7 +978,6 @@ static innodb_conn_data_t *innodb_conn_init(
     conn_data->cmd_buf = malloc(1024);
     if (!conn_data->cmd_buf) {
       UNLOCK_CONN_IF_NOT_LOCKED(has_lock, engine);
-      free(conn_data->row_buf[0]);
       free(conn_data->row_buf);
       free(conn_data->result);
       free(conn_data);
@@ -894,6 +990,7 @@ static innodb_conn_data_t *innodb_conn_init(
 #ifdef UNIV_MEMCACHED_SDI
     conn_data->sdi_buf = NULL;
 #endif /* UNIV_MEMCACHED_SDI */
+    conn_data->is_flushing = false;
 
     conn_data->conn_cookie = (void *)cookie;
 
@@ -915,9 +1012,6 @@ have_conn:
 
   assert(engine->conn_data.count > 0);
 
-  assert(conn_data->thd);
-  handler_thd_attach(conn_data->thd, nullptr);
-
   if (conn_option == CONN_MODE_NONE) {
     return (conn_data);
   }
@@ -930,6 +1024,13 @@ have_conn:
 
   LOCK_CURRENT_CONN_IF_NOT_LOCKED(has_lock, conn_data);
 
+  /* If flush is running, then wait for it complete. */
+  if (conn_data->is_flushing) {
+    /* Request flush_mutex for waiting for flush completed. */
+    pthread_mutex_lock(&engine->flush_mutex);
+    pthread_mutex_unlock(&engine->flush_mutex);
+  }
+
   /* This special case added to facilitate unlocking
      of MDL lock during FLUSH TABLE WITH READ LOCK */
   if (engine && release_mdl_lock &&
@@ -967,7 +1068,12 @@ have_conn:
                            &conn_data->idx_crsr, lock_mode);
 
     if (err != DB_SUCCESS) {
-      innodb_commit_and_release_crsr_trx(conn_data);
+      innodb_cb_cursor_close(conn_data->crsr);
+      conn_data->crsr = NULL;
+      innodb_cb_trx_commit(conn_data->crsr_trx);
+      err = ib_cb_trx_release(conn_data->crsr_trx);
+      assert(err == DB_SUCCESS);
+      conn_data->crsr_trx = NULL;
       conn_data->in_use = false;
       UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(has_lock, conn_data);
       return (NULL);
@@ -999,7 +1105,12 @@ have_conn:
                              &conn_data->idx_crsr, lock_mode);
 
       if (err != DB_SUCCESS) {
-        innodb_commit_and_release_crsr_trx(conn_data);
+        innodb_cb_cursor_close(conn_data->crsr);
+        conn_data->crsr = NULL;
+        innodb_cb_trx_commit(conn_data->crsr_trx);
+        err = ib_cb_trx_release(conn_data->crsr_trx);
+        assert(err == DB_SUCCESS);
+        conn_data->crsr_trx = NULL;
         conn_data->in_use = false;
 
         UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(has_lock, conn_data);
@@ -1018,7 +1129,12 @@ have_conn:
       err = innodb_cb_cursor_lock(engine, crsr, lock_mode);
 
       if (err != DB_SUCCESS) {
-        innodb_commit_and_release_crsr_trx(conn_data);
+        innodb_cb_cursor_close(conn_data->crsr);
+        conn_data->crsr = NULL;
+        innodb_cb_trx_commit(conn_data->crsr_trx);
+        err = ib_cb_trx_release(conn_data->crsr_trx);
+        assert(err == DB_SUCCESS);
+        conn_data->crsr_trx = NULL;
         conn_data->in_use = false;
         UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(has_lock, conn_data);
         return (NULL);
@@ -1040,7 +1156,12 @@ have_conn:
       err = innodb_cb_cursor_lock(engine, crsr, lock_mode);
 
       if (err != DB_SUCCESS) {
-        innodb_commit_and_release_crsr_trx(conn_data);
+        innodb_cb_cursor_close(conn_data->crsr);
+        conn_data->crsr = NULL;
+        innodb_cb_trx_commit(conn_data->crsr_trx);
+        err = ib_cb_trx_release(conn_data->crsr_trx);
+        assert(err == DB_SUCCESS);
+        conn_data->crsr_trx = NULL;
         conn_data->in_use = false;
         UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(has_lock, conn_data);
         return (NULL);
@@ -1057,19 +1178,10 @@ have_conn:
       }
     }
   } else {
-    /* This is a read operation, but depending on isolation level, it might
-    require an InnoDB shared table intention lock, and/or a shared record lock.
-    We need to tell in advance if our trx is "read_write" and "autocommit".
-    These concepts are mapped through API and InnoDB's trx_start_low() in a
-    complicated way. In particular the auto_commit maps to trx->api_auto_commit
-    and if trx->api_auto_commit is true then trx_start_low does not set
-    trx->will_lock (it stays 0). When we later request a lock, we violate
-      !trx_is_autocommit_non_locking((t))
-    assert in lock sys. So, if we plan to take locks, we should not set
-    auto_commit, as InnoDB would interpret it as "autocommit non locking".*/
-    const bool will_lock = lock_mode != IB_LOCK_NONE;
-    const bool auto_commit = (!will_lock && engine->read_batch_size == 1 &&
-                              !(engine->cfg_status & IB_CFG_DISABLE_ROWLOCK));
+    bool auto_commit = (engine->read_batch_size == 1 &&
+                        !(engine->cfg_status & IB_CFG_DISABLE_ROWLOCK))
+                           ? true
+                           : false;
     assert(conn_option == CONN_MODE_READ);
 
     if (!read_crsr) {
@@ -1091,7 +1203,12 @@ have_conn:
                              lock_mode);
 
       if (err != DB_SUCCESS) {
-        innodb_commit_and_release_crsr_trx(conn_data);
+        innodb_cb_cursor_close(conn_data->read_crsr);
+        innodb_cb_trx_commit(conn_data->crsr_trx);
+        err = ib_cb_trx_release(conn_data->crsr_trx);
+        assert(err == DB_SUCCESS);
+        conn_data->crsr_trx = NULL;
+        conn_data->read_crsr = NULL;
         conn_data->in_use = false;
         UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(has_lock, conn_data);
 
@@ -1115,7 +1232,12 @@ have_conn:
       err = innodb_cb_cursor_lock(engine, conn_data->read_crsr, lock_mode);
 
       if (err != DB_SUCCESS) {
-        innodb_commit_and_release_crsr_trx(conn_data);
+        innodb_cb_cursor_close(conn_data->read_crsr);
+        innodb_cb_trx_commit(conn_data->crsr_trx);
+        err = ib_cb_trx_release(conn_data->crsr_trx);
+        assert(err == DB_SUCCESS);
+        conn_data->crsr_trx = NULL;
+        conn_data->read_crsr = NULL;
         conn_data->in_use = false;
         UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(has_lock, conn_data);
 
@@ -1139,7 +1261,12 @@ have_conn:
       err = innodb_cb_cursor_lock(engine, conn_data->read_crsr, lock_mode);
 
       if (err != DB_SUCCESS) {
-        innodb_commit_and_release_crsr_trx(conn_data);
+        innodb_cb_cursor_close(conn_data->read_crsr);
+        innodb_cb_trx_commit(conn_data->crsr_trx);
+        err = ib_cb_trx_release(conn_data->crsr_trx);
+        assert(err == DB_SUCCESS);
+        conn_data->crsr_trx = NULL;
+        conn_data->read_crsr = NULL;
         conn_data->in_use = false;
         UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(has_lock, conn_data);
 
@@ -1424,9 +1551,6 @@ static ENGINE_ERROR_CODE innodb_switch_mapping(
 
   conn_data = innodb_conn_init(innodb_eng, cookie, CONN_MODE_NONE,
                                ib_lck_mode_t(0), false, new_meta_info);
-  if (conn_data == nullptr) {
-    return (ENGINE_TMPFAIL);
-  }
 
   assert(conn_data->conn_meta == new_meta_info);
 
@@ -1511,15 +1635,19 @@ static void innodb_clean_engine(
   UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(false, conn_data);
 }
 
-/** Release the resources used to store query response item
-@param[in]  handle  Engine handle
-@param[in]  cookie  connection cookie
-@param[in]  item    item to free
-*/
-static void innodb_release(ENGINE_HANDLE *handle, const void *cookie,
-                           item *item) {
+/*******************************************************************/ /**
+ Release the connection, free resource allocated in innodb_allocate */
+static void innodb_release(
+    /*===========*/
+    ENGINE_HANDLE *handle, /*!< in: Engine handle */
+    const void *cookie __attribute__((unused)),
+    /*!< in: connection cookie */
+    item *item __attribute__((unused)))
+/*!< in: item to free */
+{
   struct innodb_engine *innodb_eng = innodb_handle(handle);
   innodb_conn_data_t *conn_data;
+  mem_buf_t *mem_buf;
 
   conn_data =
       (innodb_conn_data_t *)innodb_eng->server.cookie->get_engine_specific(
@@ -1536,7 +1664,13 @@ static void innodb_release(ENGINE_HANDLE *handle, const void *cookie,
   conn_data->multi_get = false;
   conn_data->mul_col_buf_used = 0;
 
-  innodb_conn_free_used_buffers(conn_data);
+  mem_buf = UT_LIST_GET_FIRST(conn_data->mul_used_buf);
+
+  while (mem_buf) {
+    UT_LIST_REMOVE(mem_list, conn_data->mul_used_buf, mem_buf);
+    free(mem_buf->mem);
+    mem_buf = UT_LIST_GET_FIRST(conn_data->mul_used_buf);
+  }
 
   /* If item's memory comes from Memcached default engine, release it
   through Memcached APIs */
@@ -1603,16 +1737,13 @@ static int convert_to_char(
       int8_t int_val = *(int8_t *)value;
       snprintf(buf, buf_len, "%" PRIi8, int_val);
     }
-  } else {
-    assert(!"invalid byte length of integer");
-    return 0;
   }
 
   return (strlen(buf));
 }
 
 /*******************************************************************/ /**
- Free value associated with key */
+ Free value assocaited with key */
 static void innodb_free_item(
     /*=====================*/
     void *item) /*!< in: Item to be freed */
@@ -1631,24 +1762,6 @@ static void innodb_free_item(
     result->col_value[MCI_COL_VALUE].allocated = false;
   }
 }
-static void innodb_ensure_mul_col_buf_capacity(innodb_conn_data_t *conn_data,
-                                               size_t total_len) {
-  if (conn_data->mul_col_buf_len < total_len + conn_data->mul_col_buf_used) {
-    /* Need to keep the old result buffer, since its
-    point is already registered with memcached output
-    buffer. These result buffers will be release
-    once results are all reported */
-    if (conn_data->mul_col_buf) {
-      mem_buf_t *new_temp = (mem_buf_t *)malloc(sizeof(mem_buf_t));
-      new_temp->mem = conn_data->mul_col_buf;
-      UT_LIST_ADD_LAST(mem_list, conn_data->mul_used_buf, new_temp);
-    }
-
-    conn_data->mul_col_buf = (char *)malloc(total_len);
-    conn_data->mul_col_buf_len = total_len;
-    conn_data->mul_col_buf_used = 0;
-  }
-}
 /*******************************************************************/ /**
  Support memcached "GET" command, fetch the value according to key
  @return ENGINE_SUCCESS if successfully, otherwise error code */
@@ -1662,7 +1775,7 @@ static ENGINE_ERROR_CODE innodb_get(
     uint16_t next_get)     /*!< in: has more item to get */
 {
   struct innodb_engine *innodb_eng = innodb_handle(handle);
-  ib_crsr_t crsr = nullptr;
+  ib_crsr_t crsr;
   ib_err_t err = DB_SUCCESS;
   mci_item_t *result = NULL;
   ENGINE_ERROR_CODE err_ret = ENGINE_SUCCESS;
@@ -1844,7 +1957,7 @@ static ENGINE_ERROR_CODE innodb_get(
   if (is_range_srch && err != DB_END_OF_INDEX) {
     /* we set it only after the first search. This is used to
     tell innodb_api_search() if it is the first search, which
-    might need to do the initial position of cursor */
+    might need to do the intial position of cursor */
     conn_data->range = true;
   }
 
@@ -1880,26 +1993,12 @@ search_done:
     snprintf(table_name, sizeof(table_name), "%s/%s", dbname, name);
 #endif
 
-    if (conn_data->row_buf_used + strlen(table_name) >= REC_BUF_SLOT_SIZE) {
-      conn_data->row_buf_slot++;
-
-      /* Limit the record buffer size to 16 MB */
-      if (conn_data->row_buf_slot >= 1024) {
-        err_ret = ENGINE_KEY_ENOENT;
-        goto func_exit;
-      }
-
-      if (conn_data->row_buf[conn_data->row_buf_slot] == nullptr) {
-        conn_data->row_buf[conn_data->row_buf_slot] = malloc(REC_BUF_SLOT_SIZE);
-      }
-
-      conn_data->row_buf_used = 0;
-    }
-
+    assert(!conn_data->result_in_use);
     conn_data->result_in_use = true;
     result = (mci_item_t *)(conn_data->result);
 
     memset(result, 0, sizeof(*result));
+
     memcpy((char *)(conn_data->row_buf[conn_data->row_buf_slot]) +
                conn_data->row_buf_used,
            table_name, strlen(table_name));
@@ -1908,9 +2007,6 @@ search_done:
         ((char *)conn_data->row_buf[conn_data->row_buf_slot]) +
         conn_data->row_buf_used;
     result->col_value[MCI_COL_VALUE].value_len = strlen(table_name);
-    conn_data->row_buf_used += result->col_value[MCI_COL_VALUE].value_len;
-    result->col_value[MCI_COL_VALUE].is_str = true;
-    result->col_value[MCI_COL_VALUE].is_valid = true;
   }
 
   if (!conn_data->range) {
@@ -1934,9 +2030,11 @@ search_done:
   if (result->extra_col_value) {
     int i;
     char *c_value;
-    char *value_end [[maybe_unused]];
+    char *value_end MY_ATTRIBUTE((unused));
     unsigned int total_len = 0;
     char int_buf[MAX_INT_CHAR_LEN];
+    ib_ulint_t new_len;
+
     GET_OPTION(meta_info, OPTION_ID_COL_SEP, option_delimiter, option_length);
 
     assert(option_length > 0 && option_delimiter);
@@ -1965,13 +2063,26 @@ search_done:
 
     /* No need to add the last separator */
     total_len -= option_length;
+    new_len = total_len + conn_data->mul_col_buf_used;
 
-    innodb_ensure_mul_col_buf_capacity(conn_data, total_len);
+    if (new_len >= conn_data->mul_col_buf_len) {
+      /* Need to keep the old result buffer, since its
+      point is already registered with memcached output
+      buffer. These result buffers will be release
+      once results are all reported */
+      if (conn_data->mul_col_buf) {
+        mem_buf_t *new_temp = (mem_buf_t *)malloc(sizeof(mem_buf_t));
+        new_temp->mem = conn_data->mul_col_buf;
+        UT_LIST_ADD_LAST(mem_list, conn_data->mul_used_buf, new_temp);
+      }
+
+      conn_data->mul_col_buf = (char *)malloc(new_len + 1);
+      conn_data->mul_col_buf_len = new_len + 1;
+      conn_data->mul_col_buf_used = 0;
+    }
 
     c_value = &conn_data->mul_col_buf[conn_data->mul_col_buf_used];
-    assert(conn_data->mul_col_buf_used + total_len <=
-           conn_data->mul_col_buf_len);
-    value_end = c_value + total_len;
+    value_end = &conn_data->mul_col_buf[new_len];
 
     for (i = 0; i < result->n_extra_col; i++) {
       mci_column_t *col_value;
@@ -1980,7 +2091,7 @@ search_done:
 
       if (col_value->value_len != 0) {
         if (!col_value->is_str) {
-          uint64_t int_len;
+          ib_ulint_t int_len;
           memset(int_buf, 0, sizeof int_buf);
 
           int_len =
@@ -1988,7 +2099,6 @@ search_done:
                               col_value->value_len, col_value->is_unsigned);
 
           assert(int_len <= conn_data->mul_col_buf_len);
-          assert(c_value + int_len <= value_end);
 
           memcpy(c_value, int_buf, int_len);
           c_value += int_len;
@@ -2007,44 +2117,39 @@ search_done:
 
       if (col_value->allocated) {
         free(col_value->value_str);
-        col_value->value_str = nullptr;
-        col_value->allocated = false;
-        col_value->value_len = 0;
-        col_value->is_str = false;
-        col_value->is_valid = false;
       }
     }
-    assert(c_value == value_end);
 
     result->col_value[MCI_COL_VALUE].value_str =
         &conn_data->mul_col_buf[conn_data->mul_col_buf_used];
     result->col_value[MCI_COL_VALUE].value_len = total_len;
-    result->col_value[MCI_COL_VALUE].is_str = true;
-    result->col_value[MCI_COL_VALUE].is_valid = true;
     conn_data->mul_col_buf_used += total_len;
+    ((char *)result->col_value[MCI_COL_VALUE].value_str)[total_len] = 0;
 
     free(result->extra_col_value);
-    result->extra_col_value = nullptr;
   } else if (!result->col_value[MCI_COL_VALUE].is_str &&
              result->col_value[MCI_COL_VALUE].value_len != 0) {
     unsigned int int_len;
-    char int_buf[MAX_INT_CHAR_LEN] = {};
+    char int_buf[MAX_INT_CHAR_LEN];
 
     int_len = convert_to_char(int_buf, sizeof int_buf,
                               &result->col_value[MCI_COL_VALUE].value_int,
                               result->col_value[MCI_COL_VALUE].value_len,
                               result->col_value[MCI_COL_VALUE].is_unsigned);
 
-    assert(int_len > 0);
-    innodb_ensure_mul_col_buf_capacity(conn_data, int_len);
-    result->col_value[MCI_COL_VALUE].value_str =
-        conn_data->mul_col_buf + conn_data->mul_col_buf_used;
+    if (int_len > conn_data->mul_col_buf_len) {
+      if (conn_data->mul_col_buf) {
+        free(conn_data->mul_col_buf);
+      }
+
+      conn_data->mul_col_buf = (char *)malloc(int_len + 1);
+      conn_data->mul_col_buf_len = int_len;
+    }
+
+    if (int_len > 0) memcpy(conn_data->mul_col_buf, int_buf, int_len);
+    result->col_value[MCI_COL_VALUE].value_str = conn_data->mul_col_buf;
+
     result->col_value[MCI_COL_VALUE].value_len = int_len;
-    conn_data->mul_col_buf_used += int_len;
-    memcpy(result->col_value[MCI_COL_VALUE].value_str, int_buf,
-           result->col_value[MCI_COL_VALUE].value_len);
-    result->col_value[MCI_COL_VALUE].is_str = true;
-    result->col_value[MCI_COL_VALUE].is_valid = true;
   }
 
   *item = result;
@@ -2091,7 +2196,8 @@ static ENGINE_ERROR_CODE innodb_get_stats(
 }
 
 /*******************************************************************/ /**
- reset statistics */
+ reset statistics
+ @return ENGINE_SUCCESS if successfully, otherwise error code */
 static void innodb_reset_stats(
     /*===============*/
     ENGINE_HANDLE *handle, /*!< in: Engine Handle */
@@ -2194,7 +2300,7 @@ static ENGINE_ERROR_CODE innodb_arithmetic(
                               or decrement */
     const bool create,        /*!< in: whether to create the key
                               value pair if can't find */
-    const uint64_t delta,     /*!< in: value to add/subtract */
+    const uint64_t delta,     /*!< in: value to add/substract */
     const uint64_t initial,   /*!< in: initial */
     const rel_time_t exptime, /*!< in: expiration time */
     uint64_t *cas,            /*!< out: new cas value */
@@ -2241,6 +2347,60 @@ static ENGINE_ERROR_CODE innodb_arithmetic(
   return (err_ret);
 }
 
+/*******************************************************************/ /**
+ Cleanup idle connections if "clear_all" is false, and clean up all
+ connections if "clear_all" is true.
+ @return number of connection cleaned */
+static bool innodb_flush_sync_conn(
+    /*===================*/
+    innodb_engine_t *engine, /*!< in/out: InnoDB memcached
+                             engine */
+    const void *cookie,      /*!< in: connection cookie */
+    bool flush_flag)         /*!< in: flush is running or not */
+{
+  innodb_conn_data_t *conn_data = NULL;
+  innodb_conn_data_t *curr_conn_data;
+  bool ret = true;
+
+  curr_conn_data =
+      (innodb_conn_data_t *)engine->server.cookie->get_engine_specific(cookie);
+  assert(curr_conn_data);
+
+  conn_data = UT_LIST_GET_FIRST(engine->conn_data);
+
+  while (conn_data) {
+    if (conn_data != curr_conn_data && (!conn_data->is_stale)) {
+      if (conn_data->thd) {
+        handler_thd_attach(conn_data->thd, NULL);
+      }
+      LOCK_CURRENT_CONN_IF_NOT_LOCKED(false, conn_data);
+      if (flush_flag == false) {
+        conn_data->is_flushing = flush_flag;
+        UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(false, conn_data);
+        conn_data = UT_LIST_GET_NEXT(conn_list, conn_data);
+        continue;
+      }
+      if (!conn_data->in_use) {
+        /* Set flushing flag to conn_data for preventing
+        it is get by other request.  */
+        conn_data->is_flushing = flush_flag;
+        UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(false, conn_data);
+      } else {
+        ret = false;
+        UNLOCK_CURRENT_CONN_IF_NOT_LOCKED(false, conn_data);
+        break;
+      }
+    }
+    conn_data = UT_LIST_GET_NEXT(conn_list, conn_data);
+  }
+
+  if (curr_conn_data->thd) {
+    handler_thd_attach(curr_conn_data->thd, NULL);
+  }
+
+  return (ret);
+}
+
 /*******************************************************************/ /**
  Support memcached "FLUSH_ALL" command, clean up storage (trunate InnoDB Table)
  @return ENGINE_SUCCESS if successfully, otherwise error code */
@@ -2272,23 +2432,35 @@ static ENGINE_ERROR_CODE innodb_flush(
     }
   }
 
+  /* Lock the whole engine, so no other connection can start
+  new opeartion */
+  pthread_mutex_lock(&innodb_eng->conn_mutex);
+
+  /* Lock the flush_mutex for blocking other DMLs. */
+  pthread_mutex_lock(&innodb_eng->flush_mutex);
+
   conn_data =
       (innodb_conn_data_t *)innodb_eng->server.cookie->get_engine_specific(
           cookie);
 
   if (conn_data) {
-    /* Commit any work on this connection before waiting for locks.
-    This is to avoid a deadlock, in which another thread also doing flush_all
-    will wait for a lock we hold on our conn_data */
+    /* Commit any work on this connection */
     innodb_api_cursor_reset(innodb_eng, conn_data, CONN_OP_FLUSH, true);
-    innodb_conn_clean_data(conn_data, false, false);
   }
 
   conn_data = innodb_conn_init(innodb_eng, cookie, CONN_MODE_WRITE,
-                               IB_LOCK_TABLE_X, false, NULL);
+                               IB_LOCK_TABLE_X, true, NULL);
 
   if (!conn_data) {
-    /* TBD: why in the past we've returned ENGINE_SUCCESS in this case? */
+    pthread_mutex_unlock(&innodb_eng->flush_mutex);
+    pthread_mutex_unlock(&innodb_eng->conn_mutex);
+    return (ENGINE_SUCCESS);
+  }
+
+  if (!innodb_flush_sync_conn(innodb_eng, cookie, true)) {
+    pthread_mutex_unlock(&innodb_eng->flush_mutex);
+    pthread_mutex_unlock(&innodb_eng->conn_mutex);
+    innodb_flush_sync_conn(innodb_eng, cookie, false);
     return (ENGINE_TMPFAIL);
   }
 
@@ -2301,6 +2473,11 @@ static ENGINE_ERROR_CODE innodb_flush(
   innodb_api_cursor_reset(innodb_eng, conn_data, CONN_OP_FLUSH, true);
   innodb_conn_clean_data(conn_data, false, false);
 
+  pthread_mutex_unlock(&innodb_eng->flush_mutex);
+  pthread_mutex_unlock(&innodb_eng->conn_mutex);
+
+  innodb_flush_sync_conn(innodb_eng, cookie, false);
+
   return ((ib_err == DB_SUCCESS) ? ENGINE_SUCCESS : ENGINE_TMPFAIL);
 }
 
@@ -2321,8 +2498,19 @@ static ENGINE_ERROR_CODE innodb_unknown_command(
                                           request, response));
 }
 
-static bool innodb_get_item_info(ENGINE_HANDLE *handle, const void *cookie,
-                                 const item *item, item_info *item_info) {
+/*******************************************************************/ /**
+ Callback functions used by Memcached's process_command() function
+ to get the result key/value information
+ @return true if info fetched */
+static bool innodb_get_item_info(
+    /*=================*/
+    ENGINE_HANDLE *handle __attribute__((unused)),
+    /*!< in: Engine Handle */
+    const void *cookie __attribute__((unused)),
+    /*!< in: connection cookie */
+    const item *item,     /*!< in: item in question */
+    item_info *item_info) /*!< out: item info got */
+{
   struct innodb_engine *innodb_eng = innodb_handle(handle);
   innodb_conn_data_t *conn_data;
 
@@ -2412,7 +2600,7 @@ static bool innodb_sdi_remove(struct innodb_engine *innodb_eng,
   }
 
   ib_trx_t trx = conn_data->crsr_trx;
-  ib_crsr_t crsr = nullptr;
+  ib_crsr_t crsr;
   /* +2 for the '/' and trailing '\0' */
   char table_name[MAX_TABLE_NAME_LEN + MAX_DATABASE_NAME_LEN + 2];
   char *name;
@@ -2470,7 +2658,7 @@ static bool innodb_sdi_get(innodb_conn_data_t *conn_data,
   mci_item_t *result = (mci_item_t *)conn_data->result;
 
   ib_trx_t trx = conn_data->crsr_trx;
-  ib_crsr_t crsr = nullptr;
+  ib_crsr_t crsr;
 
   /* +2 for the '/' and trailing '\0' */
   char table_name[MAX_TABLE_NAME_LEN + MAX_DATABASE_NAME_LEN + 2];
@@ -2529,7 +2717,7 @@ static bool innodb_sdi_get(innodb_conn_data_t *conn_data,
   } else {
     /* Allocate memory of 64 KB, assuming SDI will fit into
     it. If retrieval fails, we will get actual length of SDI.
-    We retry after allocating the required memory */
+    We retry afer allocating the required memory */
     const uint32_t mem_size = 64 * 1024;
     void *new_mem = realloc(conn_data->sdi_buf, mem_size);
 
@@ -2610,7 +2798,7 @@ static bool innodb_sdi_store(struct innodb_engine *innodb_eng,
   }
 
   ib_trx_t trx = conn_data->crsr_trx;
-  ib_crsr_t crsr = nullptr;
+  ib_crsr_t crsr;
 
   /* +2 for the '/' and trailing '\0' */
   char table_name[MAX_TABLE_NAME_LEN + MAX_DATABASE_NAME_LEN + 2];
