@@ -1,6 +1,5 @@
 /* Copyright (C) 2007 Google Inc.
-   Copyright (c) 2008, 2023, Oracle and/or its affiliates.
-   Use is subject to license terms
+   Copyright (c) 2008, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -20,19 +19,17 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-
-#include "semisync.h"
+#include "plugin/semisync/semisync.h"
 
 const unsigned char ReplSemiSyncBase::kPacketMagicNum = 0xef;
 const unsigned char ReplSemiSyncBase::kPacketFlagSync = 0x01;
 
-
-const unsigned long Trace::kTraceGeneral  = 0x0001;
-const unsigned long Trace::kTraceDetail   = 0x0010;
-const unsigned long Trace::kTraceNetWait  = 0x0020;
+const unsigned long Trace::kTraceGeneral = 0x0001;
+const unsigned long Trace::kTraceDetail = 0x0010;
+const unsigned long Trace::kTraceNetWait = 0x0020;
 const unsigned long Trace::kTraceFunction = 0x0040;
 
-const unsigned char  ReplSemiSyncBase::kSyncHeader[2] =
-  {ReplSemiSyncBase::kPacketMagicNum, 0};
+const unsigned char ReplSemiSyncBase::kSyncHeader[2] = {
+    ReplSemiSyncBase::kPacketMagicNum, 0};
