@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -132,7 +131,7 @@ struct st_plugin_ctx {
   st_plugin_ctx() { reset(); }
 
   void reset() {
-    resultcs = nullptr;
+    resultcs = NULL;
     server_status = 0;
     current_col = 0;
     warn_count = 0;
@@ -170,7 +169,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
-  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->m_coll_name));
+  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
   pctx->num_cols = num_cols;
   pctx->resultcs = resultcs;
   pctx->current_col = 0;
@@ -292,7 +291,7 @@ static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
 static const char *test_decimal_as_string(char *buff, const decimal_t *val,
                                           int *length) {
   if (!val) return "NULL";
-  (void)decimal2string(val, buff, length);
+  (void)decimal2string(val, buff, length, 0, 0, 0);
   return buff;
 }
 
@@ -479,7 +478,6 @@ const struct st_command_service_cbs sql_cbs = {
     sql_handle_ok,
     sql_handle_error,
     sql_shutdown,
-    nullptr,
 };
 
 enum cs_text_or_binary txt_or_bin;
@@ -764,7 +762,7 @@ static void handle_error(struct st_plugin_ctx *pctx) {
 }
 
 static void exec_test_cmd(MYSQL_SESSION session, const char *test_cmd,
-                          void *p [[maybe_unused]], void *ctx,
+                          void *p MY_ATTRIBUTE((unused)), void *ctx,
                           enum cs_text_or_binary text_or_binary) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
@@ -772,12 +770,11 @@ static void exec_test_cmd(MYSQL_SESSION session, const char *test_cmd,
 
   WRITE_VAL("%s\n", test_cmd);
   pctx->reset();
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = test_cmd;
   cmd.com_query.length = strlen(cmd.com_query.query);
   int fail = command_service_run_command(session, COM_QUERY, &cmd,
-                                         &my_charset_utf8mb3_general_ci,
-                                         &sql_cbs, text_or_binary, ctx);
+                                         &my_charset_utf8_general_ci, &sql_cbs,
+                                         text_or_binary, ctx);
   if (fail)
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG,
                     "test_sql_all_col_types - ret code : %d", fail);
@@ -926,7 +923,7 @@ static void *test_sql_threaded_wrapper(void *param) {
   srv_session_deinit_thread();
 
   context->thread_finished = true;
-  return nullptr;
+  return NULL;
 }
 
 static void create_log_file(const char *log_name) {
@@ -955,7 +952,7 @@ static void test_in_spawned_thread(void *p, void (*test_function)(void *)) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG,
                  "Could not create test session thread");
   else
-    my_thread_join(&context.thread, nullptr);
+    my_thread_join(&context.thread, NULL);
 }
 
 static int test_sql_service_plugin_init(void *p) {
@@ -979,7 +976,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
+static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
@@ -997,15 +994,15 @@ mysql_declare_plugin(test_daemon){
     MYSQL_DAEMON_PLUGIN,
     &test_sql_service_plugin,
     "test_sql_all_col_types",
-    PLUGIN_AUTHOR_ORACLE,
+    "Horst Hunger, Andrey Hristov",
     "Test sql all column types",
     PLUGIN_LICENSE_GPL,
     test_sql_service_plugin_init,   /* Plugin Init      */
-    nullptr,                        /* Plugin Check uninstall    */
+    NULL,                           /* Plugin Check uninstall    */
     test_sql_service_plugin_deinit, /* Plugin Deinit    */
     0x0100,                         /* 1.0              */
-    nullptr,                        /* status variables */
-    nullptr,                        /* system variables */
-    nullptr,                        /* config options   */
+    NULL,                           /* status variables */
+    NULL,                           /* system variables */
+    NULL,                           /* config options   */
     0,                              /* flags            */
 } mysql_declare_plugin_end;
