@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,7 +40,7 @@
 #include "template_utils.h"
 #include "thr_cond.h"
 
-static constexpr int STRING_BUFFER = 1024 * 4;
+#define STRING_BUFFER 1024
 
 static const char *sep =
     "======================================================\n";
@@ -129,7 +128,7 @@ struct Callback_data {
     affected_rows = 0;
     last_insert_id = 0;
     shutdown = 0;
-    shutdown_called = false;
+    shutdown_called = 0;
   }
 };
 
@@ -146,7 +145,7 @@ struct st_send_field_n {
   enum_field_types type;
 };
 
-const CHARSET_INFO *sql_resultcs = nullptr;
+const CHARSET_INFO *sql_resultcs = NULL;
 uint sql_num_meta_rows = 0;
 uint sql_num_rows = 0;
 uint col_count = 0;
@@ -161,7 +160,7 @@ static int sql_start_result_metadata(void *, uint num_cols, uint,
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
-  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->m_coll_name));
+  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
   row_count = 0;
   sql_num_cols = num_cols;
   sql_resultcs = resultcs;
@@ -326,17 +325,16 @@ const struct st_command_service_cbs sql_cbs = {
     sql_handle_ok,
     sql_handle_error,
     sql_shutdown,
-    nullptr,
 };
 
 /****************************************************************************************/
 
-static void test_com_query(void *p [[maybe_unused]]) {
+static void test_com_query(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
 
   /* Session declarations */
   MYSQL_SESSION st_session;
-  void *plugin_ctx = nullptr;
+  void *plugin_ctx = NULL;
   bool session_ret = false;
   bool fail = false;
   COM_DATA cmd;
@@ -355,12 +353,11 @@ static void test_com_query(void *p [[maybe_unused]]) {
       "-----------------------------------------------------------------\n");
   memset(&sql_str_value, 0, 64 * 64 * 256 * sizeof(char));
   memset(&sql_str_len, 0, 64 * 64 * sizeof(size_t));
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = "SELECT id,info FROM information_schema.processlist";
   cmd.com_query.length = strlen(cmd.com_query.query);
   WRITE_VAL("%s\n", cmd.com_query.query);
   fail = command_service_run_command(st_session, COM_QUERY, &cmd,
-                                     &my_charset_utf8mb3_general_ci, &sql_cbs,
+                                     &my_charset_utf8_general_ci, &sql_cbs,
                                      CS_TEXT_REPRESENTATION, &cbd);
   if (fail)
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "sql_simple ret code: %d\n",
@@ -369,7 +366,7 @@ static void test_com_query(void *p [[maybe_unused]]) {
     /* get values */
     WRITE_STR(
         "-----------------------------------------------------------------\n");
-    WRITE_VAL("%s\t%s\n", sql_field[0][0].col_name, sql_field[0][1].col_name);
+    WRITE_VAL("%s\t\%s\n", sql_field[0][0].col_name, sql_field[0][1].col_name);
     for (uint row = 0; row < sql_num_rows; row++) {
       for (uint col = 0; col < sql_num_cols; col++) {
         WRITE_VAL("%s\n", sql_str_value[col][row]);
@@ -384,7 +381,7 @@ static void test_com_query(void *p [[maybe_unused]]) {
     } else {
       WRITE_VAL("server status: %d\n", cbd.server_status);
       WRITE_VAL("warn count: %d\n", cbd.warn_count);
-      //           WRITE_VAL("message: %s\n",msg);
+      //           WRITE_VAL("messsage: %s\n",msg);
     }
   }
 
@@ -393,7 +390,6 @@ static void test_com_query(void *p [[maybe_unused]]) {
       "-----------------------------------------------------------------\n");
   memset(&sql_str_value, 0, 64 * 64 * 256 * sizeof(char));
   memset(&sql_str_len, 0, 64 * 64 * sizeof(size_t));
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query =
       "SELECT * FROM performance_schema.global_variables WHERE variable_name "
       "LIKE 'INNODB_READ_IO_THREADS'";
@@ -401,7 +397,7 @@ static void test_com_query(void *p [[maybe_unused]]) {
   WRITE_VAL("%s\n", cmd.com_query.query);
   cbd.reset();
   fail = command_service_run_command(st_session, COM_QUERY, &cmd,
-                                     &my_charset_utf8mb3_general_ci, &sql_cbs,
+                                     &my_charset_utf8_general_ci, &sql_cbs,
                                      CS_TEXT_REPRESENTATION, &cbd);
   if (fail)
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "sql_simple ret code: %d\n",
@@ -410,10 +406,10 @@ static void test_com_query(void *p [[maybe_unused]]) {
     /* get values */
     WRITE_STR(
         "-----------------------------------------------------------------\n");
-    WRITE_VAL("%s\t%s\n", sql_field[0][0].col_name, sql_field[0][1].col_name);
+    WRITE_VAL("%s\t\%s\n", sql_field[0][0].col_name, sql_field[0][1].col_name);
     for (uint row = 0; row < sql_num_rows; row++) {
       for (uint col = 0; col < sql_num_cols; col += 2) {
-        WRITE_VAL("%s\t%s\n", sql_str_value[col][row],
+        WRITE_VAL("%s\t\%s\n", sql_str_value[col][row],
                   sql_str_value[col + 1][row]);
       }
     }
@@ -431,12 +427,11 @@ static void test_com_query(void *p [[maybe_unused]]) {
 
   // 3. statement must fail
   cbd.reset();
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = "garbage";
   cmd.com_query.length = strlen(cmd.com_query.query);
 
   ENSURE_API_OK(command_service_run_command(
-      st_session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
+      st_session, COM_QUERY, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
       CS_TEXT_REPRESENTATION, &cbd));
 
   WRITE_VAL("error after bad SQL: %i: %s\n", cbd.err, cbd.errmsg.c_str());
@@ -465,7 +460,7 @@ static int test_com_init_db(void *p) {
   cmd.com_init_db.length = strlen("mysql");
   Callback_data cbd;
   ENSURE_API_OK(command_service_run_command(
-      st_session, COM_INIT_DB, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
+      st_session, COM_INIT_DB, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
       CS_TEXT_REPRESENTATION, &cbd));
 
   db_name = srv_session_info_get_current_db(st_session);
@@ -490,7 +485,7 @@ static int test_com_list_fields(void *p)
   cmd.com_init_db.db_name = "mysql";
   cmd.com_init_db.length = strlen("mysql");
   ENSURE_API_OK(command_service_run_command(st_session, COM_INIT_DB, &cmd,
-&my_charset_utf8mb3_general_ci, &sql_cbs, CS_TEXT_REPRESENTATION, p));
+&my_charset_utf8_general_ci, &sql_cbs, CS_TEXT_REPRESENTATION, p));
 
   WRITE_VAL("switched default db to: %s\n",
 srv_session_info_get_current_db(st_session));
@@ -503,7 +498,7 @@ char*)cmd.com_field_list.table_name); cmd.com_field_list.query = (unsigned
 char*)"%"; cmd.com_field_list.query_length = strlen((const
 char*)cmd.com_field_list.query);
   ENSURE_API_OK(command_service_run_command(st_session, COM_FIELD_LIST, &cmd,
-&my_charset_utf8mb3_general_ci, &sql_cbs, CS_TEXT_REPRESENTATION, p));
+&my_charset_utf8_general_ci, &sql_cbs, CS_TEXT_REPRESENTATION, p));
 
   WRITE_STR("-----------------------------------------------------------------\n");
   for (uint row_count=0;row_count < sql_num_rows;row_count++){
@@ -529,7 +524,7 @@ struct Test_data {
   Test_data() {
     ready = 0;
     native_cond_init(&cond);
-    native_mutex_init(&mutex, nullptr);
+    native_mutex_init(&mutex, NULL);
   }
 
   ~Test_data() {
@@ -562,7 +557,6 @@ static void *test_session_thread(Test_data *tdata) {
   WRITE_VAL("session is dead? %i\n",
             thd_killed(srv_session_info_get_thd(tdata->session)));
 
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = "select sleep(10)";
   cmd.com_query.length = strlen("select sleep(10)");
 
@@ -571,7 +565,7 @@ static void *test_session_thread(Test_data *tdata) {
   tdata->go();
 
   int r = command_service_run_command(tdata->session, COM_QUERY, &cmd,
-                                      &my_charset_utf8mb3_general_ci, &sql_cbs,
+                                      &my_charset_utf8_general_ci, &sql_cbs,
                                       CS_TEXT_REPRESENTATION, &cbdata);
   WRITE_VAL("Killed run_command return value: %i\n", r);
 
@@ -587,7 +581,7 @@ static void *test_session_thread(Test_data *tdata) {
 
   srv_session_deinit_thread();
 
-  return nullptr;
+  return NULL;
 }
 
 static void session_error_cb(void *, unsigned int sql_errno,
@@ -641,11 +635,10 @@ static int test_query_kill(void *p) {
   snprintf(buffer, sizeof(buffer), "kill query %i",
            srv_session_info_get_session_id(st_session_victim));
   WRITE_STR("run KILL QUERY\n");
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = buffer;
   cmd.com_query.length = strlen(buffer);
   ENSURE_API_OK(command_service_run_command(
-      st_session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
+      st_session, COM_QUERY, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
       CS_TEXT_REPRESENTATION, &cbd));
 
   void *ret;
@@ -680,8 +673,8 @@ static int test_com_process_kill(void *p) {
 
   cmd.com_kill.id = srv_session_info_get_session_id(st_session_victim);
   ENSURE_API_OK(command_service_run_command(
-      st_session, COM_PROCESS_KILL, &cmd, &my_charset_utf8mb3_general_ci,
-      &sql_cbs, CS_TEXT_REPRESENTATION, &cbd));
+      st_session, COM_PROCESS_KILL, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
+      CS_TEXT_REPRESENTATION, &cbd));
 
   WRITE_VAL("session is dead now? %i\n",
             thd_killed(srv_session_info_get_thd(st_session_victim)));
@@ -705,11 +698,10 @@ static int test_priv(void *p) {
 
   switch_user(root_session, user_privileged);
 
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = "create user ordinary@localhost";
   cmd.com_query.length = strlen(cmd.com_query.query);
   ENSURE_API_OK(command_service_run_command(
-      root_session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
+      root_session, COM_QUERY, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
       CS_TEXT_REPRESENTATION, &cbd));
   WRITE_VAL("create user as root: %i %s\n", cbd.err, cbd.errmsg.c_str());
 
@@ -720,11 +712,10 @@ static int test_priv(void *p) {
     switch_user(ordinary_session, user_ordinary);
 
     cbd.reset();
-    memset(&cmd, 0, sizeof(cmd));
     cmd.com_query.query = "create user bogus@localhost";
     cmd.com_query.length = strlen(cmd.com_query.query);
     ENSURE_API_OK(command_service_run_command(
-        ordinary_session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci,
+        ordinary_session, COM_QUERY, &cmd, &my_charset_utf8_general_ci,
         &sql_cbs, CS_TEXT_REPRESENTATION, &cbd));
 
     WRITE_VAL("create user supposed to fail: %i %s\n", cbd.err,
@@ -734,11 +725,10 @@ static int test_priv(void *p) {
   }
 
   cbd.reset();
-  memset(&cmd, 0, sizeof(cmd));
   cmd.com_query.query = "drop user ordinary@localhost";
   cmd.com_query.length = strlen(cmd.com_query.query);
   ENSURE_API_OK(command_service_run_command(
-      root_session, COM_QUERY, &cmd, &my_charset_utf8mb3_general_ci, &sql_cbs,
+      root_session, COM_QUERY, &cmd, &my_charset_utf8_general_ci, &sql_cbs,
       CS_TEXT_REPRESENTATION, &cbd));
   WRITE_VAL("drop user as root: %i %s\n", cbd.err, cbd.errmsg.c_str());
 
@@ -747,7 +737,7 @@ static int test_priv(void *p) {
   return 0;
 }
 
-static void test_sql(void *p [[maybe_unused]]) {
+static void test_sql(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Installation.");
 
@@ -790,7 +780,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
+static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
@@ -808,15 +798,15 @@ mysql_declare_plugin(test_daemon){
     MYSQL_DAEMON_PLUGIN,
     &test_sql_service_plugin,
     "test_sql_cmds_1",
-    PLUGIN_AUTHOR_ORACLE,
+    "Horst Hunger, Andrey Hristov",
     "Test sql service commands",
     PLUGIN_LICENSE_GPL,
     test_sql_service_plugin_init,   /* Plugin Init */
-    nullptr,                        /* Plugin Check uninstall */
+    NULL,                           /* Plugin Check uninstall */
     test_sql_service_plugin_deinit, /* Plugin Deinit */
     0x0100 /* 1.0 */,
-    nullptr, /* status variables                */
-    nullptr, /* system variables                */
-    nullptr, /* config options                  */
-    0,       /* flags                           */
+    NULL, /* status variables                */
+    NULL, /* system variables                */
+    NULL, /* config options                  */
+    0,    /* flags                           */
 } mysql_declare_plugin_end;
