@@ -242,7 +242,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
                                      const CHARSET_INFO *resultcs) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_start_result_metadata\n");
-  DBUG_ENTER("sql_start_result_metadata");
+  DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
   DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
@@ -250,7 +250,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
   pctx->resultcs = resultcs;
   pctx->current_col = 0;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_field_metadata(void *ctx, struct st_send_field *field,
@@ -258,7 +258,7 @@ static int sql_field_metadata(void *ctx, struct st_send_field *field,
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   st_send_field_n *cfield = &pctx->sql_field[pctx->current_col];
   //  WRITE_STR("sql_field_metadata\n");
-  DBUG_ENTER("sql_field_metadata");
+  DBUG_TRACE;
   DBUG_PRINT("info", ("field->db_name: %s", field->db_name));
   DBUG_PRINT("info", ("field->table_name: %s", field->table_name));
   DBUG_PRINT("info", ("field->org_table_name: %s", field->org_table_name));
@@ -282,50 +282,47 @@ static int sql_field_metadata(void *ctx, struct st_send_field *field,
   cfield->type = field->type;
 
   pctx->current_col++;
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_end_result_metadata(void *ctx, uint server_status,
                                    uint warn_count) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_end_result_metadata\n");
-  DBUG_ENTER("sql_end_result_metadata");
+  DBUG_TRACE;
   pctx->meta_server_status = server_status;
   pctx->meta_warn_count = warn_count;
   pctx->num_rows = 0;
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_start_row(void *ctx) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_start_row\n");
-  DBUG_ENTER("sql_start_row");
+  DBUG_TRACE;
   pctx->current_col = 0;
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_end_row(void *ctx) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_end_row\n");
-  DBUG_ENTER("sql_end_row");
+  DBUG_TRACE;
   pctx->num_rows++;
-  DBUG_RETURN(false);
+  return false;
 }
 
-static void sql_abort_row(void *) {
-  DBUG_ENTER("sql_abort_row");
-  DBUG_VOID_RETURN;
-}
+static void sql_abort_row(void *) { DBUG_TRACE; }
 
 static ulong sql_get_client_capabilities(void *) {
-  DBUG_ENTER("sql_get_client_capabilities");
-  DBUG_RETURN(0);
+  DBUG_TRACE;
+  return 0;
 }
 
 static int sql_get_null(void *ctx) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_null\n");
-  DBUG_ENTER("sql_get_null");
+  DBUG_TRACE;
   uint row = pctx->num_rows;
   uint col = pctx->current_col;
   pctx->current_col++;
@@ -333,13 +330,13 @@ static int sql_get_null(void *ctx) {
   memcpy(pctx->sql_str_value[row][col], "[NULL]", sizeof("[NULL]"));
   pctx->sql_str_len[row][col] = sizeof("[NULL]") - 1;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_get_integer(void *ctx, longlong value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_integer\n");
-  DBUG_ENTER("sql_get_integer");
+  DBUG_TRACE;
   uint row = pctx->num_rows;
   uint col = pctx->current_col;
   pctx->current_col++;
@@ -349,13 +346,13 @@ static int sql_get_integer(void *ctx, longlong value) {
 
   pctx->sql_str_len[row][col] = len;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_longlong\n");
-  DBUG_ENTER("sql_get_longlong");
+  DBUG_TRACE;
   uint row = pctx->num_rows;
   uint col = pctx->current_col;
   pctx->current_col++;
@@ -366,7 +363,7 @@ static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
 
   pctx->sql_str_len[row][col] = len;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 static const char *test_decimal_as_string(char *buff, const decimal_t *val,
@@ -379,7 +376,7 @@ static const char *test_decimal_as_string(char *buff, const decimal_t *val,
 static int sql_get_decimal(void *ctx, const decimal_t *value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_decimal\n");
-  DBUG_ENTER("sql_get_decimal");
+  DBUG_TRACE;
   uint row = pctx->num_rows;
   uint col = pctx->current_col;
   pctx->current_col++;
@@ -388,13 +385,13 @@ static int sql_get_decimal(void *ctx, const decimal_t *value) {
   test_decimal_as_string(pctx->sql_str_value[row][col], value, &len);
   pctx->sql_str_len[row][col] = len;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_get_double(void *ctx, double value, uint32) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_double\n");
-  DBUG_ENTER("sql_get_double");
+  DBUG_TRACE;
   uint row = pctx->num_rows;
   uint col = pctx->current_col;
   pctx->current_col++;
@@ -404,13 +401,13 @@ static int sql_get_double(void *ctx, double value, uint32) {
 
   pctx->sql_str_len[row][col] = len;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_date\n");
-  DBUG_ENTER("sql_get_date");
+  DBUG_TRACE;
   uint row = pctx->num_rows;
   uint col = pctx->current_col;
   pctx->current_col++;
@@ -421,13 +418,13 @@ static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
                value->neg ? "-" : "", value->year, value->month, value->day);
   pctx->sql_str_len[row][col] = len;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_time\n");
-  DBUG_ENTER("sql_get_time");
+  DBUG_TRACE;
   uint row = pctx->num_rows;
   uint col = pctx->current_col;
   pctx->current_col++;
@@ -439,13 +436,13 @@ static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint) {
       value->second);
 
   pctx->sql_str_len[row][col] = len;
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_get_datetime(void *ctx, const MYSQL_TIME *value, uint) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_datetime\n");
-  DBUG_ENTER("sql_get_datetime");
+  DBUG_TRACE;
   uint row = pctx->num_rows;
   uint col = pctx->current_col;
   pctx->current_col++;
@@ -457,14 +454,14 @@ static int sql_get_datetime(void *ctx, const MYSQL_TIME *value, uint) {
 
   pctx->sql_str_len[row][col] = len;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 static int sql_get_string(void *ctx, const char *const value, size_t length,
                           const CHARSET_INFO *const) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_string\n");
-  DBUG_ENTER("sql_get_string");
+  DBUG_TRACE;
   uint row = pctx->num_rows;
   uint col = pctx->current_col;
   pctx->current_col++;
@@ -472,7 +469,7 @@ static int sql_get_string(void *ctx, const char *const value, size_t length,
   strncpy(pctx->sql_str_value[row][col], value, length);
   pctx->sql_str_len[row][col] = length;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 static void sql_handle_ok(void *ctx, uint server_status,
@@ -480,7 +477,7 @@ static void sql_handle_ok(void *ctx, uint server_status,
                           ulonglong last_insert_id, const char *const message) {
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_handle_ok\n");
-  DBUG_ENTER("sql_handle_ok");
+  DBUG_TRACE;
   /* This could be an EOF */
   if (!pctx->num_cols) pctx->num_rows = 0;
   pctx->server_status = server_status;
@@ -489,8 +486,6 @@ static void sql_handle_ok(void *ctx, uint server_status,
   pctx->last_insert_id = last_insert_id;
   if (message) strncpy(pctx->message, message, sizeof(pctx->message) - 1);
   pctx->message[sizeof(pctx->message) - 1] = '\0';
-
-  DBUG_VOID_RETURN;
 }
 
 static void sql_handle_error(void *ctx, uint sql_errno,
@@ -499,17 +494,13 @@ static void sql_handle_error(void *ctx, uint sql_errno,
   char buffer[1024];
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_handle_error\n");
-  DBUG_ENTER("sql_handle_error");
+  DBUG_TRACE;
   WRITE_VAL2("[%u][%s]", sql_errno, sqlstate);
   WRITE_VAL("[%s]", err_msg);
   pctx->num_rows = 0;
-  DBUG_VOID_RETURN;
 }
 
-static void sql_shutdown(void *, int) {
-  DBUG_ENTER("sql_shutdown");
-  DBUG_VOID_RETURN;
-}
+static void sql_shutdown(void *, int) { DBUG_TRACE; }
 
 const struct st_command_service_cbs protocol_callbacks = {
     sql_start_result_metadata,
@@ -840,7 +831,7 @@ void static change_current_db(MYSQL_SESSION session, const char *db,
 }
 
 static void test_selects(MYSQL_SESSION session, void *p) {
-  DBUG_ENTER("test_selects");
+  DBUG_TRACE;
   char buffer[STRING_BUFFER_SIZE];
 
   struct st_plugin_ctx *plugin_ctx = new st_plugin_ctx();
@@ -861,7 +852,6 @@ static void test_selects(MYSQL_SESSION session, void *p) {
   WRITE_DASHED_LINE();
 
   delete plugin_ctx;
-  DBUG_VOID_RETURN;
 }
 
 static const char *user_localhost = "localhost";
@@ -877,7 +867,7 @@ static void switch_user(MYSQL_SESSION session, const char *user) {
 }
 
 static void test_sql(void *p) {
-  DBUG_ENTER("test_sql");
+  DBUG_TRACE;
 
   char buffer[STRING_BUFFER_SIZE];
 
@@ -902,7 +892,7 @@ static void test_sql(void *p) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "srv_session_close failed.");
 
 end:
-  DBUG_VOID_RETURN;
+  return;
 }
 
 struct test_thread_context {
@@ -968,9 +958,8 @@ static void test_in_spawned_thread(void *p, void (*test_function)(void *)) {
 
 static int test_sql_service_plugin_init(void *p) {
   char buffer[STRING_BUFFER_SIZE];
-  DBUG_ENTER("test_sql_service_plugin_init");
-  if (init_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs))
-    DBUG_RETURN(1);
+  DBUG_TRACE;
+  if (init_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs)) return 1;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Installation.");
 
   create_log_file(log_filename);
@@ -985,14 +974,14 @@ static int test_sql_service_plugin_init(void *p) {
 
   my_close(outfile, MYF(0));
 
-  DBUG_RETURN(0);
+  return 0;
 }
 
 static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
-  DBUG_ENTER("test_sql_service_plugin_deinit");
+  DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
-  DBUG_RETURN(0);
+  return 0;
 }
 
 static struct st_mysql_daemon test_sql_service_plugin = {
