@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -50,7 +50,7 @@ struct st_test_statement {
 
 static struct st_test_statement test_query_plan[] = {
     /* DB   RESULT   QUERY */
-    {nullptr, true, "SELECT 'first complex command' as a"},
+    {NULL, true, "SELECT 'first complex command' as a"},
     {"test", false,
      "INSERT INTO t1 VALUES (1,1), (2,2), (3,3), (4,1), (5,2), (6,3)"},
     {"test", true, "SELECT * FROM t1"},
@@ -133,7 +133,7 @@ struct st_plugin_ctx {
   st_plugin_ctx() { reset(); }
 
   void reset() {
-    resultcs = nullptr;
+    resultcs = NULL;
     meta_server_status = 0;
     meta_warn_count = 0;
     server_status = 0;
@@ -164,8 +164,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
   struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
-  DBUG_PRINT("info",
-             ("resultcs->csname: %s", replace_utf8_utf8mb3(resultcs->csname)));
+  DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
   DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
   pctx->num_cols = num_cols;
   pctx->resultcs = resultcs;
@@ -282,7 +281,7 @@ static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
 static const char *test_decimal_as_string(char *buff, const decimal_t *val,
                                           int *length) {
   if (!val) return "NULL";
-  (void)decimal2string(val, buff, length);
+  (void)decimal2string(val, buff, length, 0, 0, 0);
   return buff;
 }
 
@@ -428,7 +427,6 @@ const struct st_command_service_cbs protocol_callbacks = {
     sql_handle_ok,
     sql_handle_error,
     sql_shutdown,
-    nullptr,
 };
 
 static const char *fieldtype2str(enum enum_field_types type) {
@@ -554,8 +552,7 @@ static void dump_cs_info(const CHARSET_INFO *cs) {
   }
 
   WRITE_VAL("\t\t[meta][charset result] number: %d\n", cs->number);
-  WRITE_VAL("\t\t[meta][charset result] name: %s\n",
-            replace_utf8_utf8mb3(cs->csname));
+  WRITE_VAL("\t\t[meta][charset result] name: %s\n", cs->csname);
   WRITE_VAL("\t\t[meta][charset result] collation: %s\n", cs->name);
   WRITE_VAL("\t\t[meta][charset result] sort order: %s\n", cs->sort_order);
 }
@@ -674,7 +671,6 @@ static void dump_closing_ok(struct st_plugin_ctx *ctx) {
 static void set_query_in_com_data(const char *query, union COM_DATA *cmd) {
   char buffer[STRING_BUFFER_SIZE];
 
-  memset(cmd, 0, sizeof(union COM_DATA));
   cmd->com_query.query = query;
   cmd->com_query.length = strlen(query);
   WRITE_VAL2("EXECUTING:[%u][%s]\n", cmd->com_query.length, query);
@@ -682,7 +678,7 @@ static void set_query_in_com_data(const char *query, union COM_DATA *cmd) {
 
 static void run_statement(MYSQL_SESSION session, const char *query,
                           struct st_plugin_ctx *ctx, bool generates_result_set,
-                          void *p [[maybe_unused]]) {
+                          void *p MY_ATTRIBUTE((unused))) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
 
@@ -720,7 +716,7 @@ again:
 
 void static change_current_db(MYSQL_SESSION session, const char *db,
                               struct st_plugin_ctx *ctx,
-                              void *p [[maybe_unused]]) {
+                              void *p MY_ATTRIBUTE((unused))) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
   cmd.com_init_db.db_name = db;
@@ -741,7 +737,7 @@ static void test_selects(MYSQL_SESSION session, void *p) {
 
   struct st_plugin_ctx *plugin_ctx = new st_plugin_ctx();
 
-  const char *last_db = nullptr;
+  const char *last_db = NULL;
   size_t stmt_count = sizeof(test_query_plan) / sizeof(test_query_plan[0]);
   for (size_t i = 0; i < stmt_count; i++) {
     /* Change current DB if needed */
@@ -753,7 +749,7 @@ static void test_selects(MYSQL_SESSION session, void *p) {
     run_statement(session, test_query_plan[i].query, plugin_ctx,
                   test_query_plan[i].generates_result_set, p);
 
-    last_db = nullptr;
+    last_db = NULL;
   }
 
   WRITE_SEP();
@@ -804,7 +800,7 @@ static void *test_sql_threaded_wrapper(void *param) {
   srv_session_deinit_thread();
 
   context->thread_finished = true;
-  return nullptr;
+  return NULL;
 }
 
 static void create_log_file(const char *log_name) {
@@ -833,7 +829,7 @@ static void test_in_spawned_thread(void *p, void (*test_function)(void *)) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG,
                  "Could not create test session thread");
   else
-    my_thread_join(&context.thread, nullptr);
+    my_thread_join(&context.thread, NULL);
 }
 
 static int test_sql_service_plugin_init(void *p) {
@@ -857,7 +853,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
+static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
@@ -875,15 +871,15 @@ mysql_declare_plugin(test_daemon){
     MYSQL_DAEMON_PLUGIN,
     &test_sql_service_plugin,
     "test_sql_replication",
-    PLUGIN_AUTHOR_ORACLE,
+    "Pavan Naik, Andrey Hristov",
     "Test sql replication",
     PLUGIN_LICENSE_GPL,
     test_sql_service_plugin_init,   /* Plugin Init      */
-    nullptr,                        /* Plugin Check uninstall    */
+    NULL,                           /* Plugin Check uninstall    */
     test_sql_service_plugin_deinit, /* Plugin Deinit    */
     0x0100,                         /* 1.0              */
-    nullptr,                        /* status variables */
-    nullptr,                        /* system variables */
-    nullptr,                        /* config options   */
+    NULL,                           /* status variables */
+    NULL,                           /* system variables */
+    NULL,                           /* config options   */
     0,                              /* flags            */
 } mysql_declare_plugin_end;
