@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,7 +51,7 @@ struct st_test_statement {
 
 static struct st_test_statement test_query_plan[] = {
     /* DB   RESULT   QUERY */
-    {nullptr, true, "SELECT 'first complex command' as a"},
+    {NULL, true, "SELECT 'first complex command' as a"},
     // Procedure
     {"test", false, "CALL test.proc()"},
     {"test", false, "CALL test.sp_no_results()"},
@@ -63,12 +62,12 @@ static struct st_test_statement test_query_plan[] = {
     {"test", false, "CALL test.sp_two_results_out(@a)"},
     {"test", true, "CALL test.sp_with_error()"},
     {"test", true, "CALL test.sp_result_error()"},
-    {nullptr, true,
+    {NULL, true,
      "SELECT SPECIFIC_NAME FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_TYPE "
      "= 'PROCEDURE' AND ROUTINE_NAME = 'proc'"},
     // Function
     {"test", true, "SELECT avg_func(10, 20)"},
-    {nullptr, true,
+    {NULL, true,
      "SELECT SPECIFIC_NAME FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_TYPE "
      "= 'FUNCTION' AND ROUTINE_NAME = 'avg_func'"}};
 
@@ -161,7 +160,7 @@ struct st_plugin_ctx {
   st_plugin_ctx() { reset(); }
 
   void reset() {
-    resultcs = nullptr;
+    resultcs = NULL;
     meta_server_status = 0;
     meta_warn_count = 0;
     server_status = 0;
@@ -193,7 +192,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
   DBUG_PRINT("info", ("resultcs->csname: %s", resultcs->csname));
-  DBUG_PRINT("info", ("resultcs->m_coll_name: %s", resultcs->m_coll_name));
+  DBUG_PRINT("info", ("resultcs->name: %s", resultcs->name));
   pctx->num_cols = num_cols;
   pctx->resultcs = resultcs;
   pctx->current_col = 0;
@@ -311,7 +310,7 @@ static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
 static const char *test_decimal_as_string(char *buff, const decimal_t *val,
                                           int *length) {
   if (!val) return "NULL";
-  (void)decimal2string(val, buff, length);
+  (void)decimal2string(val, buff, length, 0, 0, 0);
   return buff;
 }
 
@@ -456,7 +455,6 @@ const struct st_command_service_cbs protocol_callbacks = {
     sql_handle_ok,
     sql_handle_error,
     sql_shutdown,
-    nullptr,
 };
 
 #define WRITE_DASHED_LINE() \
@@ -587,7 +585,7 @@ static void dump_cs_info(const CHARSET_INFO *cs) {
 
   WRITE_VAL("\t\t[meta][charset result] number: %d\n", cs->number);
   WRITE_VAL("\t\t[meta][charset result] name: %s\n", cs->csname);
-  WRITE_VAL("\t\t[meta][charset result] collation: %s\n", cs->m_coll_name);
+  WRITE_VAL("\t\t[meta][charset result] collation: %s\n", cs->name);
   WRITE_VAL("\t\t[meta][charset result] sort order: %s\n", cs->sort_order);
 }
 
@@ -705,7 +703,6 @@ static void dump_closing_ok(struct st_plugin_ctx *ctx) {
 static void set_query_in_com_data(const char *query, union COM_DATA *cmd) {
   char buffer[STRING_BUFFER_SIZE];
 
-  memset(cmd, 0, sizeof(union COM_DATA));
   cmd->com_query.query = query;
   cmd->com_query.length = strlen(query);
   WRITE_VAL2("EXECUTING:[%u][%s]\n", cmd->com_query.length, query);
@@ -713,7 +710,7 @@ static void set_query_in_com_data(const char *query, union COM_DATA *cmd) {
 
 static void run_statement(MYSQL_SESSION session, const char *query,
                           struct st_plugin_ctx *ctx, bool generates_result_set,
-                          void *p [[maybe_unused]]) {
+                          void *p MY_ATTRIBUTE((unused))) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
 
@@ -726,7 +723,7 @@ static void run_statement(MYSQL_SESSION session, const char *query,
 again:
   ctx->reset();
   int fail = command_service_run_command(session, COM_QUERY, &cmd,
-                                         &my_charset_utf8mb3_general_ci,
+                                         &my_charset_utf8_general_ci,
                                          &protocol_callbacks, txt_or_bin, ctx);
   if (fail) {
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "run_statement code: %d\n",
@@ -751,7 +748,7 @@ again:
 
 void static change_current_db(MYSQL_SESSION session, const char *db,
                               struct st_plugin_ctx *ctx,
-                              void *p [[maybe_unused]]) {
+                              void *p MY_ATTRIBUTE((unused))) {
   char buffer[STRING_BUFFER_SIZE];
   COM_DATA cmd;
   cmd.com_init_db.db_name = db;
@@ -761,7 +758,7 @@ void static change_current_db(MYSQL_SESSION session, const char *db,
   WRITE_VAL("EXECUTING:[COM_INIT_DB][%s]\n", db);
   ctx->reset();
   int fail = command_service_run_command(
-      session, COM_INIT_DB, &cmd, &my_charset_utf8mb3_general_ci,
+      session, COM_INIT_DB, &cmd, &my_charset_utf8_general_ci,
       &protocol_callbacks, CS_TEXT_REPRESENTATION, ctx);
   if (fail)
     LogPluginErrMsg(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "change db code: %d\n",
@@ -774,7 +771,7 @@ static void test_selects(MYSQL_SESSION session, void *p) {
 
   struct st_plugin_ctx *plugin_ctx = new st_plugin_ctx();
 
-  const char *last_db = nullptr;
+  const char *last_db = NULL;
   size_t stmt_count = sizeof(test_query_plan) / sizeof(test_query_plan[0]);
   for (size_t i = 0; i < stmt_count; i++) {
     /* Change current DB if needed */
@@ -786,7 +783,7 @@ static void test_selects(MYSQL_SESSION session, void *p) {
     run_statement(session, test_query_plan[i].query, plugin_ctx,
                   test_query_plan[i].generates_result_set, p);
 
-    last_db = nullptr;
+    last_db = NULL;
   }
 
   WRITE_DASHED_LINE();
@@ -838,7 +835,7 @@ static void *test_sql_threaded_wrapper(void *param) {
   srv_session_deinit_thread();
 
   context->thread_finished = true;
-  return nullptr;
+  return NULL;
 }
 
 static void create_log_file(const char *log_name) {
@@ -867,7 +864,7 @@ static void test_in_spawned_thread(void *p, void (*test_function)(void *)) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG,
                  "Could not create test session thread");
   else
-    my_thread_join(&context.thread, nullptr);
+    my_thread_join(&context.thread, NULL);
 }
 
 static int test_sql_service_plugin_init(void *p) {
@@ -892,7 +889,7 @@ static int test_sql_service_plugin_init(void *p) {
   return 0;
 }
 
-static int test_sql_service_plugin_deinit(void *p [[maybe_unused]]) {
+static int test_sql_service_plugin_deinit(void *p MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG, "Uninstallation.");
   deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
@@ -910,15 +907,15 @@ mysql_declare_plugin(test_daemon){
     MYSQL_DAEMON_PLUGIN,
     &test_sql_service_plugin,
     "test_sql_stored_procedures_functions",
-    PLUGIN_AUTHOR_ORACLE,
+    "Pavan Naik, Andrey Hristov",
     "Test SQL stored procedures/functions",
     PLUGIN_LICENSE_GPL,
     test_sql_service_plugin_init,   /* Plugin Init      */
-    nullptr,                        /* Plugin check uninstall    */
+    NULL,                           /* Plugin check uninstall    */
     test_sql_service_plugin_deinit, /* Plugin Deinit    */
     0x0100,                         /* 1.0              */
-    nullptr,                        /* status variables */
-    nullptr,                        /* system variables */
-    nullptr,                        /* config options   */
+    NULL,                           /* status variables */
+    NULL,                           /* system variables */
+    NULL,                           /* config options   */
     0,                              /* flags            */
 } mysql_declare_plugin_end;
