@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,15 +24,15 @@
 
 #include "plugin/x/client/authentication/password_hasher.h"
 
-#include <assert.h>
-#include <openssl/rand.h>
-#include <openssl/sha.h>
 #include <sys/types.h>
-
 #include <cstdint>
 #include <cstring>
 #include <stdexcept>
 
+#include <openssl/rand.h>
+#include <openssl/sha.h>
+
+#include "my_dbug.h"
 #include "plugin/x/client/authentication/mysql41_hash.h"
 
 #define PVERSION41_CHAR '*'
@@ -45,14 +44,14 @@ namespace {
 
 const char *_dig_vec_upper = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
-void compute_two_stage_mysql41_hash(const std::string &password,
+void compute_two_stage_mysql41_hash(const char *password, size_t pass_len,
                                     uint8_t *hash_stage1,
                                     uint8_t *hash_stage2) {
   /* Stage 1: hash pwd */
-  compute_mysql41_hash(hash_stage1, password.c_str(), password.size());
+  compute_mysql41_hash(hash_stage1, password, static_cast<unsigned>(pass_len));
 
   /* Stage 2 : hash first stage's output. */
-  compute_mysql41_hash(hash_stage2, reinterpret_cast<const char *>(hash_stage1),
+  compute_mysql41_hash(hash_stage2, (const char *)hash_stage1,
                        MYSQL41_HASH_SIZE);
 }
 
@@ -91,7 +90,7 @@ char *octet2hex(char *to, const char *str, size_t len) {
  *          human readable version of hash_stage2.
  */
 std::string get_password_from_salt(const std::string &hash_stage2) {
-  const uint8_t result_size =
+  const std::uint8_t result_size =
       2 * MYSQL41_HASH_SIZE + 1 /* '\0' sign */ + 1 /* '*' sign */;
   char result[result_size] = {0};
 
@@ -109,7 +108,7 @@ std::string generate_user_salt() {
   char *buffer = &result[0];
   char *end = buffer + result.length() - 1;
 
-  RAND_bytes(reinterpret_cast<unsigned char *>(buffer), SCRAMBLE_LENGTH);
+  RAND_bytes((unsigned char *)buffer, SCRAMBLE_LENGTH);
 
   /* Sequence must be a legal UTF8 string */
   for (; buffer < end; buffer++) {
@@ -120,50 +119,48 @@ std::string generate_user_salt() {
   return result;
 }
 
-bool check_scramble_mysql41_hash(const std::string &scramble_arg,
-                                 const std::string &message,
+bool check_scramble_mysql41_hash(const char *scramble_arg, const char *message,
                                  const uint8_t *hash_stage2) {
   char buf[MYSQL41_HASH_SIZE];
   uint8_t hash_stage2_reassured[MYSQL41_HASH_SIZE];
 
-  assert(MYSQL41_HASH_SIZE == SCRAMBLE_LENGTH);
+  DBUG_ASSERT(MYSQL41_HASH_SIZE == SCRAMBLE_LENGTH);
   /* create key to encrypt scramble */
-  compute_mysql41_hash_multi(
-      reinterpret_cast<uint8_t *>(buf), message.c_str(), message.size(),
-      reinterpret_cast<const char *>(hash_stage2), MYSQL41_HASH_SIZE);
+  compute_mysql41_hash_multi(reinterpret_cast<uint8_t *>(buf), message,
+                             SCRAMBLE_LENGTH, (const char *)hash_stage2,
+                             MYSQL41_HASH_SIZE);
 
   /* encrypt scramble */
-  my_crypt(buf, reinterpret_cast<const uint8_t *>(buf),
-           reinterpret_cast<const uint8_t *>(scramble_arg.c_str()),
+  my_crypt(buf, (const uint8_t *)buf, (const uint8_t *)scramble_arg,
            SCRAMBLE_LENGTH);
 
   /* now buf supposedly contains hash_stage1: so we can get hash_stage2 */
   compute_mysql41_hash(reinterpret_cast<uint8_t *>(hash_stage2_reassured),
-                       reinterpret_cast<const char *>(buf), MYSQL41_HASH_SIZE);
+                       (const char *)buf, MYSQL41_HASH_SIZE);
 
   return 0 == memcmp(hash_stage2, hash_stage2_reassured, MYSQL41_HASH_SIZE);
 }
 
-std::string scramble(const std::string &message, const std::string &password) {
+std::string scramble(const char *message, const char *password) {
   uint8_t hash_stage1[MYSQL41_HASH_SIZE];
   uint8_t hash_stage2[MYSQL41_HASH_SIZE];
   std::string result(SCRAMBLE_LENGTH, '\0');
 
   result.at(SCRAMBLE_LENGTH - 1) = '\0';
 
-  assert(MYSQL41_HASH_SIZE == SCRAMBLE_LENGTH);
+  DBUG_ASSERT(MYSQL41_HASH_SIZE == SCRAMBLE_LENGTH);
 
   /* Two stage SHA1 hash of the pwd */
-  compute_two_stage_mysql41_hash(password,
+  compute_two_stage_mysql41_hash(password, strlen(password),
                                  reinterpret_cast<uint8_t *>(hash_stage1),
                                  reinterpret_cast<uint8_t *>(hash_stage2));
 
   /* create crypt string as sha1(message, hash_stage2) */
-  compute_mysql41_hash_multi(
-      reinterpret_cast<uint8_t *>(&result[0]), message.c_str(), message.size(),
-      reinterpret_cast<const char *>(hash_stage2), MYSQL41_HASH_SIZE);
-  my_crypt(&result[0], reinterpret_cast<const uint8_t *>(&result[0]),
-           hash_stage1, SCRAMBLE_LENGTH);
+  compute_mysql41_hash_multi(reinterpret_cast<uint8_t *>(&result[0]), message,
+                             SCRAMBLE_LENGTH, (const char *)hash_stage2,
+                             MYSQL41_HASH_SIZE);
+  my_crypt(&result[0], (const uint8_t *)&result[0], hash_stage1,
+           SCRAMBLE_LENGTH);
 
   return result;
 }
