@@ -1,32 +1,31 @@
 /*
-Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License, version 2.0,
-as published by the Free Software Foundation.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License, version 2.0, for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License, version 2.0, for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "plugin/x/client/authentication/sha256_scramble_generator.h"
 
 #include <cstring>
 
-#include "my_config.h"  // NOLINT(build/include_subdir)
+#include "my_config.h"
 
 #ifdef HAVE_ALLOCA_H
 #include <alloca.h>
@@ -34,8 +33,8 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <openssl/evp.h>
 
-#include <cstdint>
-#include "my_dbug.h"  // DBUG instrumentation  NOLINT(build/include_subdir)
+#include "my_dbug.h"     /* DBUG instrumentation */
+#include "my_inttypes.h" /* typedefs */
 
 namespace xcl {
 namespace sha256_password {
@@ -66,7 +65,7 @@ SHA256_digest::~SHA256_digest() { deinit(); }
     @retval true Problem updating digest
     @retval false Success
 */
-bool SHA256_digest::update_digest(const void *src, const uint32_t length) {
+bool SHA256_digest::update_digest(const void *src, const std::uint32_t length) {
   DBUG_TRACE;
   if (!m_ok || !src) {
     DBUG_PRINT("info", ("Either digest context is not ok or "
@@ -90,7 +89,7 @@ bool SHA256_digest::update_digest(const void *src, const uint32_t length) {
     @retval false Success
 */
 bool SHA256_digest::retrieve_digest(unsigned char *digest,
-                                    const uint32_t length) {
+                                    const std::uint32_t length) {
   DBUG_TRACE;
   if (!m_ok || !digest || length != CACHING_SHA2_DIGEST_LENGTH) {
     DBUG_PRINT("info", ("Either digest context is not ok or "
@@ -167,7 +166,7 @@ Generate_scramble::Generate_scramble(
       break;
     }
     default: {
-      assert(false);
+      DBUG_ASSERT(false);
     }
   }
 }
@@ -189,14 +188,14 @@ Generate_scramble::Generate_scramble(
     @retval false Success
 */
 bool Generate_scramble::scramble(unsigned char *out_scramble,
-                                 const uint32_t scramble_length) {
+                                 const std::uint32_t scramble_length) {
   DBUG_TRACE;
   unsigned char *digest_stage1;
   unsigned char *digest_stage2;
   unsigned char *scramble_stage1;
 
   if (!out_scramble || scramble_length != m_digest_length) {
-    DBUG_PRINT("info", ("Unexpected scramble length"
+    DBUG_PRINT("info", ("Unexpected scrable length"
                         "Expected: %d, Actual: %d",
                         m_digest_length, !out_scramble ? 0 : scramble_length));
     return true;
@@ -213,7 +212,7 @@ bool Generate_scramble::scramble(unsigned char *out_scramble,
       break;
     }
     default: {
-      assert(false);
+      DBUG_ASSERT(false);
       return true;
     }
   }
@@ -239,13 +238,13 @@ bool Generate_scramble::scramble(unsigned char *out_scramble,
   if (m_digest_generator->update_digest(digest_stage2, m_digest_length) ||
       m_digest_generator->update_digest(m_rnd.c_str(), m_rnd.length()) ||
       m_digest_generator->retrieve_digest(scramble_stage1, m_digest_length)) {
-    DBUG_PRINT("info", ("Failed to generate scramble_stage1: "
+    DBUG_PRINT("info", ("Failed to generate scrmable_stage1: "
                         "SHA2(digest_stage2, m_rnd)"));
     return true;
   }
 
   /* XOR(digest_stage1, scramble_stage1) => out_scramble */
-  for (uint32_t i = 0; i < m_digest_length; ++i)
+  for (uint i = 0; i < m_digest_length; ++i)
     out_scramble[i] = (digest_stage1[i] ^ scramble_stage1[i]);
 
   return false;
@@ -266,7 +265,7 @@ bool Generate_scramble::scramble(unsigned char *out_scramble,
   @note
     SHA2(src) => X
     SHA2(X) => Y
-    SHA2(Y, salt) => Z
+    SHA2(XOR(salt, Y) => Z
     XOR(X, Z) => scramble
 
   @returns Status of scramble generation
