@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -381,42 +381,6 @@ class XProtocol {
   virtual XError send(const Header_message_type_id mid, const uint8_t *buffer,
                       const std::size_t length) = 0;
 
-  /**
-    Serialize, compress and send protobuf message.
-
-    This method compresses 'message', and places it into 'payload'
-    field of `Compression` message. `Compression` message is serialized
-    and send to the wire.
-    Such construction is send using XConnection interface.
-
-    @param message_id   message identifier
-    @param message      to be serialized and sent
-
-    @return Error code with description
-      @retval != true     OK
-      @retval == true     I/O error or timeout error occurred
-  */
-  virtual XError send_compressed_frame(const Client_message_type_id message_id,
-                                       const Message &message) = 0;
-
-  /**
-    Serialize, compress and send multiple protobuf message of different type.
-
-    This method builds "Compression" message that encodes and compresses all
-    'messages' into "payload" field. Later on "Compression" message is
-    serialized and send to the wire. size. Such construction is send using
-    XConnection interface.
-
-    @param messages     messages to be serialized, compressed and sent
-
-    @return Error code with description
-      @retval != true     OK
-      @retval == true     I/O error or timeout error occurred
-  */
-  virtual XError send_compressed_multiple_frames(
-      const std::vector<std::pair<Client_message_type_id, const Message *>>
-          &messages) = 0;
-
   /**
     Serialize and send protobuf message.
 
@@ -914,13 +878,6 @@ class XProtocol {
                                       const std::string &pass,
                                       const std::string &schema,
                                       const std::string &method = "") = 0;
-
-  virtual void use_compression(const Compression_algorithm algo) = 0;
-
-  virtual void use_compression(const Compression_algorithm algo,
-                               const int32_t level) = 0;
-
-  virtual void reset_buffering() = 0;
 };
 
 }  // namespace xcl
