@@ -79,6 +79,19 @@ class XSession {
       Capability type: BOOL. Value: enable/disable the support.
      */
     Capability_client_interactive,
+
+    /**
+      The server may want know more about the client. MySQL X clients connection
+      the server may differ in: application version, MySQL library version,
+      OS, CPU, CPU endianess, programming language. Having more information
+      about clients/statistics is going to help server administrators with
+      finding faulty clients, targeting potential issues and allow better
+      optimizing.
+      Capability type: OBJECT. Value: associative array of strings where
+      key name must not exceed 32 characters and value must not exceed
+      1024 characters.
+     */
+    Capability_session_connect_attrs,
   };
 
   /**
@@ -243,7 +256,7 @@ class XSession {
       Option type: BOOL
      */
     Consume_all_notices,
-    /** Determine what should be the lenght of a DATETIME field so that it
+    /** Determine what should be the length of a DATETIME field so that it
         would be possible to distinguish if it contain only date or both
         date and time parts.
 
@@ -257,7 +270,7 @@ class XSession {
       Default:
       Option type: STRING
     */
-    Network_namespace
+    Network_namespace,
   };
 
  public:
@@ -370,13 +383,16 @@ class XSession {
 
     @param capability   capability to set or modify
     @param value        assign bool value to the capability
+    @param required     define if connection should be accepted by client when
+                        server rejected the capability
 
     @return Error code with description
       @retval != true     OK
       @retval == true     error occurred
   */
   virtual XError set_capability(const Mysqlx_capability capability,
-                                const bool value) = 0;
+                                const bool value,
+                                const bool required = true) = 0;
 
   /**
     Set X protocol capabilities.
@@ -386,13 +402,16 @@ class XSession {
 
     @param capability   capability to set or modify
     @param value        assign string value to the capability
+    @param required     define if connection should be accepted by client when
+                        server rejected the capability
 
     @return Error code with description
       @retval != true     OK
       @retval == true     error occurred
   */
   virtual XError set_capability(const Mysqlx_capability capability,
-                                const std::string &value) = 0;
+                                const std::string &value,
+                                const bool required = true) = 0;
 
   /**
     Set X protocol capabilities.
@@ -402,13 +421,16 @@ class XSession {
 
     @param capability   capability to set or modify
     @param value        assign "C" string value to the capability
+    @param required     define if connection should be accepted by client when
+                        server rejected the capability
 
     @return Error code with description
       @retval != true     OK
       @retval == true     error occurred
   */
   virtual XError set_capability(const Mysqlx_capability capability,
-                                const char *value) = 0;
+                                const char *value,
+                                const bool required = true) = 0;
 
   /**
     Set X protocol capabilities.
@@ -418,13 +440,54 @@ class XSession {
 
     @param capability   capability to set or modify
     @param value        assign integer value to the capability
+    @param required     define if connection should be accepted by client when
+                        server rejected the capability
 
     @return Error code with description
       @retval != true     OK
       @retval == true     error occurred
   */
   virtual XError set_capability(const Mysqlx_capability capability,
-                                const int64_t value) = 0;
+                                const int64_t value,
+                                const bool required = true) = 0;
+
+  /**
+    Set X protocol capabilities.
+
+    All capabilities set before calling `XSession::connect` method are
+    committed to the server (other side of the connection).
+
+    @param capability   capability to set or modify
+    @param value        assign object value to the capability
+    @param required     define if connection should be accepted by client when
+                        server rejected the capability
+
+    @return Error code with description
+      @retval != true     OK
+      @retval == true     error occurred
+  */
+  virtual XError set_capability(const Mysqlx_capability capability,
+                                const Argument_object &value,
+                                const bool required = true) = 0;
+
+  /**
+    Set X protocol capabilities.
+
+    All capabilities set before calling `XSession::connect` method are
+    committed to the server (other side of the connection).
+
+    @param capability   capability to set or modify
+    @param value        assign 'unordered' object value to the capability
+    @param required     define if connection should be accepted by client when
+                        server rejected the capability
+
+    @return Error code with description
+      @retval != true     OK
+      @retval == true     error occurred
+  */
+  virtual XError set_capability(const Mysqlx_capability capability,
+                                const Argument_uobject &value,
+                                const bool required = true) = 0;
 
   /**
     Establish and authenticate connection using TCP.
@@ -515,10 +578,9 @@ class XSession {
       @retval != nullptr  OK
       @retval == nullptr  error occurred
   */
-  virtual std::unique_ptr<XQuery_result> execute_stmt(const std::string &ns,
-                                                      const std::string &stmt,
-                                                      const Arguments &args,
-                                                      XError *out_error) = 0;
+  virtual std::unique_ptr<XQuery_result> execute_stmt(
+      const std::string &ns, const std::string &stmt,
+      const Argument_array &args, XError *out_error) = 0;
 
   /**
     Graceful shutdown maintaing the close connection message flow.
@@ -527,6 +589,13 @@ class XSession {
     object.
   */
   virtual void close() = 0;
+
+  /**
+   Get pre-filled session connection attributes.
+   If necessary could be supplemented with additional information before
+   sending it as capability to the server.
+   */
+  virtual Argument_uobject get_connect_attrs() const = 0;
 };
 
 /**
