@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -35,20 +35,31 @@
 #include "mysqlxclient/xcompression.h"
 #include "mysqlxclient/xerror.h"
 
+#include "plugin/x/client/xcapability_builder.h"
+
 namespace xcl {
 
+enum class Compression_style { k_none, k_single, k_multiple, k_group };
+
 class Capabilities_negotiator {
  public:
+  using Compression_styles = std::vector<Compression_style>;
   using Compression_algorithms = std::vector<Compression_algorithm>;
   using Array_of_strings = std::vector<std::string>;
 
  public:
+  void server_supports_client_styles(
+      const Array_of_strings &server_supported_client_styles);
+  void server_supports_server_styles(
+      const Array_of_strings &server_supported_server_styles);
   void server_supports_algorithms(
       const Array_of_strings &server_supported_algorithms);
 
   bool is_negotiation_needed() const;
-  bool is_compression_preferred() const;
   bool update_compression_options(Compression_algorithm *out_algorithm,
+                                  Compression_style *out_client_style,
+                                  Compression_style *out_server_style,
+                                  Capabilities_builder *out_builder,
                                   XError *out_Error);
 
  public:
@@ -58,8 +69,15 @@ class Capabilities_negotiator {
   // The order is important, its sorted from most preferred.
   //
   Compression_algorithms m_compression_negotiation_algorithm{
-      Compression_algorithm::k_deflate, Compression_algorithm::k_lz4,
-      Compression_algorithm::k_zstd};
+      Compression_algorithm::k_deflate, Compression_algorithm::k_lz4};
+
+  Compression_styles m_compression_negotiation_server_style{
+      Compression_style::k_group, Compression_style::k_multiple,
+      Compression_style::k_single};
+
+  Compression_styles m_compression_negotiation_client_style{
+      Compression_style::k_single, Compression_style::k_multiple,
+      Compression_style::k_group};
 
   Compression_negotiation m_compression_mode =
       Compression_negotiation::k_disabled;
@@ -89,11 +107,16 @@ class Capabilities_negotiator {
       }
     }
   }
-  bool was_chooses() const;
+
   bool is_compression_required() const;
+  bool was_chooses() const;
 
   Compression_algorithm m_choosen_algorithm = Compression_algorithm::k_none;
+  Compression_style m_choosen_client_style = Compression_style::k_none;
+  Compression_style m_choosen_server_style = Compression_style::k_none;
   std::string m_choosen_algorithm_txt;
+  std::string m_choosen_client_style_txt;
+  std::string m_choosen_server_style_txt;
 };
 
 }  // namespace xcl
