@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -32,12 +32,12 @@
 #include <limits>
 #include <string>
 
-#include "errmsg.h"     // NOLINT(build/include_subdir)
-#include "my_config.h"  // NOLINT(build/include_subdir)
-#include "my_dbug.h"    // NOLINT(build/include_subdir)
-#include "my_io.h"      // NOLINT(build/include_subdir)
-#include "mysql_com.h"  // NOLINT(build/include_subdir)
-#include "sha2.h"       // NOLINT(build/include_subdir)
+#include "errmsg.h"
+#include "my_config.h"
+#include "my_dbug.h"
+#include "my_io.h"
+#include "mysql_com.h"
+#include "sha2.h"
 
 #include "plugin/x/client/authentication/password_hasher.h"
 #include "plugin/x/client/authentication/sha256_scramble_generator.h"
@@ -69,8 +69,6 @@ const char *const ER_TEXT_RECEIVE_HANDLER_FAILED =
 const char *const ER_TEXT_NOTICE_HANDLER_FAILED =
     "Aborted by internal callback at send message processing";
 const char *const ER_TEXT_RECEIVE_BUFFER_TO_SMALL = "Receive buffer to small";
-const char *const ER_TEXT_COMPRESSION_NOT_CONFIGURED =
-    "Compression is disabled or required compression style was not selected";
 
 namespace details {
 
@@ -88,7 +86,8 @@ namespace details {
 
     @param stream   Protobuf zero output stream
 
-    @retval true stream has more data
+    @return
+      @retval == true stream has more data
   */
 bool has_data(ZeroCopyInputStream *stream) {
   const void *data;
@@ -112,16 +111,6 @@ bool is_timeout_error(const XError &error) {
   return (CR_X_READ_TIMEOUT == error.error());
 }
 
-bool is_compressed(const XProtocol::Header_message_type_id id) {
-  switch (id) {
-    case Mysqlx::ServerMessages::COMPRESSION:
-      return true;
-
-    default:
-      return false;
-  }
-}
-
 class Query_sequencer : public Query_instances {
  public:
   Instance_id instances_fetch_begin() override { return m_last_instance++; }
@@ -137,14 +126,6 @@ class Query_sequencer : public Query_instances {
   Instance_id m_last_instance{0};
 };
 
-inline size_t message_byte_size(const XProtocol::Message &msg) {
-#if (defined(GOOGLE_PROTOBUF_VERSION) && GOOGLE_PROTOBUF_VERSION > 3000000)
-  return msg.ByteSizeLong();
-#else
-  return msg.ByteSize();
-#endif
-}
-
 }  // namespace details
 
 Protocol_impl::Protocol_impl(std::shared_ptr<Context> context,
@@ -189,17 +170,6 @@ XError Protocol_impl::execute_authenticate(const std::string &user,
   return error;
 }
 
-void Protocol_impl::use_compression(const Compression_algorithm algo) {
-  DBUG_TRACE;
-  m_compression->reinitialize(algo);
-}
-
-void Protocol_impl::use_compression(const Compression_algorithm algo,
-                                    const int32_t level) {
-  DBUG_TRACE;
-  m_compression->reinitialize(algo, level);
-}
-
 std::unique_ptr<XProtocol::Capabilities>
 Protocol_impl::execute_fetch_capabilities(XError *out_error) {
   DBUG_TRACE;
@@ -325,10 +295,14 @@ XError Protocol_impl::authenticate_mysql41(const std::string &user,
     XError operator()(
         const std::string &user, const std::string &pass, const std::string &db,
         const Mysqlx::Session::AuthenticateContinue &auth_continue) {
+      std::string data;
       std::string password_hash;
+
+      Mysqlx::Session::AuthenticateContinue auth_continue_response;
+
       if (pass.length()) {
-        password_hash =
-            password_hasher::scramble(auth_continue.auth_data(), pass);
+        password_hash = password_hasher::scramble(
+            auth_continue.auth_data().c_str(), pass.c_str());
         password_hash = password_hasher::get_password_from_salt(password_hash);
 
         if (password_hash.empty()) {
@@ -336,12 +310,9 @@ XError Protocol_impl::authenticate_mysql41(const std::string &user,
         }
       }
 
-      std::string data;
       data.append(db).push_back('\0');    // authz
       data.append(user).push_back('\0');  // authc
       data.append(password_hash);         // pass
-
-      Mysqlx::Session::AuthenticateContinue auth_continue_response;
       auth_continue_response.set_auth_data(data);
 
       return m_protocol->send(auth_continue_response);
@@ -437,7 +408,7 @@ XError Protocol_impl::send(const Header_message_type_id mid,
   };
   /*
     Use dummy, otherwise g++ 4.4 reports: unused variable 'dummy'
-    [[maybe_unused]] did not work, so we must use it.
+    MY_ATTRIBUTE((unused)) did not work, so we must use it.
   */
   dummy = 0;
 
@@ -467,7 +438,7 @@ bool Protocol_impl::send_impl(const Client_message_type_id mid,
   const Header_message_type_id header_mesage_id = mid;
   const int header_message_type_size = sizeof(Header_message_type_id);
   const std::size_t header_whole_message_size =
-      details::message_byte_size(msg) + header_message_type_size;
+      msg.ByteSize() + header_message_type_size;
 
   cos.WriteLittleEndian32(header_whole_message_size);
   cos.WriteRaw(&header_mesage_id, header_message_type_size);
@@ -856,60 +827,6 @@ std::unique_ptr<XProtocol::Message> Protocol_impl::recv_single_message(
   }
 }
 
-XError Protocol_impl::send_compressed_frame(
-    const Client_message_type_id message_id, const Message &message) {
-  DBUG_TRACE;
-
-  return send_compressed_multiple_frames({{message_id, &message}});
-}
-
-XError Protocol_impl::send_compressed_multiple_frames(
-    const std::vector<std::pair<Client_message_type_id, const Message *>>
-        &messages) {
-  DBUG_TRACE;
-
-  std::string compressed_messages;
-  size_t total_size = 0;
-
-  {
-    for (const auto &message : messages)
-      total_size += details::message_byte_size(*message.second) + 5;
-
-    auto algo = m_compression->compression_algorithm();
-    if (algo) algo->set_pledged_source_size(total_size);
-
-    StringOutputStream out_stream(&compressed_messages);
-    auto compressed_out_stream = m_compression->uplink(&out_stream);
-
-    if (!compressed_out_stream) {
-      return XError{CR_X_COMPRESSION_NOT_CONFIGURED,
-                    ER_TEXT_COMPRESSION_NOT_CONFIGURED};
-    }
-
-    CodedOutputStream cos(compressed_out_stream.get());
-
-    for (const auto &message : messages) {
-      const auto msg_id = message.first;
-      const auto header_msg_id = static_cast<Header_message_type_id>(msg_id);
-      const auto msg = message.second;
-
-      dispatch_send_message(msg_id, *msg);
-
-      cos.WriteLittleEndian32(details::message_byte_size(*msg) + 1);
-      cos.WriteRaw(&header_msg_id, 1);
-      msg->SerializeToCodedStream(&cos);
-    }
-  }
-
-  Mysqlx::Connection::Compression compression;
-  compression.set_payload(compressed_messages);
-  DBUG_DUMP("compressed-payload", (const uint8_t *)compressed_messages.c_str(),
-            compressed_messages.length());
-  compression.set_uncompressed_size(total_size);
-
-  return send(Mysqlx::ClientMessages::COMPRESSION, compression);
-}
-
 std::unique_ptr<Protocol_impl::Message> Protocol_impl::alloc_message(
     const Header_message_type_id mid) {
   DBUG_TRACE;
@@ -955,8 +872,6 @@ std::unique_ptr<Protocol_impl::Message> Protocol_impl::alloc_message(
     case Mysqlx::ServerMessages::RESULTSET_FETCH_DONE_MORE_OUT_PARAMS:
       ret_val.reset(new Mysqlx::Resultset::FetchDoneMoreOutParams());
       break;
-    case Mysqlx::ServerMessages::COMPRESSION:
-      return nullptr;
   }
 
   return ret_val;
@@ -1011,75 +926,11 @@ XProtocol::Message *Protocol_impl::recv_id(
   return msg.release();
 }
 
-XProtocol::Message *Protocol_impl::read_compressed(Server_message_type_id *mid,
-                                                   XError *out_error) {
-  DBUG_TRACE;
-
-  if (nullptr == m_compressed_input_stream.get()) {
-    *out_error = XError{CR_X_COMPRESSION_NOT_CONFIGURED,
-                        ER_TEXT_COMPRESSION_NOT_CONFIGURED};
-    return nullptr;
-  }
-
-  std::unique_ptr<XProtocol::Message> message;
-
-  {
-    CodedInputStream cis(m_compressed_input_stream.get());
-
-    // Currently only XQuery_result class sets the m_context->m_global_error
-    // on invalid sequence of fetched resultsets.
-    //
-    // Fatal errors might be also set in m_global_error, where fatal errors
-    // might
-    //
-    // * IO errors
-    // * Server fatal Mysqlx.Error messages
-    // * compression errors
-    //
-    // This topic needs to be investigated
-    Header_message_type_id id;
-    uint32_t size;
-    cis.ReadLittleEndian32(&size);
-    cis.ReadRaw(&id, 1);
-    cis.PushLimit(size - 1);
-    DBUG_LOG("debug", "COMPRESSION_GROUP HEADER id:" << static_cast<int>(id)
-                                                     << ", size:" << size);
-    *mid = static_cast<Server_message_type_id>(id);
-
-    message = deserialize_message(*mid, &cis, out_error);
-
-    if (!*out_error) *out_error = m_connection_input_stream->GetIOError();
-  }
-
-  if (!details::has_data(m_compressed_input_stream.get())) {
-    DBUG_LOG("debug", "No more data in compressed packet,"
-                          << " removing compression stream");
-    m_compressed_input_stream.reset();
-    m_compressed_payload_input_stream.reset();
-    m_compressed.Clear();
-  }
-
-  if (*out_error) return nullptr;
-
-  return message.release();
-}
-
-void Protocol_impl::reset_buffering() {
-  m_connection_input_stream.reset(
-      new Connection_input_stream(m_connection.get()));
-}
-
 XProtocol::Message *Protocol_impl::recv_message_with_header(
     Server_message_type_id *mid, XError *out_error) {
   DBUG_TRACE;
   uint32_t payload_size = 0;
 
-  /* If the pointer is set, then we are in middle of reading compressed message.
-   */
-  if (m_compressed_input_stream) {
-    return read_compressed(mid, out_error);
-  }
-
   Header_message_type_id header_mid;
   *out_error = recv_header(&header_mid, &payload_size);
   *mid = static_cast<Server_message_type_id>(header_mid);
@@ -1090,68 +941,27 @@ XProtocol::Message *Protocol_impl::recv_message_with_header(
            "Reading X message of type: " << static_cast<int>(header_mid)
                                          << ", and size: " << payload_size);
 
-  const bool is_mid_compressed = details::is_compressed(header_mid);
-
   m_connection_input_stream->AllowedRead(payload_size);
 
-  {
-    // The 'cis' variable must be destroyed before doing read_buffered
-    google::protobuf::io::CodedInputStream cis(m_connection_input_stream.get());
+  // The 'cis' variable must be destroyed before doing read_buffered
+  google::protobuf::io::CodedInputStream cis(m_connection_input_stream.get());
 
-    cis.PushLimit(payload_size);
+  cis.PushLimit(payload_size);
 
-    if (!is_mid_compressed) {
-      DBUG_PRINT("info", ("Reading uncompressed X message"));
-      auto result = deserialize_message(header_mid, &cis, out_error);
+  DBUG_PRINT("info", ("Reading uncompressed X message"));
+  auto result = deserialize_message(header_mid, &cis, out_error);
 
-      if (!*out_error) {
-        *out_error = m_connection_input_stream->GetIOError();
-      }
-
-      if (*out_error) {
-        return nullptr;
-      }
-
-      *mid = static_cast<XProtocol::Server_message_type_id>(header_mid);
-
-      return result.release();
-    }
-
-    bool read_ok = true;
-
-    if (!m_compressed.ParseFromCodedStream(&cis)) {
-      std::string error_message(ERR_MSG_MESSAGE_NOT_INITIALIZED);
-      error_message += "Name:" + m_compressed.GetTypeName() + ", ";
-      error_message += m_compressed.InitializationErrorString();
-      *out_error = XError(CR_MALFORMED_PACKET, error_message);
-      return nullptr;
-    }
-
-    m_compression_inner_message_id = Mysqlx::ServerMessages::COMPRESSION;
-
-    if (!read_ok) {
-      *out_error = m_connection_input_stream->GetIOError();
-      return nullptr;
-    }
-
-    bool out_ignore = false;
-    *out_error =
-        dispatch_received(*mid, Mysqlx::Connection::Compression(), &out_ignore);
-
-    if (*out_error || out_ignore) {
-      skip_not_parsed(&cis, out_error);
-
-      return nullptr;
-    }
+  if (!*out_error) {
+    *out_error = m_connection_input_stream->GetIOError();
   }
-  const auto &payload = m_compressed.payload();
-  m_compressed_payload_input_stream.reset(
-      new google::protobuf::io::ArrayInputStream(payload.c_str(),
-                                                 payload.length()));
-  m_compressed_input_stream =
-      m_compression->downlink(m_compressed_payload_input_stream.get());
 
-  return read_compressed(mid, out_error);
+  if (*out_error) {
+    return nullptr;
+  }
+
+  *mid = static_cast<XProtocol::Server_message_type_id>(header_mid);
+
+  return result.release();
 }
 
 }  // namespace xcl
