@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -104,7 +103,6 @@ class Session_impl : public XSession {
   void close() override;
 
   Argument_uobject get_connect_attrs() const override;
-  bool is_protocol() const { return m_protocol.use_count(); }
 
  private:
   using Context_ptr = std::shared_ptr<Context>;
@@ -134,9 +132,7 @@ class Session_impl : public XSession {
                                        const char *, const uint32_t);
 
   std::pair<XError, std::vector<std::string>> validate_and_adjust_auth_methods(
-      const std::vector<Auth> &auth_methods, bool can_use_plain);
-  Argument_value get_compression_capability(
-      const bool include_compression_level = true) const;
+      std::vector<Auth> auth_methods, const bool can_use_plain);
 
   Argument_object m_required_capabilities;
   Argument_object m_optional_capabilities;
