@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -26,19 +26,15 @@
 #define PLUGIN_X_NGS_INCLUDE_NGS_INTERFACE_SQL_SESSION_INTERFACE_H_
 
 #include <string>
-#include "plugin/x/ngs/include/ngs/command_delegate.h"
 
 #include "plugin/x/ngs/include/ngs/interface/authentication_interface.h"
 #include "plugin/x/ngs/include/ngs/interface/resultset_interface.h"
-#include "plugin/x/ngs/include/ngs/protocol_encoder.h"
 #include "plugin/x/src/io/connection_type.h"
 
 struct PS_PARAM;
 
 namespace ngs {
 
-using Arg_list = ::google::protobuf::RepeatedPtrField<::Mysqlx::Datatypes::Any>;
-
 class Sql_session_interface {
  public:
   virtual ~Sql_session_interface() {}
@@ -58,6 +54,8 @@ class Sql_session_interface {
       bool allow_expired_passwords) = 0;
   virtual Error_code execute(const char *sql, std::size_t sql_len,
                              Resultset_interface *rset) = 0;
+  virtual Error_code execute_sql(const char *sql, std::size_t sql_len,
+                                 Resultset_interface *rset) = 0;
   virtual Error_code fetch_cursor(const std::uint32_t id,
                                   const std::uint32_t row_count,
                                   Resultset_interface *rset) = 0;
@@ -67,12 +65,13 @@ class Sql_session_interface {
                                           Resultset_interface *rset) = 0;
   virtual Error_code execute_prep_stmt(const uint32_t stmt_id,
                                        const bool has_cursor,
-                                       PS_PARAM *parameters,
+                                       const PS_PARAM *parameters,
                                        const std::size_t parameter_count,
                                        Resultset_interface *rset) = 0;
   virtual Error_code attach() = 0;
   virtual Error_code detach() = 0;
   virtual Error_code reset() = 0;
+  virtual bool is_sql_mode_set(const std::string &mode) = 0;
 };
 
 }  // namespace ngs
