@@ -31,30 +31,28 @@
 #include <string.h>
 #include <sys/types.h>
 #include <algorithm>
-#include <cinttypes>
 #include <functional>
 #include <sstream>
 
-#include "my_dbug.h"  // NOLINT(build/include_subdir)
+#include "my_dbug.h"
 
-#include "my_macros.h"   // NOLINT(build/include_subdir)
-#include "my_systime.h"  // my_sleep NOLINT(build/include_subdir)
+#include "my_macros.h"
+#include "my_systime.h"  // my_sleep
 
+#include "plugin/x/ngs/include/ngs/interface/protocol_monitor_interface.h"
+#include "plugin/x/ngs/include/ngs/interface/server_interface.h"
+#include "plugin/x/ngs/include/ngs/interface/session_interface.h"
+#include "plugin/x/ngs/include/ngs/interface/ssl_context_interface.h"
 #include "plugin/x/ngs/include/ngs/log.h"
 #include "plugin/x/ngs/include/ngs/protocol/protocol_config.h"
 #include "plugin/x/ngs/include/ngs/protocol/protocol_protobuf.h"
 #include "plugin/x/ngs/include/ngs/protocol_encoder.h"
 #include "plugin/x/ngs/include/ngs/scheduler.h"
-#include "plugin/x/src/capabilities/capability_compression.h"
 #include "plugin/x/src/capabilities/handler_auth_mech.h"
 #include "plugin/x/src/capabilities/handler_client_interactive.h"
 #include "plugin/x/src/capabilities/handler_connection_attributes.h"
 #include "plugin/x/src/capabilities/handler_readonly_value.h"
 #include "plugin/x/src/capabilities/handler_tls.h"
-#include "plugin/x/src/interface/protocol_monitor.h"
-#include "plugin/x/src/interface/server.h"
-#include "plugin/x/src/interface/session.h"
-#include "plugin/x/src/interface/ssl_context.h"
 #include "plugin/x/src/operations_factory.h"
 #include "plugin/x/src/xpl_error.h"
 #include "plugin/x/src/xpl_global_status_variables.h"
@@ -74,9 +72,9 @@ class No_idle_processing : public xpl::iface::Waiting_for_io {
 
 }  // namespace details
 
-Client::Client(std::shared_ptr<xpl::iface::Vio> connection,
-               xpl::iface::Server &server, Client_id client_id,
-               xpl::iface::Protocol_monitor *pmon,
+Client::Client(std::shared_ptr<Vio_interface> connection,
+               Server_interface &server, Client_id client_id,
+               Protocol_monitor_interface *pmon,
                const Global_timeouts &timeouts)
     : m_client_id(client_id),
       m_server(server),
@@ -157,8 +155,6 @@ xpl::Capabilities_configurator *Client::capabilities_configurator() {
   handlers.push_back(
       ngs::allocate_shared<xpl::Capability_connection_attributes>());
 
-  handlers.push_back(ngs::allocate_shared<xpl::Capability_compression>(this));
-
   return ngs::allocate_object<xpl::Capabilities_configurator>(handlers);
 }
 
@@ -314,9 +310,8 @@ void Client::on_read_timeout() {
   warning.set_msg("IO Read error: read_timeout exceeded");
   std::string warning_data;
   warning.SerializeToString(&warning_data);
-  m_encoder->send_notice(xpl::iface::Frame_type::k_warning,
-                         xpl::iface::Frame_scope::k_global, warning_data,
-                         force_flush);
+  m_encoder->send_notice(Frame_type::k_warning, Frame_scope::k_global,
+                         warning_data, force_flush);
 }
 
 // this will be called on socket errors, but also when halt_and_wait() is called
@@ -327,8 +322,8 @@ void Client::on_network_error(const int error) {
     set_close_reason_if_non_fatal(Close_reason::k_write_timeout);
   }
 
-  log_debug("%s, %" PRIu32 ": on_network_error(error:%i)", client_id(),
-            static_cast<uint32_t>(m_state.load()), error);
+  log_debug("%s, %i: on_network_error(error:%i)", client_id(),
+            static_cast<int>(m_state.load()), error);
 
   if (m_state != State::k_closing && error != 0)
     set_close_reason_if_non_fatal(Close_reason::k_net_error);
@@ -366,7 +361,7 @@ void Client::on_client_addr(const bool skip_resolve) {
 
   switch (m_connection->get_type()) {
     case xpl::Connection_tcpip: {
-      m_connection->peer_addr(&m_client_addr, &m_client_port);
+      m_connection->peer_addr(m_client_addr, m_client_port);
     } break;
 
     case xpl::Connection_namedpipe:
@@ -399,8 +394,8 @@ void Client::on_accept() {
             client_address());
 
   DBUG_EXECUTE_IF("client_accept_timeout", {
-    int32_t i = 0;
-    const int32_t max_iterations = 1000;
+    std::int32_t i = 0;
+    const std::int32_t max_iterations = 1000;
     while (m_server.is_running() && i < max_iterations) {
       my_sleep(10000);
       ++i;
@@ -412,7 +407,7 @@ void Client::on_accept() {
   // it can be accessed directly (no other thread access thus object)
   m_state = State::k_accepted;
 
-  set_encoder(ngs::allocate_object<Protocol_encoder>(
+  set_encoder(allocate_object<Protocol_encoder>(
       m_connection,
       std::bind(&Client::on_network_error, this, std::placeholders::_1),
       m_protocol_monitor, &m_memory_block_pool));
@@ -428,47 +423,31 @@ void Client::on_accept() {
   }
 
   if (xpl::Plugin_system_variables::m_enable_hello_notice)
-    m_encoder->send_notice(xpl::iface::Frame_type::k_server_hello,
-                           xpl::iface::Frame_scope::k_global, "", true);
+    m_encoder->send_notice(Frame_type::k_server_hello, Frame_scope::k_global,
+                           "", true);
 }
 
-void Client::on_session_auth_success(xpl::iface::Session *) {
-  log_debug("%s: on_session_auth_success", client_id());
+void Client::on_session_auth_success(Session_interface &) {
   // this is called from worker thread
   State expected = State::k_authenticating_first;
   m_state.compare_exchange_strong(expected, State::k_running);
-
-  if (Compression_algorithm::k_none != m_cached_compression_algorithm) {
-    Compression_style style = m_cached_combine_msg
-                                  ? Compression_style::k_group
-                                  : Compression_style::k_multiple;
-
-    if (m_cached_max_msg == 1) {
-      style = Compression_style::k_single;
-    }
-
-    get_protocol_compression_or_install_it()->set_compression_options(
-        m_cached_compression_algorithm, style, m_cached_max_msg);
-
-    m_config->m_compression_algorithm = m_cached_compression_algorithm;
-  }
 }
 
-void Client::on_session_close(xpl::iface::Session *s MY_ATTRIBUTE((unused))) {
-  log_debug("%s: Session %i removed", client_id(), s->session_id());
+void Client::on_session_close(Session_interface &s MY_ATTRIBUTE((unused))) {
+  log_debug("%s: Session %i removed", client_id(), s.session_id());
 
   // no more open sessions, disconnect
   disconnect_and_trigger_close();
 
-  if (s->state_before_close() != xpl::iface::Session::State::k_authenticating) {
+  if (s.state_before_close() != ngs::Session_interface::k_authenticating) {
     ++xpl::Global_status_variables::instance().m_closed_sessions_count;
   }
 
   remove_client_from_server();
 }
 
-void Client::on_session_reset(xpl::iface::Session *s MY_ATTRIBUTE((unused))) {
-  log_debug("%s: Resetting session %i", client_id(), s->session_id());
+void Client::on_session_reset(Session_interface &s MY_ATTRIBUTE((unused))) {
+  log_debug("%s: Resetting session %i", client_id(), s.session_id());
 
   if (!create_session()) {
     m_state = State::k_closing;
@@ -479,9 +458,9 @@ void Client::on_session_reset(xpl::iface::Session *s MY_ATTRIBUTE((unused))) {
 }
 
 void Client::on_server_shutdown() {
-  log_debug("%s: closing client because of shutdown (state: %" PRIu32 ")",
-            client_id(), static_cast<uint32_t>(m_state.load()));
-  std::shared_ptr<xpl::iface::Session> local_copy = m_session;
+  log_debug("%s: closing client because of shutdown (state: %i)", client_id(),
+            static_cast<int>(m_state.load()));
+  std::shared_ptr<ngs::Session_interface> local_copy = m_session;
 
   if (local_copy) local_copy->on_kill();
 
@@ -489,12 +468,12 @@ void Client::on_server_shutdown() {
   disconnect_and_trigger_close();
 }
 
-xpl::iface::Protocol_monitor &Client::get_protocol_monitor() {
+Protocol_monitor_interface &Client::get_protocol_monitor() {
   return *m_protocol_monitor;
 }
 
-void Client::set_encoder(xpl::iface::Protocol_encoder *enc) {
-  m_encoder.reset(enc);
+void Client::set_encoder(Protocol_encoder_interface *enc) {
+  m_encoder = Memory_instrumented<Protocol_encoder_interface>::Unique_ptr(enc);
   m_encoder->get_flusher()->set_write_timeout(m_write_timeout);
 
   if (m_session) m_session->set_proto(m_encoder.get());
@@ -576,30 +555,9 @@ xpl::iface::Waiting_for_io *Client::get_idle_processing() {
   return m_session->get_notice_output_queue().get_callbacks_waiting_for_io();
 }
 
-Protocol_encoder_compression *Client::get_protocol_compression_or_install_it() {
-  if (!m_is_compression_encoder_injected) {
-    m_is_compression_encoder_injected = true;
-    auto encoder = ngs::allocate_object<Protocol_encoder_compression>(
-        std::move(m_encoder), m_protocol_monitor,
-        std::bind(&Client::on_network_error, this, std::placeholders::_1),
-        &m_memory_block_pool);
-    set_encoder(encoder);
-  }
-
-  return reinterpret_cast<Protocol_encoder_compression *>(m_encoder.get());
-}
-
-void Client::configure_compression_opts(const Compression_algorithm algo,
-                                        const int64_t max_msg,
-                                        const bool combine) {
-  m_cached_compression_algorithm = algo;
-  m_cached_max_msg = max_msg;
-  m_cached_combine_msg = combine;
-}
-
 bool Client::create_session() {
-  std::shared_ptr<xpl::iface::Session> session(
-      m_server.create_session(this, m_encoder.get(), 1));
+  std::shared_ptr<Session_interface> session(
+      m_server.create_session(*this, *m_encoder, 1));
   if (!session) {
     log_warning(ER_XPLUGIN_FAILED_TO_CREATE_SESSION_FOR_CONN, client_id(),
                 m_client_addr.c_str());
