@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2019, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,13 +25,7 @@
 #ifndef PLUGIN_X_PROTOCOL_ENCODERS_ENCODING_XPROTOCOL_H_
 #define PLUGIN_X_PROTOCOL_ENCODERS_ENCODING_XPROTOCOL_H_
 
-#include "my_compiler.h"
-MY_COMPILER_DIAGNOSTIC_PUSH()
-// Suppress warning C4251 'type' : class 'type1' needs to have dll-interface
-// to be used by clients of class 'type2'
-MY_COMPILER_MSVC_DIAGNOSTIC_IGNORE(4251)
 #include <google/protobuf/wire_format_lite.h>
-MY_COMPILER_DIAGNOSTIC_POP()
 #include <cassert>
 #include <cstdint>
 #include <string>
@@ -43,6 +36,16 @@ MY_COMPILER_DIAGNOSTIC_POP()
 
 namespace protocol {
 
+namespace tags {
+
+enum Raw_payload_ids {
+  COMPRESSION_SINGLE = 19,
+  COMPRESSION_MULTIPLE = 20,
+  COMPRESSION_GROUP = 21,
+};
+
+}  // namespace tags
+
 enum class Compression_type { k_single, k_multiple, k_group };
 
 class Compression_buffer_interface {
@@ -136,8 +139,8 @@ class XProtocol_encoder : public Protobuf_encoder {
   struct Compression_position : Position {
     Encoding_buffer *m_compressed_buffer;
     Compression_type m_compression_type;
-    Delayed_fixed_varuint32 m_uncompressed_size;
-    Field_delimiter<5> m_payload;
+    uint32_t m_compressed_data_size;
+    uint32_t m_uncompressed_data_size;
     uint8_t m_msg_id;
   };
 
@@ -160,29 +163,31 @@ class XProtocol_encoder : public Protobuf_encoder {
                                          Encoding_buffer *to_compress) {
     Compression_position result;
 
-    result.m_msg_id = msg_id;
-    begin_xmessage<tags::Compression::server_id, 100>(&result);
-
     switch (type) {
       case Compression_type::k_single:
+        begin_xmessage<tags::Raw_payload_ids::COMPRESSION_SINGLE, 100>(&result);
+        set_header_config(Header_configuration::k_none);
+        m_buffer->m_current->m_current_data += 5;
+        break;
       case Compression_type::k_multiple:
-        set_header_config(Header_configuration::k_full);
-        encode_field_var_uint32<tags::Compression::server_messages>(msg_id);
+        begin_xmessage<tags::Raw_payload_ids::COMPRESSION_MULTIPLE, 100>(
+            &result);
+        set_header_config(Header_configuration::k_size_only);
+        m_buffer->m_current->m_current_data += 5;
         break;
       case Compression_type::k_group:
+        begin_xmessage<tags::Raw_payload_ids::COMPRESSION_GROUP, 100>(&result);
         set_header_config(Header_configuration::k_full);
+        m_buffer->m_current->m_current_data += 4;
         break;
     }
 
-    result.m_uncompressed_size =
-        encode_field_fixed_uint32<tags::Compression::uncompressed_size>();
-    begin_delimited_field<tags::Compression::payload>(&result.m_payload);
-
-    assert(to_compress->m_current == to_compress->m_front);
-    assert(to_compress->m_current->m_begin_data ==
-           to_compress->m_current->m_current_data);
+    DBUG_ASSERT(to_compress->m_current == to_compress->m_front);
+    DBUG_ASSERT(to_compress->m_current->m_begin_data ==
+                to_compress->m_current->m_current_data);
     result.m_compressed_buffer = m_buffer;
     result.m_compression_type = type;
+    result.m_msg_id = msg_id;
     // Reset buffer, and initialize the 'handy' data hold inside
     // 'Encoder_primitives'
     buffer_set(to_compress);
@@ -197,8 +202,6 @@ class XProtocol_encoder : public Protobuf_encoder {
     const auto before_compression_size =
         before_compression.bytes_until_page(m_page);
 
-    position.m_uncompressed_size.encode(before_compression_size);
-
     if (!compress->process(position.m_compressed_buffer, m_buffer))
       return false;
 
@@ -206,14 +209,39 @@ class XProtocol_encoder : public Protobuf_encoder {
     const auto message_size =
         position.bytes_until_page(position.m_compressed_buffer->m_current);
 
+    switch (position.m_compression_type) {
+      case Compression_type::k_single:
+        primitives::base::Fixint_length<4>::encode_value(ptr, message_size - 4);
+        primitives::base::Fixint_length<1>::encode_value(
+            ptr, tags::Raw_payload_ids::COMPRESSION_SINGLE);
+        primitives::base::Fixint_length<1>::encode_value(ptr,
+                                                         position.m_msg_id);
+        primitives::base::Fixint_length<4>::encode_value(
+            ptr, before_compression_size);
+        break;
+      case Compression_type::k_multiple:
+        primitives::base::Fixint_length<4>::encode_value(ptr, message_size - 4);
+        primitives::base::Fixint_length<1>::encode_value(
+            ptr, tags::Raw_payload_ids::COMPRESSION_MULTIPLE);
+        primitives::base::Fixint_length<1>::encode_value(ptr,
+                                                         position.m_msg_id);
+        primitives::base::Fixint_length<4>::encode_value(
+            ptr, before_compression_size);
+        break;
+      case Compression_type::k_group:
+        primitives::base::Fixint_length<4>::encode_value(ptr, message_size - 4);
+        primitives::base::Fixint_length<1>::encode_value(
+            ptr, tags::Raw_payload_ids::COMPRESSION_GROUP);
+        primitives::base::Fixint_length<4>::encode_value(
+            ptr, before_compression_size);
+        break;
+    }
     // Lets discard data inside new/compression buffer
     // in case when 'compress' call didn't do that.
     m_buffer->reset();
 
     // and now we restore original buffer
     buffer_set(position.m_compressed_buffer);
-    end_delimited_field(position.m_payload);
-    primitives::base::Fixint_length<4>::encode_value(ptr, message_size - 4);
 
     set_header_config(Header_configuration::k_full);
 
