@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -27,36 +27,220 @@
 #include <algorithm>
 #include <limits>
 
-#include "plugin/x/src/helper/sql_commands.h"
-#include "plugin/x/src/ngs/mysqlx/getter_any.h"
+#include "plugin/x/ngs/include/ngs/mysqlx/getter_any.h"
 #include "plugin/x/src/xpl_error.h"
 #include "plugin/x/src/xpl_regex.h"
 
 namespace xpl {
 
 namespace {
-using Argument_appearance = iface::Admin_command_arguments::Appearance_type;
+using Argument_appearance =
+    Admin_command_handler::Command_arguments::Appearance_type;
 inline bool is_optional(const Argument_appearance appearance) {
   return appearance == Argument_appearance::k_optional;
 }
+}  // namespace
 
+Admin_command_arguments_list::Admin_command_arguments_list(const List &args)
+    : m_args(args), m_current(m_args.begin()), m_args_consumed(0) {}
+
+Admin_command_arguments_list &Admin_command_arguments_list::string_arg(
+    Argument_name_list name, std::string *ret_value,
+    const Appearance_type appearance) {
+  if (check_scalar_arg(name, Mysqlx::Datatypes::Scalar::V_STRING, "string",
+                       appearance)) {
+    const std::string &value = m_current->scalar().v_string().value();
+    if (memchr(value.data(), 0, value.length())) {
+      m_error = ngs::Error(ER_X_CMD_ARGUMENT_VALUE,
+                           "Invalid value for argument '%s'", *name.begin());
+      return *this;
+    }
+    *ret_value = value;
+    ++m_current;
+  }
+  return *this;
+}
+
+Admin_command_arguments_list &Admin_command_arguments_list::string_list(
+    Argument_name_list name, std::vector<std::string> *ret_value,
+    const Appearance_type appearance) {
+  std::string value;
+  do {
+    string_arg(name, &value, appearance);
+    ret_value->push_back(value);
+    value.clear();
+  } while (!is_end());
+  return *this;
+}
+
+Admin_command_arguments_list &Admin_command_arguments_list::sint_arg(
+    Argument_name_list name, int64_t *ret_value,
+    const Appearance_type appearance) {
+  if (check_scalar_arg(name, Mysqlx::Datatypes::Scalar::V_SINT, "signed int",
+                       appearance)) {
+    if (m_current->scalar().type() == Mysqlx::Datatypes::Scalar::V_UINT)
+      *ret_value = (int64_t)m_current->scalar().v_unsigned_int();
+    else if (m_current->scalar().type() == Mysqlx::Datatypes::Scalar::V_SINT)
+      *ret_value = m_current->scalar().v_signed_int();
+    ++m_current;
+  }
+  return *this;
+}
+
+Admin_command_arguments_list &Admin_command_arguments_list::uint_arg(
+    Argument_name_list name, uint64_t *ret_value,
+    const Appearance_type appearance) {
+  if (check_scalar_arg(name, Mysqlx::Datatypes::Scalar::V_UINT, "unsigned int",
+                       appearance)) {
+    if (m_current->scalar().type() == Mysqlx::Datatypes::Scalar::V_UINT)
+      *ret_value = m_current->scalar().v_unsigned_int();
+    else if (m_current->scalar().type() == Mysqlx::Datatypes::Scalar::V_SINT)
+      *ret_value = (uint64_t)m_current->scalar().v_signed_int();
+    ++m_current;
+  }
+  return *this;
+}
+
+Admin_command_arguments_list &Admin_command_arguments_list::bool_arg(
+    Argument_name_list name, bool *ret_value,
+    const Appearance_type appearance) {
+  if (check_scalar_arg(name, Mysqlx::Datatypes::Scalar::V_BOOL, "bool",
+                       appearance)) {
+    *ret_value = m_current->scalar().v_bool();
+    ++m_current;
+  }
+  return *this;
+}
+
+Admin_command_arguments_list &Admin_command_arguments_list::docpath_arg(
+    Argument_name_list name, std::string *ret_value, const Appearance_type) {
+  m_args_consumed++;
+  if (!m_error) {
+    if (m_current == m_args.end()) {
+      m_error = ngs::Error(ER_X_CMD_NUM_ARGUMENTS, "Too few arguments");
+    } else {
+      if (m_current->type() == Mysqlx::Datatypes::Any::SCALAR &&
+          m_current->has_scalar() &&
+          (m_current->scalar().type() == Mysqlx::Datatypes::Scalar::V_STRING &&
+           m_current->scalar().has_v_string())) {
+        *ret_value = m_current->scalar().v_string().value();
+        // We could perform some extra validation on the document path here, but
+        // since the path will be quoted and escaped when used, it would be
+        // redundant.
+        // Plus, the best way to have the exact same syntax as the server
+        // is to let the server do it.
+        if (ret_value->empty() || ret_value->size() < 2)
+          m_error = ngs::Error(ER_X_CMD_ARGUMENT_VALUE,
+                               "Invalid document path value for argument %s",
+                               *name.begin());
+      } else {
+        arg_type_mismatch(*name.begin(), m_args_consumed,
+                          "document path string");
+      }
+    }
+    ++m_current;
+  }
+  return *this;
+}
+
+Admin_command_arguments_list &Admin_command_arguments_list::object_list(
+    Argument_name_list name, std::vector<Command_arguments *> *ret_value,
+    const Appearance_type, unsigned expected_members_count) {
+  List::difference_type left = m_args.end() - m_current;
+  if (left % expected_members_count > 0) {
+    m_error = ngs::Error(ER_X_CMD_NUM_ARGUMENTS,
+                         "Too few values for argument '%s'", *name.begin());
+    return *this;
+  }
+  for (unsigned i = 0; i < left / expected_members_count; ++i)
+    ret_value->push_back(this);
+  return *this;
+}
+
+bool Admin_command_arguments_list::is_end() const {
+  return !(m_error.error == 0 && m_args.size() > m_args_consumed);
+}
+
+const ngs::Error_code &Admin_command_arguments_list::end() {
+  if (m_error.error == ER_X_CMD_NUM_ARGUMENTS ||
+      (m_error.error == 0 && m_args.size() > m_args_consumed)) {
+    m_error = ngs::Error(ER_X_CMD_NUM_ARGUMENTS,
+                         "Invalid number of arguments, expected %i but got %i",
+                         m_args_consumed, m_args.size());
+  }
+  return m_error;
+}
+
+void Admin_command_arguments_list::arg_type_mismatch(const char *argname,
+                                                     int argpos,
+                                                     const char *type) {
+  m_error = ngs::Error(ER_X_CMD_ARGUMENT_TYPE,
+                       "Invalid type for argument '%s' at #%i (should be %s)",
+                       argname, argpos, type);
+}
+
+bool Admin_command_arguments_list::check_scalar_arg(
+    const Argument_name_list &argname, Mysqlx::Datatypes::Scalar::Type type,
+    const char *type_name, const Appearance_type appearance) {
+  m_args_consumed++;
+  if (!m_error) {
+    if (m_current == m_args.end()) {
+      if (!is_optional(appearance))
+        m_error = ngs::Error(ER_X_CMD_NUM_ARGUMENTS,
+                             "Insufficient number of arguments");
+    } else {
+      if (m_current->type() == Mysqlx::Datatypes::Any::SCALAR &&
+          m_current->has_scalar()) {
+        if (m_current->scalar().type() == type) {
+          // TODO(user): add charset check for strings?
+          // return true only if value to be consumed is available
+          return true;
+        } else if (type == Mysqlx::Datatypes::Scalar::V_SINT &&
+                   m_current->scalar().type() ==
+                       Mysqlx::Datatypes::Scalar::V_UINT &&
+                   m_current->scalar().v_unsigned_int() <
+                       static_cast<uint64_t>(
+                           std::numeric_limits<int64_t>::max())) {
+          return true;
+        } else if (type == Mysqlx::Datatypes::Scalar::V_UINT &&
+                   m_current->scalar().type() ==
+                       Mysqlx::Datatypes::Scalar::V_SINT &&
+                   m_current->scalar().v_signed_int() >= 0) {
+          return true;
+        } else {
+          if (!(is_optional(appearance) &&
+                m_current->scalar().type() ==
+                    Mysqlx::Datatypes::Scalar::V_NULL)) {
+            arg_type_mismatch(*argname.begin(), m_args_consumed, type_name);
+          }
+        }
+      } else {
+        arg_type_mismatch(*argname.begin(), m_args_consumed, type_name);
+      }
+      ++m_current;
+    }
+  }
+  return false;
+}
+
+namespace {
 using Object_field = ::Mysqlx::Datatypes::Object_ObjectField;
 
 template <typename T>
 class General_argument_validator {
  public:
-  explicit General_argument_validator(bool *) {}
+  General_argument_validator(const char *, bool *) {}
   void operator()(const T &input, T *output) { *output = input; }
 };
 
 template <typename Type, typename Validator = General_argument_validator<Type>>
 class Argument_type_handler {
  public:
-  explicit Argument_type_handler(Type *value)
-      : m_validator(&m_error), m_value(value), m_error(false) {}
+  Argument_type_handler(const char *name, Type *value)
+      : m_validator(name, &m_error), m_value(value), m_error(false) {}
 
-  Argument_type_handler()
-      : m_validator(&m_error), m_value(nullptr), m_error(false) {}
+  explicit Argument_type_handler(const char *name)
+      : m_validator(name, &m_error), m_value(nullptr), m_error(false) {}
 
   void assign(Type *value) { m_value = value; }
   void operator()(const Type &value) { m_validator(value, m_value); }
@@ -76,7 +260,8 @@ class Argument_type_handler {
 
 class String_argument_validator {
  public:
-  explicit String_argument_validator(bool *error) : m_error(error) {}
+  String_argument_validator(const char *name, bool *error)
+      : m_name(name), m_error(error) {}
 
   void operator()(const std::string &input, std::string *output) {
     if (memchr(input.data(), 0, input.length())) {
@@ -87,13 +272,14 @@ class String_argument_validator {
   }
 
  protected:
+  const char *m_name;
   bool *m_error;
 };
 
 class Docpath_argument_validator : String_argument_validator {
  public:
-  explicit Docpath_argument_validator(bool *error)
-      : String_argument_validator(error) {}
+  Docpath_argument_validator(const char *name, bool *error)
+      : String_argument_validator(name, error) {}
 
   void operator()(const std::string &input, std::string *output) {
     static const std::string k_doc_member_regex =
@@ -118,11 +304,15 @@ inline std::string get_indexed_name(const std::string &name, const int i) {
 Admin_command_arguments_object::Admin_command_arguments_object(const List &args)
     : m_args_empty(args.size() == 0),
       m_is_object(args.size() == 1 && args.Get(0).has_obj()),
-      m_object(m_is_object ? args.Get(0).obj() : Object::default_instance()) {}
+      m_object(m_is_object ? args.Get(0).obj() : Object::default_instance()),
+      m_args_consumed(0) {}
 
 Admin_command_arguments_object::Admin_command_arguments_object(
     const Object &obj)
-    : m_args_empty(true), m_is_object(true), m_object(obj) {}
+    : m_args_empty(true),
+      m_is_object(true),
+      m_object(obj),
+      m_args_consumed(0) {}
 
 void Admin_command_arguments_object::set_number_args_error(
     const std::string &name) {
@@ -138,14 +328,6 @@ void Admin_command_arguments_object::set_arg_value_error(
                  get_path(name).c_str());
 }
 
-void Admin_command_arguments_object::set_arg_type_error(
-    const std::string &name, const std::string &expected_type) {
-  m_error =
-      ngs::Error(ER_X_CMD_ARGUMENT_TYPE,
-                 "Invalid data type for argument '%s', expected '%s' type",
-                 name.c_str(), expected_type.c_str());
-}
-
 template <typename H>
 void Admin_command_arguments_object::get_scalar_arg(
     const Argument_name_list &name, const Appearance_type appearance,
@@ -162,6 +344,8 @@ Admin_command_arguments_object::get_object_field(
     const Argument_name_list &name, const Appearance_type appearance) {
   if (m_error) return nullptr;
 
+  ++m_args_consumed;
+
   if (!m_is_object) {
     if (!is_optional(appearance)) set_number_args_error(*name.begin());
     return nullptr;
@@ -174,7 +358,6 @@ Admin_command_arguments_object::get_object_field(
           return fld.has_key() && fld.key() == arg_name;
         });
     if (i != fld.end()) {
-      m_allowed_keys.push_back(i->key());
       return &(*i);
     }
   }
@@ -194,16 +377,16 @@ void Admin_command_arguments_object::get_scalar_value(const Any &value,
 }
 
 Admin_command_arguments_object &Admin_command_arguments_object::string_arg(
-    const Argument_name_list &name, std::string *ret_value,
+    Argument_name_list name, std::string *ret_value,
     const Appearance_type appearance) {
   Argument_type_handler<std::string, String_argument_validator> handler(
-      ret_value);
+      *name.begin(), ret_value);
   get_scalar_arg(name, appearance, &handler);
   return *this;
 }
 
 Admin_command_arguments_object &Admin_command_arguments_object::string_list(
-    const Argument_name_list &name, std::vector<std::string> *ret_value,
+    Argument_name_list name, std::vector<std::string> *ret_value,
     const Appearance_type appearance) {
   const Object::ObjectField *field = get_object_field(name, appearance);
   if (!field) return *this;
@@ -214,7 +397,8 @@ Admin_command_arguments_object &Admin_command_arguments_object::string_list(
   }
 
   std::vector<std::string> values;
-  Argument_type_handler<std::string, String_argument_validator> handler{};
+  Argument_type_handler<std::string, String_argument_validator> handler(
+      *name.begin());
 
   switch (field->value().type()) {
     case ::Mysqlx::Datatypes::Any_Type_ARRAY:
@@ -248,41 +432,42 @@ Admin_command_arguments_object &Admin_command_arguments_object::string_list(
 }
 
 Admin_command_arguments_object &Admin_command_arguments_object::sint_arg(
-    const Argument_name_list &name, int64_t *ret_value,
+    Argument_name_list name, int64_t *ret_value,
     const Appearance_type appearance) {
-  Argument_type_handler<google::protobuf::int64> handler(ret_value);
+  Argument_type_handler<google::protobuf::int64> handler(*name.begin(),
+                                                         ret_value);
   get_scalar_arg(name, appearance, &handler);
   return *this;
 }
 
 Admin_command_arguments_object &Admin_command_arguments_object::uint_arg(
-    const Argument_name_list &name, uint64_t *ret_value,
+    Argument_name_list name, uint64_t *ret_value,
     const Appearance_type appearance) {
-  Argument_type_handler<google::protobuf::uint64> handler(ret_value);
+  Argument_type_handler<google::protobuf::uint64> handler(*name.begin(),
+                                                          ret_value);
   get_scalar_arg(name, appearance, &handler);
   return *this;
 }
 
 Admin_command_arguments_object &Admin_command_arguments_object::bool_arg(
-    const Argument_name_list &name, bool *ret_value,
+    Argument_name_list name, bool *ret_value,
     const Appearance_type appearance) {
-  Argument_type_handler<bool> handler(ret_value);
+  Argument_type_handler<bool> handler(*name.begin(), ret_value);
   get_scalar_arg(name, appearance, &handler);
   return *this;
 }
 
 Admin_command_arguments_object &Admin_command_arguments_object::docpath_arg(
-    const Argument_name_list &name, std::string *ret_value,
+    Argument_name_list name, std::string *ret_value,
     const Appearance_type appearance) {
   Argument_type_handler<std::string, Docpath_argument_validator> handler(
-      ret_value);
+      *name.begin(), ret_value);
   get_scalar_arg(name, appearance, &handler);
   return *this;
 }
 
 Admin_command_arguments_object &Admin_command_arguments_object::object_list(
-    const Argument_name_list &name,
-    std::vector<iface::Admin_command_arguments *> *ret_value,
+    Argument_name_list name, std::vector<Command_arguments *> *ret_value,
     const Appearance_type appearance, unsigned) {
   const Object::ObjectField *field = get_object_field(name, appearance);
   if (!field) return *this;
@@ -292,7 +477,7 @@ Admin_command_arguments_object &Admin_command_arguments_object::object_list(
     return *this;
   }
 
-  std::vector<iface::Admin_command_arguments *> values;
+  std::vector<Command_arguments *> values;
   switch (field->value().type()) {
     case ::Mysqlx::Datatypes::Any_Type_ARRAY:
       if (field->value().array().value_size() == 0) {
@@ -325,38 +510,6 @@ Admin_command_arguments_object &Admin_command_arguments_object::object_list(
   return *this;
 }
 
-Admin_command_arguments_object &Admin_command_arguments_object::any_arg(
-    const Argument_name_list &name, Any *ret_value,
-    const Argument_appearance appearance) {
-  const Object::ObjectField *field = get_object_field(name, appearance);
-  if (!field) return *this;
-
-  if (!field->value().has_type()) {
-    set_number_args_error(*name.begin());
-    return *this;
-  }
-
-  if (!m_error) *ret_value = field->value();
-
-  return *this;
-}
-
-Admin_command_arguments_object &Admin_command_arguments_object::object_arg(
-    const Argument_name_list &name, Object *ret_value,
-    const Argument_appearance appearance) {
-  const Object::ObjectField *field = get_object_field(name, appearance);
-  if (!field) return *this;
-
-  if (field->value().type() != Mysqlx::Datatypes::Any_Type::Any_Type_OBJECT) {
-    set_arg_type_error(name[0], "object");
-    return *this;
-  }
-
-  if (!m_error) *ret_value = field->value().obj();
-
-  return *this;
-}
-
 Admin_command_arguments_object *Admin_command_arguments_object::add_sub_object(
     const Object &object, const std::string &path) {
   Admin_command_arguments_object *obj =
@@ -370,18 +523,11 @@ const ngs::Error_code &Admin_command_arguments_object::end() {
   if (m_error) return m_error;
 
   if (m_is_object) {
-    for (const auto &fld : m_object.fld()) {
-      if (fld.has_key()) {
-        const auto key = fld.key();
-        if (std::none_of(
-                std::begin(m_allowed_keys), std::end(m_allowed_keys),
-                [&key](const std::string &val) { return key == val; })) {
-          m_error = ngs::Error(ER_X_CMD_INVALID_ARGUMENT,
-                               "'%s' is not a valid field", key.c_str());
-          return m_error;
-        }
-      }
-    }
+    if (m_object.fld().size() > m_args_consumed)
+      m_error =
+          ngs::Error(ER_X_CMD_NUM_ARGUMENTS,
+                     "Invalid number of arguments, expected %i but got %i",
+                     m_args_consumed, m_object.fld().size());
   } else {
     if (!m_args_empty)
       m_error =
@@ -391,12 +537,9 @@ const ngs::Error_code &Admin_command_arguments_object::end() {
   return m_error;
 }
 
-std::vector<std::string> Admin_command_arguments_object::get_obj_keys() const {
-  std::vector<std::string> ret_value;
-  for (const auto &fld : m_object.fld()) {
-    if (fld.has_key()) ret_value.push_back(fld.key());
-  }
-  return ret_value;
+bool Admin_command_arguments_object::is_end() const {
+  return !(m_error.error == 0 && m_is_object &&
+           m_object.fld().size() > m_args_consumed);
 }
 
 }  // namespace xpl
