@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,14 +28,13 @@
 #include <cstring>
 #include <memory>
 
-#include "plugin/x/src/helper/generate_hash.h"
 #include "plugin/x/src/index_array_field.h"
 #include "plugin/x/src/index_field.h"
 #include "plugin/x/src/query_string_builder.h"
-#include "plugin/x/src/session.h"
 #include "plugin/x/src/sql_data_result.h"
 #include "plugin/x/src/xpl_error.h"
 #include "plugin/x/src/xpl_log.h"
+#include "plugin/x/src/xpl_session.h"
 
 namespace xpl {
 
@@ -125,7 +123,6 @@ std::string Admin_command_index::get_default_field_type(
  * - unique: bool - whether the index should be a unique index
  * - type: string, optional - name of index's type
  *   {"INDEX"|"SPATIAL"|"FULLTEXT"}
- * - with_parser: string, optional - name of parser for fulltext index
  * - fields|constraint: object, list - detailed information for the generated
  *   column
  *   - field|member: string - path to document member for which the index
@@ -144,7 +141,8 @@ std::string Admin_command_index::get_default_field_type(
  *   be usable unless queries also specify left(), which is not desired.
  */
 
-ngs::Error_code Admin_command_index::create(Command_arguments *args) {
+ngs::Error_code Admin_command_index::create(const std::string &name_space,
+                                            Command_arguments *args) {
   std::string schema;
   std::string collection;
   std::string index_name;
@@ -153,17 +151,33 @@ ngs::Error_code Admin_command_index::create(Command_arguments *args) {
   bool is_unique = false;
   std::vector<Command_arguments *> constraints;
 
-  ngs::Error_code error =
-      args->string_arg({"schema"}, &schema, Argument_appearance::k_obligatory)
-          .string_arg({"collection"}, &collection,
-                      Argument_appearance::k_obligatory)
-          .string_arg({"name"}, &index_name, Argument_appearance::k_obligatory)
-          .bool_arg({"unique"}, &is_unique, Argument_appearance::k_obligatory)
-          .string_arg({"type"}, &index_type, Argument_appearance::k_optional)
-          .string_arg({"with_parser"}, &parser, Argument_appearance::k_optional)
-          .object_list({"fields", "constraint"}, &constraints,
-                       Argument_appearance::k_obligatory)
-          .error();
+  ngs::Error_code error;
+  if (name_space == Admin_command_handler::k_mysqlx_namespace)
+    error =
+        args->string_arg({"schema"}, &schema, Argument_appearance::k_obligatory)
+            .string_arg({"collection"}, &collection,
+                        Argument_appearance::k_obligatory)
+            .string_arg({"name"}, &index_name,
+                        Argument_appearance::k_obligatory)
+            .bool_arg({"unique"}, &is_unique, Argument_appearance::k_obligatory)
+            .string_arg({"type"}, &index_type, Argument_appearance::k_optional)
+            .string_arg({"with_parser"}, &parser,
+                        Argument_appearance::k_optional)
+            .object_list({"fields", "constraint"}, &constraints,
+                         Argument_appearance::k_obligatory)
+            .error();
+  else
+    error =
+        args->string_arg({"schema"}, &schema, Argument_appearance::k_obligatory)
+            .string_arg({"collection"}, &collection,
+                        Argument_appearance::k_obligatory)
+            .string_arg({"name"}, &index_name,
+                        Argument_appearance::k_obligatory)
+            .bool_arg({"unique"}, &is_unique, Argument_appearance::k_obligatory)
+            .object_list({"constraint"}, &constraints,
+                         Argument_appearance::k_obligatory)
+            .error();
+
   if (error) return error;
 
   if (schema.empty())
@@ -210,7 +224,8 @@ ngs::Error_code Admin_command_index::create(Command_arguments *args) {
   using Fields = std::vector<std::unique_ptr<const Index_field_interface>>;
   Fields fields;
   for (auto c : constraints) {
-    fields.emplace_back(create_field(virtual_supported, type_id, c, &error));
+    fields.emplace_back(
+        create_field(name_space, virtual_supported, type_id, c, &error));
     if (error) return error;
   }
   error = args->end();
@@ -327,7 +342,8 @@ ngs::Error_code Admin_command_index::get_index_generated_column_names(
  * - collection: string - name of collection with dropped index
  * - schema: string - name of collection's schema
  */
-ngs::Error_code Admin_command_index::drop(Command_arguments *args) {
+ngs::Error_code Admin_command_index::drop(const std::string & /*name_space*/,
+                                          Command_arguments *args) {
   Query_string_builder qb;
   std::string schema;
   std::string collection;
@@ -384,24 +400,36 @@ ngs::Error_code Admin_command_index::drop(Command_arguments *args) {
 }
 
 const Admin_command_index::Index_field_interface *
-Admin_command_index::create_field(const bool is_virtual_allowed,
+Admin_command_index::create_field(const std::string &name_space,
+                                  const bool is_virtual_allowed,
                                   const Index_type_id &index_type,
                                   Command_arguments *constraint,
                                   ngs::Error_code *error) const {
   Index_field_info info;
   bool is_array{false};
-  *error =
-      constraint
-          ->docpath_arg({"field", "member"}, &info.m_path,
-                        Argument_appearance::k_obligatory)
-          .string_arg({"type"}, &info.m_type, Argument_appearance::k_optional)
-          .bool_arg({"required"}, &info.m_is_required,
-                    Argument_appearance::k_optional)
-          .uint_arg({"options"}, &info.m_options,
-                    Argument_appearance::k_optional)
-          .uint_arg({"srid"}, &info.m_srid, Argument_appearance::k_optional)
-          .bool_arg({"array"}, &is_array, Argument_appearance::k_optional)
-          .error();
+  if (name_space == Admin_command_handler::k_mysqlx_namespace)
+    *error =
+        constraint
+            ->docpath_arg({"field", "member"}, &info.m_path,
+                          Argument_appearance::k_obligatory)
+            .string_arg({"type"}, &info.m_type, Argument_appearance::k_optional)
+            .bool_arg({"required"}, &info.m_is_required,
+                      Argument_appearance::k_optional)
+            .uint_arg({"options"}, &info.m_options,
+                      Argument_appearance::k_optional)
+            .uint_arg({"srid"}, &info.m_srid, Argument_appearance::k_optional)
+            .bool_arg({"array"}, &is_array, Argument_appearance::k_optional)
+            .error();
+  else
+    *error =
+        constraint
+            ->docpath_arg({"member"}, &info.m_path,
+                          Argument_appearance::k_obligatory)
+            .string_arg({"type"}, &info.m_type, Argument_appearance::k_optional)
+            .bool_arg({"required"}, &info.m_is_required,
+                      Argument_appearance::k_obligatory)
+            .error();
+
   if (*error) return nullptr;
 
   if (info.m_type.empty())
