@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -24,6 +24,8 @@
 
 #include "plugin/x/src/callback_command_delegate.h"
 
+#include <string>
+
 #include <stddef.h>
 
 #include "plugin/x/ngs/include/ngs/memory.h"
@@ -41,28 +43,6 @@ Callback_command_delegate::Field_value::Field_value(const Field_value &other)
   if (other.is_string) value.v_string = new std::string(*other.value.v_string);
 }
 
-/*
-NOTE: Commented for coverage. Uncomment when needed.
-
-Callback_command_delegate::Field_value&
-Callback_command_delegate::Field_value::operator = (const Field_value& other)
-{
-  if (&other != this)
-  {
-    this->~Field_value();
-
-    value = other.value;
-    is_unsigned = other.is_unsigned;
-    is_string = other.is_string;
-
-    if (other.is_string)
-      value.v_string = new std::string(*other.value.v_string);
-  }
-
-  return *this;
-}
-*/
-
 Callback_command_delegate::Field_value::Field_value(const longlong &num,
                                                     bool unsign) {
   value.v_long = num;
@@ -102,9 +82,7 @@ Callback_command_delegate::Field_value::~Field_value() {
 Callback_command_delegate::Row_data::~Row_data() { clear(); }
 
 void Callback_command_delegate::Row_data::clear() {
-  std::vector<Field_value *>::iterator i = fields.begin();
-
-  for (; i != fields.end(); ++i) ngs::free_object(*i);
+  for (auto f : fields) ngs::free_object(f);
 
   fields.clear();
 }
@@ -125,10 +103,9 @@ operator=(const Row_data &other) {
 
 void Callback_command_delegate::Row_data::clone_fields(const Row_data &other) {
   fields.reserve(other.fields.size());
-  std::vector<Field_value *>::const_iterator i = other.fields.begin();
-  for (; i != other.fields.end(); ++i) {
-    this->fields.push_back((*i) ? ngs::allocate_object<Field_value>(**i)
-                                : NULL);
+
+  for (auto f : other.fields) {
+    this->fields.push_back(f ? ngs::allocate_object<Field_value>(*f) : nullptr);
   }
 }
 
@@ -153,8 +130,9 @@ int Callback_command_delegate::start_row() {
   if (m_start_row) {
     m_current_row = m_start_row();
     if (!m_current_row) return true;
-  } else
+  } else {
     m_current_row = nullptr;
+  }
   return false;
 }
 
@@ -173,7 +151,7 @@ ulong Callback_command_delegate::get_client_capabilities() {
 int Callback_command_delegate::get_null() {
   try {
     if (m_current_row) m_current_row->fields.push_back(NULL);
-  } catch (std::exception &e) {
+  } catch (std::exception &DEBUG_VAR(e)) {
     log_debug("Error getting result data: %s", e.what());
     return true;
   }
@@ -184,7 +162,7 @@ int Callback_command_delegate::get_integer(longlong value) {
   try {
     if (m_current_row)
       m_current_row->fields.push_back(ngs::allocate_object<Field_value>(value));
-  } catch (std::exception &e) {
+  } catch (std::exception &DEBUG_VAR(e)) {
     log_debug("Error getting result data: %s", e.what());
     return true;
   }
@@ -197,7 +175,7 @@ int Callback_command_delegate::get_longlong(longlong value,
     if (m_current_row)
       m_current_row->fields.push_back(
           ngs::allocate_object<Field_value>(value, unsigned_flag));
-  } catch (std::exception &e) {
+  } catch (std::exception &DEBUG_VAR(e)) {
     log_debug("Error getting result data: %s", e.what());
     return true;
   }
@@ -209,7 +187,7 @@ int Callback_command_delegate::get_decimal(const decimal_t *value) {
     if (m_current_row)
       m_current_row->fields.push_back(
           ngs::allocate_object<Field_value>(*value));
-  } catch (std::exception &e) {
+  } catch (std::exception &DEBUG_VAR(e)) {
     log_debug("Error getting result data: %s", e.what());
     return true;
   }
@@ -220,7 +198,7 @@ int Callback_command_delegate::get_double(double value, uint32) {
   try {
     if (m_current_row)
       m_current_row->fields.push_back(ngs::allocate_object<Field_value>(value));
-  } catch (std::exception &e) {
+  } catch (std::exception &DEBUG_VAR(e)) {
     log_debug("Error getting result data: %s", e.what());
     return true;
   }
@@ -232,7 +210,7 @@ int Callback_command_delegate::get_date(const MYSQL_TIME *value) {
     if (m_current_row)
       m_current_row->fields.push_back(
           ngs::allocate_object<Field_value>(*value));
-  } catch (std::exception &e) {
+  } catch (std::exception &DEBUG_VAR(e)) {
     log_debug("Error getting result data: %s", e.what());
     return true;
   }
@@ -244,7 +222,7 @@ int Callback_command_delegate::get_time(const MYSQL_TIME *value, uint) {
     if (m_current_row)
       m_current_row->fields.push_back(
           ngs::allocate_object<Field_value>(*value));
-  } catch (std::exception &e) {
+  } catch (std::exception &DEBUG_VAR(e)) {
     log_debug("Error getting result data: %s", e.what());
     return true;
   }
@@ -256,7 +234,7 @@ int Callback_command_delegate::get_datetime(const MYSQL_TIME *value, uint) {
     if (m_current_row)
       m_current_row->fields.push_back(
           ngs::allocate_object<Field_value>(*value));
-  } catch (std::exception &e) {
+  } catch (std::exception &DEBUG_VAR(e)) {
     log_debug("Error getting result data: %s", e.what());
     return true;
   }
@@ -270,7 +248,7 @@ int Callback_command_delegate::get_string(const char *const value,
     if (m_current_row)
       m_current_row->fields.push_back(
           ngs::allocate_object<Field_value>(value, length));
-  } catch (std::exception &e) {
+  } catch (std::exception &DEBUG_VAR(e)) {
     log_debug("Error getting result data: %s", e.what());
     return true;
   }
