@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,34 +22,32 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
-#include <string>
-
 #include "plugin/x/src/crud_cmd_handler.h"
 
+#include "plugin/x/ngs/include/ngs/interface/client_interface.h"
+#include "plugin/x/ngs/include/ngs/interface/document_id_generator_interface.h"
+#include "plugin/x/ngs/include/ngs/interface/server_interface.h"
+#include "plugin/x/ngs/include/ngs/protocol/protocol_protobuf.h"
 #include "plugin/x/src/delete_statement_builder.h"
 #include "plugin/x/src/expr_generator.h"
 #include "plugin/x/src/find_statement_builder.h"
-#include "plugin/x/src/get_detailed_validation_error.h"
 #include "plugin/x/src/insert_statement_builder.h"
-#include "plugin/x/src/interface/client.h"
-#include "plugin/x/src/interface/document_id_generator.h"
-#include "plugin/x/src/interface/server.h"
-#include "plugin/x/src/ngs/protocol/protocol_protobuf.h"
 #include "plugin/x/src/notices.h"
-#include "plugin/x/src/session.h"
 #include "plugin/x/src/sql_data_result.h"
 #include "plugin/x/src/update_statement_builder.h"
 #include "plugin/x/src/view_statement_builder.h"
 #include "plugin/x/src/xpl_error.h"
 #include "plugin/x/src/xpl_log.h"
 #include "plugin/x/src/xpl_resultset.h"
+#include "plugin/x/src/xpl_session.h"
 
 namespace xpl {
 
 template <typename B, typename M>
 ngs::Error_code Crud_command_handler::execute(
-    const B &builder, const M &msg, iface::Resultset &resultset,
-    Status_variable variable, bool (iface::Protocol_encoder::*send_ok)()) {
+    const B &builder, const M &msg, ngs::Resultset_interface &resultset,
+    Status_variable variable,
+    bool (ngs::Protocol_encoder_interface::*send_ok)()) {
   m_session->update_status(variable);
   m_qb.clear();
   try {
@@ -70,18 +67,18 @@ ngs::Error_code Crud_command_handler::execute(
 }
 
 template <typename B, typename M>
-void Crud_command_handler::notice_handling(const iface::Resultset::Info &info,
-                                           const B & /*builder*/,
-                                           const M & /*msg*/) const {
+void Crud_command_handler::notice_handling(
+    const ngs::Resultset_interface::Info &info, const B & /*builder*/,
+    const M & /*msg*/) const {
   notice_handling_common(info);
 }
 
 void Crud_command_handler::notice_handling_common(
-    const iface::Resultset::Info &info) const {
+    const ngs::Resultset_interface::Info &info) const {
   const auto &notice_config = m_session->get_notice_configuration();
   if (info.num_warnings > 0 &&
       notice_config.is_notice_enabled(ngs::Notice_type::k_warning))
-    notices::send_warnings(&m_session->data_context(), &m_session->proto());
+    notices::send_warnings(m_session->data_context(), m_session->proto());
 
   if (!info.message.empty())
     m_session->proto().send_notice_txt_message(info.message);
@@ -98,17 +95,16 @@ inline bool check_message(const std::string &msg, const char *pattern,
 ngs::Error_code Crud_command_handler::execute_crud_insert(
     const Mysqlx::Crud::Insert &msg) {
   auto &id_agg = m_session->get_document_id_aggregator();
-  iface::Document_id_aggregator::Retention_guard g(&id_agg);
+  ngs::Document_id_aggregator_interface::Retention_guard g(&id_agg);
   ngs::Error_code error = id_agg.configue(&m_session->data_context());
   if (error) return error;
 
-  const auto is_relational = is_table_data_model(msg);
   Expression_generator gen(&m_qb, msg.args(), msg.collection().schema(),
-                           is_relational);
+                           is_table_data_model(msg));
   Empty_resultset rset;
   return execute(Insert_statement_builder(gen, &id_agg), msg, rset,
                  &ngs::Common_status_variables::m_crud_insert,
-                 &iface::Protocol_encoder::send_exec_ok);
+                 &ngs::Protocol_encoder_interface::send_exec_ok);
 }
 
 template <>
@@ -127,21 +123,23 @@ ngs::Error_code Crud_command_handler::error_handling(
                         msg.collection().name().c_str());
 
     case ER_DUP_ENTRY:
-    case ER_X_BAD_UPSERT_DATA:
       return ngs::Error(
-          ER_X_DUPLICATE_ENTRY,
+          ER_X_DOC_ID_DUPLICATE,
           "Document contains a field value that is not unique but "
           "required to be");
 
-    case ER_CHECK_CONSTRAINT_VIOLATED:
-      return get_detailed_validation_error(m_session->data_context());
+    case ER_X_BAD_UPSERT_DATA:
+      return ngs::Error(ER_X_BAD_UPSERT_DATA,
+                        "Unable upsert data in document collection '%s'",
+                        msg.collection().name().c_str());
   }
   return error;
 }
 
 template <>
 void Crud_command_handler::notice_handling(
-    const iface::Resultset::Info &info, const Insert_statement_builder &builder,
+    const ngs::Resultset_interface::Info &info,
+    const Insert_statement_builder &builder,
     const Mysqlx::Crud::Insert &msg) const {
   notice_handling_common(info);
   m_session->proto().send_notice_rows_affected(info.affected_rows);
@@ -157,13 +155,12 @@ void Crud_command_handler::notice_handling(
 // -- Update
 ngs::Error_code Crud_command_handler::execute_crud_update(
     const Mysqlx::Crud::Update &msg) {
-  const auto is_relational = is_table_data_model(msg);
   Expression_generator gen(&m_qb, msg.args(), msg.collection().schema(),
-                           is_relational);
+                           is_table_data_model(msg));
   Empty_resultset rset;
   return execute(Update_statement_builder(gen), msg, rset,
                  &ngs::Common_status_variables::m_crud_update,
-                 &iface::Protocol_encoder::send_exec_ok);
+                 &ngs::Protocol_encoder_interface::send_exec_ok);
 }
 
 template <>
@@ -180,16 +177,13 @@ ngs::Error_code Crud_command_handler::error_handling(
       return ngs::Error(ER_X_BAD_UPDATE_DATA,
                         "Invalid data for update operation on "
                         "document collection table");
-
-    case ER_CHECK_CONSTRAINT_VIOLATED:
-      return get_detailed_validation_error(m_session->data_context());
   }
   return error;
 }
 
 template <>
 void Crud_command_handler::notice_handling(
-    const iface::Resultset::Info &info,
+    const ngs::Resultset_interface::Info &info,
     const Update_statement_builder & /*builder*/,
     const Mysqlx::Crud::Update & /*msg*/) const {
   notice_handling_common(info);
@@ -199,18 +193,17 @@ void Crud_command_handler::notice_handling(
 // -- Delete
 ngs::Error_code Crud_command_handler::execute_crud_delete(
     const Mysqlx::Crud::Delete &msg) {
-  const auto is_relational = is_table_data_model(msg);
   Expression_generator gen(&m_qb, msg.args(), msg.collection().schema(),
-                           is_relational);
+                           is_table_data_model(msg));
   Empty_resultset rset;
   return execute(Delete_statement_builder(gen), msg, rset,
                  &ngs::Common_status_variables::m_crud_delete,
-                 &iface::Protocol_encoder::send_exec_ok);
+                 &ngs::Protocol_encoder_interface::send_exec_ok);
 }
 
 template <>
 void Crud_command_handler::notice_handling(
-    const iface::Resultset::Info &info,
+    const ngs::Resultset_interface::Info &info,
     const Delete_statement_builder & /*builder*/,
     const Mysqlx::Crud::Delete & /*msg*/) const {
   notice_handling_common(info);
@@ -220,9 +213,8 @@ void Crud_command_handler::notice_handling(
 // -- Find
 ngs::Error_code Crud_command_handler::execute_crud_find(
     const Mysqlx::Crud::Find &msg) {
-  const auto is_relational = is_table_data_model(msg);
   Expression_generator gen(&m_qb, msg.args(), msg.collection().schema(),
-                           is_relational);
+                           is_table_data_model(msg));
   Streaming_resultset<Crud_command_delegate> rset(m_session, false);
   return execute(Find_statement_builder(gen), msg, rset,
                  &ngs::Common_status_variables::m_crud_find, nullptr);
@@ -230,7 +222,7 @@ ngs::Error_code Crud_command_handler::execute_crud_find(
 
 template <>
 void Crud_command_handler::notice_handling(
-    const iface::Resultset::Info &info,
+    const ngs::Resultset_interface::Info &info,
     const Find_statement_builder & /*builder*/,
     const Mysqlx::Crud::Find & /*msg*/) const {}
 
@@ -266,7 +258,7 @@ ngs::Error_code Crud_command_handler::execute_create_view(
   Empty_resultset rset;
   return execute(View_statement_builder(gen), msg, rset,
                  &ngs::Common_status_variables::m_crud_create_view,
-                 &iface::Protocol_encoder::send_ok);
+                 &ngs::Protocol_encoder_interface::send_ok);
 }
 
 ngs::Error_code Crud_command_handler::execute_modify_view(
@@ -276,7 +268,7 @@ ngs::Error_code Crud_command_handler::execute_modify_view(
   Empty_resultset rset;
   return execute(View_statement_builder(gen), msg, rset,
                  &ngs::Common_status_variables::m_crud_modify_view,
-                 &iface::Protocol_encoder::send_ok);
+                 &ngs::Protocol_encoder_interface::send_ok);
 }
 
 ngs::Error_code Crud_command_handler::execute_drop_view(
@@ -286,7 +278,7 @@ ngs::Error_code Crud_command_handler::execute_drop_view(
   Empty_resultset rset;
   return execute(View_statement_builder(gen), msg, rset,
                  &ngs::Common_status_variables::m_crud_drop_view,
-                 &iface::Protocol_encoder::send_ok);
+                 &ngs::Protocol_encoder_interface::send_ok);
 }
 
 }  // namespace xpl
