@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,7 @@
 #include <string>
 #include <utility>
 
+#include "plugin/x/src/helper/string_case.h"
 #include "plugin/x/src/helper/to_string.h"
 #include "plugin/x/src/json_utils.h"
 #include "plugin/x/src/mysql_function_names.h"
@@ -36,6 +37,7 @@
 #include "plugin/x/src/xpl_regex.h"
 
 namespace xpl {
+using Placeholder_type = Placeholder_info::Type;
 
 Expression_generator::Error::Error(int error_code, const std::string &message)
     : std::invalid_argument(message), m_error(error_code) {}
@@ -271,24 +273,18 @@ void Expression_generator::generate(
   }
 }
 
-void Expression_generator::generate_placeholder(
-    const Placeholder &arg,
-    void (Expression_generator::*generate_fun)(
-        const Mysqlx::Datatypes::Scalar &) const) const {
+void Expression_generator::generate(const Placeholder &arg) const {
   if (arg < static_cast<Placeholder>(m_args.size())) {
-    (this->*generate_fun)(m_args.Get(arg));
+    generate(m_args.Get(arg));
     return;
   }
   if (!is_prep_stmt_mode())
     throw Error(ER_X_EXPR_BAD_VALUE, "Invalid value of placeholder");
-  m_placeholder_ids->push_back(arg - static_cast<Placeholder>(m_args.size()));
+  m_placeholders->emplace_back(arg - static_cast<Placeholder>(m_args.size()),
+                               Placeholder_type::k_raw);
   m_qb->put("?");
 }
 
-void Expression_generator::generate(const Placeholder &arg) const {
-  generate_placeholder(arg, &Expression_generator::generate);
-}
-
 void Expression_generator::generate(const Mysqlx::Expr::Object &arg) const {
   m_qb->put("JSON_OBJECT(");
   generate_for_each(arg.fld(), &Expression_generator::generate);
@@ -463,40 +459,40 @@ void Expression_generator::generate_json_literal_param(
   }
 }
 
-void Expression_generator::generate_cont_in_param(
-    const Mysqlx::Expr::Expr &arg) const {
+void Expression_generator::generate_json_only_param(
+    const Mysqlx::Expr::Expr &arg, const std::string &expr_name) const {
   switch (arg.type()) {
-    case Mysqlx::Expr::Expr::IDENT:
-      if (arg.identifier().document_path_size() < 1)
-        throw Error(ER_X_EXPR_BAD_VALUE,
-                    "CONT_IN expression requires identifier"
-                    " that produce a JSON value.");
-      generate(arg);
-      break;
-
     case Mysqlx::Expr::Expr::LITERAL:
       generate_json_literal_param(arg.literal());
       break;
 
     case Mysqlx::Expr::Expr::FUNC_CALL:
       if (!is_json_function_call(arg.function_call()))
-        throw Error(ER_X_EXPR_BAD_VALUE,
-                    "CONT_IN expression requires function"
-                    " that produce a JSON value.");
+        throw Error(ER_X_EXPR_BAD_VALUE, expr_name +
+                                             " expression requires function"
+                                             " that produce a JSON value.");
       generate(arg);
       break;
 
     case Mysqlx::Expr::Expr::OPERATOR:
       if (!is_cast_to_json(arg.operator_()))
-        throw Error(ER_X_EXPR_BAD_VALUE,
-                    "CONT_IN expression requires operator"
-                    " that produce a JSON value.");
+        throw Error(ER_X_EXPR_BAD_VALUE, expr_name +
+                                             " expression requires operator"
+                                             " that produce a JSON value.");
       generate(arg);
       break;
 
     case Mysqlx::Expr::Expr::PLACEHOLDER:
-      generate_placeholder(arg.position(),
-                           &Expression_generator::generate_json_literal_param);
+      if (arg.position() < static_cast<Placeholder>(m_args.size())) {
+        generate_json_literal_param(m_args.Get(arg.position()));
+        break;
+      }
+      if (!is_prep_stmt_mode())
+        throw Error(ER_X_EXPR_BAD_VALUE, "Invalid value of placeholder");
+      m_placeholders->emplace_back(
+          arg.position() - static_cast<Placeholder>(m_args.size()),
+          Placeholder_type::k_json);
+      m_qb->put("CAST(? AS JSON)");
       break;
 
     default:
@@ -511,9 +507,9 @@ void Expression_generator::cont_in_expression(const Mysqlx::Expr::Operator &arg,
                 "CONT_IN expression requires two parameters.");
 
   m_qb->put(str).put("JSON_CONTAINS(");
-  generate_cont_in_param(arg.param(1));
+  generate_json_only_param(arg.param(1), "CONT_IN");
   m_qb->put(",");
-  generate_cont_in_param(arg.param(0));
+  generate_json_only_param(arg.param(0), "CONT_IN");
   m_qb->put(")");
 }
 
@@ -594,14 +590,12 @@ struct Cast_type_validator {
       : m_error_msg(error_msg) {}
 
   bool operator()(const char *str) const {
-    static const Regex re(
-        "^("
-        "BINARY(\\([[:digit:]]+\\))?|"
+    static const xpl::Regex re(
+        "BINARY(?:\\([[:digit:]]+\\))?|"
         "DATE|DATETIME|TIME|JSON|"
-        "CHAR(\\([[:digit:]]+\\))?|"
-        "DECIMAL(\\([[:digit:]]+(,[[:digit:]]+)?\\))?|"
-        "SIGNED( INTEGER)?|UNSIGNED( INTEGER)?"
-        "){1}$");
+        "CHAR(?:\\([[:digit:]]+\\))?|"
+        "DECIMAL(?:\\([[:digit:]]+(?:,[[:digit:]]+)?\\))?|"
+        "SIGNED(?: INTEGER)?|UNSIGNED(?: INTEGER)?");
     return re.match(str);
   }
 
@@ -643,11 +637,22 @@ void Expression_generator::cast_expression(
     throw Error(ER_X_EXPR_BAD_NUM_ARGS,
                 "CAST expression requires exactly two parameters.");
 
+  std::string as_type =
+      get_valid_string(arg.param(1), Cast_type_validator("CAST type invalid."));
+
   m_qb->put("CAST(");
-  generate_unquote_param(arg.param(0));
+  if (is_prep_stmt_mode() && as_type == "JSON" &&
+      arg.param(0).type() == Mysqlx::Expr::Expr::PLACEHOLDER &&
+      arg.param(0).position() >= static_cast<Placeholder>(m_args.size())) {
+    m_placeholders->emplace_back(
+        arg.param(0).position() - static_cast<Placeholder>(m_args.size()),
+        Placeholder_type::k_json);
+    m_qb->put("?");
+  } else {
+    generate_unquote_param(arg.param(0));
+  }
   m_qb->put(" AS ");
-  m_qb->put(get_valid_string(arg.param(1),
-                             Cast_type_validator("CAST type invalid.")));
+  m_qb->put(as_type);
   m_qb->put(")");
 }
 
@@ -721,8 +726,10 @@ void Expression_generator::generate(const Mysqlx::Expr::Operator &arg) const {
       {"not_cont_in", std::bind(&Gen::cont_in_expression, _1, _2, "NOT ")},
       {"not_in", std::bind(&Gen::in_expression, _1, _2, "NOT ")},
       {"not_like", std::bind(&Gen::like_expression, _1, _2, " NOT LIKE ")},
+      {"not_overlaps", std::bind(&Gen::overlaps_expression, _1, _2, "NOT ")},
       {"not_regexp",
        std::bind(&Gen::binary_expression, _1, _2, " NOT REGEXP ")},
+      {"overlaps", std::bind(&Gen::overlaps_expression, _1, _2, "")},
       {"regexp", std::bind(&Gen::binary_expression, _1, _2, " REGEXP ")},
       {"sign_minus", std::bind(&Gen::unary_operator, _1, _2, "-")},
       {"sign_plus", std::bind(&Gen::unary_operator, _1, _2, "+")},
@@ -778,4 +785,17 @@ Expression_generator Expression_generator::clone(
   return Expression_generator(qb, m_args, m_default_schema, m_is_relational);
 }
 
+void Expression_generator::overlaps_expression(
+    const Mysqlx::Expr::Operator &arg, const char *str) const {
+  if (arg.param_size() != 2)
+    throw Error(ER_X_EXPR_BAD_NUM_ARGS,
+                "OVERLAPS expression requires two parameters.");
+
+  m_qb->put(str).put("JSON_OVERLAPS(");
+  generate_json_only_param(arg.param(0), "OVERLAPS");
+  m_qb->put(",");
+  generate_json_only_param(arg.param(1), "OVERLAPS");
+  m_qb->put(")");
+}
+
 }  // namespace xpl
