@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -27,8 +27,8 @@
 
 #include <string>
 
-#include "plugin/x/ngs/include/ngs/ngs_error.h"
 #include "plugin/x/ngs/include/ngs/protocol/protocol_protobuf.h"
+#include "plugin/x/src/xpl_error.h"
 
 namespace xpl {
 namespace iface {
@@ -45,7 +45,7 @@ class Capability_handler {
   virtual void commit() = 0;
 
  protected:
-  virtual void get_impl(::Mysqlx::Datatypes::Any &any) = 0;
+  virtual void get_impl(::Mysqlx::Datatypes::Any *any) = 0;
   virtual ngs::Error_code set_impl(const ::Mysqlx::Datatypes::Any &any) = 0;
   virtual bool is_supported_impl() const = 0;
 };
