@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -38,9 +38,11 @@ class Vio_input_stream : public google::protobuf::io::ZeroCopyInputStream {
   using gint64 = google::protobuf::int64;
 
  public:
-  Vio_input_stream(const std::shared_ptr<ngs::Vio_interface> &connection);
+  explicit Vio_input_stream(
+      const std::shared_ptr<ngs::Vio_interface> &connection);
   ~Vio_input_stream() override;
 
+  bool was_io_error() const;
   bool was_io_error(int *error_code) const;
   void reset_byte_count();
 
@@ -70,7 +72,7 @@ class Vio_input_stream : public google::protobuf::io::ZeroCopyInputStream {
 
   int m_bytes_count{0};
 
-  int m_locked_data_count{0};
+  int m_locked_data_count{-1};
   int m_locked_data_pos{0};
 
   // IO error/handling
