@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,8 +31,7 @@
 
 #include <memory>
 
-#include "my_io.h"  // NOLINT(build/include_subdir)
-
+#include "my_io.h"
 #include "plugin/x/generated/mysqlx_version.h"
 #include "plugin/x/src/helper/string_formatter.h"
 #include "plugin/x/src/operations_factory.h"
@@ -50,7 +48,7 @@ const char *BIND_IPv6_ADDRESS = "::";
 
 class Tcp_creator {
  public:
-  explicit Tcp_creator(iface::Operations_factory &factory)
+  Tcp_creator(ngs::Operations_factory_interface &factory)
       : m_factory(factory),
         m_system_interface(m_factory.create_system_interface()) {}
 
@@ -88,25 +86,26 @@ class Tcp_creator {
     }
 
     return std::shared_ptr<addrinfo>(
-        result, std::bind(&iface::System::freeaddrinfo, m_system_interface,
-                          std::placeholders::_1));
+        result, std::bind(&ngs::System_interface::freeaddrinfo,
+                          m_system_interface, std::placeholders::_1));
   }
 
-  std::shared_ptr<iface::Socket> create_and_bind_socket(
-      std::shared_ptr<addrinfo> ai, const uint32_t backlog, int &error_code,
+  ngs::Socket_interface::Shared_ptr create_and_bind_socket(
+      std::shared_ptr<addrinfo> ai, const uint32 backlog, int &error_code,
       std::string &error_message) {
     addrinfo *used_ai = nullptr;
     std::string errstr;
 
-    std::shared_ptr<iface::Socket> result_socket = create_socket_from_addrinfo(
-        ai.get(), KEY_socket_x_tcpip, AF_INET, &used_ai);
+    ngs::Socket_interface::Shared_ptr result_socket =
+        create_socket_from_addrinfo(ai.get(), KEY_socket_x_tcpip, AF_INET,
+                                    &used_ai);
 
     if (nullptr == result_socket.get())
       result_socket = create_socket_from_addrinfo(ai.get(), KEY_socket_x_tcpip,
                                                   AF_INET6, &used_ai);
 
     if (nullptr == result_socket.get()) {
-      m_system_interface->get_socket_error_and_message(&error_code, &errstr);
+      m_system_interface->get_socket_error_and_message(error_code, errstr);
 
       error_message = String_formatter()
                           .append("`socket()` failed with error: ")
@@ -116,7 +115,7 @@ class Tcp_creator {
                           .append(")")
                           .get_result();
 
-      return std::shared_ptr<iface::Socket>();
+      return ngs::Socket_interface::Shared_ptr();
     }
 
 #ifdef IPV6_V6ONLY
@@ -133,10 +132,10 @@ class Tcp_creator {
       int option_flag = 0;
 
       if (result_socket->set_socket_opt(IPPROTO_IPV6, IPV6_V6ONLY,
-                                        reinterpret_cast<char *>(&option_flag),
+                                        (char *)&option_flag,
                                         sizeof(option_flag))) {
         log_error(ER_XPLUGIN_FAILED_TO_RESET_IPV6_V6ONLY_FLAG,
-                  static_cast<int>(socket_errno));
+                  (int)socket_errno);
       }
     }
 #endif
@@ -148,7 +147,7 @@ class Tcp_creator {
       if (result_socket->set_socket_opt(SOL_SOCKET, SO_REUSEADDR,
                                         (const char *)&one, sizeof(one))) {
         log_error(ER_XPLUGIN_FAILED_TO_SET_SO_REUSEADDR_FLAG,
-                  static_cast<int>(m_system_interface->get_socket_errno()));
+                  (int)m_system_interface->get_socket_errno());
       }
     }
 
@@ -157,7 +156,7 @@ class Tcp_creator {
     if (result_socket->bind((const struct sockaddr *)used_ai->ai_addr,
                             static_cast<socklen_t>(used_ai->ai_addrlen)) < 0) {
       // lets decide later if its an error or not
-      m_system_interface->get_socket_error_and_message(&error_code, &errstr);
+      m_system_interface->get_socket_error_and_message(error_code, errstr);
 
       error_message = String_formatter()
                           .append("`bind()` failed with error: ")
@@ -169,12 +168,12 @@ class Tcp_creator {
                               "running with Mysqlx ?")
                           .get_result();
 
-      return std::shared_ptr<iface::Socket>();
+      return ngs::Socket_interface::Shared_ptr();
     }
 
     if (result_socket->listen(backlog) < 0) {
       // lets decide later if its an error or not
-      m_system_interface->get_socket_error_and_message(&error_code, &errstr);
+      m_system_interface->get_socket_error_and_message(error_code, errstr);
 
       error_message = String_formatter()
                           .append("`listen()` failed with error: ")
@@ -184,7 +183,7 @@ class Tcp_creator {
                           .append(")")
                           .get_result();
 
-      return std::shared_ptr<iface::Socket>();
+      return ngs::Socket_interface::Shared_ptr();
     }
 
     m_used_address.resize(200, '\0');
@@ -202,13 +201,13 @@ class Tcp_creator {
   std::string get_used_address() { return m_used_address; }
 
  private:
-  std::shared_ptr<iface::Socket> create_socket_from_addrinfo(
+  ngs::Socket_interface::Shared_ptr create_socket_from_addrinfo(
       addrinfo *ai, PSI_socket_key psi_key, const int family,
       addrinfo **used_ai) {
     for (addrinfo *cur_ai = ai; nullptr != cur_ai; cur_ai = cur_ai->ai_next) {
       if (family != cur_ai->ai_family) continue;
 
-      std::shared_ptr<iface::Socket> result =
+      ngs::Socket_interface::Shared_ptr result =
           m_factory.create_socket(psi_key, family, SOCK_STREAM, 0);
 
       if (INVALID_SOCKET != result->get_socket_fd()) {
@@ -217,11 +216,11 @@ class Tcp_creator {
       }
     }
 
-    return std::shared_ptr<iface::Socket>();
+    return ngs::Socket_interface::Shared_ptr();
   }
 
   bool is_ipv6_avaiable() {
-    std::shared_ptr<iface::Socket> socket(m_factory.create_socket(
+    ngs::Socket_interface::Shared_ptr socket(m_factory.create_socket(
         KEY_socket_x_diagnostics, AF_INET6, SOCK_STREAM, 0));
     const bool has_ipv6 = INVALID_SOCKET != socket->get_socket_fd();
 
@@ -246,19 +245,19 @@ class Tcp_creator {
   }
 
   std::string m_used_address;
-  iface::Operations_factory &m_factory;
-  std::shared_ptr<iface::System> m_system_interface;
+  ngs::Operations_factory_interface &m_factory;
+  ngs::System_interface::Shared_ptr m_system_interface;
 };
 
 Listener_tcp::Listener_tcp(Factory_ptr operations_factory,
-                           const std::string &bind_address,
+                           std::string &bind_address,
                            const std::string &network_namespace,
-                           const uint16_t port,
-                           const uint32_t port_open_timeout,
-                           iface::Socket_events &event, const uint32_t backlog)
+                           const uint16 port, const uint32 port_open_timeout,
+                           ngs::Socket_events_interface &event,
+                           const uint32 backlog)
     : m_operations_factory(operations_factory),
-      m_state(iface::Listener::State::k_initializing,
-              KEY_mutex_x_listener_tcp_sync, KEY_cond_x_listener_tcp_sync),
+      m_state(ngs::State_listener_initializing, KEY_mutex_x_listener_tcp_sync,
+              KEY_cond_x_listener_tcp_sync),
       m_bind_address(bind_address),
       m_network_namespace(network_namespace),
       m_port(port),
@@ -271,17 +270,31 @@ Listener_tcp::~Listener_tcp() {
   close_listener();
 }
 
-const Listener_tcp::Sync_variable_state &Listener_tcp::get_state() const {
-  return m_state;
+Listener_tcp::Sync_variable_state &Listener_tcp::get_state() { return m_state; }
+
+std::string Listener_tcp::get_last_error() { return m_last_error; }
+
+std::string Listener_tcp::get_name_and_configuration() const {
+  return String_formatter()
+      .append("bind-address: '")
+      .append(m_bind_address)
+      .append("' ")
+      .append("port: ")
+      .append(m_port)
+      .get_result();
 }
 
-std::string Listener_tcp::get_configuration_variable() const {
-  return std::string(MYSQLX_SYSTEM_VARIABLE_PREFIX("port")) + "," +
-         MYSQLX_SYSTEM_VARIABLE_PREFIX("bind_address");
+std::vector<std::string> Listener_tcp::get_configuration_variables() const {
+  std::vector<std::string> result;
+
+  result.push_back(MYSQLX_SYSTEM_VARIABLE_PREFIX("port"));
+  result.push_back(MYSQLX_SYSTEM_VARIABLE_PREFIX("bind_address"));
+
+  return result;
 }
 
 bool Listener_tcp::setup_listener(On_connection on_connection) {
-  if (!m_state.is(iface::Listener::State::k_initializing)) return false;
+  if (!m_state.is(ngs::State_listener_initializing)) return false;
 
   m_tcp_socket = create_socket();
 
@@ -293,7 +306,7 @@ bool Listener_tcp::setup_listener(On_connection on_connection) {
   }
 
   if (m_event.listen(m_tcp_socket, on_connection)) {
-    m_state.set(iface::Listener::State::k_prepared);
+    m_state.set(ngs::State_listener_prepared);
     return true;
   }
 
@@ -305,33 +318,28 @@ bool Listener_tcp::setup_listener(On_connection on_connection) {
 }
 
 void Listener_tcp::close_listener() {
-  // iface::Socket::close can be called multiple times
+  // ngs::Socket_interface::close can be called multiple times
   // it invalidates the content of m_mysql_socket thus at next call
   // it does nothing
   //
   // Same applies to close_listener()
-  m_state.set(iface::Listener::State::k_stopped);
+  m_state.set(ngs::State_listener_stopped);
 
   if (m_tcp_socket) m_tcp_socket->close();
 }
 
-void Listener_tcp::pre_loop() {
-  if (m_tcp_socket) m_tcp_socket->set_socket_thread_owner();
-  m_state.set(xpl::iface::Listener::State::k_running);
-}
-
 void Listener_tcp::loop() {}
 
-std::shared_ptr<iface::Socket> Listener_tcp::create_socket() {
+ngs::Socket_interface::Shared_ptr Listener_tcp::create_socket() {
   Tcp_creator creator(*m_operations_factory);
   int error_code;
 
-  std::shared_ptr<iface::Socket> result_socket;
-  std::shared_ptr<iface::System> system_interface(
+  ngs::Socket_interface::Shared_ptr result_socket;
+  ngs::System_interface::Shared_ptr system_interface(
       m_operations_factory->create_system_interface());
 
   log_debug("TCP Sockets address is '%s' and port is %i",
-            m_bind_address.c_str(), static_cast<int>(m_port));
+            m_bind_address.c_str(), (int)m_port);
   if (!m_network_namespace.empty()) {
 #ifdef HAVE_SETNS
     if (set_network_namespace(m_network_namespace)) return nullptr;
@@ -343,9 +351,9 @@ std::shared_ptr<iface::Socket> Listener_tcp::create_socket() {
   std::shared_ptr<addrinfo> ai =
       creator.resolve_bind_address(m_bind_address, m_port, m_last_error);
 
-  if (nullptr == ai.get()) return std::shared_ptr<iface::Socket>();
+  if (nullptr == ai.get()) return ngs::Socket_interface::Shared_ptr();
 
-  for (uint32_t waited = 0, retry = 1; waited <= m_port_open_timeout; ++retry) {
+  for (uint32 waited = 0, retry = 1; waited <= m_port_open_timeout; ++retry) {
     result_socket =
         creator.create_and_bind_socket(ai, m_backlog, error_code, m_last_error);
 
@@ -360,7 +368,7 @@ std::shared_ptr<iface::Socket> Listener_tcp::create_socket() {
     // Critical failure, lets break the loop
     if (SOCKET_EADDRINUSE != system_interface->get_socket_errno()) break;
 
-    log_info(ER_XPLUGIN_RETRYING_BIND_ON_PORT, static_cast<int>(m_port));
+    log_info(ER_XPLUGIN_RETRYING_BIND_ON_PORT, (int)m_port);
 
     const int time_to_wait = retry * retry / 3 + 1;
     system_interface->sleep(time_to_wait);
@@ -378,13 +386,13 @@ std::shared_ptr<iface::Socket> Listener_tcp::create_socket() {
 std::string Listener_tcp::choose_property_value(
     const std::string &value) const {
   switch (m_state.get()) {
-    case State::k_prepared:
+    case ngs::State_listener_prepared:
       return value;
 
-    case State::k_running:
+    case ngs::State_listener_running:
       return value;
 
-    case State::k_stopped:
+    case ngs::State_listener_stopped:
       return ngs::PROPERTY_NOT_CONFIGURED;
 
     default:
@@ -400,23 +408,4 @@ void Listener_tcp::report_properties(On_report_properties on_prop) {
           choose_property_value(std::to_string(m_port)));
 }
 
-bool Listener_tcp::report_status() const {
-  const std::string name = m_network_namespace.empty()
-                               ? m_bind_address
-                               : m_bind_address + '/' + m_network_namespace;
-
-  const std::string msg =
-      "bind-address: '" + name + "' port: " + std::to_string(m_port);
-
-  if (m_state.is(State::k_prepared)) {
-    log_info(ER_XPLUGIN_LISTENER_STATUS_MSG, msg.c_str());
-    return true;
-  }
-
-  log_error(ER_XPLUGIN_LISTENER_SETUP_FAILED, msg.c_str(),
-            m_last_error.c_str());
-  log_error(ER_XPLUGIN_FAILED_TO_BIND_INTERFACE_ADDRESS, name.c_str());
-  return false;
-}
-
 }  // namespace xpl
