@@ -1,5 +1,6 @@
+
 /*
- * Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -29,14 +30,9 @@
 #include <sstream>
 #include <string>
 
-#include "mysql/components/my_service.h"
-#include "mysql/components/services/mysql_admin_session.h"
 #include "mysql/plugin.h"
 #include "mysql/service_command.h"
-#include "sql/debug_sync.h"
-
 #include "plugin/x/src/helper/get_system_variable.h"
-#include "plugin/x/src/module_mysqlx.h"
 #include "plugin/x/src/mysql_variables.h"
 #include "plugin/x/src/notices.h"
 #include "plugin/x/src/query_string_builder.h"
@@ -49,36 +45,9 @@
 
 namespace xpl {
 
-namespace details {
-
-class Admin_session_factory {
- public:
-  MYSQL_SESSION create(srv_session_error_cb error_cb, void *context) {
-    if (!m_admin_session.is_valid()) return nullptr;
-
-    return m_admin_session->open(error_cb, context);
-  }
-
- private:
-  class Registry_holder {
-   public:
-    Registry_holder() : m_registry{mysql_plugin_registry_acquire()} {}
-    ~Registry_holder() { mysql_plugin_registry_release(m_registry); }
-
-    SERVICE_TYPE(registry) *const m_registry;
-  };
-
-  Registry_holder m_registry_holder;
-  my_service<SERVICE_TYPE(mysql_admin_session)> m_admin_session{
-      "mysql_admin_session", m_registry_holder.m_registry};
-};
-
-}  // namespace details
-
 ngs::Error_code Sql_data_context::init(const int client_port,
-                                       const Connection_type type,
-                                       const bool is_admin) {
-  ngs::Error_code error = init(is_admin);
+                                       const Connection_type type) {
+  ngs::Error_code error = init();
   if (error) return error;
 
   if ((error = set_connection_type(type))) return error;
@@ -89,77 +58,39 @@ ngs::Error_code Sql_data_context::init(const int client_port,
   return ngs::Error_code();
 }
 
-ngs::Error_code Sql_data_context::init(const bool is_admin) {
-  if (is_admin) {
-    details::Admin_session_factory factory;
-    m_mysql_session =
-        factory.create(&Sql_data_context::default_completion_handler, this);
-
-  } else {
-    m_mysql_session =
-        srv_session_open(&Sql_data_context::default_completion_handler, this);
-  }
-
+ngs::Error_code Sql_data_context::init() {
+  m_mysql_session =
+      srv_session_open(&Sql_data_context::default_completion_handler, this);
   log_debug(
       "sqlsession init: %p [%i]", m_mysql_session,
       m_mysql_session ? srv_session_info_get_session_id(m_mysql_session) : -1);
   if (!m_mysql_session) {
     if (ER_SERVER_ISNT_AVAILABLE == m_last_sql_errno)
       return ngs::Error_code(ER_SERVER_ISNT_AVAILABLE, "Server API not ready");
-
-    if (ER_CON_COUNT_ERROR == m_last_sql_errno)
-      return ngs::Error_code(m_last_sql_errno, m_last_sql_error);
-
-    if (ER_CON_COUNT_ERROR == m_last_sql_errno)
-      return ngs::Error_code(m_last_sql_errno, m_last_sql_error);
-
     log_error(ER_XPLUGIN_FAILED_TO_OPEN_INTERNAL_SESSION);
     return ngs::Error_code(ER_X_SESSION, "Could not open session");
   }
-
-  m_pre_authenticate_event_fired = false;
-  m_authentication_code = ngs::Error_code();
-
   return ngs::Error_code();
 }
 
 void Sql_data_context::deinit() {
   if (m_mysql_session) {
-    if (m_pre_authenticate_event_fired && !attach()) {
-      if (!m_authentication_code) {
-        // In case of successfull login, connect event has been generated
-        // earlier. Time for generating disconnect event.
-        modules::Module_mysqlx::get_instance_services()->m_audit_api->emit(
-            get_thd(), MYSQL_AUDIT_CONNECTION_DISCONNECT);
-      } else {
-        modules::Module_mysqlx::get_instance_services()
-            ->m_audit_api->emit_with_errorcode(get_thd(),
-                                               MYSQL_AUDIT_CONNECTION_CONNECT,
-                                               m_authentication_code.error);
-      }
-    }
-
-    m_pre_authenticate_event_fired = false;
-    m_authentication_code = ngs::Error_code();
-
     srv_session_detach(m_mysql_session);
 
     log_debug("sqlsession deinit: %p [%i]", m_mysql_session,
               srv_session_info_get_session_id(m_mysql_session));
-
     srv_session_close(m_mysql_session);
-    m_mysql_session = nullptr;
+    m_mysql_session = NULL;
   }
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
   PSI_THREAD_CALL(delete_current_thread)();
 
-  PSI_thread *psi =
-      PSI_THREAD_CALL(new_thread)(KEY_thread_x_worker, 0, nullptr, 0);
+  PSI_thread *psi = PSI_THREAD_CALL(new_thread)(KEY_thread_x_worker, NULL, 0);
   PSI_THREAD_CALL(set_thread_os_id)(psi);
   PSI_THREAD_CALL(set_thread)(psi);
 #endif /* HAVE_PSI_THREAD_INTERFACE */
-}  // namespace xpl
+}
 
 static void kill_completion_handler(void *, unsigned int sql_errno,
                                     const char *err_msg) {
@@ -171,7 +102,7 @@ bool Sql_data_context::kill() {
     log_debug("sqlsession init (for kill): %p [%i]", m_mysql_session,
               m_mysql_session ? srv_session_info_get_session_id(m_mysql_session)
                               : -1);
-    MYSQL_SESSION session = srv_session_open(kill_completion_handler, nullptr);
+    MYSQL_SESSION session = srv_session_open(kill_completion_handler, NULL);
     bool ok = false;
     if (session) {
       MYSQL_SECURITY_CONTEXT scontext;
@@ -182,7 +113,7 @@ bool Sql_data_context::kill() {
       } else {
         const char *user = MYSQL_SESSION_USER;
         const char *host = MYSQLXSYS_HOST;
-        if (security_context_lookup(scontext, user, host, nullptr, nullptr)) {
+        if (security_context_lookup(scontext, user, host, NULL, NULL)) {
           log_warning(ER_XPLUGIN_FAILED_TO_SWITCH_SECURITY_CTX, user);
         } else {
           COM_DATA data;
@@ -190,13 +121,13 @@ bool Sql_data_context::kill() {
           Query_string_builder qb;
           qb.put("KILL ").put(mysql_session_id());
 
-          memset(&data, 0, sizeof(data));
           data.com_query.query = qb.get().c_str();
           data.com_query.length = static_cast<unsigned int>(qb.get().length());
 
           if (!command_service_run_command(
-                  session, COM_QUERY, &data, mysqld::get_default_charset(),
-                  deleg.callbacks(), deleg.representation(), &deleg)) {
+                  session, COM_QUERY, &data,
+                  mysqld::get_charset_utf8mb4_general_ci(), deleg.callbacks(),
+                  deleg.representation(), &deleg)) {
             if (!deleg.get_error())
               ok = true;
             else
@@ -248,21 +179,25 @@ Sql_data_context::~Sql_data_context() {
 }
 
 void Sql_data_context::switch_to_local_user(const std::string &user) {
-  ngs::Error_code error =
-      switch_to_user(user.c_str(), "localhost", nullptr, nullptr);
+  ngs::Error_code error = switch_to_user(user.c_str(), "localhost", NULL, NULL);
   if (error) throw error;
 }
 
-ngs::Error_code Sql_data_context::authenticate_internal(
+ngs::Error_code Sql_data_context::authenticate(
     const char *user, const char *host, const char *ip, const char *db,
     const std::string &passwd,
-    const iface::Authentication &account_verification,
+    const ngs::Authentication_interface &account_verification,
     bool allow_expired_passwords) {
+  m_password_expired = false;
+
+  ngs::Error_code error = switch_to_user(user, host, ip, db);
+
+  if (error) return ngs::SQLError_access_denied();
+
   std::string authenticated_user_name = get_authenticated_user_name();
   std::string authenticated_user_host = get_authenticated_user_host();
 
-  ngs::Error_code error =
-      switch_to_user(MYSQL_SESSION_USER, MYSQLXSYS_HOST, nullptr, nullptr);
+  error = switch_to_user(MYSQL_SESSION_USER, MYSQLXSYS_HOST, NULL, NULL);
 
   if (error) {
     const char *session_user = MYSQL_SESSION_USER;
@@ -275,8 +210,6 @@ ngs::Error_code Sql_data_context::authenticate_internal(
         authenticated_user_name, authenticated_user_host, passwd);
   }
 
-  ngs::Error_code switch_error = switch_to_user(user, host, ip, db);
-
   if (error.error == ER_MUST_CHANGE_PASSWORD_LOGIN) {
     m_password_expired = true;
 
@@ -287,8 +220,7 @@ ngs::Error_code Sql_data_context::authenticate_internal(
   } else {
     if (error) return error;
   }
-
-  error = switch_error;
+  error = switch_to_user(user, host, ip, db);
 
   if (!error) {
     if (db && *db) {
@@ -299,10 +231,11 @@ ngs::Error_code Sql_data_context::authenticate_internal(
           static_cast<unsigned long>(strlen(db));  // NOLINT(runtime/int)
 
       Callback_command_delegate callback_delegate;
-      if (command_service_run_command(
-              m_mysql_session, COM_INIT_DB, &data,
-              mysqld::get_default_charset(), callback_delegate.callbacks(),
-              callback_delegate.representation(), &callback_delegate))
+      if (command_service_run_command(m_mysql_session, COM_INIT_DB, &data,
+                                      mysqld::get_charset_utf8mb4_general_ci(),
+                                      callback_delegate.callbacks(),
+                                      callback_delegate.representation(),
+                                      &callback_delegate))
         return ngs::Error_code(ER_NO_DB_ERROR, "Could not set database");
       error = callback_delegate.get_error();
     }
@@ -310,6 +243,19 @@ ngs::Error_code Sql_data_context::authenticate_internal(
     std::string user_name = get_user_name();
     std::string host_or_ip = get_host_or_ip();
 
+    /*
+      Instead of modifying the current security context in switch_user()
+      method above, we must create a security_context to do the
+      security_context_lookup() on newly created security_context then set
+      that in the THD. Until that happens, we have to get the existing security
+      context and set that again in the THD. The latter opertion is nedded as
+      it may toggle the system_user flag in THD iff security_context has
+      SYSTEM_USER privilege.
+    */
+    MYSQL_SECURITY_CONTEXT scontext;
+    thd_get_security_context(get_thd(), &scontext);
+    thd_set_security_context(get_thd(), scontext);
+
 #ifdef HAVE_PSI_THREAD_INTERFACE
     PSI_THREAD_CALL(set_thread_account)
     (user_name.c_str(), static_cast<int>(user_name.length()),
@@ -324,60 +270,6 @@ ngs::Error_code Sql_data_context::authenticate_internal(
   return error;
 }
 
-ngs::Error_code Sql_data_context::authenticate(
-    const char *user, const char *host, const char *ip, const char *db,
-    const std::string &passwd,
-    const iface::Authentication &account_verification,
-    bool allow_expired_passwords) {
-  m_password_expired = false;
-
-  m_using_password = passwd.size() > 0;
-
-  ngs::Error_code error = switch_to_user(user, host, ip, db);
-
-  if (error) error = ngs::SQLError_access_denied();
-
-  m_authentication_code = error;
-
-  if (!m_pre_authenticate_event_fired) {
-    attach();
-    int audit_result =
-        modules::Module_mysqlx::get_instance_services()->m_audit_api->emit(
-            static_cast<void *>(get_thd()),
-            MYSQL_AUDIT_CONNECTION_PRE_AUTHENTICATE);
-    detach();
-
-    if (audit_result != 0)
-      return ngs::Fatal(ER_AUDIT_API_ABORT, my_get_err_msg(ER_AUDIT_API_ABORT),
-                        "MYSQL_AUDIT_CONNECTION_PRE_AUTHENTICATE",
-                        audit_result);
-
-    m_pre_authenticate_event_fired = true;
-  }
-
-  if (error) return error;
-
-  error = authenticate_internal(user, host, ip, db, passwd,
-                                account_verification, allow_expired_passwords);
-
-  m_authentication_code = error;
-
-  if (!error) {
-    attach();
-    int audit_result =
-        modules::Module_mysqlx::get_instance_services()->m_audit_api->emit(
-            static_cast<void *>(get_thd()), MYSQL_AUDIT_CONNECTION_CONNECT);
-    detach();
-
-    if (audit_result != 0)
-      m_authentication_code = error =
-          ngs::Fatal(ER_AUDIT_API_ABORT, my_get_err_msg(ER_AUDIT_API_ABORT),
-                     "MYSQL_AUDIT_CONNECTION_CONNECT", audit_result);
-  }
-
-  return error;
-}
-
 template <typename Result_type>
 bool get_security_context_value(MYSQL_THD thd, const char *option,
                                 Result_type *result) {
@@ -392,7 +284,7 @@ bool Sql_data_context::is_acl_disabled() {
   MYSQL_LEX_CSTRING value{"", 0};
 
   if (get_security_context_value(get_thd(), "priv_user", &value)) {
-    return 0 != value.length && nullptr != strstr(value.str, "skip-grants ");
+    return 0 != value.length && NULL != strstr(value.str, "skip-grants ");
   }
 
   return false;
@@ -451,46 +343,19 @@ ngs::Error_code Sql_data_context::switch_to_user(const char *username,
     return ngs::Fatal(ER_X_SERVICE_ERROR,
                       "Error getting security context for session");
 
-  attach();
-  CONDITIONAL_SYNC_POINT("xpl_switch_to_user_enter");
-
   // security_context_lookup - doesn't make a copy of username, hostname, addres
   //                           or db thus we need to make a copy of them
-  //                           and pass our pointers to security_context_lookup.
-  //                           If we change pass different pointers than in last
-  //                           invocation of `switch_to_user`, then it might
-  //                           cause an invalid memory access while listing all
-  //                           sessions. Thus we must use memory regions that
-  //                           can hold values of maximum length.
-  memset(m_username, 0, sizeof(m_username));
-  memset(m_hostname, 0, sizeof(m_hostname));
-  memset(m_address, 0, sizeof(m_address));
-  memset(m_db, 0, sizeof(m_db));
-
-  if (username) {
-    strncpy(m_username, username, sizeof(m_username) - 1);
-  }
-
-  if (hostname) {
-    strncpy(m_hostname, hostname, sizeof(m_hostname) - 1);
-  }
-
-  if (address) {
-    strncpy(m_address, address, sizeof(m_address) - 1);
-  }
-
-  if (db) {
-    strncpy(m_db, db, sizeof(m_db) - 1);
-  }
-
-  CONDITIONAL_SYNC_POINT("xpl_switch_to_user_execute");
-  detach();
+  //                           and pass our pointers to security_context_lookup
+  m_username = username ? username : "";
+  m_hostname = hostname ? hostname : "";
+  m_address = address ? address : "";
+  m_db = db ? db : "";
 
   log_debug("Switching security context to user %s@%s [%s]", username, hostname,
             address);
 
-  if (security_context_lookup(scontext, m_username, m_hostname, m_address,
-                              m_db)) {
+  if (security_context_lookup(scontext, m_username.c_str(), hostname,
+                              m_address.c_str(), m_db.c_str())) {
     log_debug("Unable to switch security context to user %s@%s [%s]", username,
               hostname, address);
     return ngs::Fatal(ER_X_SERVICE_ERROR, "Unable to switch context to user %s",
@@ -517,15 +382,10 @@ void Sql_data_context::default_completion_handler(void *ctx,
 }
 
 bool Sql_data_context::is_killed() const {
-  const auto kill = thd_killed(get_thd());
-  DBUG_LOG("debug", "is_killed:" << kill);
-
-  if (0 == kill) return false;
-
-  return ER_QUERY_INTERRUPTED != kill;
+  return srv_session_info_killed(m_mysql_session);
 }
 
-bool Sql_data_context::is_api_ready() {
+bool Sql_data_context::is_api_ready() const {
   return 0 != srv_session_server_is_available();
 }
 
@@ -538,11 +398,9 @@ MYSQL_THD Sql_data_context::get_thd() const {
 }
 
 ngs::Error_code Sql_data_context::execute(const char *sql, std::size_t sql_len,
-                                          iface::Resultset *rset) {
+                                          ngs::Resultset_interface *rset) {
   const auto error = execute_sql(sql, sql_len, rset);
-  if (error) return error;
-
-  if (m_password_expired) {
+  if (m_password_expired && !error) {
     // if a SQL command succeeded while password is expired, it means the user
     // probably changed the password
     // we run a command to check just in case... (some commands are still
@@ -552,22 +410,25 @@ ngs::Error_code Sql_data_context::execute(const char *sql, std::size_t sql_len,
     if (!execute_sql(cmd.c_str(), cmd.length(), &rs))
       m_password_expired = false;
   }
-  return ngs::Success();
+
+  if (is_killed())
+    throw ngs::Fatal(ER_QUERY_INTERRUPTED, "Query execution was interrupted");
+
+  return error;
 }
 
 ngs::Error_code Sql_data_context::execute_sql(const char *sql,
                                               std::size_t sql_len,
-                                              iface::Resultset *rset) {
+                                              ngs::Resultset_interface *rset) {
   COM_DATA data;
-  memset(&data, 0, sizeof(data));
   data.com_query.query = sql;
   data.com_query.length = static_cast<unsigned int>(sql_len);
   return execute_server_command(COM_QUERY, data, rset);
 }
 
-ngs::Error_code Sql_data_context::fetch_cursor(const uint32_t statement_id,
-                                               const uint32_t row_count,
-                                               iface::Resultset *rset) {
+ngs::Error_code Sql_data_context::fetch_cursor(const std::uint32_t statement_id,
+                                               const std::uint32_t row_count,
+                                               ngs::Resultset_interface *rset) {
   COM_DATA data;
   data.com_stmt_fetch.stmt_id = statement_id;
   data.com_stmt_fetch.num_rows = row_count;
@@ -593,16 +454,15 @@ ngs::Error_code Sql_data_context::detach() {
   return {};
 }
 
-ngs::Error_code Sql_data_context::prepare_prep_stmt(const char *sql,
-                                                    std::size_t sql_len,
-                                                    iface::Resultset *rset) {
+ngs::Error_code Sql_data_context::prepare_prep_stmt(
+    const char *sql, std::size_t sql_len, ngs::Resultset_interface *rset) {
   COM_DATA data;
   data.com_stmt_prepare = {sql, static_cast<unsigned>(sql_len)};
   return execute_server_command(COM_STMT_PREPARE, data, rset);
 }
 
-ngs::Error_code Sql_data_context::deallocate_prep_stmt(const uint32_t stmt_id,
-                                                       iface::Resultset *rset) {
+ngs::Error_code Sql_data_context::deallocate_prep_stmt(
+    const uint32_t stmt_id, ngs::Resultset_interface *rset) {
   COM_DATA data;
   data.com_stmt_close = {static_cast<unsigned>(stmt_id)};
   return execute_server_command(COM_STMT_CLOSE, data, rset);
@@ -610,7 +470,7 @@ ngs::Error_code Sql_data_context::deallocate_prep_stmt(const uint32_t stmt_id,
 
 ngs::Error_code Sql_data_context::execute_prep_stmt(
     const uint32_t stmt_id, const bool has_cursor, const PS_PARAM *parameters,
-    const std::size_t parameters_count, iface::Resultset *rset) {
+    const std::size_t parameters_count, ngs::Resultset_interface *rset) {
   COM_DATA cmd;
   cmd.com_stmt_execute = {
       static_cast<unsigned long>(stmt_id),     // NOLINT(runtime/int)
@@ -622,39 +482,40 @@ ngs::Error_code Sql_data_context::execute_prep_stmt(
   return execute_server_command(COM_STMT_EXECUTE, cmd, rset);
 }
 
-ngs::Error_code Sql_data_context::reset() {
-  COM_DATA data{};
-  Empty_resultset rset;
-  return execute_server_command(COM_RESET_CONNECTION, data, &rset);
-}
-
 ngs::Error_code Sql_data_context::execute_server_command(
     const enum_server_command cmd, const COM_DATA &cmd_data,
-    iface::Resultset *rset) {
+    ngs::Resultset_interface *rset) {
   ngs::Command_delegate &deleg = rset->get_callbacks();
   deleg.reset();
-  const auto fail = command_service_run_command(
-      m_mysql_session, cmd, &cmd_data, mysqld::get_default_charset(),
-      deleg.callbacks(), deleg.representation(), &deleg);
-
-  ngs::Error_code error = deleg.get_error();
-
-  if (fail) {
-    if (error) {
-      log_debug("Error running server command: (%i:%s)", error.error,
-                error.message.c_str());
-      return error;
-    }
-    return ngs::Error(ER_X_SERVICE_ERROR, "Internal error executing command");
+  if (command_service_run_command(m_mysql_session, cmd, &cmd_data,
+                                  mysqld::get_charset_utf8mb4_general_ci(),
+                                  deleg.callbacks(), deleg.representation(),
+                                  &deleg)) {
+    return ngs::Error_code(ER_X_SERVICE_ERROR,
+                           "Internal error executing command");
   }
+  const ngs::Error_code error = deleg.get_error();
+  if (error)
+    log_debug("Error running server command: (%i %s)", error.error,
+              error.message.c_str());
+  return error;
+}
 
-  if (!error) return ngs::Success();
-
-  log_debug("Error running server command: (%i:%s; killed: %i:%i)", error.error,
-            error.message.c_str(), static_cast<int>(is_killed()),
-            static_cast<int>(deleg.killed()));
-
-  return is_killed() ? ngs::Fatal(error) : error;
+ngs::Error_code Sql_data_context::reset() {
+  COM_DATA data;
+  Callback_command_delegate deleg;
+  if (command_service_run_command(m_mysql_session, COM_RESET_CONNECTION, &data,
+                                  mysqld::get_charset_utf8mb4_general_ci(),
+                                  deleg.callbacks(), deleg.representation(),
+                                  &deleg)) {
+    return ngs::Error_code(ER_X_SERVICE_ERROR,
+                           "Internal error executing command");
+  }
+  const ngs::Error_code &error = deleg.get_error();
+  if (error)
+    log_debug("Error reseting sql session: (%i %s)", error.error,
+              error.message.c_str());
+  return error;
 }
 
 bool Sql_data_context::is_sql_mode_set(const std::string &mode) {
