@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -27,25 +27,26 @@
 #include <string>
 
 #include "plugin/x/src/admin_cmd_arguments.h"
-#include "plugin/x/src/client.h"
 #include "plugin/x/src/notices.h"
-#include "plugin/x/src/session.h"
 #include "plugin/x/src/sql_statement_builder.h"
+#include "plugin/x/src/xpl_client.h"
 #include "plugin/x/src/xpl_error.h"
 #include "plugin/x/src/xpl_log.h"
 #include "plugin/x/src/xpl_resultset.h"
+#include "plugin/x/src/xpl_session.h"
 
 namespace xpl {
 
 ngs::Error_code Stmt_command_handler::execute(
     const Mysqlx::Sql::StmtExecute &msg) {
-  log_debug("%s: %s", m_session->client().client_id(),
-            msg.stmt().substr(0, MBYTE(1)).c_str());
+  log_debug("%s: %s", m_session->client().client_id(), msg.stmt().c_str());
 
   if (msg.namespace_() == Sql_statement_builder::k_sql_namespace ||
       !msg.has_namespace_())
     return sql_stmt_execute(msg);
 
+  if (msg.namespace_() == "xplugin") return deprecated_admin_stmt_execute(msg);
+
   if (msg.namespace_() == Admin_command_handler::k_mysqlx_namespace)
     return admin_stmt_execute(msg);
 
@@ -76,19 +77,38 @@ ngs::Error_code Stmt_command_handler::sql_stmt_execute(
       da.execute(m_qb.get().data(), m_qb.get().length(), &resultset);
 
   if (error) {
-    if (show_warnings) notices::send_warnings(&da, &proto, true);
+    if (show_warnings) notices::send_warnings(da, proto, true);
     return error;
   }
 
   return ngs::Success();
 }
 
+ngs::Error_code Stmt_command_handler::deprecated_admin_stmt_execute(
+    const Mysqlx::Sql::StmtExecute &msg) {
+  m_session->update_status(
+      &ngs::Common_status_variables::m_stmt_execute_xplugin);
+  auto &notice_config = m_session->get_notice_configuration();
+
+  if (notice_config.is_notice_enabled(
+          ngs::Notice_type::k_xplugin_deprecation)) {
+    m_session->proto().send_notice_txt_message(
+        "Namespace 'xplugin' is deprecated, please use 'mysqlx' instead");
+
+    notice_config.set_notice(ngs::Notice_type::k_xplugin_deprecation, false);
+  }
+  Admin_command_arguments_list args(msg.args());
+  return Admin_command_handler(m_session).execute(msg.namespace_(), msg.stmt(),
+                                                  &args);
+}
+
 ngs::Error_code Stmt_command_handler::admin_stmt_execute(
     const Mysqlx::Sql::StmtExecute &msg) {
   m_session->update_status(
       &ngs::Common_status_variables::m_stmt_execute_mysqlx);
   Admin_command_arguments_object args(msg.args());
-  return Admin_command_handler(m_session).execute(msg.stmt(), &args);
+  return Admin_command_handler(m_session).execute(msg.namespace_(), msg.stmt(),
+                                                  &args);
 }
 
 }  // namespace xpl
