@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,14 +22,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
-#include "plugin/x/src/interface/notice_output_queue.h"
-
-#include "plugin/x/src/session.h"
 #include "plugin/x/src/xpl_dispatcher.h"
+#include "plugin/x/ngs/include/ngs/interface/notice_output_queue_interface.h"
 #include "plugin/x/src/xpl_error.h"
+#include "plugin/x/src/xpl_session.h"
 
 namespace xpl {
-ngs::Error_code Dispatcher::execute(const ngs::Message_request &command) {
+bool Dispatcher::execute(const ngs::Message_request &command) {
   ngs::Error_code error =
       m_expect_stack.pre_client_stmt(command.get_message_type());
   if (!error) {
@@ -40,7 +38,7 @@ ngs::Error_code Dispatcher::execute(const ngs::Message_request &command) {
   } else {
     m_session->proto().send_result(error);
   }
-  return error;
+  return error.error != ER_UNKNOWN_COM_ERROR;
 }
 
 ngs::Error_code Dispatcher::dispatch(const ngs::Message_request &command) {
