@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -27,15 +27,15 @@
 
 #include <vector>
 
+#include "plugin/x/ngs/include/ngs/interface/notice_output_queue_interface.h"
+#include "plugin/x/ngs/include/ngs/interface/resultset_interface.h"
 #include "plugin/x/src/buffering_command_delegate.h"
 #include "plugin/x/src/custom_command_delegates.h"
-#include "plugin/x/src/interface/notice_output_queue.h"
-#include "plugin/x/src/interface/resultset.h"
 #include "plugin/x/src/streaming_command_delegate.h"
 
 namespace xpl {
 
-class Process_resultset : public iface::Resultset {
+class Process_resultset : public ngs::Resultset_interface {
  public:
   using Row = Callback_command_delegate::Row_data;
   using Field = Callback_command_delegate::Field_value;
@@ -59,7 +59,7 @@ class Process_resultset : public iface::Resultset {
   Callback_command_delegate m_callback_delegate;
 };
 
-class Empty_resultset : public iface::Resultset {
+class Empty_resultset : public ngs::Resultset_interface {
  public:
   Empty_resultset() : m_callback_delegate() {}
   ngs::Command_delegate &get_callbacks() override {
@@ -73,7 +73,7 @@ class Empty_resultset : public iface::Resultset {
   Callback_command_delegate m_callback_delegate;
 };
 
-class Collect_resultset : public iface::Resultset {
+class Collect_resultset : public ngs::Resultset_interface {
  public:
   using Row_list = Buffering_command_delegate::Resultset;
   using Row = Buffering_command_delegate::Row_data;
@@ -102,9 +102,10 @@ class Collect_resultset : public iface::Resultset {
 };
 
 template <typename T = Streaming_command_delegate>
-class Streaming_resultset : public iface::Resultset {
+class Streaming_resultset : public ngs::Resultset_interface {
  public:
-  Streaming_resultset(iface::Session *session, const bool compact_metadata)
+  Streaming_resultset(ngs::Session_interface *session,
+                      const bool compact_metadata)
       : m_streaming_delegate(session) {
     m_streaming_delegate.set_compact_metadata(compact_metadata);
   }
@@ -121,9 +122,9 @@ class Streaming_resultset : public iface::Resultset {
   T m_streaming_delegate;
 };
 
-class Cursor_resultset : public iface::Resultset {
+class Cursor_resultset : public ngs::Resultset_interface {
  public:
-  Cursor_resultset(iface::Session *session, const bool compact_metadata,
+  Cursor_resultset(ngs::Session_interface *session, const bool compact_metadata,
                    const bool ignore_fetch_suspended)
       : m_cursor_delegate(session, ignore_fetch_suspended) {
     m_cursor_delegate.set_compact_metadata(compact_metadata);
