@@ -34,7 +34,6 @@
 #include <vector>
 
 #include "my_inttypes.h"
-#include "plugin/x/src/set_variable.h"
 
 #ifdef max_allowed_packet
 #undef max_allowed_packet
@@ -79,8 +78,6 @@ class Plugin_system_variables {
   static uint32_t m_document_id_unique_prefix;
   static bool m_enable_hello_notice;
 
-  static Set_variable m_compression_algorithms;
-
   static Ssl_config ssl_config;
 
  public:
