@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -24,17 +24,11 @@
 
 #include "plugin/x/tests/driver/connector/session_holder.h"
 
-#include <array>
+#include "my_dbug.h"
 
-#include "my_dbug.h"         // NOLINT(build/include_subdir)
-#include "template_utils.h"  // NOLINT(build/include_subdir)
-
-#include "plugin/x/protocol/stream/compression/compression_algorithm_lz4.h"
-#include "plugin/x/protocol/stream/compression/compression_algorithm_zlib.h"
-#include "plugin/x/protocol/stream/compression/compression_algorithm_zstd.h"
 #include "plugin/x/tests/driver/connector/mysqlx_all_msgs.h"
 
-namespace {
+namespace details {
 
 std::string get_ip_mode_to_text(const xcl::Internet_protocol ip) {
   switch (ip) {
@@ -50,75 +44,17 @@ std::string get_ip_mode_to_text(const xcl::Internet_protocol ip) {
   }
 }
 
-size_t get_message_size(const xcl::XProtocol::Message &msg) {
-#if (defined(GOOGLE_PROTOBUF_VERSION) && GOOGLE_PROTOBUF_VERSION > 3000000)
-  return msg.ByteSizeLong() + 1;
-#else
-  return msg.ByteSize() + 1;
-#endif
-}
-
-template <typename Msg = xcl::XProtocol::Message>
-void print_message(const Console &console, const std::string &direction,
-                   const size_t size, const Msg &msg) {
-  console.print(direction, size, " ", msg);
-}
-
-}  // namespace
-
-const int k_maximum_size_of_message_in_flow_history = 1000;
+}  // namespace details
 
 Session_holder::Session_holder(std::unique_ptr<xcl::XSession> session,
-                               const Console &console_with_flow_history,
                                const Console &console,
                                const Connection_options &options)
-    : m_session(std::move(session)),
-      m_console_with_flow_history(console_with_flow_history),
-      m_console(console),
-      m_options(options) {
-  const auto env_variable_enable_tracing = getenv("MYSQLX_TRACE_CONNECTION");
-  m_enable_tracing_in_console =
-      m_options.trace_protocol ||
-      (env_variable_enable_tracing && strlen(env_variable_enable_tracing));
-}
-
-bool Session_holder::enable_compression(
-    const xcl::Compression_algorithm algorithm, const int64_t level) {
-  m_algorithm.reset();
-
-  get_session()->get_protocol().use_compression(algorithm, level);
-
-  switch (algorithm) {
-    case xcl::Compression_algorithm::k_lz4:
-      m_algorithm.reset(new protocol::Compression_algorithm_lz4(level));
-      break;
-
-    case xcl::Compression_algorithm::k_deflate:
-      m_algorithm.reset(new protocol::Compression_algorithm_zlib(level));
-      break;
-
-    case xcl::Compression_algorithm::k_zstd:
-      m_algorithm.reset(new protocol::Compression_algorithm_zstd(level));
-      break;
-
-    default:
-    case xcl::Compression_algorithm::k_none:
-      return false;
-  }
-
-  return true;
-}
-
-protocol::Compression_algorithm_interface *Session_holder::get_algorithm() {
-  return m_algorithm.get();
-}
+    : m_session(std::move(session)), m_console(console), m_options(options) {}
 
 xcl::XSession *Session_holder::get_session() { return m_session.get(); }
 
 xcl::XError Session_holder::connect(const bool is_raw_connection) {
   DBUG_TRACE;
-  setup_compression();
-
   setup_ssl();
   setup_msg_callbacks();
   setup_other_options();
@@ -186,7 +122,7 @@ xcl::XError Session_holder::setup_connection() {
 void Session_holder::setup_other_options() {
   DBUG_TRACE;
   using Mysqlx_option = xcl::XSession::Mysqlx_option;
-  const auto text_ip_mode = get_ip_mode_to_text(m_options.ip_mode);
+  const auto text_ip_mode = details::get_ip_mode_to_text(m_options.ip_mode);
 
   if (m_options.compatible) {
     m_session->set_mysql_option(Mysqlx_option::Authentication_method,
@@ -201,58 +137,6 @@ void Session_holder::setup_other_options() {
                                 m_options.auth_methods);
 }
 
-void Session_holder::setup_compression() {
-  DBUG_TRACE;
-  using Mysqlx_option = xcl::XSession::Mysqlx_option;
-
-  if (m_options.compression_mode.empty()) return;
-
-  xcl::XError error;
-  error = m_session->set_mysql_option(
-      Mysqlx_option::Compression_negotiation_mode, m_options.compression_mode);
-  if (error)
-    throw xcl::XError{CR_X_UNSUPPORTED_OPTION_VALUE,
-                      "Unsupported value for \"compression-mode\" option"};
-
-  error = m_session->set_mysql_option(Mysqlx_option::Compression_algorithms,
-                                      m_options.compression_algorithm);
-  if (error)
-    throw xcl::XError{CR_X_UNSUPPORTED_OPTION_VALUE,
-                      "Unsupported value for \"compression-algorithm\" option"};
-
-  error = m_session->set_mysql_option(
-      Mysqlx_option::Compression_combine_mixed_messages,
-      m_options.compression_combine_mixed_messages);
-  if (error)
-    throw xcl::XError{
-        CR_X_UNSUPPORTED_OPTION_VALUE,
-        "Unsupported value for \"compression-combine-mixed-messages\" option"};
-
-  error = m_session->set_mysql_option(
-      Mysqlx_option::Compression_max_combine_messages,
-      m_options.compression_max_combine_messages);
-  if (error)
-    throw xcl::XError{
-        CR_X_UNSUPPORTED_OPTION_VALUE,
-        "Unsupported value for \"compression-max-combine-messages\" option"};
-
-  if (m_options.compression_level.has_value()) {
-    error = m_session->set_mysql_option(
-        Mysqlx_option::Compression_level_server,
-        static_cast<int64_t>(m_options.compression_level.value()));
-    if (error)
-      throw xcl::XError{CR_X_UNSUPPORTED_OPTION_VALUE,
-                        "Unsupported value for \"compression-level\" option"};
-
-    error = m_session->set_mysql_option(
-        Mysqlx_option::Compression_level_client,
-        static_cast<int64_t>(m_options.compression_level.value()));
-    if (error)
-      throw xcl::XError{CR_X_UNSUPPORTED_OPTION_VALUE,
-                        "Unsupported value for \"compression-level\" option"};
-  }
-}
-
 void Session_holder::setup_ssl() {
   DBUG_TRACE;
 
@@ -299,6 +183,7 @@ void Session_holder::setup_ssl() {
 }
 
 void Session_holder::setup_msg_callbacks() {
+  const bool force_trace_protocol = m_options.trace_protocol;
   auto &protocol = m_session->get_protocol();
 
   m_session->set_mysql_option(xcl::XSession::Mysqlx_option::Consume_all_notices,
@@ -318,28 +203,28 @@ void Session_holder::setup_msg_callbacks() {
         return count_received_messages(protocol, msg_id, msg);
       });
 
-  protocol.add_received_message_handler(
-      [this](xcl::XProtocol *protocol,
-             const xcl::XProtocol::Server_message_type_id msg_id,
-             const xcl::XProtocol::Message &msg) -> xcl::Handler_result {
-        print_message_to_consoles("<<<< RECEIVE ", msg);
+  /** Push message handlers that are responsible for tracing.
+   The functionality is enabled by setting "MYSQLX_TRACE_CONNECTION"
+   environment variable to any value.
+   */
+  const auto should_enable_tracing = getenv("MYSQLX_TRACE_CONNECTION");
 
-        DBUG_LOG("debug", "log message "
-                              << "recv: " << msg);
-        return xcl::Handler_result::Continue;
-      });
+  if (force_trace_protocol ||
+      (should_enable_tracing && strlen(should_enable_tracing))) {
+    protocol.add_received_message_handler(
+        [this](xcl::XProtocol *protocol,
+               const xcl::XProtocol::Server_message_type_id msg_id,
+               const xcl::XProtocol::Message &msg) -> xcl::Handler_result {
+          return trace_received_messages(protocol, msg_id, msg);
+        });
 
-  protocol.add_send_message_handler(
-      [this](xcl::XProtocol *protocol,
-             const xcl::XProtocol::Client_message_type_id msg_id,
-             const xcl::XProtocol::Message &msg) -> xcl::Handler_result {
-        print_message_to_consoles(">>>> SEND ", msg);
-
-        DBUG_LOG("debug", "log message "
-                              << "send: " << msg);
-        /** None of processed messages should be filtered out*/
-        return xcl::Handler_result::Continue;
-      });
+    protocol.add_send_message_handler(
+        [this](xcl::XProtocol *protocol,
+               const xcl::XProtocol::Client_message_type_id msg_id,
+               const xcl::XProtocol::Message &msg) -> xcl::Handler_result {
+          return trace_send_messages(protocol, msg_id, msg);
+        });
+  }
 }
 
 void Session_holder::remove_notice_handler() {
@@ -348,6 +233,26 @@ void Session_holder::remove_notice_handler() {
   }
 }
 
+xcl::Handler_result Session_holder::trace_send_messages(
+    xcl::XProtocol *protocol,
+    const xcl::XProtocol::Client_message_type_id msg_id,
+    const xcl::XProtocol::Message &msg) {
+  print_message(">>>> SEND ", msg);
+
+  /** None of processed messages should be filtered out*/
+  return xcl::Handler_result::Continue;
+}
+
+xcl::Handler_result Session_holder::trace_received_messages(
+    xcl::XProtocol *protocol,
+    const xcl::XProtocol::Server_message_type_id msg_id,
+    const xcl::XProtocol::Message &msg) {
+  print_message("<<<< RECEIVE ", msg);
+
+  /** None of processed messages should be filtered out*/
+  return xcl::Handler_result::Continue;
+}
+
 xcl::Handler_result Session_holder::count_received_messages(
     xcl::XProtocol *protocol,
     const xcl::XProtocol::Server_message_type_id msg_id,
@@ -365,18 +270,17 @@ xcl::Handler_result Session_holder::count_received_messages(
   if (msg_name != Mysqlx::Notice::Frame::descriptor()->full_name())
     return xcl::Handler_result::Continue;
 
-  static const std::array<std::string, 5> k_notice_type_id = {
+  static const std::string notice_type_id[] = {
       Mysqlx::Notice::Warning::descriptor()->full_name(),
       Mysqlx::Notice::SessionVariableChanged::descriptor()->full_name(),
       Mysqlx::Notice::SessionStateChanged::descriptor()->full_name(),
       Mysqlx::Notice::GroupReplicationStateChanged::descriptor()->full_name(),
-      Mysqlx::Notice::ServerHello::descriptor()->full_name(),
-  };
+      Mysqlx::Notice::ServerHello::descriptor()->full_name()};
 
   const auto notice_type =
       static_cast<const Mysqlx::Notice::Frame *>(&msg)->type() - 1u;
-  if (notice_type < k_notice_type_id.size())
-    ++m_received_msg_counters[k_notice_type_id[notice_type]];
+  if (notice_type < array_elements(notice_type_id))
+    ++m_received_msg_counters[notice_type_id[notice_type]];
 
   /** None of processed messages should be filtered out*/
   return xcl::Handler_result::Continue;
@@ -408,23 +312,7 @@ xcl::Handler_result Session_holder::dump_notices(const xcl::XProtocol *protocol,
   return xcl::Handler_result::Continue;
 }
 
-void Session_holder::print_message_to_consoles(
-    const std::string &direction, const xcl::XProtocol::Message &msg) {
-  const auto message_size = get_message_size(msg);
-  const auto is_msg_too_big =
-      message_size > k_maximum_size_of_message_in_flow_history;
-
-  if (m_enable_tracing_in_console) {
-    if (is_msg_too_big)
-      print_message(m_console, direction + " BIG ", message_size,
-                    msg.GetTypeName());
-    else
-      print_message(m_console, direction, message_size, msg);
-  }
-
-  if (is_msg_too_big)
-    print_message(m_console_with_flow_history, direction + " BIG ",
-                  message_size, msg.GetTypeName());
-  else
-    print_message(m_console_with_flow_history, direction, message_size, msg);
+void Session_holder::print_message(const std::string &direction,
+                                   const xcl::XProtocol::Message &msg) {
+  m_console.print(direction, msg.ByteSize() + 1, " ", msg);
 }
