@@ -183,6 +183,8 @@ class Command {
   Result cmd_macro_delimiter_compress(std::istream &input,
                                       Execution_context *context,
                                       const std::string &args);
+  Result cmd_assert(std::istream &input, Execution_context *context,
+                    const std::string &args);
   Result cmd_assert_eq(std::istream &input, Execution_context *context,
                        const std::string &args);
   Result cmd_assert_ne(std::istream &input, Execution_context *context,
@@ -191,6 +193,10 @@ class Command {
                        const std::string &args);
   Result cmd_assert_ge(std::istream &input, Execution_context *context,
                        const std::string &args);
+  Result cmd_assert_le(std::istream &input, Execution_context *context,
+                       const std::string &args);
+  Result cmd_assert_lt(std::istream &input, Execution_context *context,
+                       const std::string &args);
   Result cmd_query(std::istream &input, Execution_context *context,
                    const std::string &args);
   Result cmd_noquery(std::istream &input, Execution_context *context,
@@ -224,6 +230,41 @@ class Command {
                        const bool print_column_info);
 
   static void try_result(Result result);
+
+  template <typename Equal_operator>
+  Result cmd_assert_generic(std::istream &input, Execution_context *context,
+                            const std::string &args) {
+    std::vector<std::string> vargs;
+
+    try {
+      aux::split(vargs, args, "\t", true);
+
+      if (2 != vargs.size()) {
+        context->print_error(context->m_script_stack,
+                             "Specified invalid number of arguments for ",
+                             context->m_command_name,
+                             " command: ", vargs.size(), " expecting 2.\n",
+                             "Executed with following argument: \"", args,
+                             "\"");
+        return Result::Stop_with_failure;
+      }
+
+      context->m_variables->replace(&vargs[0]);
+      context->m_variables->replace(&vargs[1]);
+
+      Equal_operator op;
+      if (op(vargs[0], vargs[1])) return Result::Continue;
+    } catch (const std::exception &e) {
+    }
+
+    context->print_error(
+        context->m_script_stack, "Execution of '", context->m_command_name, " ",
+        context->m_command_arguments, "', resulted in an error.\n");
+    context->print_error("Where argument0='", vargs[0], "', argument1='",
+                         vargs[1], "'\n");
+
+    return Result::Stop_with_failure;
+  }
 };
 
 void print_help_commands();
