@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,7 +37,7 @@
 
 namespace {
 
-std::string data_to_bindump(const std::string &bindump) {
+static std::string data_to_bindump(const std::string &bindump) {
   std::string res;
 
   for (size_t i = 0; i < bindump.length(); i++) {
@@ -75,9 +74,7 @@ Block_processor::Result Send_message_block_processor::feed(
   }
 
   if (linebuf[0] == '}') {
-    xcl::XProtocol::Client_message_type_id msg_id =
-        static_cast<xcl::XProtocol::Client_message_type_id>(
-            Mysqlx::ClientMessages::Type_MIN);
+    xcl::XProtocol::Client_message_type_id msg_id;
     std::string processed_buffer = m_buffer;
 
     m_context->m_variables->replace(&m_full_name);
@@ -143,7 +140,7 @@ int Send_message_block_processor::process_client_message(
   return 0;
 }
 
-std::string Send_message_block_processor::message_serialize(
+std::string Send_message_block_processor::message_to_bindump(
     const xcl::XProtocol::Message &message) {
   std::string res;
   std::string out;
@@ -159,15 +156,10 @@ std::string Send_message_block_processor::message_serialize(
   std::swap(res[1], res[2]);
 #endif
 
-  res[4] = client_msgs_by_name[client_msgs_by_full_name[std::string(
-                                   message.GetDescriptor()->full_name())]]
-               .second;
+  res[4] = client_msgs_by_name
+               [client_msgs_by_full_name[message.GetDescriptor()->full_name()]]
+                   .second;
   res.append(out);
 
-  return res;
-}
-
-std::string Send_message_block_processor::message_to_bindump(
-    const xcl::XProtocol::Message &message) {
-  return data_to_bindump(message_serialize(message));
+  return data_to_bindump(res);
 }
