@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,13 +40,11 @@ const std::string k_variable_option_port = "%OPTION_CLIENT_PORT%";
 const std::string k_variable_option_ssl_mode = "%OPTION_SSL_MODE%";
 const std::string k_variable_option_ssl_cipher = "%OPTION_SSL_CIPHER%";
 const std::string k_variable_option_tls_version = "%OPTION_TLS_VERSION%";
+const std::string k_variable_option_compression_client_style =
+    "%OPTION_COMPRESSION_STYLE_CLIENT%";
+const std::string k_variable_option_compression_server_style =
+    "%OPTION_COMPRESSION_STYLE_SERVER%";
 const std::string k_variable_option_compression_algorithm =
     "%OPTION_COMPRESSION_ALGORITHM%";
-const std::string k_variable_option_compression_max_combine_messages =
-    "%OPTION_COMPRESSION_MAX_COMBINE_MESSAGES%";
-const std::string k_variable_option_compression_combine_mixed_messages =
-    "%OPTION_COMPRESSION_COMBINE_MIXED_MESSAGES%";
-const std::string k_variable_option_compression_level =
-    "%OPTION_COMPRESSION_LEVEL%";
 
 #endif  // PLUGIN_X_TESTS_DRIVER_PROCESSOR_VARIABLE_NAMES_H_
