@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -35,7 +35,6 @@
 
 #include <cstdint>
 #include <functional>
-#include <map>
 #include <memory>
 #include <string>
 #include <vector>
@@ -46,6 +45,10 @@ enum class CmdOptionValueReq {
   optional = 0x03,
 };
 
+using ActionFunc = std::function<void(const std::string &)>;
+using AtEndActionFunc = std::function<void()>;
+using OptionNames = std::vector<std::string>;
+
 /** @brief CmdOption stores information about command line options
  *
  * The CmdOption structure stores information about command line options.
@@ -53,7 +56,7 @@ enum class CmdOptionValueReq {
  */
 struct CmdOption {
   using ActionFunc = std::function<void(const std::string &)>;
-  using AtEndActionFunc = std::function<void(const std::string &)>;
+  using AtEndActionFunc = std::function<void()>;
   using OptionNames = std::vector<std::string>;
 
   OptionNames names;
@@ -68,8 +71,7 @@ struct CmdOption {
   CmdOption(
       OptionNames names_, std::string description_,
       CmdOptionValueReq value_req_, const std::string metavar_,
-      ActionFunc action_,
-      AtEndActionFunc at_end_action_ = [](const std::string &) {})
+      ActionFunc action_, AtEndActionFunc at_end_action_ = [] {})
       : names(names_),
         description(description_),
         value_req(value_req_),
@@ -142,20 +144,15 @@ class HARNESS_EXPORT CmdArgHandler {
   /** @brief Constructor
    *
    * @param allow_rest_arguments_ whether we allow rest arguments or not
-   * @param ignore_unknown_arguments_ whether we ignore unknown arguments or
-   * give an error
    */
-  explicit CmdArgHandler(bool allow_rest_arguments_,
-                         bool ignore_unknown_arguments_ = false)
-      : allow_rest_arguments(allow_rest_arguments_),
-        ignore_unknown_arguments(ignore_unknown_arguments_) {}
+  explicit CmdArgHandler(bool allow_rest_arguments_)
+      : allow_rest_arguments(allow_rest_arguments_) {}
 
   /** @brief Default constructor
    *
-   * By default, rest arguments are not allowed and unknown arguments are not
-   * ignored.
+   * By default, rest arguments are not allowed.
    */
-  CmdArgHandler() : CmdArgHandler(false, false) {}
+  CmdArgHandler() : CmdArgHandler(false) {}
 
   /** @brief Adds a command line option
    *
@@ -217,8 +214,7 @@ class HARNESS_EXPORT CmdArgHandler {
       const CmdOption::OptionNames &names, const std::string &description,
       const CmdOptionValueReq &value_req, const std::string &metavar,
       CmdOption::ActionFunc action,
-      CmdOption::AtEndActionFunc at_end_action = [](const std::string &) {
-      }) noexcept;
+      CmdOption::AtEndActionFunc at_end_action = [] {}) noexcept;
 
   void add_option(const CmdOption &other) noexcept;
 
@@ -294,8 +290,8 @@ class HARNESS_EXPORT CmdArgHandler {
    * @param name name of the option as string
    * @returns iterator object
    */
-  OptionContainer::const_iterator find_option(
-      const std::string &name) const noexcept;
+  OptionContainer::const_iterator find_option(const std::string &name) const
+      noexcept;
 
   using UsagePredicate =
       std::function<std::pair<bool, CmdOption>(const CmdOption &)>;
@@ -335,9 +331,11 @@ class HARNESS_EXPORT CmdArgHandler {
         });
   }
 
-  std::vector<std::string> usage_lines_if(
-      const std::string &prefix, const std::string &rest_metavar, size_t width,
-      UsagePredicate predicate) const noexcept;
+  std::vector<std::string> usage_lines_if(const std::string &prefix,
+                                          const std::string &rest_metavar,
+                                          size_t width,
+                                          UsagePredicate predicate) const
+      noexcept;
 
   /** @brief Produces description of all options
    *
@@ -364,8 +362,9 @@ class HARNESS_EXPORT CmdArgHandler {
    * @param indent how much the description should be indented.
    * @return vector of strings
    */
-  std::vector<std::string> option_descriptions(
-      const size_t width, const size_t indent) const noexcept;
+  std::vector<std::string> option_descriptions(const size_t width,
+                                               const size_t indent) const
+      noexcept;
 
   /** @brief Returns an iterator to first option
    *
@@ -416,25 +415,11 @@ class HARNESS_EXPORT CmdArgHandler {
   /** @brief Whether to allow rest arguments or not **/
   bool allow_rest_arguments;
 
-  /** @brief Whether to ignore unknown arguments **/
-  bool ignore_unknown_arguments;
-
-  /** @brief The key is a section identificator (section name and optional
-   * section key), the value is a map of all the overrides for a given section
-   * (option/value pairs) **/
-  using ConfigOverwrites = std::map<std::pair<std::string, std::string>,
-                                    std::map<std::string, std::string>>;
-  const ConfigOverwrites &get_config_overwrites() const noexcept {
-    return config_overwrites_;
-  }
-
  private:
   /** @brief Vector with registered options **/
   std::vector<CmdOption> options_;
   /** @brief Vector with arguments as strings not processed as options **/
   std::vector<std::string> rest_arguments_;
-  /** @brief Keeps configuration options overwrites **/
-  ConfigOverwrites config_overwrites_;
 };
 
 #endif  // HARNESS_ARG_HANDLER_INCLUDED
