@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -28,18 +28,15 @@
 #include "harness_export.h"
 
 #include <memory>
-#include <ostream>
 #include <stdexcept>
 #include <string>
-#include <system_error>
-#include <vector>
-
 #ifndef _WIN32
 #include <fcntl.h>
 #endif
 
-#include "mysql/harness/access_rights.h"
-#include "mysql/harness/stdx/expected.h"
+#ifdef _WIN32
+#include <aclapi.h>
+#endif
 
 namespace mysql_harness {
 
@@ -163,17 +160,6 @@ class HARNESS_EXPORT Path {
    */
   bool is_regular() const;
 
-  /**
-   * Check if the path is absolute or not
-   *
-   * The path is considered absolute if it starts with one of:
-   *   Unix:    '/'
-   *   Windows: '/' or '\' or '.:' (where . is any character)
-   * else:
-   *   it's considered relative (empty path is also relative in such respect)
-   */
-  bool is_absolute() const;
-
   /**
    * Check if path exists
    */
@@ -320,16 +306,12 @@ class HARNESS_EXPORT Directory : public Path {
    *
    * A directory iterator is an input iterator.
    */
-  class HARNESS_EXPORT DirectoryIterator {
+  using DirectoryIteratorBase = std::iterator<std::input_iterator_tag, Path>;
+
+  class HARNESS_EXPORT DirectoryIterator : public DirectoryIteratorBase {
     friend class Directory;
 
    public:
-    using value_type = Path;
-    using iterator_category = std::input_iterator_tag;
-    using difference_type = std::ptrdiff_t;
-    using pointer = value_type *;
-    using reference = value_type &;
-
     DirectoryIterator(const Path &path,
                       const std::string &pattern = std::string());
 
@@ -419,13 +401,6 @@ class HARNESS_EXPORT Directory : public Path {
    */
   DirectoryIterator begin();
 
-  /**
-   * Constant iterator to first entry.
-   *
-   * @return Returns a constant iterator pointing to the first entry.
-   */
-  DirectoryIterator cbegin() const;
-
   /**
    * Iterator past-the-end of entries.
    *
@@ -433,31 +408,6 @@ class HARNESS_EXPORT Directory : public Path {
    */
   DirectoryIterator end();
 
-  /**
-   * Constant iterator past-the-end of entries.
-   *
-   * @return Returns a constant iterator pointing *past-the-end* of the entries.
-   */
-  DirectoryIterator cend() const;
-
-  /**
-   * Check if the directory is empty.
-   *
-   * @retval true Directory is empty.
-   * @retval false Directory is no empty.
-   */
-  bool is_empty() const;
-
-  /**
-   * Recursively list all paths in a directory.
-   *
-   * Recursively create a list of relative paths from a directory. Path will
-   * be relative to the given directory. Empty directories are also listed.
-   *
-   * @return Recursive list of paths from a direcotry.
-   */
-  std::vector<Path> list_recursive() const;
-
   /**
    * Iterate over entries matching a glob.
    */
@@ -476,11 +426,10 @@ class HARNESS_EXPORT Directory : public Path {
  *
  * @param dir path of the directory to be removed; this directory must be empty
  *
- * @return void on success, error_code on failure
+ * @return 0 on success, -1 on error and sets errno
  */
 HARNESS_EXPORT
-stdx::expected<void, std::error_code> delete_dir(
-    const std::string &dir) noexcept;
+int delete_dir(const std::string &dir) noexcept;
 
 /** @brief Removes a file.
  *
@@ -488,11 +437,10 @@ stdx::expected<void, std::error_code> delete_dir(
  *
  * @param path of the file to be removed
  *
- * @return void on success, error_code on failure
+ * @return 0 on success, -1 on error
  */
 HARNESS_EXPORT
-stdx::expected<void, std::error_code> delete_file(
-    const std::string &path) noexcept;
+int delete_file(const std::string &path) noexcept;
 
 /** @brief Removes directory and all its contents.
  *
@@ -500,11 +448,10 @@ stdx::expected<void, std::error_code> delete_file(
  *
  * @param dir path of the directory to be removed
  *
- * @return void on success, error_code on failure
+ * @return 0 on success, -1 on error
  */
 HARNESS_EXPORT
-stdx::expected<void, std::error_code> delete_dir_recursive(
-    const std::string &dir) noexcept;
+int delete_dir_recursive(const std::string &dir) noexcept;
 
 /** @brief Creates a temporary directory with partially-random name and returns
  * its path.
@@ -535,13 +482,35 @@ std::string get_plugin_dir(const std::string &runtime_dir);
 HARNESS_EXPORT
 std::string get_tests_data_dir(const std::string &runtime_dir);
 
+#ifdef _WIN32
+/**
+ * Deleter for smart pointers pointing to objects allocated with `std::malloc`.
+ */
+template <typename T>
+class StdFreeDeleter {
+ public:
+  void operator()(T *ptr) { std::free(ptr); }
+};
+
+/**
+ * Retrieves file's DACL security descriptor.
+ *
+ * @param[in] file_name File name.
+ *
+ * @return File's DACL security descriptor.
+ *
+ * @throw std::exception Failed to retrieve security descriptor.
+ */
+HARNESS_EXPORT std::unique_ptr<SECURITY_DESCRIPTOR, decltype(&free)>
+get_security_descriptor(const std::string &file_name);
+
+#endif
+
 #ifndef _WIN32
 using perm_mode = mode_t;
-HARNESS_EXPORT
 extern const perm_mode kStrictDirectoryPerm;
 #else
 using perm_mode = int;
-HARNESS_EXPORT
 extern const perm_mode kStrictDirectoryPerm;
 #endif
 
@@ -570,19 +539,6 @@ int mkdir(const std::string &dir, perm_mode mode, bool recursive = false);
  */
 void HARNESS_EXPORT make_file_public(const std::string &file_name);
 
-#ifdef _WIN32
-/**
- * Changes file access permissions to be readable by all users.
- *
- * On Windows, Everyone group is granted read access to the file.
- *
- * @param[in] file_name File name.
- *
- * @throw std::exception Failed to change file permissions.
- */
-void make_file_readable_for_everyone(const std::string &file_name);
-#endif
-
 /**
  * Changes file access permissions to be accessible only by a limited set of
  * users.
@@ -602,19 +558,6 @@ void HARNESS_EXPORT
 make_file_private(const std::string &file_name,
                   const bool read_only_for_local_service = true);
 
-/**
- * Changes file access permissions to be read only.
- *
- * On Unix, the function sets file permission mask to 555.
- * On Windows, all permissions to this file are read access only for Everyone
- * group, LocalService account gets read access.
- *
- * @param[in] file_name File name.
- *
- * @throw std::exception Failed to change file permissions.
- */
-void HARNESS_EXPORT make_file_readonly(const std::string &file_name);
-
 /**
  * Verifies access permissions of a file.
  *
