@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +25,15 @@
 /** @file
  * Module for implementing the Logger functionality.
  */
+
 #include "mysql/harness/logging/handler.h"
+#include "mysql/harness/logging/logging.h"
+
+#include "common.h"
+#include "mysql/harness/config_parser.h"
+#include "mysql/harness/filesystem.h"
+#include "mysql/harness/plugin.h"
+#include "utilities.h"  // string_format()
 
 #include <algorithm>
 #include <cerrno>
@@ -41,18 +48,22 @@
 #include <unistd.h>
 #endif
 
-#include "mysql/harness/config_parser.h"
-#include "mysql/harness/filesystem.h"
-#include "mysql/harness/logging/logging.h"
-#include "mysql/harness/plugin.h"
-#include "mysql/harness/utility/string.h"  // string_format
-
 using mysql_harness::Path;
 
+#if defined(_MSC_VER) && defined(logger_EXPORTS)
+/* We are building this library */
+#define LOGGER_API __declspec(dllexport)
+#else
+#define LOGGER_API
+#endif
+
+using std::ofstream;
+using std::ostringstream;
+
 using namespace std::chrono_literals;
 
-static const char *const level_str[] = {"FATAL", "SYSTEM", "ERROR", "WARNING",
-                                        "INFO",  "NOTE",   "DEBUG"};
+static const char *const level_str[] = {"FATAL", "ERROR", "WARNING", "INFO",
+                                        "DEBUG"};
 
 namespace mysql_harness {
 
@@ -133,6 +144,9 @@ std::string Handler::format(const Record &record) const {
 
 void Handler::handle(const Record &record) { do_log(record); }
 
+// satisfy ODR
+constexpr const char *StreamHandler::kDefaultName;
+
 ////////////////////////////////////////////////////////////////
 // class StreamHandler
 
@@ -144,19 +158,8 @@ StreamHandler::StreamHandler(std::ostream &out, bool format_messages,
 void StreamHandler::do_log(const Record &record) {
   std::lock_guard<std::mutex> lock(stream_mutex_);
   stream_ << format(record) << std::endl;
-
-  has_logged(true);
 }
 
-////////////////////////////////////////////////////////////////
-// class NullHandler
-
-NullHandler::NullHandler(bool format_messages, LogLevel level,
-                         LogTimestampPrecision timestamp_precision)
-    : Handler(format_messages, level, timestamp_precision) {}
-
-void NullHandler::do_log(const Record & /*record*/) {}
-
 ////////////////////////////////////////////////////////////////
 // class FileHandler
 
@@ -171,11 +174,6 @@ FileHandler::FileHandler(const Path &path, bool format_messages, LogLevel level,
     pos = log_path.find_last_of('/');
     if (pos != std::string::npos) log_path.erase(pos);  // log_path = /path/to
 
-    if (log_path.empty()) {
-      throw std::runtime_error("filelog sink configured but the filename '" +
-                               path.str() + "' is not a valid log filename");
-    }
-
     // mkdir if it doesn't exist
     if (mysql_harness::Path(log_path).exists() == false &&
         mkdir(log_path, kStrictDirectoryPerm) != 0) {
@@ -195,9 +193,7 @@ FileHandler::FileHandler(const Path &path, bool format_messages, LogLevel level,
   reopen();  // not opened yet so it's just for open in this context
 }
 
-void FileHandler::reopen(const std::string dst) {  // namespace logging
-  std::exception_ptr eptr = nullptr;
-
+void FileHandler::reopen() {
   // here we need to lock the mutex that's used while logging
   // to prevent other threads from trying to log to invalid stream
   std::lock_guard<std::mutex> lock(stream_mutex_);
@@ -205,45 +201,10 @@ void FileHandler::reopen(const std::string dst) {  // namespace logging
   // if was open before, close first
   if (fstream_.is_open()) {
     fstream_.close();
-
-    // With closed stream we may rename file on any platform, windows included
-    if (!dst.empty()) {
-      if (rename(file_path_.str().c_str(), dst.c_str())) {
-        auto last_error =
-#ifdef _WIN32
-            GetLastError()
-#else
-            errno
-#endif
-            ;
-        if (last_error != 0) {
-          // Exceptions cannot be thrown directly here, but stashed until after
-          // reopening the logfile again. Otherwise all logging ends up on
-          // console due to closed logfile.
-          mysql_harness::Path dstpath(dst);
-          if (dstpath.exists()) {
-            eptr = make_exception_ptr(std::system_error(
-                last_error, std::system_category(),
-                "File exists. Cannot rename to " + dstpath.str()));
-          } else {
-            if (last_error != ENOENT) {
-              eptr = make_exception_ptr(
-                  std::system_error(last_error, std::system_category(),
-                                    "Cannot rename file in directory " +
-                                        dstpath.dirname().str()));
-            }
-          }
-        }
-      }
-    }
-
     fstream_.clear();
   }
 
-#ifdef _WIN32
-  const bool created = !file_path_.exists();
-#endif
-  fstream_.open(file_path_.str(), std::ofstream::app);
+  fstream_.open(file_path_.str(), ofstream::app);
   if (fstream_.fail()) {
     // get the last-error early as with VS2015 it has been seen
     // that something in std::system_error() called SetLastError(0)
@@ -265,21 +226,6 @@ void FileHandler::reopen(const std::string dst) {  // namespace logging
           "Cannot create file in directory " + file_path_.dirname().str());
     }
   }
-
-#ifdef _WIN32
-  // If we are running in the context of the Windows service and we created the
-  // file it will be unaccessible by anyone else which makes it kind of
-  // pointless as this is a log file. Let's make sure this file is readable if
-  // we just created it.
-  if (created) {
-    make_file_readable_for_everyone(file_path_.str());
-  }
-#endif
-
-  // After reopening the logfile, is it safe to throw earlier exceptions
-  if (eptr) {
-    std::rethrow_exception(eptr);
-  }
 }  // namespace logging
 
 void FileHandler::do_log(const Record &record) {
@@ -287,14 +233,13 @@ void FileHandler::do_log(const Record &record) {
   stream_ << format(record) << std::endl;
   // something is wrong with the logging file, let's at least log it on the
   // std error as a fallback
-  if (stream_.fail()) {
-    std::cerr << format(record) << std::endl;
-  } else {
-    has_logged(true);
-  }
+  if (stream_.fail()) std::cerr << format(record) << std::endl;
 }
 
-FileHandler::~FileHandler() = default;
+// satisfy ODR
+constexpr const char *FileHandler::kDefaultName;
+
+FileHandler::~FileHandler() {}
 
 }  // namespace logging
 
