@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+  Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,14 +31,14 @@
 #include <string>
 #include <system_error>
 
-#include "mysqlrouter/http_auth_backend_lib_export.h"
+#include "mysqlrouter/http_auth_backend_export.h"
 
 class HttpAuthBackend;
 
 /**
  * public API of the HttpAuthBackend plugin.
  */
-class HTTP_AUTH_BACKEND_LIB_EXPORT HttpAuthBackendComponent {
+class HTTP_AUTH_BACKEND_EXPORT HttpAuthBackendComponent {
  public:
   using value_type = std::map<std::string, std::shared_ptr<HttpAuthBackend>>;
 
@@ -49,15 +48,9 @@ class HTTP_AUTH_BACKEND_LIB_EXPORT HttpAuthBackendComponent {
   static HttpAuthBackendComponent &get_instance();
 
   /**
-   * register an authentication backend.
+   * initialize component from backends.
    */
-  void add_backend(const std::string &name,
-                   std::shared_ptr<HttpAuthBackend> backend);
-
-  /**
-   * unregister an authentication backend.
-   */
-  void remove_backend(const std::string &name);
+  void init(std::shared_ptr<value_type> backends);
 
   /**
    * authenticate user against auth-backend.
@@ -74,8 +67,8 @@ class HTTP_AUTH_BACKEND_LIB_EXPORT HttpAuthBackendComponent {
   HttpAuthBackendComponent(HttpAuthBackendComponent const &) = delete;
   void operator=(HttpAuthBackendComponent const &) = delete;
 
-  std::mutex backends_m_;
-  value_type auth_backends_;
+  std::mutex mu;  // request handler mutex
+  std::weak_ptr<value_type> auth_backends_;
 
   HttpAuthBackendComponent() = default;
 };
