@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+  Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,17 +26,20 @@
 #define MYSQLROUTER_HTTP_SERVER_PLUGIN_INCLUDED
 
 #include <mutex>
-#include <regex>
 #include <string>
 #include <thread>
 #include <vector>
 
-#include "mysql/harness/net_ts/impl/socket_constants.h"
-#include "mysql/harness/net_ts/internet.h"
-#include "mysql/harness/net_ts/io_context.h"
-#include "mysql/harness/stdx/monitor.h"
-#include "mysqlrouter/http_common.h"
+#include <event2/event.h>  // event_base
+#include <event2/http.h>   // evhttp_new
+#include <event2/util.h>   // evutil_socket_t
+
 #include "mysqlrouter/http_server_component.h"
+#include "posix_re.h"
+
+using harness_socket_t = evutil_socket_t;
+
+void stop_eventloop(evutil_socket_t, short, void *cb_arg);
 
 class HttpRequestRouter {
  public:
@@ -57,7 +59,7 @@ class HttpRequestRouter {
  private:
   struct RouterData {
     std::string url_regex_str;
-    std::regex url_regex;
+    PosixRE url_regex;
     std::unique_ptr<BaseRequestHandler> handler;
   };
   std::vector<RouterData> request_handlers_;
@@ -82,43 +84,35 @@ class HttpRequestRouter {
  */
 class HttpRequestThread {
  public:
-  HttpRequestThread() {
+  HttpRequestThread()
+      : ev_base(event_base_new(), &event_base_free),
+        ev_http(evhttp_new(ev_base.get()), &evhttp_free),
+        ev_shutdown_timer(event_new(ev_base.get(), -1, EV_PERSIST,
+                                    stop_eventloop, ev_base.get()),
+                          &event_free) {
     // enable all methods to allow the higher layers to handle them
     //
     // CONNECT, TRACE and OPTIONS are disabled by default if not explicitly
     // enabled.
-    event_http_.set_allowed_http_methods(
-        HttpMethod::Bitset().set(/*all types*/));
+    evhttp_set_allowed_methods(
+        ev_http.get(), EVHTTP_REQ_CONNECT | EVHTTP_REQ_DELETE | EVHTTP_REQ_GET |
+                           EVHTTP_REQ_HEAD | EVHTTP_REQ_OPTIONS |
+                           EVHTTP_REQ_PATCH | EVHTTP_REQ_POST | EVHTTP_REQ_PUT |
+                           EVHTTP_REQ_TRACE);
   }
 
-  HttpRequestThread(HttpRequestThread &&object)
-      : event_base_(std::move(object.event_base_)),
-        event_http_(std::move(object.event_http_)),
-        accept_fd_(object.accept_fd_) /* just copy */,
-        initialized_(object.is_initalized()) /* just copy */ {}
-
-  using native_handle_type = EventBaseSocket;
-
-  native_handle_type get_socket_fd() { return accept_fd_; }
+  harness_socket_t get_socket_fd() { return accept_fd_; }
 
   void accept_socket();
   void set_request_router(HttpRequestRouter &router);
   void wait_and_dispatch();
 
- public:  // Thread safe methods
-  void break_dispatching();
-  void wait_until_ready();
-
  protected:
-  static void on_event_loop_ready(native_handle_type, short, void *);
+  std::unique_ptr<event_base, decltype(&event_base_free)> ev_base;
+  std::unique_ptr<evhttp, decltype(&evhttp_free)> ev_http;
+  std::unique_ptr<event, decltype(&event_free)> ev_shutdown_timer;
 
-  bool is_initalized() const;
-  void initialization_finished();
-
-  EventBase event_base_;
-  EventHttp event_http_{&event_base_};
-  native_handle_type accept_fd_{kEventBaseInvalidSocket};
-  WaitableMonitor<bool> initialized_{false};
+  harness_socket_t accept_fd_{-1};
 };
 
 class HttpServer {
@@ -137,8 +131,6 @@ class HttpServer {
   virtual ~HttpServer() { join_all(); }
 
   virtual void start(size_t max_threads);
-  void stop();
-
   void add_route(const std::string &url_regex,
                  std::unique_ptr<BaseRequestHandler> cb);
   void remove_route(const std::string &url_regex);
@@ -151,9 +143,6 @@ class HttpServer {
   uint16_t port_;
   HttpRequestRouter request_router_;
 
-  net::io_context io_ctx_;
-  net::ip::tcp::acceptor listen_sock_{io_ctx_};
-
   std::vector<std::thread> sys_threads_;
 };
 
