@@ -495,19 +495,24 @@ MetadataCache::MetadataCache(
     std::shared_ptr<MetaData>
         cluster_metadata,  // this could be changed to UniquePtr
     std::chrono::milliseconds ttl, const mysqlrouter::SSLOptions &ssl_options,
-    const std::string &cluster, size_t thread_stack_size)
+    const std::string &cluster, size_t thread_stack_size,
+    bool use_gr_notifications)
     : group_replication_id_(group_replication_id),
-      refresh_thread_(thread_stack_size) {
+      refresh_thread_(thread_stack_size),
+      use_gr_notifications_(use_gr_notifications) {
   for (const auto &s : metadata_servers) {
     metadata_servers_.emplace_back(s);
   }
   ttl_ = ttl;
   cluster_name_ = cluster;
-  terminated_ = terminator_.get_future();
   meta_data_ = cluster_metadata;
   ssl_options_ = ssl_options;
 }
 
+MetadataCache::~MetadataCache() {
+  meta_data_->shutdown_gr_notifications_listener();
+}
+
 void *MetadataCache::run_thread(void *context) {
   mysqlrouter::MySQLClientThreadToken api_token;
   MetadataCache *metadata_cache = static_cast<MetadataCache *>(context);
@@ -526,8 +531,7 @@ void MetadataCache::refresh_thread() {
   const std::chrono::milliseconds kTerminateOrForcedRefreshCheckInterval =
       std::chrono::seconds(1);
 
-  while (terminated_.wait_for(std::chrono::seconds(0)) !=
-         std::future_status::ready) {
+  while (!terminated_) {
     refresh();
 
     auto ttl_left = ttl_;
@@ -538,7 +542,15 @@ void MetadataCache::refresh_thread() {
       auto sleep_for =
           std::min(ttl_left, kTerminateOrForcedRefreshCheckInterval);
 
-      if (terminated_.wait_for(sleep_for) == std::future_status::ready) return;
+      {
+        std::unique_lock<std::mutex> lock(refresh_wait_mtx_);
+        refresh_wait_.wait_for(lock, sleep_for);
+        if (terminated_) return;
+        if (refresh_requested_) {
+          refresh_requested_ = false;
+          break;  // go to the refresh() in the outer loop
+        }
+      }
 
       ttl_left -= sleep_for;
       {
@@ -556,13 +568,20 @@ void MetadataCache::refresh_thread() {
  * Connect to the metadata servers and refresh the metadata information in the
  * cache.
  */
-void MetadataCache::start() { refresh_thread_.run(&run_thread, this); }
+void MetadataCache::start() {
+  // start refresh thread that uses classic protocol
+  refresh_thread_.run(&run_thread, this);
+}
 
 /**
  * Stop the refresh thread.
  */
 void MetadataCache::stop() noexcept {
-  terminator_.set_value(0);
+  {
+    std::unique_lock<std::mutex> lk(refresh_wait_mtx_);
+    terminated_ = true;
+  }
+  refresh_wait_.notify_one();
   refresh_thread_.join();
 }
 
@@ -594,24 +613,22 @@ bool metadata_cache::ManagedInstance::operator==(
          std::fabs(weight - other.weight) <
              0.001 &&  // 0.001 = reasonable guess, change if needed
          host == other.host &&
-         location == other.location && port == other.port &&
-         version_token == other.version_token && xport == other.xport;
+         port == other.port && version_token == other.version_token &&
+         xport == other.xport;
 }
 
 metadata_cache::ManagedInstance::ManagedInstance(
     const std::string &p_replicaset_name,
     const std::string &p_mysql_server_uuid, const std::string &p_role,
     const ServerMode p_mode, const float p_weight,
-    const unsigned int p_version_token, const std::string &p_location,
-    const std::string &p_host, const unsigned int p_port,
-    const unsigned int p_xport)
+    const unsigned int p_version_token, const std::string &p_host,
+    const uint16_t p_port, const uint16_t p_xport)
     : replicaset_name(p_replicaset_name),
       mysql_server_uuid(p_mysql_server_uuid),
       role(p_role),
       mode(p_mode),
       weight(p_weight),
       version_token(p_version_token),
-      location(p_location),
       host(p_host),
       port(p_port),
       xport(p_xport) {}
@@ -633,15 +650,18 @@ inline bool compare_instance_lists(const MetaData::ReplicaSetsByName &map_a,
   auto ai = map_a.begin();
   auto bi = map_b.begin();
   for (; ai != map_a.end(); ++ai, ++bi) {
-    if ((ai->first != bi->first) ||
-        (ai->second.members.size() != bi->second.members.size()))
+    if ((ai->first != bi->first)) return false;
+    // we need to compare 2 vectors if their content is the same
+    // but order of their elements can be different as we use
+    // SQL with no "ORDER BY" to fetch them from different nodes
+    if (ai->second.members.size() != bi->second.members.size()) return false;
+    if (!std::is_permutation(ai->second.members.begin(),
+                             ai->second.members.end(),
+                             bi->second.members.begin())) {
       return false;
-    auto a = ai->second.members.begin();
-    auto b = bi->second.members.begin();
-    for (; a != ai->second.members.end(); ++a, ++b) {
-      if (!(*a == *b)) return false;
     }
   }
+
   return true;
 }
 
@@ -666,8 +686,7 @@ void MetadataCache::refresh() {
   // fetch metadata
   bool broke_loop = false;
   for (auto &metadata_server : metadata_servers_) {
-    if (terminated_.wait_for(std::chrono::seconds(0)) ==
-        std::future_status::ready) {
+    if (terminated_) {
       broke_loop = true;
       break;
     }
@@ -679,6 +698,10 @@ void MetadataCache::refresh() {
     }
     fetched = fetch_metadata_from_connected_instance(metadata_server, changed);
     if (fetched) {
+      last_refresh_succeeded_ = std::chrono::system_clock::now();
+      last_metadata_server_host_ = metadata_server.host;
+      last_metadata_server_port_ = metadata_server.port;
+      refresh_succeeded_++;
       break;  // successfully updated metadata
     }
   }
@@ -698,6 +721,9 @@ void MetadataCache::refresh() {
     return;
   }
 
+  refresh_failed_++;
+  last_refresh_failed_ = std::chrono::system_clock::now();
+
   // we failed to fetch metadata from any of the metadata servers
   if (!broke_loop)
     log_error("Failed fetching metadata from any of the %u metadata servers.",
@@ -731,7 +757,7 @@ bool MetadataCache::fetch_metadata_from_connected_instance(
     if (replicaset_data_temp.empty()) {
       log_warning(
           "Tried node %s on host %s, port %d as a metadata server, it does not "
-          "contan metadata for replication group %s",
+          "contain metadata for replication group %s",
           instance.mysql_server_uuid.c_str(), instance.host.c_str(),
           instance.port, group_replication_id_.c_str());
       return false;
@@ -803,17 +829,32 @@ bool MetadataCache::fetch_metadata_from_connected_instance(
 }
 
 void MetadataCache::on_instances_changed(const bool md_servers_reachable) {
-  std::lock_guard<std::mutex> lock(replicaset_instances_change_callbacks_mtx_);
-
   auto instances = replicaset_lookup("" /*cluster_name_*/);
+  {
+    std::lock_guard<std::mutex> lock(
+        replicaset_instances_change_callbacks_mtx_);
 
-  for (auto &replicaset_clb : listeners_) {
-    const std::string replicaset_name = replicaset_clb.first;
+    for (auto &replicaset_clb : listeners_) {
+      const std::string replicaset_name = replicaset_clb.first;
 
-    for (auto each : listeners_[replicaset_name]) {
-      each->notify(instances, md_servers_reachable);
+      for (auto each : listeners_[replicaset_name]) {
+        each->notify(instances, md_servers_reachable);
+      }
     }
   }
+
+  if (use_gr_notifications_) {
+    meta_data_->setup_gr_notifications_listener(
+        instances, [this]() { on_refresh_requested(); });
+  }
+}
+
+void MetadataCache::on_refresh_requested() {
+  {
+    std::unique_lock<std::mutex> lock(refresh_wait_mtx_);
+    refresh_requested_ = true;
+  }
+  refresh_wait_.notify_one();
 }
 
 void MetadataCache::mark_instance_reachability(
