@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+  Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,7 +39,7 @@
 #include "mysql/harness/vt100.h"
 #include "mysql/harness/vt100_filter.h"
 #include "print_version.h"
-#include "router_config.h"  // MYSQL_ROUTER_PACKAGE_NAME
+#include "router_config.h"
 #include "welcome_copyright_notice.h"
 
 using mysql_harness::Path;
@@ -50,6 +49,8 @@ using testing::StrEq;
 using testing::ValuesIn;
 using testing::WithParamInterface;
 
+using std::string;
+
 Path g_origin_path;
 
 class PluginInfoAppTest : public ::testing::Test {
@@ -61,11 +62,10 @@ class PluginInfoAppTest : public ::testing::Test {
   void SetUp() override;
 
   void verify_version_output();
-  void verify_plugin_info(const std::string &brief, const std::string &version,
-                          const std::string &requires_plugins,
-                          const std::string &conflicts);
+  void verify_plugin_info(const string &brief, const string &version,
+                          const string &requires, const string &conflicts);
 
-  std::string get_plugin_file_path(const std::string &plugin_name);
+  string get_plugin_file_path(const string &plugin_name);
 
   std::stringstream out_stream_;
   Vt100Filter filtered_out_streambuf_;
@@ -84,10 +84,9 @@ void PluginInfoAppTest::SetUp() {
   plugin_dir_ = mysql_harness::get_plugin_dir(g_origin_path.str());
 }
 
-std::string PluginInfoAppTest::get_plugin_file_path(
-    const std::string &plugin_name) {
+string PluginInfoAppTest::get_plugin_file_path(const string &plugin_name) {
   Path plugin_path = plugin_dir_;
-  std::string plugin_file = plugin_name;
+  string plugin_file = plugin_name;
 
 #ifndef _WIN32
   plugin_file += ".so";
@@ -132,17 +131,17 @@ void PluginInfoAppTest::verify_version_output() {
   std::string version_string;
   build_version(std::string(MYSQL_ROUTER_PACKAGE_NAME), &version_string);
 
-  const std::string kVersionOutput =
+  const string kVersionOutput =
       version_string + "\n" + ORACLE_WELCOME_COPYRIGHT_NOTICE("2015") + "\n";
 
   EXPECT_EQ(out_stream_.str(), kVersionOutput);
   EXPECT_THAT(out_stream_err_.str(), StrEq(""));
 }
 
-void PluginInfoAppTest::verify_plugin_info(const std::string &brief,
-                                           const std::string &version,
-                                           const std::string &requires_plugins,
-                                           const std::string &conflicts) {
+void PluginInfoAppTest::verify_plugin_info(const string &brief,
+                                           const string &version,
+                                           const string &requires,
+                                           const string &conflicts) {
   EXPECT_THAT(out_stream_err_.str(), StrEq(""));
 
   const auto abi_version = ::mysql_harness::PLUGIN_ABI_VERSION;
@@ -150,13 +149,13 @@ void PluginInfoAppTest::verify_plugin_info(const std::string &brief,
       std::to_string(ABI_VERSION_MAJOR(abi_version)) + "." +
       std::to_string(ABI_VERSION_MINOR(abi_version));
 
-  const std::string expected_json =
+  const string expected_json =
       "{\n"
       "    \"abi-version\": \"" +
       abi_version_str +
       "\",\n"
       "    \"arch-descriptor\": \"" +
-      std::string(mysql_harness::ARCHITECTURE_DESCRIPTOR) +
+      string(mysql_harness::ARCHITECTURE_DESCRIPTOR) +
       "\",\n"
       "    \"brief\": \"" +
       brief +
@@ -165,7 +164,7 @@ void PluginInfoAppTest::verify_plugin_info(const std::string &brief,
       version +
       "\",\n"
       "    \"requires\": [" +
-      requires_plugins +
+      requires +
       "],\n"
       "    \"conflicts\": [" +
       conflicts +
@@ -176,8 +175,8 @@ void PluginInfoAppTest::verify_plugin_info(const std::string &brief,
 }
 
 TEST_F(PluginInfoAppTest, NoParametersPassed) {
-  PluginInfoFrontend plugin_info_app(kPluginInfoAppExeFileName, {},
-                                     out_stream_);
+  PluginInfoFrontend plugin_info_app(kPluginInfoAppExeFileName, {}, out_stream_,
+                                     out_stream_err_);
 
   EXPECT_THROW(plugin_info_app.run(), UsageError);
 }
@@ -189,7 +188,7 @@ TEST_F(PluginInfoAppTest, NoParametersPassed) {
 TEST_F(PluginInfoAppTest, HelpRequested_plain) {
   std::vector<std::string> args{"--help"};
   PluginInfoFrontend plugin_info_app(kPluginInfoAppExeFileName, args,
-                                     filtered_out_stream_);
+                                     filtered_out_stream_, out_stream_err_);
 
   int res = plugin_info_app.run();
 
@@ -204,7 +203,7 @@ TEST_F(PluginInfoAppTest, HelpRequested_plain) {
 TEST_F(PluginInfoAppTest, HelpRequested_vt100) {
   std::vector<std::string> args{"--help"};
   PluginInfoFrontend plugin_info_app(kPluginInfoAppExeFileName, args,
-                                     out_stream_);
+                                     out_stream_, out_stream_err_);
 
   int res = plugin_info_app.run();
 
@@ -219,7 +218,7 @@ TEST_F(PluginInfoAppTest, HelpRequested_vt100) {
 TEST_F(PluginInfoAppTest, VersionRequested) {
   std::vector<std::string> args{"--version"};
   PluginInfoFrontend plugin_info_app(kPluginInfoAppExeFileName, args,
-                                     out_stream_);
+                                     out_stream_, out_stream_err_);
 
   int res = plugin_info_app.run();
 
@@ -230,7 +229,7 @@ TEST_F(PluginInfoAppTest, VersionRequested) {
 TEST_F(PluginInfoAppTest, WrongNumberOfParams) {
   std::vector<std::string> args{"one", "two", "three"};
   PluginInfoFrontend plugin_info_app(kPluginInfoAppExeFileName, args,
-                                     out_stream_);
+                                     out_stream_, out_stream_err_);
 
   EXPECT_THROW(plugin_info_app.run(), UsageError);
 }
@@ -238,11 +237,11 @@ TEST_F(PluginInfoAppTest, WrongNumberOfParams) {
 TEST_F(PluginInfoAppTest, NonExistingLibrary) {
   const char *plugin_name = "non_existing_plugin";
   std::string lib_path = get_plugin_file_path(plugin_name);
-  std::vector<std::string> args{lib_path, plugin_name};
+  std::vector<std::string> args{lib_path.c_str(), plugin_name};
   PluginInfoFrontend plugin_info_app(kPluginInfoAppExeFileName, args,
-                                     out_stream_);
+                                     out_stream_, out_stream_err_);
 
-  const std::string expected_error = "Could not load plugin file ";
+  const std::string expected_error = "Could not load plugin file: ";
   EXPECT_THROW_LIKE(plugin_info_app.run(), FrontendError, expected_error);
 
   // that nothing else is printed
@@ -250,18 +249,15 @@ TEST_F(PluginInfoAppTest, NonExistingLibrary) {
   EXPECT_EQ(out_stream_err_.str(), "");
 }
 
-/**
- * check if loading an existing library which doesn't export the plugin struct
- * works.
- *
- * DISABLED as currently no just library exists in the plugin directory.
- */
-TEST_F(PluginInfoAppTest, DISABLED_NonPluginExistingLibrary) {
+// we use mysql_protocol which is an existing library but it's not a
+// plugin so should not have Plugin struct exported/defined
+TEST_F(PluginInfoAppTest, NonPluginExistingLibrary) {
   const char *plugin_name = "mysql_protocol";
   std::string lib_path = get_plugin_file_path(plugin_name);
 
-  PluginInfoFrontend plugin_info_app(kPluginInfoAppExeFileName,
-                                     {lib_path, plugin_name}, out_stream_);
+  std::vector<std::string> args{lib_path.c_str(), plugin_name};
+  PluginInfoFrontend plugin_info_app(kPluginInfoAppExeFileName, args,
+                                     out_stream_, out_stream_err_);
 
   const std::string expected_error = "Loading plugin information for ";
   EXPECT_THROW_LIKE(plugin_info_app.run(), FrontendError, expected_error);
@@ -277,24 +273,23 @@ TEST_F(PluginInfoAppTest, DISABLED_NonPluginExistingLibrary) {
 //
 
 //                            <name,   brief,  version ,requires, conflicts>
-using Plugin_data =
-    std::tuple<std::string, std::string, std::string, std::string, std::string>;
+using Plugin_data = std::tuple<string, string, string, string, string>;
 
 class PluginInfoAppTestReadInfo : public PluginInfoAppTest,
                                   public WithParamInterface<Plugin_data> {};
 
 TEST_P(PluginInfoAppTestReadInfo, ReadInfo) {
-  const std::string plugin_name = std::get<0>(GetParam());
-  const std::string plugin_brief = std::get<1>(GetParam());
-  const std::string plugin_version = std::get<2>(GetParam());
-  const std::string plugin_requires = std::get<3>(GetParam());
-  const std::string plugin_conflicts = std::get<4>(GetParam());
-  const std::string plugin_file_path = get_plugin_file_path(plugin_name);
+  const string plugin_name = std::get<0>(GetParam());
+  const string plugin_brief = std::get<1>(GetParam());
+  const string plugin_version = std::get<2>(GetParam());
+  const string plugin_requires = std::get<3>(GetParam());
+  const string plugin_conflicts = std::get<4>(GetParam());
+  const string plugin_file_path = get_plugin_file_path(plugin_name);
 
   std::vector<std::string> args{plugin_file_path.c_str(), plugin_name.c_str()};
 
   PluginInfoFrontend plugin_info_app(kPluginInfoAppExeFileName, args,
-                                     out_stream_);
+                                     out_stream_, out_stream_err_);
 
   int res = plugin_info_app.run();
 
@@ -304,30 +299,19 @@ TEST_P(PluginInfoAppTestReadInfo, ReadInfo) {
 }
 
 const Plugin_data router_plugins[]{
-    {"routing",
-     "Routing MySQL connections between MySQL clients/connectors "
-     "and servers",
-     "0.0.1", R"(
-        "logger",
-        "router_protobuf",
-        "router_openssl",
-        "io",
-        "connection_pool",
-        "destination_status"
-    )",
-     ""},
-    {"metadata_cache",
-     "Metadata Cache, managing information fetched from the Metadata Server",
-     "0.0.1", R"(
-        "logger",
-        "router_protobuf"
-    )",
-     ""},
-    {"keepalive", "Keepalive Plugin", "0.0.1", "", ""},
+    Plugin_data{"routing",
+                "Routing MySQL connections between MySQL clients/connectors "
+                "and servers",
+                "0.0.1", "", ""},
+    Plugin_data{
+        "metadata_cache",
+        "Metadata Cache, managing information fetched from the Metadata Server",
+        "0.0.1", "", ""},
+    Plugin_data{"keepalive", "Keepalive Plugin", "0.0.1", "", ""},
 };
 
-INSTANTIATE_TEST_SUITE_P(CheckReadInfo, PluginInfoAppTestReadInfo,
-                         ValuesIn(router_plugins));
+INSTANTIATE_TEST_CASE_P(CheckReadInfo, PluginInfoAppTestReadInfo,
+                        ValuesIn(router_plugins));
 
 int main(int argc, char *argv[]) {
   g_origin_path = Path(argv[0]).dirname();
