@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -23,19 +23,21 @@
 */
 
 #include "mysqlrouter/mysql_session.h"
-#define MYSQL_ROUTER_LOG_DOMAIN "sql"
-#include "mysql/harness/logging/logging.h"
-
-#include <assert.h>  // <cassert> is flawed: assert() lands in global namespace on Ubuntu 14.04, not std::
-#include <ctype.h>  // not <cctype> because we don't want std::toupper(), which causes problems with std::transform()
-#include <mysql.h>
 #include <algorithm>
+#include <cassert>
+#include <cctype>
 #include <chrono>
 #include <cstdlib>
 #include <fstream>
 #include <iostream>
 #include <sstream>
 
+#include <mysql.h>
+
+#include "mysqlrouter/mysql_client_thread_token.h"
+#define MYSQL_ROUTER_LOG_DOMAIN "sql"
+#include "mysql/harness/logging/logging.h"
+
 IMPORT_LOG_FUNCTIONS()
 
 using namespace mysqlrouter;
@@ -398,6 +400,8 @@ static GoogleMockRecorder g_mock_recorder;
 #endif  // !MOCK_RECORDER
 
 MySQLSession::MySQLSession() {
+  MySQLClientThreadToken api_token;
+
   connection_ = new MYSQL();
   connected_ = false;
   if (!mysql_init(connection_)) {
@@ -624,6 +628,7 @@ void MySQLSession::disconnect() {
 
   // initialize the connection handle again as _close() is also free()ing
   // a lot of internal data.
+  MySQLClientThreadToken api_token;
   mysql_init(connection_);
   connected_ = false;
   connection_address_.clear();
@@ -711,9 +716,8 @@ void MySQLSession::query(const std::string &q, const RowProcessor &processor) {
 
 class RealResultRow : public MySQLSession::ResultRow {
  public:
-  RealResultRow(const MySQLSession::Row &row, MYSQL_RES *res) : res_(res) {
-    row_ = row;
-  }
+  RealResultRow(MySQLSession::Row row, MYSQL_RES *res)
+      : ResultRow(std::move(row)), res_(res) {}
 
   virtual ~RealResultRow() { mysql_free_result(res_); }
 
@@ -721,7 +725,8 @@ class RealResultRow : public MySQLSession::ResultRow {
   MYSQL_RES *res_;
 };
 
-MySQLSession::ResultRow *MySQLSession::query_one(const std::string &q) {
+std::unique_ptr<MySQLSession::ResultRow> MySQLSession::query_one(
+    const std::string &q) {
   if (connection_) {
     MOCK_REC_QUERY_ONE(q);
     if (mysql_real_query(connection_, q.data(), q.length()) != 0) {
@@ -751,7 +756,7 @@ MySQLSession::ResultRow *MySQLSession::query_one(const std::string &q) {
         mysql_free_result(res);
         return nullptr;
       }
-      return new RealResultRow(outrow, res);
+      return std::make_unique<RealResultRow>(outrow, res);
     } else {
       std::stringstream ss;
       ss << "Error fetching query results: ";
