@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,10 +22,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "mysqlrouter/utils_sqlstring.h"
-
-#include <cstring>
-#include <string>
+#include "utils_sqlstring.h"
+#include <string.h>
 
 #ifdef _WIN32
 #define strcasecmp _stricmp
@@ -265,7 +262,7 @@ static const char *reserved_keywords[] = {"ACCESSIBLE",
                                           "XOR",
                                           "YEAR_MONTH",
                                           "ZEROFILL",
-                                          nullptr};
+                                          NULL};
 
 namespace mysqlrouter {
 //--------------------------------------------------------------------------------------------------
@@ -361,7 +358,7 @@ std::string escape_backticks(const std::string &s) {
 //--------------------------------------------------------------------------------------------------
 
 bool is_reserved_word(const std::string &word) {
-  for (const char **kw = reserved_keywords; *kw != nullptr; ++kw) {
+  for (const char **kw = reserved_keywords; *kw != NULL; ++kw) {
     if (strcasecmp(word.c_str(), *kw) == 0) return true;
   }
   return false;
@@ -416,7 +413,10 @@ sqlstring::sqlstring(const char *format_string, const sqlstringformat format)
   append(consume_until_next_escape());
 }
 
-sqlstring::sqlstring(const sqlstring &) = default;
+sqlstring::sqlstring(const sqlstring &copy)
+    : _formatted(copy._formatted),
+      _format_string_left(copy._format_string_left),
+      _format(copy._format) {}
 
 sqlstring::sqlstring() : _format(0) {}
 
