@@ -6,18 +6,15 @@ var options = {
 
 var common_responses = common_stmts.prepare_statement_responses([
   "router_start_transaction",
-  "router_select_view_id_bootstrap_ar",
   "router_commit",
 ], options);
 
 var common_responses_regex = common_stmts.prepare_statement_responses_regex([
-  "router_create_user_if_not_exists",
-  "router_select_router_address",
+  "router_select_hosts_join_routers",
+  "router_delete_old_accounts",
   "router_create_user",
   "router_grant_on_metadata_db",
   "router_grant_on_pfs_db",
-  "router_grant_on_routers",
-  "router_grant_on_v2_routers",
   "router_update_routers_in_metadata",
   "router_drop_users",
 ], options);
@@ -31,7 +28,7 @@ var common_responses_regex = common_stmts.prepare_statement_responses_regex([
       return res;
     }
     else {
-      return common_stmts.unknown_statement_response("TODO:" + stmt);
+      return common_stmts.unknown_statement_response(stmt);
     }
   }
 })
