@@ -1,42 +1,35 @@
 var common_stmts = require("common_statements");
 
 var options = {
-  cluster_type: "gr",
-  innodb_cluster_name: "mycluster",
-  innodb_cluster_instances: [ ["localhost", 5500], ["localhost", 5510], ["localhost", 5520] ],
+  innodb_cluster_cluster_name: "mycluster",
+  innodb_cluster_insances: [ ["localhost", 5500], ["localhost", 5510], ["localhost", 5520] ],
   bootstrap_report_host_pattern: "host.foo.bar",
 };
 
 var common_responses = common_stmts.prepare_statement_responses([
   "router_select_schema_version",
-  "router_select_cluster_type_v2",
   "router_select_group_membership_with_primary_mode",
   "router_select_group_replication_primary_member",
-  "router_select_metadata_v2",
-  "router_count_clusters_v2",
+  "router_select_metadata",
+  "router_count_clusters_and_replicasets",
   "router_check_member_state",
   "router_select_members_count",
   "router_select_replication_group_name",
   "router_show_cipher_status",
-  "router_select_cluster_instances_v2",
-  "router_select_cluster_instance_addresses_v2",
+  "router_select_cluster_instances",
   "router_start_transaction",
   "router_commit",
 ], options);
 
 var common_responses_regex = common_stmts.prepare_statement_responses_regex([
+  "router_select_hosts",
+  "router_insert_into_hosts",
   "router_insert_into_routers",
-  "router_create_user_if_not_exists",
+  "router_delete_old_accounts",
+  "router_create_user",
   "router_grant_on_metadata_db",
   "router_grant_on_pfs_db",
-  "router_grant_on_routers",
-  "router_grant_on_v2_routers",
   "router_update_routers_in_metadata",
-
-  // to avoid creating yet another .js just for one test, below entry was added
-  // so this .js can be reused by test:
-  // RouterBootstrapTest.bootstrap_report_not_shown_until_bootstrap_succeeds
-  "router_drop_users",
 ], options);
 
 ({
