@@ -29,6 +29,8 @@
 #include "filesystem_utils.h"
 #include "gmock/gmock.h"
 #include "keyring/keyring_manager.h"
+#include "mock_server_rest_client.h"
+#include "mock_server_testutils.h"
 #include "random_generator.h"
 #include "router_component_test.h"
 #include "script_generator.h"
@@ -45,6 +47,8 @@
  * @brief Component Tests for the bootstrap operation
  */
 
+using namespace std::chrono_literals;
+
 // we create a number of classes to logically group tests together. But to avoid
 // code duplication, we derive them from a class which contains the common code
 // they need.
@@ -60,8 +64,8 @@ class CommonBootstrapTest : public RouterComponentTest {
   struct Config {
     std::string ip;
     unsigned int port;
-    std::string in_filename;
-    std::string out_filename;
+    uint16_t http_port;
+    std::string js_filename;
   };
 
   void bootstrap_failover(
@@ -69,7 +73,7 @@ class CommonBootstrapTest : public RouterComponentTest {
       const std::vector<std::string> &router_options = {},
       int expected_exitcode = 0,
       const std::vector<std::string> &expected_output_regex = {},
-      unsigned wait_for_exit_timeout_ms = 10000);
+      std::chrono::milliseconds wait_for_exit_timeout = 10000ms);
 
   friend std::ostream &operator<<(
       std::ostream &os,
@@ -103,51 +107,33 @@ void CommonBootstrapTest::bootstrap_failover(
     const std::vector<Config> &mock_server_configs,
     const std::vector<std::string> &router_options, int expected_exitcode,
     const std::vector<std::string> &expected_output_regex,
-    unsigned wait_for_exit_timeout_ms) {
+    std::chrono::milliseconds wait_for_exit_timeout) {
   std::string cluster_name("mycluster");
 
-  // build environment
-  std::map<std::string, std::string> env_vars = {
-      {"MYSQL_SERVER_MOCK_CLUSTER_NAME", cluster_name},
-      {"MYSQL_SERVER_MOCK_HOST_NAME", my_hostname},
-  };
-
-  unsigned int ndx = 1;
-
+  std::vector<std::pair<std::string, unsigned>> gr_members;
   for (const auto &mock_server_config : mock_server_configs) {
-    env_vars.emplace("MYSQL_SERVER_MOCK_HOST_" + std::to_string(ndx),
-                     mock_server_config.ip);
-    env_vars.emplace("MYSQL_SERVER_MOCK_PORT_" + std::to_string(ndx),
-                     std::to_string(mock_server_config.port));
-    ndx++;
+    gr_members.emplace_back(mock_server_config.ip, mock_server_config.port);
   }
 
   std::vector<std::tuple<ProcessWrapper &, unsigned int>> mock_servers;
 
   // start the mocks
   for (const auto &mock_server_config : mock_server_configs) {
-    unsigned int port = mock_server_config.port;
-    const std::string &in_filename = mock_server_config.in_filename;
-    const std::string &out_filename = mock_server_config.out_filename;
+    if (mock_server_config.js_filename.empty()) continue;
 
-    if (in_filename.size()) {
-      rewrite_js_to_tracefile(in_filename, out_filename, env_vars);
-    }
+    const auto port = mock_server_config.port;
+    const auto http_port = mock_server_config.http_port;
+    mock_servers.emplace_back(
+        launch_mysql_server_mock(mock_server_config.js_filename, port,
+                                 EXIT_SUCCESS, false, http_port),
+        port);
 
-    if (out_filename.size()) {
-      mock_servers.emplace_back(
-          launch_mysql_server_mock(out_filename, port, EXIT_SUCCESS, false),
-          port);
-    }
-  }
+    ProcessWrapper &mock_server = std::get<0>(mock_servers.back());
+    ASSERT_NO_FATAL_FAILURE(
+        check_port_ready(mock_server, static_cast<uint16_t>(port)));
 
-  // wait for all mocks to be up
-  for (auto &mock_server : mock_servers) {
-    auto &proc = std::get<0>(mock_server);
-    unsigned int port = std::get<1>(mock_server);
-
-    bool ready = wait_for_port_ready(port);
-    EXPECT_TRUE(ready) << proc.get_full_output();
+    EXPECT_TRUE(MockServerRestClient(http_port).wait_for_rest_endpoint_ready());
+    set_mock_bootstrap_data(http_port, cluster_name, gr_members);
   }
 
   std::vector<std::string> router_cmdline;
@@ -155,9 +141,8 @@ void CommonBootstrapTest::bootstrap_failover(
   if (router_options.size()) {
     router_cmdline = router_options;
   } else {
-    router_cmdline.emplace_back(
-        "--bootstrap=" + env_vars.at("MYSQL_SERVER_MOCK_HOST_1") + ":" +
-        env_vars.at("MYSQL_SERVER_MOCK_PORT_1"));
+    router_cmdline.emplace_back("--bootstrap=" + gr_members[0].first + ":" +
+                                std::to_string(gr_members[0].second));
 
     router_cmdline.emplace_back("--report-host");
     router_cmdline.emplace_back(my_hostname);
@@ -172,10 +157,9 @@ void CommonBootstrapTest::bootstrap_failover(
   router.register_response("Please enter MySQL password for root: ",
                            "fake-pass\n");
 
-  // wait_for_exit() throws at timeout.
-  EXPECT_NO_THROW(EXPECT_EQ(router.wait_for_exit(wait_for_exit_timeout_ms),
-                            expected_exitcode)
-                  << router.get_full_output());
+  ASSERT_NO_FATAL_FAILURE(
+      check_exit_code(router, expected_exitcode, wait_for_exit_timeout))
+      << std::get<0>(mock_servers[0]).get_full_output();
 
   // split the output into lines
   std::vector<std::string> lines;
@@ -245,7 +229,8 @@ class RouterBootstrapTest : public CommonBootstrapTest {};
  */
 TEST_F(RouterBootstrapTest, BootstrapOk) {
   std::vector<Config> config{
-      {"127.0.0.1", port_pool_.get_next_available(), "",
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir().join("bootstrap.js").str()},
   };
 
@@ -274,7 +259,8 @@ TEST_F(RouterBootstrapTest, BootstrapUserIsCurrentUser) {
     const char *current_username = current_userpw->pw_name;
 
     std::vector<Config> mock_servers{
-        {"127.0.0.1", port_pool_.get_next_available(), "",
+        {"127.0.0.1", port_pool_.get_next_available(),
+         port_pool_.get_next_available(),
          get_data_dir().join("bootstrap.js").str()},
     };
 
@@ -306,7 +292,8 @@ TEST_F(RouterBootstrapTest, BootstrapUserIsCurrentUser) {
  */
 TEST_F(RouterBootstrapTest, BootstrapOnlySockets) {
   std::vector<Config> mock_servers{
-      {"127.0.0.1", port_pool_.get_next_available(), "",
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir().join("bootstrap.js").str()},
   };
 
@@ -345,7 +332,8 @@ TEST_F(RouterBootstrapTest, BootstrapOnlySockets) {
  */
 TEST_F(RouterBootstrapTest, BootstrapUnsupportedSchemaVersion) {
   std::vector<Config> mock_servers{
-      {"127.0.0.1", port_pool_.get_next_available(), "",
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir().join("bootstrap_unsupported_schema_version.js").str()},
   };
 
@@ -368,13 +356,13 @@ TEST_F(RouterBootstrapTest, BootstrapUnsupportedSchemaVersion) {
 TEST_F(RouterBootstrapTest, BootstrapFailoverSuperReadonly) {
   std::vector<Config> config{
       {"127.0.0.1", port_pool_.get_next_available(),
-       get_data_dir().join("bootstrap_failover_super_read_only_1.js").str(),
-       Path(tmp_dir.name())
-           .join("bootstrap_failover_super_read_only_1.json")
-           .str()},
-      {"127.0.0.1", port_pool_.get_next_available(), "",
+       port_pool_.get_next_available(),
+       get_data_dir().join("bootstrap_failover_super_read_only_1.js").str()},
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir().join("bootstrap_failover_super_read_only_2.js").str()},
-      {"127.0.0.1", port_pool_.get_next_available(), "", ""},
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(), ""},
   };
 
   bootstrap_failover(config);
@@ -406,14 +394,15 @@ TEST_F(RouterBootstrapTest, BootstrapFailoverSuperReadonly2ndNodeDead) {
   std::vector<Config> config{
       // member-1, PRIMARY, fails at first write
       {"127.0.0.1", port_pool_.get_next_available(),
-       get_data_dir().join("bootstrap_failover_super_read_only_1.js").str(),
-       Path(tmp_dir.name()).join("member-1.json").str()},
+       port_pool_.get_next_available(),
+       get_data_dir().join("bootstrap_failover_super_read_only_1.js").str()},
       // member-2, unreachable
       {"127.0.0.1", 65536,  // 65536 % 0xffff = 0 (port 0), but we bypass
                             // libmysqlclient's default-port assignment
-       "", ""},
+       port_pool_.get_next_available(), ""},
       // member-3, succeeds
-      {"127.0.0.1", port_pool_.get_next_available(), "",
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir().join("bootstrap_failover_super_read_only_2.js").str()},
   };
 
@@ -439,18 +428,19 @@ TEST_F(RouterBootstrapTest, BootstrapFailoverSuperReadonlyCreateAccountFails) {
   std::vector<Config> config{
       // member-1: SECONDARY, fails at DROP USER due to RW request on RO node
       {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir()
            .join("bootstrap_failover_super_read_only_dead_2nd_1.js")
-           .str(),
-       Path(tmp_dir.name()).join("member-1.json").str()},
+           .str()},
 
       // member-2: PRIMARY, succeeds
       {"127.0.0.1", port_pool_.get_next_available(),
-       get_data_dir().join("bootstrap_failover_reconfigure_ok.js").str(),
-       Path(tmp_dir.name()).join("member-2.json").str()},
+       port_pool_.get_next_available(),
+       get_data_dir().join("bootstrap_failover_reconfigure_ok.js").str()},
 
       // member-3: defined, but unused
-      {"127.0.0.1", port_pool_.get_next_available(), "", ""},
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(), ""},
   };
 
   bootstrap_failover(config);
@@ -472,20 +462,21 @@ TEST_F(RouterBootstrapTest,
   std::vector<Config> config{
       // member-1: SECONDARY, fails on CREATE USER due to RW request on RO node
       {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir()
            .join("bootstrap_failover_super_read_only_delete_user.js")
-           .str(),
-       Path(tmp_dir.name()).join("member-1.json").str()},
+           .str()},
 
       // member-2: PRIMARY, succeeds
       {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir()
            .join("bootstrap_failover_reconfigure_ok_3_old_users.js")
-           .str(),
-       Path(tmp_dir.name()).join("member-2.json").str()},
+           .str()},
 
       // member-3: defined, but unused
-      {"127.0.0.1", port_pool_.get_next_available(), "", ""},
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(), ""},
   };
 
   bootstrap_failover(config);
@@ -507,16 +498,17 @@ TEST_F(RouterBootstrapTest,
   std::vector<Config> config{
       // member-1: PRIMARY, fails after GRANT
       {"127.0.0.1", port_pool_.get_next_available(),
-       get_data_dir().join("bootstrap_failover_at_grant.js").str(),
-       Path(tmp_dir.name()).join("member-1.json").str()},
+       port_pool_.get_next_available(),
+       get_data_dir().join("bootstrap_failover_at_grant.js").str()},
 
       // member-2: PRIMARY, succeeds
       {"127.0.0.1", port_pool_.get_next_available(),
-       get_data_dir().join("bootstrap_failover_reconfigure_ok.js").str(),
-       Path(tmp_dir.name()).join("member-2.json").str()},
+       port_pool_.get_next_available(),
+       get_data_dir().join("bootstrap_failover_reconfigure_ok.js").str()},
 
       // member-3: defined, but unused
-      {"127.0.0.1", port_pool_.get_next_available(), "", ""},
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(), ""},
   };
 
   bootstrap_failover(config);
@@ -540,13 +532,13 @@ TEST_F(RouterBootstrapTest,
 TEST_F(RouterBootstrapTest, DISABLED_BootstrapFailoverSuperReadonlyFromSocket) {
   std::vector<Config> mock_servers{
       {"127.0.0.1", port_pool_.get_next_available(),
-       get_data_dir().join("bootstrap_failover_super_read_only_1.js").str(),
-       Path(tmp_dir.name())
-           .join("bootstrap_failover_super_read_only_1.json")
-           .str()},
-      {"127.0.0.1", port_pool_.get_next_available(), "",
+       port_pool_.get_next_available(),
+       get_data_dir().join("bootstrap_failover_super_read_only_1.js").str()},
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir().join("bootstrap_failover_super_read_only_2.js").str()},
-      {"127.0.0.1", port_pool_.get_next_available(), "", ""},
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(), ""},
   };
 
   std::vector<std::string> router_options = {
@@ -570,20 +562,20 @@ TEST_F(RouterBootstrapTest, BootstrapFailoverSuperReadonlyNewPrimaryCrash) {
   std::vector<Config> mock_servers{
       // member-1: PRIMARY, fails at DROP USER
       {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir()
            .join("bootstrap_failover_super_read_only_dead_2nd_1.js")
-           .str(),
-       Path(tmp_dir.name()).join("member-1.json").str()},
+           .str()},
 
       // member-2: PRIMARY, but crashing
       {"127.0.0.1", port_pool_.get_next_available(),
-       get_data_dir().join("bootstrap_failover_at_crash.js").str(),
-       Path(tmp_dir.name()).join("member-2.json").str()},
+       port_pool_.get_next_available(),
+       get_data_dir().join("bootstrap_failover_at_crash.js").str()},
 
       // member-3: newly elected PRIMARY, succeeds
       {"127.0.0.1", port_pool_.get_next_available(),
-       get_data_dir().join("bootstrap_failover_reconfigure_ok.js").str(),
-       Path(tmp_dir.name()).join("member-3.json").str()},
+       port_pool_.get_next_available(),
+       get_data_dir().join("bootstrap_failover_reconfigure_ok.js").str()},
   };
 
   bootstrap_failover(mock_servers);
@@ -596,7 +588,8 @@ TEST_F(RouterBootstrapTest, BootstrapFailoverSuperReadonlyNewPrimaryCrash) {
 TEST_F(RouterBootstrapTest,
        BootstrapSucceedWhenServerResponseLessThanReadTimeout) {
   std::vector<Config> mock_servers{
-      {"127.0.0.1", port_pool_.get_next_available(), "",
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir().join("bootstrap_exec_time_2_seconds.js").str()},
   };
 
@@ -617,14 +610,16 @@ TEST_F(RouterBootstrapTest, BootstrapAccessErrorAtGrantStatement) {
   std::vector<Config> config{
       // member-1: PRIMARY, fails after GRANT
       {"127.0.0.1", port_pool_.get_next_available(),
-       get_data_dir().join("bootstrap_access_error_at_grant.js").str(),
-       Path(tmp_dir.name()).join("member-1.json").str()},
+       port_pool_.get_next_available(),
+       get_data_dir().join("bootstrap_access_error_at_grant.js").str()},
 
       // member-2: defined, but unused
-      {"127.0.0.1", port_pool_.get_next_available(), "", ""},
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(), ""},
 
       // member-3: defined, but unused
-      {"127.0.0.1", port_pool_.get_next_available(), "", ""},
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(), ""},
   };
 
   bootstrap_failover(config, {}, EXIT_FAILURE,
@@ -643,7 +638,7 @@ TEST_F(RouterBootstrapTest, BootstrapNoGroupReplicationSetup) {
       {
           "127.0.0.1",
           port_pool_.get_next_available(),
-          "",
+          port_pool_.get_next_available(),
           get_data_dir().join("bootstrap_no_gr.js").str(),
       },
   };
@@ -662,7 +657,7 @@ TEST_F(RouterBootstrapTest, BootstrapNoMetadataSchema) {
       {
           "127.0.0.1",
           port_pool_.get_next_available(),
-          "",
+          port_pool_.get_next_available(),
           get_data_dir().join("bootstrap_no_schema.js").str(),
       },
   };
@@ -677,7 +672,8 @@ TEST_F(RouterBootstrapTest, BootstrapNoMetadataSchema) {
  */
 TEST_F(RouterBootstrapTest, BootstrapFailWhenServerResponseExceedsReadTimeout) {
   std::vector<Config> mock_servers{
-      {"127.0.0.1", port_pool_.get_next_available(), "",
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir().join("bootstrap_exec_time_2_seconds.js").str()},
   };
 
@@ -704,7 +700,7 @@ TEST_F(RouterAccountHostTest, multiple_host_patterns) {
   // to avoid duplication of tracefiles, we run the same test twice, with the
   // only difference that 1st time we run --bootstrap before the --account-host,
   // and second time we run it after
-  const unsigned server_port = port_pool_.get_next_available();
+  const auto server_port = port_pool_.get_next_available();
 
   auto test_it = [&](const std::vector<std::string> &cmdline) -> void {
     const std::string json_stmts =
@@ -715,8 +711,7 @@ TEST_F(RouterAccountHostTest, multiple_host_patterns) {
     // launch mock server and wait for it to start accepting connections
     auto &server_mock =
         launch_mysql_server_mock(json_stmts, server_port, EXIT_SUCCESS, false);
-    bool ready = wait_for_port_ready(server_port);
-    EXPECT_TRUE(ready) << server_mock.get_full_output();
+    ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
     // launch the router in bootstrap mode
     auto &router = launch_router(cmdline);
@@ -727,10 +722,12 @@ TEST_F(RouterAccountHostTest, multiple_host_patterns) {
 
     // check if the bootstraping was successful
     EXPECT_TRUE(router.expect_output(
-        "MySQL Router configured for the InnoDB cluster 'test'"))
-        << router.get_full_output() << std::endl
+        "MySQL Router configured for the InnoDB cluster 'mycluster'", false,
+        5s))
+        << "router: " << router.get_full_output() << std::endl
         << "server: " << server_mock.get_full_output();
-    EXPECT_EQ(router.wait_for_exit(), EXIT_SUCCESS);
+
+    check_exit_code(router, EXIT_SUCCESS);
 
     server_mock.kill();
   };
@@ -781,7 +778,7 @@ TEST_F(RouterAccountHostTest, argument_missing) {
   EXPECT_TRUE(router.expect_output(
       "option '--account-host' expects a value, got nothing"))
       << router.get_full_output() << std::endl;
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 /**
@@ -797,7 +794,7 @@ TEST_F(RouterAccountHostTest, without_bootstrap_flag) {
   EXPECT_TRUE(router.expect_output(
       "Option --account-host can only be used together with -B/--bootstrap"))
       << router.get_full_output() << std::endl;
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 /**
@@ -809,13 +806,12 @@ TEST_F(RouterAccountHostTest, illegal_hostname) {
   const std::string json_stmts =
       get_data_dir().join("bootstrap_account_host_pattern_too_long.js").str();
   TempDirectory bootstrap_directory;
-  const unsigned server_port = port_pool_.get_next_available();
+  const auto server_port = port_pool_.get_next_available();
 
   // launch mock server and wait for it to start accepting connections
   auto &server_mock =
       launch_mysql_server_mock(json_stmts, server_port, EXIT_SUCCESS, false);
-  bool ready = wait_for_port_ready(server_port);
-  EXPECT_TRUE(ready) << server_mock.get_full_output();
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
   // launch the router in bootstrap mode
   auto &router = launch_router(
@@ -835,7 +831,7 @@ TEST_F(RouterAccountHostTest, illegal_hostname) {
       << router.get_full_output() << std::endl
       << "server:\n"
       << server_mock.get_full_output();
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 class RouterReportHostTest : public CommonBootstrapTest {};
@@ -845,7 +841,7 @@ class RouterReportHostTest : public CommonBootstrapTest {};
  *        verify that --report-host works for the typical use case
  */
 TEST_F(RouterReportHostTest, typical_usage) {
-  const unsigned server_port = port_pool_.get_next_available();
+  const auto server_port = port_pool_.get_next_available();
 
   auto test_it = [&](const std::vector<std::string> &cmdline) -> void {
     const std::string json_stmts =
@@ -854,8 +850,7 @@ TEST_F(RouterReportHostTest, typical_usage) {
     // launch mock server and wait for it to start accepting connections
     auto &server_mock =
         launch_mysql_server_mock(json_stmts, server_port, EXIT_SUCCESS, false);
-    bool ready = wait_for_port_ready(server_port);
-    EXPECT_TRUE(ready) << server_mock.get_full_output();
+    ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
     // launch the router in bootstrap mode
     auto &router = launch_router(cmdline);
@@ -870,7 +865,7 @@ TEST_F(RouterReportHostTest, typical_usage) {
                              "InnoDB cluster 'mycluster'"))
         << router.get_full_output() << std::endl
         << "server: " << server_mock.get_full_output();
-    EXPECT_EQ(router.wait_for_exit(), EXIT_SUCCESS);
+    check_exit_code(router, EXIT_SUCCESS);
 
     server_mock.kill();
   };
@@ -905,7 +900,7 @@ TEST_F(RouterReportHostTest, multiple_hostnames) {
   EXPECT_TRUE(
       router.expect_output("Option --report-host can only be used once."))
       << router.get_full_output() << std::endl;
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 /**
@@ -922,7 +917,7 @@ TEST_F(RouterReportHostTest, argument_missing) {
   EXPECT_TRUE(router.expect_output(
       "option '--report-host' expects a value, got nothing"))
       << router.get_full_output() << std::endl;
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 /**
@@ -938,7 +933,7 @@ TEST_F(RouterReportHostTest, without_bootstrap_flag) {
   EXPECT_TRUE(router.expect_output(
       "Option --report-host can only be used together with -B/--bootstrap"))
       << router.get_full_output() << std::endl;
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 /**
@@ -961,7 +956,7 @@ TEST_F(RouterReportHostTest, invalid_hostname) {
   EXPECT_TRUE(
       router.expect_output("Error: Option --report-host has an invalid value."))
       << router.get_full_output() << std::endl;
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 /**
@@ -972,7 +967,8 @@ TEST_F(RouterReportHostTest, invalid_hostname) {
 TEST_F(RouterBootstrapTest,
        NoMasterKeyFileWhenBootstrapPassWithMasterKeyReader) {
   std::vector<Config> config{
-      {"127.0.0.1", port_pool_.get_next_available(), "",
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir().join("bootstrap.js").str()},
   };
 
@@ -1008,23 +1004,22 @@ TEST_F(RouterBootstrapTest,
  *       verify that master key file is not overridden by sunsequent bootstrap.
  */
 TEST_F(RouterBootstrapTest, MasterKeyFileNotChangedAfterSecondBootstrap) {
-  mysql_harness::DIM &dim = mysql_harness::DIM::instance();
-  // RandomGenerator
-  dim.set_RandomGenerator(
-      []() {
-        static mysql_harness::RandomGenerator rg;
-        return &rg;
-      },
-      [](mysql_harness::RandomGeneratorInterface *) {});
-
-  mysql_harness::mkdir(Path(bootstrap_dir.name()).str(), 0777);
   std::string master_key_path =
       Path(bootstrap_dir.name()).join("master_key").str();
-  mysql_harness::mkdir(Path(bootstrap_dir.name()).join("data").str(), 0777);
   std::string keyring_path =
       Path(bootstrap_dir.name()).join("data").join("keyring").str();
 
-  mysql_harness::init_keyring(keyring_path, master_key_path, true);
+  mysql_harness::mkdir(Path(bootstrap_dir.name()).str(), 0777);
+  mysql_harness::mkdir(Path(bootstrap_dir.name()).join("data").str(), 0777);
+
+  auto &proc = launch_command(get_origin().join("mysqlrouter_keyring").str(),
+                              {
+                                  "init",
+                                  keyring_path,
+                                  "--master-key-file",
+                                  master_key_path,
+                              });
+  ASSERT_NO_THROW(proc.wait_for_exit());
 
   std::string master_key;
   {
@@ -1035,7 +1030,8 @@ TEST_F(RouterBootstrapTest, MasterKeyFileNotChangedAfterSecondBootstrap) {
   }
 
   std::vector<Config> mock_servers{
-      {"127.0.0.1", port_pool_.get_next_available(), "",
+      {"127.0.0.1", port_pool_.get_next_available(),
+       port_pool_.get_next_available(),
        get_data_dir().join("bootstrap.js").str()},
   };
 
@@ -1064,14 +1060,13 @@ TEST_F(RouterBootstrapTest, MasterKeyFileNotChangedAfterSecondBootstrap) {
  */
 TEST_F(RouterBootstrapTest, ConfUseGrNotificationsYes) {
   TempDirectory bootstrap_directory;
-  const unsigned server_port = port_pool_.get_next_available();
+  const auto server_port = port_pool_.get_next_available();
   const std::string json_stmts = get_data_dir().join("bootstrap.js").str();
 
   // launch mock server and wait for it to start accepting connections
   auto &server_mock =
       launch_mysql_server_mock(json_stmts, server_port, EXIT_SUCCESS, false);
-  bool ready = wait_for_port_ready(server_port);
-  EXPECT_TRUE(ready) << server_mock.get_full_output();
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
   // launch the router in bootstrap mode
   auto &router = launch_router(
@@ -1082,8 +1077,7 @@ TEST_F(RouterBootstrapTest, ConfUseGrNotificationsYes) {
   router.register_response("Please enter MySQL password for root: ",
                            "fake-pass\n");
 
-  EXPECT_EQ(router.wait_for_exit(), EXIT_SUCCESS)
-      << "output:" << router.get_full_output();
+  check_exit_code(router, EXIT_SUCCESS);
 
   // check if the valid config option was added to the file
   EXPECT_TRUE(find_in_file(
@@ -1108,15 +1102,14 @@ TEST_F(RouterBootstrapTest, ConfUseGrNotificationsYes) {
  */
 TEST_F(RouterBootstrapTest, ConfUseGrNotificationsNo) {
   TempDirectory bootstrap_directory;
-  const unsigned server_port = port_pool_.get_next_available();
+  const auto server_port = port_pool_.get_next_available();
 
   const std::string json_stmts = get_data_dir().join("bootstrap.js").str();
 
   // launch mock server and wait for it to start accepting connections
   auto &server_mock =
       launch_mysql_server_mock(json_stmts, server_port, EXIT_SUCCESS, false);
-  bool ready = wait_for_port_ready(server_port);
-  EXPECT_TRUE(ready) << server_mock.get_full_output();
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port));
 
   // launch the router in bootstrap mode
   auto &router =
@@ -1127,8 +1120,7 @@ TEST_F(RouterBootstrapTest, ConfUseGrNotificationsNo) {
   router.register_response("Please enter MySQL password for root: ",
                            "fake-pass\n");
 
-  EXPECT_EQ(router.wait_for_exit(), EXIT_SUCCESS)
-      << "output:" << router.get_full_output();
+  check_exit_code(router, EXIT_SUCCESS);
 
   // check if valid config option was added to the file
   EXPECT_TRUE(find_in_file(
@@ -1157,7 +1149,7 @@ TEST_F(RouterReportHostTest, ConfUseGrNotificationsNoBootstrap) {
       router.expect_output("Error: Option --conf-use-gr-notifications can only "
                            "be used together with -B/--bootstrap"))
       << router.get_full_output() << std::endl;
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 /**
@@ -1173,7 +1165,7 @@ TEST_F(RouterReportHostTest, ConfUseGrNotificationsHasValue) {
       router.expect_output("Error: option '--conf-use-gr-notifications' does "
                            "not expect a value, but got a value"))
       << router.get_full_output() << std::endl;
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 class ErrorReportTest : public CommonBootstrapTest {};
@@ -1221,7 +1213,7 @@ TEST_F(ErrorReportTest, bootstrap_dir_exists_and_is_not_empty) {
                         "' already contains files\n"
                         "Error: Directory already exits";
 
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 // unfortunately it's not (reasonably) possible to make folders read-only on
@@ -1274,7 +1266,7 @@ TEST_F(ErrorReportTest, bootstrap_dir_exists_but_is_inaccessible) {
       "This may be caused by insufficient rights or AppArmor settings.\n.*"
       "Error: Could not check contents of existing deployment directory";
 
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 
 /**
@@ -1327,7 +1319,7 @@ TEST_F(ErrorReportTest,
       "This may be caused by insufficient rights or AppArmor settings.\n.*"
       "Error: Could not create deployment directory";
 
-  EXPECT_EQ(router.wait_for_exit(), EXIT_FAILURE);
+  check_exit_code(router, EXIT_FAILURE);
 }
 #endif
 
