@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -22,8 +22,11 @@
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
+#include <array>
+
+#include <gmock/gmock.h>
+
 #include "common.h"
-#include "gmock/gmock.h"
 #include "router_component_system_layout.h"
 #include "router_component_test.h"
 #include "tcp_port_pool.h"
@@ -43,28 +46,29 @@ Path g_origin_path;
 #ifndef SKIP_BOOTSTRAP_SYSTEM_DEPLOYMENT_TESTS
 
 class RouterBootstrapSystemDeploymentTest : public RouterComponentTest,
-                                            public RouterSystemLayout,
-                                            public ::testing::Test {
+                                            public RouterSystemLayout {
  protected:
   void SetUp() override {
-    set_origin(g_origin_path);
-    RouterComponentTest::init();
-    init_system_layout_dir(get_mysqlrouter_exec(), g_origin_path);
+    RouterComponentTest::SetUp();
+    // this test modifies the origin path so we need to restore it
+    ProcessManager::set_origin(g_origin_path);
+    init_system_layout_dir(get_mysqlrouter_exec(),
+                           ProcessManager::get_origin());
 
     set_mysqlrouter_exec(Path(exec_file_));
   }
 
-  void TearDown() override { cleanup_system_layout(); }
+  void TearDown() override {
+    RouterComponentTest::TearDown();
+    cleanup_system_layout();
+  }
 
-  RouterComponentTest::CommandHandle run_server_mock() {
+  auto &run_server_mock() {
     const std::string json_stmts = get_data_dir().join("bootstrap.js").str();
     server_port_ = port_pool_.get_next_available();
 
     // launch mock server and wait for it to start accepting connections
-    auto server_mock = launch_mysql_server_mock(json_stmts, server_port_);
-    EXPECT_TRUE(wait_for_port_ready(server_port_, 1000))
-        << "Timed out waiting for mock server port ready\n"
-        << server_mock.get_full_output();
+    auto &server_mock = launch_mysql_server_mock(json_stmts, server_port_);
     return server_mock;
   }
 
@@ -78,20 +82,23 @@ class RouterBootstrapSystemDeploymentTest : public RouterComponentTest,
  * have access (see install_layout.cmake).
  */
 TEST_F(RouterBootstrapSystemDeploymentTest, BootstrapPass) {
-  auto server_mock = run_server_mock();
+  auto &server_mock = run_server_mock();
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port_));
 
   // launch the router in bootstrap mode
-  auto router =
-      launch_router("--bootstrap=127.0.0.1:" + std::to_string(server_port_) +
-                    " --report-host dont.query.dns");
+  auto &router = launch_router({
+      "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
+      "--connect-timeout=1",
+      "--report-host",
+      "dont.query.dns",
+  });
 
   // add login hook
   router.register_response("Please enter MySQL password for root: ",
                            "fake-pass\n");
 
   // check if the bootstraping was successful
-  EXPECT_NO_THROW(EXPECT_EQ(router.wait_for_exit(), 0))
-      << router.get_full_output();
+  check_exit_code(router, EXIT_SUCCESS);
 
   EXPECT_TRUE(
       router.expect_output("MySQL Router configured for the "
@@ -112,19 +119,24 @@ TEST_F(RouterBootstrapSystemDeploymentTest,
    * bootstrap to fail.
    */
   mysql_harness::mkdir(config_file_, 0700);
-  auto server_mock = run_server_mock();
+  auto &server_mock = run_server_mock();
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port_));
 
   // launch the router in bootstrap mode
-  auto router =
-      launch_router("--bootstrap=127.0.0.1:" + std::to_string(server_port_) +
-                    " --report-host dont.query.dns");
+  auto &router = launch_router(
+      {
+          "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
+          "--connect-timeout=1",
+          "--report-host",
+          "dont.query.dns",
+      },
+      EXIT_FAILURE);
 
   // add login hook
   router.register_response("Please enter MySQL password for root: ",
                            "fake-pass\n");
 
-  EXPECT_NO_THROW(EXPECT_EQ(router.wait_for_exit(), 1))
-      << router.get_full_output();
+  check_exit_code(router, EXIT_FAILURE);
 
   EXPECT_TRUE(router.expect_output(
       "Error: Could not save configuration file to final location", false))
@@ -148,19 +160,24 @@ TEST_F(RouterBootstrapSystemDeploymentTest,
    * bootstrap to fail.
    */
   mysql_harness::mkdir(config_file_, 0700);
-  auto server_mock = run_server_mock();
+  auto &server_mock = run_server_mock();
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port_));
 
   // launch the router in bootstrap mode
-  auto router =
-      launch_router("--bootstrap=127.0.0.1:" + std::to_string(server_port_) +
-                    " --report-host dont.query.dns");
+  auto &router = launch_router(
+      {
+          "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
+          "--connect-timeout=1",
+          "--report-host",
+          "dont.query.dns",
+      },
+      EXIT_FAILURE);
 
   // add login hook
   router.register_response("Please enter MySQL password for root: ",
                            "fake-pass\n");
 
-  EXPECT_NO_THROW(EXPECT_EQ(router.wait_for_exit(), 1))
-      << router.get_full_output();
+  check_exit_code(router, EXIT_FAILURE);
 
   EXPECT_TRUE(router.expect_output(
       "Error: Could not save configuration file to final location", false))
@@ -178,7 +195,8 @@ TEST_F(RouterBootstrapSystemDeploymentTest,
  */
 TEST_F(RouterBootstrapSystemDeploymentTest,
        IsKeyringRevertedWhenBootstrapFail) {
-  static const char kMasterKeyFileSignature[] = "MRKF";
+  const std::array<char, 5> kMasterKeyFileSignature = {'M', 'R', 'K', 'F',
+                                                       '\0'};
 
   {
     std::ofstream keyring_file(
@@ -186,8 +204,8 @@ TEST_F(RouterBootstrapSystemDeploymentTest,
         std::ios_base::binary | std::ios_base::trunc | std::ios_base::out);
 
     mysql_harness::make_file_private(tmp_dir_ + "/stage/mysqlrouter.key");
-    keyring_file.write(kMasterKeyFileSignature,
-                       strlen(kMasterKeyFileSignature));
+    keyring_file.write(kMasterKeyFileSignature.data(),
+                       kMasterKeyFileSignature.size());
   }
 
   /*
@@ -195,19 +213,24 @@ TEST_F(RouterBootstrapSystemDeploymentTest,
    * bootstrap to fail.
    */
   mysql_harness::mkdir(config_file_, 0700);
-  auto server_mock = run_server_mock();
+  auto &server_mock = run_server_mock();
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port_));
 
   // launch the router in bootstrap mode
-  auto router =
-      launch_router("--bootstrap=127.0.0.1:" + std::to_string(server_port_) +
-                    " --report-host dont.query.dns");
+  auto &router = launch_router(
+      {
+          "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
+          "--connect-timeout=1",
+          "--report-host",
+          "dont.query.dns",
+      },
+      EXIT_FAILURE);
 
   // add login hook
   router.register_response("Please enter MySQL password for root: ",
                            "fake-pass\n");
 
-  EXPECT_NO_THROW(EXPECT_EQ(router.wait_for_exit(), 1))
-      << router.get_full_output();
+  check_exit_code(router, EXIT_FAILURE);
 
   EXPECT_TRUE(router.expect_output(
       "Error: Could not save configuration file to final location", false))
@@ -220,10 +243,10 @@ TEST_F(RouterBootstrapSystemDeploymentTest,
   std::ifstream keyring_file(tmp_dir_ + "/stage/mysqlrouter.key",
                              std::ios_base::binary | std::ios_base::in);
 
-  char buf[10] = {0};
-  keyring_file.read(buf, sizeof(buf));
-  EXPECT_THAT(keyring_file.gcount(), 4);
-  EXPECT_THAT(std::strncmp(buf, kMasterKeyFileSignature, 4), testing::Eq(0));
+  std::array<char, 5> buf;
+  keyring_file.read(buf.data(), buf.size());
+  EXPECT_THAT(keyring_file.gcount(), 5);
+  EXPECT_EQ(buf, kMasterKeyFileSignature);
 }
 
 /*
@@ -238,19 +261,24 @@ TEST_F(RouterBootstrapSystemDeploymentTest,
    * bootstrap to fail.
    */
   mysql_harness::mkdir(config_file_, 0700);
-  auto server_mock = run_server_mock();
+  auto &server_mock = run_server_mock();
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(server_mock, server_port_));
 
   // launch the router in bootstrap mode
-  auto router =
-      launch_router("--bootstrap=127.0.0.1:" + std::to_string(server_port_) +
-                    " --report-host dont.query.dns");
+  auto &router = launch_router(
+      {
+          "--bootstrap=127.0.0.1:" + std::to_string(server_port_),
+          "--connect-timeout=1",
+          "--report-host",
+          "dont.query.dns",
+      },
+      EXIT_FAILURE);
 
   // add login hook
   router.register_response("Please enter MySQL password for root: ",
                            "fake-pass\n");
 
-  EXPECT_NO_THROW(EXPECT_EQ(router.wait_for_exit(), 1))
-      << router.get_full_output();
+  check_exit_code(router, EXIT_FAILURE);
 
   EXPECT_TRUE(router.expect_output(
       "Error: Could not save configuration file to final location", false))
