@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -44,36 +44,31 @@
  *
  */
 
-Path g_origin_path;
+using namespace std::chrono_literals;
+
 const char *g_this_exec_path;
 
-constexpr int kSleepDurationMs =
-    2000;  // you may want to decrease this to speed up tests
+constexpr auto kSleepDuration =
+    2000ms;  // you may want to decrease this to speed up tests
 
-class ComponentTestFrameworkTest : public RouterComponentTest,
-                                   public ::testing::Test {
+class ComponentTestFrameworkTest : public RouterComponentTest {
  protected:
-  virtual void SetUp() {
-    set_origin(g_origin_path);
-    RouterComponentTest::init();
-  }
-
-  static std::string show_output(CommandHandle &process,
+  static std::string show_output(ProcessWrapper &process,
                                  const std::string &process_description) {
     return process_description + ":\n" + process.get_full_output() +
            "-(end)-\n";
   }
 
   const std::string arglist_prefix_ =
-      "--gtest_also_run_disabled_tests "
       "--gtest_filter=ComponentTestFrameworkTest.DISABLED_";
 };
 
-static void autoresponder_testee(unsigned leak_interval_ms = 0) {
-  if (leak_interval_ms) {
-    auto slow_cout = [leak_interval_ms](char c) {
+static void autoresponder_testee(
+    std::chrono::milliseconds leak_interval = 0ms) {
+  if (leak_interval.count()) {
+    auto slow_cout = [leak_interval](char c) {
       std::cout << c << std::flush;
-      std::this_thread::sleep_for(std::chrono::milliseconds(leak_interval_ms));
+      std::this_thread::sleep_for(leak_interval);
     };
 
     slow_cout('S');
@@ -128,8 +123,9 @@ TEST_F(ComponentTestFrameworkTest, autoresponder_simple_tester) {
 
   // launch the DISABLED_autoresponder_simple_testee testcase as a separate
   // executable
-  CommandHandle testee = launch_command(
-      g_this_exec_path, arglist_prefix_ + "autoresponder_simple_testee");
+  ProcessWrapper &testee = launch_command(
+      g_this_exec_path, {"--gtest_also_run_disabled_tests",
+                         arglist_prefix_ + "autoresponder_simple_testee"});
   // register autoresponses
   testee.register_response("Syn", "Syn+Ack\n");
   testee.register_response("Fin", "Ack\n");
@@ -141,8 +137,9 @@ TEST_F(ComponentTestFrameworkTest, autoresponder_simple_tester) {
       << show_output(testee, "ROUTER OUTPUT");
 
   // wait for child
-  EXPECT_EQ(testee.wait_for_exit(), 0);
+  check_exit_code(testee, EXIT_SUCCESS);
 }
+
 TEST_F(ComponentTestFrameworkTest, DISABLED_autoresponder_simple_testee) {
   autoresponder_testee();
 }
@@ -175,7 +172,7 @@ std::this_thread::sleep_for(std::chrono::milliseconds(100));  // [THIS_LINE]
 
   EXPECT_TRUE(testee.expect_output("Syn\nAck\nFin\nOK", false, 2 * kSleepDurationMs)) << show_output(testee, "ROUTER OUTPUT");
 
-  EXPECT_EQ(testee.wait_for_exit(), 0);
+  check_exit_code(testee, EXIT_SUCCESS);
 
 }
 TEST_F(ComponentTestFrameworkTest, DISABLED_autoresponder_segmented_triggers_testee) {
@@ -185,7 +182,7 @@ TEST_F(ComponentTestFrameworkTest, DISABLED_autoresponder_segmented_triggers_tes
 
 static void sleepy_testee() {
   std::cout << "Hello, I'm feeling sleepy. Yawn." << std::endl;
-  std::this_thread::sleep_for(std::chrono::milliseconds(kSleepDurationMs));
+  std::this_thread::sleep_for(kSleepDuration);
   std::cout << "Yes, I'm still alive." << std::endl;
 }
 
@@ -195,17 +192,18 @@ TEST_F(ComponentTestFrameworkTest, sleepy_tester) {
    * for a longer period of time
    */
 
-  CommandHandle testee =
-      launch_command(g_this_exec_path, arglist_prefix_ + "sleepy_testee");
+  ProcessWrapper &testee = launch_command(
+      g_this_exec_path,
+      {"--gtest_also_run_disabled_tests", arglist_prefix_ + "sleepy_testee"});
 
   // first and second sentence should arrive kSleepDurationMs ms apart,
   // expect_output() should not give up reading during that time
   EXPECT_TRUE(testee.expect_output(
       "Hello, I'm feeling sleepy. Yawn.\nYes, I'm still alive.\n", false,
-      1.5 * kSleepDurationMs))
+      kSleepDuration + kSleepDuration / 2))
       << show_output(testee, "TESTED PROCESS");
 
-  EXPECT_EQ(testee.wait_for_exit(), 0);
+  check_exit_code(testee, EXIT_SUCCESS);
 }
 TEST_F(ComponentTestFrameworkTest, DISABLED_sleepy_testee) { sleepy_testee(); }
 
@@ -216,10 +214,10 @@ TEST_F(ComponentTestFrameworkTest, sleepy_blind_tester) {
    * it.
    */
 
-  CommandHandle testee =
-      launch_command(g_this_exec_path, arglist_prefix_ + "sleepy_blind_testee");
+  ProcessWrapper &testee = launch_command(
+      g_this_exec_path, {arglist_prefix_ + "sleepy_blind_testee"});
 
-  EXPECT_EQ(testee.wait_for_exit(1.5 * kSleepDurationMs), 0);
+  EXPECT_EQ(testee.wait_for_exit(kSleepDuration + kSleepDuration / 2), 0);
 }
 TEST_F(ComponentTestFrameworkTest, DISABLED_sleepy_blind_testee) {
   sleepy_testee();
@@ -238,17 +236,18 @@ TEST_F(ComponentTestFrameworkTest, sleepy_blind_autoresponder_tester) {
    * out waiting for the process to exit: No child processes"
    */
 
-  CommandHandle testee = launch_command(
-      g_this_exec_path, arglist_prefix_ + "sleepy_blind_autoresponder_testee");
+  ProcessWrapper &testee =
+      launch_command(g_this_exec_path,
+                     {arglist_prefix_ + "sleepy_blind_autoresponder_testee"});
 
   testee.register_response("Syn", "Syn+Ack\n");
   testee.register_response("Fin", "Ack\n");
 
   // wait for child (while reading and issuing autoresponses)
-  EXPECT_EQ(testee.wait_for_exit(1.5 * kSleepDurationMs), 0);
+  EXPECT_EQ(testee.wait_for_exit(kSleepDuration + kSleepDuration / 2), 0);
 }
 TEST_F(ComponentTestFrameworkTest, DISABLED_sleepy_blind_autoresponder_testee) {
-  std::this_thread::sleep_for(std::chrono::milliseconds(kSleepDurationMs));
+  std::this_thread::sleep_for(kSleepDuration);
   autoresponder_testee();
 }
 
@@ -256,7 +255,7 @@ int main(int argc, char *argv[]) {
   g_this_exec_path = argv[0];
 
   init_windows_sockets();
-  g_origin_path = Path(argv[0]).dirname();
+  ProcessManager::set_origin(Path(argv[0]).dirname());
   ::testing::InitGoogleTest(&argc, argv);
   return RUN_ALL_TESTS();
 }
