@@ -39,7 +39,6 @@
 #include "mock_server_rest_client.h"
 #include "mock_server_testutils.h"
 #include "mysql_session.h"
-#include "mysqlrouter/cluster_metadata.h"
 #include "mysqlrouter/rest_client.h"
 #include "random_generator.h"
 #include "rest_metadata_client.h"
@@ -49,7 +48,6 @@
 #define ASSERT_NO_ERROR(expr) \
   ASSERT_THAT(expr, ::testing::Eq(std::error_code{}))
 
-using mysqlrouter::ClusterType;
 using mysqlrouter::MySQLSession;
 using namespace std::chrono_literals;
 static constexpr const char kMockServerConnectionsUri[] =
@@ -103,9 +101,7 @@ class ConfigGenerator {
 
   // void metadata_refresh_ttl(unsigned ttl) { metadata_refresh_ttl_ = ttl; }
 
-  void add_metadata_cache_section(
-      std::chrono::milliseconds ttl,
-      ClusterType cluster_type = ClusterType::GR_V2) {
+  void add_metadata_cache_section(std::chrono::milliseconds ttl) {
     // NOT: Those tests are using bootstrap_server_addresses in the static
     // configuration which is now moved to the dynamic state file. This way we
     // are testing the backward compatibility of the old
@@ -113,16 +109,11 @@ class ConfigGenerator {
     // use
     // dynamic state file,  a new test should be added to test that
     // bootstrap_server_addresses is still handled properly.
-    const std::string cluster_type_str =
-        (cluster_type == ClusterType::RS_V2) ? "ar" : "gr";
     metadata_cache_section_ =
         "[logger]\n"
         "level = INFO\n\n"
 
         "[metadata_cache:test]\n"
-        "cluster_type=" +
-        cluster_type_str +
-        "\n"
         "router_id=1\n"
         "bootstrap_server_addresses=";
     size_t i = 0;
@@ -240,9 +231,8 @@ class ConfigGenerator {
   }
 
   std::string build_config_file(const std::string &temp_test_dir,
-                                ClusterType cluster_type,
                                 bool is_primary_and_secondary = false) {
-    add_metadata_cache_section(metadata_refresh_ttl_, cluster_type);
+    add_metadata_cache_section(metadata_refresh_ttl_);
     add_routing_primary_section();
     add_monitoring_section(temp_test_dir);
 
@@ -468,9 +458,9 @@ TEST_F(RouterRoutingConnectionTest, OldSchemaVersion) {
       check_port_ready(*cluster_nodes_[0], cluster_nodes_ports_[0]));
 
   SCOPED_TRACE("// [prep] launching router");
-  auto &router = launch_router(router_rw_port_,
-                               config_generator_->build_config_file(
-                                   temp_test_dir_.name(), ClusterType::GR_V2));
+  auto &router = launch_router(
+      router_rw_port_,
+      config_generator_->build_config_file(temp_test_dir_.name()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_rw_port_));
 
   SCOPED_TRACE("// [prep] waiting " +
@@ -503,11 +493,10 @@ TEST_F(RouterRoutingConnectionTest, OldSchemaVersion) {
   constexpr const char log_msg_re[]{
 #ifdef GTEST_USES_POSIX_RE
       "Unsupported metadata schema on .*\\. Expected Metadata Schema version "
-      "compatible to [0-9]\\.[0-9]\\.[0-9], [0-9]\\.[0-9]\\.[0-9], got "
-      "0\\.0\\.1"
+      "compatible to [0-9]\\.[0-9]\\.[0-9], got 0\\.0\\.0"
 #else
       "Unsupported metadata schema on .*\\. Expected Metadata Schema version "
-      "compatible to \\d\\.\\d\\.\\d, \\d\\.\\d\\.\\d, got 0\\.0\\.1"
+      "compatible to \\d\\.\\d\\.\\d, got 0\\.0\\.0"
 #endif
   };
 
@@ -522,11 +511,12 @@ TEST_F(RouterRoutingConnectionTest, OldSchemaVersion) {
  */
 TEST_F(RouterRoutingConnectionTest,
        IsRouterFailToStartWhen_disconnect_on_promoted_to_primary_invalid) {
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(
+      "metadata_3_secondaries_server_removed_from_cluster.js", 4));
   config_generator_->disconnect_on_promoted_to_primary(
       "&disconnect_on_promoted_to_primary=bogus");
   auto &router = ProcessManager::launch_router(
-      {"-c", config_generator_->build_config_file(temp_test_dir_.name(),
-                                                  ClusterType::GR_V2)},
+      {"-c", config_generator_->build_config_file(temp_test_dir_.name())},
       EXIT_FAILURE);
   check_port_not_ready(router, router_ro_port_);
 }
@@ -538,40 +528,24 @@ TEST_F(RouterRoutingConnectionTest,
  */
 TEST_F(RouterRoutingConnectionTest,
        IsRouterFailToStartWhen_disconnect_on_metadata_unavailable_invalid) {
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(
+      "metadata_3_secondaries_server_removed_from_cluster.js", 4));
   config_generator_->disconnect_on_metadata_unavailable(
       "&disconnect_on_metadata_unavailable=bogus");
   auto &router = ProcessManager::launch_router(
-      {"-c", config_generator_->build_config_file(temp_test_dir_.name(),
-                                                  ClusterType::GR_V2)},
+      {"-c", config_generator_->build_config_file(temp_test_dir_.name())},
       EXIT_FAILURE);
   check_port_not_ready(router, router_ro_port_);
 }
 
-struct TracefileTestParam {
-  std::string tracefile;
-  ClusterType cluster_type;
-  std::string param{};
-
-  TracefileTestParam(const std::string tracefile_,
-                     const ClusterType cluster_type_,
-                     const std::string param_ = "")
-      : tracefile(tracefile_), cluster_type(cluster_type_), param(param_) {}
-};
-
-class IsConnectionsClosedWhenPrimaryRemovedFromClusterTest
-    : public RouterRoutingConnectionTest,
-      public ::testing::WithParamInterface<TracefileTestParam> {};
-
 /**
  * @test
  *      Verify that all connections to Primary are closed when Primary is
- * removed from Cluster
+ * removed from GR;
  */
-TEST_P(IsConnectionsClosedWhenPrimaryRemovedFromClusterTest,
-       IsConnectionsClosedWhenPrimaryRemovedFromCluster) {
+TEST_F(RouterRoutingConnectionTest,
+       IsConnectionsClosedWhenPrimaryRemovedFromGR) {
   TempDirectory tmp_dir("conf");
-  const std::string tracefile = GetParam().tracefile;
-
   config_generator_.reset(new ConfigGenerator(
       get_DEFAULT_defaults(), tmp_dir.name(),
       {cluster_nodes_ports_[0], cluster_nodes_ports_[1]}, router_rw_port_,
@@ -580,19 +554,23 @@ TEST_P(IsConnectionsClosedWhenPrimaryRemovedFromClusterTest,
   SCOPED_TRACE("// launch the primary node on port " +
                std::to_string(cluster_nodes_ports_.at(0)) +
                " working also as metadata server");
-  cluster_nodes_.push_back(&launch_server(cluster_nodes_ports_[0], tracefile,
-                                          cluster_nodes_http_ports_[0], 4));
+  cluster_nodes_.push_back(
+      &launch_server(cluster_nodes_ports_[0],
+                     "metadata_3_secondaries_server_removed_from_cluster.js",
+                     cluster_nodes_http_ports_[0], 4));
 
   SCOPED_TRACE("// launch the secondary node on port " +
                std::to_string(cluster_nodes_ports_.at(1)) +
                " working also as metadata server");
-  cluster_nodes_.push_back(&launch_server(cluster_nodes_ports_[1], tracefile,
-                                          cluster_nodes_http_ports_[1], 4));
+  cluster_nodes_.push_back(
+      &launch_server(cluster_nodes_ports_[1],
+                     "metadata_3_secondaries_server_removed_from_cluster.js",
+                     cluster_nodes_http_ports_[1], 4));
 
   SCOPED_TRACE("// launch the rest of secondary cluster nodes");
   for (unsigned port = 2; port < 4; ++port) {
     cluster_nodes_.push_back(
-        &launch_server(cluster_nodes_ports_[port], tracefile,
+        &launch_server(cluster_nodes_ports_[port], "rest_server_mock.js",
                        cluster_nodes_http_ports_[port], 4));
   }
 
@@ -604,8 +582,8 @@ TEST_P(IsConnectionsClosedWhenPrimaryRemovedFromClusterTest,
 
   SCOPED_TRACE("// launching router");
   auto &router = launch_router(
-      router_rw_port_, config_generator_->build_config_file(
-                           temp_test_dir_.name(), GetParam().cluster_type));
+      router_rw_port_,
+      config_generator_->build_config_file(temp_test_dir_.name()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_rw_port_));
 
   SCOPED_TRACE("// waiting " +
@@ -659,35 +637,20 @@ TEST_P(IsConnectionsClosedWhenPrimaryRemovedFromClusterTest,
   }
 }
 
-INSTANTIATE_TEST_CASE_P(
-    IsConnectionsClosedWhenPrimaryRemovedFromCluster,
-    IsConnectionsClosedWhenPrimaryRemovedFromClusterTest,
-    ::testing::Values(
-        TracefileTestParam(
-            "metadata_3_secondaries_server_removed_from_cluster_v2_gr.js",
-            ClusterType::GR_V2),
-        TracefileTestParam(
-            "metadata_3_secondaries_server_removed_from_cluster.js",
-            ClusterType::GR_V1)));
-
-class IsConnectionsClosedWhenSecondaryRemovedFromClusterTest
-    : public RouterRoutingConnectionTest,
-      public ::testing::WithParamInterface<TracefileTestParam> {};
-
 /**
  * @test
  *      Verify that all connections to Secondary are closed when Secondary is
- *      removed from Cluster.
+ *      removed from GR.
  *
  */
-TEST_P(IsConnectionsClosedWhenSecondaryRemovedFromClusterTest,
-       IsConnectionsClosedWhenSecondaryRemovedFromCluster) {
-  const std::string tracefile = GetParam().tracefile;
-  ASSERT_NO_FATAL_FAILURE(setup_cluster(tracefile, 4));
+TEST_F(RouterRoutingConnectionTest,
+       IsConnectionsClosedWhenSecondaryRemovedFromGR) {
+  ASSERT_NO_FATAL_FAILURE(setup_cluster(
+      "metadata_3_secondaries_server_removed_from_cluster.js", 4));
 
   auto &router = launch_router(
-      router_rw_port_, config_generator_->build_config_file(
-                           temp_test_dir_.name(), GetParam().cluster_type));
+      router_rw_port_,
+      config_generator_->build_config_file(temp_test_dir_.name()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_rw_port_));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_ro_port_));
 
@@ -737,34 +700,17 @@ TEST_P(IsConnectionsClosedWhenSecondaryRemovedFromClusterTest,
   }
 }
 
-INSTANTIATE_TEST_CASE_P(
-    IsConnectionsClosedWhenSecondaryRemovedFromCluster,
-    IsConnectionsClosedWhenSecondaryRemovedFromClusterTest,
-    ::testing::Values(
-        TracefileTestParam(
-            "metadata_3_secondaries_server_removed_from_cluster_v2_gr.js",
-            ClusterType::GR_V2),
-        TracefileTestParam(
-            "metadata_3_secondaries_server_removed_from_cluster.js",
-            ClusterType::GR_V1)));
-
-class IsRWConnectionsClosedWhenPrimaryFailoverTest
-    : public RouterRoutingConnectionTest,
-      public ::testing::WithParamInterface<TracefileTestParam> {};
-
 /**
  * @test
  *       Verify that when Primary is demoted, then all RW connections
  *       to that server are closed.
  */
-TEST_P(IsRWConnectionsClosedWhenPrimaryFailoverTest,
-       IsRWConnectionsClosedWhenPrimaryFailover) {
-  const std::string tracefile = GetParam().tracefile;
-
-  ASSERT_NO_FATAL_FAILURE(setup_cluster(tracefile, 4));
+TEST_F(RouterRoutingConnectionTest, IsRWConnectionsClosedWhenPrimaryFailover) {
+  ASSERT_NO_FATAL_FAILURE(
+      setup_cluster("metadata_3_secondaries_primary_failover.js", 4));
   auto &router = launch_router(
-      router_ro_port_, config_generator_->build_config_file(
-                           temp_test_dir_.name(), GetParam().cluster_type));
+      router_ro_port_,
+      config_generator_->build_config_file(temp_test_dir_.name()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_rw_port_));
 
   /*
@@ -806,36 +752,20 @@ TEST_P(IsRWConnectionsClosedWhenPrimaryFailoverTest,
   }
 }
 
-INSTANTIATE_TEST_CASE_P(
-    IsRWConnectionsClosedWhenPrimaryFailover,
-    IsRWConnectionsClosedWhenPrimaryFailoverTest,
-    ::testing::Values(
-        TracefileTestParam("metadata_3_secondaries_primary_failover_v2_gr.js",
-                           ClusterType::GR_V2),
-        TracefileTestParam("metadata_3_secondaries_primary_failover.js",
-                           ClusterType::GR_V1)));
-
-class IsROConnectionsKeptWhenPrimaryFailoverTest
-    : public RouterRoutingConnectionTest,
-      public ::testing::WithParamInterface<TracefileTestParam> {};
-
 /**
  * @test
  *       Verify that when Primary is demoted, then RO connections
  *       to that server are kept.
  */
-TEST_P(IsROConnectionsKeptWhenPrimaryFailoverTest,
-       IsROConnectionsKeptWhenPrimaryFailover) {
-  const std::string tracefile = GetParam().tracefile;
-
-  ASSERT_NO_FATAL_FAILURE(setup_cluster(tracefile, 4));
+TEST_F(RouterRoutingConnectionTest, IsROConnectionsKeptWhenPrimaryFailover) {
+  ASSERT_NO_FATAL_FAILURE(
+      setup_cluster("metadata_3_secondaries_primary_failover.js", 4));
 
   config_generator_->disconnect_on_promoted_to_primary("");
 
-  auto &router =
-      launch_router(router_ro_port_,
-                    config_generator_->build_config_file(
-                        temp_test_dir_.name(), GetParam().cluster_type, true));
+  auto &router = launch_router(
+      router_ro_port_,
+      config_generator_->build_config_file(temp_test_dir_.name(), true));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_rw_port_));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_ro_port_));
 
@@ -877,18 +807,9 @@ TEST_P(IsROConnectionsKeptWhenPrimaryFailoverTest,
   }
 }
 
-INSTANTIATE_TEST_CASE_P(
-    IsROConnectionsKeptWhenPrimaryFailover,
-    IsROConnectionsKeptWhenPrimaryFailoverTest,
-    ::testing::Values(
-        TracefileTestParam("metadata_3_secondaries_primary_failover_v2_gr.js",
-                           ClusterType::GR_V2),
-        TracefileTestParam("metadata_3_secondaries_primary_failover.js",
-                           ClusterType::GR_V1)));
-
 class RouterRoutingConnectionPromotedTest
     : public RouterRoutingConnectionCommonTest,
-      public testing::WithParamInterface<TracefileTestParam> {};
+      public testing::WithParamInterface<std::string> {};
 
 /**
  * @test
@@ -898,16 +819,14 @@ class RouterRoutingConnectionPromotedTest
  */
 TEST_P(RouterRoutingConnectionPromotedTest,
        IsConnectionsToSecondaryKeptWhenPromotedToPrimary) {
-  const std::string tracefile = GetParam().tracefile;
-  const std::string param = GetParam().param;
+  ASSERT_NO_FATAL_FAILURE(
+      setup_cluster("metadata_3_secondaries_primary_failover.js", 4));
 
-  ASSERT_NO_FATAL_FAILURE(setup_cluster(tracefile, 4));
-
-  config_generator_->disconnect_on_promoted_to_primary(param);
+  config_generator_->disconnect_on_promoted_to_primary(GetParam());
 
   auto &router = launch_router(
-      router_ro_port_, config_generator_->build_config_file(
-                           temp_test_dir_.name(), GetParam().cluster_type));
+      router_ro_port_,
+      config_generator_->build_config_file(temp_test_dir_.name()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_rw_port_));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_ro_port_));
 
@@ -951,24 +870,14 @@ TEST_P(RouterRoutingConnectionPromotedTest,
   }
 }
 
-INSTANTIATE_TEST_CASE_P(
-    RouterRoutingIsConnectionNotClosedWhenPromoted,
-    RouterRoutingConnectionPromotedTest,
-    ::testing::Values(
-        TracefileTestParam("metadata_3_secondaries_primary_failover_v2_gr.js",
-                           ClusterType::GR_V2,
-                           "&disconnect_on_promoted_to_primary=no"),
-        TracefileTestParam("metadata_3_secondaries_primary_failover.js",
-                           ClusterType::GR_V1,
-                           "&disconnect_on_promoted_to_primary=no"),
-        TracefileTestParam("metadata_3_secondaries_primary_failover_v2_gr.js",
-                           ClusterType::GR_V2, ""),
-        TracefileTestParam("metadata_3_secondaries_primary_failover.js",
-                           ClusterType::GR_V1, "")));
+std::string promoted_flags[] = {
+    "&disconnect_on_promoted_to_primary=no",
+    "",
+};
 
-class IsConnectionToSecondaryClosedWhenPromotedToPrimaryTest
-    : public RouterRoutingConnectionTest,
-      public ::testing::WithParamInterface<TracefileTestParam> {};
+INSTANTIATE_TEST_CASE_P(RouterRoutingIsConnectionNotClosedWhenPromoted,
+                        RouterRoutingConnectionPromotedTest,
+                        testing::ValuesIn(promoted_flags));
 
 /**
  * @test
@@ -976,18 +885,17 @@ class IsConnectionToSecondaryClosedWhenPromotedToPrimaryTest
  *       disconnect_on_promoted_to_primary is set to 'yes' then connections
  *       to that server are closed.
  */
-TEST_P(IsConnectionToSecondaryClosedWhenPromotedToPrimaryTest,
+TEST_F(RouterRoutingConnectionTest,
        IsConnectionToSecondaryClosedWhenPromotedToPrimary) {
-  const std::string tracefile = GetParam().tracefile;
-
-  ASSERT_NO_FATAL_FAILURE(setup_cluster(tracefile, 4));
+  ASSERT_NO_FATAL_FAILURE(
+      setup_cluster("metadata_3_secondaries_primary_failover.js", 4));
 
   config_generator_->disconnect_on_promoted_to_primary(
       "&disconnect_on_promoted_to_primary=yes");
 
   auto &router = launch_router(
-      router_ro_port_, config_generator_->build_config_file(
-                           temp_test_dir_.name(), GetParam().cluster_type));
+      router_ro_port_,
+      config_generator_->build_config_file(temp_test_dir_.name()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_ro_port_));
 
   /*
@@ -1033,36 +941,22 @@ TEST_P(IsConnectionToSecondaryClosedWhenPromotedToPrimaryTest,
   }
 }
 
-INSTANTIATE_TEST_CASE_P(
-    IsConnectionToSecondaryClosedWhenPromotedToPrimary,
-    IsConnectionToSecondaryClosedWhenPromotedToPrimaryTest,
-    ::testing::Values(
-        TracefileTestParam("metadata_3_secondaries_primary_failover_v2_gr.js",
-                           ClusterType::GR_V2),
-        TracefileTestParam("metadata_3_secondaries_primary_failover.js",
-                           ClusterType::GR_V1)));
-
-class IsConnectionToMinorityClosedWhenClusterPartitionTest
-    : public RouterRoutingConnectionTest,
-      public ::testing::WithParamInterface<TracefileTestParam> {};
-
 /**
  * @test
  *       Verify that when GR is partitioned, then connections to servers that
  *       are not in majority are closed.
  */
-TEST_P(IsConnectionToMinorityClosedWhenClusterPartitionTest,
+TEST_F(RouterRoutingConnectionTest,
        IsConnectionToMinorityClosedWhenClusterPartition) {
-  const std::string tracefile = GetParam().tracefile;
-
   /*
    * create cluster with 5 servers
    */
-  ASSERT_NO_FATAL_FAILURE(setup_cluster(tracefile, 5));
+  ASSERT_NO_FATAL_FAILURE(
+      setup_cluster("metadata_4_secondaries_partitioning.js", 5));
 
   auto &router = launch_router(
-      router_ro_port_, config_generator_->build_config_file(
-                           temp_test_dir_.name(), GetParam().cluster_type));
+      router_ro_port_,
+      config_generator_->build_config_file(temp_test_dir_.name()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_ro_port_));
 
   /*
@@ -1073,8 +967,7 @@ TEST_P(IsConnectionToMinorityClosedWhenClusterPartitionTest,
                                           kRestApiUsername, kRestApiPassword);
 
   ASSERT_NO_ERROR(rest_metadata_client.wait_for_cache_ready(
-      wait_for_cache_ready_timeout, metadata_status))
-      << router.get_full_logfile();
+      wait_for_cache_ready_timeout, metadata_status));
 
   // connect clients
   std::vector<std::pair<MySQLSession, uint16_t>> clients(10);
@@ -1139,22 +1032,13 @@ TEST_P(IsConnectionToMinorityClosedWhenClusterPartitionTest,
   for (auto &client_and_port : clients) {
     auto &client = client_and_port.first;
     ASSERT_ANY_THROW(client.query_one("select @@port"))
-        << router.get_full_logfile();
+        << router.get_full_output();
   }
 }
 
-INSTANTIATE_TEST_CASE_P(
-    IsConnectionToMinorityClosedWhenClusterPartition,
-    IsConnectionToMinorityClosedWhenClusterPartitionTest,
-    ::testing::Values(
-        TracefileTestParam("metadata_4_secondaries_partitioning_v2_gr.js",
-                           ClusterType::GR_V2),
-        TracefileTestParam("metadata_4_secondaries_partitioning.js",
-                           ClusterType::GR_V1)));
-
-class IsConnectionClosedWhenClusterOverloadedTest
+class RouterRoutingConnectionClusterOverloadTest
     : public RouterRoutingConnectionCommonTest,
-      public testing::WithParamInterface<TracefileTestParam> {};
+      public testing::WithParamInterface<std::string> {};
 
 /**
  * @test
@@ -1162,18 +1046,15 @@ class IsConnectionClosedWhenClusterOverloadedTest
  * disconnect_on_metadata_unavailable is set to 'yes' then all connection to
  * GR are closed
  */
-TEST_P(IsConnectionClosedWhenClusterOverloadedTest,
-       IsConnectionClosedWhenClusterOverloaded) {
-  const std::string tracefile = GetParam().tracefile;
-
-  ASSERT_NO_FATAL_FAILURE(setup_cluster(tracefile, 4));
+TEST_F(RouterRoutingConnectionTest, IsConnectionClosedWhenClusterOverloaded) {
+  ASSERT_NO_FATAL_FAILURE(setup_cluster("metadata_3_secondaries_pass.js", 4));
 
   config_generator_->disconnect_on_metadata_unavailable(
       "&disconnect_on_metadata_unavailable=yes");
 
   auto &router = launch_router(
-      router_ro_port_, config_generator_->build_config_file(
-                           temp_test_dir_.name(), GetParam().cluster_type));
+      router_ro_port_,
+      config_generator_->build_config_file(temp_test_dir_.name()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_ro_port_));
 
   /*
@@ -1230,17 +1111,9 @@ TEST_P(IsConnectionClosedWhenClusterOverloadedTest,
   }
 }
 
-INSTANTIATE_TEST_CASE_P(
-    IsConnectionClosedWhenClusterOverloaded,
-    IsConnectionClosedWhenClusterOverloadedTest,
-    ::testing::Values(TracefileTestParam("metadata_3_secondaries_pass_v2_gr.js",
-                                         ClusterType::GR_V2),
-                      TracefileTestParam("metadata_3_secondaries_pass.js",
-                                         ClusterType::GR_V1)));
-
 class RouterRoutingConnectionMDUnavailableTest
     : public RouterRoutingConnectionCommonTest,
-      public testing::WithParamInterface<TracefileTestParam> {};
+      public testing::WithParamInterface<std::string> {};
 
 /**
  * @test
@@ -1250,15 +1123,12 @@ class RouterRoutingConnectionMDUnavailableTest
  */
 TEST_P(RouterRoutingConnectionMDUnavailableTest,
        IsConnectionKeptWhenClusterOverloaded) {
-  const std::string tracefile = GetParam().tracefile;
-  const std::string param = GetParam().param;
+  ASSERT_NO_FATAL_FAILURE(setup_cluster("metadata_3_secondaries_pass.js", 4));
 
-  ASSERT_NO_FATAL_FAILURE(setup_cluster(tracefile, 4));
-
-  config_generator_->disconnect_on_promoted_to_primary(param);
+  config_generator_->disconnect_on_promoted_to_primary(GetParam());
   auto &router = launch_router(
-      router_ro_port_, config_generator_->build_config_file(
-                           temp_test_dir_.name(), GetParam().cluster_type));
+      router_ro_port_,
+      config_generator_->build_config_file(temp_test_dir_.name()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_rw_port_));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_ro_port_));
 
@@ -1306,39 +1176,20 @@ TEST_P(RouterRoutingConnectionMDUnavailableTest,
   }
 }
 
-INSTANTIATE_TEST_CASE_P(
-    RouterRoutingIsConnectionNotClosedWhenMDUnavailable,
-    RouterRoutingConnectionMDUnavailableTest,
-    ::testing::Values(
-        TracefileTestParam("metadata_3_secondaries_pass_v2_gr.js",
-                           ClusterType::GR_V2,
-                           "&disconnect_on_metadata_unavailable=no"),
-        TracefileTestParam("metadata_3_secondaries_pass.js", ClusterType::GR_V1,
-                           "&disconnect_on_metadata_unavailable=no"),
-        TracefileTestParam("metadata_3_secondaries_pass_v2_gr.js",
-                           ClusterType::GR_V2, ""),
-        TracefileTestParam("metadata_3_secondaries_pass.js", ClusterType::GR_V1,
-                           "")));
+std::string metadata_unavailable_flags[] = {
+    "&disconnect_on_metadata_unavailable=no",
+    "",
+};
+
+INSTANTIATE_TEST_CASE_P(RouterRoutingIsConnectionNotClosedWhenMDUnavailable,
+                        RouterRoutingConnectionMDUnavailableTest,
+                        testing::ValuesIn(metadata_unavailable_flags));
 
 using server_globals = RouterRoutingConnectionCommonTest::server_globals;
 
-struct MDRefreshTestParam {
-  ClusterType cluster_type;
-  std::string tracefile1;
-  std::string tracefile2;
-  server_globals globals;
-
-  MDRefreshTestParam(ClusterType cluster_type_, std::string tracefile1_,
-                     std::string tracefile2_, server_globals globals_)
-      : cluster_type(cluster_type_),
-        tracefile1(tracefile1_),
-        tracefile2(tracefile2_),
-        globals(globals_) {}
-};
-
 class RouterRoutingConnectionMDRefreshTest
     : public RouterRoutingConnectionCommonTest,
-      public testing::WithParamInterface<MDRefreshTestParam> {};
+      public testing::WithParamInterface<server_globals> {};
 
 /**
  * @test
@@ -1364,13 +1215,13 @@ TEST_P(RouterRoutingConnectionMDRefreshTest,
       router_ro_port_, monitoring_port_, metadata_refresh_ttl_));
 
   // launch the primary node working also as metadata server
-  cluster_nodes_.push_back(&launch_server(cluster_nodes_ports_[0],
-                                          GetParam().tracefile1,
-                                          cluster_nodes_http_ports_[0], 4));
+  cluster_nodes_.push_back(&launch_server(
+      cluster_nodes_ports_[0], "metadata_3_secondaries_failed_to_update.js",
+      cluster_nodes_http_ports_[0], 4));
 
   // launch the secondary node working also as metadata server
   cluster_nodes_.push_back(&launch_server(cluster_nodes_ports_[1],
-                                          GetParam().tracefile2,
+                                          "metadata_3_secondaries_pass.js",
                                           cluster_nodes_http_ports_[1], 4));
 
   // launch the rest of secondary cluster nodes
@@ -1383,8 +1234,8 @@ TEST_P(RouterRoutingConnectionMDRefreshTest,
   config_generator_->disconnect_on_metadata_unavailable(
       "&disconnect_on_metadata_unavailable=yes");
   auto &router = launch_router(
-      router_ro_port_, config_generator_->build_config_file(
-                           temp_test_dir_.name(), GetParam().cluster_type));
+      router_ro_port_,
+      config_generator_->build_config_file(temp_test_dir_.name()));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_rw_port_));
   ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_ro_port_));
 
@@ -1425,7 +1276,7 @@ TEST_P(RouterRoutingConnectionMDRefreshTest,
 
   ASSERT_TRUE(MockServerRestClient(cluster_nodes_http_ports_[0])
                   .wait_for_rest_endpoint_ready());
-  set_additional_globals(cluster_nodes_http_ports_[0], GetParam().globals);
+  set_additional_globals(cluster_nodes_http_ports_[0], GetParam());
   ASSERT_NO_ERROR(rest_metadata_client.wait_for_cache_updated(
       wait_for_cache_update_timeout, metadata_status));
 
@@ -1438,35 +1289,9 @@ TEST_P(RouterRoutingConnectionMDRefreshTest,
   }
 }
 
-MDRefreshTestParam steps[] = {
-    MDRefreshTestParam(ClusterType::GR_V2,
-                       "metadata_3_secondaries_failed_to_update_v2_gr.js",
-                       "metadata_3_secondaries_pass_v2_gr.js",
-                       server_globals().set_MD_failed()),
-
-    MDRefreshTestParam(
-        ClusterType::GR_V1, "metadata_3_secondaries_failed_to_update.js",
-        "metadata_3_secondaries_pass.js", server_globals().set_MD_failed()),
-
-    MDRefreshTestParam(ClusterType::GR_V2,
-                       "metadata_3_secondaries_failed_to_update_v2_gr.js",
-                       "metadata_3_secondaries_pass_v2_gr.js",
-                       server_globals().set_GR_primary_failed()),
-
-    MDRefreshTestParam(ClusterType::GR_V1,
-                       "metadata_3_secondaries_failed_to_update.js",
-                       "metadata_3_secondaries_pass.js",
-                       server_globals().set_GR_primary_failed()),
-
-    MDRefreshTestParam(ClusterType::GR_V2,
-                       "metadata_3_secondaries_failed_to_update_v2_gr.js",
-                       "metadata_3_secondaries_pass_v2_gr.js",
-                       server_globals().set_GR_health_failed()),
-
-    MDRefreshTestParam(ClusterType::GR_V1,
-                       "metadata_3_secondaries_failed_to_update.js",
-                       "metadata_3_secondaries_pass.js",
-                       server_globals().set_GR_health_failed())};
+server_globals steps[] = {server_globals().set_MD_failed(),
+                          server_globals().set_GR_primary_failed(),
+                          server_globals().set_GR_health_failed()};
 
 INSTANTIATE_TEST_CASE_P(RouterRoutingIsConnectionNotDisabledWhenMDRefresh,
                         RouterRoutingConnectionMDRefreshTest,
