@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -22,20 +22,33 @@
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "gmock/gmock.h"
-#include "router_component_test.h"
-#include "tcp_port_pool.h"
-
 #include <chrono>
 #include <thread>
 
-Path g_origin_path;
+#include <gmock/gmock.h>
+
+#include "mock_server_rest_client.h"
+#include "mock_server_testutils.h"
+#include "mysql_session.h"
+#include "rest_metadata_client.h"
+#include "router_component_test.h"
+#include "tcp_port_pool.h"
+
+/**
+ * assert std::error_code has no 'error'
+ */
+#define ASSERT_NO_ERROR(expr) \
+  ASSERT_THAT(expr, ::testing::Eq(std::error_code{})) << expr.message()
+
+using mysqlrouter::MySQLSession;
+
+static const std::string kRestApiUsername("someuser");
+static const std::string kRestApiPassword("somepass");
 
 class RouterRoutingStrategyTest : public RouterComponentTest {
  protected:
   virtual void SetUp() {
-    set_origin(g_origin_path);
-    RouterComponentTest::init();
+    RouterComponentTest::SetUp();
 
     // Valgrind needs way more time
     if (getenv("WITH_VALGRIND")) {
@@ -57,7 +70,7 @@ class RouterRoutingStrategyTest : public RouterComponentTest {
   }
 
   std::string get_static_routing_section(
-      unsigned router_port, const std::vector<unsigned> &destinations,
+      unsigned router_port, const std::vector<uint16_t> &destinations,
       const std::string &strategy, const std::string &mode = "") {
     std::string result =
         "[routing:test_default]\n"
@@ -121,39 +134,79 @@ class RouterRoutingStrategyTest : public RouterComponentTest {
     return result;
   }
 
-  RouterComponentTest::CommandHandle launch_cluster_node(
-      unsigned cluster_port, const std::string &data_dir,
-      const std::string &tmp_dir) {
-    const std::string json_my_port_template =
-        Path(data_dir).join("my_port.js").str();
-    const std::string json_my_port =
-        Path(tmp_dir)
-            .join("my_port_" + std::to_string(cluster_port) + ".json")
-            .str();
-    std::map<std::string, std::string> env_vars = {
-        {"MY_PORT", std::to_string(cluster_port)},
-    };
-    RouterComponentTest::rewrite_js_to_tracefile(json_my_port_template,
-                                                 json_my_port, env_vars);
-    auto cluster_node = RouterComponentTest::launch_mysql_server_mock(
-        json_my_port, cluster_port, false);
-    bool ready = wait_for_port_ready(cluster_port, 1000);
-    EXPECT_TRUE(ready) << cluster_node.get_full_output();
+  std::string get_monitoring_section(unsigned monitoring_port,
+                                     const std::string &config_dir) {
+    std::string passwd_filename =
+        mysql_harness::Path(config_dir).join("users").str();
+
+    {
+      auto &cmd = launch_command(get_origin().join("mysqlrouter_passwd").str(),
+                                 {"set", passwd_filename, kRestApiUsername},
+                                 EXIT_SUCCESS, true);
+      cmd.register_response("Please enter password", kRestApiPassword + "\n");
+      check_exit_code(cmd, EXIT_SUCCESS);
+    }
+
+    return "[rest_api]\n"
+           "[rest_metadata_cache]\n"
+           "require_realm=somerealm\n"
+           "[http_auth_realm:somerealm]\n"
+           "backend=somebackend\n"
+           "method=basic\n"
+           "name=somerealm\n"
+           "[http_auth_backend:somebackend]\n"
+           "backend=file\n"
+           "filename=" +
+           passwd_filename +
+           "\n"
+           "[http_server]\n"
+           "port=" +
+           std::to_string(monitoring_port) + "\n";
+  }
+
+  // need to return void to be able to use ASSERT_ macros
+  void connect_client_and_query_port(unsigned router_port,
+                                     std::string &out_port,
+                                     bool should_fail = false) {
+    MySQLSession client;
+
+    if (should_fail) {
+      EXPECT_THROW_LIKE(client.connect("127.0.0.1", router_port, "username",
+                                       "password", "", ""),
+                        std::exception, "Error connecting to MySQL server");
+      out_port = "";
+      return;
+    } else {
+      ASSERT_NO_THROW(client.connect("127.0.0.1", router_port, "username",
+                                     "password", "", ""));
+    }
+
+    std::unique_ptr<MySQLSession::ResultRow> result{
+        client.query_one("select @@port")};
+    ASSERT_NE(nullptr, result.get());
+    ASSERT_EQ(1u, result->size());
+    out_port = std::string((*result)[0]);
+  }
+
+  ProcessWrapper &launch_cluster_node(unsigned cluster_port,
+                                      const std::string &data_dir) {
+    const std::string js_file = Path(data_dir).join("my_port.js").str();
+    auto &cluster_node = ProcessManager::launch_mysql_server_mock(
+        js_file, cluster_port, EXIT_SUCCESS, false);
 
     return cluster_node;
   }
 
-  RouterComponentTest::CommandHandle launch_standalone_server(
-      unsigned server_port, const std::string &data_dir,
-      const std::string &tmp_dir) {
+  ProcessWrapper &launch_standalone_server(unsigned server_port,
+                                           const std::string &data_dir) {
     // it' does the same thing, just an alias  for less confusion
-    return launch_cluster_node(server_port, data_dir, tmp_dir);
+    return launch_cluster_node(server_port, data_dir);
   }
 
-  RouterComponentTest::CommandHandle launch_router_static(
-      const std::string &conf_dir, unsigned router_port,
-      const std::string &routing_section, bool expect_error = false,
-      bool log_to_console = true) {
+  ProcessWrapper &launch_router_static(const std::string &conf_dir,
+                                       const std::string &routing_section,
+                                       bool expect_error = false,
+                                       bool log_to_console = true) {
     auto def_section = get_DEFAULT_defaults();
     if (log_to_console) {
       def_section["logging_folder"] = "";
@@ -161,46 +214,31 @@ class RouterRoutingStrategyTest : public RouterComponentTest {
     // launch the router with the static routing configuration
     const std::string conf_file =
         create_config_file(conf_dir, routing_section, &def_section);
-    auto router = RouterComponentTest::launch_router("-c " + conf_file);
-    if (!expect_error) {
-      bool ready = wait_for_port_ready(router_port, 1000);
-      EXPECT_TRUE(ready) << (log_to_console ? router.get_full_output()
-                                            : get_router_log_output());
-    }
+    const int expected_exit_code = expect_error ? EXIT_FAILURE : EXIT_SUCCESS;
+    auto &router =
+        ProcessManager::launch_router({"-c", conf_file}, expected_exit_code);
 
     return router;
   }
 
-  RouterComponentTest::CommandHandle launch_router(
-      unsigned router_port, const std::string &temp_test_dir,
-      const std::string &metadata_cache_section,
-      const std::string &routing_section, bool catch_stderr = true,
-      bool with_sudo = false, bool wait_ready = true,
-      bool log_to_stdout = false) {
+  ProcessWrapper &launch_router(const std::string &temp_test_dir,
+                                const std::string &metadata_cache_section,
+                                const std::string &routing_section) {
     auto default_section = get_DEFAULT_defaults();
     init_keyring(default_section, temp_test_dir);
 
     // launch the router with metadata-cache configuration
-    if (log_to_stdout) {
-      default_section["logging_folder"] = "";
-    }
-    const std::string conf_dir = get_tmp_dir("conf");
-    std::shared_ptr<void> exit_guard(nullptr,
-                                     [&](void *) { purge_dir(conf_dir); });
     const std::string conf_file = create_config_file(
-        conf_dir, metadata_cache_section + routing_section, &default_section);
-    auto router = RouterComponentTest::launch_router("-c " + conf_file,
-                                                     catch_stderr, with_sudo);
-    if (wait_ready) {
-      bool ready = wait_for_port_ready(router_port, 1000);
-      EXPECT_TRUE(ready) << get_router_log_output();
-    }
+        temp_test_dir, metadata_cache_section + routing_section,
+        &default_section);
+    auto &router = ProcessManager::launch_router({"-c", conf_file},
+                                                 EXIT_SUCCESS, true, false);
 
     return router;
   }
 
-  void kill_server(RouterComponentTest::CommandHandle &server) {
-    EXPECT_NO_THROW(server.kill()) << server.get_full_output();
+  void kill_server(ProcessWrapper *server) {
+    EXPECT_NO_THROW(server->kill()) << server->get_full_output();
   }
 
   TcpPortPool port_pool_;
@@ -239,7 +277,7 @@ struct MetadataCacheTestParams {
 
 class RouterRoutingStrategyMetadataCache
     : public RouterRoutingStrategyTest,
-      public ::testing::TestWithParam<MetadataCacheTestParams> {
+      public ::testing::WithParamInterface<MetadataCacheTestParams> {
  protected:
   virtual void SetUp() { RouterRoutingStrategyTest::SetUp(); }
 };
@@ -251,46 +289,30 @@ class RouterRoutingStrategyMetadataCache
 TEST_P(RouterRoutingStrategyMetadataCache, MetadataCacheRoutingStrategy) {
   auto test_params = GetParam();
 
-  const std::string temp_test_dir = get_tmp_dir();
-  std::shared_ptr<void> exit_guard(nullptr,
-                                   [&](void *) { purge_dir(temp_test_dir); });
+  TempDirectory temp_test_dir;
 
-  const std::vector<unsigned> cluster_nodes_ports{
+  const std::vector<uint16_t> cluster_nodes_ports{
+      port_pool_.get_next_available(),  // first is PRIMARY
+      port_pool_.get_next_available(), port_pool_.get_next_available(),
+      port_pool_.get_next_available()};
+  const std::vector<uint16_t> cluster_nodes_http_ports{
       port_pool_.get_next_available(),  // first is PRIMARY
       port_pool_.get_next_available(), port_pool_.get_next_available(),
       port_pool_.get_next_available()};
 
-  std::vector<RouterComponentTest::CommandHandle> cluster_nodes;
-
-  std::map<std::string, std::string> primary_json_env_vars = {
-      {"PRIMARY_HOST", "127.0.0.1:" + std::to_string(cluster_nodes_ports[0])},
-      {"SECONDARY_1_HOST",
-       "127.0.0.1:" + std::to_string(cluster_nodes_ports[1])},
-      {"SECONDARY_2_HOST",
-       "127.0.0.1:" + std::to_string(cluster_nodes_ports[2])},
-      {"SECONDARY_3_HOST",
-       "127.0.0.1:" + std::to_string(cluster_nodes_ports[3])},
-
-      {"PRIMARY_PORT", std::to_string(cluster_nodes_ports[0])},
-      {"SECONDARY_1_PORT", std::to_string(cluster_nodes_ports[1])},
-      {"SECONDARY_2_PORT", std::to_string(cluster_nodes_ports[2])},
-      {"SECONDARY_3_PORT", std::to_string(cluster_nodes_ports[3])},
-
-      {"MY_PORT", std::to_string(cluster_nodes_ports[0])},
-  };
+  std::vector<ProcessWrapper *> cluster_nodes;
 
   // launch the primary node working also as metadata server
-  const std::string json_primary_node_template =
-      get_data_dir().join("metadata_3_secondaries.js").str();
-  const std::string json_primary_node =
-      Path(temp_test_dir).join("metadata_3_secondaries.js").str();
-  rewrite_js_to_tracefile(json_primary_node_template, json_primary_node,
-                          primary_json_env_vars);
-  auto primary_node = launch_mysql_server_mock(json_primary_node,
-                                               cluster_nodes_ports[0], false);
-  bool ready = wait_for_port_ready(cluster_nodes_ports[0], 1000);
-  EXPECT_TRUE(ready) << primary_node.get_full_output();
-  cluster_nodes.emplace_back(std::move(primary_node));
+  const auto json_file =
+      get_data_dir().join("metadata_3_secondaries_pass.js").str();
+  const auto http_port = cluster_nodes_http_ports[0];
+  auto &primary_node = launch_mysql_server_mock(
+      json_file, cluster_nodes_ports[0], EXIT_SUCCESS, false, http_port);
+  ASSERT_NO_FATAL_FAILURE(
+      check_port_ready(primary_node, cluster_nodes_ports[0]));
+  EXPECT_TRUE(MockServerRestClient(http_port).wait_for_rest_endpoint_ready());
+  set_mock_metadata(http_port, "", cluster_nodes_ports);
+  cluster_nodes.emplace_back(&primary_node);
 
   // launch the router with metadata-cache configuration
   const auto router_port = port_pool_.get_next_available();
@@ -299,20 +321,36 @@ TEST_P(RouterRoutingStrategyMetadataCache, MetadataCacheRoutingStrategy) {
   const std::string routing_section = get_metadata_cache_routing_section(
       router_port, test_params.role, test_params.routing_strategy,
       test_params.mode);
-  auto router = launch_router(router_port, temp_test_dir,
-                              metadata_cache_section, routing_section);
+  const auto monitoring_port = port_pool_.get_next_available();
+  const std::string monitoring_section =
+      get_monitoring_section(monitoring_port, temp_test_dir.name());
+
+  auto &router = launch_router(temp_test_dir.name(),
+                               metadata_cache_section + monitoring_section,
+                               routing_section);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_port))
+      << router.get_full_output();
 
   // launch the secondary cluster nodes
   for (unsigned port = 1; port < cluster_nodes_ports.size(); ++port) {
-    auto secondary_node = launch_cluster_node(
-        cluster_nodes_ports[port], get_data_dir().str(), temp_test_dir);
-    cluster_nodes.emplace_back(std::move(secondary_node));
+    auto &secondary_node =
+        launch_cluster_node(cluster_nodes_ports[port], get_data_dir().str());
+    cluster_nodes.emplace_back(&secondary_node);
+    ASSERT_NO_FATAL_FAILURE(
+        check_port_ready(secondary_node, cluster_nodes_ports[port]));
   }
 
   // give the router a chance to initialise metadata-cache module
-  // there is currently no easy way to check that
-  std::this_thread::sleep_for(
-      std::chrono::milliseconds(wait_for_cache_ready_timeout));
+  // there is currently now easy way to check that
+  SCOPED_TRACE("// waiting " + std::to_string(wait_for_cache_ready_timeout) +
+               "ms until metadata is initialized");
+  RestMetadataClient::MetadataStatus metadata_status;
+  RestMetadataClient rest_metadata_client("127.0.0.1", monitoring_port,
+                                          kRestApiUsername, kRestApiPassword);
+
+  ASSERT_NO_ERROR(rest_metadata_client.wait_for_cache_ready(
+      std::chrono::milliseconds(wait_for_cache_ready_timeout), metadata_status))
+      << router.get_full_logfile();
 
   if (!test_params.round_robin) {
     // check if the server nodes are being used in the expected order
@@ -339,7 +377,7 @@ TEST_P(RouterRoutingStrategyMetadataCache, MetadataCacheRoutingStrategy) {
       if (i == 0) {  // first-connection
         const auto &real_port_iter =
             std::find(cluster_nodes_ports.begin(), cluster_nodes_ports.end(),
-                      (unsigned)std::atoi(node_port.c_str()));
+                      static_cast<uint16_t>(std::atoi(node_port.c_str())));
         ASSERT_NE(real_port_iter, cluster_nodes_ports.end());
         auto port_id_ = real_port_iter - std::begin(cluster_nodes_ports);
 
@@ -411,30 +449,27 @@ INSTANTIATE_TEST_CASE_P(
 class RouterRoutingStrategyTestRoundRobin
     : public RouterRoutingStrategyTest,
       // r. strategy, mode
-      public ::testing::TestWithParam<std::pair<std::string, std::string>> {
+      public ::testing::WithParamInterface<
+          std::pair<std::string, std::string>> {
  protected:
   virtual void SetUp() { RouterRoutingStrategyTest::SetUp(); }
 };
 
 TEST_P(RouterRoutingStrategyTestRoundRobin, StaticRoutingStrategyRoundRobin) {
-  // create and RAII-remove tmp dirs
-  const std::string temp_test_dir = get_tmp_dir();
-  std::shared_ptr<void> exit_guard1(nullptr,
-                                    [&](void *) { purge_dir(temp_test_dir); });
-  const std::string conf_dir = get_tmp_dir("conf");
-  std::shared_ptr<void> exit_guard2(nullptr,
-                                    [&](void *) { purge_dir(conf_dir); });
+  TempDirectory temp_test_dir;
+  TempDirectory conf_dir("conf");
 
-  const std::vector<unsigned> server_ports{port_pool_.get_next_available(),
+  const std::vector<uint16_t> server_ports{port_pool_.get_next_available(),
                                            port_pool_.get_next_available(),
                                            port_pool_.get_next_available()};
 
   // launch the standalone servers
-  std::vector<RouterComponentTest::CommandHandle> server_instances;
+  std::vector<ProcessWrapper *> server_instances;
   for (auto &server_port : server_ports) {
-    auto secondary_node = launch_standalone_server(
-        server_port, get_data_dir().str(), temp_test_dir);
-    server_instances.emplace_back(std::move(secondary_node));
+    auto &secondary_node =
+        launch_standalone_server(server_port, get_data_dir().str());
+    ASSERT_NO_FATAL_FAILURE(check_port_ready(secondary_node, server_port));
+    server_instances.emplace_back(&secondary_node);
   }
 
   // launch the router with the static configuration
@@ -444,7 +479,8 @@ TEST_P(RouterRoutingStrategyTestRoundRobin, StaticRoutingStrategyRoundRobin) {
   const auto mode = GetParam().second;
   const std::string routing_section = get_static_routing_section(
       router_port, server_ports, routing_strategy, mode);
-  auto router = launch_router_static(conf_dir, router_port, routing_section);
+  auto &router = launch_router_static(conf_dir.name(), routing_section);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_port));
 
   std::this_thread::sleep_for(
       std::chrono::milliseconds(wait_for_static_ready_timeout));
@@ -476,31 +512,29 @@ INSTANTIATE_TEST_CASE_P(
 class RouterRoutingStrategyTestFirstAvailable
     : public RouterRoutingStrategyTest,
       // r. strategy, mode
-      public ::testing::TestWithParam<std::pair<std::string, std::string>> {
+      public ::testing::WithParamInterface<
+          std::pair<std::string, std::string>> {
  protected:
   virtual void SetUp() { RouterRoutingStrategyTest::SetUp(); }
 };
 
 TEST_P(RouterRoutingStrategyTestFirstAvailable,
        StaticRoutingStrategyFirstAvailable) {
-  // create and RAII-remove tmp dirs
-  const std::string temp_test_dir = get_tmp_dir();
-  std::shared_ptr<void> exit_guard1(nullptr,
-                                    [&](void *) { purge_dir(temp_test_dir); });
-  const std::string conf_dir = get_tmp_dir("conf");
-  std::shared_ptr<void> exit_guard2(nullptr,
-                                    [&](void *) { purge_dir(conf_dir); });
+  TempDirectory temp_test_dir;
+  TempDirectory conf_dir("conf");
 
-  const std::vector<unsigned> server_ports{port_pool_.get_next_available(),
+  const std::vector<uint16_t> server_ports{port_pool_.get_next_available(),
                                            port_pool_.get_next_available(),
                                            port_pool_.get_next_available()};
 
   // launch the standalone servers
-  std::vector<RouterComponentTest::CommandHandle> server_instances;
+  std::vector<ProcessWrapper *> server_instances;
   for (auto &server_port : server_ports) {
-    auto secondary_node = launch_standalone_server(
-        server_port, get_data_dir().str(), temp_test_dir);
-    server_instances.emplace_back(std::move(secondary_node));
+    auto &secondary_node =
+        launch_standalone_server(server_port, get_data_dir().str());
+    ASSERT_NO_FATAL_FAILURE(check_port_ready(secondary_node, server_port));
+
+    server_instances.emplace_back(&secondary_node);
   }
 
   // launch the router with the static configuration
@@ -510,7 +544,8 @@ TEST_P(RouterRoutingStrategyTestFirstAvailable,
   const auto mode = GetParam().second;
   const std::string routing_section = get_static_routing_section(
       router_port, server_ports, routing_strategy, mode);
-  auto router = launch_router_static(conf_dir, router_port, routing_section);
+  auto &router = launch_router_static(conf_dir.name(), routing_section);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_port));
 
   std::this_thread::sleep_for(std::chrono::milliseconds(100));
 
@@ -535,8 +570,10 @@ TEST_P(RouterRoutingStrategyTestFirstAvailable,
   EXPECT_EQ("", node_port);
 
   // bring back 1st server
-  server_instances.emplace_back(launch_standalone_server(
-      server_ports[0], get_data_dir().str(), temp_test_dir));
+  server_instances.emplace_back(
+      &launch_standalone_server(server_ports[0], get_data_dir().str()));
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(
+      *server_instances[server_instances.size() - 1], server_ports[0]));
   // we should now succesfully connect to this server
   connect_client_and_query_port(router_port, node_port);
   EXPECT_EQ(std::to_string(server_ports[0]), node_port);
@@ -556,38 +593,34 @@ INSTANTIATE_TEST_CASE_P(
         std::make_pair(std::string(""), std::string("read-write"))));
 
 // for non-param tests
-class RouterRoutingStrategyStatic : public RouterRoutingStrategyTest,
-                                    public ::testing::Test {
+class RouterRoutingStrategyStatic : public RouterRoutingStrategyTest {
  protected:
   virtual void SetUp() { RouterRoutingStrategyTest::SetUp(); }
 };
 
 TEST_F(RouterRoutingStrategyStatic, StaticRoutingStrategyNextAvailable) {
-  // create and RAII-remove tmp dirs
-  const std::string temp_test_dir = get_tmp_dir();
-  std::shared_ptr<void> exit_guard1(nullptr,
-                                    [&](void *) { purge_dir(temp_test_dir); });
-  const std::string conf_dir = get_tmp_dir("conf");
-  std::shared_ptr<void> exit_guard2(nullptr,
-                                    [&](void *) { purge_dir(conf_dir); });
+  TempDirectory temp_test_dir;
+  TempDirectory conf_dir("conf");
 
-  const std::vector<unsigned> server_ports{port_pool_.get_next_available(),
+  const std::vector<uint16_t> server_ports{port_pool_.get_next_available(),
                                            port_pool_.get_next_available(),
                                            port_pool_.get_next_available()};
 
   // launch the standalone servers
-  std::vector<RouterComponentTest::CommandHandle> server_instances;
+  std::vector<ProcessWrapper *> server_instances;
   for (auto &server_port : server_ports) {
-    auto secondary_node = launch_standalone_server(
-        server_port, get_data_dir().str(), temp_test_dir);
-    server_instances.emplace_back(std::move(secondary_node));
+    auto &secondary_node =
+        launch_standalone_server(server_port, get_data_dir().str());
+    ASSERT_NO_FATAL_FAILURE(check_port_ready(secondary_node, server_port));
+    server_instances.emplace_back(&secondary_node);
   }
 
   // launch the router with the static configuration
   const auto router_port = port_pool_.get_next_available();
   const std::string routing_section =
       get_static_routing_section(router_port, server_ports, "next-available");
-  auto router = launch_router_static(conf_dir, router_port, routing_section);
+  auto &router = launch_router_static(conf_dir.name(), routing_section);
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(router, router_port));
 
   std::this_thread::sleep_for(std::chrono::milliseconds(100));
 
@@ -612,8 +645,10 @@ TEST_F(RouterRoutingStrategyStatic, StaticRoutingStrategyNextAvailable) {
   EXPECT_EQ("", node_port);
 
   // bring back 1st server
-  server_instances.emplace_back(launch_standalone_server(
-      server_ports[0], get_data_dir().str(), temp_test_dir));
+  server_instances.emplace_back(
+      &launch_standalone_server(server_ports[0], get_data_dir().str()));
+  ASSERT_NO_FATAL_FAILURE(check_port_ready(
+      *server_instances[server_instances.size() - 1], server_ports[0]));
   // we should NOT connect to this server (in next-available we NEVER go back)
   connect_client_and_query_port(router_port, node_port, /*should_fail=*/true);
   EXPECT_EQ("", node_port);
@@ -622,124 +657,96 @@ TEST_F(RouterRoutingStrategyStatic, StaticRoutingStrategyNextAvailable) {
 // configuration error scenarios
 
 TEST_F(RouterRoutingStrategyStatic, InvalidStrategyName) {
-  // create and RAII-remove tmp dirs
-  const std::string temp_test_dir = get_tmp_dir();
-  std::shared_ptr<void> exit_guard1(nullptr,
-                                    [&](void *) { purge_dir(temp_test_dir); });
-  const std::string conf_dir = get_tmp_dir("conf");
-  std::shared_ptr<void> exit_guard2(nullptr,
-                                    [&](void *) { purge_dir(conf_dir); });
+  TempDirectory temp_test_dir;
+  TempDirectory conf_dir("conf");
 
   // launch the router with the static configuration
   const auto router_port = port_pool_.get_next_available();
   const std::string routing_section = get_static_routing_section_error(
       router_port, {1, 2}, "round-robin-with-fallback", "read-only");
-  auto router = launch_router_static(conf_dir, router_port, routing_section,
-                                     /*expect_error=*/true);
+  auto &router = launch_router_static(conf_dir.name(), routing_section,
+                                      /*expect_error=*/true);
 
-  EXPECT_EQ(router.wait_for_exit(wait_for_process_exit_timeout), 1);
+  check_exit_code(router, EXIT_FAILURE);
   EXPECT_TRUE(
       router.expect_output("Configuration error: option routing_strategy in "
                            "[routing:test_default] is invalid; "
                            "valid are first-available, next-available, and "
                            "round-robin (was 'round-robin-with-fallback'"))
-      << get_router_log_output();
+      << router.get_full_logfile();
 }
 
 TEST_F(RouterRoutingStrategyStatic, InvalidMode) {
-  const std::string temp_test_dir = get_tmp_dir();
-  std::shared_ptr<void> exit_guard1(nullptr,
-                                    [&](void *) { purge_dir(temp_test_dir); });
-  const std::string conf_dir = get_tmp_dir("conf");
-  std::shared_ptr<void> exit_guard2(nullptr,
-                                    [&](void *) { purge_dir(conf_dir); });
+  TempDirectory conf_dir("conf");
 
   // launch the router with the static configuration
   const auto router_port = port_pool_.get_next_available();
   const std::string routing_section = get_static_routing_section_error(
       router_port, {1, 2}, "invalid", "read-only");
-  auto router = launch_router_static(conf_dir, router_port, routing_section,
-                                     /*expect_error=*/true);
+  auto &router = launch_router_static(conf_dir.name(), routing_section,
+                                      /*expect_error=*/true);
 
-  EXPECT_EQ(router.wait_for_exit(wait_for_process_exit_timeout), 1);
+  check_exit_code(router, EXIT_FAILURE);
   EXPECT_TRUE(router.expect_output(
       "option routing_strategy in [routing:test_default] is invalid; valid are "
       "first-available, next-available, and round-robin (was 'invalid')"))
-      << get_router_log_output();
+      << router.get_full_logfile();
 }
 
 TEST_F(RouterRoutingStrategyStatic, BothStrategyAndModeMissing) {
-  // create and RAII-remove tmp dirs
-  const std::string temp_test_dir = get_tmp_dir();
-  std::shared_ptr<void> exit_guard1(nullptr,
-                                    [&](void *) { purge_dir(temp_test_dir); });
-  const std::string conf_dir = get_tmp_dir("conf");
-  std::shared_ptr<void> exit_guard2(nullptr,
-                                    [&](void *) { purge_dir(conf_dir); });
+  TempDirectory conf_dir("conf");
 
   // launch the router with the static configuration
   const auto router_port = port_pool_.get_next_available();
   const std::string routing_section =
       get_static_routing_section(router_port, {1, 2}, "");
-  auto router = launch_router_static(conf_dir, router_port, routing_section,
-                                     /*expect_error=*/true);
+  auto &router = launch_router_static(conf_dir.name(), routing_section,
+                                      /*expect_error=*/true);
 
-  EXPECT_EQ(router.wait_for_exit(wait_for_process_exit_timeout), 1);
+  check_exit_code(router, EXIT_FAILURE);
   EXPECT_TRUE(
       router.expect_output("Configuration error: option routing_strategy in "
                            "[routing:test_default] is required"))
-      << get_router_log_output();
+      << router.get_full_logfile();
 }
 
 TEST_F(RouterRoutingStrategyStatic, RoutingSrtategyEmptyValue) {
-  // create and RAII-remove tmp dirs
-  const std::string temp_test_dir = get_tmp_dir();
-  std::shared_ptr<void> exit_guard1(nullptr,
-                                    [&](void *) { purge_dir(temp_test_dir); });
-  const std::string conf_dir = get_tmp_dir("conf");
-  std::shared_ptr<void> exit_guard2(nullptr,
-                                    [&](void *) { purge_dir(conf_dir); });
+  TempDirectory conf_dir("conf");
 
   // launch the router with the static configuration
   const auto router_port = port_pool_.get_next_available();
   const std::string routing_section =
       get_static_routing_section_error(router_port, {1, 2}, "", "read-only");
-  auto router = launch_router_static(conf_dir, router_port, routing_section,
-                                     /*expect_error=*/true);
+  auto &router = launch_router_static(conf_dir.name(), routing_section,
+                                      /*expect_error=*/true);
 
-  EXPECT_EQ(router.wait_for_exit(wait_for_process_exit_timeout), 1);
+  check_exit_code(router, EXIT_FAILURE);
   EXPECT_TRUE(
       router.expect_output("Configuration error: option routing_strategy in "
                            "[routing:test_default] needs a value"))
-      << get_router_log_output();
+      << router.get_full_logfile();
 }
 
 TEST_F(RouterRoutingStrategyStatic, ModeEmptyValue) {
-  // create and RAII-remove tmp dirs
-  const std::string temp_test_dir = get_tmp_dir();
-  std::shared_ptr<void> exit_guard1(nullptr,
-                                    [&](void *) { purge_dir(temp_test_dir); });
-  const std::string conf_dir = get_tmp_dir("conf");
-  std::shared_ptr<void> exit_guard2(nullptr,
-                                    [&](void *) { purge_dir(conf_dir); });
+  TempDirectory conf_dir("conf");
 
   // launch the router with the static configuration
   const auto router_port = port_pool_.get_next_available();
   const std::string routing_section = get_static_routing_section_error(
       router_port, {1, 2}, "first-available", "");
-  auto router = launch_router_static(conf_dir, router_port, routing_section,
-                                     /*expect_error=*/true);
+  auto &router = launch_router_static(conf_dir.name(), routing_section,
+                                      /*expect_error=*/true);
 
-  EXPECT_EQ(router.wait_for_exit(wait_for_process_exit_timeout), 1);
+  check_exit_code(router, EXIT_FAILURE);
   EXPECT_TRUE(
       router.expect_output("Configuration error: option mode in "
                            "[routing:test_default] needs a value"))
-      << get_router_log_output();
+      << router.get_full_logfile();
 }
 
 int main(int argc, char *argv[]) {
   init_windows_sockets();
-  g_origin_path = Path(argv[0]).dirname();
+  ProcessManager::set_origin(Path(argv[0]).dirname());
   ::testing::InitGoogleTest(&argc, argv);
   return RUN_ALL_TESTS();
 }
