@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2019, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -25,10 +25,6 @@
 #ifndef MYSQLROUTER_TESTS_FILESYSTEM_UTILS_INCLUDED
 #define MYSQLROUTER_TESTS_FILESYSTEM_UTILS_INCLUDED
 
-#include <string>
-
-#include "mysql/harness/filesystem.h"
-
 void check_config_file_access_rights(const std::string &file_name,
                                      const bool read_only);
 
