@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -31,8 +31,6 @@
 #include <stdexcept>
 #include <typeinfo>
 
-#include "mysql/harness/stdx/attribute.h"
-
 #define SKIP_GIT_TESTS(COND)                                       \
   if (COND) {                                                      \
     std::cout << "[  SKIPPED ] Tests using Git repository skipped" \
@@ -174,48 +172,10 @@ void init_windows_sockets();
  *
  * @returns true if the selected port accepts connections, false otherwise
  */
-STDX_NODISCARD
 bool wait_for_port_ready(
     uint16_t port, std::chrono::milliseconds timeout = kDefaultPortReadyTimeout,
     const std::string &hostname = "127.0.0.1");
 
-/** @brief Check if a given port is open / not used by any application.
- *
- * @param port TCP port that will be checked
- *
- * @returns true if the selected port is available, false otherwise
- */
-STDX_NODISCARD
-bool is_port_available(const uint16_t port);
-
-/**
- * Wait until the port is not available (is used by any application).
- *
- * @param port      TCP port number to check
- * @param timeout   maximum timeout to wait for the port
- *
- * @return false if the port is still available after the timeout expiry,
- *         true otherwise.
- */
-STDX_NODISCARD
-bool wait_for_port_not_available(
-    const uint16_t port,
-    std::chrono::milliseconds timeout = std::chrono::seconds(10));
-
-/**
- * Wait until the port is available (is not used by any application).
- *
- * @param port      TCP port number to check
- * @param timeout   maximum timeout to wait for the port
- *
- * @return false if the port is still not available after the timeout expiry,
- *         true otherwise.
- */
-STDX_NODISCARD
-bool wait_for_port_available(
-    const uint16_t port,
-    std::chrono::milliseconds timeout = std::chrono::seconds(10));
-
 /** @brief Initializes keyring and adds keyring-related config items to
  * [DEFAULT] section
  *
@@ -245,41 +205,18 @@ bool find_in_file(
  *
  * @param file_name name of the file
  * @param file_path path to the file
- * @param throw_on_error if true, throws std::runtime_error on failure
- *                       if false, returns error message instead of actual file
- * contents
  */
 std::string get_file_output(const std::string &file_name,
-                            const std::string &file_path,
-                            bool throw_on_error = false);
+                            const std::string &file_path);
 
 /** @brief returns the content of selected file as a string
  *
  * @param file_name full path and name of the file
- * @param throw_on_error if true, throws std::runtime_error on failure
- *                       if false, returns error message instead of actual file
- * contents
  */
-std::string get_file_output(const std::string &file_name,
-                            bool throw_on_error = false);
+std::string get_file_output(const std::string &file_name);
 
 // need to return void to be able to use ASSERT_ macros
 void connect_client_and_query_port(unsigned router_port, std::string &out_port,
                                    bool should_fail = false);
 
-/**
- * Add a "<key>=<value>" line in a configuration file in a given config section.
- *
- * @param config_path configuration file path
- * @param section_name configuration section name
- * @param key part of configuration to be added
- * @param value part of configuration to be added
- *
- * @retval true config line inserted successfully
- * @retval false config line not inserted
- */
-bool add_line_to_config_file(const std::string &config_path,
-                             const std::string &section_name,
-                             const std::string &key, const std::string &value);
-
 #endif  // ROUTER_TESTS_TEST_HELPERS_INCLUDED
