@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+  Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,14 +23,12 @@
 */
 
 #include "mysql/harness/filesystem.h"
-#include "mysql/harness/net_ts/io_context.h"
 
 #ifndef _WIN32
 #include <netdb.h>
 #include <netinet/in.h>
 #include <sys/file.h>
 #include <sys/socket.h>
-#include <sys/stat.h>  // chmod
 #include <sys/un.h>
 #include <unistd.h>
 #else
@@ -41,26 +38,23 @@
 #endif
 
 #include <fcntl.h>
-#include <cstring>
+#include <string.h>
 #include <stdexcept>
 
-#include "mysql/harness/net_ts/internet.h"
 #include "mysqlrouter/utils.h"
-#include "router_test_helpers.h"
+#include "socket_operations.h"
 #include "tcp_port_pool.h"
 
 using mysql_harness::Path;
+using mysqlrouter::get_socket_errno;
+
+const unsigned TcpPortPool::kPortsRange;
 
 #ifndef _WIN32
 bool UniqueId::lock_file(const std::string &file_name) {
   lock_file_fd_ = open(file_name.c_str(), O_RDWR | O_CREAT, 0666);
 
   if (lock_file_fd_ >= 0) {
-    // don't pass the lock-fd to the child processes
-    int flag = fcntl(lock_file_fd_, F_GETFD);
-    flag |= FD_CLOEXEC;
-    fcntl(lock_file_fd_, F_SETFD, &flag);
-
     // open() honours umask and we want to make sure this directory is
     // accessible for every user regardless of umask settings
     ::chmod(file_name.c_str(), 0666);
@@ -79,7 +73,6 @@ bool UniqueId::lock_file(const std::string &file_name) {
     if (lock) {
       // no lock so no luck, try the next one
       close(lock_file_fd_);
-      lock_file_fd_ = -1;
       return false;
     }
 
@@ -213,7 +206,73 @@ UniqueId::UniqueId(UniqueId &&other) {
   other.lock_file_name_ = "";
 }
 
-uint16_t TcpPortPool::get_next_available() {
+/*
+ * Check whether we can connect on a given port.
+ * It returns false if the connect returns any error (ECONNREFUSED, ENETUNREACH,
+ * EACCESS etc.)
+ * */
+static bool try_to_connect(uint16_t port,
+                           const std::chrono::milliseconds socket_probe_timeout,
+                           const std::string &hostname = "127.0.0.1") {
+  struct addrinfo hints, *ainfo;
+  memset(&hints, 0, sizeof hints);
+  hints.ai_family = AF_UNSPEC;
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_flags = AI_PASSIVE;
+
+  auto socket_ops = mysql_harness::SocketOperations::instance();
+
+  int status = getaddrinfo(hostname.c_str(), std::to_string(port).c_str(),
+                           &hints, &ainfo);
+  if (status != 0) {
+    throw std::runtime_error(
+        std::string("try_to_connect(): getaddrinfo() failed: ") +
+        gai_strerror(status));
+  }
+  std::shared_ptr<void> exit_freeaddrinfo(nullptr,
+                                          [&](void *) { freeaddrinfo(ainfo); });
+
+  auto sock_id =
+      socket(ainfo->ai_family, ainfo->ai_socktype, ainfo->ai_protocol);
+  if (sock_id < 0) {
+    throw std::runtime_error("try_to_connect(): socket() failed: " +
+                             std::to_string(get_socket_errno()));
+  }
+  std::shared_ptr<void> exit_close_socket(
+      nullptr, [&](void *) { socket_ops->close(sock_id); });
+
+  socket_ops->set_socket_blocking(sock_id, false);
+  status = connect(sock_id, ainfo->ai_addr, ainfo->ai_addrlen);
+  if (status >= 0) {
+    return true;
+  }
+
+  switch (socket_ops->get_errno()) {
+#ifdef _WIN32
+    case WSAEINPROGRESS:
+    case WSAEWOULDBLOCK:
+#else
+    case EINPROGRESS:
+#endif
+      if (0 != socket_ops->connect_non_blocking_wait(
+                   sock_id, std::chrono::milliseconds(socket_probe_timeout))) {
+        return false;
+      }
+
+      {
+        int so_error = 0;
+        return (0 ==
+                socket_ops->connect_non_blocking_status(sock_id, so_error));
+      }
+    default:;
+      // fallback
+  }
+
+  return false;
+}
+
+uint16_t TcpPortPool::get_next_available(
+    const std::chrono::milliseconds socket_probe_timeout) {
   while (true) {
     if (number_of_ids_used_ % kPortsPerFile == 0) {
       number_of_ids_used_ = 0;
@@ -230,6 +289,12 @@ uint16_t TcpPortPool::get_next_available() {
     unsigned result = 10000 + unique_ids_.back().get() * kPortsPerFile +
                       number_of_ids_used_++;
 
-    if (is_port_bindable(result)) return result;
+    // there is no lock file for a given port but let's also check if there
+    // really is nothing that will accept our connection attempt on that port
+    if (!try_to_connect(result, socket_probe_timeout, "127.0.0.1"))
+      return result;
+
+    std::cerr << "get_next_available(): port " << result
+              << " seems busy, not using\n";
   }
 }
