@@ -1,4 +1,4 @@
---  Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+--  Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 -- 
 --  This program is free software; you can redistribute it and/or modify
 --  it under the terms of the GNU General Public License as published by
@@ -48,6 +48,10 @@ Some of the sys schema views are calculated as initial (optional), overall, delt
 
 Requires the SUPER privilege for "SET sql_log_bin = 0;".
 
+Versions supported:
+   * MySQL 5.6: 5.6.10 and later
+   * MySQL 5.7: 5.7.9 and later
+
 Parameters
 -----------
 
@@ -108,7 +112,7 @@ mysql> NOTEE;
 BEGIN
     DECLARE v_start, v_runtime, v_iter_start, v_sleep DECIMAL(20,2) DEFAULT 0.0;
     DECLARE v_has_innodb, v_has_ndb, v_has_ps, v_has_replication, v_has_ps_replication VARCHAR(8) CHARSET utf8mb4 DEFAULT 'NO';
-    DECLARE v_this_thread_enabled  ENUM('YES', 'NO');
+    DECLARE v_this_thread_enabled, v_has_metrics ENUM('YES', 'NO');
     DECLARE v_table_name, v_banner VARCHAR(64) CHARSET utf8mb4;
     DECLARE v_sql_status_summary_select, v_sql_status_summary_delta, v_sql_status_summary_from, v_no_delta_names TEXT;
     DECLARE v_output_time, v_output_time_prev DECIMAL(20,3) UNSIGNED;
@@ -181,7 +185,7 @@ BEGIN
 
     -- Temporary table are used - disable sql_log_bin if necessary to prevent them replicating
     SET @log_bin := @@sql_log_bin;
-    IF ((@log_bin = 1) AND (@@binlog_format = 'STATEMENT')) THEN
+    IF (@log_bin = 1) THEN
         SET sql_log_bin = 0;
     END IF;
 
@@ -299,15 +303,21 @@ BEGIN
     SET v_has_innodb         = IFNULL((SELECT SUPPORT FROM information_schema.ENGINES WHERE ENGINE = 'InnoDB'), 'NO'),
         v_has_ndb            = IFNULL((SELECT SUPPORT FROM information_schema.ENGINES WHERE ENGINE = 'NDBCluster'), 'NO'),
         v_has_ps             = IFNULL((SELECT SUPPORT FROM information_schema.ENGINES WHERE ENGINE = 'PERFORMANCE_SCHEMA'), 'NO'),
-        v_has_ps_replication = v_has_ps,
-        v_has_replication    = IF(v_has_ps_replication = 'YES', IF((SELECT COUNT(*) FROM performance_schema.replication_connection_status) > 0, 'YES', 'NO'),
+        v_has_ps_replication = IF(v_has_ps = 'YES'
+                                   AND EXISTS(SELECT 1 FROM information_schema.TABLES WHERE TABLE_SCHEMA = 'performance_schema' AND TABLE_NAME = 'replication_applier_status'),
+                                   'YES',
+                                   'NO'
+                               ),
+        v_has_replication    = /*!50707 IF(v_has_ps_replication = 'YES', IF((SELECT COUNT(*) FROM performance_schema.replication_connection_status) > 0, 'YES', 'NO'),*/
                                   IF(@@master_info_repository = 'TABLE', IF((SELECT COUNT(*) FROM mysql.slave_master_info) > 0, 'YES', 'NO'),
                                      IF(@@relay_log_info_repository = 'TABLE', IF((SELECT COUNT(*) FROM mysql.slave_relay_log_info) > 0, 'YES', 'NO'),
-                                        'MAYBE')));
+                                        'MAYBE'))/*!50707 )*/,
+        v_has_metrics        = IF(v_has_ps = 'YES' OR (sys.version_major() = 5 AND sys.version_minor() = 6), 'YES', 'NO');
 
     IF (@sys.debug = 'ON') THEN
        SELECT v_has_innodb AS 'Has_InnoDB', v_has_ndb AS 'Has_NDBCluster',
               v_has_ps AS 'Has_Performance_Schema',
+              v_has_metrics AS 'Has_metrics',
               v_has_ps_replication 'AS Has_P_S_Replication', v_has_replication AS 'Has_Replication';
     END IF;
 
@@ -339,12 +349,12 @@ BEGIN
                                 (''io_global_by_file_by_bytes'', ''total''),
                                 (''io_global_by_wait_by_bytes'', ''total_requested'')
                          )
-                         THEN CONCAT(''format_bytes('', COLUMN_NAME, '') AS '', COLUMN_NAME)
+                         THEN CONCAT(''sys.format_bytes('', COLUMN_NAME, '') AS '', COLUMN_NAME)
                     WHEN COLUMN_NAME LIKE ''%latency''
-                         THEN CONCAT(''format_pico_time('', COLUMN_NAME, '') AS '', COLUMN_NAME)
+                         THEN CONCAT(''sys.format_time('', COLUMN_NAME, '') AS '', COLUMN_NAME)
                     WHEN SUBSTRING(COLUMN_NAME, -7) = ''_memory'' OR SUBSTRING(COLUMN_NAME, -17) = ''_memory_allocated''
                          OR ((SUBSTRING(COLUMN_NAME, -5) = ''_read'' OR SUBSTRING(COLUMN_NAME, -8) = ''_written'' OR SUBSTRING(COLUMN_NAME, -6) = ''_write'') AND SUBSTRING(COLUMN_NAME, 1, 6) <> ''COUNT_'')
-                         THEN CONCAT(''format_bytes('', COLUMN_NAME, '') AS '', COLUMN_NAME)
+                         THEN CONCAT(''sys.format_bytes('', COLUMN_NAME, '') AS '', COLUMN_NAME)
                     ELSE COLUMN_NAME
                END
                ORDER BY ORDINAL_POSITION
@@ -359,9 +369,9 @@ BEGIN
     SET @sys.diagnostics.sql_gen_query_delta = 'SELECT CONCAT(
            ''SELECT '',
            GROUP_CONCAT(
-               CASE WHEN FIND_IN_SET(COLUMN_NAME COLLATE utf8mb3_general_ci, diag.pk)
+               CASE WHEN FIND_IN_SET(COLUMN_NAME COLLATE utf8_general_ci, diag.pk)
                          THEN COLUMN_NAME
-                    WHEN diag.TABLE_NAME = ''io_global_by_file_by_bytes'' AND COLUMN_NAME COLLATE utf8mb3_general_ci = ''write_pct''
+                    WHEN diag.TABLE_NAME = ''io_global_by_file_by_bytes'' AND COLUMN_NAME COLLATE utf8_general_ci = ''write_pct''
                          THEN CONCAT(''IFNULL(ROUND(100-(((e.total_read-IFNULL(s.total_read, 0))'',
                                      ''/NULLIF(((e.total_read-IFNULL(s.total_read, 0))+(e.total_written-IFNULL(s.total_written, 0))), 0))*100), 2), 0.00) AS '',
                                      COLUMN_NAME)
@@ -369,24 +379,24 @@ BEGIN
                                 (''io_global_by_file_by_bytes'', ''total''),
                                 (''io_global_by_wait_by_bytes'', ''total_requested'')
                          )
-                         THEN CONCAT(''format_bytes(e.'', COLUMN_NAME, ''-IFNULL(s.'', COLUMN_NAME, '', 0)) AS '', COLUMN_NAME)
+                         THEN CONCAT(''sys.format_bytes(e.'', COLUMN_NAME, ''-IFNULL(s.'', COLUMN_NAME, '', 0)) AS '', COLUMN_NAME)
                     WHEN SUBSTRING(COLUMN_NAME, 1, 4) IN (''max_'', ''min_'') AND SUBSTRING(COLUMN_NAME, -8) = ''_latency''
-                         THEN CONCAT(''format_pico_time(e.'', COLUMN_NAME, '') AS '', COLUMN_NAME)
-                    WHEN COLUMN_NAME COLLATE utf8mb3_general_ci = ''avg_latency''
-                         THEN CONCAT(''format_pico_time((e.total_latency - IFNULL(s.total_latency, 0))'',
+                         THEN CONCAT(''sys.format_time(e.'', COLUMN_NAME, '') AS '', COLUMN_NAME)
+                    WHEN COLUMN_NAME COLLATE utf8_general_ci = ''avg_latency''
+                         THEN CONCAT(''sys.format_time((e.total_latency - IFNULL(s.total_latency, 0))'',
                                      ''/NULLIF(e.total - IFNULL(s.total, 0), 0)) AS '', COLUMN_NAME)
                     WHEN SUBSTRING(COLUMN_NAME, -12) = ''_avg_latency''
-                         THEN CONCAT(''format_pico_time((e.'', SUBSTRING(COLUMN_NAME FROM 1 FOR CHAR_LENGTH(COLUMN_NAME)-12), ''_latency - IFNULL(s.'', SUBSTRING(COLUMN_NAME FROM 1 FOR CHAR_LENGTH(COLUMN_NAME)-12), ''_latency, 0))'',
+                         THEN CONCAT(''sys.format_time((e.'', SUBSTRING(COLUMN_NAME FROM 1 FOR CHAR_LENGTH(COLUMN_NAME)-12), ''_latency - IFNULL(s.'', SUBSTRING(COLUMN_NAME FROM 1 FOR CHAR_LENGTH(COLUMN_NAME)-12), ''_latency, 0))'',
                                      ''/NULLIF(e.'', SUBSTRING(COLUMN_NAME FROM 1 FOR CHAR_LENGTH(COLUMN_NAME)-12), ''s - IFNULL(s.'', SUBSTRING(COLUMN_NAME FROM 1 FOR CHAR_LENGTH(COLUMN_NAME)-12), ''s, 0), 0)) AS '', COLUMN_NAME)
                     WHEN COLUMN_NAME LIKE ''%latency''
-                         THEN CONCAT(''format_pico_time(e.'', COLUMN_NAME, '' - IFNULL(s.'', COLUMN_NAME, '', 0)) AS '', COLUMN_NAME)
+                         THEN CONCAT(''sys.format_time(e.'', COLUMN_NAME, '' - IFNULL(s.'', COLUMN_NAME, '', 0)) AS '', COLUMN_NAME)
                     WHEN COLUMN_NAME IN (''avg_read'', ''avg_write'', ''avg_written'')
-                         THEN CONCAT(''format_bytes(IFNULL((e.total_'', IF(COLUMN_NAME = ''avg_read'', ''read'', ''written''), ''-IFNULL(s.total_'', IF(COLUMN_NAME = ''avg_read'', ''read'', ''written''), '', 0))'',
+                         THEN CONCAT(''sys.format_bytes(IFNULL((e.total_'', IF(COLUMN_NAME = ''avg_read'', ''read'', ''written''), ''-IFNULL(s.total_'', IF(COLUMN_NAME = ''avg_read'', ''read'', ''written''), '', 0))'',
                                      ''/NULLIF(e.count_'', IF(COLUMN_NAME = ''avg_read'', ''read'', ''write''), ''-IFNULL(s.count_'', IF(COLUMN_NAME = ''avg_read'', ''read'', ''write''), '', 0), 0), 0)) AS '',
                                      COLUMN_NAME)
                     WHEN SUBSTRING(COLUMN_NAME, -7) = ''_memory'' OR SUBSTRING(COLUMN_NAME, -17) = ''_memory_allocated''
                          OR ((SUBSTRING(COLUMN_NAME, -5) = ''_read'' OR SUBSTRING(COLUMN_NAME, -8) = ''_written'' OR SUBSTRING(COLUMN_NAME, -6) = ''_write'') AND SUBSTRING(COLUMN_NAME, 1, 6) <> ''COUNT_'')
-                         THEN CONCAT(''format_bytes(e.'', COLUMN_NAME, '' - IFNULL(s.'', COLUMN_NAME, '', 0)) AS '', COLUMN_NAME)
+                         THEN CONCAT(''sys.format_bytes(e.'', COLUMN_NAME, '' - IFNULL(s.'', COLUMN_NAME, '', 0)) AS '', COLUMN_NAME)
                     ELSE CONCAT(''(e.'', COLUMN_NAME, '' - IFNULL(s.'', COLUMN_NAME, '', 0)) AS '', COLUMN_NAME)
                END
                ORDER BY ORDINAL_POSITION
@@ -396,7 +406,7 @@ BEGIN
        LEFT OUTER JOIN tmp_'', diag.TABLE_NAME, ''_start s USING ('', diag.pk, '')''
        ) AS Query INTO @sys.diagnostics.sql_select
   FROM tmp_sys_views_delta diag
-       INNER JOIN information_schema.COLUMNS c ON c.TABLE_NAME COLLATE utf8mb3_general_ci = CONCAT(''x$'', diag.TABLE_NAME)
+       INNER JOIN information_schema.COLUMNS c ON c.TABLE_NAME COLLATE utf8_general_ci = CONCAT(''x$'', diag.TABLE_NAME)
  WHERE c.TABLE_SCHEMA = ''sys'' AND diag.TABLE_NAME = ?
  GROUP BY diag.TABLE_NAME';
 
@@ -546,7 +556,7 @@ BEGIN
 
         IF (v_has_ps_replication = 'YES') THEN
             SELECT 'Replication - Connection Configuration' AS 'The following output is:';
-            SELECT * FROM performance_schema.replication_connection_configuration ORDER BY CHANNEL_NAME;
+            SELECT * FROM performance_schema.replication_connection_configuration/*!50706 ORDER BY CHANNEL_NAME*/;
         END IF;
 
         IF (v_has_ps_replication = 'YES') THEN
@@ -558,17 +568,17 @@ BEGIN
             SELECT 'Replication - Master Info Repository Configuration' AS 'The following output is:';
             -- Can't just do SELECT *  as the password may be present in plain text
             -- Don't include binary log file and position as that will be determined in each iteration as well
-            SELECT Channel_name, Host, User_name, Port, Connect_retry,
+            SELECT /*!50706 Channel_name, */Host, User_name, Port, Connect_retry,
                    Enabled_ssl, Ssl_ca, Ssl_capath, Ssl_cert, Ssl_cipher, Ssl_key, Ssl_verify_server_cert,
                    Heartbeat, Bind, Ignored_server_ids, Uuid, Retry_count, Ssl_crl, Ssl_crlpath,
                    Tls_version, Enabled_auto_position
-              FROM mysql.slave_master_info ORDER BY Channel_name;
+              FROM mysql.slave_master_info/*!50706 ORDER BY Channel_name*/;
         END IF;
 
         IF (@@relay_log_info_repository = 'TABLE') THEN
             SELECT 'Replication - Relay Log Repository Configuration' AS 'The following output is:';
-            SELECT Channel_name, Sql_delay, Number_of_workers, Id
-              FROM mysql.slave_relay_log_info ORDER BY Channel_name;
+            SELECT /*!50706 Channel_name, */Sql_delay, Number_of_workers, Id
+              FROM mysql.slave_relay_log_info/*!50706 ORDER BY Channel_name*/;
         END IF;
     END IF;
 
@@ -701,9 +711,9 @@ BEGIN
                 SELECT Id, sys.format_path(Relay_log_name) AS Relay_log_name, Relay_log_pos, Master_log_name, Master_log_pos,
                        sys.format_path(Checkpoint_relay_log_name) AS Checkpoint_relay_log_name, Checkpoint_relay_log_pos,
                        Checkpoint_master_log_name, Checkpoint_master_log_pos, Checkpoint_seqno, Checkpoint_group_size,
-                       HEX(Checkpoint_group_bitmap) AS Checkpoint_group_bitmap, Channel_name
+                       HEX(Checkpoint_group_bitmap) AS Checkpoint_group_bitmap/*!50706 , Channel_name*/
                   FROM mysql.slave_worker_info
-              ORDER BY Channel_name, Id;
+              ORDER BY /*!50706 Channel_name, */Id;
             END IF;
         END IF;
 
@@ -721,10 +731,42 @@ BEGIN
   PRIMARY KEY (Type, Variable_name)
 ) ENGINE = InnoDB DEFAULT CHARSET=utf8mb4'));
 
-        SET @sys.diagnostics.sql = CONCAT(
-            'INSERT INTO ', v_table_name,
-            ' SELECT Variable_name, REPLACE(Variable_value, ''\n'', ''\\\\n'') AS Variable_value, Type, Enabled FROM sys.metrics'
-        );
+        IF (v_has_metrics) THEN
+            SET @sys.diagnostics.sql = CONCAT(
+                'INSERT INTO ', v_table_name,
+                ' SELECT Variable_name, REPLACE(Variable_value, ''\n'', ''\\\\n'') AS Variable_value, Type, Enabled FROM sys.metrics'
+            );
+        ELSE
+            -- 5.7+ and the Performance Schema disabled. Use information_schema.GLOBAL_STATUS instead like in 5.6.
+            SET @sys.diagnostics.sql = CONCAT(
+                'INSERT INTO ', v_table_name,
+                '(SELECT LOWER(VARIABLE_NAME) AS Variable_name, REPLACE(VARIABLE_VALUE, ''\n'', ''\\\\n'') AS Variable_value,
+                         ''Global Status'' AS Type, ''YES'' AS Enabled
+  FROM performance_schema.global_status
+) UNION ALL (
+SELECT NAME AS Variable_name, COUNT AS Variable_value,
+       CONCAT(''InnoDB Metrics - '', SUBSYSTEM) AS Type,
+       IF(STATUS = ''enabled'', ''YES'', ''NO'') AS Enabled
+  FROM information_schema.INNODB_METRICS
+  -- Deduplication - some variables exists both in GLOBAL_STATUS and INNODB_METRICS
+  -- Keep the one from GLOBAL_STATUS as it is always enabled and it''s more likely to be used for existing tools.
+ WHERE NAME NOT IN (
+     ''lock_row_lock_time'', ''lock_row_lock_time_avg'', ''lock_row_lock_time_max'', ''lock_row_lock_waits'',
+     ''buffer_pool_reads'', ''buffer_pool_read_requests'', ''buffer_pool_write_requests'', ''buffer_pool_wait_free'',
+     ''buffer_pool_read_ahead'', ''buffer_pool_read_ahead_evicted'', ''buffer_pool_pages_total'', ''buffer_pool_pages_misc'',
+     ''buffer_pool_pages_data'', ''buffer_pool_bytes_data'', ''buffer_pool_pages_dirty'', ''buffer_pool_bytes_dirty'',
+     ''buffer_pool_pages_free'', ''buffer_pages_created'', ''buffer_pages_written'', ''buffer_pages_read'',
+     ''buffer_data_reads'', ''buffer_data_written'', ''file_num_open_files'',
+     ''os_log_bytes_written'', ''os_log_fsyncs'', ''os_log_pending_fsyncs'', ''os_log_pending_writes'',
+     ''log_waits'', ''log_write_requests'', ''log_writes'', ''innodb_dblwr_writes'', ''innodb_dblwr_pages_written'', ''innodb_page_size'')
+) UNION ALL (
+SELECT ''NOW()'' AS Variable_name, NOW(3) AS Variable_value, ''System Time'' AS Type, ''YES'' AS Enabled
+) UNION ALL (
+SELECT ''UNIX_TIMESTAMP()'' AS Variable_name, ROUND(UNIX_TIMESTAMP(NOW(3)), 3) AS Variable_value, ''System Time'' AS Type, ''YES'' AS Enabled
+)
+ ORDER BY Type, Variable_name;'
+            );
+        END IF;
         CALL sys.execute_prepared_stmt(@sys.diagnostics.sql);
 
         -- Prepare the query to retrieve the summary
@@ -761,7 +803,11 @@ BEGIN
         SET v_output_time_prev = v_output_time;
 
         IF (@sys.diagnostics.include_raw = 'ON') THEN
-            SELECT 'SELECT * FROM sys.metrics' AS 'The following output is:';
+            IF (v_has_metrics) THEN
+                SELECT 'SELECT * FROM sys.metrics' AS 'The following output is:';
+            ELSE
+                SELECT 'sys.metrics equivalent' AS 'The following output is:';
+            END IF;
             -- Ensures that the output here is the same as the one used in the status summary at the end
             CALL sys.execute_prepared_stmt(CONCAT('SELECT Type, Variable_name, Enabled, Variable_value FROM ', v_table_name, ' ORDER BY Type, Variable_name'));
         END IF;
@@ -780,7 +826,7 @@ BEGIN
             EXECUTE stmt_ndbcluster_status;
 
             SELECT 'ndbinfo.memoryusage' AS 'The following output is:';
-            SELECT node_id, memory_type, format_bytes(used) AS used, used_pages, format_bytes(total) AS total, total_pages,
+            SELECT node_id, memory_type, sys.format_bytes(used) AS used, used_pages, sys.format_bytes(total) AS total, total_pages,
                    ROUND(100*(used/total), 2) AS 'Used %'
             FROM ndbinfo.memoryusage;
 
@@ -865,9 +911,9 @@ BEGIN
     IF (@sys.diagnostics.allow_i_s_tables = 'ON') THEN
         SELECT 'Storage Engine Usage' AS 'The following output is:';
         SELECT ENGINE, COUNT(*) AS NUM_TABLES,
-                format_bytes(SUM(DATA_LENGTH)) AS DATA_LENGTH,
-                format_bytes(SUM(INDEX_LENGTH)) AS INDEX_LENGTH,
-                format_bytes(SUM(DATA_LENGTH+INDEX_LENGTH)) AS TOTAL
+                sys.format_bytes(SUM(DATA_LENGTH)) AS DATA_LENGTH,
+                sys.format_bytes(SUM(INDEX_LENGTH)) AS INDEX_LENGTH,
+                sys.format_bytes(SUM(DATA_LENGTH+INDEX_LENGTH)) AS TOTAL
             FROM information_schema.TABLES
             GROUP BY ENGINE;
 
@@ -1011,7 +1057,11 @@ BEGIN
         DROP TEMPORARY TABLE tmp_sys_views_delta;
     END IF;
 
-    SELECT 'SELECT * FROM sys.metrics' AS 'The following output is:';
+    IF (v_has_metrics) THEN
+        SELECT 'SELECT * FROM sys.metrics' AS 'The following output is:';
+    ELSE
+        SELECT 'sys.metrics equivalent' AS 'The following output is:';
+    END IF;
     CALL sys.execute_prepared_stmt(
         CONCAT(v_sql_status_summary_select, v_sql_status_summary_delta, ', Type, s1.Enabled', v_sql_status_summary_from,
                '
@@ -1029,9 +1079,7 @@ BEGIN
 
     IF (in_auto_config <> 'current') THEN
         CALL sys.ps_setup_reload_saved();
-        IF ((@log_bin = 1) AND (@@binlog_format = 'STATEMENT')) THEN
-            SET sql_log_bin = @log_bin;
-        END IF;
+        SET sql_log_bin = @log_bin;
     END IF;
 
     -- Reset the @sys.diagnostics.% user variables internal to this procedure
@@ -1047,7 +1095,7 @@ BEGIN
         CALL sys.ps_setup_enable_thread(CONNECTION_ID());
     END IF;
 
-    IF ((@log_bin = 1) AND (@@binlog_format = 'STATEMENT')) THEN
+    IF (@log_bin = 1) THEN
         SET sql_log_bin = @log_bin;
     END IF;
 END$$
