@@ -1,4 +1,4 @@
--- Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+-- Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 -- 
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -196,8 +196,8 @@ mysql> CREATE OR REPLACE VIEW mydb.my_statements AS
     -> SELECT sys.format_statement(DIGEST_TEXT) AS query,
     ->        SCHEMA_NAME AS db,
     ->        COUNT_STAR AS exec_count,
-    ->        format_pico_time(SUM_TIMER_WAIT) AS total_latency,
-    ->        format_pico_time(AVG_TIMER_WAIT) AS avg_latency,
+    ->        sys.format_time(SUM_TIMER_WAIT) AS total_latency,
+    ->        sys.format_time(AVG_TIMER_WAIT) AS avg_latency,
     ->        ROUND(IFNULL(SUM_ROWS_SENT / NULLIF(COUNT_STAR, 0), 0)) AS rows_sent_avg,
     ->        ROUND(IFNULL(SUM_ROWS_EXAMINED / NULLIF(COUNT_STAR, 0), 0)) AS rows_examined_avg,
     ->        ROUND(IFNULL(SUM_ROWS_AFFECTED / NULLIF(COUNT_STAR, 0), 0)) AS rows_affected_avg,
@@ -254,7 +254,7 @@ BEGIN
 
     -- Temporary table are used - disable sql_log_bin if necessary to prevent them replicating
     SET @log_bin := @@sql_log_bin;
-    IF ((@log_bin = 1) AND (@@binlog_format = 'STATEMENT')) THEN
+    IF (@log_bin = 1) THEN
         SET sql_log_bin = 0;
     END IF;
 
@@ -404,42 +404,38 @@ BEGIN
   `SCHEMA_NAME` varchar(64) DEFAULT NULL,
   `DIGEST` varchar(64) DEFAULT NULL,
   `DIGEST_TEXT` longtext,
-  `COUNT_STAR` bigint unsigned NOT NULL,
-  `SUM_TIMER_WAIT` bigint unsigned NOT NULL,
-  `MIN_TIMER_WAIT` bigint unsigned NOT NULL,
-  `AVG_TIMER_WAIT` bigint unsigned NOT NULL,
-  `MAX_TIMER_WAIT` bigint unsigned NOT NULL,
-  `SUM_LOCK_TIME` bigint unsigned NOT NULL,
-  `SUM_ERRORS` bigint unsigned NOT NULL,
-  `SUM_WARNINGS` bigint unsigned NOT NULL,
-  `SUM_ROWS_AFFECTED` bigint unsigned NOT NULL,
-  `SUM_ROWS_SENT` bigint unsigned NOT NULL,
-  `SUM_ROWS_EXAMINED` bigint unsigned NOT NULL,
-  `SUM_CREATED_TMP_DISK_TABLES` bigint unsigned NOT NULL,
-  `SUM_CREATED_TMP_TABLES` bigint unsigned NOT NULL,
-  `SUM_SELECT_FULL_JOIN` bigint unsigned NOT NULL,
-  `SUM_SELECT_FULL_RANGE_JOIN` bigint unsigned NOT NULL,
-  `SUM_SELECT_RANGE` bigint unsigned NOT NULL,
-  `SUM_SELECT_RANGE_CHECK` bigint unsigned NOT NULL,
-  `SUM_SELECT_SCAN` bigint unsigned NOT NULL,
-  `SUM_SORT_MERGE_PASSES` bigint unsigned NOT NULL,
-  `SUM_SORT_RANGE` bigint unsigned NOT NULL,
-  `SUM_SORT_ROWS` bigint unsigned NOT NULL,
-  `SUM_SORT_SCAN` bigint unsigned NOT NULL,
-  `SUM_NO_INDEX_USED` bigint unsigned NOT NULL,
-  `SUM_NO_GOOD_INDEX_USED` bigint unsigned NOT NULL,
-  `SUM_CPU_TIME` bigint unsigned NOT NULL,
-  `MAX_CONTROLLED_MEMORY` bigint unsigned NOT NULL,
-  `MAX_TOTAL_MEMORY` bigint unsigned NOT NULL,
-  `COUNT_SECONDARY` bigint unsigned NOT NULL,
+  `COUNT_STAR` bigint(20) unsigned NOT NULL,
+  `SUM_TIMER_WAIT` bigint(20) unsigned NOT NULL,
+  `MIN_TIMER_WAIT` bigint(20) unsigned NOT NULL,
+  `AVG_TIMER_WAIT` bigint(20) unsigned NOT NULL,
+  `MAX_TIMER_WAIT` bigint(20) unsigned NOT NULL,
+  `SUM_LOCK_TIME` bigint(20) unsigned NOT NULL,
+  `SUM_ERRORS` bigint(20) unsigned NOT NULL,
+  `SUM_WARNINGS` bigint(20) unsigned NOT NULL,
+  `SUM_ROWS_AFFECTED` bigint(20) unsigned NOT NULL,
+  `SUM_ROWS_SENT` bigint(20) unsigned NOT NULL,
+  `SUM_ROWS_EXAMINED` bigint(20) unsigned NOT NULL,
+  `SUM_CREATED_TMP_DISK_TABLES` bigint(20) unsigned NOT NULL,
+  `SUM_CREATED_TMP_TABLES` bigint(20) unsigned NOT NULL,
+  `SUM_SELECT_FULL_JOIN` bigint(20) unsigned NOT NULL,
+  `SUM_SELECT_FULL_RANGE_JOIN` bigint(20) unsigned NOT NULL,
+  `SUM_SELECT_RANGE` bigint(20) unsigned NOT NULL,
+  `SUM_SELECT_RANGE_CHECK` bigint(20) unsigned NOT NULL,
+  `SUM_SELECT_SCAN` bigint(20) unsigned NOT NULL,
+  `SUM_SORT_MERGE_PASSES` bigint(20) unsigned NOT NULL,
+  `SUM_SORT_RANGE` bigint(20) unsigned NOT NULL,
+  `SUM_SORT_ROWS` bigint(20) unsigned NOT NULL,
+  `SUM_SORT_SCAN` bigint(20) unsigned NOT NULL,
+  `SUM_NO_INDEX_USED` bigint(20) unsigned NOT NULL,
+  `SUM_NO_GOOD_INDEX_USED` bigint(20) unsigned NOT NULL,
   `FIRST_SEEN` timestamp(6) NULL DEFAULT NULL,
   `LAST_SEEN` timestamp(6) NULL DEFAULT NULL,
-  `QUANTILE_95` bigint unsigned NOT NULL,
-  `QUANTILE_99` bigint unsigned NOT NULL,
-  `QUANTILE_999` bigint unsigned NOT NULL,
+  `QUANTILE_95` bigint(20) unsigned NOT NULL,
+  `QUANTILE_99` bigint(20) unsigned NOT NULL,
+  `QUANTILE_999` bigint(20) unsigned NOT NULL,
   `QUERY_SAMPLE_TEXT` longtext,
   `QUERY_SAMPLE_SEEN` timestamp(6) NULL DEFAULT NULL,
-  `QUERY_SAMPLE_TIMER_WAIT` bigint unsigned NOT NULL,
+  `QUERY_SAMPLE_TIMER_WAIT` bigint(20) unsigned NOT NULL,
   INDEX (SCHEMA_NAME, DIGEST)
 ) DEFAULT CHARSET=utf8mb4';
 
@@ -517,10 +513,6 @@ SELECT `d_end`.`SCHEMA_NAME`,
        `d_end`.`SUM_SORT_SCAN`-IFNULL(`d_start`.`SUM_SORT_SCAN`, 0) AS ''SUM_SORT_SCAN'',
        `d_end`.`SUM_NO_INDEX_USED`-IFNULL(`d_start`.`SUM_NO_INDEX_USED`, 0) AS ''SUM_NO_INDEX_USED'',
        `d_end`.`SUM_NO_GOOD_INDEX_USED`-IFNULL(`d_start`.`SUM_NO_GOOD_INDEX_USED`, 0) AS ''SUM_NO_GOOD_INDEX_USED'',
-       `d_end`.`SUM_CPU_TIME`-IFNULL(`d_start`.`SUM_CPU_TIME`, 0) AS ''SUM_CPU_TIME'',
-       `d_end`.`MAX_CONTROLLED_MEMORY` AS ''MAX_CONTROLLED_MEMORY'',
-       `d_end`.`MAX_TOTAL_MEMORY` AS ''MAX_TOTAL_MEMORY'',
-       `d_end`.`COUNT_SECONDARY`-IFNULL(`d_start`.`COUNT_SECONDARY`, 0) AS ''COUNT_SECONDARY'',
        `d_end`.`FIRST_SEEN`,
        `d_end`.`LAST_SEEN`,
        `d_end`.`QUANTILE_95`,
@@ -738,7 +730,7 @@ HAVING percentile > 0.95
         CALL sys.ps_setup_enable_thread(CONNECTION_ID());
     END IF;
 
-    IF ((@log_bin = 1) AND (@@binlog_format = 'STATEMENT')) THEN
+    IF (@log_bin = 1) THEN
         SET sql_log_bin = @log_bin;
     END IF;
 END$$
