@@ -1,4 +1,4 @@
--- Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+-- Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -44,8 +44,8 @@ VIEW innodb_buffer_stats_by_schema (
   rows_cached
 ) AS
 SELECT IF(LOCATE('.', ibp.table_name) = 0, 'InnoDB System', REPLACE(SUBSTRING_INDEX(ibp.table_name, '.', 1), '`', '')) AS object_schema,
-       format_bytes(SUM(IF(ibp.compressed_size = 0, 16384, compressed_size))) AS allocated,
-       format_bytes(SUM(ibp.data_size)) AS data,
+       sys.format_bytes(SUM(IF(ibp.compressed_size = 0, 16384, compressed_size))) AS allocated,
+       sys.format_bytes(SUM(ibp.data_size)) AS data,
        COUNT(ibp.page_number) AS pages,
        COUNT(IF(ibp.is_hashed = 'YES', 1, NULL)) AS pages_hashed,
        COUNT(IF(ibp.is_old = 'YES', 1, NULL)) AS pages_old,
