@@ -1,4 +1,4 @@
--- Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+-- Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -63,18 +63,18 @@ VIEW io_global_by_wait_by_latency (
 ) AS
 SELECT SUBSTRING_INDEX(event_name, '/', -2) AS event_name,
        count_star AS total,
-       format_pico_time(sum_timer_wait) AS total_latency,
-       format_pico_time(avg_timer_wait) AS avg_latency,
-       format_pico_time(max_timer_wait) AS max_latency,
-       format_pico_time(sum_timer_read) AS read_latency,
-       format_pico_time(sum_timer_write) AS write_latency,
-       format_pico_time(sum_timer_misc) AS misc_latency,
+       sys.format_time(sum_timer_wait) AS total_latency,
+       sys.format_time(avg_timer_wait) AS avg_latency,
+       sys.format_time(max_timer_wait) AS max_latency,
+       sys.format_time(sum_timer_read) AS read_latency,
+       sys.format_time(sum_timer_write) AS write_latency,
+       sys.format_time(sum_timer_misc) AS misc_latency,
        count_read,
-       format_bytes(sum_number_of_bytes_read) AS total_read,
-       format_bytes(IFNULL(sum_number_of_bytes_read / NULLIF(count_read, 0), 0)) AS avg_read,
+       sys.format_bytes(sum_number_of_bytes_read) AS total_read,
+       sys.format_bytes(IFNULL(sum_number_of_bytes_read / NULLIF(count_read, 0), 0)) AS avg_read,
        count_write,
-       format_bytes(sum_number_of_bytes_write) AS total_written,
-       format_bytes(IFNULL(sum_number_of_bytes_write / NULLIF(count_write, 0), 0)) AS avg_written
+       sys.format_bytes(sum_number_of_bytes_write) AS total_written,
+       sys.format_bytes(IFNULL(sum_number_of_bytes_write / NULLIF(count_write, 0), 0)) AS avg_written
   FROM performance_schema.file_summary_by_event_name 
  WHERE event_name LIKE 'wait/io/file/%'
    AND count_star > 0
