@@ -1,4 +1,4 @@
--- Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+-- Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -53,13 +53,13 @@ SELECT OBJECT_SCHEMA AS table_schema,
        OBJECT_NAME AS table_name,
        INDEX_NAME as index_name,
        COUNT_FETCH AS rows_selected,
-       format_pico_time(SUM_TIMER_FETCH) AS select_latency,
+       sys.format_time(SUM_TIMER_FETCH) AS select_latency,
        COUNT_INSERT AS rows_inserted,
-       format_pico_time(SUM_TIMER_INSERT) AS insert_latency,
+       sys.format_time(SUM_TIMER_INSERT) AS insert_latency,
        COUNT_UPDATE AS rows_updated,
-       format_pico_time(SUM_TIMER_UPDATE) AS update_latency,
+       sys.format_time(SUM_TIMER_UPDATE) AS update_latency,
        COUNT_DELETE AS rows_deleted,
-       format_pico_time(SUM_TIMER_DELETE) AS delete_latency
+       sys.format_time(SUM_TIMER_DELETE) AS delete_latency
   FROM performance_schema.table_io_waits_summary_by_index_usage
  WHERE index_name IS NOT NULL
  ORDER BY sum_timer_wait DESC;
