@@ -1,4 +1,4 @@
--- Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+-- Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -70,23 +70,23 @@ VIEW schema_table_statistics (
 ) AS
 SELECT pst.object_schema AS table_schema,
        pst.object_name AS table_name,
-       format_pico_time(pst.sum_timer_wait) AS total_latency,
+       sys.format_time(pst.sum_timer_wait) AS total_latency,
        pst.count_fetch AS rows_fetched,
-       format_pico_time(pst.sum_timer_fetch) AS fetch_latency,
+       sys.format_time(pst.sum_timer_fetch) AS fetch_latency,
        pst.count_insert AS rows_inserted,
-       format_pico_time(pst.sum_timer_insert) AS insert_latency,
+       sys.format_time(pst.sum_timer_insert) AS insert_latency,
        pst.count_update AS rows_updated,
-       format_pico_time(pst.sum_timer_update) AS update_latency,
+       sys.format_time(pst.sum_timer_update) AS update_latency,
        pst.count_delete AS rows_deleted,
-       format_pico_time(pst.sum_timer_delete) AS delete_latency,
+       sys.format_time(pst.sum_timer_delete) AS delete_latency,
        fsbi.count_read AS io_read_requests,
-       format_bytes(fsbi.sum_number_of_bytes_read) AS io_read,
-       format_pico_time(fsbi.sum_timer_read) AS io_read_latency,
+       sys.format_bytes(fsbi.sum_number_of_bytes_read) AS io_read,
+       sys.format_time(fsbi.sum_timer_read) AS io_read_latency,
        fsbi.count_write AS io_write_requests,
-       format_bytes(fsbi.sum_number_of_bytes_write) AS io_write,
-       format_pico_time(fsbi.sum_timer_write) AS io_write_latency,
+       sys.format_bytes(fsbi.sum_number_of_bytes_write) AS io_write,
+       sys.format_time(fsbi.sum_timer_write) AS io_write_latency,
        fsbi.count_misc AS io_misc_requests,
-       format_pico_time(fsbi.sum_timer_misc) AS io_misc_latency
+       sys.format_time(fsbi.sum_timer_misc) AS io_misc_latency
   FROM performance_schema.table_io_waits_summary_by_table AS pst
   LEFT JOIN x$ps_schema_table_statistics_io AS fsbi
     ON pst.object_schema = fsbi.table_schema
