@@ -1,4 +1,4 @@
--- Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+-- Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -49,16 +49,16 @@ VIEW user_summary (
 ) AS
 SELECT IF(accounts.user IS NULL, 'background', accounts.user) AS user,
        SUM(stmt.total) AS statements,
-       format_pico_time(SUM(stmt.total_latency)) AS statement_latency,
-       format_pico_time(IFNULL(SUM(stmt.total_latency) / NULLIF(SUM(stmt.total), 0), 0)) AS statement_avg_latency,
+       sys.format_time(SUM(stmt.total_latency)) AS statement_latency,
+       sys.format_time(IFNULL(SUM(stmt.total_latency) / NULLIF(SUM(stmt.total), 0), 0)) AS statement_avg_latency,
        SUM(stmt.full_scans) AS table_scans,
        SUM(io.ios) AS file_ios,
-       format_pico_time(SUM(io.io_latency)) AS file_io_latency,
+       sys.format_time(SUM(io.io_latency)) AS file_io_latency,
        SUM(accounts.current_connections) AS current_connections,
        SUM(accounts.total_connections) AS total_connections,
        COUNT(DISTINCT host) AS unique_hosts,
-       format_bytes(SUM(mem.current_allocated)) AS current_memory,
-       format_bytes(SUM(mem.total_allocated)) AS total_memory_allocated
+       sys.format_bytes(SUM(mem.current_allocated)) AS current_memory,
+       sys.format_bytes(SUM(mem.total_allocated)) AS total_memory_allocated
   FROM performance_schema.accounts
   LEFT JOIN sys.x$user_summary_by_statement_latency AS stmt ON IF(accounts.user IS NULL, 'background', accounts.user) = stmt.user
   LEFT JOIN sys.x$user_summary_by_file_io AS io ON IF(accounts.user IS NULL, 'background', accounts.user) = io.user
