@@ -1,4 +1,4 @@
--- Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+-- Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -44,7 +44,6 @@ VIEW user_summary_by_statement_type (
   total_latency,
   max_latency,
   lock_latency,
-  cpu_latency,
   rows_sent,
   rows_examined,
   rows_affected,
@@ -53,10 +52,9 @@ VIEW user_summary_by_statement_type (
 SELECT IF(user IS NULL, 'background', user) AS user,
        SUBSTRING_INDEX(event_name, '/', -1) AS statement,
        count_star AS total,
-       format_pico_time(sum_timer_wait) AS total_latency,
-       format_pico_time(max_timer_wait) AS max_latency,
-       format_pico_time(sum_lock_time) AS lock_latency,
-       format_pico_time(sum_cpu_time) AS cpu_latency,
+       sys.format_time(sum_timer_wait) AS total_latency,
+       sys.format_time(max_timer_wait) AS max_latency,
+       sys.format_time(sum_lock_time) AS lock_latency,
        sum_rows_sent AS rows_sent,
        sum_rows_examined AS rows_examined,
        sum_rows_affected AS rows_affected,
