@@ -1,4 +1,4 @@
--- Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+-- Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 --
 -- This program is free software; you can redistribute it and/or modify
 -- it under the terms of the GNU General Public License as published by
@@ -45,9 +45,9 @@ VIEW waits_by_host_by_latency (
 SELECT IF(host IS NULL, 'background', host) AS host,
        event_name AS event,
        count_star AS total,
-       format_pico_time(sum_timer_wait) AS total_latency,
-       format_pico_time(avg_timer_wait) AS avg_latency,
-       format_pico_time(max_timer_wait) AS max_latency
+       sys.format_time(sum_timer_wait) AS total_latency,
+       sys.format_time(avg_timer_wait) AS avg_latency,
+       sys.format_time(max_timer_wait) AS max_latency
   FROM performance_schema.events_waits_summary_by_host_by_event_name
  WHERE event_name != 'idle'
    AND sum_timer_wait > 0
