@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -35,6 +34,7 @@
 #include "my_dbug.h"
 #include "my_inttypes.h"
 
+#if defined(HAVE_OPENSSL)
 #include <openssl/err.h>
 #include <openssl/pem.h>
 #include <openssl/rsa.h>
@@ -56,7 +56,7 @@
 #include "violite.h"
 
 #define MAX_CIPHER_LENGTH 1024
-#define PASSWORD_SCRAMBLE_LENGTH 512
+
 #define SHA2_SCRAMBLE_LENGTH SHA256_DIGEST_LENGTH
 
 mysql_mutex_t g_public_key_mutex;
@@ -72,12 +72,6 @@ int sha256_password_deinit(void) {
   return 0;
 }
 
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-static EVP_PKEY *g_public_key = nullptr;
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-static RSA *g_public_key = nullptr;
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-
 /**
   Reads and parse RSA public key data from a file.
 
@@ -85,33 +79,31 @@ static RSA *g_public_key = nullptr;
 
   @return Pointer to the RSA public key storage buffer
 */
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-static EVP_PKEY *rsa_init(MYSQL *mysql) {
-  EVP_PKEY *key = nullptr;
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+
+static RSA *g_public_key = NULL;
+
 static RSA *rsa_init(MYSQL *mysql) {
-  RSA *key = nullptr;
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+  RSA *key = NULL;
 
   mysql_mutex_lock(&g_public_key_mutex);
   key = g_public_key;
   mysql_mutex_unlock(&g_public_key_mutex);
 
-  if (key != nullptr) return key;
+  if (key != NULL) return key;
 
-  FILE *pub_key_file = nullptr;
+  FILE *pub_key_file = NULL;
 
-  if (mysql->options.extension != nullptr &&
-      mysql->options.extension->server_public_key_path != nullptr &&
+  if (mysql->options.extension != NULL &&
+      mysql->options.extension->server_public_key_path != NULL &&
       mysql->options.extension->server_public_key_path[0] != '\0') {
     pub_key_file =
         fopen(mysql->options.extension->server_public_key_path, "rb");
   }
   /* No public key is used; return 0 without errors to indicate this. */
   else
-    return nullptr;
+    return 0;
 
-  if (pub_key_file == nullptr) {
+  if (pub_key_file == NULL) {
     /*
       If a key path was submitted but no key located then we print an error
       message. Else we just report that there is no public key.
@@ -119,54 +111,23 @@ static RSA *rsa_init(MYSQL *mysql) {
     my_message_local(WARNING_LEVEL, EE_FAILED_TO_LOCATE_SERVER_PUBLIC_KEY,
                      mysql->options.extension->server_public_key_path);
 
-    return nullptr;
+    return 0;
   }
 
   mysql_mutex_lock(&g_public_key_mutex);
-  key = g_public_key =
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-      PEM_read_PUBKEY(pub_key_file, nullptr, nullptr, nullptr);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-      PEM_read_RSA_PUBKEY(pub_key_file, nullptr, nullptr, nullptr);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+  key = g_public_key = PEM_read_RSA_PUBKEY(pub_key_file, 0, 0, 0);
   mysql_mutex_unlock(&g_public_key_mutex);
   fclose(pub_key_file);
-  if (g_public_key == nullptr) {
+  if (g_public_key == NULL) {
     ERR_clear_error();
     my_message_local(WARNING_LEVEL, EE_PUBLIC_KEY_NOT_IN_PEM_FORMAT,
                      mysql->options.extension->server_public_key_path);
-    return nullptr;
+    return 0;
   }
 
   return key;
 }
 
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-static bool encrypt_RSA_public_key(const unsigned char *password,
-                                   int password_len, unsigned char *to,
-                                   size_t *to_len, EVP_PKEY *public_key) {
-  EVP_PKEY_CTX *key_ctx = EVP_PKEY_CTX_new(public_key, nullptr);
-  if (!key_ctx) return true;
-  if (EVP_PKEY_encrypt_init(key_ctx) <= 0 ||
-      EVP_PKEY_CTX_set_rsa_padding(key_ctx, RSA_PKCS1_OAEP_PADDING) <= 0 ||
-      EVP_PKEY_encrypt(key_ctx, to, to_len, password, password_len) <= 0) {
-    EVP_PKEY_CTX_free(key_ctx);
-    return true;
-  }
-  EVP_PKEY_CTX_free(key_ctx);
-  return false;
-}
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-static bool encrypt_RSA_public_key(const unsigned char *password,
-                                   int password_len, unsigned char *to,
-                                   RSA *public_key) {
-  if (RSA_public_encrypt(password_len, password, to, public_key,
-                         RSA_PKCS1_OAEP_PADDING) == -1)
-    return true;
-  return false;
-}
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-
 /**
   Authenticate the client using the RSA or TLS and a SHA256 salted password.
 
@@ -182,14 +143,10 @@ int sha256_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
   bool uses_password = mysql->passwd[0] != 0;
   unsigned char encrypted_password[MAX_CIPHER_LENGTH];
   static char request_public_key = '\1';
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-  EVP_PKEY *public_key = nullptr;
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-  RSA *public_key = nullptr;
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+  RSA *public_key = NULL;
   bool got_public_key_from_server = false;
   bool connection_is_secure = false;
-  unsigned char scramble_pkt[SCRAMBLE_LENGTH]{};
+  unsigned char scramble_pkt[20];
   unsigned char *pkt;
 
   DBUG_TRACE;
@@ -212,7 +169,7 @@ int sha256_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
   */
   memcpy(scramble_pkt, pkt, SCRAMBLE_LENGTH);
 
-  if (mysql_get_ssl_cipher(mysql) != nullptr) connection_is_secure = true;
+  if (mysql_get_ssl_cipher(mysql) != NULL) connection_is_secure = true;
 
   /* If connection isn't secure attempt to get the RSA public key file */
   if (!connection_is_secure) {
@@ -231,23 +188,18 @@ int sha256_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
       /*
         If no public key; request one from the server.
       */
-      if (public_key == nullptr) {
+      if (public_key == NULL) {
         if (vio->write_packet(vio, (const unsigned char *)&request_public_key,
                               1))
           return CR_ERROR;
 
-        int packet_len = 0;
-        unsigned char *packet;
-        if ((packet_len = vio->read_packet(vio, &packet)) == -1)
-          return CR_ERROR;
-        BIO *bio = BIO_new_mem_buf(packet, packet_len);
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-        public_key = PEM_read_bio_PUBKEY(bio, nullptr, nullptr, nullptr);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-        public_key = PEM_read_bio_RSA_PUBKEY(bio, nullptr, nullptr, nullptr);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+        int pkt_len = 0;
+        unsigned char *pkt;
+        if ((pkt_len = vio->read_packet(vio, &pkt)) == -1) return CR_ERROR;
+        BIO *bio = BIO_new_mem_buf(pkt, pkt_len);
+        public_key = PEM_read_bio_RSA_PUBKEY(bio, NULL, NULL, NULL);
         BIO_free(bio);
-        if (public_key == nullptr) {
+        if (public_key == 0) {
           ERR_clear_error();
           return CR_ERROR;
         }
@@ -257,16 +209,17 @@ int sha256_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
       /*
         An arbitrary limitation based on the assumption that passwords
         larger than e.g. 15 symbols don't contribute to security.
-        Note also that it's further restricted to RSA_size() - 41 down
+        Note also that it's furter restricted to RSA_size() - 41 down
         below, so this leaves 471 bytes of possible RSA key sizes which
         should be reasonably future-proof.
         We avoid heap allocation for speed reasons.
       */
-      char passwd_scramble[PASSWORD_SCRAMBLE_LENGTH];
+      char passwd_scramble[512];
 
       if (passwd_len > sizeof(passwd_scramble)) {
         /* password too long for the buffer */
-        goto err;
+        if (got_public_key_from_server) RSA_free(public_key);
+        return CR_ERROR;
       }
       memmove(passwd_scramble, mysql->passwd, passwd_len);
 
@@ -274,37 +227,21 @@ int sha256_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
       xor_string(passwd_scramble, passwd_len - 1, (char *)scramble_pkt,
                  SCRAMBLE_LENGTH);
       /* Encrypt the password and send it to the server */
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-      int cipher_length = EVP_PKEY_get_size(public_key);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
       int cipher_length = RSA_size(public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
       /*
         When using RSA_PKCS1_OAEP_PADDING the password length must be less
         than RSA_size(rsa) - 41.
       */
       if (passwd_len + 41 >= (unsigned)cipher_length) {
         /* password message is to long */
-        goto err;
+        if (got_public_key_from_server) RSA_free(public_key);
+        return CR_ERROR;
       }
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-      {
-        size_t encrypted_password_len = sizeof(encrypted_password);
-
-        if (encrypt_RSA_public_key((unsigned char *)passwd_scramble, passwd_len,
-                                   encrypted_password, &encrypted_password_len,
-                                   public_key))
-          goto err;
-
-        if (got_public_key_from_server) EVP_PKEY_free(public_key);
-      }
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-      if (encrypt_RSA_public_key((unsigned char *)passwd_scramble, passwd_len,
-                                 encrypted_password, public_key))
-        goto err;
-
+      RSA_public_encrypt(passwd_len, (unsigned char *)passwd_scramble,
+                         encrypted_password, public_key,
+                         RSA_PKCS1_OAEP_PADDING);
       if (got_public_key_from_server) RSA_free(public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+
       if (vio->write_packet(vio, (uchar *)encrypted_password, cipher_length))
         return CR_ERROR;
     } else {
@@ -315,121 +252,6 @@ int sha256_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
   }
 
   return CR_OK;
-
-err:
-  if (got_public_key_from_server)
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-    EVP_PKEY_free(public_key);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-    RSA_free(public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-  return CR_ERROR;
-}
-
-/**
-  Read RSA public key sent by server - used by nonblocking
-  version of caching_sha2_password and sha256_password
-  plugins
-
-  @param [in]      vio       VIO handle to read data from server
-  @param [in, out] ctx       Async authentication context to store data
-  @param [out]     result    Authentication process result
-  @param [out]     got_public_key_from_server Flag to be used for cleanup
-  @param [out]     status    Async status
-
-  @returns status of read operation
-    @retval false Success
-    @retval true  Failure
-*/
-static bool read_public_key_nonblocking(MYSQL_PLUGIN_VIO *vio,
-                                        mysql_async_auth *ctx, int *result,
-                                        bool &got_public_key_from_server,
-                                        net_async_status &status) {
-  unsigned char *pkt = nullptr;
-  int io_result;
-  status = vio->read_packet_nonblocking(vio, &pkt, &io_result);
-  if (status == NET_ASYNC_NOT_READY) return true;
-
-  if (io_result <= 0) {
-    *result = CR_ERROR;
-    status = NET_ASYNC_COMPLETE;
-    return true;
-  }
-
-  BIO *bio = BIO_new_mem_buf(pkt, io_result);
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-  ctx->sha2_auth.public_key =
-      PEM_read_bio_PUBKEY(bio, nullptr, nullptr, nullptr);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-  ctx->sha2_auth.public_key =
-      PEM_read_bio_RSA_PUBKEY(bio, nullptr, nullptr, nullptr);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-  BIO_free(bio);
-  if (ctx->sha2_auth.public_key == nullptr) {
-    ERR_clear_error();
-    DBUG_PRINT("info", ("Failed to parse public key"));
-    *result = CR_ERROR;
-    status = NET_ASYNC_COMPLETE;
-    return true;
-  }
-  got_public_key_from_server = true;
-  return false;
-}
-
-/** Helper function to free RSA key */
-void free_rsa_key(mysql_async_auth *ctx) {
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-  EVP_PKEY_free(ctx->sha2_auth.public_key);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-  RSA_free(ctx->sha2_auth.public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-}
-
-/**
-  Prepare public key and password for encryption
-
-  @param [in]  ctx              Async authentication context to retrieve data
-  @param [out] passwd_scramble  Buffer to store scramble. Must be allocated
-  @param [in]  scramble_length  Length of the out buffer
-  @param [in]  passwd           Password
-  @param [in]  passwd_len       Length of password
-
-  @returns Result of the processing
-    @retval false Success
-    @retval true  Failure
-*/
-static bool process_public_key_and_prepare_scramble_nonblocking(
-    mysql_async_auth *ctx, char *passwd_scramble, size_t scramble_length,
-    const char *passwd, unsigned int passwd_len) {
-  if (passwd_len > scramble_length) {
-    /* password too long for the buffer */
-    DBUG_PRINT("info", ("Password is too long."));
-    return true;
-  }
-
-  memmove(passwd_scramble, passwd, passwd_len);
-  /* Obfuscate the plain text password with the session scramble */
-  xor_string(passwd_scramble, passwd_len - 1,
-             (char *)ctx->sha2_auth.scramble_pkt,
-             sizeof(ctx->sha2_auth.scramble_pkt));
-
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-  ctx->sha2_auth.cipher_length = EVP_PKEY_get_size(ctx->sha2_auth.public_key);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-  ctx->sha2_auth.cipher_length = RSA_size(ctx->sha2_auth.public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-
-  /*
-    When using RSA_PKCS1_OAEP_PADDING the password length
-    must be less than RSA_size(rsa) - 41.
-  */
-  if (passwd_len + 41 >= (unsigned)ctx->sha2_auth.cipher_length) {
-    /* password message is to long */
-    DBUG_PRINT("info", ("Password is too long to be encrypted using "
-                        "given public key."));
-    return true;
-  }
-  return false;
 }
 
 /**
@@ -440,11 +262,13 @@ net_async_status sha256_password_auth_client_nonblocking(MYSQL_PLUGIN_VIO *vio,
                                                          int *result) {
   DBUG_TRACE;
   net_async_status status = NET_ASYNC_NOT_READY;
-  bool uses_password = mysql->passwd[0] != 0;
+  unsigned char encrypted_password[MAX_CIPHER_LENGTH];
   static char request_public_key = '\1';
+  static RSA *public_key = NULL;
   bool got_public_key_from_server = false;
   int io_result;
-  bool connection_is_secure = (mysql_get_ssl_cipher(mysql) != nullptr);
+  bool connection_is_secure = (mysql_get_ssl_cipher(mysql) != NULL);
+  unsigned char scramble_pkt[20];
   unsigned char *pkt;
   unsigned int passwd_len =
       static_cast<unsigned int>(strlen(mysql->passwd) + 1);
@@ -467,8 +291,7 @@ net_async_status sha256_password_auth_client_nonblocking(MYSQL_PLUGIN_VIO *vio,
         *result = CR_ERROR;
         return NET_ASYNC_COMPLETE;
       }
-      memcpy(ctx->sha2_auth.scramble_pkt, pkt,
-             sizeof(ctx->sha2_auth.scramble_pkt));
+      memcpy(scramble_pkt, pkt, SCRAMBLE_LENGTH);
       if (connection_is_secure)
         ctx->client_auth_plugin_state =
             client_auth_sha256_password_plugin_status::
@@ -478,10 +301,10 @@ net_async_status sha256_password_auth_client_nonblocking(MYSQL_PLUGIN_VIO *vio,
             client_auth_sha256_password_plugin_status::
                 SHA256_REQUEST_PUBLIC_KEY;
       return NET_ASYNC_NOT_READY;
-    case client_auth_sha256_password_plugin_status::SHA256_REQUEST_PUBLIC_KEY:
-      ctx->sha2_auth.public_key = rsa_init(mysql);
+    case client_auth_sha256_password_plugin_status::SHA256_REQUEST_PUBLIC_KEY: {
+      public_key = rsa_init(mysql);
       /* If no public key; request one from the server. */
-      if (ctx->sha2_auth.public_key == nullptr) {
+      if (public_key == NULL) {
         status = vio->write_packet_nonblocking(
             vio, (const unsigned char *)&request_public_key, 1, &io_result);
         if (status == NET_ASYNC_NOT_READY) {
@@ -492,63 +315,72 @@ net_async_status sha256_password_auth_client_nonblocking(MYSQL_PLUGIN_VIO *vio,
           return NET_ASYNC_COMPLETE;
         }
       }
+      set_mysql_extended_error(mysql, CR_AUTH_PLUGIN_ERR, unknown_sqlstate,
+                               ER_CLIENT(CR_AUTH_PLUGIN_ERR), "sha256_password",
+                               "Authentication requires SSL encryption");
+      *result = CR_ERROR;
+      return NET_ASYNC_COMPLETE;
+    }
       ctx->client_auth_plugin_state =
           client_auth_sha256_password_plugin_status::SHA256_READ_PUBLIC_KEY;
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case client_auth_sha256_password_plugin_status::SHA256_READ_PUBLIC_KEY:
-      if (ctx->sha2_auth.public_key == nullptr) {
-        if (read_public_key_nonblocking(vio, ctx, result,
-                                        got_public_key_from_server, status))
-          return status;
-      }
-      if (ctx->sha2_auth.public_key) {
-        char passwd_scramble[PASSWORD_SCRAMBLE_LENGTH];
-        if (process_public_key_and_prepare_scramble_nonblocking(
-                ctx, passwd_scramble, sizeof(passwd_scramble), mysql->passwd,
-                passwd_len))
-          goto err;
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-        {
-          size_t encrypted_password_len =
-              sizeof(ctx->sha2_auth.encrypted_password);
-          if (encrypt_RSA_public_key(
-                  (unsigned char *)passwd_scramble, passwd_len,
-                  ctx->sha2_auth.encrypted_password, &encrypted_password_len,
-                  ctx->sha2_auth.public_key))
-            goto err;
-          if (got_public_key_from_server)
-            EVP_PKEY_free(ctx->sha2_auth.public_key);
+      if (public_key == NULL) {
+        status = vio->read_packet_nonblocking(vio, &pkt, &io_result);
+        if (status == NET_ASYNC_NOT_READY) {
+          return NET_ASYNC_NOT_READY;
         }
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-        if (encrypt_RSA_public_key((unsigned char *)passwd_scramble, passwd_len,
-                                   ctx->sha2_auth.encrypted_password,
-                                   ctx->sha2_auth.public_key))
-          goto err;
-        if (got_public_key_from_server) RSA_free(ctx->sha2_auth.public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-      } else {
-        set_mysql_extended_error(mysql, CR_AUTH_PLUGIN_ERR, unknown_sqlstate,
-                                 ER_CLIENT(CR_AUTH_PLUGIN_ERR),
-                                 "sha256_password",
-                                 "Authentication requires SSL encryption");
-        *result = CR_ERROR;
-        return NET_ASYNC_COMPLETE;
+        if (io_result <= 0) {
+          *result = CR_ERROR;
+          return NET_ASYNC_COMPLETE;
+        }
+        BIO *bio = BIO_new_mem_buf(pkt, io_result);
+        public_key = PEM_read_bio_RSA_PUBKEY(bio, NULL, NULL, NULL);
+        BIO_free(bio);
+        if (public_key == 0) {
+          ERR_clear_error();
+          *result = CR_ERROR;
+          return NET_ASYNC_COMPLETE;
+        }
+        got_public_key_from_server = true;
       }
       ctx->client_auth_plugin_state =
           client_auth_sha256_password_plugin_status::
               SHA256_SEND_ENCRYPTED_PASSWORD;
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case client_auth_sha256_password_plugin_status::
         SHA256_SEND_ENCRYPTED_PASSWORD: {
-      if (uses_password) {
-        status = vio->write_packet_nonblocking(
-            vio, (uchar *)ctx->sha2_auth.encrypted_password,
-            ctx->sha2_auth.cipher_length, &io_result);
-      } else {
-        /* We're not using a password */
-        static const unsigned char zero_byte = '\0';
-        status = vio->write_packet_nonblocking(vio, &zero_byte, 1, &io_result);
+      char passwd_scramble[512];
+
+      if (passwd_len > sizeof(passwd_scramble)) {
+        /* password too long for the buffer */
+        if (got_public_key_from_server) RSA_free(public_key);
+        *result = CR_ERROR;
+        return NET_ASYNC_COMPLETE;
       }
+      memmove(passwd_scramble, mysql->passwd, passwd_len);
+
+      /* Obfuscate the plain text password with the session scramble */
+      xor_string(passwd_scramble, passwd_len - 1, (char *)scramble_pkt,
+                 SCRAMBLE_LENGTH);
+      /* Encrypt the password and send it to the server */
+      int cipher_length = RSA_size(public_key);
+      /*
+        When using RSA_PKCS1_OAEP_PADDING the password length must be less
+        than RSA_size(rsa) - 41.
+      */
+      if (passwd_len + 41 >= (unsigned)cipher_length) {
+        /* password message is to long */
+        if (got_public_key_from_server) RSA_free(public_key);
+        *result = CR_ERROR;
+        return NET_ASYNC_COMPLETE;
+      }
+      RSA_public_encrypt(passwd_len, (unsigned char *)passwd_scramble,
+                         encrypted_password, public_key,
+                         RSA_PKCS1_OAEP_PADDING);
+      if (got_public_key_from_server) RSA_free(public_key);
+      status = vio->write_packet_nonblocking(vio, (uchar *)encrypted_password,
+                                             cipher_length, &io_result);
       if (status == NET_ASYNC_NOT_READY) {
         return NET_ASYNC_NOT_READY;
       }
@@ -574,11 +406,6 @@ net_async_status sha256_password_auth_client_nonblocking(MYSQL_PLUGIN_VIO *vio,
   }
   *result = CR_OK;
   return NET_ASYNC_COMPLETE;
-
-err:
-  if (got_public_key_from_server) free_rsa_key(ctx);
-  result = CR_ERROR;
-  return NET_ASYNC_COMPLETE;
 }
 /* caching_sha2_password */
 
@@ -590,11 +417,11 @@ static bool is_secure_transport(MYSQL *mysql) {
   if (!mysql || !mysql->net.vio) return false;
   switch (mysql->net.vio->type) {
     case VIO_TYPE_SSL: {
-      if (mysql_get_ssl_cipher(mysql) == nullptr) return false;
+      if (mysql_get_ssl_cipher(mysql) == NULL) return false;
     }
-      [[fallthrough]];
+    // Fall through
     case VIO_TYPE_SHARED_MEMORY:
-      [[fallthrough]];
+    // Fall through
     case VIO_TYPE_SOCKET:
       return true;
     default:
@@ -603,10 +430,6 @@ static bool is_secure_transport(MYSQL *mysql) {
   return false;
 }
 
-static char request_public_key = '\2';
-static char fast_auth_success = '\3';
-static char perform_full_authentication = '\4';
-
 /**
   Authenticate the client using the RSA or TLS and a SHA2 salted password.
 
@@ -617,18 +440,19 @@ static char perform_full_authentication = '\4';
     @retval CR_ERROR An error occurred.
     @retval CR_OK Authentication succeeded.
 */
+
+static char request_public_key = '\2';
+static char fast_auth_success = '\3';
+static char perform_full_authentication = '\4';
+
 int caching_sha2_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
   bool uses_password = mysql->passwd[0] != 0;
   unsigned char encrypted_password[MAX_CIPHER_LENGTH];
   // static char request_public_key= '\1';
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-  EVP_PKEY *public_key = nullptr;
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-  RSA *public_key = nullptr;
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+  RSA *public_key = NULL;
   bool got_public_key_from_server = false;
   bool connection_is_secure = false;
-  unsigned char scramble_pkt[SCRAMBLE_LENGTH]{};
+  unsigned char scramble_pkt[20];
   unsigned char *pkt;
 
   DBUG_TRACE;
@@ -696,7 +520,7 @@ int caching_sha2_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
     if (!connection_is_secure) {
       public_key = rsa_init(mysql);
 
-      if (public_key == nullptr && mysql->options.extension &&
+      if (public_key == NULL && mysql->options.extension &&
           mysql->options.extension->get_server_public_key) {
         // If no public key; request one from the server.
         if (vio->write_packet(vio, (const unsigned char *)&request_public_key,
@@ -705,13 +529,9 @@ int caching_sha2_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
 
         if ((pkt_len = vio->read_packet(vio, &pkt)) <= 0) return CR_ERROR;
         BIO *bio = BIO_new_mem_buf(pkt, pkt_len);
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-        public_key = PEM_read_bio_PUBKEY(bio, nullptr, nullptr, nullptr);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-        public_key = PEM_read_bio_RSA_PUBKEY(bio, nullptr, nullptr, nullptr);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+        public_key = PEM_read_bio_RSA_PUBKEY(bio, NULL, NULL, NULL);
         BIO_free(bio);
-        if (public_key == nullptr) {
+        if (public_key == 0) {
           ERR_clear_error();
           DBUG_PRINT("info", ("Failed to parse public key"));
           return CR_ERROR;
@@ -728,12 +548,13 @@ int caching_sha2_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
            should be reasonably future-proof.
            We avoid heap allocation for speed reasons.
          */
-        char passwd_scramble[PASSWORD_SCRAMBLE_LENGTH];
+        char passwd_scramble[512];
 
         if (passwd_len > sizeof(passwd_scramble)) {
           /* password too long for the buffer */
+          if (got_public_key_from_server) RSA_free(public_key);
           DBUG_PRINT("info", ("Password is too long."));
-          goto err;
+          return CR_ERROR;
         }
         memmove(passwd_scramble, mysql->passwd, passwd_len);
 
@@ -741,38 +562,22 @@ int caching_sha2_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
         xor_string(passwd_scramble, passwd_len - 1, (char *)scramble_pkt,
                    SCRAMBLE_LENGTH);
         /* Encrypt the password and send it to the server */
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-        int cipher_length = EVP_PKEY_get_size(public_key);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
         int cipher_length = RSA_size(public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
         /*
            When using RSA_PKCS1_OAEP_PADDING the password length must be less
            than RSA_size(rsa) - 41.
          */
         if (passwd_len + 41 >= (unsigned)cipher_length) {
           /* password message is to long */
+          if (got_public_key_from_server) RSA_free(public_key);
           DBUG_PRINT("info", ("Password is too long to be encrypted using "
                               "given public key."));
-          goto err;
+          return CR_ERROR;
         }
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-        {
-          size_t encrypted_password_len = sizeof(encrypted_password);
-          if (encrypt_RSA_public_key((unsigned char *)passwd_scramble,
-                                     passwd_len, encrypted_password,
-                                     &encrypted_password_len, public_key))
-            goto err;
-
-          if (got_public_key_from_server) EVP_PKEY_free(public_key);
-        }
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-        if (encrypt_RSA_public_key((unsigned char *)passwd_scramble, passwd_len,
-                                   encrypted_password, public_key))
-          goto err;
-
+        RSA_public_encrypt(passwd_len, (unsigned char *)passwd_scramble,
+                           encrypted_password, public_key,
+                           RSA_PKCS1_OAEP_PADDING);
         if (got_public_key_from_server) RSA_free(public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
 
         if (vio->write_packet(vio, (uchar *)encrypted_password, cipher_length))
           return CR_ERROR;
@@ -791,15 +596,6 @@ int caching_sha2_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql) {
   }
 
   return CR_OK;
-
-err:
-  if (got_public_key_from_server)
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-    EVP_PKEY_free(public_key);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-    RSA_free(public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-  return CR_ERROR;
 }
 
 /**
@@ -808,13 +604,15 @@ err:
 net_async_status caching_sha2_password_auth_client_nonblocking(
     MYSQL_PLUGIN_VIO *vio, MYSQL *mysql, int *result) {
   DBUG_TRACE;
-  bool uses_password = mysql->passwd[0] != 0;
   int io_result;
   net_async_status status = NET_ASYNC_NOT_READY;
+  static unsigned char encrypted_password[MAX_CIPHER_LENGTH];
+  static RSA *public_key = NULL;
   bool connection_is_secure = is_secure_transport(mysql);
   bool got_public_key_from_server = false;
-  unsigned int passwd_len =
-      static_cast<unsigned int>(strlen(mysql->passwd) + 1);
+  static unsigned char scramble_pkt[20];
+  static int cipher_length = 0;
+  static unsigned int passwd_len = 0;
   unsigned char *pkt;
   mysql_async_auth *ctx = ASYNC_DATA(mysql)->connect_context->auth_context;
 
@@ -844,15 +642,14 @@ net_async_status caching_sha2_password_auth_client_nonblocking(
         Copy the scramble to the stack or it will be lost on the next use
         of the net buffer.
       */
-      memcpy(ctx->sha2_auth.scramble_pkt, pkt,
-             sizeof(ctx->sha2_auth.scramble_pkt));
+      memcpy(scramble_pkt, pkt, SCRAMBLE_LENGTH);
       ctx->client_auth_plugin_state =
           client_auth_caching_sha2_password_plugin_status::
               CACHING_SHA2_WRITING_RESPONSE;
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case client_auth_caching_sha2_password_plugin_status::
         CACHING_SHA2_WRITING_RESPONSE:
-      if (!uses_password) {
+      if (mysql->passwd[0] == 0) {
         /* We're not using a password */
         static const unsigned char zero_byte = '\0';
         status = vio->write_packet_nonblocking(vio, &zero_byte, 1, &io_result);
@@ -866,12 +663,13 @@ net_async_status caching_sha2_password_auth_client_nonblocking(
         *result = CR_OK;
         return NET_ASYNC_COMPLETE;
       } else {
+        /* Password is a 0-terminated byte array ('\0' character included) */
+        passwd_len = static_cast<unsigned int>(strlen(mysql->passwd) + 1);
         /* First try with SHA2 scramble */
         unsigned char sha2_scramble[SHA2_SCRAMBLE_LENGTH];
         if (generate_sha256_scramble(sha2_scramble, SHA2_SCRAMBLE_LENGTH,
                                      mysql->passwd, passwd_len - 1,
-                                     (char *)ctx->sha2_auth.scramble_pkt,
-                                     sizeof(ctx->sha2_auth.scramble_pkt))) {
+                                     (char *)scramble_pkt, SCRAMBLE_LENGTH)) {
           set_mysql_extended_error(mysql, CR_AUTH_PLUGIN_ERR, unknown_sqlstate,
                                    ER_CLIENT(CR_AUTH_PLUGIN_ERR),
                                    "caching_sha2_password",
@@ -892,7 +690,7 @@ net_async_status caching_sha2_password_auth_client_nonblocking(
       ctx->client_auth_plugin_state =
           client_auth_caching_sha2_password_plugin_status::
               CACHING_SHA2_CHALLENGE_RESPONSE;
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case client_auth_caching_sha2_password_plugin_status::
         CACHING_SHA2_CHALLENGE_RESPONSE:
       status = vio->read_packet_nonblocking(vio, &pkt, &io_result);
@@ -921,13 +719,14 @@ net_async_status caching_sha2_password_auth_client_nonblocking(
             client_auth_caching_sha2_password_plugin_status::
                 CACHING_SHA2_SEND_PLAIN_PASSWORD;
       return NET_ASYNC_NOT_READY;
+      /* FALLTHROUGH */
     case client_auth_caching_sha2_password_plugin_status::
         CACHING_SHA2_REQUEST_PUBLIC_KEY:
       /* If connection isn't secure attempt to get the RSA public key file */
       {
-        ctx->sha2_auth.public_key = rsa_init(mysql);
+        public_key = rsa_init(mysql);
 
-        if (ctx->sha2_auth.public_key == nullptr && mysql->options.extension &&
+        if (public_key == NULL && mysql->options.extension &&
             mysql->options.extension->get_server_public_key) {
           status = vio->write_packet_nonblocking(
               vio, (const unsigned char *)&request_public_key, 1, &io_result);
@@ -943,48 +742,62 @@ net_async_status caching_sha2_password_auth_client_nonblocking(
       ctx->client_auth_plugin_state =
           client_auth_caching_sha2_password_plugin_status::
               CACHING_SHA2_READ_PUBLIC_KEY;
-      [[fallthrough]];
+      /* FALLTHROUGH */
     case client_auth_caching_sha2_password_plugin_status::
         CACHING_SHA2_READ_PUBLIC_KEY: {
-      if (ctx->sha2_auth.public_key == nullptr && mysql->options.extension &&
+      if (public_key == NULL && mysql->options.extension &&
           mysql->options.extension->get_server_public_key) {
-        if (read_public_key_nonblocking(vio, ctx, result,
-                                        got_public_key_from_server, status))
-          return status;
+        status = vio->read_packet_nonblocking(vio, &pkt, &io_result);
+        if (status == NET_ASYNC_NOT_READY) {
+          return NET_ASYNC_NOT_READY;
+        }
+        if (io_result <= 0) {
+          *result = CR_ERROR;
+          return NET_ASYNC_COMPLETE;
+        }
+        int pkt_len = 0;
+        BIO *bio = BIO_new_mem_buf(pkt, pkt_len);
+        public_key = PEM_read_bio_RSA_PUBKEY(bio, NULL, NULL, NULL);
+        BIO_free(bio);
+        if (public_key == 0) {
+          ERR_clear_error();
+          DBUG_PRINT("info", ("Failed to parse public key"));
+          *result = CR_ERROR;
+          return NET_ASYNC_COMPLETE;
+        }
+        got_public_key_from_server = true;
       }
-      if (ctx->sha2_auth.public_key) {
-        char passwd_scramble[PASSWORD_SCRAMBLE_LENGTH];
-        if (process_public_key_and_prepare_scramble_nonblocking(
-                ctx, passwd_scramble, sizeof(passwd_scramble), mysql->passwd,
-                passwd_len))
-          goto err;
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-        {
-          size_t encrypted_password_len =
-              sizeof(ctx->sha2_auth.encrypted_password);
-          EVP_PKEY_CTX *key_ctx =
-              EVP_PKEY_CTX_new(ctx->sha2_auth.public_key, nullptr);
-          if (!key_ctx) goto err;
-          if (EVP_PKEY_encrypt_init(key_ctx) <= 0 ||
-              EVP_PKEY_CTX_set_rsa_padding(key_ctx, RSA_PKCS1_OAEP_PADDING) <=
-                  0 ||
-              EVP_PKEY_encrypt(key_ctx, ctx->sha2_auth.encrypted_password,
-                               &encrypted_password_len,
-                               (unsigned char *)passwd_scramble,
-                               passwd_len) <= 0) {
-            EVP_PKEY_CTX_free(key_ctx);
-            goto err;
-          }
-          EVP_PKEY_CTX_free(key_ctx);
-          if (got_public_key_from_server)
-            EVP_PKEY_free(ctx->sha2_auth.public_key);
+      if (public_key) {
+        char passwd_scramble[512];
+        if (passwd_len > sizeof(passwd_scramble)) {
+          /* password too long for the buffer */
+          if (got_public_key_from_server) RSA_free(public_key);
+          DBUG_PRINT("info", ("Password is too long."));
+          *result = CR_ERROR;
+          return NET_ASYNC_COMPLETE;
+        }
+        memmove(passwd_scramble, mysql->passwd, passwd_len);
+        /* Obfuscate the plain text password with the session scramble */
+        xor_string(passwd_scramble, passwd_len - 1, (char *)scramble_pkt,
+                   SCRAMBLE_LENGTH);
+        /* Encrypt the password and send it to the server */
+        cipher_length = RSA_size(public_key);
+        /*
+          When using RSA_PKCS1_OAEP_PADDING the password length
+          must be less than RSA_size(rsa) - 41.
+        */
+        if (passwd_len + 41 >= (unsigned)cipher_length) {
+          /* password message is to long */
+          if (got_public_key_from_server) RSA_free(public_key);
+          DBUG_PRINT("info", ("Password is too long to be encrypted using "
+                              "given public key."));
+          *result = CR_ERROR;
+          return NET_ASYNC_COMPLETE;
         }
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
         RSA_public_encrypt(passwd_len, (unsigned char *)passwd_scramble,
-                           ctx->sha2_auth.encrypted_password,
-                           ctx->sha2_auth.public_key, RSA_PKCS1_OAEP_PADDING);
-        if (got_public_key_from_server) RSA_free(ctx->sha2_auth.public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+                           encrypted_password, public_key,
+                           RSA_PKCS1_OAEP_PADDING);
+        if (got_public_key_from_server) RSA_free(public_key);
       } else {
         set_mysql_extended_error(mysql, CR_AUTH_PLUGIN_ERR, unknown_sqlstate,
                                  ER_CLIENT(CR_AUTH_PLUGIN_ERR),
@@ -997,12 +810,11 @@ net_async_status caching_sha2_password_auth_client_nonblocking(
       ctx->client_auth_plugin_state =
           client_auth_caching_sha2_password_plugin_status::
               CACHING_SHA2_SEND_ENCRYPTED_PASSWORD;
-      [[fallthrough]];
+    /* FALLTHROUGH */
     case client_auth_caching_sha2_password_plugin_status::
         CACHING_SHA2_SEND_ENCRYPTED_PASSWORD: {
-      status = vio->write_packet_nonblocking(
-          vio, (uchar *)ctx->sha2_auth.encrypted_password,
-          ctx->sha2_auth.cipher_length, &io_result);
+      status = vio->write_packet_nonblocking(vio, (uchar *)encrypted_password,
+                                             cipher_length, &io_result);
       if (status == NET_ASYNC_NOT_READY) {
         return NET_ASYNC_NOT_READY;
       }
@@ -1028,22 +840,14 @@ net_async_status caching_sha2_password_auth_client_nonblocking(
   }
   *result = CR_OK;
   return NET_ASYNC_COMPLETE;
-
-err:
-  if (got_public_key_from_server) free_rsa_key(ctx);
-  result = CR_ERROR;
-  return NET_ASYNC_COMPLETE;
 }
 
 void STDCALL mysql_reset_server_public_key(void) {
   DBUG_TRACE;
   mysql_mutex_lock(&g_public_key_mutex);
-  if (g_public_key)
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-    EVP_PKEY_free(g_public_key);
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-    RSA_free(g_public_key);
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-  g_public_key = nullptr;
+  if (g_public_key) RSA_free(g_public_key);
+  g_public_key = NULL;
   mysql_mutex_unlock(&g_public_key_mutex);
 }
+
+#endif
