@@ -407,11 +407,11 @@ struct st_mysql_client_plugin *mysql_load_plugin_v(MYSQL *mysql,
   char win_errormsg[2048];
 #endif
 
-  DBUG_ENTER("mysql_load_plugin_v");
+  DBUG_TRACE;
   DBUG_PRINT("entry", ("name=%s type=%d int argc=%d", name, type, argc));
   if (is_not_initialized(mysql, name)) {
     DBUG_PRINT("leave", ("mysql not initialized"));
-    DBUG_RETURN(NULL);
+    return NULL;
   }
 
   mysql_mutex_lock(&LOCK_load_client_plugin);
@@ -489,14 +489,14 @@ have_plugin:
   mysql_mutex_unlock(&LOCK_load_client_plugin);
 
   DBUG_PRINT("leave", ("plugin loaded ok"));
-  DBUG_RETURN(plugin);
+  return plugin;
 
 err:
   mysql_mutex_unlock(&LOCK_load_client_plugin);
   DBUG_PRINT("leave", ("plugin load error : %s", errmsg));
   set_mysql_extended_error(mysql, CR_AUTH_PLUGIN_CANNOT_LOAD, unknown_sqlstate,
                            ER_CLIENT(CR_AUTH_PLUGIN_CANNOT_LOAD), name, errmsg);
-  DBUG_RETURN(NULL);
+  return NULL;
 }
 
 /* see <mysql/client_plugin.h> for a full description */
@@ -516,9 +516,9 @@ struct st_mysql_client_plugin *mysql_client_find_plugin(MYSQL *mysql,
                                                         int type) {
   struct st_mysql_client_plugin *p;
 
-  DBUG_ENTER("mysql_client_find_plugin");
+  DBUG_TRACE;
   DBUG_PRINT("entry", ("name=%s, type=%d", name, type));
-  if (is_not_initialized(mysql, name)) DBUG_RETURN(NULL);
+  if (is_not_initialized(mysql, name)) return NULL;
 
   if (type < 0 || type >= MYSQL_CLIENT_MAX_PLUGINS) {
     set_mysql_extended_error(
@@ -528,20 +528,20 @@ struct st_mysql_client_plugin *mysql_client_find_plugin(MYSQL *mysql,
 
   if ((p = find_plugin(name, type))) {
     DBUG_PRINT("leave", ("found %p", p));
-    DBUG_RETURN(p);
+    return p;
   }
 
   /* not found, load it */
   p = mysql_load_plugin(mysql, name, type, 0);
   DBUG_PRINT("leave", ("loaded %p", p));
-  DBUG_RETURN(p);
+  return p;
 }
 
 /* see <mysql/client_plugin.h> for a full description */
 int mysql_plugin_options(struct st_mysql_client_plugin *plugin,
                          const char *option, const void *value) {
-  DBUG_ENTER("mysql_plugin_options");
+  DBUG_TRACE;
   /* does the plugin support options call? */
-  if (!plugin || !plugin->options) DBUG_RETURN(1);
-  DBUG_RETURN(plugin->options(option, value));
+  if (!plugin || !plugin->options) return 1;
+  return plugin->options(option, value);
 }
