@@ -1,16 +1,15 @@
-/* Copyright (c) 2019, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,6 @@
 
 #include "compression.h"
 #include "m_ctype.h"
-#include "my_dbug.h"
 #include "my_sys.h"
 #include "mysqld_error.h"
 
@@ -58,6 +56,8 @@ enum_compression_algorithm get_compression_algorithm(std::string name) {
 
   @param       name    comma separated list of compression algorithm names
   @param[out]  list    list containing algorithm names
+
+  @retval void
 */
 void parse_compression_algorithms_list(std::string name,
                                        std::vector<std::string> &list) {
@@ -91,43 +91,20 @@ bool is_zstd_compression_level_valid(uint level) {
   @retval 1  error or warnings
 */
 bool validate_compression_attributes(std::string algorithm_names,
-                                     std::string channel_name [[maybe_unused]],
-                                     bool ignore_errors [[maybe_unused]]) {
+                                     std::string channel_name,
+                                     bool ignore_errors) {
   DBUG_TRACE;
-  /*
-    Note: there's no real limit like that to the string. But, since the
-    replication
-  */
-  if (algorithm_names.length() >= COMPRESSION_ALGORITHM_NAME_BUFFER_SIZE) {
-#ifdef MYSQL_SERVER
-    if (!ignore_errors) {
-      my_error(ER_CHANGE_RPL_SRC_WRONG_COMPRESSION_ALGORITHM_SIZE, MYF(0),
-               algorithm_names.length(), channel_name.data());
-    }
-#endif
-    return true;
-  }
+  DBUG_ASSERT(algorithm_names.length() <
+              COMPRESSION_ALGORITHM_NAME_BUFFER_SIZE);
   std::vector<std::string> algorithm_name_list;
 
   parse_compression_algorithms_list(algorithm_names, algorithm_name_list);
   unsigned int total_names = algorithm_name_list.size();
 
-  if (!total_names) {
-#ifdef MYSQL_SERVER
-    if (!ignore_errors) {
-      my_error(ER_CHANGE_SOURCE_WRONG_COMPRESSION_ALGORITHM_CLIENT, MYF(0),
-               algorithm_names.c_str(), channel_name.c_str());
-    }
-#endif
-    return true;
-  }
   if (total_names > COMPRESSION_ALGORITHM_COUNT_MAX) {
-#ifdef MYSQL_SERVER
-    if (!ignore_errors) {
-      my_error(ER_CHANGE_SOURCE_WRONG_COMPRESSION_ALGORITHM_LIST_CLIENT, MYF(0),
+    if (!ignore_errors)
+      my_error(ER_CHANGE_MASTER_WRONG_COMPRESSION_ALGORITHM_LIST_CLIENT, MYF(0),
                algorithm_names.c_str(), channel_name.c_str());
-    }
-#endif
     return true;
   }
   /* validate compression algorithm names */
@@ -138,12 +115,9 @@ bool validate_compression_attributes(std::string algorithm_names,
     /* validate algorithm name */
     method = get_compression_algorithm(algorithm_name);
     if (method == enum_compression_algorithm::MYSQL_INVALID) {
-#ifdef MYSQL_SERVER
-      if (!ignore_errors) {
-        my_error(ER_CHANGE_SOURCE_WRONG_COMPRESSION_ALGORITHM_CLIENT, MYF(0),
+      if (!ignore_errors)
+        my_error(ER_CHANGE_MASTER_WRONG_COMPRESSION_ALGORITHM_CLIENT, MYF(0),
                  algorithm_name.c_str(), channel_name.c_str());
-      }
-#endif
       return true;
     }
     name_it++;
