@@ -26,6 +26,7 @@
 #include <stddef.h>
 #include <sys/types.h>
 #include <functional>
+#include <list>
 #include <memory>
 #include <set>
 #include <utility>
@@ -34,7 +35,9 @@
 #include "lex_string.h"
 #include "my_command.h"
 #include "my_dbug.h"
+#include "my_hostname.h"  // HOSTNAME_LENGTH
 #include "my_inttypes.h"
+#include "mysql_com.h"  // USERNAME_LENGTH
 #include "template_utils.h"
 
 /* Forward Declarations */
@@ -168,7 +171,7 @@ class ACL_internal_schema_access {
 */
 class ACL_internal_schema_registry {
  public:
-  static void register_schema(const LEX_STRING &name,
+  static void register_schema(const LEX_CSTRING &name,
                               const ACL_internal_schema_access *access);
   static const ACL_internal_schema_access *lookup(const char *name);
 };
@@ -673,7 +676,7 @@ bool acl_check_host(THD *thd, const char *host, const char *ip);
 void log_user(THD *thd, String *str, LEX_USER *user, bool comma);
 bool check_change_password(THD *thd, const char *host, const char *user,
                            bool retain_current_password);
-bool change_password(THD *thd, LEX_USER *user, char *password,
+bool change_password(THD *thd, LEX_USER *user, const char *password,
                      const char *current_password,
                      bool retain_current_password);
 bool mysql_create_user(THD *thd, List<LEX_USER> &list, bool if_not_exists,
@@ -691,16 +694,16 @@ int wild_case_compare(CHARSET_INFO *cs, const char *str, size_t str_len,
 bool hostname_requires_resolving(const char *hostname);
 bool acl_init(bool dont_read_acl_tables);
 void acl_free(bool end = false);
-bool check_engine_type_for_acl_table(THD *thd);
+bool check_engine_type_for_acl_table(THD *thd, bool mdl_locked);
 bool grant_init(bool skip_grant_tables);
 void grant_free(void);
-bool reload_acl_caches(THD *thd);
+bool reload_acl_caches(THD *thd, bool mdl_locked);
 ulong acl_get(THD *thd, const char *host, const char *ip, const char *user,
               const char *db, bool db_is_pattern);
 bool is_acl_user(THD *thd, const char *host, const char *user);
-bool acl_getroot(THD *thd, Security_context *sctx, char *user, char *host,
-                 char *ip, const char *db);
-bool check_acl_tables_intact(THD *thd);
+bool acl_getroot(THD *thd, Security_context *sctx, const char *user,
+                 const char *host, const char *ip, const char *db);
+bool check_acl_tables_intact(THD *thd, bool mdl_locked);
 bool check_acl_tables_intact(THD *thd, TABLE_LIST *tables);
 void notify_flush_event(THD *thd);
 bool wildcard_db_grant_exists();
@@ -816,11 +819,12 @@ typedef enum ssl_artifacts_status {
 } ssl_artifacts_status;
 
 ulong get_global_acl_cache_size();
-#if defined(HAVE_OPENSSL) && !defined(HAVE_WOLFSSL)
+#if defined(HAVE_OPENSSL)
 extern bool opt_auto_generate_certs;
 bool do_auto_cert_generation(ssl_artifacts_status auto_detection_status,
-                             char **ssl_ca, char **ssl_key, char **ssl_cert);
-#endif /* HAVE_OPENSSL && !HAVE_WOLFSSL */
+                             const char **ssl_ca, const char **ssl_key,
+                             const char **ssl_cert);
+#endif /* HAVE_OPENSSL */
 
 #define DEFAULT_SSL_CA_CERT "ca.pem"
 #define DEFAULT_SSL_CA_KEY "ca-key.pem"
@@ -828,10 +832,10 @@ bool do_auto_cert_generation(ssl_artifacts_status auto_detection_status,
 #define DEFAULT_SSL_SERVER_KEY "server-key.pem"
 
 void update_mandatory_roles(void);
-bool check_authorization_id_string(THD *thd, const char *buffer, size_t length);
+bool check_authorization_id_string(THD *thd, LEX_STRING &mandatory_roles);
 void func_current_role(const THD *thd, String *active_role);
 
-extern volatile uint32 global_password_history, global_password_reuse_interval;
+extern uint32 global_password_history, global_password_reuse_interval;
 
 struct Security_context_policy {
   enum Operation { Precheck, Execute };
@@ -1001,10 +1005,17 @@ class Auth_id {
   const std::string &host() const;
 
  private:
+  void create_key();
   /** User part */
   std::string m_user;
   /** Host part */
   std::string m_host;
+  /**
+    Key: Internal representation mainly to facilitate use of
+         Auth_id class in standard container.
+         Format: 'user\0host\0'
+  */
+  std::string m_key;
 };
 
 /*
@@ -1014,4 +1025,17 @@ class Auth_id {
 */
 using Role_id = Auth_id;
 
+/**
+  Length of string buffer, that is enough to contain
+  username and hostname parts of the user identifier with trailing zero in
+  MySQL standard format:
+  user_name_part\@host_name_part\\0
+*/
+static constexpr int USER_HOST_BUFF_SIZE =
+    HOSTNAME_LENGTH + USERNAME_LENGTH + 2;
+
+void generate_random_password(std::string *password, uint32_t);
+typedef std::list<std::vector<std::string>> Userhostpassword_list;
+bool send_password_result_set(THD *thd,
+                              const Userhostpassword_list &generated_passwords);
 #endif /* AUTH_COMMON_INCLUDED */
