@@ -1,4 +1,4 @@
-/* Copyright (c) 2018, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -28,11 +28,11 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include "mysql/components/services/log_shared.h"
 #include "mysqld_error.h"
 #include "partial_revokes.h"
-#include "sql-common/json_dom.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"
 #include "sql/auth/auth_internal.h"
 #include "sql/auth/sql_auth_cache.h"
+#include "sql/json_dom.h"
 #include "sql/mysqld.h"
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
@@ -45,38 +45,50 @@ const std::string Restrictions("Restrictions");
 
 /**
   Abstract restriction constructor
+
+  @param [in] mem_root MEM_ROOT handle to be used to store restrictions.
+                       Can be nullptr.
 */
-Abstract_restrictions::Abstract_restrictions() = default;
+Abstract_restrictions::Abstract_restrictions(MEM_ROOT *mem_root)
+    : m_mem_root_base(mem_root) {}
 
 /** Abstract restriction destructor */
-Abstract_restrictions::~Abstract_restrictions() = default;
+Abstract_restrictions::~Abstract_restrictions() {}
 
 /**
   DB Restrictions constructor
+
+  @param [in] mem_root MEM_ROOT handle. Can be nullptr.
+
 */
-DB_restrictions::DB_restrictions() : Abstract_restrictions() {}
+DB_restrictions::DB_restrictions(MEM_ROOT *mem_root)
+    : Abstract_restrictions(mem_root),
+      m_restrictions(system_charset_info ? system_charset_info
+                                         : &my_charset_utf8_general_ci,
+                     m_mem_root_base.get_mem_root()) {}
 
 /**
   Copy constructor for DB Restrictions
 
-  @param [in] other Source DB restrictions
+  @param [in] restrictions Source DB restrictions
 */
-DB_restrictions::DB_restrictions(const DB_restrictions &other) {
-  copy_restrictions(other);
+DB_restrictions::DB_restrictions(const DB_restrictions &restrictions)
+    : DB_restrictions(nullptr) {
+  add(restrictions);
 }
-
 /** Destructor */
-DB_restrictions::~DB_restrictions() { clear(); }
+DB_restrictions::~DB_restrictions() { m_restrictions.clear(); }
 
 /**
   Assignment operator
 
-  @param [in] other Source DB restrictions
+  @param [in] restrictions Source DB restrictions
 */
-DB_restrictions &DB_restrictions::operator=(const DB_restrictions &other) {
-  if (this != &other) {
-    clear();
-    copy_restrictions(other);
+DB_restrictions &DB_restrictions::operator=(
+    const DB_restrictions &restrictions) {
+  if (this != &restrictions) {
+    this->clear();
+    this->add(restrictions);
   }
   return *this;
 }
@@ -88,8 +100,8 @@ DB_restrictions &DB_restrictions::operator=(const DB_restrictions &other) {
 */
 DB_restrictions &DB_restrictions::operator=(DB_restrictions &&restrictions) {
   if (this != &restrictions) {
-    clear();
-    add(restrictions);
+    this->clear();
+    this->add(restrictions);
   }
   return *this;
 }
@@ -98,14 +110,28 @@ DB_restrictions &DB_restrictions::operator=(DB_restrictions &&restrictions) {
 
   @param [in] restrictions DB_restrictions object to be compared with this
 
-  @retval true    If both DB_restrictions are same
-  @retval false   Otherwise
+  @returns
+    @retval true    If both DB_restrictions are same
+    @retval false   Otherwise
 
 */
 bool DB_restrictions::operator==(const DB_restrictions &restrictions) const {
-  if (is_empty() && restrictions.is_empty()) return true;
-  if (is_empty() || restrictions.is_empty()) return false;
-  return (*m_restrictions == *restrictions.m_restrictions);
+  return (m_restrictions == restrictions.m_restrictions);
+}
+
+/**
+  Add given set of privileges (in string format) and
+  add them to restriction list for given database.
+
+  @param [in] db_name Database information
+  @param [in] privs   Restricted access - In string format
+*/
+void DB_restrictions::add(const std::string &db_name,
+                          const std::set<std::string> &privs) {
+  for (const auto &priv : privs) {
+    auto priv_itr = global_acls_map.find(priv);
+    if (priv_itr != global_acls_map.end()) add(db_name, priv_itr->second);
+  }
 }
 
 /**
@@ -116,13 +142,12 @@ bool DB_restrictions::operator==(const DB_restrictions &restrictions) const {
 */
 void DB_restrictions::add(const std::string &db_name,
                           const ulong revoke_privs) {
-  auto restrictions = create_restrictions_if_needed();
-  if (restrictions->find(db_name) != restrictions->end()) {
+  if (m_restrictions.find(db_name) != m_restrictions.end()) {
     /* Partial revokes already exists on this DB so update them */
-    (*restrictions)[db_name] |= (revoke_privs);
+    m_restrictions[db_name] |= (revoke_privs);
   } else {
     /* Partial revokes do not exist on this DB so add them */
-    restrictions->emplace(db_name, revoke_privs);
+    m_restrictions.emplace(db_name, revoke_privs);
   }
 }
 
@@ -132,10 +157,7 @@ void DB_restrictions::add(const std::string &db_name,
   @param [in] restrictions List of <database, privileges>
 */
 void DB_restrictions::add(const DB_restrictions &restrictions) {
-  if (!restrictions.m_restrictions) {
-    return;
-  }
-  for (auto &itr : *(restrictions.m_restrictions))
+  for (auto &itr : restrictions.m_restrictions)
     add(itr.first.c_str(), itr.second);
 }
 
@@ -193,6 +215,26 @@ bool DB_restrictions::add(const Json_object &json_object) {
   return false;
 }
 
+/**
+  Remove given set of privileges for a database from restriction list
+
+  @param [in] db_name      Database information
+  @param [in] revoke_privs List of privileges to remove
+*/
+void DB_restrictions::remove(const std::string &db_name,
+                             const std::set<std::string> &revoke_privs) {
+  auto rest_itr = m_restrictions.find(db_name);
+  if (rest_itr != m_restrictions.end()) {
+    for (auto &priv : revoke_privs) {
+      const auto &itr = global_acls_map.find(priv);
+      if (itr != global_acls_map.end()) {
+        auto &priv_mask = rest_itr->second;
+        priv_mask ^= (1UL << itr->second);
+      }
+    }
+  }
+}
+
 /**
   Remove given set of privilegs for a database from restriction list
 
@@ -201,11 +243,9 @@ bool DB_restrictions::add(const Json_object &json_object) {
 */
 void DB_restrictions::remove(const std::string &db_name,
                              const ulong revoke_privs) {
-  if (m_restrictions != nullptr) {
-    auto rest_itr = m_restrictions->find(db_name);
-    if (rest_itr != m_restrictions->end()) {
-      remove(revoke_privs, rest_itr->second);
-    }
+  auto rest_itr = m_restrictions.find(db_name);
+  if (rest_itr != m_restrictions.end()) {
+    remove(revoke_privs, rest_itr->second);
   }
 }
 
@@ -219,15 +259,13 @@ void DB_restrictions::remove(const std::string &db_name,
   @param [in] revoke_privs Privileges to be removed
 */
 void DB_restrictions::remove(const ulong revoke_privs) {
-  if (m_restrictions != nullptr) {
-    for (auto rest_itr = m_restrictions->begin();
-         rest_itr != m_restrictions->end();) {
-      remove(revoke_privs, rest_itr->second);
-      if (rest_itr->second == 0)
-        rest_itr = m_restrictions->erase(rest_itr);
-      else
-        ++rest_itr;
-    }
+  for (auto rest_itr = m_restrictions.begin();
+       rest_itr != m_restrictions.end();) {
+    remove(revoke_privs, rest_itr->second);
+    if (rest_itr->second == 0)
+      rest_itr = m_restrictions.erase(rest_itr);
+    else
+      ++rest_itr;
   }
 }
 
@@ -254,36 +292,25 @@ void DB_restrictions::remove(const ulong remove_restrictions,
     @retval false Entry not found. Do not rely on access.
 */
 bool DB_restrictions::find(const std::string &db_name, ulong &access) const {
-  if (!is_empty()) {
-    const auto &itr = m_restrictions->find(db_name);
-    if (itr != m_restrictions->end()) {
-      access = itr->second;
-      return true;
-    }
+  const auto &itr = m_restrictions.find(db_name);
+  if (itr != m_restrictions.end()) {
+    access = itr->second;
+    return true;
   }
   return false;
 }
 
-/**
-  Status function to check if restriction list is empty.
-  m_restrictions may not be initialized yet, that also indicates that the
-  restrictions are empty.
-*/
-bool DB_restrictions::is_empty() const {
-  return (m_restrictions != nullptr ? m_restrictions->empty() : true);
-}
+/** Status function to check if restriction list is empty */
+bool DB_restrictions::is_empty() const { return m_restrictions.empty(); }
+
+/** Status function to check if restriction list is non-empty */
+bool DB_restrictions::is_not_empty() const { return !is_empty(); }
 
 /** Status function to get number of entries in restriction list */
-size_t DB_restrictions::size() const {
-  if (is_empty()) return 0;
-  return m_restrictions->size();
-}
+size_t DB_restrictions::size() const { return m_restrictions.size(); }
 
 /** Clear restriction list */
-void DB_restrictions::clear() {
-  delete m_restrictions;
-  m_restrictions = nullptr;
-}
+void DB_restrictions::clear() { m_restrictions.clear(); }
 
 /**
   Serializer. Converts restriction list to JSON format.
@@ -291,8 +318,7 @@ void DB_restrictions::clear() {
   This is used while storing restriction list in ACL table.
 */
 void DB_restrictions::get_as_json(Json_array &restrictions_array) const {
-  assert(m_restrictions != nullptr);
-  for (auto &revocations_itr : *m_restrictions) {
+  for (auto &revocations_itr : m_restrictions) {
     Json_array privileges;
     Json_object revocations_obj;
     Json_string db_name(revocations_itr.first.c_str());
@@ -357,7 +383,7 @@ bool DB_restrictions::has_more_restrictions(const DB_restrictions &other,
   A factory method that creates objects from Restrictions_aggregator
   hierarchy.
 
-  Creates an object if --partial_revokes system variable is ON.
+  Creates an object iif --partial_revokes system variable is ON.
   It also records the CURRENT_USER in the binlog so that partial_revokes can
   be executed on slave with context of current user
 
@@ -367,8 +393,9 @@ bool DB_restrictions::has_more_restrictions(const DB_restrictions &other,
   @param [in]     rights    access specified in the SQL statement
   @param [in]     is_grant_revoke_all_on_db  flag that indicates if the
                             REVOKE/GRANT ALL was executed on a DB
-  @returns A restriction aggregator object
-  @retval  nullptr if partial_revokes system variable is OFF
+  @returns
+    @retval   A restriction aggregator object
+    @retval   nullptr, if partial_revokes system variable is OFF
 */
 std::unique_ptr<Restrictions_aggregator>
 Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
@@ -389,12 +416,12 @@ Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
   const Auth_id grantee = fetch_grantee(acl_user);
   /* Fetch access information of grantor */
   ulong grantor_global_access;
-  Restrictions grantor_restrictions;
+  Restrictions grantor_restrictions(nullptr);
   fetch_grantor_access(security_context, db, grantor_global_access,
                        grantor_restrictions);
-  /* Fetch access information of grantee */
+  /* Fetch access infomation of grantee */
   ulong grantee_global_access;
-  Restrictions grantee_restrictions;
+  Restrictions grantee_restrictions(nullptr);
   fetch_grantee_access(acl_user, grantee_global_access, grantee_restrictions);
   if (db) {
     /* Fetch DB privileges of grantor */
@@ -417,7 +444,7 @@ Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
             std::move(security_context)));
         break;
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         break;
     }
   } else {
@@ -441,11 +468,11 @@ Restrictions_aggregator_factory::create(THD *thd, const ACL_USER *acl_user,
             std::move(security_context)));
         break;
       default:
-        assert(false);
+        DBUG_ASSERT(false);
         break;
     }
   }
-  assert(aggregator);
+  DBUG_ASSERT(aggregator);
   return aggregator;
 }
 
@@ -462,7 +489,7 @@ Restrictions_aggregator_factory::create(
   aggregator.reset(new DB_restrictions_aggregator_set_role(
       grantor, grantee, grantor_access, grantee_access, grantor_db_restrictions,
       grantee_db_restrictions, required_access, db_map));
-  assert(aggregator);
+  DBUG_ASSERT(aggregator);
   return aggregator;
 }
 
@@ -470,7 +497,8 @@ Restrictions_aggregator_factory::create(
   Returns the grantor user name and host id.
   @param  [in]  sctx Security context
 
-  @returns Grantor's user name and host info
+  @returns
+    @retval Grantor's user name and host info
 */
 Auth_id Restrictions_aggregator_factory::fetch_grantor(
     const Security_context *sctx) {
@@ -487,7 +515,8 @@ Auth_id Restrictions_aggregator_factory::fetch_grantor(
 
   @param [in]  acl_user user handle from ACL_Cache
 
-  @returns Grantee's user name and host info
+  @returns
+    @retval Grantee's user name and host info
 */
 Auth_id Restrictions_aggregator_factory::fetch_grantee(
     const ACL_USER *acl_user) {
@@ -505,7 +534,8 @@ Auth_id Restrictions_aggregator_factory::fetch_grantee(
   @param  [in]  thd Thread handle
   @param  [in]  db  Database name for which privileges to be fetched.
 
-  @returns privilege access to the grantor on the specified database
+  @returns
+    @retval privilege access to the grantor on the specified database
 */
 ulong Restrictions_aggregator_factory::fetch_grantor_db_access(THD *thd,
                                                                const char *db) {
@@ -522,7 +552,8 @@ ulong Restrictions_aggregator_factory::fetch_grantor_db_access(THD *thd,
   @param  [in]  acl_user  user handle from ACL_Cache
   @param  [in]  db        Database name for which privileges to be fetched.
 
-  @returns privilege access to the grantee on the specified database
+  @returns
+    @retval privilege access to the grantee on the specified database
 */
 ulong Restrictions_aggregator_factory::fetch_grantee_db_access(
     THD *thd, const ACL_USER *acl_user, const char *db) {
@@ -537,6 +568,9 @@ ulong Restrictions_aggregator_factory::fetch_grantee_db_access(
   @param  [in]  db            Database name for which privileges to be fetched.
   @param  [out] global_access fetch grantor's global access
   @param  [out] restrictions  fetch grantor's restrictions
+
+  @returns
+    @retval global privilege access to the grantor
 */
 void Restrictions_aggregator_factory::fetch_grantor_access(
     const Security_context *sctx, const char *db, ulong &global_access,
@@ -549,7 +583,7 @@ void Restrictions_aggregator_factory::fetch_grantor_access(
 
 void Restrictions_aggregator_factory::fetch_grantee_access(
     const ACL_USER *grantee, ulong &global_access, Restrictions &restrictions) {
-  assert(assert_acl_cache_read_lock(current_thd));
+  DBUG_ASSERT(assert_acl_cache_read_lock(current_thd));
   global_access = grantee->access;
   restrictions = acl_restrictions->find_restrictions(grantee);
 }
@@ -575,11 +609,11 @@ Restrictions_aggregator::Restrictions_aggregator(
       m_requested_access(requested_access),
       m_status(Status::No_op) {
   // partial_revokes system variable is ON
-  assert(mysqld_partial_revokes());
+  DBUG_ASSERT(mysqld_partial_revokes());
 }
 
 /** Destructor */
-Restrictions_aggregator::~Restrictions_aggregator() = default;
+Restrictions_aggregator::~Restrictions_aggregator() {}
 
 /**
   Constructor for database level restrictions aggregator
@@ -617,7 +651,7 @@ DB_restrictions_aggregator::DB_restrictions_aggregator(
   grantor & grantee's restrictions, global access and grantee access.
 
   We also perform dynamic cast here once and call method of respective derived
-  classes. This way, derived classes do not have to override aggregate and
+  classes. This way, dervied classes do not have to override aggregate and
   perform similar dynamic casting before proceeding.
 
   @param [out] restrictions Aggreatated restrictions for grantee
@@ -629,7 +663,7 @@ DB_restrictions_aggregator::DB_restrictions_aggregator(
 bool DB_restrictions_aggregator::generate(Abstract_restrictions &restrictions) {
   DB_restrictions *db_restrictions =
       dynamic_cast<DB_restrictions *>(&restrictions);
-  assert(db_restrictions);
+  DBUG_ASSERT(db_restrictions);
   m_status = validate();
   if (m_status == Status::Validated) {
     aggregate(*db_restrictions);
@@ -642,7 +676,7 @@ bool DB_restrictions_aggregator::generate(Abstract_restrictions &restrictions) {
 /**
   Get list of privileges that are not restricted through restriction list
 
-  @param [out] rights Bitmask of privileges to be processed further
+  @param [out] rights Bitmask of privileges to be processed futher
 
   @returns Any more privilegs remaining?
     @retval false No privileges to be processed further
@@ -712,7 +746,7 @@ void DB_restrictions_aggregator::set_if_db_level_operation(
     const ulong requested_access, const ulong restrictions_mask) noexcept {
   /*
     DB level privilegss are filtered as following -
-    1. If restriction_mask is 0 then there is nothing to filter from the
+    1. If restriction_mask is 0 then there is nothin to filter from the
        requested_access.
     2. If restriction_mask has some access restrictions then remove only
        those access in the requested_access.
@@ -742,55 +776,38 @@ void DB_restrictions_aggregator::set_if_db_level_operation(
                                 is to be done.
   @param  [in]    db_map        DB_access_map used to fetch grantee's db access
                                 for SET ROLE
-  @param  [out]   restrictions  Fills the parameter with the generated
+  @param  [out]   restrictions  Fills the paramter with the generated
                                 DB_restrictions.
 
 */
 void DB_restrictions_aggregator::aggregate_restrictions(
     SQL_OP sql_op, const Db_access_map *db_map, DB_restrictions &restrictions) {
-  if (!m_grantor_rl.is_empty()) {
+  if (m_grantor_rl.is_not_empty()) {
     if (test_all_bits(m_grantee_global_access, m_requested_access) &&
         m_grantee_rl.is_empty()) {
       /*
-        If grantee does not have any restrictions but grantor has, that
-        implies grantee already has more privileges, therefore don't do
-        any further processing. Keep the empty restrictions list.
+        If grantee does not have any restrictions but grantor has, that implies
+        grantee already has more privileges, therefore retain its restrictions
       */
       restrictions = m_grantee_rl;
     } else {
       ulong restrictions_mask;
-      DB_restrictions grantee_rl = m_grantee_rl;
       for (const auto &grantor_rl_itr : m_grantor_rl()) {
         ulong grantee_revokes;
-        if (grantee_rl.find(grantor_rl_itr.first, grantee_revokes)) {
+        if (m_grantee_rl.find(grantor_rl_itr.first.c_str(), grantee_revokes)) {
           /*
             If grantor and grantee both have restriction list then aggregration
             of them will be OR operation of the following :
-             - Grantee's global privileges negates grantor's restrictions
-               (consider relevant restrictions in context of privileges
-                specified in statement i.e. requested_access)
-             - Grantor and Grantee both might have same restrictions as well
-               as global privileges. Remaining restrictions after negation
-               by Grantor and Grantee's global privileges.
+              - Grantor's global privileges negates grantee's restrictions
+              - Grantee's global privileges negates grantor's restrictions
+              - Grantor and grantee both might have same restrictions as well
+                as global privileges
           */
-          ulong rm1 = ((grantor_rl_itr.second & m_requested_access) &
-                       ~m_grantee_global_access);
-          ulong rm2 = (grantor_rl_itr.second & m_grantee_global_access &
-                       grantee_revokes & m_grantor_global_access);
-          restrictions_mask = rm1 | rm2;
-
-          /*
-            Retain restrictions that are not applicable w.r.t.
-            requested operation in the restrictions list.
-            For instance :
-            Grantee may have restrictions SELECT, INSERT on db,
-            the GRANT statement has only SELECT on db, then
-            retain INSERT on db.
-          */
-          ulong remove_mask = grantee_revokes & m_requested_access;
-          grantee_rl.remove(grantor_rl_itr.first, remove_mask);
-          /* Remove the schemas without any restrictions remaining */
-          grantee_rl.remove(0);
+          ulong rm1 = (grantor_rl_itr.second & ~m_grantee_global_access);
+          ulong rm2 = (grantee_revokes & ~m_grantor_global_access);
+          ulong rm3 = (grantor_rl_itr.second & m_grantee_global_access &
+                       m_grantor_global_access & grantee_revokes);
+          restrictions_mask = rm1 | rm2 | rm3;
         } else {
           /*
             Filter restrictions relevant to access from the grantor's
@@ -800,7 +817,8 @@ void DB_restrictions_aggregator::aggregate_restrictions(
 
           ulong grantee_db_access = 0;
           if (sql_op == SQL_OP::GLOBAL_GRANT) {
-            grantee_db_access = get_grantee_db_access(grantor_rl_itr.first);
+            grantee_db_access =
+                get_grantee_db_access(grantor_rl_itr.first.c_str());
           } else if (sql_op == SQL_OP::SET_ROLE && db_map != nullptr) {
             const std::string key(grantor_rl_itr.first.c_str(),
                                   grantor_rl_itr.first.length());
@@ -814,31 +832,16 @@ void DB_restrictions_aggregator::aggregate_restrictions(
           */
           restrictions_mask &= ~grantee_db_access;
 
-          /* Remove any global privilege that grantee already has */
+          /*
+            Remove any global privilege that grantee already has
+          */
           restrictions_mask &= ~m_grantee_global_access;
         }
         if (restrictions_mask)
-          restrictions.add(grantor_rl_itr.first, restrictions_mask);
-      }
-      /*
-       We are left with two types of grantee's restrictions to process.
-       (1) Grantee's schemas that did not match grantor's restrictions but
-           restrictions on these schemas could be negated due to grantor's
-           global privileges.
-       (2) Restrictions in grantee's schemas which are not relevant with
-           the requested access. Grantee must retain these restrictions.
-      */
-      if (!grantee_rl.is_empty()) {
-        for (auto &grantee_rl_itr : grantee_rl()) {
-          grantee_rl_itr.second &=
-              ~(m_grantor_global_access & m_requested_access);
-        }
-        /* Remove the schemas without any restrictions remaining */
-        grantee_rl.remove(0);
-        restrictions.add(grantee_rl);
+          restrictions.add(grantor_rl_itr.first.c_str(), restrictions_mask);
       }
     }
-  } else if (!m_grantee_rl.is_empty()) {
+  } else if (m_grantee_rl.is_not_empty()) {
     restrictions = m_grantee_rl;
     for (auto &grantee_rl_itr : restrictions()) {
       restrictions.remove(grantee_rl_itr.first.c_str(), m_requested_access);
@@ -861,7 +864,8 @@ void DB_restrictions_aggregator::aggregate_restrictions(
 
   @param [in] db_name   Database name for which we need to fetch the DB level
                         access.
-  @returns DB level access.
+  @returns
+    @retval DB level access.
 */
 ulong DB_restrictions_aggregator::get_grantee_db_access(
     const std::string &db_name) const {
@@ -924,9 +928,10 @@ DB_restrictions_aggregator_set_role::DB_restrictions_aggregator_set_role(
   -  Checks possible descrepancy between DB access being granted and
      existing restrictions.
 
-  @returns Status   Moves the object in the appropriate status.
+  @returns
+    @retval Status  Moves the object in the appropiate status.
                     For instance :
-                    - Validated, if validation was performed successfully
+                    - Validated, if validation was performed successfuly
                     - NO_op, if there is no aggregation of privileges required.
                     - Error, if an error encountered
 */
@@ -953,14 +958,14 @@ DB_restrictions_aggregator_set_role::validate() {
   - else if grantee has restrictions
     - Remove the restrictions on which global grant is requested.
 
-  @param  [out]  db_restrictions  Fills the parameter with the generated
-                                  DB_restrictions
+  @param  [out]  restrictions  Fills the paramter with the generated
+                               DB_restrictions
 */
 void DB_restrictions_aggregator_set_role::aggregate(
-    DB_restrictions &db_restrictions) {
-  assert(m_status == Status::Validated);
+    DB_restrictions &restrictions) {
+  DBUG_ASSERT(m_status == Status::Validated);
 
-  if (!m_grantee_rl.is_empty()) {
+  if (m_grantee_rl.is_not_empty()) {
     /*
       At this point, we already have aggregated DB privileges and grantee's
       restrictions. Therefore, negate the restrictions with the DB privileges.
@@ -986,7 +991,7 @@ void DB_restrictions_aggregator_set_role::aggregate(
       }
     }
   }
-  aggregate_restrictions(SQL_OP::SET_ROLE, m_db_map, db_restrictions);
+  aggregate_restrictions(SQL_OP::SET_ROLE, m_db_map, restrictions);
   m_status = Status::Aggregated;
 }
 
@@ -1019,32 +1024,31 @@ DB_restrictions_aggregator_global_grant::
   -  Checks possible descrepancy between DB access being granted and
      existing restrictions.
 
-  @returns Status   Moves the object in the appropriate status.
+  @returns
+    @retval Status  Moves the object in the appropiate status.
                     For instance :
-                    - Validated, if validation was performed successfully
+                    - Validated, if validation was performed successfuly
                     - NO_op, if there is no aggregation of privileges required.
                     - Error, if an error encountered
 */
 Restrictions_aggregator::Status
 DB_restrictions_aggregator_global_grant::validate() {
-  if (!m_grantee_rl.is_empty()) {
-    for (auto &grantee_rl_itr : m_grantee_rl()) {
-      ulong grantee_db_access =
-          get_grantee_db_access(grantee_rl_itr.first.c_str());
-      /*
-        Remove the restrictions from grantor's restrictions for which
-        grantee already has DB level privileges.
-      */
-      if (check_db_access_and_restrictions_collision(
-              grantee_db_access, grantee_rl_itr.second,
-              grantee_rl_itr.first.c_str())) {
-        return (m_status = Status::Error);
-        break;
-      }
+  for (auto &grantee_rl_itr : m_grantee_rl()) {
+    ulong grantee_db_access =
+        get_grantee_db_access(grantee_rl_itr.first.c_str());
+    /*
+      Remove the restrictions from grantor's restrictions for which
+      grantee already has DB level privileges.
+    */
+    if (check_db_access_and_restrictions_collision(
+            grantee_db_access, grantee_rl_itr.second,
+            grantee_rl_itr.first.c_str())) {
+      return (m_status = Status::Error);
+      break;
     }
   }
   /*
-    It must be only Global Grant, if grantor and grantee both does not have any
+    It must be only Global Grant, if grantor and grantee both doesnot have any
     restrictions attached.
   */
   if (m_grantor_rl.is_empty() && m_grantee_rl.is_empty())
@@ -1064,12 +1068,12 @@ DB_restrictions_aggregator_global_grant::validate() {
   - else if grantee has restrictions
     - Remove the restrictions on which global grant is requested.
 
-  @param  [out]  restrictions  Fills the parameter with the generated
+  @param  [out]  restrictions  Fills the paramter with the generated
                                DB_restrictions
 */
 void DB_restrictions_aggregator_global_grant::aggregate(
     DB_restrictions &restrictions) {
-  assert(m_status == Status::Validated);
+  DBUG_ASSERT(m_status == Status::Validated);
   aggregate_restrictions(SQL_OP::GLOBAL_GRANT, nullptr, restrictions);
   m_status = Status::Aggregated;
 }
@@ -1102,9 +1106,10 @@ DB_restrictions_aggregator_global_revoke::
   Evaluates the restrictions list of grantor and grantee, as well as requested
   privilege.
 
-  @returns Status   Moves the object in the appropriate status.
+  @returns
+    @retval Status  Moves the object in the appropiate status.
                     For instance :
-                    - Validated, if validation was performed successfully
+                    - Validated, if validation was performed successfuly
                     - NO_op, if there is no aggregation of privileges required.
                     - Error, if an error encountered
 */
@@ -1118,9 +1123,9 @@ DB_restrictions_aggregator_global_revoke::validate() {
       revoke from grantee.
     */
     return (m_status = Status::No_op);
-  } else if (!m_grantee_rl.is_empty()) {
+  } else if (m_grantee_rl.is_not_empty()) {
     return validate_if_grantee_rl_not_empty();
-  } else if (!m_grantor_rl.is_empty()) {
+  } else if (m_grantor_rl.is_not_empty()) {
     for (auto &itr : m_grantor_rl.get()) {
       /*
         Grantor cannot revoke the privileges from grantee which are in
@@ -1138,18 +1143,25 @@ DB_restrictions_aggregator_global_revoke::validate() {
 }
 
 /**
+  Generates DB_restrictions based on the requested access, grantor and
+  grantee's DB_restrictions in the ACL cache.
   - If grantee has the restriction list
     - Remove only requested restrictions from restriction_list
+  - Else clear the DB_restrictions paramater
 
-  @param  [out]  restrictions  Fills the parater with the aggregated
-                               DB_restrictions
+  @param  [out]  restrictions  Fills the paramter with the aggregated
+                                  DB_restrictions
 */
 void DB_restrictions_aggregator_global_revoke::aggregate(
     DB_restrictions &restrictions) {
-  assert(m_status == Status::Validated);
-  restrictions = m_grantee_rl;
+  DBUG_ASSERT(m_status == Status::Validated);
   if (test_all_bits(m_grantee_global_access, m_requested_access)) {
-    restrictions.remove(m_requested_access);
+    if (m_grantee_rl.is_not_empty()) {
+      restrictions = m_grantee_rl;
+      restrictions.remove(m_requested_access);
+    } else {
+      restrictions.clear();
+    }
   }
   m_status = Status::Aggregated;
 }
@@ -1161,9 +1173,10 @@ void DB_restrictions_aggregator_global_revoke::aggregate(
   - Check if grantee has same DB level privilege as well as
     restriction list on the  database
 
-  @returns  Status  Moves the object in the appropriate status.
+  @returns
+    @retval Status  Moves the object in the appropiate status.
                     For instance :
-                    - Validated, if validation was performed successfully
+                    - Validated, if validation was performed successfuly
                     - NO_op, if there is no aggregation of privileges required.
                     - Error, if an error encountered
 */
@@ -1228,9 +1241,9 @@ Restrictions_aggregator::Status
 DB_restrictions_aggregator_global_revoke_all::validate() {
   if (m_grantor_rl == m_grantee_rl) {
     return (m_status = Status::No_op);
-  } else if (!m_grantee_rl.is_empty()) {
+  } else if (m_grantee_rl.is_not_empty()) {
     return validate_if_grantee_rl_not_empty();
-  } else if (!m_grantor_rl.is_empty()) {
+  } else if (m_grantor_rl.is_not_empty()) {
     for (auto &itr : m_grantor_rl.get()) {
       /*
         Grantor cannot revoke the privileges from grantee which are in
@@ -1254,7 +1267,7 @@ DB_restrictions_aggregator_global_revoke_all::validate() {
 */
 void DB_restrictions_aggregator_global_revoke_all::aggregate(
     DB_restrictions &restrictions) {
-  assert(m_status == Status::Validated);
+  DBUG_ASSERT(m_status == Status::Validated);
   restrictions.clear();
   m_status = Status::Aggregated;
 }
@@ -1339,7 +1352,7 @@ void DB_restrictions_aggregator_db_grant::aggregate(
        should be recorded in mysql.db table. Hence, we call
        set_if_db_level_operation().
   */
-  assert(m_status == Status::Validated);
+  DBUG_ASSERT(m_status == Status::Validated);
   restrictions = m_grantee_rl;
   ulong grantee_db_revokes = 0;
 
@@ -1426,7 +1439,7 @@ DB_restrictions_aggregator_db_revoke::validate() {
 */
 void DB_restrictions_aggregator_db_revoke::aggregate(
     DB_restrictions &restrictions) {
-  assert(m_status == Status::Validated);
+  DBUG_ASSERT(m_status == Status::Validated);
   restrictions = m_grantee_rl;
   if ((test_all_bits(m_grantee_db_access, m_requested_access))) {
     /*
@@ -1455,14 +1468,26 @@ void DB_restrictions_aggregator_db_revoke::aggregate(
 
 /**
   Constructor for Restrictions
+
+  @param [in] mem_root MEM_ROOT to be used to store restrictions
 */
-Restrictions::Restrictions() : m_db_restrictions() {}
+Restrictions::Restrictions(MEM_ROOT *mem_root) : m_db_restrictions(mem_root) {}
 
 /** Destructor */
 Restrictions ::~Restrictions() { m_db_restrictions.clear(); }
 
+/**
+  Copy constructor for Restrictions
+
+  @param [in] restrictions Restrictions to be copied
+*/
+Restrictions::Restrictions(const Restrictions &restrictions)
+    : m_db_restrictions(restrictions.m_db_restrictions) {}
+
 /**
   Move constructor for Restrictions
+
+  @param [in] restrictions
 */
 Restrictions::Restrictions(Restrictions &&restrictions)
     : m_db_restrictions(restrictions.m_db_restrictions) {}
@@ -1497,6 +1522,11 @@ void Restrictions::set_db(const DB_restrictions &db_restrictions) {
   m_db_restrictions = db_restrictions;
 }
 
+/** Add given database restrictions */
+void Restrictions::add_db(const DB_restrictions &db_restrictions) {
+  m_db_restrictions.add(db_restrictions);
+}
+
 /** Clear database restrictions */
 void Restrictions::clear_db() { m_db_restrictions.clear(); }
 
