@@ -69,51 +69,16 @@ extern Role_index_map *g_authid_to_vertex;
 #define MYSQL_DEFAULT_ROLE_FIELD_ROLE_HOST 2
 #define MYSQL_DEFAULT_ROLE_FIELD_ROLE_USER 3
 
-TABLE *open_role_edges_table(THD *thd) {
-  DBUG_ENTER("open_role_edges_table");
-  TABLE_LIST tablelst;
-  tablelst.init_one_table(STRING_WITH_LEN("mysql"),
-                          STRING_WITH_LEN("role_edges"), "role_edges", TL_WRITE,
-                          MDL_SHARED_NO_READ_WRITE);
-  tablelst.next_local = tablelst.next_global = 0;
-
-  if (open_and_lock_tables(thd, &tablelst, MYSQL_LOCK_IGNORE_TIMEOUT)) {
-    DBUG_PRINT("error", ("An error occurred while trying to open the "
-                         "mysql.roles_edges table"));
-    DBUG_RETURN(0);
-  }
-
-  DBUG_RETURN(tablelst.table);
-}
-
-TABLE *open_default_role_table(THD *thd) {
-  DBUG_ENTER("open_role_edges_table");
-  TABLE_LIST tablelst;
-  tablelst.init_one_table(STRING_WITH_LEN("mysql"),
-                          STRING_WITH_LEN("default_roles"), "default_roles",
-                          TL_WRITE, MDL_SHARED_NO_READ_WRITE);
-  tablelst.next_local = tablelst.next_global = 0;
-
-  if (open_and_lock_tables(thd, &tablelst, MYSQL_LOCK_IGNORE_TIMEOUT)) {
-    DBUG_PRINT("error", ("En error occurred while trying to open the "
-                         "mysql.roles_edges table"));
-    DBUG_RETURN(0);
-  }
-
-  DBUG_RETURN(tablelst.table);
-}
-
 bool modify_role_edges_in_table(THD *thd, TABLE *table,
                                 const Auth_id_ref &from_user,
                                 const Auth_id_ref &to_user,
                                 bool with_admin_option, bool delete_option) {
-  DBUG_ENTER("modify_role_edges_in_table");
+  DBUG_TRACE;
   int ret = 0;
   uchar user_key[MAX_KEY_LENGTH];
   Acl_table_intact table_intact(thd);
 
-  if (table_intact.check(table, ACL_TABLES::TABLE_ROLE_EDGES))
-    DBUG_RETURN(true);
+  if (table_intact.check(table, ACL_TABLES::TABLE_ROLE_EDGES)) return true;
 
   table->use_all_columns();
 
@@ -142,7 +107,7 @@ bool modify_role_edges_in_table(THD *thd, TABLE *table,
       ret = table->file->ha_delete_row(table->record[0]);
     } else if (ret == HA_ERR_KEY_NOT_FOUND) {
       /* If the key didn't exist the record is already gone and all is well. */
-      DBUG_RETURN(false);
+      return false;
     }
   } else if (ret == HA_ERR_KEY_NOT_FOUND && !delete_option) {
     /* Insert new edge into table */
@@ -152,20 +117,19 @@ bool modify_role_edges_in_table(THD *thd, TABLE *table,
                 from_user.second.str, to_user.first.str, to_user.second.str));
     ret = table->file->ha_write_row(table->record[0]);
   }
-  DBUG_RETURN(ret != 0);
+  return ret != 0;
 }
 
 bool modify_default_roles_in_table(THD *thd, TABLE *table,
                                    const Auth_id_ref &auth_id,
                                    const Auth_id_ref &role,
                                    bool delete_option) {
-  DBUG_ENTER("modify_default_roles_in_table");
+  DBUG_TRACE;
   int ret = 0;
   uchar user_key[MAX_KEY_LENGTH];
   Acl_table_intact table_intact(thd);
 
-  if (table_intact.check(table, ACL_TABLES::TABLE_DEFAULT_ROLES))
-    DBUG_RETURN(true);
+  if (table_intact.check(table, ACL_TABLES::TABLE_DEFAULT_ROLES)) return true;
 
   table->use_all_columns();
   table->field[MYSQL_DEFAULT_ROLE_FIELD_HOST]->store(
@@ -195,7 +159,7 @@ bool modify_default_roles_in_table(THD *thd, TABLE *table,
                         role.second.str));
     ret = table->file->ha_write_row(table->record[0]);
   }
-  DBUG_RETURN(false);
+  return false;
 }
 
 /*
@@ -212,9 +176,9 @@ bool modify_default_roles_in_table(THD *thd, TABLE *table,
 */
 
 bool populate_roles_caches(THD *thd, TABLE_LIST *tablelst) {
-  DBUG_ENTER("populate_roles_caches");
+  DBUG_TRACE;
   DBUG_ASSERT(assert_acl_cache_write_lock(thd));
-  READ_RECORD read_record_info;
+  unique_ptr_destroy_only<RowIterator> iterator;
   TABLE *roles_edges_table = tablelst[0].table;
   TABLE *default_role_table = tablelst[1].table;
 
@@ -226,11 +190,12 @@ bool populate_roles_caches(THD *thd, TABLE_LIST *tablelst) {
 
   {
     roles_edges_table->use_all_columns();
-    if (init_read_record(&read_record_info, thd, roles_edges_table, NULL, false,
-                         /*ignore_not_found_rows=*/false)) {
+    iterator = init_table_iterator(thd, roles_edges_table, NULL, false,
+                                   /*ignore_not_found_rows=*/false);
+    if (iterator == nullptr) {
       my_error(ER_TABLE_CORRUPT, MYF(0), roles_edges_table->s->db.str,
                roles_edges_table->s->table_name.str);
-      DBUG_RETURN(true);
+      return true;
     }
 
     ACL_USER *acl_role;
@@ -240,7 +205,7 @@ bool populate_roles_caches(THD *thd, TABLE_LIST *tablelst) {
     init_alloc_root(PSI_NOT_INSTRUMENTED, &tmp_mem, 128, 0);
     g_authid_to_vertex->clear();
     g_granted_roles->clear();
-    while (!(read_rec_errcode = read_record_info->Read())) {
+    while (!(read_rec_errcode = iterator->Read())) {
       char *from_host = get_field(
           &tmp_mem, roles_edges_table->field[MYSQL_ROLE_EDGES_FIELD_FROM_HOST]);
       char *from_user = get_field(
@@ -260,27 +225,26 @@ bool populate_roles_caches(THD *thd, TABLE_LIST *tablelst) {
                         "Unknown authorization identifier `%s`@`%s`", MYF(0),
                         to_user, to_host);
         rebuild_vertex_index(thd);
-        DBUG_RETURN(true);
+        return true;
       }
       grant_role(acl_role, acl_user, *with_admin_opt == 'Y' ? 1 : 0);
     }
-    read_record_info.iterator.reset();
+    iterator.reset();
 
     default_role_table->use_all_columns();
 
-    bool ret =
-        init_read_record(&read_record_info, thd, default_role_table, NULL,
-                         false, /*ignore_not_found_records=*/false);
-    DBUG_EXECUTE_IF("dbug_fail_in_role_cache_reinit", ret = true;);
-    if (ret) {
+    iterator = init_table_iterator(thd, default_role_table, NULL, false,
+                                   /*ignore_not_found_records=*/false);
+    DBUG_EXECUTE_IF("dbug_fail_in_role_cache_reinit", iterator = nullptr;);
+    if (iterator == nullptr) {
       my_error(ER_TABLE_CORRUPT, MYF(0), default_role_table->s->db.str,
                default_role_table->s->table_name.str);
 
       rebuild_vertex_index(thd);
-      DBUG_RETURN(true);
+      return true;
     }
     g_default_roles->clear();
-    while (!(read_rec_errcode = read_record_info->Read())) {
+    while (!(read_rec_errcode = iterator->Read())) {
       char *host = get_field(
           &tmp_mem, default_role_table->field[MYSQL_DEFAULT_ROLE_FIELD_HOST]);
       char *user = get_field(
@@ -303,5 +267,5 @@ bool populate_roles_caches(THD *thd, TABLE_LIST *tablelst) {
     opt_mandatory_roles_cache = false;
   }
 
-  DBUG_RETURN(false);
+  return false;
 }
