@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -46,15 +45,15 @@
 #include "my_inttypes.h" /* typedefs                    */
 #include "my_macros.h"
 #include "mysql/components/my_service.h"
-#include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/components/services/bits/psi_rwlock_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/components/services/psi_rwlock_bits.h"
 #include "mysql/mysql_lex_string.h"
 #include "mysql/plugin.h"
 #include "mysql/plugin_audit.h"
 #include "mysql/plugin_auth.h"        /* MYSQL_SERVER_AUTH_INFO      */
 #include "mysql/plugin_auth_common.h" /* MYSQL_PLUGIN_VIO            */
 #include "mysql/psi/mysql_rwlock.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/service_my_plugin_log.h" /* plugin_log_level            */
 #include "mysql/service_mysql_password_policy.h"
 #include "mysql_com.h"
@@ -65,10 +64,8 @@
 #include "sql/auth/i_sha2_password_common.h"
 #include "sql/auth/sql_auth_cache.h" /* ACL_USER                    */
 #include "sql/auth/sql_authentication.h"
-#include "sql/debug_sync.h"       // DEBUG_SYNC
 #include "sql/protocol_classic.h" /* Protocol_classic            */
-#include "sql/sql_class.h"
-#include "sql/sql_const.h" /* MAX_FIELD_WIDTH             */
+#include "sql/sql_const.h"        /* MAX_FIELD_WIDTH             */
 #include "violite.h"
 
 class THD;
@@ -79,8 +76,7 @@ struct SYS_VAR;
 char *caching_sha2_rsa_private_key_path;
 char *caching_sha2_rsa_public_key_path;
 bool caching_sha2_auto_generate_rsa_keys = true;
-Rsa_authentication_keys *g_caching_sha2_rsa_keys = nullptr;
-int caching_sha2_digest_rounds = 0;
+Rsa_authentication_keys *g_caching_sha2_rsa_keys = 0;
 
 namespace sha2_password {
 using std::min;
@@ -184,7 +180,8 @@ static PSI_rwlock_info all_rwlocks[] = {
 */
 
 Caching_sha2_password::Caching_sha2_password(
-    MYSQL_PLUGIN plugin_handle, size_t stored_digest_rounds,
+    MYSQL_PLUGIN plugin_handle,
+    size_t stored_digest_rounds,     /* = DEFAULT_STORED_DIGEST_ROUNDS */
     unsigned int fast_digest_rounds, /* = DEFAULT_FAST_DIGEST_ROUNDS */
     Digest_info digest_type)         /* = Digest_info::SHA256_DIGEST */
     : m_plugin_info(plugin_handle),
@@ -219,10 +216,10 @@ Caching_sha2_password::~Caching_sha2_password() {
     b. Hash iteration count
     c. Expected hash
   2. Use plaintext password, salt and hash iteration count to generate
-     hash.
-  3. Validate generated hash against expected hash.
+     hash
+  3. Validate generated hash against expected hash
 
-  In case of successful authentication, update password cache.
+  In case of successful authentication, update password cache
 
   @param [in] authorization_id   User information
   @param [in] serialized_string        Information retrieved from
@@ -238,11 +235,11 @@ std::pair<bool, bool> Caching_sha2_password::authenticate(
     const std::string &plaintext_password) {
   DBUG_TRACE;
 
-  /* Don't process the password if it is longer than maximum limit. */
+  /* Don't process the password if it is longer than maximum limit */
   if (plaintext_password.length() > CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH)
     return std::make_pair(true, false);
 
-  /* Empty authentication string. */
+  /* Empty authentication string */
   if (!serialized_string[0].length())
     return std::make_pair(plaintext_password.length() ? true : false, false);
 
@@ -274,7 +271,8 @@ std::pair<bool, bool> Caching_sha2_password::authenticate(
     */
 
     if (this->generate_sha2_multi_hash(plaintext_password, random,
-                                       &generated_digest, iterations)) {
+                                       &generated_digest,
+                                       m_stored_digest_rounds)) {
       if (m_plugin_info)
         LogPluginErr(ERROR_LEVEL,
                      ER_SHA_PWD_FAILED_TO_GENERATE_MULTI_ROUND_HASH,
@@ -309,12 +307,12 @@ std::pair<bool, bool> Caching_sha2_password::authenticate(
         sha2_cache_entry stored_digest;
         m_cache.search(authorization_id, stored_digest);
 
-        /* Same digest is already added, so just return. */
+        /* Same digest is already added, so just return */
         if (memcmp(fast_digest.digest_buffer[i], stored_digest.digest_buffer[i],
                    sizeof(fast_digest.digest_buffer[i])) == 0)
           return std::make_pair(false, second);
 
-        /* Update the digest. */
+        /* Update the digest */
         uint retain_index = i ? 0 : 1;
         memcpy(fast_digest.digest_buffer[retain_index],
                stored_digest.digest_buffer[retain_index],
@@ -475,14 +473,14 @@ bool Caching_sha2_password::deserialize(const std::string &serialized_string,
   }
   std::string iteration_info =
       serialized_string.substr(delimiter + 1, ITERATION_LENGTH);
-  unsigned long int iteration_count =
-      strtoul(iteration_info.c_str(), nullptr, 16);
-  if (!iteration_count) {
+  iterations =
+      std::min((std::stoul(iteration_info, nullptr)) * ITERATION_MULTIPLIER,
+               MAX_ITERATIONS);
+  if (!iterations) {
     DBUG_PRINT("info", ("Digest string is not in expected format."
                         "Invalid iteration count information."));
     return true;
   }
-  iterations = std::min(iteration_count * ITERATION_MULTIPLIER, MAX_ITERATIONS);
 
   /* Salt */
   delimiter = delimiter_2;
@@ -555,13 +553,12 @@ bool Caching_sha2_password::serialize(std::string &serialized_string,
   }
 
   /* Iterations */
-  if (iterations < ITERATION_MULTIPLIER || iterations > MAX_ITERATIONS) {
+  unsigned int iteration_info = iterations / ITERATION_MULTIPLIER;
+  if (!iteration_info || iterations > MAX_ITERATIONS) {
     DBUG_PRINT("info", ("Invalid iteration count information."));
     return true;
   }
-  unsigned int iteration_info = iterations / ITERATION_MULTIPLIER;
-  ss << std::setfill('0') << std::setw(3) << std::uppercase << std::hex
-     << iteration_info << DELIMITER;
+  ss << std::setfill('0') << std::setw(3) << iteration_info << DELIMITER;
   serialized_string = ss.str();
 
   /* Salt */
@@ -602,10 +599,10 @@ bool Caching_sha2_password::generate_fast_digest(
     const std::string &plaintext_password, sha2_cache_entry &digest,
     unsigned int pos) {
   DBUG_TRACE;
-  assert(pos < MAX_PASSWORDS);
+  DBUG_ASSERT(pos < MAX_PASSWORDS);
   SHA256_digest sha256_digest;
   unsigned char digest_buffer[CACHING_SHA2_DIGEST_LENGTH];
-  assert(sizeof(digest.digest_buffer[pos]) == sizeof(digest_buffer));
+  DBUG_ASSERT(sizeof(digest.digest_buffer[pos]) == sizeof(digest_buffer));
 
   if (sha256_digest.update_digest(plaintext_password.c_str(),
                                   plaintext_password.length()) ||
@@ -660,7 +657,7 @@ bool Caching_sha2_password::generate_sha2_multi_hash(const std::string &source,
     case Digest_info::SHA256_DIGEST: {
       char buffer[CRYPT_MAX_PASSWORD_SIZE + 1];
       memset(buffer, 0, sizeof(buffer));
-      assert(source.length() <= CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH);
+      DBUG_ASSERT(source.length() <= CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH);
       my_crypt_genhash(buffer, CRYPT_MAX_PASSWORD_SIZE, source.c_str(),
                        source.length(), random.c_str(), nullptr, &iterations);
 
@@ -673,7 +670,7 @@ bool Caching_sha2_password::generate_sha2_multi_hash(const std::string &source,
       break;
     }
     default:
-      assert(false);
+      DBUG_ASSERT(false);
       return true;
   }
   return false;
@@ -727,8 +724,12 @@ bool Caching_sha2_password::validate_hash(const std::string serialized_string) {
 /** Length of encrypted packet */
 const int MAX_CIPHER_LENGTH = 1024;
 
+/** Default iteration count */
+const int CACHING_SHA2_PASSWORD_ITERATIONS =
+    sha2_password::DEFAULT_STORED_DIGEST_ROUNDS;
+
 /** Caching_sha2_password handle */
-sha2_password::Caching_sha2_password *g_caching_sha2_password = nullptr;
+sha2_password::Caching_sha2_password *g_caching_sha2_password = 0;
 
 /** caching_sha2_password plugin handle - Mostly used for logging */
 static MYSQL_PLUGIN caching_sha2_auth_plugin_ref;
@@ -739,6 +740,19 @@ static int my_vio_is_secure(MYSQL_PLUGIN_VIO *vio) {
   return is_secure_transport(mpvio->protocol->get_vio()->type);
 }
 
+/**
+  Check if server has valid public key/private key
+  pair for RSA communication.
+
+  @return
+    @retval false RSA support is available
+    @retval true RSA support is not available
+*/
+bool caching_sha2_rsa_auth_status() {
+  return (!g_caching_sha2_rsa_keys->get_private_key() ||
+          !g_caching_sha2_rsa_keys->get_public_key());
+}
+
 /**
   Save the scramble in mpvio for future re-use.
 
@@ -878,7 +892,7 @@ static char perform_full_authentication = '\4';
        if (Client: Connection check) then
          -[#red]-> [Priority#1:\nTCP with TLS OR Socket Or\nShared Memory connection:\nSend password to server] "Server: received password"
        else
-         if (Priority#2:\nPublic key available) then
+         if (Priorty#2:\nPublic key available) then
            -[#yellow]-> [yes] "Client: Public key available"
          else
            if (No\nPriority#3: Should client get\nserver's public key) then
@@ -929,13 +943,8 @@ static int caching_sha2_password_authenticate(MYSQL_PLUGIN_VIO *vio,
   char scramble[SCRAMBLE_LENGTH + 1];
   int cipher_length = 0;
   unsigned char plain_text[MAX_CIPHER_LENGTH + 1];
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-  EVP_PKEY *private_key = nullptr;
-  EVP_PKEY *public_key = nullptr;
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-  RSA *private_key = nullptr;
-  RSA *public_key = nullptr;
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+  RSA *private_key = NULL;
+  RSA *public_key = NULL;
 
   generate_user_salt(scramble, SCRAMBLE_LENGTH + 1);
 
@@ -981,7 +990,7 @@ static int caching_sha2_password_authenticate(MYSQL_PLUGIN_VIO *vio,
   MPVIO_EXT *mpvio = (MPVIO_EXT *)vio;
   std::string authorization_id;
   const char *hostname = mpvio->acl_user->host.get_host();
-  make_hash_key(info->authenticated_as, hostname ? hostname : nullptr,
+  make_hash_key(info->authenticated_as, hostname ? hostname : NULL,
                 authorization_id);
 
   if (pkt_len != sha2_password::CACHING_SHA2_DIGEST_LENGTH) return CR_ERROR;
@@ -1031,7 +1040,7 @@ static int caching_sha2_password_authenticate(MYSQL_PLUGIN_VIO *vio,
     public_key = g_caching_sha2_rsa_keys->get_public_key();
 
     /* Without the keys encryption isn't possible. */
-    if (private_key == nullptr || public_key == nullptr) {
+    if (private_key == NULL || public_key == NULL) {
       if (caching_sha2_auth_plugin_ref)
         LogPluginErr(ERROR_LEVEL, ER_SHA_PWD_AUTH_REQUIRES_RSA_OR_SSL);
       return CR_ERROR;
@@ -1071,9 +1080,8 @@ static int caching_sha2_password_authenticate(MYSQL_PLUGIN_VIO *vio,
     if (pkt_len != cipher_length) return CR_ERROR;
 
     /* Decrypt password */
-    if (decrypt_RSA_private_key(pkt, cipher_length, plain_text,
-                                sizeof(plain_text) - 1, private_key))
-      return CR_ERROR;
+    RSA_private_decrypt(cipher_length, pkt, plain_text, private_key,
+                        RSA_PKCS1_OAEP_PADDING);
 
     plain_text[cipher_length] = '\0';  // safety
     xor_string((char *)plain_text, cipher_length, (char *)scramble,
@@ -1152,8 +1160,7 @@ static int caching_sha2_password_generate(char *outbuf, unsigned int *buflen,
   random.assign(salt, sha2_password::SALT_LENGTH);
 
   if (g_caching_sha2_password->generate_sha2_multi_hash(
-          source, random, &digest,
-          g_caching_sha2_password->get_digest_rounds()))
+          source, random, &digest, CACHING_SHA2_PASSWORD_ITERATIONS))
     return 1;
 
   if (g_caching_sha2_password->serialize(
@@ -1202,32 +1209,30 @@ static int caching_sha2_password_validate(char *const inbuf,
   @returns Always returns success (0)
 */
 
-static int caching_sha2_password_salt(const char *password [[maybe_unused]],
-                                      unsigned int password_len
-                                      [[maybe_unused]],
-                                      unsigned char *salt [[maybe_unused]],
-                                      unsigned char *salt_len) {
+static int caching_sha2_password_salt(
+    const char *password MY_ATTRIBUTE((unused)),
+    unsigned int password_len MY_ATTRIBUTE((unused)),
+    unsigned char *salt MY_ATTRIBUTE((unused)), unsigned char *salt_len) {
   DBUG_TRACE;
   *salt_len = 0;
   return 0;
 }
 
 /*
-Initialize caching_sha2_password plugin
+  Initialize caching_sha2_password plugin
 
-@param [in] plugin_ref Plugin structure handle
+  @param [in] plugin_ref Plugin structure handle
 
-@returns Status of plugin initialization
-@retval 0 Success
-@retval 1 Error
+  @returns Status of plugin initialization
+    @retval 0 Success
+    @retval 1 Error
 */
 
 static int caching_sha2_authentication_init(MYSQL_PLUGIN plugin_ref) {
   DBUG_TRACE;
-
   caching_sha2_auth_plugin_ref = plugin_ref;
-  g_caching_sha2_password = new sha2_password::Caching_sha2_password(
-      caching_sha2_auth_plugin_ref, caching_sha2_digest_rounds);
+  g_caching_sha2_password =
+      new sha2_password::Caching_sha2_password(caching_sha2_auth_plugin_ref);
   if (!g_caching_sha2_password) return 1;
 
   return 0;
@@ -1241,11 +1246,12 @@ static int caching_sha2_authentication_init(MYSQL_PLUGIN plugin_ref) {
   @returns Always returns success
 */
 
-static int caching_sha2_authentication_deinit(void *arg [[maybe_unused]]) {
+static int caching_sha2_authentication_deinit(
+    void *arg MY_ATTRIBUTE((unused))) {
   DBUG_TRACE;
   if (g_caching_sha2_password) {
     delete g_caching_sha2_password;
-    g_caching_sha2_password = nullptr;
+    g_caching_sha2_password = 0;
   }
   return 0;
 }
@@ -1280,8 +1286,8 @@ static int compare_caching_sha2_password_with_hash(
   sha2_password::Digest_info digest_type;
   size_t iterations;
 
-  assert(cleartext_length <=
-         sha2_password::CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH);
+  DBUG_ASSERT(cleartext_length <=
+              sha2_password::CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH);
   if (cleartext_length >
       sha2_password::CACHING_SHA2_PASSWORD_MAX_PASSWORD_LENGTH)
     return -1;
@@ -1313,11 +1319,9 @@ static int compare_caching_sha2_password_with_hash(
   @param [out] var Status variable structure
   @param [in]  buff Value buffer
 */
-static int show_caching_sha2_password_rsa_public_key(MYSQL_THD thd
-                                                     [[maybe_unused]],
-                                                     SHOW_VAR *var,
-                                                     char *buff
-                                                     [[maybe_unused]]) {
+static int show_caching_sha2_password_rsa_public_key(
+    MYSQL_THD thd MY_ATTRIBUTE((unused)), SHOW_VAR *var,
+    char *buff MY_ATTRIBUTE((unused))) {
   var->type = SHOW_CHAR;
   var->value =
       const_cast<char *>(g_caching_sha2_rsa_keys->get_public_key_as_pem());
@@ -1338,13 +1342,13 @@ static MYSQL_SYSVAR_STR(
     private_key_path, caching_sha2_rsa_private_key_path,
     PLUGIN_VAR_READONLY | PLUGIN_VAR_NOPERSIST,
     "A fully qualified path to the private RSA key used for authentication.",
-    nullptr, nullptr, AUTH_DEFAULT_RSA_PRIVATE_KEY);
+    NULL, NULL, AUTH_DEFAULT_RSA_PRIVATE_KEY);
 
 static MYSQL_SYSVAR_STR(
     public_key_path, caching_sha2_rsa_public_key_path,
     PLUGIN_VAR_READONLY | PLUGIN_VAR_NOPERSIST,
     "A fully qualified path to the public RSA key used for authentication.",
-    nullptr, nullptr, AUTH_DEFAULT_RSA_PUBLIC_KEY);
+    NULL, NULL, AUTH_DEFAULT_RSA_PUBLIC_KEY);
 
 static MYSQL_SYSVAR_BOOL(
     auto_generate_rsa_keys, caching_sha2_auto_generate_rsa_keys,
@@ -1352,32 +1356,19 @@ static MYSQL_SYSVAR_BOOL(
     "Auto generate RSA keys at server startup if corresponding "
     "system variables are not specified and key files are not present "
     "at the default location.",
-    nullptr, nullptr, true);
-
-static MYSQL_SYSVAR_INT(
-    digest_rounds,               // Name.
-    caching_sha2_digest_rounds,  // Variable.
-    PLUGIN_VAR_READONLY,         // Argument optional for cmd line
-    "Number of SHA2 rounds to be done when storing a password hash onto disk.",
-    nullptr,                                      // Check function.
-    nullptr,                                      // Update function.
-    sha2_password::DEFAULT_STORED_DIGEST_ROUNDS,  // Default value.
-    sha2_password::MIN_STORED_DIGEST_ROUNDS,      // Min value.
-    sha2_password::MAX_STORED_DIGEST_ROUNDS,      // Max value.
-    1                                             // Block size.
-);
+    NULL, NULL, true);
 
 /** Array of system variables. Used in plugin declaration. */
 static SYS_VAR *caching_sha2_password_sysvars[] = {
     MYSQL_SYSVAR(private_key_path), MYSQL_SYSVAR(public_key_path),
-    MYSQL_SYSVAR(auto_generate_rsa_keys), MYSQL_SYSVAR(digest_rounds), nullptr};
+    MYSQL_SYSVAR(auto_generate_rsa_keys), 0};
 
 /** Array of status variables. Used in plugin declaration. */
 static SHOW_VAR caching_sha2_password_status_variables[] = {
     {"Caching_sha2_password_rsa_public_key",
      (char *)&show_caching_sha2_password_rsa_public_key, SHOW_FUNC,
      SHOW_SCOPE_GLOBAL},
-    {nullptr, nullptr, enum_mysql_show_type(0), enum_mysql_show_scope(0)}};
+    {0, 0, enum_mysql_show_type(0), enum_mysql_show_scope(0)}};
 
 /**
   Handle an authentication audit event.
@@ -1412,7 +1403,7 @@ static int sha2_cache_cleaner_notify(MYSQL_THD, mysql_event_class_t event_class,
     if (subclass == MYSQL_AUDIT_AUTHENTICATION_CREDENTIAL_CHANGE ||
         subclass == MYSQL_AUDIT_AUTHENTICATION_AUTHID_RENAME ||
         subclass == MYSQL_AUDIT_AUTHENTICATION_AUTHID_DROP) {
-      assert(
+      DBUG_ASSERT(
           authentication_event->user.str[authentication_event->user.length] ==
           '\0');
       std::string authorization_id;
@@ -1427,7 +1418,7 @@ static int sha2_cache_cleaner_notify(MYSQL_THD, mysql_event_class_t event_class,
 /** st_mysql_audit for sha2_cache_cleaner plugin */
 struct st_mysql_audit sha2_cache_cleaner = {
     MYSQL_AUDIT_INTERFACE_VERSION, /* interface version */
-    nullptr,                       /* release_thd() */
+    NULL,                          /* release_thd() */
     sha2_cache_cleaner_notify,     /* event_notify() */
     {
         0, /* MYSQL_AUDIT_GENERAL_CLASS */
@@ -1447,13 +1438,13 @@ struct st_mysql_audit sha2_cache_cleaner = {
     }};
 
 /** Init function for sha2_cache_cleaner */
-static int caching_sha2_cache_cleaner_init(MYSQL_PLUGIN plugin_info
-                                           [[maybe_unused]]) {
+static int caching_sha2_cache_cleaner_init(
+    MYSQL_PLUGIN plugin_info MY_ATTRIBUTE((unused))) {
   return 0;
 }
 
 /** Deinit function for sha2_cache_cleaner */
-static int caching_sha2_cache_cleaner_deinit(void *arg [[maybe_unused]]) {
+static int caching_sha2_cache_cleaner_deinit(void *arg MY_ATTRIBUTE((unused))) {
   return 0;
 }
 
@@ -1466,31 +1457,31 @@ mysql_declare_plugin(caching_sha2_password){
     &caching_sha2_auth_handler,  /* type specific descriptor      */
     Cached_authentication_plugins::get_plugin_name(
         PLUGIN_CACHING_SHA2_PASSWORD),      /* plugin name          */
-    PLUGIN_AUTHOR_ORACLE,                   /* author                        */
+    "Oracle",                               /* author                        */
     "Caching sha2 authentication",          /* description                   */
     PLUGIN_LICENSE_GPL,                     /* license                       */
     caching_sha2_authentication_init,       /* plugin initializer            */
-    nullptr,                                /* Uninstall notifier            */
+    NULL,                                   /* Uninstall notifier            */
     caching_sha2_authentication_deinit,     /* plugin deinitializer          */
     0x0100,                                 /* version (1.0)                 */
     caching_sha2_password_status_variables, /* status variables              */
     caching_sha2_password_sysvars,          /* system variables              */
-    nullptr,                                /* reserved                      */
+    NULL,                                   /* reserverd                     */
     0,                                      /* flags                         */
 },
     {
         MYSQL_AUDIT_PLUGIN,   /* plugin type                   */
         &sha2_cache_cleaner,  /* type specific descriptor      */
         "sha2_cache_cleaner", /* plugin name                   */
-        PLUGIN_AUTHOR_ORACLE, /* author                        */
+        "Oracle Inc",         /* author                        */
         "Cache cleaner for Caching sha2 authentication", /* description */
         PLUGIN_LICENSE_GPL,                /* license                       */
         caching_sha2_cache_cleaner_init,   /* plugin initializer            */
-        nullptr,                           /* Uninstall notifier            */
+        NULL,                              /* Uninstall notifier            */
         caching_sha2_cache_cleaner_deinit, /* plugin deinitializer          */
         0x0100,                            /* version (1.0)                 */
-        nullptr,                           /* status variables              */
-        nullptr,                           /* system variables              */
-        nullptr,                           /* reserved                      */
+        NULL,                              /* status variables              */
+        NULL,                              /* system variables              */
+        NULL,                              /* reserverd                     */
         0                                  /* flags                         */
     } mysql_declare_plugin_end;
