@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,6 @@
 #ifndef SQL_USER_CACHE_INCLUDED
 #define SQL_USER_CACHE_INCLUDED
 
-#include <assert.h>
 #include <string.h>
 #include <sys/types.h>
 #include <atomic>
@@ -32,7 +30,6 @@
 #include <boost/graph/graph_traits.hpp>
 #include <boost/graph/properties.hpp>
 #include <boost/pending/property.hpp>
-#include <list>
 #include <memory>
 #include <string>
 #include <unordered_map>
@@ -43,21 +40,17 @@
 #include "map_helpers.h"
 #include "mf_wcomp.h"  // wild_many, wild_one, wild_prefix
 #include "my_alloc.h"
-#include "my_compiler.h"
 #include "my_inttypes.h"
 #include "my_sharedlib.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/mysql_mutex_bits.h"
+#include "mysql/components/services/mysql_mutex_bits.h"
 #include "mysql/mysql_lex_string.h"
 #include "mysql_com.h"   // SCRAMBLE_LENGTH
 #include "mysql_time.h"  // MYSQL_TIME
 #include "sql/auth/auth_common.h"
 #include "sql/auth/auth_internal.h"  // List_of_authid, Authid
-#include "sql/auth/partial_revokes.h"
-#include "sql/malloc_allocator.h"
-#include "sql/psi_memory_key.h"
-#include "sql/sql_connect.h"  // USER_RESOURCES
-#include "violite.h"          // SSL_type
+#include "sql/sql_connect.h"         // USER_RESOURCES
+#include "violite.h"                 // SSL_type
 
 /* Forward declarations */
 class Security_context;
@@ -67,85 +60,26 @@ struct TABLE;
 template <typename Element_type, size_t Prealloc>
 class Prealloced_array;
 class Acl_restrictions;
-enum class Lex_acl_attrib_udyn;
+class Restrictions;
 
 /* Classes */
 
 class ACL_HOST_AND_IP {
- public:
-  /**
-    IP mask type enum.
-  */
-  enum enum_ip_mask_type {
-    /**
-    Only IP is specified.
-    */
-    ip_mask_type_implicit,
-    /**
-    IP specified with a mask in a CIDR form.
-    */
-    ip_mask_type_cidr,
-    /**
-    IP specified with a mask in a form of a subnet.
-    */
-    ip_mask_type_subnet
-  };
-
   const char *hostname;
   size_t hostname_length;
   long ip, ip_mask;  // Used with masked ip:s
-  /**
-    IP mask type.
-  */
-  enum_ip_mask_type ip_mask_type;
 
-  /**
-    IP mask parsing in the CIDR format.
-
-    @param[in]  ip_arg Buffer containing CIDR mask value.
-    @param[out] val    Numeric IP mask value on success.
-
-    @retval false Parsing succeeded.
-    @retval true  Parsing failed.
-  */
-  static bool calc_cidr_mask(const char *ip_arg, long *val);
-
-  /**
-    IP mask parsing in the subnet format.
-
-    @param[in]  ip_arg Buffer containing subnet mask value.
-    @param[out] val    Numeric IP mask value on success.
-
-    @retval false Parsing succeeded.
-    @retval true  Parsing failed.
-  */
-  static bool calc_ip_mask(const char *ip_arg, long *val);
-
-  /**
-    IP parsing.
-
-    @param[in]  ip_arg Buffer containing IP value.
-    @param[out] val    Numeric IP value on success.
-
-    @retval !nullptr Parsing succeeded. Returned value is the pointer following
-    the buffer holding the IP.
-    @retval nullptr  Parsing failed. The buffer does not contain valid IP value.
-  */
-  static const char *calc_ip(const char *ip_arg, long *val);
+  const char *calc_ip(const char *ip_arg, long *val, char end);
 
  public:
   ACL_HOST_AND_IP()
-      : hostname(nullptr),
-        hostname_length(0),
-        ip(0),
-        ip_mask(0),
-        ip_mask_type(ip_mask_type_implicit) {}
-  const char *get_host() const { return hostname ? hostname : ""; }
+      : hostname(nullptr), hostname_length(0), ip(0), ip_mask(0) {}
+  const char *get_host() const { return hostname; }
   size_t get_host_len() const { return hostname_length; }
 
   bool has_wildcard() {
-    return (strchr(get_host(), wild_many) || strchr(get_host(), wild_one) ||
-            (ip_mask && (ip_mask != (long)UINT_MAX32)));
+    return (strchr(hostname, wild_many) || strchr(hostname, wild_one) ||
+            ip_mask);
   }
 
   bool check_allow_all_hosts() {
@@ -162,55 +96,15 @@ class ACL_ACCESS {
   ACL_ACCESS() : host(), sort(0), access(0) {}
   ACL_HOST_AND_IP host;
   ulong sort;
-  Access_bitmask access;
+  ulong access;
 };
 
-/**
-  @class ACL_compare
-
-  Class that compares ACL_ACCESS objects. Used in std::sort functions.
-*/
 class ACL_compare {
  public:
-  /**
-    Determine sort order of two user accounts.
-
-    ACL_ACCESS with IP specified is sorted before host name.
-
-    @param [in] a First object to compare.
-    @param [in] b Second object to compare.
-
-    @retval true  First element goes first.
-    @retval false Second element goes first.
-  */
   bool operator()(const ACL_ACCESS &a, const ACL_ACCESS &b);
   bool operator()(const ACL_ACCESS *a, const ACL_ACCESS *b);
 };
 
-/**
-  @class ACL_USER_compare
-
-  Class that compares ACL_USER objects.
-*/
-class ACL_USER_compare {
- public:
-  /**
-    Determine sort order of two user accounts.
-
-    ACL_USER with IP specified is sorted before host name.
-    Non anonymous user is sorted before anonymous user, when properties of both
-    are equal.
-
-    @param [in] a First object to compare.
-    @param [in] b Second object to compare.
-
-    @retval true  First element goes first.
-    @retval false Second element goes first.
-  */
-  bool operator()(const ACL_USER &a, const ACL_USER &b);
-  bool operator()(const ACL_USER *a, const ACL_USER *b);
-};
-
 /* ACL_HOST is used if no host is specified */
 
 class ACL_HOST : public ACL_ACCESS {
@@ -300,66 +194,11 @@ class ACL_USER : public ACL_ACCESS {
 
   ACL_USER *copy(MEM_ROOT *root);
   ACL_USER();
-
-  void set_user(MEM_ROOT *mem, const char *user_arg);
-  void set_host(MEM_ROOT *mem, const char *host_arg);
-  void set_mfa(MEM_ROOT *mem, I_multi_factor_auth *m);
-  size_t get_username_length() const { return user ? strlen(user) : 0; }
-  class Password_locked_state {
-   public:
-    bool is_active() const {
-      return m_password_lock_time_days != 0 && m_failed_login_attempts != 0;
-    }
-    bool is_default() const {
-      return (m_remaining_login_attempts == m_failed_login_attempts &&
-              m_daynr_locked == 0);
-    }
-    int get_password_lock_time_days() const {
-      return m_password_lock_time_days;
-    }
-    uint get_failed_login_attempts() const { return m_failed_login_attempts; }
-    uint get_remaining_login_attempts() const {
-      return m_remaining_login_attempts;
-    }
-    long get_daynr_locked() const { return m_daynr_locked; }
-    void set_temporary_lock_state_parameters(uint remaining_login_attempts,
-                                             long daynr_locked);
-    void set_parameters(int password_lock_time_days,
-                        uint failed_login_attempts);
-    bool update(THD *thd, bool successful_login, long *ret_days_remaining);
-    Password_locked_state()
-        : m_password_lock_time_days(0),
-          m_failed_login_attempts(0),
-          m_remaining_login_attempts(0),
-          m_daynr_locked(0) {}
-
-   protected:
-    /**
-      read from the user config. The number of days to keep the account locked
-    */
-    int m_password_lock_time_days;
-    /**
-      read from the user config. The number of failed login attempts before the
-      account is locked
-    */
-    uint m_failed_login_attempts;
-    /**
-      The remaining login tries, valid only if @ref m_failed_login_attempts and
-      @ref m_password_lock_time_days are non-zero
-    */
-    uint m_remaining_login_attempts;
-    /** The day the account is locked, 0 if not locked */
-    long m_daynr_locked;
-  } password_locked_state;
-  I_multi_factor_auth *m_mfa;
 };
 
 class ACL_DB : public ACL_ACCESS {
  public:
   char *user, *db;
-
-  void set_user(MEM_ROOT *mem, const char *user_arg);
-  void set_host(MEM_ROOT *mem, const char *host_arg);
 };
 
 class ACL_PROXY_USER : public ACL_ACCESS {
@@ -379,7 +218,7 @@ class ACL_PROXY_USER : public ACL_ACCESS {
   } old_acl_proxy_users;
 
  public:
-  ACL_PROXY_USER() = default;
+  ACL_PROXY_USER() {}
 
   void init(const char *host_arg, const char *user_arg,
             const char *proxied_host_arg, const char *proxied_user_arg,
@@ -395,8 +234,9 @@ class ACL_PROXY_USER : public ACL_ACCESS {
   const char *get_user() { return user; }
   const char *get_proxied_user() { return proxied_user; }
   const char *get_proxied_host() { return proxied_host.get_host(); }
-  void set_user(MEM_ROOT *mem, const char *user_arg);
-  void set_host(MEM_ROOT *mem, const char *host_arg);
+  void set_user(MEM_ROOT *mem, const char *user_arg) {
+    user = user_arg && *user_arg ? strdup_root(mem, user_arg) : NULL;
+  }
 
   bool check_validity(bool check_no_resolve);
 
@@ -404,7 +244,7 @@ class ACL_PROXY_USER : public ACL_ACCESS {
                const char *proxied_user_arg, bool any_proxy_user);
 
   inline static bool auth_element_equals(const char *a, const char *b) {
-    return (a == b || (a != nullptr && b != nullptr && !strcmp(a, b)));
+    return (a == b || (a != NULL && b != NULL && !strcmp(a, b)));
   }
 
   bool pk_equals(ACL_PROXY_USER *grant);
@@ -417,7 +257,7 @@ class ACL_PROXY_USER : public ACL_ACCESS {
          (host.get_host() && host_arg && !strcmp(host.get_host(), host_arg))));
   }
 
-  void print_grant(THD *thd, String *str);
+  void print_grant(String *str);
 
   void set_data(ACL_PROXY_USER *grant) { with_grant = grant->with_grant; }
 
@@ -432,26 +272,20 @@ class ACL_PROXY_USER : public ACL_ACCESS {
                                const LEX_CSTRING &proxied_host,
                                const LEX_CSTRING &proxied_user, bool with_grant,
                                const char *grantor);
-
-  size_t get_user_length() const { return user ? strlen(user) : 0; }
-
-  size_t get_proxied_user_length() const {
-    return proxied_user ? strlen(proxied_user) : 0;
-  }
 };
 
 class acl_entry {
  public:
-  Access_bitmask access;
+  ulong access;
   uint16 length;
   char key[1];  // Key will be stored here
 };
 
 class GRANT_COLUMN {
  public:
-  Access_bitmask rights;
+  ulong rights;
   std::string column;
-  GRANT_COLUMN(String &c, Access_bitmask y);
+  GRANT_COLUMN(String &c, ulong y);
 };
 
 class GRANT_NAME {
@@ -460,13 +294,13 @@ class GRANT_NAME {
   char *db;
   const char *user;
   char *tname;
-  Access_bitmask privs;
+  ulong privs;
   ulong sort;
   std::string hash_key;
   GRANT_NAME(const char *h, const char *d, const char *u, const char *t,
-             Access_bitmask p, bool is_routine);
+             ulong p, bool is_routine);
   GRANT_NAME(TABLE *form, bool is_routine);
-  virtual ~GRANT_NAME() = default;
+  virtual ~GRANT_NAME() {}
   virtual bool ok() { return privs != 0; }
   void set_user_details(const char *h, const char *d, const char *u,
                         const char *t, bool is_routine);
@@ -474,17 +308,17 @@ class GRANT_NAME {
 
 class GRANT_TABLE : public GRANT_NAME {
  public:
-  Access_bitmask cols;
+  ulong cols;
   collation_unordered_multimap<std::string,
                                unique_ptr_destroy_only<GRANT_COLUMN>>
       hash_columns;
 
   GRANT_TABLE(const char *h, const char *d, const char *u, const char *t,
-              Access_bitmask p, Access_bitmask c);
+              ulong p, ulong c);
   explicit GRANT_TABLE(TABLE *form);
   bool init(TABLE *col_privs);
-  ~GRANT_TABLE() override;
-  bool ok() override { return privs != 0 || cols != 0; }
+  ~GRANT_TABLE();
+  bool ok() { return privs != 0 || cols != 0; }
 };
 
 /*
@@ -504,12 +338,13 @@ class Acl_cache_allocator : public Malloc_allocator<T> {
   };
 
   template <class U>
-  Acl_cache_allocator(const Acl_cache_allocator<U> &other [[maybe_unused]])
+  Acl_cache_allocator(
+      const Acl_cache_allocator<U> &other MY_ATTRIBUTE((unused)))
       : Malloc_allocator<T>(key_memory_acl_cache) {}
 
   template <class U>
-  Acl_cache_allocator &operator=(const Acl_cache_allocator<U> &other
-                                 [[maybe_unused]]) {}
+  Acl_cache_allocator &operator=(
+      const Acl_cache_allocator<U> &other MY_ATTRIBUTE((unused))) {}
 };
 typedef Acl_cache_allocator<ACL_USER *> Acl_user_ptr_allocator;
 typedef std::list<ACL_USER *, Acl_user_ptr_allocator> Acl_user_ptr_list;
@@ -577,7 +412,6 @@ inline GRANT_NAME *routine_hash_search(const char *host, const char *ip,
                                        const char *db, const char *user,
                                        const char *tname, bool proc,
                                        bool exact) {
-  assert(proc ? proc_priv_hash : func_priv_hash);
   return name_hash_search(proc ? *proc_priv_hash : *func_priv_hash, host, ip,
                           db, user, tname, exact, true);
 }
@@ -585,7 +419,6 @@ inline GRANT_NAME *routine_hash_search(const char *host, const char *ip,
 inline GRANT_TABLE *table_hash_search(const char *host, const char *ip,
                                       const char *db, const char *user,
                                       const char *tname, bool exact) {
-  assert(column_priv_hash);
   return name_hash_search(*column_priv_hash, host, ip, db, user, tname, exact,
                           false);
 }
@@ -634,8 +467,7 @@ typedef boost::graph_traits<Granted_roles_graph>::edge_descriptor
 /** The datatype of the map between authids and graph vertex descriptors */
 typedef std::unordered_map<std::string, Role_vertex_descriptor> Role_index_map;
 
-/** The type used for the number of edges incident to a vertex in the graph.
- */
+/** The type used for the number of edges incident to a vertex in the graph. */
 using degree_s_t = boost::graph_traits<Granted_roles_graph>::degree_size_type;
 
 /** The type for the iterator returned by out_edges(). */
@@ -664,7 +496,7 @@ class Acl_map {
   void increase_reference_count();
   void decrease_reference_count();
 
-  Access_bitmask global_acl();
+  ulong global_acl();
   Db_access_map *db_acls();
   Db_access_map *db_wild_acls();
   Table_access_map *table_acls();
@@ -682,7 +514,7 @@ class Acl_map {
   Db_access_map m_db_acls;
   Db_access_map m_db_wild_acls;
   Table_access_map m_table_acls;
-  Access_bitmask m_global_acl;
+  ulong m_global_acl;
   SP_access_map m_sp_acls;
   SP_access_map m_func_acls;
   Grant_acl_set m_with_admin_acls;
@@ -717,30 +549,21 @@ class Acl_cache {
     A new object will also be created if the role graph version counter is
     different than the acl map object's version.
 
-    @param sctx The target Security_context
-    @param uid The target authid
-    @param active_roles A list of active roles
-
-    @return A pointer to an Acl_map
-    @retval !NULL Success
-    @retval NULL A fatal OOM error happened.
+    @param uid
+    @return
   */
   Acl_map *checkout_acl_map(Security_context *sctx, Auth_id_ref &uid,
                             List_of_auth_id_refs &active_roles);
   /**
     When the security context is done with the acl map it calls the cache
     to decrease the reference count on that object.
-    @param map acl map
+    @param map
   */
   void return_acl_map(Acl_map *map);
   /**
     Removes all acl map objects with a references count of zero.
   */
   void flush_cache();
-  /**
-    Removes all acl map objects when shutdown_acl_cache is called.
-  */
-  void clear_acl_cache();
   /**
     Return a lower boundary to the current version count.
   */
@@ -756,6 +579,7 @@ class Acl_cache {
 
     @param version The version of the new map
     @param sctx The associated security context
+    @return
   */
   Acl_map *create_acl_map(uint64 version, Security_context *sctx);
   /** Role graph version counter */
@@ -809,9 +633,9 @@ class Acl_cache_lock_guard {
   Callers must acquire acl_cache_write_lock before to amend the cache.
   Callers should acquire acl_cache_read_lock to probe the cache.
 
-  Acl_restrictions is not part of ACL_USER because as of now latter is POD
-  type class. We use copy-POD for ACL_USER that makes the explicit memory
-  management of its members hard.
+  Acl_restrictions is not part of ACL_USER because as of now latter is POD type
+  class. We use copy-POD for ACL_USER that makes the explicit memory management
+  of its members hard.
 */
 class Acl_restrictions {
  public:
@@ -833,56 +657,4 @@ class Acl_restrictions {
   malloc_unordered_map<std::string, Restrictions> m_restrictions_map;
 };
 
-/**
-  Enables preserving temporary account locking attributes during ACL DDL.
-  Enables restoring temporary account locking attributes after ACL reload.
-
-  This class is used to preserve the state of the accounts being altered by the
-  current ACL statement. The account locking data needs to be preserved since
-  the current state of account locking is not stored into the table and can't be
-  restored from it when the code needs to re-create the ACL caches from the
-  tables.
-
-  When an ACL DDL statement that can modify account locking data starts, a new
-  instance of this class is created and the current in-memory account locking
-  data is preserved for each user that is modified by the statement, if account
-  locking data is not default.
-
-  ACL DDL rollback results in the in-memory ACL cache being re-created during
-  ACL reload.
-
-  After ACL reload:
-  - For all users in the new ACL cache, the temporary account locking state is
-    restored from the old ACL cache, if account locking data is not default.
-  - For specific users in the new ACL cache, the temporary account locking state
-    is restored from the instances of this class that were created at the start
-    of the ACL DDL statement. This needs to be done since these accounts could
-    be dropped (mysql_drop_user), renamed (mysql_rename_user) or altered
-    (mysql_alter_user) in the old ACL cache.
-*/
-class ACL_temporary_lock_state {
- public:
-  ACL_temporary_lock_state(const char *host, const char *user,
-                           uint remaining_login_attempts, long daynr_locked);
-
-  static bool is_modified(ACL_USER *acl_user);
-
-  static ACL_USER *preserve_user_lock_state(const char *host, const char *user,
-                                            Lock_state_list &user_list);
-
-  static void restore_user_lock_state(const char *host, const char *user,
-                                      uint remaining_login_attempts,
-                                      long daynr_locked);
-
-  static void restore_temporary_account_locking(
-      Prealloced_array<ACL_USER, ACL_PREALLOC_SIZE> *old_acl_users,
-      Lock_state_list *modified_user_lock_state_list);
-
- private:
-  const char *m_host;
-  const char *m_user;
-  const uint m_remaining_login_attempts;
-  const long m_daynr_locked;
-};
-
 #endif /* SQL_USER_CACHE_INCLUDED */
