@@ -45,6 +45,7 @@
 #include "sql/current_thd.h"
 #include "sql/mysqld.h"
 #include "sql/sql_class.h"
+#include "sql/table.h"
 
 extern bool initialized;
 
@@ -67,18 +68,18 @@ Security_context::Security_context(const Security_context &src_sctx)
 
 Security_context &Security_context::operator=(
     const Security_context &src_sctx) {
-  DBUG_ENTER("Security_context::operator =");
+  DBUG_TRACE;
 
   if (this != &src_sctx) {
     destroy();
     copy_security_ctx(src_sctx);
   }
 
-  DBUG_RETURN(*this);
+  return *this;
 }
 
 void Security_context::init() {
-  DBUG_ENTER("Security_context::init");
+  DBUG_TRACE;
 
   m_user.set((const char *)0, 0, system_charset_info);
   m_host.set("", 0, system_charset_info);
@@ -96,7 +97,6 @@ void Security_context::init() {
   m_is_skip_grants_user = false;
   m_has_drop_policy = false;
   m_executed_drop_policy = false;
-  DBUG_VOID_RETURN;
 }
 
 void Security_context::logout() {
@@ -133,7 +133,7 @@ void Security_context::set_drop_policy(
 }
 
 void Security_context::destroy() {
-  DBUG_ENTER("Security_context::destroy");
+  DBUG_TRACE;
   execute_drop_policy();
   if (m_acl_map) {
     DBUG_PRINT(
@@ -162,7 +162,6 @@ void Security_context::destroy() {
   m_password_expired = false;
   m_is_skip_grants_user = false;
   clear_db_restrictions();
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -175,7 +174,7 @@ void Security_context::destroy() {
 */
 void Security_context::skip_grants(const char *user /*= "skip-grants user"*/,
                                    const char *host /*= "skip-grants host"*/) {
-  DBUG_ENTER("Security_context::skip_grants");
+  DBUG_TRACE;
 
   /* privileges for the user are unknown everything is allowed */
   set_host_or_ip_ptr("", 0);
@@ -191,7 +190,6 @@ void Security_context::skip_grants(const char *user /*= "skip-grants user"*/,
   if (m_thd && m_thd->security_context() == this) {
     m_thd->set_system_user(true);
   }
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -201,7 +199,7 @@ void Security_context::skip_grants(const char *user /*= "skip-grants user"*/,
 */
 
 void Security_context::copy_security_ctx(const Security_context &src_sctx) {
-  DBUG_ENTER("Security_context::copy_security_ctx");
+  DBUG_TRACE;
 
   assign_user(src_sctx.m_user.ptr(), src_sctx.m_user.length());
   assign_host(src_sctx.m_host.ptr(), src_sctx.m_host.length());
@@ -222,7 +220,6 @@ void Security_context::copy_security_ctx(const Security_context &src_sctx) {
   m_has_drop_policy = false;  // you cannot copy a drop policy
   m_executed_drop_policy = false;
   m_restrictions = src_sctx.restrictions();
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -280,10 +277,10 @@ void Security_context::copy_security_ctx(const Security_context &src_sctx) {
 
 bool Security_context::change_security_context(
     THD *thd, const LEX_CSTRING &definer_user, const LEX_CSTRING &definer_host,
-    LEX_STRING *db, Security_context **backup, bool force) {
+    const char *db, Security_context **backup, bool force) {
   bool needs_change;
 
-  DBUG_ENTER("Security_context::change_security_context");
+  DBUG_TRACE;
 
   DBUG_ASSERT(definer_user.str && definer_host.str);
 
@@ -293,18 +290,16 @@ bool Security_context::change_security_context(
        my_strcasecmp(system_charset_info, definer_host.str,
                      thd->security_context()->priv_host().str));
   if (needs_change || force) {
-    if (acl_getroot(thd, this, const_cast<char *>(definer_user.str),
-                    const_cast<char *>(definer_host.str),
-                    const_cast<char *>(definer_host.str),
-                    (db ? db->str : nullptr))) {
+    if (acl_getroot(thd, this, definer_user.str, definer_host.str,
+                    definer_host.str, db)) {
       my_error(ER_NO_SUCH_USER, MYF(0), definer_user.str, definer_host.str);
-      DBUG_RETURN(true);
+      return true;
     }
     *backup = thd->security_context();
     thd->set_security_context(this);
   }
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 void Security_context::restore_security_context(THD *thd,
@@ -313,24 +308,24 @@ void Security_context::restore_security_context(THD *thd,
 }
 
 bool Security_context::user_matches(Security_context *them) {
-  DBUG_ENTER("Security_context::user_matches");
+  DBUG_TRACE;
 
   const char *them_user = them->user().str;
 
-  DBUG_RETURN((m_user.ptr() != NULL) && (them_user != NULL) &&
-              !strcmp(m_user.ptr(), them_user));
+  return (m_user.ptr() != NULL) && (them_user != NULL) &&
+         !strcmp(m_user.ptr(), them_user);
 }
 
 bool Security_context::check_access(ulong want_access,
                                     const std::string &db_name /* = "" */,
                                     bool match_any) {
-  DBUG_ENTER("Security_context::check_access");
+  DBUG_TRACE;
   if ((want_access & DB_ACLS) &&
       (is_access_restricted_on_db(want_access, db_name))) {
-    DBUG_RETURN(false);
+    return false;
   }
-  DBUG_RETURN((match_any ? (m_master_access & want_access)
-                         : ((m_master_access & want_access) == want_access)));
+  return (match_any ? (m_master_access & want_access)
+                    : ((m_master_access & want_access) == want_access));
 }
 
 ulong Security_context::master_access(const std::string &db_name) const {
@@ -381,7 +376,7 @@ int Security_context::activate_role(LEX_CSTRING role, LEX_CSTRING role_host,
   grant/revoke or flush due to roles is per statement.
 */
 void Security_context::checkout_access_maps(void) {
-  DBUG_ENTER("Security_context::checkout_access_maps");
+  DBUG_TRACE;
 
   /*
     If we're checkout out a map before we return it now, because we're only
@@ -399,7 +394,7 @@ void Security_context::checkout_access_maps(void) {
     m_acl_map = nullptr;
   }
 
-  if (m_active_roles.size() == 0) DBUG_VOID_RETURN;
+  if (m_active_roles.size() == 0) return;
   ++m_map_checkout_count;
   Auth_id_ref uid;
   uid.first.str = this->m_user.ptr();
@@ -416,7 +411,6 @@ void Security_context::checkout_access_maps(void) {
   } else {
     set_master_access(0);
   }
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -476,8 +470,8 @@ void Security_context::get_active_roles(THD *thd, List<LEX_USER> &list) {
 }
 
 ulong Security_context::db_acl(LEX_CSTRING db, bool use_pattern_scan) const {
-  DBUG_ENTER("Security_context::db_acl");
-  if (m_acl_map == 0 || db.length == 0) DBUG_RETURN(0);
+  DBUG_TRACE;
+  if (m_acl_map == 0 || db.length == 0) return 0;
 
   Db_access_map::iterator it;
   std::string key(db.str, db.length);
@@ -502,15 +496,15 @@ ulong Security_context::db_acl(LEX_CSTRING db, bool use_pattern_scan) const {
           access |= it->second;
         }
       }
-      DBUG_RETURN(filter_access(access, key));
+      return filter_access(access, key);
     } else {
       DBUG_PRINT("info", ("Db %s not found in cache (no pattern matching)",
                           key.c_str()));
-      DBUG_RETURN(0);
+      return 0;
     }
   } else {
     DBUG_PRINT("info", ("Found exact match for db %s", key.c_str()));
-    DBUG_RETURN(filter_access(it->second, key));
+    return filter_access(it->second, key);
   }
 }
 
@@ -567,16 +561,16 @@ ulong Security_context::table_acl(LEX_CSTRING db, LEX_CSTRING table) {
 
 bool Security_context::has_with_admin_acl(const LEX_CSTRING &role_name,
                                           const LEX_CSTRING &role_host) {
-  DBUG_ENTER("Security_context::has_with_admin_acl");
-  if (m_acl_map == 0) DBUG_RETURN(false);
+  DBUG_TRACE;
+  if (m_acl_map == 0) return false;
   String q_name;
   append_identifier(&q_name, role_name.str, role_name.length);
   q_name.append("@");
   append_identifier(&q_name, role_host.str, role_host.length);
   Grant_acl_set::iterator it =
       m_acl_map->grant_acls()->find(std::string(q_name.c_ptr_quick()));
-  if (it != m_acl_map->grant_acls()->end()) DBUG_RETURN(true);
-  DBUG_RETURN(false);
+  if (it != m_acl_map->grant_acls()->end()) return true;
+  return false;
 }
 
 bool Security_context::any_sp_acl(const LEX_CSTRING &db) {
@@ -715,22 +709,22 @@ bool Security_context::can_operate_with(const Auth_id &auth_id,
                                         const std::string &privilege,
                                         bool cumulative /*= false */,
                                         bool ignore_if_nonextant /*= true */) {
-  DBUG_ENTER("Security_context::can_operate_with");
+  DBUG_TRACE;
   Acl_cache_lock_guard acl_cache_lock(current_thd,
                                       Acl_cache_lock_mode::READ_MODE);
   if (!acl_cache_lock.lock()) {
     DBUG_PRINT("error", ("Could not check for the SYSTEM_USER privilege. "
                          "Could not lock Acl caches.\n"));
-    DBUG_RETURN(true);
+    return true;
   }
   ACL_USER *acl_user =
       find_acl_user(auth_id.host().c_str(), auth_id.user().c_str(), true);
   if (!acl_user) {
     if (ignore_if_nonextant)
-      DBUG_RETURN(false);
+      return false;
     else {
       my_error(ER_USER_DOES_NOT_EXIST, MYF(0), auth_id.auth_str().c_str());
-      DBUG_RETURN(true);
+      return true;
     }
   }
 
@@ -743,15 +737,15 @@ bool Security_context::can_operate_with(const Auth_id &auth_id,
   if (is_mismatch) {
     my_error(ER_SPECIFIC_ACCESS_DENIED_ERROR, MYF(0), privilege.c_str());
   }
-  DBUG_RETURN(is_mismatch);
+  return is_mismatch;
 }
 
 LEX_CSTRING Security_context::priv_user() const {
   LEX_CSTRING priv_user;
-  DBUG_ENTER("Security_context::priv_user");
+  DBUG_TRACE;
   priv_user.str = m_priv_user;
   priv_user.length = m_priv_user_length;
-  DBUG_RETURN(priv_user);
+  return priv_user;
 }
 
 /**
@@ -763,12 +757,12 @@ LEX_CSTRING Security_context::priv_user() const {
 LEX_CSTRING Security_context::user() const {
   LEX_CSTRING user;
 
-  DBUG_ENTER("Security_context::user");
+  DBUG_TRACE;
 
   user.str = m_user.ptr();
   user.length = m_user.length();
 
-  DBUG_RETURN(user);
+  return user;
 }
 
 /**
@@ -782,14 +776,12 @@ LEX_CSTRING Security_context::user() const {
 
 void Security_context::set_user_ptr(const char *user_arg,
                                     const size_t user_arg_length) {
-  DBUG_ENTER("Security_context::set_user_ptr");
+  DBUG_TRACE;
 
-  if (user_arg == m_user.ptr()) DBUG_VOID_RETURN;
+  if (user_arg == m_user.ptr()) return;
 
   // set new user value to m_user.
   m_user.set(user_arg, user_arg_length, system_charset_info);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -804,16 +796,14 @@ void Security_context::set_user_ptr(const char *user_arg,
 
 void Security_context::assign_user(const char *user_arg,
                                    const size_t user_arg_length) {
-  DBUG_ENTER("Security_context::assign_user");
+  DBUG_TRACE;
 
-  if (user_arg == m_user.ptr()) DBUG_VOID_RETURN;
+  if (user_arg == m_user.ptr()) return;
 
   if (user_arg)
     m_user.copy(user_arg, user_arg_length, system_charset_info);
   else
     m_user.set((const char *)0, 0, system_charset_info);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -825,12 +815,12 @@ void Security_context::assign_user(const char *user_arg,
 LEX_CSTRING Security_context::host() const {
   LEX_CSTRING host;
 
-  DBUG_ENTER("Security_context::host");
+  DBUG_TRACE;
 
   host.str = m_host.ptr();
   host.length = m_host.length();
 
-  DBUG_RETURN(host);
+  return host;
 }
 
 /**
@@ -844,16 +834,14 @@ LEX_CSTRING Security_context::host() const {
 */
 void Security_context::set_host_ptr(const char *host_arg,
                                     const size_t host_arg_length) {
-  DBUG_ENTER("Security_context::set_host_ptr");
+  DBUG_TRACE;
 
   DBUG_ASSERT(host_arg != nullptr);
 
-  if (host_arg == m_host.ptr()) DBUG_VOID_RETURN;
+  if (host_arg == m_host.ptr()) return;
 
   // set new host value to m_host.
   m_host.set(host_arg, host_arg_length, system_charset_info);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -869,7 +857,7 @@ void Security_context::set_host_ptr(const char *host_arg,
 
 void Security_context::assign_host(const char *host_arg,
                                    const size_t host_arg_length) {
-  DBUG_ENTER("Security_context::assign_host");
+  DBUG_TRACE;
 
   if (host_arg == nullptr) {
     m_host.set("", 0, system_charset_info);
@@ -882,7 +870,7 @@ void Security_context::assign_host(const char *host_arg,
   }
 
 end:
-  DBUG_VOID_RETURN;
+  return;
 }
 
 /**
@@ -894,12 +882,12 @@ end:
 LEX_CSTRING Security_context::ip() const {
   LEX_CSTRING ip;
 
-  DBUG_ENTER("Security_context::ip");
+  DBUG_TRACE;
 
   ip.str = m_ip.ptr();
   ip.length = m_ip.length();
 
-  DBUG_RETURN(ip);
+  return ip;
 }
 
 /**
@@ -912,14 +900,12 @@ LEX_CSTRING Security_context::ip() const {
 */
 
 void Security_context::set_ip_ptr(const char *ip_arg, const int ip_arg_length) {
-  DBUG_ENTER("Security_context::set_ip_ptr");
+  DBUG_TRACE;
 
-  if (ip_arg == m_ip.ptr()) DBUG_VOID_RETURN;
+  if (ip_arg == m_ip.ptr()) return;
 
   // set new ip value to m_ip.
   m_ip.set(ip_arg, ip_arg_length, system_charset_info);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -934,16 +920,14 @@ void Security_context::set_ip_ptr(const char *ip_arg, const int ip_arg_length) {
 */
 
 void Security_context::assign_ip(const char *ip_arg, const int ip_arg_length) {
-  DBUG_ENTER("Security_context::assign_ip");
+  DBUG_TRACE;
 
-  if (ip_arg == m_ip.ptr()) DBUG_VOID_RETURN;
+  if (ip_arg == m_ip.ptr()) return;
 
   if (ip_arg)
     m_ip.copy(ip_arg, ip_arg_length, system_charset_info);
   else
     m_ip.set((const char *)0, 0, system_charset_info);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -957,14 +941,12 @@ void Security_context::assign_ip(const char *ip_arg, const int ip_arg_length) {
 
 void Security_context::set_external_user_ptr(const char *ext_user_arg,
                                              const int ext_user_arg_length) {
-  DBUG_ENTER("Security_context::set_external_user_ptr");
+  DBUG_TRACE;
 
-  if (ext_user_arg == m_external_user.ptr()) DBUG_VOID_RETURN;
+  if (ext_user_arg == m_external_user.ptr()) return;
 
   // set new ip value to m_ip.
   m_external_user.set(ext_user_arg, ext_user_arg_length, system_charset_info);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -979,17 +961,15 @@ void Security_context::set_external_user_ptr(const char *ext_user_arg,
 
 void Security_context::assign_external_user(const char *ext_user_arg,
                                             const int ext_user_arg_length) {
-  DBUG_ENTER("Security_context::assign_external_user");
+  DBUG_TRACE;
 
-  if (ext_user_arg == m_external_user.ptr()) DBUG_VOID_RETURN;
+  if (ext_user_arg == m_external_user.ptr()) return;
 
   if (ext_user_arg)
     m_external_user.copy(ext_user_arg, ext_user_arg_length,
                          system_charset_info);
   else
     m_external_user.set((const char *)0, 0, system_charset_info);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -1001,7 +981,7 @@ void Security_context::assign_external_user(const char *ext_user_arg,
 
 void Security_context::assign_priv_user(const char *priv_user_arg,
                                         const size_t priv_user_arg_length) {
-  DBUG_ENTER("Security_context::assign_priv_user");
+  DBUG_TRACE;
 
   if (priv_user_arg_length) {
     m_priv_user_length =
@@ -1011,8 +991,6 @@ void Security_context::assign_priv_user(const char *priv_user_arg,
     *m_priv_user = 0;
     m_priv_user_length = 0;
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -1024,12 +1002,12 @@ void Security_context::assign_priv_user(const char *priv_user_arg,
 LEX_CSTRING Security_context::proxy_user() const {
   LEX_CSTRING proxy_user;
 
-  DBUG_ENTER("Security_context::proxy_user");
+  DBUG_TRACE;
 
   proxy_user.str = m_proxy_user;
   proxy_user.length = m_proxy_user_length;
 
-  DBUG_RETURN(proxy_user);
+  return proxy_user;
 }
 
 /**
@@ -1041,7 +1019,7 @@ LEX_CSTRING Security_context::proxy_user() const {
 
 void Security_context::assign_proxy_user(const char *proxy_user_arg,
                                          const size_t proxy_user_arg_length) {
-  DBUG_ENTER("Security_context::assign_proxy_user");
+  DBUG_TRACE;
 
   if (proxy_user_arg_length) {
     m_proxy_user_length =
@@ -1051,8 +1029,6 @@ void Security_context::assign_proxy_user(const char *proxy_user_arg,
     *m_proxy_user = 0;
     m_proxy_user_length = 0;
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -1064,12 +1040,12 @@ void Security_context::assign_proxy_user(const char *proxy_user_arg,
 LEX_CSTRING Security_context::priv_host() const {
   LEX_CSTRING priv_host;
 
-  DBUG_ENTER("Security_context::priv_host");
+  DBUG_TRACE;
 
   priv_host.str = m_priv_host;
   priv_host.length = m_priv_host_length;
 
-  DBUG_RETURN(priv_host);
+  return priv_host;
 }
 
 /**
@@ -1081,7 +1057,7 @@ LEX_CSTRING Security_context::priv_host() const {
 
 void Security_context::assign_priv_host(const char *priv_host_arg,
                                         const size_t priv_host_arg_length) {
-  DBUG_ENTER("Security_context::assign_priv_host");
+  DBUG_TRACE;
 
   if (priv_host_arg_length) {
     m_priv_host_length =
@@ -1091,8 +1067,6 @@ void Security_context::assign_priv_host(const char *priv_host_arg,
     *m_priv_host = 0;
     m_priv_host_length = 0;
   }
-
-  DBUG_VOID_RETURN;
 }
 
 void Security_context::init_restrictions(const Restrictions &restrictions) {
@@ -1166,3 +1140,137 @@ std::pair<bool, bool> Security_context::fetch_global_grant(
   has_privilege = sctx.has_global_grant(privilege.c_str(), privilege.length());
   return has_privilege;
 }
+
+/**
+  Check if required access to given table is granted.
+
+  @param [in]     priv Required access
+  @param [in,out] tables Table list object
+
+  @returns access information
+  @retval true Sucess
+  @retval false Failure
+ */
+bool Security_context::has_table_access(ulong priv, TABLE_LIST *tables) {
+  DBUG_TRACE;
+  DBUG_ASSERT(tables != nullptr);
+  TABLE const *table = tables->table;
+  LEX_CSTRING db, table_name;
+  db.str = table->s->db.str;
+  db.length = table->s->db.length;
+
+  table_name.str = table->alias;
+  table_name.length = strlen(table->alias);
+
+  ulong acls = master_access({db.str, db.length});
+  if (m_acl_map) {
+    if (priv & acls) return true;
+
+    acls = db_acl(db);
+    if (priv & acls) return true;
+
+    Grant_table_aggregate aggr = table_and_column_acls(db, table_name);
+    acls = aggr.table_access | aggr.cols;
+    if (priv & acls) return true;
+  } else {
+    /* Global and DB priv check */
+    if (::check_access(m_thd, priv, db.str, &acls, nullptr, false, true))
+      return false;
+
+    if (priv & acls) return true;
+
+    if (::check_grant(m_thd, priv, tables, false, 1, true)) return false;
+    return true;
+  }
+  return false;
+}
+
+/**
+  Check if required access to given table is not restricted.
+
+  @param [in]     priv Required access
+  @param [in,out] table Table object
+
+  @returns access information
+  @retval true Access to the table is blocked
+  @retval false Access to the table is not blocked
+ */
+bool Security_context::is_table_blocked(ulong priv, TABLE const *table) {
+  DBUG_TRACE;
+  DBUG_ASSERT(table != nullptr);
+  LEX_CSTRING db, table_name;
+  db.str = table->s->db.str;
+  db.length = table->s->db.length;
+
+  table_name.str = table->alias;
+  table_name.length = strlen(table->alias);
+
+  /* Table privs */
+  TABLE_LIST tables;
+  tables.table = const_cast<TABLE *>(table);
+  tables.db = db.str;
+  tables.db_length = db.length;
+  tables.table_name = table_name.str;
+  tables.table_name_length = table_name.length;
+  tables.grant.privilege = NO_ACCESS;
+
+  return !has_table_access(priv, &tables);
+}
+
+/**
+  Check if required access to given table column is granted.
+
+  @param [in] priv Required access
+  @param [in] table Table object
+  @param [in] columns List of column names to check
+
+  @returns access information
+  @retval true Sucess
+  @retval false Failure
+ */
+bool Security_context::has_column_access(ulong priv, TABLE const *table,
+                                         std::vector<std::string> columns) {
+  DBUG_TRACE;
+  DBUG_ASSERT(table != nullptr);
+  LEX_CSTRING db, table_name;
+  db.str = table->s->db.str;
+  db.length = table->s->db.length;
+
+  table_name.str = table->alias;
+  table_name.length = strlen(table->alias);
+
+  /* Table privs */
+  TABLE_LIST tables;
+  tables.table = const_cast<TABLE *>(table);
+  tables.db = db.str;
+  tables.db_length = db.length;
+  tables.table_name = table_name.str;
+  tables.table_name_length = table_name.length;
+  tables.grant.privilege = NO_ACCESS;
+
+  // Check that general table access is possible
+  if (!has_table_access(priv, &tables)) return false;
+
+  // Try to get info about table specific grants
+  {
+    Acl_cache_lock_guard acl_cache_lock{this->m_thd,
+                                        Acl_cache_lock_mode::READ_MODE};
+    if (!acl_cache_lock.lock()) return false;
+
+    if (table_hash_search(this->host().str, this->ip().str, db.str,
+                          this->priv_user().str, table_name.str,
+                          0) == nullptr) {
+      // If there is no specific info about the table specific privileges, it
+      // means that there are no column privileges configured for the table
+      // columns. So, we let the general table access above to prevail.
+      return true;
+    }
+  }
+
+  for (auto column : columns) {
+    if (check_column_grant_in_table_ref(m_thd, &tables, column.data(),
+                                        column.length(), priv))
+      return false;
+  }
+  return true;
+}
