@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,14 +22,11 @@
 
 #include "sql/clone_handler.h"
 
-#include <assert.h>
 #include <string.h>
-#include <cctype>
 #include <chrono>
-#include <cstdlib>
-#include <string>
 #include <thread>
 
+#include "my_dbug.h"
 #include "my_dir.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
@@ -45,87 +41,17 @@
 #include "sql/sql_class.h"
 #include "sql/sql_parse.h"
 #include "sql/sql_plugin.h"  // plugin_unlock
-#include "sql/srv_session.h"
-#include "sql_string.h"  // to_lex_cstring
+#include "sql_string.h"      // to_lex_cstring
 #include "violite.h"
 
 class THD;
 
-/* To get current session thread default THD */
-THD *thd_get_current_thd();
-
 /** Clone handler global */
 Clone_handler *clone_handle = nullptr;
 
 /** Clone plugin name */
 const char *clone_plugin_nm = "clone";
 
-bool Clone_handler::get_donor_error(Srv_session *session, int &error,
-                                    const char *&message) {
-  error = 0;
-  message = nullptr;
-  THD *thd = nullptr;
-
-  if (session != nullptr) {
-    thd = session->get_thd();
-  }
-
-  /* Check if THD exists. */
-  if (thd == nullptr) {
-    /* Try to get current THD handle. */
-    thd = thd_get_current_thd();
-    if (thd == nullptr) {
-      return (false);
-    }
-  }
-
-  /* Check if DA exists. */
-  auto da = thd->get_stmt_da();
-  if (da == nullptr || !da->is_error()) {
-    return (false);
-  }
-
-  if (da->mysql_errno() != ER_CLONE_DONOR) {
-    return (false);
-  }
-
-  /* Assign current error from DA */
-  error = da->mysql_errno();
-  message = da->message_text();
-
-  /* Parse and find out donor error and message. */
-  size_t err_pos = 0;
-  std::string msg_string(message);
-
-  while (!std::isdigit(message[err_pos])) {
-    /* Find position of next ":". */
-    err_pos = msg_string.find(": ", err_pos);
-
-    /* No more separator, return. */
-    if (err_pos == std::string::npos) {
-      assert(false);
-      return (false);
-    }
-    /* Skip ":" and space. */
-    err_pos += 2;
-  }
-
-  error = std::atoi(message + err_pos);
-
-  if (error != 0) {
-    err_pos = msg_string.find(": ", err_pos);
-    /* Should find the error message following the error code. */
-    if (err_pos == std::string::npos) {
-      assert(false);
-      return (false);
-    }
-    /* Skip ":" and space. */
-    err_pos += 2;
-    message = message + err_pos;
-  }
-  return (true);
-}
-
 int Clone_handler::clone_local(THD *thd, const char *data_dir) {
   int error;
   char dir_name[FN_REFLEN];
@@ -194,8 +120,8 @@ int Clone_handler::init() {
 
   plugin_ref plugin;
 
-  plugin = my_plugin_lock_by_name(
-      nullptr, to_lex_cstring(m_plugin_name.c_str()), MYSQL_CLONE_PLUGIN);
+  plugin = my_plugin_lock_by_name(0, to_lex_cstring(m_plugin_name.c_str()),
+                                  MYSQL_CLONE_PLUGIN);
   if (plugin == nullptr) {
     m_plugin_handle = nullptr;
     LogErr(ERROR_LEVEL, ER_CLONE_PLUGIN_NOT_LOADED_TRACE);
@@ -203,7 +129,7 @@ int Clone_handler::init() {
   }
 
   m_plugin_handle = (Mysql_clone *)plugin_decl(plugin)->info;
-  plugin_unlock(nullptr, plugin);
+  plugin_unlock(0, plugin);
 
   if (opt_initialize) {
     /* Inform that database initialization in progress. */
@@ -265,7 +191,7 @@ int Clone_handler::validate_dir(const char *in_dir, char *out_dir) {
 
     /* length must always decrease for the loop to terminate */
     if (length <= new_length) {
-      assert(false);
+      DBUG_ASSERT(false);
       break;
     }
 
@@ -291,20 +217,6 @@ int clone_handle_create(const char *plugin_name) {
   return clone_handle->init();
 }
 
-int clone_handle_check_drop(MYSQL_PLUGIN plugin_info) {
-  auto plugin = static_cast<st_plugin_int *>(plugin_info);
-  int error = 0;
-
-  mysql_mutex_lock(&LOCK_plugin);
-  assert(plugin->state == PLUGIN_IS_DYING);
-
-  if (plugin->ref_count > 0) {
-    error = ER_PLUGIN_CANNOT_BE_UNINSTALLED; /* purecov: inspected */
-  }
-  mysql_mutex_unlock(&LOCK_plugin);
-  return error;
-}
-
 int clone_handle_drop() {
   if (clone_handle == nullptr) {
     return 1;
@@ -332,7 +244,7 @@ Clone_handler *clone_plugin_lock(THD *thd, plugin_ref *plugin) {
   if (*plugin != nullptr && plugin_state(*plugin) == PLUGIN_IS_READY) {
     mysql_mutex_unlock(&LOCK_plugin);
 
-    assert(clone_handle != nullptr);
+    DBUG_ASSERT(clone_handle != nullptr);
     return clone_handle;
   }
 
@@ -348,7 +260,6 @@ void clone_plugin_unlock(THD *thd, plugin_ref plugin) {
 std::atomic<int> Clone_handler::s_xa_counter{0};
 std::atomic<bool> Clone_handler::s_xa_block_op{false};
 std::atomic<int> Clone_handler::s_provision_in_progress{0};
-std::atomic<bool> Clone_handler::s_is_data_dropped{false};
 
 mysql_mutex_t Clone_handler::s_xa_mutex;
 
@@ -427,7 +338,7 @@ bool Clone_handler::XA_Block::failed() const { return (!m_success); }
 
 bool Clone_handler::block_xa_operation(THD *thd) {
   bool ret = true;
-  assert(!s_xa_block_op.load());
+  DBUG_ASSERT(!s_xa_block_op.load());
 
   mysql_mutex_lock(&s_xa_mutex);
   /* Block new xa prepare/commit/rollback. No new XA operation can start after
