@@ -31,9 +31,8 @@
   to a single table or similar, but are the inner (non-leaf) nodes of the
   iterator execution tree. They consistently own their source iterator, although
   not its memory (since we never allocate row iterators on the heap--usually on
-  a MEM_ROOT or as static parts of READ_RECORD). This means that in the end,
-  you'll end up with a single root iterator which then owns everything else
-  recursively.
+  a MEM_ROOT>). This means that in the end, you'll end up with a single root
+  iterator which then owns everything else recursively.
 
   SortingIterator is also a composite iterator, but is defined in its own file.
  */
@@ -48,10 +47,12 @@
 #include "my_base.h"
 #include "my_dbug.h"
 #include "my_table_map.h"
+#include "prealloced_array.h"
 #include "sql/item.h"
 #include "sql/row_iterator.h"
 #include "sql/table.h"
 
+class FollowTailIterator;
 template <class T>
 class List;
 class JOIN;
@@ -80,6 +81,10 @@ class FilterIterator final : public RowIterator {
     m_source->SetNullRowFlag(is_null_row);
   }
 
+  void StartPSIBatchMode() override { m_source->StartPSIBatchMode(); }
+  void EndPSIBatchModeIfStarted() override {
+    m_source->EndPSIBatchModeIfStarted();
+  }
   void UnlockRow() override { m_source->UnlockRow(); }
 
   std::vector<Child> children() const override;
@@ -133,6 +138,10 @@ class LimitOffsetIterator final : public RowIterator {
     m_source->SetNullRowFlag(is_null_row);
   }
 
+  void StartPSIBatchMode() override { m_source->StartPSIBatchMode(); }
+  void EndPSIBatchModeIfStarted() override {
+    m_source->EndPSIBatchModeIfStarted();
+  }
   void UnlockRow() override { m_source->UnlockRow(); }
 
   std::vector<Child> children() const override {
@@ -149,7 +158,11 @@ class LimitOffsetIterator final : public RowIterator {
       snprintf(buf, sizeof(buf), "Limit/Offset: %llu/%llu row(s)",
                m_limit - m_offset, m_offset);
     }
-    return {std::string(buf)};
+    if (m_count_all_rows) {
+      return {std::string(buf) + " (no early end due to SQL_CALC_FOUND_ROWS)"};
+    } else {
+      return {std::string(buf)};
+    }
   }
 
  private:
@@ -213,7 +226,17 @@ class AggregateIterator final : public RowIterator {
   void SetNullRowFlag(bool is_null_row) override {
     m_source->SetNullRowFlag(is_null_row);
   }
-  void UnlockRow() override;
+
+  void StartPSIBatchMode() override { m_source->StartPSIBatchMode(); }
+  void EndPSIBatchModeIfStarted() override {
+    m_source->EndPSIBatchModeIfStarted();
+  }
+  void UnlockRow() override {
+    // Most likely, HAVING failed. Ideally, we'd like to backtrack and
+    // unlock all rows that went into this aggregate, but we can't do that,
+    // and we also can't unlock the _current_ row, since that belongs to a
+    // different group. Thus, do nothing.
+  }
 
   std::vector<Child> children() const override {
     return std::vector<Child>{{m_source.get(), ""}};
@@ -347,7 +370,14 @@ class PrecomputedAggregateIterator final : public RowIterator {
   void SetNullRowFlag(bool is_null_row) override {
     m_source->SetNullRowFlag(is_null_row);
   }
-  void UnlockRow() override;
+
+  void StartPSIBatchMode() override { m_source->StartPSIBatchMode(); }
+  void EndPSIBatchModeIfStarted() override {
+    m_source->EndPSIBatchModeIfStarted();
+  }
+  void UnlockRow() override {
+    // See AggregateIterator::UnlockRow().
+  }
 
   std::vector<Child> children() const override {
     return std::vector<Child>{{m_source.get(), ""}};
@@ -420,6 +450,11 @@ class NestedLoopIterator final : public RowIterator {
     m_source_inner->SetNullRowFlag(is_null_row);
   }
 
+  void EndPSIBatchModeIfStarted() override {
+    m_source_outer->EndPSIBatchModeIfStarted();
+    m_source_inner->EndPSIBatchModeIfStarted();
+  }
+
   void UnlockRow() override {
     // Since we don't know which condition that caused the row to be rejected,
     // we can't know whether we could also unlock the outer row
@@ -511,35 +546,146 @@ class CacheInvalidatorIterator final : public RowIterator {
   which items is governed by the temporary table parameters.
 
   Conceptually (although not performance-wise!), the MaterializeIterator is a
-  no-op if you don't ask for deduplication, and in some cases, we probably
-  should elide it (e.g. when scanning a table only once). However, it's not
-  necessarily straightforward to do so by just not inserting the iterator,
-  as the optimizer will have set up everything (e.g., read sets, or what table
-  upstream items will read from) assuming the materialization will happen.
+  no-op if you don't ask for deduplication, and in some cases (e.g. when
+  scanning a table only once), we elide it. However, it's not necessarily
+  straightforward to do so by just not inserting the iterator, as the optimizer
+  will have set up everything (e.g., read sets, or what table upstream items
+  will read from) assuming the materialization will happen, so the realistic
+  option is setting up everything as if materialization would happen but not
+  actually write to the table; see StreamingIterator for details.
+
+  MaterializeIterator conceptually materializes iterators, not JOINs or
+  SELECT_LEX_UNITs. However, there are many details that leak out
+  (e.g., setting performance schema batch mode, slices, reusing CTEs,
+  etc.), so we need to send them in anyway.
  */
 class MaterializeIterator final : public TableRowIterator {
  public:
-  // SELECT_LEX is used only to get active options.
-  //
-  // If “copy_fields_and_items” is set to false, the Field objects in the
-  // output row is presumed already to be filled out. This is the case
-  // iff there's an AggregateIterator earlier in the chain.
-  //
-  // The “limit_rows” parameter does the same job as a LimitOffsetIterator
-  // right before the MaterializeIterator would have done, except that it
-  // works _after_ deduplication (if that is active). It is used for when
-  // pushing LIMIT down to MaterializeIterator, so that we can stop
-  // materializing when there are enough rows. The deduplication is the
-  // reason why this specific limit has to be handled in MaterializeIterator
-  // and not using a regular LimitOffsetIterator. Set to HA_POS_ERROR
-  // for no limit.
+  struct QueryBlock {
+    /// The iterator to read the actual rows from.
+    unique_ptr_destroy_only<RowIterator> subquery_iterator;
+
+    /// Used only for optimizer trace.
+    int select_number;
+
+    /// The JOIN that this query block represents. Used for performance
+    /// schema batch mode: When materializing a query block that consists of
+    /// a single table, MaterializeIterator needs to set up schema batch mode,
+    /// since there is no nested loop iterator to do it. (This is similar to
+    /// what ExecuteIteratorQuery() needs to do at the top level.)
+    JOIN *join;
+
+    /// If true, unique constraint checking via hash key is disabled
+    /// when materializing this query block (ie., we simply avoid calling
+    /// check_unique_constraint() for each row). Used when materializing
+    /// UNION DISTINCT and UNION ALL parts into the same table.
+    /// We'd like to just use a unique constraint via unique index instead,
+    /// but there might be other indexes on the destination table
+    /// that we'd like to keep, and the implementation doesn't allow
+    /// disabling only one index.
+    ///
+    /// If you use this on a query block, doing_hash_deduplication()
+    /// must be true.
+    bool disable_deduplication_by_hash_field = false;
+
+    /// If set to false, the Field objects in the output row are
+    /// presumed already to be filled out. This is the case iff
+    /// there's an AggregateIterator earlier in the chain.
+    bool copy_fields_and_items;
+
+    /// If copy_fields_and_items is true, used for copying the Field objects
+    /// into the temporary table row. Otherwise unused.
+    Temp_table_param *temp_table_param;
+
+    // Whether this query block is a recursive reference back to the
+    // output of the materialization.
+    bool is_recursive_reference = false;
+
+    // If is_recursive_reference is true, contains the FollowTailIterator
+    // in the query block (there can be at most one recursive reference
+    // in a join list, as per the SQL standard, so there should be exactly one).
+    // Used for informing the iterators about various shared state in the
+    // materialization (including coordinating rematerializations).
+    FollowTailIterator *recursive_reader = nullptr;
+  };
+
+  /**
+    @param thd Thread handler.
+    @param query_blocks_to_materialize List of query blocks to materialize.
+    @param table Handle to table to materialize into.
+    @param table_iterator Iterator used for scanning the temporary table
+      after materialization.
+    @param cte If materializing a CTE, points to it (see m_cte), otherwise
+      nullptr.
+    @param unit The query expression we are materializing (see m_unit).
+    @param join
+      When materializing within the same JOIN (e.g., into a temporary table
+      before sorting), as opposed to a derived table or a CTE, we may need
+      to change the slice on the join before returning rows from the result
+      table. If so, join and ref_slice would need to be set, and
+      query_blocks_to_materialize should contain only one member, with the same
+      join.
+    @param ref_slice See join. If we are materializing across JOINs,
+      e.g. derived tables, ref_slice should be left at -1.
+    @param rematerialize true if rematerializing on every Init() call
+      (e.g., because we have a dependency on a value from outside the query
+      block).
+    @param limit_rows
+      Does the same job as a LimitOffsetIterator right before the
+      MaterializeIterator would have done, except that it works _after_
+      deduplication (if that is active). It is used for when pushing LIMIT down
+      to MaterializeIterator, so that we can stop materializing when there are
+      enough rows. The deduplication is the reason why this specific limit has
+      to be handled in MaterializeIterator and not using a regular
+      LimitOffsetIterator. Set to HA_POS_ERROR for no limit.
+   */
+  MaterializeIterator(THD *thd,
+                      Mem_root_array<QueryBlock> query_blocks_to_materialize,
+                      TABLE *table,
+                      unique_ptr_destroy_only<RowIterator> table_iterator,
+                      const Common_table_expr *cte, SELECT_LEX_UNIT *unit,
+                      JOIN *join, int ref_slice, bool rematerialize,
+                      ha_rows limit_rows);
+
+  /**
+    A convenience form for materializing a single table only.
+
+    @param thd Thread handler.
+    @param subquery_iterator The iterator to read the actual rows from.
+    @param temp_table_param If copy_fields_and_items is true, used for copying
+      the Field objects into the temporary table row. Otherwise unused.
+    @param table Handle to table to materialize into.
+    @param table_iterator Iterator used for scanning the temporary table
+      after materialization.
+    @param cte If materializing a CTE, points to it (see m_cte), otherwise
+      nullptr.
+    @param select_number Used only for optimizer trace.
+    @param unit The query expression we are materializing (see m_unit).
+    @param join
+      When materializing within the same JOIN (e.g., into a temporary table
+      before sorting), as opposed to a derived table or a CTE, we may need
+      to change the slice on the join before returning rows from the result
+      table. If so, join and ref_slice would need to be set, and
+      query_blocks_to_materialize should contain only one member, with the same
+      join.
+    @param ref_slice See join. If we are materializing across JOINs,
+      e.g. derived tables, ref_slice should be left at -1.
+    @param copy_fields_and_items If set to false, the Field objects in the
+      output row are presumed already to be filled out. This is the case iff
+      there's an AggregateIterator earlier in the chain.
+    @param rematerialize true if rematerializing on every Init() call
+      (e.g., because we have a dependency on a value from outside the query
+      block).
+    @param limit_rows See limit_rows on the other constructor.
+   */
   MaterializeIterator(THD *thd,
                       unique_ptr_destroy_only<RowIterator> subquery_iterator,
                       Temp_table_param *temp_table_param, TABLE *table,
                       unique_ptr_destroy_only<RowIterator> table_iterator,
-                      const Common_table_expr *cte, SELECT_LEX *select_lex,
-                      JOIN *join, int ref_slice, bool copy_fields_and_items,
-                      bool rematerialize, ha_rows limit_rows);
+                      const Common_table_expr *cte, int select_number,
+                      SELECT_LEX_UNIT *unit, JOIN *join, int ref_slice,
+                      bool copy_fields_and_items, bool rematerialize,
+                      ha_rows limit_rows);
 
   bool Init() override;
   int Read() override;
@@ -551,6 +697,9 @@ class MaterializeIterator final : public TableRowIterator {
     m_table_iterator->SetNullRowFlag(is_null_row);
   }
 
+  void StartPSIBatchMode() override { m_table_iterator->StartPSIBatchMode(); }
+  void EndPSIBatchModeIfStarted() override;
+
   // The temporary table is private to us, so there's no need to worry about
   // locks to other transactions.
   void UnlockRow() override {}
@@ -563,24 +712,30 @@ class MaterializeIterator final : public TableRowIterator {
   void AddInvalidator(const CacheInvalidatorIterator *invalidator);
 
  private:
-  unique_ptr_destroy_only<RowIterator> m_subquery_iterator;
+  Mem_root_array<QueryBlock> m_query_blocks_to_materialize;
   unique_ptr_destroy_only<RowIterator> m_table_iterator;
 
-  /// If we are materializing a CTE, points to it. Otherwise nullptr.
+  /// If we are materializing a CTE, points to it (otherwise nullptr).
+  /// Used so that we see if some other iterator already materialized the table,
+  /// avoiding duplicate work.
   const Common_table_expr *m_cte;
 
-  Temp_table_param *m_tmp_table_param;
-  SELECT_LEX *m_select_lex;
+  /// The query expression we are materializing. For derived tables,
+  /// we materialize the entire query expression; for materialization within
+  /// a query expression (e.g. for sorting or for windowing functions),
+  /// we materialize only parts of it. Used to clear correlated CTEs within
+  /// the unit when we rematerialize, since they depend on values from
+  /// outside the query expression, and those values may have changed
+  /// since last materialization.
+  SELECT_LEX_UNIT *m_unit;
 
-  /// The join we are materializing.
+  /// See constructor.
   JOIN *const m_join;
 
   /// The slice to set when accessing temporary table; used if anything upstream
   /// (e.g. WHERE, HAVING) wants to evaluate values based on its contents.
-  const int m_ref_slice;
-
   /// See constructor.
-  const bool m_copy_fields_and_items;
+  const int m_ref_slice;
 
   /// If true, we need to materialize anew for each Init() (because the contents
   /// of the table will depend on some outer non-constant value).
@@ -608,9 +763,67 @@ class MaterializeIterator final : public TableRowIterator {
   /// check_unique_constraint() is not called. However, B-tree indexes
   /// have limitations, in particular on length, that sometimes require us
   /// to do this instead. See create_tmp_table() for details.
-  bool doing_hash_deduplication() const {
-    return table()->hash_field && !table()->no_keyread;
+  bool doing_hash_deduplication() const { return table()->hash_field; }
+
+  /// Whether we are deduplicating, whether through a hash field
+  /// or a regular unique index.
+  bool doing_deduplication() const;
+
+  bool MaterializeRecursive();
+  bool MaterializeQueryBlock(const QueryBlock &query_block,
+                             ha_rows *stored_rows);
+};
+
+/**
+  StreamingIterator is a minimal version of MaterializeIterator that does not
+  actually materialize; instead, every Read() just forwards the call to the
+  subquery iterator and does the required copying from one set of fields to
+  another.
+
+  It is used for when the optimizer would normally set up a materialization,
+  but you don't actually need one, ie. you don't want to read the rows multiple
+  times after writing them, and you don't want to access them by index (only
+  a single table scan). If you don't need the copy functionality (ie., you
+  have an AggregateIterator, which does this job already), you still need a
+  StreamingIterator, to set the NULL row flag on the temporary table.
+ */
+class StreamingIterator final : public TableRowIterator {
+ public:
+  StreamingIterator(THD *thd,
+                    unique_ptr_destroy_only<RowIterator> subquery_iterator,
+                    Temp_table_param *temp_table_param, TABLE *table,
+                    bool copy_fields_and_items);
+
+  bool Init() override;
+
+  int Read() override;
+
+  std::vector<std::string> DebugString() const override {
+    return {"Stream results"};
   }
+
+  std::vector<Child> children() const override {
+    return std::vector<Child>{{m_subquery_iterator.get(), ""}};
+  }
+
+  void StartPSIBatchMode() override {
+    m_subquery_iterator->StartPSIBatchMode();
+  }
+  void EndPSIBatchModeIfStarted() override {
+    m_subquery_iterator->EndPSIBatchModeIfStarted();
+  }
+  void UnlockRow() override { m_subquery_iterator->UnlockRow(); }
+
+ private:
+  unique_ptr_destroy_only<RowIterator> m_subquery_iterator;
+  Temp_table_param *m_temp_table_param;
+  const bool m_copy_fields_and_items;
+  ha_rows m_row_number;
+
+  // Whether the iterator should generate and provide a row ID. Only true if the
+  // iterator is part of weedout, where the iterator will create a fake row ID
+  // to uniquely identify the rows it produces.
+  bool m_provide_rowid{false};
 };
 
 /**
@@ -631,6 +844,10 @@ class TemptableAggregateIterator final : public TableRowIterator {
   void SetNullRowFlag(bool is_null_row) override {
     m_table_iterator->SetNullRowFlag(is_null_row);
   }
+  void EndPSIBatchModeIfStarted() override {
+    m_table_iterator->EndPSIBatchModeIfStarted();
+    m_subquery_iterator->EndPSIBatchModeIfStarted();
+  }
   void UnlockRow() override {}
   std::vector<std::string> DebugString() const override;
 
@@ -643,7 +860,7 @@ class TemptableAggregateIterator final : public TableRowIterator {
   /// The iterator used to scan the resulting temporary table.
   unique_ptr_destroy_only<RowIterator> m_table_iterator;
 
-  Temp_table_param *m_tmp_table_param;
+  Temp_table_param *m_temp_table_param;
   SELECT_LEX *m_select_lex;
   JOIN *const m_join;
   const int m_ref_slice;
@@ -677,6 +894,11 @@ class MaterializedTableFunctionIterator final : public TableRowIterator {
     m_table_iterator->SetNullRowFlag(is_null_row);
   }
 
+  void StartPSIBatchMode() override { m_table_iterator->StartPSIBatchMode(); }
+  void EndPSIBatchModeIfStarted() override {
+    m_table_iterator->EndPSIBatchModeIfStarted();
+  }
+
   // The temporary table is private to us, so there's no need to worry about
   // locks to other transactions.
   void UnlockRow() override {}
@@ -723,11 +945,19 @@ class WeedoutIterator final : public RowIterator {
     m_source->SetNullRowFlag(is_null_row);
   }
 
+  void EndPSIBatchModeIfStarted() override {
+    m_source->EndPSIBatchModeIfStarted();
+  }
   void UnlockRow() override { m_source->UnlockRow(); }
 
  private:
   unique_ptr_destroy_only<RowIterator> m_source;
   SJ_TMP_TABLE *m_sj;
+
+  // The cached value of QEP_TAB::rowid_status for each of the tables in the
+  // weedout. Index 0 corresponds to the first table in m_sj.
+  // See QEP_TAB::rowid_status for why we need to cache this value.
+  Prealloced_array<rowid_statuses, 4> m_rowid_status;
 };
 
 /**
@@ -757,6 +987,10 @@ class RemoveDuplicatesIterator final : public RowIterator {
     m_source->SetNullRowFlag(is_null_row);
   }
 
+  void StartPSIBatchMode() override { m_source->StartPSIBatchMode(); }
+  void EndPSIBatchModeIfStarted() override {
+    m_source->EndPSIBatchModeIfStarted();
+  }
   void UnlockRow() override { m_source->UnlockRow(); }
 
  private:
@@ -815,6 +1049,11 @@ class NestedLoopSemiJoinWithDuplicateRemovalIterator final
     m_source_inner->SetNullRowFlag(is_null_row);
   }
 
+  void EndPSIBatchModeIfStarted() override {
+    m_source_outer->EndPSIBatchModeIfStarted();
+    m_source_inner->EndPSIBatchModeIfStarted();
+  }
+
   void UnlockRow() override {
     m_source_outer->UnlockRow();
     m_source_inner->UnlockRow();
@@ -862,6 +1101,11 @@ class WindowingIterator final : public RowIterator {
     m_source->SetNullRowFlag(is_null_row);
   }
 
+  void StartPSIBatchMode() override { m_source->StartPSIBatchMode(); }
+  void EndPSIBatchModeIfStarted() override {
+    m_source->EndPSIBatchModeIfStarted();
+  }
+
   void UnlockRow() override {
     // There's nothing we can do here.
   }
@@ -911,6 +1155,11 @@ class BufferingWindowingIterator final : public RowIterator {
     m_source->SetNullRowFlag(is_null_row);
   }
 
+  void StartPSIBatchMode() override { m_source->StartPSIBatchMode(); }
+  void EndPSIBatchModeIfStarted() override {
+    m_source->EndPSIBatchModeIfStarted();
+  }
+
   void UnlockRow() override {
     // There's nothing we can do here.
   }
@@ -955,4 +1204,73 @@ class BufferingWindowingIterator final : public RowIterator {
   bool m_eof;
 };
 
+/**
+  MaterializeInformationSchemaTableIterator makes sure a given I_S temporary
+  table is materialized (filled out) before we try to scan it.
+ */
+class MaterializeInformationSchemaTableIterator final : public RowIterator {
+ public:
+  MaterializeInformationSchemaTableIterator(
+      THD *thd, QEP_TAB *qep_tab,
+      unique_ptr_destroy_only<RowIterator> table_iterator);
+
+  bool Init() override;
+  int Read() override { return m_table_iterator->Read(); }
+  std::vector<std::string> DebugString() const override;
+
+  std::vector<Child> children() const override {
+    // We don't list the table iterator as an explicit child; we mark it in
+    // our DebugString() instead. (Anything else would look confusingly much
+    // like a join.)
+    return {};
+  }
+
+  void SetNullRowFlag(bool is_null_row) override {
+    m_table_iterator->SetNullRowFlag(is_null_row);
+  }
+
+  void StartPSIBatchMode() override { m_table_iterator->StartPSIBatchMode(); }
+  void EndPSIBatchModeIfStarted() override {
+    m_table_iterator->EndPSIBatchModeIfStarted();
+  }
+
+  // The temporary table is private to us, so there's no need to worry about
+  // locks to other transactions.
+  void UnlockRow() override {}
+
+ private:
+  /// The iterator that reads from the materialized table.
+  unique_ptr_destroy_only<RowIterator> m_table_iterator;
+  QEP_TAB *m_qep_tab;
+};
+
+/**
+  Takes in two or more iterators and output rows from them sequentially
+  (first all rows from the first one, the all from the second one, etc.).
+  Used for implementing UNION ALL, typically together with StreamingIterator.
+ */
+class AppendIterator final : public RowIterator {
+ public:
+  AppendIterator(
+      THD *thd,
+      std::vector<unique_ptr_destroy_only<RowIterator>> &&sub_iterators);
+
+  bool Init() override;
+  int Read() override;
+
+  std::vector<std::string> DebugString() const override { return {"Append"}; }
+  std::vector<Child> children() const override;
+
+  void StartPSIBatchMode() override;
+  void EndPSIBatchModeIfStarted() override;
+
+  void SetNullRowFlag(bool is_null_row) override;
+  void UnlockRow() override;
+
+ private:
+  std::vector<unique_ptr_destroy_only<RowIterator>> m_sub_iterators;
+  size_t m_current_iterator_index = 0;
+  bool m_pfs_batch_mode_enabled = false;
+};
+
 #endif  // SQL_COMPOSITE_ITERATORS_INCLUDED
