@@ -68,7 +68,10 @@ class Create_field {
   ///          location where the actual data is stored. So LONGBLOB would
   ///          return 4 bytes for the length variable + 8 bytes for the pointer
   ///          to the data (12 bytes in total).
-  size_t pack_length() const;
+  ///  @param dont_override  Don't use pack_length_override even if non-zero
+  ///                        Used by multi-valued index, where pack_length
+  ///                        and key_length aren't the same.
+  size_t pack_length(bool dont_override = false) const;
 
   /// @returns the key length for this column.
   size_t key_length() const;
@@ -93,8 +96,8 @@ class Create_field {
     NULL for columns added.
   */
   const char *change;
-  const char *after;   // Put column after this one
-  LEX_STRING comment;  // Comment for field
+  const char *after;    // Put column after this one
+  LEX_CSTRING comment;  // Comment for field
 
   /**
      The declared default value, if any, otherwise NULL. Note that this member
@@ -173,6 +176,9 @@ class Create_field {
   Value_generator *m_default_val_expr{nullptr};
   Nullable<gis::srid_t> m_srid;
 
+  // Whether the field is actually an array of the field's type;
+  bool is_array{false};
+
   Create_field()
       : after(NULL),
         is_explicit_collation(false),
@@ -208,12 +214,12 @@ class Create_field {
 
   bool init(THD *thd, const char *field_name, enum_field_types type,
             const char *length, const char *decimals, uint type_modifier,
-            Item *default_value, Item *on_update_value, LEX_STRING *comment,
+            Item *default_value, Item *on_update_value, LEX_CSTRING *comment,
             const char *change, List<String> *interval_list,
             const CHARSET_INFO *cs, bool has_explicit_collation,
             uint uint_geom_type, Value_generator *gcol_info,
             Value_generator *default_val_expr, Nullable<gis::srid_t> srid,
-            dd::Column::enum_hidden_type hidden);
+            dd::Column::enum_hidden_type hidden, bool is_array = false);
 
   ha_storage_media field_storage_type() const {
     return (ha_storage_media)((flags >> FIELD_FLAGS_STORAGE_MEDIA) & 3);
@@ -262,4 +268,8 @@ class Create_field {
   static constexpr size_t LONGBLOB_MAX_SIZE_IN_BYTES{4294967295};
 };
 
+/// @returns whether or not this field is a hidden column that represents a
+///          functional index.
+bool is_field_for_functional_index(const Create_field *create_field);
+
 #endif
