@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,7 +58,7 @@ namespace dd {
 template <typename T>
 T &Collection<T>::Collection_iterator::operator*() {
   // Dereferencing an invalid iterator is prohibited.
-  assert(m_current != m_array->end());
+  DBUG_ASSERT(m_current != m_array->end());
 
   // Need a non-tmp placeholder of correct type since reference is returned.
   m_current_obj = *m_current;
@@ -67,10 +66,10 @@ T &Collection<T>::Collection_iterator::operator*() {
 }
 
 template <typename T>
-const typename Collection<T>::abstract_type *&
-Collection<T>::Collection_const_iterator::operator*() {
+const typename Collection<T>::abstract_type *
+    &Collection<T>::Collection_const_iterator::operator*() {
   // Dereferencing an invalid iterator is prohibited.
-  assert(m_current != m_array->end());
+  DBUG_ASSERT(m_current != m_array->end());
 
   // Need a non-tmp placeholder of correct type since reference is returned.
   m_current_obj = *m_current;
@@ -110,7 +109,10 @@ void Collection<T>::remove(typename Collection<T>::impl_type *item) {
 template <typename T>
 typename Collection<T>::iterator Collection<T>::find(const impl_type *item) {
   // Find the element and prepare iterator pointing to found element.
-  return iterator(&m_items, std::find(m_items.begin(), m_items.end(), item));
+  typename Collection<T>::iterator iterator(
+      &m_items, std::find(m_items.begin(), m_items.end(), item));
+
+  return iterator;
 }
 
 /**
@@ -127,9 +129,9 @@ bool Collection<T>::restore_items(Parent_item *parent,
 
   // NOTE: if this assert is firing, that means the table was not registered
   // for that transaction. Use Open_dictionary_tables_ctx::register_tables().
-  assert(table);
+  DBUG_ASSERT(table);
 
-  assert(empty());
+  DBUG_ASSERT(empty());
 
   std::unique_ptr<Object_key> key_holder(key);
 
@@ -256,7 +258,7 @@ bool Collection<T>::drop_items(Open_dictionary_tables_ctx *otx,
 
 template <typename T>
 const typename Collection<T>::abstract_type *Collection<T>::at(size_t n) const {
-  assert(n < size());
+  DBUG_ASSERT(n < size());
   return m_items[n];
 }
 
@@ -281,58 +283,58 @@ void Collection<T>::deep_copy(const Collection<T> &src, Parent_item *parent) {
 */
 
 template Column *&Collection<Column *>::Collection_iterator::operator*();
-template Column_type_element *&
-Collection<Column_type_element *>::Collection_iterator::operator*();
-template Foreign_key *&
-Collection<Foreign_key *>::Collection_iterator::operator*();
-template Foreign_key_element *&
-Collection<Foreign_key_element *>::Collection_iterator::operator*();
+template Column_type_element *
+    &Collection<Column_type_element *>::Collection_iterator::operator*();
+template Foreign_key *&Collection<Foreign_key *>::Collection_iterator::operator
+    *();
+template Foreign_key_element *
+    &Collection<Foreign_key_element *>::Collection_iterator::operator*();
 template Index *&Collection<Index *>::Collection_iterator::operator*();
-template Index_element *&
-Collection<Index_element *>::Collection_iterator::operator*();
-template Parameter_type_element *&
-Collection<Parameter_type_element *>::Collection_iterator::operator*();
+template Index_element *
+    &Collection<Index_element *>::Collection_iterator::operator*();
+template Parameter_type_element *
+    &Collection<Parameter_type_element *>::Collection_iterator::operator*();
 template Partition *&Collection<Partition *>::Collection_iterator::operator*();
-template dd::Partition_index *&
-Collection<dd::Partition_index *>::Collection_iterator::operator*();
-template Tablespace_file *&
-Collection<Tablespace_file *>::Collection_iterator::operator*();
+template dd::Partition_index *
+    &Collection<dd::Partition_index *>::Collection_iterator::operator*();
+template Tablespace_file *
+    &Collection<Tablespace_file *>::Collection_iterator::operator*();
 template Trigger *&Collection<Trigger *>::Collection_iterator::operator*();
-template Check_constraint *&
-Collection<Check_constraint *>::Collection_iterator::operator*();
+template Check_constraint *
+    &Collection<Check_constraint *>::Collection_iterator::operator*();
 
-template const Column *&
-Collection<Column *>::Collection_const_iterator::operator*();
-template const Column_type_element *&
-Collection<Column_type_element *>::Collection_const_iterator::operator*();
-template const Foreign_key *&
-Collection<Foreign_key *>::Collection_const_iterator::operator*();
-template const Foreign_key_element *&
-Collection<Foreign_key_element *>::Collection_const_iterator::operator*();
-template const Index *&
-Collection<Index *>::Collection_const_iterator::operator*();
-template const Index_element *&
-Collection<Index_element *>::Collection_const_iterator::operator*();
-template const Parameter *&
-Collection<Parameter *>::Collection_const_iterator::operator*();
-template const Parameter_type_element *&
-Collection<Parameter_type_element *>::Collection_const_iterator::operator*();
-template const Partition *&
-Collection<Partition *>::Collection_const_iterator::operator*();
-template const Partition_index *&
-Collection<Partition_index *>::Collection_const_iterator::operator*();
-template const Partition_value *&
-Collection<Partition_value *>::Collection_const_iterator::operator*();
-template const Tablespace_file *&
-Collection<Tablespace_file *>::Collection_const_iterator::operator*();
-template const View_routine *&
-Collection<View_routine *>::Collection_const_iterator::operator*();
-template const View_table *&
-Collection<View_table *>::Collection_const_iterator::operator*();
-template const Trigger *&
-Collection<Trigger *>::Collection_const_iterator::operator*();
-template const Check_constraint *&
-Collection<Check_constraint *>::Collection_const_iterator::operator*();
+template const Column *
+    &Collection<Column *>::Collection_const_iterator::operator*();
+template const Column_type_element *
+    &Collection<Column_type_element *>::Collection_const_iterator::operator*();
+template const Foreign_key *
+    &Collection<Foreign_key *>::Collection_const_iterator::operator*();
+template const Foreign_key_element *
+    &Collection<Foreign_key_element *>::Collection_const_iterator::operator*();
+template const Index *&Collection<Index *>::Collection_const_iterator::operator
+    *();
+template const Index_element *
+    &Collection<Index_element *>::Collection_const_iterator::operator*();
+template const Parameter *
+    &Collection<Parameter *>::Collection_const_iterator::operator*();
+template const Parameter_type_element *&Collection<
+    Parameter_type_element *>::Collection_const_iterator::operator*();
+template const Partition *
+    &Collection<Partition *>::Collection_const_iterator::operator*();
+template const Partition_index *
+    &Collection<Partition_index *>::Collection_const_iterator::operator*();
+template const Partition_value *
+    &Collection<Partition_value *>::Collection_const_iterator::operator*();
+template const Tablespace_file *
+    &Collection<Tablespace_file *>::Collection_const_iterator::operator*();
+template const View_routine *
+    &Collection<View_routine *>::Collection_const_iterator::operator*();
+template const View_table *
+    &Collection<View_table *>::Collection_const_iterator::operator*();
+template const Trigger *
+    &Collection<Trigger *>::Collection_const_iterator::operator*();
+template const Check_constraint *
+    &Collection<Check_constraint *>::Collection_const_iterator::operator*();
 
 template bool Collection<Column *>::restore_items<Abstract_table_impl>(
     Abstract_table_impl *, Open_dictionary_tables_ctx *, Raw_table *,
