@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,8 +34,7 @@ enum class enum_dd_init_type {
   DD_RESTART_OR_UPGRADE,
   DD_POPULATE_UPGRADE,
   DD_DELETE,
-  DD_UPDATE_I_S_METADATA,
-  DD_INITIALIZE_NON_DD_BASED_SYSTEM_VIEWS
+  DD_UPDATE_I_S_METADATA
 };
 
 /**
