@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,7 +25,6 @@
 #include <string.h>
 #include <sys/types.h>
 #include <time.h>
-
 #include <memory>
 #include <string>
 
@@ -95,7 +94,7 @@ static ulonglong dd_get_old_view_check_type(dd::View::enum_check_option type) {
 
   /* purecov: begin deadcode */
   LogErr(ERROR_LEVEL, ER_DD_FAILSAFE, "view check option.");
-  assert(false);
+  DBUG_ASSERT(false);
 
   return VIEW_CHECK_NONE;
   /* purecov: end */
@@ -116,7 +115,7 @@ static dd::View::enum_check_option dd_get_new_view_check_type(ulonglong type) {
 
   /* purecov: begin deadcode */
   LogErr(ERROR_LEVEL, ER_DD_FAILSAFE, "view check option.");
-  assert(false);
+  DBUG_ASSERT(false);
 
   return dd::View::CO_NONE;
   /* purecov: end */
@@ -137,7 +136,7 @@ static enum enum_view_algorithm dd_get_old_view_algorithm_type(
 
   /* purecov: begin deadcode */
   LogErr(ERROR_LEVEL, ER_DD_FAILSAFE, "view algorithm.");
-  assert(false);
+  DBUG_ASSERT(false);
 
   return VIEW_ALGORITHM_UNDEFINED;
   /* purecov: end */
@@ -158,7 +157,7 @@ static dd::View::enum_algorithm dd_get_new_view_algorithm_type(
 
   /* purecov: begin deadcode */
   LogErr(ERROR_LEVEL, ER_DD_FAILSAFE, "view algorithm.");
-  assert(false);
+  DBUG_ASSERT(false);
 
   return dd::View::VA_UNDEFINED;
   /* purecov: end */
@@ -179,7 +178,7 @@ static ulonglong dd_get_old_view_security_type(
 
   /* purecov: begin deadcode */
   LogErr(ERROR_LEVEL, ER_DD_FAILSAFE, "view security type.");
-  assert(false);
+  DBUG_ASSERT(false);
 
   return VIEW_SUID_DEFAULT;
   /* purecov: end */
@@ -200,25 +199,25 @@ static dd::View::enum_security_type dd_get_new_view_security_type(
 
   /* purecov: begin deadcode */
   LogErr(ERROR_LEVEL, ER_DD_FAILSAFE, "view security type.");
-  assert(false);
+  DBUG_ASSERT(false);
 
   return dd::View::ST_DEFAULT;
   /* purecov: end */
 }
 
 /**
-  Method to fill view columns from the first Query_block of view query.
+  Method to fill view columns from the first SELECT_LEX of view query.
 
   @param  thd       Thread Handle.
   @param  view_obj  DD view object.
-  @param  view      Table_ref object of view.
+  @param  view      TABLE_LIST object of view.
 
   @retval false     On Success.
   @retval true      On failure.
 */
 
 static bool fill_dd_view_columns(THD *thd, View *view_obj,
-                                 const Table_ref *view) {
+                                 const TABLE_LIST *view) {
   DBUG_TRACE;
 
   // Helper class which takes care restoration of THD::variables.sql_mode and
@@ -246,7 +245,7 @@ static bool fill_dd_view_columns(THD *thd, View *view_obj,
   };
 
   // Creating dummy TABLE and TABLE_SHARE objects to prepare Field objects from
-  // the items of first Query_block of the view query. We prepare these once and
+  // the items of first SELECT_LEX of the view query. We prepare these once and
   // reuse them for all the fields.
   TABLE table;
   TABLE_SHARE share;
@@ -264,13 +263,15 @@ static bool fill_dd_view_columns(THD *thd, View *view_obj,
   const dd::Properties &names_dict = view_obj->column_names();
 
   /*
-    Iterate through all the items of first Query_block if view query is of
+    Iterate through all the items of first SELECT_LEX if view query is of
     single query block. Otherwise iterate through all the type holders items
     created for unioned column types of all the query blocks.
   */
+  List_iterator_fast<Item> it(*(thd->lex->unit->get_unit_column_types()));
   List<Create_field> create_fields;
+  Item *item;
   uint i = 0;
-  for (Item *item : VisibleFields(*(thd->lex->unit->get_unit_column_types()))) {
+  while ((item = it++) != nullptr) {
     i++;
     bool is_sp_func_item = false;
     // Create temporary Field object from the item.
@@ -286,7 +287,7 @@ static bool fill_dd_view_columns(THD *thd, View *view_obj,
         if (is_sp_func_item == false && item->result_type() == INT_RESULT &&
             item->max_char_length() >= (MY_INT32_NUM_DECIMAL_DIGITS - 1)) {
           tmp_field = new (thd->mem_root)
-              Field_longlong(item->max_char_length(), item->is_nullable(),
+              Field_longlong(item->max_char_length(), item->maybe_null,
                              item->item_name.ptr(), item->unsigned_flag);
           if (tmp_field) tmp_field->init(&table);
         } else
@@ -301,7 +302,7 @@ static bool fill_dd_view_columns(THD *thd, View *view_obj,
             // tmp_table_field_from_field_type, so creating the blob field by
             // passing set_packlenth value as "true" here.
             tmp_field = new (thd->mem_root) Field_blob(
-                item->max_length, item->is_nullable(), item->item_name.ptr(),
+                item->max_length, item->maybe_null, item->item_name.ptr(),
                 item->collation.collation, true);
 
             if (tmp_field) tmp_field->init(&table);
@@ -348,23 +349,22 @@ static bool fill_dd_view_columns(THD *thd, View *view_obj,
       }
       if (!name) return true; /* purecov: inspected */
       cr_field->field_name = name;
-    } else if (thd->lex->unit->is_set_operation()) {
+    } else if (thd->lex->unit->is_union()) {
       /*
         If view query has any duplicate column names then generated unique name
-        is stored only with the first Query_block. So when Create_field instance
-        is created with type holder item, store name from first Query_block.
+        is stored only with the first SELECT_LEX. So when Create_field instance
+        is created with type holder item, store name from first SELECT_LEX.
       */
       cr_field->field_name =
-          GetNthVisibleField(thd->lex->query_block->fields, i - 1)
-              ->item_name.ptr();
+          thd->lex->select_lex->item_list[i - 1]->item_name.ptr();
     }
 
     cr_field->after = nullptr;
     cr_field->offset = 0;
     cr_field->pack_length_override = 0;
-    cr_field->is_nullable = !tmp_field->is_flag_set(NOT_NULL_FLAG);
-    cr_field->is_zerofill = tmp_field->is_flag_set(ZEROFILL_FLAG);
-    cr_field->is_unsigned = tmp_field->is_flag_set(UNSIGNED_FLAG);
+    cr_field->maybe_null = !(tmp_field->flags & NOT_NULL_FLAG);
+    cr_field->is_zerofill = (tmp_field->flags & ZEROFILL_FLAG);
+    cr_field->is_unsigned = (tmp_field->flags & UNSIGNED_FLAG);
 
     create_fields.push_back(cr_field);
   }
@@ -378,15 +378,15 @@ static bool fill_dd_view_columns(THD *thd, View *view_obj,
   View object.
 
   @param  view_obj       DD view object.
-  @param  view           Table_ref object of view.
+  @param  view           TABLE_LIST object of view.
   @param  query_tables   View query tables list.
 */
 
-static void fill_dd_view_tables(View *view_obj, const Table_ref *view,
-                                const Table_ref *query_tables) {
+static void fill_dd_view_tables(View *view_obj, const TABLE_LIST *view,
+                                const TABLE_LIST *query_tables) {
   DBUG_TRACE;
 
-  for (const Table_ref *table = query_tables; table != nullptr;
+  for (const TABLE_LIST *table = query_tables; table != nullptr;
        table = table->next_global) {
     /*
       Skip if table is not directly referred by a view or if table is a
@@ -395,11 +395,21 @@ static void fill_dd_view_tables(View *view_obj, const Table_ref *view,
     if ((table->referencing_view && table->referencing_view != view) ||
         get_dictionary()->is_dd_table_name(table->get_db_name(),
                                            table->get_table_name()) ||
-        is_temporary_table(const_cast<Table_ref *>(table)))
+        is_temporary_table(const_cast<TABLE_LIST *>(table)))
       continue;
 
-    LEX_CSTRING db_name = {table->db, table->db_length};
-    LEX_CSTRING table_name = {table->table_name, table->table_name_length};
+    LEX_CSTRING db_name;
+    LEX_CSTRING table_name;
+    if (table->schema_table_name) {
+      db_name = {table->db, table->db_length};
+      table_name = {table->schema_table_name, strlen(table->schema_table_name)};
+    } else if (table->is_view()) {
+      db_name = table->view_db;
+      table_name = table->view_name;
+    } else {
+      db_name = {table->db, table->db_length};
+      table_name = {table->table_name, table->table_name_length};
+    }
 
     // Avoid duplicate entries.
     {
@@ -450,7 +460,7 @@ static void fill_dd_view_routines(View *view_obj,
       We should get only stored functions here, as procedures are not directly
       used by views, and thus not stored as dependencies.
     */
-    assert(rt->type() == Sroutine_hash_entry::FUNCTION);
+    DBUG_ASSERT(rt->type() == Sroutine_hash_entry::FUNCTION);
 
     // view routine catalog
     view_sf_obj->set_routine_catalog(Dictionary_impl::default_catalog_name());
@@ -474,7 +484,8 @@ static void fill_dd_view_routines(View *view_obj,
   @retval true         On failure.
 */
 
-static bool fill_dd_view_definition(THD *thd, View *view_obj, Table_ref *view) {
+static bool fill_dd_view_definition(THD *thd, View *view_obj,
+                                    TABLE_LIST *view) {
   // View name.
   view_obj->set_name(view->table_name);
 
@@ -536,7 +547,7 @@ static bool fill_dd_view_definition(THD *thd, View *view_obj, Table_ref *view) {
     }
   }
 
-  time_t tm = time(nullptr);
+  time_t tm = my_time(0);
   get_date(view->timestamp.str,
            GETDATE_DATE_TIME | GETDATE_GMT | GETDATE_FIXEDLENGTH, tm);
   view->timestamp.length = PARSE_FILE_TIMESTAMPLENGTH;
@@ -554,7 +565,7 @@ static bool fill_dd_view_definition(THD *thd, View *view_obj, Table_ref *view) {
     metadata stored with column information. Fill view columns only when view
     metadata is stored with column information.
   */
-  if (!thd->lex->query_block->field_list_is_empty() &&
+  if ((thd->lex->select_lex->item_list.elements > 0) &&
       fill_dd_view_columns(thd, view_obj, view))
     return true;
 
@@ -571,7 +582,7 @@ static bool fill_dd_view_definition(THD *thd, View *view_obj, Table_ref *view) {
   return false;
 }
 
-bool update_view(THD *thd, dd::View *new_view, Table_ref *view) {
+bool update_view(THD *thd, dd::View *new_view, TABLE_LIST *view) {
   // Clear the columns, tables and routines since it will be added later.
   new_view->remove_children();
 
@@ -586,7 +597,7 @@ bool update_view(THD *thd, dd::View *new_view, Table_ref *view) {
   return thd->dd_client()->update(new_view);
 }
 
-bool create_view(THD *thd, const dd::Schema &schema, Table_ref *view) {
+bool create_view(THD *thd, const dd::Schema &schema, TABLE_LIST *view) {
   // Create dd::View object.
   bool hidden_system_view = false;
   std::unique_ptr<dd::View> view_obj;
@@ -606,8 +617,8 @@ bool create_view(THD *thd, const dd::Schema &schema, Table_ref *view) {
   return thd->dd_client()->store(view_obj.get());
 }
 
-bool read_view(Table_ref *view, const dd::View &view_obj, MEM_ROOT *mem_root) {
-  // Fill Table_ref 'view' with view details.
+bool read_view(TABLE_LIST *view, const dd::View &view_obj, MEM_ROOT *mem_root) {
+  // Fill TABLE_LIST 'view' with view details.
   String_type definer_user = view_obj.definer_user();
   view->definer.user.length = definer_user.length();
   view->definer.user.str = (char *)strmake_root(mem_root, definer_user.c_str(),
@@ -648,17 +659,15 @@ bool read_view(Table_ref *view, const dd::View &view_obj, MEM_ROOT *mem_root) {
   // Get view_client_cs_name. Note that this is the character set name.
   CHARSET_INFO *collation =
       dd_get_mysql_charset(view_obj.client_collation_id());
-  assert(collation);
-  const char *csname = collation->csname;
-  view->view_client_cs_name.length = strlen(csname);
-  view->view_client_cs_name.str = strdup_root(mem_root, csname);
+  DBUG_ASSERT(collation);
+  view->view_client_cs_name.length = strlen(collation->csname);
+  view->view_client_cs_name.str = strdup_root(mem_root, collation->csname);
 
   // Get view_connection_cl_name. Note that this is the collation name.
   collation = dd_get_mysql_charset(view_obj.connection_collation_id());
-  assert(collation);
-  view->view_connection_cl_name.length = strlen(collation->m_coll_name);
-  view->view_connection_cl_name.str =
-      strdup_root(mem_root, collation->m_coll_name);
+  DBUG_ASSERT(collation);
+  view->view_connection_cl_name.length = strlen(collation->name);
+  view->view_connection_cl_name.str = strdup_root(mem_root, collation->name);
 
   if (!(view->definer.user.str && view->definer.host.str &&  // OOM
         view->view_body_utf8.str && view->select_stmt.str &&
@@ -703,7 +712,7 @@ bool update_view_status(THD *thd, const char *schema_name,
   dd::Properties *view_options = &new_view->options();
   view_options->set("view_valid", status);
 
-  Implicit_substatement_state_guard substatement_guard(thd);
+  Disable_gtid_state_update_guard disabler(thd);
 
   // Update DD tables.
   if (client->update(new_view)) {
