@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,7 +57,6 @@
 #include "sql/dd/impl/upgrade/dd.h"             // dd::upgrade::upgrade_tables
 #include "sql/dd/impl/upgrade/server.h"  // dd::upgrade::do_server_upgrade_checks
 #include "sql/dd/impl/utils.h"           // dd::execute_query
-#include "sql/dd/info_schema/metadata.h"  // IS_DD_VERSION
 #include "sql/dd/object_id.h"
 #include "sql/dd/types/abstract_table.h"
 #include "sql/dd/types/object_table.h"             // dd::Object_table
@@ -73,7 +71,6 @@
 #include "sql/mysqld.h"
 #include "sql/sd_notify.h"  // sysd::notify
 #include "sql/thd_raii.h"
-#include "storage/perfschema/pfs_dd_version.h"  // PFS_DD_VERSION
 
 using namespace dd;
 
@@ -152,7 +149,7 @@ bool update_system_tables(THD *thd) {
 
       const Object_table *table_def = System_tables::instance()->find_table(
           MYSQL_SCHEMA_NAME.str, it->first);
-      assert(table_def);
+      DBUG_ASSERT(table_def);
 
       std::unique_ptr<dd::Properties> tbl_props(
           Properties::parse_properties(tbl_prop_str));
@@ -175,7 +172,7 @@ bool update_system_tables(THD *thd) {
 
 // Create a DD table using the target table definition.
 bool create_target_table(THD *thd, const Object_table *object_table) {
-  assert(object_table != nullptr);
+  DBUG_ASSERT(object_table != nullptr);
 
   /*
    The target table definition may not be present if the table
@@ -187,9 +184,9 @@ bool create_target_table(THD *thd, const Object_table *object_table) {
   const Object_table_definition *target_table_def =
       object_table->target_table_definition();
 
-  assert(target_table_def != nullptr);
+  DBUG_ASSERT(target_table_def != nullptr);
   target_ddl_statement = target_table_def->get_ddl();
-  assert(!target_ddl_statement.empty());
+  DBUG_ASSERT(!target_ddl_statement.empty());
 
   return dd::execute_query(thd, target_ddl_statement);
 }
@@ -203,7 +200,7 @@ bool create_actual_table(THD *thd, const Object_table *object_table) {
     case, we just skip them.
   */
   if (object_table == nullptr) {
-    assert(bootstrap::DD_bootstrap_ctx::instance().is_minor_downgrade());
+    DBUG_ASSERT(bootstrap::DD_bootstrap_ctx::instance().is_minor_downgrade());
     return false;
   }
 
@@ -219,7 +216,7 @@ bool create_actual_table(THD *thd, const Object_table *object_table) {
   if (actual_table_def == nullptr) return false;
 
   actual_ddl_statement = actual_table_def->get_ddl();
-  assert(!actual_ddl_statement.empty());
+  DBUG_ASSERT(!actual_ddl_statement.empty());
 
   return dd::execute_query(thd, actual_ddl_statement);
 }
@@ -252,7 +249,7 @@ bool acquire_exclusive_mdl(THD *thd) {
     if ((*it)->entity() == nullptr) continue;
 
     MDL_request *table_request = new (thd->mem_root) MDL_request;
-    if (table_request == nullptr) return true;
+    if (table_request == NULL) return true;
     MDL_REQUEST_INIT(table_request, MDL_key::TABLE, MYSQL_SCHEMA_NAME.str,
                      (*it)->entity()->name().c_str(), MDL_EXCLUSIVE,
                      MDL_TRANSACTION);
@@ -392,7 +389,7 @@ bool flush_meta_data(THD *thd) {
           (*it)->property() == System_tables::Types::SYSTEM)
         continue;
 
-      assert((*it)->entity()->name() == (*clone_it)->name());
+      DBUG_ASSERT((*it)->entity()->name() == (*clone_it)->name());
 
       // We must set the ID to INVALID to let the object get an auto inc ID.
       (*clone_it)->set_id(INVALID_OBJECT_ID);
@@ -458,8 +455,8 @@ bool flush_meta_data(THD *thd) {
         thd, static_cast<Tablespace *>(dd_tspace_clone.get()));
 
     // Make sure the IDs after storing are as expected.
-    assert(dd_schema_clone->id() == 1);
-    assert(dd_tspace_clone->id() == 1);
+    DBUG_ASSERT(dd_schema_clone->id() == 1);
+    DBUG_ASSERT(dd_tspace_clone->id() == 1);
 
     /*
       Finally, we update the core registry of the DD tables. This must be
@@ -475,7 +472,7 @@ bool flush_meta_data(THD *thd) {
           (*it)->property() == System_tables::Types::SYSTEM)
         continue;
 
-      assert((*it)->entity()->name() == (*table_it)->name());
+      DBUG_ASSERT((*it)->entity()->name() == (*table_it)->name());
       dd::cache::Storage_adapter::instance()->core_drop(
           thd, static_cast<const Table *>(*table_it));
     }
@@ -491,7 +488,7 @@ bool flush_meta_data(THD *thd) {
         continue;
 
       if ((*it)->property() == System_tables::Types::CORE) {
-        assert((*it)->entity()->name() == (*clone_it)->name());
+        DBUG_ASSERT((*it)->entity()->name() == (*clone_it)->name());
         dd::cache::Storage_adapter::instance()->core_store(
             thd, static_cast<Table *>((*clone_it).get()));
       }
@@ -662,15 +659,15 @@ bool verify_contents(THD *thd) {
   Object_id dd_schema_id =
       cache::Storage_adapter::instance()->core_get_id<Schema>(schema_key);
 
-  assert(dd_schema_id == MYSQL_SCHEMA_DD_ID);
+  DBUG_ASSERT(dd_schema_id == MYSQL_SCHEMA_DD_ID);
   if (dd_schema_id == INVALID_OBJECT_ID) {
     LogErr(ERROR_LEVEL, ER_DD_SCHEMA_NOT_FOUND, MYSQL_SCHEMA_NAME.str);
     return dd::end_transaction(thd, true);
   }
-  assert(cache::Storage_adapter::instance()->core_size<Schema>() == 1);
+  DBUG_ASSERT(cache::Storage_adapter::instance()->core_size<Schema>() == 1);
 
   // Verify that the core DD tables are present.
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   size_t n_core_tables = 0;
 #endif
   for (System_tables::Const_iterator it =
@@ -680,7 +677,7 @@ bool verify_contents(THD *thd) {
     // Skip extraneous tables for minor downgrade.
     if ((*it)->entity() == nullptr) continue;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     n_core_tables++;
 #endif
 
@@ -689,15 +686,15 @@ bool verify_contents(THD *thd) {
     Object_id dd_table_id =
         cache::Storage_adapter::instance()->core_get_id<Table>(table_key);
 
-    assert(dd_table_id != INVALID_OBJECT_ID);
+    DBUG_ASSERT(dd_table_id != INVALID_OBJECT_ID);
     if (dd_table_id == INVALID_OBJECT_ID) {
       LogErr(ERROR_LEVEL, ER_DD_TABLE_NOT_FOUND,
              (*it)->entity()->name().c_str());
       return dd::end_transaction(thd, true);
     }
   }
-  assert(cache::Storage_adapter::instance()->core_size<Abstract_table>() ==
-         n_core_tables);
+  DBUG_ASSERT(cache::Storage_adapter::instance()->core_size<Abstract_table>() ==
+              n_core_tables);
 
   // Verify that the dictionary tablespace is present and that its id == 1.
   Tablespace::Name_key tspace_key;
@@ -705,12 +702,12 @@ bool verify_contents(THD *thd) {
   Object_id dd_tspace_id =
       cache::Storage_adapter::instance()->core_get_id<Tablespace>(tspace_key);
 
-  assert(dd_tspace_id == MYSQL_TABLESPACE_DD_ID);
+  DBUG_ASSERT(dd_tspace_id == MYSQL_TABLESPACE_DD_ID);
   if (dd_tspace_id == INVALID_OBJECT_ID) {
     LogErr(ERROR_LEVEL, ER_DD_TABLESPACE_NOT_FOUND, MYSQL_TABLESPACE_NAME.str);
     return dd::end_transaction(thd, true);
   }
-  assert(cache::Storage_adapter::instance()->core_size<Tablespace>() == 1);
+  DBUG_ASSERT(cache::Storage_adapter::instance()->core_size<Tablespace>() == 1);
 
   return dd::end_transaction(thd, false);
 }
@@ -781,14 +778,8 @@ bool DDSE_dict_init(THD *thd, dict_init_mode_t dict_init_mode, uint version) {
       }
       if (!DD_bootstrap_ctx::instance().supported_server_version(
               server_version)) {
-        if (server_version > MYSQL_VERSION_ID &&
-            !DD_bootstrap_ctx::instance().is_server_patch_downgrade(
-                server_version))
-          LogErr(ERROR_LEVEL, ER_INVALID_SERVER_DOWNGRADE_NOT_PATCH,
-                 server_version, MYSQL_VERSION_ID);
-        else
-          LogErr(ERROR_LEVEL, ER_SERVER_UPGRADE_VERSION_NOT_SUPPORTED,
-                 server_version);
+        LogErr(ERROR_LEVEL, ER_SERVER_UPGRADE_VERSION_NOT_SUPPORTED,
+               server_version);
         return true;
       }
     }
@@ -866,17 +857,9 @@ bool initialize_dictionary(THD *thd, bool is_dd_upgrade_57,
       populate_tables(thd) ||
       update_properties(thd, nullptr, nullptr,
                         String_type(MYSQL_SCHEMA_NAME.str)) ||
-      verify_contents(thd) || update_versions(thd, is_dd_upgrade_57))
+      verify_contents(thd) | update_versions(thd, is_dd_upgrade_57))
     return true;
 
-  DBUG_EXECUTE_IF(
-      "schema_read_only",
-      if (dd::execute_query(thd, "CREATE SCHEMA schema_read_only") ||
-          dd::execute_query(thd, "ALTER SCHEMA schema_read_only READ ONLY=1") ||
-          dd::execute_query(thd, "CREATE TABLE schema_read_only.t(i INT)") ||
-          dd::execute_query(thd, "DROP SCHEMA schema_read_only"))
-          assert(false););
-
   bootstrap::DD_bootstrap_ctx::instance().set_stage(bootstrap::Stage::FINISHED);
 
   return false;
@@ -896,7 +879,7 @@ bool initialize(THD *thd) {
   Disable_autocommit_guard autocommit_guard(thd);
 
   Dictionary_impl *d = dd::Dictionary_impl::instance();
-  assert(d);
+  DBUG_ASSERT(d);
   cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
 
   /*
@@ -909,7 +892,7 @@ bool initialize(THD *thd) {
       initialize_dictionary(thd, false, d))
     return true;
 
-  assert(d->get_target_dd_version() == d->get_actual_dd_version(thd));
+  DBUG_ASSERT(d->get_target_dd_version() == d->get_actual_dd_version(thd));
   LogErr(INFORMATION_LEVEL, ER_DD_VERSION_INSTALLED,
          d->get_target_dd_version());
   return false;
@@ -932,7 +915,7 @@ bool restart(THD *thd) {
   Disable_autocommit_guard autocommit_guard(thd);
 
   Dictionary_impl *d = dd::Dictionary_impl::instance();
-  assert(d);
+  DBUG_ASSERT(d);
   cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
 
   store_predefined_tablespace_metadata(thd);
@@ -947,15 +930,6 @@ bool restart(THD *thd) {
     return true;
   }
 
-  DBUG_EXECUTE_IF(
-      "schema_read_only",
-      if (dd::execute_query(thd, "CREATE SCHEMA schema_read_only") ||
-          dd::execute_query(thd, "ALTER SCHEMA schema_read_only READ ONLY=1") ||
-          dd::execute_query(thd, "CREATE TABLE schema_read_only.t(i INT)") ||
-          dd::execute_query(thd, "DROP SCHEMA schema_read_only") ||
-          dd::execute_query(thd, "CREATE TABLE IF NOT EXISTS S.restart(i INT)"))
-          assert(false););
-
   bootstrap::DD_bootstrap_ctx::instance().set_stage(bootstrap::Stage::FINISHED);
   LogErr(INFORMATION_LEVEL, ER_DD_VERSION_FOUND, d->get_actual_dd_version(thd));
 
@@ -1001,9 +975,9 @@ bool setup_dd_objects_and_collations(THD *thd) {
   Disable_autocommit_guard autocommit_guard(thd);
 
   Dictionary_impl *d = dd::Dictionary_impl::instance();
-  assert(d);
+  DBUG_ASSERT(d);
 
-  assert(d->get_target_dd_version() == d->get_actual_dd_version(thd));
+  DBUG_ASSERT(d->get_target_dd_version() == d->get_actual_dd_version(thd));
 
   /*
     In this context, we initialize the target tables directly since this
@@ -1047,14 +1021,14 @@ void store_predefined_tablespace_metadata(THD *thd) {
         tablespace_def->get_files();
     List_iterator<const Plugin_tablespace::Plugin_tablespace_file> file_it(
         files);
-    const Plugin_tablespace::Plugin_tablespace_file *file = nullptr;
+    const Plugin_tablespace::Plugin_tablespace_file *file = NULL;
     while ((file = file_it++)) {
       Tablespace_file *space_file = tablespace->add_file();
       space_file->set_filename(file->get_name());
       space_file->set_se_private_data(file->get_se_private_data());
     }
 
-    // All the predefined tablespace are unencrypted (at least for now).
+    // All the predefined tablespace are unencrypted (atleast for now).
     tablespace->options().set("encryption", "N");
 
     /*
@@ -1071,61 +1045,15 @@ void store_predefined_tablespace_metadata(THD *thd) {
 }
 
 bool create_dd_schema(THD *thd) {
-  return dd::execute_query(
-             thd, dd::String_type("CREATE SCHEMA ") +
-                      dd::String_type(MYSQL_SCHEMA_NAME.str) +
-                      dd::String_type(" DEFAULT COLLATE ") +
-                      dd::String_type(default_charset_info->m_coll_name)) ||
+  return dd::execute_query(thd,
+                           dd::String_type("CREATE SCHEMA ") +
+                               dd::String_type(MYSQL_SCHEMA_NAME.str) +
+                               dd::String_type(" DEFAULT COLLATE ") +
+                               dd::String_type(default_charset_info->name)) ||
          dd::execute_query(thd, dd::String_type("USE ") +
                                     dd::String_type(MYSQL_SCHEMA_NAME.str));
 }
 
-bool getprop(THD *thd, const char *key, uint *value, bool silent = false) {
-  bool exists = false;
-  if (dd::tables::DD_properties::instance().get(thd, key, value, &exists) ||
-      !exists) {
-    /* purecov: begin inspected */
-    if (!silent) LogErr(ERROR_LEVEL, ER_FAILED_GET_DD_PROPERTY, key);
-    return true;
-    /* purecov: end */
-  }
-  return false;
-}
-
-bool getprop(THD *thd, const char *key, String_type *value,
-             bool silent = false) {
-  bool exists = false;
-  if (dd::tables::DD_properties::instance().get(thd, key, value, &exists) ||
-      !exists) {
-    /* purecov: begin inspected */
-    if (!silent) LogErr(ERROR_LEVEL, ER_FAILED_GET_DD_PROPERTY, key);
-    return true;
-    /* purecov: end */
-  }
-  return false;
-}
-
-bool setprop(THD *thd, const char *key, const uint value, bool silent = false) {
-  if (dd::tables::DD_properties::instance().set(thd, key, value)) {
-    /* purecov: begin inspected */
-    if (!silent) LogErr(ERROR_LEVEL, ER_FAILED_SET_DD_PROPERTY, key);
-    return true;
-    /* purecov: end */
-  }
-  return false;
-}
-
-bool setprop(THD *thd, const char *key, const String_type &value,
-             bool silent = false) {
-  if (dd::tables::DD_properties::instance().set(thd, key, value)) {
-    /* purecov: begin inspected */
-    if (!silent) LogErr(ERROR_LEVEL, ER_FAILED_SET_DD_PROPERTY, key);
-    return true;
-    /* purecov: end */
-  }
-  return false;
-}
-
 bool initialize_dd_properties(THD *thd) {
   // Create the dd_properties table.
   const Object_table_definition *dd_properties_def =
@@ -1136,92 +1064,31 @@ bool initialize_dd_properties(THD *thd) {
     We can now decide which version number we will use for the DD, and
     initialize the DD_bootstrap_ctx with the relevant version number.
   */
-  uint actual_dd_version = dd::DD_VERSION;
+  uint actual_version = dd::DD_VERSION;
   uint actual_server_version = MYSQL_VERSION_ID;
   uint upgraded_server_version = MYSQL_VERSION_ID;
 
-  bootstrap::DD_bootstrap_ctx::instance().set_actual_dd_version(
-      actual_dd_version);
+  bootstrap::DD_bootstrap_ctx::instance().set_actual_dd_version(actual_version);
   bootstrap::DD_bootstrap_ctx::instance().set_upgraded_server_version(
       actual_server_version);
 
   if (!opt_initialize) {
-    /*
-      First get the DD version, the actual server version and the last
-      completed version upgrade (which may be older in case e.g. the
-      system table upgrade step failed).
-    */
     bool exists = false;
-    if (dd::tables::DD_properties::instance().get(
-            thd, "DD_VERSION", &actual_dd_version, &exists) ||
+    bool exists_server = false;
+    bool exists_upgraded_version = false;
+
+    // Check 'DD_version' too in order to catch an upgrade from 8.0.3.
+    if (dd::tables::DD_properties::instance().get(thd, "DD_VERSION",
+                                                  &actual_version, &exists) ||
         !exists) {
       LogErr(ERROR_LEVEL, ER_DD_NO_VERSION_FOUND);
       return true;
     }
 
-    if (dd::tables::DD_properties::instance().get(
-            thd, "MYSQLD_VERSION", &actual_server_version, &exists) ||
-        !exists)
-      return true;
-
-    if (dd::tables::DD_properties::instance().get(
-            thd, "MYSQLD_VERSION_UPGRADED", &upgraded_server_version,
-            &exists) ||
-        !exists)
-      upgraded_server_version = actual_server_version;
-
-    /*
-      Get information from DD properties. Do this after 8.2.0 / 8.0.35.
-      Older versions do not have the required information available.
-    */
-    String_type mysql_version_stability{"INNOVATION"};
-    uint server_downgrade_threshold = 0;
-    uint server_upgrade_threshold = 0;
     /* purecov: begin inspected */
-    if (actual_server_version >= 80035 && actual_server_version != 80100 &&
-        (getprop(thd, "MYSQL_VERSION_STABILITY", &mysql_version_stability) ||
-         getprop(thd, "SERVER_DOWNGRADE_THRESHOLD",
-                 &server_downgrade_threshold) ||
-         getprop(thd, "SERVER_UPGRADE_THRESHOLD", &server_upgrade_threshold)))
-      return true;
-
-    /* Is there a server version change? */
-    if (MYSQL_VERSION_ID != actual_server_version) {
-      if (MYSQL_VERSION_ID > actual_server_version) {
-        // This is an upgrade attempt.
-        if ((MYSQL_VERSION_ID / 10000) != (actual_server_version / 10000) &&
-            (mysql_version_stability != "LTS" ||
-             actual_server_version / 100 == 800 ||
-             MYSQL_VERSION_ID / 10000 != actual_server_version / 10000 + 1)) {
-          LogErr(ERROR_LEVEL, ER_INVALID_SERVER_UPGRADE_NOT_LTS,
-                 actual_server_version, MYSQL_VERSION_ID,
-                 actual_server_version);
-          return true;
-        } else if (MYSQL_VERSION_ID < server_upgrade_threshold &&
-                   MYSQL_VERSION_ID / 100 != actual_server_version / 100) {
-          LogErr(ERROR_LEVEL, ER_BEYOND_SERVER_UPGRADE_THRESHOLD,
-                 actual_server_version, MYSQL_VERSION_ID,
-                 server_upgrade_threshold);
-          return true;
-        }
-      } else {
-        // This is a downgrade attempt.
-        if (MYSQL_VERSION_ID / 100 != actual_server_version / 100) {
-          LogErr(ERROR_LEVEL, ER_INVALID_SERVER_DOWNGRADE_NOT_PATCH,
-                 actual_server_version, MYSQL_VERSION_ID);
-          return true;
-        } else if (MYSQL_VERSION_ID < server_downgrade_threshold) {
-          LogErr(ERROR_LEVEL, ER_BEYOND_SERVER_DOWNGRADE_THRESHOLD,
-                 actual_server_version, MYSQL_VERSION_ID,
-                 server_downgrade_threshold);
-          return true;
-        }
-      }
-    }
-
-    if (actual_dd_version != dd::DD_VERSION) {
+    if (actual_version != dd::DD_VERSION) {
       bootstrap::DD_bootstrap_ctx::instance().set_actual_dd_version(
-          actual_dd_version);
+          actual_version);
       if (opt_no_dd_upgrade) {
         push_deprecated_warn(thd, "--no-dd-upgrade", "--upgrade=NONE");
         LogErr(ERROR_LEVEL, ER_DD_UPGRADE_OFF);
@@ -1235,7 +1102,7 @@ bool initialize_dd_properties(THD *thd) {
         */
         if (!bootstrap::DD_bootstrap_ctx::instance().is_minor_downgrade()) {
           LogErr(ERROR_LEVEL, ER_DD_UPGRADE_VERSION_NOT_SUPPORTED,
-                 actual_dd_version);
+                 actual_version);
           return true;
         }
 
@@ -1245,22 +1112,26 @@ bool initialize_dd_properties(THD *thd) {
                 &exists) ||
             !exists || minor_downgrade_threshold > dd::DD_VERSION) {
           LogErr(ERROR_LEVEL, ER_DD_MINOR_DOWNGRADE_VERSION_NOT_SUPPORTED,
-                 actual_dd_version);
+                 actual_version);
           return true;
         }
       }
     }
     /* purecov: end */
 
+    if (dd::tables::DD_properties::instance().get(
+            thd, "MYSQLD_VERSION", &actual_server_version, &exists_server) ||
+        !exists_server)
+      return true;
+
+    if (dd::tables::DD_properties::instance().get(
+            thd, "MYSQLD_VERSION_UPGRADED", &upgraded_server_version,
+            &exists_upgraded_version) ||
+        !exists_upgraded_version)
+      upgraded_server_version = actual_server_version;
     bootstrap::DD_bootstrap_ctx::instance().set_upgraded_server_version(
         upgraded_server_version);
 
-    /*
-      If the previous upgrade was not completed, e.g. because system table
-      upgrade failed, then we will not accept a new upgrade attempt to an
-      even newer version. First, the previous upgrade must be completed so
-      that actual_server_version == upgraded_server_version.
-    */
     if (DBUG_EVALUATE_IF("simulate_mysql_upgrade_skip_pending", true,
                          actual_server_version != upgraded_server_version &&
                              actual_server_version != MYSQL_VERSION_ID)) {
@@ -1269,10 +1140,6 @@ bool initialize_dd_properties(THD *thd) {
       return true;
     }
 
-    /*
-      Check if we are doing a server upgrade or a server downgrade. An
-      upgrade of the DD will of course imply a server upgrade.
-    */
     if (upgraded_server_version != MYSQL_VERSION_ID) {
       /*
         This check is also done in DDSE_dict_init() based on the version
@@ -1284,12 +1151,11 @@ bool initialize_dd_properties(THD *thd) {
       if (!bootstrap::DD_bootstrap_ctx::instance().supported_server_version()) {
         LogErr(ERROR_LEVEL, ER_SERVER_UPGRADE_VERSION_NOT_SUPPORTED,
                actual_server_version);
-        assert(false);
+        DBUG_ASSERT(false);
         return true;
       }
     }
 
-    DBUG_EXECUTE_IF("error_during_bootstrap", return true;);
     /*
       Reject restarting with a changed LCTN setting, since the collation
       for LCTN-dependent columns is decided during server initialization.
@@ -1311,15 +1177,16 @@ bool initialize_dd_properties(THD *thd) {
   else if (bootstrap::DD_bootstrap_ctx::instance().is_restart())
     LogErr(INFORMATION_LEVEL, ER_DD_RESTART, dd::DD_VERSION);
   else if (bootstrap::DD_bootstrap_ctx::instance().is_minor_downgrade())
-    LogErr(INFORMATION_LEVEL, ER_DD_MINOR_DOWNGRADE, actual_dd_version,
+    LogErr(INFORMATION_LEVEL, ER_DD_MINOR_DOWNGRADE, actual_version,
            dd::DD_VERSION);
   else {
     /*
-      If none of the above, then this must be DD upgrade, server
-      upgrade, or patch downgrade.
+      If none of the above, then this must be DD upgrade or server
+      upgrade, or both.
     */
     if (bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade()) {
-      LogErr(SYSTEM_LEVEL, ER_DD_UPGRADE, actual_dd_version, dd::DD_VERSION);
+      LogErr(SYSTEM_LEVEL, ER_DD_UPGRADE, actual_version, dd::DD_VERSION);
+      log_sink_buffer_check_timeout();
       sysd::notify("STATUS=Data Dictionary upgrade in progress\n");
     }
     if (bootstrap::DD_bootstrap_ctx::instance().is_server_upgrade()) {
@@ -1330,20 +1197,9 @@ bool initialize_dd_properties(THD *thd) {
       }
       LogErr(INFORMATION_LEVEL, ER_SERVER_UPGRADE_FROM_VERSION,
              upgraded_server_version, MYSQL_VERSION_ID);
-    } else if (bootstrap::DD_bootstrap_ctx::instance()
-                   .is_server_patch_downgrade()) {
-      /* purecov: begin inspected */
-      if (opt_upgrade_mode == UPGRADE_NONE) {
-        LogErr(ERROR_LEVEL, ER_SERVER_UPGRADE_OFF);
-        return true;
-      }
-      LogErr(INFORMATION_LEVEL, ER_SERVER_DOWNGRADE_FROM_VERSION,
-             upgraded_server_version, MYSQL_VERSION_ID);
-      /* purecov: end */
     }
-    assert(bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade() ||
-           bootstrap::DD_bootstrap_ctx::instance().is_server_upgrade() ||
-           bootstrap::DD_bootstrap_ctx::instance().is_server_patch_downgrade());
+    DBUG_ASSERT(bootstrap::DD_bootstrap_ctx::instance().is_dd_upgrade() ||
+                bootstrap::DD_bootstrap_ctx::instance().is_server_upgrade());
   }
 
   /*
@@ -1483,7 +1339,7 @@ bool sync_meta_data(THD *thd) {
             thd, tspace_key, ISO_READ_COMMITTED, true, &tmp_tspace))
       return dd::end_transaction(thd, true);
 
-    assert(tmp_schema != nullptr && tmp_tspace != nullptr);
+    DBUG_ASSERT(tmp_schema != nullptr && tmp_tspace != nullptr);
     std::unique_ptr<Schema> persisted_dd_schema(
         const_cast<Schema *>(tmp_schema));
     std::unique_ptr<Tablespace> persisted_dd_tspace(
@@ -1491,7 +1347,7 @@ bool sync_meta_data(THD *thd) {
 
     // If the persisted meta data indicates that the DD tablespace is
     // encrypted, then we record this fact to make sure the DDL statements
-    // that are generated during e.g. upgrade will have the correct
+    // that are genereated during e.g. upgrade will have the correct
     // encryption option.
     String_type encryption("");
     Object_table_definition_impl::set_dd_tablespace_encrypted(
@@ -1531,56 +1387,6 @@ bool sync_meta_data(THD *thd) {
       if (thd->dd_client()->acquire((*it)->entity()->get_name(), &tspace))
         return dd::end_transaction(thd, true);
 
-      /*
-        There is a possibility of the InnoDB system tablespace being extended by
-        adding additional datafiles during server restart. Hence, we would need
-        to check the DD tables to verify which tablespace datafiles have been
-        persisted already and then add the extra datafiles to system tablespace
-        and persist the updated metadata.
-
-        The documentation mentions that datafiles can only be added to the sytem
-        tablespace and can not be removed.
-      */
-      Tablespace::Name_key predef_tspace_key;
-      tspace->update_name_key(&predef_tspace_key);
-      const Tablespace *predef_tspace = nullptr;
-
-      if (dd::cache::Storage_adapter::instance()->get(
-              thd, predef_tspace_key, ISO_READ_COMMITTED, true, &predef_tspace))
-        return dd::end_transaction(thd, true);
-
-      std::unique_ptr<Tablespace> predef_tspace_persist(
-          const_cast<Tablespace *>(predef_tspace));
-
-      size_t existing_datafiles, added_datafiles;
-      existing_datafiles = predef_tspace->files().size();
-      added_datafiles = tspace->files().size() - existing_datafiles;
-      if (added_datafiles) {
-        std::unordered_set<std::string> predef_tspace_files;
-        for (auto tspace_file_it = predef_tspace->files().begin();
-             tspace_file_it != predef_tspace->files().end(); ++tspace_file_it) {
-          predef_tspace_files.insert((*tspace_file_it)->filename().c_str());
-        }
-
-        List<const Plugin_tablespace::Plugin_tablespace_file> files =
-            (*it)->entity()->get_files();
-        List_iterator<const Plugin_tablespace::Plugin_tablespace_file> file_it(
-            files);
-        const Plugin_tablespace::Plugin_tablespace_file *file = nullptr;
-        while ((file = file_it++)) {
-          if (predef_tspace_files.find(file->get_name()) ==
-              predef_tspace_files.end()) {
-            Tablespace_file *space_file = predef_tspace_persist->add_file();
-            space_file->set_filename(file->get_name());
-            space_file->set_se_private_data(file->get_se_private_data());
-          }
-        }
-        dd::cache::Storage_adapter::instance()->store(
-            thd, predef_tspace_persist.get());
-        DBUG_PRINT("info", ("Persisted metadata for additional datafile(s) "
-                            "added to the predefined tablespace %s",
-                            predef_tspace_persist->name().c_str()));
-      }
       dd::cache::Storage_adapter::instance()->core_drop(thd, tspace);
     }
     /*
@@ -1609,8 +1415,8 @@ bool sync_meta_data(THD *thd) {
         thd, persisted_dd_tspace.get());
 
     // Make sure the IDs after storing are as expected.
-    assert(persisted_dd_schema->id() == 1);
-    assert(persisted_dd_tspace->id() == 1);
+    DBUG_ASSERT(persisted_dd_schema->id() == 1);
+    DBUG_ASSERT(persisted_dd_tspace->id() == 1);
 
     /*
       Finally, we update the core registry of the DD tables. This must be
@@ -1626,7 +1432,7 @@ bool sync_meta_data(THD *thd) {
         version we are upgrading from.
       */
       if ((*table_it) != nullptr) {
-        assert((*it)->entity()->name() == (*table_it)->name());
+        DBUG_ASSERT((*it)->entity()->name() == (*table_it)->name());
         dd::cache::Storage_adapter::instance()->core_drop(thd, *table_it);
       }
     }
@@ -1729,7 +1535,7 @@ bool sync_meta_data(THD *thd) {
                                                        "UPGRADE_TARGET_SCHEMA");
 
   if (actual_schema_exists || target_schema_exists)
-    return dd::end_transaction(thd, false);
+    return end_transaction(thd, false);
 
   return false;
 }
@@ -1752,7 +1558,7 @@ bool update_properties(THD *thd, const std::set<String_type> *create_set,
         This will not be called for minor downgrade, so all tables
         will have a corresponding Object_table.
       */
-      assert((*it)->entity() != nullptr);
+      DBUG_ASSERT((*it)->entity() != nullptr);
       const Object_table_definition *table_def =
           (*it)->entity()->target_table_definition();
 
@@ -1792,7 +1598,7 @@ bool update_properties(THD *thd, const std::set<String_type> *create_set,
         return dd::end_transaction(thd, true);
 
       // All non-abandoned tables should have a table object present.
-      assert(dd_table != nullptr);
+      DBUG_ASSERT(dd_table != nullptr);
 
       std::unique_ptr<dd::Properties> tbl_props(
           dd::Properties::parse_properties(""));
@@ -1853,28 +1659,32 @@ bool update_versions(THD *thd, bool is_dd_upgrade_57) {
     mysqld server version.
   */
   if (opt_initialize) {
-    if (setprop(thd, "DD_VERSION", dd::DD_VERSION) ||
-        setprop(thd, "MINOR_DOWNGRADE_THRESHOLD",
-                dd::DD_VERSION_MINOR_DOWNGRADE_THRESHOLD) ||
-        setprop(thd, "SDI_VERSION", dd::SDI_VERSION) ||
-        setprop(thd, "LCTN", lower_case_table_names) ||
-        setprop(thd, "MYSQL_VERSION_STABILITY", MYSQL_VERSION_STABILITY) ||
-        setprop(thd, "SERVER_DOWNGRADE_THRESHOLD",
-                SERVER_DOWNGRADE_THRESHOLD) ||
-        setprop(thd, "SERVER_UPGRADE_THRESHOLD", SERVER_UPGRADE_THRESHOLD) ||
-        setprop(thd, "MYSQLD_VERSION_LO", MYSQL_VERSION_ID) ||
-        setprop(thd, "MYSQLD_VERSION_HI", MYSQL_VERSION_ID) ||
-        setprop(thd, "MYSQLD_VERSION", MYSQL_VERSION_ID))
+    if (dd::tables::DD_properties::instance().set(thd, "DD_VERSION",
+                                                  dd::DD_VERSION) ||
+        dd::tables::DD_properties::instance().set(
+            thd, "MINOR_DOWNGRADE_THRESHOLD",
+            dd::DD_VERSION_MINOR_DOWNGRADE_THRESHOLD) ||
+        dd::tables::DD_properties::instance().set(thd, "SDI_VERSION",
+                                                  dd::SDI_VERSION) ||
+        dd::tables::DD_properties::instance().set(thd, "LCTN",
+                                                  lower_case_table_names) ||
+        dd::tables::DD_properties::instance().set(thd, "MYSQLD_VERSION_LO",
+                                                  MYSQL_VERSION_ID) ||
+        dd::tables::DD_properties::instance().set(thd, "MYSQLD_VERSION_HI",
+                                                  MYSQL_VERSION_ID) ||
+        dd::tables::DD_properties::instance().set(thd, "MYSQLD_VERSION",
+                                                  MYSQL_VERSION_ID))
       return dd::end_transaction(thd, true);
 
     if (is_dd_upgrade_57) {
-      if (setprop(thd, "MYSQLD_VERSION_UPGRADED",
-                  bootstrap::SERVER_VERSION_50700))
+      if (dd::tables::DD_properties::instance().set(
+              thd, "MYSQLD_VERSION_UPGRADED", bootstrap::SERVER_VERSION_50700))
         return true;
       bootstrap::DD_bootstrap_ctx::instance().set_upgraded_server_version(
           bootstrap::SERVER_VERSION_50700);
     } else {
-      if (setprop(thd, "MYSQLD_VERSION_UPGRADED", MYSQL_VERSION_ID))
+      if (dd::tables::DD_properties::instance().set(
+              thd, "MYSQLD_VERSION_UPGRADED", MYSQL_VERSION_ID))
         return true;
       bootstrap::DD_bootstrap_ctx::instance().set_upgraded_server_version(
           MYSQL_VERSION_ID);
@@ -1884,30 +1694,42 @@ bool update_versions(THD *thd, bool is_dd_upgrade_57) {
     uint mysqld_version_hi = 0;
     uint mysqld_version = 0;
     uint upgraded_server_version = 0;
-
-    if (getprop(thd, "MYSQLD_VERSION_LO", &mysqld_version_lo) ||
-        getprop(thd, "MYSQLD_VERSION_HI", &mysqld_version_hi) ||
-        getprop(thd, "MYSQLD_VERSION", &mysqld_version))
+    bool exists_lo = false;
+    bool exists_hi = false;
+    bool exists = false;
+    bool exists_upgraded_version = false;
+    if ((dd::tables::DD_properties::instance().get(
+             thd, "MYSQLD_VERSION_LO", &mysqld_version_lo, &exists_lo) ||
+         !exists_lo) ||
+        (dd::tables::DD_properties::instance().get(
+             thd, "MYSQLD_VERSION_HI", &mysqld_version_hi, &exists_hi) ||
+         !exists_hi) ||
+        (dd::tables::DD_properties::instance().get(thd, "MYSQLD_VERSION",
+                                                   &mysqld_version, &exists) ||
+         !exists))
       return dd::end_transaction(thd, true);
 
-    if (getprop(thd, "MYSQLD_VERSION_UPGRADED", &upgraded_server_version,
-                true)) {
-      if (setprop(thd, "MYSQLD_VERSION_UPGRADED", mysqld_version)) return true;
+    if (dd::tables::DD_properties::instance().get(
+            thd, "MYSQLD_VERSION_UPGRADED", &upgraded_server_version,
+            &exists_upgraded_version) ||
+        !exists_upgraded_version) {
+      if (dd::tables::DD_properties::instance().set(
+              thd, "MYSQLD_VERSION_UPGRADED", mysqld_version))
+        return true;
       upgraded_server_version = mysqld_version;
     }
     bootstrap::DD_bootstrap_ctx::instance().set_upgraded_server_version(
         upgraded_server_version);
 
     if ((mysqld_version_lo > MYSQL_VERSION_ID &&
-         setprop(thd, "MYSQLD_VERSION_LO", MYSQL_VERSION_ID)) ||
+         dd::tables::DD_properties::instance().set(thd, "MYSQLD_VERSION_LO",
+                                                   MYSQL_VERSION_ID)) ||
         (mysqld_version_hi < MYSQL_VERSION_ID &&
-         setprop(thd, "MYSQLD_VERSION_HI", MYSQL_VERSION_ID)) ||
+         dd::tables::DD_properties::instance().set(thd, "MYSQLD_VERSION_HI",
+                                                   MYSQL_VERSION_ID)) ||
         (mysqld_version != MYSQL_VERSION_ID &&
-         (setprop(thd, "MYSQLD_VERSION", MYSQL_VERSION_ID) ||
-          setprop(thd, "MYSQL_VERSION_STABILITY", MYSQL_VERSION_STABILITY) ||
-          setprop(thd, "SERVER_DOWNGRADE_THRESHOLD",
-                  SERVER_DOWNGRADE_THRESHOLD) ||
-          setprop(thd, "SERVER_UPGRADE_THRESHOLD", SERVER_UPGRADE_THRESHOLD))))
+         dd::tables::DD_properties::instance().set(thd, "MYSQLD_VERSION",
+                                                   MYSQL_VERSION_ID)))
       return dd::end_transaction(thd, true);
 
     /*
@@ -1915,9 +1737,13 @@ bool update_versions(THD *thd, bool is_dd_upgrade_57) {
       Note that on downgrade, we keep the old SDI version.
     */
     uint stored_sdi_version = 0;
-    if (getprop(thd, "SDI_VERSION", &stored_sdi_version) ||
+    bool exists_sdi = false;
+    if ((dd::tables::DD_properties::instance().get(
+             thd, "SDI_VERSION", &stored_sdi_version, &exists_sdi) ||
+         !exists_sdi) ||
         (stored_sdi_version < dd::SDI_VERSION &&
-         setprop(thd, "SDI_VERSION", dd::SDI_VERSION)))
+         dd::tables::DD_properties::instance().set(thd, "SDI_VERSION",
+                                                   dd::SDI_VERSION)))
       return dd::end_transaction(thd, true);
 
     /*
@@ -1925,9 +1751,13 @@ bool update_versions(THD *thd, bool is_dd_upgrade_57) {
       Note that on downgrade, we keep the old DD version.
     */
     uint dd_version = 0;
-    if (getprop(thd, "DD_VERSION", &dd_version) ||
+    bool exists_dd = false;
+    if ((dd::tables::DD_properties::instance().get(thd, "DD_VERSION",
+                                                   &dd_version, &exists_dd) ||
+         !exists_dd) ||
         (dd_version < dd::DD_VERSION &&
-         setprop(thd, "DD_VERSION", dd::DD_VERSION)))
+         dd::tables::DD_properties::instance().set(thd, "DD_VERSION",
+                                                   dd::DD_VERSION)))
       return dd::end_transaction(thd, true);
 
     /*
@@ -1936,8 +1766,9 @@ bool update_versions(THD *thd, bool is_dd_upgrade_57) {
       already present.
     */
     if (dd_version < dd::DD_VERSION &&
-        setprop(thd, "MINOR_DOWNGRADE_THRESHOLD",
-                dd::DD_VERSION_MINOR_DOWNGRADE_THRESHOLD))
+        dd::tables::DD_properties::instance().set(
+            thd, "MINOR_DOWNGRADE_THRESHOLD",
+            dd::DD_VERSION_MINOR_DOWNGRADE_THRESHOLD))
       return dd::end_transaction(thd, true);
   }
 
@@ -1949,8 +1780,7 @@ bool update_versions(THD *thd, bool is_dd_upgrade_57) {
     do before committing.
   */
   handlerton *ddse = ha_resolve_by_legacy_type(thd, DB_TYPE_INNODB);
-  if (bootstrap::DD_bootstrap_ctx::instance().is_server_upgrade() ||
-      bootstrap::DD_bootstrap_ctx::instance().is_server_patch_downgrade()) {
+  if (bootstrap::DD_bootstrap_ctx::instance().is_server_upgrade()) {
     if (ddse->dict_set_server_version == nullptr ||
         ddse->dict_set_server_version()) {
       LogErr(ERROR_LEVEL, ER_CANNOT_SET_SERVER_VERSION_IN_TABLESPACE_HEADER);
@@ -1958,7 +1788,7 @@ bool update_versions(THD *thd, bool is_dd_upgrade_57) {
     }
   }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   /*
     Debug code to make sure that after updating version numbers, regardless
     of the type of initialization, restart or upgrade, the server version
@@ -1966,9 +1796,9 @@ bool update_versions(THD *thd, bool is_dd_upgrade_57) {
     version number.
   */
   uint version = 0;
-  assert(ddse->dict_get_server_version != nullptr);
-  assert(!ddse->dict_get_server_version(&version));
-  assert(version == MYSQL_VERSION_ID);
+  DBUG_ASSERT(ddse->dict_get_server_version != nullptr);
+  DBUG_ASSERT(!ddse->dict_get_server_version(&version));
+  DBUG_ASSERT(version == MYSQL_VERSION_ID);
 #endif
 
   bootstrap::DD_bootstrap_ctx::instance().set_stage(
