@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,9 +22,9 @@
 
 #include "sql/dd/impl/cache/shared_dictionary_cache.h"
 
-#include <assert.h>
 #include <atomic>
 
+#include "my_dbug.h"
 #include "sql/dd/impl/cache/shared_multi_map.h"
 #include "sql/dd/impl/cache/storage_adapter.h"  // Storage_adapter
 #include "sql/mysqld.h"
@@ -91,10 +91,10 @@ template <typename K, typename T>
 bool Shared_dictionary_cache::get(THD *thd, const K &key,
                                   Cache_element<T> **element) {
   bool error = false;
-  assert(element);
+  DBUG_ASSERT(element);
   if (m_map<T>()->get(key, element)) {
     // Handle cache miss.
-    const T *new_object = nullptr;
+    const T *new_object = NULL;
     error = get_uncached(thd, key, ISO_READ_COMMITTED, &new_object);
 
     // Add the new object, and assign the output element, even in the case of
@@ -109,9 +109,10 @@ template <typename K, typename T>
 bool Shared_dictionary_cache::get_uncached(THD *thd, const K &key,
                                            enum_tx_isolation isolation,
                                            const T **object) const {
-  assert(object);
+  DBUG_ASSERT(object);
   bool error = Storage_adapter::get(thd, key, isolation, false, object);
-  assert(!error || thd->is_system_thread() || thd->killed || thd->is_error());
+  DBUG_ASSERT(!error || thd->is_system_thread() || thd->killed ||
+              thd->is_error());
 
   return error;
 }
@@ -119,9 +120,9 @@ bool Shared_dictionary_cache::get_uncached(THD *thd, const K &key,
 // Add an object to the shared cache.
 template <typename T>
 void Shared_dictionary_cache::put(const T *object, Cache_element<T> **element) {
-  assert(object);
+  DBUG_ASSERT(object);
   // Cast needed to help the compiler choose the correct template instance..
-  m_map<T>()->put(static_cast<const typename T::Id_key *>(nullptr), object,
+  m_map<T>()->put(static_cast<const typename T::Id_key *>(NULL), object,
                   element);
 }
 
