@@ -130,13 +130,14 @@ bool Dictionary_impl::init(enum_dd_init_type dd_init) {
 
   // Creation of Data Dictionary through current server
   if (dd_init == enum_dd_init_type::DD_INITIALIZE)
-    result = ::bootstrap::run_bootstrap_thread(NULL, &bootstrap::initialize,
-                                               SYSTEM_THREAD_DD_INITIALIZE);
+    result = ::bootstrap::run_bootstrap_thread(
+        nullptr, nullptr, &bootstrap::initialize, SYSTEM_THREAD_DD_INITIALIZE);
 
   // Creation of INFORMATION_SCHEMA system views.
   else if (dd_init == enum_dd_init_type::DD_INITIALIZE_SYSTEM_VIEWS)
-    result = ::bootstrap::run_bootstrap_thread(
-        NULL, &dd::info_schema::initialize, SYSTEM_THREAD_DD_INITIALIZE);
+    result = ::bootstrap::run_bootstrap_thread(nullptr, nullptr,
+                                               &dd::info_schema::initialize,
+                                               SYSTEM_THREAD_DD_INITIALIZE);
 
   /*
     Creation of Dictionary Tables in old Data Directory
@@ -144,23 +145,24 @@ bool Dictionary_impl::init(enum_dd_init_type dd_init) {
   */
   else if (dd_init == enum_dd_init_type::DD_RESTART_OR_UPGRADE)
     result = ::bootstrap::run_bootstrap_thread(
-        NULL, &upgrade_57::do_pre_checks_and_initialize_dd,
+        nullptr, nullptr, &upgrade_57::do_pre_checks_and_initialize_dd,
         SYSTEM_THREAD_DD_INITIALIZE);
 
   // Populate metadata in DD tables from old data directory and do cleanup.
   else if (dd_init == enum_dd_init_type::DD_POPULATE_UPGRADE)
     result = ::bootstrap::run_bootstrap_thread(
-        NULL, &upgrade_57::fill_dd_and_finalize, SYSTEM_THREAD_DD_INITIALIZE);
+        nullptr, nullptr, &upgrade_57::fill_dd_and_finalize,
+        SYSTEM_THREAD_DD_INITIALIZE);
 
   // Delete DD tables and do cleanup in case of error in upgrade
   else if (dd_init == enum_dd_init_type::DD_DELETE)
-    result = ::bootstrap::run_bootstrap_thread(NULL, &upgrade_57::terminate,
-                                               SYSTEM_THREAD_DD_INITIALIZE);
+    result = ::bootstrap::run_bootstrap_thread(
+        nullptr, nullptr, &upgrade_57::terminate, SYSTEM_THREAD_DD_INITIALIZE);
 
   // Update server and plugin I_S table metadata into DD tables.
   else if (dd_init == enum_dd_init_type::DD_UPDATE_I_S_METADATA)
     result = ::bootstrap::run_bootstrap_thread(
-        NULL, &dd::info_schema::update_I_S_metadata,
+        nullptr, nullptr, &dd::info_schema::update_I_S_metadata,
         SYSTEM_THREAD_DD_INITIALIZE);
 
   // Restore the table_encryption_privilege_check.
@@ -396,7 +398,7 @@ static bool acquire_mdl(THD *thd, MDL_key::enum_mdl_namespace lock_namespace,
                         enum_mdl_type lock_type,
                         enum_mdl_duration lock_duration,
                         MDL_ticket **out_mdl_ticket) {
-  DBUG_ENTER("dd::acquire_mdl");
+  DBUG_TRACE;
 
   MDL_request mdl_request;
   MDL_REQUEST_INIT(&mdl_request, lock_namespace, schema_name, table_name,
@@ -439,14 +441,14 @@ static bool acquire_mdl(THD *thd, MDL_key::enum_mdl_namespace lock_namespace,
          thd->mdl_context.try_acquire_lock(bl_request)) ||
         (bl_request != nullptr &&
          thd->mdl_context.try_acquire_lock(grl_request))) {
-      DBUG_RETURN(true);
+      return true;
     }
   } else if (thd->mdl_context.acquire_locks(&mdl_requests, lock_wait_timeout))
-    DBUG_RETURN(true);
+    return true;
 
   if (out_mdl_ticket) *out_mdl_ticket = mdl_request.ticket;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 bool acquire_shared_table_mdl(THD *thd, const char *schema_name,
@@ -525,11 +527,9 @@ bool acquire_exclusive_schema_mdl(THD *thd, const char *schema_name,
 }
 
 void release_mdl(THD *thd, MDL_ticket *mdl_ticket) {
-  DBUG_ENTER("dd::release_mdl");
+  DBUG_TRACE;
 
   thd->mdl_context.release_lock(mdl_ticket);
-
-  DBUG_VOID_RETURN;
 }
 
 /* purecov: begin deadcode */
@@ -540,8 +540,8 @@ bool create_native_table(THD *thd, const Plugin_table *pt) {
   if (dd::get_dictionary()->is_dd_table_name(pt->get_schema_name(),
                                              pt->get_name())) {
     my_error(ER_NO_SYSTEM_TABLE_ACCESS, MYF(0),
-             ER_THD(thd, dd::get_dictionary()->table_type_error_code(
-                             pt->get_schema_name(), pt->get_name())),
+             ER_THD_NONCONST(thd, dd::get_dictionary()->table_type_error_code(
+                                      pt->get_schema_name(), pt->get_name())),
              pt->get_schema_name(), pt->get_name());
 
     return true;
@@ -561,7 +561,8 @@ bool create_native_table(THD *thd, const Plugin_table *pt) {
   /*
     1. Mark that we are executing a special DDL during
     plugin initialization. This will enable DDL to not be
-    committed. The called of this API would commit the transaction.
+    committed or binlogged. The called of this API would commit
+    the transaction.
 
     2. Remove metadata of native table if already exists. This could
     happen if server was crashed and restarted.
@@ -578,16 +579,19 @@ bool create_native_table(THD *thd, const Plugin_table *pt) {
   thd->mark_plugin_fake_ddl(true);
   ulong master_access = thd->security_context()->master_access();
   thd->security_context()->set_master_access(~(ulong)0);
+  {
+    Disable_binlog_guard guard(thd);
 
-  // Drop the table and related dynamic statistics too.
-  if (table_def) {
-    error =
-        client->drop(table_def) || client->remove_table_dynamic_statistics(
-                                       pt->get_schema_name(), pt->get_name());
+    // Drop the table and related dynamic statistics too.
+    if (table_def) {
+      error =
+          client->drop(table_def) || client->remove_table_dynamic_statistics(
+                                         pt->get_schema_name(), pt->get_name());
+    }
+
+    if (!error) error = dd::execute_query(thd, pt->get_ddl());
   }
 
-  if (!error) error = dd::execute_query(thd, pt->get_ddl());
-
   thd->security_context()->set_master_access(master_access);
   thd->mark_plugin_fake_ddl(false);
 
@@ -599,8 +603,8 @@ bool drop_native_table(THD *thd, const char *schema_name,
                        const char *table_name) {
   if (dd::get_dictionary()->is_dd_table_name(schema_name, table_name)) {
     my_error(ER_NO_SYSTEM_TABLE_ACCESS, MYF(0),
-             ER_THD(thd, dd::get_dictionary()->table_type_error_code(
-                             schema_name, table_name)),
+             ER_THD_NONCONST(thd, dd::get_dictionary()->table_type_error_code(
+                                      schema_name, table_name)),
              schema_name, table_name);
 
     return true;
