@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -65,15 +64,15 @@ bool Raw_record::update() {
 
   /**
     We ignore HA_ERR_RECORD_IS_THE_SAME here for following reason.
-    If in case we are updating children of some DD object,
+    If in case we are updating childrens of some DD object,
     and only one of the children has really changed and other have
     not. Then we get HA_ERR_RECORD_IS_THE_SAME for children (rows)
     which has not really been modified.
 
-    Currently DD framework creates/updates *all* children at once
+    Currently DD framework creates/updates *all* childrens at once
     and we don't have machinism to update only required child.
-    Maybe this is part of a task which will implement inplace
-    alter in a better way, updating only the changed child (or row)
+    May be this is part of task which will implement inplace
+    alter in better way, updating only the changed child (or row)
     and ignore others. Then we can remove the below check which
     ignores HA_ERR_RECORD_IS_THE_SAME.
   */
@@ -133,7 +132,7 @@ bool Raw_record::store_ref_id(int field_no, Object_id id) {
   set_null(field_no, false);
   type_conversion_status rc = field(field_no)->store(id, true);
 
-  assert(rc == TYPE_OK);
+  DBUG_ASSERT(rc == TYPE_OK);
   return rc != TYPE_OK;
 }
 
@@ -156,7 +155,7 @@ bool Raw_record::store(int field_no, const String_type &s, bool is_null) {
   type_conversion_status rc =
       field(field_no)->store(s.c_str(), s.length(), system_charset_info);
 
-  assert(rc == TYPE_OK);
+  DBUG_ASSERT(rc == TYPE_OK);
   return rc != TYPE_OK;
 }
 
@@ -169,7 +168,7 @@ bool Raw_record::store(int field_no, ulonglong ull, bool is_null) {
 
   type_conversion_status rc = field(field_no)->store(ull, true);
 
-  assert(rc == TYPE_OK);
+  DBUG_ASSERT(rc == TYPE_OK);
   return rc != TYPE_OK;
 }
 
@@ -182,7 +181,7 @@ bool Raw_record::store(int field_no, longlong ll, bool is_null) {
 
   type_conversion_status rc = field(field_no)->store(ll, false);
 
-  assert(rc == TYPE_OK);
+  DBUG_ASSERT(rc == TYPE_OK);
   return rc != TYPE_OK;
 }
 
@@ -207,7 +206,7 @@ bool Raw_record::store_time(int field_no, my_time_t val, bool is_null) {
 
 ///////////////////////////////////////////////////////////////////////////
 
-bool Raw_record::store_timestamp(int field_no, const my_timeval &tv) {
+bool Raw_record::store_timestamp(int field_no, const timeval &tv) {
   field(field_no)->store_timestamp(&tv);
   return false;
 }
@@ -267,11 +266,11 @@ my_time_t Raw_record::read_time(int field_no) const {
 
 ///////////////////////////////////////////////////////////////////////////
 
-my_timeval Raw_record::read_timestamp(int field_no) const {
+timeval Raw_record::read_timestamp(int field_no) const {
   int warnings = 0;
-  my_timeval tv;
+  timeval tv;
   if (field(field_no)->get_timestamp(&tv, &warnings)) {
-    assert(false);
+    DBUG_ASSERT(false);
     return {0, 0};
   }
   return tv;
@@ -335,9 +334,9 @@ void Raw_new_record::finalize() {
 
   m_table->autoinc_field_has_explicit_non_null_value = false;
   m_table->file->ha_release_auto_increment();
-  m_table->next_number_field = nullptr;
+  m_table->next_number_field = NULL;
 
-  m_table = nullptr;
+  m_table = NULL;
 }
 
 ///////////////////////////////////////////////////////////////////////////
