@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -55,7 +55,7 @@ class St_spatial_reference_systems
     return s_view_name;
   }
 
-  const String_type &name() const override {
+  virtual const String_type &name() const {
     return St_spatial_reference_systems::view_name();
   }
 };
