@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -39,7 +39,7 @@ const Indexes &Indexes::instance() {
 ///////////////////////////////////////////////////////////////////////////
 
 const CHARSET_INFO *Indexes::name_collation() {
-  return &my_charset_utf8mb3_tolower_ci;
+  return &my_charset_utf8_tolower_ci;
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -53,7 +53,7 @@ Indexes::Indexes() {
                          "table_id BIGINT UNSIGNED NOT NULL");
   m_target_def.add_field(FIELD_NAME, "FIELD_NAME",
                          "name VARCHAR(64) NOT NULL COLLATE " +
-                             String_type(name_collation()->m_coll_name));
+                             String_type(name_collation()->name));
   m_target_def.add_field(FIELD_TYPE, "FIELD_TYPE",
                          "type ENUM(\n"
                          "  'PRIMARY',\n"
@@ -87,14 +87,8 @@ Indexes::Indexes() {
                          "se_private_data MEDIUMTEXT");
   m_target_def.add_field(FIELD_TABLESPACE_ID, "FIELD_TABLESPACE_ID",
                          "tablespace_id BIGINT UNSIGNED");
-  m_target_def.add_field(
-      FIELD_ENGINE, "FIELD_ENGINE",
-      "engine VARCHAR(64) NOT NULL COLLATE utf8mb3_general_ci");
-  m_target_def.add_field(FIELD_ENGINE_ATTRIBUTE, "FIELD_ENGINE_ATTRIBUTE",
-                         "engine_attribute JSON");
-  m_target_def.add_field(FIELD_SECONDARY_ENGINE_ATTRIBUTE,
-                         "FIELD_SECONDARY_ENGINE_ATTRIBUTE",
-                         "secondary_engine_attribute JSON");
+  m_target_def.add_field(FIELD_ENGINE, "FIELD_ENGINE",
+                         "engine VARCHAR(64) NOT NULL COLLATE utf8_general_ci");
 
   m_target_def.add_index(INDEX_PK_ID, "INDEX_PK_ID", "PRIMARY KEY(id)");
   m_target_def.add_index(INDEX_UK_TABLE_ID_NAME, "INDEX_UK_TABLE_ID_NAME",
