@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,7 +43,7 @@ const Routines &Routines::instance() {
 ///////////////////////////////////////////////////////////////////////////
 
 const CHARSET_INFO *Routines::name_collation() {
-  return &my_charset_utf8mb3_general_ci;
+  return &my_charset_utf8_general_ci;
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -58,7 +57,7 @@ Routines::Routines() {
                          "schema_id BIGINT UNSIGNED NOT NULL");
   m_target_def.add_field(FIELD_NAME, "FIELD_NAME",
                          "name VARCHAR(64) NOT NULL COLLATE " +
-                             String_type(name_collation()->m_coll_name));
+                             String_type(name_collation()->name));
   m_target_def.add_field(FIELD_TYPE, "FIELD_TYPE",
                          "type ENUM('FUNCTION', 'PROCEDURE') NOT NULL");
   m_target_def.add_field(FIELD_RESULT_DATA_TYPE, "FIELD_RESULT_DATA_TYPE",
@@ -153,7 +152,6 @@ Routines::Routines() {
   m_target_def.add_index(INDEX_K_SCHEMA_COLLATION_ID,
                          "INDEX_K_SCHEMA_COLLATION_ID",
                          "KEY(schema_collation_id)");
-  m_target_def.add_index(INDEX_K_DEFINER, "INDEX_K_DEFINER", "KEY(definer)");
 
   m_target_def.add_foreign_key(FK_SCHEMA_ID, "FK_SCHEMA_ID",
                                "FOREIGN KEY (schema_id) "
@@ -205,12 +203,5 @@ Object_key *Routines::create_key_by_schema_id(Object_id schema_id) {
 
 ///////////////////////////////////////////////////////////////////////////
 
-Object_key *Routines::create_key_by_definer(const String_type &definer) {
-  return new (std::nothrow)
-      Definer_reference_range_key(INDEX_K_DEFINER, FIELD_DEFINER, definer);
-}
-
-///////////////////////////////////////////////////////////////////////////
-
 }  // namespace tables
 }  // namespace dd
