@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -55,12 +55,12 @@ void Open_dictionary_tables_ctx::add_table(const String_type &name) {
 bool Open_dictionary_tables_ctx::open_tables() {
   DBUG_TRACE;
 
-  assert(!m_tables.empty());
+  DBUG_ASSERT(!m_tables.empty());
 
   Object_table_map::iterator it = m_tables.begin();
   Object_table_map::iterator it_next = m_tables.begin();
 
-  Table_ref *table_list = it_next->second->get_table_ref();
+  TABLE_LIST *table_list = it_next->second->get_table_list();
 
   ++it_next;
 
@@ -68,7 +68,8 @@ bool Open_dictionary_tables_ctx::open_tables() {
   while (it_next != m_tables.end()) {
     // fprintf(stderr, "  - '%s'\n", it->first.c_str());
 
-    it->second->get_table_ref()->next_global = it_next->second->get_table_ref();
+    it->second->get_table_list()->next_global =
+        it_next->second->get_table_list();
 
     ++it;
     ++it_next;
@@ -114,13 +115,14 @@ bool Open_dictionary_tables_ctx::open_tables() {
     is not really necessary for replicating data-dictionary changes
     (as we do not aim to replicate exact IDs in the data-dictionary).
   */
-  for (Table_ref *t = table_list; t; t = t->next_global) {
-    assert(t->table->file->ha_table_flags() & HA_ATTACHABLE_TRX_COMPATIBLE);
+  for (TABLE_LIST *t = table_list; t; t = t->next_global) {
+    DBUG_ASSERT(t->table->file->ha_table_flags() &
+                HA_ATTACHABLE_TRX_COMPATIBLE);
     if (t->table->file->ha_extra(HA_EXTRA_NO_AUTOINC_LOCKING)) return true;
   }
 
   // Lock the tables.
-  if (lock_dictionary_tables(m_thd, table_list, counter, flags)) return true;
+  if (lock_tables(m_thd, table_list, counter, flags)) return true;
 
   return false;
 }
@@ -138,7 +140,7 @@ Update_dictionary_tables_ctx::Update_dictionary_tables_ctx(THD *thd)
     : otx(thd, TL_WRITE),
       m_thd(thd),
       m_kill_immunizer(thd),
-      m_query_tables_list_backup(new Query_tables_list()),
+      m_lex_saved(NULL),
       m_saved_in_sub_stmt(thd->in_sub_stmt),
       m_saved_time_zone_used(thd->time_zone_used),
       m_saved_auto_increment_increment(
@@ -148,11 +150,10 @@ Update_dictionary_tables_ctx::Update_dictionary_tables_ctx(THD *thd)
   m_saved_mode = m_thd->variables.sql_mode;
   m_thd->variables.sql_mode = 0;  // Reset during DD operations
 
-  /*
-    Backup and reset part of LEX which will be accessed while opening
-    and closing data-dictionary tables.
-  */
-  m_thd->lex->reset_n_backup_query_tables_list(m_query_tables_list_backup);
+  // Save old lex
+  m_lex_saved = m_thd->lex;
+  m_thd->lex = new (m_thd->mem_root) st_lex_local;
+  lex_start(m_thd);
 
   m_thd->reset_n_backup_open_tables_state(&m_open_tables_state_backup,
                                           Open_tables_state::SYSTEM_TABLES);
@@ -178,8 +179,8 @@ Update_dictionary_tables_ctx::Update_dictionary_tables_ctx(THD *thd)
     mode. This means that all DDL statements using Update_dictionary_tables_ctx
     to update data-dictionary need to turn off @@autocommit for its duration.
   */
-  assert((m_thd->variables.option_bits & OPTION_NOT_AUTOCOMMIT) &&
-         !(m_thd->variables.option_bits & OPTION_AUTOCOMMIT));
+  DBUG_ASSERT((m_thd->variables.option_bits & OPTION_NOT_AUTOCOMMIT) &&
+              !(m_thd->variables.option_bits & OPTION_AUTOCOMMIT));
 
   // Store current intervals.
   m_thd->auto_inc_intervals_in_cur_stmt_for_binlog.swap(
@@ -205,14 +206,17 @@ Update_dictionary_tables_ctx::~Update_dictionary_tables_ctx() {
   m_thd->variables.option_bits = m_saved_options;
 
   if (m_saved_binlog_row_based) m_thd->set_current_stmt_binlog_format_row();
-  m_saved_binlog_row_based = false;
+  m_saved_binlog_row_based = 0;
 
   m_thd->restore_backup_open_tables_state(&m_open_tables_state_backup);
 
-  m_thd->lex->restore_backup_query_tables_list(m_query_tables_list_backup);
+  // Restore the lex
+  lex_end(m_thd->lex);
+  delete (st_lex_local *)m_thd->lex;
+  m_thd->lex = m_lex_saved;
 
   // Restore auto_inc_intervals_in_cur_stmt_for_binlog
-  m_auto_inc_intervals_in_cur_stmt_for_binlog_saved.clear();  // XXX: remove?
+  m_auto_inc_intervals_in_cur_stmt_for_binlog_saved.empty();  // XXX: remove?
   m_auto_inc_intervals_in_cur_stmt_for_binlog_saved.swap(
       &m_thd->auto_inc_intervals_in_cur_stmt_for_binlog);
 
@@ -224,8 +228,6 @@ Update_dictionary_tables_ctx::~Update_dictionary_tables_ctx() {
   m_thd->in_sub_stmt = m_saved_in_sub_stmt;
 
   m_thd->time_zone_used = m_saved_time_zone_used;
-
-  delete m_query_tables_list_backup;
 }
 
 }  // namespace dd
