@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -58,18 +58,18 @@ class Column_type_element_impl : public Weak_object_impl,
   Column_type_element_impl(const Column_type_element_impl &src,
                            Column_impl *parent);
 
-  ~Column_type_element_impl() override = default;
+  virtual ~Column_type_element_impl() {}
 
  public:
-  const Object_table &object_table() const override;
+  virtual const Object_table &object_table() const;
 
   static void register_tables(Open_dictionary_tables_ctx *otx);
 
-  bool validate() const override;
+  virtual bool validate() const;
 
-  bool store_attributes(Raw_record *r) override;
+  virtual bool store_attributes(Raw_record *r);
 
-  bool restore_attributes(const Raw_record &r) override;
+  virtual bool restore_attributes(const Raw_record &r);
 
   void set_ordinal_position(uint ordinal_position) {
     m_index = ordinal_position;
@@ -77,9 +77,9 @@ class Column_type_element_impl : public Weak_object_impl,
 
   virtual uint ordinal_position() const { return index(); }
 
-  void serialize(Sdi_wcontext *wctx, Sdi_writer *w) const override;
+  void serialize(Sdi_wcontext *wctx, Sdi_writer *w) const;
 
-  bool deserialize(Sdi_rcontext *rctx, const RJ_Value &val) override;
+  bool deserialize(Sdi_rcontext *rctx, const RJ_Value &val);
 
  public:
   static Column_type_element_impl *restore_item(Column_impl *column) {
@@ -96,28 +96,28 @@ class Column_type_element_impl : public Weak_object_impl,
   // Name.
   /////////////////////////////////////////////////////////////////////////
 
-  const String_type &name() const override { return m_name; }
+  virtual const String_type &name() const { return m_name; }
 
-  void set_name(const String_type &name) override { m_name = name; }
+  virtual void set_name(const String_type &name) { m_name = name; }
 
   /////////////////////////////////////////////////////////////////////////
   // Column
   /////////////////////////////////////////////////////////////////////////
 
-  const Column &column() const override;
+  virtual const Column &column() const;
 
   /////////////////////////////////////////////////////////////////////////
   // index.
   /////////////////////////////////////////////////////////////////////////
 
-  uint index() const override { return m_index; }
+  virtual uint index() const { return m_index; }
 
  public:
-  void debug_print(String_type &outb) const override;
+  virtual void debug_print(String_type &outb) const;
 
  protected:
-  Object_key *create_primary_key() const override;
-  bool has_new_primary_key() const override;
+  virtual Object_key *create_primary_key() const;
+  virtual bool has_new_primary_key() const;
 
  protected:
   // Fields
