@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,8 +24,6 @@
 
 #include <stdint.h>
 
-#include <optional>
-
 #include "my_rapidjson_size_t.h"  // IWYU pragma: keep
 
 #include <rapidjson/document.h>
@@ -114,9 +111,7 @@ bool Spatial_reference_system_impl::store_attributes(Raw_record *r) {
 }
 
 ///////////////////////////////////////////////////////////////////////////
-static_assert(Spatial_reference_systems::NUMBER_OF_FIELDS == 10,
-              "Spatial_reference_systems definition has changed, check if "
-              "serialize() and deserialize() need to be updated!");
+
 void Spatial_reference_system_impl::serialize(Sdi_wcontext *wctx,
                                               Sdi_writer *w) const {
   w->StartObject();
@@ -147,25 +142,25 @@ bool Spatial_reference_system_impl::deserialize(Sdi_rcontext *rctx,
   Entity_object_impl::deserialize(rctx, val);
   read(&m_last_altered, val, "last_altered");
   read(&m_created, val, "created");
-  bool is_null{false};
+  bool is_null;
   read(&is_null, val, "organization_null");
   if (!is_null) {
     String_type s;
     read(&s, val, "organization");
-    m_organization = std::optional<String_type>(s);
+    m_organization = Mysql::Nullable<String_type>(s);
   }
   read(&is_null, val, "organization_coordsys_id_null");
   if (!is_null) {
     gis::srid_t id = 0;
     read(&id, val, "organization_coordsys_id");
-    m_organization_coordsys_id = std::optional<gis::srid_t>(id);
+    m_organization_coordsys_id = Mysql::Nullable<gis::srid_t>(id);
   }
   read(&m_definition, val, "definition");
   read(&is_null, val, "description_null");
   if (!is_null) {
     String_type s;
     read(&s, val, "description");
-    m_description = std::optional<String_type>(s);
+    m_description = Mysql::Nullable<String_type>(s);
   }
 
   return parse_definition();
