@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,9 +37,7 @@
 #include "sql/dd/impl/types/entity_object_impl.h"
 #include "sql/dd/string_type.h"
 #include "sql/dd/types/object_table.h"  // Object_table
-#include "sql/debug_sync.h"             // DEBUG_SYNC
 #include "sql/log.h"
-#include "sql/sql_class.h"  // current_thd, THD
 
 namespace dd {
 
@@ -55,8 +52,7 @@ namespace dd {
   @return true - on failure and error is reported.
   @return false - on success.
 */
-template <bool use_pfs>
-bool Weak_object_impl_<use_pfs>::store(Open_dictionary_tables_ctx *otx) {
+bool Weak_object_impl::store(Open_dictionary_tables_ctx *otx) {
   DBUG_TRACE;
 
   DBUG_EXECUTE_IF("fail_while_storing_dd_object", {
@@ -70,7 +66,7 @@ bool Weak_object_impl_<use_pfs>::store(Open_dictionary_tables_ctx *otx) {
 
   Raw_table *t = otx->get_table(obj_table.name());
 
-  assert(t);
+  DBUG_ASSERT(t);
 
   // Insert or update record.
 
@@ -92,7 +88,7 @@ bool Weak_object_impl_<use_pfs>::store(Open_dictionary_tables_ctx *otx) {
     if (!obj_key.get()) {
       /* purecov: begin deadcode */
       LogErr(ERROR_LEVEL, ER_DD_CANT_GET_OBJECT_KEY);
-      assert(false);
+      DBUG_ASSERT(false);
       return true;
       /* purecov: end */
     }
@@ -125,8 +121,6 @@ bool Weak_object_impl_<use_pfs>::store(Open_dictionary_tables_ctx *otx) {
     return true;
   }
 
-  DEBUG_SYNC(current_thd, "before_insert_into_dd");
-
   if (r->insert()) return true;
 
   DBUG_EXECUTE_IF("weak_object_impl_store_fail_before_store_children", {
@@ -174,8 +168,7 @@ bool Weak_object_impl_<use_pfs>::store(Open_dictionary_tables_ctx *otx) {
   @return true - on failure and error is reported.
   @return false - on success.
 */
-template <bool use_pfs>
-bool Weak_object_impl_<use_pfs>::drop(Open_dictionary_tables_ctx *otx) const {
+bool Weak_object_impl::drop(Open_dictionary_tables_ctx *otx) const {
   DBUG_TRACE;
 
   DBUG_EXECUTE_IF("fail_while_dropping_dd_object", {
@@ -189,7 +182,7 @@ bool Weak_object_impl_<use_pfs>::drop(Open_dictionary_tables_ctx *otx) const {
 
   Raw_table *t = otx->get_table(obj_table.name());
 
-  assert(t);
+  DBUG_ASSERT(t);
 
   // Find object to be dropped
 
@@ -201,7 +194,7 @@ bool Weak_object_impl_<use_pfs>::drop(Open_dictionary_tables_ctx *otx) const {
   if (!r.get()) {
     /* purecov: begin deadcode */
     LogErr(ERROR_LEVEL, ER_DD_CANT_CREATE_OBJECT_KEY);
-    assert(false);
+    DBUG_ASSERT(false);
     return true;
     /* purecov: end */
   }
@@ -210,7 +203,7 @@ bool Weak_object_impl_<use_pfs>::drop(Open_dictionary_tables_ctx *otx) const {
     Drop collections and then drop the object
 
     We should drop collections first and then parent object
-    as we have referential constraints. Mostly the reverse
+    as we have referencial constraints. Mostly the reverse
     order of restore/store operation.
   */
 
@@ -221,11 +214,10 @@ bool Weak_object_impl_<use_pfs>::drop(Open_dictionary_tables_ctx *otx) const {
 
 ///////////////////////////////////////////////////////////////////////////
 
-template <bool use_pfs>
-bool Weak_object_impl_<use_pfs>::check_parent_consistency(
-    Entity_object_impl *parent, Object_id parent_id) const {
-  assert(parent);
-  assert(parent->id() == parent_id);
+bool Weak_object_impl::check_parent_consistency(Entity_object_impl *parent,
+                                                Object_id parent_id) const {
+  DBUG_ASSERT(parent);
+  DBUG_ASSERT(parent->id() == parent_id);
 
   if (!parent) {
     my_error(ER_INVALID_DD_OBJECT, MYF(0), this->object_table().name().c_str(),
@@ -246,14 +238,3 @@ bool Weak_object_impl_<use_pfs>::check_parent_consistency(
 ///////////////////////////////////////////////////////////////////////////
 
 }  // namespace dd
-
-template bool dd::Weak_object_impl_<true>::store(Open_dictionary_tables_ctx *);
-template bool dd::Weak_object_impl_<false>::store(Open_dictionary_tables_ctx *);
-template bool dd::Weak_object_impl_<true>::drop(
-    Open_dictionary_tables_ctx *) const;
-template bool dd::Weak_object_impl_<false>::drop(
-    Open_dictionary_tables_ctx *) const;
-template bool dd::Weak_object_impl_<true>::check_parent_consistency(
-    Entity_object_impl *, Object_id) const;
-template bool dd::Weak_object_impl_<false>::check_parent_consistency(
-    Entity_object_impl *, Object_id) const;
