@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,8 +30,9 @@
 #include "sql/dd/info_schema/show_query_builder.h"  // Select_lex_builder
 #include "sql/dd/info_schema/table_stats.h"
 #include "sql/dd/string_type.h"
-#include "sql/item_cmpfunc.h"    // Item_func_case
-#include "sql/parse_location.h"  // POS
+#include "sql/item_cmpfunc.h"  // Item_func_case
+#include "sql/key.h"
+#include "sql/parse_tree_node_base.h"
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
 #include "sql/table.h"
@@ -43,9 +43,9 @@ namespace info_schema {
 
 //  Build a substitute query for SHOW CHARSETS.
 
-Query_block *build_show_character_set_query(const POS &pos, THD *thd,
-                                            const String *wild,
-                                            Item *where_cond) {
+SELECT_LEX *build_show_character_set_query(const POS &pos, THD *thd,
+                                           const String *wild,
+                                           Item *where_cond) {
   static const LEX_CSTRING system_view_name = {
       STRING_WITH_LEN("CHARACTER_SETS")};
 
@@ -148,7 +148,7 @@ Query_block *build_show_character_set_query(const POS &pos, THD *thd,
   // ... ORDER BY 'Charset' ...
   if (top_query.add_order_by(alias_charset)) return nullptr;
 
-  Query_block *sl = top_query.prepare_query_block();
+  SELECT_LEX *sl = top_query.prepare_select_lex();
 
   // sql_command is set to SQL_QUERY after above call, so.
   thd->lex->sql_command = SQLCOM_SHOW_CHARSETS;
@@ -158,8 +158,8 @@ Query_block *build_show_character_set_query(const POS &pos, THD *thd,
 
 // Build a substitute query for SHOW COLLATION.
 
-Query_block *build_show_collation_query(const POS &pos, THD *thd,
-                                        const String *wild, Item *where_cond) {
+SELECT_LEX *build_show_collation_query(const POS &pos, THD *thd,
+                                       const String *wild, Item *where_cond) {
   static const LEX_CSTRING system_view_name = {STRING_WITH_LEN("COLLATIONS")};
 
   // Define field name literal used in query to be built.
@@ -218,31 +218,35 @@ Query_block *build_show_collation_query(const POS &pos, THD *thd,
         new (mem_root) Item_field(pos, NullS, NullS, field_id.str);
     if (case_value_item == nullptr) return nullptr;  // OOM
 
-    mem_root_deque<Item *> case_when_list(mem_root);
+    List<Item> case_when_list;
 
     // ... WHEN <ID of utf8mb4_general_ci> ...
     Item *old_default_collation =
         new (mem_root) Item_uint(my_charset_utf8mb4_general_ci.number);
-    if (old_default_collation == nullptr) return nullptr;  // OOM
-    case_when_list.push_back(old_default_collation);
+    if (old_default_collation == nullptr ||
+        case_when_list.push_back(old_default_collation))
+      return nullptr;  // OOM
 
     // ... THEN 'Yes' ...
     Item *force_old_default_collation =
         new (mem_root) Item_string(STRING_WITH_LEN("Yes"), system_charset_info);
-    if (force_old_default_collation == nullptr) return nullptr;  // OOM
-    case_when_list.push_back(force_old_default_collation);
+    if (force_old_default_collation == nullptr ||
+        case_when_list.push_back(force_old_default_collation))
+      return nullptr;  // OOM
 
     // ... WHEN <ID of utf8mb4_0900_ai_ci> ...
     Item *new_default_collation =
         new (mem_root) Item_uint(my_charset_utf8mb4_0900_ai_ci.number);
-    if (new_default_collation == nullptr) return nullptr;  // OOM
-    case_when_list.push_back(new_default_collation);
+    if (new_default_collation == nullptr ||
+        case_when_list.push_back(new_default_collation))
+      return nullptr;  // OOM
 
     // ... THEN '' ...
     Item *suppress_new_default_collation =
         new (mem_root) Item_string(STRING_WITH_LEN(""), system_charset_info);
-    if (suppress_new_default_collation == nullptr) return nullptr;  // OOM
-    case_when_list.push_back(suppress_new_default_collation);
+    if (suppress_new_default_collation == nullptr ||
+        case_when_list.push_back(suppress_new_default_collation))
+      return nullptr;  // OOM
 
     // ... ELSE IS_DEFAULT ...
     Item *case_else_item =
@@ -254,7 +258,7 @@ Query_block *build_show_collation_query(const POS &pos, THD *thd,
     //   WHEN <ID of utf8mb4_0900_ai_ci> THEN FALSE
     //   ELSE `IS_DEFAULT`
     Item *case_func = new (mem_root)
-        Item_func_case(pos, &case_when_list, case_value_item, case_else_item);
+        Item_func_case(pos, case_when_list, case_value_item, case_else_item);
     if (case_func == nullptr ||
         sub_query.add_select_expr(case_func, alias_default))
       return nullptr;
@@ -291,7 +295,7 @@ Query_block *build_show_collation_query(const POS &pos, THD *thd,
   // ... ORDER BY 'Collation' ...
   if (top_query.add_order_by(alias_collation)) return nullptr;
 
-  Query_block *sl = top_query.prepare_query_block();
+  SELECT_LEX *sl = top_query.prepare_select_lex();
 
   // sql_command is set to SQL_QUERY after above call, so.
   thd->lex->sql_command = SQLCOM_SHOW_COLLATIONS;
@@ -301,8 +305,8 @@ Query_block *build_show_collation_query(const POS &pos, THD *thd,
 
 // Build a substitute query for SHOW DATABASES.
 
-Query_block *build_show_databases_query(const POS &pos, THD *thd, String *wild,
-                                        Item *where_cond) {
+SELECT_LEX *build_show_databases_query(const POS &pos, THD *thd, String *wild,
+                                       Item *where_cond) {
   static const LEX_CSTRING system_view_name = {STRING_WITH_LEN("SCHEMATA")};
 
   // Define field name literal used in query to be built.
@@ -360,7 +364,7 @@ Query_block *build_show_databases_query(const POS &pos, THD *thd, String *wild,
   // ... ORDER BY 'Database' ...
   if (top_query.add_order_by(alias_database)) return nullptr;
 
-  Query_block *sl = top_query.prepare_query_block();
+  SELECT_LEX *sl = top_query.prepare_select_lex();
 
   // sql_command is set to SQL_QUERY after above call, so.
   thd->lex->sql_command = SQLCOM_SHOW_DATABASES;
@@ -375,8 +379,8 @@ Query_block *build_show_databases_query(const POS &pos, THD *thd, String *wild,
   @param alias_as_alias - Add select items of the form,
                           'alias AS alias'. This is required
                           for top level query, as the real field
-                          names would be valid only the
-                          subquery that we are building for SHOW
+                          names would be valied only the sub
+                          query that we are building for SHOW
                           TABLE STATUS. Check comments in
                           sql/dd/show.h for more details.
 
@@ -494,9 +498,9 @@ static bool add_table_status_fields(Select_lex_builder *query,
 
 // Build a substitute query for SHOW TABLES / TABLE STATUS.
 
-Query_block *build_show_tables_query(const POS &pos, THD *thd, String *wild,
-                                     Item *where_cond,
-                                     bool include_status_fields) {
+SELECT_LEX *build_show_tables_query(const POS &pos, THD *thd, String *wild,
+                                    Item *where_cond,
+                                    bool include_status_fields) {
   static const LEX_CSTRING system_view_name = {STRING_WITH_LEN("TABLES")};
 
   // Define field name literal used in query to be built.
@@ -511,16 +515,16 @@ Query_block *build_show_tables_query(const POS &pos, THD *thd, String *wild,
 
   // Get the current logged in schema name
   size_t dummy;
-  if (thd->lex->query_block->db == nullptr &&
-      thd->lex->copy_db_to(&thd->lex->query_block->db, &dummy))
+  if (thd->lex->select_lex->db == nullptr &&
+      thd->lex->copy_db_to(&thd->lex->select_lex->db, &dummy))
     return nullptr;
 
   // Convert IS db and table name to desired form.
-  dd::info_schema::convert_table_name_case(thd->lex->query_block->db,
+  dd::info_schema::convert_table_name_case(thd->lex->select_lex->db,
                                            wild ? wild->ptr() : nullptr);
 
-  LEX_STRING cur_db = {thd->lex->query_block->db,
-                       strlen(thd->lex->query_block->db)};
+  LEX_STRING cur_db = {thd->lex->select_lex->db,
+                       strlen(thd->lex->select_lex->db)};
 
   if (check_and_convert_db_name(&cur_db, false) != Ident_name_check::OK)
     return nullptr;
@@ -599,7 +603,7 @@ Query_block *build_show_tables_query(const POS &pos, THD *thd, String *wild,
   // ... ORDER BY 'Table' ...
   if (top_query.add_order_by(alias_lex_string)) return nullptr;
 
-  Query_block *sl = top_query.prepare_query_block();
+  SELECT_LEX *sl = top_query.prepare_select_lex();
 
   // sql_command is set to SQL_QUERY after above call, so.
   if (include_status_fields)
@@ -612,9 +616,9 @@ Query_block *build_show_tables_query(const POS &pos, THD *thd, String *wild,
 
 // Build a substitute query for SHOW COLUMNS/FIELDS OR DESCRIBE.
 
-Query_block *build_show_columns_query(const POS &pos, THD *thd,
-                                      Table_ident *table_ident,
-                                      const String *wild, Item *where_cond) {
+SELECT_LEX *build_show_columns_query(const POS &pos, THD *thd,
+                                     Table_ident *table_ident,
+                                     const String *wild, Item *where_cond) {
   static const LEX_CSTRING system_view_name = {STRING_WITH_LEN("COLUMNS")};
 
   // Define field name literal used in query to be built.
@@ -743,8 +747,8 @@ Query_block *build_show_columns_query(const POS &pos, THD *thd,
   // ... ORDER BY 'Ordinal_position' ...
   if (top_query.add_order_by(alias_ordinal_position)) return nullptr;
 
-  // Prepare the Query_block
-  Query_block *sl = top_query.prepare_query_block();
+  // Prepare the SELECT_LEX
+  SELECT_LEX *sl = top_query.prepare_select_lex();
   if (sl == nullptr) return nullptr;
 
   // sql_command is set to SQL_QUERY after above call, so.
@@ -755,8 +759,8 @@ Query_block *build_show_columns_query(const POS &pos, THD *thd,
 
 // Build a substitute query for SHOW INDEX|KEYS|INDEXES
 
-Query_block *build_show_keys_query(const POS &pos, THD *thd,
-                                   Table_ident *table_ident, Item *where_cond) {
+SELECT_LEX *build_show_keys_query(const POS &pos, THD *thd,
+                                  Table_ident *table_ident, Item *where_cond) {
   LEX_CSTRING system_view_name = {STRING_WITH_LEN("SHOW_STATISTICS")};
 
   // Define field name literal used in query to be built.
@@ -907,8 +911,8 @@ Query_block *build_show_keys_query(const POS &pos, THD *thd,
       top_query.add_order_by(alias_column_pos))
     return nullptr;
 
-  // Prepare the Query_block
-  Query_block *sl = top_query.prepare_query_block();
+  // Prepare the SELECT_LEX
+  SELECT_LEX *sl = top_query.prepare_select_lex();
   if (sl == nullptr) return nullptr;
 
   // sql_command is set to SQL_QUERY after above call, so.
@@ -919,8 +923,8 @@ Query_block *build_show_keys_query(const POS &pos, THD *thd,
 
 // Build a substitute query for SHOW TRIGGERS
 
-Query_block *build_show_triggers_query(const POS &pos, THD *thd, String *wild,
-                                       Item *where_cond) {
+SELECT_LEX *build_show_triggers_query(const POS &pos, THD *thd, String *wild,
+                                      Item *where_cond) {
   static const LEX_CSTRING system_view_name = {STRING_WITH_LEN("TRIGGERS")};
 
   // Define field name literal used in query to be built.
@@ -977,16 +981,16 @@ Query_block *build_show_triggers_query(const POS &pos, THD *thd, String *wild,
 
   // Get the current logged in schema name
   size_t dummy;
-  if (thd->lex->query_block->db == nullptr &&
-      thd->lex->copy_db_to(&thd->lex->query_block->db, &dummy))
+  if (thd->lex->select_lex->db == nullptr &&
+      thd->lex->copy_db_to(&thd->lex->select_lex->db, &dummy))
     return nullptr;
 
   // Convert IS db and table name to desired form.
-  dd::info_schema::convert_table_name_case(thd->lex->query_block->db,
+  dd::info_schema::convert_table_name_case(thd->lex->select_lex->db,
                                            wild ? wild->ptr() : nullptr);
 
-  LEX_STRING cur_db = {thd->lex->query_block->db,
-                       strlen(thd->lex->query_block->db)};
+  LEX_STRING cur_db = {thd->lex->select_lex->db,
+                       strlen(thd->lex->select_lex->db)};
 
   if (check_and_convert_db_name(&cur_db, false) != Ident_name_check::OK)
     return nullptr;
@@ -1058,7 +1062,7 @@ Query_block *build_show_triggers_query(const POS &pos, THD *thd, String *wild,
       top_query.add_order_by(alias_action_order))
     return nullptr;
 
-  Query_block *sl = top_query.prepare_query_block();
+  SELECT_LEX *sl = top_query.prepare_select_lex();
 
   // sql_command is set to SQL_QUERY after above call, so.
   thd->lex->sql_command = SQLCOM_SHOW_TRIGGERS;
@@ -1068,8 +1072,8 @@ Query_block *build_show_triggers_query(const POS &pos, THD *thd, String *wild,
 
 // Build a substitute query for SHOW PROCEDURES / FUNCTIONS
 
-Query_block *build_show_procedures_query(const POS &pos, THD *thd, String *wild,
-                                         Item *where_cond) {
+SELECT_LEX *build_show_procedures_query(const POS &pos, THD *thd, String *wild,
+                                        Item *where_cond) {
   enum_sql_command current_cmd = thd->lex->sql_command;
 
   static const LEX_CSTRING system_view_name = {STRING_WITH_LEN("ROUTINES")};
@@ -1172,7 +1176,7 @@ Query_block *build_show_procedures_query(const POS &pos, THD *thd, String *wild,
   if (top_query.add_order_by(alias_db) || top_query.add_order_by(alias_name))
     return nullptr;
 
-  Query_block *sl = top_query.prepare_query_block();
+  SELECT_LEX *sl = top_query.prepare_select_lex();
 
   // sql_command is set to SQL_QUERY after above call, so.
   thd->lex->sql_command = current_cmd;
@@ -1182,8 +1186,8 @@ Query_block *build_show_procedures_query(const POS &pos, THD *thd, String *wild,
 
 // Build a substitute query for SHOW EVENTS
 
-Query_block *build_show_events_query(const POS &pos, THD *thd, String *wild,
-                                     Item *where_cond) {
+SELECT_LEX *build_show_events_query(const POS &pos, THD *thd, String *wild,
+                                    Item *where_cond) {
   static const LEX_CSTRING system_view_name = {STRING_WITH_LEN("EVENTS")};
 
   // Define field name literal used in query to be built.
@@ -1244,16 +1248,16 @@ Query_block *build_show_events_query(const POS &pos, THD *thd, String *wild,
 
   // Get the current logged in schema name
   size_t dummy;
-  if (thd->lex->query_block->db == nullptr &&
-      thd->lex->copy_db_to(&thd->lex->query_block->db, &dummy))
+  if (thd->lex->select_lex->db == nullptr &&
+      thd->lex->copy_db_to(&thd->lex->select_lex->db, &dummy))
     return nullptr;
 
   // Convert IS db and table name to desired form.
-  dd::info_schema::convert_table_name_case(thd->lex->query_block->db,
+  dd::info_schema::convert_table_name_case(thd->lex->select_lex->db,
                                            wild ? wild->ptr() : nullptr);
 
-  LEX_STRING cur_db = {thd->lex->query_block->db,
-                       strlen(thd->lex->query_block->db)};
+  LEX_STRING cur_db = {thd->lex->select_lex->db,
+                       strlen(thd->lex->select_lex->db)};
 
   if (check_and_convert_db_name(&cur_db, false) != Ident_name_check::OK)
     return nullptr;
@@ -1314,7 +1318,7 @@ Query_block *build_show_events_query(const POS &pos, THD *thd, String *wild,
   if (top_query.add_order_by(alias_db) || top_query.add_order_by(alias_name))
     return nullptr;
 
-  Query_block *sl = top_query.prepare_query_block();
+  SELECT_LEX *sl = top_query.prepare_select_lex();
 
   // sql_command is set to SQL_QUERY after above call, so.
   thd->lex->sql_command = SQLCOM_SHOW_EVENTS;
