@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -46,7 +46,7 @@ class Table;
 
 class Object_table_definition {
  public:
-  virtual ~Object_table_definition() = default;
+  virtual ~Object_table_definition() {}
 
   /**
     Set the name of the table.
