@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -35,8 +35,8 @@
 #include "my_sys.h"
 #include "my_time.h"
 #include "my_user.h"  // parse_user
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
@@ -87,16 +87,14 @@ const TABLE_FIELD_TYPE event_table_fields[ET_FIELD_COUNT] = {
     {{STRING_WITH_LEN("name")},
      {STRING_WITH_LEN("char(64)")},
      {STRING_WITH_LEN("utf8")}},
-    {{STRING_WITH_LEN("body")}, {STRING_WITH_LEN("longblob")}, {nullptr, 0}},
+    {{STRING_WITH_LEN("body")}, {STRING_WITH_LEN("longblob")}, {NULL, 0}},
     {{STRING_WITH_LEN("definer")},
      {STRING_WITH_LEN("char(93)")},
      {STRING_WITH_LEN("utf8")}},
-    {{STRING_WITH_LEN("execute_at")},
-     {STRING_WITH_LEN("datetime")},
-     {nullptr, 0}},
+    {{STRING_WITH_LEN("execute_at")}, {STRING_WITH_LEN("datetime")}, {NULL, 0}},
     {{STRING_WITH_LEN("interval_value")},
-     {STRING_WITH_LEN("int")},
-     {nullptr, 0}},
+     {STRING_WITH_LEN("int(11)")},
+     {NULL, 0}},
     {{STRING_WITH_LEN("interval_field")},
      {STRING_WITH_LEN(
          "enum('YEAR','QUARTER','MONTH','DAY',"
@@ -106,24 +104,20 @@ const TABLE_FIELD_TYPE event_table_fields[ET_FIELD_COUNT] = {
          ","
          "'DAY_MICROSECOND','HOUR_MICROSECOND','MINUTE_MICROSECOND',"
          "'SECOND_MICROSECOND')")},
-     {nullptr, 0}},
-    {{STRING_WITH_LEN("created")},
-     {STRING_WITH_LEN("timestamp")},
-     {nullptr, 0}},
-    {{STRING_WITH_LEN("modified")},
-     {STRING_WITH_LEN("timestamp")},
-     {nullptr, 0}},
+     {NULL, 0}},
+    {{STRING_WITH_LEN("created")}, {STRING_WITH_LEN("timestamp")}, {NULL, 0}},
+    {{STRING_WITH_LEN("modified")}, {STRING_WITH_LEN("timestamp")}, {NULL, 0}},
     {{STRING_WITH_LEN("last_executed")},
      {STRING_WITH_LEN("datetime")},
-     {nullptr, 0}},
-    {{STRING_WITH_LEN("starts")}, {STRING_WITH_LEN("datetime")}, {nullptr, 0}},
-    {{STRING_WITH_LEN("ends")}, {STRING_WITH_LEN("datetime")}, {nullptr, 0}},
+     {NULL, 0}},
+    {{STRING_WITH_LEN("starts")}, {STRING_WITH_LEN("datetime")}, {NULL, 0}},
+    {{STRING_WITH_LEN("ends")}, {STRING_WITH_LEN("datetime")}, {NULL, 0}},
     {{STRING_WITH_LEN("status")},
      {STRING_WITH_LEN("enum('ENABLED','DISABLED','SLAVESIDE_DISABLED')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("on_completion")},
      {STRING_WITH_LEN("enum('DROP','PRESERVE')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("sql_mode")},
      {STRING_WITH_LEN(
          "set('REAL_AS_FLOAT','PIPES_AS_CONCAT','ANSI_QUOTES',"
@@ -138,11 +132,11 @@ const TABLE_FIELD_TYPE event_table_fields[ET_FIELD_COUNT] = {
          "'ERROR_FOR_DIVISION_BY_ZERO','TRADITIONAL','NO_AUTO_CREATE_USER',"
          "'HIGH_NOT_PRECEDENCE','NO_ENGINE_SUBSTITUTION','PAD_CHAR_TO_FULL_"
          "LENGTH')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("comment")},
      {STRING_WITH_LEN("char(64)")},
      {STRING_WITH_LEN("utf8")}},
-    {{STRING_WITH_LEN("originator")}, {STRING_WITH_LEN("int")}, {nullptr, 0}},
+    {{STRING_WITH_LEN("originator")}, {STRING_WITH_LEN("int(10)")}, {NULL, 0}},
     {{STRING_WITH_LEN("time_zone")},
      {STRING_WITH_LEN("char(64)")},
      {STRING_WITH_LEN("latin1")}},
@@ -155,9 +149,7 @@ const TABLE_FIELD_TYPE event_table_fields[ET_FIELD_COUNT] = {
     {{STRING_WITH_LEN("db_collation")},
      {STRING_WITH_LEN("char(32)")},
      {STRING_WITH_LEN("utf8")}},
-    {{STRING_WITH_LEN("body_utf8")},
-     {STRING_WITH_LEN("longblob")},
-     {nullptr, 0}}};
+    {{STRING_WITH_LEN("body_utf8")}, {STRING_WITH_LEN("longblob")}, {NULL, 0}}};
 
 static const TABLE_FIELD_DEF event_table_def = {ET_FIELD_COUNT,
                                                 event_table_fields};
@@ -173,16 +165,14 @@ static const TABLE_FIELD_TYPE event_table_fields_old[ET_FIELD_COUNT] = {
     {{STRING_WITH_LEN("name")},
      {STRING_WITH_LEN("char(64)")},
      {STRING_WITH_LEN("utf8")}},
-    {{STRING_WITH_LEN("body")}, {STRING_WITH_LEN("longblob")}, {nullptr, 0}},
+    {{STRING_WITH_LEN("body")}, {STRING_WITH_LEN("longblob")}, {NULL, 0}},
     {{STRING_WITH_LEN("definer")},
      {STRING_WITH_LEN("char(77)")},
      {STRING_WITH_LEN("utf8")}},
-    {{STRING_WITH_LEN("execute_at")},
-     {STRING_WITH_LEN("datetime")},
-     {nullptr, 0}},
+    {{STRING_WITH_LEN("execute_at")}, {STRING_WITH_LEN("datetime")}, {NULL, 0}},
     {{STRING_WITH_LEN("interval_value")},
-     {STRING_WITH_LEN("int")},
-     {nullptr, 0}},
+     {STRING_WITH_LEN("int(11)")},
+     {NULL, 0}},
     {{STRING_WITH_LEN("interval_field")},
      {STRING_WITH_LEN(
          "enum('YEAR','QUARTER','MONTH','DAY',"
@@ -192,24 +182,20 @@ static const TABLE_FIELD_TYPE event_table_fields_old[ET_FIELD_COUNT] = {
          ","
          "'DAY_MICROSECOND','HOUR_MICROSECOND','MINUTE_MICROSECOND',"
          "'SECOND_MICROSECOND')")},
-     {nullptr, 0}},
-    {{STRING_WITH_LEN("created")},
-     {STRING_WITH_LEN("timestamp")},
-     {nullptr, 0}},
-    {{STRING_WITH_LEN("modified")},
-     {STRING_WITH_LEN("timestamp")},
-     {nullptr, 0}},
+     {NULL, 0}},
+    {{STRING_WITH_LEN("created")}, {STRING_WITH_LEN("timestamp")}, {NULL, 0}},
+    {{STRING_WITH_LEN("modified")}, {STRING_WITH_LEN("timestamp")}, {NULL, 0}},
     {{STRING_WITH_LEN("last_executed")},
      {STRING_WITH_LEN("datetime")},
-     {nullptr, 0}},
-    {{STRING_WITH_LEN("starts")}, {STRING_WITH_LEN("datetime")}, {nullptr, 0}},
-    {{STRING_WITH_LEN("ends")}, {STRING_WITH_LEN("datetime")}, {nullptr, 0}},
+     {NULL, 0}},
+    {{STRING_WITH_LEN("starts")}, {STRING_WITH_LEN("datetime")}, {NULL, 0}},
+    {{STRING_WITH_LEN("ends")}, {STRING_WITH_LEN("datetime")}, {NULL, 0}},
     {{STRING_WITH_LEN("status")},
      {STRING_WITH_LEN("enum('ENABLED','DISABLED','SLAVESIDE_DISABLED')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("on_completion")},
      {STRING_WITH_LEN("enum('DROP','PRESERVE')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("sql_mode")},
      {STRING_WITH_LEN(
          "set('REAL_AS_FLOAT','PIPES_AS_CONCAT','ANSI_QUOTES',"
@@ -224,11 +210,11 @@ static const TABLE_FIELD_TYPE event_table_fields_old[ET_FIELD_COUNT] = {
          "'ERROR_FOR_DIVISION_BY_ZERO','TRADITIONAL','NO_AUTO_CREATE_USER',"
          "'HIGH_NOT_PRECEDENCE','NO_ENGINE_SUBSTITUTION','PAD_CHAR_TO_FULL_"
          "LENGTH')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("comment")},
      {STRING_WITH_LEN("char(64)")},
      {STRING_WITH_LEN("utf8")}},
-    {{STRING_WITH_LEN("originator")}, {STRING_WITH_LEN("int")}, {nullptr, 0}},
+    {{STRING_WITH_LEN("originator")}, {STRING_WITH_LEN("int(10)")}, {NULL, 0}},
     {{STRING_WITH_LEN("time_zone")},
      {STRING_WITH_LEN("char(64)")},
      {STRING_WITH_LEN("latin1")}},
@@ -241,9 +227,7 @@ static const TABLE_FIELD_TYPE event_table_fields_old[ET_FIELD_COUNT] = {
     {{STRING_WITH_LEN("db_collation")},
      {STRING_WITH_LEN("char(32)")},
      {STRING_WITH_LEN("utf8")}},
-    {{STRING_WITH_LEN("body_utf8")},
-     {STRING_WITH_LEN("longblob")},
-     {nullptr, 0}}};
+    {{STRING_WITH_LEN("body_utf8")}, {STRING_WITH_LEN("longblob")}, {NULL, 0}}};
 
 static const TABLE_FIELD_DEF event_table_def_old = {ET_FIELD_COUNT,
                                                     event_table_fields_old};
@@ -259,13 +243,13 @@ static void load_event_creation_context(THD *thd, TABLE *table,
   thd->variables.time_zone = my_tz_SYSTEM;
 
   if ((tz_name.str = get_field(thd->mem_root,
-                               table->field[ET_FIELD_TIME_ZONE])) == nullptr) {
+                               table->field[ET_FIELD_TIME_ZONE])) == NULL) {
     LogErr(WARNING_LEVEL, ER_EVENT_CANT_GET_TIMEZONE_FROM_FIELD,
            et_parse_data->dbname.str, et_parse_data->name.str);
   } else {
     tz_name.length = strlen(tz_name.str);
     String tz_str(tz_name.str, &my_charset_latin1);
-    if ((thd->variables.time_zone = my_tz_find(thd, &tz_str)) == nullptr) {
+    if ((thd->variables.time_zone = my_tz_find(thd, &tz_str)) == NULL) {
       thd->variables.time_zone = my_tz_SYSTEM;
       LogErr(WARNING_LEVEL, ER_EVENT_CANT_FIND_TIMEZONE,
              et_parse_data->dbname.str, et_parse_data->name.str);
@@ -403,8 +387,7 @@ static bool set_status_and_interval_for_event(THD *thd, TABLE *table,
     et_parse_data->interval = (interval_type)i;
   }
 
-  if ((ptr = get_field(thd->mem_root, table->field[ET_FIELD_STATUS])) ==
-      nullptr)
+  if ((ptr = get_field(thd->mem_root, table->field[ET_FIELD_STATUS])) == NULL)
     return true;
 
   switch (ptr[0]) {
@@ -434,25 +417,25 @@ static bool migrate_event_to_dd(THD *thd, TABLE *event_table) {
   LEX_STRING event_body, event_body_utf8;
 
   et_parse_data.interval = INTERVAL_LAST;
-  et_parse_data.identifier = nullptr;
+  et_parse_data.identifier = NULL;
 
   if ((et_parse_data.definer.str = get_field(
-           thd->mem_root, event_table->field[ET_FIELD_DEFINER])) == nullptr)
+           thd->mem_root, event_table->field[ET_FIELD_DEFINER])) == NULL)
     return true;
   et_parse_data.definer.length = strlen(et_parse_data.definer.str);
 
-  if ((et_parse_data.name.str = get_field(
-           thd->mem_root, event_table->field[ET_FIELD_NAME])) == nullptr)
+  if ((et_parse_data.name.str =
+           get_field(thd->mem_root, event_table->field[ET_FIELD_NAME])) == NULL)
     return true;
   et_parse_data.name.length = strlen(et_parse_data.name.str);
 
-  if ((et_parse_data.dbname.str = get_field(
-           thd->mem_root, event_table->field[ET_FIELD_DB])) == nullptr)
+  if ((et_parse_data.dbname.str =
+           get_field(thd->mem_root, event_table->field[ET_FIELD_DB])) == NULL)
     return true;
   et_parse_data.dbname.length = strlen(et_parse_data.dbname.str);
 
   if ((et_parse_data.comment.str = get_field(
-           thd->mem_root, event_table->field[ET_FIELD_COMMENT])) == nullptr)
+           thd->mem_root, event_table->field[ET_FIELD_COMMENT])) == NULL)
     et_parse_data.comment.length = 0;
   else
     et_parse_data.comment.length = strlen(et_parse_data.comment.str);
@@ -476,11 +459,11 @@ static bool migrate_event_to_dd(THD *thd, TABLE *event_table) {
     return true;
 
   if ((ptr = get_field(thd->mem_root,
-                       event_table->field[ET_FIELD_ORIGINATOR])) == nullptr)
+                       event_table->field[ET_FIELD_ORIGINATOR])) == NULL)
     return true;
 
   if ((ptr = get_field(thd->mem_root,
-                       event_table->field[ET_FIELD_ON_COMPLETION])) == nullptr)
+                       event_table->field[ET_FIELD_ON_COMPLETION])) == NULL)
     return true;
 
   et_parse_data.on_completion =
@@ -488,13 +471,13 @@ static bool migrate_event_to_dd(THD *thd, TABLE *event_table) {
                      : Event_parse_data::ON_COMPLETION_PRESERVE);
 
   // Set up the event body.
-  if ((event_body.str = get_field(
-           thd->mem_root, event_table->field[ET_FIELD_BODY])) == nullptr)
+  if ((event_body.str =
+           get_field(thd->mem_root, event_table->field[ET_FIELD_BODY])) == NULL)
     return true;
   event_body.length = strlen(event_body.str);
 
   if ((event_body_utf8.str = get_field(
-           thd->mem_root, event_table->field[ET_FIELD_BODY_UTF8])) == nullptr)
+           thd->mem_root, event_table->field[ET_FIELD_BODY_UTF8])) == NULL)
     return true;
   event_body_utf8.length = strlen(event_body_utf8.str);
   et_parse_data.body_changed = true;
@@ -507,6 +490,7 @@ static bool migrate_event_to_dd(THD *thd, TABLE *event_table) {
   // Holders for user name and host name used in parse user.
   char definer_user_name_holder[USERNAME_LENGTH + 1];
   char definer_host_name_holder[HOSTNAME_LENGTH + 1];
+  memset(&user_info, 0, sizeof(LEX_USER));
   user_info.user = {definer_user_name_holder, USERNAME_LENGTH};
   user_info.host = {definer_host_name_holder, HOSTNAME_LENGTH};
 
@@ -533,14 +517,7 @@ static bool migrate_event_to_dd(THD *thd, TABLE *event_table) {
   dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   const dd::Schema *schema = nullptr;
   if (thd->dd_client()->acquire(et_parse_data.dbname.str, &schema)) return true;
-
-  if (schema == nullptr) {
-    // Schema does not exist. Fail with an error indicating the presence of an
-    // orphan event.
-    LogErr(ERROR_LEVEL, ER_UPGRADE_NONEXISTENT_SCHEMA, et_parse_data.dbname.str,
-           "event", et_parse_data.name.str, "events");
-    return true;
-  }
+  DBUG_ASSERT(schema != nullptr);
 
   if (dd::create_event(thd, *schema, et_parse_data.name.str, event_body.str,
                        event_body_utf8.str, &user_info, &et_parse_data)) {
@@ -591,9 +568,9 @@ bool migrate_events_to_dd(THD *thd) {
 
   // Initialize time zone support infrastructure since the information
   // is not available during upgrade.
-  my_tz_init(thd, default_tz_name, false);
+  my_tz_init(thd, default_tz_name, 0);
 
-  if (event_table->file->ha_index_init(0, true)) {
+  if (event_table->file->ha_index_init(0, 1)) {
     LogErr(ERROR_LEVEL, ER_EVENT_CANT_OPEN_TABLE_MYSQL_EVENT);
     goto err;
   }
