@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,13 +33,12 @@
 #include "my_loglevel.h"
 #include "my_sys.h"
 #include "my_user.h"  // parse_user
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
-#include "sql/dd/impl/upgrade/server.h"
 #include "sql/dd/types/schema.h"
 #include "sql/dd/upgrade_57/global.h"
 #include "sql/dd_sp.h"  // prepare_sp_chistics_from_dd_routine
@@ -63,6 +61,8 @@
 #include "sql_string.h"
 #include "thr_lock.h"
 
+#include "sql/dd/impl/upgrade/server.h"
+
 namespace dd {
 
 namespace upgrade_57 {
@@ -75,42 +75,38 @@ static Check_table_intact table_intact;
 static const TABLE_FIELD_TYPE proc_table_fields[MYSQL_PROC_FIELD_COUNT] = {
     {{STRING_WITH_LEN("db")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("name")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("type")},
      {STRING_WITH_LEN("enum('FUNCTION','PROCEDURE')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("specific_name")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("language")},
      {STRING_WITH_LEN("enum('SQL')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("sql_data_access")},
      {STRING_WITH_LEN(
          "enum('CONTAINS_SQL','NO_SQL','READS_SQL_DATA','MODIFIES_SQL_DATA')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("is_deterministic")},
      {STRING_WITH_LEN("enum('YES','NO')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("security_type")},
      {STRING_WITH_LEN("enum('INVOKER','DEFINER')")},
-     {nullptr, 0}},
-    {{STRING_WITH_LEN("param_list")}, {STRING_WITH_LEN("blob")}, {nullptr, 0}},
+     {NULL, 0}},
+    {{STRING_WITH_LEN("param_list")}, {STRING_WITH_LEN("blob")}, {NULL, 0}},
 
-    {{STRING_WITH_LEN("returns")}, {STRING_WITH_LEN("longblob")}, {nullptr, 0}},
-    {{STRING_WITH_LEN("body")}, {STRING_WITH_LEN("longblob")}, {nullptr, 0}},
+    {{STRING_WITH_LEN("returns")}, {STRING_WITH_LEN("longblob")}, {NULL, 0}},
+    {{STRING_WITH_LEN("body")}, {STRING_WITH_LEN("longblob")}, {NULL, 0}},
     {{STRING_WITH_LEN("definer")},
      {STRING_WITH_LEN("char(93)")},
-     {STRING_WITH_LEN("utf8mb3")}},
-    {{STRING_WITH_LEN("created")},
-     {STRING_WITH_LEN("timestamp")},
-     {nullptr, 0}},
-    {{STRING_WITH_LEN("modified")},
-     {STRING_WITH_LEN("timestamp")},
-     {nullptr, 0}},
+     {STRING_WITH_LEN("utf8")}},
+    {{STRING_WITH_LEN("created")}, {STRING_WITH_LEN("timestamp")}, {NULL, 0}},
+    {{STRING_WITH_LEN("modified")}, {STRING_WITH_LEN("timestamp")}, {NULL, 0}},
     {{STRING_WITH_LEN("sql_mode")},
      {STRING_WITH_LEN(
          "set('REAL_AS_FLOAT','PIPES_AS_CONCAT','ANSI_QUOTES',"
@@ -125,22 +121,20 @@ static const TABLE_FIELD_TYPE proc_table_fields[MYSQL_PROC_FIELD_COUNT] = {
          "'ERROR_FOR_DIVISION_BY_ZERO','TRADITIONAL','NO_AUTO_CREATE_USER',"
          "'HIGH_NOT_PRECEDENCE','NO_ENGINE_SUBSTITUTION','PAD_CHAR_TO_FULL_"
          "LENGTH')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("comment")},
      {STRING_WITH_LEN("text")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("character_set_client")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("collation_connection")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("db_collation")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8mb3")}},
-    {{STRING_WITH_LEN("body_utf8")},
-     {STRING_WITH_LEN("longblob")},
-     {nullptr, 0}}};
+     {STRING_WITH_LEN("utf8")}},
+    {{STRING_WITH_LEN("body_utf8")}, {STRING_WITH_LEN("longblob")}, {NULL, 0}}};
 
 static const TABLE_FIELD_DEF proc_table_def = {MYSQL_PROC_FIELD_COUNT,
                                                proc_table_fields};
@@ -152,42 +146,38 @@ static const TABLE_FIELD_DEF proc_table_def = {MYSQL_PROC_FIELD_COUNT,
 static const TABLE_FIELD_TYPE proc_table_fields_old[MYSQL_PROC_FIELD_COUNT] = {
     {{STRING_WITH_LEN("db")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("name")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("type")},
      {STRING_WITH_LEN("enum('FUNCTION','PROCEDURE')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("specific_name")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("language")},
      {STRING_WITH_LEN("enum('SQL')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("sql_data_access")},
      {STRING_WITH_LEN(
          "enum('CONTAINS_SQL','NO_SQL','READS_SQL_DATA','MODIFIES_SQL_DATA')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("is_deterministic")},
      {STRING_WITH_LEN("enum('YES','NO')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("security_type")},
      {STRING_WITH_LEN("enum('INVOKER','DEFINER')")},
-     {nullptr, 0}},
-    {{STRING_WITH_LEN("param_list")}, {STRING_WITH_LEN("blob")}, {nullptr, 0}},
+     {NULL, 0}},
+    {{STRING_WITH_LEN("param_list")}, {STRING_WITH_LEN("blob")}, {NULL, 0}},
 
-    {{STRING_WITH_LEN("returns")}, {STRING_WITH_LEN("longblob")}, {nullptr, 0}},
-    {{STRING_WITH_LEN("body")}, {STRING_WITH_LEN("longblob")}, {nullptr, 0}},
+    {{STRING_WITH_LEN("returns")}, {STRING_WITH_LEN("longblob")}, {NULL, 0}},
+    {{STRING_WITH_LEN("body")}, {STRING_WITH_LEN("longblob")}, {NULL, 0}},
     {{STRING_WITH_LEN("definer")},
      {STRING_WITH_LEN("char(77)")},
-     {STRING_WITH_LEN("utf8mb3")}},
-    {{STRING_WITH_LEN("created")},
-     {STRING_WITH_LEN("timestamp")},
-     {nullptr, 0}},
-    {{STRING_WITH_LEN("modified")},
-     {STRING_WITH_LEN("timestamp")},
-     {nullptr, 0}},
+     {STRING_WITH_LEN("utf8")}},
+    {{STRING_WITH_LEN("created")}, {STRING_WITH_LEN("timestamp")}, {NULL, 0}},
+    {{STRING_WITH_LEN("modified")}, {STRING_WITH_LEN("timestamp")}, {NULL, 0}},
     {{STRING_WITH_LEN("sql_mode")},
      {STRING_WITH_LEN(
          "set('REAL_AS_FLOAT','PIPES_AS_CONCAT','ANSI_QUOTES',"
@@ -202,22 +192,20 @@ static const TABLE_FIELD_TYPE proc_table_fields_old[MYSQL_PROC_FIELD_COUNT] = {
          "'ERROR_FOR_DIVISION_BY_ZERO','TRADITIONAL','NO_AUTO_CREATE_USER',"
          "'HIGH_NOT_PRECEDENCE','NO_ENGINE_SUBSTITUTION','PAD_CHAR_TO_FULL_"
          "LENGTH')")},
-     {nullptr, 0}},
+     {NULL, 0}},
     {{STRING_WITH_LEN("comment")},
      {STRING_WITH_LEN("text")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("character_set_client")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("collation_connection")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8mb3")}},
+     {STRING_WITH_LEN("utf8")}},
     {{STRING_WITH_LEN("db_collation")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8mb3")}},
-    {{STRING_WITH_LEN("body_utf8")},
-     {STRING_WITH_LEN("longblob")},
-     {nullptr, 0}}};
+     {STRING_WITH_LEN("utf8")}},
+    {{STRING_WITH_LEN("body_utf8")}, {STRING_WITH_LEN("longblob")}, {NULL, 0}}};
 
 static const TABLE_FIELD_DEF proc_table_def_old = {MYSQL_PROC_FIELD_COUNT,
                                                    proc_table_fields_old};
@@ -236,7 +224,7 @@ static bool set_st_sp_chistics(THD *thd, TABLE *proc_table,
   memset(chistics, 0, sizeof(st_sp_chistics));
 
   if ((ptr = get_field(thd->mem_root,
-                       proc_table->field[MYSQL_PROC_FIELD_ACCESS])) == nullptr)
+                       proc_table->field[MYSQL_PROC_FIELD_ACCESS])) == NULL)
     return true;
 
   switch (ptr[0]) {
@@ -259,7 +247,7 @@ static bool set_st_sp_chistics(THD *thd, TABLE *proc_table,
   // Deterministic
   if ((ptr = get_field(thd->mem_root,
                        proc_table->field[MYSQL_PROC_FIELD_DETERMINISTIC])) ==
-      nullptr)
+      NULL)
     return true;
 
   chistics->detistic = (ptr[0] != 'N');
@@ -267,7 +255,7 @@ static bool set_st_sp_chistics(THD *thd, TABLE *proc_table,
   // Security type
   if ((ptr = get_field(thd->mem_root,
                        proc_table->field[MYSQL_PROC_FIELD_SECURITY_TYPE])) ==
-      nullptr)
+      NULL)
     return true;
 
   chistics->suid = (ptr[0] == 'I' ? SP_IS_NOT_SUID : SP_IS_SUID);
@@ -275,7 +263,7 @@ static bool set_st_sp_chistics(THD *thd, TABLE *proc_table,
   // Fetch SP/SF comment
   proc_table->field[MYSQL_PROC_FIELD_COMMENT]->val_str(&str, &str);
 
-  ptr = nullptr;
+  ptr = 0;
   if ((length = str.length()))
     ptr = strmake_root(thd->mem_root, str.ptr(), length);
   chistics->comment.str = ptr;
@@ -296,20 +284,18 @@ static bool migrate_routine_to_dd(THD *thd, TABLE *proc_table) {
   sp_head *sp = nullptr;
   enum_sp_type routine_type;
   LEX_USER user_info;
-  bool dummy_is_sp_created = false;
 
-  // Fetch SP/SF name, database name, definer and type.
+  // Fetch SP/SF name, datbase name, definer and type.
   if ((sp_db = get_field(thd->mem_root,
-                         proc_table->field[MYSQL_PROC_FIELD_DB])) == nullptr)
+                         proc_table->field[MYSQL_PROC_FIELD_DB])) == NULL)
     return true;
 
-  if ((sp_name1 = get_field(
-           thd->mem_root, proc_table->field[MYSQL_PROC_FIELD_NAME])) == nullptr)
+  if ((sp_name1 = get_field(thd->mem_root,
+                            proc_table->field[MYSQL_PROC_FIELD_NAME])) == NULL)
     return true;
 
-  if ((definer = get_field(thd->mem_root,
-                           proc_table->field[MYSQL_PROC_FIELD_DEFINER])) ==
-      nullptr)
+  if ((definer = get_field(
+           thd->mem_root, proc_table->field[MYSQL_PROC_FIELD_DEFINER])) == NULL)
     return true;
 
   routine_type =
@@ -318,7 +304,7 @@ static bool migrate_routine_to_dd(THD *thd, TABLE *proc_table) {
   // Fetch SP/SF parameters string
   if ((params = get_field(thd->mem_root,
                           proc_table->field[MYSQL_PROC_FIELD_PARAM_LIST])) ==
-      nullptr)
+      NULL)
     params = "";
 
   // Create return type string for SF
@@ -326,7 +312,7 @@ static bool migrate_routine_to_dd(THD *thd, TABLE *proc_table) {
     returns = "";
   else if ((returns = get_field(thd->mem_root,
                                 proc_table->field[MYSQL_PROC_FIELD_RETURNS])) ==
-           nullptr)
+           NULL)
     return true;
 
   st_sp_chistics chistics;
@@ -338,7 +324,7 @@ static bool migrate_routine_to_dd(THD *thd, TABLE *proc_table) {
 
   // Fetch SP/SF body
   if ((body = get_field(thd->mem_root,
-                        proc_table->field[MYSQL_PROC_FIELD_BODY])) == nullptr)
+                        proc_table->field[MYSQL_PROC_FIELD_BODY])) == NULL)
     return true;
 
   dd::upgrade::Routine_event_context_guard routine_ctx_guard(thd);
@@ -373,6 +359,7 @@ static bool migrate_routine_to_dd(THD *thd, TABLE *proc_table) {
   // Holders for user name and host name used in parse user.
   char definer_user_name_holder[USERNAME_LENGTH + 1];
   char definer_host_name_holder[HOSTNAME_LENGTH + 1];
+  memset(&user_info, 0, sizeof(LEX_USER));
   user_info.user = {definer_user_name_holder, USERNAME_LENGTH};
   user_info.host = {definer_host_name_holder, HOSTNAME_LENGTH};
 
@@ -385,21 +372,11 @@ static bool migrate_routine_to_dd(THD *thd, TABLE *proc_table) {
   Disable_autocommit_guard autocommit_guard(thd);
 
   // This function fixes sp_head to use in sp_create_routine()
-  enum_sp_return_code sp_ret_code = db_load_routine(
-      thd, routine_type, sp_db_str.str, sp_db_str.length, sp_name_str.str,
-      sp_name_str.length, &sp, thd->variables.sql_mode, params, returns, body,
-      &chistics, definer_user_name_holder, definer_host_name_holder, created,
-      modified, creation_ctx);
-
-  if (sp_ret_code != SP_OK) {
-    if (sp_ret_code == SP_NO_DB_ERROR) {
-      // Schema does not exist. Fail with an error indicating the presence of an
-      // orphan routine.
-      LogErr(ERROR_LEVEL, ER_UPGRADE_NONEXISTENT_SCHEMA, sp_db_str.str,
-             "routine", sp_name_str.str, "routines");
-      goto err;
-    }
-
+  if (db_load_routine(thd, routine_type, sp_db_str.str, sp_db_str.length,
+                      sp_name_str.str, sp_name_str.length, &sp,
+                      thd->variables.sql_mode, params, returns, body, &chistics,
+                      definer_user_name_holder, definer_host_name_holder,
+                      created, modified, creation_ctx)) {
     /*
       Parsing of routine body failed. Use empty routine body and report a
       warning if the routine does not belong to sys schema. Sys schema routines
@@ -435,8 +412,7 @@ static bool migrate_routine_to_dd(THD *thd, TABLE *proc_table) {
   }
 
   // Create entry for SP/SF in DD table.
-  if (sp_create_routine(thd, sp, &user_info, false, dummy_is_sp_created))
-    goto err;
+  if (sp_create_routine(thd, sp, &user_info)) goto err;
 
   if (sp != nullptr)  // To be safe
     sp_head::destroy(sp);
@@ -463,7 +439,7 @@ bool migrate_routines_to_dd(THD *thd) {
   MEM_ROOT records_mem_root;
   Thd_mem_root_guard root_guard(thd, &records_mem_root);
 
-  Table_ref tables("mysql", "proc", TL_READ);
+  TABLE_LIST tables("mysql", "proc", TL_READ);
   auto table_list = &tables;
 
   if (open_and_lock_tables(thd, table_list, flags, &prelocking_strategy)) {
@@ -484,7 +460,7 @@ bool migrate_routines_to_dd(THD *thd) {
 
   System_table_close_guard proc_table_guard(thd, proc_table);
 
-  if (proc_table->file->ha_index_init(0, true)) {
+  if (proc_table->file->ha_index_init(0, 1)) {
     LogErr(ERROR_LEVEL, ER_CANT_READ_TABLE_MYSQL_PROC);
     return true;
   }
