@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -54,15 +54,13 @@
 #include "sql/dd/impl/dictionary_impl.h"          // dd::Dictionary_impl
 #include "sql/dd/impl/sdi.h"                      // sdi::store()
 #include "sql/dd/impl/system_registry.h"          // dd::System_tables
-#include "sql/dd/impl/upgrade/server.h"
-#include "sql/dd/impl/utils.h"               // execute_query
+#include "sql/dd/impl/utils.h"                    // execute_query
 #include "sql/dd/info_schema/metadata.h"     // dd::info_schema::install_IS...
 #include "sql/dd/performance_schema/init.h"  // create_pfs_schema
 #include "sql/dd/sdi_file.h"                 // dd::sdi_file::EXT
 #include "sql/dd/types/object_table.h"
 #include "sql/dd/types/table.h"  // dd::Table
 #include "sql/dd/types/tablespace.h"
-#include "sql/dd/upgrade/server.h"
 #include "sql/dd/upgrade_57/event.h"
 #include "sql/dd/upgrade_57/global.h"
 #include "sql/dd/upgrade_57/routine.h"
@@ -86,6 +84,9 @@
 #include "sql/thd_raii.h"
 #include "sql/transaction.h"  // trans_rollback
 
+#include "sql/dd/impl/upgrade/server.h"
+#include "sql/dd/upgrade/server.h"
+
 namespace dd {
 
 namespace upgrade_57 {
@@ -261,7 +262,7 @@ bool finalize_upgrade(THD *thd) {
       // Get the name without the file extension.
       if (check_file_extension(file_ext)) {
         if (fn_format(from_path, file.c_str(), mysql_real_data_home, "",
-                      MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == nullptr)
+                      MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == NULL)
           return true;
 
         (void)mysql_file_delete(key_file_misc, from_path, MYF(0));
@@ -275,7 +276,7 @@ bool finalize_upgrade(THD *thd) {
     char dir_path[FN_REFLEN];
 
     if (fn_format(dir_path, dir_name.c_str(), path.c_str(), "",
-                  MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == nullptr)
+                  MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == NULL)
       continue;
 
     if (!(b = my_dir(dir_path, MYF(MY_WANT_STAT)))) continue;
@@ -293,7 +294,7 @@ bool finalize_upgrade(THD *thd) {
       // Get the name without the file extension.
       if (check_file_extension(file_ext)) {
         if (fn_format(from_path, file.c_str(), dir_path, "",
-                      MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == nullptr)
+                      MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == NULL)
           continue;
 
         (void)mysql_file_delete(key_file_misc, from_path, MYF(0));
@@ -414,7 +415,7 @@ static void drop_sdi_files() {
     if (MY_S_ISDIR(a->dir_entry[i].mystat->st_mode)) {
       char dir_path[FN_REFLEN];
       if (fn_format(dir_path, file.c_str(), path.c_str(), "",
-                    MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == nullptr) {
+                    MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == NULL) {
         LogErr(ERROR_LEVEL, ER_CANT_SET_PATH_FOR, file.c_str());
         continue;
       }
@@ -436,7 +437,7 @@ static void drop_sdi_files() {
         if (file_ext.compare(0, 4, dd::sdi_file::EXT) == 0) {
           char to_path[FN_REFLEN];
           if (fn_format(to_path, file2.c_str(), dir_path, "",
-                        MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == nullptr) {
+                        MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == NULL) {
             LogErr(ERROR_LEVEL, ER_CANT_SET_PATH_FOR, file2.c_str());
             continue;
           }
@@ -454,7 +455,7 @@ static void drop_sdi_files() {
       if (file_ext.compare(0, 4, dd::sdi_file::EXT) == 0) {
         char to_path[FN_REFLEN];
         if (fn_format(to_path, file.c_str(), path.c_str(), "",
-                      MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == nullptr) {
+                      MYF(MY_UNPACK_FILENAME | MY_SAFE_PATH)) == NULL) {
           LogErr(ERROR_LEVEL, ER_CANT_SET_PATH_FOR, file.c_str());
           continue;
         }
@@ -569,6 +570,7 @@ bool add_sdi_info(THD *thd) {
     return true;
 
   LogErr(SYSTEM_LEVEL, ER_DD_UPGRADE_DD_POPULATED);
+  log_sink_buffer_check_timeout();
   sysd::notify("STATUS=Data Dictionary upgrade from MySQL 5.7 complete\n");
 
   return false;
@@ -593,7 +595,7 @@ Upgrade_status::enum_stage Upgrade_status::get() {
 bool Upgrade_status::update(Upgrade_status::enum_stage stage) {
   if (open(O_TRUNC | O_WRONLY)) return true;
 
-  if (write(stage)) return true;
+  write(stage);
 
   if (close()) return true;
 
@@ -608,7 +610,7 @@ Upgrade_status::Upgrade_status()
 bool Upgrade_status::create() {
   if (open(O_TRUNC | O_WRONLY)) return true;
 
-  if (write(enum_stage::STARTED)) return true;
+  write(enum_stage::STARTED);
 
   if (exists() == false || close()) return true;
 
@@ -617,7 +619,7 @@ bool Upgrade_status::create() {
 
 // Open status file.
 bool Upgrade_status::open(int flags) {
-  assert(m_file == nullptr);
+  DBUG_ASSERT(m_file == nullptr);
 
   if (!(m_file = my_fopen(m_filename.c_str(), flags, MYF(0)))) {
     LogErr(ERROR_LEVEL, ER_DD_UPGRADE_INFO_FILE_OPEN_FAILED, m_filename.c_str(),
@@ -630,10 +632,10 @@ bool Upgrade_status::open(int flags) {
 
 // Read status from file.
 Upgrade_status::enum_stage Upgrade_status::read() {
-  assert(m_file);
+  DBUG_ASSERT(m_file);
 
   enum_stage stage = enum_stage::NONE;
-  size_t items_read [[maybe_unused]];
+  size_t items_read MY_ATTRIBUTE((unused));
 
   if (!feof(m_file)) items_read = fread(&stage, sizeof(int), 1, m_file);
 
@@ -642,14 +644,9 @@ Upgrade_status::enum_stage Upgrade_status::read() {
 
 // Write status to file.
 bool Upgrade_status::write(Upgrade_status::enum_stage stage) {
-  assert(m_file);
+  DBUG_ASSERT(m_file);
 
-  if (fwrite(&stage, sizeof(int), 1, m_file) != 1) {
-    char errbuf[MYSYS_STRERROR_SIZE];
-    LogErr(ERROR_LEVEL, ER_FAILED_TO_WRITE_TO_FILE, m_filename.c_str(),
-           ferror(m_file), my_strerror(errbuf, sizeof(errbuf), ferror(m_file)));
-    return true;
-  }
+  fwrite(&stage, sizeof(int), 1, m_file);
   fflush(m_file);
   return false;
 }
@@ -661,7 +658,7 @@ bool Upgrade_status::exists() {
 
 // Close status file.
 bool Upgrade_status::close() {
-  assert(m_file);
+  DBUG_ASSERT(m_file);
 
   if (my_fclose(m_file, MYF(0))) {
     LogErr(ERROR_LEVEL, ER_DD_UPGRADE_INFO_FILE_CLOSE_FAILED,
@@ -676,7 +673,7 @@ bool Upgrade_status::close() {
 
 // Delete status file.
 bool Upgrade_status::remove() {
-  assert(!m_file);
+  DBUG_ASSERT(!m_file);
   (void)mysql_file_delete(key_file_misc, m_filename.c_str(), MYF(MY_WME));
   return false;
 }
@@ -744,7 +741,7 @@ static bool ha_migrate_tablespaces(THD *thd, plugin_ref plugin, void *) {
 */
 static bool ha_migrate_tablespaces(THD *thd) {
   return (plugin_foreach(thd, ha_migrate_tablespaces,
-                         MYSQL_STORAGE_ENGINE_PLUGIN, nullptr));
+                         MYSQL_STORAGE_ENGINE_PLUGIN, 0));
 }
 
 /**
@@ -757,16 +754,14 @@ static bool migrate_stats(THD *thd) {
   error_handler.set_log_error(false);
   if (dd::execute_query(thd,
                         "INSERT IGNORE INTO mysql.innodb_table_stats "
-                        "SELECT * FROM mysql.innodb_table_stats_backup57 "
-                        "WHERE table_name not like '%#P#%'"))
+                        "SELECT * FROM mysql.innodb_table_stats_backup57"))
     LogErr(WARNING_LEVEL, ER_DD_UPGRADE_FAILED_TO_CREATE_TABLE_STATS);
   else
     LogErr(INFORMATION_LEVEL, ER_DD_UPGRADE_TABLE_STATS_MIGRATE_COMPLETED);
 
   if (dd::execute_query(thd,
                         "INSERT IGNORE INTO mysql.innodb_index_stats "
-                        "SELECT * FROM mysql.innodb_index_stats_backup57 "
-                        "WHERE table_name not like '%#P#%'"))
+                        "SELECT * FROM mysql.innodb_index_stats_backup57"))
     LogErr(WARNING_LEVEL, ER_DD_UPGRADE_FAILED_TO_CREATE_INDEX_STATS);
   else
     LogErr(INFORMATION_LEVEL, ER_DD_UPGRADE_TABLE_STATS_MIGRATE_COMPLETED);
@@ -826,7 +821,7 @@ static bool upgrade_logs(THD *thd, plugin_ref plugin, void *) {
   @retval true   ON FAILURE
 */
 static bool ha_upgrade_engine_logs(THD *thd) {
-  if (plugin_foreach(thd, upgrade_logs, MYSQL_STORAGE_ENGINE_PLUGIN, nullptr))
+  if (plugin_foreach(thd, upgrade_logs, MYSQL_STORAGE_ENGINE_PLUGIN, 0))
     return true;
 
   return false;
@@ -841,7 +836,7 @@ bool do_pre_checks_and_initialize_dd(THD *thd) {
 
   Disable_autocommit_guard autocommit_guard(thd);
   Dictionary_impl *d = dd::Dictionary_impl::instance();
-  assert(d);
+  DBUG_ASSERT(d);
   cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
 
   char path[FN_REFLEN + 1];
@@ -886,22 +881,15 @@ bool do_pre_checks_and_initialize_dd(THD *thd) {
       return true;
     }
 
-    // Safeguard against upgrading from 5.7 to 9.x
-    if (MYSQL_VERSION_ID >= 90000) {
-      LogErr(ERROR_LEVEL, ER_INVALID_SERVER_UPGRADE_NOT_LTS,
-             bootstrap::SERVER_VERSION_50700, MYSQL_VERSION_ID,
-             bootstrap::SERVER_VERSION_50700);
-      return true;
-    }
-
     // Create the file to track stages of upgrade.
     if (upgrade_status.create()) return true;
 
     /*
-      If mysql.idb does not exist and upgrade stage tracking file
+      If mysql.idb does not exist and updgrade stage tracking file
       does not exist, we are in upgrade mode.
     */
     LogErr(SYSTEM_LEVEL, ER_DD_UPGRADE_START);
+    log_sink_buffer_check_timeout();
     sysd::notify("STATUS=Data Dictionary upgrade from MySQL 5.7 in progress\n");
   }
 
@@ -1207,14 +1195,9 @@ bool fill_dd_and_finalize(THD *thd) {
   */
   bootstrap_error_handler.set_log_error(false);
 
-  std::set<uint> allowed_errors = {ER_DEFINITION_CONTAINS_INVALID_STRING};
-  bootstrap_error_handler.set_allowlist_errors(allowed_errors);
-
   error |= migrate_events_to_dd(thd);
   error |= migrate_routines_to_dd(thd);
 
-  bootstrap_error_handler.clear_allowlist_errors();
-
   // We will not get error in this step unless its a fatal error.
   for (it = db_name.begin(); it != db_name.end(); it++) {
     // Upgrade view resolving dependency
@@ -1259,6 +1242,8 @@ bool fill_dd_and_finalize(THD *thd) {
   if (Upgrade_status().update(Upgrade_status::enum_stage::USER_TABLE_UPGRADED))
     return true;
 
+  log_sink_buffer_check_timeout();
+
   // Add SDI information to all tablespaces
   if (add_sdi_info(thd))
     LogErr(ERROR_LEVEL, ER_DD_UPGRADE_SDI_INFO_UPDATE_FAILED);
