@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -109,7 +108,7 @@ static Field *make_field(const dd::Parameter &param, TABLE_SHARE *share,
                     0, dd_get_old_field_type(param.data_type()),
                     dd_get_mysql_charset(param.collation_id()), geom_type,
                     Field::NONE, interval, "", false, param.is_zerofill(),
-                    param.is_unsigned(), numeric_scale, false, 0, {}, false);
+                    param.is_unsigned(), numeric_scale, 0, 0, {}, false);
 }
 
 /**
@@ -130,7 +129,7 @@ static void prepare_type_string_from_dd_param(THD *thd,
   DBUG_TRACE;
 
   // ENUM/SET elements.
-  TYPELIB *interval = nullptr;
+  TYPELIB *interval = NULL;
   if (param->data_type() == dd::enum_column_types::ENUM ||
       param->data_type() == dd::enum_column_types::SET) {
     // Allocate space for interval.
@@ -139,12 +138,12 @@ static void prepare_type_string_from_dd_param(THD *thd,
     interval = static_cast<TYPELIB *>(thd->mem_root->Alloc(sizeof(TYPELIB)));
     interval->type_names = static_cast<const char **>(
         thd->mem_root->Alloc((sizeof(char *) * (interval_parts + 1))));
-    interval->type_names[interval_parts] = nullptr;
+    interval->type_names[interval_parts] = 0;
 
     interval->type_lengths = static_cast<uint *>(
         thd->mem_root->Alloc(sizeof(uint) * interval_parts));
     interval->count = interval_parts;
-    interval->name = nullptr;
+    interval->name = NULL;
 
     for (const dd::Parameter_type_element *pe : param->elements()) {
       // Read the enum/set element name
@@ -182,7 +181,7 @@ static void prepare_type_string_from_dd_param(THD *thd,
     type_str->append(field->charset()->csname);
     if (!(field->charset()->state & MY_CS_PRIMARY)) {
       type_str->append(STRING_WITH_LEN(" COLLATE "));
-      type_str->append(field->charset()->m_coll_name);
+      type_str->append(field->charset()->name);
     }
   }
 }
@@ -204,7 +203,7 @@ void prepare_return_type_string_from_dd_routine(
 
     if (!parameters.empty()) {
       const dd::Parameter *param = *parameters.begin();
-      assert(param->ordinal_position() == 1);
+      DBUG_ASSERT(param->ordinal_position() == 1);
 
       String type_str(64);
       type_str.set_charset(system_charset_info);
