@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,7 +28,6 @@
 #include <vector>
 
 #include "lex_string.h"
-#include "m_ctype.h"  // my_casedn_str
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sqlcommand.h"
@@ -49,14 +48,12 @@
 #include "sql/error_handler.h"  // Internal_error_handler
 #include "sql/handler.h"        // HA_LEX_CREATE_TMP_TABLE
 #include "sql/mdl.h"
-#include "sql/mysqld.h"  // lower_case_table_names
 #include "sql/set_var.h"
 #include "sql/sp_head.h"    // sp_name
 #include "sql/sql_alter.h"  // Alter_info
 #include "sql/sql_base.h"   // open_tables
 #include "sql/sql_class.h"
 #include "sql/sql_const.h"
-#include "sql/sql_db.h"  // check_schema_readonly
 #include "sql/sql_error.h"
 #include "sql/sql_lex.h"   // LEX
 #include "sql/sql_view.h"  // mysql_register_view
@@ -106,8 +103,7 @@ class View_metadata_updater_context {
     m_thd->set_open_tables_state(&m_open_tables_state_backup);
 
     // Restore lex.
-    m_thd->lex->cleanup(m_thd, true);
-    m_thd->lex->destroy();
+    m_thd->lex->unit->cleanup(m_thd, true);
     lex_end(m_thd->lex);
     delete static_cast<st_lex_local *>(m_thd->lex);
     m_thd->lex = m_saved_lex;
@@ -142,9 +138,9 @@ class View_metadata_updater_context {
 class View_metadata_updater_error_handler final
     : public Internal_error_handler {
  public:
-  bool handle_condition(THD *, uint sql_errno, const char *,
-                        Sql_condition::enum_severity_level *,
-                        const char *msg) override {
+  virtual bool handle_condition(THD *, uint sql_errno, const char *,
+                                Sql_condition::enum_severity_level *,
+                                const char *msg) override {
     switch (sql_errno) {
       case ER_LOCK_WAIT_TIMEOUT:
       case ER_LOCK_DEADLOCK:
@@ -196,7 +192,7 @@ class View_metadata_updater_error_handler final
   }
 
  private:
-  ErrorHandlerFunctionPointer m_old_error_handler_hook;
+  void (*m_old_error_handler_hook)(uint, const char *, myf);
 
  private:
   uint m_sql_errno = 0;
@@ -220,12 +216,6 @@ Uncommitted_tables_guard::~Uncommitted_tables_guard() {
   @param      thd             Current thread.
   @param      db              Database name.
   @param      tbl_or_sf_name  Base table/ View/ Stored function name.
-  @param      skip_same_db    Indicates whether it is OK to skip
-                              views belonging to the same database
-                              as table (as they will be dropped anyway).
-  @param      mem_root        Memory root for allocation of temporary
-                              objects which will be cleared after
-                              processing this table/view/routine.
   @param[out] views           TABLE_LIST objects for views.
 
   @retval     false           Success.
@@ -236,7 +226,6 @@ Uncommitted_tables_guard::~Uncommitted_tables_guard() {
 template <typename T>
 static bool prepare_view_tables_list(THD *thd, const char *db,
                                      const char *tbl_or_sf_name,
-                                     bool skip_same_db, MEM_ROOT *mem_root,
                                      std::vector<TABLE_LIST *> *views) {
   DBUG_TRACE;
   std::vector<dd::Object_id> view_ids;
@@ -253,7 +242,7 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
     dd::String_type schema_name;
     // Get schema name and view name from the object id of the view.
     {
-      std::unique_ptr<dd::View> view;
+      dd::View *view = nullptr;
       // We need to use READ_UNCOMMITTED here as the view could be changed
       // by the same statement (e.g. RENAME TABLE).
       if (thd->dd_client()->acquire_uncached_uncommitted(view_ids.at(idx),
@@ -261,7 +250,7 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
         return true;
       if (!view) continue;
 
-      std::unique_ptr<dd::Schema> schema;
+      dd::Schema *schema = nullptr;
       if (thd->dd_client()->acquire_uncached_uncommitted(view->schema_id(),
                                                          &schema))
         return true;
@@ -270,47 +259,19 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
       schema_name = schema->name();
     }
 
-    if (skip_same_db) {
-      /*
-        DROP DATABASE acquires exclusive metadata lock on database being
-        dropped. So we have guarantee that dependent view belonging to
-        the same database won't be moved into other database using
-        RENAME TABLE or doing DROP/CREATE. Hence it is safe to skip
-        view belonging to the database being dropped even if we don't
-        have any lock on it yet.
-      */
-      assert(thd->mdl_context.owns_equal_or_stronger_lock(MDL_key::SCHEMA, db,
-                                                          "", MDL_EXCLUSIVE));
-      if (my_strcasecmp(table_alias_charset, db, schema_name.c_str()) == 0)
-        continue;
-    }
-
     // If TABLE_LIST object is already prepared for view name then skip it.
     if (prepared_view_ids.find(view_ids.at(idx)) == prepared_view_ids.end()) {
       // Prepare TABLE_LIST object for the view and push_back
 
-      char *db_name =
-          strmake_root(mem_root, schema_name.c_str(), schema_name.length());
+      const char *db_name = strmake_root(thd->mem_root, schema_name.c_str(),
+                                         schema_name.length());
       if (db_name == nullptr) return true;
 
-      char *vw_name =
-          strmake_root(mem_root, view_name.c_str(), view_name.length());
+      const char *vw_name =
+          strmake_root(thd->mem_root, view_name.c_str(), view_name.length());
       if (vw_name == nullptr) return true;
 
-      /*
-        With l_c_t_n == 2, the original lettercase is stored in the DD, so we
-        need to convert to lowercase to make sure we always lock with the same
-        lettercase. Note that there is an exception for information schema
-        view names, where the convention is to use capital letters, as stored
-        in the DD.
-      */
-      if (lower_case_table_names == 2) {
-        my_casedn_str(system_charset_info, db_name);
-        if (!is_infoschema_db(db_name))
-          my_casedn_str(system_charset_info, vw_name);
-      }
-
-      auto vw = new (mem_root)
+      auto vw = new (thd->mem_root)
           TABLE_LIST(db_name, schema_name.length(), vw_name, view_name.length(),
                      TL_IGNORE, MDL_EXCLUSIVE);
       if (vw == nullptr) return true;
@@ -342,15 +303,8 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
   @param          tbl_or_sf_name      Base table/ View/ Stored function name.
   @param          views_list          TABLE_LIST objects of the referencing
                                       views.
-  @param          skip_same_db        Indicates whether it is OK to skip
-                                      views belonging to the same database
-                                      as table (as they will be dropped
-                                      anyway).
   @param          commit_dd_changes   Indicates whether changes to DD need
                                       to be committed.
-  @param          mem_root            Memory root for allocation of temporary
-                                      objects which will be cleared after
-                                      processing referenced table/view/routine.
 
   @retval     false           Success.
   @retval     true            Failure.
@@ -360,16 +314,15 @@ static bool prepare_view_tables_list(THD *thd, const char *db,
 template <typename T>
 static bool mark_all_views_invalid(THD *thd, const char *db,
                                    const char *tbl_or_sf_name,
-                                   const std::vector<TABLE_LIST *> *views_list,
-                                   bool skip_same_db, bool commit_dd_changes,
-                                   MEM_ROOT *mem_root) {
+                                   std::vector<TABLE_LIST *> *views_list,
+                                   bool commit_dd_changes) {
   DBUG_TRACE;
-  assert(!views_list->empty());
+  DBUG_ASSERT(!views_list->empty());
 
   // Acquire lock on all the views.
   MDL_request_list mdl_requests;
   for (auto view : *views_list) {
-    MDL_request *schema_request = new (mem_root) MDL_request;
+    MDL_request *schema_request = new (thd->mem_root) MDL_request;
     ;
     MDL_REQUEST_INIT(schema_request, MDL_key::SCHEMA, view->db, "",
                      MDL_INTENTION_EXCLUSIVE, MDL_STATEMENT);
@@ -380,11 +333,6 @@ static bool mark_all_views_invalid(THD *thd, const char *db,
                                      thd->variables.lock_wait_timeout))
     return true;
 
-  // Check schema read only for all views.
-  for (auto view : *views_list) {
-    if (check_schema_readonly(thd, view->db)) return true;
-  }
-
   /*
     In the time gap of listing referencing views and acquiring MDL lock on them
     if any view definition is updated or dropped then it should not be
@@ -392,8 +340,7 @@ static bool mark_all_views_invalid(THD *thd, const char *db,
     Hence preparing updated list of view tables after acquiring the lock.
   */
   std::vector<TABLE_LIST *> updated_views_list;
-  if (prepare_view_tables_list<T>(thd, db, tbl_or_sf_name, skip_same_db,
-                                  mem_root, &updated_views_list))
+  if (prepare_view_tables_list<T>(thd, db, tbl_or_sf_name, &updated_views_list))
     return true;
   if (updated_views_list.empty()) return false;
 
@@ -469,11 +416,6 @@ static bool open_views_and_update_metadata(
     }
   }
 
-  // Check schema read only for all views.
-  for (auto view : *views) {
-    if (check_schema_readonly(thd, view->db)) return true;
-  }
-
   for (auto view : *views) {
     View_metadata_updater_context vw_metadata_update_context(thd);
 
@@ -498,26 +440,13 @@ static bool open_views_and_update_metadata(
     */
     uint counter = 0;
     DML_prelocking_strategy prelocking_strategy;
-    view->query_block = thd->lex->query_block;
-    /*
-      open_tables() will normally switch the current memory allocation context
-      from the execution context to the prepared statement context, if
-      such context is active. However, open_views_and_update_metadata() should
-      be done only as part of a single execution, and will be fully cleaned up
-      as part of that execution.
-      The following code forces the preparation code in open_tables() to
-      be performed on the execution mem_root, regardless of the current
-      embedding memory allocation context.
-    */
-    MEM_ROOT *saved_arena_mem_root = thd->stmt_arena->mem_root;
-    thd->stmt_arena->mem_root = thd->mem_root;
+    view->select_lex = thd->lex->select_lex;
     if (open_tables(thd, &view, &counter, MYSQL_OPEN_NO_NEW_TABLE_IN_SE,
                     &prelocking_strategy)) {
-      thd->stmt_arena->mem_root = saved_arena_mem_root;
       thd->pop_internal_handler();
 
       if (error_handler.is_view_invalid()) {
-        if (view->mdl_request.ticket != nullptr) {
+        if (view->mdl_request.ticket != NULL) {
           // Update view status in tables.options.view_valid.
           if (dd::update_view_status(thd, view->get_db_name(),
                                      view->get_table_name(), false,
@@ -533,7 +462,6 @@ static bool open_views_and_update_metadata(
       }
       continue;
     }
-    thd->stmt_arena->mem_root = saved_arena_mem_root;
     if (view->is_view() == false) {
       // In between listing views and locking(opening), if view is dropped and
       // created as table then skip it.
@@ -546,7 +474,7 @@ static bool open_views_and_update_metadata(
     LEX *org_lex = thd->lex;
     thd->lex = view_lex;
     view_lex->context_analysis_only |= CONTEXT_ANALYSIS_ONLY_VIEW;
-    if (view_lex->unit->prepare(thd, nullptr, nullptr, 0, 0)) {
+    if (view_lex->unit->prepare(thd, 0, 0, 0)) {
       thd->lex = org_lex;
       thd->pop_internal_handler();
       // Please refer comments in the view open error handling block above.
@@ -580,15 +508,12 @@ static bool open_views_and_update_metadata(
 
     if (thd->lex->unit->is_mergeable() &&
         view->algorithm != VIEW_ALGORITHM_TEMPTABLE) {
-      for (ORDER *order = thd->lex->query_block->order_list.first; order;
+      for (ORDER *order = thd->lex->select_lex->order_list.first; order;
            order = order->next)
         order->used_alias = false;  /// @see Item::print_for_order()
     }
     Sql_mode_parse_guard parse_guard(thd);
-    thd->lex->unit->print(
-        thd, &view_query,
-        static_cast<enum_query_type>(QT_TO_ARGUMENT_CHARSET |
-                                     QT_HIDE_ROLLUP_FUNCTIONS));
+    thd->lex->unit->print(thd, &view_query, QT_TO_ARGUMENT_CHARSET);
     if (lex_string_strmake(thd->mem_root, &view->select_stmt, view_query.ptr(),
                            view_query.length()))
       return true;
@@ -599,11 +524,11 @@ static bool open_views_and_update_metadata(
     if (thd->dd_client()->acquire_for_modification(view->db, view->table_name,
                                                    &new_view))
       return true;
-    assert(new_view != nullptr);
+    DBUG_ASSERT(new_view != nullptr);
     bool res = dd::update_view(thd, new_view, view);
 
     if (commit_dd_changes) {
-      Implicit_substatement_state_guard substatement_guard(thd);
+      Disable_gtid_state_update_guard disabler(thd);
       if (res) {
         trans_rollback_stmt(thd);
         // Full rollback in case we have THD::transaction_rollback_request.
@@ -612,12 +537,16 @@ static bool open_views_and_update_metadata(
         res = trans_commit_stmt(thd) || trans_commit(thd);
     }
     if (res) {
+      view_lex->unit->cleanup(thd, true);
+      lex_end(view_lex);
       thd->lex = org_lex;
       return true;
     }
     tdc_remove_table(thd, TDC_RT_REMOVE_ALL, view->get_db_name(),
                      view->get_table_name(), false);
 
+    view_lex->unit->cleanup(thd, true);
+    lex_end(view_lex);
     thd->lex = org_lex;
   }
   DEBUG_SYNC(thd, "after_updating_view_metadata");
@@ -659,7 +588,7 @@ static bool is_view_metadata_update_needed(THD *thd, const char *db,
       retval = is_non_dictionary_or_temp_table();
       break;
     case SQLCOM_ALTER_TABLE: {
-      assert(thd->lex->alter_info);
+      DBUG_ASSERT(thd->lex->alter_info);
 
       // Alter operations which affects view column metadata.
       const uint alter_operations =
@@ -719,11 +648,12 @@ static bool update_view_metadata(THD *thd, const char *db,
   if (is_view_metadata_update_needed(thd, db, tbl_or_sf_name)) {
     // Prepare list of all views referencing the db.table_name.
     std::vector<TABLE_LIST *> views;
-    if (prepare_view_tables_list<T>(thd, db, tbl_or_sf_name, false,
-                                    thd->mem_root, &views))
+    if (prepare_view_tables_list<T>(thd, db, tbl_or_sf_name, &views))
       return true;
     if (views.empty()) return false;
 
+    DEBUG_SYNC(thd, "after_preparing_view_tables_list");
+
     bool is_drop_operation = (thd->lex->sql_command == SQLCOM_DROP_TABLE ||
                               thd->lex->sql_command == SQLCOM_DROP_VIEW ||
                               thd->lex->sql_command == SQLCOM_DROP_FUNCTION ||
@@ -733,8 +663,8 @@ static bool update_view_metadata(THD *thd, const char *db,
     // If operation is drop operation then view referencing it becomes invalid.
     // Hence mark all view as invalid.
     if (is_drop_operation)
-      return mark_all_views_invalid<T>(thd, db, tbl_or_sf_name, &views, false,
-                                       commit_dd_changes, thd->mem_root);
+      return mark_all_views_invalid<T>(thd, db, tbl_or_sf_name, &views,
+                                       commit_dd_changes);
 
     /*
        Open views and update views metadata.
@@ -781,7 +711,7 @@ bool update_referencing_views_metadata(
     const char *new_table_name, bool commit_dd_changes,
     Uncommitted_tables_guard *uncommitted_tables) {
   DBUG_TRACE;
-  assert(table != nullptr);
+  DBUG_ASSERT(table != nullptr);
 
   bool error = update_referencing_views_metadata(
       thd, table->get_db_name(), table->get_table_name(), new_db,
@@ -800,7 +730,7 @@ bool update_referencing_views_metadata(
     THD *thd, const char *db_name, const char *table_name,
     bool commit_dd_changes, Uncommitted_tables_guard *uncommitted_tables) {
   DBUG_TRACE;
-  assert(db_name && table_name);
+  DBUG_ASSERT(db_name && table_name);
 
   bool error = update_referencing_views_metadata(
       thd, db_name, table_name, nullptr, nullptr, commit_dd_changes,
@@ -810,7 +740,7 @@ bool update_referencing_views_metadata(
 
 bool update_referencing_views_metadata(THD *thd, const sp_name *spname) {
   DBUG_TRACE;
-  assert(spname);
+  DBUG_ASSERT(spname);
 
   /*
     Updates to view metatdata for DDL on stored routines does not include
@@ -823,74 +753,6 @@ bool update_referencing_views_metadata(THD *thd, const sp_name *spname) {
   return error;
 }
 
-/**
-  Helper method to mark referencing views as invalid.
-
-  @tparam         T                   Type of object (View_table/View_routine)
-                                      to fetch referencing view names.
-  @param          thd                 Current thread.
-  @param          db                  Database name.
-  @param          tbl_or_sf_name      Base table/ View/ Stored function name.
-  @param          skip_same_db        Indicates whether it is OK to skip
-                                      views belonging to the same database
-                                      as table (as they will be dropped
-                                      anyway).
-  @param          commit_dd_changes   Indicates whether changes to DD need
-                                      to be committed.
-  @param          mem_root            Memory root for allocation of temporary
-                                      objects which will be cleared after
-                                      each call to this function.
-
-  @retval     false           Success.
-  @retval     true            Failure.
-
-*/
-
-template <typename T>
-static bool mark_referencing_views_invalid(THD *thd, const char *db,
-                                           const char *tbl_or_sf_name,
-                                           bool skip_same_db,
-                                           bool commit_dd_changes,
-                                           MEM_ROOT *mem_root) {
-  std::vector<TABLE_LIST *> views;
-  if (prepare_view_tables_list<T>(thd, db, tbl_or_sf_name, skip_same_db,
-                                  mem_root, &views))
-    return true;
-
-  if (views.empty()) return false;
-
-  DEBUG_SYNC(thd, "after_preparing_view_tables_list");
-
-  return mark_all_views_invalid<T>(thd, db, tbl_or_sf_name, &views,
-                                   skip_same_db, commit_dd_changes, mem_root);
-}
-
-bool mark_referencing_views_invalid(THD *thd, const TABLE_LIST *table,
-                                    bool skip_same_db, bool commit_dd_changes,
-                                    MEM_ROOT *mem_root) {
-  DBUG_TRACE;
-
-  const char *db_name = table->get_db_name();
-  const char *table_name = table->get_table_name();
-
-  // Don't mark referencing view invalid if we drop dictionary table.
-  if (dd::get_dictionary()->is_dd_table_name(db_name, table_name)) return false;
-
-  return mark_referencing_views_invalid<dd::View_table>(
-      thd, db_name, table_name, skip_same_db, commit_dd_changes, mem_root);
-}
-
-bool mark_referencing_views_invalid(THD *thd, const sp_name *spname,
-                                    MEM_ROOT *mem_root) {
-  /*
-    DROP DATABASE always drops routines atomically so we don't need to commit
-    transaction here. Also since DROP DATABASE drops routines views are dropped
-    there is no point in using skip-views-in-the-same-db optimization here.
-  */
-  return mark_referencing_views_invalid<dd::View_routine>(
-      thd, spname->m_db.str, spname->m_name.str, false, false, mem_root);
-}
-
 std::string push_view_warning_or_error(THD *thd, const char *db,
                                        const char *view_name) {
   // Report error for "SHOW FIELDS/DESCRIBE" operations.
