@@ -574,7 +574,7 @@ static const char *debug_sync_thd_proc_info(THD *thd, const char *info) {
 */
 
 int debug_sync_init(void) {
-  DBUG_ENTER("debug_sync_init");
+  DBUG_TRACE;
 
 #ifdef HAVE_PSI_INTERFACE
   init_debug_sync_psi_keys();
@@ -589,13 +589,13 @@ int debug_sync_init(void) {
         (rc =
              mysql_mutex_init(key_debug_sync_globals_ds_mutex,
                               &debug_sync_global.ds_mutex, MY_MUTEX_INIT_FAST)))
-      DBUG_RETURN(rc); /* purecov: inspected */
+      return rc; /* purecov: inspected */
 
     /* Set the call back pointer in C files. */
     debug_sync_C_callback_ptr = debug_sync_C_callback;
   }
 
-  DBUG_RETURN(0);
+  return 0;
 }
 
 /**
@@ -606,7 +606,7 @@ int debug_sync_init(void) {
 */
 
 void debug_sync_end(void) {
-  DBUG_ENTER("debug_sync_end");
+  DBUG_TRACE;
 
   /* End the facility only if it had been initialized. */
   if (debug_sync_C_callback_ptr) {
@@ -629,8 +629,6 @@ void debug_sync_end(void) {
              llstr(debug_sync_global.dsp_max_active, llbuff));
     }
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /* purecov: begin tested */
@@ -644,15 +642,13 @@ void debug_sync_end(void) {
 */
 
 static void debug_sync_emergency_disable(void) {
-  DBUG_ENTER("debug_sync_emergency_disable");
+  DBUG_TRACE;
 
   opt_debug_sync_timeout = 0;
 
   DBUG_PRINT("debug_sync",
              ("Debug Sync Facility disabled due to lack of memory."));
   LogErr(ERROR_LEVEL, ER_DEBUG_SYNC_OOM);
-
-  DBUG_VOID_RETURN;
 }
 
 /* purecov: end */
@@ -664,7 +660,7 @@ static void debug_sync_emergency_disable(void) {
 */
 
 void debug_sync_init_thread(THD *thd) {
-  DBUG_ENTER("debug_sync_init_thread");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
 
   if (opt_debug_sync_timeout) {
@@ -679,12 +675,10 @@ void debug_sync_init_thread(THD *thd) {
       debug_sync_emergency_disable(); /* purecov: tested */
     }
   }
-
-  DBUG_VOID_RETURN;
 }
 
 void debug_sync_claim_memory_ownership(THD *thd) {
-  DBUG_ENTER("debug_sync_claim_memory_ownership");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
 
   st_debug_sync_control *ds_control = thd->debug_sync_control;
@@ -703,8 +697,6 @@ void debug_sync_claim_memory_ownership(THD *thd) {
 
     my_claim(ds_control);
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -714,7 +706,7 @@ void debug_sync_claim_memory_ownership(THD *thd) {
 */
 
 void debug_sync_end_thread(THD *thd) {
-  DBUG_ENTER("debug_sync_end_thread");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
 
   if (thd->debug_sync_control) {
@@ -748,8 +740,6 @@ void debug_sync_end_thread(THD *thd) {
     my_free(ds_control);
     thd->debug_sync_control = NULL;
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -835,10 +825,10 @@ static void debug_sync_action_string(char *result, uint size,
 static void debug_sync_print_actions(THD *thd) {
   st_debug_sync_control *ds_control = thd->debug_sync_control;
   uint idx;
-  DBUG_ENTER("debug_sync_print_actions");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
 
-  if (!ds_control) DBUG_VOID_RETURN;
+  if (!ds_control) return;
 
   for (idx = 0; idx < ds_control->ds_active; idx++) {
     const char *dsp_name = ds_control->ds_action[idx].sync_point.c_ptr();
@@ -848,8 +838,6 @@ static void debug_sync_print_actions(THD *thd) {
                              ds_control->ds_action + idx);
     DBUG_PRINT("debug_sync_list", ("%s %s", dsp_name, action_string));
   }
-
-  DBUG_VOID_RETURN;
 }
 
 #endif /* !defined(DBUG_OFF) */
@@ -919,7 +907,7 @@ static st_debug_sync_action *debug_sync_find(st_debug_sync_action *actionarr,
 
 static void debug_sync_reset(THD *thd) {
   st_debug_sync_control *ds_control = thd->debug_sync_control;
-  DBUG_ENTER("debug_sync_reset");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
   DBUG_ASSERT(ds_control);
 
@@ -930,8 +918,6 @@ static void debug_sync_reset(THD *thd) {
   mysql_mutex_lock(&debug_sync_global.ds_mutex);
   debug_sync_global.ds_signal_set.clear();
   mysql_mutex_unlock(&debug_sync_global.ds_mutex);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -950,7 +936,7 @@ static void debug_sync_reset(THD *thd) {
 static void debug_sync_remove_action(st_debug_sync_control *ds_control,
                                      st_debug_sync_action *action) {
   uint dsp_idx = static_cast<uint>(action - ds_control->ds_action);
-  DBUG_ENTER("debug_sync_remove_action");
+  DBUG_TRACE;
   DBUG_ASSERT(ds_control);
   DBUG_ASSERT(ds_control == current_thd->debug_sync_control);
   DBUG_ASSERT(action);
@@ -985,8 +971,6 @@ static void debug_sync_remove_action(st_debug_sync_control *ds_control,
     dest_action = ds_control->ds_action + ds_control->ds_active;
     *dest_action = std::move(save_action);
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -1009,7 +993,7 @@ static st_debug_sync_action *debug_sync_get_action(THD *thd,
                                                    size_t name_len) {
   st_debug_sync_control *ds_control = thd->debug_sync_control;
   st_debug_sync_action *action;
-  DBUG_ENTER("debug_sync_get_action");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
   DBUG_ASSERT(dsp_name);
   DBUG_ASSERT(name_len);
@@ -1042,7 +1026,7 @@ static st_debug_sync_action *debug_sync_get_action(THD *thd,
                     new_alloc * sizeof(st_debug_sync_action), MYF(MY_WME));
       if (!new_action) {
         /* Error is reported by my_malloc(). */
-        DBUG_RETURN(nullptr); /* purecov: tested */
+        return nullptr; /* purecov: tested */
       }
       // Move objects into newly allocated memory.
       // TODO: use std::uninitialized_move in C++17
@@ -1068,7 +1052,7 @@ static st_debug_sync_action *debug_sync_get_action(THD *thd,
     action = ds_control->ds_action + dsp_idx;
     if (action->sync_point.copy(dsp_name, name_len, system_charset_info)) {
       /* Error is reported by my_malloc(). */
-      DBUG_RETURN(nullptr); /* purecov: tested */
+      return nullptr; /* purecov: tested */
     }
     action->need_sort = true;
   }
@@ -1077,7 +1061,7 @@ static st_debug_sync_action *debug_sync_get_action(THD *thd,
   DBUG_PRINT("debug_sync", ("action: %p  array: %p  count: %u", action,
                             ds_control->ds_action, ds_control->ds_active));
 
-  DBUG_RETURN(action);
+  return action;
 }
 
 /**
@@ -1117,7 +1101,7 @@ static st_debug_sync_action *debug_sync_get_action(THD *thd,
 static bool debug_sync_set_action(THD *thd, st_debug_sync_action *action) {
   st_debug_sync_control *ds_control = thd->debug_sync_control;
   bool is_dsp_now = false;
-  DBUG_ENTER("debug_sync_set_action");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
   DBUG_ASSERT(action);
   DBUG_ASSERT(ds_control);
@@ -1176,10 +1160,10 @@ static bool debug_sync_set_action(THD *thd, st_debug_sync_action *action) {
       and shall not be reported as a result of SET DEBUG_SYNC.
       Hence, we check for the first condition above.
     */
-    if (thd->is_error()) DBUG_RETURN(true);
+    if (thd->is_error()) return true;
   }
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /*
@@ -1394,7 +1378,7 @@ static bool debug_sync_eval_action(THD *thd, char *action_str) {
   char *ptr;
   char *token;
   size_t token_length = 0;
-  DBUG_ENTER("debug_sync_eval_action");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
   DBUG_ASSERT(action_str);
 
@@ -1414,7 +1398,7 @@ static bool debug_sync_eval_action(THD *thd, char *action_str) {
     action = debug_sync_get_action(thd, token, token_length);
     if (!action) {
       /* Error message is sent. */
-      DBUG_RETURN(true); /* purecov: tested */
+      return true; /* purecov: tested */
     }
   }
 
@@ -1614,13 +1598,13 @@ err:
     my_printf_error(ER_PARSE_ERROR, errmsg, MYF(0), token_length, token);
   }
   if (action) debug_sync_remove_action(thd->debug_sync_control, action);
-  DBUG_RETURN(true);
+  return true;
 
 set_action:
-  DBUG_RETURN(debug_sync_set_action(thd, action));
+  return debug_sync_set_action(thd, action);
 
 end:
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -1644,15 +1628,14 @@ end:
 */
 
 bool debug_sync_update(THD *thd, char *val_str) {
-  DBUG_ENTER("debug_sync_update");
+  DBUG_TRACE;
   DBUG_PRINT("debug_sync", ("set action: '%s'", val_str));
 
   /*
     debug_sync_eval_action() places '\0' in the string, which itself
     must be '\0' terminated.
   */
-  DBUG_RETURN(opt_debug_sync_timeout ? debug_sync_eval_action(thd, val_str)
-                                     : false);
+  return opt_debug_sync_timeout ? debug_sync_eval_action(thd, val_str) : false;
 }
 
 /**
@@ -1673,7 +1656,7 @@ bool debug_sync_update(THD *thd, char *val_str) {
 
 uchar *debug_sync_value_ptr(THD *thd) {
   char *value;
-  DBUG_ENTER("debug_sync_value_ptr");
+  DBUG_TRACE;
 
   if (opt_debug_sync_timeout) {
     std::string signals_on("ON - signals: '");
@@ -1703,7 +1686,7 @@ uchar *debug_sync_value_ptr(THD *thd) {
     /* purecov: end */
   }
 
-  DBUG_RETURN((uchar *)value);
+  return (uchar *)value;
 }
 
 /**
@@ -1772,7 +1755,7 @@ static void debug_sync_execute(THD *thd, st_debug_sync_action *action) {
   const char *sig_emit = action->signal.c_ptr();
   const char *sig_wait = action->wait_for.c_ptr();
 #endif
-  DBUG_ENTER("debug_sync_execute");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
   DBUG_ASSERT(action);
   DBUG_PRINT("debug_sync",
@@ -1921,8 +1904,6 @@ static void debug_sync_execute(THD *thd, st_debug_sync_action *action) {
     DBUG_PRINT("debug_sync_exec",
                ("hit_limit: %lu  at: '%s'", action->hit_limit, dsp_name));
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -1940,7 +1921,7 @@ void debug_sync(THD *thd, const char *sync_point_name, size_t name_len) {
 
   st_debug_sync_control *ds_control = thd->debug_sync_control;
   st_debug_sync_action *action;
-  DBUG_ENTER("debug_sync");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
   DBUG_ASSERT(sync_point_name);
   DBUG_ASSERT(name_len);
@@ -1963,8 +1944,6 @@ void debug_sync(THD *thd, const char *sync_point_name, size_t name_len) {
     /* If action became inactive, remove it to shrink the search array. */
     if (!action->activation_count) debug_sync_remove_action(ds_control, action);
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -1994,13 +1973,13 @@ void debug_sync(THD *thd, const char *sync_point_name, size_t name_len) {
 bool debug_sync_set_action(THD *thd, const char *action_str, size_t len) {
   bool rc;
   char *value;
-  DBUG_ENTER("debug_sync_set_action");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
   DBUG_ASSERT(action_str);
 
   value = strmake_root(thd->mem_root, action_str, len);
   rc = debug_sync_eval_action(thd, value);
-  DBUG_RETURN(rc);
+  return rc;
 }
 
 #endif /* defined(ENABLED_DEBUG_SYNC) */
