@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -95,6 +95,7 @@ const char *MY_LOCALE_ERRMSGS::lookup(int mysql_errno) {
   return "Invalid error code";
 }
 
+#ifndef CHECK_ERRMSG_FORMAT
 const char *ER_DEFAULT(int mysql_errno) {
   return my_default_lc_messages->errmsgs->lookup(mysql_errno);
 }
@@ -102,6 +103,14 @@ const char *ER_DEFAULT(int mysql_errno) {
 const char *ER_THD(const THD *thd, int mysql_errno) {
   return thd->variables.lc_messages->errmsgs->lookup(mysql_errno);
 }
+#endif  // CHECK_ERRMSG_FORMAT
+const char *ER_DEFAULT_NONCONST(int mysql_errno) {
+  return my_default_lc_messages->errmsgs->lookup(mysql_errno);
+}
+
+const char *ER_THD_NONCONST(const THD *thd, int mysql_errno) {
+  return thd->variables.lc_messages->errmsgs->lookup(mysql_errno);
+}
 
 /**
   Get the error-message corresponding to the given MySQL error-code,
@@ -118,7 +127,7 @@ const char *ER_THD(const THD *thd, int mysql_errno) {
 static const char *error_message_fetch(int mysql_errno) {
   if ((my_default_lc_messages != nullptr) &&
       (my_default_lc_messages->errmsgs->is_loaded()))
-    return ER_DEFAULT(mysql_errno);
+    return ER_DEFAULT_NONCONST(mysql_errno);
 
   {
     server_error *sqlstate_map = &error_names_array[1];
@@ -166,20 +175,20 @@ const char *error_message_for_error_log(int mysql_errno) {
   @retval  an error-message if available, or nullptr
 */
 const char *error_message_for_client(int mysql_errno) {
-  if (current_thd) return ER_THD(current_thd, mysql_errno);
+  if (current_thd) return ER_THD_NONCONST(current_thd, mysql_errno);
 
   return error_message_fetch(mysql_errno);
 }
 C_MODE_END
 
 bool init_errmessage() {
-  DBUG_ENTER("init_errmessage");
+  DBUG_TRACE;
 
   /* Read messages from file. */
   (void)my_default_lc_messages->errmsgs->read_texts();
 
   if (!my_default_lc_messages->errmsgs->is_loaded())
-    DBUG_RETURN(true); /* Fatal error, not able to allocate memory. */
+    return true; /* Fatal error, not able to allocate memory. */
 
   /* Register messages for use with my_error(). */
   for (int i = 0; i < NUM_SECTIONS; i++) {
@@ -187,11 +196,11 @@ bool init_errmessage() {
             error_message_for_client, errmsg_section_start[i],
             errmsg_section_start[i] + errmsg_section_size[i] - 1)) {
       my_default_lc_messages->errmsgs->destroy();
-      DBUG_RETURN(true);
+      return true;
     }
   }
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 void deinit_errmessage() {
@@ -222,7 +231,7 @@ bool MY_LOCALE_ERRMSGS::read_texts() {
   uchar head[32];
   uint error_messages = 0;
 
-  DBUG_ENTER("read_texts");
+  DBUG_TRACE;
 
   for (int i = 0; i < NUM_SECTIONS; i++)
     error_messages += errmsg_section_size[i];
@@ -273,7 +282,7 @@ bool MY_LOCALE_ERRMSGS::read_texts() {
             MYF(0)))) {
     LogErr(ERROR_LEVEL, ER_ERRMSG_OOM, name);
     (void)mysql_file_close(file, MYF(MY_WME));
-    DBUG_RETURN(true);
+    return true;
   }
 
   // Get pointer to Section2.
@@ -300,7 +309,7 @@ bool MY_LOCALE_ERRMSGS::read_texts() {
 
   (void)mysql_file_close(file, MYF(0));
 
-  DBUG_RETURN(false);
+  return false;
 
 read_err_init:
   /*
@@ -335,7 +344,7 @@ open_err:
     }
   }
 
-  DBUG_RETURN(true);
+  return true;
 } /* read_texts */
 
 void MY_LOCALE_ERRMSGS::destroy() {
