@@ -24,6 +24,7 @@
 
 #include <stdio.h>
 #include <atomic>
+#include <memory>
 
 #include "my_compiler.h"
 #include "my_dbug.h"
@@ -112,7 +113,7 @@ Event_queue::~Event_queue() {
 */
 
 bool Event_queue::init_queue() {
-  DBUG_ENTER("Event_queue::init_queue");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("this: %p", this));
 
   LOCK_QUEUE_DATA();
@@ -123,11 +124,11 @@ bool Event_queue::init_queue() {
   }
 
   UNLOCK_QUEUE_DATA();
-  DBUG_RETURN(false);
+  return false;
 
 err:
   UNLOCK_QUEUE_DATA();
-  DBUG_RETURN(true);
+  return true;
 }
 
 /*
@@ -139,13 +140,11 @@ err:
 */
 
 void Event_queue::deinit_queue() {
-  DBUG_ENTER("Event_queue::deinit_queue");
+  DBUG_TRACE;
 
   LOCK_QUEUE_DATA();
   empty_queue();
   UNLOCK_QUEUE_DATA();
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -168,7 +167,7 @@ void Event_queue::deinit_queue() {
 
 bool Event_queue::create_event(THD *thd, Event_queue_element *new_element,
                                bool *created) {
-  DBUG_ENTER("Event_queue::create_event");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("thd: %p et=%s.%s", thd, new_element->m_schema_name.str,
                        new_element->m_event_name.str));
 
@@ -176,7 +175,7 @@ bool Event_queue::create_event(THD *thd, Event_queue_element *new_element,
   new_element->compute_next_execution_time(thd);
   if (new_element->m_status != Event_parse_data::ENABLED) {
     *created = false;
-    DBUG_RETURN(false);
+    return false;
   }
 
   DBUG_PRINT("info", ("new event in the queue: %p", new_element));
@@ -187,7 +186,7 @@ bool Event_queue::create_event(THD *thd, Event_queue_element *new_element,
   mysql_cond_broadcast(&COND_queue_state);
   UNLOCK_QUEUE_DATA();
 
-  DBUG_RETURN(!*created);
+  return !*created;
 }
 
 /*
@@ -202,9 +201,9 @@ bool Event_queue::create_event(THD *thd, Event_queue_element *new_element,
       new_name   New name, in case of RENAME TO, otherwise NULL
 */
 
-void Event_queue::update_event(THD *thd, LEX_STRING dbname, LEX_STRING name,
+void Event_queue::update_event(THD *thd, LEX_CSTRING dbname, LEX_CSTRING name,
                                Event_queue_element *new_element) {
-  DBUG_ENTER("Event_queue::update_event");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("thd: %p  et=[%s.%s]", thd, dbname.str, name.str));
 
   if ((new_element->m_status == Event_parse_data::DISABLED) ||
@@ -231,8 +230,6 @@ void Event_queue::update_event(THD *thd, LEX_STRING dbname, LEX_STRING name,
 
   dbug_dump_queue(thd->query_start_in_secs());
   UNLOCK_QUEUE_DATA();
-
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -245,8 +242,8 @@ void Event_queue::update_event(THD *thd, LEX_STRING dbname, LEX_STRING name,
       name    Name of the event to drop
 */
 
-void Event_queue::drop_event(THD *thd, LEX_STRING dbname, LEX_STRING name) {
-  DBUG_ENTER("Event_queue::drop_event");
+void Event_queue::drop_event(THD *thd, LEX_CSTRING dbname, LEX_CSTRING name) {
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("thd: %p  db :%s  name: %s", thd, dbname.str, name.str));
 
   LOCK_QUEUE_DATA();
@@ -258,8 +255,6 @@ void Event_queue::drop_event(THD *thd, LEX_STRING dbname, LEX_STRING name) {
     We don't signal here because the scheduler will catch the change
     next time it wakes up.
   */
-
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -278,11 +273,11 @@ void Event_queue::drop_event(THD *thd, LEX_STRING dbname, LEX_STRING name) {
     Expected is the caller to acquire lock on LOCK_event_queue
 */
 
-void Event_queue::drop_matching_events(LEX_STRING pattern,
-                                       bool (*comparator)(LEX_STRING,
+void Event_queue::drop_matching_events(LEX_CSTRING pattern,
+                                       bool (*comparator)(LEX_CSTRING,
                                                           Event_basic *)) {
   size_t i = 0;
-  DBUG_ENTER("Event_queue::drop_matching_events");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("pattern=%s", pattern.str));
 
   while (i < queue.size()) {
@@ -318,8 +313,6 @@ void Event_queue::drop_matching_events(LEX_STRING pattern,
        we may not notify the scheduler and it will realize the change when it
        wakes up from timedwait.
   */
-
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -331,12 +324,11 @@ void Event_queue::drop_matching_events(LEX_STRING pattern,
       schema    The schema name
 */
 
-void Event_queue::drop_schema_events(LEX_STRING schema) {
-  DBUG_ENTER("Event_queue::drop_schema_events");
+void Event_queue::drop_schema_events(LEX_CSTRING schema) {
+  DBUG_TRACE;
   LOCK_QUEUE_DATA();
   drop_matching_events(schema, event_basic_db_equal);
   UNLOCK_QUEUE_DATA();
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -352,8 +344,8 @@ void Event_queue::drop_schema_events(LEX_STRING schema) {
     actual signalling in case an event is removed from the queue.
 */
 
-void Event_queue::find_n_remove_event(LEX_STRING db, LEX_STRING name) {
-  DBUG_ENTER("Event_queue::find_n_remove_event");
+void Event_queue::find_n_remove_event(LEX_CSTRING db, LEX_CSTRING name) {
+  DBUG_TRACE;
 
   for (size_t i = 0; i < queue.size(); ++i) {
     Event_queue_element *et = queue[i];
@@ -365,8 +357,6 @@ void Event_queue::find_n_remove_event(LEX_STRING db, LEX_STRING name) {
       break;
     }
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -383,7 +373,7 @@ void Event_queue::find_n_remove_event(LEX_STRING db, LEX_STRING name) {
 */
 
 void Event_queue::recalculate_activation_times(THD *thd) {
-  DBUG_ENTER("Event_queue::recalculate_activation_times");
+  DBUG_TRACE;
 
   LOCK_QUEUE_DATA();
   DBUG_PRINT("info", ("%u loaded events to be recalculated",
@@ -408,12 +398,10 @@ void Event_queue::recalculate_activation_times(THD *thd) {
   */
   dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   Disable_gtid_state_update_guard disabler(thd);
+  std::vector<std::unique_ptr<Event_queue_element>> events_to_drop;
   for (size_t i = queue.size(); i > 0; i--) {
     Event_queue_element *element = queue[i - 1];
     if (element->m_status != Event_parse_data::DISABLED) break;
-    if (lock_object_name(thd, MDL_key::EVENT, element->m_schema_name.str,
-                         element->m_event_name.str))
-      break;
     /*
       This won't cause queue re-order, because we remove
       always the last element.
@@ -422,40 +410,48 @@ void Event_queue::recalculate_activation_times(THD *thd) {
     /*
       Dropping the event from Data Dictionary.
     */
-    if (element->m_dropped) {
-      bool ret;
-      bool event_exists;
-      if (!(ret = Event_db_repository::drop_event(thd, element->m_schema_name,
-                                                  element->m_event_name, false,
-                                                  &event_exists))) {
-        String sp_sql;
-        if ((ret =
-                 construct_drop_event_sql(thd, &sp_sql, element->m_schema_name,
+    if (element->m_dropped)
+      events_to_drop.push_back(std::unique_ptr<Event_queue_element>(element));
+    else
+      delete element;
+  }
+  UNLOCK_QUEUE_DATA();
+
+  for (const auto &element : events_to_drop) {
+    bool ret;
+    bool event_exists;
+
+    if (lock_object_name(thd, MDL_key::EVENT, element->m_schema_name.str,
+                         element->m_event_name.str)) {
+      continue;
+    }
+    if (!(ret = Event_db_repository::drop_event(thd, element->m_schema_name,
+                                                element->m_event_name, false,
+                                                &event_exists))) {
+      String sp_sql;
+      if ((ret = construct_drop_event_sql(thd, &sp_sql, element->m_schema_name,
                                           element->m_event_name))) {
-          LogErr(WARNING_LEVEL, ER_FAILED_TO_CONSTRUCT_DROP_EVENT_QUERY);
-        } else {
-          // Write drop event to bin log.
-          thd->add_to_binlog_accessed_dbs(element->m_schema_name.str);
-          if ((ret = write_bin_log(thd, true, sp_sql.c_ptr_safe(),
-                                   sp_sql.length(), event_exists))) {
-            LogErr(WARNING_LEVEL, ER_FAILED_TO_BINLOG_DROP_EVENT,
-                   element->m_schema_name.str, element->m_event_name.str);
-          }
+        LogErr(WARNING_LEVEL, ER_FAILED_TO_CONSTRUCT_DROP_EVENT_QUERY);
+      } else {
+        // Write drop event to bin log.
+        thd->add_to_binlog_accessed_dbs(element->m_schema_name.str);
+        if ((ret = write_bin_log(thd, true, sp_sql.c_ptr_safe(),
+                                 sp_sql.length(), event_exists))) {
+          LogErr(WARNING_LEVEL, ER_FAILED_TO_BINLOG_DROP_EVENT,
+                 element->m_schema_name.str, element->m_event_name.str);
         }
       }
-
-      if (!ret)
-        ret = trans_commit_stmt(thd) || trans_commit(thd);
-      else {
-        trans_rollback_stmt(thd);
-        trans_rollback(thd);
-      }
     }
-    delete element;
+
+    if (!ret)
+      ret = trans_commit_stmt(thd) || trans_commit(thd);
+    else {
+      trans_rollback_stmt(thd);
+      trans_rollback(thd);
+    }
   }
   // Release locks taken before drop_event()
   thd->mdl_context.release_transactional_locks();
-  UNLOCK_QUEUE_DATA();
 
   /*
     XXX: The events are dropped only from memory and not from disk
@@ -463,8 +459,6 @@ void Event_queue::recalculate_activation_times(THD *thd) {
          disk till next server restart.
          Please add code here to do it.
   */
-
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -479,15 +473,13 @@ void Event_queue::recalculate_activation_times(THD *thd) {
 */
 
 void Event_queue::empty_queue() {
-  DBUG_ENTER("Event_queue::empty_queue");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("Purging the queue. %u element(s)",
                        static_cast<unsigned>(queue.size())));
   LogErr(INFORMATION_LEVEL, ER_EVENT_PURGING_QUEUE,
          static_cast<unsigned>(queue.size()));
   /* empty the queue */
   queue.delete_elements();
-
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -500,7 +492,7 @@ void Event_queue::empty_queue() {
 
 void Event_queue::dbug_dump_queue(time_t now MY_ATTRIBUTE((unused))) {
 #ifndef DBUG_OFF
-  DBUG_ENTER("Event_queue::dbug_dump_queue");
+  DBUG_TRACE;
   DBUG_PRINT("info", ("Dumping queue . Elements=%u",
                       static_cast<unsigned>(queue.size())));
   for (size_t i = 0; i < queue.size(); i++) {
@@ -516,7 +508,7 @@ void Event_queue::dbug_dump_queue(time_t now MY_ATTRIBUTE((unused))) {
          et->m_execution_count, (long)et->m_expression, (long)et->m_execute_at,
          (long)now, (int)(et->m_execute_at - now), et->m_execute_at <= now));
   }
-  DBUG_VOID_RETURN;
+  return;
 #endif
 }
 
@@ -541,7 +533,7 @@ bool Event_queue::get_top_for_execution_if_time(
   *event_name = nullptr;
   my_time_t last_executed = 0;
   int status = 0;
-  DBUG_ENTER("Event_queue::get_top_for_execution_if_time");
+  DBUG_TRACE;
 
   LOCK_QUEUE_DATA();
   for (;;) {
@@ -640,7 +632,7 @@ end:
     // Acquire exclusive MDL lock on the event and it's parent schema.
     if (lock_object_name(thd, MDL_key::EVENT, (*event_name)->dbname.str,
                          (*event_name)->name.str))
-      DBUG_RETURN(true);
+      return true;
 
     (void)Event_db_repository::update_timing_fields_for_event(
         thd, (*event_name)->dbname, (*event_name)->name, last_executed,
@@ -649,7 +641,7 @@ end:
     thd->mdl_context.release_transactional_locks();
   }
 
-  DBUG_RETURN(ret);
+  return ret;
 }
 
 /*
@@ -663,7 +655,7 @@ end:
 */
 
 void Event_queue::lock_data(const char *func, uint line) {
-  DBUG_ENTER("Event_queue::lock_data");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("func=%s line=%u", func, line));
   mutex_last_attempted_lock_in_func = func;
   mutex_last_attempted_lock_at_line = line;
@@ -676,8 +668,6 @@ void Event_queue::lock_data(const char *func, uint line) {
   mutex_last_locked_in_func = func;
   mutex_last_locked_at_line = line;
   mutex_queue_data_locked = true;
-
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -691,13 +681,12 @@ void Event_queue::lock_data(const char *func, uint line) {
 */
 
 void Event_queue::unlock_data(const char *func, uint line) {
-  DBUG_ENTER("Event_queue::unlock_data");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("func=%s line=%u", func, line));
   mutex_last_unlocked_at_line = line;
   mutex_queue_data_locked = false;
   mutex_last_unlocked_in_func = func;
   mysql_mutex_unlock(&LOCK_event_queue);
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -715,7 +704,7 @@ void Event_queue::unlock_data(const char *func, uint line) {
 void Event_queue::cond_wait(THD *thd, struct timespec *abstime,
                             const PSI_stage_info *stage, const char *src_func,
                             const char *src_file, uint src_line) {
-  DBUG_ENTER("Event_queue::cond_wait");
+  DBUG_TRACE;
   waiting_on_cond = true;
   mutex_last_unlocked_at_line = src_line;
   mutex_queue_data_locked = false;
@@ -743,8 +732,6 @@ void Event_queue::cond_wait(THD *thd, struct timespec *abstime,
   unlock_data(src_func, src_line);
   thd->exit_cond(NULL, src_func, src_file, src_line);
   lock_data(src_func, src_line);
-
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -755,7 +742,7 @@ void Event_queue::cond_wait(THD *thd, struct timespec *abstime,
 */
 
 void Event_queue::dump_internal_status() {
-  DBUG_ENTER("Event_queue::dump_internal_status");
+  DBUG_TRACE;
 
   /* element count */
   puts("");
@@ -781,8 +768,6 @@ void Event_queue::dump_internal_status() {
            time.month, time.day, time.hour, time.minute, time.second);
   else
     printf("Next activation : never");
-
-  DBUG_VOID_RETURN;
 }
 
 /**
