@@ -1,16 +1,15 @@
-/* Copyright (c) 2006, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +24,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <time.h>
 
 #include "lex_string.h"
 #include "m_ctype.h"
@@ -36,8 +34,8 @@
 #include "my_psi_config.h"
 #include "my_sys.h"
 #include "my_thread.h"
-#include "mysql/components/services/bits/psi_statement_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/components/services/psi_statement_bits.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/psi/mysql_statement.h"
@@ -57,7 +55,7 @@
 #include "sql/events.h"
 #include "sql/log.h"
 #include "sql/mdl.h"
-#include "sql/mysqld.h"              // my_localhost replica_net_timeout
+#include "sql/mysqld.h"              // my_localhost slave_net_timeout
 #include "sql/mysqld_thd_manager.h"  // Global_THD_manager
 #include "sql/protocol_classic.h"
 #include "sql/psi_memory_key.h"
@@ -145,7 +143,7 @@ void Event_worker_thread::print_warnings(THD *thd, Event_job_data *et) {
         break;
       default:
         ll = ERROR_LEVEL;
-        assert(false);
+        DBUG_ASSERT(false);
     }
     LogErr(ll, ER_EVENT_MESSAGE_STACK, static_cast<int>(err_msg.length()),
            err_msg.c_ptr());
@@ -187,7 +185,7 @@ bool post_init_event_thread(THD *thd) {
 void deinit_event_thread(THD *thd) {
   Global_THD_manager *thd_manager = Global_THD_manager::get_instance();
 
-  thd->set_proc_info("Clearing");
+  thd->proc_info = "Clearing";
   thd->get_protocol_classic()->end_net();
   DBUG_PRINT("exit", ("Event thread finishing"));
   thd->release_resources();
@@ -216,21 +214,20 @@ void pre_init_event_thread(THD *thd) {
   thd->security_context()->cache_current_db_access(0);
   thd->security_context()->set_host_or_ip_ptr(my_localhost,
                                               strlen(my_localhost));
-  thd->get_protocol_classic()->init_net(nullptr);
+  thd->get_protocol_classic()->init_net(NULL);
   thd->security_context()->set_user_ptr(STRING_WITH_LEN("event_scheduler"));
-  thd->get_protocol_classic()->get_net()->read_timeout = replica_net_timeout;
-  thd->slave_thread = false;
+  thd->get_protocol_classic()->get_net()->read_timeout = slave_net_timeout;
+  thd->slave_thread = 0;
   thd->variables.option_bits |= OPTION_AUTO_IS_NULL;
   thd->get_protocol_classic()->set_client_capabilities(CLIENT_MULTI_RESULTS);
 
   thd->set_new_thread_id();
-
   /*
     Guarantees that we will see the thread in SHOW PROCESSLIST though its
     vio is NULL.
   */
 
-  thd->set_proc_info("Initialized");
+  thd->proc_info = "Initialized";
   thd->set_time();
 
   /* Do not use user-supplied timeout value for system threads. */
@@ -259,36 +256,24 @@ static void *event_scheduler_thread(void *arg) {
 
   mysql_thread_set_psi_id(thd->thread_id());
 
-#ifdef HAVE_PSI_THREAD_INTERFACE
-  /* Update the thread instrumentation. */
-  PSI_thread *psi = PSI_THREAD_CALL(get_thread)();
-  thd_set_psi(thd, psi);
-  PSI_THREAD_CALL(set_thread_account)
-  (thd->security_context()->user().str, thd->security_context()->user().length,
-   thd->security_context()->host_or_ip().str,
-   thd->security_context()->host_or_ip().length);
-  PSI_THREAD_CALL(set_thread_command)(thd->get_command());
-  PSI_THREAD_CALL(set_thread_start_time)(thd->query_start_in_secs());
-#endif /* HAVE_PSI_THREAD_INTERFACE */
-
   res = post_init_event_thread(thd);
 
   {
     DBUG_TRACE;
-    my_claim(arg, true);
-    thd->claim_memory_ownership(true);
+    my_claim(arg);
+    thd->claim_memory_ownership();
     my_free(arg);
     if (!res)
       scheduler->run(thd);
     else {
-      thd->set_proc_info("Clearing");
+      thd->proc_info = "Clearing";
       thd->get_protocol_classic()->end_net();
       delete thd;
     }
 
   }  // Against gcc warnings
   my_thread_end();
-  return nullptr;
+  return 0;
 }
 
 /**
@@ -307,31 +292,19 @@ static void *event_worker_thread(void *arg) {
   THD *thd;
   Event_queue_element_for_exec *event = (Event_queue_element_for_exec *)arg;
 
-  event->claim_memory_ownership(true);
+  event->claim_memory_ownership();
 
   thd = event->thd;
 
-  thd->claim_memory_ownership(true);
+  thd->claim_memory_ownership();
 
   mysql_thread_set_psi_id(thd->thread_id());
 
-#ifdef HAVE_PSI_THREAD_INTERFACE
-  /* Update the thread instrumentation. */
-  PSI_thread *psi = PSI_THREAD_CALL(get_thread)();
-  thd_set_psi(thd, psi);
-  PSI_THREAD_CALL(set_thread_account)
-  (thd->security_context()->user().str, thd->security_context()->user().length,
-   thd->security_context()->host_or_ip().str,
-   thd->security_context()->host_or_ip().length);
-  PSI_THREAD_CALL(set_thread_command)(thd->get_command());
-  PSI_THREAD_CALL(set_thread_start_time)(thd->query_start_in_secs());
-#endif /* HAVE_PSI_THREAD_INTERFACE */
-
   Event_worker_thread worker_thread;
   worker_thread.run(thd, event);
 
   my_thread_end();
-  return nullptr;  // Can't return anything here
+  return 0;  // Can't return anything here
 }
 }  // extern "C"
 
@@ -351,13 +324,13 @@ void Event_worker_thread::run(THD *thd, Event_queue_element_for_exec *event) {
   Event_job_data job_data;
   bool res;
 
-  assert(thd->m_digest == nullptr);
+  DBUG_ASSERT(thd->m_digest == NULL);
 
   thd->thread_stack = &my_stack;  // remember where our stack is
   res = post_init_event_thread(thd);
 
   DBUG_TRACE;
-  DBUG_PRINT("info", ("Time is %ld, THD: %p", (long)time(nullptr), thd));
+  DBUG_PRINT("info", ("Time is %ld, THD: %p", (long)my_time(0), thd));
 
   if (res) {
     delete event;
@@ -365,19 +338,12 @@ void Event_worker_thread::run(THD *thd, Event_queue_element_for_exec *event) {
     return;
   }
 
-  mysql_thread_set_secondary_engine(false);
-
 #ifdef HAVE_PSI_STATEMENT_INTERFACE
-  PSI_thread *thread = thd_get_psi(thd);
-  if (thread != nullptr) {
-    PSI_THREAD_CALL(detect_telemetry)(thread);
-  }
-
   PSI_statement_locker_state state;
-  assert(thd->m_statement_psi == nullptr);
+  DBUG_ASSERT(thd->m_statement_psi == NULL);
   thd->m_statement_psi = MYSQL_START_STATEMENT(
       &state, event->get_psi_info()->m_key, event->dbname.str,
-      event->dbname.length, thd->charset(), nullptr);
+      event->dbname.length, thd->charset(), NULL);
 #endif
   /*
     We must make sure the schema is released and unlocked in the right
@@ -423,10 +389,10 @@ void Event_worker_thread::run(THD *thd, Event_queue_element_for_exec *event) {
 end:
 #ifdef HAVE_PSI_STATEMENT_INTERFACE
   MYSQL_END_STATEMENT(thd->m_statement_psi, thd->get_stmt_da());
-  thd->m_statement_psi = nullptr;
+  thd->m_statement_psi = NULL;
 #endif
 
-  assert(thd->m_digest == nullptr);
+  DBUG_ASSERT(thd->m_digest == NULL);
 
   DBUG_PRINT("info",
              ("Done with Event %s.%s", event->dbname.str, event->name.str));
@@ -437,7 +403,7 @@ end:
 
 Event_scheduler::Event_scheduler(Event_queue *queue_arg)
     : state(INITIALIZED),
-      scheduler_thd(nullptr),
+      scheduler_thd(NULL),
       queue(queue_arg),
       mutex_last_locked_at_line(0),
       mutex_last_unlocked_at_line(0),
@@ -467,12 +433,13 @@ Event_scheduler::~Event_scheduler() {
   @param[out] err_no - errno indicating type of error which caused
                        failure to start scheduler thread.
 
-  @retval false Success.
-  @retval true  Error.
+  @return
+    @retval false Success.
+    @retval true  Error.
 */
 
 bool Event_scheduler::start(int *err_no) {
-  THD *new_thd = nullptr;
+  THD *new_thd = NULL;
   bool ret = false;
   my_thread_handle th;
   struct scheduler_param *scheduler_param_value;
@@ -525,13 +492,6 @@ bool Event_scheduler::start(int *err_no) {
   DBUG_PRINT("info", ("Setting state go RUNNING"));
   state = RUNNING;
   DBUG_PRINT("info", ("Forking new thread for scheduler. THD: %p", new_thd));
-
-  /*
-    Transfer memory ownership to the child scheduler thread.
-  */
-  my_claim(scheduler_param_value, false);
-  new_thd->claim_memory_ownership(false);
-
   if ((*err_no = mysql_thread_create(key_thread_event_scheduler, &th,
                                      &connection_attrib, event_scheduler_thread,
                                      (void *)scheduler_param_value))) {
@@ -539,11 +499,11 @@ bool Event_scheduler::start(int *err_no) {
     LogErr(ERROR_LEVEL, ER_CANT_CREATE_SCHEDULER_THREAD, *err_no)
         .os_errno(*err_no);
 
-    new_thd->set_proc_info("Clearing");
+    new_thd->proc_info = "Clearing";
     new_thd->get_protocol_classic()->end_net();
 
     state = INITIALIZED;
-    scheduler_thd = nullptr;
+    scheduler_thd = NULL;
     delete new_thd;
 
     my_free(scheduler_param_value);
@@ -579,32 +539,11 @@ bool Event_scheduler::run(THD *thd) {
   queue->recalculate_activation_times(thd);
 
   while (is_running()) {
-    Event_queue_element_for_exec *event_name = nullptr;
+    Event_queue_element_for_exec *event_name;
 
     /* Gets a minimized version */
     if (queue->get_top_for_execution_if_time(thd, &event_name)) {
-      /* Report scheduling errors to the error log. */
-      if (thd->get_stmt_da()->cond_count() > 0) {
-        Diagnostics_area::Sql_condition_iterator it =
-            thd->get_stmt_da()->sql_conditions();
-        const Sql_condition *err;
-        while ((err = it++)) {
-          if (err->severity() == Sql_condition::SL_ERROR) {
-            char msg_buf[10 * STRING_BUFFER_USUAL_SIZE];
-            String err_msg(msg_buf, sizeof(msg_buf), system_charset_info);
-            err_msg.length(0);
-            err_msg.append("Event Scheduler: Unable to schedule event: ");
-            err_msg.append(err->message_text(), err->message_octet_length(),
-                           system_charset_info);
-
-            LogErr(ERROR_LEVEL, ER_EVENT_MESSAGE_STACK,
-                   static_cast<int>(err_msg.length()), err_msg.c_ptr());
-          }
-        }
-      }
-
       LogErr(INFORMATION_LEVEL, ER_SCHEDULER_STOPPING_FAILED_TO_GET_EVENT);
-      if (event_name != nullptr) delete event_name;
       break;
     }
 
@@ -614,16 +553,15 @@ bool Event_scheduler::run(THD *thd) {
     if (event_name) {
       if ((res = execute_top(event_name))) break;
     } else {
-      assert(thd->killed);
+      DBUG_ASSERT(thd->killed);
       DBUG_PRINT("info", ("job_data is NULL, the thread was killed"));
     }
     DBUG_PRINT("info", ("state=%s", scheduler_states_names[state].str));
-    thd->mem_root->Clear();
   }
 
   LOCK_DATA();
   deinit_event_thread(thd);
-  scheduler_thd = nullptr;
+  scheduler_thd = NULL;
   state = INITIALIZED;
   DBUG_PRINT("info", ("Broadcasting COND_state back to the stoppers"));
   mysql_cond_broadcast(&COND_state);
@@ -657,12 +595,6 @@ bool Event_scheduler::execute_top(Event_queue_element_for_exec *event_name) {
   DBUG_PRINT("info", ("Event %s@%s ready for start", event_name->dbname.str,
                       event_name->name.str));
 
-  /*
-    Transfer memory ownership to the child worker thread.
-  */
-  event_name->claim_memory_ownership(false);
-  new_thd->claim_memory_ownership(false);
-
   /*
     TODO: should use thread pool here, preferably with an upper limit
     on number of threads: if too many events are scheduled for the
@@ -681,7 +613,7 @@ bool Event_scheduler::execute_top(Event_queue_element_for_exec *event_name) {
 
     LogErr(ERROR_LEVEL, ER_SCHEDULER_STOPPING_FAILED_TO_CREATE_WORKER, res);
 
-    new_thd->set_proc_info("Clearing");
+    new_thd->proc_info = "Clearing";
     new_thd->get_protocol_classic()->end_net();
 
     goto error;
@@ -744,7 +676,7 @@ bool Event_scheduler::stop() {
   if (state != RUNNING) {
     /* Synchronously wait until the scheduler stops. */
     while (state != INITIALIZED)
-      COND_STATE_WAIT(thd, nullptr, &stage_waiting_for_scheduler_to_stop);
+      COND_STATE_WAIT(thd, NULL, &stage_waiting_for_scheduler_to_stop);
     goto end;
   }
 
@@ -776,7 +708,7 @@ bool Event_scheduler::stop() {
 
     /* thd could be 0x0, when shutting down */
     LogErr(INFORMATION_LEVEL, ER_SCHEDULER_WAITING);
-    COND_STATE_WAIT(thd, nullptr, &stage_waiting_for_scheduler_to_stop);
+    COND_STATE_WAIT(thd, NULL, &stage_waiting_for_scheduler_to_stop);
   } while (state == STOPPING);
   DBUG_PRINT("info", ("Scheduler thread has cleaned up. Set state to INIT"));
   LogErr(INFORMATION_LEVEL, ER_SCHEDULER_STOPPED);
@@ -794,7 +726,7 @@ end:
 class Is_worker : public Do_THD_Impl {
  public:
   Is_worker() : m_count(0) {}
-  void operator()(THD *thd) override {
+  virtual void operator()(THD *thd) {
     if (thd->system_thread == SYSTEM_THREAD_EVENT_WORKER) m_count++;
     return;
   }
@@ -881,8 +813,8 @@ void Event_scheduler::cond_wait(THD *thd, struct timespec *abstime,
   mutex_scheduler_data_locked = false;
   mutex_last_unlocked_in_func = src_func;
   if (thd)
-    thd->enter_cond(&COND_state, &LOCK_scheduler_state, stage, nullptr,
-                    src_func, src_file, src_line);
+    thd->enter_cond(&COND_state, &LOCK_scheduler_state, stage, NULL, src_func,
+                    src_file, src_line);
 
   DBUG_PRINT("info", ("mysql_cond_%swait", abstime ? "timed" : ""));
   if (!abstime)
@@ -895,7 +827,7 @@ void Event_scheduler::cond_wait(THD *thd, struct timespec *abstime,
       Not the best thing to do but we need to obey cond_wait()
     */
     UNLOCK_DATA();
-    thd->exit_cond(nullptr, src_func, src_file, src_line);
+    thd->exit_cond(NULL, src_func, src_file, src_line);
     LOCK_DATA();
   }
   mutex_last_locked_in_func = src_func;
