@@ -1,16 +1,15 @@
-/* Copyright (c) 2005, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,16 +38,16 @@
 #include "my_loglevel.h"
 #include "my_macros.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/components/services/bits/psi_memory_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
+#include "mysql/components/services/psi_memory_bits.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/psi/mysql_sp.h"
 #include "mysql/psi/mysql_stage.h"
 #include "mysql/psi/mysql_thread.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"  // ER_*
 #include "sql/auth/auth_acls.h"
@@ -148,14 +147,14 @@ static bool load_events_from_db(THD *thd, Event_queue *event_queue);
     1   s > t
 */
 
-int sortcmp_lex_string(LEX_CSTRING s, LEX_CSTRING t, const CHARSET_INFO *cs) {
+int sortcmp_lex_string(LEX_CSTRING s, LEX_CSTRING t, CHARSET_INFO *cs) {
   return cs->coll->strnncollsp(cs, pointer_cast<const uchar *>(s.str), s.length,
                                pointer_cast<const uchar *>(t.str), t.length);
 }
 
 /*
   Reconstructs interval expression from interval type and expression
-  value that is in form of a value of the smallest entity:
+  value that is in form of a value of the smalles entity:
   For
     YEAR_MONTH - expression is in months
     DAY_MINUTE - expression is in minutes
@@ -348,10 +347,9 @@ bool Events::create_event(THD *thd, Event_parse_data *parse_data,
   if (parse_data->check_parse_data(thd)) return true;
 
   /* At create, one of them must be set */
-  assert(parse_data->expression || parse_data->execute_at);
+  DBUG_ASSERT(parse_data->expression || parse_data->execute_at);
 
-  if (check_access(thd, EVENT_ACL, parse_data->dbname.str, nullptr, nullptr,
-                   false, false))
+  if (check_access(thd, EVENT_ACL, parse_data->dbname.str, NULL, NULL, 0, 0))
     return true;
 
   // Acquire exclusive MDL lock.
@@ -400,7 +398,7 @@ bool Events::create_event(THD *thd, Event_parse_data *parse_data,
 
   // Binlog the create event.
   {
-    assert(thd->query().str && thd->query().length);
+    DBUG_ASSERT(thd->query().str && thd->query().length);
     String log_query;
     if (create_query_string(thd, &log_query)) {
       LogErr(ERROR_LEVEL, ER_EVENT_ERROR_CREATING_QUERY_TO_WRITE_TO_BINLOG);
@@ -485,15 +483,14 @@ bool Events::update_event(THD *thd, Event_parse_data *parse_data,
   if (parse_data->check_parse_data(thd) || parse_data->do_not_create)
     return true;
 
-  if (check_access(thd, EVENT_ACL, parse_data->dbname.str, nullptr, nullptr,
-                   false, false))
+  if (check_access(thd, EVENT_ACL, parse_data->dbname.str, NULL, NULL, 0, 0))
     return true;
 
   if (lock_object_name(thd, MDL_key::EVENT, parse_data->dbname.str,
                        parse_data->name.str))
     return true;
 
-  if (new_dbname != nullptr) /* It's a rename */
+  if (new_dbname != NULL) /* It's a rename */
   {
     /* Check that the new and the old names differ. */
     if (!sortcmp_lex_string(parse_data->dbname, *new_dbname,
@@ -509,8 +506,7 @@ bool Events::update_event(THD *thd, Event_parse_data *parse_data,
       to tell the user that a database doesn't exist if they can not
       access it.
     */
-    if (check_access(thd, EVENT_ACL, new_dbname->str, nullptr, nullptr, false,
-                     false))
+    if (check_access(thd, EVENT_ACL, new_dbname->str, NULL, NULL, 0, 0))
       return true;
 
     //  Acquire mdl exclusive lock on target database name.
@@ -550,7 +546,7 @@ bool Events::update_event(THD *thd, Event_parse_data *parse_data,
 
   /* Binlog the alter event. */
   {
-    assert(thd->query().str && thd->query().length);
+    DBUG_ASSERT(thd->query().str && thd->query().length);
 
     thd->add_to_binlog_accessed_dbs(parse_data->dbname.str);
     if (new_dbname) thd->add_to_binlog_accessed_dbs(new_dbname->str);
@@ -564,13 +560,6 @@ bool Events::update_event(THD *thd, Event_parse_data *parse_data,
       trans_commit_stmt(thd) || trans_commit(thd))
     goto err_with_rollback;
 
-  if (new_dbname != nullptr) {
-    /* RENAME: Drop the old event instrumentation. */
-    MYSQL_DROP_SP(to_uint(enum_sp_type::EVENT), parse_data->dbname.str,
-                  parse_data->dbname.length, parse_data->name.str,
-                  parse_data->name.length);
-  }
-
   // Update element in event queue.
   if (event_queue && new_element != nullptr) {
     /*
@@ -639,8 +628,7 @@ bool Events::drop_event(THD *thd, LEX_CSTRING dbname, LEX_CSTRING name,
                         bool if_exists) {
   DBUG_TRACE;
 
-  if (check_access(thd, EVENT_ACL, dbname.str, nullptr, nullptr, false, false))
-    return true;
+  if (check_access(thd, EVENT_ACL, dbname.str, NULL, NULL, 0, 0)) return true;
 
   // Acquire exclusive MDL lock.
   if (lock_object_name(thd, MDL_key::EVENT, dbname.str, name.str)) return true;
@@ -657,7 +645,7 @@ bool Events::drop_event(THD *thd, LEX_CSTRING dbname, LEX_CSTRING name,
 
   // Binlog the drop event.
   {
-    assert(thd->query().str && thd->query().length);
+    DBUG_ASSERT(thd->query().str && thd->query().length);
 
     thd->add_to_binlog_accessed_dbs(dbname.str);
     if (write_bin_log(thd, true, thd->query().str, thd->query().length,
@@ -722,13 +710,6 @@ bool Events::lock_schema_events(THD *thd, const dd::Schema &schema) {
     schema_name = schema_name_buf;
   }
 
-  /*
-    Ensure that we don't hold memory used by MDL_requests after locks have
-    been acquired. This reduces memory usage in cases when we have DROP
-    DATABASE that needs to drop lots of different objects.
-  */
-  MEM_ROOT mdl_reqs_root(key_memory_rm_db_mdl_reqs_root, MEM_ROOT_BLOCK_SIZE);
-
   MDL_request_list mdl_requests;
   for (std::vector<dd::String_type>::const_iterator name = event_names.begin();
        name != event_names.end(); ++name) {
@@ -736,7 +717,7 @@ bool Events::lock_schema_events(THD *thd, const dd::Schema &schema) {
     dd::Event::create_mdl_key(dd::String_type(schema_name), *name, &mdl_key);
 
     // Add MDL_request for routine to mdl_requests list.
-    MDL_request *mdl_request = new (&mdl_reqs_root) MDL_request;
+    MDL_request *mdl_request = new (thd->mem_root) MDL_request;
     MDL_REQUEST_INIT_BY_KEY(mdl_request, &mdl_key, MDL_EXCLUSIVE,
                             MDL_TRANSACTION);
     mdl_requests.push_front(mdl_request);
@@ -778,7 +759,7 @@ static bool send_show_create_event(THD *thd, Event_timed *et,
                                    Protocol *protocol) {
   char show_str_buf[10 * STRING_BUFFER_USUAL_SIZE];
   String show_str(show_str_buf, sizeof(show_str_buf), system_charset_info);
-  mem_root_deque<Item *> field_list(thd->mem_root);
+  List<Item> field_list;
   LEX_STRING sql_mode;
   const String *tz_name;
 
@@ -811,7 +792,7 @@ static bool send_show_create_event(THD *thd, Event_timed *et,
   field_list.push_back(
       new Item_empty_string("Database Collation", MY_CS_NAME_SIZE));
 
-  if (thd->send_result_metadata(field_list,
+  if (thd->send_result_metadata(&field_list,
                                 Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF))
     return true;
 
@@ -824,14 +805,14 @@ static bool send_show_create_event(THD *thd, Event_timed *et,
                          system_charset_info);
   protocol->store_string(show_str.c_ptr(), show_str.length(),
                          et->m_creation_ctx->get_client_cs());
-  const char *csname = et->m_creation_ctx->get_client_cs()->csname;
-  protocol->store_string(csname, strlen(csname), system_charset_info);
-  protocol->store_string(
-      et->m_creation_ctx->get_connection_cl()->m_coll_name,
-      strlen(et->m_creation_ctx->get_connection_cl()->m_coll_name),
-      system_charset_info);
-  protocol->store_string(et->m_creation_ctx->get_db_cl()->m_coll_name,
-                         strlen(et->m_creation_ctx->get_db_cl()->m_coll_name),
+  protocol->store_string(et->m_creation_ctx->get_client_cs()->csname,
+                         strlen(et->m_creation_ctx->get_client_cs()->csname),
+                         system_charset_info);
+  protocol->store_string(et->m_creation_ctx->get_connection_cl()->name,
+                         strlen(et->m_creation_ctx->get_connection_cl()->name),
+                         system_charset_info);
+  protocol->store_string(et->m_creation_ctx->get_db_cl()->name,
+                         strlen(et->m_creation_ctx->get_db_cl()->name),
                          system_charset_info);
 
   if (protocol->end_row()) return true;
@@ -858,8 +839,7 @@ bool Events::show_create_event(THD *thd, LEX_CSTRING dbname, LEX_CSTRING name) {
   DBUG_TRACE;
   DBUG_PRINT("enter", ("name: %s@%s", dbname.str, name.str));
 
-  if (check_access(thd, EVENT_ACL, dbname.str, nullptr, nullptr, false, false))
-    return true;
+  if (check_access(thd, EVENT_ACL, dbname.str, NULL, NULL, 0, 0)) return true;
 
   // We must make sure the schema is released and unlocked in the right
   // order. Fail if we are unable to get a meta data lock on the schema
@@ -952,8 +932,8 @@ bool Events::init(bool opt_noacl_or_bootstrap) {
   thd->thread_stack = (char *)&thd;
   thd->store_globals();
 
-  assert(opt_event_scheduler == Events::EVENTS_ON ||
-         opt_event_scheduler == Events::EVENTS_OFF);
+  DBUG_ASSERT(opt_event_scheduler == Events::EVENTS_ON ||
+              opt_event_scheduler == Events::EVENTS_OFF);
 
   if (!(event_queue = new Event_queue) ||
       !(scheduler = new Event_scheduler(event_queue))) {
@@ -971,9 +951,9 @@ bool Events::init(bool opt_noacl_or_bootstrap) {
 end:
   if (res) {
     delete event_queue;
-    event_queue = nullptr;
+    event_queue = NULL;
     delete scheduler;
-    scheduler = nullptr;
+    scheduler = NULL;
   }
   delete thd;
 
@@ -995,9 +975,9 @@ void Events::deinit() {
 
   if (opt_event_scheduler != EVENTS_DISABLED) {
     delete scheduler;
-    scheduler = nullptr; /* safety */
+    scheduler = NULL; /* safety */
     delete event_queue;
-    event_queue = nullptr; /* safety */
+    event_queue = NULL; /* safety */
   }
 }
 
@@ -1024,10 +1004,9 @@ static PSI_cond_info all_events_conds[] = {
 PSI_thread_key key_thread_event_scheduler, key_thread_event_worker;
 
 static PSI_thread_info all_events_threads[] = {
-    {&key_thread_event_scheduler, "event_scheduler", "evt_sched",
-     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME},
-    {&key_thread_event_worker, "event_worker", "evt_wkr", 0, 0,
-     PSI_DOCUMENT_ME}};
+    {&key_thread_event_scheduler, "event_scheduler", PSI_FLAG_SINGLETON, 0,
+     PSI_DOCUMENT_ME},
+    {&key_thread_event_worker, "event_worker", 0, 0, PSI_DOCUMENT_ME}};
 #endif /* HAVE_PSI_INTERFACE */
 
 PSI_stage_info stage_waiting_on_empty_queue = {0, "Waiting on empty queue", 0,
@@ -1046,8 +1025,7 @@ PSI_stage_info *all_events_stages[] = {&stage_waiting_on_empty_queue,
 
 static PSI_memory_info all_events_memory[] = {
     {&key_memory_event_basic_root, "Event_basic::mem_root",
-     PSI_FLAG_ONLY_GLOBAL_STAT, 0,
-     "Event base class with root used for definiton etc."}};
+     PSI_FLAG_ONLY_GLOBAL_STAT, 0, PSI_DOCUMENT_ME}};
 
 static void init_events_psi_keys(void) {
   const char *category = "sql";
@@ -1164,15 +1142,6 @@ static bool load_events_from_db(THD *thd, Event_queue *event_queue) {
     if (thd->dd_client()->fetch_schema_components(schema_obj, &events))
       return true;
 
-    const char *converted_schema_name = schema_obj->name().c_str();
-    char name_buf[NAME_LEN + 1];
-    if (lower_case_table_names == 2) {
-      // Lower case table names == 2 is tested on OSX.
-      my_stpcpy(name_buf, converted_schema_name);
-      my_casedn_str(&my_charset_utf8mb3_tolower_ci, name_buf);
-      converted_schema_name = name_buf;
-    }
-
     for (const dd::Event *ev_obj : events) {
       std::unique_ptr<Event_queue_element> et(new (std::nothrow)
                                                   Event_queue_element);
@@ -1181,7 +1150,7 @@ static bool load_events_from_db(THD *thd, Event_queue *event_queue) {
         return true;
       }
 
-      if (et->fill_event_info(thd, *ev_obj, converted_schema_name)) {
+      if (et->fill_event_info(thd, *ev_obj, schema_obj->name().c_str())) {
         LogErr(ERROR_LEVEL, ER_EVENT_SCHEDULER_GOT_BAD_DATA_FROM_TABLE);
         return true;
       }
