@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -21,13 +21,16 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <cstdio>
-#include <cstdlib>
-#include <cstring>
+#include <limits.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+/* We only need the tokens here */
+#define YYSTYPE_IS_DECLARED
 
 #include "sql/lex.h"
 #include "sql/lex_symbol.h"
-#include "sql/sql_hints.yy.h"
 #include "sql/sql_yacc.h"
 #include "welcome_copyright_notice.h" /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
 
@@ -49,14 +52,12 @@
   This is done to ensure stability in digest computed values.
 
   As of now (8.0.0), the mapping looks like this:
-  - PART 1: [0 .. 255] tokens of single-character lexemes
-  - PART 2: [256 .. ...] tokens < YYUNDEF from sql_yacc.yy
-  - PART 3: [... .. 999] reserved for sql_yacc.yy new tokens < YYUNDEF
-  - PART 4: [1000 .. ...] tokens from sql_hints.yy
-  - PART 5: [... .. 1099] reserved for sql_hints.yy new tokens
-  - PART 6: [1100 .. ...] digest special fake tokens
-  - PART 7: [... .. 1149] reserved for new digest special fake tokens
-  - PART 8: [1150 .. ...] tokens > YYUNDEF from sql_yacc.yy
+  - [0 .. 255] character terminal tokens.
+  - [256 .. 907] non terminal tokens from sql_yacc.yy
+  - [908 .. 999] reserved for sql_yacc.yy new tokens
+  - [1000 .. 1017] non terminal tokens from sql_hints.yy
+  - [1018 .. 1099] reserved for sql_hints.yy new tokens
+  - [1100 .. 1111] non terminal tokens for digests
 
   Should gen_lex_token fail when tokens are exhausted
   (maybe you are reading this comment because of a fprintf(stderr) below),
@@ -71,9 +72,9 @@
   to complexity, by making the parser tables bigger,
   so adding tokens all the time is not a good practice.
 
-  3) Expand boundary values for
-     - range_for_sql_hints
-     - range_for_digests
+  3) Expand the values for
+     - start_token_range_for_sql_hints
+     - start_token_range_for_digests
   and record again all the MTR tests that print a DIGEST,
   because DIGEST values have now changed.
 
@@ -84,171 +85,35 @@
   - likewise for sql/sql_hints.yy
 */
 
-/** Generated token. */
-struct gen_lex_token_string {
-  gen_lex_token_string(const char *token_string, int token_length,
-                       bool append_space, bool start_expr)
-      : m_token_string{token_string},
-        m_token_length{token_length},
-        m_append_space{append_space},
-        m_start_expr{start_expr} {}
-
-  gen_lex_token_string()
-      : m_token_string{},
-        m_token_length{},
-        m_append_space{true},
-        m_start_expr{false} {}
-
-  /**
-    Real lexeme string or user-specified text to output with a normalized
-    query string.
-  */
-  const char *m_token_string;
-
-  /**
-    Byte length of m_token_string.
-  */
-  int m_token_length;
-
-  /**
-    If true, output ' ' after this token to a normalized query string.
-    See digest_add_token().
-  */
-  bool m_append_space;
-
-  /**
-    See digest_add_token().
-  */
-  bool m_start_expr;
-
-  /**
-    The structure is uninitialized if false.
-  */
-  bool m_initialized{false};
-};
-
+int start_token_range_for_sql_hints = 1000;
+int start_token_range_for_digests = 1100;
 /*
   This is a tool used during build only,
   so MY_MAX_TOKEN does not need to be exact,
   only big enough to hold:
-  - 256 of single-character lexeme tokens
-  - up to 1000 named tokens from bison (sql_yacc.yy).
-  - padding
-  - tokens from bison (sql_hints.yy).
-  - padding
+  - 256 character terminal tokens
+  - YYNTOKENS named terminal tokens from bison (sql_yacc.yy).
+  - padding (see start_token_range_for_sql_hints)
+  - YYNTOKENS named terminal tokens from bison (sql_hints.yy).
+  - padding (see start_token_range_for_digests)
   - DIGEST special tokens.
-  - padding
-  - mode named tokens from bison (sql_yacc.yy).
   See also YYMAXUTOK.
 */
-const int MY_MAX_TOKEN = 1251;
-
-gen_lex_token_string compiled_token_array[MY_MAX_TOKEN];
-
-struct range {
-  range(const char *title, int start, int end)
-      : title{title}, start{start}, end{end}, max_seen{0} {}
-
-  void set_token(int tok, const char *str, int line) {
-    if (tok <= 0) {
-      fprintf(stderr, "%s:%d: Bad token found\n", __FILE__, line);
-      exit(1);
-    }
-
-    if (tok > end) {
-      fprintf(stderr,
-              "%s:%d: Token reserve for %s exhausted: %d (should be <= %d).\n"
-              "Please see MAINTAINER instructions in sql/gen_lex_token.cc\n",
-              __FILE__, line, title, tok, end);
-      exit(1);
-    }
-
-    if (tok >= MY_MAX_TOKEN) {
-      fprintf(stderr,
-              "%s:%d: Added that many new keywords ? Increase MY_MAX_TOKEN\n",
-              __FILE__, line);
-      exit(1);
-    }
-
-    if (tok > max_seen) {
-      max_seen = tok;
-    }
-
-    compiled_token_array[tok].m_initialized = true;
-    compiled_token_array[tok].m_token_string = str;
-    compiled_token_array[tok].m_token_length = strlen(str);
-    compiled_token_array[tok].m_append_space = true;
-    compiled_token_array[tok].m_start_expr = false;
-  }
-
-  int add_token(const char *str, int line) {
-    set_token(max_seen ? max_seen + 1 : start, str, line);
-    return max_seen;
-  }
-
-  void print(const char *header1, const char *header2 = nullptr) const {
-    puts(header1);
-    for (int tok = start; tok <= max_seen; tok++) {
-      print_token(tok);
-    }
-
-    if (header2 == nullptr) {
-      return;
-    }
-
-    puts(header2);
-    for (int tok = max_seen + 1; tok <= end; tok++) {
-      printf("/* reserved %03d for %s */  { \"\", 0, false, false},\n", tok,
-             title);
-    }
-  }
-
- private:
-  void print_token(int tok) const {
-    const gen_lex_token_string *x = &compiled_token_array[tok];
-    if (tok < 256) {
-      printf("/* %03d */  { \"\\x%02x\", 1, %s, %s},\n", tok, tok,
-             x->m_append_space ? "true" : "false",
-             x->m_start_expr ? "true" : "false");
-      return;
-    }
-
-    if (!x->m_initialized) {
-      static const gen_lex_token_string dummy{"(unknown)", 9, true, false};
-      x = &dummy;
-    }
-    printf("/* %03d */  { \"%s\", %d, %s, %s},\n", tok, x->m_token_string,
-           x->m_token_length, x->m_append_space ? "true" : "false",
-           x->m_start_expr ? "true" : "false");
-  }
-
- private:
-  const char *const title;
-
- public:
-  const int start;
-
- private:
-  const int end;
-
-  int max_seen;
+#define MY_MAX_TOKEN 1200
+/** Generated token. */
+struct gen_lex_token_string {
+  const char *m_token_string;
+  int m_token_length;
+  bool m_append_space;
+  bool m_start_expr;
 };
 
-static_assert(YYUNDEF == 1150,
-              "YYUNDEF must be stable, because raw token numbers are used in "
-              "PFS digest calculations");
-range range_for_sql_yacc2{"sql/sql_yacc.yy (before YYUNDEF)", YYUNDEF,
-                          MY_MAX_TOKEN};
+gen_lex_token_string compiled_token_array[MY_MAX_TOKEN];
+int max_token_seen = 0;
+int max_token_seen_in_sql_yacc = 0;
+int max_token_seen_in_sql_hints = 0;
 
-range range_for_digests{"digest specials", 1100, range_for_sql_yacc2.start - 1};
-
-static_assert(MAX_EXECUTION_TIME_HINT == 1000,
-              "MAX_EXECUTION_TIME_HINT should be equal to 1000");
-range range_for_sql_hints{"sql/sql_hints.yy", MAX_EXECUTION_TIME_HINT,
-                          range_for_digests.start - 1};
-
-range range_for_sql_yacc1{"sql/sql_yacc.yy (after YYUNDEF)", 256,
-                          range_for_sql_hints.start - 1};
+char char_tokens[256];
 
 int tok_generic_value = 0;
 int tok_generic_value_list = 0;
@@ -265,71 +130,159 @@ int tok_hint_comment_close =
     0;  ///< Fake token value for "*/" of hint comments.
 int tok_unused = 0;
 
+/**
+  Adjustment value to translate hint parser's internal token values to generally
+  visible token values. This adjustment is necessary, since keyword token values
+  of separate parsers may interfere.
+*/
+int tok_hint_adjust = 0;
+
+static void set_token(int tok, const char *str) {
+  if (tok <= 0) {
+    fprintf(stderr, "Bad token found\n");
+    exit(1);
+  }
+
+  if (tok > max_token_seen) {
+    max_token_seen = tok;
+  }
+
+  if (max_token_seen >= MY_MAX_TOKEN) {
+    fprintf(stderr, "Added that many new keywords ? Increase MY_MAX_TOKEN\n");
+    exit(1);
+  }
+
+  compiled_token_array[tok].m_token_string = str;
+  compiled_token_array[tok].m_token_length = strlen(str);
+  compiled_token_array[tok].m_append_space = true;
+  compiled_token_array[tok].m_start_expr = false;
+}
+
 static void set_start_expr_token(int tok) {
   compiled_token_array[tok].m_start_expr = true;
 }
 
 static void compute_tokens() {
+  int tok;
+  unsigned int i;
+  char *str;
+
+  /*
+    Default value.
+  */
+  for (tok = 0; tok < MY_MAX_TOKEN; tok++) {
+    compiled_token_array[tok].m_token_string = "(unknown)";
+    compiled_token_array[tok].m_token_length = 9;
+    compiled_token_array[tok].m_append_space = true;
+    compiled_token_array[tok].m_start_expr = false;
+  }
+
   /*
     Tokens made of just one terminal character
   */
+  for (tok = 0; tok < 256; tok++) {
+    str = &char_tokens[tok];
+    str[0] = (char)tok;
+    compiled_token_array[tok].m_token_string = str;
+    compiled_token_array[tok].m_token_length = 1;
+    compiled_token_array[tok].m_append_space = true;
+  }
 
-  // Do nothing -- see range::print() for token numbers in [0 .. 255]
+  max_token_seen = 255;
+
+  /*
+    String terminal tokens, used in sql_yacc.yy
+  */
+  set_token(NEG, "~");
 
   /*
     Tokens hard coded in sql_lex.cc
   */
 
-  range_for_sql_yacc1.set_token(WITH_ROLLUP_SYM, "WITH ROLLUP", __LINE__);
-  range_for_sql_yacc1.set_token(NOT2_SYM, "!", __LINE__);
-  range_for_sql_yacc1.set_token(OR2_SYM, "||", __LINE__);
-  range_for_sql_yacc1.set_token(PARAM_MARKER, "?", __LINE__);
-  range_for_sql_yacc1.set_token(SET_VAR, ":=", __LINE__);
-  range_for_sql_yacc1.set_token(UNDERSCORE_CHARSET, "(_charset)", __LINE__);
-  range_for_sql_yacc1.set_token(END_OF_INPUT, "", __LINE__);
-  range_for_sql_yacc1.set_token(JSON_SEPARATOR_SYM, "->", __LINE__);
-  range_for_sql_yacc1.set_token(JSON_UNQUOTED_SEPARATOR_SYM, "->>", __LINE__);
+  set_token(WITH_ROLLUP_SYM, "WITH ROLLUP");
+  set_token(NOT2_SYM, "!");
+  set_token(OR2_SYM, "|");
+  set_token(PARAM_MARKER, "?");
+  set_token(SET_VAR, ":=");
+  set_token(UNDERSCORE_CHARSET, "(_charset)");
+  set_token(END_OF_INPUT, "");
+  set_token(JSON_SEPARATOR_SYM, "->");
+  set_token(JSON_UNQUOTED_SEPARATOR_SYM, "->>");
 
   /*
     Values.
     These tokens are all normalized later,
     so this strings will never be displayed.
   */
-  range_for_sql_yacc1.set_token(BIN_NUM, "(bin)", __LINE__);
-  range_for_sql_yacc1.set_token(DECIMAL_NUM, "(decimal)", __LINE__);
-  range_for_sql_yacc1.set_token(FLOAT_NUM, "(float)", __LINE__);
-  range_for_sql_yacc1.set_token(HEX_NUM, "(hex)", __LINE__);
-  range_for_sql_yacc1.set_token(LEX_HOSTNAME, "(hostname)", __LINE__);
-  range_for_sql_yacc1.set_token(LONG_NUM, "(long)", __LINE__);
-  range_for_sql_yacc1.set_token(NUM, "(num)", __LINE__);
-  range_for_sql_yacc1.set_token(TEXT_STRING, "(text)", __LINE__);
-  range_for_sql_yacc1.set_token(NCHAR_STRING, "(nchar)", __LINE__);
-  range_for_sql_yacc1.set_token(ULONGLONG_NUM, "(ulonglong)", __LINE__);
+  set_token(BIN_NUM, "(bin)");
+  set_token(DECIMAL_NUM, "(decimal)");
+  set_token(FLOAT_NUM, "(float)");
+  set_token(HEX_NUM, "(hex)");
+  set_token(LEX_HOSTNAME, "(hostname)");
+  set_token(LONG_NUM, "(long)");
+  set_token(NUM, "(num)");
+  set_token(TEXT_STRING, "(text)");
+  set_token(NCHAR_STRING, "(nchar)");
+  set_token(ULONGLONG_NUM, "(ulonglong)");
 
   /*
     Identifiers.
   */
-  range_for_sql_yacc1.set_token(IDENT, "(id)", __LINE__);
-  range_for_sql_yacc1.set_token(IDENT_QUOTED, "(id_quoted)", __LINE__);
+  set_token(IDENT, "(id)");
+  set_token(IDENT_QUOTED, "(id_quoted)");
 
   /*
     See symbols[] in sql/lex.h
   */
-  for (const SYMBOL &sym : symbols) {
-    if ((sym.group & SG_MAIN_PARSER) != 0) {
-      if (sym.tok < YYUNDEF)
-        range_for_sql_yacc1.set_token(sym.tok, sym.name, __LINE__);
-      else
-        range_for_sql_yacc2.set_token(sym.tok, sym.name, __LINE__);
-    } else if ((sym.group & SG_HINTS) != 0) {
-      range_for_sql_hints.set_token(sym.tok, sym.name, __LINE__);
-    } else {
-      fprintf(stderr, "%s:%d: Unknown symbol group flag: %x\n", __FILE__,
-              __LINE__, sym.group & ~(SG_MAIN_PARSER | SG_HINTS));
-      exit(1);
-    }
+  for (i = 0; i < sizeof(symbols) / sizeof(symbols[0]); i++) {
+    if (!(symbols[i].group & SG_MAIN_PARSER)) continue;
+    set_token(symbols[i].tok, symbols[i].name);
   }
 
+  max_token_seen_in_sql_yacc = max_token_seen;
+
+  if (max_token_seen_in_sql_yacc >= start_token_range_for_sql_hints) {
+    fprintf(stderr,
+            "sql/sql_yacc.yy token reserve exhausted.\n"
+            "Please see MAINTAINER instructions in sql/gen_lex_token.cc\n");
+    exit(1);
+  }
+
+  /*
+    FAKE tokens to output "optimizer hint" keywords.
+
+    Hint keyword token values may interfere with token values of the main SQL
+    parser, so the tok_hint_adjust adjustment is needed to add them into
+    compiled_token_array and lex_token_array.
+
+    Also see the TOK_HINT_ADJUST() adjustment macro definition.
+  */
+  int tok_hint_min = INT_MAX;
+  for (unsigned int i = 0; i < sizeof(symbols) / sizeof(symbols[0]); i++) {
+    if (!(symbols[i].group & SG_HINTS)) continue;
+
+    if (static_cast<int>(symbols[i].tok) < tok_hint_min)
+      tok_hint_min = symbols[i].tok;  // Calculate the minimal hint token value.
+  }
+
+  tok_hint_adjust = start_token_range_for_sql_hints - tok_hint_min;
+
+  for (unsigned int i = 0; i < sizeof(symbols) / sizeof(symbols[0]); i++) {
+    if (!(symbols[i].group & SG_HINTS)) continue;
+    set_token(symbols[i].tok + tok_hint_adjust, symbols[i].name);
+  }
+
+  max_token_seen_in_sql_hints = max_token_seen;
+
+  if (max_token_seen_in_sql_hints >= start_token_range_for_digests) {
+    fprintf(stderr,
+            "sql/sql_hints.yy token reserve exhausted.\n"
+            "Please see MAINTAINER instructions in sql/gen_lex_token.cc\n");
+    exit(1);
+  }
+
+  max_token_seen = start_token_range_for_digests;
+
   /*
     Additional FAKE tokens,
     used internally to normalize a digest text.
@@ -337,29 +290,45 @@ static void compute_tokens() {
 
   /* Digest tokens in 5.7 */
 
-  tok_generic_value = range_for_digests.add_token("?", __LINE__);
-  tok_generic_value_list = range_for_digests.add_token("?, ...", __LINE__);
-  tok_row_single_value = range_for_digests.add_token("(?)", __LINE__);
-  tok_row_single_value_list =
-      range_for_digests.add_token("(?) /* , ... */", __LINE__);
-  tok_row_multiple_value = range_for_digests.add_token("(...)", __LINE__);
-  tok_row_multiple_value_list =
-      range_for_digests.add_token("(...) /* , ... */", __LINE__);
-  tok_ident = range_for_digests.add_token("(tok_id)", __LINE__);
-  tok_ident_at = range_for_digests.add_token("(tok_id_at)", __LINE__);
-  tok_hint_comment_open =
-      range_for_digests.add_token(HINT_COMMENT_STARTER, __LINE__);
-  tok_hint_comment_close =
-      range_for_digests.add_token(HINT_COMMENT_TERMINATOR, __LINE__);
+  tok_generic_value = max_token_seen++;
+  set_token(tok_generic_value, "?");
+
+  tok_generic_value_list = max_token_seen++;
+  set_token(tok_generic_value_list, "?, ...");
+
+  tok_row_single_value = max_token_seen++;
+  set_token(tok_row_single_value, "(?)");
+
+  tok_row_single_value_list = max_token_seen++;
+  set_token(tok_row_single_value_list, "(?) /* , ... */");
+
+  tok_row_multiple_value = max_token_seen++;
+  set_token(tok_row_multiple_value, "(...)");
+
+  tok_row_multiple_value_list = max_token_seen++;
+  set_token(tok_row_multiple_value_list, "(...) /* , ... */");
+
+  tok_ident = max_token_seen++;
+  set_token(tok_ident, "(tok_id)");
+
+  tok_ident_at = max_token_seen++;
+  set_token(tok_ident_at, "(tok_id_at)");
+
+  tok_hint_comment_open = max_token_seen++;
+  set_token(tok_hint_comment_open, HINT_COMMENT_STARTER);
+
+  tok_hint_comment_close = max_token_seen++;
+  set_token(tok_hint_comment_close, HINT_COMMENT_TERMINATOR);
 
   /* New in 8.0 */
 
-  tok_in_generic_value_expression =
-      range_for_digests.add_token("IN (...)", __LINE__);
+  tok_in_generic_value_expression = max_token_seen++;
+  set_token(tok_in_generic_value_expression, "IN (...)");
 
   /* Add new digest tokens here */
 
-  tok_unused = range_for_digests.add_token("UNUSED", __LINE__);
+  tok_unused = max_token_seen++;
+  set_token(tok_unused, "UNUSED");
 
   /*
     Fix whitespace for some special tokens.
@@ -441,22 +410,56 @@ static void print_tokens() {
            compiled_token_array[tok].m_start_expr ? "true" : "false");
   }
 
-  range_for_sql_yacc1.print(
-      "/* PART 2: named tokens from sql/sql_yacc.yy (chunk 1). */",
-      "/* PART 3: padding reserved for sql/sql_yacc.yy extensions. */");
+  printf("/* PART 2: named tokens from sql/sql_yacc.yy. */\n");
 
-  range_for_sql_hints.print(
-      "/* PART 4: named tokens from sql/sql_hints.yy. */",
-      "/* PART 5: padding reserved for sql/sql_hints.yy extensions. */");
+  for (tok = 256; tok <= max_token_seen_in_sql_yacc; tok++) {
+    printf("/* %03d */  { \"%s\", %d, %s, %s},\n", tok,
+           compiled_token_array[tok].m_token_string,
+           compiled_token_array[tok].m_token_length,
+           compiled_token_array[tok].m_append_space ? "true" : "false",
+           compiled_token_array[tok].m_start_expr ? "true" : "false");
+  }
 
-  range_for_digests.print(
-      "/* PART 6: Digest special tokens. */",
-      "/* PART 7: padding reserved for digest special tokens. */");
+  printf("/* PART 3: padding reserved for sql/sql_yacc.yy extensions. */\n");
 
-  range_for_sql_yacc2.print(
-      "/* PART 8: named tokens from sql/sql_yacc.yy (chunk 2). */");
+  for (tok = max_token_seen_in_sql_yacc + 1;
+       tok < start_token_range_for_sql_hints; tok++) {
+    printf(
+        "/* reserved %03d for sql/sql_yacc.yy */  { \"\", 0, false, false},\n",
+        tok);
+  }
 
-  printf("/* PART 9: End of token list. */\n");
+  printf("/* PART 4: named tokens from sql/sql_hints.yy. */\n");
+
+  for (tok = start_token_range_for_sql_hints;
+       tok <= max_token_seen_in_sql_hints; tok++) {
+    printf("/* %03d */  { \"%s\", %d, %s, %s},\n", tok,
+           compiled_token_array[tok].m_token_string,
+           compiled_token_array[tok].m_token_length,
+           compiled_token_array[tok].m_append_space ? "true" : "false",
+           compiled_token_array[tok].m_start_expr ? "true" : "false");
+  }
+
+  printf("/* PART 5: padding reserved for sql/sql_hints.yy extensions. */\n");
+
+  for (tok = max_token_seen_in_sql_hints + 1;
+       tok < start_token_range_for_digests; tok++) {
+    printf(
+        "/* reserved %03d for sql/sql_hints.yy */  { \"\", 0, false, false},\n",
+        tok);
+  }
+
+  printf("/* PART 6: Digest special tokens. */\n");
+
+  for (tok = start_token_range_for_digests; tok < max_token_seen; tok++) {
+    printf("/* %03d */  { \"%s\", %d, %s, %s},\n", tok,
+           compiled_token_array[tok].m_token_string,
+           compiled_token_array[tok].m_token_length,
+           compiled_token_array[tok].m_append_space ? "true" : "false",
+           compiled_token_array[tok].m_start_expr ? "true" : "false");
+  }
+
+  printf("/* PART 7: End of token list. */\n");
 
   printf("/* DUMMY */ { \"\", 0, false, false}\n");
   printf("};\n");
@@ -476,6 +479,7 @@ static void print_tokens() {
   printf("#define TOK_HINT_COMMENT_CLOSE %d\n", tok_hint_comment_close);
   printf("#define TOK_IN_GENERIC_VALUE_EXPRESSION %d\n",
          tok_in_generic_value_expression);
+  printf("#define TOK_HINT_ADJUST(x) ((x) + %d)\n", tok_hint_adjust);
   printf("#define TOK_UNUSED %d\n", tok_unused);
 }
 
@@ -487,14 +491,21 @@ static void print_tokens() {
 */
 static const int zerofill_expected_value = 906;
 
-static_assert(!(ZEROFILL_SYM < zerofill_expected_value),
-              "Token deleted. "
-              "Please read MAINTAINER instructions in sql/sql_yacc.yy");
-static_assert(!(ZEROFILL_SYM > zerofill_expected_value),
-              "Token added in the wrong place. "
-              "Please read MAINTAINER instructions in sql/sql_yacc.yy");
-
 int main(int, char **) {
+  if (ZEROFILL_SYM < zerofill_expected_value) {
+    fprintf(stderr,
+            "Token deleted.\n"
+            "Please read MAINTAINER instructions in sql/sql_yacc.yy\n");
+    return 1;
+  }
+
+  if (ZEROFILL_SYM > zerofill_expected_value) {
+    fprintf(stderr,
+            "Token added in the wrong place.\n"
+            "Please read MAINTAINER instructions in sql/sql_yacc.yy\n");
+    return 1;
+  }
+
   puts(ORACLE_GPL_COPYRIGHT_NOTICE("2016"));
 
   printf("/*\n");
