@@ -1,19 +1,18 @@
 #ifndef SQL_GIS_TRANSFORM_FUNCTOR_H_INCLUDED
 #define SQL_GIS_TRANSFORM_FUNCTOR_H_INCLUDED
 
-// Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+// Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0,
 // as published by the Free Software Foundation.
 //
-// This program is designed to work with certain software (including
+// This program is also distributed with certain software (including
 // but not limited to OpenSSL) that is licensed under separate terms,
 // as designated in a particular file or component or in included license
 // documentation.  The authors of MySQL hereby grant you an additional
 // permission to link the program and your derivative works with the
-// separately licensed software that they have either included with
-// the program or referenced in the documentation.
+// separately licensed software that they have included with MySQL.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,13 +35,8 @@
 #include <memory>  // std::unique_ptr
 #include <string>
 
-#include "my_compiler.h"
-
-MY_COMPILER_DIAGNOSTIC_PUSH()
-MY_COMPILER_GCC_DIAGNOSTIC_IGNORE("-Wmaybe-uninitialized")
 #include <boost/geometry.hpp>
 #include <boost/geometry/srs/transformation.hpp>
-MY_COMPILER_DIAGNOSTIC_POP()
 
 #include "sql/gis/functor.h"
 #include "sql/gis/geometries.h"
@@ -79,19 +73,12 @@ class Transform : public Unary_functor<std::unique_ptr<Geometry>> {
 
   std::unique_ptr<Geometry> operator()(const Geometry &g) const override;
   std::unique_ptr<Geometry> eval(const Geometry &g) const;
-  std::unique_ptr<Geometry> eval(const Cartesian_point &g) const;
   std::unique_ptr<Geometry> eval(const Geographic_point &g) const;
-  std::unique_ptr<Geometry> eval(const Cartesian_linestring &g) const;
   std::unique_ptr<Geometry> eval(const Geographic_linestring &g) const;
-  std::unique_ptr<Geometry> eval(const Cartesian_polygon &g) const;
   std::unique_ptr<Geometry> eval(const Geographic_polygon &g) const;
-  std::unique_ptr<Geometry> eval(const Cartesian_geometrycollection &g) const;
   std::unique_ptr<Geometry> eval(const Geographic_geometrycollection &g) const;
-  std::unique_ptr<Geometry> eval(const Cartesian_multipoint &g) const;
   std::unique_ptr<Geometry> eval(const Geographic_multipoint &g) const;
-  std::unique_ptr<Geometry> eval(const Cartesian_multilinestring &g) const;
   std::unique_ptr<Geometry> eval(const Geographic_multilinestring &g) const;
-  std::unique_ptr<Geometry> eval(const Cartesian_multipolygon &g) const;
   std::unique_ptr<Geometry> eval(const Geographic_multipolygon &g) const;
 };
 
