@@ -1,16 +1,15 @@
-/* Copyright (c) 2002, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,7 +36,7 @@
 #include "sql/psi_memory_key.h"
 
 static inline bool is_numeric_beginning(const char *pc, const size_t len) {
-  return (pc != nullptr &&
+  return (pc != NULL &&
           ((*pc >= '0' && *pc <= '9') || *pc == '-' || *pc == '+' ||
            (*pc == '.' && len > 1 && pc[1] >= '0' && pc[1] <= '9')));
 }
@@ -57,8 +56,7 @@ bool Gis_read_stream::get_next_word(LEX_CSTRING *res) {
   skip_space();
   res->str = m_cur;
   /* The following will also test for \0 */
-  if ((m_cur >= m_limit) || !my_isvar_start(&my_charset_bin, *m_cur))
-    return true;
+  if ((m_cur >= m_limit) || !my_isvar_start(&my_charset_bin, *m_cur)) return 1;
 
   /*
     We can't combine the following increment with my_isvar() because
@@ -68,7 +66,7 @@ bool Gis_read_stream::get_next_word(LEX_CSTRING *res) {
   while ((m_cur < m_limit) && my_isvar(&my_charset_bin, *m_cur)) m_cur++;
 
   res->length = (uint32)(m_cur - res->str);
-  return false;
+  return 0;
 }
 
 /* Read a floating point number. */
@@ -80,13 +78,13 @@ bool Gis_read_stream::get_next_number(double *d) {
 
   if ((m_cur >= m_limit) || !is_numeric_beginning(m_cur, m_limit - m_cur)) {
     set_error_msg("Numeric constant expected");
-    return true;
+    return 1;
   }
 
   *d = my_strntod(m_charset, m_cur, (uint)(m_limit - m_cur), &endptr, &err);
-  if (err) return true;
+  if (err) return 1;
   if (endptr) m_cur = endptr;
-  return false;
+  return 0;
 }
 
 bool Gis_read_stream::check_next_symbol(char symbol) {
@@ -96,10 +94,10 @@ bool Gis_read_stream::check_next_symbol(char symbol) {
     my_stpcpy(buff, "'?' expected");
     buff[2] = symbol;
     set_error_msg(buff);
-    return true;
+    return 1;
   }
   m_cur++;
-  return false;
+  return 0;
 }
 
 /*
