@@ -1,19 +1,18 @@
 #ifndef GSTREAM_INCLUDED
 #define GSTREAM_INCLUDED
 
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,7 +40,7 @@ class Gis_read_stream {
                   int size)
       : m_cur(buffer),
         m_limit(buffer + size),
-        m_err_msg(nullptr),
+        m_err_msg(NULL),
         m_charset(charset),
         m_thd(thd) {}
   Gis_read_stream() : m_cur(NullS), m_limit(NullS), m_err_msg(NullS) {}
@@ -60,10 +59,9 @@ class Gis_read_stream {
   /* Skip next character, if match. Return 1 if no match */
   inline bool skip_char(char skip) {
     skip_space();
-    if ((m_cur >= m_limit) || *m_cur != skip)
-      return true; /* Didn't find char */
+    if ((m_cur >= m_limit) || *m_cur != skip) return 1; /* Didn't find char */
     m_cur++;
-    return false;
+    return 0;
   }
   void set_error_msg(const char *msg);
 
