@@ -1,7 +1,7 @@
 #ifndef SQL_HASH_JOIN_BUFFER_H_
 #define SQL_HASH_JOIN_BUFFER_H_
 
-/* Copyright (c) 2019, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -54,6 +54,12 @@
 /// buffer. As such, we will probably use a little bit more memory than
 /// specified by join_buffer_size.
 ///
+/// Some basic profiling shows that the majority of the time is used on
+/// constructing the hash table. It would be a future improvement to replace
+/// std::unordered_multimap with a more performant hash table, as the literature
+/// suggests that there are better alternatives (see e.g.
+/// https://probablydance.com/2017/02/26/i-wrote-the-fastest-hashtable/).
+///
 /// The primary use case for these classes is, as the name implies,
 /// for implementing hash join.
 
@@ -62,16 +68,14 @@
 #include <memory>
 #include <vector>
 
-#include "extra/robin-hood-hashing/robin_hood.h"
+#include "extra/lz4/my_xxhash.h"
 #include "field_types.h"
 #include "map_helpers.h"
 #include "my_alloc.h"
 #include "my_inttypes.h"
 #include "my_table_map.h"
 #include "prealloced_array.h"
-#include "sql/immutable_string.h"
 #include "sql/item_cmpfunc.h"
-#include "sql/pack_rows.h"
 #include "sql/table.h"
 #include "sql_string.h"
 
@@ -80,6 +84,85 @@ class QEP_TAB;
 
 namespace hash_join_buffer {
 
+/// A class that represents a field, which also holds a cached value of the
+/// field's data type.
+struct Column {
+  explicit Column(Field *field);
+  Field *const field;
+
+  // The field type is used frequently, and caching it gains around 30% in some
+  // of our microbenchmarks.
+  const enum_field_types field_type;
+};
+
+/// This struct is primarily used for holding the extracted columns in a hash
+/// join. When the hash join iterator is constructed, we extract the columns
+/// that are needed to satisfy the SQL query.
+struct Table {
+  explicit Table(QEP_TAB *qep_tab);
+  QEP_TAB *qep_tab;
+  Prealloced_array<Column, 8> columns;
+
+  // Whether to copy the NULL flags or not.
+  bool copy_null_flags{false};
+
+  // A cached value of QEP_TAB::rowid_status. This determines whether we need to
+  // copy/restore the row ID for each row, and how we should retrieve the row ID
+  // (i.e., should we call handler::position() or not). See the comment on
+  // QEP_TAB::rowid_status for more details.
+  rowid_statuses rowid_status;
+};
+
+/// A structure that contains a list of tables for the hash join operation,
+/// and some pre-computed properties for the tables.
+class TableCollection {
+ public:
+  TableCollection() = default;
+
+  explicit TableCollection(const std::vector<QEP_TAB *> &tables);
+
+  const Prealloced_array<Table, 4> &tables() const { return m_tables; }
+
+  table_map tables_bitmap() const { return m_tables_bitmap; }
+
+  size_t ref_and_null_bytes_size() const { return m_ref_and_null_bytes_size; }
+
+  bool has_blob_column() const { return m_has_blob_column; }
+
+ private:
+  Prealloced_array<Table, 4> m_tables{PSI_NOT_INSTRUMENTED};
+
+  // We frequently use the bitmap to determine which side of the join an Item
+  // belongs to, so precomputing the bitmap saves quite some time.
+  table_map m_tables_bitmap;
+
+  // Sum of the NULL bytes and the row ID for all of the tables.
+  size_t m_ref_and_null_bytes_size;
+
+  // Whether any of the tables has a BLOB/TEXT column. This is used to determine
+  // whether we need to estimate the row size every time we store a row to the
+  // row buffer or to a chunk file on disk. If this is set to false, we can
+  // pre-allocate any necessary buffers we need during the hash join, and thus
+  // eliminate the need for recalculating the row size every time.
+  bool m_has_blob_column;
+};
+
+/// Count up how many bytes a single row from the given tables will occupy,
+/// in "packed" format. Note that this is an upper bound, so the length after
+/// calling Field::pack may very well be shorter than the size returned by this
+/// function.
+///
+/// The value returned from this function will sum up
+/// 1) The row-id if that is to be kept.
+/// 2) Size of the NULL flags.
+/// 3) Size of the buffer returned by pack() on all columns marked in the
+///    read_set.
+///
+/// Note that if any of the tables has a BLOB/TEXT column, this function looks
+/// at the data stored in the record buffers. This means that the function can
+/// not be called before reading any rows if tables.has_blob_column is true.
+size_t ComputeRowSizeUpperBound(const TableCollection &tables);
+
 /// The key type for the hash structure in HashJoinRowBuffer.
 ///
 /// A key consists of the value from one or more columns, taken from the join
@@ -132,64 +215,47 @@ class Key {
   const size_t m_size;
 };
 
-class KeyEquals {
- public:
-  // This is a marker from C++17 that signals to the container that
-  // operator() can be called with arguments of which one of the types
-  // differs from the container's key type (ImmutableStringWithLength),
-  // and thus enables map.find(Key). The type itself does not matter.
-  using is_transparent = void;
-
-  bool operator()(const Key &str1,
-                  const ImmutableStringWithLength &other) const {
-    ImmutableStringWithLength::Decoded str2 = other.Decode();
-    if (str1.size() != str2.size) {
-      return false;
-    } else if (str1.size() == 0) {
-      return true;
-    } else {
-      return memcmp(str1.data(), str2.data, str1.size()) == 0;
-    }
-  }
-
-  bool operator()(const ImmutableStringWithLength &str1,
-                  const ImmutableStringWithLength &str2) const {
-    return str1 == str2;
-  }
-};
-
 // A row in the hash join buffer is generally the same as the Key class. In
 // C++17, both of them should most likely be replaced with std::string_view. For
 // now, we create an alias to distinguish between Key and a row.
 using BufferRow = Key;
 
+/// We rely on xxHash64 to do the hashing of the key. xxHash64 was chosen as it
+/// both is very fast _and_ produces reasonably good-quality hashes
+/// (see https://github.com/rurban/smhasher).
 class KeyHasher {
  public:
-  // This is a marker from C++17 that signals to the container that
-  // operator() can be called with an argument that differs from the
-  // container's key type (ImmutableStringWithLength), and thus enables
-  // map.find(Key). The type itself does not matter.
-  using is_transparent = void;
+  explicit KeyHasher(uint32_t seed) : m_seed(seed) {}
 
   size_t operator()(hash_join_buffer::Key key) const {
-    return robin_hood::hash_bytes(key.data(), key.size());
+    return MY_XXH64(key.data(), key.size(), m_seed);
   }
 
-  size_t operator()(ImmutableStringWithLength key) const {
-    ImmutableStringWithLength::Decoded decoded = key.Decode();
-    return robin_hood::hash_bytes(decoded.data, decoded.size);
-  }
+ private:
+  const uint32_t m_seed;
 };
 
-// A convenience form of LoadIntoTableBuffers() that also verifies the end
-// pointer for us.
-void LoadBufferRowIntoTableBuffers(const pack_rows::TableCollection &tables,
-                                   BufferRow row);
+/// Take the data marked for reading in "tables" and store it in the provided
+/// buffer. What data to store is determined by the read set of each table.
+/// Note that any existing data in "buffer" will be overwritten.
+///
+/// The output buffer will contain three things:
+///
+/// 1) NULL flags for each nullable column.
+/// 2) The row ID for each row. This is only stored if QEP_TAB::rowid_status !=
+///    NO_ROWID_NEEDED.
+/// 3) The actual data from the columns.
+///
+/// @retval true if error, false otherwise
+bool StoreFromTableBuffers(const TableCollection &tables, String *buffer);
 
-// A convenience form of the above that also decodes the LinkedImmutableString
-// for us.
-void LoadImmutableStringIntoTableBuffers(
-    const pack_rows::TableCollection &tables, LinkedImmutableString row);
+/// Take the data in "row" and put it back to the tables' record buffers. The
+/// tables must be _exactly_ the same as when the BufferRow was created.
+/// That is, it must contain the same tables in the same order, and the read set
+/// of each table must be identical when storing and restoring the row.
+/// If that's not the case, you will end up with undefined and unpredictable
+/// behavior.
+void LoadIntoTableBuffers(const TableCollection &tables, BufferRow row);
 
 enum class StoreRowResult { ROW_STORED, BUFFER_FULL, FATAL_ERROR };
 
@@ -197,105 +263,64 @@ class HashJoinRowBuffer {
  public:
   // Construct the buffer. Note that Init() must be called before the buffer can
   // be used.
-  HashJoinRowBuffer(pack_rows::TableCollection tables,
+  HashJoinRowBuffer(TableCollection tables,
                     std::vector<HashJoinCondition> join_conditions,
                     size_t max_mem_available_bytes);
 
   // Initialize the HashJoinRowBuffer so it is ready to store rows. This
   // function can be called multiple times; subsequent calls will only clear the
   // buffer for existing rows.
-  bool Init();
+  bool Init(uint32_t hash_seed);
 
   /// Store the row that is currently lying in the tables record buffers.
   /// The hash map key is extracted from the join conditions that the row buffer
   /// holds.
   ///
   /// @param thd the thread handler
-  /// @param reject_duplicate_keys If true, reject rows with duplicate keys.
-  ///        If a row is rejected, the function will still return ROW_STORED.
-  /// @param store_rows_with_null_in_condition Whether to store rows where the
-  ///        join conditions contains SQL NULL.
   ///
   /// @retval ROW_STORED the row was stored.
   /// @retval BUFFER_FULL the row was stored, and the buffer is full.
   /// @retval FATAL_ERROR an unrecoverable error occured (most likely,
   ///         malloc failed). It is the callers responsibility to call
   ///         my_error().
-  StoreRowResult StoreRow(THD *thd, bool reject_duplicate_keys,
-                          bool store_rows_with_null_in_condition);
+  StoreRowResult StoreRow(THD *thd);
 
   size_t size() const { return m_hash_map->size(); }
 
   bool empty() const { return m_hash_map->empty(); }
 
-  bool inited() const { return m_hash_map != nullptr; }
-
-  using hash_map_type = robin_hood::unordered_flat_map<
-      ImmutableStringWithLength, LinkedImmutableString, KeyHasher, KeyEquals>;
+  using hash_map_type = memroot_unordered_multimap<Key, BufferRow, KeyHasher>;
 
   using hash_map_iterator = hash_map_type::const_iterator;
 
-  hash_map_iterator find(const Key &key) const { return m_hash_map->find(key); }
+  std::pair<hash_map_iterator, hash_map_iterator> equal_range(
+      const Key &key) const {
+    return m_hash_map->equal_range(key);
+  }
 
   hash_map_iterator begin() const { return m_hash_map->begin(); }
 
   hash_map_iterator end() const { return m_hash_map->end(); }
 
-  LinkedImmutableString LastRowStored() const {
-    assert(Initialized());
-    return m_last_row_stored;
-  }
-
-  bool Initialized() const { return m_hash_map.get() != nullptr; }
-
-  bool contains(const Key &key) const { return find(key) != end(); }
-
  private:
   const std::vector<HashJoinCondition> m_join_conditions;
 
   // A row can consist of parts from different tables. This structure tells us
   // which tables that are involved.
-  const pack_rows::TableCollection m_tables;
+  const TableCollection m_tables;
 
-  // The MEM_ROOT on which all of the hash table keys and values are allocated.
-  // The actual hash map is on the regular heap.
+  // The MEM_ROOT on which all of the hash table data is allocated.
   MEM_ROOT m_mem_root;
 
-  // A MEM_ROOT used only for storing the final row (possibly both key and
-  // value). The code assumes fairly deeply that inserting a row never fails, so
-  // when m_mem_root goes full (we set a capacity on it to ensure that the last
-  // allocated block does not get too big), we allocate the very last row on
-  // this MEM_ROOT and the signal fullness so that we can start spilling to
-  // disk.
-  MEM_ROOT m_overflow_mem_root;
-
   // The hash table where the rows are stored.
-  std::unique_ptr<hash_map_type> m_hash_map;
+  unique_ptr_destroy_only<hash_map_type> m_hash_map;
 
   // A buffer we can use when we are constructing a join key from a join
   // condition. In order to avoid reallocating memory, the buffer never shrinks.
   String m_buffer;
-  size_t m_row_size_upper_bound;
 
   // The maximum size of the buffer, given in bytes.
   const size_t m_max_mem_available;
-
-  // The last row that was stored in the hash table, or nullptr if the hash
-  // table is empty. We may have to put this row back into the tables' record
-  // buffers if we have a child iterator that expects the record buffers to
-  // contain the last row returned by the storage engine (the probe phase of
-  // hash join may put any row in the hash table in the tables' record buffer).
-  // See HashJoinIterator::BuildHashTable() for an example of this.
-  LinkedImmutableString m_last_row_stored{nullptr};
-
-  // Fetch the relevant fields from each table, and pack them into m_mem_root
-  // as a LinkedImmutableString where the “next” pointer points to “next_ptr”.
-  // If that does not work (capacity reached), pack into m_overflow_mem_root
-  // instead and set “full” to true. If _that_ does not work (fatally out
-  // of memory), returns nullptr. Otherwise, returns a pointer to the newly
-  // packed string.
-  LinkedImmutableString StoreLinkedImmutableStringFromTableBuffers(
-      LinkedImmutableString next_ptr, bool *full);
 };
 
 }  // namespace hash_join_buffer
