@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,9 +39,9 @@
 #include "my_loglevel.h"
 #include "my_psi_config.h"
 #include "my_systime.h"  // my_micro_time()
-#include "mysql/components/services/bits/mysql_mutex_bits.h"
-#include "mysql/components/services/bits/psi_mutex_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/components/services/mysql_mutex_bits.h"
+#include "mysql/components/services/psi_mutex_bits.h"
 #include "thr_mutex.h"
 
 #ifndef _WIN32
@@ -111,7 +110,7 @@ Host_errors::Host_errors()
       m_init_connect(0),
       m_local(0) {}
 
-Host_errors::~Host_errors() = default;
+Host_errors::~Host_errors() {}
 
 void Host_errors::reset() {
   m_connect = 0;
@@ -201,14 +200,14 @@ bool hostname_cache_init(uint size) {
                    MY_MUTEX_INIT_FAST);
   hostname_cache_mutex_inited = true;
 
-  return false;
+  return 0;
 }
 
 void hostname_cache_free() {
   delete hostname_cache_by_ip;
-  hostname_cache_by_ip = nullptr;
+  hostname_cache_by_ip = NULL;
   delete hostname_cache_lru;
-  hostname_cache_lru = nullptr;
+  hostname_cache_lru = NULL;
 
   if (hostname_cache_mutex_inited) {
     mysql_mutex_destroy(&hostname_cache_mutex);
@@ -259,9 +258,9 @@ static void add_hostname_impl(const char *ip_string, const char *hostname,
 
   entry = hostname_cache_search(ip_string);
 
-  if (likely(entry == nullptr)) {
+  if (likely(entry == NULL)) {
     entry = new (std::nothrow) Host_entry;
-    if (entry == nullptr) return;
+    if (entry == NULL) return;
 
     need_add = true;
     strcpy(entry->ip_key, ip_string);
@@ -277,7 +276,7 @@ static void add_hostname_impl(const char *ip_string, const char *hostname,
   }
 
   if (validated) {
-    if (hostname != nullptr) {
+    if (hostname != NULL) {
       size_t len = strlen(hostname);
       if (len > sizeof(entry->m_hostname) - 1)
         len = sizeof(entry->m_hostname) - 1;
@@ -321,7 +320,7 @@ static void add_hostname_impl(const char *ip_string, const char *hostname,
       hostname_cache_by_ip->erase(hostname_cache_lru->front()->ip_key);
       hostname_cache_lru->pop_front();
     }
-    assert(hostname_cache_lru->size() < hostname_cache_max_size);
+    DBUG_ASSERT(hostname_cache_lru->size() < hostname_cache_max_size);
     hostname_cache_lru->emplace_front(entry);
     hostname_cache_by_ip->emplace(entry->ip_key, hostname_cache_lru->begin());
   }
@@ -430,13 +429,12 @@ static inline bool is_hostname_valid(const char *hostname) {
 
   @param [in]  ip_storage IP address (sockaddr). Must be set.
   @param [in]  ip_string  IP address (string). Must be set.
-  @param [out] hostname   Hostname if IP-address is valid.
-  @param [out] connect_errors  Represents number of connection errors.
+  @param [out] hostname
+  @param [out] connect_errors
 
   @return Error status
   @retval 0 Success
   @retval RC_BLOCKED_HOST The host is blocked.
-  @retval RC_LONG_HOSTNAME The hostname is longer than HOSTNAME_LENGTH.
 
   The function does not set/report MySQL server error in case of failure.
   It's caller's responsibility to handle failures of this function
@@ -456,7 +454,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
              ("IP address: '%s'; family: %d.", ip_string, (int)ip->sa_family));
 
   /* Default output values, for most cases. */
-  *hostname = nullptr;
+  *hostname = NULL;
   *connect_errors = 0;
 
   /* Check if we have loopback address (127.0.0.1 or ::1). */
@@ -518,7 +516,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
   DBUG_PRINT("info", ("Resolving '%s'...", (const char *)ip_string));
 
   err_code =
-      vio_getnameinfo(ip, hostname_buffer, NI_MAXHOST, nullptr, 0, NI_NAMEREQD);
+      vio_getnameinfo(ip, hostname_buffer, NI_MAXHOST, NULL, 0, NI_NAMEREQD);
 
   /*
   ===========================================================================
@@ -558,13 +556,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
   });
 
   DBUG_EXECUTE_IF("getnameinfo_fake_max_length", {
-    std::string s(HOSTNAME_LENGTH, 'a');
-    strcpy(hostname_buffer, s.c_str());
-    err_code = 0;
-  });
-
-  DBUG_EXECUTE_IF("getnameinfo_fake_max_length_plus_1", {
-    std::string s(HOSTNAME_LENGTH + 1, 'a');
+    std::string s(NI_MAXHOST - 1, 'a');
     strcpy(hostname_buffer, s.c_str());
     err_code = 0;
   });
@@ -601,7 +593,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
       errors.m_nameinfo_transient = 1;
       validated = false;
     }
-    add_hostname(ip_string, nullptr, validated, &errors);
+    add_hostname(ip_string, NULL, validated, &errors);
 
     return 0;
   }
@@ -639,11 +631,6 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
     return false;
   }
 
-  /* Prevent hostnames longer than HOSTNAME_LENGTH from connecting */
-  if (strlen(hostname_buffer) > HOSTNAME_LENGTH) {
-    return RC_LONG_HOSTNAME;
-  }
-
   /* Get IP-addresses for the resolved host name (FCrDNS technique). */
 
   struct addrinfo hints;
@@ -662,7 +649,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
   DBUG_PRINT("info",
              ("Getting IP addresses for hostname '%s'...", hostname_buffer));
 
-  err_code = getaddrinfo(hostname_buffer, nullptr, &hints, &addr_info_list);
+  err_code = getaddrinfo(hostname_buffer, NULL, &hints, &addr_info_list);
   if (err_code == 0) free_addr_info_list = true;
 
   /*
@@ -674,7 +661,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
   DBUG_EXECUTE_IF("getaddrinfo_error_noname", {
     if (free_addr_info_list) freeaddrinfo(addr_info_list);
 
-    addr_info_list = nullptr;
+    addr_info_list = NULL;
     err_code = EAI_NONAME;
     free_addr_info_list = false;
   });
@@ -682,7 +669,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
   DBUG_EXECUTE_IF("getaddrinfo_error_again", {
     if (free_addr_info_list) freeaddrinfo(addr_info_list);
 
-    addr_info_list = nullptr;
+    addr_info_list = NULL;
     err_code = EAI_AGAIN;
     free_addr_info_list = false;
   });
@@ -714,7 +701,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
 
     debug_addr_info[1].ai_addr = (struct sockaddr *)&debug_sock_addr[1];
     debug_addr_info[1].ai_addrlen = sizeof(struct sockaddr_in);
-    debug_addr_info[1].ai_next = nullptr;
+    debug_addr_info[1].ai_next = NULL;
 
     addr_info_list = &debug_addr_info[0];
     err_code = 0;
@@ -743,7 +730,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
 
     debug_addr_info[1].ai_addr = (struct sockaddr *)&debug_sock_addr[1];
     debug_addr_info[1].ai_addrlen = sizeof(struct sockaddr_in);
-    debug_addr_info[1].ai_next = nullptr;
+    debug_addr_info[1].ai_next = NULL;
 
     addr_info_list = &debug_addr_info[0];
     err_code = 0;
@@ -811,7 +798,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
 
     debug_addr_info[1].ai_addr = (struct sockaddr *)&debug_sock_addr[1];
     debug_addr_info[1].ai_addrlen = sizeof(struct sockaddr_in6);
-    debug_addr_info[1].ai_next = nullptr;
+    debug_addr_info[1].ai_next = NULL;
 
     addr_info_list = &debug_addr_info[0];
     err_code = 0;
@@ -878,7 +865,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
 
     debug_addr_info[1].ai_addr = (struct sockaddr *)&debug_sock_addr[1];
     debug_addr_info[1].ai_addrlen = sizeof(struct sockaddr_in6);
-    debug_addr_info[1].ai_next = nullptr;
+    debug_addr_info[1].ai_next = NULL;
 
     addr_info_list = &debug_addr_info[0];
     err_code = 0;
@@ -911,7 +898,7 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
       errors.m_addrinfo_transient = 1;
       validated = false;
     }
-    add_hostname(ip_string, nullptr, validated, &errors);
+    add_hostname(ip_string, NULL, validated, &errors);
 
     return false;
   }
@@ -926,11 +913,11 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
     char ip_buffer[HOST_ENTRY_KEY_SIZE];
 
     {
-      bool err_status [[maybe_unused]];
+      bool err_status MY_ATTRIBUTE((unused));
       err_status = vio_get_normalized_ip_string(addr_info->ai_addr,
                                                 addr_info->ai_addrlen,
                                                 ip_buffer, sizeof(ip_buffer));
-      assert(!err_status);
+      DBUG_ASSERT(!err_status);
     }
 
     DBUG_PRINT("info", ("  - '%s'", ip_buffer));
@@ -966,13 +953,13 @@ int ip_to_hostname(struct sockaddr_storage *ip_storage, const char *ip_string,
          addr_info = addr_info->ai_next) {
       char ip_buffer[HOST_ENTRY_KEY_SIZE];
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       bool err_status =
 #endif
           vio_get_normalized_ip_string(addr_info->ai_addr,
                                        addr_info->ai_addrlen, ip_buffer,
                                        sizeof(ip_buffer));
-      assert(!err_status);
+      DBUG_ASSERT(!err_status);
 
       LogErr(INFORMATION_LEVEL, ER_ADDRESSES_FOR_HOSTNAME_LIST_ITEM, ip_buffer);
     }
