@@ -27,13 +27,16 @@
 
 #include <string.h>
 #include <sys/types.h>
-#include <memory>
 
 #include "field_types.h"
+#include "json_dom.h"  // Json_wrapper
+//#include "extra/regex/my_regex.h"  // my_regex_t
+#include "m_ctype.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
+#include "my_sys.h"
 #include "my_table_map.h"
 #include "my_time.h"
 #include "mysql/udf_registration_types.h"
@@ -45,6 +48,7 @@
 #include "sql/mem_root_array.h"  // Mem_root_array
 #include "sql/my_decimal.h"
 #include "sql/parse_tree_node_base.h"
+#include "sql/psi_memory_key.h"  // key_memory_JSON
 #include "sql/sql_const.h"
 #include "sql/sql_list.h"
 #include "sql/table.h"
@@ -63,7 +67,6 @@ class PT_item_list;
 class QEP_TAB;
 class SELECT_LEX;
 class THD;
-struct CHARSET_INFO;
 struct MY_BITMAP;
 
 Item *make_condition(Parse_context *pc, Item *item);
@@ -213,7 +216,7 @@ class Arg_comparator {
   int compare_json();
   bool compare_null_values();
 
-  static bool can_compare_as_dates(const Item *a, const Item *b);
+  static bool can_compare_as_dates(Item *a, Item *b);
 
   Item **cache_converted_constant(THD *thd, Item **value, Item **cache,
                                   Item_result type);
@@ -472,7 +475,7 @@ class Item_in_optimizer final : public Item_bool_func {
   Item_in_optimizer(Item *a, Item_in_subselect *b)
       : Item_bool_func(a, reinterpret_cast<Item *>(b)),
         cache(0),
-        save_cache(false),
+        save_cache(0),
         result_for_null_param(UNKNOWN) {
     set_subquery();
   }
@@ -558,8 +561,8 @@ class Gt_creator : public Comp_creator {
   virtual ~Gt_creator() {} /* Remove gcc warning */
   virtual Item_bool_func *create(Item *a, Item *b) const;
   virtual const char *symbol(bool invert) const { return invert ? "<=" : ">"; }
-  virtual bool eqne_op() const { return false; }
-  virtual bool l_op() const { return false; }
+  virtual bool eqne_op() const { return 0; }
+  virtual bool l_op() const { return 0; }
 };
 
 class Lt_creator : public Comp_creator {
@@ -568,8 +571,8 @@ class Lt_creator : public Comp_creator {
   virtual ~Lt_creator() {} /* Remove gcc warning */
   virtual Item_bool_func *create(Item *a, Item *b) const;
   virtual const char *symbol(bool invert) const { return invert ? ">=" : "<"; }
-  virtual bool eqne_op() const { return false; }
-  virtual bool l_op() const { return true; }
+  virtual bool eqne_op() const { return 0; }
+  virtual bool l_op() const { return 1; }
 };
 
 class Ge_creator : public Comp_creator {
@@ -578,8 +581,8 @@ class Ge_creator : public Comp_creator {
   virtual ~Ge_creator() {} /* Remove gcc warning */
   virtual Item_bool_func *create(Item *a, Item *b) const;
   virtual const char *symbol(bool invert) const { return invert ? "<" : ">="; }
-  virtual bool eqne_op() const { return false; }
-  virtual bool l_op() const { return false; }
+  virtual bool eqne_op() const { return 0; }
+  virtual bool l_op() const { return 0; }
 };
 
 class Le_creator : public Comp_creator {
@@ -588,8 +591,8 @@ class Le_creator : public Comp_creator {
   virtual ~Le_creator() {} /* Remove gcc warning */
   virtual Item_bool_func *create(Item *a, Item *b) const;
   virtual const char *symbol(bool invert) const { return invert ? ">" : "<="; }
-  virtual bool eqne_op() const { return false; }
-  virtual bool l_op() const { return true; }
+  virtual bool eqne_op() const { return 0; }
+  virtual bool l_op() const { return 1; }
 };
 
 class Item_bool_func2 : public Item_bool_func { /* Bool with 2 string args */
@@ -883,8 +886,8 @@ class Item_func_not_all : public Item_func_not {
         test_sum_item(0),
         test_sub_item(0),
         subselect(0),
-        abort_on_null(false),
-        show(false) {}
+        abort_on_null(0),
+        show(0) {}
   void apply_is_true() override { abort_on_null = true; }
   /// Treat UNKNOWN result like FALSE because callers see no difference
   bool ignore_unknown() const { return abort_on_null; }
@@ -1141,17 +1144,17 @@ class Item_func_opt_neg : public Item_int_func {
   bool pred_level; /* <=> [NOT] <func> is used on a predicate level */
  public:
   Item_func_opt_neg(const POS &pos, Item *a, Item *b, Item *c, bool is_negation)
-      : Item_int_func(pos, a, b, c), negated(false), pred_level(false) {
+      : Item_int_func(pos, a, b, c), negated(0), pred_level(0) {
     if (is_negation) negate();
   }
   Item_func_opt_neg(const POS &pos, PT_item_list *list, bool is_negation)
-      : Item_int_func(pos, list), negated(false), pred_level(false) {
+      : Item_int_func(pos, list), negated(0), pred_level(0) {
     if (is_negation) negate();
   }
 
  public:
   inline void negate() { negated = !negated; }
-  inline void apply_is_true() override { pred_level = true; }
+  inline void apply_is_true() override { pred_level = 1; }
   bool ignore_unknown() const { return pred_level; }
   Item *truth_transformer(THD *, Bool_test test) override {
     if (test != BOOL_NEGATED) return nullptr;
@@ -1210,7 +1213,6 @@ class Item_func_strcmp final : public Item_bool_func2 {
   longlong val_int() override;
   optimize_type select_optimize(const THD *) override { return OPTIMIZE_NONE; }
   const char *func_name() const override { return "strcmp"; }
-  enum Functype functype() const override { return STRCMP_FUNC; }
 
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override {
@@ -1697,27 +1699,21 @@ class cmp_item_string final : public cmp_item_scalar {
 class cmp_item_json final : public cmp_item_scalar {
  private:
   /// Cached JSON value to look up
-  unique_ptr_destroy_only<Json_wrapper> m_value;
+  Json_wrapper m_value;
   /// Cache for the value above
-  unique_ptr_destroy_only<Json_scalar_holder> m_holder;
+  Json_scalar_holder m_holder;
   /// String buffer
   String m_str_value;
+  /// Scalar holder for the RHS value
+  Json_scalar_holder m_itm_holder;
 
  public:
-  /**
-    Construct a cmp_item_json object.
-    @param wrapper a Json_wrapper for holding the JSON value in the comparison
-    @param holder  pre-alloced memory for creating JSON scalar values without
-                   using the heap
-  */
-  cmp_item_json(unique_ptr_destroy_only<Json_wrapper> wrapper,
-                unique_ptr_destroy_only<Json_scalar_holder> holder);
-  ~cmp_item_json() override;
+  cmp_item_json() {}
 
-  int compare(const cmp_item *ci) const override;
-  void store_value(Item *item) override;
-  int cmp(Item *arg) override;
-  cmp_item *make_same() override;
+  virtual int compare(const cmp_item *ci) const;
+  virtual void store_value(Item *item);
+  virtual int cmp(Item *arg);
+  virtual cmp_item *make_same();
 };
 
 class cmp_item_int final : public cmp_item_scalar {
@@ -2221,20 +2217,20 @@ class Item_cond : public Item_bool_func {
 
  public:
   /* Item_cond() is only used to create top level items */
-  Item_cond() : Item_bool_func(), abort_on_null(true) {}
-  Item_cond(Item *i1, Item *i2) : Item_bool_func(), abort_on_null(false) {
+  Item_cond() : Item_bool_func(), abort_on_null(1) {}
+  Item_cond(Item *i1, Item *i2) : Item_bool_func(), abort_on_null(0) {
     list.push_back(i1);
     list.push_back(i2);
   }
   Item_cond(const POS &pos, Item *i1, Item *i2)
-      : Item_bool_func(pos), abort_on_null(false) {
+      : Item_bool_func(pos), abort_on_null(0) {
     list.push_back(i1);
     list.push_back(i2);
   }
 
   Item_cond(THD *thd, Item_cond *item);
   Item_cond(List<Item> &nlist)
-      : Item_bool_func(), list(nlist), abort_on_null(false) {}
+      : Item_bool_func(), list(nlist), abort_on_null(0) {}
   bool add(Item *item) {
     DBUG_ASSERT(item);
     return list.push_back(item);
@@ -2362,7 +2358,7 @@ class Item_equal final : public Item_bool_func {
 
  public:
   inline Item_equal()
-      : Item_bool_func(), const_item(0), eval_item(0), cond_false(false) {}
+      : Item_bool_func(), const_item(0), eval_item(0), cond_false(0) {}
   Item_equal(Item_field *f1, Item_field *f2);
   Item_equal(Item *c, Item_field *f);
   Item_equal(Item_equal *item_equal);
