@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,11 +22,8 @@
 
 #include "sql/item_geofunc_internal.h"
 
+#include <string.h>
 #include <algorithm>
-#include <cstring>
-#include <iterator>
-#include <memory>
-
 #include <boost/concept/usage.hpp>
 #include <boost/geometry/algorithms/centroid.hpp>
 #include <boost/geometry/algorithms/is_valid.hpp>
@@ -36,6 +33,8 @@
 #include <boost/geometry/index/predicates.hpp>
 #include <boost/geometry/strategies/strategies.hpp>
 #include <boost/iterator/iterator_facade.hpp>
+#include <iterator>
+#include <memory>
 
 #include "m_ctype.h"
 #include "m_string.h"
@@ -46,8 +45,8 @@
 #include "sql/dd/cache/dictionary_client.h"
 #include "sql/item_func.h"
 #include "sql/mdl.h"
-#include "sql/parse_location.h"  // POS
-#include "sql/sql_class.h"       // THD
+#include "sql/parse_tree_node_base.h"
+#include "sql/sql_class.h"  // THD
 #include "sql/srs_fetcher.h"
 #include "sql/system_variables.h"
 #include "sql_string.h"
@@ -59,10 +58,10 @@ class Spatial_reference_system;
 
 bool Srs_fetcher::lock(gis::srid_t srid, enum_mdl_type lock_type) {
   DBUG_TRACE;
-  assert(srid != 0);
+  DBUG_ASSERT(srid != 0);
 
   char id_str[11];  // uint32 => max 10 digits + \0
-  longlong10_to_str(srid, id_str, 10);
+  int10_to_str(srid, id_str, 10);
 
   MDL_request mdl_request;
   mdl_request.init_with_source(MDL_key::SRID, "", id_str, lock_type,
@@ -97,7 +96,7 @@ bool Srs_fetcher::acquire_for_modification(gis::srid_t srid,
 }
 
 bool Srs_fetcher::srs_exists(THD *thd, gis::srid_t srid, bool *exists) {
-  assert(exists);
+  DBUG_ASSERT(exists);
   std::unique_ptr<dd::cache::Dictionary_client::Auto_releaser> releaser(
       new dd::cache::Dictionary_client::Auto_releaser(thd->dd_client()));
   Srs_fetcher fetcher(thd);
@@ -107,12 +106,20 @@ bool Srs_fetcher::srs_exists(THD *thd, gis::srid_t srid, bool *exists) {
   return false;
 }
 
+/**
+  Merge all components as appropriate so that the object contains only
+  components that don't overlap.
+
+  @tparam Coordsys Coordinate system type, specified using those defined in
+          boost::geometry::cs.
+  @param[out] pnull_value takes back null_value set during the operation.
+ */
 template <typename Coordsys>
 void BG_geometry_collection::merge_components(bool *pnull_value) {
   if (is_comp_no_overlapped()) return;
 
-  POS pos{{nullptr, nullptr}, {nullptr, nullptr}};
-  Item_func_st_union ifsu(pos, nullptr, nullptr);
+  POS pos;
+  Item_func_st_union ifsu(pos, NULL, NULL);
   bool do_again = true;
   uint32 last_composition[6] = {0}, num_unchanged_composition = 0;
   size_t last_num_geos = 0;
@@ -206,7 +213,7 @@ bool linear_areal_intersect_infinite(Geometry *g1, Geometry *g2,
                               g2->get_flags(), g2->get_srid());
       res = bg::is_valid(mplgn);
     } else
-      assert(false);
+      DBUG_ASSERT(false);
 
     return res;
   }
@@ -280,7 +287,7 @@ class Is_target_geometry {
 
 class Rtree_entry_compare {
  public:
-  Rtree_entry_compare() = default;
+  Rtree_entry_compare() {}
 
   bool operator()(const BG_rtree_entry &re1, const BG_rtree_entry &re2) const {
     return re1.second < re2.second;
@@ -300,7 +307,7 @@ class Rtree_entry_compare {
 template <typename Coordsys>
 bool BG_geometry_collection::merge_one_run(Item_func_st_union *ifsu,
                                            bool *pnull_value) {
-  Geometry *gres = nullptr;
+  Geometry *gres = NULL;
   bool has_new = false;
   bool &null_value = *pnull_value;
   Pointer_vector<Geometry> added;
@@ -329,7 +336,7 @@ bool BG_geometry_collection::merge_one_run(Item_func_st_union *ifsu,
       min corner point have greater coordinates than the max corner point,
       the box isn't valid and the rtree can be empty.
      */
-    assert(rtree_result.size() != 0);
+    DBUG_ASSERT(rtree_result.size() != 0);
 
     // Sort rtree_result by Rtree_entry::second in order to make
     // components in fixed order.
@@ -342,7 +349,7 @@ bool BG_geometry_collection::merge_one_run(Item_func_st_union *ifsu,
     for (Rtree_result::iterator j = rtree_result.begin();
          j != rtree_result.end(); ++j) {
       Geometry *geom2 = m_geos[j->second];
-      if (*i == geom2 || geom2 == nullptr) continue;
+      if (*i == geom2 || geom2 == NULL) continue;
 
       // Equals is much easier and faster to check, so check it first.
       if (Item_func_spatial_rel::bg_geo_relation_check(
@@ -372,7 +379,7 @@ bool BG_geometry_collection::merge_one_run(Item_func_st_union *ifsu,
       if (Item_func_spatial_rel::bg_geo_relation_check(
               geom2, *i, Item_func::SP_WITHIN_FUNC, &null_value) &&
           !null_value) {
-        m_geos[j->second] = nullptr;
+        m_geos[j->second] = NULL;
         continue;
       }
 
@@ -390,8 +397,8 @@ bool BG_geometry_collection::merge_one_run(Item_func_st_union *ifsu,
       */
       char d11 = (*i)->feature_dimension();
       char d12 = geom2->feature_dimension();
-      Geometry *geom_d1 = nullptr;
-      Geometry *geom_d2 = nullptr;
+      Geometry *geom_d1 = NULL;
+      Geometry *geom_d2 = NULL;
       bool is_linear_areal = false;
 
       if (((d11 == 1 && d12 == 2) || (d12 == 1 && d11 == 2))) {
@@ -429,20 +436,20 @@ bool BG_geometry_collection::merge_one_run(Item_func_st_union *ifsu,
         null_value = ifsu->null_value;
 
         if (null_value) {
-          if (gres != nullptr && gres != *i && gres != geom2) delete gres;
+          if (gres != NULL && gres != *i && gres != geom2) delete gres;
           stop_it = true;
           break;
         }
 
         if (gres != *i) *i = NULL;
-        if (gres != geom2) m_geos[j->second] = nullptr;
-        if (gres != nullptr && gres != *i && gres != geom2) {
+        if (gres != geom2) m_geos[j->second] = NULL;
+        if (gres != NULL && gres != *i && gres != geom2) {
           added.push_back(gres);
           String tmp_wkbbuf;
           added_wkbbufs.push_back(tmp_wkbbuf);
           added_wkbbufs.back().takeover(wkbres);
           has_new = true;
-          gres = nullptr;
+          gres = NULL;
         }
         /*
           Done with *i, it's either adopted, or removed or merged to a new
@@ -463,7 +470,7 @@ bool BG_geometry_collection::merge_one_run(Item_func_st_union *ifsu,
   }  // for (*i)
 
   // Remove deleted Geometry object pointers, then append new components if any.
-  Is_target_geometry pred(nullptr);
+  Is_target_geometry pred(NULL);
   Geometry_list::iterator jj =
       std::remove_if(m_geos.begin(), m_geos.end(), pred);
   m_geos.resize(jj - m_geos.begin());
@@ -495,14 +502,14 @@ inline void reassemble_geometry(Geometry *g) {
 template <typename BG_geotype>
 bool post_fix_result(BG_result_buf_mgr *resbuf_mgr, BG_geotype &geout,
                      String *res) {
-  assert(geout.has_geom_header_space());
+  DBUG_ASSERT(geout.has_geom_header_space());
   reassemble_geometry(&geout);
 
   // Such objects returned by BG never have overlapped components.
   if (geout.get_type() == Geometry::wkb_multilinestring ||
       geout.get_type() == Geometry::wkb_multipolygon)
     geout.set_components_no_overlapped(true);
-  if (geout.get_ptr() == nullptr) return true;
+  if (geout.get_ptr() == NULL) return true;
   if (res) {
     char *resptr = geout.get_cptr() - GEOM_HEADER_SIZE;
     size_t len = geout.get_nbytes();
@@ -513,11 +520,9 @@ bool post_fix_result(BG_result_buf_mgr *resbuf_mgr, BG_geotype &geout,
      */
     resbuf_mgr->add_buffer(resptr);
     /*
-      The memory for the result is owned by a BG_result_buf_mgr,
-      so use String::set(char*, size_t, const CHARSET_INFO)
-      which points the internall buffer to the input argument,
-      and sets m_is_alloced = false, signifying the String object
-      does not own the buffer.
+      Pass resptr as const pointer so that the memory space won't be reused
+      by res object. Reuse is forbidden because the memory comes from BG
+      operations and will be freed upon next same val_str call.
     */
     res->set(resptr, len + GEOM_HEADER_SIZE, &my_charset_bin);
 
@@ -555,15 +560,15 @@ class Is_empty_geometry : public WKB_scanner_event_handler {
 
   Is_empty_geometry() : is_empty(true) {}
 
-  void on_wkb_start(Geometry::wkbByteOrder, Geometry::wkbType geotype,
-                    const void *, uint32, bool) override {
+  virtual void on_wkb_start(Geometry::wkbByteOrder, Geometry::wkbType geotype,
+                            const void *, uint32, bool) {
     if (is_empty && geotype != Geometry::wkb_geometrycollection)
       is_empty = false;
   }
 
-  void on_wkb_end(const void *) override {}
+  virtual void on_wkb_end(const void *) {}
 
-  bool continue_scan() const override { return is_empty; }
+  virtual bool continue_scan() const { return is_empty; }
 };
 
 bool is_empty_geocollection(const Geometry *g) {
@@ -580,7 +585,7 @@ bool is_empty_geocollection(const Geometry *g) {
 }
 
 bool is_empty_geocollection(const String &wkbres) {
-  if (wkbres.ptr() == nullptr) return true;
+  if (wkbres.ptr() == NULL) return true;
 
   uint32 geotype = uint4korr(wkbres.ptr() + SRID_SIZE + 1);
 
