@@ -1,19 +1,18 @@
 #ifndef ITEM_SUBSELECT_INCLUDED
 #define ITEM_SUBSELECT_INCLUDED
 
-/* Copyright (c) 2002, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,30 +25,19 @@
 
 /* subselect Item */
 
-#include <assert.h>
+#include <stddef.h>
 #include <sys/types.h>
 
-#include <cstddef>
-#include <memory>  // unique_ptr
-#include <vector>
-
 #include "field_types.h"  // enum_field_types
-#include "my_alloc.h"     // Destroy_only
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_table_map.h"
 #include "my_time.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_time.h"
-#include "sql/comp_creator.h"
 #include "sql/enum_query_type.h"
-#include "sql/item.h"                    // Item_result_field
-#include "sql/iterators/row_iterator.h"  // IWYU pragma: keep
-#include "sql/parse_location.h"          // POS
+#include "sql/item.h"  // Item_result_field
 #include "sql/parse_tree_node_base.h"
-#include "sql/sql_const.h"
-#include "sql/sql_opt_exec_shared.h"
-#include "template_utils.h"
 
 class Comp_creator;
 class Field;
@@ -58,23 +46,28 @@ class Item_in_optimizer;
 class JOIN;
 class Json_wrapper;
 class PT_subquery;
+class QEP_TAB;
 class Query_result_interceptor;
 class Query_result_subquery;
-class Query_result_union;
-class Query_block;
-class Query_expression;
+class SELECT_LEX;
+class SELECT_LEX_UNIT;
 class String;
-class SubqueryWithResult;
 class THD;
 class Temp_table_param;
 class my_decimal;
-class subselect_indexsubquery_engine;
-struct AccessPath;
-class Table_ref;
-
+class subselect_engine;
+struct TABLE_LIST;
 template <class T>
 class List;
 
+/**
+  Convenience typedef used in this file, and further used by any files
+  including this file.
+
+  @retval NULL In case of semantic errors.
+*/
+typedef Comp_creator *(*chooser_compare_func_creator)(bool invert);
+
 /* base class for subselects */
 
 class Item_subselect : public Item_result_field {
@@ -94,67 +87,48 @@ class Item_subselect : public Item_result_field {
   /*
     Used inside Item_subselect::fix_fields() according to this scenario:
       > Item_subselect::fix_fields
-        > subquery->prepare
+        > engine->prepare
           > query_block->prepare
             (Here we realize we need to do the rewrite and set
              substitution= some new Item, eg. Item_in_optimizer )
           < query_block->prepare
-        < subquery->prepare
+        < engine->prepare
         *ref= substitution;
       < Item_subselect::fix_fields
   */
   Item *substitution;
 
+ public:
   /* unit of subquery */
-  Query_expression *unit;
+  SELECT_LEX_UNIT *unit;
   /**
      If !=NO_PLAN_IDX: this Item is in the condition attached to the JOIN_TAB
      having this index in the parent JOIN.
   */
   int in_cond_of_tab;
 
-  // For EXPLAIN.
-  enum enum_engine_type { OTHER_ENGINE, INDEXSUBQUERY_ENGINE, HASH_SJ_ENGINE };
-  enum_engine_type engine_type() const;
-
-  // For EXPLAIN. Only valid if engine_type() == HASH_SJ_ENGINE.
-  const TABLE *get_table() const;
-  const Index_lookup &index_lookup() const;
-  join_type get_join_type() const;
-
-  void create_iterators(THD *thd);
-  virtual AccessPath *root_access_path() const { return nullptr; }
+  /// EXPLAIN needs read-only access to the engine
+  const subselect_engine *get_engine_for_explain() const { return engine; }
 
  protected:
-  /**
-    We need this method, because some compilers do not allow 'this'
-    pointer in constructor initialization list, but we need to pass a pointer
-    to subselect Item class to Query_result_interceptor's constructor.
-  */
-  void init(Query_block *select, Query_result_subquery *result);
-
-  /// The inner part of the subquery.
-  unique_ptr_destroy_only<SubqueryWithResult> subquery;
-
-  /// Only relevant for Item_in_subselect; optimized structure used for
-  /// execution in place of running the entire subquery.
-  subselect_indexsubquery_engine *indexsubquery_engine = nullptr;
-
-  /// cache of used tables
+  /* engine that perform execution of subselect (single select or union) */
+  subselect_engine *engine;
+  /* old engine if engine was changed */
+  subselect_engine *old_engine;
+  /* cache of used external tables */
   table_map used_tables_cache;
-  /// allowed number of columns (1 for scalar subqueries)
+  /* allowed number of columns (1 for single value subqueries) */
   uint max_columns;
-  /// where subquery is placed
+  /* where subquery is placed */
   enum_parsing_context parsing_place;
-  /// work with 'substitution'
+  /* work with 'substitution' */
   bool have_to_be_excluded;
 
  public:
-  /// cache of used tables, used to identify tables added for IN subquery
-  table_map m_subquery_used_tables{0};
-  /// subquery is transformed
+  /* subquery is transformed */
   bool changed;
 
+  enum trans_res { RES_OK, RES_REDUCE, RES_ERROR };
   enum subs_type {
     UNKNOWN_SUBS,
     SINGLEROW_SUBS,
@@ -171,32 +145,48 @@ class Item_subselect : public Item_result_field {
   /// Accumulate properties from underlying query expression
   void accumulate_properties();
   /// Accumulate properties from underlying query block
-  void accumulate_properties(Query_block *select);
+  void accumulate_properties(SELECT_LEX *select);
   /// Accumulate properties from a selected expression within a query block.
   void accumulate_expression(Item *item);
   /// Accumulate properties from a condition or GROUP/ORDER within a query
   /// block.
   void accumulate_condition(Item *item);
+  /// Accumulate properties from a join condition within a query block.
+  void accumulate_join_condition(List<TABLE_LIST> *tables);
 
  public:
   /// Accumulate used tables
   void accumulate_used_tables(table_map add_tables) {
     used_tables_cache |= add_tables;
-    m_subquery_used_tables |= add_tables;
   }
 
   virtual subs_type substype() const { return UNKNOWN_SUBS; }
 
+  /*
+    We need this method, because some compilers do not allow 'this'
+    pointer in constructor initialization list, but we need to pass a pointer
+    to subselect Item class to Query_result_interceptor's constructor.
+  */
+  void init(SELECT_LEX *select, Query_result_subquery *result);
+
+  ~Item_subselect() override;
   void cleanup() override;
-  virtual void reset() { null_value = true; }
-  virtual bool select_transformer(THD *thd, Query_block *select) = 0;
+  virtual void reset() { null_value = 1; }
+  virtual trans_res select_transformer(THD *thd, SELECT_LEX *select) = 0;
   bool assigned() const { return value_assigned; }
   void assigned(bool a) { value_assigned = a; }
   enum Type type() const override;
-  bool is_null() override { return update_null_value() || null_value; }
+  bool is_null() override {
+    /*
+      TODO : Implement error handling for this function as
+      update_null_value() can return error.
+    */
+    (void)update_null_value();
+    return null_value;
+  }
   bool fix_fields(THD *thd, Item **ref) override;
-  void fix_after_pullout(Query_block *parent_query_block,
-                         Query_block *removed_query_block) override;
+  void fix_after_pullout(SELECT_LEX *parent_select,
+                         SELECT_LEX *removed_select) override;
   virtual bool exec(THD *thd);
   bool resolve_type(THD *) override;
   table_map used_tables() const override { return used_tables_cache; }
@@ -205,9 +195,11 @@ class Item_subselect : public Item_result_field {
   void update_used_tables() override;
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
-
-  void set_indexsubquery_engine(subselect_indexsubquery_engine *eng) {
-    indexsubquery_engine = eng;
+  virtual bool have_guarded_conds() { return false; }
+  bool change_engine(subselect_engine *eng) {
+    old_engine = engine;
+    engine = eng;
+    return eng == 0;
   }
 
   /*
@@ -223,13 +215,14 @@ class Item_subselect : public Item_result_field {
   */
   virtual void reset_value_registration() {}
   enum_parsing_context place() { return parsing_place; }
+  bool walk_body(Item_processor processor, enum_walk walk, uchar *arg);
   bool walk(Item_processor processor, enum_walk walk, uchar *arg) override;
   bool explain_subquery_checker(uchar **arg) override;
   bool inform_item_in_cond_of_tab(uchar *arg) override;
   bool clean_up_after_removal(uchar *arg) override;
 
   const char *func_name() const override {
-    assert(0);
+    DBUG_ASSERT(0);
     return "subselect";
   }
 
@@ -240,38 +233,16 @@ class Item_subselect : public Item_result_field {
     return true;
   }
 
-  /// argument used by walk method collect_scalar_subqueries ("css")
-  struct Collect_subq_info {
-    ///< accumulated all subq (or aggregates) found
-    std::vector<Item_subselect *> list;
-    Query_block *m_query_block{nullptr};
-    Collect_subq_info(Query_block *owner) : m_query_block(owner) {}
-    bool contains(Query_expression *candidate) {
-      for (auto sq : list) {
-        if (sq->unit == candidate) return true;
-      }
-      return false;
-    }
-  };
-
-  bool collect_subqueries(uchar *) override;
-  Item *replace_item_field(uchar *arg) override;
-  Item *replace_item_view_ref(uchar *arg) override;
-  Item *replace_item(Item_transformer t, uchar *arg);
-
   friend class Query_result_interceptor;
   friend class Item_in_optimizer;
   friend bool Item_field::fix_fields(THD *, Item **);
   friend int Item_field::fix_outer_field(THD *, Field **, Item **);
   friend bool Item_ref::fix_fields(THD *, Item **);
-  friend void Item_ident::fix_after_pullout(Query_block *parent_query_block,
-                                            Query_block *removed_query_block);
+  friend void Item_ident::fix_after_pullout(SELECT_LEX *parent_select,
+                                            SELECT_LEX *removed_select);
 
  private:
   bool subq_opt_away_processor(uchar *arg) override;
-
- protected:
-  uint unit_cols() const;
 };
 
 /* single value subselect */
@@ -281,15 +252,15 @@ class Item_singlerow_subselect : public Item_subselect {
   Item_cache *value, **row;
   bool no_rows;  ///< @c no_rows_in_result
  public:
-  Item_singlerow_subselect(Query_block *query_block);
+  Item_singlerow_subselect(SELECT_LEX *select_lex);
   Item_singlerow_subselect()
-      : Item_subselect(), value(nullptr), row(nullptr), no_rows(false) {}
+      : Item_subselect(), value(0), row(0), no_rows(false) {}
 
   void cleanup() override;
   subs_type substype() const override { return SINGLEROW_SUBS; }
 
   void reset() override;
-  bool select_transformer(THD *thd, Query_block *select) override;
+  trans_res select_transformer(THD *thd, SELECT_LEX *select) override;
   void store(uint i, Item *item);
   double val_real() override;
   longlong val_int() override;
@@ -309,8 +280,7 @@ class Item_singlerow_subselect : public Item_subselect {
   */
   void no_rows_in_result() override;
 
-  uint cols() const override { return unit_cols(); }
-
+  uint cols() const override;
   /**
     @note that this returns the i-th element of the SELECT list.
     To check for nullability, look at this->maybe_null and not
@@ -325,45 +295,20 @@ class Item_singlerow_subselect : public Item_subselect {
   bool null_inside() override;
   void bring_value() override;
 
-  bool collect_scalar_subqueries(uchar *) override;
-  virtual bool is_maxmin() const { return false; }
-
-  /**
-    Argument for walk method replace_scalar_subquery
-  */
-  struct Scalar_subquery_replacement {
-    Item_singlerow_subselect *m_target;  ///< subquery to be replaced with field
-    Field *m_field;                      ///< the replacement field
-    Query_block *m_outer_query_block;    ///< The transformed query block.
-    Query_block *m_inner_query_block;    ///< The immediately surrounding query
-                                       ///< block. This will be the transformed
-                                       ///< block or a subquery of it
-    bool m_add_coalesce{false};
-    Scalar_subquery_replacement(Item_singlerow_subselect *target, Field *field,
-                                Query_block *select, bool add_coalesce)
-        : m_target(target),
-          m_field(field),
-          m_outer_query_block(select),
-          m_inner_query_block(select),
-          m_add_coalesce(add_coalesce) {}
-  };
-
-  Item *replace_scalar_subquery(uchar *arge) override;
   /**
     This method is used to implement a special case of semantic tree
     rewriting, mandated by a SQL:2003 exception in the specification.
     The only caller of this method is handle_sql2003_note184_exception(),
     see the code there for more details.
     Note that this method breaks the object internal integrity, by
-    removing it's association with the corresponding Query_block,
+    removing it's association with the corresponding SELECT_LEX,
     making this object orphan from the parse tree.
     No other method, beside the destructor, should be called on this
     object, as it is now invalid.
-    @return the Query_block structure that was given in the constructor.
+    @return the SELECT_LEX structure that was given in the constructor.
   */
-  Query_block *invalidate_and_restore_query_block();
-  std::optional<ContainedSubquery> get_contained_subquery(
-      const Query_block *outer_query_block) override;
+  SELECT_LEX *invalidate_and_restore_select_lex();
+
   friend class Query_result_scalar_subquery;
 };
 
@@ -373,7 +318,7 @@ class Item_maxmin_subselect final : public Item_singlerow_subselect {
   bool max;
   bool was_values;  // Set if we have found at least one row
  public:
-  Item_maxmin_subselect(Item_subselect *parent, Query_block *query_block,
+  Item_maxmin_subselect(Item_subselect *parent, SELECT_LEX *select_lex,
                         bool max, bool ignore_nulls);
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
@@ -381,39 +326,10 @@ class Item_maxmin_subselect final : public Item_singlerow_subselect {
   bool any_value() { return was_values; }
   void register_value() { was_values = true; }
   void reset_value_registration() override { was_values = false; }
-  bool is_maxmin() const override { return true; }
 };
 
 /* exists subselect */
 
-/**
-  Strategy which will be used to handle this subquery: flattening to a
-  semi-join, conversion to a derived table, rewrite of IN to EXISTS...
-  Sometimes the strategy is first only a candidate, then the real decision
-  happens in a second phase. Other times the first decision is final.
- */
-enum class Subquery_strategy : int {
-  /// Nothing decided yet
-  UNSPECIFIED,
-  /// Candidate for rewriting IN(subquery) to EXISTS, or subquery
-  /// materialization
-  CANDIDATE_FOR_IN2EXISTS_OR_MAT,
-  /// Candidate for semi-join flattening
-  CANDIDATE_FOR_SEMIJOIN,
-  /// Candidate for rewriting to joined derived table
-  CANDIDATE_FOR_DERIVED_TABLE,
-  /// Semi-join flattening
-  SEMIJOIN,
-  /// Rewrite to joined derived table
-  DERIVED_TABLE,
-  /// Evaluate as EXISTS subquery (possibly after rewriting from another type)
-  SUBQ_EXISTS,
-  /// Subquery materialization (HASH_SJ_ENGINE)
-  SUBQ_MATERIALIZATION,
-  /// Subquery has been deleted, probably because it was always false
-  DELETED,
-};
-
 class Item_exists_subselect : public Item_subselect {
   typedef Item_subselect super;
 
@@ -422,45 +338,74 @@ class Item_exists_subselect : public Item_subselect {
   bool value{false};
 
  public:
+  /**
+    The method chosen to execute the predicate, currently used for IN, =ANY
+    and EXISTS predicates.
+  */
+  enum enum_exec_method {
+    EXEC_UNSPECIFIED,  ///< No execution method specified yet.
+    EXEC_SEMI_JOIN,    ///< Predicate is converted to semi-join nest.
+    /// IN was converted to correlated EXISTS, and this is a final decision.
+    EXEC_EXISTS,
+    /**
+       Decision between EXEC_EXISTS and EXEC_MATERIALIZATION is not yet taken.
+       IN was temporarily converted to correlated EXISTS.
+       All descendants of Item_in_subselect must go through this method
+       before they can reach EXEC_EXISTS.
+    */
+    EXEC_EXISTS_OR_MAT,
+    /// Predicate executed via materialization, and this is a final decision.
+    EXEC_MATERIALIZATION
+  };
+  enum_exec_method exec_method;
   /// Priority of this predicate in the convert-to-semi-join-nest process.
-  int sj_convert_priority{0};
-  /// Execution strategy chosen for this Item
-  Subquery_strategy strategy{Subquery_strategy::UNSPECIFIED};
-  /// Used by the transformation to derived table
-  enum_condition_context outer_condition_context{enum_condition_context::ANDS};
-
+  int sj_convert_priority;
+  /// Decision on whether predicate is selected for semi-join transformation
+  enum enum_sj_selection {
+    /// Not selected for semi-join, evaluate as subquery predicate, or
+    /// replace with a substitution for the Item (e.g. Item_in_optimizer)
+    SJ_NOT_SELECTED,
+    /// Selected for semi-join, replace predicate with "true"
+    SJ_SELECTED,
+    /// Subquery's WHERE is always false, replace predicate with "false"
+    SJ_ALWAYS_FALSE
+  };
+  enum_sj_selection sj_selection{SJ_NOT_SELECTED};
   /**
     Used by subquery optimizations to keep track about where this subquery
     predicate is located, and whether it is a candidate for transformation.
-      (Table_ref*) 1 - the predicate is an AND-part of the WHERE
-      join nest pointer    - the predicate is an AND-part of ON expression
-                             of a join nest
-      NULL                 - for all other locations. It also means that the
-                             predicate is not a candidate for transformation.
+      (TABLE_LIST*) 1   - the predicate is an AND-part of the WHERE
+      join nest pointer - the predicate is an AND-part of ON expression
+                          of a join nest
+      NULL              - for all other locations. It also means that the
+                          predicate is not a candidate for transformation.
     See also THD::emb_on_expr_nest.
-
-    As for the second case above (the join nest pointer), note that this value
-    may change if scalar subqueries are transformed to derived tables,
-    cf. transform_scalar_subqueries_to_join_with_derived, due to the need to
-    build new join nests. The change is performed in Query_block::nest_derived.
   */
-  Table_ref *embedding_join_nest{nullptr};
+  TABLE_LIST *embedding_join_nest;
 
-  Item_exists_subselect(Query_block *select);
+  Item_exists_subselect(SELECT_LEX *select);
 
-  Item_exists_subselect() : Item_subselect() {}
+  Item_exists_subselect()
+      : Item_subselect(),
+        value(false),
+        exec_method(EXEC_UNSPECIFIED),
+        sj_convert_priority(0),
+        embedding_join_nest(NULL) {}
 
-  explicit Item_exists_subselect(const POS &pos) : super(pos) {}
+  explicit Item_exists_subselect(const POS &pos)
+      : super(pos),
+        value(false),
+        exec_method(EXEC_UNSPECIFIED),
+        sj_convert_priority(0),
+        embedding_join_nest(NULL) {}
 
-  void notify_removal() override { strategy = Subquery_strategy::DELETED; }
-
-  bool select_transformer(THD *, Query_block *) override {
-    strategy = Subquery_strategy::SUBQ_EXISTS;
-    return false;
+  trans_res select_transformer(THD *, SELECT_LEX *) override {
+    exec_method = EXEC_EXISTS;
+    return RES_OK;
   }
   subs_type substype() const override { return EXISTS_SUBS; }
   bool is_bool_func() const override { return true; }
-  void reset() override { value = false; }
+  void reset() override { value = 0; }
 
   enum Item_result result_type() const override { return INT_RESULT; }
   /*
@@ -506,6 +451,7 @@ class Item_exists_subselect : public Item_subselect {
              enum_query_type query_type) const override;
 
   friend class Query_result_exists_subquery;
+  friend class subselect_indexsubquery_engine;
 };
 
 /**
@@ -598,8 +544,9 @@ class Item_in_subselect : public Item_exists_subselect {
        was done.
     */
     bool dependent_after;
-  } *in2exists_info;
+  } * in2exists_info;
 
+  Item *remove_in2exists_conds(Item *conds);
   bool mark_as_outer(Item *left_row, size_t col);
 
  public:
@@ -622,29 +569,30 @@ class Item_in_subselect : public Item_exists_subselect {
   }
 
   bool *get_cond_guard(int i) {
-    return pushed_cond_guards ? pushed_cond_guards + i : nullptr;
+    return pushed_cond_guards ? pushed_cond_guards + i : NULL;
   }
   void set_cond_guard_var(int i, bool v) {
     if (pushed_cond_guards) pushed_cond_guards[i] = v;
   }
+  bool have_guarded_conds() override { return pushed_cond_guards != nullptr; }
 
-  Item_in_subselect(Item *left_expr, Query_block *query_block);
+  Item_in_subselect(Item *left_expr, SELECT_LEX *select_lex);
   Item_in_subselect(const POS &pos, Item *left_expr,
                     PT_subquery *pt_subquery_arg);
 
   Item_in_subselect()
       : Item_exists_subselect(),
-        left_expr(nullptr),
-        left_expr_cache(nullptr),
+        left_expr(NULL),
+        left_expr_cache(NULL),
         left_expr_cache_filled(false),
         need_expr_cache(true),
-        m_injected_left_expr(nullptr),
-        optimizer(nullptr),
+        m_injected_left_expr(NULL),
+        optimizer(NULL),
         was_null(false),
         abort_on_null(false),
-        in2exists_info(nullptr),
-        pushed_cond_guards(nullptr),
-        upper_item(nullptr) {}
+        in2exists_info(NULL),
+        pushed_cond_guards(NULL),
+        upper_item(NULL) {}
 
   bool itemize(Parse_context *pc, Item **res) override;
 
@@ -652,26 +600,22 @@ class Item_in_subselect : public Item_exists_subselect {
   subs_type substype() const override { return IN_SUBS; }
 
   void reset() override {
-    value = false;
-    null_value = false;
-    was_null = false;
+    value = 0;
+    null_value = 0;
+    was_null = 0;
   }
-  bool select_transformer(THD *thd, Query_block *select) override;
-  bool select_in_like_transformer(THD *thd, Query_block *select,
-                                  Comp_creator *func);
-  bool single_value_transformer(THD *thd, Query_block *select,
-                                Comp_creator *func);
-  bool row_value_transformer(THD *thd, Query_block *select);
-  bool single_value_in_to_exists_transformer(THD *thd, Query_block *select,
-                                             Comp_creator *func);
-  bool row_value_in_to_exists_transformer(THD *thd, Query_block *select);
-  bool subquery_allows_materialization(THD *thd, Query_block *query_block,
-                                       const Query_block *outer);
+  trans_res select_transformer(THD *thd, SELECT_LEX *select) override;
+  trans_res select_in_like_transformer(THD *thd, SELECT_LEX *select,
+                                       Comp_creator *func);
+  trans_res single_value_transformer(THD *thd, SELECT_LEX *select,
+                                     Comp_creator *func);
+  trans_res row_value_transformer(THD *thd, SELECT_LEX *select);
+  trans_res single_value_in_to_exists_transformer(THD *thd, SELECT_LEX *select,
+                                                  Comp_creator *func);
+  trans_res row_value_in_to_exists_transformer(THD *thd, SELECT_LEX *select);
+  bool subquery_allows_materialization(THD *thd, SELECT_LEX *select_lex,
+                                       const SELECT_LEX *outer);
   bool walk(Item_processor processor, enum_walk walk, uchar *arg) override;
-  Item *transform(Item_transformer transformer, uchar *arg) override;
-  Item *compile(Item_analyzer analyzer, uchar **arg_p,
-                Item_transformer transformer, uchar *arg_t) override;
-
   bool exec(THD *thd) override;
   longlong val_int() override;
   double val_real() override;
@@ -682,24 +626,20 @@ class Item_in_subselect : public Item_exists_subselect {
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
   bool fix_fields(THD *thd, Item **ref) override;
-  void fix_after_pullout(Query_block *parent_query_block,
-                         Query_block *removed_query_block) override;
-  void update_used_tables() override;
+  void fix_after_pullout(SELECT_LEX *parent_select,
+                         SELECT_LEX *removed_select) override;
   bool init_left_expr_cache(THD *thd);
 
   /**
      Once the decision to use IN->EXISTS has been taken, performs some last
      steps of this transformation.
   */
-  bool finalize_exists_transform(THD *thd, Query_block *select);
+  bool finalize_exists_transform(THD *thd, SELECT_LEX *select);
   /**
      Once the decision to use materialization has been taken, performs some
      last steps of this transformation.
   */
   bool finalize_materialization_transform(THD *thd, JOIN *join);
-  AccessPath *root_access_path() const override;
-  std::optional<ContainedSubquery> get_contained_subquery(
-      const Query_block *outer_query_block) override;
 
   friend class Item_ref_null_helper;
   friend class Item_is_not_null_test;
@@ -719,60 +659,19 @@ class Item_allany_subselect final : public Item_in_subselect {
   bool all;
 
   Item_allany_subselect(Item *left_expr, chooser_compare_func_creator fc,
-                        Query_block *select, bool all);
+                        SELECT_LEX *select, bool all);
 
   // only ALL subquery has upper not
   subs_type substype() const override { return all ? ALL_SUBS : ANY_SUBS; }
-  bool select_transformer(THD *thd, Query_block *select) override;
+  trans_res select_transformer(THD *thd, SELECT_LEX *select) override;
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
 };
 
-class SubqueryWithResult {
- public:
-  SubqueryWithResult(Query_expression *u, Query_result_interceptor *res,
-                     Item_subselect *si);
-  /**
-    Cleanup subquery after complete query execution, free all resources.
-  */
-  void cleanup();
-  bool prepare(THD *thd);
-  void fix_length_and_dec(Item_cache **row);
-  /**
-    Execute the subquery
-
-    SYNOPSIS
-      exec()
-
-    DESCRIPTION
-      Execute the subquery. The result of execution is subquery value that is
-      captured by previously set up Query_result-based 'sink'.
-
-    RETURN
-      false - OK
-      true  - Execution error.
-  */
-  bool exec(THD *thd);
-  void print(const THD *thd, String *str, enum_query_type query_type);
-  bool change_query_result(THD *thd, Item_subselect *si,
-                           Query_result_subquery *result);
-  Query_block *single_query_block() const;  // Only if unit is simple.
-
-  enum Item_result type() const { return res_type; }
-  enum_field_types field_type() const { return res_field_type; }
-  bool may_be_null() const { return maybe_null; }
-
-#ifndef NDEBUG
-  /**
-     @returns the internal Item. Defined only in debug builds, because should
-     be used only for debug asserts.
-  */
-  const Item_subselect *get_item() const { return item; }
-#endif
-
- private:
+class subselect_engine {
+ protected:
   Query_result_interceptor *result; /* results storage class */
-  Item_subselect *item;             /* item, that use this subquery */
+  Item_subselect *item;             /* item, that use this engine */
   enum Item_result res_type;        /* type of results */
   enum_field_types res_field_type;  /* column type of the results */
   /**
@@ -781,10 +680,90 @@ class SubqueryWithResult {
   */
   bool maybe_null;
 
-  Query_expression *unit; /* corresponding unit structure */
+ public:
+  enum enum_engine_type {
+    ABSTRACT_ENGINE,
+    ITERATOR_ENGINE,
+    INDEXSUBQUERY_ENGINE,
+    HASH_SJ_ENGINE
+  };
 
-  void set_row(const mem_root_deque<Item *> &item_list, Item_cache **row,
-               bool never_empty);
+  subselect_engine(Item_subselect *si, Query_result_interceptor *res)
+      : result(res),
+        item(si),
+        res_type(STRING_RESULT),
+        res_field_type(MYSQL_TYPE_VAR_STRING),
+        maybe_null(false) {}
+  virtual ~subselect_engine() {}  // to satisfy compiler
+  /**
+    Cleanup engine after complete query execution, free all resources.
+  */
+  virtual void cleanup(THD *thd) = 0;
+
+  virtual bool prepare(THD *thd) = 0;
+  virtual void fix_length_and_dec(Item_cache **row) = 0;
+  /*
+    Execute the engine
+
+    SYNOPSIS
+      exec()
+
+    DESCRIPTION
+      Execute the engine. The result of execution is subquery value that is
+      either captured by previously set up Query_result-based 'sink' or
+      stored somewhere by the exec() method itself.
+
+    RETURN
+      0 - OK
+      1 - Either an execution error, or the engine was "changed", and the
+          caller should call exec() again for the new engine.
+  */
+  virtual bool exec(THD *thd) = 0;
+  virtual uint cols() const = 0; /* return number of columns in select */
+  virtual uint8 uncacheable() const = 0; /* query is uncacheable */
+  virtual enum Item_result type() const { return res_type; }
+  virtual enum_field_types field_type() const { return res_field_type; }
+  virtual void exclude() = 0;
+  bool may_be_null() const { return maybe_null; }
+  virtual table_map upper_select_const_tables() const = 0;
+  static table_map calc_const_tables(TABLE_LIST *);
+  virtual void print(const THD *thd, String *str,
+                     enum_query_type query_type) = 0;
+  virtual bool change_query_result(THD *thd, Item_subselect *si,
+                                   Query_result_subquery *result) = 0;
+  virtual enum_engine_type engine_type() const { return ABSTRACT_ENGINE; }
+#ifndef DBUG_OFF
+  /**
+     @returns the internal Item. Defined only in debug builds, because should
+     be used only for debug asserts.
+  */
+  const Item_subselect *get_item() const { return item; }
+#endif
+};
+
+class subselect_iterator_engine final : public subselect_engine {
+ public:
+  subselect_iterator_engine(SELECT_LEX_UNIT *u,
+                            Query_result_interceptor *result,
+                            Item_subselect *item);
+  void cleanup(THD *thd) override;
+  bool prepare(THD *thd) override;
+  void fix_length_and_dec(Item_cache **row) override;
+  bool exec(THD *thd) override;
+  uint cols() const override;
+  uint8 uncacheable() const override;
+  void exclude() override;
+  table_map upper_select_const_tables() const override;
+  void print(const THD *thd, String *str, enum_query_type query_type) override;
+  bool change_query_result(THD *thd, Item_subselect *si,
+                           Query_result_subquery *result) override;
+  enum_engine_type engine_type() const override { return ITERATOR_ENGINE; }
+  SELECT_LEX *single_select_lex() const;  // Only if unit is simple.
+
+ private:
+  SELECT_LEX_UNIT *unit; /* corresponding unit structure */
+
+  void set_row(List<Item> &item_list, Item_cache **row, bool never_empty);
 
   friend class subselect_hash_sj_engine;
 };
@@ -805,16 +784,13 @@ class SubqueryWithResult {
   i.e. the subquery is a single table SELECT without GROUP BY, aggregate
   functions, etc.
 */
-class subselect_indexsubquery_engine {
+class subselect_indexsubquery_engine : public subselect_engine {
  protected:
-  Query_result_union *result = nullptr; /* results storage class */
   /// Table which is read, using one of eq_ref, ref, ref_or_null.
-  TABLE *table{nullptr};
-  Table_ref *table_ref{nullptr};
-  Index_lookup ref;
-  join_type type{JT_UNKNOWN};
+  QEP_TAB *tab;
   Item *cond;     /* The WHERE condition of subselect */
-  ulonglong hash; /* Hash value calculated by RefIterator, when needed. */
+  ulonglong hash; /* Hash value calculated by copy_ref_key, when needed. */
+ private:
   /*
     The "having" clause. This clause (further referred to as "artificial
     having") was inserted by subquery transformation code. It contains
@@ -827,29 +803,27 @@ class subselect_indexsubquery_engine {
   */
   Item *having;
 
-  Item_in_subselect *item; /* item that uses this engine */
-
  public:
-  enum enum_engine_type { INDEXSUBQUERY_ENGINE, HASH_SJ_ENGINE };
-
-  subselect_indexsubquery_engine(TABLE *table, Table_ref *table_ref,
-                                 const Index_lookup &ref,
-                                 enum join_type join_type,
-                                 Item_in_subselect *subs, Item *where,
-                                 Item *having_arg)
-      : table(table),
-        table_ref(table_ref),
-        ref(ref),
-        type(join_type),
+  subselect_indexsubquery_engine(QEP_TAB *tab_arg, Item_subselect *subs,
+                                 Item *where, Item *having_arg)
+      : subselect_engine(subs, 0),
+        tab(tab_arg),
         cond(where),
-        having(having_arg),
-        item(subs) {}
-  virtual ~subselect_indexsubquery_engine() = default;
-  virtual bool exec(THD *thd);
-  virtual void print(const THD *thd, String *str, enum_query_type query_type);
-  virtual enum_engine_type engine_type() const { return INDEXSUBQUERY_ENGINE; }
-  virtual void cleanup() {}
-  virtual void create_iterators(THD *) {}
+        having(having_arg) {}
+  bool exec(THD *thd) override;
+  void print(const THD *thd, String *str, enum_query_type query_type) override;
+  enum_engine_type engine_type() const override { return INDEXSUBQUERY_ENGINE; }
+  void cleanup(THD *) override {}
+  bool prepare(THD *thd) override;
+  void fix_length_and_dec(Item_cache **row) override;
+  uint cols() const override { return 1; }
+  uint8 uncacheable() const override { return UNCACHEABLE_DEPENDENT; }
+  void exclude() override;
+  table_map upper_select_const_tables() const override { return 0; }
+  bool change_query_result(THD *thd, Item_subselect *si,
+                           Query_result_subquery *result) override;
+  bool scan_table();
+  void copy_ref_key(bool *require_scan, bool *convert_error);
 };
 
 /*
@@ -858,7 +832,11 @@ class subselect_indexsubquery_engine {
  */
 Item *all_any_subquery_creator(Item *left_expr,
                                chooser_compare_func_creator cmp, bool all,
-                               Query_block *select);
+                               SELECT_LEX *select);
+
+inline bool Item_subselect::is_uncacheable() const {
+  return engine->uncacheable();
+}
 
 /**
   Compute an IN predicate via a hash semi-join. The subquery is materialized
@@ -868,11 +846,8 @@ Item *all_any_subquery_creator(Item *left_expr,
 
 class subselect_hash_sj_engine final : public subselect_indexsubquery_engine {
  private:
-  /* true if the subquery was materialized into a temp table. */
+  /* TRUE if the subquery was materialized into a temp table. */
   bool is_materialized;
-  // true if we know for sure that there are zero rows in the table.
-  // Set only after is_materialized is true.
-  bool has_zero_rows = false;
   /**
      Existence of inner NULLs in materialized table:
      By design, other values than IRRELEVANT_OR_FALSE are possible only if the
@@ -887,55 +862,34 @@ class subselect_hash_sj_engine final : public subselect_indexsubquery_engine {
     NEX_TRUE = 2
   };
   enum nulls_exist mat_table_has_nulls;
-  Query_expression *const unit;
-  unique_ptr_destroy_only<RowIterator> m_iterator;
-  AccessPath *m_root_access_path;
-
-  /// Saved result object, must be restored after use
-  Query_result_interceptor *saved_result{nullptr};
+  /*
+    The old engine already chosen at parse time and stored in permanent memory.
+    Through this member we can re-create and re-prepare the join object
+    used to materialize the subquery for each execution of a prepared
+    statement. We also reuse the functionality of
+    subselect_iterator_engine::[prepare | cols].
+  */
+  subselect_iterator_engine *materialize_engine;
+  /* Temp table context of the outer select's JOIN. */
+  Temp_table_param *tmp_param;
 
  public:
-  subselect_hash_sj_engine(Item_in_subselect *in_predicate,
-                           Query_expression *unit_arg)
-      : subselect_indexsubquery_engine(nullptr, nullptr, {}, JT_UNKNOWN,
-                                       in_predicate, nullptr, nullptr),
+  subselect_hash_sj_engine(Item_subselect *in_predicate,
+                           subselect_iterator_engine *old_engine)
+      : subselect_indexsubquery_engine(NULL, in_predicate, NULL, NULL),
         is_materialized(false),
-        unit(unit_arg) {}
+        materialize_engine(old_engine),
+        tmp_param(NULL) {}
   ~subselect_hash_sj_engine() override;
 
-  bool setup(THD *thd, const mem_root_deque<Item *> &tmp_columns);
-  void cleanup() override;
+  bool setup(THD *thd, List<Item> *tmp_columns);
+  void cleanup(THD *thd) override;
+  bool prepare(THD *thd) override { return materialize_engine->prepare(thd); }
   bool exec(THD *thd) override;
   void print(const THD *thd, String *str, enum_query_type query_type) override;
+  uint cols() const override { return materialize_engine->cols(); }
   enum_engine_type engine_type() const override { return HASH_SJ_ENGINE; }
 
-  TABLE *get_table() const { return table; }
-  const Index_lookup &index_lookup() const { return ref; }
-  enum join_type get_join_type() const { return type; }
-  AccessPath *root_access_path() const { return m_root_access_path; }
-  void create_iterators(THD *thd) override;
+  const QEP_TAB *get_qep_tab() const { return tab; }
 };
-
-/**
-  Removes every predicate injected by IN->EXISTS.
-
-  This function is different from others:
-  - it wants to remove all traces of IN->EXISTS (for
-  materialization)
-  - remove_subq_pushed_predicates() and remove_additional_cond() want to
-  remove only the conditions of IN->EXISTS which index lookup already
-  satisfies (they are just an optimization).
-
-  If there are no in2exists conditions, it will return the exact same
-  pointer. If it returns a new Item, the old Item is left alone, so it
-  can be reused in other settings.
-
-  @param conds  Condition; may be nullptr.
-  @returns      new condition
- */
-Item *remove_in2exists_conds(Item *conds);
-
-/// Returns whether the Item is an IN-subselect.
-bool IsItemInSubSelect(Item *item);
-
 #endif /* ITEM_SUBSELECT_INCLUDED */
