@@ -1,16 +1,15 @@
-/* Copyright (c) 2005, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,17 +22,15 @@
 
 #include "sql/item_xmlfunc.h"
 
-#include <assert.h>
+#include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
 
-#include <algorithm>
-#include <cstdio>
-#include <cstring>
-#include <memory>
-
+#include "lex_string.h"
 #include "m_ctype.h"
 #include "m_string.h"
-
+#include "my_dbug.h"
+#include "my_macros.h"
 #include "my_sys.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_com.h"
@@ -74,7 +71,7 @@
        - preceding
 */
 
-/* Lexical analyzer token */
+/* Lexical analizer token */
 struct MY_XPATH_LEX {
   int term;        /* token type, see MY_XPATH_LEX_XXXXX below */
   const char *beg; /* beginnign of the token                   */
@@ -185,7 +182,7 @@ class Item_nodeset_func : public Item_str_func {
   }
   enum Item_result result_type() const override { return STRING_RESULT; }
   bool resolve_type(THD *) override {
-    set_data_type_string(uint32{MAX_BLOB_WIDTH});
+    max_length = MAX_BLOB_WIDTH;
     // To avoid premature evaluation, mark all nodeset functions as non-const.
     used_tables_cache = RAND_TABLE_BIT;
     return false;
@@ -407,6 +404,10 @@ class Item_nodeset_context_cache : public Item_nodeset_func {
     if (!m_is_empty)
       nodeset->push_back({m_num, m_pos, static_cast<uint>(m_size)});
   }
+  bool resolve_type(THD *) override {
+    max_length = MAX_BLOB_WIDTH;
+    return false;
+  }
   void set_element(uint32 num, uint32 pos, size_t size) {
     m_num = num;
     m_pos = pos;
@@ -420,7 +421,7 @@ class Item_func_xpath_position : public Item_int_func {
   explicit Item_func_xpath_position(Item *a) : Item_int_func(a) {}
   const char *func_name() const override { return "xpath_position"; }
   bool resolve_type(THD *) override {
-    set_data_type_string(10U);
+    max_length = 10;
     return false;
   }
   longlong val_int() override {
@@ -437,7 +438,7 @@ class Item_func_xpath_count : public Item_int_func {
   explicit Item_func_xpath_count(Item *a) : Item_int_func(a) {}
   const char *func_name() const override { return "xpath_count"; }
   bool resolve_type(THD *) override {
-    set_data_type_string(10U);
+    max_length = 10;
     return false;
   }
   longlong val_int() override {
@@ -760,7 +761,7 @@ static Item *nodeset2bool(Item *item) {
 /*
   Create scalar comparator
 
-  SYNOPSIS
+  SYNOPSYS
     Create a comparator function for scalar arguments,
     for the given arguments and operation.
 
@@ -782,13 +783,13 @@ static Item_bool_func *eq_func(int oper, Item *a, Item *b) {
     case MY_XPATH_LEX_LESS:
       return new Item_func_lt(a, b);
   }
-  return nullptr;
+  return 0;
 }
 
 /*
   Create scalar comparator
 
-  SYNOPSIS
+  SYNOPSYS
     Create a comparator function for scalar arguments,
     for the given arguments and reverse operation, e.g.
 
@@ -812,13 +813,13 @@ static Item_bool_func *eq_func_reverse(int oper, Item *a, Item *b) {
     case MY_XPATH_LEX_LESS:
       return new Item_func_gt(a, b);
   }
-  return nullptr;
+  return 0;
 }
 
 /*
   Create a comparator
 
-  SYNOPSIS
+  SYNOPSYS
     Create a comparator for scalar or non-scalar arguments,
     for the given arguments and operation.
 
@@ -832,13 +833,13 @@ static Item *create_comparator(MY_XPATH *xpath, int oper, MY_XPATH_LEX *context,
   } else if (a->type() == Item::XPATH_NODESET &&
              b->type() == Item::XPATH_NODESET) {
     size_t len = xpath->query.end - context->beg;
-    len = std::min(len, size_t(32));
+    set_if_smaller(len, 32);
     my_printf_error(ER_UNKNOWN_ERROR,
                     "XPATH error: "
                     "comparison of two nodesets is not supported: '%.*s'",
                     MYF(0), static_cast<int>(len), context->beg);
 
-    return nullptr;  // TODO: Comparison of two nodesets
+    return 0;  // TODO: Comparison of two nodesets
   } else {
     /*
      Compare a node set to a scalar value.
@@ -869,7 +870,7 @@ static Item *create_comparator(MY_XPATH *xpath, int oper, MY_XPATH_LEX *context,
 /*
   Create a step
 
-  SYNOPSIS
+  SYNOPSYS
     Create a step function for the given argument and axis.
 
   RETURN
@@ -877,10 +878,10 @@ static Item *create_comparator(MY_XPATH *xpath, int oper, MY_XPATH_LEX *context,
 */
 static Item_nodeset_func *nametestfunc(MY_XPATH *xpath, int type, Item *arg,
                                        const char *beg, size_t len) {
-  assert(arg != nullptr);
-  assert(arg->type() == Item::XPATH_NODESET);
-  assert(beg != nullptr);
-  assert(len > 0);
+  DBUG_ASSERT(arg != 0);
+  DBUG_ASSERT(arg->type() == Item::XPATH_NODESET);
+  DBUG_ASSERT(beg != 0);
+  DBUG_ASSERT(len > 0);
 
   Item_nodeset_func *res;
   switch (type) {
@@ -920,7 +921,7 @@ static Item_nodeset_func *nametestfunc(MY_XPATH *xpath, int type, Item *arg,
 }
 
 /*
-  Tokens consisting of one character, for faster lexical analyzer.
+  Tokens consisting of one character, for faster lexical analizer.
 */
 static char simpletok[128] = {
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -950,7 +951,7 @@ static struct my_xpath_keyword_names_st my_keyword_names[] = {
     {MY_XPATH_LEX_OR, "or", 2, 0},
     {MY_XPATH_LEX_DIV, "div", 3, 0},
     {MY_XPATH_LEX_MOD, "mod", 3, 0},
-    {0, nullptr, 0, 0}};
+    {0, NULL, 0, 0}};
 
 static struct my_xpath_keyword_names_st my_axis_names[] = {
     {MY_XPATH_LEX_AXIS, "ancestor", 8, MY_XPATH_AXIS_ANCESTOR},
@@ -969,19 +970,19 @@ static struct my_xpath_keyword_names_st my_axis_names[] = {
     {MY_XPATH_LEX_AXIS, "preceding-sibling", 17,
      MY_XPATH_AXIS_PRECEDING_SIBLING},
     {MY_XPATH_LEX_AXIS, "self", 4, MY_XPATH_AXIS_SELF},
-    {0, nullptr, 0, 0}};
+    {0, NULL, 0, 0}};
 
 static struct my_xpath_keyword_names_st my_nodetype_names[] = {
     {MY_XPATH_LEX_NODETYPE, "comment", 7, 0},
     {MY_XPATH_LEX_NODETYPE, "text", 4, 0},
     {MY_XPATH_LEX_NODETYPE, "processing-instruction", 22, 0},
     {MY_XPATH_LEX_NODETYPE, "node", 4, 0},
-    {0, nullptr, 0, 0}};
+    {0, NULL, 0, 0}};
 
 /*
   Lookup a keyword
 
-  SYNOPSIS
+  SYNOPSYS
     Check that the last scanned identifier is a keyword.
 
   RETURN
@@ -1036,7 +1037,7 @@ static Item *create_func_number(MY_XPATH *xpath, Item **args, uint nargs) {
   if (nargs > 0) {
     arg = args[0];
   } else {
-    arg = xpath->context != nullptr ? xpath->context : xpath->rootelement;
+    arg = xpath->context != NULL ? xpath->context : xpath->rootelement;
   }
   return new Item_xpath_cast_number(arg);
 }
@@ -1044,20 +1045,19 @@ static Item *create_func_number(MY_XPATH *xpath, Item **args, uint nargs) {
 static Item *create_func_string_length(MY_XPATH *xpath, Item **args,
                                        uint nargs) {
   Item *arg = nargs ? args[0] : xpath->context;
-  return arg ? new Item_func_char_length(arg) : nullptr;
+  return arg ? new Item_func_char_length(arg) : 0;
 }
 
 static Item *create_func_round(MY_XPATH *, Item **args, uint) {
-  return new Item_func_round(args[0], new Item_int_0(), false);
+  return new Item_func_round(args[0], new Item_int_0(), 0);
 }
 
 static Item *create_func_last(MY_XPATH *xpath, Item **, uint) {
-  return xpath->context ? new Item_func_xpath_count(xpath->context) : nullptr;
+  return xpath->context ? new Item_func_xpath_count(xpath->context) : NULL;
 }
 
 static Item *create_func_position(MY_XPATH *xpath, Item **, uint) {
-  return xpath->context ? new Item_func_xpath_position(xpath->context)
-                        : nullptr;
+  return xpath->context ? new Item_func_xpath_position(xpath->context) : NULL;
 }
 
 static Item *create_func_contains(MY_XPATH *, Item **args, uint) {
@@ -1076,12 +1076,12 @@ static Item *create_func_substr(MY_XPATH *, Item **args, uint nargs) {
 }
 
 static Item *create_func_count(MY_XPATH *, Item **args, uint) {
-  if (args[0]->type() != Item::XPATH_NODESET) return nullptr;
+  if (args[0]->type() != Item::XPATH_NODESET) return 0;
   return new Item_func_xpath_count(args[0]);
 }
 
 static Item *create_func_sum(MY_XPATH *xpath, Item **args, uint) {
-  if (args[0]->type() != Item::XPATH_NODESET) return nullptr;
+  if (args[0]->type() != Item::XPATH_NODESET) return 0;
   return new Item_func_xpath_sum(args[0], xpath->pxml);
 }
 
@@ -1091,50 +1091,50 @@ static Item *create_func_sum(MY_XPATH *xpath, Item **args, uint) {
 */
 static MY_XPATH_FUNC my_func_names3[] = {{"sum", 3, 1, 1, create_func_sum},
                                          {"not", 3, 1, 1, create_func_not},
-                                         {nullptr, 0, 0, 0, nullptr}};
+                                         {0, 0, 0, 0, 0}};
 
 static MY_XPATH_FUNC my_func_names4[] = {{"last", 4, 0, 0, create_func_last},
                                          {"true", 4, 0, 0, create_func_true},
-                                         {"name", 4, 0, 1, nullptr},
-                                         {"lang", 4, 1, 1, nullptr},
-                                         {nullptr, 0, 0, 0, nullptr}};
+                                         {"name", 4, 0, 1, 0},
+                                         {"lang", 4, 1, 1, 0},
+                                         {0, 0, 0, 0, 0}};
 
 static MY_XPATH_FUNC my_func_names5[] = {{"count", 5, 1, 1, create_func_count},
                                          {"false", 5, 0, 0, create_func_false},
                                          {"floor", 5, 1, 1, create_func_floor},
                                          {"round", 5, 1, 1, create_func_round},
-                                         {nullptr, 0, 0, 0, nullptr}};
+                                         {0, 0, 0, 0, 0}};
 
 static MY_XPATH_FUNC my_func_names6[] = {
     {"concat", 6, 2, 255, create_func_concat},
     {"number", 6, 0, 1, create_func_number},
-    {"string", 6, 0, 1, nullptr},
-    {nullptr, 0, 0, 0, nullptr}};
+    {"string", 6, 0, 1, 0},
+    {0, 0, 0, 0, 0}};
 
 /* Other functions, with name longer than 6, all together */
 static MY_XPATH_FUNC my_func_names[] = {
-    {"id", 2, 1, 1, nullptr},
+    {"id", 2, 1, 1, 0},
     {"boolean", 7, 1, 1, create_func_bool},
     {"ceiling", 7, 1, 1, create_func_ceiling},
     {"position", 8, 0, 0, create_func_position},
     {"contains", 8, 2, 2, create_func_contains},
     {"substring", 9, 2, 3, create_func_substr},
-    {"translate", 9, 3, 3, nullptr},
+    {"translate", 9, 3, 3, 0},
 
-    {"local-name", 10, 0, 1, nullptr},
-    {"starts-with", 11, 2, 2, nullptr},
-    {"namespace-uri", 13, 0, 1, nullptr},
+    {"local-name", 10, 0, 1, 0},
+    {"starts-with", 11, 2, 2, 0},
+    {"namespace-uri", 13, 0, 1, 0},
     {"string-length", 13, 0, 1, create_func_string_length},
-    {"substring-after", 15, 2, 2, nullptr},
-    {"normalize-space", 15, 0, 1, nullptr},
-    {"substring-before", 16, 2, 2, nullptr},
+    {"substring-after", 15, 2, 2, 0},
+    {"normalize-space", 15, 0, 1, 0},
+    {"substring-before", 16, 2, 2, 0},
 
-    {nullptr, 0, 0, 0, nullptr}};
+    {NULL, 0, 0, 0, 0}};
 
 /*
   Lookup a function by name
 
-  SYNOPSIS
+  SYNOPSYS
     Lookup a function by its name.
 
   RETURN
@@ -1147,7 +1147,7 @@ static MY_XPATH_FUNC *my_xpath_function(const char *beg, const char *end) {
   size_t length = end - beg;
   switch (length) {
     case 1:
-      return nullptr;
+      return 0;
     case 3:
       function_names = my_func_names3;
       break;
@@ -1167,10 +1167,10 @@ static MY_XPATH_FUNC *my_xpath_function(const char *beg, const char *end) {
     if (k->create && length == k->length &&
         !native_strncasecmp(beg, k->name, length))
       return k;
-  return nullptr;
+  return NULL;
 }
 
-/* Initialize a lex analyzer token */
+/* Initialize a lex analizer token */
 static void my_xpath_lex_init(MY_XPATH_LEX *lex, const char *str,
                               const char *strend) {
   lex->beg = str;
@@ -1187,10 +1187,10 @@ static int my_xdigit(int c) { return ((c) >= '0' && (c) <= '9'); }
 /*
   Scan the next token
 
-  SYNOPSIS
+  SYNOPSYS
     Scan the next token from the input.
     lex->term is set to the scanned token type.
-    lex->beg and lex->end are set to the beginning
+    lex->beg and lex->end are set to the beginnig
     and to the end of the token.
   RETURN
     N/A
@@ -1213,7 +1213,7 @@ static void my_xpath_lex_scan(MY_XPATH *xpath, MY_XPATH_LEX *lex,
            xpath->cs, &ctype, reinterpret_cast<const uchar *>(beg),
            reinterpret_cast<const uchar *>(end))) > 0 &&
       ((ctype & (_MY_L | _MY_U)) || *beg == '_')) {
-    // scan until the end of the idenfitier
+    // scan untill the end of the idenfitier
     for (beg += length;
          (length = xpath->cs->cset->ctype(
               xpath->cs, &ctype, reinterpret_cast<const uchar *>(beg),
@@ -1289,7 +1289,7 @@ static void my_xpath_lex_scan(MY_XPATH *xpath, MY_XPATH_LEX *lex,
 /*
   Scan the given token
 
-  SYNOPSIS
+  SYNOPSYS
     Scan the given token and rotate lasttok to prevtok on success.
 
   RETURN
@@ -1309,7 +1309,7 @@ static int my_xpath_parse_term(MY_XPATH *xpath, int term) {
 /*
   Scan AxisName
 
-  SYNOPSIS
+  SYNOPSYS
     Scan an axis name and store the scanned axis type into xpath->axis.
 
   RETURN
@@ -1326,7 +1326,7 @@ static int my_xpath_parse_AxisName(MY_XPATH *xpath) {
 ** Grammar rules, according to http://www.w3.org/TR/xpath
 ** Implemented using recursive descendant method.
 ** All the following grammar processing functions accept
-** a single "xpath" argument and return 1 on success and 0 on error.
+** a signle "xpath" argument and return 1 on success and 0 on error.
 ** They also modify "xpath" argument by creating new items.
 */
 
@@ -1361,7 +1361,7 @@ static int my_xpath_parse_VariableReference(MY_XPATH *xpath);
 /*
   Scan LocationPath
 
-  SYNOPSIS
+  SYNOPSYS
 
     [1] LocationPath ::=   RelativeLocationPath
                          | AbsoluteLocationPath
@@ -1386,7 +1386,7 @@ static int my_xpath_parse_LocationPath(MY_XPATH *xpath) {
 /*
   Scan Absolute Location Path
 
-  SYNOPSIS
+  SYNOPSYS
 
     [2]     AbsoluteLocationPath ::=   '/' RelativeLocationPath?
                                      | AbbreviatedAbsoluteLocationPath
@@ -1420,7 +1420,7 @@ static int my_xpath_parse_AbsoluteLocationPath(MY_XPATH *xpath) {
 /*
   Scan Relative Location Path
 
-  SYNOPSIS
+  SYNOPSYS
 
     For better performance we combine these two rules
 
@@ -1456,7 +1456,7 @@ static int my_xpath_parse_RelativeLocationPath(MY_XPATH *xpath) {
 /*
   Scan non-abbreviated or abbreviated Step
 
-  SYNOPSIS
+  SYNOPSYS
 
   [4] Step ::=   AxisSpecifier NodeTest Predicate*
                | AbbreviatedStep
@@ -1514,7 +1514,7 @@ static int my_xpath_parse_Step(MY_XPATH *xpath) {
 /*
   Scan Abbreviated Axis Specifier
 
-  SYNOPSIS
+  SYNOPSYS
   [5] AxisSpecifier ::=  AxisName '::'
                          | AbbreviatedAxisSpecifier
 
@@ -1533,7 +1533,7 @@ static int my_xpath_parse_AbbreviatedAxisSpecifier(MY_XPATH *xpath) {
 /*
   Scan non-abbreviated axis specifier
 
-  SYNOPSIS
+  SYNOPSYS
 
   RETURN
     1 - success
@@ -1548,7 +1548,7 @@ static int my_xpath_parse_AxisName_colon_colon(MY_XPATH *xpath) {
 /*
   Scan Abbreviated AxisSpecifier
 
-  SYNOPSIS
+  SYNOPSYS
     [13] AbbreviatedAxisSpecifier  ::=  '@'?
 
   RETURN
@@ -1563,7 +1563,7 @@ static int my_xpath_parse_AxisSpecifier(MY_XPATH *xpath) {
 /*
   Scan NodeType followed by parens
 
-  SYNOPSIS
+  SYNOPSYS
 
   RETURN
     1 - success
@@ -1578,7 +1578,7 @@ static int my_xpath_parse_NodeTest_lp_rp(MY_XPATH *xpath) {
 /*
   Scan NodeTest
 
-  SYNOPSIS
+  SYNOPSYS
 
   [7] NodeTest ::=   NameTest
                    | NodeType '(' ')'
@@ -1594,7 +1594,7 @@ static int my_xpath_parse_NodeTest(MY_XPATH *xpath) {
 /*
   Scan Abbreviated Step
 
-  SYNOPSIS
+  SYNOPSYS
 
   [12] AbbreviatedStep  ::= '.'	| '..'
 
@@ -1613,7 +1613,7 @@ static int my_xpath_parse_AbbreviatedStep(MY_XPATH *xpath) {
 /*
   Scan Primary Expression
 
-  SYNOPSIS
+  SYNOPSYS
 
   [15] PrimaryExpr ::= VariableReference
                        | '(' Expr ')'   (RECURSIVE)
@@ -1652,7 +1652,7 @@ static int my_xpath_parse_PrimaryExpr(MY_XPATH *xpath) {
 /*
   Scan Function Call
 
-  SYNOPSIS
+  SYNOPSYS
     [16] FunctionCall ::= FunctionName '(' ( Argument ( ',' Argument )* )? ')'
     [17] Argument      ::= Expr (RECURSIVE)
     [14] Expr ::= OrExpr
@@ -1700,7 +1700,7 @@ right_paren:
 /*
   Scan Union Expression
 
-  SYNOPSIS
+  SYNOPSYS
     [18] UnionExpr ::=   PathExpr
                        | UnionExpr '|' PathExpr
 
@@ -1729,7 +1729,7 @@ static int my_xpath_parse_UnionExpr(MY_XPATH *xpath) {
 /*
   Scan Path Expression
 
-  SYNOPSIS
+  SYNOPSYS
 
   [19] PathExpr ::=   LocationPath
                     | FilterExpr
@@ -1779,7 +1779,7 @@ static int my_xpath_parse_PathExpr(MY_XPATH *xpath) {
 /*
   Scan Filter Expression
 
-  SYNOPSIS
+  SYNOPSYS
     [20]  FilterExpr ::=   PrimaryExpr
                          | FilterExpr Predicate
 
@@ -1799,7 +1799,7 @@ static int my_xpath_parse_FilterExpr(MY_XPATH *xpath) {
 /*
   Scan Or Expression
 
-  SYNOPSIS
+  SYNOPSYS
     [21] OrExpr ::=   AndExpr
                     | OrExpr 'or' AndExpr
 
@@ -1830,7 +1830,7 @@ static int my_xpath_parse_OrExpr(MY_XPATH *xpath) {
 /*
   Scan And Expression
 
-  SYNOPSIS
+  SYNOPSYS
     [22] AndExpr ::=   EqualityExpr
                      | AndExpr 'and' EqualityExpr
 
@@ -1857,7 +1857,7 @@ static int my_xpath_parse_AndExpr(MY_XPATH *xpath) {
 /*
   Scan Equality Expression
 
-  SYNOPSIS
+  SYNOPSYS
     [23] EqualityExpr ::=   RelationalExpr
                           | EqualityExpr '=' RelationalExpr
                           | EqualityExpr '!=' RelationalExpr
@@ -1916,7 +1916,7 @@ static int my_xpath_parse_EqualityExpr(MY_XPATH *xpath) {
 /*
   Scan Relational Expression
 
-  SYNOPSIS
+  SYNOPSYS
 
     [24] RelationalExpr ::=   AdditiveExpr
                             | RelationalExpr '<' AdditiveExpr
@@ -1969,7 +1969,7 @@ static int my_xpath_parse_RelationalExpr(MY_XPATH *xpath) {
 /*
   Scan Additive Expression
 
-  SYNOPSIS
+  SYNOPSYS
 
     [25] AdditiveExpr ::=   MultiplicativeExpr
                           | AdditiveExpr '+' MultiplicativeExpr
@@ -2004,7 +2004,7 @@ static int my_xpath_parse_AdditiveExpr(MY_XPATH *xpath) {
 /*
   Scan Multiplicative Expression
 
-  SYNOPSIS
+  SYNOPSYS
 
     [26] MultiplicativeExpr ::=   UnaryExpr
                                 | MultiplicativeExpr MultiplyOperator UnaryExpr
@@ -2038,7 +2038,7 @@ static int my_xpath_parse_MultiplicativeExpr(MY_XPATH *xpath) {
         xpath->item = new Item_func_mul(prev, xpath->item);
         break;
       case MY_XPATH_LEX_DIV:
-        xpath->item = new Item_func_div_int(prev, xpath->item);
+        xpath->item = new Item_func_int_div(prev, xpath->item);
         break;
       case MY_XPATH_LEX_MOD:
         xpath->item = new Item_func_mod(prev, xpath->item);
@@ -2051,7 +2051,7 @@ static int my_xpath_parse_MultiplicativeExpr(MY_XPATH *xpath) {
 /*
   Scan Unary Expression
 
-  SYNOPSIS
+  SYNOPSYS
 
     [27] UnaryExpr ::=   UnionExpr
                        | '-' UnaryExpr
@@ -2075,7 +2075,7 @@ static int my_xpath_parse_UnaryExpr(MY_XPATH *xpath) {
 /*
   Scan Number
 
-  SYNOPSIS
+  SYNOPSYS
 
     [30] Number ::= Digits ('.' Digits?)? | '.' Digits)
 
@@ -2090,7 +2090,7 @@ static int my_xpath_parse_UnaryExpr(MY_XPATH *xpath) {
   as it is in conflict with abbreviated step.
   1 + .123    does not work,
   1 + 0.123   does.
-  Perhaps it is better to move this code into lex analyzer.
+  Perhaps it is better to move this code into lex analizer.
 
   RETURN
     1 - success
@@ -2114,7 +2114,7 @@ static int my_xpath_parse_Number(MY_XPATH *xpath) {
 /*
   Scan NCName.
 
-  SYNOPSIS
+  SYNOPSYS
 
     The keywords AND, OR, MOD, DIV are valid identitiers
     when they are in identifier context:
@@ -2210,15 +2210,15 @@ static int my_xpath_parse_VariableReference(MY_XPATH *xpath) {
         (spv = spc->find_variable(name_str, name_length, false))) {
       Item_splocal *splocal = new Item_splocal(
           Name_string(name_str, name_length, false), spv->offset, spv->type, 0);
-#ifndef NDEBUG
+#ifndef DBUG_OFF
       if (splocal) splocal->m_sp = lex->sphead;
 #endif
-      xpath->item = splocal;
+      xpath->item = down_cast<Item *>(splocal);
     } else {
-      xpath->item = nullptr;
-      assert(xpath->query.end > dollar_pos);
+      xpath->item = NULL;
+      DBUG_ASSERT(xpath->query.end > dollar_pos);
       size_t len = xpath->query.end - dollar_pos;
-      len = std::min(len, size_t(32));
+      set_if_smaller(len, 32);
       my_printf_error(ER_UNKNOWN_ERROR, "Unknown XPATH variable at: '%.*s'",
                       MYF(0), static_cast<int>(len), dollar_pos);
     }
@@ -2229,7 +2229,7 @@ static int my_xpath_parse_VariableReference(MY_XPATH *xpath) {
 /*
   Scan Name Test
 
-  SYNOPSIS
+  SYNOPSYS
 
     [37] NameTest ::=  '*'
                       | NCName ':' '*'
@@ -2240,7 +2240,7 @@ static int my_xpath_parse_VariableReference(MY_XPATH *xpath) {
 */
 static int my_xpath_parse_NodeTest_QName(MY_XPATH *xpath) {
   if (!my_xpath_parse_QName(xpath)) return 0;
-  assert(xpath->context);
+  DBUG_ASSERT(xpath->context);
   size_t len = xpath->prevtok.end - xpath->prevtok.beg;
   xpath->context =
       nametestfunc(xpath, xpath->axis, xpath->context, xpath->prevtok.beg, len);
@@ -2248,7 +2248,7 @@ static int my_xpath_parse_NodeTest_QName(MY_XPATH *xpath) {
 }
 static int my_xpath_parse_NodeTest_asterisk(MY_XPATH *xpath) {
   if (!my_xpath_parse_term(xpath, MY_XPATH_LEX_ASTERISK)) return 0;
-  assert(xpath->context);
+  DBUG_ASSERT(xpath->context);
   xpath->context = nametestfunc(xpath, xpath->axis, xpath->context, "*", 1);
   return 1;
 }
@@ -2260,7 +2260,7 @@ static int my_xpath_parse_NameTest(MY_XPATH *xpath) {
 /*
   Scan an XPath expression
 
-  SYNOPSIS
+  SYNOPSYS
     Scan xpath expression.
     The expression is returned in xpath->expr.
 
@@ -2281,8 +2281,8 @@ static int my_xpath_parse(MY_XPATH *xpath, const char *str,
          my_xpath_parse_term(xpath, MY_XPATH_LEX_EOF);
 }
 
-bool Item_xml_str_func::resolve_type(THD *thd) {
-  if (param_type_is_default(thd, 0, -1)) return true;
+bool Item_xml_str_func::resolve_type(THD *) {
+  nodeset_func = NULL;
 
   if (agg_arg_charsets_for_comparison(collation, args, arg_count)) return true;
 
@@ -2304,8 +2304,6 @@ bool Item_xml_str_func::resolve_type(THD *thd) {
 
   if (args[1]->const_item() && parse_xpath(args[1])) return true;
 
-  if (nodeset_func != nullptr) nodeset_func_permanent = true;
-
   return false;
 }
 
@@ -2325,13 +2323,12 @@ bool Item_xml_str_func::parse_xpath(Item *xpath_expr) {
 
   if (!rc) {
     size_t clen = xpath.query.end - xpath.lasttok.beg;
-    clen = std::min(clen, size_t(32));
+    set_if_smaller(clen, 32);
     my_printf_error(ER_UNKNOWN_ERROR, "XPATH syntax error: '%.*s'", MYF(0),
                     static_cast<int>(clen), xpath.lasttok.beg);
     return true;
   }
 
-  assert(nodeset_func == nullptr);
   nodeset_func = xpath.item;
   if (nodeset_func && nodeset_func->fix_fields(current_thd, &nodeset_func))
     return true;
@@ -2350,10 +2347,10 @@ typedef struct {
 /*
   Process tag beginning
 
-  SYNOPSIS
+  SYNOPSYS
 
     A call-back function executed when XML parser
-    is entering a tag or an attribute.
+    is entering a tag or an attribue.
     Appends the new node into data->pxml.
     Increments data->level.
 
@@ -2368,7 +2365,7 @@ int xml_enter(MY_XML_PARSER *st, const char *attr, size_t len) {
 
   node.parent = data->parent;  // Set parent for the new node to old parent
   data->parent = data->pxml->size();  // Remember current node as new parent
-  assert(data->level < MAX_LEVEL);
+  DBUG_ASSERT(data->level < MAX_LEVEL);
   data->pos[data->level] = data->pxml->size();
   if (data->level < MAX_LEVEL - 1)
     node.level = data->level++;
@@ -2384,10 +2381,10 @@ int xml_enter(MY_XML_PARSER *st, const char *attr, size_t len) {
 /*
   Process text node
 
-  SYNOPSIS
+  SYNOPSYS
 
     A call-back function executed when XML parser
-    is entering into a tag or an attribute textual value.
+    is entering into a tag or an attribue textual value.
     The value is appended into data->pxml.
 
   RETURN
@@ -2411,7 +2408,7 @@ int xml_value(MY_XML_PARSER *st, const char *attr, size_t len) {
 /*
   Leave a tag or an attribute
 
-  SYNOPSIS
+  SYNOPSYS
 
     A call-back function executed when XML parser
     is leaving a tag or an attribute.
@@ -2424,7 +2421,7 @@ extern "C" int xml_leave(MY_XML_PARSER *st, const char *attr, size_t len);
 
 int xml_leave(MY_XML_PARSER *st, const char *, size_t) {
   auto *data = reinterpret_cast<MY_XML_USER_DATA *>(st->user_data);
-  assert(data->level > 0);
+  DBUG_ASSERT(data->level > 0);
   data->level--;
 
   data->parent = data->pxml->at(data->parent).parent;
@@ -2436,7 +2433,7 @@ int xml_leave(MY_XML_PARSER *st, const char *, size_t) {
 /*
   Parse raw XML
 
-  SYNOPSIS
+  SYNOPSYS
 
 
   RETURN
@@ -2484,39 +2481,36 @@ String *Item_func_xml_extractvalue::val_str(String *str) {
   String *res;
   null_value = false;
   if (!nodeset_func && parse_xpath(args[1])) {
-    assert(is_nullable());
+    DBUG_ASSERT(maybe_null);
     null_value = true;
-    return nullptr;
+    return NULL;
   }
 
   tmp_value.set("", 0, collation.collation);
   if (!nodeset_func || !(res = args[0]->val_str(str)) ||
       !parse_xml(res, &pxml) || !(res = nodeset_func->val_str(&tmp_value))) {
-    null_value = true;
-    return nullptr;
+    null_value = 1;
+    return 0;
   }
   return res;
 }
 
 String *Item_func_xml_update::val_str(String *str) {
+  String *res, *rep;
+
   null_value = false;
   if (!nodeset_func && parse_xpath(args[1])) {
-    assert(is_nullable());
-    return error_str();
+    DBUG_ASSERT(maybe_null);
+    null_value = true;
+    return nullptr;
   }
 
-  if (nodeset_func == nullptr) return error_str();
-
-  String *res = eval_string_arg(collation.collation, args[0], str);
-  if (res == nullptr) return error_str();
-
-  StringBuffer<STRING_BUFFER_USUAL_SIZE> rep_buf(nullptr, 0,
-                                                 collation.collation);
-  String *rep = eval_string_arg(collation.collation, args[2], &rep_buf);
-  if (rep == nullptr) return error_str();
-
-  if (!parse_xml(res, &pxml)) return error_str();
-  if (nodeset_func->type() != XPATH_NODESET) return error_str();
+  if (!nodeset_func || !(res = args[0]->val_str(str)) ||
+      !(rep = args[2]->val_str(&tmp_value)) || !parse_xml(res, &pxml) ||
+      (nodeset_func->type() != XPATH_NODESET)) {
+    null_value = true;
+    return nullptr;
+  }
 
   XPathFilter nodeset;
   down_cast<const Item_nodeset_func *>(nodeset_func)->val_nodeset(&nodeset);
@@ -2529,14 +2523,13 @@ String *Item_func_xml_update::val_str(String *str) {
 
   const MY_XML_NODE *node = &pxml.at(nodeset.at(0).num);
 
-  if (node->level == 0) {
+  if (!node->level) {
     /*
       Root element, without NameTest:
       UpdateXML(xml, '/', 'replacement');
       Just return the replacement string.
     */
-    tmp_value.copy(*rep);
-    return &tmp_value;
+    return rep;
   }
 
   tmp_value.length(0);
