@@ -1,19 +1,18 @@
 #ifndef JSON_DIFF_INCLUDED
 #define JSON_DIFF_INCLUDED
 
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,9 +39,8 @@
 #include <memory>  // std::unique_ptr
 #include <vector>
 
-#include "sql-common/json_path.h"
-#include "sql/mem_root_allocator.h"
-#include "sql/psi_memory_key.h"  // key_memory_JSON
+#include "sql/json_path.h"
+#include "sql/memroot_allocator.h"
 
 class Field_json;
 class Json_dom;
@@ -104,9 +102,7 @@ class Json_diff final {
   */
   Json_diff(const Json_seekable_path &path, enum_json_diff_operation operation,
             std::unique_ptr<Json_dom> value)
-      : m_path(key_memory_JSON),
-        m_operation(operation),
-        m_value(std::move(value)) {
+      : m_path(), m_operation(operation), m_value(std::move(value)) {
     for (const Json_path_leg *leg : path) m_path.append(*leg);
   }
 
@@ -141,7 +137,7 @@ class Json_diff final {
 class Json_diff_vector {
  public:
   /// Type of the allocator for the underlying invector.
-  typedef Mem_root_allocator<Json_diff> allocator_type;
+  typedef Memroot_allocator<Json_diff> allocator_type;
   /// Type of the underlying vector
   typedef std::vector<Json_diff, allocator_type> vector;
   /// Type of iterator over the underlying vector
@@ -150,9 +146,9 @@ class Json_diff_vector {
   typedef vector::const_iterator const_iterator;
   /**
     Constructor
-    @param arg Mem_root_allocator to use for the vector
+    @param arg Memroot_allocator to use for the vector
   */
-  explicit Json_diff_vector(allocator_type arg);
+  Json_diff_vector(allocator_type arg);
   /**
     Append a new diff at the end of this vector.
     @param path Path to update
