@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -89,7 +88,7 @@ bool foreign_key_prefix(const Key_spec *a, const Key_spec *b) {
 }
 
 bool Key_part_spec::resolve_expression(THD *thd) {
-  assert(has_expression());
+  DBUG_ASSERT(has_expression());
   if (get_expression()->fixed) {
     return false;
   }
@@ -124,7 +123,7 @@ bool Foreign_key_spec::validate(THD *thd, const char *table_name,
   List_iterator<Create_field> it(table_fields);
   if (ref_columns.size() != columns.size()) {
     my_error(ER_WRONG_FK_DEF, MYF(0),
-             (has_explicit_name ? name.str : "foreign key without name"),
+             (name.str ? name.str : "foreign key without name"),
              ER_THD(thd, ER_KEY_REF_DO_NOT_MATCH_TABLE_REF));
     return true;
   }
@@ -164,7 +163,7 @@ bool Foreign_key_spec::validate(THD *thd, const char *table_name,
   }
 
   for (const Key_part_spec *fk_col : ref_columns) {
-    if (check_column_name(to_lex_cstring(fk_col->get_field_name()))) {
+    if (check_column_name(fk_col->get_field_name())) {
       my_error(ER_WRONG_COLUMN_NAME, MYF(0), fk_col->get_field_name());
       return true;
     }
