@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -107,7 +106,6 @@
 #include "sql/sql_base.h"  // MYSQL_LOCK_LOG_TABLE
 #include "sql/sql_class.h"
 #include "sql/sql_const.h"
-#include "sql/sql_db.h"  // check_schema_readonly
 #include "sql/sql_lex.h"
 #include "sql/sql_parse.h"  // is_log_table_write_query
 #include "sql/system_variables.h"
@@ -158,7 +156,7 @@ static int lock_tables_check(THD *thd, TABLE **tables, size_t count,
     TABLE *t = tables[i];
 
     /* Protect against 'fake' partially initialized TABLE_SHARE */
-    assert(t->s->table_category != TABLE_UNKNOWN_CATEGORY);
+    DBUG_ASSERT(t->s->table_category != TABLE_UNKNOWN_CATEGORY);
 
     /*
       Table I/O to performance schema tables is performed
@@ -196,11 +194,12 @@ static int lock_tables_check(THD *thd, TABLE **tables, size_t count,
       type. For table to be locked for read we must own metadata lock
       of MDL_SHARED_READ or stronger type).
     */
-    assert(t->s->tmp_table ||
-           thd->mdl_context.owns_equal_or_stronger_lock(
-               MDL_key::TABLE, t->s->db.str, t->s->table_name.str,
-               t->reginfo.lock_type >= TL_WRITE_ALLOW_WRITE ? MDL_SHARED_WRITE
-                                                            : MDL_SHARED_READ));
+    DBUG_ASSERT(t->s->tmp_table ||
+                thd->mdl_context.owns_equal_or_stronger_lock(
+                    MDL_key::TABLE, t->s->db.str, t->s->table_name.str,
+                    t->reginfo.lock_type >= TL_WRITE_ALLOW_WRITE
+                        ? MDL_SHARED_WRITE
+                        : MDL_SHARED_READ));
 
     /*
       Prevent modifications to base tables if READ_ONLY is activated.
@@ -210,10 +209,8 @@ static int lock_tables_check(THD *thd, TABLE **tables, size_t count,
     if (!(flags & MYSQL_LOCK_IGNORE_GLOBAL_READ_ONLY) && !t->s->tmp_table &&
         !is_perfschema_db(t->s->db.str, t->s->db.length)) {
       if (t->reginfo.lock_type >= TL_WRITE_ALLOW_WRITE &&
-          (check_readonly(thd, true) ||
-           check_schema_readonly(thd, t->s->db.str, t->s))) {
+          check_readonly(thd, true))
         return 1;
-      }
     }
   }
 
@@ -272,7 +269,9 @@ static void reset_lock_data(MYSQL_LOCK *sql_lock) {
 */
 
 static void track_table_access(THD *thd, TABLE **tables, size_t count) {
-  TX_TRACKER_GET(tst);
+  Transaction_state_tracker *tst =
+      (Transaction_state_tracker *)thd->session_tracker.get_tracker(
+          TRANSACTION_INFO_TRACKER);
   enum enum_tx_state s;
 
   while (count--) {
@@ -296,7 +295,7 @@ static void track_table_access(THD *thd, TABLE **tables, size_t count) {
 static void reset_lock_data_and_free(MYSQL_LOCK **mysql_lock) {
   reset_lock_data(*mysql_lock);
   my_free(*mysql_lock);
-  *mysql_lock = nullptr;
+  *mysql_lock = 0;
 }
 
 /**
@@ -323,17 +322,15 @@ MYSQL_LOCK *mysql_lock_tables(THD *thd, TABLE **tables, size_t count,
 
   DBUG_TRACE;
 
-  if (lock_tables_check(thd, tables, count, flags)) return nullptr;
+  if (lock_tables_check(thd, tables, count, flags)) return NULL;
 
   if (!(sql_lock = get_lock_data(thd, tables, count, GET_LOCK_STORE_LOCKS)))
-    return nullptr;
+    return NULL;
 
   if (!(thd->state_flags & Open_tables_state::SYSTEM_TABLES))
     THD_STAGE_INFO(thd, stage_system_lock);
 
-  ulonglong lock_start_usec = my_micro_time();
-
-  DBUG_PRINT("info", ("thd->proc_info %s", thd->proc_info()));
+  DBUG_PRINT("info", ("thd->proc_info %s", thd->proc_info));
   if (sql_lock->table_count &&
       lock_external(thd, sql_lock->table, sql_lock->table_count)) {
     /* Clear the lock type of all lock data to avoid reusage. */
@@ -364,16 +361,14 @@ end:
     thd->send_kill_message();
     if (sql_lock) {
       mysql_unlock_tables(thd, sql_lock);
-      sql_lock = nullptr;
+      sql_lock = 0;
     }
   }
 
   if (thd->variables.session_track_transaction_info > TX_TRACK_NONE)
     track_table_access(thd, tables, count);
 
-  ulonglong lock_end_usec = my_micro_time();
-  thd->inc_lock_usec(lock_end_usec - lock_start_usec);
-
+  thd->set_time_after_lock();
   return sql_lock;
 }
 
@@ -384,7 +379,7 @@ static int lock_external(THD *thd, TABLE **tables, uint count) {
 
   DBUG_PRINT("info", ("count %d", count));
   for (i = 1; i <= count; i++, tables++) {
-    assert((*tables)->reginfo.lock_type >= TL_READ);
+    DBUG_ASSERT((*tables)->reginfo.lock_type >= TL_READ);
     lock_type = F_WRLCK; /* Lock exclusive */
     if ((*tables)->db_stat & HA_READ_ONLY ||
         ((*tables)->reginfo.lock_type >= TL_READ &&
@@ -455,7 +450,7 @@ void mysql_unlock_read_tables(THD *thd, MYSQL_LOCK *sql_lock) {
   /* Move all write locked tables first */
   TABLE **table = sql_lock->table;
   for (i = found = 0; i < sql_lock->table_count; i++) {
-    assert(sql_lock->table[i]->lock_position == i);
+    DBUG_ASSERT(sql_lock->table[i]->lock_position == i);
     if ((uint)sql_lock->table[i]->reginfo.lock_type > TL_WRITE_ALLOW_WRITE) {
       std::swap(*table, sql_lock->table[i]);
       table++;
@@ -498,7 +493,7 @@ void mysql_lock_remove(THD *thd, MYSQL_LOCK *locked, TABLE *table) {
         TABLE *tbl;
         uint lock_data_end;
 
-        assert(table->lock_position == i);
+        DBUG_ASSERT(table->lock_position == i);
 
         /* Unlock the table. */
         mysql_unlock_some_tables(thd, &table, /* table count */ 1);
@@ -532,7 +527,7 @@ void mysql_lock_remove(THD *thd, MYSQL_LOCK *locked, TABLE *table) {
         for (j = i; j < old_tables; j++) {
           tbl = locked->table[j];
           tbl->lock_position--;
-          assert(tbl->lock_position == j);
+          DBUG_ASSERT(tbl->lock_position == j);
           tbl->lock_data_start -= removed_locks;
         }
 
@@ -575,7 +570,7 @@ MYSQL_LOCK *mysql_lock_merge(MYSQL_LOCK *a, MYSQL_LOCK *b) {
                 sizeof(THR_LOCK_DATA *) * (a->lock_count + b->lock_count) +
                 sizeof(TABLE *) * (a->table_count + b->table_count),
             MYF(MY_WME))))
-    return nullptr;  // Fatal error
+    return 0;  // Fatal error
   sql_lock->lock_count = a->lock_count + b->lock_count;
   sql_lock->table_count = a->table_count + b->table_count;
   sql_lock->locks = (THR_LOCK_DATA **)(sql_lock + 1);
@@ -645,7 +640,7 @@ static MYSQL_LOCK *get_lock_data(THD *thd, TABLE **table_ptr, size_t count,
   TABLE **to, **table_buf;
   DBUG_TRACE;
 
-  assert((flags == GET_LOCK_UNLOCK) || (flags == GET_LOCK_STORE_LOCKS));
+  DBUG_ASSERT((flags == GET_LOCK_UNLOCK) || (flags == GET_LOCK_STORE_LOCKS));
   DBUG_PRINT("info", ("count %zu", count));
 
   for (i = tables = lock_count = 0; i < count; i++) {
@@ -668,7 +663,7 @@ static MYSQL_LOCK *get_lock_data(THD *thd, TABLE **table_ptr, size_t count,
             sizeof(*sql_lock) + sizeof(THR_LOCK_DATA *) * tables * 2 +
                 sizeof(table_ptr) * lock_count,
             MYF(0))))
-    return nullptr;
+    return 0;
   locks = locks_buf = sql_lock->locks = (THR_LOCK_DATA **)(sql_lock + 1);
   to = table_buf = sql_lock->table = (TABLE **)(locks + tables * 2);
   sql_lock->table_count = lock_count;
@@ -681,8 +676,8 @@ static MYSQL_LOCK *get_lock_data(THD *thd, TABLE **table_ptr, size_t count,
     if ((table = table_ptr[i])->s->tmp_table == NON_TRANSACTIONAL_TMP_TABLE)
       continue;
     lock_type = table->reginfo.lock_type;
-    assert(lock_type != TL_WRITE_DEFAULT && lock_type != TL_READ_DEFAULT &&
-           lock_type != TL_WRITE_CONCURRENT_DEFAULT);
+    DBUG_ASSERT(lock_type != TL_WRITE_DEFAULT && lock_type != TL_READ_DEFAULT &&
+                lock_type != TL_WRITE_CONCURRENT_DEFAULT);
     locks_start = locks;
     locks = table->file->store_lock(
         thd, locks, (flags & GET_LOCK_UNLOCK) ? TL_IGNORE : lock_type);
@@ -763,19 +758,6 @@ bool lock_schema_name(THD *thd, const char *db) {
                                      thd->variables.lock_wait_timeout))
     return true;
 
-  /*
-    Now when we have protection against concurrent change of read_only
-    option we can safely re-check its value.
-  */
-  if (check_readonly(thd, true)) return true;
-
-  /*
-    We have an IX lock on the schema name, so we can check the read
-    only option of the schema without worrying about a concurrent
-    ALTER SCHEMA.
-  */
-  if (check_schema_readonly(thd, db)) return true;
-
   DEBUG_SYNC(thd, "after_wait_locked_schema_name");
   return false;
 }
@@ -786,24 +768,22 @@ bool lock_schema_name(THD *thd, const char *db) {
   @param thd               - Thread invoking this function.
   @param tablespace_set    - Set of tablespace names to be lock.
   @param lock_wait_timeout - Lock timeout.
-  @param mem_root          - Memory root on which MDL_request objects
-                             can be allocated.
 
   @return true - On failure
   @return false - On Success.
 */
 bool lock_tablespace_names(THD *thd, Tablespace_hash_set *tablespace_set,
-                           ulong lock_wait_timeout, MEM_ROOT *mem_root) {
+                           ulong lock_wait_timeout) {
   // Stop if we have nothing to lock
   if (tablespace_set->empty()) return false;
 
   // Prepare MDL_request's for all tablespace names.
   MDL_request_list mdl_tablespace_requests;
   for (const std::string &tablespace : *tablespace_set) {
-    assert(!tablespace.empty());
+    DBUG_ASSERT(!tablespace.empty());
 
-    MDL_request *tablespace_request = new (mem_root) MDL_request;
-    if (tablespace_request == nullptr) return true;
+    MDL_request *tablespace_request = new (thd->mem_root) MDL_request;
+    if (tablespace_request == NULL) return true;
     MDL_REQUEST_INIT(tablespace_request, MDL_key::TABLESPACE, "",
                      tablespace.c_str(), MDL_INTENTION_EXCLUSIVE,
                      MDL_TRANSACTION);
@@ -858,7 +838,7 @@ bool lock_object_name(THD *thd, MDL_key::enum_mdl_namespace mdl_type,
     return true;
   }
 
-  assert(name);
+  DBUG_ASSERT(name);
 
   switch (mdl_type) {
     case MDL_key::FUNCTION:
@@ -874,7 +854,7 @@ bool lock_object_name(THD *thd, MDL_key::enum_mdl_namespace mdl_type,
       dd::Resource_group::create_mdl_key(name, &mdl_key);
       break;
     default:
-      assert(false);
+      DBUG_ASSERT(false);
       return true;
   }
 
@@ -899,19 +879,6 @@ bool lock_object_name(THD *thd, MDL_key::enum_mdl_namespace mdl_type,
                                      thd->variables.lock_wait_timeout))
     return true;
 
-  /*
-    Now when we have protection against concurrent change of read_only
-    option we can safely re-check its value.
-  */
-  if (check_readonly(thd, true)) return true;
-
-  /*
-    We have an IX lock on the schema name, so we can check the read
-    only option of the schema without worrying about a concurrent
-    ALTER SCHEMA.
-  */
-  if (check_schema_readonly(thd, db)) return true;
-
   DEBUG_SYNC(thd, "after_wait_locked_pname");
   return false;
 }
@@ -1012,23 +979,14 @@ std::atomic<int32> Global_read_lock::m_atomic_active_requests;
 
 bool acquire_shared_global_read_lock(THD *thd,
                                      unsigned long lock_wait_timeout) {
-  // If we cannot acquire protection against GRL, err out.
+  // If we cannot acuqire protection against GRL, err out.
   if (thd->global_read_lock.can_acquire_protection()) return true;
 
   MDL_request grl_request;
   MDL_REQUEST_INIT(&grl_request, MDL_key::GLOBAL, "", "",
                    MDL_INTENTION_EXCLUSIVE, MDL_TRANSACTION);
 
-  if (thd->mdl_context.acquire_lock(&grl_request, lock_wait_timeout))
-    return true;
-
-  /*
-    Now when we have protection against concurrent change of read_only
-    option we can safely re-check its value.
-  */
-  if (check_readonly(thd, true)) return true;
-
-  return false;
+  return thd->mdl_context.acquire_lock(&grl_request, lock_wait_timeout);
 }
 
 /**
@@ -1050,8 +1008,8 @@ bool Global_read_lock::lock_global_read_lock(THD *thd) {
   if (!m_state) {
     MDL_request mdl_request;
 
-    assert(!thd->mdl_context.owns_equal_or_stronger_lock(MDL_key::GLOBAL, "",
-                                                         "", MDL_SHARED));
+    DBUG_ASSERT(!thd->mdl_context.owns_equal_or_stronger_lock(
+        MDL_key::GLOBAL, "", "", MDL_SHARED));
     MDL_REQUEST_INIT(&mdl_request, MDL_key::GLOBAL, "", "", MDL_SHARED,
                      MDL_EXPLICIT);
 
@@ -1061,7 +1019,7 @@ bool Global_read_lock::lock_global_read_lock(THD *thd) {
     if (thd->mdl_context.acquire_lock(&mdl_request,
                                       thd->variables.lock_wait_timeout)) {
       Global_read_lock::m_atomic_active_requests--;
-      return true;
+      return 1;
     }
 
     m_mdl_global_shared_lock = mdl_request.ticket;
@@ -1075,7 +1033,7 @@ bool Global_read_lock::lock_global_read_lock(THD *thd) {
     forbid it before, or we can have a 3-thread deadlock if 2 do SELECT FOR
     UPDATE and one does FLUSH TABLES WITH READ LOCK).
   */
-  return false;
+  return 0;
 }
 
 /**
@@ -1091,15 +1049,15 @@ bool Global_read_lock::lock_global_read_lock(THD *thd) {
 void Global_read_lock::unlock_global_read_lock(THD *thd) {
   DBUG_TRACE;
 
-  assert(m_mdl_global_shared_lock && m_state);
+  DBUG_ASSERT(m_mdl_global_shared_lock && m_state);
 
   if (m_mdl_blocks_commits_lock) {
     thd->mdl_context.release_lock(m_mdl_blocks_commits_lock);
-    m_mdl_blocks_commits_lock = nullptr;
+    m_mdl_blocks_commits_lock = NULL;
   }
   thd->mdl_context.release_lock(m_mdl_global_shared_lock);
   Global_read_lock::m_atomic_active_requests--;
-  m_mdl_global_shared_lock = nullptr;
+  m_mdl_global_shared_lock = NULL;
   m_state = GRL_NONE;
 }
 
@@ -1122,10 +1080,10 @@ bool Global_read_lock::make_global_read_lock_block_commit(THD *thd) {
   MDL_request mdl_request;
   DBUG_TRACE;
   /*
-    If we didn't succeed lock_global_read_lock(), or if we already succeeded
+    If we didn't succeed lock_global_read_lock(), or if we already suceeded
     make_global_read_lock_block_commit(), do nothing.
   */
-  if (m_state != GRL_ACQUIRED) return false;
+  if (m_state != GRL_ACQUIRED) return 0;
 
   MDL_REQUEST_INIT(&mdl_request, MDL_key::COMMIT, "", "", MDL_SHARED,
                    MDL_EXPLICIT);
