@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,20 +26,23 @@
 #include "sql/mysqld.h"     // table_alias_charset
 #include "sql/sql_base.h"   // close_thread_tables
 #include "sql/sql_class.h"  // THD
-#include "sql/table.h"      // TABLE Table_ref
+#include "sql/table.h"      // TABLE TABLE_LIST
 
 Locked_tables_list::Locked_tables_list()
     : m_locked_tables(nullptr),
       m_locked_tables_last(&m_locked_tables),
       m_reopen_array(nullptr),
-      m_locked_tables_count(0) {}
+      m_locked_tables_count(0) {
+  init_sql_alloc(key_memory_locked_table_list, &m_locked_tables_root,
+                 MEM_ROOT_BLOCK_SIZE, 0);
+}
 
 /**
   Enter LTM_LOCK_TABLES mode.
 
   Enter the LOCK TABLES mode using all the tables that are
   currently open and locked in this connection.
-  Initializes a Table_ref instance for every locked table.
+  Initializes a TABLE_LIST instance for every locked table.
 
   @param  thd  thread handle
 
@@ -48,24 +50,24 @@ Locked_tables_list::Locked_tables_list()
 */
 
 bool Locked_tables_list::init_locked_tables(THD *thd) {
-  assert(thd->locked_tables_mode == LTM_NONE);
-  assert(m_locked_tables == nullptr);
-  assert(m_reopen_array == nullptr);
-  assert(m_locked_tables_count == 0);
+  DBUG_ASSERT(thd->locked_tables_mode == LTM_NONE);
+  DBUG_ASSERT(m_locked_tables == NULL);
+  DBUG_ASSERT(m_reopen_array == NULL);
+  DBUG_ASSERT(m_locked_tables_count == 0);
 
   for (TABLE *table = thd->open_tables; table;
        table = table->next, m_locked_tables_count++) {
-    Table_ref *src_table_list = table->pos_in_table_list;
+    TABLE_LIST *src_table_list = table->pos_in_table_list;
     char *db, *table_name, *alias;
     size_t db_len = src_table_list->db_length;
     size_t table_name_len = src_table_list->table_name_length;
     size_t alias_len = strlen(src_table_list->alias);
-    Table_ref *dst_table_list;
+    TABLE_LIST *dst_table_list;
 
     if (!multi_alloc_root(&m_locked_tables_root, &dst_table_list,
                           sizeof(*dst_table_list), &db, db_len + 1, &table_name,
                           table_name_len + 1, &alias, alias_len + 1, NullS)) {
-      unlock_locked_tables(nullptr);
+      unlock_locked_tables(0);
       return true;
     }
 
@@ -80,9 +82,11 @@ bool Locked_tables_list::init_locked_tables(THD *thd) {
       thd->update_lock_default.
     */
     new (dst_table_list)
-        Table_ref(table, db, db_len, table_name, table_name_len, alias,
-                  src_table_list->table->reginfo.lock_type);
+        TABLE_LIST(table, alias, src_table_list->table->reginfo.lock_type);
 
+    dst_table_list->db = db;
+    dst_table_list->table_name = table_name;
+    dst_table_list->alias = alias;
     dst_table_list->mdl_request.ticket = src_table_list->mdl_request.ticket;
 
     /* Link last into the list of tables */
@@ -98,15 +102,16 @@ bool Locked_tables_list::init_locked_tables(THD *thd) {
     */
     m_reopen_array = (TABLE **)m_locked_tables_root.Alloc(
         sizeof(TABLE *) * (m_locked_tables_count + 1));
-    if (m_reopen_array == nullptr) {
-      unlock_locked_tables(nullptr);
+    if (m_reopen_array == NULL) {
+      unlock_locked_tables(0);
       return true;
     }
   }
 
   if (thd->variables.session_track_transaction_info > TX_TRACK_NONE) {
-    TX_TRACKER_GET(tst);
-    tst->add_trx_state(thd, TX_LOCKED_TABLES);
+    ((Transaction_state_tracker *)thd->session_tracker.get_tracker(
+         TRANSACTION_INFO_TRACKER))
+        ->add_trx_state(thd, TX_LOCKED_TABLES);
   }
 
   thd->enter_locked_tables_mode(LTM_LOCK_TABLES);
@@ -126,8 +131,8 @@ void Locked_tables_list::unlock_locked_tables(THD *thd)
 
 {
   if (thd) {
-    assert(!thd->in_sub_stmt &&
-           !(thd->state_flags & Open_tables_state::BACKUPS_AVAIL));
+    DBUG_ASSERT(!thd->in_sub_stmt &&
+                !(thd->state_flags & Open_tables_state::BACKUPS_AVAIL));
     /*
       Sic: we must be careful to not close open tables if
       we're not in LOCK TABLES mode: unlock_locked_tables() is
@@ -136,22 +141,23 @@ void Locked_tables_list::unlock_locked_tables(THD *thd)
     */
     if (thd->locked_tables_mode != LTM_LOCK_TABLES) return;
 
-    for (Table_ref *table_list = m_locked_tables; table_list;
+    for (TABLE_LIST *table_list = m_locked_tables; table_list;
          table_list = table_list->next_global) {
       /*
         Clear the position in the list, the TABLE object will be
         returned to the table cache.
       */
-      table_list->table->pos_in_locked_tables = nullptr;
+      table_list->table->pos_in_locked_tables = NULL;
     }
     thd->leave_locked_tables_mode();
 
     if (thd->variables.session_track_transaction_info > TX_TRACK_NONE) {
-      TX_TRACKER_GET(tst);
-      tst->clear_trx_state(thd, TX_LOCKED_TABLES);
+      ((Transaction_state_tracker *)thd->session_tracker.get_tracker(
+           TRANSACTION_INFO_TRACKER))
+          ->clear_trx_state(thd, TX_LOCKED_TABLES);
     }
 
-    assert(thd->get_transaction()->is_empty(Transaction_ctx::STMT));
+    DBUG_ASSERT(thd->get_transaction()->is_empty(Transaction_ctx::STMT));
     close_thread_tables(thd);
     /*
       We rely on the caller to implicitly commit the
@@ -160,12 +166,12 @@ void Locked_tables_list::unlock_locked_tables(THD *thd)
   }
   /*
     After closing tables we can free memory used for storing lock
-    request for metadata locks and Table_ref elements.
+    request for metadata locks and TABLE_LIST elements.
   */
-  m_locked_tables_root.Clear();
-  m_locked_tables = nullptr;
+  free_root(&m_locked_tables_root, MYF(0));
+  m_locked_tables = NULL;
   m_locked_tables_last = &m_locked_tables;
-  m_reopen_array = nullptr;
+  m_reopen_array = NULL;
   m_locked_tables_count = 0;
 }
 
@@ -176,8 +182,8 @@ void Locked_tables_list::unlock_locked_tables(THD *thd)
   @param  thd        thread handle
   @param  table_list the element of locked tables list.
                      The implementation assumes that this argument
-                     points to a Table_ref element linked into
-                     the locked tables list. Passing a Table_ref
+                     points to a TABLE_LIST element linked into
+                     the locked tables list. Passing a TABLE_LIST
                      instance that is not part of locked tables
                      list will lead to a crash.
   @param  remove_from_locked_tables
@@ -189,7 +195,8 @@ void Locked_tables_list::unlock_locked_tables(THD *thd)
   @sa Locked_tables_list::reopen_tables()
 */
 
-void Locked_tables_list::unlink_from_list(const THD *thd, Table_ref *table_list,
+void Locked_tables_list::unlink_from_list(const THD *thd,
+                                          TABLE_LIST *table_list,
                                           bool remove_from_locked_tables) {
   /*
     If mode is not LTM_LOCK_TABLES, we needn't do anything. Moreover,
@@ -201,13 +208,13 @@ void Locked_tables_list::unlink_from_list(const THD *thd, Table_ref *table_list,
     table_list must be set and point to pos_in_locked_tables of some
     table.
   */
-  assert(table_list->table->pos_in_locked_tables == table_list);
+  DBUG_ASSERT(table_list->table->pos_in_locked_tables == table_list);
 
   /* Clear the pointer, the table will be returned to the table cache. */
-  table_list->table->pos_in_locked_tables = nullptr;
+  table_list->table->pos_in_locked_tables = NULL;
 
   /* Mark the table as closed in the locked tables list. */
-  table_list->table = nullptr;
+  table_list->table = NULL;
 
   /*
     If the table is being dropped or renamed, remove it from
@@ -216,7 +223,7 @@ void Locked_tables_list::unlink_from_list(const THD *thd, Table_ref *table_list,
   */
   if (remove_from_locked_tables) {
     *table_list->prev_global = table_list->next_global;
-    if (table_list->next_global == nullptr)
+    if (table_list->next_global == NULL)
       m_locked_tables_last = table_list->prev_global;
     else
       table_list->next_global->prev_global = table_list->prev_global;
@@ -250,20 +257,20 @@ void Locked_tables_list::unlink_all_closed_tables(THD *thd, MYSQL_LOCK *lock,
         in reopen_tables() always links the opened table
         to the beginning of the open_tables list.
       */
-      assert(thd->open_tables == m_reopen_array[reopen_count]);
+      DBUG_ASSERT(thd->open_tables == m_reopen_array[reopen_count]);
 
-      thd->open_tables->pos_in_locked_tables->table = nullptr;
+      thd->open_tables->pos_in_locked_tables->table = NULL;
 
       close_thread_table(thd, &thd->open_tables);
     }
   }
   /* Exclude all closed tables from the LOCK TABLES list. */
-  for (Table_ref *table_list = m_locked_tables; table_list;
+  for (TABLE_LIST *table_list = m_locked_tables; table_list;
        table_list = table_list->next_global) {
-    if (table_list->table == nullptr) {
+    if (table_list->table == NULL) {
       /* Unlink from list. */
       *table_list->prev_global = table_list->next_global;
-      if (table_list->next_global == nullptr)
+      if (table_list->next_global == NULL)
         m_locked_tables_last = table_list->prev_global;
       else
         table_list->next_global->prev_global = table_list->prev_global;
@@ -298,14 +305,14 @@ bool Locked_tables_list::reopen_tables(THD *thd) {
   Diagnostics_area tmp_da(false);
   thd->push_diagnostics_area(&tmp_da, false);
 
-  for (Table_ref *table_list = m_locked_tables; table_list;
+  for (TABLE_LIST *table_list = m_locked_tables; table_list;
        table_list = table_list->next_global) {
     if (table_list->table) /* The table was not closed */
       continue;
 
     /* Links into thd->open_tables upon success */
     if (open_table(thd, table_list, &ot_ctx)) {
-      unlink_all_closed_tables(thd, nullptr, reopen_count);
+      unlink_all_closed_tables(thd, 0, reopen_count);
       thd->pop_diagnostics_area();
       if (!thd->get_stmt_da()->is_error() && tmp_da.is_error()) {
         // Copy the exception condition information.
@@ -320,14 +327,14 @@ bool Locked_tables_list::reopen_tables(THD *thd) {
     /* See also the comment on lock type in init_locked_tables(). */
     table_list->table->reginfo.lock_type = table_list->lock_descriptor().type;
 
-    assert(reopen_count < m_locked_tables_count);
+    DBUG_ASSERT(reopen_count < m_locked_tables_count);
     m_reopen_array[reopen_count++] = table_list->table;
   }
 
   thd->pop_diagnostics_area();
 
   if (reopen_count) {
-    thd->in_lock_tables = true;
+    thd->in_lock_tables = 1;
     /*
       We re-lock all tables with mysql_lock_tables() at once rather
       than locking one table at a time because of the case
@@ -341,9 +348,9 @@ bool Locked_tables_list::reopen_tables(THD *thd) {
     */
     lock =
         mysql_lock_tables(thd, m_reopen_array, reopen_count, MYSQL_OPEN_REOPEN);
-    thd->in_lock_tables = false;
-    if (lock == nullptr ||
-        (merged_lock = mysql_lock_merge(thd->lock, lock)) == nullptr) {
+    thd->in_lock_tables = 0;
+    if (lock == NULL ||
+        (merged_lock = mysql_lock_merge(thd->lock, lock)) == NULL) {
       unlink_all_closed_tables(thd, lock, reopen_count);
       if (!thd->killed) my_error(ER_LOCK_DEADLOCK, MYF(0));
       return true;
@@ -366,20 +373,20 @@ bool Locked_tables_list::reopen_tables(THD *thd) {
   @note This function is a no-op if we're not under LOCK TABLES.
 */
 
-void Locked_tables_list::rename_locked_table(Table_ref *old_table_list,
+void Locked_tables_list::rename_locked_table(TABLE_LIST *old_table_list,
                                              const char *new_db,
                                              const char *new_table_name,
                                              MDL_ticket *target_mdl_ticket) {
-  for (Table_ref *table_list = m_locked_tables; table_list;
+  for (TABLE_LIST *table_list = m_locked_tables; table_list;
        table_list = table_list->next_global) {
     if (my_strcasecmp(table_alias_charset, table_list->db,
                       old_table_list->db) == 0 &&
         my_strcasecmp(table_alias_charset, table_list->table_name,
                       old_table_list->table_name) == 0) {
-      assert(table_list->table == nullptr);
+      DBUG_ASSERT(table_list->table == nullptr);
 
       /*
-        Update Table_ref element with new db and name. Allocate
+        Update TABLE_LIST element with new db and name. Allocate
         them on Locked_tables_list private memory root.
       */
       size_t new_db_len = strlen(new_db);
@@ -390,8 +397,8 @@ void Locked_tables_list::rename_locked_table(Table_ref *old_table_list,
           &m_locked_tables_root, new_table_name, new_table_name_len);
 
       if (new_db_root != nullptr && new_table_name_root != nullptr) {
-        Table_ref *save_next_global = table_list->next_global;
-        Table_ref **save_prev_global = table_list->prev_global;
+        TABLE_LIST *save_next_global = table_list->next_global;
+        TABLE_LIST **save_prev_global = table_list->prev_global;
 
         /*
           If explicit alias was used in LOCK TABLES then it makes sense
@@ -403,7 +410,7 @@ void Locked_tables_list::rename_locked_table(Table_ref *old_table_list,
             my_strcasecmp(table_alias_charset, table_list->table_name,
                           table_list->alias) != 0;
 
-        *table_list = Table_ref(
+        *table_list = TABLE_LIST(
             new_db_root, new_db_len, new_table_name_root, new_table_name_len,
             real_alias ? table_list->alias : new_table_name_root,
             table_list->lock_descriptor().type);
@@ -425,8 +432,8 @@ void Locked_tables_list::rename_locked_table(Table_ref *old_table_list,
 
 void Locked_tables_list::add_rename_tablespace_mdls(MDL_ticket *src,
                                                     MDL_ticket *dst) {
-  assert(m_locked_tables != nullptr);
-  assert(dst->get_duration() == MDL_TRANSACTION);
+  DBUG_ASSERT(m_locked_tables != nullptr);
+  DBUG_ASSERT(dst->get_duration() == MDL_TRANSACTION);
   m_rename_tablespace_mdls.push_back({src, dst});
 }
 
