@@ -82,6 +82,7 @@
 #include "sql/field.h"
 #include "sql/handler.h"
 #include "sql/mysqld.h"
+#include "sql/protocol_classic.h"
 #include "sql/psi_memory_key.h"  // key_memory_File_query_log_name
 #include "sql/query_options.h"
 #include "sql/sql_audit.h"  // mysql_audit_general_log
@@ -486,11 +487,11 @@ bool File_query_log::open() {
   my_off_t pos = 0;
   char buff[FN_REFLEN];
   MY_STAT f_stat;
-  DBUG_ENTER("File_query_log::open");
+  DBUG_TRACE;
 
   DBUG_ASSERT(name != nullptr);
 
-  if (is_open()) DBUG_RETURN(false);
+  if (is_open()) return false;
 
   write_error = false;
 
@@ -562,7 +563,7 @@ bool File_query_log::open() {
   }
 
   log_open = true;
-  DBUG_RETURN(false);
+  return false;
 
 err:
   char log_open_file_error_message[96] = "";
@@ -588,12 +589,12 @@ err:
   end_io_cache(&log_file);
 
   log_open = false;
-  DBUG_RETURN(true);
+  return true;
 }
 
 void File_query_log::close() {
-  DBUG_ENTER("File_query_log::close");
-  if (!is_open()) DBUG_VOID_RETURN;
+  DBUG_TRACE;
+  if (!is_open()) return;
 
   end_io_cache(&log_file);
 
@@ -604,8 +605,6 @@ void File_query_log::close() {
     check_and_print_write_error();
 
   log_open = false;
-
-  DBUG_VOID_RETURN;
 }
 
 void File_query_log::check_and_print_write_error() {
@@ -988,7 +987,7 @@ bool Log_to_csv_event_handler::log_slow(
   struct timeval tv;
   const char *reason = "";
 
-  DBUG_ENTER("Log_to_csv_event_handler::log_slow");
+  DBUG_TRACE;
 
   /*
     CSV uses TIME_to_timestamp() internally if table needs to be repaired
@@ -1140,37 +1139,39 @@ err:
   if (need_close) close_log_table(thd, &open_tables_backup);
 
   thd->time_zone_used = save_time_zone_used;
-  DBUG_RETURN(result);
+  return result;
 }
 
 bool Log_to_csv_event_handler::activate_log(
     THD *thd, enum_log_table_type log_table_type) {
-  TABLE_LIST table_list;
+  DBUG_TRACE;
 
-  DBUG_ENTER("Log_to_csv_event_handler::activate_log");
+  const char *log_name = nullptr;
+  size_t log_name_length = 0;
 
   switch (log_table_type) {
     case QUERY_LOG_GENERAL:
-      table_list.init_one_table(MYSQL_SCHEMA_NAME.str, MYSQL_SCHEMA_NAME.length,
-                                GENERAL_LOG_NAME.str, GENERAL_LOG_NAME.length,
-                                GENERAL_LOG_NAME.str,
-                                TL_WRITE_CONCURRENT_INSERT);
+      log_name = GENERAL_LOG_NAME.str;
+      log_name_length = GENERAL_LOG_NAME.length;
       break;
     case QUERY_LOG_SLOW:
-      table_list.init_one_table(MYSQL_SCHEMA_NAME.str, MYSQL_SCHEMA_NAME.length,
-                                SLOW_LOG_NAME.str, SLOW_LOG_NAME.length,
-                                SLOW_LOG_NAME.str, TL_WRITE_CONCURRENT_INSERT);
+      log_name = SLOW_LOG_NAME.str;
+      log_name_length = SLOW_LOG_NAME.length;
       break;
     default:
       DBUG_ASSERT(false);
   }
 
+  TABLE_LIST table_list(MYSQL_SCHEMA_NAME.str, MYSQL_SCHEMA_NAME.length,
+                        log_name, log_name_length, log_name,
+                        TL_WRITE_CONCURRENT_INSERT);
+
   Open_tables_backup open_tables_backup;
   if (open_log_table(thd, &table_list, &open_tables_backup) != NULL) {
     close_log_table(thd, &open_tables_backup);
-    DBUG_RETURN(false);
+    return false;
   }
-  DBUG_RETURN(true);
+  return true;
 }
 
 /**
@@ -1580,15 +1581,14 @@ char *make_query_log_name(char *buff, enum_log_table_type log_type) {
 }
 
 bool log_slow_applicable(THD *thd) {
-  DBUG_ENTER("log_slow_applicable");
+  DBUG_TRACE;
 
   /*
     The following should never be true with our current code base,
     but better to keep this here so we don't accidently try to log a
     statement in a trigger or stored function
   */
-  if (unlikely(thd->in_sub_stmt))
-    DBUG_RETURN(false);  // Don't set time for sub stmt
+  if (unlikely(thd->in_sub_stmt)) return false;  // Don't set time for sub stmt
 
   /*
     Do not log administrative statements unless the appropriate option is
@@ -1606,9 +1606,9 @@ bool log_slow_applicable(THD *thd) {
          thd->variables.min_examined_row_limit);
     bool suppress_logging = log_throttle_qni.log(thd, warn_no_index);
 
-    if (!suppress_logging && log_this_query) DBUG_RETURN(true);
+    if (!suppress_logging && log_this_query) return true;
   }
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -1987,7 +1987,7 @@ bool reopen_error_log() {
   @retval          int                  number of added fields, if any
 */
 void log_write_errstream(const char *buffer, size_t length) {
-  DBUG_ENTER("log_write_errstream");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("buffer: %s", buffer));
 
   /*
@@ -2001,8 +2001,6 @@ void log_write_errstream(const char *buffer, size_t length) {
   fflush(stderr);
 
   if (error_log_initialized) mysql_mutex_unlock(&LOCK_error_log);
-
-  DBUG_VOID_RETURN;
 }
 
 my_thread_id log_get_thread_id(THD *thd) { return thd->thread_id(); }
@@ -2057,7 +2055,7 @@ int log_vmessage(int log_type MY_ATTRIBUTE((unused)), va_list fili) {
   bool dedup;
   int wk;
 
-  DBUG_ENTER("log_message");
+  DBUG_TRACE;
 
   ll.count = 0;
   ll.seen = 0;
@@ -2149,7 +2147,7 @@ int log_vmessage(int log_type MY_ATTRIBUTE((unused)), va_list fili) {
           to free anything here.
         */
         DBUG_ASSERT(false);
-        DBUG_RETURN(-1);
+        return -1;
     }
 
     /*
@@ -2278,7 +2276,7 @@ int log_vmessage(int log_type MY_ATTRIBUTE((unused)), va_list fili) {
 
   va_end(fili);
 
-  DBUG_RETURN(log_line_submit(&ll));
+  return log_line_submit(&ll);
 }
 
 /**
@@ -2293,15 +2291,13 @@ int log_vmessage(int log_type MY_ATTRIBUTE((unused)), va_list fili) {
 
 */
 void error_log_print(enum loglevel level, uint ecode, va_list args) {
-  DBUG_ENTER("error_log_print");
+  DBUG_TRACE;
 
   LogEvent()
       .type(LOG_TYPE_ERROR)
       .errcode(ecode)
       .prio(level)
       .messagev(EE(ecode), args);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
