@@ -1,18 +1,17 @@
 #ifndef MDL_H
 #define MDL_H
-/* Copyright (c) 2009, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <assert.h>
 #include <string.h>
 #include <sys/types.h>
 #include <algorithm>
@@ -33,16 +31,16 @@
 #include "m_string.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
 #include "my_systime.h"  // Timout_type
-#include "mysql/components/services/bits/mysql_cond_bits.h"
-#include "mysql/components/services/bits/mysql_mutex_bits.h"
-#include "mysql/components/services/bits/mysql_rwlock_bits.h"
-#include "mysql/components/services/bits/psi_mdl_bits.h"
-#include "mysql/components/services/bits/psi_stage_bits.h"
+#include "mysql/components/services/mysql_cond_bits.h"
+#include "mysql/components/services/mysql_mutex_bits.h"
+#include "mysql/components/services/mysql_rwlock_bits.h"
+#include "mysql/components/services/psi_mdl_bits.h"
+#include "mysql/components/services/psi_stage_bits.h"
 #include "mysql/psi/mysql_rwlock.h"
 #include "mysql_com.h"
 #include "sql/sql_plist.h"
@@ -55,9 +53,7 @@ class THD;
 struct LF_PINS;
 struct MDL_key;
 struct MEM_ROOT;
-namespace mdl_unittest {
-bool test_drive_fix_pins(MDL_context *);
-}
+
 /**
   @def ENTER_COND(C, M, S, O)
   Start a wait on a condition.
@@ -84,7 +80,7 @@ bool test_drive_fix_pins(MDL_context *);
 
 class MDL_context_owner {
  public:
-  virtual ~MDL_context_owner() = default;
+  virtual ~MDL_context_owner() {}
 
   /**
     Enter a condition wait.
@@ -126,20 +122,7 @@ class MDL_context_owner {
   /**
     Does the owner still have connection to the client?
   */
-  virtual bool is_connected(bool = false) = 0;
-
-  /**
-    Indicates that owner thread might have some commit order (non-MDL) waits
-    for it which are still taken into account by MDL deadlock detection,
-    even in cases when it doesn't have any MDL locks acquired and therefore
-    can't have any MDL waiters.
-
-    @note It is important for this check to be non-racy and quick (perhaps
-          at expense of being pretty pessimistic), so it can be used to
-          make decisions reliably about whether we can skip deadlock
-          detection in some cases.
-  */
-  virtual bool might_have_commit_order_waiters() const = 0;
+  virtual bool is_connected() = 0;
 
   /**
      Within MDL subsystem this one is only used for DEBUG_SYNC.
@@ -375,9 +358,6 @@ struct MDL_key {
 
     Different types of objects exist in different namespaces
      - GLOBAL is used for the global read lock.
-     - BACKUP_LOCK is to block any operations that could cause
-       inconsistent backup. Such operations are most DDL statements,
-       and some administrative statements.
      - TABLESPACE is for tablespaces.
      - SCHEMA is for schemas (aka databases).
      - TABLE is for tables and views.
@@ -391,6 +371,9 @@ struct MDL_key {
      - SRID is for spatial reference systems
      - ACL_CACHE is for ACL caches
      - COLUMN_STATISTICS is for column statistics, such as histograms
+     - BACKUP_LOCK is to block any operations that could cause
+       inconsistent backup. Such operations are most DDL statements,
+       and some administrative statements.
      - RESOURCE_GROUPS is for resource groups.
      - FOREIGN_KEY is for foreign key names.
      - CHECK_CONSTRAINT is for check constraint names.
@@ -399,7 +382,6 @@ struct MDL_key {
   */
   enum enum_mdl_namespace {
     GLOBAL = 0,
-    BACKUP_LOCK,
     TABLESPACE,
     SCHEMA,
     TABLE,
@@ -413,6 +395,7 @@ struct MDL_key {
     SRID,
     ACL_CACHE,
     COLUMN_STATISTICS,
+    BACKUP_LOCK,
     RESOURCE_GROUPS,
     FOREIGN_KEY,
     CHECK_CONSTRAINT,
@@ -433,7 +416,7 @@ struct MDL_key {
   uint name_length() const { return m_object_name_length; }
 
   const char *col_name() const {
-    assert(!use_normalized_object_name());
+    DBUG_ASSERT(!use_normalized_object_name());
 
     if (m_db_name_length + m_object_name_length + 3 < m_length) {
       /* A column name was stored in the key buffer. */
@@ -441,11 +424,11 @@ struct MDL_key {
     }
 
     /* No column name stored. */
-    return nullptr;
+    return NULL;
   }
 
   uint col_name_length() const {
-    assert(!use_normalized_object_name());
+    DBUG_ASSERT(!use_normalized_object_name());
 
     if (m_db_name_length + m_object_name_length + 3 < m_length) {
       /* A column name was stored in the key buffer. */
@@ -475,7 +458,7 @@ struct MDL_key {
                     const char *name) {
     m_ptr[0] = (char)mdl_namespace;
 
-    assert(!use_normalized_object_name());
+    DBUG_ASSERT(!use_normalized_object_name());
 
     /*
       It is responsibility of caller to ensure that db and object names
@@ -496,8 +479,8 @@ struct MDL_key {
             implementation, possibly relying on the key format,
             must be investigated first, though.
     */
-    assert(strlen(db) <= NAME_LEN);
-    assert((mdl_namespace == TABLESPACE) || (strlen(name) <= NAME_LEN));
+    DBUG_ASSERT(strlen(db) <= NAME_LEN);
+    DBUG_ASSERT((mdl_namespace == TABLESPACE) || (strlen(name) <= NAME_LEN));
     m_db_name_length =
         static_cast<uint16>(strmake(m_ptr + 1, db, NAME_LEN) - m_ptr - 1);
     m_object_name_length = static_cast<uint16>(
@@ -524,20 +507,20 @@ struct MDL_key {
     char *start;
     char *end;
 
-    assert(!use_normalized_object_name());
+    DBUG_ASSERT(!use_normalized_object_name());
 
-    assert(strlen(db) <= NAME_LEN);
+    DBUG_ASSERT(strlen(db) <= NAME_LEN);
     start = m_ptr + 1;
     end = strmake(start, db, NAME_LEN);
     m_db_name_length = static_cast<uint16>(end - start);
 
-    assert(strlen(name) <= NAME_LEN);
+    DBUG_ASSERT(strlen(name) <= NAME_LEN);
     start = end + 1;
     end = strmake(start, name, NAME_LEN);
     m_object_name_length = static_cast<uint16>(end - start);
 
     size_t col_len = strlen(column_name);
-    assert(col_len <= NAME_LEN);
+    DBUG_ASSERT(col_len <= NAME_LEN);
     start = end + 1;
     size_t remaining =
         MAX_MDLKEY_LENGTH - m_db_name_length - m_object_name_length - 3;
@@ -600,7 +583,7 @@ struct MDL_key {
       extra_length = static_cast<uint16>(end - start) + 1;  // With \0
     }
     m_length = m_db_name_length + m_object_name_length + 3 + extra_length;
-    assert(m_length <= MAX_MDLKEY_LENGTH);
+    DBUG_ASSERT(m_length <= MAX_MDLKEY_LENGTH);
   }
 
   /**
@@ -609,25 +592,23 @@ struct MDL_key {
 
     @remark The key for a routine/event/resource group/trigger is
       @<mdl_namespace@>+@<database name@>+@<normalized object name@>
-      additionally @<object name@> is stored in the same buffer for information
-      purpose if buffer has sufficient space.
+      additionaly @<object name@> is stored in the same buffer for information
+      purpose if buffer has sufficent space.
 
     Routine, Event and Resource group names are case sensitive and accent
     sensitive. So normalized object name is used to form a MDL_key.
 
-    With the UTF8MB3 charset space reserved for the db name/object
-    name is 64 * 3 bytes. utf8mb3_general_ci collation is used for the
-    Routine, Event and Resource group names. With this collation, the
-    normalized object name uses just 2 bytes for each character (max
-    length = 64 * 2 bytes). MDL_key has still some space to store the
-    object names. If there is a sufficient space for the object name
-    in the MDL_key then it is stored in the MDL_key (similar to the
-    column names in the MDL_key). Actual object name is used by the
-    PFS.  Not listing actual object name from the PFS should be OK
-    when there is no space to store it (instead of increasing the
-    MDL_key size). Object name is not used in the key comparisons. So
-    only (mdl_namespace + strlen(db) + 1 + normalized_name_len + 1)
-    value is stored in the m_length member.
+    With the UTF8MB3 charset space reserved for the db name/object name is
+    64 * 3  bytes. utf8_general_ci collation is used for the Routine, Event and
+    Resource group names. With this collation, the normalized object name uses
+    just 2 bytes for each character (max length = 64 * 2 bytes). MDL_key has
+    still some space to store the object names. If there is a sufficient space
+    for the object name in the MDL_key then it is stored in the MDL_key (similar
+    to the column names in the MDL_key). Actual object name is used by the PFS.
+    Not listing actual object name from the PFS should be OK when there is no
+    space to store it (instead of increasing the MDL_key size). Object name is
+    not used in the key comparisons. So only (mdl_namespace + strlen(db) + 1 +
+    normalized_name_len + 1) value is stored in the m_length member.
 
     @param  mdl_namespace       Id of namespace of object to be locked.
     @param  db                  Name of database to which the object belongs.
@@ -644,10 +625,10 @@ struct MDL_key {
       FUNCTION, PROCEDURE, EVENT and RESOURCE_GROUPS names are case and accent
       insensitive. For other objects key should not be formed from this method.
     */
-    assert(use_normalized_object_name());
+    DBUG_ASSERT(use_normalized_object_name());
 
-    assert(strlen(db) <= NAME_LEN && strlen(name) <= NAME_LEN &&
-           normalized_name_len <= NAME_CHAR_LEN * 2);
+    DBUG_ASSERT(strlen(db) <= NAME_LEN && strlen(name) <= NAME_LEN &&
+                normalized_name_len <= NAME_CHAR_LEN * 2);
 
     // Database name.
     m_db_name_length =
@@ -672,7 +653,7 @@ struct MDL_key {
       *(m_ptr + m_length) = 0;
     }
 
-    assert(m_length + m_object_name_length < MAX_MDLKEY_LENGTH);
+    DBUG_ASSERT(m_length + m_object_name_length < MAX_MDLKEY_LENGTH);
   }
 
   /**
@@ -692,18 +673,18 @@ struct MDL_key {
       Key suffix provided should be in compatible format and
       its components should adhere to length restrictions.
     */
-    assert(strlen(part_key) == db_length);
-    assert(db_length + 1 + strlen(part_key + db_length + 1) + 1 ==
-           part_key_length);
-    assert(db_length <= NAME_LEN);
-    assert(part_key_length <= NAME_LEN + 1 + NAME_LEN + 1);
+    DBUG_ASSERT(strlen(part_key) == db_length);
+    DBUG_ASSERT(db_length + 1 + strlen(part_key + db_length + 1) + 1 ==
+                part_key_length);
+    DBUG_ASSERT(db_length <= NAME_LEN);
+    DBUG_ASSERT(part_key_length <= NAME_LEN + 1 + NAME_LEN + 1);
 
     m_ptr[0] = (char)mdl_namespace;
     /*
       Partial key of objects with normalized object name can not be used to
       initialize MDL key.
     */
-    assert(!use_normalized_object_name());
+    DBUG_ASSERT(!use_normalized_object_name());
 
     memcpy(m_ptr + 1, part_key, part_key_length);
     m_length = static_cast<uint16>(part_key_length + 1);
@@ -757,7 +738,7 @@ struct MDL_key {
           const char *name_arg) {
     mdl_key_init(namespace_arg, db_arg, name_arg);
   }
-  MDL_key() = default; /* To use when part of MDL_request. */
+  MDL_key() {} /* To use when part of MDL_request. */
 
   /**
     Get thread state name to be used in case when we have to
@@ -824,13 +805,13 @@ class MDL_request {
 
  public:
   static void *operator new(size_t size, MEM_ROOT *mem_root,
-                            const std::nothrow_t &arg
-                            [[maybe_unused]] = std::nothrow) noexcept {
+                            const std::nothrow_t &arg MY_ATTRIBUTE((unused)) =
+                                std::nothrow) noexcept {
     return mem_root->Alloc(size);
   }
 
   static void operator delete(void *, MEM_ROOT *,
-                              const std::nothrow_t &) noexcept {}
+                              const std::nothrow_t &)noexcept {}
 
   void init_with_source(MDL_key::enum_mdl_namespace namespace_arg,
                         const char *db_arg, const char *name_arg,
@@ -850,7 +831,7 @@ class MDL_request {
                                     const char *src_file, uint src_line);
   /** Set type of lock request. Can be only applied to pending locks. */
   inline void set_type(enum_mdl_type type_arg) {
-    assert(ticket == nullptr);
+    DBUG_ASSERT(ticket == NULL);
     type = type_arg;
   }
 
@@ -873,29 +854,28 @@ class MDL_request {
   /**
     This constructor exists for two reasons:
 
-    - Table_ref objects are sometimes default-constructed. We plan to
-      remove this as there is no practical reason, the call to the default
-      constructor is always followed by either a call to
-      Table_ref::operator= or memberwise assignments.
+    - TABLE_LIST objects are sometimes default-constructed. We plan to remove
+      this as there is no practical reason, the call to the default
+      constructor is always followed by either a call to TABLE_LIST::operator=
+      or memberwise assignments.
 
-    - In some legacy cases Table_ref objects are copy-assigned without
-      intention to copy the Table_ref::mdl_request member. In this cases
-      they are overwritten with an uninitialized MDL_request object. The cases
-      are:
+    - In some legacy cases TABLE_LIST objects are copy-assigned without
+      intention to copy the TABLE_LIST::mdl_request member. In this cases they
+      are overwritten with an uninitialized MDL_request object. The cases are:
 
       - Sql_cmd_handler_open::execute()
       - mysql_execute_command()
-      - Query_expression::prepare()
+      - SELECT_LEX_UNIT::prepare()
       - fill_defined_view_parts()
 
       No new cases are expected.  In all other cases, so far only
       Locked_tables_list::rename_locked_table(), a move assignment is actually
       what is intended.
   */
-  MDL_request() = default;
+  MDL_request() {}
 
   MDL_request(const MDL_request &rhs)
-      : type(rhs.type), duration(rhs.duration), ticket(nullptr), key(rhs.key) {}
+      : type(rhs.type), duration(rhs.duration), ticket(NULL), key(rhs.key) {}
 
   MDL_request(MDL_request &&) = default;
 
@@ -952,8 +932,7 @@ class MDL_wait_for_subgraph {
   */
   virtual bool accept_visitor(MDL_wait_for_graph_visitor *gvisitor) = 0;
 
-  static const uint DEADLOCK_WEIGHT_CO = 0;
-  static const uint DEADLOCK_WEIGHT_DML = 25;
+  static const uint DEADLOCK_WEIGHT_DML = 0;
   static const uint DEADLOCK_WEIGHT_ULL = 50;
   static const uint DEADLOCK_WEIGHT_DDL = 100;
 
@@ -1016,10 +995,10 @@ class MDL_ticket : public MDL_wait_for_subgraph {
   bool is_incompatible_when_waiting(enum_mdl_type type) const;
 
   /** Implement MDL_wait_for_subgraph interface. */
-  bool accept_visitor(MDL_wait_for_graph_visitor *dvisitor) override;
-  uint get_deadlock_weight() const override;
+  virtual bool accept_visitor(MDL_wait_for_graph_visitor *dvisitor);
+  virtual uint get_deadlock_weight() const;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   enum_mdl_duration get_duration() const { return m_duration; }
   void set_duration(enum_mdl_duration dur) { m_duration = dur; }
 #endif
@@ -1039,26 +1018,26 @@ class MDL_ticket : public MDL_wait_for_subgraph {
   friend class MDL_context;
 
   MDL_ticket(MDL_context *ctx_arg, enum_mdl_type type_arg
-#ifndef NDEBUG
+#ifndef DBUG_OFF
              ,
              enum_mdl_duration duration_arg
 #endif
              )
       : m_type(type_arg),
-#ifndef NDEBUG
+#ifndef DBUG_OFF
         m_duration(duration_arg),
 #endif
         m_ctx(ctx_arg),
-        m_lock(nullptr),
+        m_lock(NULL),
         m_is_fast_path(false),
         m_hton_notified(false),
-        m_psi(nullptr) {
+        m_psi(NULL) {
   }
 
-  ~MDL_ticket() override { assert(m_psi == nullptr); }
+  virtual ~MDL_ticket() { DBUG_ASSERT(m_psi == NULL); }
 
   static MDL_ticket *create(MDL_context *ctx_arg, enum_mdl_type type_arg
-#ifndef NDEBUG
+#ifndef DBUG_OFF
                             ,
                             enum_mdl_duration duration_arg
 #endif
@@ -1068,7 +1047,7 @@ class MDL_ticket : public MDL_wait_for_subgraph {
  private:
   /** Type of metadata lock. Externally accessible. */
   enum enum_mdl_type m_type;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   /**
     Duration of lock represented by this ticket.
     Context private. Debug-only.
@@ -1315,7 +1294,7 @@ class MDL_ticket_store {
 
 class MDL_savepoint {
  public:
-  MDL_savepoint() = default;
+  MDL_savepoint() {}
 
  private:
   MDL_savepoint(MDL_ticket *stmt_ticket, MDL_ticket *trans_ticket)
@@ -1378,7 +1357,7 @@ class MDL_wait {
 
 class MDL_release_locks_visitor {
  public:
-  virtual ~MDL_release_locks_visitor() = default;
+  virtual ~MDL_release_locks_visitor() {}
   /**
     Check if the given ticket represents a lock that should be released.
 
@@ -1393,7 +1372,7 @@ class MDL_release_locks_visitor {
 
 class MDL_context_visitor {
  public:
-  virtual ~MDL_context_visitor() = default;
+  virtual ~MDL_context_visitor() {}
   virtual void visit_context(const MDL_context *ctx) = 0;
 };
 
@@ -1466,7 +1445,7 @@ class MDL_context {
 
   bool has_locks_waited_for() const;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   bool has_locks(enum_mdl_duration duration) {
     return !m_ticket_store.is_empty(duration);
   }
@@ -1667,9 +1646,7 @@ class MDL_context {
   void release_lock(enum_mdl_duration duration, MDL_ticket *ticket);
   bool try_acquire_lock_impl(MDL_request *mdl_request, MDL_ticket **out_ticket);
   void materialize_fast_path_locks();
-
-  friend bool mdl_unittest::test_drive_fix_pins(MDL_context *);
-  bool fix_pins();
+  inline bool fix_pins();
 
  public:
   void find_deadlock();
@@ -1698,7 +1675,7 @@ class MDL_context {
   /** Remove the wait-for edge from the graph after we're done waiting. */
   void done_waiting_for() {
     mysql_prlock_wrlock(&m_LOCK_waiting_for);
-    m_waiting_for = nullptr;
+    m_waiting_for = NULL;
     mysql_prlock_unlock(&m_LOCK_waiting_for);
   }
   void lock_deadlock_victim() { mysql_prlock_rdlock(&m_LOCK_waiting_for); }
@@ -1712,7 +1689,7 @@ class MDL_context {
 void mdl_init();
 void mdl_destroy();
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 extern mysql_mutex_t LOCK_open;
 #endif
 
@@ -1743,4 +1720,37 @@ const double MDL_LOCKS_UNUSED_LOCKS_MIN_RATIO = 0.25;
 
 int32 mdl_get_unused_locks_count();
 
+/**
+  Inspect if MDL_context is owned by any thread.
+*/
+class MDL_lock_is_owned_visitor : public MDL_context_visitor {
+ public:
+  MDL_lock_is_owned_visitor() : m_exists(false) {}
+
+  /**
+    Collects relevant information about the MDL lock owner.
+
+    This function is only called by MDL_context::find_lock_owner() when
+    searching for MDL lock owners to collect extra information about the
+    owner. As we only need to know that the MDL lock is owned, setting
+    m_exists to true is enough.
+  */
+
+  void visit_context(const MDL_context *ctx MY_ATTRIBUTE((unused))) override {
+    m_exists = true;
+  }
+
+  /**
+    Returns if an owner for the MDL lock being inspected exists.
+
+    @return true when MDL lock is owned, false otherwise.
+  */
+
+  bool exists() const { return m_exists; }
+
+ private:
+  /* holds information about MDL being owned by any thread */
+  bool m_exists;
+};
+
 #endif
