@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,7 +36,7 @@ class THD;
 /**
   Merges buffers to make < MERGEBUFF2 buffers.
 
-  @param thd          thread context
+  @param thd
   @param param        Sort parameters.
   @param sort_buffer  The main memory buffer.
   @param chunk_array  Array of chunk descriptors to merge.
@@ -68,8 +67,8 @@ bool merge_many_buff(THD *thd, Merge_param *param, Sort_buffer sort_buffer,
   IO_CACHE *to_file = &t_file2;
 
   while (num_chunks > MERGEBUFF2) {
-    if (reinit_io_cache(from_file, READ_CACHE, 0L, false, false)) goto cleanup;
-    if (reinit_io_cache(to_file, WRITE_CACHE, 0L, false, false)) goto cleanup;
+    if (reinit_io_cache(from_file, READ_CACHE, 0L, 0, 0)) goto cleanup;
+    if (reinit_io_cache(to_file, WRITE_CACHE, 0L, 0, 0)) goto cleanup;
     Merge_chunk *last_chunk = chunk_array.begin();
     uint i;
     for (i = 0; i < num_chunks - MERGEBUFF * 3U / 2U; i += MERGEBUFF) {
