@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -45,9 +44,10 @@
   std sorting functions instead of this.
 */
 
-#include <assert.h>
 #include <queue>
 
+#include "my_dbug.h"
+
 /**
  Sorts the elements in the range [first,last) into ascending order
  using insertion sort.
@@ -121,7 +121,7 @@ void merge_sort(Element_type **first, Element_type **last, Comp_func comp) {
   Element_type **cur2 = middle;
 
   for (uint i = 0; i < elements; i++) {
-    assert(cur1 < middle || cur2 < last);
+    DBUG_ASSERT(cur1 < middle || cur2 < last);
 
     if (cur1 == middle)
       merged.push(*cur2++);
