@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,94 +29,11 @@
 #include "violite.h"
 
 using std::string;
-using keyring_load_t = SERVICE_TYPE_NO_CONST(keyring_load);
-using keyring_writer_t = SERVICE_TYPE_NO_CONST(keyring_writer);
-
-Destination_keyring_component::Destination_keyring_component(
-    const std::string component_path, const std::string implementation_name)
-    : component_path_(component_path),
-      keyring_load_service_(nullptr),
-      keyring_writer_service_(nullptr),
-      component_loaded_(false),
-      ok_(false) {
-  {
-    my_service<SERVICE_TYPE(registry_registration)> registrator(
-        "registry_registration", srv_registry);
-    const char *urn[] = {component_path_.c_str()};
-    bool load_status = dynamic_loader_srv->load(urn, 1);
-    if (load_status == true) return;
-    component_loaded_ = true;
-  }
-
-  std::string load_service_name("keyring_load");
-  load_service_name += "." + implementation_name;
-
-  my_h_service h_keyring_load_service = nullptr;
-  my_h_service h_keyring_writer_service = nullptr;
-
-  auto local_cleanup = [&]() {
-    if (h_keyring_load_service != nullptr)
-      srv_registry->release(h_keyring_load_service);
-    h_keyring_load_service = nullptr;
-
-    if (h_keyring_writer_service != nullptr)
-      srv_registry->release(h_keyring_writer_service);
-    h_keyring_writer_service = nullptr;
-  };
-
-  if (srv_registry->acquire(load_service_name.c_str(),
-                            &h_keyring_load_service) ||
-      srv_registry->acquire_related("keyring_writer", h_keyring_load_service,
-                                    &h_keyring_writer_service)) {
-    local_cleanup();
-    return;
-  }
-
-  keyring_load_service_ =
-      reinterpret_cast<const_keyring_load_t *>(h_keyring_load_service);
-  keyring_writer_service_ =
-      reinterpret_cast<const_keyring_writer_t *>(h_keyring_writer_service);
-
-  /*
-    In case of migration from keyring plugin to keyring
-    component, we only support reading configuration from
-    plugin directory
-  */
-  if (keyring_load_service_->load(opt_plugin_dir, nullptr) != 0) {
-    return;
-  }
-
-  ok_ = true;
-}
-
-Destination_keyring_component::~Destination_keyring_component() {
-  if (keyring_load_service_)
-    srv_registry->release(reinterpret_cast<my_h_service>(
-        const_cast<keyring_load_t *>(keyring_load_service_)));
-  keyring_load_service_ = nullptr;
-
-  if (keyring_writer_service_ != nullptr)
-    srv_registry->release(reinterpret_cast<my_h_service>(
-        const_cast<keyring_writer_t *>(keyring_writer_service_)));
-  keyring_writer_service_ = nullptr;
-
-  if (component_loaded_ == true) {
-    my_service<SERVICE_TYPE(registry_registration)> registrator(
-        "registry_registration", srv_registry);
-    const char *urn[] = {component_path_.c_str()};
-    dynamic_loader_srv->unload(urn, 1);
-  }
-  component_loaded_ = false;
-  ok_ = false;
-}
 
 Migrate_keyring::Migrate_keyring() {
   m_source_plugin_handle = nullptr;
   m_destination_plugin_handle = nullptr;
-  mysql = nullptr;
-  m_destination_component = nullptr;
-  m_argc = 0;
-  m_argv = nullptr;
+  mysql = NULL;
 }
 
 /**
@@ -126,23 +42,15 @@ Migrate_keyring::Migrate_keyring() {
     2. Get plugin_dir value.
     3. Get a connection handle by connecting to server.
 
-   @param [in] argc                            Pointer to argc of original
-  program
-   @param [in] argv                            Pointer to argv of original
-  program
-   @param [in] source_plugin                   Pointer to source plugin option
-   @param [in] destination_plugin              Pointer to destination plugin
-  option
-   @param [in] user                            User to login to server
-   @param [in] host                            Host on which to connect to
-  server
-   @param [in] password                        Password used to connect to
-  server
-   @param [in] socket                          The socket file to use for
-  connection
-   @param [in] port                            Port number to use for connection
-   @param [in] migrate_to_component            Migrate from plugin to component
-  destination component
+   @param [in] argc               Pointer to argc of original program
+   @param [in] argv               Pointer to argv of original program
+   @param [in] source_plugin      Pointer to source plugin option
+   @param [in] destination_plugin Pointer to destination plugin option
+   @param [in] user               User to login to server
+   @param [in] host               Host on which to connect to server
+   @param [in] password           Password used to connect to server
+   @param [in] socket             The socket file to use for connection
+   @param [in] port               Port number to use for connection
 
    @return 0 Success
    @return 1 Failure
@@ -150,8 +58,7 @@ Migrate_keyring::Migrate_keyring() {
 */
 bool Migrate_keyring::init(int argc, char **argv, char *source_plugin,
                            char *destination_plugin, char *user, char *host,
-                           char *password, char *socket, ulong port,
-                           bool migrate_to_component) {
+                           char *password, char *socket, ulong port) {
   DBUG_TRACE;
 
   std::size_t found = std::string::npos;
@@ -199,26 +106,15 @@ bool Migrate_keyring::init(int argc, char **argv, char *source_plugin,
     return true;
   }
 
-  m_migrate_to_component = migrate_to_component;
-  if (m_migrate_to_component) {
-    /* If we are migrating to component, construct complete URI */
-    std::string uri("file://");
-    if (check_valid_path(m_destination_plugin_option.c_str(),
-                         m_destination_plugin_option.length())) {
-      LogErr(ERROR_LEVEL, ER_KEYRING_MIGRATE_FAILED,
-             "No paths allowed for shared library");
-      return true;
-    }
-    uri.append(m_destination_plugin_name);
-    m_destination_plugin_option = uri;
-  }
-
   /* if connect options are provided then initiate connection */
   if (migrate_connect_options) {
     ssl_start();
     /* initiate connection */
-    mysql = mysql_init(nullptr);
-    net_server_ext_init(&server_extn);
+    mysql = mysql_init(NULL);
+    server_extn.m_user_data = nullptr;
+    server_extn.m_before_header = nullptr;
+    server_extn.m_after_header = nullptr;
+    server_extn.compress_ctx.algorithm = MYSQL_UNCOMPRESSED;
 
     mysql_extension_set_server_extn(mysql, &server_extn);
     /* set default compression method */
@@ -226,7 +122,7 @@ bool Migrate_keyring::init(int argc, char **argv, char *source_plugin,
                   compression_method.c_str());
     enum mysql_ssl_mode ssl_mode = SSL_MODE_REQUIRED;
     mysql_options(mysql, MYSQL_OPT_SSL_MODE, &ssl_mode);
-    mysql_options(mysql, MYSQL_OPT_CONNECT_ATTR_RESET, nullptr);
+    mysql_options(mysql, MYSQL_OPT_CONNECT_ATTR_RESET, 0);
     mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "program_name", "mysqld");
     mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "_client_role",
                    "keyring_migration_tool");
@@ -275,7 +171,7 @@ bool Migrate_keyring::init(int argc, char **argv, char *source_plugin,
     2. Load source plugin.
     3. Load destination plugin.
     4. Fetch all keys from source plugin and upon
-       success store in destination plugin.
+       sucess store in destination plugin.
     5. Enable access to keyring service APIs.
     6. Unload source plugin.
     7. Unload destination plugin.
@@ -301,34 +197,25 @@ bool Migrate_keyring::execute() {
     goto error;
   }
 
-  if (!m_migrate_to_component) {
-    /* Load destination plugin. */
-    if (load_plugin(enum_plugin_type::DESTINATION_PLUGIN)) {
-      LogErr(ERROR_LEVEL, ER_KEYRING_MIGRATE_FAILED,
-             "Failed to initialize destination keyring");
-      goto error;
-    }
-  } else {
-    if (load_component()) {
-      LogErr(ERROR_LEVEL, ER_KEYRING_MIGRATE_FAILED,
-             "Failed to initialize destination keyring");
-      goto error;
-    }
+  /* Load destination plugin. */
+  if (load_plugin(enum_plugin_type::DESTINATION_PLUGIN)) {
+    LogErr(ERROR_LEVEL, ER_KEYRING_MIGRATE_FAILED,
+           "Failed to initialize destination keyring");
+    goto error;
   }
 
   /* skip program name */
   m_argc--;
   /* We use a tmp ptr instead of m_argv since if the latter gets changed, we
-   * lose access to the allocated mem and hence there would be leak */
+   * lose access to the alloced mem and hence there would be leak */
   tmp_m_argv = m_argv + 1;
   /* check for invalid options */
   if (m_argc > 1) {
-    struct my_option no_opts[] = {{nullptr, 0, nullptr, nullptr, nullptr,
-                                   nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0,
-                                   nullptr, 0, nullptr}};
-    my_getopt_skip_unknown = false;
+    struct my_option no_opts[] = {
+        {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
+    my_getopt_skip_unknown = 0;
     my_getopt_use_args_separator = true;
-    if (handle_options(&m_argc, &tmp_m_argv, no_opts, nullptr)) return true;
+    if (handle_options(&m_argc, &tmp_m_argv, no_opts, NULL)) return true;
 
     if (m_argc > 1) {
       LogErr(WARNING_LEVEL, ER_KEYRING_MIGRATION_EXTRA_OPTIONS);
@@ -351,24 +238,6 @@ error:
   return true;
 }
 
-/**
-  Load component.
-
-  @return false Success
-  @return true Failure
-*/
-bool Migrate_keyring::load_component() {
-  m_destination_component = new (std::nothrow) Destination_keyring_component(
-      m_destination_plugin_option, m_destination_plugin_name);
-
-  if (m_destination_component && !m_destination_component->ok()) {
-    delete m_destination_component;
-    m_destination_component = nullptr;
-    return true;
-  }
-  return false;
-}
-
 /**
   Load plugin.
 
@@ -380,11 +249,11 @@ bool Migrate_keyring::load_component() {
 bool Migrate_keyring::load_plugin(enum_plugin_type plugin_type) {
   DBUG_TRACE;
 
-  char *keyring_plugin = nullptr;
-  char *plugin_name = nullptr;
-  bool is_source_plugin = false;
+  char *keyring_plugin = NULL;
+  char *plugin_name = NULL;
+  bool is_source_plugin = 0;
 
-  if (plugin_type == enum_plugin_type::SOURCE_PLUGIN) is_source_plugin = true;
+  if (plugin_type == enum_plugin_type::SOURCE_PLUGIN) is_source_plugin = 1;
 
   if (is_source_plugin) {
     keyring_plugin = const_cast<char *>(m_source_plugin_option.c_str());
@@ -399,7 +268,7 @@ bool Migrate_keyring::load_plugin(enum_plugin_type plugin_type) {
   else {
     /* set plugin handle */
     plugin_ref plugin;
-    plugin = my_plugin_lock_by_name(nullptr, to_lex_cstring(plugin_name),
+    plugin = my_plugin_lock_by_name(0, to_lex_cstring(plugin_name),
                                     MYSQL_KEYRING_PLUGIN);
     if (plugin == nullptr) goto error;
 
@@ -409,7 +278,7 @@ bool Migrate_keyring::load_plugin(enum_plugin_type plugin_type) {
       m_destination_plugin_handle =
           (st_mysql_keyring *)plugin_decl(plugin)->info;
 
-    plugin_unlock(nullptr, plugin);
+    plugin_unlock(0, plugin);
   }
   return false;
 
@@ -441,22 +310,17 @@ bool Migrate_keyring::fetch_and_store_keys() {
   bool error = false;
   char key_id[MAX_KEY_LEN] = {0};
   char user_id[USERNAME_LENGTH] = {0};
-  void *key = nullptr;
+  void *key = NULL;
   size_t key_len = 0;
-  char *key_type = nullptr;
-  void *key_iterator = nullptr;
-  const_keyring_writer_t *keyring_writer_service = nullptr;
+  char *key_type = NULL;
+  void *key_iterator = NULL;
 
   m_source_plugin_handle->mysql_key_iterator_init(&key_iterator);
-  if (key_iterator == nullptr) {
+  if (key_iterator == NULL) {
     LogErr(ERROR_LEVEL, ER_KEYRING_MIGRATE_FAILED,
            "Initializing source keyring iterator failed.");
     return true;
   }
-
-  if (m_migrate_to_component) {
-    keyring_writer_service = m_destination_component->writer();
-  }
   while (!error) {
     if (m_source_plugin_handle->mysql_key_iterator_get_key(key_iterator, key_id,
                                                            user_id))
@@ -470,18 +334,14 @@ bool Migrate_keyring::fetch_and_store_keys() {
           "Fetching key (" + string(key_id) + ") from source plugin failed.";
       LogErr(ERROR_LEVEL, ER_KEYRING_MIGRATE_FAILED, errmsg.c_str());
       error = true;
-    } else /* store the fetched key into destination plugin */ {
-      error =
-          m_migrate_to_component
-              ? keyring_writer_service->store(
-                    key_id, user_id, static_cast<const unsigned char *>(key),
-                    key_len, key_type)
-              : m_destination_plugin_handle->mysql_key_store(
-                    key_id, key_type, user_id, key, key_len);
-      if (error) {
+    } else /* store the fetched key into destination plugin */
+    {
+      if (m_destination_plugin_handle->mysql_key_store(key_id, key_type,
+                                                       user_id, key, key_len)) {
         string errmsg = "Storing key (" + string(key_id) +
                         ") into destination plugin failed.";
         LogErr(ERROR_LEVEL, ER_KEYRING_MIGRATE_FAILED, errmsg.c_str());
+        error = true;
       } else {
         /*
          keep track of keys stored in successfully so that they can be
@@ -494,12 +354,6 @@ bool Migrate_keyring::fetch_and_store_keys() {
     if (key) my_free((char *)key);
     if (key_type) my_free(key_type);
   }
-
-  /* If there are zero keys in the keyring, it means no keys were migrated */
-  if (!error && m_source_keys.size() == 0) {
-    LogErr(WARNING_LEVEL, ER_WARN_MIGRATION_EMPTY_SOURCE_KEYRING);
-  }
-
   if (error) {
     /* something went wrong remove keys from destination plugin. */
     while (m_source_keys.size()) {
@@ -543,10 +397,6 @@ bool Migrate_keyring::disable_keyring_operations() {
 bool Migrate_keyring::enable_keyring_operations() {
   DBUG_TRACE;
   const char query[] = "SET GLOBAL KEYRING_OPERATIONS=1";
-
-  /* clear the SSL error stack first as the connection could be encrypted */
-  ERR_clear_error();
-
   if (mysql && mysql_real_query(mysql, query, strlen(query))) {
     LogErr(ERROR_LEVEL, ER_KEYRING_MIGRATE_FAILED,
            "Failed to enable keyring_operations variable.");
@@ -559,13 +409,11 @@ bool Migrate_keyring::enable_keyring_operations() {
   Standard destructor to close connection handle.
 */
 Migrate_keyring::~Migrate_keyring() {
-  delete[] m_argv;
-  m_argv = nullptr;
   if (mysql) {
+    delete[] m_argv;
+    m_argv = NULL;
     mysql_close(mysql);
-    mysql = nullptr;
+    mysql = NULL;
     if (migrate_connect_options) vio_end();
   }
-  if (m_destination_component != nullptr) delete m_destination_component;
-  m_destination_component = nullptr;
 }
