@@ -1,16 +1,15 @@
-/* Copyright (c) 2005, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,17 +35,15 @@
   Most function are just inline wrappers around library calls
 */
 
-#include <assert.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <algorithm>
 
 #include "decimal.h"
 #include "m_ctype.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
-#include "my_time_t.h"
 
 class String;
 struct MYSQL_TIME;
@@ -98,13 +95,13 @@ class my_decimal : public decimal_t {
   To catch them, we allocate dummy fields around the buffer,
   and test that their values do not change.
  */
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   int foo1;
 #endif
 
   decimal_digit_t buffer[DECIMAL_BUFF_LENGTH];
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   int foo2;
   static const int test_value = 123;
 #endif
@@ -112,7 +109,7 @@ class my_decimal : public decimal_t {
  public:
   my_decimal(const my_decimal &rhs) : decimal_t(rhs) {
     rhs.sanity_check();
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
     foo1 = test_value;
     foo2 = test_value;
 #endif
@@ -131,7 +128,7 @@ class my_decimal : public decimal_t {
   }
 
   void init() {
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
     foo1 = test_value;
     foo2 = test_value;
 #endif
@@ -145,14 +142,14 @@ class my_decimal : public decimal_t {
 
   my_decimal() { init(); }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   ~my_decimal() { sanity_check(); }
-#endif  // NDEBUG
+#endif  // DBUG_OFF
 
   void sanity_check() const {
-    assert(foo1 == test_value);
-    assert(foo2 == test_value);
-    assert(buf == buffer);
+    DBUG_ASSERT(foo1 == test_value);
+    DBUG_ASSERT(foo2 == test_value);
+    DBUG_ASSERT(buf == buffer);
   }
 
   bool sign() const { return decimal_t::sign; }
@@ -170,7 +167,7 @@ class my_decimal : public decimal_t {
 #endif
 };
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 void print_decimal(const my_decimal *dec);
 void print_decimal_buff(const my_decimal *dec, const uchar *ptr, int length);
 const char *dbug_decimal_as_string(char *buff, const my_decimal *val);
@@ -178,13 +175,15 @@ const char *dbug_decimal_as_string(char *buff, const my_decimal *val);
 #define dbug_decimal_as_string(A) NULL
 #endif
 
-bool str_set_decimal(uint mask, const my_decimal *val, String *str,
-                     const CHARSET_INFO *cs, uint decimals);
+bool str_set_decimal(uint mask, const my_decimal *val, uint fixed_prec,
+                     uint fixed_dec, char filler, String *str,
+                     const CHARSET_INFO *cs);
 
 extern my_decimal decimal_zero;
 
 inline void max_my_decimal(my_decimal *to, int precision, int frac) {
-  assert((precision <= DECIMAL_MAX_PRECISION) && (frac <= DECIMAL_MAX_SCALE));
+  DBUG_ASSERT((precision <= DECIMAL_MAX_PRECISION) &&
+              (frac <= DECIMAL_MAX_SCALE));
   max_decimal(precision, frac, to);
 }
 
@@ -211,7 +210,7 @@ inline int check_result_and_overflow(uint mask, int result, my_decimal *val) {
 inline uint my_decimal_length_to_precision(uint length, uint scale,
                                            bool unsigned_flag) {
   /* Precision can't be negative thus ignore unsigned_flag when length is 0. */
-  assert(length || !scale);
+  DBUG_ASSERT(length || !scale);
   uint retval =
       (uint)(length - (scale > 0 ? 1 : 0) - (unsigned_flag || !length ? 0 : 1));
   return retval;
@@ -224,10 +223,9 @@ inline uint32 my_decimal_precision_to_length_no_truncation(uint precision,
     When precision is 0 it means that original length was also 0. Thus
     unsigned_flag is ignored in this case.
   */
-  assert(precision || !scale);
+  DBUG_ASSERT(precision || !scale);
   uint32 retval = (uint32)(precision + (scale > 0 ? 1 : 0) +
                            (unsigned_flag || !precision ? 0 : 1));
-  if (retval == 0) return 1;
   return retval;
 }
 
@@ -237,8 +235,8 @@ inline uint32 my_decimal_precision_to_length(uint precision, uint8 scale,
     When precision is 0 it means that original length was also 0. Thus
     unsigned_flag is ignored in this case.
   */
-  assert(precision || !scale);
-  precision = std::min(precision, uint(DECIMAL_MAX_PRECISION));
+  DBUG_ASSERT(precision || !scale);
+  set_if_smaller(precision, DECIMAL_MAX_PRECISION);
   return my_decimal_precision_to_length_no_truncation(precision, scale,
                                                       unsigned_flag);
 }
@@ -287,7 +285,7 @@ inline int my_decimal_set_zero(my_decimal *d) {
   /*
     We need the up-cast here, since my_decimal has sign() member functions,
     which conflicts with decimal_t::size
-    (and decimal_make_zero is a macro, rather than a function).
+    (and decimal_make_zero is a macro, rather than a funcion).
   */
   decimal_make_zero(static_cast<decimal_t *>(d));
   return 0;
@@ -312,13 +310,14 @@ inline int my_decimal_ceiling(uint mask, const my_decimal *from,
   return from->check_result(mask, decimal_round(from, to, 0, CEILING));
 }
 
-int my_decimal2string(uint mask, const my_decimal *d, uint fixed_prec,
-                      uint fixed_dec, String *str);
-
-inline int my_decimal2string(uint mask, const my_decimal *d, String *str) {
-  return my_decimal2string(mask, d, 0, 0, str);
+inline bool str_set_decimal(const my_decimal *val, String *str,
+                            const CHARSET_INFO *cs) {
+  return str_set_decimal(E_DEC_FATAL_ERROR, val, 0, 0, 0, str, cs);
 }
 
+int my_decimal2string(uint mask, const my_decimal *d, uint fixed_prec,
+                      uint fixed_dec, char filler, String *str);
+
 inline int my_decimal2int(uint mask, const my_decimal *d, bool unsigned_flag,
                           longlong *l) {
   my_decimal rounded;
@@ -348,7 +347,7 @@ int str2my_decimal(uint mask, const char *from, size_t length,
 
 my_decimal *date2my_decimal(const MYSQL_TIME *ltime, my_decimal *dec);
 my_decimal *time2my_decimal(const MYSQL_TIME *ltime, my_decimal *dec);
-my_decimal *timeval2my_decimal(const my_timeval *tm, my_decimal *dec);
+my_decimal *timeval2my_decimal(const struct timeval *tm, my_decimal *dec);
 
 inline int double2my_decimal(uint mask, double val, my_decimal *d) {
   return check_result_and_overflow(mask, double2decimal(val, d), d);
@@ -364,7 +363,7 @@ inline int int2my_decimal(uint mask, longlong i, bool unsigned_flag,
 inline void my_decimal_neg(decimal_t *arg) {
   // Avoid returning negative zero, cfr. decimal_cmp()
   if (decimal_is_zero(arg)) {
-    arg->sign = false;
+    arg->sign = 0;
     return;
   }
   arg->sign ^= 1;
@@ -397,9 +396,10 @@ inline int my_decimal_mod(uint mask, my_decimal *res, const my_decimal *a,
 }
 
 /**
-  @retval -1 if a @< b
-  @retval 1 if a @> b
-  @retval 0 if a == b
+  @return
+    @retval -1 if a @< b
+    @retval 1 if a @> b
+    @retval 0 if a == b
 */
 inline int my_decimal_cmp(const my_decimal *a, const my_decimal *b) {
   return decimal_cmp(a, b);
