@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,20 +20,18 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include "sql/named_pipe.h"
-
 #include <AclAPI.h>
 #include <accctrl.h>
 #include <errno.h>
+#include "sql_class.h"
 
 #include <mysql/components/services/log_builtins.h>
 #include "my_config.h"
 #include "my_sys.h"
 #include "mysqld_error.h"
-#include "sql/current_thd.h"
 #include "sql/log.h"
 #include "sql/mysqld.h"
-#include "sql/sql_error.h"
+#include "sql/named_pipe.h"
 
 bool is_existing_windows_group_name(const char *group_name) {
   // First, let's get a SID for the given group name...
@@ -67,7 +64,7 @@ static bool check_windows_group_for_everyone(const char *group_name,
     return false;
   }
 
-  if (strcmp(group_name, NAMED_PIPE_FULL_ACCESS_GROUP_EVERYONE) == 0) {
+  if (strcmp(group_name, DEFAULT_NAMED_PIPE_FULL_ACCESS_GROUP) == 0) {
     *is_everyone_group = true;
     return false;
   } else {
@@ -150,9 +147,9 @@ bool my_security_attr_add_rights_to_group(SECURITY_ATTRIBUTES *psa,
     }
   }
 
-  // Treat the NAMED_PIPE_FULL_ACCESS_GROUP_EVERYONE value
+  // Treat the DEFAULT_NAMED_PIPE_FULL_ACCESS_GROUP value
   // as a special case: we  convert it to the "world" SID
-  if (strcmp(group_name, NAMED_PIPE_FULL_ACCESS_GROUP_EVERYONE) == 0) {
+  if (strcmp(group_name, DEFAULT_NAMED_PIPE_FULL_ACCESS_GROUP) == 0) {
     if (!CreateWellKnownSid(WinWorldSid, NULL, soughtSID, &size_sid)) {
       DWORD last_error_num = GetLastError();
       FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
@@ -235,7 +232,7 @@ bool my_security_attr_add_rights_to_group(SECURITY_ATTRIBUTES *psa,
   DWORD dwRes = SetEntriesInAcl(1, &ea, pOldDACL, &pNewDACL);
   if (ERROR_SUCCESS != dwRes) {
     char num_buff[20];
-    longlong10_to_str(dwRes, num_buff, 10);
+    int10_to_str(dwRes, num_buff, 10);
     log_message(LOG_TYPE_ERROR, LOG_ITEM_LOG_PRIO, (longlong)ERROR_LEVEL,
                 LOG_ITEM_LOG_LOOKUP, ER_NPIPE_CANT_CREATE,
                 "SetEntriesInAcl to add group permissions failed", num_buff);
@@ -332,7 +329,7 @@ HANDLE create_server_named_pipe(SECURITY_ATTRIBUTES **ppsec_attr,
                     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), last_error_msg,
                     sizeof(last_error_msg) / sizeof(TCHAR), NULL);
       char num_buff[20];
-      longlong10_to_str(last_error_num, num_buff, 10);
+      int10_to_str(last_error_num, num_buff, 10);
 
       log_message(LOG_TYPE_ERROR, LOG_ITEM_LOG_PRIO, (longlong)ERROR_LEVEL,
                   LOG_ITEM_LOG_LOOKUP, ER_NPIPE_CANT_CREATE, last_error_msg,
