@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -46,7 +46,7 @@
 #include "sql/key.h"
 #include "sql/key_spec.h"
 #include "sql/malloc_allocator.h"  // IWYU pragma: keep
-#include "sql/records.h"           // READ_RECORD
+#include "sql/records.h"           // unique_ptr_destroy_only<RowIterator>
 #include "sql/sql_bitmap.h"
 #include "sql/sql_const.h"
 #include "sql/sql_list.h"
@@ -677,7 +677,7 @@ class QUICK_INDEX_MERGE_SELECT : public QUICK_SELECT_I {
   }
 
   /* used to get rows collected in Unique */
-  READ_RECORD read_record;
+  unique_ptr_destroy_only<RowIterator> read_record;
 };
 
 /*
