@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,16 +27,16 @@
 
 #include "sql/opt_trace.h"
 
+#include <float.h>
 #include <stdio.h>
 #include <new>
 
 #include "lex_string.h"
-#include "m_ctype.h"
 #include "m_string.h"  // _dig_vec_lower
 #include "my_dbug.h"
 #include "my_pointer_arithmetic.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/psi/psi_base.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"
 #include "sql/current_thd.h"
@@ -47,7 +46,8 @@
 #include "sql/table.h"
 #include "sql_string.h"  // String
 
-namespace {
+// gcc.gnu.org/bugzilla/show_bug.cgi?id=29365
+namespace random_name_to_avoid_gcc_bug_29365 {
 /**
   A wrapper of class String, for storing query or trace.
   Any memory allocation error in this class is reported by my_error(), see
@@ -95,7 +95,9 @@ class Buffer {
   void set_allowed_mem_size(size_t a) { allowed_mem_size = a; }
 };
 
-}  // namespace
+}  // namespace random_name_to_avoid_gcc_bug_29365
+
+using random_name_to_avoid_gcc_bug_29365::Buffer;
 
 /**
   @class Opt_trace_stmt
@@ -180,13 +182,17 @@ class Opt_trace_stmt {
   /// Fills user-level information @sa Opt_trace_iterator
   void fill_info(Opt_trace_info *info) const;
 
+  /// @returns 'size' last bytes of the trace buffer
+  const char *trace_buffer_tail(size_t size);
+
   /// @returns total memory used by this trace
   size_t alloced_length() const {
     return trace_buffer.alloced_length() + query_buffer.alloced_length();
   }
 
-  void assert_current_struct(const Opt_trace_struct *s [[maybe_unused]]) const {
-    assert(current_struct == s);
+  void assert_current_struct(
+      const Opt_trace_struct *s MY_ATTRIBUTE((unused))) const {
+    DBUG_ASSERT(current_struct == s);
   }
 
   /// @see Opt_trace_context::missing_privilege()
@@ -269,7 +275,7 @@ void Opt_trace_struct::do_construct(Opt_trace_context *ctx,
 
   DBUG_PRINT("opt", ("%s: starting struct", key));
   stmt = ctx->get_current_stmt_in_gen();
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   previous_key[0] = 0;
 #endif
   has_disabled_I_S = !ctx->feature_enabled(feature);
@@ -281,7 +287,7 @@ void Opt_trace_struct::do_construct(Opt_trace_context *ctx,
 
 void Opt_trace_struct::do_destruct() {
   DBUG_PRINT("opt", ("%s: ending struct", saved_key));
-  assert(started);
+  DBUG_ASSERT(started);
   stmt->close_struct(saved_key, has_disabled_I_S,
                      closing_bracket(requires_key));
   started = false;
@@ -289,7 +295,7 @@ void Opt_trace_struct::do_destruct() {
 
 Opt_trace_struct &Opt_trace_struct::do_add(const char *key, const char *val,
                                            size_t val_length, bool escape) {
-  assert(started);
+  DBUG_ASSERT(started);
   DBUG_PRINT("opt", ("%s: \"%.*s\"", key, (int)val_length, val));
   stmt->add(key, val, val_length, true, escape);
   return *this;
@@ -302,7 +308,7 @@ LEX_CSTRING bool_as_text[] = {{STRING_WITH_LEN("false")},
 }  // namespace
 
 Opt_trace_struct &Opt_trace_struct::do_add(const char *key, bool val) {
-  assert(started);
+  DBUG_ASSERT(started);
   DBUG_PRINT("opt", ("%s: %d", key, (int)val));
   const LEX_CSTRING *text = &bool_as_text[val];
   stmt->add(key, text->str, text->length, false, false);
@@ -310,7 +316,7 @@ Opt_trace_struct &Opt_trace_struct::do_add(const char *key, bool val) {
 }
 
 Opt_trace_struct &Opt_trace_struct::do_add(const char *key, longlong val) {
-  assert(started);
+  DBUG_ASSERT(started);
   char buf[22];  // 22 is enough for digits of a 64-bit int
   llstr(val, buf);
   DBUG_PRINT("opt", ("%s: %s", key, buf));
@@ -319,7 +325,7 @@ Opt_trace_struct &Opt_trace_struct::do_add(const char *key, longlong val) {
 }
 
 Opt_trace_struct &Opt_trace_struct::do_add(const char *key, ulonglong val) {
-  assert(started);
+  DBUG_ASSERT(started);
   char buf[22];
   ullstr(val, buf);
   DBUG_PRINT("opt", ("%s: %s", key, buf));
@@ -328,18 +334,16 @@ Opt_trace_struct &Opt_trace_struct::do_add(const char *key, ulonglong val) {
 }
 
 Opt_trace_struct &Opt_trace_struct::do_add(const char *key, double val) {
-  assert(started);
-  // The buffer must be big enough to hold six digits of precision, sign,
-  // decimal point and exponent.
-  char buf[sizeof("-1.23456e+308")];
-  const int len = std::snprintf(buf, sizeof(buf), "%g", val);
+  DBUG_ASSERT(started);
+  char buf[32];  // 32 is enough for digits of a double
+  my_gcvt(val, MY_GCVT_ARG_DOUBLE, FLT_DIG, buf, nullptr);
   DBUG_PRINT("opt", ("%s: %s", key, buf));
-  stmt->add(key, buf, len, false, false);
+  stmt->add(key, buf, strlen(buf), false, false);
   return *this;
 }
 
 Opt_trace_struct &Opt_trace_struct::do_add_null(const char *key) {
-  assert(started);
+  DBUG_ASSERT(started);
   DBUG_PRINT("opt", ("%s: null", key));
   stmt->add(key, STRING_WITH_LEN("null"), false, false);
   return *this;
@@ -349,7 +353,7 @@ Opt_trace_struct &Opt_trace_struct::do_add(const char *key, Item *item) {
   char buff[256];
   String str(buff, sizeof(buff), system_charset_info);
   str.length(0);
-  if (item != nullptr) {
+  if (item != NULL) {
     // QT_TO_SYSTEM_CHARSET because trace must be in UTF8
     item->print(current_thd, &str,
                 enum_query_type(QT_TO_SYSTEM_CHARSET | QT_SHOW_SELECT_NUMBER |
@@ -362,11 +366,32 @@ Opt_trace_struct &Opt_trace_struct::do_add(const char *key, Item *item) {
 
 Opt_trace_struct &Opt_trace_struct::do_add(const char *key,
                                            const Cost_estimate &value) {
-  return do_add(key, value.total_cost());
+  char buf[32];  // 32 is enough for digits of a double
+  my_gcvt(value.total_cost(), MY_GCVT_ARG_DOUBLE, FLT_DIG, buf, nullptr);
+  DBUG_PRINT("opt", ("%s: %s", key, buf));
+  stmt->add(key, buf, strlen(buf), false, false);
+  return *this;
 }
 
-Opt_trace_struct &Opt_trace_struct::do_add_utf8_table(const Table_ref *tl) {
-  if (tl != nullptr) {
+Opt_trace_struct &Opt_trace_struct::do_add_hex(const char *key, uint64 val) {
+  DBUG_ASSERT(started);
+  char buf[2 + 16], *p_end = buf + sizeof(buf) - 1, *p = p_end;
+  for (;;) {
+    *p-- = _dig_vec_lower[val & 15];
+    *p-- = _dig_vec_lower[(val & 240) >> 4];
+    val >>= 8;
+    if (val == 0) break;
+  }
+  *p-- = 'x';
+  *p = '0';
+  const size_t len = p_end + 1 - p;
+  DBUG_PRINT("opt", ("%s: %.*s", key, static_cast<int>(len), p));
+  stmt->add(check_key(key), p, len, false, false);
+  return *this;
+}
+
+Opt_trace_struct &Opt_trace_struct::do_add_utf8_table(const TABLE_LIST *tl) {
+  if (tl != NULL) {
     StringBuffer<32> str;
     tl->print(current_thd, &str,
               enum_query_type(QT_TO_SYSTEM_CHARSET | QT_SHOW_SELECT_NUMBER |
@@ -377,22 +402,22 @@ Opt_trace_struct &Opt_trace_struct::do_add_utf8_table(const Table_ref *tl) {
 }
 
 const char *Opt_trace_struct::check_key(const char *key) {
-  assert(started);
+  DBUG_ASSERT(started);
   //  User should always add to the innermost open object, not outside.
   stmt->assert_current_struct(this);
-  bool has_key = key != nullptr;
+  bool has_key = key != NULL;
   if (unlikely(has_key != requires_key)) {
     // fix the key to produce correct JSON syntax:
-    key = has_key ? nullptr : stmt->make_unknown_key();
+    key = has_key ? NULL : stmt->make_unknown_key();
     has_key = !has_key;
   }
   if (has_key) {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     /*
       Check that we're not having two identical consecutive keys in one
       object; though the real restriction should not have 'consecutive'.
     */
-    assert(strncmp(previous_key, key, sizeof(previous_key) - 1) != 0);
+    DBUG_ASSERT(strncmp(previous_key, key, sizeof(previous_key) - 1) != 0);
     strncpy(previous_key, key, sizeof(previous_key) - 1);
     previous_key[sizeof(previous_key) - 1] = 0;
 #endif
@@ -407,17 +432,17 @@ Opt_trace_stmt::Opt_trace_stmt(Opt_trace_context *ctx_arg)
       I_S_disabled(0),
       missing_priv(false),
       ctx(ctx_arg),
-      current_struct(nullptr),
+      current_struct(NULL),
       stack_of_current_structs(PSI_INSTRUMENT_ME),
       unknown_key_count(0) {
   // Trace is always in UTF8. This is the only charset which JSON accepts.
   trace_buffer.set_charset(system_charset_info);
-  assert(system_charset_info == &my_charset_utf8mb3_general_ci);
+  DBUG_ASSERT(system_charset_info == &my_charset_utf8_general_ci);
 }
 
 void Opt_trace_stmt::end() {
-  assert(stack_of_current_structs.size() == 0);
-  assert(I_S_disabled >= 0);
+  DBUG_ASSERT(stack_of_current_structs.size() == 0);
+  DBUG_ASSERT(I_S_disabled >= 0);
   ended = true;
   /*
     Because allocation is done in big chunks, buffer->Ptr[str_length]
@@ -448,7 +473,7 @@ void Opt_trace_stmt::set_allowed_mem_size(size_t size) {
 void Opt_trace_stmt::set_query(const char *query, size_t length,
                                const CHARSET_INFO *charset) {
   // Should be called only once per statement.
-  assert(query_buffer.ptr() == nullptr);
+  DBUG_ASSERT(query_buffer.ptr() == NULL);
   query_buffer.set_charset(charset);
   if (!support_I_S()) {
     /*
@@ -488,8 +513,8 @@ bool Opt_trace_stmt::open_struct(const char *key, Opt_trace_struct *ots,
         disabled too.
         When the structure is destroyed, the initial setting is restored.
       */
-      if (current_struct != nullptr) {
-        if (key != nullptr)
+      if (current_struct != NULL) {
+        if (key != NULL)
           current_struct->add_alnum(key, "...");
         else
           current_struct->add_alnum("...");
@@ -528,7 +553,7 @@ void Opt_trace_stmt::close_struct(const char *saved_key, bool has_disabled_I_S,
   if (support_I_S()) {
     next_line();
     trace_buffer.append(closing_bracket);
-    if (ctx->get_end_marker() && saved_key != nullptr) {
+    if (ctx->get_end_marker() && saved_key != NULL) {
       trace_buffer.append(STRING_WITH_LEN(" /* "));
       trace_buffer.append(saved_key);
       trace_buffer.append(STRING_WITH_LEN(" */"));
@@ -538,9 +563,9 @@ void Opt_trace_stmt::close_struct(const char *saved_key, bool has_disabled_I_S,
 }
 
 void Opt_trace_stmt::separator() {
-  assert(support_I_S());
+  DBUG_ASSERT(support_I_S());
   // Put a comma first, if we have already written an object at this level.
-  if (current_struct != nullptr) {
+  if (current_struct != NULL) {
     if (!current_struct->set_not_empty()) trace_buffer.append(',');
     next_line();
   }
@@ -576,8 +601,8 @@ void Opt_trace_stmt::add(const char *key, const char *val, size_t val_length,
                          bool quotes, bool escape) {
   if (!support_I_S()) return;
   separator();
-  if (current_struct != nullptr) key = current_struct->check_key(key);
-  if (key != nullptr) {
+  if (current_struct != NULL) key = current_struct->check_key(key);
+  if (key != NULL) {
     trace_buffer.append('"');
     trace_buffer.append(key);
     trace_buffer.append(STRING_WITH_LEN("\": "));
@@ -612,6 +637,13 @@ void Opt_trace_stmt::fill_info(Opt_trace_info *info) const {
   }
 }
 
+const char *Opt_trace_stmt::trace_buffer_tail(size_t size) {
+  size_t buffer_len = trace_buffer.length();
+  const char *ptr = trace_buffer.c_ptr_safe();
+  if (buffer_len > size) ptr += buffer_len - size;
+  return ptr;
+}
+
 void Opt_trace_stmt::missing_privilege() {
   if (!missing_priv) {
     DBUG_PRINT("opt", ("trace denied"));
@@ -624,7 +656,7 @@ void Opt_trace_stmt::missing_privilege() {
 
 // Implementation of class Buffer
 
-namespace {
+namespace random_name_to_avoid_gcc_bug_29365 {
 
 void Buffer::append_escaped(const char *str, size_t length) {
   if (alloced_length() >= allowed_mem_size) {
@@ -747,7 +779,7 @@ void Buffer::prealloc() {
       We jump from 0 to first_increment and then multiply by 1.5. Unlike
       addition of a constant length, multiplying is expected to give amortized
       constant reallocation time; 1.5 is a commonly seen factor in the
-      literature.
+      litterature.
     */
     size_t new_size = (alloced == 0) ? first_increment : (alloced * 15 / 10);
     size_t max_size = allowed_mem_size;
@@ -771,7 +803,7 @@ void Buffer::prealloc() {
   }
 }
 
-}  // namespace
+}  // namespace random_name_to_avoid_gcc_bug_29365
 
 // Implementation of Opt_trace_context class
 
@@ -789,13 +821,13 @@ const Opt_trace_context::feature_value Opt_trace_context::default_features =
                                      Opt_trace_context::REPEATED_SUBSELECT);
 
 Opt_trace_context::~Opt_trace_context() {
-  if (unlikely(pimpl != nullptr)) {
+  if (unlikely(pimpl != NULL)) {
     /* There may well be some few ended traces left: */
     purge_stmts(true);
     /* All should have moved to 'del' list: */
-    assert(pimpl->all_stmts_for_I_S.size() == 0);
+    DBUG_ASSERT(pimpl->all_stmts_for_I_S.size() == 0);
     /* All of 'del' list should have been deleted: */
-    assert(pimpl->all_stmts_to_del.size() == 0);
+    DBUG_ASSERT(pimpl->all_stmts_to_del.size() == 0);
     delete pimpl;
   }
 }
@@ -803,14 +835,14 @@ Opt_trace_context::~Opt_trace_context() {
 template <class T>
 T *new_nothrow_w_my_error() {
   T *const t = new (std::nothrow) T();
-  if (unlikely(t == nullptr))
+  if (unlikely(t == NULL))
     my_error(ER_OUTOFMEMORY, MYF(ME_FATALERROR), static_cast<int>(sizeof(T)));
   return t;
 }
 template <class T, class Arg>
 T *new_nothrow_w_my_error(Arg a) {
   T *const t = new (std::nothrow) T(a);
-  if (unlikely(t == nullptr))
+  if (unlikely(t == NULL))
     my_error(ER_OUTOFMEMORY, MYF(ME_FATALERROR), static_cast<int>(sizeof(T)));
   return t;
 }
@@ -835,7 +867,7 @@ bool Opt_trace_context::start(bool support_I_S_arg,
   */
   if (!support_I_S_arg && !support_dbug_or_missing_priv) {
     // The statement will not do tracing.
-    if (likely(pimpl == nullptr) || pimpl->current_stmt_in_gen == nullptr) {
+    if (likely(pimpl == NULL) || pimpl->current_stmt_in_gen == NULL) {
       /*
         This should be the most commonly taken branch in a release binary,
         when the connection rarely has optimizer tracing runtime-enabled.
@@ -858,10 +890,10 @@ bool Opt_trace_context::start(bool support_I_S_arg,
     */
   }
 
-  DBUG_EXECUTE_IF("no_new_opt_trace_stmt", assert(0););
+  DBUG_EXECUTE_IF("no_new_opt_trace_stmt", DBUG_ASSERT(0););
 
-  if (pimpl == nullptr &&
-      ((pimpl = new_nothrow_w_my_error<Opt_trace_context_impl>()) == nullptr))
+  if (pimpl == NULL &&
+      ((pimpl = new_nothrow_w_my_error<Opt_trace_context_impl>()) == NULL))
     return true;
 
   /*
@@ -907,8 +939,8 @@ bool Opt_trace_context::start(bool support_I_S_arg,
 
     DBUG_PRINT("opt", ("new stmt %p support_I_S %d", stmt, support_I_S_arg));
 
-    if (unlikely(stmt == nullptr || pimpl->stack_of_current_stmts.push_back(
-                                        pimpl->current_stmt_in_gen)))
+    if (unlikely(stmt == NULL || pimpl->stack_of_current_stmts.push_back(
+                                     pimpl->current_stmt_in_gen)))
       goto err;  // push_back() above called my_error()
 
     /*
@@ -936,15 +968,15 @@ bool Opt_trace_context::start(bool support_I_S_arg,
     return false;
   err:
     delete stmt;
-    assert(0);
+    DBUG_ASSERT(0);
     return true;
   }
 }
 
 void Opt_trace_context::end() {
-  assert(I_S_disabled >= 0);
-  if (likely(pimpl == nullptr)) return;
-  if (pimpl->current_stmt_in_gen != nullptr) {
+  DBUG_ASSERT(I_S_disabled >= 0);
+  if (likely(pimpl == NULL)) return;
+  if (pimpl->current_stmt_in_gen != NULL) {
     pimpl->current_stmt_in_gen->end();
     /*
       pimpl was constructed with current_stmt_in_gen=NULL which was pushed in
@@ -953,7 +985,7 @@ void Opt_trace_context::end() {
     Opt_trace_stmt *const parent = pimpl->stack_of_current_stmts.back();
     pimpl->stack_of_current_stmts.pop_back();
     pimpl->current_stmt_in_gen = parent;
-    if (parent != nullptr) {
+    if (parent != NULL) {
       /*
         Parent regains control, now it needs to be told that its child has
         used space, and thus parent's allowance has shrunk.
@@ -980,11 +1012,11 @@ void Opt_trace_context::end() {
     */
     purge_stmts(false);
   } else
-    assert(pimpl->stack_of_current_stmts.size() == 0);
+    DBUG_ASSERT(pimpl->stack_of_current_stmts.size() == 0);
 }
 
 bool Opt_trace_context::support_I_S() const {
-  return (pimpl != nullptr) && (pimpl->current_stmt_in_gen != nullptr) &&
+  return (pimpl != NULL) && (pimpl->current_stmt_in_gen != NULL) &&
          pimpl->current_stmt_in_gen->support_I_S();
 }
 
@@ -1035,7 +1067,7 @@ void Opt_trace_context::purge_stmts(bool purge_all) {
   /* Examine list of "to be freed" traces and free what can be */
   for (idx = (pimpl->all_stmts_to_del.size() - 1); idx >= 0; idx--) {
     Opt_trace_stmt *stmt = pimpl->all_stmts_to_del.at(idx);
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     bool skip_del = false;
     DBUG_EXECUTE_IF("opt_trace_oom_in_purge", skip_del = true;);
 #else
@@ -1099,18 +1131,18 @@ void Opt_trace_context::set_query(const char *query, size_t length,
 }
 
 void Opt_trace_context::reset() {
-  if (pimpl == nullptr) return;
+  if (pimpl == NULL) return;
   purge_stmts(true);
   pimpl->since_offset_0 = 0;
 }
 
 void Opt_trace_context::Opt_trace_context_impl::
     disable_I_S_for_this_and_children() {
-  if (current_stmt_in_gen != nullptr) current_stmt_in_gen->disable_I_S();
+  if (current_stmt_in_gen != NULL) current_stmt_in_gen->disable_I_S();
 }
 
 void Opt_trace_context::Opt_trace_context_impl::restore_I_S() {
-  if (current_stmt_in_gen != nullptr) current_stmt_in_gen->restore_I_S();
+  if (current_stmt_in_gen != NULL) current_stmt_in_gen->restore_I_S();
 }
 
 void Opt_trace_context::missing_privilege() {
@@ -1121,8 +1153,8 @@ void Opt_trace_context::missing_privilege() {
     disabled.
     Storing in Opt_trace_context would require an external memory (probably a
     RAII object), which would not be possible in
-    Table_ref::prepare_security(), where I_S must be disabled even after
-    the end of that function - so RAII would not work.
+    TABLE_LIST::prepare_security(), where I_S must be disabled even after the
+    end of that function - so RAII would not work.
 
     Which is why this function needs an existing current_stmt_in_gen.
   */
@@ -1132,12 +1164,12 @@ void Opt_trace_context::missing_privilege() {
 const Opt_trace_stmt *Opt_trace_context::get_next_stmt_for_I_S(
     long *got_so_far) const {
   const Opt_trace_stmt *p;
-  if ((pimpl == nullptr) || (*got_so_far >= pimpl->limit) ||
+  if ((pimpl == NULL) || (*got_so_far >= pimpl->limit) ||
       (*got_so_far >= static_cast<long>(pimpl->all_stmts_for_I_S.size())))
-    p = nullptr;
+    p = NULL;
   else {
     p = pimpl->all_stmts_for_I_S.at(*got_so_far);
-    assert(p != nullptr);
+    DBUG_ASSERT(p != NULL);
     (*got_so_far)++;
   }
   return p;
