@@ -34,20 +34,18 @@
 #include "my_inttypes.h"
 #include "my_sys.h"
 #include "my_time.h"
-#include "mysql_com.h"
 #include "mysqld_error.h"
 #include "sql/field.h"
 #include "sql/item.h"
+#include "sql/item_cmpfunc.h"
 #include "sql/item_create.h"  // Create_func
 #include "sql/item_func.h"
-#include "sql/item_strfunc.h"
 #include "sql/item_subselect.h"
 #include "sql/item_sum.h"       // Item_sum_count
 #include "sql/item_timefunc.h"  // Item_func_now_local
 #include "sql/parse_location.h"
 #include "sql/parse_tree_helpers.h"  // Parse_tree_item
 #include "sql/parse_tree_node_base.h"
-#include "sql/protocol.h"
 #include "sql/set_var.h"
 #include "sql/sp_head.h"  // sp_head
 #include "sql/sql_class.h"
@@ -72,7 +70,7 @@ class PTI_table_wild : public Parse_tree_item {
                           const char *table_arg)
       : super(pos), schema(schema_arg), table(table_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **item);
+  bool itemize(Parse_context *pc, Item **item) override;
 };
 
 class PTI_truth_transform : public Parse_tree_item {
@@ -85,7 +83,7 @@ class PTI_truth_transform : public Parse_tree_item {
   PTI_truth_transform(const POS &pos, Item *expr_arg, Bool_test truth_test)
       : super(pos), expr(expr_arg), truth_test(truth_test) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_comp_op : public Parse_tree_item {
@@ -104,7 +102,7 @@ class PTI_comp_op : public Parse_tree_item {
         boolfunc2creator(boolfunc2creator_arg),
         right(right_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_comp_op_all : public Parse_tree_item {
@@ -125,20 +123,20 @@ class PTI_comp_op_all : public Parse_tree_item {
         is_all(is_all_arg),
         subselect(subselect_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_simple_ident_ident : public Parse_tree_item {
   typedef Parse_tree_item super;
 
-  LEX_STRING ident;
+  LEX_CSTRING ident;
   Symbol_location raw;
 
  public:
-  PTI_simple_ident_ident(const POS &pos, const LEX_STRING &ident_arg)
+  PTI_simple_ident_ident(const POS &pos, const LEX_CSTRING &ident_arg)
       : super(pos), ident(ident_arg), raw(pos.raw) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 /**
@@ -157,24 +155,7 @@ class PTI_simple_ident_q_3d : public Parse_tree_item {
                         const char *table_arg, const char *field_arg)
       : super(pos), db(db_arg), table(table_arg), field(field_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
-    if (super::itemize(pc, res)) return true;
-
-    THD *thd = pc->thd;
-    const char *schema =
-        thd->get_protocol()->has_client_capability(CLIENT_NO_SCHEMA) ? NULL
-                                                                     : db;
-    if (pc->select->no_table_names_allowed) {
-      my_error(ER_TABLENAME_NOT_ALLOWED_HERE, MYF(0), table, thd->where);
-    }
-    if ((pc->select->parsing_place != CTX_HAVING) ||
-        (pc->select->get_in_sum_expr() > 0)) {
-      *res = new (pc->mem_root) Item_field(POS(), schema, table, field);
-    } else {
-      *res = new (pc->mem_root) Item_ref(POS(), schema, table, field);
-    }
-    return *res == NULL || (*res)->itemize(pc, res);
-  }
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 /**
@@ -188,7 +169,7 @@ class PTI_simple_ident_q_2d : public PTI_simple_ident_q_3d {
                         const char *field_arg)
       : super(pos, NULL, table_arg, field_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_simple_ident_nospvar_ident : public Parse_tree_item {
@@ -200,7 +181,7 @@ class PTI_simple_ident_nospvar_ident : public Parse_tree_item {
   PTI_simple_ident_nospvar_ident(const POS &pos, const LEX_STRING &ident_arg)
       : super(pos), ident(ident_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     if ((pc->select->parsing_place != CTX_HAVING) ||
@@ -237,7 +218,7 @@ class PTI_function_call_nonkeyword_sysdate : public Parse_tree_item {
   explicit PTI_function_call_nonkeyword_sysdate(const POS &pos, uint8 dec_arg)
       : super(pos), dec(dec_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_udf_expr : public Parse_tree_item {
@@ -256,7 +237,7 @@ class PTI_udf_expr : public Parse_tree_item {
         select_alias(select_alias_arg),
         expr_loc(expr_loc_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_function_call_generic_ident_sys : public Parse_tree_item {
@@ -275,7 +256,7 @@ class PTI_function_call_generic_ident_sys : public Parse_tree_item {
         ident(ident_arg),
         opt_udf_expr_list(opt_udf_expr_list_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 /**
@@ -297,7 +278,7 @@ class PTI_function_call_generic_2d : public Parse_tree_item {
         func(func_arg),
         opt_expr_list(opt_expr_list_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_text_literal : public Item_string {
@@ -320,7 +301,7 @@ class PTI_text_literal_text_string : public PTI_text_literal {
                                const LEX_STRING &literal_arg)
       : super(pos, is_7bit_arg, literal_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     THD *thd = pc->thd;
@@ -352,7 +333,7 @@ class PTI_text_literal_nchar_string : public PTI_text_literal {
                                 const LEX_STRING &literal_arg)
       : super(pos, is_7bit_arg, literal_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_text_literal_underscore_charset : public PTI_text_literal {
@@ -366,7 +347,7 @@ class PTI_text_literal_underscore_charset : public PTI_text_literal {
                                       const LEX_STRING &literal_arg)
       : super(pos, is_7bit_arg, literal_arg), cs(cs_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     init(literal.str, literal.length, cs, DERIVATION_COERCIBLE,
@@ -387,7 +368,7 @@ class PTI_text_literal_concat : public PTI_text_literal {
                           PTI_text_literal *head_arg, const LEX_STRING &tail)
       : super(pos, is_7bit_arg, tail), head(head_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     Item *tmp_head;
     if (super::itemize(pc, res) || head->itemize(pc, &tmp_head)) return true;
 
@@ -425,7 +406,7 @@ class PTI_temporal_literal : public Parse_tree_item {
         field_type(field_type_arg),
         cs(cs_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     *res = create_temporal_literal(pc->thd, literal.str, literal.length, cs,
@@ -445,7 +426,7 @@ class PTI_literal_underscore_charset_hex_num : public Item_string {
               Item_hex_string::make_hex_str(literal.str, literal.length),
               charset) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     set_repertoire_from_value();
@@ -465,7 +446,7 @@ class PTI_literal_underscore_charset_bin_num : public Item_string {
               Item_bin_string::make_bin_str(literal.str, literal.length),
               charset) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     set_cs_specified(true);
@@ -480,7 +461,7 @@ class PTI_variable_aux_set_var final : public Item_func_set_user_var {
   PTI_variable_aux_set_var(const POS &pos, const LEX_STRING &var, Item *expr)
       : super(pos, var, expr, false) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     LEX *lex = pc->thd->lex;
@@ -501,7 +482,7 @@ class PTI_variable_aux_ident_or_text final : public Item_func_get_user_var {
   PTI_variable_aux_ident_or_text(const POS &pos, const LEX_STRING &var)
       : super(pos, var) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     LEX *lex = pc->thd->lex;
@@ -537,7 +518,7 @@ class PTI_variable_aux_3d : public Parse_tree_item {
         ident1_pos(ident1_pos_arg),
         ident2(ident2_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     LEX *lex = pc->thd->lex;
@@ -547,7 +528,7 @@ class PTI_variable_aux_3d : public Parse_tree_item {
     }
 
     /* disallow "SELECT @@global.global.variable" */
-    if (ident1.str && ident2.str && check_reserved_words(&ident1)) {
+    if (ident1.str && ident2.str && check_reserved_words(ident1.str)) {
       error(pc, ident1_pos);
       return true;
     }
@@ -594,7 +575,7 @@ class PTI_count_sym : public Item_sum_count {
  public:
   PTI_count_sym(const POS &pos, PT_window *w) : super(pos, (Item *)NULL, w) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     args[0] = new (pc->mem_root) Item_int((int32)0L, 1);
     if (args[0] == NULL) return true;
 
@@ -611,7 +592,7 @@ class PTI_in_sum_expr : public Parse_tree_item {
   PTI_in_sum_expr(const POS &pos, Item *expr_arg)
       : super(pos), expr(expr_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     pc->select->in_sum_expr++;
 
     if (super::itemize(pc, res) || expr->itemize(pc, &expr)) return true;
@@ -632,7 +613,7 @@ class PTI_singlerow_subselect : public Parse_tree_item {
   PTI_singlerow_subselect(const POS &pos, PT_subquery *subselect_arg)
       : super(pos), subselect(subselect_arg) {}
 
-  bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_exists_subselect : public Parse_tree_item {
@@ -644,7 +625,7 @@ class PTI_exists_subselect : public Parse_tree_item {
   PTI_exists_subselect(const POS &pos, PT_subquery *subselect_arg)
       : super(pos), subselect(subselect_arg) {}
 
-  bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_odbc_date : public Parse_tree_item {
@@ -657,7 +638,7 @@ class PTI_odbc_date : public Parse_tree_item {
   PTI_odbc_date(const POS &pos, const LEX_STRING &ident_arg, Item *expr_arg)
       : super(pos), ident(ident_arg), expr(expr_arg) {}
 
-  bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res) || expr->itemize(pc, &expr)) return true;
 
     *res = NULL;
@@ -712,7 +693,7 @@ class PTI_handle_sql2003_note184_exception : public Parse_tree_item {
         is_negation(is_negation_arg),
         right(right_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_expr_with_alias : public Parse_tree_item {
@@ -720,29 +701,29 @@ class PTI_expr_with_alias : public Parse_tree_item {
 
   Item *expr;
   Symbol_location expr_loc;
-  LEX_STRING alias;
+  LEX_CSTRING alias;
 
  public:
   PTI_expr_with_alias(const POS &pos, Item *expr_arg,
                       const Symbol_location &expr_loc_arg,
-                      const LEX_STRING &alias_arg)
+                      const LEX_CSTRING &alias_arg)
       : super(pos), expr(expr_arg), expr_loc(expr_loc_arg), alias(alias_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res);
+  bool itemize(Parse_context *pc, Item **res) override;
 };
 
 class PTI_limit_option_ident : public Parse_tree_item {
   typedef Parse_tree_item super;
 
-  LEX_STRING ident;
+  LEX_CSTRING ident;
   Symbol_location ident_loc;
 
  public:
-  explicit PTI_limit_option_ident(const POS &pos, const LEX_STRING &ident_arg,
-                                  const Symbol_location &ident_loc_arg)
+  PTI_limit_option_ident(const POS &pos, const LEX_CSTRING &ident_arg,
+                         const Symbol_location &ident_loc_arg)
       : super(pos), ident(ident_arg), ident_loc(ident_loc_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     LEX *lex = pc->thd->lex;
@@ -777,7 +758,7 @@ class PTI_limit_option_param_marker : public Parse_tree_item {
                                          Item_param *param_marker_arg)
       : super(pos), param_marker(param_marker_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     Item *tmp_param;
     if (super::itemize(pc, res) || param_marker->itemize(pc, &tmp_param))
       return true;
@@ -806,7 +787,7 @@ class PTI_context : public Parse_tree_item {
  public:
   PTI_context(const POS &pos, Item *expr_arg) : super(pos), expr(expr_arg) {}
 
-  virtual bool itemize(Parse_context *pc, Item **res) {
+  bool itemize(Parse_context *pc, Item **res) override {
     if (super::itemize(pc, res)) return true;
 
     pc->select->parsing_place = Context;
@@ -828,4 +809,7 @@ class PTI_context : public Parse_tree_item {
   }
 };
 
+using PTI_where = PTI_context<CTX_WHERE>;
+using PTI_having = PTI_context<CTX_HAVING>;
+
 #endif /* PARSE_TREE_ITEMS_INCLUDED */
