@@ -1,4 +1,4 @@
-/* Copyright (c) 2006, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,7 +25,6 @@
 #include <limits.h>
 #include <stdio.h>
 #include <string.h>
-
 #include <algorithm>
 #include <memory>
 #include <string>
@@ -39,8 +38,8 @@
 #include "my_dbug.h"
 #include "my_sqlcommand.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/plugin.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/udf_registration_types.h"
 #include "mysqld_error.h"
 #include "sql/auth/auth_acls.h"
@@ -80,13 +79,13 @@ partition_info *partition_info::get_clone(THD *thd, bool reset /* = false */) {
   partition_info *clone = new (thd->mem_root) partition_info(*this);
   if (!clone) {
     mem_alloc_error(sizeof(partition_info));
-    return nullptr;
+    return NULL;
   }
   new (&(clone->read_partitions)) MY_BITMAP;
   new (&(clone->lock_partitions)) MY_BITMAP;
   clone->bitmaps_are_initialized = false;
-  clone->partitions.clear();
-  clone->temp_partitions.clear();
+  clone->partitions.empty();
+  clone->temp_partitions.empty();
 
   while ((part = (part_it++))) {
     List_iterator<partition_element> subpart_it(part->subpartitions);
@@ -95,7 +94,7 @@ partition_info *partition_info::get_clone(THD *thd, bool reset /* = false */) {
         new (thd->mem_root) partition_element(*part);
     if (!part_clone) {
       mem_alloc_error(sizeof(partition_element));
-      return nullptr;
+      return NULL;
     }
 
     /* Explicitly copy the tablespace name, use the thd->mem_root. */
@@ -120,13 +119,13 @@ partition_info *partition_info::get_clone(THD *thd, bool reset /* = false */) {
       }
     }
 
-    part_clone->subpartitions.clear();
+    part_clone->subpartitions.empty();
     while ((subpart = (subpart_it++))) {
       partition_element *subpart_clone =
           new (thd->mem_root) partition_element(*subpart);
       if (!subpart_clone) {
         mem_alloc_error(sizeof(partition_element));
-        return nullptr;
+        return NULL;
       }
 
       /* Explicitly copy the tablespace name, use the thd->mem_root. */
@@ -146,7 +145,7 @@ partition_info *partition_info::get_full_clone(THD *thd) {
   partition_info *clone;
   DBUG_TRACE;
   clone = get_clone(thd);
-  if (!clone) return nullptr;
+  if (!clone) return NULL;
   memcpy(&clone->read_partitions, &read_partitions, sizeof(read_partitions));
   memcpy(&clone->lock_partitions, &lock_partitions, sizeof(lock_partitions));
   clone->bitmaps_are_initialized = bitmaps_are_initialized;
@@ -168,11 +167,11 @@ bool partition_info::add_named_partition(const char *part_name, size_t length) {
   PART_NAME_DEF *part_def;
   Partition_share *part_share;
   DBUG_TRACE;
-  assert(table && table->s && table->s->ha_share);
+  DBUG_ASSERT(table && table->s && table->s->ha_share);
   part_share = static_cast<Partition_share *>((table->s->ha_share));
-  assert(part_share->partition_name_hash != nullptr);
+  DBUG_ASSERT(part_share->partition_name_hash != nullptr);
   auto part_name_hash = part_share->partition_name_hash.get();
-  assert(!part_name_hash->empty());
+  DBUG_ASSERT(!part_name_hash->empty());
 
   part_def = find_or_nullptr(*part_name_hash, string(part_name, length));
   if (!part_def) {
@@ -261,8 +260,8 @@ bool partition_info::set_read_partitions(List<String> *partition_names) {
 bool partition_info::set_partition_bitmaps(TABLE_LIST *table_list) {
   DBUG_TRACE;
 
-  assert(bitmaps_are_initialized);
-  assert(table);
+  DBUG_ASSERT(bitmaps_are_initialized);
+  DBUG_ASSERT(table);
   is_pruning_completed = false;
   if (!bitmaps_are_initialized) return true;
 
@@ -283,7 +282,7 @@ bool partition_info::set_partition_bitmaps(TABLE_LIST *table_list) {
     DBUG_PRINT("info", ("Set all partitions"));
   }
   bitmap_copy(&lock_partitions, &read_partitions);
-  assert(bitmap_get_first_set(&lock_partitions) != MY_BIT_NONE);
+  DBUG_ASSERT(bitmap_get_first_set(&lock_partitions) != MY_BIT_NONE);
   return false;
 }
 
@@ -307,14 +306,15 @@ bool partition_info::set_partition_bitmaps(TABLE_LIST *table_list) {
     @retval true   Failure
 */
 
-bool partition_info::can_prune_insert(
-    THD *thd, enum_duplicates duplic, COPY_INFO &update,
-    const mem_root_deque<Item *> &update_fields,
-    const mem_root_deque<Item *> &fields, bool empty_values,
-    enum_can_prune *can_prune_partitions, bool *prune_needs_default_values,
-    MY_BITMAP *used_partitions) {
+bool partition_info::can_prune_insert(THD *thd, enum_duplicates duplic,
+                                      COPY_INFO &update,
+                                      List<Item> &update_fields,
+                                      List<Item> &fields, bool empty_values,
+                                      enum_can_prune *can_prune_partitions,
+                                      bool *prune_needs_default_values,
+                                      MY_BITMAP *used_partitions) {
   *can_prune_partitions = PRUNE_NO;
-  assert(bitmaps_are_initialized);
+  DBUG_ASSERT(bitmaps_are_initialized);
   DBUG_TRACE;
 
   if (table->s->db_type()->partition_flags() & HA_USE_AUTO_PARTITION)
@@ -344,7 +344,7 @@ bool partition_info::can_prune_insert(
   if (table->vfield) {
     Field **fld;
     for (fld = table->vfield; *fld; fld++) {
-      if (bitmap_is_set(&full_part_field_set, (*fld)->field_index()))
+      if (bitmap_is_set(&full_part_field_set, (*fld)->field_index))
         return false;
     }
   }
@@ -356,7 +356,7 @@ bool partition_info::can_prune_insert(
   if (table->gen_def_fields_ptr) {
     Field **fld;
     for (fld = table->gen_def_fields_ptr; *fld; fld++) {
-      if (bitmap_is_set(&full_part_field_set, (*fld)->field_index()))
+      if (bitmap_is_set(&full_part_field_set, (*fld)->field_index))
         return false;
     }
   }
@@ -368,7 +368,7 @@ bool partition_info::can_prune_insert(
       is not 0 (with NO_AUTO_VALUE_ON_ZERO sql_mode), then pruning is possible!
     */
     if (bitmap_is_set(&full_part_field_set,
-                      table->found_next_number_field->field_index()))
+                      table->found_next_number_field->field_index))
       return false;
   }
 
@@ -419,11 +419,11 @@ bool partition_info::can_prune_insert(
       2) only copy those fields from the default record.
   */
   *prune_needs_default_values = false;
-  if (empty_values) {
-    *prune_needs_default_values = true;  // like 'INSERT INTO t () VALUES ()'
-  } else if (!fields.empty()) {
+  if (fields.elements) {
     if (!is_full_part_expr_in_fields(fields))
       *prune_needs_default_values = true;
+  } else if (empty_values) {
+    *prune_needs_default_values = true;  // like 'INSERT INTO t () VALUES ()'
   } else {
     /*
       In case of INSERT INTO t VALUES (...) we must get values for
@@ -441,7 +441,7 @@ bool partition_info::can_prune_insert(
   /*
     If no partitioning field in set (e.g. defaults) check pruning only once.
   */
-  if (!fields.empty() && !is_fields_in_part_expr(fields))
+  if (fields.elements && !is_fields_in_part_expr(fields))
     *can_prune_partitions = PRUNE_DEFAULTS;
   else
     *can_prune_partitions = PRUNE_YES;
@@ -452,7 +452,6 @@ bool partition_info::can_prune_insert(
 /**
   Mark the partition, the record belongs to, as used.
 
-  @param thd              Thread handler
   @param fields           Fields to set
   @param values           Values to use
   @param info             COPY_INFO used for default values handling
@@ -466,31 +465,33 @@ bool partition_info::can_prune_insert(
   so caller must check thd->is_error().
 */
 
-bool partition_info::set_used_partition(THD *thd,
-                                        const mem_root_deque<Item *> &fields,
-                                        const mem_root_deque<Item *> &values,
+bool partition_info::set_used_partition(List<Item> &fields, List<Item> &values,
                                         COPY_INFO &info,
                                         bool copy_default_values,
                                         MY_BITMAP *used_partitions) {
+  THD *thd = table->in_use;
   uint32 part_id;
   longlong func_value;
 
-  assert(thd);
+  DBUG_ASSERT(thd);
 
   /* Only allow checking of constant values */
-  for (Item *item : values) {
+  List_iterator_fast<Item> v(values);
+  Item *item;
+
+  while ((item = v++)) {
     if (!item->const_item()) return true;
   }
 
   if (copy_default_values) restore_record(table, s->default_values);
 
-  if (!fields.empty() || values.empty()) {
-    if (fill_record(thd, table, fields, values, &full_part_field_set, nullptr,
+  if (fields.elements || !values.elements) {
+    if (fill_record(thd, table, fields, values, &full_part_field_set, NULL,
                     false))
       return true;
   } else {
     if (fill_record(thd, table, table->field, values, &full_part_field_set,
-                    nullptr, false))
+                    NULL, false))
       return true;
   }
 
@@ -500,9 +501,9 @@ bool partition_info::set_used_partition(THD *thd,
     overhead. Not yet done, since mostly only one DEFAULT function per
     table, or at least very few such columns.
   */
-  if (info.function_defaults_apply_on_columns(&full_part_field_set)) {
-    if (info.set_function_defaults(table)) return true;
-  }
+  if (info.function_defaults_apply_on_columns(&full_part_field_set))
+    info.set_function_defaults(table);
+
   {
     /*
       This function is used in INSERT; 'values' are supplied by user,
@@ -548,7 +549,7 @@ char *partition_info::create_default_partition_names(uint num_parts_arg,
   uint i = 0;
   DBUG_TRACE;
 
-  if (likely(ptr != nullptr)) {
+  if (likely(ptr != 0)) {
     do {
       sprintf(move_ptr, "p%u", (start_no + i));
       move_ptr += MAX_PART_NAME_SIZE;
@@ -584,8 +585,10 @@ void partition_info::set_show_version_string(String *packet) {
       /* No new functions in partition function */
       packet->append(STRING_WITH_LEN("\n/*!50100"));
     } else {
+      char buf[65];
+      char *buf_ptr = longlong10_to_str((longlong)version, buf, 10);
       packet->append(STRING_WITH_LEN("\n/*!"));
-      packet->append_longlong(version);
+      packet->append(buf, (size_t)(buf_ptr - buf));
     }
   }
 }
@@ -607,7 +610,7 @@ char *partition_info::create_default_subpartition_name(uint subpart_no,
   char *ptr = (char *)sql_calloc(size_alloc);
   DBUG_TRACE;
 
-  if (likely(ptr != nullptr)) {
+  if (likely(ptr != NULL)) {
     snprintf(ptr, size_alloc, "%ssp%u", part_name, subpart_no);
   } else {
     mem_alloc_error(size_alloc);
@@ -679,7 +682,7 @@ bool partition_info::set_up_default_partitions(Partition_handler *part_handler,
   i = 0;
   do {
     partition_element *part_elem = new (*THR_MALLOC) partition_element();
-    if (likely(part_elem != nullptr && (!partitions.push_back(part_elem)))) {
+    if (likely(part_elem != 0 && (!partitions.push_back(part_elem)))) {
       part_elem->engine_type = default_engine_type;
       part_elem->partition_name = default_name;
       default_name += MAX_PART_NAME_SIZE;
@@ -742,7 +745,7 @@ bool partition_info::set_up_default_subpartitions(
     do {
       partition_element *subpart_elem =
           new (*THR_MALLOC) partition_element(part_elem);
-      if (likely(subpart_elem != nullptr &&
+      if (likely(subpart_elem != 0 &&
                  (!part_elem->subpartitions.push_back(subpart_elem)))) {
         char *ptr =
             create_default_subpartition_name(j, part_elem->partition_name);
@@ -826,14 +829,18 @@ char *partition_info::find_duplicate_field() {
       }
     }
   }
-  return nullptr;
+  return NULL;
 }
 
 /**
   @brief Get part_elem and part_id from partition name
 
   @param partition_name Name of partition to search for.
-
+  @param [out] file_name Partition file name (part after table name,
+                        @code
+                        #P#<part>[#SP#<subpart>]
+                        @endcode
+                        ), skipped if NULL.
   @param [out] part_id   Id of found partition or NOT_A_PARTITION_ID.
 
   @retval Pointer to part_elem of [sub]partition, if not found NULL
@@ -844,11 +851,12 @@ char *partition_info::find_duplicate_field() {
   the partition, but part_id will be NOT_A_PARTITION_ID and file_name not set.
 */
 partition_element *partition_info::get_part_elem(const char *partition_name,
+                                                 char *file_name,
                                                  uint32 *part_id) {
   List_iterator<partition_element> part_it(partitions);
   uint i = 0;
   DBUG_TRACE;
-  assert(part_id);
+  DBUG_ASSERT(part_id);
   *part_id = NOT_A_PARTITION_ID;
   do {
     partition_element *part_elem = part_it++;
@@ -859,6 +867,9 @@ partition_element *partition_info::get_part_elem(const char *partition_name,
         partition_element *sub_part_elem = sub_part_it++;
         if (!my_strcasecmp(system_charset_info, sub_part_elem->partition_name,
                            partition_name)) {
+          if (file_name)
+            create_subpartition_name(file_name, "", part_elem->partition_name,
+                                     partition_name);
           *part_id = j + (i * num_subparts);
           return sub_part_elem;
         }
@@ -870,11 +881,12 @@ partition_element *partition_info::get_part_elem(const char *partition_name,
         return part_elem;
     } else if (!my_strcasecmp(system_charset_info, part_elem->partition_name,
                               partition_name)) {
+      if (file_name) create_partition_name(file_name, "", partition_name, true);
       *part_id = i;
       return part_elem;
     }
   } while (++i < num_parts);
-  return nullptr;
+  return NULL;
 }
 
 /*
@@ -896,6 +908,7 @@ partition_element *partition_info::get_part_elem(const char *partition_name,
 const char *partition_info::find_duplicate_name() {
   collation_unordered_set<string> partition_names{system_charset_info,
                                                   PSI_INSTRUMENT_ME};
+  uint max_names;
   List_iterator<partition_element> parts_it(partitions);
   partition_element *p_elem;
 
@@ -907,6 +920,8 @@ const char *partition_info::find_duplicate_name() {
     And this only happens when in ALTER TABLE with full table copy.
   */
 
+  max_names = num_parts;
+  if (is_sub_partitioned()) max_names += num_parts * num_subparts;
   while ((p_elem = (parts_it++))) {
     const char *partition_name = p_elem->partition_name;
     if (!partition_names.insert(partition_name).second) return partition_name;
@@ -1104,12 +1119,12 @@ bool partition_info::check_range_constants(THD *thd) {
 
   if (column_list) {
     part_column_list_val *loc_range_col_array;
-    part_column_list_val *current_largest_col_val = nullptr;
+    part_column_list_val *current_largest_col_val = NULL;
     uint num_column_values = part_field_list.elements;
     uint size_entries = sizeof(part_column_list_val) * num_column_values;
     range_col_array =
         (part_column_list_val *)sql_calloc(num_parts * size_entries);
-    if (unlikely(range_col_array == nullptr)) {
+    if (unlikely(range_col_array == NULL)) {
       mem_alloc_error(num_parts * size_entries);
       goto end;
     }
@@ -1121,7 +1136,7 @@ bool partition_info::check_range_constants(THD *thd) {
         List_iterator<part_elem_value> list_val_it(part_def->list_val_list);
         part_elem_value *range_val = list_val_it++;
         part_column_list_val *col_val = range_val->col_val_array;
-        assert(part_def->list_val_list.elements == 1);
+        DBUG_ASSERT(part_def->list_val_list.elements == 1);
 
         if (fix_column_value_functions(thd, range_val, i)) goto end;
         memcpy(loc_range_col_array, (const void *)col_val, size_entries);
@@ -1141,7 +1156,7 @@ bool partition_info::check_range_constants(THD *thd) {
 
     range_int_array =
         (longlong *)(*THR_MALLOC)->Alloc(num_parts * sizeof(longlong));
-    if (unlikely(range_int_array == nullptr)) {
+    if (unlikely(range_int_array == NULL)) {
       mem_alloc_error(num_parts * sizeof(longlong));
       goto end;
     }
@@ -1238,7 +1253,7 @@ bool partition_info::compare_column_values(
 */
 
 bool partition_info::check_list_constants(THD *thd) {
-  uint size_entries, num_column_values;
+  uint i, size_entries, num_column_values;
   uint list_index = 0;
   part_elem_value *list_value;
   bool result = true;
@@ -1265,7 +1280,7 @@ bool partition_info::check_list_constants(THD *thd) {
     list.
   */
 
-  uint part_id = 0;
+  i = 0;
   do {
     part_def = list_func_it++;
     if (part_def->has_null_value) {
@@ -1274,19 +1289,19 @@ bool partition_info::check_list_constants(THD *thd) {
         goto end;
       }
       has_null_value = true;
-      has_null_part_id = part_id;
+      has_null_part_id = i;
       found_null = true;
     }
     List_iterator<part_elem_value> list_val_it1(part_def->list_val_list);
     while (list_val_it1++) num_list_values++;
-  } while (++part_id < num_parts);
+  } while (++i < num_parts);
   list_func_it.rewind();
   num_column_values = part_field_list.elements;
   size_entries = column_list
                      ? (num_column_values * sizeof(part_column_list_val))
                      : sizeof(LIST_PART_ENTRY);
   ptr = sql_calloc((num_list_values + 1) * size_entries);
-  if (unlikely(ptr == nullptr)) {
+  if (unlikely(ptr == NULL)) {
     mem_alloc_error(num_list_values * size_entries);
     goto end;
   }
@@ -1294,19 +1309,19 @@ bool partition_info::check_list_constants(THD *thd) {
     part_column_list_val *loc_list_col_array;
     loc_list_col_array = (part_column_list_val *)ptr;
     list_col_array = (part_column_list_val *)ptr;
-    part_id = 0;
+    i = 0;
     do {
       part_def = list_func_it++;
       List_iterator<part_elem_value> list_val_it2(part_def->list_val_list);
       while ((list_value = list_val_it2++)) {
         part_column_list_val *col_val = list_value->col_val_array;
-        if (unlikely(fix_column_value_functions(thd, list_value, part_id))) {
+        if (unlikely(fix_column_value_functions(thd, list_value, i))) {
           return true;
         }
         memcpy(loc_list_col_array, (const void *)col_val, size_entries);
         loc_list_col_array += num_column_values;
       }
-    } while (++part_id < num_parts);
+    } while (++i < num_parts);
 
     varlen_sort(list_col_array,
                 list_col_array + num_list_values * num_column_values,
@@ -1322,7 +1337,7 @@ bool partition_info::check_list_constants(THD *thd) {
     }
   } else {
     list_array = (LIST_PART_ENTRY *)ptr;
-    part_id = 0;
+    i = 0;
     /*
       Fix to be able to reuse signed sort functions also for unsigned
       partition functions.
@@ -1336,9 +1351,9 @@ bool partition_info::check_list_constants(THD *thd) {
       while ((list_value = list_val_it2++)) {
         calc_value = list_value->value | type_add;
         list_array[list_index].list_value = calc_value;
-        list_array[list_index++].partition_id = part_id;
+        list_array[list_index++].partition_id = i;
       }
-    } while (++part_id < num_parts);
+    } while (++i < num_parts);
 
     LIST_PART_ENTRY *list_array_end = list_array + num_list_values;
     std::sort(list_array, list_array_end,
@@ -1354,7 +1369,7 @@ bool partition_info::check_list_constants(THD *thd) {
       goto end;
     }
   }
-  assert(fixed);
+  DBUG_ASSERT(fixed);
   result = false;
 end:
   return result;
@@ -1374,7 +1389,7 @@ static void warn_if_dir_in_part_elem(THD *thd, partition_element *part_elem) {
     if (part_elem->index_file_name)
       push_warning_printf(thd, Sql_condition::SL_WARNING, WARN_OPTION_IGNORED,
                           ER_THD(thd, WARN_OPTION_IGNORED), "INDEX DIRECTORY");
-    part_elem->data_file_name = part_elem->index_file_name = nullptr;
+    part_elem->data_file_name = part_elem->index_file_name = NULL;
   }
 }
 
@@ -1417,16 +1432,16 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
 
     /* Check for partition expression. */
     if (!list_of_part_fields) {
-      assert(part_expr);
+      DBUG_ASSERT(part_expr);
       err = part_expr->walk(&Item::check_partition_func_processor,
-                            enum_walk::POSTFIX, nullptr);
+                            enum_walk::POSTFIX, NULL);
     }
 
     /* Check for sub partition expression. */
     if (!err && is_sub_partitioned() && !list_of_subpart_fields) {
-      assert(subpart_expr);
+      DBUG_ASSERT(subpart_expr);
       err = subpart_expr->walk(&Item::check_partition_func_processor,
-                               enum_walk::POSTFIX, nullptr);
+                               enum_walk::POSTFIX, NULL);
     }
 
     if (err) {
@@ -1501,7 +1516,7 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
       partition_element *part_elem = part_it++;
       warn_if_dir_in_part_elem(thd, part_elem);
       if (!is_sub_partitioned()) {
-        if (part_elem->engine_type == nullptr) {
+        if (part_elem->engine_type == NULL) {
           num_parts_not_set++;
           part_elem->engine_type = default_engine_type;
         }
@@ -1534,8 +1549,8 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
             my_error(ER_TOO_LONG_IDENT, MYF(0));
             goto end;
           }
-          if (sub_elem->engine_type == nullptr) {
-            if (part_elem->engine_type != nullptr)
+          if (sub_elem->engine_type == NULL) {
+            if (part_elem->engine_type != NULL)
               sub_elem->engine_type = part_elem->engine_type;
             else {
               sub_elem->engine_type = default_engine_type;
@@ -1559,7 +1574,7 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
           goto end;
         }
 
-        if (part_elem->engine_type == nullptr) {
+        if (part_elem->engine_type == NULL) {
           if (num_subparts_not_set == 0)
             part_elem->engine_type = sub_elem->engine_type;
           else {
@@ -1582,7 +1597,7 @@ bool partition_info::check_partition_info(THD *thd, handlerton **eng_type,
     goto end;
   }
 
-  assert(table_engine == default_engine_type);
+  DBUG_ASSERT(table_engine == default_engine_type);
 
   if (eng_type) *eng_type = table_engine;
 
@@ -1635,7 +1650,7 @@ void partition_info::print_no_partition_found(THD *thd, TABLE *table_arg) {
       if (part_expr->null_value)
         buf_ptr = "NULL";
       else
-        longlong10_to_str(err_value, buf, part_expr->unsigned_flag ? 10 : -10);
+        longlong2str(err_value, buf, part_expr->unsigned_flag ? 10 : -10);
       dbug_tmp_restore_column_map(table_arg->read_set, old_map);
     }
     my_error(ER_NO_PARTITION_FOR_GIVEN_VALUE, MYF(0), buf_ptr);
@@ -1764,7 +1779,7 @@ bool partition_info::set_up_charset_field_preps() {
         part_field_buffers[i++] = field_buf;
       }
     }
-    part_charset_field_array[i] = nullptr;
+    part_charset_field_array[i] = NULL;
   }
   if (is_sub_partitioned() && !list_of_subpart_fields &&
       check_part_func_fields(subpart_field_array, false)) {
@@ -1787,7 +1802,7 @@ bool partition_info::set_up_charset_field_preps() {
     ptr = subpart_field_array;
     i = 0;
     while ((field = *(ptr++))) {
-      uchar *field_buf = nullptr;
+      uchar *field_buf = NULL;
 
       if (!field_is_partition_charset(field)) continue;
       size = field->pack_length();
@@ -1795,7 +1810,7 @@ bool partition_info::set_up_charset_field_preps() {
       subpart_charset_field_array[i] = field;
       subpart_field_buffers[i++] = field_buf;
     }
-    subpart_charset_field_array[i] = nullptr;
+    subpart_charset_field_array[i] = NULL;
   }
   return false;
 error:
@@ -1901,13 +1916,15 @@ void partition_info::report_part_expr_error(bool use_subpart_expr) {
     @retval false No field is within any partitioning expression.
 */
 
-bool partition_info::is_fields_in_part_expr(
-    const mem_root_deque<Item *> &fields) {
+bool partition_info::is_fields_in_part_expr(List<Item> &fields) {
+  List_iterator<Item> it(fields);
+  Item *item;
+  Item_field *field;
   DBUG_TRACE;
-  for (Item *item : fields) {
-    Item_field *field = item->field_for_view_update();
-    assert(field->field->table == table);
-    if (bitmap_is_set(&full_part_field_set, field->field->field_index()))
+  while ((item = it++)) {
+    field = item->field_for_view_update();
+    DBUG_ASSERT(field->field->table == table);
+    if (bitmap_is_set(&full_part_field_set, field->field->field_index))
       return true;
   }
   return false;
@@ -1917,10 +1934,9 @@ bool partition_info::is_fields_in_part_expr(
   Check if all partitioning fields are included.
 */
 
-bool partition_info::is_full_part_expr_in_fields(
-    const mem_root_deque<Item *> &fields) {
+bool partition_info::is_full_part_expr_in_fields(List<Item> &fields) {
   Field **part_field = full_part_field_array;
-  assert(*part_field);
+  DBUG_ASSERT(*part_field);
   DBUG_TRACE;
   /*
     It is very seldom many fields in full_part_field_array, so it is OK
@@ -1928,11 +1944,14 @@ bool partition_info::is_full_part_expr_in_fields(
     to compare with.
   */
   do {
+    List_iterator<Item> it(fields);
+    Item *item;
+    Item_field *field;
     bool found = false;
 
-    for (Item *item : fields) {
-      Item_field *field = item->field_for_view_update();
-      assert(field->field->table == table);
+    while ((item = it++)) {
+      field = item->field_for_view_update();
+      DBUG_ASSERT(field->field->table == table);
       if (*part_field == field->field) {
         found = true;
         break;
@@ -1998,7 +2017,7 @@ part_column_list_val *Parser_partition_info::add_column_value() {
     if (!reorganize_into_single_field_col_val() && !init_column_part()) {
       return add_column_value();
     }
-    return nullptr;
+    return NULL;
   }
   if (part_info->column_list) {
     my_error(ER_PARTITION_COLUMN_LIST_ERROR, MYF(0));
@@ -2008,7 +2027,7 @@ part_column_list_val *Parser_partition_info::add_column_value() {
     else
       my_error(ER_TOO_MANY_VALUES_ERROR, MYF(0), "LIST");
   }
-  return nullptr;
+  return NULL;
 }
 
 /**
@@ -2038,7 +2057,7 @@ void Parser_partition_info::init_col_val(part_column_list_val *col_val,
       curr_list_val->unsigned_flag = false;
     if (!curr_list_val->unsigned_flag) curr_part_elem->signed_flag = true;
   }
-  col_val->part_info = nullptr;
+  col_val->part_info = NULL;
 }
 
 /**
@@ -2056,7 +2075,7 @@ void Parser_partition_info::init_col_val(part_column_list_val *col_val,
 
 bool Parser_partition_info::add_column_list_value(THD *thd, Item *item) {
   part_column_list_val *col_val;
-  Name_resolution_context *context = &thd->lex->current_query_block()->context;
+  Name_resolution_context *context = &thd->lex->current_select()->context;
   TABLE_LIST *save_list = context->table_list;
   const char *save_where = thd->where;
   DBUG_TRACE;
@@ -2068,18 +2087,18 @@ bool Parser_partition_info::add_column_list_value(THD *thd, Item *item) {
     }
   }
 
-  context->table_list = nullptr;
+  context->table_list = 0;
   if (part_info->column_list)
     thd->where = "field list";
   else
     thd->where = "partition function";
 
   if (item->walk(&Item::check_partition_func_processor, enum_walk::POSTFIX,
-                 nullptr)) {
+                 NULL)) {
     my_error(ER_PARTITION_FUNCTION_IS_NOT_ALLOWED, MYF(0));
     return true;
   }
-  if (item->fix_fields(thd, (Item **)nullptr) ||
+  if (item->fix_fields(thd, (Item **)0) ||
       ((context->table_list = save_list), false) || (!item->const_item())) {
     context->table_list = save_list;
     thd->where = save_where;
@@ -2162,8 +2181,9 @@ bool Parser_partition_info::reorganize_into_single_field_col_val() {
   uint num_values = part_info->num_columns;
   uint i;
   DBUG_TRACE;
-  assert(part_info->part_type == partition_type::LIST);
-  assert(!part_info->num_columns || part_info->num_columns == val->added_items);
+  DBUG_ASSERT(part_info->part_type == partition_type::LIST);
+  DBUG_ASSERT(!part_info->num_columns ||
+              part_info->num_columns == val->added_items);
 
   if (!num_values) num_values = val->added_items;
   part_info->num_columns = 1;
@@ -2216,7 +2236,7 @@ bool partition_info::fix_partition_values(part_elem_value *val,
   }
   if (col_val->max_value) {
     /* The parser ensures we're not LIST partitioned here */
-    assert(part_type == partition_type::RANGE);
+    DBUG_ASSERT(part_type == partition_type::RANGE);
     if (defined_max_value) {
       my_error(ER_PARTITION_MAXVALUE_ERROR, MYF(0));
       return true;
@@ -2270,7 +2290,7 @@ Item *partition_info::get_column_item(Item *item, Field *field) {
       item->collation.collation != field->charset()) {
     if (!(item = convert_charset_partition_constant(item, field->charset()))) {
       my_error(ER_PARTITION_FUNCTION_IS_NOT_ALLOWED, MYF(0));
-      return nullptr;
+      return NULL;
     }
   }
   return item;
@@ -2307,9 +2327,9 @@ bool partition_info::fix_column_value_functions(THD *thd, part_elem_value *val,
     col_val->part_info = this;
     col_val->partition_id = part_id;
     if (col_val->max_value)
-      col_val->column_value.field_image = nullptr;
+      col_val->column_value.field_image = NULL;
     else {
-      col_val->column_value.field_image = nullptr;
+      col_val->column_value.field_image = NULL;
       if (!col_val->null_value) {
         uchar *val_ptr;
         uint len = field->pack_length();
@@ -2335,7 +2355,7 @@ bool partition_info::fix_column_value_functions(THD *thd, part_elem_value *val,
           goto end;
         }
         col_val->column_value.field_image = val_ptr;
-        memcpy(val_ptr, field->field_ptr(), len);
+        memcpy(val_ptr, field->ptr, len);
       }
     }
     col_val->fixed = 2;
@@ -2426,7 +2446,7 @@ bool partition_info::fix_parser_data(THD *thd) {
     part_elem = it++;
     List_iterator<part_elem_value> list_val_it(part_elem->list_val_list);
     num_elements = part_elem->list_val_list.elements;
-    assert(part_type == partition_type::RANGE ? num_elements == 1U : true);
+    DBUG_ASSERT(part_type == partition_type::RANGE ? num_elements == 1U : true);
     for (j = 0; j < num_elements; j++) {
       part_elem_value *val = list_val_it++;
       if (column_list) {
@@ -2496,7 +2516,7 @@ static bool strcmp_null(const char *a, const char *b) {
 bool partition_info::has_same_partitioning(partition_info *new_part_info) {
   DBUG_TRACE;
 
-  assert(part_field_array && part_field_array[0]);
+  DBUG_ASSERT(part_field_array && part_field_array[0]);
 
   /*
     Only consider pre 5.5.3 .frm's to have same partitioning as
@@ -2532,15 +2552,13 @@ bool partition_info::has_same_partitioning(partition_info *new_part_info) {
   }
 
   /* Check that it will use the same fields in KEY (fields) list. */
-  {
-    List_iterator<char> old_field_name_it(part_field_list);
-    List_iterator<char> new_field_name_it(new_part_info->part_field_list);
-    char *old_name, *new_name;
-    while ((old_name = old_field_name_it++)) {
-      new_name = new_field_name_it++;
-      if (!new_name || my_strcasecmp(system_charset_info, new_name, old_name))
-        return false;
-    }
+  List_iterator<char> old_field_name_it(part_field_list);
+  List_iterator<char> new_field_name_it(new_part_info->part_field_list);
+  char *old_name, *new_name;
+  while ((old_name = old_field_name_it++)) {
+    new_name = new_field_name_it++;
+    if (!new_name || my_strcasecmp(system_charset_info, new_name, old_name))
+      return false;
   }
 
   if (is_sub_partitioned()) {
@@ -2610,7 +2628,7 @@ bool partition_info::has_same_partitioning(partition_info *new_part_info) {
           }
           if (new_list_vals++) return false;
         } else {
-          assert(part_type == partition_type::RANGE);
+          DBUG_ASSERT(part_type == partition_type::RANGE);
           if (new_part_elem->range_value != part_elem->range_value)
             return false;
         }
@@ -2682,7 +2700,7 @@ static bool has_same_column_order(List<Create_field> *create_list,
                                   Field **field_array) {
   Field **f_ptr;
   List_iterator_fast<Create_field> new_field_it;
-  Create_field *new_field = nullptr;
+  Create_field *new_field = NULL;
   new_field_it.init(*create_list);
 
   for (f_ptr = field_array; *f_ptr; f_ptr++) {
@@ -2719,7 +2737,7 @@ bool partition_info::same_key_column_order(List<Create_field> *create_list) {
   return true;
 }
 
-void partition_info::print_debug(const char *str [[maybe_unused]],
+void partition_info::print_debug(const char *str MY_ATTRIBUTE((unused)),
                                  uint *value) {
   DBUG_TRACE;
   if (value)
@@ -2731,14 +2749,13 @@ void partition_info::print_debug(const char *str [[maybe_unused]],
 bool has_external_data_or_index_dir(partition_info &pi) {
   List_iterator<partition_element> part_it(pi.partitions);
   for (partition_element *part = part_it++; part; part = part_it++) {
-    if (part->data_file_name != nullptr || part->index_file_name != nullptr) {
+    if (part->data_file_name != NULL || part->index_file_name != NULL) {
       return true;
     }
     List_iterator<partition_element> subpart_it(part->subpartitions);
     for (const partition_element *subpart = subpart_it++; subpart;
          subpart = subpart_it++) {
-      if (subpart->data_file_name != nullptr ||
-          subpart->index_file_name != nullptr) {
+      if (subpart->data_file_name != NULL || subpart->index_file_name != NULL) {
         return true;
       }
     }
@@ -2814,7 +2831,7 @@ bool validate_partition_tablespace_name_lengths(partition_info *part_info) {
 
 bool validate_partition_tablespace_names(partition_info *part_info,
                                          const handlerton *default_engine) {
-  assert(default_engine);
+  DBUG_ASSERT(default_engine);
 
   // Do nothing if the table is not partitioned.
   if (!part_info) return false;
@@ -2862,6 +2879,6 @@ bool partition_info::init_partition_bitmap(MY_BITMAP *bitmap,
     mem_alloc_error(bitmap_bytes);
     return true;
   }
-  bitmap_init(bitmap, bitmap_buf, bitmap_bits);
+  bitmap_init(bitmap, bitmap_buf, bitmap_bits, false);
   return false;
 }
