@@ -2,19 +2,18 @@
 #define PARTITION_HANDLER_INCLUDED
 
 /*
-   Copyright (c) 2005, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +25,6 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include <assert.h>
 #include <string.h>
 #include <sys/types.h>
 #include <memory>
@@ -36,7 +34,7 @@
 #include "my_base.h"  // ha_rows.
 #include "my_bitmap.h"
 #include "my_compiler.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
 #include "mysql/psi/mysql_mutex.h"
@@ -104,7 +102,7 @@ void partitioning_init();
 class Partition_share : public Handler_share {
  public:
   Partition_share();
-  ~Partition_share() override;
+  ~Partition_share();
 
   /** Set if auto increment is used an initialized. */
   bool auto_inc_initialized;
@@ -143,12 +141,12 @@ class Partition_share : public Handler_share {
 
   /** lock mutex protecting auto increment value next_auto_inc_val. */
   inline void lock_auto_inc() {
-    assert(auto_inc_mutex);
+    DBUG_ASSERT(auto_inc_mutex);
     mysql_mutex_lock(auto_inc_mutex);
   }
   /** unlock mutex protecting auto increment value next_auto_inc_val. */
   inline void unlock_auto_inc() {
-    assert(auto_inc_mutex);
+    DBUG_ASSERT(auto_inc_mutex);
     mysql_mutex_unlock(auto_inc_mutex);
   }
   /**
@@ -193,8 +191,8 @@ class Partition_share : public Handler_share {
 */
 class Partition_handler {
  public:
-  Partition_handler() = default;
-  virtual ~Partition_handler() = default;
+  Partition_handler() {}
+  virtual ~Partition_handler() {}
 
   /**
     Get dynamic table information from partition.
@@ -217,8 +215,8 @@ class Partition_handler {
     @param info  Create info.
     @return Number of default partitions.
   */
-  virtual int get_default_num_partitions(HA_CREATE_INFO *info
-                                         [[maybe_unused]]) {
+  virtual int get_default_num_partitions(
+      HA_CREATE_INFO *info MY_ATTRIBUTE((unused))) {
     return 1;
   }
   /**
@@ -229,8 +227,8 @@ class Partition_handler {
 
     @param[in,out] part_info  Partition object to setup.
   */
-  virtual void set_auto_partitions(partition_info *part_info [[maybe_unused]]) {
-  }
+  virtual void set_auto_partitions(
+      partition_info *part_info MY_ATTRIBUTE((unused))) {}
   /**
     Get number of partitions for table in SE
 
@@ -241,7 +239,7 @@ class Partition_handler {
     @retval false for success
     @retval true for failure, for example table didn't exist in engine
   */
-  virtual bool get_num_parts(const char *name [[maybe_unused]],
+  virtual bool get_num_parts(const char *name MY_ATTRIBUTE((unused)),
                              uint *num_parts) {
     *num_parts = 0;
     return false;
@@ -276,6 +274,10 @@ class Partition_handler {
   /**
     Exchange partition.
 
+    @param[in]      part_table_path   Path to partition in partitioned table
+                                      to be exchanged.
+    @param[in]      swap_table_path   Path to non-partitioned table to be
+                                      exchanged with partition.
     @param[in]      part_id           Id of partition to be exchanged.
     @param[in,out]  part_table_def    dd::Table object for partitioned table.
     @param[in,out]  swap_table_def    dd::Table object for non-partitioned
@@ -291,8 +293,9 @@ class Partition_handler {
       @retval    0  Success.
       @retval != 0  Error code.
   */
-  int exchange_partition(uint part_id, dd::Table *part_table_def,
-                         dd::Table *swap_table_def);
+  int exchange_partition(const char *part_table_path,
+                         const char *swap_table_path, uint part_id,
+                         dd::Table *part_table_def, dd::Table *swap_table_def);
 
   /**
     Alter flags.
@@ -303,7 +306,9 @@ class Partition_handler {
 
     @return Supported alter table flags.
   */
-  virtual uint alter_flags(uint flags [[maybe_unused]]) const { return 0; }
+  virtual uint alter_flags(uint flags MY_ATTRIBUTE((unused))) const {
+    return 0;
+  }
 
   /**
     Get partition row type from SE
@@ -334,10 +339,12 @@ class Partition_handler {
 
     @sa Partition_handler::exchange_partition().
   */
-  virtual int exchange_partition_low(uint part_id [[maybe_unused]],
-                                     dd::Table *part_table_def [[maybe_unused]],
-                                     dd::Table *swap_table_def
-                                     [[maybe_unused]]) {
+  virtual int exchange_partition_low(
+      const char *part_table_path MY_ATTRIBUTE((unused)),
+      const char *swap_table_path MY_ATTRIBUTE((unused)),
+      uint part_id MY_ATTRIBUTE((unused)),
+      dd::Table *part_table_def MY_ATTRIBUTE((unused)),
+      dd::Table *swap_table_def MY_ATTRIBUTE((unused))) {
     return HA_ERR_WRONG_COMMAND;
   }
 
@@ -350,7 +357,7 @@ class Partition_handler {
 
     @return handler or NULL if not supported.
   */
-  virtual handler *get_handler() { return nullptr; }
+  virtual handler *get_handler() { return NULL; }
 };
 
 /// Maps compare function to strict weak ordering required by Priority_queue.
@@ -414,9 +421,9 @@ class Partition_helper {
       @retval false success.
       @retval true  failure.
   */
-  bool init_partitioning(MEM_ROOT *mem_root [[maybe_unused]]) {
-#ifndef NDEBUG
-    m_key_not_found_partitions.bitmap = nullptr;
+  bool init_partitioning(MEM_ROOT *mem_root MY_ATTRIBUTE((unused))) {
+#ifndef DBUG_OFF
+    m_key_not_found_partitions.bitmap = NULL;
 #endif
     return false;
   }
@@ -429,9 +436,12 @@ class Partition_helper {
 
   /**
     Insert a row to the partitioned table.
-      @returns Operation status.
-      @returns    0 Success
-      @returns != 0 Error code
+
+    @param buf The row in MySQL Row Format.
+
+    @return Operation status.
+      @retval    0 Success
+      @retval != 0 Error code
   */
   int ph_write_row(uchar *buf);
   /**
@@ -449,9 +459,12 @@ class Partition_helper {
     new_data is always record[0]
     old_data is always record[1]
 
+    @param old_data  The old record in MySQL Row Format.
+    @param new_data  The new record in MySQL Row Format.
+
     @return Operation status.
-      @returns    0 Success
-      @returns != 0 Error code
+      @retval    0 Success
+      @retval != 0 Error code
   */
   int ph_update_row(const uchar *old_data, uchar *new_data);
   /**
@@ -618,8 +631,8 @@ class Partition_helper {
     Set m_part_share, Allocate internal bitmaps etc. used by open tables.
 
     @return Operation status.
-      @returns false success.
-      @returns true  failure.
+      @retval false success.
+      @retval true  failure.
   */
   bool open_partitioning(Partition_share *part_share);
   /**
@@ -648,25 +661,17 @@ class Partition_helper {
       m_auto_increment_lock = false;
     }
   }
-
   /**
-    Get a range of auto increment values.
+    Get auto increment.
 
-    Can only be used if the auto increment field is the first field in an index.
+    Only to be used for auto increment values that are the first field in
+    an unique index.
 
-    This method is called by update_auto_increment which in turn is called
-    by the individual handlers as part of write_row. We use the
-    part_share->next_auto_inc_val, or search all
-    partitions for the highest auto_increment_value if not initialized or
-    if auto_increment field is a secondary part of a key, we must search
-    every partition when holding a mutex to be sure of correctness.
-
-    @param[in]   increment           Increment value.
-    @param[in]   nb_desired_values   Number of desired values.
-    @param[out]  first_value         First auto inc value reserved
-                                        or MAX if failure.
-    @param[out]  nb_reserved_values  Number of values reserved.
-   */
+    @param[in]  increment           Increment between generated numbers.
+    @param[in]  nb_desired_values   Number of values requested.
+    @param[out] first_value         First reserved value (ULLONG_MAX on error).
+    @param[out] nb_reserved_values  Number of values reserved.
+  */
   void get_auto_increment_first_field(ulonglong increment,
                                       ulonglong nb_desired_values,
                                       ulonglong *first_value,
@@ -826,7 +831,7 @@ class Partition_helper {
   /** Release auto_increment in all underlying partitions. */
   virtual void release_auto_increment_all_parts() {}
   /** Save or persist the current max auto increment. */
-  virtual void save_auto_increment(ulonglong nr [[maybe_unused]]) {}
+  virtual void save_auto_increment(ulonglong nr MY_ATTRIBUTE((unused))) {}
   /**
     Per partition equivalent of rnd_* and index_* functions.
 
@@ -877,8 +882,8 @@ class Partition_helper {
       @retval    0   Success.
       @retval != 0   Error code.
   */
-  virtual int init_record_priority_queue_for_parts(uint used_parts
-                                                   [[maybe_unused]]) {
+  virtual int init_record_priority_queue_for_parts(
+      uint used_parts MY_ATTRIBUTE((unused))) {
     return 0;
   }
   /**
@@ -891,8 +896,9 @@ class Partition_helper {
 
     @param part_id  Partition to checksum.
   */
-  virtual ha_checksum checksum_in_part(uint part_id [[maybe_unused]]) const {
-    assert(0);
+  virtual ha_checksum checksum_in_part(
+      uint part_id MY_ATTRIBUTE((unused))) const {
+    DBUG_ASSERT(0);
     return 0;
   }
   /**
@@ -933,9 +939,17 @@ class Partition_helper {
     If we need to scan only one partition, set m_ordered_scan_ongoing=false
     as we will not need to do merge ordering.
 
+    @param buf            Buffer to later return record in (this function
+                          needs it to calculate partitioning function values)
+
+    @param idx_read_flag  True <=> m_start_key has range start endpoint which
+                          probably can be used to determine the set of
+                          partitions to scan.
+                          False <=> there is no start endpoint.
+
     @return Operation status.
-      @returns   0  Success
-      @returns !=0  Error code
+      @retval   0  Success
+      @retval !=0  Error code
   */
   int partition_scan_set_up(uchar *buf, bool idx_read_flag);
   /**
@@ -949,10 +963,13 @@ class Partition_helper {
     scan is performed on only one partition and thus it isn't necessary to
     perform any sort.
 
+    @param[out] buf        Read row in MySQL Row Format.
+    @param[in]  is_next_same  Called from index_next_same.
+
     @return Operation status.
-      @returns HA_ERR_END_OF_FILE  End of scan
-      @returns 0                   Success
-      @returns other               Error code
+      @retval HA_ERR_END_OF_FILE  End of scan
+      @retval 0                   Success
+      @retval other               Error code
   */
   int handle_unordered_next(uchar *buf, bool is_next_same);
   /**
@@ -972,12 +989,13 @@ class Partition_helper {
   /**
     Common routine to start index scan with ordered results.
 
+    @param[out] buf  Read row in MySQL Row Format
 
-      @returns Operation status
-      @returns HA_ERR_END_OF_FILE    End of scan
-      @returns HA_ERR_KEY_NOT_FOUND  End of scan
-      @returns 0                     Success
-      @returns other                 Error code
+    @return Operation status
+      @retval HA_ERR_END_OF_FILE    End of scan
+      @retval HA_ERR_KEY_NOT_FOUND  End of scan
+      @retval 0                     Success
+      @retval other                 Error code
   */
   int handle_ordered_index_scan(uchar *buf);
   /**
@@ -987,10 +1005,10 @@ class Partition_helper {
     ha_index_read_map was done, those partitions must be included in the
     following index_next/prev call.
 
-      @returns Operation status
-      @returns HA_ERR_END_OF_FILE    End of scan
-      @returns 0                     Success
-      @returns other                 Error code
+    @return Operation status
+      @retval HA_ERR_END_OF_FILE    End of scan
+      @retval 0                     Success
+      @retval other                 Error code
   */
   int handle_ordered_index_scan_key_not_found();
   /**
@@ -1034,18 +1052,7 @@ class Partition_helper {
       @retval HA_ERR_KEY_NOT_FOUND Record not found, but index cursor
     positioned.
       @retval other                Error code.
-
-    @details
-      Start scanning the range (when invoked from read_range_first()) or doing
-      an index lookup (when invoked from index_read_XXX):
-       - If possible, perform partition selection
-       - Find the set of partitions we're going to use
-       - Depending on whether we need ordering:
-          NO:  Get the first record from first used partition (see
-               handle_unordered_scan_next_partition)
-          YES: Fill the priority queue and get the record that is the first in
-               the ordering
-   */
+  */
   int common_index_read(uchar *buf, bool have_start_key);
   /**
     Common routine for index_first/index_last.
