@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -49,12 +49,17 @@ struct st_server_ssl_variables {
   char *ssl_ca;
   char *ssl_capath;
   char *tls_version;
+  char *tls_ciphersuites;
   char *ssl_cert;
   char *ssl_cipher;
   char *ssl_key;
   char *ssl_crl;
   char *ssl_crlpath;
   unsigned int ssl_fips_mode;
+
+  void init();
+
+  void deinit();
 };
 
 /**
@@ -170,7 +175,7 @@ typedef struct Trans_param {
   Trans_context_info trans_ctx_info;
 
   /// pointer to the status var original_commit_timestamp
-  uint64 *original_commit_timestamp;
+  unsigned long long *original_commit_timestamp;
 
   /** Replication channel info associated to this transaction/THD */
   enum_rpl_channel_type rpl_channel_type;
@@ -363,6 +368,17 @@ typedef int (*before_server_shutdown_t)(Server_state_param *param);
 */
 typedef int (*after_server_shutdown_t)(Server_state_param *param);
 
+/**
+  This is called just after an upgrade from MySQL 5.7 populates the data
+  dictionary for the first time.
+
+  @param[in]  param Observer common parameter
+
+  @retval 0 Success
+  @retval >0 Failure
+*/
+typedef int (*after_dd_upgrade_t)(Server_state_param *param);
+
 /**
   Observer server state
  */
@@ -375,6 +391,7 @@ typedef struct Server_state_observer {
   after_recovery_t after_recovery;
   before_server_shutdown_t before_server_shutdown;
   after_server_shutdown_t after_server_shutdown;
+  after_dd_upgrade_t after_dd_upgrade_from_57;
 } Server_state_observer;
 
 /**
