@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -38,7 +38,6 @@
 #include "my_loglevel.h"
 #include "my_pointer_arithmetic.h"
 #include "my_sys.h"
-#include "my_systime.h"
 #include "my_thread.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/psi_stage_bits.h"
@@ -76,6 +75,20 @@ const ushort Binlog_sender::PACKET_SHRINK_COUNTER_THRESHOLD = 100;
 const float Binlog_sender::PACKET_GROW_FACTOR = 2.0;
 const float Binlog_sender::PACKET_SHRINK_FACTOR = 0.5;
 
+/**
+  Simple function to help readability w.r.t. chrono operations.
+
+  This function SHALL return a nanoseconds duration representing
+  the current time. It is just a convenience function to interface
+  with std::chrono::high_resolution_clock::now().
+
+  @return a std::chrono::nanoseconds duration since the epoch.
+ */
+static std::chrono::nanoseconds now_in_nanosecs() {
+  return std::chrono::duration_cast<std::chrono::nanoseconds>(
+      std::chrono::high_resolution_clock::now().time_since_epoch());
+}
+
 /**
   Binlog_sender reads events one by one. It uses the preallocated memory
   (A String object) to store all event_data instead of allocating memory when
@@ -91,8 +104,8 @@ class Binlog_sender::Event_allocator {
     if (m_sender->grow_packet(size)) return nullptr;
 
     m_sender->m_packet.length(event_offset + size);
-    return reinterpret_cast<unsigned char *>(
-        const_cast<char *>(m_sender->m_packet.ptr() + event_offset));
+    return pointer_cast<unsigned char *>(m_sender->m_packet.ptr() +
+                                         event_offset);
   }
 
   void deallocate(unsigned char *ptr MY_ATTRIBUTE((unused))) {}
@@ -109,13 +122,13 @@ Binlog_sender::Binlog_sender(THD *thd, const char *start_file,
       m_start_file(start_file),
       m_start_pos(start_pos),
       m_exclude_gtid(exclude_gtids),
-      m_using_gtid_protocol(exclude_gtids != NULL),
-      m_check_previous_gtid_event(exclude_gtids != NULL),
-      m_gtid_clear_fd_created_flag(exclude_gtids == NULL),
+      m_using_gtid_protocol(exclude_gtids != nullptr),
+      m_check_previous_gtid_event(exclude_gtids != nullptr),
+      m_gtid_clear_fd_created_flag(exclude_gtids == nullptr),
       m_diag_area(false),
-      m_errmsg(NULL),
+      m_errmsg(nullptr),
       m_errno(0),
-      m_last_file(NULL),
+      m_last_file(nullptr),
       m_last_pos(0),
       m_half_buffer_size_req_counter(0),
       m_new_shrink_size(PACKET_MIN_SIZE),
@@ -124,12 +137,12 @@ Binlog_sender::Binlog_sender(THD *thd, const char *start_file,
       m_transmit_started(false) {}
 
 void Binlog_sender::init() {
-  DBUG_ENTER("Binlog_sender::init");
+  DBUG_TRACE;
   THD *thd = m_thd;
 
   thd->push_diagnostics_area(&m_diag_area);
   init_heartbeat_period();
-  m_last_event_sent_ts = time(0);
+  m_last_event_sent_ts = now_in_nanosecs();
 
   mysql_mutex_lock(&thd->LOCK_thd_data);
   thd->current_linfo = &m_linfo;
@@ -143,12 +156,12 @@ void Binlog_sender::init() {
 
   if (!mysql_bin_log.is_open()) {
     set_fatal_error("Binary log is not open");
-    DBUG_VOID_RETURN;
+    return;
   }
 
   if (DBUG_EVALUATE_IF("simulate_no_server_id", true, server_id == 0)) {
     set_fatal_error("Misconfigured master - master server_id is 0");
-    DBUG_VOID_RETURN;
+    return;
   }
 
   if (m_using_gtid_protocol) {
@@ -161,11 +174,11 @@ void Binlog_sender::init() {
               "instead of ON.",
               get_gtid_mode_string(gtid_mode));
       set_fatal_error(buf);
-      DBUG_VOID_RETURN;
+      return;
     }
   }
 
-  if (check_start_file()) DBUG_VOID_RETURN;
+  if (check_start_file()) return;
 
   LogErr(INFORMATION_LEVEL, ER_RPL_BINLOG_STARTING_DUMP, thd->thread_id(),
          thd->server_id, m_start_file, m_start_pos);
@@ -174,7 +187,7 @@ void Binlog_sender::init() {
           binlog_transmit, transmit_start,
           (thd, m_flag, m_start_file, m_start_pos, &m_observe_transmission))) {
     set_unknown_error("Failed to run hook 'transmit_start'");
-    DBUG_VOID_RETURN;
+    return;
   }
   m_transmit_started = true;
 
@@ -224,11 +237,10 @@ void Binlog_sender::init() {
         "--sporadic-binlog-dump-fail");
   m_event_count = 0;
 #endif
-  DBUG_VOID_RETURN;
 }
 
 void Binlog_sender::cleanup() {
-  DBUG_ENTER("Binlog_sender::cleanup");
+  DBUG_TRACE;
 
   THD *thd = m_thd;
 
@@ -236,7 +248,7 @@ void Binlog_sender::cleanup() {
     (void)RUN_HOOK(binlog_transmit, transmit_stop, (thd, m_flag));
 
   mysql_mutex_lock(&thd->LOCK_thd_data);
-  thd->current_linfo = NULL;
+  thd->current_linfo = nullptr;
   mysql_mutex_unlock(&thd->LOCK_thd_data);
 
   thd->variables.max_allowed_packet =
@@ -247,12 +259,10 @@ void Binlog_sender::cleanup() {
     my_message(m_errno, m_errmsg, MYF(0));
   else
     my_eof(thd);
-
-  DBUG_VOID_RETURN;
 }
 
 void Binlog_sender::run() {
-  DBUG_ENTER("Binlog_sender::run");
+  DBUG_TRACE;
   init();
 
   unsigned int max_event_size =
@@ -359,7 +369,6 @@ void Binlog_sender::run() {
   }
 
   cleanup();
-  DBUG_VOID_RETURN;
 }
 
 int Binlog_sender::send_binlog(File_reader *reader, my_off_t start_pos) {
@@ -403,7 +412,7 @@ int Binlog_sender::send_binlog(File_reader *reader, my_off_t start_pos) {
 }
 
 int Binlog_sender::get_binlog_end_pos(File_reader *reader, my_off_t *end_pos) {
-  DBUG_ENTER("Binlog_sender::get_binlog_end_pos()");
+  DBUG_TRACE;
   my_off_t read_pos = reader->position();
 
   do {
@@ -417,26 +426,26 @@ int Binlog_sender::get_binlog_end_pos(File_reader *reader, my_off_t *end_pos) {
     /* If this is a cold binlog file, we are done getting the end pos */
     if (unlikely(!mysql_bin_log.is_active(m_linfo.log_file_name))) {
       *end_pos = 0;
-      DBUG_RETURN(0);
+      return 0;
     }
 
     DBUG_PRINT("info", ("Reading file %s, seek pos %llu, end_pos is %llu",
                         m_linfo.log_file_name, read_pos, *end_pos));
     DBUG_PRINT("info", ("Active file is %s", mysql_bin_log.get_log_fname()));
 
-    if (read_pos < *end_pos) DBUG_RETURN(0);
+    if (read_pos < *end_pos) return 0;
 
     /* Some data may be in net buffer, it should be flushed before waiting */
-    if (!m_wait_new_events || flush_net()) DBUG_RETURN(1);
+    if (!m_wait_new_events || flush_net()) return 1;
 
-    if (unlikely(wait_new_events(read_pos))) DBUG_RETURN(1);
+    if (unlikely(wait_new_events(read_pos))) return 1;
   } while (unlikely(!m_thd->killed));
 
-  DBUG_RETURN(1);
+  return 1;
 }
 
 int Binlog_sender::send_events(File_reader *reader, my_off_t end_pos) {
-  DBUG_ENTER("Binlog_sender::send_events");
+  DBUG_TRACE;
 
   THD *thd = m_thd;
   const char *log_file = m_linfo.log_file_name;
@@ -448,12 +457,12 @@ int Binlog_sender::send_events(File_reader *reader, my_off_t end_pos) {
     uchar *event_ptr = nullptr;
     uint32 event_len = 0;
 
-    if (unlikely(thd->killed)) DBUG_RETURN(1);
+    if (unlikely(thd->killed)) return 1;
 
-    if (unlikely(read_event(reader, &event_ptr, &event_len))) DBUG_RETURN(1);
+    if (unlikely(read_event(reader, &event_ptr, &event_len))) return 1;
 
     if (event_ptr == nullptr) {
-      if (end_pos == 0) DBUG_RETURN(0);  // Arrive the end of inactive file
+      if (end_pos == 0) return 0;  // Arrive the end of inactive file
 
       /*
         It is reading events before end_pos of active binlog file. In theory,
@@ -462,12 +471,11 @@ int Binlog_sender::send_events(File_reader *reader, my_off_t end_pos) {
         file is reopened and truncated to 0 after RESET MASTER.
       */
       set_fatal_error(log_read_error_msg(Binlog_read_error::SYSTEM_IO));
-      DBUG_RETURN(1);
+      return 1;
     }
 
     Log_event_type event_type = (Log_event_type)event_ptr[EVENT_TYPE_OFFSET];
-    if (unlikely(check_event_type(event_type, log_file, log_pos)))
-      DBUG_RETURN(1);
+    if (unlikely(check_event_type(event_type, log_file, log_pos))) return 1;
 
     DBUG_EXECUTE_IF("dump_thread_wait_before_send_xid", {
       if (event_type == binary_log::XID_EVENT) {
@@ -482,7 +490,7 @@ int Binlog_sender::send_events(File_reader *reader, my_off_t end_pos) {
 
     log_pos = reader->position();
 
-    if (before_send_hook(log_file, log_pos)) DBUG_RETURN(1);
+    if (before_send_hook(log_file, log_pos)) return 1;
     /*
       TODO: Set m_exclude_gtid to NULL if all gtids in m_exclude_gtid has
       be skipped. and maybe removing the gtid from m_exclude_gtid will make
@@ -496,12 +504,12 @@ int Binlog_sender::send_events(File_reader *reader, my_off_t end_pos) {
        */
       DBUG_EXECUTE_IF("inject_2sec_sleep_when_skipping_an_event",
                       { my_sleep(2000000); });
-      time_t now = time(0);
+      auto now = now_in_nanosecs();
       DBUG_ASSERT(now >= m_last_event_sent_ts);
-      bool time_for_hb_event = ((ulonglong)(now - m_last_event_sent_ts) >=
-                                (ulonglong)(m_heartbeat_period / 1000000000UL));
-      if (time_for_hb_event) {
-        if (unlikely(send_heartbeat_event(log_pos))) DBUG_RETURN(1);
+
+      // if enough time has elapsed so that we should send another heartbeat
+      if ((now - m_last_event_sent_ts) >= m_heartbeat_period) {
+        if (unlikely(send_heartbeat_event(log_pos))) return 1;
         exclude_group_end_pos = 0;
       } else {
         exclude_group_end_pos = log_pos;
@@ -520,8 +528,7 @@ int Binlog_sender::send_events(File_reader *reader, my_off_t end_pos) {
         tmp.copy(m_packet);
         tmp.length(m_packet.length());
 
-        if (unlikely(send_heartbeat_event(exclude_group_end_pos)))
-          DBUG_RETURN(1);
+        if (unlikely(send_heartbeat_event(exclude_group_end_pos))) return 1;
         exclude_group_end_pos = 0;
 
         /* Restore the copy back. */
@@ -529,11 +536,11 @@ int Binlog_sender::send_events(File_reader *reader, my_off_t end_pos) {
         m_packet.length(tmp.length());
       }
 
-      if (unlikely(send_packet())) DBUG_RETURN(1);
+      if (unlikely(send_packet())) return 1;
     }
 
     if (unlikely(after_send_hook(log_file, in_exclude_group ? log_pos : 0)))
-      DBUG_RETURN(1);
+      return 1;
   }
 
   /*
@@ -542,9 +549,9 @@ int Binlog_sender::send_events(File_reader *reader, my_off_t end_pos) {
     master_log_pos correctly.
   */
   if (unlikely(in_exclude_group)) {
-    if (send_heartbeat_event(log_pos)) DBUG_RETURN(1);
+    if (send_heartbeat_event(log_pos)) return 1;
   }
-  DBUG_RETURN(0);
+  return 0;
 }
 
 bool Binlog_sender::check_event_type(Log_event_type type, const char *log_file,
@@ -564,9 +571,9 @@ bool Binlog_sender::check_event_type(Log_event_type type, const char *log_file,
       DBUG_EXECUTE_IF("skip_sender_anon_autoposition_error",
                       { return false; };);
       char buf[MYSQL_ERRMSG_SIZE];
-      sprintf(buf,
-              ER_THD(m_thd, ER_CANT_REPLICATE_ANONYMOUS_WITH_AUTO_POSITION),
-              log_file, log_pos);
+      snprintf(buf, MYSQL_ERRMSG_SIZE,
+               ER_THD(m_thd, ER_CANT_REPLICATE_ANONYMOUS_WITH_AUTO_POSITION),
+               log_file, log_pos);
       set_fatal_error(buf);
       return true;
     }
@@ -579,8 +586,9 @@ bool Binlog_sender::check_event_type(Log_event_type type, const char *log_file,
     */
     else if (get_gtid_mode_from_copy(GTID_MODE_LOCK_NONE) == GTID_MODE_ON) {
       char buf[MYSQL_ERRMSG_SIZE];
-      sprintf(buf, ER_THD(m_thd, ER_CANT_REPLICATE_ANONYMOUS_WITH_GTID_MODE_ON),
-              log_file, log_pos);
+      snprintf(buf, MYSQL_ERRMSG_SIZE,
+               ER_THD(m_thd, ER_CANT_REPLICATE_ANONYMOUS_WITH_GTID_MODE_ON),
+               log_file, log_pos);
       set_fatal_error(buf);
       return true;
     }
@@ -594,8 +602,9 @@ bool Binlog_sender::check_event_type(Log_event_type type, const char *log_file,
     */
     if (get_gtid_mode_from_copy(GTID_MODE_LOCK_NONE) == GTID_MODE_OFF) {
       char buf[MYSQL_ERRMSG_SIZE];
-      sprintf(buf, ER_THD(m_thd, ER_CANT_REPLICATE_GTID_WITH_GTID_MODE_OFF),
-              log_file, log_pos);
+      snprintf(buf, MYSQL_ERRMSG_SIZE,
+               ER_THD(m_thd, ER_CANT_REPLICATE_GTID_WITH_GTID_MODE_OFF),
+               log_file, log_pos);
       set_fatal_error(buf);
       return true;
     }
@@ -605,7 +614,7 @@ bool Binlog_sender::check_event_type(Log_event_type type, const char *log_file,
 
 inline bool Binlog_sender::skip_event(const uchar *event_ptr,
                                       bool in_exclude_group) {
-  DBUG_ENTER("Binlog_sender::skip_event");
+  DBUG_TRACE;
 
   uint8 event_type = (Log_event_type)event_ptr[LOG_EVENT_OFFSET];
   switch (event_type) {
@@ -616,12 +625,12 @@ inline bool Binlog_sender::skip_event(const uchar *event_ptr,
       Gtid gtid;
       gtid.sidno = gtid_ev.get_sidno(m_exclude_gtid->get_sid_map());
       gtid.gno = gtid_ev.get_gno();
-      DBUG_RETURN(m_exclude_gtid->contains_gtid(gtid));
+      return m_exclude_gtid->contains_gtid(gtid);
     }
     case binary_log::ROTATE_EVENT:
-      DBUG_RETURN(false);
+      return false;
   }
-  DBUG_RETURN(in_exclude_group);
+  return in_exclude_group;
 }
 
 int Binlog_sender::wait_new_events(my_off_t log_pos) {
@@ -643,7 +652,7 @@ int Binlog_sender::wait_new_events(my_off_t log_pos) {
                     mysql_bin_log.get_binlog_end_pos_lock(),
                     &stage_master_has_sent_all_binlog_to_slave, &old_stage);
 
-  if (m_heartbeat_period)
+  if (m_heartbeat_period.count() > 0)
     ret = wait_with_heartbeat(log_pos);
   else
     ret = wait_without_heartbeat();
@@ -661,7 +670,7 @@ inline int Binlog_sender::wait_with_heartbeat(my_off_t log_pos) {
   int ret;
 
   do {
-    set_timespec_nsec(&ts, m_heartbeat_period);
+    set_timespec_nsec(&ts, m_heartbeat_period.count());
     ret = mysql_bin_log.wait_for_update(&ts);
     if (!is_timeout(ret)) break;
 
@@ -681,28 +690,26 @@ inline int Binlog_sender::wait_with_heartbeat(my_off_t log_pos) {
 }
 
 inline int Binlog_sender::wait_without_heartbeat() {
-  return mysql_bin_log.wait_for_update(NULL);
+  return mysql_bin_log.wait_for_update(nullptr);
 }
 
 void Binlog_sender::init_heartbeat_period() {
   bool null_value;
-  LEX_STRING name = {C_STRING_WITH_LEN("master_heartbeat_period")};
+  LEX_CSTRING name = {STRING_WITH_LEN("master_heartbeat_period")};
 
   /* Protects m_thd->user_vars. */
   mysql_mutex_lock(&m_thd->LOCK_thd_data);
 
   const auto it = m_thd->user_vars.find(to_string(name));
-  if (it == m_thd->user_vars.end())
-    m_heartbeat_period = 0;
-  else
-    m_heartbeat_period = it->second->val_int(&null_value);
+  m_heartbeat_period = std::chrono::nanoseconds(
+      it == m_thd->user_vars.end() ? 0 : it->second->val_int(&null_value));
 
   mysql_mutex_unlock(&m_thd->LOCK_thd_data);
 }
 
 int Binlog_sender::check_start_file() {
   char index_entry_name[FN_REFLEN];
-  char *name_ptr = NULL;
+  char *name_ptr = nullptr;
   const char *errmsg;
 
   if (m_start_file[0] != '\0') {
@@ -779,8 +786,8 @@ int Binlog_sender::check_start_file() {
 
       String tmp_uuid;
       get_slave_uuid(m_thd, &tmp_uuid);
-      char *missing_gtids = NULL;
-      gtid_missing.to_string(&missing_gtids, false, NULL);
+      char *missing_gtids = nullptr;
+      gtid_missing.to_string(&missing_gtids, false, nullptr);
       LogErr(WARNING_LEVEL, ER_FOUND_MISSING_GTIDS, tmp_uuid.ptr(),
              missing_gtids);
       my_free(missing_gtids);
@@ -853,7 +860,7 @@ int Binlog_sender::check_start_file() {
 extern TYPELIB binlog_checksum_typelib;
 
 void Binlog_sender::init_checksum_alg() {
-  DBUG_ENTER("init_binlog_checksum");
+  DBUG_TRACE;
 
   m_slave_checksum_alg = binary_log::BINLOG_CHECKSUM_ALG_UNDEF;
 
@@ -863,7 +870,7 @@ void Binlog_sender::init_checksum_alg() {
   const auto it = m_thd->user_vars.find("master_binlog_checksum");
   if (it != m_thd->user_vars.end()) {
     m_slave_checksum_alg = static_cast<enum_binlog_checksum_alg>(
-        find_type((char *)it->second->ptr(), &binlog_checksum_typelib, 1) - 1);
+        find_type(it->second->ptr(), &binlog_checksum_typelib, 1) - 1);
     DBUG_ASSERT(m_slave_checksum_alg <
                 binary_log::BINLOG_CHECKSUM_ALG_ENUM_END);
   }
@@ -877,12 +884,11 @@ void Binlog_sender::init_checksum_alg() {
     case, m_slave_checksum_alg is set as the value of m_event_checksum_alg.
   */
   m_event_checksum_alg = m_slave_checksum_alg;
-  DBUG_VOID_RETURN;
 }
 
 int Binlog_sender::fake_rotate_event(const char *next_log_file,
                                      my_off_t log_pos) {
-  DBUG_ENTER("fake_rotate_event");
+  DBUG_TRACE;
   const char *p = next_log_file + dirname_length(next_log_file);
   size_t ident_len = strlen(p);
   size_t event_len = ident_len + LOG_EVENT_HEADER_LEN +
@@ -890,11 +896,11 @@ int Binlog_sender::fake_rotate_event(const char *next_log_file,
                      (event_checksum_on() ? BINLOG_CHECKSUM_LEN : 0);
 
   /* reset transmit packet for the fake rotate event below */
-  if (reset_transmit_packet(0, event_len)) DBUG_RETURN(1);
+  if (reset_transmit_packet(0, event_len)) return 1;
 
   size_t event_offset = m_packet.length();
   m_packet.length(event_len + event_offset);
-  uchar *header = (uchar *)m_packet.ptr() + event_offset;
+  uchar *header = pointer_cast<uchar *>(m_packet.ptr()) + event_offset;
   uchar *rotate_header = header + LOG_EVENT_HEADER_LEN;
   /*
     'when' (the timestamp) is set to 0 so that slave could distinguish between
@@ -912,19 +918,19 @@ int Binlog_sender::fake_rotate_event(const char *next_log_file,
 
   if (event_checksum_on()) calc_event_checksum(header, event_len);
 
-  DBUG_RETURN(send_packet());
+  return send_packet();
 }
 
 inline void Binlog_sender::calc_event_checksum(uchar *event_ptr,
                                                size_t event_len) {
-  ha_checksum crc = checksum_crc32(0L, NULL, 0);
+  ha_checksum crc = checksum_crc32(0L, nullptr, 0);
   crc = checksum_crc32(crc, event_ptr, event_len - BINLOG_CHECKSUM_LEN);
   int4store(event_ptr + event_len - BINLOG_CHECKSUM_LEN, crc);
 }
 
 inline int Binlog_sender::reset_transmit_packet(ushort flags,
                                                 size_t event_len) {
-  DBUG_ENTER("Binlog_sender::reset_transmit_packet");
+  DBUG_TRACE;
   DBUG_PRINT("info", ("event_len: %zu, m_packet->alloced_length: %zu",
                       event_len, m_packet.alloced_length()));
   DBUG_ASSERT(m_packet.alloced_length() >= PACKET_MIN_SIZE);
@@ -936,26 +942,26 @@ inline int Binlog_sender::reset_transmit_packet(ushort flags,
   if (m_observe_transmission &&
       RUN_HOOK(binlog_transmit, reserve_header, (m_thd, flags, &m_packet))) {
     set_unknown_error("Failed to run hook 'reserve_header'");
-    DBUG_RETURN(1);
+    return 1;
   }
 
   /* Resizes the buffer if needed. */
-  if (event_len > 0 && grow_packet(event_len)) DBUG_RETURN(1);
+  if (event_len > 0 && grow_packet(event_len)) return 1;
 
   DBUG_PRINT("info", ("m_packet.alloced_length: %zu (after potential "
                       "reallocation)",
                       m_packet.alloced_length()));
 
-  DBUG_RETURN(0);
+  return 0;
 }
 
 int Binlog_sender::send_format_description_event(File_reader *reader,
                                                  my_off_t start_pos) {
-  DBUG_ENTER("Binlog_sender::send_format_description_event");
+  DBUG_TRACE;
   uchar *event_ptr = nullptr;
   uint32 event_len = 0;
 
-  if (read_event(reader, &event_ptr, &event_len)) DBUG_RETURN(1);
+  if (read_event(reader, &event_ptr, &event_len)) return 1;
 
   DBUG_PRINT(
       "info",
@@ -965,7 +971,7 @@ int Binlog_sender::send_format_description_event(File_reader *reader,
   if (event_ptr == nullptr ||
       event_ptr[EVENT_TYPE_OFFSET] != binary_log::FORMAT_DESCRIPTION_EVENT) {
     set_fatal_error("Could not find format_description_event in binlog file");
-    DBUG_RETURN(1);
+    return 1;
   }
 
   Log_event *ev = nullptr;
@@ -973,7 +979,7 @@ int Binlog_sender::send_format_description_event(File_reader *reader,
       event_ptr, event_len, reader->format_description_event(), false, &ev);
   if (binlog_read_error.has_error()) {
     set_fatal_error(binlog_read_error.get_str());
-    DBUG_RETURN(1);
+    return 1;
   }
   reader->set_format_description_event(
       dynamic_cast<Format_description_log_event &>(*ev));
@@ -994,7 +1000,7 @@ int Binlog_sender::send_format_description_event(File_reader *reader,
         "checksum that master is configured to log");
 
     LogErr(WARNING_LEVEL, ER_RPL_BINLOG_MASTER_USES_CHECKSUM_AND_SLAVE_CANT);
-    DBUG_RETURN(1);
+    return 1;
   }
 
   event_ptr[FLAGS_OFFSET] &= ~LOG_EVENT_BINLOG_IN_USE_F;
@@ -1032,7 +1038,7 @@ int Binlog_sender::send_format_description_event(File_reader *reader,
   if (event_checksum_on() && event_updated)
     calc_event_checksum(event_ptr, event_len);
 
-  DBUG_RETURN(send_packet());
+  return send_packet();
 }
 
 int Binlog_sender::has_previous_gtid_log_event(File_reader *reader,
@@ -1073,9 +1079,9 @@ const char *Binlog_sender::log_read_error_msg(
 
 inline int Binlog_sender::read_event(File_reader *reader, uchar **event_ptr,
                                      uint32 *event_len) {
-  DBUG_ENTER("Binlog_sender::read_event");
+  DBUG_TRACE;
 
-  if (reset_transmit_packet(0, 0)) DBUG_RETURN(1);
+  if (reset_transmit_packet(0, 0)) return 1;
 #ifndef DBUG_OFF
   size_t event_offset;
   event_offset = m_packet.length();
@@ -1090,10 +1096,10 @@ inline int Binlog_sender::read_event(File_reader *reader, uchar **event_ptr,
     if (reader->get_error_type() == Binlog_read_error::READ_EOF) {
       *event_ptr = nullptr;
       *event_len = 0;
-      DBUG_RETURN(0);
+      return 0;
     }
     set_fatal_error(log_read_error_msg(reader->get_error_type()));
-    DBUG_RETURN(1);
+    return 1;
   }
 
   set_last_pos(reader->position());
@@ -1110,13 +1116,13 @@ inline int Binlog_sender::read_event(File_reader *reader, uchar **event_ptr,
   DBUG_PRINT("info", ("Read event %s", Log_event::get_type_str(Log_event_type(
                                            (*event_ptr)[EVENT_TYPE_OFFSET]))));
 #ifndef DBUG_OFF
-  if (check_event_count()) DBUG_RETURN(1);
+  if (check_event_count()) return 1;
 #endif
-  DBUG_RETURN(0);
+  return 0;
 }
 
 int Binlog_sender::send_heartbeat_event(my_off_t log_pos) {
-  DBUG_ENTER("send_heartbeat_event");
+  DBUG_TRACE;
   const char *filename = m_linfo.log_file_name;
   const char *p = filename + dirname_length(filename);
   size_t ident_len = strlen(p);
@@ -1125,11 +1131,11 @@ int Binlog_sender::send_heartbeat_event(my_off_t log_pos) {
 
   DBUG_PRINT("info", ("log_file_name %s, log_pos %llu", p, log_pos));
 
-  if (reset_transmit_packet(0, event_len)) DBUG_RETURN(1);
+  if (reset_transmit_packet(0, event_len)) return 1;
 
   size_t event_offset = m_packet.length();
   m_packet.length(event_len + event_offset);
-  uchar *header = (uchar *)m_packet.ptr() + event_offset;
+  uchar *header = pointer_cast<uchar *>(m_packet.ptr()) + event_offset;
 
   /* Timestamp field */
   int4store(header, 0);
@@ -1142,7 +1148,7 @@ int Binlog_sender::send_heartbeat_event(my_off_t log_pos) {
 
   if (event_checksum_on()) calc_event_checksum(header, event_len);
 
-  DBUG_RETURN(send_packet_and_flush());
+  return send_packet_and_flush();
 }
 
 inline int Binlog_sender::flush_net() {
@@ -1155,25 +1161,25 @@ inline int Binlog_sender::flush_net() {
 }
 
 inline int Binlog_sender::send_packet() {
-  DBUG_ENTER("Binlog_sender::send_packet");
+  DBUG_TRACE;
   DBUG_PRINT("info",
              ("Sending event of type %s",
               Log_event::get_type_str(
                   (Log_event_type)m_packet.ptr()[1 + EVENT_TYPE_OFFSET])));
   // We should always use the same buffer to guarantee that the reallocation
   // logic is not broken.
-  if (DBUG_EVALUATE_IF(
-          "simulate_send_error", true,
-          my_net_write(m_thd->get_protocol_classic()->get_net(),
-                       (uchar *)m_packet.ptr(), m_packet.length()))) {
+  if (DBUG_EVALUATE_IF("simulate_send_error", true,
+                       my_net_write(m_thd->get_protocol_classic()->get_net(),
+                                    pointer_cast<const uchar *>(m_packet.ptr()),
+                                    m_packet.length()))) {
     set_unknown_error("Failed on my_net_write()");
-    DBUG_RETURN(1);
+    return 1;
   }
 
   /* Shrink the packet if needed. */
   int ret = shrink_packet() ? 1 : 0;
-  m_last_event_sent_ts = time(0);
-  DBUG_RETURN(ret);
+  m_last_event_sent_ts = now_in_nanosecs();
+  return ret;
 }
 
 inline int Binlog_sender::send_packet_and_flush() {
@@ -1225,7 +1231,7 @@ inline int Binlog_sender::check_event_count() {
 #endif
 
 inline bool Binlog_sender::grow_packet(size_t extra_size) {
-  DBUG_ENTER("Binlog_sender::grow_packet");
+  DBUG_TRACE;
   size_t cur_buffer_size = m_packet.alloced_length();
   size_t cur_buffer_used = m_packet.length();
   size_t needed_buffer_size = cur_buffer_used + extra_size;
@@ -1235,7 +1241,7 @@ inline bool Binlog_sender::grow_packet(size_t extra_size) {
        Not enough memory: requesting packet to be bigger than the max
        allowed - PACKET_MAX_SIZE.
     */
-    DBUG_RETURN(true);
+    return true;
 
   /* Grow the buffer if needed. */
   if (needed_buffer_size > cur_buffer_size) {
@@ -1243,9 +1249,9 @@ inline bool Binlog_sender::grow_packet(size_t extra_size) {
     new_buffer_size =
         calc_grow_buffer_size(cur_buffer_size, needed_buffer_size);
 
-    if (!new_buffer_size) DBUG_RETURN(true);
+    if (!new_buffer_size) return true;
 
-    if (m_packet.mem_realloc(new_buffer_size)) DBUG_RETURN(true);
+    if (m_packet.mem_realloc(new_buffer_size)) return true;
 
     /*
      Calculates the new, smaller buffer, size to use the next time
@@ -1254,11 +1260,11 @@ inline bool Binlog_sender::grow_packet(size_t extra_size) {
     calc_shrink_buffer_size(new_buffer_size);
   }
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 inline bool Binlog_sender::shrink_packet() {
-  DBUG_ENTER("Binlog_sender::shrink_packet");
+  DBUG_TRACE;
   bool res = false;
   size_t cur_buffer_size = m_packet.alloced_length();
   size_t buffer_used = m_packet.length();
@@ -1301,7 +1307,7 @@ inline bool Binlog_sender::shrink_packet() {
     DBUG_ASSERT(m_packet.alloced_length() >= PACKET_MIN_SIZE);
   }
 #endif
-  DBUG_RETURN(res);
+  return res;
 }
 
 inline size_t Binlog_sender::calc_grow_buffer_size(size_t current_size,
