@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +26,7 @@
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_stage_bits.h"
+#include "mysql/components/services/psi_stage_bits.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_mysql_alloc.h"
@@ -84,7 +83,7 @@ enum_return_status Mutex_cond_array::ensure_index(int n) {
                        nullptr);
       mysql_cond_init(key_gtid_ensure_index_cond, &mutex_cond->cond);
       m_array.push_back(mutex_cond);
-      assert(&get_mutex_cond(i)->mutex == &mutex_cond->mutex);
+      DBUG_ASSERT(&get_mutex_cond(i)->mutex == &mutex_cond->mutex);
     }
   }
   RETURN_OK;
