@@ -31,7 +31,7 @@
 #endif
 #include <list>
 
-#include "control_events.h"
+#include "libbinlogevents/include/control_events.h"
 #include "m_ctype.h"
 #include "m_string.h"
 #include "my_base.h"
@@ -68,10 +68,10 @@ using std::string;
 my_thread_handle compress_thread_id;
 static bool terminate_compress_thread = false;
 static bool should_compress = false;
-const LEX_STRING Gtid_table_access_context::TABLE_NAME = {
-    C_STRING_WITH_LEN("gtid_executed")};
-const LEX_STRING Gtid_table_access_context::DB_NAME = {
-    C_STRING_WITH_LEN("mysql")};
+const LEX_CSTRING Gtid_table_access_context::TABLE_NAME = {
+    STRING_WITH_LEN("gtid_executed")};
+const LEX_CSTRING Gtid_table_access_context::DB_NAME = {
+    STRING_WITH_LEN("mysql")};
 
 /**
   Initialize a new THD.
@@ -79,7 +79,7 @@ const LEX_STRING Gtid_table_access_context::DB_NAME = {
   @param p_thd  Pointer to pointer to thread structure
 */
 static void init_thd(THD **p_thd) {
-  DBUG_ENTER("init_thd");
+  DBUG_TRACE;
   THD *thd = *p_thd;
   thd->thread_stack = reinterpret_cast<char *>(p_thd);
   thd->set_command(COM_DAEMON);
@@ -87,7 +87,6 @@ static void init_thd(THD **p_thd) {
   thd->system_thread = SYSTEM_THREAD_COMPRESS_GTID_TABLE;
   thd->store_globals();
   thd->set_time();
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -97,12 +96,11 @@ static void init_thd(THD **p_thd) {
   @param thd Thread requesting to be destroyed
 */
 static void deinit_thd(THD *thd) {
-  DBUG_ENTER("deinit_thd");
+  DBUG_TRACE;
   thd->release_resources();
   thd->restore_globals();
   delete thd;
   current_thd = nullptr;
-  DBUG_VOID_RETURN;
 }
 
 THD *Gtid_table_access_context::create_thd() {
@@ -124,7 +122,7 @@ void Gtid_table_access_context::drop_thd(THD *thd) {
 }
 
 void Gtid_table_access_context::before_open(THD *) {
-  DBUG_ENTER("Gtid_table_access_context::before_open");
+  DBUG_TRACE;
   /*
     Allow to operate the gtid_executed table
     while disconnecting the session.
@@ -132,11 +130,10 @@ void Gtid_table_access_context::before_open(THD *) {
   m_flags = (MYSQL_OPEN_IGNORE_GLOBAL_READ_LOCK |
              MYSQL_LOCK_IGNORE_GLOBAL_READ_ONLY | MYSQL_OPEN_IGNORE_FLUSH |
              MYSQL_LOCK_IGNORE_TIMEOUT | MYSQL_OPEN_IGNORE_KILLED);
-  DBUG_VOID_RETURN;
 }
 
 bool Gtid_table_access_context::init(THD **thd, TABLE **table, bool is_write) {
-  DBUG_ENTER("Gtid_table_access_context::init");
+  DBUG_TRACE;
 
   if (!(*thd)) *thd = m_drop_thd_object = this->create_thd();
   if (!(*thd)->is_cmd_skip_readonly()) {
@@ -169,12 +166,12 @@ bool Gtid_table_access_context::init(THD **thd, TABLE **table, bool is_write) {
       *thd, DB_NAME, TABLE_NAME, Gtid_table_persistor::number_fields,
       m_is_write ? TL_WRITE : TL_READ, table, &m_backup);
 
-  DBUG_RETURN(ret);
+  return ret;
 }
 
 bool Gtid_table_access_context::deinit(THD *thd, TABLE *table, bool error,
                                        bool need_commit) {
-  DBUG_ENTER("Gtid_table_access_context::deinit");
+  DBUG_TRACE;
 
   bool err;
   err = this->close_table(thd, table, &m_backup, 0 != error, need_commit);
@@ -188,7 +185,7 @@ bool Gtid_table_access_context::deinit(THD *thd, TABLE *table, bool error,
                     ER_THD(thd, ER_ERROR_DURING_FLUSH_LOGS), MYF(ME_FATALERROR),
                     err);
     LogErr(ERROR_LEVEL, ER_ERROR_DURING_FLUSH_LOG_COMMIT_PHASE, err);
-    DBUG_RETURN(err);
+    return err;
   }
 
   /*
@@ -211,12 +208,12 @@ bool Gtid_table_access_context::deinit(THD *thd, TABLE *table, bool error,
   }
   if (m_drop_thd_object) this->drop_thd(m_drop_thd_object);
 
-  DBUG_RETURN(err);
+  return err;
 }
 
 int Gtid_table_persistor::fill_fields(Field **fields, const char *sid,
                                       rpl_gno gno_start, rpl_gno gno_end) {
-  DBUG_ENTER("Gtid_table_persistor::fill_field");
+  DBUG_TRACE;
 
   /* Store SID */
   fields[0]->set_notnull();
@@ -239,21 +236,21 @@ int Gtid_table_persistor::fill_fields(Field **fields, const char *sid,
     goto err;
   }
 
-  DBUG_RETURN(0);
+  return 0;
 err:
-  DBUG_RETURN(-1);
+  return -1;
 }
 
 int Gtid_table_persistor::write_row(TABLE *table, const char *sid,
                                     rpl_gno gno_start, rpl_gno gno_end) {
-  DBUG_ENTER("Gtid_table_persistor::write_row");
+  DBUG_TRACE;
   int error = 0;
-  Field **fields = NULL;
+  Field **fields = nullptr;
 
   fields = table->field;
   empty_record(table);
 
-  if (fill_fields(fields, sid, gno_start, gno_end)) DBUG_RETURN(-1);
+  if (fill_fields(fields, sid, gno_start, gno_end)) return -1;
 
   /* Inserts a new row into the gtid_executed table. */
   error = table->file->ha_write_row(table->record[0]);
@@ -269,18 +266,18 @@ int Gtid_table_persistor::write_row(TABLE *table, const char *sid,
         This makes sure that the error is -1 and not the status
         returned by the handler.
       */
-      DBUG_RETURN(-1);
+      return -1;
     }
   }
 
-  DBUG_RETURN(0);
+  return 0;
 }
 
 int Gtid_table_persistor::update_row(TABLE *table, const char *sid,
                                      rpl_gno gno_start, rpl_gno new_gno_end) {
-  DBUG_ENTER("Gtid_table_persistor::update_row");
+  DBUG_TRACE;
   int error = 0;
-  Field **fields = NULL;
+  Field **fields = nullptr;
   uchar user_key[MAX_KEY_LENGTH];
 
   fields = table->field;
@@ -290,14 +287,14 @@ int Gtid_table_persistor::update_row(TABLE *table, const char *sid,
   fields[0]->set_notnull();
   if (fields[0]->store(sid, binary_log::Uuid::TEXT_LENGTH, &my_charset_bin)) {
     my_error(ER_RPL_INFO_DATA_TOO_LONG, MYF(0), fields[0]->field_name);
-    DBUG_RETURN(-1);
+    return -1;
   }
 
   /* Store gno_start */
   fields[1]->set_notnull();
   if (fields[1]->store(gno_start, true /* unsigned = true*/)) {
     my_error(ER_RPL_INFO_DATA_TOO_LONG, MYF(0), fields[1]->field_name);
-    DBUG_RETURN(-1);
+    return -1;
   }
 
   key_copy(user_key, table->record[0], table->key_info,
@@ -340,15 +337,15 @@ int Gtid_table_persistor::update_row(TABLE *table, const char *sid,
 end:
   table->file->ha_index_end();
   if (error)
-    DBUG_RETURN(-1);
+    return -1;
   else
-    DBUG_RETURN(0);
+    return 0;
 }
 
 int Gtid_table_persistor::save(THD *thd, const Gtid *gtid) {
-  DBUG_ENTER("Gtid_table_persistor::save(THD *thd, Gtid *gtid)");
+  DBUG_TRACE;
   int error = 0;
-  TABLE *table = NULL;
+  TABLE *table = nullptr;
   Gtid_table_access_context table_access_ctx;
   char buf[binary_log::Uuid::TEXT_LENGTH + 1];
 
@@ -381,14 +378,14 @@ end:
     }
   }
 
-  DBUG_RETURN(error);
+  return error;
 }
 
-int Gtid_table_persistor::save(const Gtid_set *gtid_set) {
-  DBUG_ENTER("Gtid_table_persistor::save(Gtid_set *gtid_set)");
+int Gtid_table_persistor::save(const Gtid_set *gtid_set, bool compress) {
+  DBUG_TRACE;
   int ret = 0;
   int error = 0;
-  TABLE *table = NULL;
+  TABLE *table = nullptr;
   Gtid_table_access_context table_access_ctx;
   THD *thd = current_thd;
 
@@ -400,7 +397,7 @@ int Gtid_table_persistor::save(const Gtid_set *gtid_set) {
     */
     thd->clear_error();
     if (!thd->get_stmt_da()->is_set())
-      thd->get_stmt_da()->set_ok_status(0, 0, NULL);
+      thd->get_stmt_da()->set_ok_status(0, 0, nullptr);
     goto end;
   }
 
@@ -412,20 +409,19 @@ end:
   if (!ret && deinit_ret) ret = -1;
 
   /* Notify compression thread to compress gtid_executed table. */
-  if (error == 0 && DBUG_EVALUATE_IF("dont_compress_gtid_table", 0, 1)) {
+  if (error == 0 && compress &&
+      DBUG_EVALUATE_IF("dont_compress_gtid_table", 0, 1)) {
     mysql_mutex_lock(&LOCK_compress_gtid_table);
     should_compress = true;
     mysql_cond_signal(&COND_compress_gtid_table);
     mysql_mutex_unlock(&LOCK_compress_gtid_table);
   }
 
-  DBUG_RETURN(ret);
+  return ret;
 }
 
 int Gtid_table_persistor::save(TABLE *table, const Gtid_set *gtid_set) {
-  DBUG_ENTER(
-      "Gtid_table_persistor::save(TABLE* table, "
-      "Gtid_set *gtid_set)");
+  DBUG_TRACE;
   int error = 0;
   list<Gtid_interval> gtid_intervals;
   list<Gtid_interval>::iterator iter;
@@ -443,7 +439,7 @@ int Gtid_table_persistor::save(TABLE *table, const Gtid_set *gtid_set) {
   }
 
   gtid_intervals.clear();
-  DBUG_RETURN(error);
+  return error;
 }
 
 /**
@@ -458,7 +454,7 @@ int Gtid_table_persistor::save(TABLE *table, const Gtid_set *gtid_set) {
 */
 #ifndef DBUG_OFF
 static int dbug_test_on_compress(THD *thd) {
-  DBUG_ENTER("dbug_test_on_compress");
+  DBUG_TRACE;
   /*
     Sleep a little, so that notified user thread executed the statement
     completely.
@@ -483,12 +479,12 @@ static int dbug_test_on_compress(THD *thd) {
   };);
   DBUG_EXECUTE_IF("simulate_crash_on_compress_gtid_table", DBUG_SUICIDE(););
 
-  DBUG_RETURN(0);
+  return 0;
 }
 #endif
 
 int Gtid_table_persistor::compress(THD *thd) {
-  DBUG_ENTER("Gtid_table_persistor::compress");
+  DBUG_TRACE;
   int error = 0;
   bool is_complete = false;
 
@@ -503,14 +499,14 @@ int Gtid_table_persistor::compress(THD *thd) {
     DBUG_ASSERT(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
   };);
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 int Gtid_table_persistor::compress_in_single_transaction(THD *thd,
                                                          bool &is_complete) {
-  DBUG_ENTER("Gtid_table_persistor::compress_in_single_transaction");
+  DBUG_TRACE;
   int error = 0;
-  TABLE *table = NULL;
+  TABLE *table = nullptr;
   Gtid_table_access_context table_access_ctx;
 
   mysql_mutex_lock(&LOCK_reset_gtid_table);
@@ -535,12 +531,12 @@ end:
   table_access_ctx.deinit(thd, table, 0 != error, true);
   mysql_mutex_unlock(&LOCK_reset_gtid_table);
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 int Gtid_table_persistor::compress_first_consecutive_range(TABLE *table,
                                                            bool &is_complete) {
-  DBUG_ENTER("Gtid_table_persistor::compress_first_consecutive_range");
+  DBUG_TRACE;
   int ret = 0;
   int err = 0;
   /* Record the source id of the first consecutive gtid. */
@@ -561,7 +557,7 @@ int Gtid_table_persistor::compress_first_consecutive_range(TABLE *table,
   */
   bool find_first_consecutive_gtids = false;
 
-  if ((err = table->file->ha_index_init(0, true))) DBUG_RETURN(-1);
+  if ((err = table->file->ha_index_init(0, true))) return -1;
 
   /* Read each row by the PK(sid, gno_start) in increasing order. */
   err = table->file->ha_index_first(table->record[0]);
@@ -605,13 +601,13 @@ int Gtid_table_persistor::compress_first_consecutive_range(TABLE *table,
     */
     ret = update_row(table, sid.c_str(), gno_start, gno_end);
 
-  DBUG_RETURN(ret);
+  return ret;
 }
 
 int Gtid_table_persistor::reset(THD *thd) {
-  DBUG_ENTER("Gtid_table_persistor::reset");
+  DBUG_TRACE;
   int error = 0;
-  TABLE *table = NULL;
+  TABLE *table = nullptr;
   Gtid_table_access_context table_access_ctx;
 
   mysql_mutex_lock(&LOCK_reset_gtid_table);
@@ -619,18 +615,21 @@ int Gtid_table_persistor::reset(THD *thd) {
     error = 1;
     goto end;
   }
-
+  /* Reseting the counter as gtid_executed table will also
+     be emptied.
+  */
+  m_atomic_count = 0;
   error = delete_all(table);
 
 end:
   table_access_ctx.deinit(thd, table, 0 != error, true);
   mysql_mutex_unlock(&LOCK_reset_gtid_table);
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 string Gtid_table_persistor::encode_gtid_text(TABLE *table) {
-  DBUG_ENTER("Gtid_table_persistor::encode_gtid_text");
+  DBUG_TRACE;
   char buff[MAX_FIELD_WIDTH];
   String str(buff, sizeof(buff), &my_charset_bin);
 
@@ -644,13 +643,13 @@ string Gtid_table_persistor::encode_gtid_text(TABLE *table) {
   table->field[2]->val_str(&str);
   gtid_text.append(str.c_ptr_safe());
 
-  DBUG_RETURN(gtid_text);
+  return gtid_text;
 }
 
 void Gtid_table_persistor::get_gtid_interval(TABLE *table, string &sid,
                                              rpl_gno &gno_start,
                                              rpl_gno &gno_end) {
-  DBUG_ENTER("Gtid_table_persistor::get_gtid_interval");
+  DBUG_TRACE;
   char buff[MAX_FIELD_WIDTH];
   String str(buff, sizeof(buff), &my_charset_bin);
 
@@ -659,14 +658,13 @@ void Gtid_table_persistor::get_gtid_interval(TABLE *table, string &sid,
   sid = string(str.c_ptr_safe());
   gno_start = table->field[1]->val_int();
   gno_end = table->field[2]->val_int();
-  DBUG_VOID_RETURN;
 }
 
 int Gtid_table_persistor::fetch_gtids(Gtid_set *gtid_set) {
-  DBUG_ENTER("Gtid_table_persistor::fetch_gtids");
+  DBUG_TRACE;
   int ret = 0;
   int err = 0;
-  TABLE *table = NULL;
+  TABLE *table = nullptr;
   Gtid_table_access_context table_access_ctx;
   THD *thd = current_thd;
 
@@ -705,14 +703,14 @@ int Gtid_table_persistor::fetch_gtids(Gtid_set *gtid_set) {
 end:
   table_access_ctx.deinit(thd, table, 0 != ret, true);
 
-  DBUG_RETURN(ret);
+  return ret;
 }
 
 int Gtid_table_persistor::delete_all(TABLE *table) {
-  DBUG_ENTER("Gtid_table_persistor::delete_all");
+  DBUG_TRACE;
   int err = 0;
 
-  if ((err = table->file->ha_rnd_init(true))) DBUG_RETURN(-1);
+  if ((err = table->file->ha_rnd_init(true))) return -1;
 
   /*
     Delete all rows in the gtid_executed table. We cannot use truncate(),
@@ -731,9 +729,9 @@ int Gtid_table_persistor::delete_all(TABLE *table) {
   }
 
   table->file->ha_rnd_end();
-  if (err != HA_ERR_END_OF_FILE) DBUG_RETURN(-1);
+  if (err != HA_ERR_END_OF_FILE) return -1;
 
-  DBUG_RETURN(0);
+  return 0;
 }
 
 /**
@@ -752,46 +750,47 @@ static void *compress_gtid_table(void *p_thd) {
   THD *thd = (THD *)p_thd;
   mysql_thread_set_psi_id(thd->thread_id());
   my_thread_init();
-  DBUG_ENTER("compress_gtid_table");
+  {
+    DBUG_TRACE;
 
-  init_thd(&thd);
-  /*
-    Gtid table compression thread should ignore 'read-only' and
-    'super_read_only' options so that it can update 'mysql.gtid_executed'
-    replication repository tables.
-  */
-  thd->set_skip_readonly_check();
-  for (;;) {
-    mysql_mutex_lock(&LOCK_compress_gtid_table);
-    if (terminate_compress_thread) break;
-    THD_ENTER_COND(thd, &COND_compress_gtid_table, &LOCK_compress_gtid_table,
-                   &stage_suspending, NULL);
-    /* Add the check to handle spurious wakeups from system. */
-    while (!(should_compress || terminate_compress_thread))
-      mysql_cond_wait(&COND_compress_gtid_table, &LOCK_compress_gtid_table);
-    should_compress = false;
-    if (terminate_compress_thread) break;
-    mysql_mutex_unlock(&LOCK_compress_gtid_table);
-    THD_EXIT_COND(thd, NULL);
+    init_thd(&thd);
+    /*
+      Gtid table compression thread should ignore 'read-only' and
+      'super_read_only' options so that it can update 'mysql.gtid_executed'
+      replication repository tables.
+    */
+    thd->set_skip_readonly_check();
+    for (;;) {
+      mysql_mutex_lock(&LOCK_compress_gtid_table);
+      if (terminate_compress_thread) break;
+      THD_ENTER_COND(thd, &COND_compress_gtid_table, &LOCK_compress_gtid_table,
+                     &stage_suspending, nullptr);
+      /* Add the check to handle spurious wakeups from system. */
+      while (!(should_compress || terminate_compress_thread))
+        mysql_cond_wait(&COND_compress_gtid_table, &LOCK_compress_gtid_table);
+      should_compress = false;
+      if (terminate_compress_thread) break;
+      mysql_mutex_unlock(&LOCK_compress_gtid_table);
+      THD_EXIT_COND(thd, nullptr);
 
-    THD_STAGE_INFO(thd, stage_compressing_gtid_table);
-    /* Compressing the gtid_executed table. */
-    if (gtid_state->compress(thd)) {
-      LogErr(WARNING_LEVEL, ER_FAILED_TO_COMPRESS_GTID_EXECUTED_TABLE);
-      /* Clear the error for going to wait for next compression signal. */
-      thd->clear_error();
-      DBUG_EXECUTE_IF("simulate_error_on_compress_gtid_table", {
-        const char act[] = "now signal compression_failed";
-        DBUG_ASSERT(opt_debug_sync_timeout > 0);
-        DBUG_ASSERT(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
-      };);
+      THD_STAGE_INFO(thd, stage_compressing_gtid_table);
+      /* Compressing the gtid_executed table. */
+      if (gtid_state->compress(thd)) {
+        LogErr(WARNING_LEVEL, ER_FAILED_TO_COMPRESS_GTID_EXECUTED_TABLE);
+        /* Clear the error for going to wait for next compression signal. */
+        thd->clear_error();
+        DBUG_EXECUTE_IF("simulate_error_on_compress_gtid_table", {
+          const char act[] = "now signal compression_failed";
+          DBUG_ASSERT(opt_debug_sync_timeout > 0);
+          DBUG_ASSERT(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
+        };);
+      }
     }
-  }
 
-  mysql_mutex_unlock(&LOCK_compress_gtid_table);
-  thd->reset_skip_readonly_check();
-  deinit_thd(thd);
-  DBUG_LEAVE;
+    mysql_mutex_unlock(&LOCK_compress_gtid_table);
+    thd->reset_skip_readonly_check();
+    deinit_thd(thd);
+  }
   my_thread_end();
   my_thread_exit(0);
   return 0;
@@ -842,7 +841,7 @@ void create_compress_gtid_table_thread() {
   Terminate the compression thread.
 */
 void terminate_compress_gtid_table_thread() {
-  DBUG_ENTER("terminate_compress_gtid_table_thread");
+  DBUG_TRACE;
   int error = 0;
 
   /* Notify suspended compression thread. */
@@ -852,13 +851,11 @@ void terminate_compress_gtid_table_thread() {
   mysql_mutex_unlock(&LOCK_compress_gtid_table);
 
   if (compress_thread_id.thread != 0) {
-    error = my_thread_join(&compress_thread_id, NULL);
+    error = my_thread_join(&compress_thread_id, nullptr);
     compress_thread_id.thread = 0;
   }
 
   if (error != 0)
     LogErr(WARNING_LEVEL, ER_FAILED_TO_JOIN_GTID_TABLE_COMPRESSION_THREAD,
            error);
-
-  DBUG_VOID_RETURN;
 }
