@@ -89,15 +89,15 @@ Rpl_info_factory::struct_table_data Rpl_info_factory::worker_table_data;
 */
 Master_info *Rpl_info_factory::create_mi(uint mi_option, const char *channel,
                                          bool to_decide_repo) {
-  Master_info *mi = NULL;
-  Rpl_info_handler *handler_src = NULL;
-  Rpl_info_handler *handler_dest = NULL;
+  Master_info *mi = nullptr;
+  Rpl_info_handler *handler_src = nullptr;
+  Rpl_info_handler *handler_dest = nullptr;
   uint instances = 1;
   const char *msg =
       "Failed to allocate memory for the master info "
       "structure";
 
-  DBUG_ENTER("Rpl_info_factory::create_mi");
+  DBUG_TRACE;
 
   if (!(mi = new Master_info(
 #ifdef HAVE_PSI_INTERFACE
@@ -129,7 +129,7 @@ Master_info *Rpl_info_factory::create_mi(uint mi_option, const char *channel,
     delete handler_src;
   }
 
-  DBUG_RETURN(mi);
+  return mi;
 
 err:
   delete handler_src;
@@ -139,12 +139,12 @@ err:
       The handler was previously deleted so we need to remove
       any reference to it.
     */
-    mi->set_rpl_info_handler(NULL);
+    mi->set_rpl_info_handler(nullptr);
     mi->channel_wrlock();
     delete mi;
   }
   LogErr(ERROR_LEVEL, ER_RPL_ERROR_CREATING_MASTER_INFO, msg);
-  DBUG_RETURN(NULL);
+  return nullptr;
 }
 
 /**
@@ -160,26 +160,26 @@ err:
 bool Rpl_info_factory::change_mi_repository(Master_info *mi, uint mi_option,
                                             const char **msg) {
   Rpl_info_handler *handler_src = mi->get_rpl_info_handler();
-  Rpl_info_handler *handler_dest = NULL;
-  DBUG_ENTER("Rpl_info_factory::change_mi_repository");
+  Rpl_info_handler *handler_dest = nullptr;
+  DBUG_TRACE;
 
   DBUG_ASSERT(handler_src);
 
-  if (init_repositories(mi_table_data, mi_file_data, mi_option, NULL,
+  if (init_repositories(mi_table_data, mi_file_data, mi_option, nullptr,
                         &handler_dest, msg))
     goto err;
 
   if (decide_repository(mi, mi_option, &handler_src, &handler_dest, msg))
     goto err;
 
-  DBUG_RETURN(false);
+  return false;
 
 err:
   delete handler_dest;
-  handler_dest = NULL;
+  handler_dest = nullptr;
 
   LogErr(ERROR_LEVEL, ER_RPL_ERROR_CHANGING_MASTER_INFO_REPO_TYPE, *msg);
-  DBUG_RETURN(true);
+  return true;
 }
 
 /**
@@ -205,19 +205,19 @@ Relay_log_info *Rpl_info_factory::create_rli(uint rli_option,
                                              bool is_slave_recovery,
                                              const char *channel,
                                              bool to_decide_repo) {
-  Relay_log_info *rli = NULL;
-  Rpl_info_handler *handler_src = NULL;
-  Rpl_info_handler *handler_dest = NULL;
+  Relay_log_info *rli = nullptr;
+  Rpl_info_handler *handler_src = nullptr;
+  Rpl_info_handler *handler_dest = nullptr;
   uint instances = 1;
   uint worker_repository = INVALID_INFO_REPOSITORY;
   uint worker_instances = 1;
-  const char *msg = NULL;
+  const char *msg = nullptr;
   const char *msg_alloc =
       "Failed to allocate memory for the relay log info "
       "structure";
-  Rpl_filter *rpl_filter = NULL;
+  Rpl_filter *rpl_filter = nullptr;
 
-  DBUG_ENTER("Rpl_info_factory::create_rli");
+  DBUG_TRACE;
 
   /*
     Returns how many occurrences of worker's repositories exist. For example,
@@ -262,7 +262,7 @@ Relay_log_info *Rpl_info_factory::create_rli(uint rli_option,
     if (decide_repository(rli, rli_option, &handler_src, &handler_dest, &msg))
       goto err;
   } else {
-    if (channel != NULL) {
+    if (channel != nullptr) {
       /* Here dest code should be TABLE type repo. See, init_slave() */
       if (handler_dest->get_rpl_info_type() != INFO_REPOSITORY_TABLE) {
         LogErr(ERROR_LEVEL, ER_RPL_REPO_SHOULD_BE_TABLE);
@@ -279,12 +279,12 @@ Relay_log_info *Rpl_info_factory::create_rli(uint rli_option,
       which is channel_name
     */
     delete handler_src;
-    handler_src = NULL;
+    handler_src = nullptr;
   }
 
   /* Set filters here to guarantee that any rli object has a valid filter */
   rpl_filter = rpl_channel_filters.get_channel_filter(channel);
-  if (rpl_filter == NULL) {
+  if (rpl_filter == nullptr) {
     LogErr(ERROR_LEVEL, ER_RPL_SLAVE_FILTER_CREATE_FAILED, channel);
     msg = msg_alloc;
     goto err;
@@ -292,7 +292,7 @@ Relay_log_info *Rpl_info_factory::create_rli(uint rli_option,
   rli->set_filter(rpl_filter);
   rpl_filter->set_attached();
 
-  DBUG_RETURN(rli);
+  return rli;
 
 err:
   delete handler_src;
@@ -302,11 +302,11 @@ err:
       The handler was previously deleted so we need to remove
       any reference to it.
     */
-    rli->set_rpl_info_handler(NULL);
+    rli->set_rpl_info_handler(nullptr);
     delete rli;
   }
   LogErr(ERROR_LEVEL, ER_RPL_ERROR_CREATING_RELAY_LOG_INFO, msg);
-  DBUG_RETURN(NULL);
+  return nullptr;
 }
 
 /**
@@ -323,26 +323,26 @@ bool Rpl_info_factory::change_rli_repository(Relay_log_info *rli,
                                              uint rli_option,
                                              const char **msg) {
   Rpl_info_handler *handler_src = rli->get_rpl_info_handler();
-  Rpl_info_handler *handler_dest = NULL;
-  DBUG_ENTER("Rpl_info_factory::change_rli_repository");
+  Rpl_info_handler *handler_dest = nullptr;
+  DBUG_TRACE;
 
-  DBUG_ASSERT(handler_src != NULL);
+  DBUG_ASSERT(handler_src != nullptr);
 
-  if (init_repositories(rli_table_data, rli_file_data, rli_option, NULL,
+  if (init_repositories(rli_table_data, rli_file_data, rli_option, nullptr,
                         &handler_dest, msg))
     goto err;
 
   if (decide_repository(rli, rli_option, &handler_src, &handler_dest, msg))
     goto err;
 
-  DBUG_RETURN(false);
+  return false;
 
 err:
   delete handler_dest;
-  handler_dest = NULL;
+  handler_dest = nullptr;
 
   LogErr(ERROR_LEVEL, ER_RPL_ERROR_CHANGING_RELAY_LOG_INFO_REPO_TYPE, *msg);
-  DBUG_RETURN(true);
+  return true;
 }
 
 /**
@@ -355,18 +355,19 @@ err:
 bool Rpl_info_factory::reset_workers(Relay_log_info *rli) {
   bool error = true;
 
-  DBUG_ENTER("Rpl_info_factory::reset_workers");
+  DBUG_TRACE;
 
-  if (rli->recovery_parallel_workers == 0) DBUG_RETURN(0);
+  if (rli->recovery_parallel_workers == 0) return 0;
 
-  if (Rpl_info_file::do_reset_info(Slave_worker::get_number_worker_fields(),
-                                   worker_file_data.pattern,
-                                   worker_file_data.name_indexed))
+  if (Rpl_info_file::do_reset_info(
+          Slave_worker::get_number_worker_fields(), worker_file_data.pattern,
+          worker_file_data.name_indexed, &worker_file_data.nullable_fields))
     goto err;
 
   if (Rpl_info_table::do_reset_info(Slave_worker::get_number_worker_fields(),
                                     MYSQL_SCHEMA_NAME.str, WORKER_INFO_NAME.str,
-                                    rli->channel))
+                                    rli->channel,
+                                    &worker_file_data.nullable_fields))
     goto err;
 
   error = false;
@@ -383,7 +384,7 @@ err:
     error = true;
     LogErr(ERROR_LEVEL, ER_RPL_FAILED_TO_RESET_STATE_IN_SLAVE_INFO_REPOSITORY);
   }
-  DBUG_RETURN(error);
+  return error;
 }
 
 /**
@@ -405,14 +406,14 @@ err:
 Slave_worker *Rpl_info_factory::create_worker(uint rli_option, uint worker_id,
                                               Relay_log_info *rli,
                                               bool is_gaps_collecting_phase) {
-  Rpl_info_handler *handler_src = NULL;
-  Rpl_info_handler *handler_dest = NULL;
-  Slave_worker *worker = NULL;
+  Rpl_info_handler *handler_src = nullptr;
+  Rpl_info_handler *handler_dest = nullptr;
+  Slave_worker *worker = nullptr;
   const char *msg =
       "Failed to allocate memory for the worker info "
       "structure";
 
-  DBUG_ENTER("Rpl_info_factory::create_worker");
+  DBUG_TRACE;
 
   /*
     Define the name of the worker and its repository.
@@ -468,7 +469,7 @@ Slave_worker *Rpl_info_factory::create_worker(uint rli_option, uint worker_id,
     msg = "Failed to initialize worker info table";
     goto err;
   }
-  DBUG_RETURN(worker);
+  return worker;
 
 err:
   delete handler_src;
@@ -478,11 +479,11 @@ err:
       The handler was previously deleted so we need to remove
       any reference to it.
     */
-    worker->set_rpl_info_handler(NULL);
+    worker->set_rpl_info_handler(nullptr);
     delete worker;
   }
   LogErr(ERROR_LEVEL, ER_RPL_ERROR_CREATING_RELAY_LOG_INFO, msg);
-  DBUG_RETURN(NULL);
+  return nullptr;
 }
 
 static void build_worker_info_name(char *to, const char *path,
@@ -501,7 +502,7 @@ static void build_worker_info_name(char *to, const char *path,
 void Rpl_info_factory::init_repository_metadata() {
   /* Needed for the file names and paths for worker info files. */
   size_t len;
-  char *relay_log_info_file_name;
+  const char *relay_log_info_file_name;
   char relay_log_info_file_dirpart[FN_REFLEN];
 
   /* Extract the directory name from relay_log_info_file */
@@ -512,21 +513,25 @@ void Rpl_info_factory::init_repository_metadata() {
   rli_table_data.schema = MYSQL_SCHEMA_NAME.str;
   rli_table_data.name = RLI_INFO_NAME.str;
   rli_table_data.n_pk_fields = 0;
-  rli_table_data.pk_field_indexes = NULL;
+  rli_table_data.pk_field_indexes = nullptr;
+  Relay_log_info::set_nullable_fields(&rli_table_data.nullable_fields);
   rli_file_data.n_fields = Relay_log_info::get_number_info_rli_fields();
   my_stpcpy(rli_file_data.name, relay_log_info_file);
   my_stpcpy(rli_file_data.pattern, relay_log_info_file);
   rli_file_data.name_indexed = false;
+  Relay_log_info::set_nullable_fields(&rli_file_data.nullable_fields);
 
   mi_table_data.n_fields = Master_info::get_number_info_mi_fields();
   mi_table_data.schema = MYSQL_SCHEMA_NAME.str;
   mi_table_data.name = MI_INFO_NAME.str;
   mi_table_data.n_pk_fields = 1;
   mi_table_data.pk_field_indexes = Master_info::get_table_pk_field_indexes();
+  Master_info::set_nullable_fields(&mi_table_data.nullable_fields);
   mi_file_data.n_fields = Master_info::get_number_info_mi_fields();
   my_stpcpy(mi_file_data.name, master_info_file);
   my_stpcpy(mi_file_data.pattern, master_info_file);
-  rli_file_data.name_indexed = false;
+  mi_file_data.name_indexed = false;
+  Master_info::set_nullable_fields(&mi_file_data.nullable_fields);
 
   worker_table_data.n_fields = Slave_worker::get_number_worker_fields();
   worker_table_data.schema = MYSQL_SCHEMA_NAME.str;
@@ -534,12 +539,14 @@ void Rpl_info_factory::init_repository_metadata() {
   worker_table_data.n_pk_fields = 2;
   worker_table_data.pk_field_indexes =
       Slave_worker::get_table_pk_field_indexes();
+  Slave_worker::set_nullable_fields(&worker_table_data.nullable_fields);
   worker_file_data.n_fields = Slave_worker::get_number_worker_fields();
   build_worker_info_name(worker_file_data.name, relay_log_info_file_dirpart,
                          relay_log_info_file_name);
   build_worker_info_name(worker_file_data.pattern, relay_log_info_file_dirpart,
                          relay_log_info_file_name);
   worker_file_data.name_indexed = true;
+  Slave_worker::set_nullable_fields(&worker_file_data.nullable_fields);
 }
 
 /**
@@ -576,17 +583,17 @@ bool Rpl_info_factory::decide_repository(Rpl_info *info, uint option,
   bool error = true;
   enum_return_check return_check_src = ERROR_CHECKING_REPOSITORY;
   enum_return_check return_check_dst = ERROR_CHECKING_REPOSITORY;
-  DBUG_ENTER("Rpl_info_factory::decide_repository");
+  DBUG_TRACE;
 
   if (option == INFO_REPOSITORY_DUMMY) {
     delete (*handler_src);
-    *handler_src = NULL;
+    *handler_src = nullptr;
     info->set_rpl_info_handler(*handler_dest);
     error = false;
     goto err;
   }
 
-  DBUG_ASSERT((*handler_src) != NULL && (*handler_dest) != NULL &&
+  DBUG_ASSERT((*handler_src) != nullptr && (*handler_dest) != nullptr &&
               (*handler_src) != (*handler_dest));
 
   return_check_src = check_src_repository(info, option, handler_src);
@@ -607,8 +614,8 @@ bool Rpl_info_factory::decide_repository(Rpl_info *info, uint option,
       If there is a problem with one of the repositories we print out
       more information and exit.
     */
-    DBUG_RETURN(check_error_repository(
-        *handler_src, *handler_dest, return_check_src, return_check_dst, msg));
+    return check_error_repository(*handler_src, *handler_dest, return_check_src,
+                                  return_check_dst, msg);
   } else {
     if ((return_check_src == REPOSITORY_EXISTS &&
          return_check_dst == REPOSITORY_DOES_NOT_EXIST) ||
@@ -653,7 +660,7 @@ bool Rpl_info_factory::decide_repository(Rpl_info *info, uint option,
       }
     } else if (return_check_src == REPOSITORY_DOES_NOT_EXIST &&
                return_check_dst == REPOSITORY_EXISTS) {
-      DBUG_ASSERT(info->get_rpl_info_handler() == NULL);
+      DBUG_ASSERT(info->get_rpl_info_handler() == nullptr);
       if ((*handler_dest)->do_init_info(info->get_internal_id())) {
         *msg = "Error reading repository";
         goto err;
@@ -665,13 +672,13 @@ bool Rpl_info_factory::decide_repository(Rpl_info *info, uint option,
     }
 
     delete (*handler_src);
-    *handler_src = NULL;
+    *handler_src = nullptr;
     info->set_rpl_info_handler(*handler_dest);
     error = false;
   }
 
 err:
-  DBUG_RETURN(error);
+  return error;
 }
 
 /**
@@ -688,7 +695,7 @@ err:
 enum_return_check Rpl_info_factory::check_src_repository(
     Rpl_info *info, uint option, Rpl_info_handler **handler_src) {
   enum_return_check return_check_src = ERROR_CHECKING_REPOSITORY;
-  bool live_migration = info->get_rpl_info_handler() != NULL;
+  bool live_migration = info->get_rpl_info_handler() != nullptr;
 
   if (!live_migration) {
     /*
@@ -788,7 +795,7 @@ bool Rpl_info_factory::init_repositories(Rpl_info *info,
                                          Rpl_info_handler **handler_src,
                                          Rpl_info_handler **handler_dest,
                                          const char **msg) {
-  bool live_migration = info->get_rpl_info_handler() != NULL;
+  bool live_migration = info->get_rpl_info_handler() != nullptr;
 
   if (!live_migration) {
     if ((*handler_src)->do_init_info(info->get_internal_id()) ||
@@ -831,30 +838,33 @@ bool Rpl_info_factory::init_repositories(const struct_table_data &table_data,
   bool error = true;
   *msg = "Failed to allocate memory for master info repositories";
 
-  DBUG_ENTER("Rpl_info_factory::init_mi_repositories");
+  DBUG_TRACE;
 
-  DBUG_ASSERT(handler_dest != NULL);
+  DBUG_ASSERT(handler_dest != nullptr);
   switch (rep_option) {
     case INFO_REPOSITORY_FILE:
-      if (!(*handler_dest =
-                new Rpl_info_file(file_data.n_fields, file_data.pattern,
-                                  file_data.name, file_data.name_indexed)))
+      if (!(*handler_dest = new Rpl_info_file(
+                file_data.n_fields, file_data.pattern, file_data.name,
+                file_data.name_indexed, &file_data.nullable_fields)))
         goto err;
       if (handler_src &&
           !(*handler_src = new Rpl_info_table(
                 table_data.n_fields, table_data.schema, table_data.name,
-                table_data.n_pk_fields, table_data.pk_field_indexes)))
+                table_data.n_pk_fields, table_data.pk_field_indexes,
+                &table_data.nullable_fields)))
         goto err;
       break;
 
     case INFO_REPOSITORY_TABLE:
       if (!(*handler_dest = new Rpl_info_table(
                 table_data.n_fields, table_data.schema, table_data.name,
-                table_data.n_pk_fields, table_data.pk_field_indexes)))
+                table_data.n_pk_fields, table_data.pk_field_indexes,
+                &table_data.nullable_fields)))
         goto err;
-      if (handler_src && !(*handler_src = new Rpl_info_file(
-                               file_data.n_fields, file_data.pattern,
-                               file_data.name, file_data.name_indexed)))
+      if (handler_src &&
+          !(*handler_src = new Rpl_info_file(
+                file_data.n_fields, file_data.pattern, file_data.name,
+                file_data.name_indexed, &file_data.nullable_fields)))
         goto err;
       break;
 
@@ -870,7 +880,7 @@ bool Rpl_info_factory::init_repositories(const struct_table_data &table_data,
   error = false;
 
 err:
-  DBUG_RETURN(error);
+  return error;
 }
 
 bool Rpl_info_factory::scan_repositories(uint *found_instances,
@@ -881,18 +891,20 @@ bool Rpl_info_factory::scan_repositories(uint *found_instances,
   bool error = false;
   uint file_instances = 0;
   uint table_instances = 0;
-  DBUG_ASSERT(found_rep_option != NULL);
+  DBUG_ASSERT(found_rep_option != nullptr);
 
-  DBUG_ENTER("Rpl_info_factory::scan_repositories");
+  DBUG_TRACE;
 
-  if (Rpl_info_table::do_count_info(table_data.n_fields, table_data.schema,
-                                    table_data.name, &table_instances)) {
+  if (Rpl_info_table::do_count_info(
+          table_data.n_fields, table_data.schema, table_data.name,
+          &table_data.nullable_fields, &table_instances)) {
     error = true;
     goto err;
   }
 
-  if (Rpl_info_file::do_count_info(file_data.n_fields, file_data.pattern,
-                                   file_data.name_indexed, &file_instances)) {
+  if (Rpl_info_file::do_count_info(
+          file_data.n_fields, file_data.pattern, file_data.name_indexed,
+          &file_data.nullable_fields, &file_instances)) {
     error = true;
     goto err;
   }
@@ -918,20 +930,19 @@ bool Rpl_info_factory::scan_repositories(uint *found_instances,
   }
 
 err:
-  DBUG_RETURN(error);
+  return error;
 }
 
 bool Rpl_info_factory::configure_channel_replication_filters(
     Relay_log_info *rli, const char *channel_name) {
-  DBUG_ENTER("configure_channel_replication_filters");
+  DBUG_TRACE;
 
   /*
     GROUP REPLICATION channels should not be configurable using
     --replicate* nor CHANGE REPLICATION FILTER, and should not
     inherit from global filters.
   */
-  if (channel_map.is_group_replication_channel_name(channel_name))
-    DBUG_RETURN(false);
+  if (channel_map.is_group_replication_channel_name(channel_name)) return false;
 
   if (Master_info::is_configured(rli->mi)) {
     /*
@@ -943,7 +954,7 @@ bool Rpl_info_factory::configure_channel_replication_filters(
     if (rli->rpl_filter->copy_global_replication_filters()) {
       LogErr(ERROR_LEVEL, ER_RPL_SLAVE_GLOBAL_FILTERS_COPY_FAILED,
              channel_name);
-      DBUG_RETURN(true);
+      return true;
     }
   } else {
     /*
@@ -956,7 +967,7 @@ bool Rpl_info_factory::configure_channel_replication_filters(
       rli->rpl_filter->reset();
     }
   }
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -1058,10 +1069,10 @@ bool Rpl_info_factory::configure_channel_replication_filters(
 bool Rpl_info_factory::create_slave_info_objects(
     uint mi_option, uint rli_option, int thread_mask,
     Multisource_info *pchannel_map) {
-  DBUG_ENTER("create_slave_info_objects");
+  DBUG_TRACE;
 
-  Master_info *mi = NULL;
-  const char *msg = NULL;
+  Master_info *mi = nullptr;
+  const char *msg = nullptr;
   bool error = false, channel_error;
   bool default_channel_existed_previously = false;
 
@@ -1163,7 +1174,7 @@ bool Rpl_info_factory::create_slave_info_objects(
     error = error || channel_error;
   }
 end:
-  DBUG_RETURN(error);
+  return error;
 }
 
 /**
@@ -1190,20 +1201,20 @@ end:
 Master_info *Rpl_info_factory::create_mi_and_rli_objects(
     uint mi_option, uint rli_option, const char *channel, bool to_decide_repo,
     Multisource_info *pchannel_map) {
-  DBUG_ENTER("Rpl_info_factory::create_mi_and_rli_objects");
+  DBUG_TRACE;
 
-  Master_info *mi = NULL;
-  Relay_log_info *rli = NULL;
+  Master_info *mi = nullptr;
+  Relay_log_info *rli = nullptr;
 
   if (!(mi = Rpl_info_factory::create_mi(mi_option, channel, to_decide_repo)))
-    DBUG_RETURN(NULL);
+    return nullptr;
 
   if (!(rli = Rpl_info_factory::create_rli(rli_option, relay_log_recovery,
                                            channel, to_decide_repo))) {
     mi->channel_wrlock();
     delete mi;
-    mi = NULL;
-    DBUG_RETURN(NULL);
+    mi = nullptr;
+    return nullptr;
   }
 
   /* Set the cross dependencies used all over the code */
@@ -1215,10 +1226,10 @@ Master_info *Rpl_info_factory::create_mi_and_rli_objects(
     mi->channel_wrlock();
     delete mi;
     delete rli;
-    DBUG_RETURN(NULL);
+    return nullptr;
   }
 
-  DBUG_RETURN(mi);
+  return mi;
 }
 
 /**
@@ -1247,7 +1258,7 @@ bool Rpl_info_factory::load_channel_names_from_repository(
     std::vector<std::string> &channel_list,
     uint mi_instances MY_ATTRIBUTE((unused)), uint mi_repository,
     const char *default_channel, bool *default_channel_existed_previously) {
-  DBUG_ENTER("Rpl_info_factory::load_channel_names_from_repository");
+  DBUG_TRACE;
 
   *default_channel_existed_previously = false;
   switch (mi_repository) {
@@ -1263,7 +1274,7 @@ bool Rpl_info_factory::load_channel_names_from_repository(
     case INFO_REPOSITORY_TABLE:
       if (load_channel_names_from_table(channel_list, default_channel,
                                         default_channel_existed_previously))
-        DBUG_RETURN(true);
+        return true;
       break;
     case INVALID_INFO_REPOSITORY:
       /* file and table instanaces are zero, nothing to be done*/
@@ -1272,7 +1283,7 @@ bool Rpl_info_factory::load_channel_names_from_repository(
       DBUG_ASSERT(0);
   }
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -1300,14 +1311,14 @@ bool Rpl_info_factory::load_channel_names_from_repository(
 bool Rpl_info_factory::load_channel_names_from_table(
     std::vector<std::string> &channel_list, const char *default_channel,
     bool *default_channel_existed_previously) {
-  DBUG_ENTER(" Rpl_info_table::load_channel_names_from_table");
+  DBUG_TRACE;
 
   int error = 1;
-  TABLE *table = 0;
+  TABLE *table = nullptr;
   ulong saved_mode;
   Open_tables_backup backup;
-  Rpl_info_table *info = 0;
-  THD *thd = 0;
+  Rpl_info_table *info = nullptr;
+  THD *thd = nullptr;
   char buff[MAX_FIELD_WIDTH];
   *default_channel_existed_previously = false;
   String str(buff, sizeof(buff),
@@ -1317,8 +1328,9 @@ bool Rpl_info_factory::load_channel_names_from_table(
 
   if (!(info = new Rpl_info_table(mi_table_data.n_fields, mi_table_data.schema,
                                   mi_table_data.name, mi_table_data.n_pk_fields,
-                                  mi_table_data.pk_field_indexes)))
-    DBUG_RETURN(true);
+                                  mi_table_data.pk_field_indexes,
+                                  &mi_table_data.nullable_fields)))
+    return true;
 
   thd = info->access->create_thd();
   saved_mode = thd->variables.sql_mode;
@@ -1326,7 +1338,8 @@ bool Rpl_info_factory::load_channel_names_from_table(
   /*
      Opens and locks the rpl_info table before accessing it.
   */
-  if (info->access->open_table(thd, info->str_schema, info->str_table,
+  if (info->access->open_table(thd, to_lex_cstring(info->str_schema),
+                               to_lex_cstring(info->str_table),
                                info->get_number_info(), TL_READ, &table,
                                &backup)) {
     /*
@@ -1338,7 +1351,7 @@ bool Rpl_info_factory::load_channel_names_from_table(
   }
 
   /* Do ha_handler random init for full scanning */
-  if ((error = table->file->ha_rnd_init(true))) DBUG_RETURN(true);
+  if ((error = table->file->ha_rnd_init(true))) return true;
 
   /* Ensure that the table pk (Channel_name) is at the correct position */
   if (info->verify_table_primary_key_fields(table)) {
@@ -1381,5 +1394,5 @@ err:
   thd->variables.sql_mode = saved_mode;
   info->access->drop_thd(thd);
   delete info;
-  DBUG_RETURN(error != HA_ERR_END_OF_FILE && error != 0);
+  return error != HA_ERR_END_OF_FILE && error != 0;
 }
