@@ -1,16 +1,15 @@
-/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -106,7 +105,7 @@ int Rpl_info_file::do_init_info() {
              my_errno());
       error = 1;
     } else if (init_io_cache(&info_file, info_fd, IO_SIZE * 2, READ_CACHE, 0L,
-                             false, MYF(MY_WME))) {
+                             0, MYF(MY_WME))) {
       LogErr(ERROR_LEVEL, ER_RPL_FAILED_TO_CREATE_CACHE_FOR_INFO_FILE,
              info_fname);
       error = 1;
@@ -119,14 +118,14 @@ int Rpl_info_file::do_init_info() {
   /* file exists */
   else if (ret_check == REPOSITORY_EXISTS) {
     if (info_fd >= 0)
-      reinit_io_cache(&info_file, READ_CACHE, 0L, false, false);
+      reinit_io_cache(&info_file, READ_CACHE, 0L, 0, 0);
     else {
       if ((info_fd = my_open(info_fname, O_RDWR, MYF(MY_WME))) < 0) {
         LogErr(ERROR_LEVEL, ER_RPL_FAILED_TO_OPEN_INFO_FILE, info_fname,
                my_errno());
         error = 1;
       } else if (init_io_cache(&info_file, info_fd, IO_SIZE * 2, READ_CACHE, 0L,
-                               false, MYF(MY_WME))) {
+                               0, MYF(MY_WME))) {
         LogErr(ERROR_LEVEL, ER_RPL_FAILED_TO_CREATE_CACHE_FOR_INFO_FILE,
                info_fname);
         error = 1;
@@ -145,14 +144,14 @@ int Rpl_info_file::do_prepare_info_for_read() {
   cursor = 0;
   prv_error = false;
   prv_get_error = Rpl_info_handler::enum_field_get_status::FIELD_VALUE_NOT_NULL;
-  return (reinit_io_cache(&info_file, READ_CACHE, 0L, false, false));
+  return (reinit_io_cache(&info_file, READ_CACHE, 0L, 0, 0));
 }
 
 int Rpl_info_file::do_prepare_info_for_write() {
   cursor = 0;
   prv_error = false;
   prv_get_error = Rpl_info_handler::enum_field_get_status::FIELD_VALUE_NOT_NULL;
-  return (reinit_io_cache(&info_file, WRITE_CACHE, 0L, false, true));
+  return (reinit_io_cache(&info_file, WRITE_CACHE, 0L, 0, 1));
 }
 
 inline enum_return_check do_check_repository_file(const char *fname) {
@@ -286,7 +285,7 @@ int Rpl_info_file::do_remove_info() {
 int Rpl_info_file::do_clean_info() {
   /*
     There is nothing to do here. Maybe we can truncate the
-    file in the future. However, for now, there is no need.
+    file in the future. Howerver, for now, there is no need.
   */
   return 0;
 }
@@ -350,7 +349,7 @@ bool Rpl_info_file::do_set_info(const int, const int value) {
 bool Rpl_info_file::do_set_info(const int, const float value) {
   /*
     64 bytes provide enough space considering that the precision is 3
-    bytes (See the appropriate set function):
+    bytes (See the appropriate set funciton):
 
     FLT_MAX  The value of this macro is the maximum number representable
              in type float. It is supposed to be at least 1E+37.
@@ -463,7 +462,10 @@ char *Rpl_info_file::do_get_description_info() { return info_fname; }
 
 bool Rpl_info_file::do_is_transactional() { return false; }
 
-bool Rpl_info_file::do_update_is_transactional() { return false; }
+bool Rpl_info_file::do_update_is_transactional() {
+  DBUG_EXECUTE_IF("simulate_update_is_transactional_error", { return true; });
+  return false;
+}
 
 uint Rpl_info_file::do_get_rpl_info_type() { return INFO_REPOSITORY_FILE; }
 
@@ -561,7 +563,7 @@ long init_ulongvar_from_file(ulong *var, IO_CACHE *f, ulong default_val) {
 
   long length;
   if ((length = static_cast<long>(my_b_gets(f, buf, sizeof(buf))))) {
-    *var = strtoul(buf, nullptr, 10);
+    *var = strtoul(buf, 0, 10);
     return length;
   } else if (default_val) {
     *var = default_val;
@@ -584,7 +586,7 @@ long init_ulongvar_from_file(ulong *var, IO_CACHE *f, ulong default_val) {
 long init_floatvar_from_file(float *var, IO_CACHE *f, float default_val) {
   /*
     64 bytes provide enough space considering that the precision is 3
-    bytes (See the appropriate set function):
+    bytes (See the appropriate set funciton):
 
     FLT_MAX  The value of this macro is the maximum number representable
              in type float. It is supposed to be at least 1E+37.
@@ -644,7 +646,7 @@ long init_dynarray_intvar_from_file(char *buffer, size_t size,
   }
   if (read_size + 1 == size && buf[size - 2] != '\n') {
     /*
-      short read happened; allocate sufficient memory and make the 2nd read
+      short read happend; allocate sufficient memory and make the 2nd read
     */
     char buf_work[(sizeof(long) * 3 + 1) * 16];
     memcpy(buf_work, buf, sizeof(buf_work));
