@@ -1,16 +1,15 @@
-/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -50,9 +49,8 @@ enum enum_info_repository {
 */
 enum enum_return_check {
   REPOSITORY_DOES_NOT_EXIST = 1,
-  REPOSITORY_EXISTS = 2,
-  ERROR_CHECKING_REPOSITORY = 4,
-  REPOSITORY_CLEARED = 10
+  REPOSITORY_EXISTS,
+  ERROR_CHECKING_REPOSITORY
 };
 
 class Rpl_info_handler {
@@ -135,6 +133,9 @@ class Rpl_info_handler {
 
   /**
     Closes access to the repository.
+
+    @retval false No error
+    @retval true  Failure
   */
   void end_info() { do_end_info(); }
 
@@ -200,8 +201,8 @@ class Rpl_info_handler {
     set the value of a field pointed at @c pk_cursor to
     @ value.
 
-    @param[in]   pk_cursor   cursor for the field value.
-    @param[in]   value       field[pk_cursor] is set to
+    @param[in]   pk_cursor   cursor for the filed value.
+    @param[in]   value       fieled[pk_cursor] would be set
                              this value.
 
     @retval      false       ok
@@ -371,7 +372,7 @@ class Rpl_info_handler {
   enum_field_get_status prv_get_error;
   /*
    Keeps track of the number of events before fsyncing. The option
-   --sync-source-info and --sync-relay-log-info determine how many
+   --sync-master-info and --sync-relay-log-info determine how many
    events should be processed before fsyncing.
   */
   uint sync_counter;
@@ -439,8 +440,8 @@ class Rpl_info_handler {
 
 bool operator!(Rpl_info_handler::enum_field_get_status status);
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
 extern ulong w_rr;
-extern uint mta_debug_concurrent_access;
+extern uint mts_debug_concurrent_access;
 #endif
 #endif /* RPL_INFO_HANDLER_H */
