@@ -1,16 +1,15 @@
-/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,7 +48,7 @@ class Rpl_info_table : public Rpl_info_handler {
   friend class Rpl_info_factory;
 
  public:
-  ~Rpl_info_table() override;
+  virtual ~Rpl_info_table();
 
  private:
   /**
@@ -66,7 +65,7 @@ class Rpl_info_table : public Rpl_info_handler {
 
   /**
     This property represents a description of the repository.
-    Specifically, "schema"."table".
+    Speciffically, "schema"."table".
   */
   char *description;
 
@@ -94,28 +93,15 @@ class Rpl_info_table : public Rpl_info_handler {
   */
   bool is_transactional;
 
-  int do_init_info() override;
-  int do_init_info(uint instance) override;
+  int do_init_info();
+  int do_init_info(uint instance);
   int do_init_info(enum_find_method method, uint instance);
-  enum_return_check do_check_info() override;
-  enum_return_check do_check_info(uint instance) override;
-  void do_end_info() override;
-
-  /**
-    Flushes and syncs in-memory information into a stable storage.
-
-    @param[in] force  If enabled ignore syncing after flushing options such as
-                      relay-log-info-sync and master-info-sync and always sync
-
-    @retval 0         Success
-    @retval nonzero   Failure  This can happen if there is an error writing the
-                               table, or if replica_preserve_commit_order is
-                               enabled and a previous transaction has failed. In
-                               both cases, the error has been reported already.
-  */
-  int do_flush_info(const bool force) override;
-  int do_remove_info() override;
-  int do_clean_info() override;
+  enum_return_check do_check_info();
+  enum_return_check do_check_info(uint instance);
+  void do_end_info();
+  int do_flush_info(const bool force);
+  int do_remove_info();
+  int do_clean_info();
   /**
     Returns the number of entries in the table identified by:
     param_schema.param_table.
@@ -132,43 +118,19 @@ class Rpl_info_table : public Rpl_info_handler {
   */
   static bool do_count_info(uint nparam, const char *param_schema,
                             const char *param_table,
-                            MY_BITMAP const *nullable_bitmap,
-                            ulonglong *counter);
-  /**
-    Returns if the table is being used, meaning it contains at least a
-    line or some concurrency related error was returned when looking at
-    the table identified by: param_schema.param_table
-
-    @param[in]  nparam           Number of fields in the table.
-    @param[in]  param_schema     Table's schema.
-    @param[in]  param_table      Table's name.
-    @param[in]  nullable_bitmap  bitmap that holds the fields that are
-                                 allowed to be `NULL`-
-
-    @retval a pair of booleans
-            First element is true if an error occurred, false otherwise.
-            Second element is true if the table is not empty or an access error
-            occurred meaning someone else is accessing it. False if the table
-            is empty.
-  */
-  static std::pair<bool, bool> table_in_use(uint nparam,
-                                            const char *param_schema,
-                                            const char *param_table,
-                                            MY_BITMAP const *nullable_bitmap);
-
+                            MY_BITMAP const *nullable_bitmap, uint *counter);
   static int do_reset_info(uint nparam, const char *param_schema,
                            const char *param_table, const char *channel_name,
                            MY_BITMAP const *nullable_bitmap);
-  int do_prepare_info_for_read() override;
-  int do_prepare_info_for_write() override;
+  int do_prepare_info_for_read();
+  int do_prepare_info_for_write();
 
-  bool do_set_info(const int pos, const char *value) override;
-  bool do_set_info(const int pos, const uchar *value,
-                   const size_t size) override;
-  bool do_set_info(const int pos, const int value) override;
-  bool do_set_info(const int pos, const ulong value) override;
-  bool do_set_info(const int pos, const float value) override;
-  bool do_set_info(const int pos, const Server_ids *value) override;
+  bool do_set_info(const int pos, const char *value);
+  bool do_set_info(const int pos, const uchar *value, const size_t size);
+  bool do_set_info(const int pos, const int value);
+  bool do_set_info(const int pos, const ulong value);
+  bool do_set_info(const int pos, const float value);
+  bool do_set_info(const int pos, const Server_ids *value);
   /**
     Setter needed to set nullable fields to `NULL`.
 
@@ -177,7 +139,7 @@ class Rpl_info_table : public Rpl_info_handler {
 
     @return true if there was an error and false otherwise.
    */
-  bool do_set_info(const int pos, const std::nullptr_t value) override;
+  bool do_set_info(const int pos, const std::nullptr_t value);
   /**
     Setter needed to set nullable fields to `NULL`.
 
@@ -188,27 +150,25 @@ class Rpl_info_table : public Rpl_info_handler {
     @return true if there was an error and false otherwise.
    */
   bool do_set_info(const int pos, const std::nullptr_t value,
-                   const size_t size) override;
+                   const size_t size);
   Rpl_info_handler::enum_field_get_status do_get_info(
-      const int pos, char *value, const size_t size,
-      const char *default_value) override;
+      const int pos, char *value, const size_t size, const char *default_value);
   Rpl_info_handler::enum_field_get_status do_get_info(
       const int pos, uchar *value, const size_t size,
-      const uchar *default_value) override;
+      const uchar *default_value);
+  Rpl_info_handler::enum_field_get_status do_get_info(const int pos, int *value,
+                                                      const int default_value);
   Rpl_info_handler::enum_field_get_status do_get_info(
-      const int pos, int *value, const int default_value) override;
+      const int pos, ulong *value, const ulong default_value);
   Rpl_info_handler::enum_field_get_status do_get_info(
-      const int pos, ulong *value, const ulong default_value) override;
+      const int pos, float *value, const float default_value);
   Rpl_info_handler::enum_field_get_status do_get_info(
-      const int pos, float *value, const float default_value) override;
-  Rpl_info_handler::enum_field_get_status do_get_info(
-      const int pos, Server_ids *value,
-      const Server_ids *default_value) override;
-  char *do_get_description_info() override;
+      const int pos, Server_ids *value, const Server_ids *default_value);
+  char *do_get_description_info();
 
-  bool do_is_transactional() override;
-  bool do_update_is_transactional() override;
-  uint do_get_rpl_info_type() override;
+  bool do_is_transactional();
+  bool do_update_is_transactional();
+  uint do_get_rpl_info_type();
 
   /**
     Verify if the table primary key fields are at the expected (column)
