@@ -75,7 +75,7 @@
 #include "typelib.h"
 
 int max_binlog_dump_events = 0;  // unlimited
-bool opt_sporadic_binlog_dump_fail = false;
+bool opt_sporadic_binlog_dump_fail = 0;
 
 malloc_unordered_map<uint32, unique_ptr_my_free<SLAVE_INFO>> slave_list{
     key_memory_SLAVE_INFO};
@@ -144,7 +144,7 @@ int register_slave(THD *thd, uchar *packet, size_t packet_length) {
   uchar *p = packet, *p_end = packet + packet_length;
   const char *errmsg = "Wrong parameters to function register_slave";
 
-  if (check_access(thd, REPL_SLAVE_ACL, any_db, nullptr, nullptr, false, false))
+  if (check_access(thd, REPL_SLAVE_ACL, any_db, nullptr, nullptr, 0, 0))
     return 1;
 
   unique_ptr_my_free<SLAVE_INFO> si((SLAVE_INFO *)my_malloc(
@@ -1303,7 +1303,7 @@ bool show_binlogs(THD *thd) {
 
   cur_dir_len = dirname_length(cur.log_file_name);
 
-  reinit_io_cache(index_file, READ_CACHE, (my_off_t)0, false, false);
+  reinit_io_cache(index_file, READ_CACHE, (my_off_t)0, 0, 0);
 
   /* The file ends with EOF or empty line */
   while ((length = my_b_gets(index_file, fname, sizeof(fname))) > 1) {
