@@ -1,16 +1,15 @@
-/* Copyright (c) 2006, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,8 +32,8 @@
 #include "my_dbug.h"
 #include "my_loglevel.h"
 #include "my_sys.h"
-#include "mysql/components/services/bits/psi_stage_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/components/services/psi_stage_bits.h"
 #include "mysql_version.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"
@@ -43,8 +42,8 @@
 #include "sql/log.h"
 #include "sql/mysqld.h"  // sync_masterinfo_period
 #include "sql/rpl_info_handler.h"
-#include "sql/rpl_msr.h"      // channel_map
-#include "sql/rpl_replica.h"  // master_retry_count
+#include "sql/rpl_msr.h"    // channel_map
+#include "sql/rpl_slave.h"  // master_retry_count
 #include "sql/sql_class.h"
 
 enum {
@@ -98,28 +97,19 @@ enum {
   /* line for master_zstd_compression_level */
   LINE_FOR_MASTER_ZSTD_COMPRESSION_LEVEL = 30,
 
-  /* line for tls_ciphersuites */
-  LINE_FOR_TLS_CIPHERSUITES = 31,
-
-  /* line for source_connection_auto_failover */
-  LINE_FOR_SOURCE_CONNECTION_AUTO_FAILOVER = 32,
-
-  /* line for gtid_only */
-  LINE_FOR_GTID_ONLY = 33,
-
   /* Number of lines currently used when saving master info file */
-  LINES_IN_MASTER_INFO = LINE_FOR_GTID_ONLY
+  LINES_IN_MASTER_INFO = LINE_FOR_MASTER_ZSTD_COMPRESSION_LEVEL
 
 };
 
 /*
-  Please every time you add a new field to the master info, update
+  Please every time you add a new field to the mater info, update
   what follows. For now, this is just used to get the number of
   fields.
 */
 const char *info_mi_fields[] = {"number_of_lines",
-                                "source_log_name",
-                                "source_log_pos",
+                                "master_log_name",
+                                "master_log_pos",
                                 "host",
                                 "user",
                                 "password",
@@ -145,11 +135,8 @@ const char *info_mi_fields[] = {"number_of_lines",
                                 "public_key_path",
                                 "get_public_key",
                                 "network_namespace",
-                                "source_compression_algorithm",
-                                "source_zstd_compression_level",
-                                "tls_ciphersuites",
-                                "source_connection_auto_failover",
-                                "gtid_only"};
+                                "master_compression_algorithm",
+                                "master_zstd_compression_level"};
 
 const uint info_mi_table_pk_field_indexes[] = {
     LINE_FOR_CHANNEL - 1,
@@ -182,8 +169,8 @@ Master_info::Master_info(
       key_info_rotate_lock(param_key_info_rotate_lock),
       key_info_rotate_cond(param_key_info_rotate_cond),
 #endif
-      ssl(false),
-      ssl_verify_server_cert(false),
+      ssl(0),
+      ssl_verify_server_cert(0),
       get_public_key(false),
       port(MYSQL_PORT),
       connect_retry(DEFAULT_CONNECT_RETRY),
@@ -196,11 +183,7 @@ Master_info::Master_info(
       retry_count(master_retry_count),
       mi_description_event(nullptr),
       auto_position(false),
-      transaction_parser(
-          Transaction_boundary_parser::TRX_BOUNDARY_PARSER_RECEIVER),
-      reset(false),
-      m_gtid_only_mode(false),
-      m_is_receiver_position_info_invalid(false) {
+      reset(false) {
   host[0] = 0;
   user[0] = 0;
   bind_addr[0] = 0;
@@ -223,7 +206,10 @@ Master_info::Master_info(
   ignore_server_ids = new Server_ids;
   strcpy(compression_algorithm, COMPRESSION_ALGORITHM_UNCOMPRESSED);
   zstd_compression_level = default_zstd_compression_level;
-  net_server_ext_init(&server_extn);
+  server_extn.m_user_data = nullptr;
+  server_extn.m_before_header = nullptr;
+  server_extn.m_after_header = nullptr;
+  server_extn.compress_ctx.algorithm = MYSQL_UNCOMPRESSED;
   gtid_monitoring_info = new Gtid_monitoring_info(&data_lock);
 
   mysql_mutex_init(*key_info_rotate_lock, &this->rotate_lock,
@@ -273,7 +259,8 @@ void Master_info::request_rotate(THD *thd) {
          transaction to end.
     */
     const char dbug_signal[] = "now SIGNAL signal.rpl_requested_for_a_flush";
-    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(dbug_signal)));
+    DBUG_ASSERT(
+        !debug_sync_set_action(current_thd, STRING_WITH_LEN(dbug_signal)));
   });
 
   while (this->rotate_requested.load() && !thd->killed)
@@ -296,7 +283,8 @@ void Master_info::clear_rotate_requests() {
            the deferred flushing of the relay log.
       */
       const char dbug_signal[] = "now SIGNAL signal.rpl_broadcasted_rotate_end";
-      assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(dbug_signal)));
+      DBUG_ASSERT(
+          !debug_sync_set_action(current_thd, STRING_WITH_LEN(dbug_signal)));
     });
   }
 }
@@ -339,7 +327,7 @@ void Master_info::end_info() {
 
   handler->end_info();
 
-  inited = false;
+  inited = 0;
   reset = true;
 }
 
@@ -363,7 +351,7 @@ void Master_info::end_info() {
 
 int Master_info::flush_info(bool force) {
   DBUG_TRACE;
-  DBUG_PRINT("enter", ("source_pos: %lu", (ulong)master_log_pos));
+  DBUG_PRINT("enter", ("master_pos: %lu", (ulong)master_log_pos));
 
   bool skip_flushing = !inited;
   /*
@@ -392,7 +380,7 @@ int Master_info::flush_info(bool force) {
   return 0;
 
 err:
-  LogErr(ERROR_LEVEL, ER_RPL_ERROR_WRITING_SOURCE_CONFIGURATION);
+  LogErr(ERROR_LEVEL, ER_RPL_ERROR_WRITING_MASTER_CONFIGURATION);
   return 1;
 }
 
@@ -420,7 +408,7 @@ int Master_info::mi_init_info() {
     if (read_info(handler)) goto err;
   }
 
-  inited = true;
+  inited = 1;
   reset = false;
   if (flush_info(true)) goto err;
 
@@ -428,8 +416,8 @@ int Master_info::mi_init_info() {
 
 err:
   handler->end_info();
-  inited = false;
-  LogErr(ERROR_LEVEL, ER_RPL_ERROR_READING_SOURCE_CONFIGURATION);
+  inited = 0;
+  LogErr(ERROR_LEVEL, ER_RPL_ERROR_READING_MASTER_CONFIGURATION);
   return 1;
 }
 
@@ -448,10 +436,8 @@ const uint *Master_info::get_table_pk_field_indexes() {
 
 void Master_info::set_nullable_fields(MY_BITMAP *nullable_fields) {
   bitmap_init(nullable_fields, nullptr,
-              Master_info::get_number_info_mi_fields());
+              Master_info::get_number_info_mi_fields(), false);
   bitmap_clear_all(nullable_fields);
-  /* Identify which fields can store a NULL value. */
-  bitmap_set_bit(nullable_fields, LINE_FOR_TLS_CIPHERSUITES - 1);
 }
 
 bool Master_info::read_info(Rpl_info_handler *from) {
@@ -499,9 +485,9 @@ bool Master_info::read_info(Rpl_info_handler *from) {
     lines = 7;
 
   if (!!from->get_info(&temp_master_log_pos, (ulong)BIN_LOG_HEADER_SIZE) ||
-      !!from->get_info(host, sizeof(host), (char *)nullptr) ||
-      !!from->get_info(user, sizeof(user), (char *)nullptr) ||
-      !!from->get_info(password, sizeof(password), (char *)nullptr) ||
+      !!from->get_info(host, sizeof(host), (char *)0) ||
+      !!from->get_info(user, sizeof(user), "test") ||
+      !!from->get_info(password, sizeof(password), (char *)0) ||
       !!from->get_info((int *)&port, (int)MYSQL_PORT) ||
       !!from->get_info((int *)&connect_retry, (int)DEFAULT_CONNECT_RETRY))
     return true;
@@ -514,11 +500,11 @@ bool Master_info::read_info(Rpl_info_handler *from) {
   */
   if (lines >= LINES_IN_MASTER_INFO_WITH_SSL) {
     if (!!from->get_info(&temp_ssl, 0) ||
-        !!from->get_info(ssl_ca, sizeof(ssl_ca), (char *)nullptr) ||
-        !!from->get_info(ssl_capath, sizeof(ssl_capath), (char *)nullptr) ||
-        !!from->get_info(ssl_cert, sizeof(ssl_cert), (char *)nullptr) ||
-        !!from->get_info(ssl_cipher, sizeof(ssl_cipher), (char *)nullptr) ||
-        !!from->get_info(ssl_key, sizeof(ssl_key), (char *)nullptr))
+        !!from->get_info(ssl_ca, sizeof(ssl_ca), (char *)0) ||
+        !!from->get_info(ssl_capath, sizeof(ssl_capath), (char *)0) ||
+        !!from->get_info(ssl_cert, sizeof(ssl_cert), (char *)0) ||
+        !!from->get_info(ssl_cipher, sizeof(ssl_cipher), (char *)0) ||
+        !!from->get_info(ssl_key, sizeof(ssl_key), (char *)0))
       return true;
   }
 
@@ -555,7 +541,7 @@ bool Master_info::read_info(Rpl_info_handler *from) {
 
   /* Starting from 5.5 the master_uuid may be in the repository. */
   if (lines >= LINE_FOR_MASTER_UUID) {
-    if (!!from->get_info(master_uuid, sizeof(master_uuid), (char *)nullptr))
+    if (!!from->get_info(master_uuid, sizeof(master_uuid), (char *)0))
       return true;
   }
 
@@ -566,8 +552,8 @@ bool Master_info::read_info(Rpl_info_handler *from) {
   }
 
   if (lines >= LINE_FOR_SSL_CRLPATH) {
-    if (!!from->get_info(ssl_crl, sizeof(ssl_crl), (char *)nullptr) ||
-        !!from->get_info(ssl_crlpath, sizeof(ssl_crlpath), (char *)nullptr))
+    if (!!from->get_info(ssl_crl, sizeof(ssl_crl), (char *)0) ||
+        !!from->get_info(ssl_crlpath, sizeof(ssl_crlpath), (char *)0))
       return true;
   }
 
@@ -576,18 +562,16 @@ bool Master_info::read_info(Rpl_info_handler *from) {
   }
 
   if (lines >= LINE_FOR_CHANNEL) {
-    if (!!from->get_info(channel, sizeof(channel), (char *)nullptr))
-      return true;
+    if (!!from->get_info(channel, sizeof(channel), (char *)0)) return true;
   }
 
   if (lines >= LINE_FOR_TLS_VERSION) {
-    if (!!from->get_info(tls_version, sizeof(tls_version), (char *)nullptr))
+    if (!!from->get_info(tls_version, sizeof(tls_version), (char *)0))
       return true;
   }
 
   if (lines >= LINE_FOR_PUBLIC_KEY_PATH) {
-    if (!!from->get_info(public_key_path, sizeof(public_key_path),
-                         (char *)nullptr))
+    if (!!from->get_info(public_key_path, sizeof(public_key_path), (char *)0))
       return true;
   }
 
@@ -597,7 +581,7 @@ bool Master_info::read_info(Rpl_info_handler *from) {
 
   if (lines >= LINE_FOR_NETWORK_NAMESPACE) {
     if (!!from->get_info(network_namespace, sizeof(network_namespace),
-                         (char *)nullptr))
+                         (char *)0))
       return true;
   }
 
@@ -607,6 +591,10 @@ bool Master_info::read_info(Rpl_info_handler *from) {
   auto_position = temp_auto_position;
   get_public_key = (bool)temp_get_public_key;
 
+#ifndef HAVE_OPENSSL
+  if (ssl) LogErr(WARNING_LEVEL, ER_RPL_SSL_INFO_IN_MASTER_INFO_IGNORED);
+#endif /* HAVE_OPENSSL */
+
   if (lines >= LINE_FOR_MASTER_COMPRESSION_ALGORITHM) {
     char algorithm_name[COMPRESSION_ALGORITHM_NAME_BUFFER_SIZE];
     if (!!from->get_info(algorithm_name, sizeof(algorithm_name), nullptr))
@@ -617,7 +605,7 @@ bool Master_info::read_info(Rpl_info_handler *from) {
                algorithm_name, channel);
         strcpy(compression_algorithm, COMPRESSION_ALGORITHM_UNCOMPRESSED);
       } else {
-        assert(strlen(algorithm_name) < sizeof(compression_algorithm));
+        DBUG_ASSERT(strlen(algorithm_name) < sizeof(compression_algorithm));
         strcpy(compression_algorithm, algorithm_name);
       }
     }
@@ -635,41 +623,6 @@ bool Master_info::read_info(Rpl_info_handler *from) {
       zstd_compression_level = default_level;
     }
   }
-
-  if (lines >= LINE_FOR_TLS_CIPHERSUITES) {
-    char buffer[FN_REFLEN_SE] = {0};
-
-    Rpl_info_handler::enum_field_get_status status =
-        from->get_info(buffer, sizeof(buffer), nullptr);
-
-    if (status == Rpl_info_handler::enum_field_get_status::FAILURE) return true;
-
-    if (status ==
-        Rpl_info_handler::enum_field_get_status::FIELD_VALUE_IS_NULL) {
-      tls_ciphersuites.first = true;
-      tls_ciphersuites.second.clear();
-    } else {
-      assert(status ==
-             Rpl_info_handler::enum_field_get_status::FIELD_VALUE_NOT_NULL);
-      tls_ciphersuites.first = false;
-      tls_ciphersuites.second.assign(buffer);
-    }
-  }
-
-  if (lines >= LINE_FOR_SOURCE_CONNECTION_AUTO_FAILOVER) {
-    auto temp_source_connection_auto_failover{0};
-    if (!!from->get_info(&temp_source_connection_auto_failover, 0)) return true;
-    m_source_connection_auto_failover = temp_source_connection_auto_failover;
-  }
-
-  auto temp_gtid_only{0};
-  if (lines >= LINE_FOR_GTID_ONLY) {
-    if (!!from->get_info(&temp_gtid_only, 0)) return true;
-  } else {
-    if (channel_map.is_group_replication_channel_name(channel))
-      temp_gtid_only = 1;
-  }
-  m_gtid_only_mode = temp_gtid_only;
   return false;
 }
 
@@ -706,11 +659,7 @@ bool Master_info::write_info(Rpl_info_handler *to) {
       to->set_info(channel) || to->set_info(tls_version) ||
       to->set_info(public_key_path) || to->set_info(get_public_key) ||
       to->set_info(network_namespace) || to->set_info(compression_algorithm) ||
-      to->set_info((int)zstd_compression_level) ||
-      to->set_info(tls_ciphersuites.first ? nullptr
-                                          : tls_ciphersuites.second.c_str()) ||
-      to->set_info((int)m_source_connection_auto_failover) ||
-      to->set_info((int)m_gtid_only_mode))
+      to->set_info((int)zstd_compression_level))
     return true;
 
   return false;
@@ -719,7 +668,7 @@ bool Master_info::write_info(Rpl_info_handler *to) {
 void Master_info::set_password(const char *password_arg) {
   DBUG_TRACE;
 
-  assert(password_arg);
+  DBUG_ASSERT(password_arg);
 
   if (password_arg && start_user_configured)
     strmake(start_password, password_arg, sizeof(start_password) - 1);
@@ -762,11 +711,6 @@ void Master_info::channel_wrlock() {
   m_channel_lock->wrlock();
 }
 
-int Master_info::channel_trywrlock() {
-  channel_map.assert_some_lock();
-  return m_channel_lock->trywrlock();
-}
-
 void Master_info::wait_until_no_reference(THD *thd) {
   PSI_stage_info *old_stage = nullptr;
 
@@ -798,17 +742,3 @@ void Master_info::get_flushed_relay_log_info(LOG_INFO *linfo) {
           sizeof(linfo->log_file_name) - 1);
   linfo->pos = flushed_relay_log_info.pos;
 }
-
-void Master_info::set_receiver_position_info_invalid(bool invalid) {
-  m_is_receiver_position_info_invalid = invalid;
-}
-
-bool Master_info::is_receiver_position_info_invalid() const {
-  return m_is_receiver_position_info_invalid;
-}
-
-void Master_info::set_gtid_only_mode(bool gtid_only_mode) {
-  m_gtid_only_mode = gtid_only_mode;
-}
-
-bool Master_info::is_gtid_only_mode() const { return m_gtid_only_mode; }
