@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -48,7 +47,7 @@ bool Multisource_info::add_mi(const char *channel_name, Master_info *mi) {
   bool res = false;
 
   /* The check of mi exceeding MAX_CHANNELS shall be done in the caller */
-  assert(current_mi_count < MAX_CHANNELS);
+  DBUG_ASSERT(current_mi_count < MAX_CHANNELS);
 
   replication_channel_map::iterator map_it;
   enum_channel_type type = is_group_replication_channel_name(channel_name)
@@ -88,7 +87,7 @@ Master_info *Multisource_info::get_mi(const char *channel_name) {
 
   m_channel_map_lock->assert_some_lock();
 
-  assert(channel_name != nullptr);
+  DBUG_ASSERT(channel_name != 0);
 
   mi_map::iterator it;
   replication_channel_map::iterator map_it;
@@ -103,18 +102,18 @@ Master_info *Multisource_info::get_mi(const char *channel_name) {
       it == map_it->second.end()) {
     map_it = rep_channel_map.find(GROUP_REPLICATION_CHANNEL);
     if (map_it == rep_channel_map.end()) {
-      return nullptr;
+      return 0;
     }
     it = map_it->second.find(channel_name);
     if (it == map_it->second.end()) {
-      return nullptr;
+      return 0;
     }
   }
 
   return it->second;
 }
 
-bool Multisource_info::delete_mi(const char *channel_name) {
+void Multisource_info::delete_mi(const char *channel_name) {
   DBUG_TRACE;
 
   m_channel_map_lock->assert_some_wrlock();
@@ -122,7 +121,7 @@ bool Multisource_info::delete_mi(const char *channel_name) {
   Master_info *mi = nullptr;
   mi_map::iterator it;
 
-  assert(channel_name != nullptr);
+  DBUG_ASSERT(channel_name != 0);
 
   replication_channel_map::iterator map_it;
   map_it = rep_channel_map.find(SLAVE_REPLICATION_CHANNEL);
@@ -134,17 +133,10 @@ bool Multisource_info::delete_mi(const char *channel_name) {
           rep_channel_map.end() ||  // If not a slave channel, maybe a group one
       it == map_it->second.end()) {
     map_it = rep_channel_map.find(GROUP_REPLICATION_CHANNEL);
-    assert(map_it != rep_channel_map.end());
+    DBUG_ASSERT(map_it != rep_channel_map.end());
 
-    if (map_it != rep_channel_map.end()) {
-      it = map_it->second.find(channel_name);
-      assert(it != map_it->second.end());
-    }
-  }
-
-  if (map_it == rep_channel_map.end() || it == map_it->second.end()) {
-    // the channel identified by channel_name could not be found
-    return true;
+    it = map_it->second.find(channel_name);
+    DBUG_ASSERT(it != map_it->second.end());
   }
 
 #ifdef WITH_PERFSCHEMA_STORAGE_ENGINE
@@ -152,10 +144,10 @@ bool Multisource_info::delete_mi(const char *channel_name) {
   /* get the index of mi from rpl_pfs_mi */
   index = get_index_from_rpl_pfs_mi(channel_name);
 
-  assert(index != -1);
+  DBUG_ASSERT(index != -1);
 
   /* set the current index to  0  and decrease current_mi_count */
-  rpl_pfs_mi[index] = nullptr;
+  rpl_pfs_mi[index] = 0;
 #endif
 
   current_mi_count--;
@@ -177,7 +169,6 @@ bool Multisource_info::delete_mi(const char *channel_name) {
     }
     delete mi;
   }
-  return false;
 }
 
 bool Multisource_info::is_group_replication_channel_name(const char *channel,
@@ -200,7 +191,7 @@ bool Multisource_info::add_mi_to_rpl_pfs_mi(Master_info *mi) {
 
   /* Point to this added mi in the rpl_pfs_mi*/
   for (uint i = 0; i < MAX_CHANNELS; i++) {
-    if (rpl_pfs_mi[i] == nullptr) {
+    if (rpl_pfs_mi[i] == 0) {
       rpl_pfs_mi[i] = mi;
       res = false;  // success
       break;
@@ -229,7 +220,7 @@ Master_info *Multisource_info::get_mi_at_pos(uint pos) {
 
   if (pos < MAX_CHANNELS) return rpl_pfs_mi[pos];
 
-  return nullptr;
+  return 0;
 }
 
 Rpl_filter *Rpl_channel_filters::create_filter(const char *channel_name) {
@@ -243,7 +234,7 @@ Rpl_filter *Rpl_channel_filters::create_filter(const char *channel_name) {
 
   m_channel_to_filter_lock->wrlock();
   it = channel_to_filter.find(channel_name);
-  assert(it == channel_to_filter.end());
+  DBUG_ASSERT(it == channel_to_filter.end());
   ret = channel_to_filter.insert(
       std::pair<std::string, Rpl_filter *>(channel_name, rpl_filter));
 #ifdef WITH_PERFSCHEMA_STORAGE_ENGINE
@@ -336,7 +327,7 @@ Rpl_filter *Rpl_channel_filters::get_channel_filter(const char *channel_name) {
   filter_map::iterator it;
   Rpl_filter *rpl_filter = nullptr;
 
-  assert(channel_name != nullptr);
+  DBUG_ASSERT(channel_name != 0);
 
   m_channel_to_filter_lock->rdlock();
   it = channel_to_filter.find(channel_name);
@@ -398,12 +389,12 @@ bool Rpl_channel_filters::build_do_and_ignore_table_hashes() {
     if (it->second->build_do_table_hash() ||
         it->second->build_ignore_table_hash()) {
       LogErr(ERROR_LEVEL, ER_FAILED_TO_BUILD_DO_AND_IGNORE_TABLE_HASHES);
-      return true;
+      return -1;
     }
   }
   m_channel_to_filter_lock->unlock();
 
-  return false;
+  return 0;
 }
 
 #endif /*WITH_PERFSCHEMA_STORAGE_ENGINE */
