@@ -1,16 +1,15 @@
-/* Copyright (c) 2007, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,17 +25,14 @@
 #include <stddef.h>
 #include <algorithm>
 
-#include "field_types.h"
 #include "lex_string.h"
 #include "m_ctype.h"
-#include "m_string.h"
 #include "my_bitmap.h"  // MY_BITMAP
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_sys.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
-#include "sql/changestreams/misc/replicated_columns_view_factory.h"  // get_columns_view
 #include "sql/current_thd.h"
 #include "sql/debug_sync.h"
 #include "sql/derror.h"  // ER_THD
@@ -44,9 +40,9 @@
 #include "sql/log_event.h"
 #include "sql/rpl_rli.h"      // Relay_log_info
 #include "sql/rpl_utility.h"  // table_def
+#include "sql/sql_class.h"
 #include "sql/sql_const.h"
 #include "sql/sql_error.h"
-#include "sql/sql_gipk.h"  // table_has_generated_invisible_primary_key
 #include "sql/system_variables.h"
 #include "sql/table.h"  // TABLE
 #include "sql_string.h"
@@ -57,20 +53,18 @@ class Json_diff_vector;
 using std::max;
 using std::min;
 
-#ifndef NDEBUG
 template <typename T, typename UT>
-void Bit_stream_base<T, UT>::dbug_print(const char *str) const {
+void Bit_stream_base<T, UT>::dbug_print(
+    const char *str MY_ATTRIBUTE((unused))) const {
+#ifndef DBUG_OFF
   StringBuffer<STRING_BUFFER_USUAL_SIZE> s;
   s.reserve(static_cast<size_t>(m_current_bit + 1));
   for (uint i = 0; i < m_current_bit; i++)
     s.append((m_ptr[i / 8] & (1 << (i % 8))) != 0 ? '1' : '0');
   s.append("\0", 1);
   DBUG_PRINT("info", ("%s: %u bits: %s", str, m_current_bit, s.ptr()));
-}
-#else
-template <typename T, typename UT>
-void Bit_stream_base<T, UT>::dbug_print(const char *) const {}
 #endif
+}
 
 /**
   Write a single field (column) of a row in a binary log row event to the
@@ -136,8 +130,8 @@ static void pack_field(uchar **pack_ptr, Field *field, size_t rec_offset,
     }
     DBUG_PRINT("info", ("stored in full format"));
   }
-  *pack_ptr = field->pack_with_metadata_bytes(
-      *pack_ptr, field->field_ptr() + rec_offset, field->max_data_length());
+  *pack_ptr = field->pack(*pack_ptr, field->ptr + rec_offset,
+                          field->max_data_length(), true);
 }
 
 /**
@@ -224,7 +218,7 @@ static bool unpack_field(const uchar **pack_ptr, Field *field, uint metadata,
         table->disable_logical_diffs_for_current_row(field);
     }
 
-    *pack_ptr = field->unpack(field->field_ptr(), *pack_ptr, metadata);
+    *pack_ptr = field->unpack(field->ptr, *pack_ptr, metadata, true);
   }
 
   return false;
@@ -271,24 +265,23 @@ size_t pack_row(TABLE *table, MY_BITMAP const *columns_in_image,
                 uchar *row_data, const uchar *record,
                 enum_row_image_type row_image_type, ulonglong value_options) {
   DBUG_TRACE;
-  cs::util::ReplicatedColumnsView fields{table};
-  fields.add_filter(
-      cs::util::ColumnFilterFactory::ColumnFilterType::outbound_func_index);
 
   // Since we don't want any hidden generated columns to be included in the
   // binlog, we must clear any bits for these columns in the bitmap. We will
   // use TABLE::pack_row_tmp_set for this purpose, so first we ensure that it
   // isn't in use somewhere else.
-  assert(bitmap_is_clear_all(&table->pack_row_tmp_set));
+  DBUG_ASSERT(bitmap_is_clear_all(&table->pack_row_tmp_set));
 
   // Copy all the bits from the "columns_in_image", and clear all the bits for
   // hidden generated columns.
   bitmap_copy(&table->pack_row_tmp_set, columns_in_image);
-  bitmap_intersect(&table->pack_row_tmp_set,
-                   &fields.get_included_fields_bitmap());
+  bitmap_subtract(&table->pack_row_tmp_set,
+                  &table->fields_for_functional_indexes);
 
   // Number of columns in image (counting only those that will be written)
   uint image_column_count = bitmap_bits_set(&table->pack_row_tmp_set);
+  // Number of columns in table (counting even those that will not be written)
+  uint table_column_count = table->pack_row_tmp_set.n_bits;
 
   ptrdiff_t const rec_offset = record - table->record[0];
 
@@ -307,7 +300,8 @@ size_t pack_row(TABLE *table, MY_BITMAP const *columns_in_image,
   bool has_any_json_diff = false;
   if ((value_options & PARTIAL_JSON_UPDATES) != 0 &&
       row_image_type == enum_row_image_type::UPDATE_AI) {
-    for (auto field : fields) {
+    for (uint field_i = 0; field_i < table_column_count; field_i++) {
+      const Field *field = table->field[field_i];
       if (field->type() == MYSQL_TYPE_JSON) {
         // Include every JSON column in the count.
         json_column_count++;
@@ -316,7 +310,7 @@ size_t pack_row(TABLE *table, MY_BITMAP const *columns_in_image,
         // needed for columns in the after-image, and of course only
         // when has_any_json_diff has not yet been set.
         if (!has_any_json_diff &&
-            bitmap_is_set(&table->pack_row_tmp_set, field->field_index())) {
+            bitmap_is_set(&table->pack_row_tmp_set, field_i)) {
           const Field_json *field_json = down_cast<const Field_json *>(field);
           const Json_diff_vector *diff_vector;
           field_json->get_diff_vector_and_length(value_options, &diff_vector);
@@ -348,9 +342,10 @@ size_t pack_row(TABLE *table, MY_BITMAP const *columns_in_image,
   Bit_writer null_bits(pack_ptr);
   pack_ptr += (image_column_count + 7) / 8;
 
-  for (auto field : fields) {
+  for (Field **p_field = table->field; *p_field != nullptr; p_field++) {
+    Field *field = *p_field;
     bool is_partial_json = false;
-    if (bitmap_is_set(&table->pack_row_tmp_set, field->field_index())) {
+    if (bitmap_is_set(&table->pack_row_tmp_set, p_field - table->field)) {
       if (field->is_null(rec_offset)) {
         null_bits.set(true);
         DBUG_PRINT("info", ("field %s: NULL", field->field_name));
@@ -358,7 +353,7 @@ size_t pack_row(TABLE *table, MY_BITMAP const *columns_in_image,
         null_bits.set(false);
 
         // Store the field when it is not NULL.
-#ifndef NDEBUG
+#ifndef DBUG_OFF
         const uchar *old_pack_ptr = pack_ptr;
 #endif
         pack_field(&pack_ptr, field, rec_offset, row_image_type, value_options,
@@ -370,7 +365,7 @@ size_t pack_row(TABLE *table, MY_BITMAP const *columns_in_image,
         DBUG_DUMP("rbr", old_pack_ptr, pack_ptr - old_pack_ptr);
       }
     }
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     else {
       DBUG_PRINT("info", ("field %s: skipped", field->field_name));
     }
@@ -383,14 +378,14 @@ size_t pack_row(TABLE *table, MY_BITMAP const *columns_in_image,
     }
   }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   DBUG_PRINT("info", ("partial_bits.tell()=%u, null_bits.tell()=%u",
                       partial_bits.tell(), null_bits.tell()));
   if (has_any_json_diff)
-    assert(partial_bits.tell() == json_column_count);
+    DBUG_ASSERT(partial_bits.tell() == json_column_count);
   else
-    assert(partial_bits.tell() == 0);
-  assert(null_bits.tell() == image_column_count);
+    DBUG_ASSERT(partial_bits.tell() == 0);
+  DBUG_ASSERT(null_bits.tell() == image_column_count);
   null_bits.dbug_print("null_bits");
   partial_bits.dbug_print("partial_bits");
   DBUG_DUMP("rbr", row_data, pack_ptr - row_data);
@@ -431,7 +426,7 @@ static const uchar *start_partial_bit_reader(const uchar *pack_ptr,
   if (net_field_length_checked<ulonglong>(&pack_ptr, &length,
                                           event_value_options) ||
       *event_value_options > 1) {
-    my_error(ER_REPLICA_CORRUPT_EVENT, MYF(0));
+    my_error(ER_SLAVE_CORRUPT_EVENT, MYF(0));
     return nullptr;
   }
   DBUG_PRINT("info", ("event_value_options=%llx", *event_value_options));
@@ -445,57 +440,174 @@ static const uchar *start_partial_bit_reader(const uchar *pack_ptr,
   return pack_ptr;
 }
 
+/**
+  Unpack a row image (either before-image or after-image) into @c
+  table->record[0].
+
+  The row is assumed to only consist of the fields for which the
+  corresponding bit in bitset @c column_image is set; the other parts
+  of the record are left alone.
+
+  If the slave table has more columns than the master table, then the
+  extra columns are not touched by this function.  If the master table
+  has more columns than the slave table, then the position is moved to
+  after the extra columns, but the values are not used.
+
+  - The layout of a row is:
+
+    For WRITE_ROWS_EVENT:
+        +--------------+
+        | after-image |
+        +--------------+
+
+    For DELETE_ROWS_EVENT:
+        +--------------+
+        | before-image |
+        +--------------+
+
+    For UPDATE_ROWS_EVENT:
+        +--------------+-------------+
+        | before-image | after-image |
+        +--------------+-------------+
+
+    For PARTIAL_UPDATE_ROWS_EVENT:
+        +--------------+--------------+-------------+
+        | before-image | shared-image | after-image |
+        +--------------+--------------+-------------+
+
+  - Each of before-image and after-image has the following format:
+        +--------+-------+-------+     +-------+
+        | length | col_1 | col_2 | ... | col_N |
+        +--------+-------+-------+     +-------+
+    length is a 4-byte integer in little-endian format, equal to the
+    total length in bytes of col_1, col_2, ..., col_N.
+
+  - The shared-image has one of the following formats:
+        +-----------------+
+        | value_options=0 |
+        +-----------------+
+    or
+        +-----------------+--------------+
+        | value_options=1 | partial_bits |
+        +-----------------+--------------+
+    where:
+
+    - value_options is a bitmap, stored as an integer, in the format
+      of net_field_length. Currently only one bit is allowed:
+      1=PARTIAL_JSON_UPDATES (so therefore the integer is always 0 or
+      1, so in reality value_options is only one byte).  When
+      PARTIAL_JSON_UPDATES=0, there is nothing else in the
+      shared-image.  When PARTIAL_JSON_UPDATES=1, there is a
+      partial_bits field.
+
+    - partial_bits has one bit for each *JSON* column in the table
+      (regardless of whether it is included in the before-image and/or
+      after-image).  The bit is 0 if the JSON update is stored as a
+      full document in the after-image, and 1 if the JSON update in
+      partial form in the after-image.
+
+    - Both when reading the before-image and when reading the
+      after-image it is necessary to know the partialness of JSON
+      columns: when reading the before-image, before looking up the
+      row in the table, we need to set the column in the table's
+      'read_set' (even if the column was not in the before-image), in
+      order to guarantee that the storage engine reads that column, so
+      that there is any base document that the diff can be applied
+      on. When reading the after-image, we need to know which columns
+      are partial so that we can correctly parse the data for that
+      column.
+
+      Therefore, when this function parses the before-image of a
+      PARTIAL_UPDATE_ROWS_LOG_EVENT, it reads both the before-image
+      and the shared-image, but leaves the read position after the
+      before-image.  So when it parses the after-image of a
+      PARTIAL_UPDATE_ROWS_LOG_EVENT, the read position is at the
+      beginning of the shared-image, so it parses both the
+      shared-image and the after-image.
+
+  @param[in] rli Applier execution context
+
+  @param[in,out] table Table to unpack into
+
+  @param[in] master_column_count Number of columns that the master had
+  in its table
+
+  @param[in] row_data Beginning of the row image
+
+  @param[in] column_image Pointer to a bit vector where the N'th bit
+  is 0 for columns that are not included in the event, and 1 for
+  columns that are included in the event.
+
+  @param[out] row_image_end_p If this function returns successfully, it
+  sets row_image_end to point to the next byte after the row image
+  that it has read.
+
+  @param[in] event_end Pointer to the end of the event.
+
+  @param[in] row_image_type The type of row image that we are going to
+  read: WRITE_AI, UPDATE_BI, UPDATE_AI, or DELETE_BI.
+
+  @param[in] event_has_value_options true for PARTIAL_UPDATE_ROWS_EVENT,
+  false for UPDATE_ROWS_EVENT.
+
+  @param only_seek If true, this is a seek operation rather than a
+  read operation.  It will only compute the row_image_end_p pointer,
+  and not read anything into the table and not apply any JSON diffs.
+  (This is used in slave_rows_search_algorithms=HASH_SCAN, which (1)
+  unpacks and hashes the before-image for all rows in the event, (2)
+  scans the table, and for each matching row it (3) unpacks the
+  after-image and applies on the table. In step (1) it needs to unpack
+  the after-image too, in order to move the read position forwards,
+  and then it should use only_seek=true.  This is an optimization, but
+  more importantly, when the after-image contains partial JSON, the
+  partial JSON cannot be applied in step (1) since there is no JSON
+  document to apply it on.)
+
+  @returns false on success, true on error.
+ */
 bool unpack_row(Relay_log_info const *rli, TABLE *table,
-                uint const source_column_count, uchar const *const row_data,
+                uint const master_column_count, uchar const *const row_data,
                 MY_BITMAP const *column_image,
                 uchar const **const row_image_end_p,
                 uchar const *const event_end,
                 enum_row_image_type row_image_type,
                 bool event_has_value_options, bool only_seek) {
   DBUG_TRACE;
-  assert(rli != nullptr);
-  assert(table != nullptr);
-  assert(row_data != nullptr);
-  assert(column_image != nullptr);
+  DBUG_ASSERT(rli != nullptr);
+  DBUG_ASSERT(table != nullptr);
+  DBUG_ASSERT(row_data != nullptr);
+  DBUG_ASSERT(column_image != nullptr);
   // This is guaranteed by the way column_image is initialized in the
   // Rows_log_event constructor.
-  assert(column_image->n_bits == source_column_count);
-  assert(row_image_end_p != nullptr);
-  assert(event_end >= row_data);
+  DBUG_ASSERT(column_image->n_bits == master_column_count);
+  DBUG_ASSERT(row_image_end_p != nullptr);
+  DBUG_ASSERT(event_end >= row_data);
   if (event_has_value_options)
-    assert(row_image_type == enum_row_image_type::UPDATE_BI ||
-           row_image_type == enum_row_image_type::UPDATE_AI);
+    DBUG_ASSERT(row_image_type == enum_row_image_type::UPDATE_BI ||
+                row_image_type == enum_row_image_type::UPDATE_AI);
 
   // Get table_def object and table used for type conversion
   table_def *tabledef = nullptr;
   TABLE *conv_table = nullptr;
   rli->get_table_data(table, &tabledef, &conv_table);
-  assert(tabledef != nullptr);
-
-  // check for mismatch between column counts in table_map_event and row_event
-  if (tabledef->size() != source_column_count) {
-    my_error(ER_REPLICA_CORRUPT_EVENT, MYF(0));
-    return true;
-  }
+  DBUG_ASSERT(tabledef != nullptr);
 
   uint image_column_count = bitmap_bits_set(column_image);
-  bool source_has_gipk = tabledef->is_gipk_present_on_source_table();
-  bool replica_has_gipk = table_has_generated_invisible_primary_key(table);
 
   DBUG_PRINT("info",
              ("table=%.*s "
-              "source_column_count=%u image_column_count=%u "
+              "master_column_count=%u image_column_count=%u "
               "tabledef=%p, conv_table=%p "
-              "row_image_type=%d event_has_value_options=%d "
-              "source_has_gipk=%d replica_has_gipk=%d",
+              "row_image_type=%d event_has_value_options=%d",
               (int)table->s->table_name.length, table->s->table_name.str,
-              source_column_count, image_column_count, tabledef, conv_table,
-              (int)row_image_type, event_has_value_options, source_has_gipk,
-              replica_has_gipk));
+              master_column_count, image_column_count, tabledef, conv_table,
+              (int)row_image_type, event_has_value_options));
 
-  std::unique_ptr<cs::util::ReplicatedColumnsView> fields =
-      cs::util::ReplicatedColumnsViewFactory::
-          get_columns_view_with_inbound_filters(rli->info_thd, table, tabledef);
+  // check for mismatch between column counts in table_map_event and row_event
+  if (tabledef->size() != master_column_count) {
+    my_error(ER_SLAVE_CORRUPT_EVENT, MYF(0));
+    return true;
+  }
 
   const uchar *pack_ptr = row_data;
 
@@ -536,30 +648,29 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
           of the statement, when close_thread_tables calls
           cleanup_partial_update.
         */
-#ifndef NDEBUG
+#ifndef DBUG_OFF
         int marked_columns = 0;
 #endif
-        for (auto it = fields->begin();
-             it != fields->end() && it.translated_pos() != source_column_count;
-             ++it) {
-          size_t col_i = it.translated_pos();
+        for (uint col_i = 0;
+             col_i < master_column_count && table->field[col_i] != nullptr;
+             col_i++)
           if (tabledef->type(col_i) == MYSQL_TYPE_JSON &&
               bitmap_is_set(column_image, col_i)) {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
             marked_columns++;
 #endif
-            if (table->mark_column_for_partial_update(*it))
+            if (table->mark_column_for_partial_update(table->field[col_i]))
               // my_error was already called
               return true; /* purecov: inspected */
           }
-        }
-#ifndef NDEBUG
+#ifndef DBUG_OFF
         DBUG_EXECUTE_IF("rpl_row_jsondiff_binarydiff", {
           if (marked_columns == 1) {
             const char act[] =
                 "now SIGNAL signal.rpl_row_jsondiff_binarydiff_marked_columns";
-            assert(opt_debug_sync_timeout > 0);
-            assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+            DBUG_ASSERT(opt_debug_sync_timeout > 0);
+            DBUG_ASSERT(
+                !debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
           }
         };);
 #endif
@@ -570,29 +681,25 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
 
   // NULL bits
   Bit_reader null_bits(pack_ptr);
-  pack_ptr = translate_beginning_of_raw_data(
-      pack_ptr, column_image, image_column_count, null_bits, tabledef,
-      source_has_gipk, replica_has_gipk);
-  size_t last_source_pos = 0;
-  // Iterate over columns that exist both in source and replica
-  for (auto it = fields->begin();
-       it != fields->end() && it.translated_pos() != source_column_count;
-       ++it) {
-    Field *field_ptr = *it;
-    size_t col_i = last_source_pos = it.translated_pos();
+  pack_ptr += (image_column_count + 7) / 8;
 
+  // Iterate over columns that exist both in master and slave
+  uint col_i = 0;
+  Field *field_ptr = nullptr;
+  for (; col_i < master_column_count &&
+         (field_ptr = table->field[col_i]) != nullptr;
+       col_i++) {
     /*
       If there is a conversion table, we pick up the field pointer to
       the conversion table.  If the conversion table or the field
       pointer is NULL, no conversions are necessary.
      */
-    Field *conv_field =
-        conv_table ? conv_table->field[it.absolute_pos()] : nullptr;
+    Field *conv_field = conv_table ? conv_table->field[col_i] : nullptr;
     Field *const f = conv_field ? conv_field : field_ptr;
-    DBUG_PRINT("debug", ("Conversion %srequired for field '%s' (#%lu)",
-                         conv_field ? "" : "not ", field_ptr->field_name,
-                         static_cast<long unsigned int>(it.absolute_pos())));
-    assert(f != nullptr);
+    DBUG_PRINT("debug",
+               ("Conversion %srequired for field '%s' (#%d)",
+                conv_field ? "" : "not ", field_ptr->field_name, col_i));
+    DBUG_ASSERT(f != nullptr);
 
     DBUG_PRINT("debug",
                ("field name: %s; field position: %p", f->field_name, pack_ptr));
@@ -610,10 +717,10 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
      */
     if (bitmap_is_set(column_image, col_i)) {
       /* Field...::unpack() cannot return 0 */
-      assert(pack_ptr != nullptr);
+      DBUG_ASSERT(pack_ptr != nullptr);
 
       if (null_bits.get()) {
-        if (f->is_nullable()) {
+        if (f->maybe_null()) {
           DBUG_PRINT("debug", ("Was NULL"));
           /**
             Calling reset just in case one is unpacking on top a
@@ -647,7 +754,7 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
           normal unpack operation.
         */
         uint const metadata = tabledef->field_metadata(col_i);
-#ifndef NDEBUG
+#ifndef DBUG_OFF
         uchar const *const old_pack_ptr = pack_ptr;
 #endif
         /// @todo calc_field_size may read out of bounds /Sven
@@ -656,7 +763,7 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
         DBUG_PRINT("info", ("calc_field_size ret=%d event_len=%d", (int)len,
                             (int)event_len));
         if (len > event_len) {
-          my_error(ER_REPLICA_CORRUPT_EVENT, MYF(0));
+          my_error(ER_SLAVE_CORRUPT_EVENT, MYF(0));
           return true;
         }
         if (only_seek)
@@ -675,9 +782,9 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
           a old decimal data type which is unsupported datatype in
           RBR mode.
          */
-        assert(tabledef->type(col_i) == MYSQL_TYPE_DECIMAL ||
-               tabledef->calc_field_size(col_i, old_pack_ptr) ==
-                   (uint32)(pack_ptr - old_pack_ptr));
+        DBUG_ASSERT(tabledef->type(col_i) == MYSQL_TYPE_DECIMAL ||
+                    tabledef->calc_field_size(col_i, old_pack_ptr) ==
+                        (uint32)(pack_ptr - old_pack_ptr));
       }
 
       /*
@@ -688,7 +795,7 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
       */
       if (conv_field) {
         Copy_field copy;
-#ifndef NDEBUG
+#ifndef DBUG_OFF
         char source_buf[MAX_FIELD_WIDTH];
         char value_buf[MAX_FIELD_WIDTH];
         String source_type(source_buf, sizeof(source_buf), system_charset_info);
@@ -699,9 +806,9 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
                              field_ptr->field_name, source_type.c_ptr_safe(),
                              value_string.c_ptr_safe()));
 #endif
-        copy.set(field_ptr, f);
-        copy.invoke_do_copy();
-#ifndef NDEBUG
+        copy.set(field_ptr, f, true);
+        copy.invoke_do_copy(&copy);
+#ifndef DBUG_OFF
         char target_buf[MAX_FIELD_WIDTH];
         String target_type(target_buf, sizeof(target_buf), system_charset_info);
         field_ptr->sql_type(target_type);
@@ -712,16 +819,15 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
 #endif
       }
     }
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     else {
       DBUG_PRINT("debug", ("Non-existent: skipped"));
     }
 #endif
   }
 
-  // move past source's extra fields
-  for (size_t col_i = last_source_pos + 1; col_i < source_column_count;
-       ++col_i) {
+  // move past master's extra fields
+  for (; col_i < master_column_count; col_i++) {
     if ((event_value_options & PARTIAL_JSON_UPDATES) != 0 &&
         tabledef->type(col_i) == MYSQL_TYPE_JSON)
       partial_bits.get();
@@ -732,7 +838,7 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
         DBUG_PRINT("info", ("Skipping field"));
         DBUG_DUMP("info", pack_ptr, len);
         if (len > event_len) {
-          my_error(ER_REPLICA_CORRUPT_EVENT, MYF(0));
+          my_error(ER_SLAVE_CORRUPT_EVENT, MYF(0));
           return true;
         }
         pack_ptr += len;
@@ -741,7 +847,7 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
   }
 
   // We have read all the null bits.
-  assert(null_bits.tell() == image_column_count);
+  DBUG_ASSERT(null_bits.tell() == image_column_count);
 
   DBUG_DUMP("info", row_data, pack_ptr - row_data);
 
@@ -755,20 +861,19 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
         start_partial_bit_reader(pack_ptr, event_end - pack_ptr, tabledef,
                                  &partial_bits, &event_value_options);
     if ((event_value_options & PARTIAL_JSON_UPDATES) != 0) {
-      for (auto it = fields->begin();
-           it != fields->end() && it.translated_pos() != source_column_count;
-           ++it) {
-        size_t col_i = it.translated_pos();
+      for (col_i = 0;
+           col_i < master_column_count && table->field[col_i] != nullptr;
+           col_i++) {
         if (tabledef->type(col_i) == MYSQL_TYPE_JSON) {
           if (partial_bits.get()) {
-            DBUG_PRINT("info",
-                       ("forcing column %s in the read_set", it->field_name));
-            bitmap_set_bit(table->read_set, it.absolute_pos());
+            DBUG_PRINT("info", ("forcing column %s in the read_set",
+                                table->field[col_i]->field_name));
+            bitmap_set_bit(table->read_set, col_i);
           }
 #ifndef NO_DBUG
           else
             DBUG_PRINT("info", ("not forcing column %s in the read_set",
-                                it->field_name));
+                                table->field[col_i]->field_name));
 #endif
         }
       }
@@ -778,21 +883,6 @@ bool unpack_row(Relay_log_info const *rli, TABLE *table,
   return false;
 }
 
-const uchar *translate_beginning_of_raw_data(
-    const uchar *raw_data, MY_BITMAP const *column_image, size_t column_count,
-    Bit_reader &null_bits, table_def *tabledef, bool source_has_gipk,
-    bool replica_has_gipk) {
-  if (source_has_gipk && !replica_has_gipk) {
-    if (bitmap_is_set(column_image, 0)) {
-      null_bits.get();
-    }
-    const uchar *ptr = raw_data + ((column_count + 7) / 8);
-    uint32 first_column_len = tabledef->calc_field_size(0, ptr);
-    return ptr + first_column_len;
-  }
-  return raw_data + (column_count + 7) / 8;
-}
-
 /**
   Fills @c table->record[0] with default values.
 
@@ -833,7 +923,7 @@ int prepare_record(TABLE *const table, const MY_BITMAP *cols,
     Just to be sure that tmp_set is currently not in use as
     the write_set already.
   */
-  assert(table->write_set != &table->tmp_set);
+  DBUG_ASSERT(table->write_set != &table->tmp_set);
   /* set the temporary write_set */
   table->column_bitmaps_set_no_signal(table->read_set, &table->tmp_set);
   /**
@@ -846,7 +936,7 @@ int prepare_record(TABLE *const table, const MY_BITMAP *cols,
     uint field_index = (uint)(field_ptr - table->field);
     if (field_index >= cols->n_bits || !bitmap_is_set(cols, field_index)) {
       Field *const f = *field_ptr;
-      if (f->is_flag_set(NO_DEFAULT_VALUE_FLAG) &&
+      if ((f->flags & NO_DEFAULT_VALUE_FLAG) &&
           (f->real_type() != MYSQL_TYPE_ENUM)) {
         f->set_default();
         push_warning_printf(
