@@ -306,7 +306,7 @@ Slave_worker::~Slave_worker() {
            non-zero   failure
 */
 int Slave_worker::init_worker(Relay_log_info *rli, ulong i) {
-  DBUG_ENTER("Slave_worker::init_worker");
+  DBUG_TRACE;
   DBUG_ASSERT(!rli->info_thd->is_error());
 
   Slave_job_item empty = Slave_job_item();
@@ -316,7 +316,13 @@ int Slave_worker::init_worker(Relay_log_info *rli, ulong i) {
 
   if (rli_init_info(false) ||
       DBUG_EVALUATE_IF("inject_init_worker_init_info_fault", true, false))
-    DBUG_RETURN(1);
+    return 1;
+
+  if (!rli->is_privilege_checks_user_null()) {
+    this->set_privilege_checks_user(
+        rli->get_privilege_checks_username().c_str(),
+        rli->get_privilege_checks_hostname().c_str());
+  }
 
   id = i;
   curr_group_exec_parts.clear();
@@ -364,7 +370,7 @@ int Slave_worker::init_worker(Relay_log_info *rli, ulong i) {
               current_mts_submode->get_type());
 
   m_order_commit_deadlock = false;
-  DBUG_RETURN(0);
+  return 0;
 }
 
 /**
@@ -390,9 +396,9 @@ int Slave_worker::init_worker(Relay_log_info *rli, ulong i) {
 int Slave_worker::rli_init_info(bool is_gaps_collecting_phase) {
   enum_return_check return_check = ERROR_CHECKING_REPOSITORY;
 
-  DBUG_ENTER("Slave_worker::rli_init_info");
+  DBUG_TRACE;
 
-  if (inited) DBUG_RETURN(0);
+  if (inited) return 0;
 
   /*
     Worker bitmap size depends on recovery mode.
@@ -425,19 +431,19 @@ int Slave_worker::rli_init_info(bool is_gaps_collecting_phase) {
   }
   inited = 1;
 
-  DBUG_RETURN(0);
+  return 0;
 
 err:
   // todo: handler->end_info(uidx, nidx);
   inited = 0;
   LogErr(ERROR_LEVEL, ER_RPL_ERROR_READING_SLAVE_WORKER_CONFIGURATION);
-  DBUG_RETURN(1);
+  return 1;
 }
 
 void Slave_worker::end_info() {
-  DBUG_ENTER("Slave_worker::end_info");
+  DBUG_TRACE;
 
-  if (!inited) DBUG_VOID_RETURN;
+  if (!inited) return;
 
   if (handler) handler->end_info();
 
@@ -446,14 +452,12 @@ void Slave_worker::end_info() {
     bitmap_free(&group_shifted);
   }
   inited = 0;
-
-  DBUG_VOID_RETURN;
 }
 
 int Slave_worker::flush_info(const bool force) {
-  DBUG_ENTER("Slave_worker::flush_info");
+  DBUG_TRACE;
 
-  if (!inited) DBUG_RETURN(0);
+  if (!inited) return 0;
 
   /*
     We update the sync_period at this point because only here we
@@ -467,15 +471,15 @@ int Slave_worker::flush_info(const bool force) {
 
   if (handler->flush_info(force)) goto err;
 
-  DBUG_RETURN(0);
+  return 0;
 
 err:
   LogErr(ERROR_LEVEL, ER_RPL_ERROR_WRITING_SLAVE_WORKER_CONFIGURATION);
-  DBUG_RETURN(1);
+  return 1;
 }
 
 bool Slave_worker::read_info(Rpl_info_handler *from) {
-  DBUG_ENTER("Slave_worker::read_info");
+  DBUG_TRACE;
 
   ulong temp_group_relay_log_pos = 0;
   ulong temp_group_master_log_pos = 0;
@@ -486,26 +490,27 @@ bool Slave_worker::read_info(Rpl_info_handler *from) {
   uchar *buffer = (uchar *)group_executed.bitmap;
   int temp_internal_id = 0;
 
-  if (from->prepare_info_for_read()) DBUG_RETURN(true);
+  if (from->prepare_info_for_read()) return true;
 
-  if (from->get_info(&temp_internal_id, 0) ||
-      from->get_info(group_relay_log_name, sizeof(group_relay_log_name), "") ||
-      from->get_info(&temp_group_relay_log_pos, 0UL) ||
-      from->get_info(group_master_log_name, sizeof(group_master_log_name),
-                     "") ||
-      from->get_info(&temp_group_master_log_pos, 0UL) ||
-      from->get_info(checkpoint_relay_log_name,
-                     sizeof(checkpoint_relay_log_name), "") ||
-      from->get_info(&temp_checkpoint_relay_log_pos, 0UL) ||
-      from->get_info(checkpoint_master_log_name,
-                     sizeof(checkpoint_master_log_name), "") ||
-      from->get_info(&temp_checkpoint_master_log_pos, 0UL) ||
-      from->get_info(&temp_checkpoint_seqno, 0UL) ||
-      from->get_info(&nbytes, 0UL) ||
-      from->get_info(buffer, (size_t)nbytes, (uchar *)0) ||
+  if (!!from->get_info(&temp_internal_id, 0) ||
+      !!from->get_info(group_relay_log_name, sizeof(group_relay_log_name),
+                       "") ||
+      !!from->get_info(&temp_group_relay_log_pos, 0UL) ||
+      !!from->get_info(group_master_log_name, sizeof(group_master_log_name),
+                       "") ||
+      !!from->get_info(&temp_group_master_log_pos, 0UL) ||
+      !!from->get_info(checkpoint_relay_log_name,
+                       sizeof(checkpoint_relay_log_name), "") ||
+      !!from->get_info(&temp_checkpoint_relay_log_pos, 0UL) ||
+      !!from->get_info(checkpoint_master_log_name,
+                       sizeof(checkpoint_master_log_name), "") ||
+      !!from->get_info(&temp_checkpoint_master_log_pos, 0UL) ||
+      !!from->get_info(&temp_checkpoint_seqno, 0UL) ||
+      !!from->get_info(&nbytes, 0UL) ||
+      !!from->get_info(buffer, (size_t)nbytes, (uchar *)0) ||
       /* default is empty string */
-      from->get_info(channel, sizeof(channel), ""))
-    DBUG_RETURN(true);
+      !!from->get_info(channel, sizeof(channel), ""))
+    return true;
 
   DBUG_ASSERT(nbytes <= no_bytes_in_map(&group_executed));
 
@@ -516,7 +521,7 @@ bool Slave_worker::read_info(Rpl_info_handler *from) {
   checkpoint_master_log_pos = temp_checkpoint_master_log_pos;
   worker_checkpoint_seqno = temp_checkpoint_seqno;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /*
@@ -537,18 +542,18 @@ void Slave_worker::copy_values_for_PFS(ulong worker_id,
 }
 
 bool Slave_worker::set_info_search_keys(Rpl_info_handler *to) {
-  DBUG_ENTER("Slave_worker::set_info_search_keys");
+  DBUG_TRACE;
 
   /* primary keys are Id and channel_name */
   if (to->set_info(0, (int)internal_id) ||
       to->set_info(LINE_FOR_CHANNEL, channel))
-    DBUG_RETURN(true);
+    return true;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 bool Slave_worker::write_info(Rpl_info_handler *to) {
-  DBUG_ENTER("Slave_worker::write_info");
+  DBUG_TRACE;
 
   ulong nbytes = (ulong)no_bytes_in_map(&group_executed);
   uchar *buffer = (uchar *)group_executed.bitmap;
@@ -565,9 +570,9 @@ bool Slave_worker::write_info(Rpl_info_handler *to) {
       to->set_info((ulong)checkpoint_master_log_pos) ||
       to->set_info(worker_checkpoint_seqno) || to->set_info(nbytes) ||
       to->set_info(buffer, (size_t)nbytes) || to->set_info(channel))
-    DBUG_RETURN(true);
+    return true;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -579,18 +584,24 @@ bool Slave_worker::write_info(Rpl_info_handler *to) {
    @return false as success true as failure
 */
 bool Slave_worker::reset_recovery_info() {
-  DBUG_ENTER("Slave_worker::reset_recovery_info");
+  DBUG_TRACE;
 
   set_group_master_log_name("");
   set_group_master_log_pos(0);
 
-  DBUG_RETURN(flush_info(true));
+  return flush_info(true);
 }
 
 size_t Slave_worker::get_number_worker_fields() {
   return sizeof(info_slave_worker_fields) / sizeof(info_slave_worker_fields[0]);
 }
 
+void Slave_worker::set_nullable_fields(MY_BITMAP *nullable_fields) {
+  bitmap_init(nullable_fields, nullptr,
+              Slave_worker::get_number_worker_fields(), false);
+  bitmap_clear_all(nullable_fields);
+}
+
 const char *Slave_worker::get_master_log_name() {
   Slave_job_group *ptr_g = c_rli->gaq->get_job_group(gaq_index);
 
@@ -600,7 +611,7 @@ const char *Slave_worker::get_master_log_name() {
 
 bool Slave_worker::commit_positions(Log_event *ev, Slave_job_group *ptr_g,
                                     bool force) {
-  DBUG_ENTER("Slave_worker::checkpoint_positions");
+  DBUG_TRACE;
 
   /*
     Initial value of checkpoint_master_log_name is learned from
@@ -671,7 +682,7 @@ bool Slave_worker::commit_positions(Log_event *ev, Slave_job_group *ptr_g,
   DBUG_EXECUTE_IF("mts_debug_concurrent_access",
                   { mts_debug_concurrent_access++; };);
 
-  DBUG_RETURN(flush_info(force));
+  return flush_info(force);
 }
 
 void Slave_worker::rollback_positions(Slave_job_group *ptr_g) {
@@ -684,7 +695,7 @@ void Slave_worker::rollback_positions(Slave_job_group *ptr_g) {
 static void free_entry(db_worker_hash_entry *entry) {
   THD *c_thd = current_thd;
 
-  DBUG_ENTER("free_entry");
+  DBUG_TRACE;
 
   DBUG_PRINT("info", ("free_entry %s, %zu", entry->db, strlen(entry->db)));
 
@@ -701,33 +712,29 @@ static void free_entry(db_worker_hash_entry *entry) {
   mts_move_temp_tables_to_thd(c_thd, entry->temporary_tables);
   entry->temporary_tables = nullptr;
 
-  my_free((void *)entry->db);
+  my_free(const_cast<char *>(entry->db));
   my_free(entry);
-
-  DBUG_VOID_RETURN;
 }
 
 bool init_hash_workers(Relay_log_info *rli) {
-  DBUG_ENTER("init_hash_workers");
+  DBUG_TRACE;
 
   rli->inited_hash_workers = true;
   mysql_mutex_init(key_mutex_slave_worker_hash, &rli->slave_worker_hash_lock,
                    MY_MUTEX_INIT_FAST);
   mysql_cond_init(key_cond_slave_worker_hash, &rli->slave_worker_hash_cond);
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 void destroy_hash_workers(Relay_log_info *rli) {
-  DBUG_ENTER("destroy_hash_workers");
+  DBUG_TRACE;
   if (rli->inited_hash_workers) {
     rli->mapping_db_to_worker.clear();
     mysql_mutex_destroy(&rli->slave_worker_hash_lock);
     mysql_cond_destroy(&rli->slave_worker_hash_cond);
     rli->inited_hash_workers = false;
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -777,9 +784,9 @@ TABLE *mts_move_temp_table_to_entry(TABLE *table, THD *thd,
    @return   the post-merge value of thd->temporary_tables.
 */
 TABLE *mts_move_temp_tables_to_thd(THD *thd, TABLE *temporary_tables) {
-  DBUG_ENTER("mts_move_temp_tables_to_thd");
+  DBUG_TRACE;
   TABLE *table = temporary_tables;
-  if (!table) DBUG_RETURN(nullptr);
+  if (!table) return nullptr;
 
   // accept only if this is the start of the list.
   DBUG_ASSERT(!table->prev);
@@ -793,7 +800,7 @@ TABLE *mts_move_temp_tables_to_thd(THD *thd, TABLE *temporary_tables) {
   if (thd->temporary_tables) thd->temporary_tables->prev = table;
   table->next = thd->temporary_tables;
   thd->temporary_tables = temporary_tables;
-  DBUG_RETURN(thd->temporary_tables);
+  return thd->temporary_tables;
 }
 
 /**
@@ -879,13 +886,13 @@ Slave_worker *map_db_to_worker(const char *dbname, Relay_log_info *rli,
 
   THD *thd = rli->info_thd;
 
-  DBUG_ENTER("map_db_to_worker");
+  DBUG_TRACE;
 
   DBUG_ASSERT(!rli->last_assigned_worker ||
               rli->last_assigned_worker == last_worker);
   DBUG_ASSERT(is_mts_db_partitioned(rli));
 
-  if (!rli->inited_hash_workers) DBUG_RETURN(nullptr);
+  if (!rli->inited_hash_workers) return nullptr;
 
   db_worker_hash_entry *entry = nullptr;
   size_t dblength = strlen(dbname);
@@ -898,7 +905,7 @@ Slave_worker *map_db_to_worker(const char *dbname, Relay_log_info *rli,
       continue;
     else if (strncmp(entry->db, const_cast<char *>(dbname), dblength) == 0) {
       *ptr_entry = entry;
-      DBUG_RETURN(last_worker);
+      return last_worker;
     }
   }
 
@@ -1058,7 +1065,7 @@ err:
     rli->curr_group_assigned_parts.push_back(entry);
     *ptr_entry = entry;
   }
-  DBUG_RETURN(entry ? entry->worker : nullptr);
+  return entry ? entry->worker : nullptr;
 }
 
 /**
@@ -1088,7 +1095,7 @@ Slave_worker *get_least_occupied_worker(Relay_log_info *rli,
    @param error  error code after processing the event by caller.
 */
 void Slave_worker::slave_worker_ends_group(Log_event *ev, int error) {
-  DBUG_ENTER("Slave_worker::slave_worker_ends_group");
+  DBUG_TRACE;
   Slave_job_group *ptr_g = nullptr;
 
   if (!error) {
@@ -1249,8 +1256,6 @@ void Slave_worker::slave_worker_ends_group(Log_event *ev, int error) {
 #endif
   }
   curr_group_seen_gtid = false;
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -1342,7 +1347,7 @@ bool Slave_committed_queue::count_done(Relay_log_info *rli) {
    @return number of discarded items
 */
 ulong Slave_committed_queue::move_queue_head(Slave_worker_array *ws) {
-  DBUG_ENTER("Slave_committed_queue::move_queue_head");
+  DBUG_TRACE;
   ulong i, cnt = 0;
 
   for (i = entry; i != avail && !empty(); cnt++, i = (i + 1) % size) {
@@ -1353,7 +1358,7 @@ ulong Slave_committed_queue::move_queue_head(Slave_worker_array *ws) {
 #ifndef DBUG_OFF
     if (DBUG_EVALUATE_IF("check_slave_debug_group", 1, 0) &&
         cnt == opt_mts_checkpoint_period)
-      DBUG_RETURN(cnt);
+      return cnt;
 #endif
 
     grl_name[0] = 0;
@@ -1425,7 +1430,7 @@ ulong Slave_committed_queue::move_queue_head(Slave_worker_array *ws) {
 
   DBUG_ASSERT(cnt <= size);
 
-  DBUG_RETURN(cnt);
+  return cnt;
 }
 
 /**
@@ -1641,7 +1646,7 @@ int Slave_worker::slave_worker_exec_event(Log_event *ev) {
   THD *thd = info_thd;
   int ret = 0;
 
-  DBUG_ENTER("slave_worker_exec_event");
+  DBUG_TRACE;
 
   thd->server_id = ev->server_id;
   thd->set_time();
@@ -1717,7 +1722,7 @@ int Slave_worker::slave_worker_exec_event(Log_event *ev) {
   set_master_log_pos(static_cast<ulong>(ev->common_header->log_pos));
   set_gaq_index(ev->mts_group_idx);
   ret = ev->do_apply_event_worker(this);
-  DBUG_RETURN(ret);
+  return ret;
 }
 
 /**
@@ -1773,9 +1778,9 @@ bool Slave_worker::retry_transaction(uint start_relay_number,
   THD *thd = info_thd;
   bool silent = false;
 
-  DBUG_ENTER("Slave_worker::retry_transaction");
+  DBUG_TRACE;
 
-  if (slave_trans_retries == 0) DBUG_RETURN(true);
+  if (slave_trans_retries == 0) return true;
 
   do {
     /* Simulate a lock deadlock error */
@@ -1828,7 +1833,7 @@ bool Slave_worker::retry_transaction(uint start_relay_number,
     if (!has_temporary_error(thd, error, &silent) ||
         thd->get_transaction()->cannot_safely_rollback(
             Transaction_ctx::SESSION))
-      DBUG_RETURN(true);
+      return true;
 
     if (trans_retries >= slave_trans_retries) {
       thd->fatal_error();
@@ -1837,7 +1842,7 @@ bool Slave_worker::retry_transaction(uint start_relay_number,
                     "in vain, giving up. Consider raising the value of "
                     "the slave_transaction_retries variable.",
                     trans_retries);
-      DBUG_RETURN(true);
+      return true;
     }
 
     if (!silent) {
@@ -1847,7 +1852,8 @@ bool Slave_worker::retry_transaction(uint start_relay_number,
         uint transient_error =
             (error == 0) ? thd->get_stmt_da()->mysql_errno() : error;
         current_thd->rli_slave->retried_processing(
-            transient_error, ER_THD(thd, transient_error), trans_retries);
+            transient_error, ER_THD_NONCONST(thd, transient_error),
+            trans_retries);
 #ifndef DBUG_OFF
         if (trans_retries == 2 || trans_retries == 6)
           DBUG_EXECUTE_IF("rpl_ps_tables_worker_retry", {
@@ -1873,7 +1879,7 @@ bool Slave_worker::retry_transaction(uint start_relay_number,
 
   } while (read_and_apply_events(start_relay_number, start_relay_pos,
                                  end_relay_number, end_relay_pos));
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -1893,7 +1899,7 @@ bool Slave_worker::read_and_apply_events(uint start_relay_number,
                                          my_off_t start_relay_pos,
                                          uint end_relay_number,
                                          my_off_t end_relay_pos) {
-  DBUG_ENTER("Slave_worker::read_and_apply_events");
+  DBUG_TRACE;
 
   Relay_log_info *rli = c_rli;
   char file_name[FN_REFLEN + 1];
@@ -1975,7 +1981,7 @@ bool Slave_worker::read_and_apply_events(uint start_relay_number,
 
   error = false;
 end:
-  DBUG_RETURN(error);
+  return error;
 }
 
 /*
@@ -2411,7 +2417,7 @@ int slave_worker_exec_job_group(Slave_worker *worker, Relay_log_info *rli) {
   uint start_relay_number;
   my_off_t start_relay_pos;
 
-  DBUG_ENTER("slave_worker_exec_job_group");
+  DBUG_TRACE;
 
   if (unlikely(worker->trans_retries > 0)) worker->trans_retries = 0;
 
@@ -2551,7 +2557,7 @@ int slave_worker_exec_job_group(Slave_worker *worker, Relay_log_info *rli) {
   remove_item_from_jobs(job_item, worker, rli);
   delete ev;
 
-  DBUG_RETURN(0);
+  return 0;
 err:
   if (error) {
     report_error_to_coordinator(worker);
@@ -2561,7 +2567,7 @@ err:
                         worker->running_status));
     worker->slave_worker_ends_group(ev, error); /* last done sets post exec */
   }
-  DBUG_RETURN(error);
+  return error;
 }
 
 const char *Slave_worker::get_for_channel_str(bool upper_case) const {
