@@ -1,16 +1,15 @@
-/* Copyright (c) 2006, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,8 +28,6 @@
 #endif
 
 #include <sys/types.h>
-#include <algorithm>
-#include <string>
 #include <unordered_map>
 
 #include "field_types.h"  // enum_field_types
@@ -46,7 +43,7 @@ struct MY_BITMAP;
 
 #include "map_helpers.h"
 #include "prealloced_array.h"  // Prealloced_array
-#include "sql/table.h"         // Table_ref
+#include "sql/table.h"         // TABLE_LIST
 
 class Log_event;
 class Relay_log_info;
@@ -96,7 +93,7 @@ struct HASH_ROW_PREAMBLE {
       const_iterator search_state;
 
   /**
-    whether this search_state is usable or not.
+    Wether this search_state is usable or not.
    */
   bool is_search_state_inited;
 };
@@ -176,7 +173,7 @@ class Hash_slave_rows {
 
   /**
      Deletes the entry pointed by entry. It also frees memory used
-     holding entry contents. This is the way to release memory
+     holding entry contents. This is the way to release memeory
      used for entry, freeing it explicitly with my_free will cause
      undefined behavior.
 
@@ -248,12 +245,12 @@ class Hash_slave_rows {
 class table_def {
  public:
   /**
-    No-op constructor. Instances of RPL_Table_ref are created by first
-    allocating memory, then placement-new-ing an RPL_Table_ref object
+    No-op constructor. Instances of RPL_TABLE_LIST are created by first
+    allocating memory, then placement-new-ing an RPL_TABLE_LIST object
     containing an uninitialized table_def object which is only conditionally
     initialized. See Table_map_log_event::do_apply_event().
   */
-  table_def() = default;
+  table_def() {}
 
   /**
     Constructor.
@@ -277,17 +274,6 @@ class table_def {
    */
   ulong size() const { return m_size; }
 
-  /**
-    Return the number of fields there is type data for minus
-    the GIPK field if this field does not exist in the replica.
-
-    @return The number of fields there is a type minus the GIPK
-   */
-  ulong filtered_size(bool replica_has_gipk) const {
-    if (m_is_gipk_on_table && !replica_has_gipk) return m_size - 1;
-    return m_size;
-  }
-
   /*
     Returns internal binlog type code for one field,
     without translation to real types.
@@ -318,7 +304,7 @@ class table_def {
     returned.
    */
   enum_field_types type(ulong index) const {
-    assert(index < m_size);
+    DBUG_ASSERT(index < m_size);
     /*
       If the source type is MYSQL_TYPE_STRING, it can in reality be
       either MYSQL_TYPE_STRING, MYSQL_TYPE_ENUM, or MYSQL_TYPE_SET, so
@@ -363,7 +349,7 @@ class table_def {
     in the event that the master's field is smaller than the slave.
   */
   uint field_metadata(uint index) const {
-    assert(index < m_size);
+    DBUG_ASSERT(index < m_size);
     if (m_field_metadata_size)
       return m_field_metadata[index];
     else
@@ -374,7 +360,7 @@ class table_def {
     Returns whether or not the field at `index` is a typed array.
    */
   bool is_array(uint index) const {
-    assert(index < m_size);
+    DBUG_ASSERT(index < m_size);
     if (m_field_metadata_size)
       return m_is_array[index];
     else
@@ -386,7 +372,7 @@ class table_def {
     This value is derived from field->maybe_null().
   */
   bool maybe_null(uint index) const {
-    assert(index < m_size);
+    DBUG_ASSERT(index < m_size);
     return ((m_null_bits[(index / 8)] & (1 << (index % 8))) ==
             (1 << (index % 8)));
   }
@@ -415,7 +401,7 @@ class table_def {
 
       - Each column on the master that also exists on the slave can be
         converted according to the current settings of @c
-        REPLICA_TYPE_CONVERSIONS.
+        SLAVE_TYPE_CONVERSIONS.
 
     @param thd   Current thread
     @param rli   Pointer to relay log info
@@ -428,7 +414,7 @@ class table_def {
     @retval 0  if the table definition is compatible with @c table
   */
   bool compatible_with(THD *thd, Relay_log_info *rli, TABLE *table,
-                       TABLE **conv_table_var);
+                       TABLE **conv_table_var) const;
 
   /**
    Create a virtual in-memory temporary table structure.
@@ -448,46 +434,12 @@ class table_def {
    @param thd Thread to allocate memory from.
    @param rli Relay log info structure, for error reporting.
    @param target_table Target table for fields.
-   @param replica_has_gipk Does the replica table contain a GIPK
 
    @return A pointer to a temporary table with memory allocated in the
    thread's memroot, NULL if the table could not be created
    */
   TABLE *create_conversion_table(THD *thd, Relay_log_info *rli,
-                                 TABLE *target_table,
-                                 bool replica_has_gipk) const;
-
-  /**
-    Evaluates if the source table might contain a GIPK
-
-    @note for servers of older versions that do not fully support GIPK, this
-    sets the info that is a guess based on available information replication
-    has.
-
-    In the case when the replica has a GIPK, the source is from an old
-    version that does not indicate if it has a GIPK or not, and the source
-    either has extra columns, or the replica has two more more extra columns
-    true is returned.
-    This function does not report an error.
-
-    @param thd   The thread object associated to the application
-    @param table The table in the replica.
-
-    @return true if this table definition is found to be incompatible with the
-    table, false otherwise.
-  */
-  bool compute_source_table_gipk_info(THD &thd, TABLE *table);
-
-  /**
-    Checks if the table contains a GIPK
-
-    @note for servers of older versions that do not fully support GIPK, this
-    method returns a guess based on available information replication has.
-
-    @return true if we believe the table to contain a GIPK, false otherwise
-   */
-  bool is_gipk_present_on_source_table() const;
-
+                                 TABLE *target_table) const;
 #endif
 
  private:
@@ -500,21 +452,19 @@ class table_def {
   uchar *m_memory;
   mutable int m_json_column_count;  // Number of JSON columns
   bool *m_is_array;
-  bool m_is_gipk_set;
-  bool m_is_gipk_on_table;
 };
 
 #ifdef MYSQL_SERVER
 /**
-   Extend the normal Table_ref with a few new fields needed by the
+   Extend the normal table list with a few new fields needed by the
    slave thread, but nowhere else.
  */
-struct RPL_Table_ref : public Table_ref {
-  RPL_Table_ref(const char *db_name_arg, size_t db_length_arg,
-                const char *table_name_arg, size_t table_name_length_arg,
-                const char *alias_arg, enum thr_lock_type lock_type_arg)
-      : Table_ref(db_name_arg, db_length_arg, table_name_arg,
-                  table_name_length_arg, alias_arg, lock_type_arg) {}
+struct RPL_TABLE_LIST : public TABLE_LIST {
+  RPL_TABLE_LIST(const char *db_name_arg, size_t db_length_arg,
+                 const char *table_name_arg, size_t table_name_length_arg,
+                 const char *alias_arg, enum thr_lock_type lock_type_arg)
+      : TABLE_LIST(db_name_arg, db_length_arg, table_name_arg,
+                   table_name_length_arg, alias_arg, lock_type_arg) {}
 
   bool m_tabledef_valid;
   table_def m_tabledef;
@@ -528,7 +478,7 @@ class Deferred_log_events {
  public:
   Deferred_log_events();
   ~Deferred_log_events();
-  /* queue for execution at Query-log-event time prior to the Query */
+  /* queue for exection at Query-log-event time prior the Query */
   int add(Log_event *ev);
   bool is_empty();
   bool execute(Relay_log_info *rli);
@@ -565,17 +515,17 @@ class Deferred_log_events {
 */
 
 std::pair<my_off_t, std::pair<uint, bool>> read_field_metadata(
-    const uchar *buffer, enum_field_types binlog_type);
+    const uchar *metadata_ptr, enum_field_types type);
 
 // NB. number of printed bit values is limited to sizeof(buf) - 1
-#define DBUG_PRINT_BITSET(N, FRM, BS)                                   \
-  do {                                                                  \
-    char buf[256];                                                      \
-    uint i;                                                             \
-    for (i = 0; i < std::min(uint{sizeof(buf) - 1}, (BS)->n_bits); i++) \
-      buf[i] = bitmap_is_set((BS), i) ? '1' : '0';                      \
-    buf[i] = '\0';                                                      \
-    DBUG_PRINT((N), ((FRM), buf));                                      \
+#define DBUG_PRINT_BITSET(N, FRM, BS)                           \
+  do {                                                          \
+    char buf[256];                                              \
+    uint i;                                                     \
+    for (i = 0; i < MY_MIN(sizeof(buf) - 1, (BS)->n_bits); i++) \
+      buf[i] = bitmap_is_set((BS), i) ? '1' : '0';              \
+    buf[i] = '\0';                                              \
+    DBUG_PRINT((N), ((FRM), buf));                              \
   } while (0)
 
 #ifdef MYSQL_SERVER
@@ -587,7 +537,7 @@ std::pair<my_off_t, std::pair<uint, bool>> read_field_metadata(
   new instance will be initialized within the constructor and disposed of in the
   destructor.
 
-  If the given `THD` object pointer passed on the constructor is not `nullptr`,
+  If the given `THD` object poitner passed on the constructor is not `nullptr`,
   the reference is kept and nothing is disposed on the destructor.
 
   Casting operator to `THD*` is also provided, to easy code replacemente.
@@ -607,7 +557,7 @@ class THD_instance_guard {
     If the given `THD` object pointer is `nullptr`, a new instance will be
     initialized within the constructor and disposed of in the destructor.
 
-    If the given `THD` object pointer is not `nullptr`, the reference is kept
+    If the given `THD` object poitner is not `nullptr`, the reference is kept
     and nothing is disposed on the destructor.
 
     @param thd `THD` object reference that determines if an existence instance
@@ -653,62 +603,4 @@ class THD_instance_guard {
 std::string replace_all_in_str(std::string from, std::string find,
                                std::string replace);
 
-#ifdef MYSQL_SERVER
-
-/**
-  This method shall evaluate if a command being executed goes against any of
-  the restrictions of server variable session.require_row_format.
-
-  @param thd The thread associated to the command
-  @return true if it violates any restrictions
-          false otherwise
- */
-bool is_require_row_format_violation(const THD *thd);
-
-/**
-  This function shall blindly replace some deprecated terms used in the
-  field names with more recent ones. This function must be removed
-  once the related syntax (SHOW SLAVE STATUS and friends) is removed.
-
-  @param thd the thread context.
-  @param field_list the list of fields that will have their name checked
-                    and altered if needed.
- */
-void rename_fields_use_old_replica_source_terms(
-    THD *thd, mem_root_deque<Item *> &field_list);
-
-/**
-  Checks if the immediate_server_version supports GIPKs or not
-
-  @param thd The THD context to check the version
-
-  @return true if the source server supports GIPK, false otherwise
- */
-bool is_immediate_server_gipk_ready(THD &thd);
-
-/**
-  Returns if the replicated table contains a GIPK or not
-
-  @note for servers of older versions that do not fully support GIPK, this
-  method returns a guess based on available information replication has.
-
-  @param rli    The relay log object associated to the channel
-  @param table  The table to check for the GIPK
-
-  @return true if we believe the table has a GIPK, false otherwise.
- */
-bool does_source_table_contain_gipk(Relay_log_info const *rli, TABLE *table);
-
-/**
-  @brief Returns a string representation for a given version
-
-  @param version a version represented using a integer
-
-  @return a string for the given version or "unknown"
-          if version is undefined or unknown.
-*/
-std::string decimal_numeric_version_to_string(uint32 version);
-
-#endif  // MYSQL_SERVER
-
 #endif /* RPL_UTILITY_H */
