@@ -1,16 +1,15 @@
-/*  Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+/*  Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License, version 2.0,
     as published by the Free Software Foundation.
 
-    This program is designed to work with certain software (including
+    This program is also distributed with certain software (including
     but not limited to OpenSSL) that is licensed under separate terms,
     as designated in a particular file or component or in included license
     documentation.  The authors of MySQL hereby grant you an additional
     permission to link the program and your derivative works with the
-    separately licensed software that they have either included with
-    the program or referenced in the documentation.
+    separately licensed software that they have included with MySQL.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,54 +34,14 @@
 #include "sql/sql_class.h"
 #include "sql/sql_show.h"
 #include "sql/sql_thd_internal_api.h"
-#include "sql/ssl_init_callback.h"
+#include "sql/ssl_acceptor_context.h"
 #include "sql/sys_vars_shared.h"
 #include "sql_common.h"
 
 #include "sql/dd/cache/dictionary_client.h"
 #include "sql/dd/dictionary.h"
 
-/** The minimum idle timeout in seconds. It is kept at 8 hours which is also
-the Server default. Currently recipient sends ACK during state transition.
-In future we could have better time controlled ACK. */
-static const uint32_t MIN_IDLE_TIME_OUT_SEC = 8 * 60 * 60;
-
-/** Minimum read timeout in seconds. Maintain above the donor ACK frequency. */
-static const uint32_t MIN_READ_TIME_OUT_SEC = 30;
-
-/** Minimum write timeout in seconds. Disallow configuring it to too low. We
-might need a separate clone configuration in future or retry on failure. */
-static const uint32_t MIN_WRITE_TIME_OUT_SEC = 60;
-
-/** Set Network read timeout.
-@param[in,out]	net	network object
-@param[in]	timeout	time out in seconds */
-static void set_read_timeout(NET *net, uint32_t timeout) {
-  if (timeout < MIN_READ_TIME_OUT_SEC) {
-    timeout = MIN_READ_TIME_OUT_SEC;
-  }
-  my_net_set_read_timeout(net, timeout);
-}
-
-/** Set Network write timeout.
-@param[in,out]	net	network object
-@param[in]	timeout	time out in seconds */
-static void set_write_timeout(NET *net, uint32_t timeout) {
-  if (timeout < MIN_WRITE_TIME_OUT_SEC) {
-    timeout = MIN_WRITE_TIME_OUT_SEC;
-  }
-  my_net_set_write_timeout(net, timeout);
-}
-
-/** Set Network idle timeout.
-@param[in,out]	net	network object
-@param[in]	timeout	time out in seconds */
-static void set_idle_timeout(NET *net, uint32_t timeout) {
-  if (timeout < MIN_IDLE_TIME_OUT_SEC) {
-    timeout = MIN_IDLE_TIME_OUT_SEC;
-  }
-  my_net_set_read_timeout(net, timeout);
-}
+void clone_protocol_service_init() { return; }
 
 DEFINE_METHOD(void, mysql_clone_start_statement,
               (THD * &thd, PSI_thread_key thread_key,
@@ -96,7 +55,7 @@ DEFINE_METHOD(void, mysql_clone_start_statement,
     my_thread_init();
 
     /* Create thread with input key for PFS */
-    thd = create_thd(true, true, true, thread_key, 0);
+    thd = create_thd(true, true, true, thread_key);
 #ifdef HAVE_PSI_THREAD_INTERFACE
     thd_created = true;
 #endif
@@ -105,15 +64,12 @@ DEFINE_METHOD(void, mysql_clone_start_statement,
 #ifdef HAVE_PSI_THREAD_INTERFACE
   /* Create and set PFS thread key */
   if (thread_key != PSI_NOT_INSTRUMENTED) {
-    assert(thd_created);
+    DBUG_ASSERT(thd_created);
     if (thd_created) {
       PSI_THREAD_CALL(set_thread)(thd->get_psi());
     }
   }
 #endif
-
-  mysql_thread_set_secondary_engine(false);
-
   /* Create and set PFS statement key */
   if (statement_key != PSI_NOT_INSTRUMENTED) {
     if (thd->m_statement_psi == nullptr) {
@@ -128,10 +84,9 @@ DEFINE_METHOD(void, mysql_clone_start_statement,
 }
 
 DEFINE_METHOD(void, mysql_clone_finish_statement, (THD * thd)) {
-  assert(thd->m_statement_psi == nullptr);
+  DBUG_ASSERT(thd->m_statement_psi == nullptr);
 
   my_thread_end();
-  thd->set_psi(nullptr);
   destroy_thd(thd);
 }
 
@@ -167,20 +122,16 @@ DEFINE_METHOD(int, mysql_clone_validate_charsets,
   if (thd == nullptr) {
     return (0);
   }
-
-  int last_error = 0;
-
   for (auto &char_set : char_sets) {
     auto charset_obj = get_charset_by_name(char_set.c_str(), MYF(0));
 
     /* Check if character set collation is available. */
     if (charset_obj == nullptr) {
       my_error(ER_CLONE_CHARSET, MYF(0), char_set.c_str());
-      /* Continue and check for all other errors. */
-      last_error = ER_CLONE_CHARSET;
+      return (ER_CLONE_CHARSET);
     }
   }
-  return last_error;
+  return (0);
 }
 
 /**
@@ -192,35 +143,36 @@ DEFINE_METHOD(int, mysql_clone_validate_charsets,
 */
 static int get_utf8_config(THD *thd, std::string config_name,
                            String &utf8_val) {
-  auto f = [thd, &utf8_val](const System_variable_tracker &, sys_var *var) {
-    char val_buf[1024];
-    SHOW_VAR show;
-    show.type = SHOW_SYS;
-    show.value = pointer_cast<char *>(var);
-    show.name = var->name.str;
+  char val_buf[1024];
+  SHOW_VAR show;
+  show.type = SHOW_SYS;
 
-    mysql_mutex_lock(&LOCK_global_system_variables);
-    size_t val_length;
-    const CHARSET_INFO *fromcs;
+  /* Get system configuration parameter. */
+  mysql_rwlock_rdlock(&LOCK_system_variables_hash);
+  auto var = intern_find_sys_var(config_name.c_str(), config_name.length());
+  mysql_rwlock_unlock(&LOCK_system_variables_hash);
 
-    const char *value =
-        get_one_variable(thd, &show, OPT_GLOBAL, SHOW_SYS, nullptr, &fromcs,
-                         val_buf, &val_length);
-
-    uint dummy_err;
-    const CHARSET_INFO *tocs = &my_charset_utf8mb4_bin;
-    utf8_val.copy(value, val_length, fromcs, tocs, &dummy_err);
-    mysql_mutex_unlock(&LOCK_global_system_variables);
-  };
-
-  System_variable_tracker sv =
-      System_variable_tracker::make_tracker(config_name);
-  if (sv.access_system_variable(thd, f, Suppress_not_found_error::YES)) {
+  if (var == nullptr) {
     my_error(ER_INTERNAL_ERROR, MYF(0),
              "Clone failed to get system configuration parameter.");
     return (ER_INTERNAL_ERROR);
   }
 
+  show.value = reinterpret_cast<char *>(var);
+  show.name = var->name.str;
+
+  mysql_mutex_lock(&LOCK_global_system_variables);
+  size_t val_length;
+  const CHARSET_INFO *fromcs;
+
+  auto value = get_one_variable(thd, &show, OPT_GLOBAL, SHOW_SYS, nullptr,
+                                &fromcs, val_buf, &val_length);
+
+  mysql_mutex_unlock(&LOCK_global_system_variables);
+
+  uint dummy_err;
+  const CHARSET_INFO *tocs = &my_charset_utf8mb4_bin;
+  utf8_val.copy(value, val_length, fromcs, tocs, &dummy_err);
   return (0);
 }
 
@@ -243,146 +195,16 @@ DEFINE_METHOD(int, mysql_clone_get_configs,
   return (err);
 }
 
-/* Clone-related macros to parse version strings and determine if clone
-should be allowed */
-/** Size of the parsed version strings array */
-constexpr unsigned int CLONE_PARSE_ARRAY_SIZE = 4;
-/** Parsed version strings array type */
-typedef std::array<std::string, CLONE_PARSE_ARRAY_SIZE> ParseArray;
-
-/** Index of the array correpsonding to parts of version */
-constexpr unsigned int MAJOR = 0;
-constexpr unsigned int MINOR = 1;
-constexpr unsigned int PATCH = 2;
-constexpr unsigned int BUILD = 3;
-
-/* Patch version in 8.0.37 where wl15989 is backported */
-constexpr unsigned long CLONE_BACKPORT_VERSION = 37;
-
-/** Parse a version string into an array of strings corresponding to the MAJOR,
-MINOR, PATCH and BUILD versions. A string of length 0 is filled in case a
-particular version string could not be parsed. For example,
-  "Major.Minor.Patch-Build" yields ["Major", "Minor", "Patch", "Build"],
-  "8.0.23-SR1"              yields ["8", "0", "23", "SR1"],
-  "8.0.-u5"                 yields ["8", "0", "", "u5"]
-@note: This function allocates the array, populates it and returns the array
-@param version input version string
-*/
-static ParseArray parse_version_string(std::string version) {
-  ParseArray parsed;
-  auto parse_next_part{[&parsed, &version](size_t index, char delimiter) {
-    const auto pos = version.find(delimiter);
-    if (pos != std::string::npos) {
-      /* pos + 1 to skip the delimiter*/
-      parsed[index] = version.substr(0, pos);
-      version.erase(0, pos + 1);
-    } else {
-      /* MAJOR part of the version string to expected to be present always */
-      assert(index != MAJOR);
-      /* unable to parse, store rest of the string and make it empty */
-      parsed[index] = version.substr(0, version.length());
-      version.erase(0, version.length());
-    }
-  }};
-
-  parse_next_part(MAJOR, '.');
-  parse_next_part(MINOR, '.');
-  parse_next_part(PATCH, '-');
-  parsed[BUILD] = version;
-  return parsed;
-}
-
-/** Test specific function to configure the version strings of the donor and
-recipient to cover various scenarios where clone is allowed or not. This
-function will modify the input to ensure correct error message is printed.
-@param config_val recipient server's version string
-@param donor_val  donor server's version string
-*/
-static void test_configure_versions([[maybe_unused]] std::string &config_val,
-                                    [[maybe_unused]] std::string &donor_val) {
-  /* Test specific code to check for cross version clone support */
-  DBUG_EXECUTE_IF("clone_across_lts_version_match",
-                  { config_val = donor_val; });
-  DBUG_EXECUTE_IF("clone_across_lts_major_mismatch", {
-    config_val = "8.4.0";
-    donor_val = "9.7.2";
-  });
-  DBUG_EXECUTE_IF("clone_across_lts_minor_mismatch", {
-    config_val = "8.4.0";
-    donor_val = "8.3.2";
-  });
-  DBUG_EXECUTE_IF("clone_across_lts_non_8_0_patch_mismatch", {
-    config_val = "8.4.2";
-    donor_val = "8.4.1";
-  });
-  DBUG_EXECUTE_IF("clone_across_lts_8_0_patch_match", {
-    config_val = "8.0.25";
-    donor_val = "8.0.25-debug";
-  });
-  DBUG_EXECUTE_IF("clone_across_lts_8_0_before_backport_patch_mismatch", {
-    config_val = "8.0.34";
-    donor_val = "8.0.35";
-  });
-  DBUG_EXECUTE_IF("clone_across_lts_8_0_before_backport_patch_mis_single", {
-    config_val = "8.0.6";
-    donor_val = "8.0.7";
-  });
-  DBUG_EXECUTE_IF("clone_across_lts_8_0_across_backport_patch_mismatch", {
-    config_val = "8.0.38";
-    donor_val = "8.0.35";
-  });
-  DBUG_EXECUTE_IF("clone_across_lts_8_0_after_backport_patch_mismatch", {
-    config_val = "8.0.38";
-    donor_val = "8.0.37";
-  });
-}
-
-/**
- Compares versions and determine if clone is allowed. Clone is allowed if both
- the donor and recipient have exactly same version string. In version series 8.1
- and above, cloning is allowed if Major and Minor versions match. In 8.0 series,
- clone is allowed if patch version is above clone backport version. In this
- comparison, suffixes are ignored: i.e. 8.0.25 should be the same as
- 8.0.25-debug, but 8.0.25 isn't the same as 8.0.251
- @param ver1 version1 string
- @param ver2 version2 string
- @return true if cloning is allowed between ver1 and ver2, false otherwise
- */
-inline bool compare_server_version(std::string ver1, std::string ver2) {
-  if (ver1 == ver2) {
-    return true;
-  }
-  const auto parse_v1 = parse_version_string(ver1);
-  const auto parse_v2 = parse_version_string(ver2);
-
-  if ((parse_v1[MAJOR] != parse_v2[MAJOR]) ||
-      (parse_v1[MINOR] != parse_v2[MINOR])) {
-    return false;
-  } else if ((parse_v1[MAJOR] == "8") && (parse_v1[MINOR] == "0")) {
-    /* Specific checks for clone across 8.0 series */
-    try {
-      return ((parse_v1[PATCH] == parse_v2[PATCH]) ||
-              (std::stoul(parse_v1[PATCH]) >= CLONE_BACKPORT_VERSION &&
-               std::stoul(parse_v2[PATCH]) >= CLONE_BACKPORT_VERSION));
-    } catch (...) {
-      return false;
-    }
-  }
-  return true;
-}
-
 DEFINE_METHOD(int, mysql_clone_validate_configs,
               (THD * thd, Mysql_Clone_Key_Values &configs)) {
-  int last_error = 0;
+  int err = 0;
 
   for (auto &key_val : configs) {
     String utf8_str;
     auto &config_name = key_val.first;
-    auto config_err = get_utf8_config(thd, config_name, utf8_str);
-    if (config_err != 0) {
-      last_error = config_err;
-      /* Continue and check for all other errors. */
-      continue;
+    err = get_utf8_config(thd, config_name, utf8_str);
+    if (err != 0) {
+      break;
     }
 
     auto &donor_val = key_val.second;
@@ -390,43 +212,29 @@ DEFINE_METHOD(int, mysql_clone_validate_configs,
     config_val.assign(utf8_str.c_ptr_quick());
 
     /* Check if the parameter value matches. */
-    if (DBUG_EVALUATE_IF(
-            "clone_across_lts_compare_versions",
-            config_val == donor_val && config_name.compare("version") != 0,
-            config_val == donor_val)) {
+    if (config_val == donor_val) {
       continue;
     }
 
-    int critical_error = 0;
-
-    /* Throw specific error for some configurations. These errors are critical
-    because user can no way clone from the current donor. */
+    /* Throw specific error for some configurations. */
     if (config_name.compare("version_compile_os") == 0) {
-      critical_error = ER_CLONE_OS;
+      err = ER_CLONE_OS;
     } else if (config_name.compare("version") == 0) {
-      /* test specific modifications to version strings */
-      test_configure_versions(config_val, donor_val);
-      if (compare_server_version(config_val, donor_val)) {
-        continue;
-      }
-      critical_error = ER_CLONE_DONOR_VERSION;
+      err = ER_CLONE_DONOR_VERSION;
     } else if (config_name.compare("version_compile_machine") == 0) {
-      critical_error = ER_CLONE_PLATFORM;
+      err = ER_CLONE_PLATFORM;
     }
 
-    /* For critical errors, exit immediately. */
-    if (critical_error != 0) {
-      last_error = critical_error;
-      my_error(last_error, MYF(0), donor_val.c_str(), config_val.c_str());
-      break;
+    if (err != 0) {
+      my_error(err, MYF(0), donor_val.c_str(), config_val.c_str());
+    } else {
+      err = ER_CLONE_CONFIG;
+      my_error(ER_CLONE_CONFIG, MYF(0), config_name.c_str(), donor_val.c_str(),
+               config_val.c_str());
     }
-
-    last_error = ER_CLONE_CONFIG;
-    my_error(ER_CLONE_CONFIG, MYF(0), config_name.c_str(), donor_val.c_str(),
-             config_val.c_str());
-    /* Continue and check for all other configuration mismatch. */
+    break;
   }
-  return last_error;
+  return (err);
 }
 
 DEFINE_METHOD(MYSQL *, mysql_clone_connect,
@@ -435,15 +243,14 @@ DEFINE_METHOD(MYSQL *, mysql_clone_connect,
                MYSQL_SOCKET *socket)) {
   DBUG_TRACE;
 
-  /* Set default */
-  uint net_read_timeout = MIN_READ_TIME_OUT_SEC;
-  uint net_write_timeout = MIN_WRITE_TIME_OUT_SEC;
+  /* Set default as 5 seconds */
+  uint net_read_timeout = 5;
+  uint net_write_timeout = 5;
 
   /* Clean any previous Error and Warnings in THD */
   if (thd != nullptr) {
     thd->clear_error();
     thd->get_stmt_da()->reset_condition_info(thd);
-
     net_read_timeout = thd->variables.net_read_timeout;
     net_write_timeout = thd->variables.net_write_timeout;
   }
@@ -468,15 +275,13 @@ DEFINE_METHOD(MYSQL *, mysql_clone_connect,
 
     OptionalString capath, cipher, ciphersuites, crl, crlpath, version;
 
-    server_main_callback.read_parameters(nullptr, &capath, &version, nullptr,
-                                         &cipher, &ciphersuites, nullptr, &crl,
-                                         &crlpath, nullptr, nullptr);
+    SslAcceptorContext::read_parameters(nullptr, &capath, &version, nullptr,
+                                        &cipher, &ciphersuites, nullptr, &crl,
+                                        &crlpath);
+
+    mysql_ssl_set(mysql, ssl_ctx->m_ssl_key, ssl_ctx->m_ssl_cert,
+                  ssl_ctx->m_ssl_ca, capath.c_str(), cipher.c_str());
 
-    mysql_options(mysql, MYSQL_OPT_SSL_KEY, ssl_ctx->m_ssl_key);
-    mysql_options(mysql, MYSQL_OPT_SSL_CERT, ssl_ctx->m_ssl_cert);
-    mysql_options(mysql, MYSQL_OPT_SSL_CIPHER, cipher.c_str());
-    mysql_options(mysql, MYSQL_OPT_SSL_CA, ssl_ctx->m_ssl_ca);
-    mysql_options(mysql, MYSQL_OPT_SSL_CAPATH, capath.c_str());
     mysql_options(mysql, MYSQL_OPT_SSL_CRL, crl.c_str());
     mysql_options(mysql, MYSQL_OPT_SSL_CRLPATH, crlpath.c_str());
     mysql_options(mysql, MYSQL_OPT_TLS_VERSION, version.c_str());
@@ -496,7 +301,7 @@ DEFINE_METHOD(MYSQL *, mysql_clone_connect,
   }
 
   ret_mysql =
-      mysql_real_connect(mysql, host, user, passwd, nullptr, port, nullptr, 0);
+      mysql_real_connect(mysql, host, user, passwd, nullptr, port, 0, 0);
 
   if (ret_mysql == nullptr) {
     char err_buf[MYSYS_ERRMSG_SIZE + 64];
@@ -519,8 +324,8 @@ DEFINE_METHOD(MYSQL *, mysql_clone_connect,
   net_clear(net, true);
 
   /* Set network read/write timeout */
-  set_read_timeout(net, net_read_timeout);
-  set_write_timeout(net, net_write_timeout);
+  my_net_set_read_timeout(net, net_read_timeout);
+  my_net_set_write_timeout(net, net_write_timeout);
 
   if (thd != nullptr) {
     /* Set current active vio so that shutdown and KILL
@@ -585,7 +390,7 @@ DEFINE_METHOD(int, mysql_clone_send_command,
     err = ER_QUERY_INTERRUPTED;
   }
 
-  assert(err != 0);
+  DBUG_ASSERT(err != 0);
   return err;
 }
 
@@ -608,22 +413,20 @@ DEFINE_METHOD(int, mysql_clone_get_response,
 
   /* Adjust read timeout if specified. */
   if (timeout != 0) {
-    set_read_timeout(net, timeout);
+    my_net_set_read_timeout(net, timeout);
   }
 
   /* Dummy function callback invoked before getting header. */
   auto func_before = [](NET *, void *, size_t) {};
 
   /* Callback function called after receiving header. */
-  auto func_after = [](NET *net_arg, void *ctx, size_t, bool) {
+  auto func_after = [](NET *net, void *ctx, size_t, bool) {
     auto net_bytes = static_cast<size_t *>(ctx);
-    *net_bytes +=
-        static_cast<size_t>(uint3korr(net_arg->buff + net_arg->where_b));
+    *net_bytes += static_cast<size_t>(uint3korr(net->buff + net->where_b));
   };
 
   /* Use server extension callback to capture network byte information. */
   NET_SERVER server_extn;
-  net_server_ext_init(&server_extn);
   server_extn.m_user_data = static_cast<void *>(net_length);
   server_extn.m_before_header = func_before;
   server_extn.m_after_header = func_after;
@@ -642,7 +445,7 @@ DEFINE_METHOD(int, mysql_clone_get_response,
   server_extn.compress_ctx.algorithm = MYSQL_UNCOMPRESSED;
 
   /* Reset timeout back to default value. */
-  set_read_timeout(net, thd->variables.net_read_timeout);
+  my_net_set_read_timeout(net, thd->variables.net_read_timeout);
 
   *packet = net->read_pos;
 
@@ -659,19 +462,6 @@ DEFINE_METHOD(int, mysql_clone_get_response,
     err = ER_QUERY_INTERRUPTED;
   }
 
-  /* This error is not relevant for client but is raised by network
-  net_read_raw_loop() as the code is compiled in server MYSQL_SERVER.
-  For clone client we need to set valid client network error. */
-  if (err == ER_CLIENT_INTERACTION_TIMEOUT) {
-    /* purecov: begin inspected */
-    thd->clear_error();
-    thd->get_stmt_da()->reset_condition_info(thd);
-    net->last_errno = ER_NET_READ_ERROR;
-    err = ER_NET_READ_ERROR;
-    my_error(ER_NET_READ_ERROR, MYF(0));
-    /* purecov: end */
-  }
-
   if (err == 0) {
     net->last_errno = ER_NET_PACKETS_OUT_OF_ORDER;
     err = ER_NET_PACKETS_OUT_OF_ORDER;
@@ -698,6 +488,17 @@ DEFINE_METHOD(int, mysql_clone_kill,
 DEFINE_METHOD(void, mysql_clone_disconnect,
               (THD * thd, MYSQL *mysql, bool is_fatal, bool clear_error)) {
   DBUG_TRACE;
+
+  if (thd != nullptr) {
+    thd->clear_clone_vio();
+
+    /* clear any session error, if requested */
+    if (clear_error) {
+      thd->clear_error();
+      thd->get_stmt_da()->reset_condition_info(thd);
+    }
+  }
+
   /* Make sure that the other end has switched back from clone protocol. */
   if (!is_fatal) {
     is_fatal = simple_command(mysql, COM_RESET_CONNECTION, nullptr, 0, 0);
@@ -709,18 +510,6 @@ DEFINE_METHOD(void, mysql_clone_disconnect,
 
   /* Disconnect */
   mysql_close(mysql);
-
-  /* There could be some n/w error during disconnect and we need to clear
-  them if requested. */
-  if (thd != nullptr) {
-    thd->clear_clone_vio();
-
-    /* clear any session error, if requested */
-    if (clear_error) {
-      thd->clear_error();
-      thd->get_stmt_da()->reset_condition_info(thd);
-    }
-  }
 }
 
 DEFINE_METHOD(void, mysql_clone_get_error,
@@ -757,15 +546,12 @@ DEFINE_METHOD(int, mysql_clone_get_command,
   if (!net_flush(net)) {
     net_new_transaction(net);
 
-    /* Set idle timeout while waiting for commands. Earlier we used server
-    configuration "wait_timeout" but this causes unwanted timeout in clone
-    when user configures the value too low. */
-    set_idle_timeout(net, thd->variables.net_wait_timeout);
+    /* Use idle timeout while waiting for commands */
+    my_net_set_read_timeout(net, thd->variables.net_wait_timeout);
 
     *buffer_length = my_net_read(net);
 
-    set_read_timeout(net, thd->variables.net_read_timeout);
-    set_write_timeout(net, thd->variables.net_write_timeout);
+    my_net_set_read_timeout(net, thd->variables.net_read_timeout);
 
     if (*buffer_length != packet_error && *buffer_length != 0) {
       *com_buffer = net->read_pos;
@@ -812,7 +598,7 @@ DEFINE_METHOD(int, mysql_clone_send_response,
 
   int err = static_cast<int>(net->last_errno);
 
-  assert(err != 0);
+  DBUG_ASSERT(err != 0);
   return err;
 }
 
@@ -820,32 +606,6 @@ DEFINE_METHOD(int, mysql_clone_send_error,
               (THD * thd, uchar err_cmd, bool is_fatal)) {
   DBUG_TRACE;
 
-  NET *net = thd->get_protocol_classic()->get_net();
-  auto da = thd->get_stmt_da();
-
-  /* Consider any previous network error as fatal. */
-  if (!is_fatal && net->last_errno != 0) {
-    is_fatal = true;
-  }
-
-  if (is_fatal) {
-    int err = 0;
-
-    /* Handle the case if network layer hasn't set the error in THD. */
-    if (da->is_error()) {
-      err = da->mysql_errno();
-    } else {
-      err = ER_NET_ERROR_ON_WRITE;
-      my_error(err, MYF(0));
-    }
-
-    mysql_mutex_lock(&thd->LOCK_thd_data);
-    thd->shutdown_active_vio();
-    mysql_mutex_unlock(&thd->LOCK_thd_data);
-
-    return err;
-  }
-
   uchar err_packet[1 + 4 + MYSQL_ERRMSG_SIZE + 1];
   uchar *buf_ptr = &err_packet[0];
   size_t packet_length = 0;
@@ -854,6 +614,8 @@ DEFINE_METHOD(int, mysql_clone_send_error,
   ++buf_ptr;
   ++packet_length;
 
+  auto da = thd->get_stmt_da();
+
   char *bufp;
 
   if (da->is_error()) {
@@ -864,6 +626,13 @@ DEFINE_METHOD(int, mysql_clone_send_error,
     bufp = reinterpret_cast<char *>(buf_ptr);
     packet_length +=
         snprintf(bufp, MYSQL_ERRMSG_SIZE, "%s", da->message_text());
+    if (is_fatal) {
+      mysql_mutex_lock(&thd->LOCK_thd_data);
+      thd->shutdown_active_vio();
+      mysql_mutex_unlock(&thd->LOCK_thd_data);
+
+      return da->mysql_errno();
+    }
   } else {
     int4store(buf_ptr, ER_INTERNAL_ERROR);
     buf_ptr += 4;
@@ -873,6 +642,14 @@ DEFINE_METHOD(int, mysql_clone_send_error,
     packet_length += snprintf(bufp, MYSQL_ERRMSG_SIZE, "%s", "Unknown Error");
   }
 
+  NET *net = thd->get_protocol_classic()->get_net();
+
+  if (net->last_errno != 0) {
+    return static_cast<int>(net->last_errno);
+  }
+
+  DBUG_ASSERT(!is_fatal);
+
   /* Clean error in THD */
   thd->clear_error();
   thd->get_stmt_da()->reset_condition_info(thd);
@@ -880,18 +657,12 @@ DEFINE_METHOD(int, mysql_clone_send_error,
 
   if (my_net_write(net, &err_packet[0], packet_length) || net_flush(net)) {
     int err = static_cast<int>(net->last_errno);
-    da = thd->get_stmt_da();
 
-    if (err == 0 || !da->is_error()) {
+    if (err == 0) {
       net->last_errno = ER_NET_PACKETS_OUT_OF_ORDER;
       err = ER_NET_PACKETS_OUT_OF_ORDER;
       my_error(err, MYF(0));
     }
-
-    mysql_mutex_lock(&thd->LOCK_thd_data);
-    thd->shutdown_active_vio();
-    mysql_mutex_unlock(&thd->LOCK_thd_data);
-
     return err;
   }
   return 0;
