@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -82,15 +82,15 @@ enum enum_component_table_field {
 };
 
 static const TABLE_FIELD_TYPE component_table_fields[CT_FIELD_COUNT] = {
-    {{C_STRING_WITH_LEN("component_id")},
-     {C_STRING_WITH_LEN("int(10)")},
+    {{STRING_WITH_LEN("component_id")},
+     {STRING_WITH_LEN("int(10)")},
      {NULL, 0}},
-    {{C_STRING_WITH_LEN("component_group_id")},
-     {C_STRING_WITH_LEN("int(10)")},
+    {{STRING_WITH_LEN("component_group_id")},
+     {STRING_WITH_LEN("int(10)")},
      {NULL, 0}},
-    {{C_STRING_WITH_LEN("component_urn")},
-     {C_STRING_WITH_LEN("text")},
-     {C_STRING_WITH_LEN("utf8")}}};
+    {{STRING_WITH_LEN("component_urn")},
+     {STRING_WITH_LEN("text")},
+     {STRING_WITH_LEN("utf8")}}};
 
 static const TABLE_FIELD_DEF component_table_def = {CT_FIELD_COUNT,
                                                     component_table_fields};
@@ -150,9 +150,7 @@ static Component_db_intact table_intact;
 */
 static bool open_component_table(THD *thd, enum thr_lock_type lock_type,
                                  TABLE **table, ulong acl_to_check) {
-  TABLE_LIST tables;
-
-  tables.init_one_table("mysql", 5, "component", 9, "component", lock_type);
+  TABLE_LIST tables("mysql", "component", lock_type);
 
   if (mysql_persistent_dynamic_loader_imp::initialized() && !opt_noacl &&
       check_one_table_access(thd, acl_to_check, &tables))
@@ -226,9 +224,10 @@ bool mysql_persistent_dynamic_loader_imp::init(void *thdp) {
     auto guard =
         create_scope_guard([&thd]() { commit_and_close_mysql_tables(thd); });
 
-    READ_RECORD read_record_info;
-    if (init_read_record(&read_record_info, thd, component_table, NULL, false,
-                         /*ignore_not_found_rows=*/false)) {
+    unique_ptr_destroy_only<RowIterator> iterator =
+        init_table_iterator(thd, component_table, NULL, false,
+                            /*ignore_not_found_rows=*/false);
+    if (iterator == nullptr) {
       push_warning(thd, Sql_condition::SL_WARNING, ER_COMPONENT_TABLE_INCORRECT,
                    ER_THD(thd, ER_COMPONENT_TABLE_INCORRECT));
       return false;
@@ -244,7 +243,7 @@ bool mysql_persistent_dynamic_loader_imp::init(void *thdp) {
     std::map<uint64, std::vector<std::string>> component_groups;
 
     for (;;) {
-      res = read_record_info->Read();
+      res = iterator->Read();
       if (res != 0) {
         break;
       }
@@ -272,7 +271,7 @@ bool mysql_persistent_dynamic_loader_imp::init(void *thdp) {
       }
     }
 
-    read_record_info.iterator.reset();
+    iterator.reset();
 
     /* res is guaranteed to be != 0, -1 means end of records encountered, which
       is interpreted as a success. */
