@@ -1,16 +1,15 @@
-/* Copyright (c) 2002, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,15 +23,15 @@
 #ifndef _SP_H_
 #define _SP_H_
 
-#include <assert.h>
 #include <stddef.h>
 #include <sys/types.h>
-
 #include <string>
 
-#include "field_types.h"
 #include "lex_string.h"
+#include "m_ctype.h"
 #include "map_helpers.h"
+#include "my_alloc.h"
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "mysql/udf_registration_types.h"
 #include "sql/item.h"     // Item::Type
@@ -42,9 +41,7 @@
 class Object_creation_ctx;
 class Query_arena;
 class THD;
-struct CHARSET_INFO;
 struct LEX_USER;
-struct MEM_ROOT;
 
 namespace dd {
 class Routine;
@@ -56,7 +53,7 @@ class Sroutine_hash_entry;
 class String;
 class sp_cache;
 struct TABLE;
-class Table_ref;
+struct TABLE_LIST;
 
 typedef ulonglong sql_mode_t;
 template <typename T>
@@ -152,15 +149,21 @@ class Stored_routine_creation_ctx : public Stored_program_creation_ctx {
                                                    TABLE *proc_tbl);
 
  public:
-  Stored_program_creation_ctx *clone(MEM_ROOT *mem_root) override;
+  virtual Stored_program_creation_ctx *clone(MEM_ROOT *mem_root) {
+    return new (mem_root)
+        Stored_routine_creation_ctx(m_client_cs, m_connection_cl, m_db_cl);
+  }
 
  protected:
-  Object_creation_ctx *create_backup_ctx(THD *thd) const override;
-  void delete_backup_ctx() override;
+  virtual Object_creation_ctx *create_backup_ctx(THD *thd) const {
+    DBUG_TRACE;
+    return new (thd->mem_root) Stored_routine_creation_ctx(thd);
+  }
+
+  virtual void delete_backup_ctx() { destroy(this); }
 
  private:
-  explicit Stored_routine_creation_ctx(THD *thd)
-      : Stored_program_creation_ctx(thd) {}
+  Stored_routine_creation_ctx(THD *thd) : Stored_program_creation_ctx(thd) {}
 
   Stored_routine_creation_ctx(const CHARSET_INFO *client_cs,
                               const CHARSET_INFO *connection_cl,
@@ -169,7 +172,7 @@ class Stored_routine_creation_ctx : public Stored_program_creation_ctx {
 };
 
 /* Drop all routines in database 'db' */
-bool sp_drop_db_routines(THD *thd, const dd::Schema &schema);
+enum_sp_return_code sp_drop_db_routines(THD *thd, const dd::Schema &schema);
 
 /**
    Acquires exclusive metadata lock on all stored routines in the
@@ -192,11 +195,10 @@ sp_head *sp_setup_routine(THD *thd, enum_sp_type type, sp_name *name,
 enum_sp_return_code sp_cache_routine(THD *thd, Sroutine_hash_entry *rt,
                                      bool lookup_only, sp_head **sp);
 
-enum_sp_return_code sp_cache_routine(THD *thd, enum_sp_type type,
-                                     const sp_name *name, bool lookup_only,
-                                     sp_head **sp);
+enum_sp_return_code sp_cache_routine(THD *thd, enum_sp_type type, sp_name *name,
+                                     bool lookup_only, sp_head **sp);
 
-bool sp_exist_routines(THD *thd, Table_ref *procs, bool is_proc);
+bool sp_exist_routines(THD *thd, TABLE_LIST *procs, bool is_proc);
 
 bool sp_show_create_routine(THD *thd, enum_sp_type type, sp_name *name);
 
@@ -208,8 +210,7 @@ enum_sp_return_code db_load_routine(
     const char *definer_host, longlong created, longlong modified,
     Stored_program_creation_ctx *creation_ctx);
 
-bool sp_create_routine(THD *thd, sp_head *sp, const LEX_USER *definer,
-                       bool if_not_exists, bool &sp_already_exists);
+bool sp_create_routine(THD *thd, sp_head *sp, const LEX_USER *definer);
 
 bool sp_update_routine(THD *thd, enum_sp_type type, sp_name *name,
                        st_sp_chistics *chistics);
@@ -304,11 +305,11 @@ class Sroutine_hash_entry {
   }
 
   const char *part_mdl_key() {
-    assert(!use_normalized_key());
+    DBUG_ASSERT(!use_normalized_key());
     return (char *)m_key + 1;
   }
   size_t part_mdl_key_length() {
-    assert(!use_normalized_key());
+    DBUG_ASSERT(!use_normalized_key());
     return m_key_length - 1U;
   }
 
@@ -322,7 +323,7 @@ class Sroutine_hash_entry {
     0 if routine is not used in view. Note that it also can be 0 if
     statement uses routine both via view and directly.
   */
-  Table_ref *belong_to_view;
+  TABLE_LIST *belong_to_view;
   /**
     This is for prepared statement validation purposes.
     A statement looks up and pre-loads all its stored functions
@@ -354,7 +355,7 @@ bool sp_add_used_routine(Query_tables_list *prelocking_ctx, Query_arena *arena,
                          size_t db_length, const char *name, size_t name_length,
                          bool lowercase_db,
                          Sp_name_normalize_type name_normalize_type,
-                         bool own_routine, Table_ref *belong_to_view);
+                         bool own_routine, TABLE_LIST *belong_to_view);
 
 /**
   Convenience wrapper around sp_add_used_routine() for most common case -
@@ -365,8 +366,8 @@ inline bool sp_add_own_used_routine(Query_tables_list *prelocking_ctx,
                                     Query_arena *arena,
                                     Sroutine_hash_entry::entry_type type,
                                     sp_name *sp_name) {
-  assert(type == Sroutine_hash_entry::FUNCTION ||
-         type == Sroutine_hash_entry::PROCEDURE);
+  DBUG_ASSERT(type == Sroutine_hash_entry::FUNCTION ||
+              type == Sroutine_hash_entry::PROCEDURE);
 
   return sp_add_used_routine(
       prelocking_ctx, arena, type, sp_name->m_db.str, sp_name->m_db.length,
@@ -378,13 +379,17 @@ void sp_remove_not_own_routines(Query_tables_list *prelocking_ctx);
 void sp_update_stmt_used_routines(
     THD *thd, Query_tables_list *prelocking_ctx,
     malloc_unordered_map<std::string, Sroutine_hash_entry *> *src,
-    Table_ref *belong_to_view);
+    TABLE_LIST *belong_to_view);
 void sp_update_stmt_used_routines(THD *thd, Query_tables_list *prelocking_ctx,
                                   SQL_I_List<Sroutine_hash_entry> *src,
-                                  Table_ref *belong_to_view);
+                                  TABLE_LIST *belong_to_view);
 
 const uchar *sp_sroutine_key(const uchar *ptr, size_t *plen);
 
+sp_head *sp_load_for_information_schema(THD *thd, LEX_CSTRING db_name,
+                                        const dd::Routine *routine,
+                                        bool *free_sp_head);
+
 bool load_charset(MEM_ROOT *mem_root, Field *field, const CHARSET_INFO *dflt_cs,
                   const CHARSET_INFO **cs);
 
@@ -405,13 +410,12 @@ uint sp_get_flags_for_command(LEX *lex);
 
 bool sp_check_name(LEX_STRING *ident);
 
-Table_ref *sp_add_to_query_tables(THD *thd, LEX *lex, const char *db,
-                                  const char *name);
+TABLE_LIST *sp_add_to_query_tables(THD *thd, LEX *lex, const char *db,
+                                   const char *name);
 
-Item *sp_prepare_func_item(THD *thd, bool standalone, Item **it_addr);
+Item *sp_prepare_func_item(THD *thd, Item **it_addr);
 
-bool sp_eval_expr(THD *thd, bool standalone, Field *result_field,
-                  Item **expr_item_ptr);
+bool sp_eval_expr(THD *thd, Field *result_field, Item **expr_item_ptr);
 
 String *sp_get_item_value(THD *thd, Item *item, String *str);
 
