@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,14 +34,13 @@
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_sqlcommand.h"
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_shared.h"
 #include "mysql/plugin.h"
 #include "mysql/psi/mysql_statement.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"  // Prealloced_array
-#include "scope_guard.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"  // check_table_access
 #include "sql/binlog.h"            // mysql_bin_log
@@ -66,7 +64,7 @@
 #include "sql/sql_const.h"
 #include "sql/sql_digest_stream.h"
 #include "sql/sql_parse.h"    // parse_sql
-#include "sql/sql_prepare.h"  // Reprepare_observer
+#include "sql/sql_prepare.h"  // reinit_stmt_before_use
 #include "sql/sql_profile.h"
 #include "sql/system_variables.h"
 #include "sql/table_trigger_dispatcher.h"  // Table_trigger_dispatcher
@@ -80,7 +78,7 @@
 class Cmp_splocal_locations {
  public:
   bool operator()(const Item_splocal *a, const Item_splocal *b) {
-    assert(a == b || a->pos_in_query != b->pos_in_query);
+    DBUG_ASSERT(a == b || a->pos_in_query != b->pos_in_query);
     return a->pos_in_query < b->pos_in_query;
   }
 };
@@ -213,7 +211,7 @@ static bool subst_spvars(THD *thd, sp_instr *instr, LEX_CSTRING query_str) {
   qbuf.length(0);
   const char *cur = query_str.str;
   int prev_pos = 0;
-  bool res = false;
+  int res = 0;
   thd->query_name_consts = 0;
 
   for (Item_splocal **splocal = sp_vars_uses.begin();
@@ -284,16 +282,15 @@ static bool subst_spvars(THD *thd, sp_instr *instr, LEX_CSTRING query_str) {
 
 class SP_instr_error_handler : public Internal_error_handler {
  public:
-  bool handle_condition(THD *thd, uint sql_errno, const char *,
-                        Sql_condition::enum_severity_level *,
-                        const char *) override {
+  virtual bool handle_condition(THD *thd, uint sql_errno, const char *,
+                                Sql_condition::enum_severity_level *,
+                                const char *) {
     /*
       Check if the "table exists" error or warning reported for the
       CREATE TABLE ... SELECT statement.
     */
     if (thd->lex && thd->lex->sql_command == SQLCOM_CREATE_TABLE &&
-        thd->lex->query_block &&
-        !thd->lex->query_block->field_list_is_empty() &&
+        thd->lex->select_lex && thd->lex->select_lex->item_list.elements > 0 &&
         sql_errno == ER_TABLE_EXISTS_ERROR)
       cts_table_exists_error = true;
 
@@ -303,83 +300,6 @@ class SP_instr_error_handler : public Internal_error_handler {
   bool cts_table_exists_error = false;
 };
 
-/**
-  Execute an expression (e.g an IF) that is not a complete SQL statement.
-
-  Expressions that may be executed in this function:
-    IF, CASE, DECLARE, SET, RETURN
-
-  @param      thd    thread context
-  @param[out] nextp  next instruction pointer
-
-  @returns false if success, true if error
-*/
-bool sp_lex_instr::execute_expression(THD *thd, uint *nextp) {
-  auto execute_guard = create_scope_guard([&]() {
-    m_lex->cleanup(true);
-    if (thd->in_sub_stmt == 0) {
-      thd->get_stmt_da()->set_overwrite_status(true);
-      thd->is_error() ? trans_rollback_stmt(thd) : trans_commit_stmt(thd);
-      thd->get_stmt_da()->set_overwrite_status(false);
-    }
-    thd_proc_info(thd, "closing tables");
-    close_thread_tables(thd);
-    thd_proc_info(thd, nullptr);
-
-    if (thd->in_sub_stmt == 0) {
-      if (thd->transaction_rollback_request) {
-        trans_rollback_implicit(thd);
-        thd->mdl_context.release_transactional_locks();
-      } else if (!thd->in_multi_stmt_transaction_mode()) {
-        thd->mdl_context.release_transactional_locks();
-      } else {
-        thd->mdl_context.release_statement_locks();
-      }
-    }
-  });
-
-  /*
-    Check privileges for tables in expression, open and lock those tables,
-    bind data to expression so that it is ready for execution.
-
-    Notice that temporary tables must be opened before privilege checking.
-    This is because a session has all privileges for any temporary table that
-    it has created, however a table must be opened in order to identify it as
-    a temporary table.
-  */
-  if (m_lex->query_tables != nullptr) {
-    if (open_temporary_tables(thd, m_lex->query_tables)) {
-      return true;
-    }
-    if (check_table_access(thd, SELECT_ACL, m_lex->query_tables, false,
-                           UINT_MAX, false)) {
-      return true;
-    }
-  }
-  if (open_and_lock_tables(thd, m_lex->query_tables, 0)) {
-    return true;
-  }
-
-  if (m_arena.get_state() != Query_arena::STMT_INITIALIZED_FOR_SP) {
-    m_lex->restore_cmd_properties();
-    bind_fields(m_arena.item_list());
-  }
-  /*
-    Trace the expression. This is not an SQL statement, but pretend it is
-    a SELECT query expression.
-  */
-  Opt_trace_start ots(thd, m_lex->query_tables, SQLCOM_SELECT, &m_lex->var_list,
-                      nullptr, 0, this, thd->variables.character_set_client);
-  const Opt_trace_object trace_command(&thd->opt_trace);
-  const Opt_trace_array trace_command_steps(&thd->opt_trace, "steps");
-
-  if (exec_core(thd, nextp)) {
-    return true;
-  }
-
-  return false;
-}
-
 bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
                                            bool open_tables) {
   /*
@@ -394,7 +314,7 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
 
   /* Check pre-conditions. */
 
-  assert(thd->change_list.is_empty());
+  DBUG_ASSERT(thd->change_list.is_empty());
 
   /*
     Use our own lex.
@@ -407,7 +327,6 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
 
   LEX *lex_saved = thd->lex;
   thd->lex = m_lex;
-  m_lex->thd = thd;
 
   /* Set new query id. */
 
@@ -430,13 +349,11 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
     }
   }
 
-  bool error = m_lex->check_preparation_invalid(thd);
-
-  m_lex->clear_execution();
+  bool error = reinit_stmt_before_use(thd, m_lex);
 
   /*
     In case a session state exists do not cache the SELECT stmt. If we
-    cache SELECT statement when session state information exists, then
+    cache SELECT statment when session state information exists, then
     the result sets of this SELECT are cached which contains changed
     session information. Next time when same query is executed when there
     is no change in session state, then result sets are picked from cache
@@ -446,7 +363,7 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
 
   if (thd->get_protocol()->has_client_capability(CLIENT_SESSION_TRACK) &&
       thd->session_tracker.enabled_any() && thd->session_tracker.changed_any())
-    thd->lex->safe_to_cache_query = false;
+    thd->lex->safe_to_cache_query = 0;
 
   SP_instr_error_handler sp_instr_error_handler;
   thd->push_internal_handler(&sp_instr_error_handler);
@@ -455,8 +372,63 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
 
   if (!error) {
     if (open_tables) {
-      error = execute_expression(thd, nextp);
-      DBUG_PRINT("info", ("exec_expression returned: %d", error));
+      // todo: break this block out into a separate function.
+      /*
+        IF, CASE, DECLARE, SET, RETURN, have 'open_tables' true; they may
+        have a subquery in parameter and are worth tracing. They don't
+        correspond to a SQL command so we pretend that they are SQLCOM_SELECT.
+      */
+      Opt_trace_start ots(thd, m_lex->query_tables, SQLCOM_SELECT,
+                          &m_lex->var_list, NULL, 0, this,
+                          thd->variables.character_set_client);
+      Opt_trace_object trace_command(&thd->opt_trace);
+      Opt_trace_array trace_command_steps(&thd->opt_trace, "steps");
+
+      /*
+        Check whenever we have access to tables for this statement
+        and open and lock them before executing instructions core function.
+        If we are not opening any tables, we don't need to check permissions
+        either.
+      */
+      if (m_lex->query_tables)
+        error = (open_temporary_tables(thd, m_lex->query_tables) ||
+                 check_table_access(thd, SELECT_ACL, m_lex->query_tables, false,
+                                    UINT_MAX, false));
+
+      if (!error) error = open_and_lock_tables(thd, m_lex->query_tables, 0);
+
+      if (!error) {
+        error = exec_core(thd, nextp);
+        DBUG_PRINT("info", ("exec_core returned: %d", error));
+      }
+
+      /*
+        Call after unit->cleanup() to close open table
+        key read.
+      */
+
+      m_lex->unit->cleanup(thd, true);
+
+      /* Here we also commit or rollback the current statement. */
+
+      if (!thd->in_sub_stmt) {
+        thd->get_stmt_da()->set_overwrite_status(true);
+        thd->is_error() ? trans_rollback_stmt(thd) : trans_commit_stmt(thd);
+        thd->get_stmt_da()->set_overwrite_status(false);
+      }
+      thd_proc_info(thd, "closing tables");
+      close_thread_tables(thd);
+      thd_proc_info(thd, 0);
+
+      if (!thd->in_sub_stmt) {
+        if (thd->transaction_rollback_request) {
+          trans_rollback_implicit(thd);
+          thd->mdl_context.release_transactional_locks();
+        } else if (!thd->in_multi_stmt_transaction_mode())
+          thd->mdl_context.release_transactional_locks();
+        else
+          thd->mdl_context.release_statement_locks();
+      }
     } else {
       DEBUG_SYNC(thd, "sp_lex_instr_before_exec_core");
       error = exec_core(thd, nextp);
@@ -479,8 +451,8 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
     */
     m_lex_query_tables_own_last = m_lex->query_tables_own_last;
     m_prelocking_tables = *m_lex_query_tables_own_last;
-    *m_lex_query_tables_own_last = nullptr;
-    m_lex->mark_as_requiring_prelocking(nullptr);
+    *m_lex_query_tables_own_last = NULL;
+    m_lex->mark_as_requiring_prelocking(NULL);
   }
 
   /* Rollback changes to the item tree during execution. */
@@ -497,14 +469,14 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
     statement. To make sure that items are created in the statement mem_root,
     change state to STMT_INITIALIZED_FOR_SP.
 
-    When a "table exists" error occurs for CREATE TABLE ... SELECT change state
+    When a "table exists" error occur for CREATE TABLE ... SELECT change state
     to STMT_INITIALIZED_FOR_SP, as if statement must be reprepared.
 
       Why is this necessary? A useful pointer would be to note how
       PREPARE/EXECUTE uses functions like select_like_stmt_test to implement
       CREATE TABLE .... SELECT. The SELECT part of the DDL is resolved first.
       Then there is an attempt to create the table. So in the execution phase,
-      if "table exists" error occurs or flush table precedes the execute, the
+      if "table exists" error occurs or flush table preceeds the execute, the
       item tree of the select is re-created and followed by an attempt to create
       the table.
 
@@ -525,7 +497,7 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
                          thd->get_stmt_da()->mysql_errno() == ER_NEED_REPREPARE;
 
   // Unless there is an error, execution must have started (and completed)
-  assert(error || m_lex->is_exec_started());
+  DBUG_ASSERT(error || m_lex->is_exec_started());
 
   if (reprepare_error || sp_instr_error_handler.cts_table_exists_error)
     thd->stmt_arena->set_state(Query_arena::STMT_INITIALIZED_FOR_SP);
@@ -541,8 +513,9 @@ bool sp_lex_instr::reset_lex_and_exec_core(THD *thd, uint *nextp,
       Transaction_ctx::STMT, parent_unsafe_rollback_flags);
 
   if (thd->variables.session_track_transaction_info > TX_TRACK_NONE) {
-    TX_TRACKER_GET(tst);
-    tst->add_trx_state_from_thd(thd);
+    ((Transaction_state_tracker *)thd->session_tracker.get_tracker(
+         TRANSACTION_INFO_TRACKER))
+        ->add_trx_state_from_thd(thd);
   }
 
   /* Restore original lex. */
@@ -566,7 +539,7 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
   String sql_query;
   sql_digest_state *parent_digest = thd->m_digest;
   PSI_statement_locker *parent_locker = thd->m_statement_psi;
-  SQL_I_List<Item_trigger_field> *next_trig_list_bkp = nullptr;
+  SQL_I_List<Item_trigger_field> *next_trig_list_bkp = NULL;
   sql_query.set_charset(system_charset_info);
 
   get_query(&sql_query);
@@ -575,9 +548,9 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
     // The instruction has returned zero-length query string. That means, the
     // re-preparation of the instruction is not possible. We should not come
     // here in the normal life.
-    assert(false);
+    DBUG_ASSERT(false);
     my_error(ER_UNKNOWN_ERROR, MYF(0));
-    return nullptr;
+    return NULL;
   }
 
   if (m_trig_field_list.elements)
@@ -586,7 +559,9 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
   cleanup_before_parsing(thd);
 
   // Cleanup and re-init the lex mem_root for re-parse.
-  m_lex_mem_root.ClearForReuse();
+  free_root(&m_lex_mem_root, MYF(0));
+  init_sql_alloc(PSI_NOT_INSTRUMENTED, &m_lex_mem_root, MEM_ROOT_BLOCK_SIZE,
+                 MEM_ROOT_PREALLOC);
 
   /*
     Switch mem-roots. We store the new LEX and its Items in the
@@ -606,7 +581,7 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
   Parser_state parser_state;
 
   if (parser_state.init(thd, sql_query.c_ptr(), sql_query.length()))
-    return nullptr;
+    return NULL;
 
   // Switch THD's item list. It is used to remember the newly created set
   // of Items during parsing. We should clean those items after each execution.
@@ -614,7 +589,7 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
   Item *execution_item_list = thd->item_list();
   thd->reset_item_list();
 
-  // Create a new LEX and initialize it.
+  // Create a new LEX and intialize it.
 
   LEX *lex_saved = thd->lex;
 
@@ -627,20 +602,15 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
 
   // Parse the just constructed SELECT-statement.
 
-  thd->m_digest = nullptr;
-  thd->m_statement_psi = nullptr;
-  bool parsing_failed = parse_sql(thd, &parser_state, nullptr);
+  thd->m_digest = NULL;
+  thd->m_statement_psi = NULL;
+  bool parsing_failed = parse_sql(thd, &parser_state, NULL);
   thd->m_digest = parent_digest;
   thd->m_statement_psi = parent_locker;
 
   if (!parsing_failed) {
-    adjust_sql_command(thd->lex);
     thd->lex->set_trg_event_type_for_tables();
 
-    // Mark statement as belonging to a stored procedure:
-    if (thd->lex->m_sql_cmd != nullptr)
-      thd->lex->m_sql_cmd->set_as_part_of_sp();
-
     // Call after-parsing callback.
     parsing_failed = on_after_expr_parsing(thd);
 
@@ -656,9 +626,9 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
 
       Trigger *t = sp->m_trg_list->find_trigger(thd->lex->sphead->m_name);
 
-      assert(t);
+      DBUG_ASSERT(t);
 
-      if (!t) return nullptr;  // Don't take chances in production.
+      if (!t) return NULL;  // Don't take chances in production.
 
       for (Item_trigger_field *trg_fld = sp->m_cur_instr_trig_field_items.first;
            trg_fld; trg_fld = trg_fld->next_trg_field) {
@@ -682,8 +652,8 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
 
   // Restore THD::lex.
 
-  thd->lex->sphead = nullptr;
-  thd->lex->set_sp_current_parsing_ctx(nullptr);
+  thd->lex->sphead = NULL;
+  thd->lex->set_sp_current_parsing_ctx(NULL);
 
   LEX *expr_lex = thd->lex;
   thd->lex = lex_saved;
@@ -695,21 +665,18 @@ LEX *sp_lex_instr::parse_expr(THD *thd, sp_head *sp) {
 
   // That's it.
 
-  return parsing_failed ? nullptr : expr_lex;
+  return parsing_failed ? NULL : expr_lex;
 }
 
 bool sp_lex_instr::validate_lex_and_execute_core(THD *thd, uint *nextp,
                                                  bool open_tables) {
   Reprepare_observer reprepare_observer;
+  int reprepare_attempt = 0;
+  const int MAX_REPREPARE_ATTEMPTS = 3;
 
   while (true) {
     DBUG_EXECUTE_IF("simulate_bug18831513", { invalidate(); });
-    if (is_invalid() ||
-        (!m_first_execution &&
-         (m_lex->has_udf() ||
-          (m_lex->m_sql_cmd != nullptr &&
-           m_lex->m_sql_cmd->reprepare_on_execute_required())))) {
-      free_lex();
+    if (is_invalid()) {
       LEX *lex = parse_expr(thd, thd->sp_runtime_ctx->sp);
 
       if (!lex) return true;
@@ -725,7 +692,7 @@ bool sp_lex_instr::validate_lex_and_execute_core(THD *thd, uint *nextp,
       the observer method will be invoked to push an error into
       the error stack.
     */
-    Reprepare_observer *stmt_reprepare_observer = nullptr;
+    Reprepare_observer *stmt_reprepare_observer = NULL;
 
     /*
       Meta-data versions are stored in the LEX-object on the first execution.
@@ -766,15 +733,17 @@ bool sp_lex_instr::validate_lex_and_execute_core(THD *thd, uint *nextp,
           raise it to the user;
     */
     if (stmt_reprepare_observer == nullptr || thd->is_fatal_error() ||
-        thd->killed || thd->get_stmt_da()->mysql_errno() != ER_NEED_REPREPARE) {
+        thd->killed || thd->get_stmt_da()->mysql_errno() != ER_NEED_REPREPARE)
       return true;
-    }
+
     /*
-      Reprepare_observer ensures that the statement is retried a maximum number
-      of times, to avoid an endless loop.
+      We take only 3 attempts to reprepare the query, otherwise we might end
+      up in the endless loop.
     */
-    assert(stmt_reprepare_observer->is_invalidated());
-    if (!stmt_reprepare_observer->can_retry()) {
+    DBUG_ASSERT(stmt_reprepare_observer->is_invalidated());
+    if ((reprepare_attempt++ >= MAX_REPREPARE_ATTEMPTS) ||
+        DBUG_EVALUATE_IF("simulate_max_reprepare_attempts_hit_case", true,
+                         false)) {
       /*
         Reprepare_observer sets error status in DA but Sql_condition is not
         added. Please check Reprepare_observer::report_error(). Pushing
@@ -787,6 +756,7 @@ bool sp_lex_instr::validate_lex_and_execute_core(THD *thd, uint *nextp,
     }
 
     thd->clear_error();
+    free_lex();
     invalidate();
   }
 }
@@ -796,7 +766,7 @@ void sp_lex_instr::set_lex(LEX *lex, bool is_lex_owner) {
 
   m_lex = lex;
   m_is_lex_owner = is_lex_owner;
-  m_lex_query_tables_own_last = nullptr;
+  m_lex_query_tables_own_last = NULL;
 
   if (m_lex) m_lex->sp_lex_in_use = true;
 }
@@ -805,15 +775,13 @@ void sp_lex_instr::free_lex() {
   if (!m_is_lex_owner || !m_lex) return;
 
   /* Prevent endless recursion. */
-  m_lex->sphead = nullptr;
+  m_lex->sphead = NULL;
   lex_end(m_lex);
-  destroy(m_lex->result);
-  m_lex->destroy();
   delete (st_lex_local *)m_lex;
 
-  m_lex = nullptr;
+  m_lex = NULL;
   m_is_lex_owner = false;
-  m_lex_query_tables_own_last = nullptr;
+  m_lex_query_tables_own_last = NULL;
 }
 
 void sp_lex_instr::cleanup_before_parsing(THD *thd) {
@@ -826,7 +794,7 @@ void sp_lex_instr::cleanup_before_parsing(THD *thd) {
   // Remove previously stored trigger-field items.
   sp_head *sp = thd->sp_runtime_ctx->sp;
 
-  if (sp->m_type == enum_sp_type::TRIGGER) m_trig_field_list.clear();
+  if (sp->m_type == enum_sp_type::TRIGGER) m_trig_field_list.empty();
 }
 
 void sp_lex_instr::get_query(String *sql_query) const {
@@ -846,8 +814,7 @@ void sp_lex_instr::get_query(String *sql_query) const {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_stmt::psi_info = {0, "stmt", 0,
-                                              "Stored Program: SQL statement"};
+PSI_statement_info sp_instr_stmt::psi_info = {0, "stmt", 0, PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_stmt::execute(THD *thd, uint *nextp) {
@@ -935,7 +902,7 @@ bool sp_instr_stmt::execute(THD *thd, uint *nextp) {
       and therefore pass in a null-pointer instead of a pointer to
       state at the beginning of execution.
     */
-    log_slow_do(thd);
+    log_slow_do(thd, nullptr);
   }
 
   /*
@@ -945,8 +912,8 @@ bool sp_instr_stmt::execute(THD *thd, uint *nextp) {
     double-check whether the potential conflict they created is a
     problem.
   */
-  assert((thd->query_name_consts == 0) ||
-         (thd->rewritten_query().length() == 0));
+  DBUG_ASSERT((thd->query_name_consts == 0) ||
+              (thd->rewritten_query.length() == 0));
 
   thd->set_query(query_backup);
   thd->query_name_consts = 0;
@@ -979,18 +946,15 @@ void sp_instr_stmt::print(const THD *, String *str) {
 }
 
 bool sp_instr_stmt::exec_core(THD *thd, uint *nextp) {
-  LEX *const lex = thd->lex;
-  lex->set_sp_current_parsing_ctx(get_parsing_ctx());
-  lex->sphead = thd->sp_runtime_ctx->sp;
+  thd->lex->set_sp_current_parsing_ctx(get_parsing_ctx());
+  thd->lex->sphead = thd->sp_runtime_ctx->sp;
 
   PSI_statement_locker *statement_psi_saved = thd->m_statement_psi;
 
-  assert(lex->m_sql_cmd == nullptr || lex->m_sql_cmd->is_part_of_sp());
-
   bool rc = mysql_execute_command(thd);
 
-  lex->set_sp_current_parsing_ctx(nullptr);
-  lex->sphead = nullptr;
+  thd->lex->set_sp_current_parsing_ctx(NULL);
+  thd->lex->sphead = NULL;
   thd->m_statement_psi = statement_psi_saved;
 
   *nextp = get_ip() + 1;
@@ -1003,21 +967,18 @@ bool sp_instr_stmt::exec_core(THD *thd, uint *nextp) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_set::psi_info = {
-    0, "set", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: SET statement"};
+PSI_statement_info sp_instr_set::psi_info = {0, "set", 0, PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_set::exec_core(THD *thd, uint *nextp) {
   *nextp = get_ip() + 1;
 
-  // LEX of instruction keeps execution state of the assignment operation
-  if (!thd->sp_runtime_ctx->set_variable(thd, true, m_offset, &m_value_item))
+  if (!thd->sp_runtime_ctx->set_variable(thd, m_offset, &m_value_item))
     return false;
 
   /* Failed to evaluate the value. Reset the variable to NULL. */
 
-  if (thd->sp_runtime_ctx->set_variable(thd, true, m_offset, nullptr)) {
+  if (thd->sp_runtime_ctx->set_variable(thd, m_offset, 0)) {
     /* If this also failed, let's abort. */
     my_error(ER_OUT_OF_RESOURCES, MYF(ME_FATALERROR));
   }
@@ -1049,8 +1010,7 @@ void sp_instr_set::print(const THD *thd, String *str) {
 
 #ifdef HAVE_PSI_INTERFACE
 PSI_statement_info sp_instr_set_trigger_field::psi_info = {
-    0, "set_trigger_field", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: SET NEW.<field> in TRIGGER"};
+    0, "set_trigger_field", 0, PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_set_trigger_field::exec_core(THD *thd, uint *nextp) {
@@ -1080,10 +1040,11 @@ void sp_instr_set_trigger_field::print(const THD *thd, String *str) {
 }
 
 bool sp_instr_set_trigger_field::on_after_expr_parsing(THD *thd) {
-  m_value_item = thd->lex->query_block->single_visible_field();
-  assert(m_value_item != nullptr);
+  DBUG_ASSERT(thd->lex->select_lex->item_list.elements == 1);
 
-  assert(!m_trigger_field);
+  m_value_item = thd->lex->select_lex->item_list.head();
+
+  DBUG_ASSERT(!m_trigger_field);
 
   m_trigger_field = new (thd->mem_root)
       Item_trigger_field(thd->lex->current_context(), TRG_NEW_ROW,
@@ -1096,13 +1057,13 @@ bool sp_instr_set_trigger_field::on_after_expr_parsing(THD *thd) {
         m_trigger_field, &m_trigger_field->next_trg_field);
   }
 
-  return m_value_item == nullptr || m_trigger_field == nullptr;
+  return m_value_item == NULL || m_trigger_field == NULL;
 }
 
 void sp_instr_set_trigger_field::cleanup_before_parsing(THD *thd) {
   sp_lex_instr::cleanup_before_parsing(thd);
 
-  m_trigger_field = nullptr;
+  m_trigger_field = NULL;
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -1110,9 +1071,7 @@ void sp_instr_set_trigger_field::cleanup_before_parsing(THD *thd) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_jump::psi_info = {
-    0, "jump", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: jump microcode instruction"};
+PSI_statement_info sp_instr_jump::psi_info = {0, "jump", 0, PSI_DOCUMENT_ME};
 #endif
 
 void sp_instr_jump::print(const THD *, String *str) {
@@ -1158,17 +1117,16 @@ void sp_instr_jump::opt_move(uint dst, List<sp_branch_instr> *bp) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_jump_if_not::psi_info = {
-    0, "jump_if_not", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: jump if false microcode instruction"};
+PSI_statement_info sp_instr_jump_if_not::psi_info = {0, "jump_if_not", 0,
+                                                     PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_jump_if_not::exec_core(THD *thd, uint *nextp) {
-  assert(m_expr_item != nullptr);
+  DBUG_ASSERT(m_expr_item);
 
-  // LEX of instruction keeps execution state of the expression evaluation
-  Item *item = sp_prepare_func_item(thd, true, &m_expr_item);
-  if (item == nullptr) return true;
+  Item *item = sp_prepare_func_item(thd, &m_expr_item);
+
+  if (!item) return true;
 
   *nextp = item->val_bool() ? get_ip() + 1 : m_dest;
 
@@ -1241,17 +1199,16 @@ void sp_lex_branch_instr::opt_move(uint dst, List<sp_branch_instr> *bp) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_jump_case_when::psi_info = {
-    0, "jump_case_when", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: jump CASE WHEN microcode instruction"};
+PSI_statement_info sp_instr_jump_case_when::psi_info = {0, "jump_case_when", 0,
+                                                        PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_jump_case_when::exec_core(THD *thd, uint *nextp) {
-  assert(m_eq_item != nullptr);
+  DBUG_ASSERT(m_eq_item);
 
-  // LEX of instruction keeps execution state of the case expression
-  Item *item = sp_prepare_func_item(thd, true, &m_eq_item);
-  if (item == nullptr) return true;
+  Item *item = sp_prepare_func_item(thd, &m_eq_item);
+
+  if (!item) return true;
 
   *nextp = item->val_bool() ? get_ip() + 1 : m_dest;
 
@@ -1278,7 +1235,7 @@ bool sp_instr_jump_case_when::on_after_expr_parsing(THD *thd) {
 
   if (!m_case_expr_item) return true;
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   m_case_expr_item->m_sp = thd->lex->sphead;
 #endif
 
@@ -1287,7 +1244,7 @@ bool sp_instr_jump_case_when::on_after_expr_parsing(THD *thd) {
   // This function can be called in two cases:
   //
   //   - during initial (regular) parsing of SP. In this case we don't have
-  //     lex->query_block (because it's not a SELECT statement), but
+  //     lex->select_lex (because it's not a SELECT statement), but
   //     m_expr_item is already set in constructor.
   //
   //   - during re-parsing after meta-data change. In this case we've just
@@ -1295,8 +1252,9 @@ bool sp_instr_jump_case_when::on_after_expr_parsing(THD *thd) {
   //     item from its list.
 
   if (!m_expr_item) {
-    m_expr_item = thd->lex->query_block->single_visible_field();
-    assert(m_expr_item != nullptr);
+    DBUG_ASSERT(thd->lex->select_lex->item_list.elements == 1);
+
+    m_expr_item = thd->lex->select_lex->item_list.head();
   }
 
   // Setup main expression item (m_expr_item).
@@ -1313,9 +1271,8 @@ bool sp_instr_jump_case_when::on_after_expr_parsing(THD *thd) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_freturn::psi_info = {
-    0, "freturn", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: RETURN from STORED FUNCTION"};
+PSI_statement_info sp_instr_freturn::psi_info = {0, "freturn", 0,
+                                                 PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_freturn::exec_core(THD *thd, uint *nextp) {
@@ -1334,7 +1291,7 @@ bool sp_instr_freturn::exec_core(THD *thd, uint *nextp) {
     do it in scope of execution the current context/block.
   */
 
-  return thd->sp_runtime_ctx->set_return_value(thd, true, &m_expr_item);
+  return thd->sp_runtime_ctx->set_return_value(thd, &m_expr_item);
 }
 
 void sp_instr_freturn::print(const THD *thd, String *str) {
@@ -1352,9 +1309,8 @@ void sp_instr_freturn::print(const THD *thd, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_hpush_jump::psi_info = {
-    0, "hpush_jump", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: install a DECLARE HANDLER microcode instruction"};
+PSI_statement_info sp_instr_hpush_jump::psi_info = {0, "hpush_jump", 0,
+                                                    PSI_DOCUMENT_ME};
 #endif
 
 sp_instr_hpush_jump::sp_instr_hpush_jump(uint ip, sp_pcontext *ctx,
@@ -1363,12 +1319,12 @@ sp_instr_hpush_jump::sp_instr_hpush_jump(uint ip, sp_pcontext *ctx,
       m_handler(handler),
       m_opt_hpop(0),
       m_frame(ctx->current_var_count()) {
-  assert(m_handler->condition_values.elements == 0);
+  DBUG_ASSERT(m_handler->condition_values.elements == 0);
 }
 
 sp_instr_hpush_jump::~sp_instr_hpush_jump() {
-  m_handler->condition_values.clear();
-  m_handler = nullptr;
+  m_handler->condition_values.empty();
+  m_handler = NULL;
 }
 
 void sp_instr_hpush_jump::add_condition(sp_condition_value *condition_value) {
@@ -1428,9 +1384,7 @@ uint sp_instr_hpush_jump::opt_mark(sp_head *sp, List<sp_instr> *leads) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_hpop::psi_info = {
-    0, "hpop", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: uninstall a DECLARE HANDLER microcode instruction"};
+PSI_statement_info sp_instr_hpop::psi_info = {0, "hpop", 0, PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_hpop::execute(THD *thd, uint *nextp) {
@@ -1444,9 +1398,8 @@ bool sp_instr_hpop::execute(THD *thd, uint *nextp) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_hreturn::psi_info = {
-    0, "hreturn", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: return from a DECLARE HANDLER microcode instruction"};
+PSI_statement_info sp_instr_hreturn::psi_info = {0, "hreturn", 0,
+                                                 PSI_DOCUMENT_ME};
 #endif
 
 sp_instr_hreturn::sp_instr_hreturn(uint ip, sp_pcontext *ctx)
@@ -1509,9 +1462,7 @@ uint sp_instr_hreturn::opt_mark(sp_head *, List<sp_instr> *) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_cpush::psi_info = {
-    0, "cpush", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: install a DECLARE CURSOR microcode instruction"};
+PSI_statement_info sp_instr_cpush::psi_info = {0, "cpush", 0, PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_cpush::execute(THD *thd, uint *nextp) {
@@ -1554,9 +1505,7 @@ void sp_instr_cpush::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_cpop::psi_info = {
-    0, "cpop", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: uninstall a DECLARE CURSOR microcode instruction"};
+PSI_statement_info sp_instr_cpop::psi_info = {0, "cpop", 0, PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_cpop::execute(THD *thd, uint *nextp) {
@@ -1578,9 +1527,7 @@ void sp_instr_cpop::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_copen::psi_info = {
-    0, "copen", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: OPEN cursor"};
+PSI_statement_info sp_instr_copen::psi_info = {0, "copen", 0, PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_copen::execute(THD *thd, uint *nextp) {
@@ -1645,9 +1592,8 @@ void sp_instr_copen::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_cclose::psi_info = {
-    0, "cclose", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: CLOSE cursor"};
+PSI_statement_info sp_instr_cclose::psi_info = {0, "cclose", 0,
+                                                PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_cclose::execute(THD *thd, uint *nextp) {
@@ -1682,9 +1628,8 @@ void sp_instr_cclose::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_cfetch::psi_info = {
-    0, "cfetch", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: FETCH cursor"};
+PSI_statement_info sp_instr_cfetch::psi_info = {0, "cfetch", 0,
+                                                PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_cfetch::execute(THD *thd, uint *nextp) {
@@ -1728,9 +1673,7 @@ void sp_instr_cfetch::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_error::psi_info = {
-    0, "error", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: CASE WHEN not found microcode instruction"};
+PSI_statement_info sp_instr_error::psi_info = {0, "error", 0, PSI_DOCUMENT_ME};
 #endif
 
 void sp_instr_error::print(const THD *, String *str) {
@@ -1745,9 +1688,8 @@ void sp_instr_error::print(const THD *, String *str) {
 ///////////////////////////////////////////////////////////////////////////
 
 #ifdef HAVE_PSI_INTERFACE
-PSI_statement_info sp_instr_set_case_expr::psi_info = {
-    0, "set_case_expr", PSI_FLAG_DISABLED | PSI_FLAG_UNTIMED,
-    "Stored Program: CASE microcode instruction"};
+PSI_statement_info sp_instr_set_case_expr::psi_info = {0, "set_case_expr", 0,
+                                                       PSI_DOCUMENT_ME};
 #endif
 
 bool sp_instr_set_case_expr::exec_core(THD *thd, uint *nextp) {
@@ -1755,21 +1697,21 @@ bool sp_instr_set_case_expr::exec_core(THD *thd, uint *nextp) {
 
   sp_rcontext *rctx = thd->sp_runtime_ctx;
 
-  // LEX of instruction keeps execution state of the case expression
-  if (rctx->set_case_expr(thd, true, m_case_expr_id, &m_expr_item)) {
-    if (!rctx->get_case_expr(m_case_expr_id)) {
-      // Failed to evaluate the value, the case expression is still not
-      // initialized. Set to NULL so we can continue.
-      Item *null_item = new Item_null();
+  if (rctx->set_case_expr(thd, m_case_expr_id, &m_expr_item) &&
+      !rctx->get_case_expr(m_case_expr_id)) {
+    // Failed to evaluate the value, the case expression is still not
+    // initialized. Set to NULL so we can continue.
 
-      if (null_item == nullptr ||
-          rctx->set_case_expr(thd, true, m_case_expr_id, &null_item)) {
-        // If this also failed, we have to abort.
-        my_error(ER_OUT_OF_RESOURCES, MYF(ME_FATALERROR));
-      }
+    Item *null_item = new Item_null();
+
+    if (!null_item || rctx->set_case_expr(thd, m_case_expr_id, &null_item)) {
+      // If this also failed, we have to abort.
+      my_error(ER_OUT_OF_RESOURCES, MYF(ME_FATALERROR));
     }
+
     return true;
   }
+
   return false;
 }
 
