@@ -1,16 +1,15 @@
-/* Copyright (c) 2019, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,26 +20,15 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include "sql/sql_check_constraint.h"
+#include "sql_check_constraint.h"
 
+#include "item_func.h"                             // print
 #include "libbinlogevents/include/binlog_event.h"  // UNDEFINED_SERVER_VERSION
-#include "m_ctype.h"                               // CHARSET_INFO
-#include "my_inttypes.h"                           // MYF, uchar
-#include "my_sys.h"                                // my_error
-#include "mysql/thread_type.h"                     // SYSTEM_THREAD_SLAVE_*
-#include "mysql_com.h"                             // NAME_CHAR_LEN
 #include "mysqld_error.h"                          // ER_*
-#include "sql/create_field.h"                      // Create_field
-#include "sql/enum_query_type.h"                   // QT_*
-#include "sql/field.h"             // pre_validate_value_generator_expr
-#include "sql/item.h"              // Item, Item_field
-#include "sql/sql_class.h"         // THD
-#include "sql/sql_const.h"         // enum_walk
-#include "sql/sql_list.h"          // List
-#include "sql/sql_parse.h"         // check_string_char_length
-#include "sql/system_variables.h"  // System_variables
-#include "sql/thd_raii.h"          // Sql_mode_parse_guard
-#include "sql_string.h"            // String
+#include "sql_parse.h"                             // check_string_char_length
+#include "sql_string.h"                            // String
+#include "strfunc.h"                               // make_lex_string_root
+#include "thd_raii.h"                              // Sql_mode_parse_guard
 
 bool Sql_check_constraint_spec::pre_validate() {
   /*
@@ -48,7 +36,7 @@ bool Sql_check_constraint_spec::pre_validate() {
     check constraint then name is generated before calling this method.
   */
   if (check_string_char_length(to_lex_cstring(name), "", NAME_CHAR_LEN,
-                               system_charset_info, true)) {
+                               system_charset_info, 1)) {
     my_error(ER_TOO_LONG_IDENT, MYF(0), name.str);
     return true;
   }
@@ -91,11 +79,13 @@ void Sql_check_constraint_spec::print_expr(THD *thd, String &out) {
 }
 
 bool Sql_check_constraint_spec::expr_refers_column(const char *column_name) {
-  mem_root_deque<Item_field *> fields(current_thd->mem_root);
+  List<Item_field> fields;
   check_expr->walk(&Item::collect_item_field_processor, enum_walk::POSTFIX,
                    (uchar *)&fields);
 
-  for (Item_field *cur_item : fields) {
+  Item_field *cur_item;
+  List_iterator<Item_field> fields_it(fields);
+  while ((cur_item = fields_it++)) {
     if (cur_item->type() == Item::FIELD_ITEM &&
         !my_strcasecmp(system_charset_info, cur_item->field_name, column_name))
       return true;
@@ -103,6 +93,10 @@ bool Sql_check_constraint_spec::expr_refers_column(const char *column_name) {
   return false;
 }
 
+Sql_table_check_constraint::~Sql_table_check_constraint() {
+  if (m_val_gen != nullptr) delete m_val_gen;
+}
+
 bool is_slave_with_master_without_check_constraints_support(THD *thd) {
   return ((thd->system_thread &
            (SYSTEM_THREAD_SLAVE_SQL | SYSTEM_THREAD_SLAVE_WORKER)) &&
@@ -112,14 +106,16 @@ bool is_slave_with_master_without_check_constraints_support(THD *thd) {
 
 bool check_constraint_expr_refers_to_only_column(Item *check_expr,
                                                  const char *column_name) {
-  mem_root_deque<Item_field *> fields(current_thd->mem_root);
+  List<Item_field> fields;
   check_expr->walk(&Item::collect_item_field_processor, enum_walk::POSTFIX,
                    (uchar *)&fields);
 
   // Expression does not refer to any columns.
-  if (fields.empty()) return false;
+  if (fields.elements == 0) return false;
 
-  for (Item_field *cur_item : fields) {
+  Item_field *cur_item;
+  List_iterator<Item_field> fields_it(fields);
+  while ((cur_item = fields_it++)) {
     // Expression refers to some other column.
     if (cur_item->type() == Item::FIELD_ITEM &&
         my_strcasecmp(system_charset_info, cur_item->field_name, column_name))
@@ -127,43 +123,3 @@ bool check_constraint_expr_refers_to_only_column(Item *check_expr,
   }
   return true;
 }
-
-bool Check_constraint_column_dependency_checker::
-    any_check_constraint_uses_column(const char *column_name) {
-  auto column_used_by_constraint =
-      [column_name](Sql_check_constraint_spec *cc_spec) {
-        if (cc_spec->expr_refers_column(column_name)) {
-          my_error(ER_DEPENDENT_BY_CHECK_CONSTRAINT, MYF(0), cc_spec->name.str,
-                   column_name);
-          return true;
-        }
-        return false;
-      };
-
-  return std::any_of(m_check_constraint_list.begin(),
-                     m_check_constraint_list.end(), column_used_by_constraint);
-}
-
-bool Check_constraint_column_dependency_checker::operator()(
-    const Alter_drop *drop) {
-  if (drop->type != Alter_drop::COLUMN) return false;
-  return any_check_constraint_uses_column(drop->name);
-}
-
-bool Check_constraint_column_dependency_checker::operator()(
-    const Alter_column *alter_column) {
-  if (alter_column->change_type() != Alter_column::Type::RENAME_COLUMN)
-    return false;
-  if (my_strcasecmp(system_charset_info, alter_column->name,
-                    alter_column->m_new_name) == 0)
-    return false;
-  return any_check_constraint_uses_column(alter_column->name);
-}
-
-bool Check_constraint_column_dependency_checker::operator()(
-    const Create_field &fld) {
-  if (fld.change == nullptr) return false;
-  if (my_strcasecmp(system_charset_info, fld.field_name, fld.change) == 0)
-    return false;
-  return any_check_constraint_uses_column(fld.change);
-}
