@@ -68,15 +68,16 @@
 #include "mysql/service_mysql_alloc.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
+#include "sql-common/net_ns.h"  // set_network_namespace
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"  // SUPER_ACL
 #include "sql/auth/sql_security_ctx.h"
-#include "sql/derror.h"     // ER_THD
-#include "sql/hostname.h"   // Host_errors
-#include "sql/item_func.h"  // mqh_used
+#include "sql/debug_sync.h"      // DEBUG_SYNC
+#include "sql/derror.h"          // ER_THD
+#include "sql/hostname_cache.h"  // Host_errors
+#include "sql/item_func.h"       // mqh_used
 #include "sql/log.h"
 #include "sql/mysqld.h"  // LOCK_user_conn
-#include "sql/net_ns.h"  // set_network_namespace
 #include "sql/protocol.h"
 #include "sql/protocol_classic.h"
 #include "sql/psi_memory_key.h"
@@ -184,7 +185,7 @@ end:
 int check_for_max_user_connections(THD *thd, const USER_CONN *uc) {
   int error = 0;
   Host_errors errors;
-  DBUG_ENTER("check_for_max_user_connections");
+  DBUG_TRACE;
 
   mysql_mutex_lock(&LOCK_user_conn);
   if (global_system_variables.max_user_connections &&
@@ -230,7 +231,7 @@ end:
   if (error) {
     inc_host_errors(thd->m_main_security_ctx.ip().str, &errors);
   }
-  DBUG_RETURN(error);
+  return error;
 }
 
 /*
@@ -252,7 +253,7 @@ end:
 */
 
 void decrease_user_connections(USER_CONN *uc) {
-  DBUG_ENTER("decrease_user_connections");
+  DBUG_TRACE;
   mysql_mutex_lock(&LOCK_user_conn);
   DBUG_ASSERT(uc->connections);
   if (!--uc->connections && !mqh_used) {
@@ -260,7 +261,6 @@ void decrease_user_connections(USER_CONN *uc) {
     hash_user_connections->erase(std::string(uc->user, uc->len));
   }
   mysql_mutex_unlock(&LOCK_user_conn);
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -273,7 +273,7 @@ void decrease_user_connections(USER_CONN *uc) {
  */
 void release_user_connection(THD *thd) {
   const USER_CONN *uc = thd->get_user_connect();
-  DBUG_ENTER("release_user_connection");
+  DBUG_TRACE;
 
   if (uc) {
     mysql_mutex_lock(&LOCK_user_conn);
@@ -286,8 +286,6 @@ void release_user_connection(THD *thd) {
     mysql_mutex_unlock(&LOCK_user_conn);
     thd->set_user_connect(NULL);
   }
-
-  DBUG_VOID_RETURN;
 }
 
 /*
@@ -298,7 +296,7 @@ void release_user_connection(THD *thd) {
 bool check_mqh(THD *thd, uint check_command) {
   bool error = 0;
   const USER_CONN *uc = thd->get_user_connect();
-  DBUG_ENTER("check_mqh");
+  DBUG_TRACE;
   DBUG_ASSERT(uc != 0);
 
   mysql_mutex_lock(&LOCK_user_conn);
@@ -330,7 +328,7 @@ bool check_mqh(THD *thd, uint check_command) {
   }
 end:
   mysql_mutex_unlock(&LOCK_user_conn);
-  DBUG_RETURN(error);
+  return error;
 }
 
 void init_max_user_conn(void) {
@@ -346,6 +344,7 @@ void free_max_user_conn(void) {
 
 void reset_mqh(THD *thd, LEX_USER *lu, bool get_them = 0) {
   mysql_mutex_lock(&LOCK_user_conn);
+  DEBUG_SYNC(thd, "in_reset_mqh_flush_privileges");
   if (lu)  // for GRANT
   {
     size_t temp_len = lu->user.length + lu->host.length + 2;
@@ -447,9 +446,11 @@ static int check_connection(THD *thd) {
   uint connect_errors = 0;
   int auth_rc;
   NET *net = thd->get_protocol_classic()->get_net();
+#ifndef DBUG_OFF
   char desc[VIO_DESCRIPTION_SIZE];
   vio_description(net->vio, desc);
   DBUG_PRINT("info", ("New connection received on %s", desc));
+#endif  // DBUG_OFF
 
   thd->set_active_vio(net->vio);
 
@@ -566,10 +567,19 @@ static int check_connection(THD *thd) {
       }
 #endif
       thd->m_main_security_ctx.assign_host(host, host ? strlen(host) : 0);
+      DBUG_EXECUTE_IF("vio_peer_addr_fake_hostname1", {
+        thd->m_main_security_ctx.assign_host(
+            "host_"
+            "1234567890abcdefghij1234567890abcdefghij1234567890abcdefghij123456"
+            "7890abcdefghij1234567890abcdefghij1234567890abcdefghij1234567890ab"
+            "cdefghij1234567890abcdefghij1234567890abcdefghij1234567890abcdefgh"
+            "ij1234567890abcdefghij1234567890abcdefghij1234567890",
+            255);
+      });
+
       main_sctx_host = thd->m_main_security_ctx.host();
       if (host && host != my_localhost) {
         my_free(host);
-        host = (char *)main_sctx_host.str;
       }
 
       /* Cut very long hostnames to avoid possible overflows */
@@ -683,7 +693,7 @@ static int check_connection(THD *thd) {
 
 static bool login_connection(THD *thd) {
   int error;
-  DBUG_ENTER("login_connection");
+  DBUG_TRACE;
   DBUG_PRINT("info",
              ("login_connection called by thread %u", thd->thread_id()));
 
@@ -699,14 +709,14 @@ static bool login_connection(THD *thd) {
     if (vio_type(thd->get_protocol_classic()->get_vio()) == VIO_TYPE_NAMEDPIPE)
       my_sleep(1000); /* must wait after eof() */
 #endif
-    DBUG_RETURN(1);
+    return 1;
   }
   /* Connect completed, set read/write timeouts back to default */
   thd->get_protocol_classic()->set_read_timeout(
       thd->variables.net_read_timeout);
   thd->get_protocol_classic()->set_write_timeout(
       thd->variables.net_write_timeout);
-  DBUG_RETURN(0);
+  return 0;
 }
 
 /*
@@ -761,8 +771,33 @@ static void prepare_new_connection_state(THD *thd) {
   NET *net = thd->get_protocol_classic()->get_net();
   Security_context *sctx = thd->security_context();
 
-  if (thd->get_protocol()->has_client_capability(CLIENT_COMPRESS))
+  if (thd->get_protocol()->has_client_capability(CLIENT_COMPRESS) ||
+      thd->get_protocol()->has_client_capability(
+          CLIENT_ZSTD_COMPRESSION_ALGORITHM)) {
     net->compress = 1;  // Use compression
+    enum enum_compression_algorithm algorithm = get_compression_algorithm(
+        thd->get_protocol()->get_compression_algorithm());
+    NET_SERVER *server_extn = static_cast<NET_SERVER *>(net->extension);
+    if (server_extn != nullptr)
+      mysql_compress_context_init(&server_extn->compress_ctx, algorithm,
+                                  thd->get_protocol()->get_compression_level());
+    if (net->extension == nullptr) {
+      LEX_CSTRING sctx_user = sctx->user();
+      Host_errors errors;
+      my_error(ER_NEW_ABORTING_CONNECTION, MYF(0), thd->thread_id(),
+               thd->db().str ? thd->db().str : "unconnected",
+               sctx_user.str ? sctx_user.str : "unauthenticated",
+               sctx->host_or_ip().str,
+               "Unable to allocate memory for compression context: Aborting "
+               "connection.");
+      thd->server_status &= ~SERVER_STATUS_CLEAR_SET;
+      thd->send_statement_status();
+      thd->killed = THD::KILL_CONNECTION;
+      errors.m_init_connect = 1;
+      inc_host_errors(thd->m_main_security_ctx.ip().str, &errors);
+      return;
+    }
+  }
 
   // Initializing session system variables.
   alloc_and_copy_thd_dynamic_variables(thd, true);
@@ -825,6 +860,9 @@ static void prepare_new_connection_state(THD *thd) {
       thd->killed = THD::KILL_CONNECTION;
       errors.m_init_connect = 1;
       inc_host_errors(thd->m_main_security_ctx.ip().str, &errors);
+      NET_SERVER *server_extn = static_cast<NET_SERVER *>(net->extension);
+      if (server_extn != nullptr)
+        mysql_compress_context_deinit(&server_extn->compress_ctx);
       return;
     }
 
@@ -858,9 +896,9 @@ bool thd_prepare_connection(THD *thd) {
 
 void close_connection(THD *thd, uint sql_errno, bool server_shutdown,
                       bool generate_event) {
-  DBUG_ENTER("close_connection");
+  DBUG_TRACE;
 
-  if (sql_errno) net_send_error(thd, sql_errno, ER_DEFAULT(sql_errno));
+  if (sql_errno) net_send_error(thd, sql_errno, ER_DEFAULT_NONCONST(sql_errno));
   thd->disconnect(server_shutdown);
 
   if (generate_event) {
@@ -872,7 +910,6 @@ void close_connection(THD *thd, uint sql_errno, bool server_shutdown,
   }
 
   thd->security_context()->logout();
-  DBUG_VOID_RETURN;
 }
 
 bool thd_connection_alive(THD *thd) {
