@@ -56,7 +56,6 @@
 #else
 #define MAX_REFLENGTH 4 /* Max length for record ref */
 #endif
-#define MAX_HOSTNAME 61 /* len+1 in mysql.user */
 
 #define MAX_MBWIDTH 3 /* Max multibyte sequence */
 #define MAX_FIELD_CHARLENGTH 255
@@ -357,7 +356,8 @@ static const ulong EVENT_DEF_CACHE_MIN = 256;
 #define OPTIMIZER_SWITCH_DERIVED_MERGE (1ULL << 18)
 #define OPTIMIZER_SWITCH_USE_INVISIBLE_INDEXES (1ULL << 19)
 #define OPTIMIZER_SKIP_SCAN (1ULL << 20)
-#define OPTIMIZER_SWITCH_LAST (1ULL << 21)
+#define OPTIMIZER_SWITCH_HASH_JOIN (1ULL << 21)
+#define OPTIMIZER_SWITCH_LAST (1ULL << 22)
 
 #define OPTIMIZER_SWITCH_DEFAULT                                          \
   (OPTIMIZER_SWITCH_INDEX_MERGE | OPTIMIZER_SWITCH_INDEX_MERGE_UNION |    \
@@ -371,7 +371,7 @@ static const ulong EVENT_DEF_CACHE_MIN = 256;
    OPTIMIZER_SWITCH_DUPSWEEDOUT | OPTIMIZER_SWITCH_SUBQ_MAT_COST_BASED |  \
    OPTIMIZER_SWITCH_USE_INDEX_EXTENSIONS |                                \
    OPTIMIZER_SWITCH_COND_FANOUT_FILTER | OPTIMIZER_SWITCH_DERIVED_MERGE | \
-   OPTIMIZER_SKIP_SCAN)
+   OPTIMIZER_SKIP_SCAN | OPTIMIZER_SWITCH_HASH_JOIN)
 
 enum SHOW_COMP_OPTION { SHOW_OPTION_YES, SHOW_OPTION_NO, SHOW_OPTION_DISABLED };
 
