@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -85,7 +84,6 @@
 #include "sql/mysqld.h"          // key_file_misc
 #include "sql/psi_memory_key.h"  // key_memory_THD_db
 #include "sql/rpl_gtid.h"
-#include "sql/rpl_replica_commit_order_manager.h"  // Commit_order_manager
 #include "sql/session_tracker.h"
 #include "sql/sp.h"         // lock_db_routines
 #include "sql/sql_base.h"   // lock_table_names
@@ -95,7 +93,7 @@
 #include "sql/sql_handler.h"  // mysql_ha_rm_tables
 #include "sql/sql_table.h"    // build_table_filename
 #include "sql/system_variables.h"
-#include "sql/table.h"  // Table_ref
+#include "sql/table.h"  // TABLE_LIST
 #include "sql/thd_raii.h"
 #include "sql/transaction.h"  // trans_rollback_stmt
 #include "sql_string.h"
@@ -108,13 +106,13 @@
 const char *del_exts[] = {".frm", ".BAK", ".TMD", ".opt",
                           ".OLD", ".cfg", ".SDI", NullS};
 static TYPELIB deletable_extentions = {array_elements(del_exts) - 1, "del_exts",
-                                       del_exts, nullptr};
+                                       del_exts, NULL};
 
 static bool find_unknown_and_remove_deletable_files(THD *thd, MY_DIR *dirp,
                                                     const char *path);
 
 static bool find_db_tables(THD *thd, const dd::Schema &schema, const char *db,
-                           Table_ref **tables);
+                           TABLE_LIST **tables);
 
 static long mysql_rm_arc_files(THD *thd, MY_DIR *dirp, const char *org_path);
 static bool rm_dir_w_symlink(const char *org_path, bool send_error);
@@ -150,7 +148,7 @@ bool get_default_db_collation(THD *thd, const char *db_name,
   // We must make sure the schema is released and unlocked in the right order.
   dd::Schema_MDL_locker mdl_handler(thd);
   dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
-  const dd::Schema *sch_obj = nullptr;
+  const dd::Schema *sch_obj = NULL;
 
   if (mdl_handler.ensure_locked(db_name) ||
       thd->dd_client()->acquire(db_name, &sch_obj))
@@ -160,100 +158,6 @@ bool get_default_db_collation(THD *thd, const char *db_name,
   return false;
 }
 
-/**
-  Check if the thread type is allowed to ignore the schema read
-  only option.
-
-  @param thd    Thread context.
-
-  @return       true if the thread is allowed to ignore read only,
-                otherwise false.
-*/
-static bool thread_can_ignore_schema_read_only(THD *thd) {
-  /*
-    We ignore read_only for server side execution during initialize,
-    restart and upgrade. We also ignore it for init-file execution.
-
-    In the same way as for check_readonly, if the thread is a replication
-    slave or if skip_read_only check is enabled for the command, we do not
-    enforce the read only check.
-  */
-  return (thd->is_bootstrap_system_thread() ||
-          thd->is_server_upgrade_thread() || thd->slave_thread ||
-          thd->is_cmd_skip_readonly());
-}
-
-/**
-  Check the read_only option for the given schema, and report error if
-  the schema is not writable.
-
-  @param thd          Thread context.
-  @param schema_name  Name of schema to check.
-  @param share        For tables, we cache the read only option in the
-                      table share, and can therefore get the read only
-                      option from the share.
-
-  Caching the read only state in the table share is done for performance
-  reasons. If a share is submitted, we get the read only state from the
-  share. Otherwise, we get the schema object from the DD cache in order
-  to see the read only state.
-
-  @return false if the schema is writable, true if not. If returning
-          true, then error is already reported.
-*/
-bool check_schema_readonly(THD *thd, const char *schema_name,
-                           TABLE_SHARE *share) {
-  // We ignore read_only for certain thread types.
-  if (thread_can_ignore_schema_read_only(thd)) return false;
-
-  /*
-    We must ignore read_only for ALTER SCHEMA to allow turning off
-    read_only. There is a special check for read_only handling within
-    mysql_alter_schema.
-
-    We also ignore read_only for CREATE SCHEMA to make sure we keep
-    the existing error handling in case the schema exists already.
-  */
-  if (thd->lex->sql_command == SQLCOM_ALTER_DB ||
-      thd->lex->sql_command == SQLCOM_CREATE_DB)
-    return false;
-
-  /*
-    If we submitted a table share, we can conclude without acquiring the
-    schema object from the DD cache.
-  */
-  if (share != nullptr) {
-    if (share->schema_read_only == TABLE_SHARE::Schema_read_only::RO_ON) {
-      my_error(ER_SCHEMA_READ_ONLY, MYF(0), schema_name);
-      return true;
-    }
-    if (share->schema_read_only == TABLE_SHARE::Schema_read_only::RO_OFF) {
-      return false;
-    }
-    assert(false);
-  }
-
-  dd::Schema_MDL_locker mdl_handler(thd);
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
-  const dd::Schema *sch_obj = nullptr;
-
-  if (mdl_handler.ensure_locked(schema_name) ||
-      thd->dd_client()->acquire(schema_name, &sch_obj)) {
-    assert(thd->is_error() || thd->killed);
-    return true;
-  }
-
-  // A non existing schema is considered not being in a read_only state.
-  if (sch_obj == nullptr) return false;
-
-  if (sch_obj->read_only()) {
-    my_error(ER_SCHEMA_READ_ONLY, MYF(0), schema_name);
-    return true;
-  }
-
-  return false;
-}
-
 /**
   Auxiliary function which writes CREATE/ALTER or DROP DATABASE statement
   to the binary log overriding connection's current database with one
@@ -264,7 +168,8 @@ static bool write_db_cmd_to_binlog(THD *thd, const char *db, bool trx_cache) {
   if (mysql_bin_log.is_open()) {
     int errcode = query_error_code(thd, true);
     Query_log_event qinfo(thd, thd->query().str, thd->query().length, trx_cache,
-                          false, /* suppress_use */ true, errcode);
+                          false,
+                          /* suppress_use */ true, errcode);
     /*
       Write should use the database being created/altered or dropped
       as the "current database" and not the threads current database,
@@ -371,7 +276,7 @@ bool mysql_create_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
   const char *lock_db_name = db;
   if (lower_case_table_names == 2) {
     my_stpcpy(name_buf, db);
-    my_casedn_str(&my_charset_utf8mb3_tolower_ci, name_buf);
+    my_casedn_str(&my_charset_utf8_tolower_ci, name_buf);
     lock_db_name = name_buf;
   }
   if (lock_schema_name(thd, lock_db_name)) return true;
@@ -414,7 +319,7 @@ bool mysql_create_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
   // already exists.
   MY_STAT stat_info;
   bool schema_dir_exists =
-      (mysql_file_stat(key_file_misc, path, &stat_info, MYF(0)) != nullptr);
+      (mysql_file_stat(key_file_misc, path, &stat_info, MYF(0)) != NULL);
   if (thd->is_dd_system_thread() &&
       (!opt_initialize || dd::upgrade_57::in_progress()) &&
       dd::get_dictionary()->is_dd_schema_name(db)) {
@@ -531,6 +436,8 @@ bool mysql_alter_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
 
   if (lock_schema_name(thd, db)) return true;
 
+  set_db_default_charset(thd, create_info);
+
   dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   dd::Schema *schema = nullptr;
   if (thd->dd_client()->acquire_for_modification(db, &schema)) return true;
@@ -540,114 +447,36 @@ bool mysql_alter_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
     return true;
   }
 
-  /*
-    If the schema is in read_only state, then the only change allowed is to:
-
-    - Turn off read_only, possibly along with other option changes.
-    - Keep read_only turned on, i.e., a no-op. In this case, other options may
-      not be changed in the same statement.
-
-     This means we fail if:
-
-     - HA_CREATE_USED_READ_ONLY is not set.
-     - Or if we set other fields as well and set READ ONLY to true.
-
-     We check this before locking tables to avoid unnecessary lock acquisition.
-  */
-  bool schema_read_only = false;
-  schema_read_only = schema->read_only();
-  if (!thread_can_ignore_schema_read_only(thd) && schema_read_only) {
-    if (!(create_info->used_fields & HA_CREATE_USED_READ_ONLY) ||
-        ((create_info->used_fields & ~HA_CREATE_USED_READ_ONLY) &&
-         create_info->schema_read_only)) {
-      my_error(ER_SCHEMA_READ_ONLY, MYF(0), db);
-      return true;
-    }
-  }
-
-  /*
-    Lock all tables while under schema lock so that we block all transactions
-    from touching these tables until the ALTER is done. Because the schema read
-    only check and the COMMIT are not done atomically, we use the table MDL
-    to serialize ALTER SCHEMA and any write statements.
-  */
-  Table_ref *tables = nullptr;
-  if (find_db_tables(thd, *schema, db, &tables) ||
-      lock_table_names(thd, tables, nullptr, thd->variables.lock_wait_timeout,
-                       0))
-    return true;
-
-  // Set new collation ID if submitted in the statement.
-  if (create_info->used_fields & HA_CREATE_USED_DEFAULT_CHARSET) {
-    set_db_default_charset(thd, create_info);
-    schema->set_default_collation_id(
-        create_info->default_table_charset->number);
-  }
+  // Set new collation ID.
+  schema->set_default_collation_id(create_info->default_table_charset->number);
 
   // Set encryption type.
-  if (create_info->used_fields & HA_CREATE_USED_DEFAULT_ENCRYPTION) {
-    assert(create_info->encrypt_type.length > 0);
+  if (create_info->encrypt_type.length > 0)
     schema->set_default_encryption(dd::is_encrypted(create_info->encrypt_type));
-  }
-
-  // Set read_only option.
-  if (create_info->used_fields & HA_CREATE_USED_READ_ONLY) {
-    schema->set_read_only(create_info->schema_read_only);
-  }
 
   // Update schema.
   if (thd->dd_client()->update(schema)) return true;
 
-  /*
-    The original query is submitted to the engine's log handler
-    because e.g. NDB needs to propagate the read only option to
-    other mysqld servers in the cluster.
-  */
-  ha_binlog_log_query(thd, nullptr, LOGCOM_ALTER_DB, thd->query().str,
+  ha_binlog_log_query(thd, 0, LOGCOM_ALTER_DB, thd->query().str,
                       thd->query().length, db, "");
 
-  /*
-    The original query is written to the binlog and hence replicated.
-    Binlogging must be switched off while executing ALTER SCHEMA if
-    this is not desired.
-  */
   if (write_db_cmd_to_binlog(thd, db, true)) return true;
 
   /*
     Commit the statement locally instead of relying on caller,
-    in order to be sure that it is successful, before changing
+    in order to be sure that it is  successfull, before changing
     options of current database.
   */
   if (trans_commit_stmt(thd) || trans_commit(thd)) return true;
 
-  /*
-    Change collation options if the current database is being
-    altered and the clause is explicitly submitted in the ALTER
-    statement.
-  */
-  if (create_info->used_fields & HA_CREATE_USED_DEFAULT_CHARSET &&
-      thd->db().str && !my_strcasecmp(table_alias_charset, thd->db().str, db)) {
+  /* Change options if current database is being altered. */
+  if (thd->db().str && !strcmp(thd->db().str, db)) {
     thd->db_charset = create_info->default_table_charset
                           ? create_info->default_table_charset
                           : thd->variables.collation_server;
     thd->variables.collation_database = thd->db_charset;
   }
 
-  /*
-    If the schema read only option was changed, then we must also evict the
-    tables belonging to the schema from the TDC since the option is cached
-    in the table shares. This is safe since we have already acquired
-    exclusive MDL for each table.
-  */
-  if (create_info->used_fields & HA_CREATE_USED_READ_ONLY) {
-    mysql_ha_flush_tables(thd, tables);
-    for (Table_ref *table = tables; table != nullptr;
-         table = table->next_global) {
-      tdc_remove_table(thd, TDC_RT_REMOVE_ALL, table->db, table->table_name,
-                       false);
-    }
-  }
-
   my_ok(thd, 1);
   return false;
 }
@@ -661,9 +490,9 @@ class Rmdir_error_handler : public Internal_error_handler {
  public:
   Rmdir_error_handler() : m_is_active(false) {}
 
-  bool handle_condition(THD *thd, uint, const char *,
-                        Sql_condition::enum_severity_level *,
-                        const char *msg) override {
+  virtual bool handle_condition(THD *thd, uint, const char *,
+                                Sql_condition::enum_severity_level *,
+                                const char *msg) {
     if (!m_is_active) {
       /* Disable the handler to avoid infinite recursion. */
       m_is_active = true;
@@ -706,8 +535,8 @@ bool mysql_rm_db(THD *thd, const LEX_CSTRING &db, bool if_exists) {
   ulong deleted_tables = 0;
   bool error = false;
   char path[2 * FN_REFLEN + 16];
-  Table_ref *tables = nullptr;
-  Table_ref *table;
+  TABLE_LIST *tables = NULL;
+  TABLE_LIST *table;
   Drop_table_error_handler err_handler;
   bool dropped_non_atomic = false;
   std::set<handlerton *> post_ddl_htons;
@@ -787,7 +616,7 @@ bool mysql_rm_db(THD *thd, const LEX_CSTRING &db, bool if_exists) {
     }
 
     /* Lock all tables and stored routines about to be dropped. */
-    if (lock_table_names(thd, tables, nullptr, thd->variables.lock_wait_timeout,
+    if (lock_table_names(thd, tables, NULL, thd->variables.lock_wait_timeout,
                          0) ||
         rm_table_do_discovery_and_lock_fk_tables(thd, tables) ||
         lock_check_constraint_names(thd, tables) ||
@@ -795,7 +624,7 @@ bool mysql_rm_db(THD *thd, const LEX_CSTRING &db, bool if_exists) {
         lock_db_routines(thd, *schema) || lock_trigger_names(thd, tables))
       return true;
 
-    /* mysql_ha_rm_tables() requires a non-null Table_ref. */
+    /* mysql_ha_rm_tables() requires a non-null TABLE_LIST. */
     if (tables) mysql_ha_rm_tables(thd, tables);
 
     for (table = tables; table; table = table->next_local) {
@@ -839,7 +668,7 @@ bool mysql_rm_db(THD *thd, const LEX_CSTRING &db, bool if_exists) {
       thd->clear_error(); /* @todo Do not ignore errors */
       Disable_binlog_guard binlog_guard(thd);
       error = Events::drop_schema_events(thd, *schema);
-      error = (error || sp_drop_db_routines(thd, *schema));
+      error = (error || (sp_drop_db_routines(thd, *schema) != SP_OK));
     }
     thd->pop_internal_handler();
 
@@ -933,8 +762,10 @@ bool mysql_rm_db(THD *thd, const LEX_CSTRING &db, bool if_exists) {
     */
     if (thd->session_tracker.get_tracker(CURRENT_SCHEMA_TRACKER)
             ->is_enabled()) {
+      LEX_CSTRING dummy = {STRING_WITH_LEN("")};
+      dummy.length = dummy.length * 1;
       thd->session_tracker.get_tracker(CURRENT_SCHEMA_TRACKER)
-          ->mark_as_changed(thd, {});
+          ->mark_as_changed(thd, &dummy);
     }
   }
 
@@ -964,7 +795,7 @@ static bool find_unknown_and_remove_deletable_files(THD *thd, MY_DIR *dirp,
     char *extension;
     DBUG_PRINT("info", ("Examining: %s", file->name));
 
-    /* skipping . and .. */
+    /* skiping . and .. */
     if (file->name[0] == '.' &&
         (!file->name[1] || (file->name[1] == '.' && !file->name[2])))
       continue;
@@ -1020,8 +851,8 @@ found_other_files:
 */
 
 static bool find_db_tables(THD *thd, const dd::Schema &schema, const char *db,
-                           Table_ref **tables) {
-  Table_ref *tot_list = nullptr, **tot_list_next_local, **tot_list_next_global;
+                           TABLE_LIST **tables) {
+  TABLE_LIST *tot_list = 0, **tot_list_next_local, **tot_list_next_global;
   DBUG_TRACE;
 
   tot_list_next_local = tot_list_next_global = &tot_list;
@@ -1037,8 +868,8 @@ static bool find_db_tables(THD *thd, const dd::Schema &schema, const char *db,
                                                                   &sch_tables))
     return true;
 
-  for (const dd::String_type &table_name : sch_tables) {
-    Table_ref *table_list = new (thd->mem_root) Table_ref;
+  for (const dd::String_type table_name : sch_tables) {
+    TABLE_LIST *table_list = new (thd->mem_root) TABLE_LIST;
     if (table_list == nullptr) return true; /* purecov: inspected */
 
     table_list->db = thd->mem_strdup(db);
@@ -1094,7 +925,7 @@ static bool rm_dir_w_symlink(const char *org_path, bool send_error) {
   pos = strend(path);
   if (pos > path && pos[-1] == FN_LIBCHAR) *--pos = 0;
 
-  if ((error = my_readlink(tmp2_path, path, MYF(MY_WME))) < 0) return true;
+  if ((error = my_readlink(tmp2_path, path, MYF(MY_WME))) < 0) return 1;
   if (!error) {
     if (mysql_file_delete(key_file_misc, path, MYF(send_error ? MY_WME : 0))) {
       return send_error;
@@ -1111,9 +942,9 @@ static bool rm_dir_w_symlink(const char *org_path, bool send_error) {
     char errbuf[MYSQL_ERRMSG_SIZE];
     my_error(ER_DB_DROP_RMDIR, MYF(0), path, errno,
              my_strerror(errbuf, MYSQL_ERRMSG_SIZE, errno));
-    return true;
+    return 1;
   }
-  return false;
+  return 0;
 }
 
 /*
@@ -1145,7 +976,7 @@ long mysql_rm_arc_files(THD *thd, MY_DIR *dirp, const char *org_path) {
     char *extension, *revision;
     DBUG_PRINT("info", ("Examining: %s", file->name));
 
-    /* skipping . and .. */
+    /* skiping . and .. */
     if (file->name[0] == '.' &&
         (!file->name[1] || (file->name[1] == '.' && !file->name[2])))
       continue;
@@ -1176,7 +1007,7 @@ long mysql_rm_arc_files(THD *thd, MY_DIR *dirp, const char *org_path) {
     If the directory is a symbolic link, remove the link first, then
     remove the directory the symbolic link pointed at
   */
-  if (!found_other_files && rm_dir_w_symlink(org_path, false)) return -1;
+  if (!found_other_files && rm_dir_w_symlink(org_path, 0)) return -1;
   return deleted;
 
 err:
@@ -1201,7 +1032,7 @@ static void mysql_change_db_impl(THD *thd, const LEX_CSTRING &new_db_name,
                                  const CHARSET_INFO *new_db_charset) {
   /* 1. Change current database in THD. */
 
-  if (new_db_name.str == nullptr) {
+  if (new_db_name.str == NULL) {
     /*
       THD::set_db() does all the job -- it frees previous database name and
       sets the new one.
@@ -1258,7 +1089,7 @@ static void backup_current_db_name(THD *thd, LEX_STRING *saved_db_name) {
   if (!thd->db().str) {
     /* No current (default) database selected. */
 
-    saved_db_name->str = nullptr;
+    saved_db_name->str = NULL;
     saved_db_name->length = 0;
   } else {
     strmake(saved_db_name->str, thd->db().str, saved_db_name->length - 1);
@@ -1356,7 +1187,7 @@ bool mysql_change_db(THD *thd, const LEX_CSTRING &new_db_name,
 
   Security_context *sctx = thd->security_context();
   ulong db_access = sctx->current_db_access();
-  const CHARSET_INFO *db_default_cl = nullptr;
+  const CHARSET_INFO *db_default_cl = NULL;
 
   // We must make sure the schema is released and unlocked in the right order.
   dd::Schema_MDL_locker mdl_handler(thd);
@@ -1366,7 +1197,7 @@ bool mysql_change_db(THD *thd, const LEX_CSTRING &new_db_name,
   DBUG_TRACE;
   DBUG_PRINT("enter", ("name: '%s'", new_db_name.str));
 
-  if (new_db_name.str == nullptr || new_db_name.length == 0) {
+  if (new_db_name.str == NULL || new_db_name.length == 0) {
     if (force_switch) {
       /*
         This can happen only if we're switching the current database back
@@ -1407,7 +1238,7 @@ bool mysql_change_db(THD *thd, const LEX_CSTRING &new_db_name,
                                     new_db_name.length, MYF(MY_WME));
   new_db_file_name.length = new_db_name.length;
 
-  if (new_db_file_name.str == nullptr) return true; /* the error is set */
+  if (new_db_file_name.str == NULL) return true; /* the error is set */
 
   /*
     NOTE: if check_db_name() fails, we should throw an error in any case,
@@ -1443,7 +1274,7 @@ bool mysql_change_db(THD *thd, const LEX_CSTRING &new_db_name,
   }
 
   if (!force_switch && !(db_access & DB_OP_ACLS) &&
-      check_grant_db(thd, new_db_file_name.str, true)) {
+      check_grant_db(thd, new_db_file_name.str)) {
     my_error(ER_DBACCESS_DENIED_ERROR, MYF(0), sctx->priv_user().str,
              sctx->priv_host().str, new_db_file_name.str);
     query_logger.general_log_print(
@@ -1489,7 +1320,7 @@ bool mysql_change_db(THD *thd, const LEX_CSTRING &new_db_name,
 
   if (get_default_db_collation(*schema, &db_default_cl)) {
     my_free(new_db_file_name.str);
-    assert(thd->is_error() || thd->killed);
+    DBUG_ASSERT(thd->is_error() || thd->killed);
     return true;
   }
 
@@ -1505,13 +1336,15 @@ done:
     Check if current database tracker is enabled. If so, set the 'changed' flag.
   */
   if (thd->session_tracker.get_tracker(CURRENT_SCHEMA_TRACKER)->is_enabled()) {
+    LEX_CSTRING dummy = {STRING_WITH_LEN("")};
+    dummy.length = dummy.length * 1;
     thd->session_tracker.get_tracker(CURRENT_SCHEMA_TRACKER)
-        ->mark_as_changed(thd, {});
+        ->mark_as_changed(thd, &dummy);
   }
   if (thd->session_tracker.get_tracker(SESSION_STATE_CHANGE_TRACKER)
           ->is_enabled())
     thd->session_tracker.get_tracker(SESSION_STATE_CHANGE_TRACKER)
-        ->mark_as_changed(thd, {});
+        ->mark_as_changed(thd, NULL);
   return false;
 }
 
@@ -1529,11 +1362,10 @@ done:
                                   and "length" is updated accordingly.
                                   Otherwise "str" is set to NULL and
                                   "length" is set to 0.
-  @param          force_switch    if the change of the current database shall be
-  forced @see mysql_change_db()
+  @param          force_switch    @see mysql_change_db()
   @param[out]     cur_db_changed  out-flag to indicate whether the current
                                   database has been changed (valid only if
-                                  the function succeeded)
+                                  the function suceeded)
 */
 
 bool mysql_opt_change_db(THD *thd, const LEX_CSTRING &new_db_name,
