@@ -53,7 +53,7 @@
 #include "sql/opt_range.h"  // prune_partitions
 #include "sql/opt_trace.h"  // Opt_trace_object
 #include "sql/query_options.h"
-#include "sql/records.h"  // READ_RECORD
+#include "sql/records.h"  // unique_ptr_destroy_only<RowIterator>
 #include "sql/row_iterator.h"
 #include "sql/sorting_iterator.h"
 #include "sql/sql_base.h"  // update_non_unique_table_error
@@ -70,6 +70,7 @@
 #include "sql/system_variables.h"
 #include "sql/table.h"
 #include "sql/table_trigger_dispatcher.h"  // Table_trigger_dispatcher
+#include "sql/timing_iterator.h"
 #include "sql/transaction_info.h"
 #include "sql/trigger_def.h"
 #include "sql/uniques.h"  // Unique
@@ -80,18 +81,18 @@ class Opt_trace_context;
 class Select_lex_visitor;
 
 bool Sql_cmd_delete::precheck(THD *thd) {
-  DBUG_ENTER("Sql_cmd_delete::precheck");
+  DBUG_TRACE;
 
   TABLE_LIST *tables = lex->query_tables;
 
   if (!multitable) {
-    if (check_one_table_access(thd, DELETE_ACL, tables)) DBUG_RETURN(true);
+    if (check_one_table_access(thd, DELETE_ACL, tables)) return true;
   } else {
     TABLE_LIST *aux_tables = delete_tables->first;
     TABLE_LIST **save_query_tables_own_last = lex->query_tables_own_last;
 
     if (check_table_access(thd, SELECT_ACL, tables, false, UINT_MAX, false))
-      DBUG_RETURN(true);
+      return true;
 
     /*
       Since aux_tables list is not part of LEX::query_tables list we
@@ -102,11 +103,11 @@ bool Sql_cmd_delete::precheck(THD *thd) {
     if (check_table_access(thd, DELETE_ACL, aux_tables, false, UINT_MAX,
                            false)) {
       lex->query_tables_own_last = save_query_tables_own_last;
-      DBUG_RETURN(true);
+      return true;
     }
     lex->query_tables_own_last = save_query_tables_own_last;
   }
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -122,7 +123,7 @@ bool Sql_cmd_delete::precheck(THD *thd) {
 */
 
 bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
-  DBUG_ENTER("Sql_cmd_delete::delete_from_single_table");
+  DBUG_TRACE;
 
   myf error_flags = MYF(0); /**< Flag for fatal errors */
   bool will_batch;
@@ -186,13 +187,13 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
                              "No matching rows after partition pruning", true,
                              0);
       bool err = explain_single_table_modification(thd, thd, &plan, select_lex);
-      DBUG_RETURN(err);
+      return err;
     }
   }
 
   Item *conds = nullptr;
   if (!no_rows && select_lex->get_optimizable_conditions(thd, &conds, nullptr))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
 
   /*
     Reset the field list to remove any hidden fields added by substitute_gc() in
@@ -214,7 +215,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
   const bool const_cond = conds == nullptr || conds->const_item();
   const bool const_cond_result = const_cond && (!conds || conds->val_int());
   if (thd->is_error())  // Error during val_int()
-    DBUG_RETURN(true);  /* purecov: inspected */
+    return true;        /* purecov: inspected */
   /*
     We are passing HA_EXTRA_IGNORE_DUP_KEY flag here to recreate query with
     IGNORE keyword within federated storage engine. If federated engine is
@@ -253,14 +254,14 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
                            maybe_deleted);
     if (lex->is_explain()) {
       bool err = explain_single_table_modification(thd, thd, &plan, select_lex);
-      DBUG_RETURN(err);
+      return err;
     }
 
     /* Do not allow deletion of all records if safe_update is set. */
     if (safe_update) {
       my_error(ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE, MYF(0),
                thd->get_stmt_da()->get_first_condition_message());
-      DBUG_RETURN(true);
+      return true;
     }
 
     DBUG_PRINT("debug", ("Trying to use delete_all_rows()"));
@@ -287,7 +288,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
     Item::cond_result result;
 
     if (optimize_cond(thd, &conds, &cond_equal, select_lex->join_list, &result))
-      DBUG_RETURN(true);
+      return true;
     if (result == Item::COND_FALSE)  // Impossible where
     {
       no_rows = true;
@@ -297,12 +298,12 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
                                0);
         bool err =
             explain_single_table_modification(thd, thd, &plan, select_lex);
-        DBUG_RETURN(err);
+        return err;
       }
     }
     if (conds) {
       conds = substitute_for_best_equal_field(thd, conds, cond_equal, 0);
-      if (conds == NULL) DBUG_RETURN(true);
+      if (conds == NULL) return true;
 
       conds->update_used_tables();
     }
@@ -321,7 +322,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
   table->covering_keys.clear_all();
 
   /* Prune a second time to be able to prune on subqueries in WHERE clause. */
-  if (prune_partitions(thd, table, conds)) DBUG_RETURN(true);
+  if (prune_partitions(thd, table, conds)) return true;
   if (table->all_partitions_pruned_away) {
     /* No matching records */
     if (lex->is_explain()) {
@@ -329,10 +330,10 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
                              "No matching rows after partition pruning", true,
                              0);
       bool err = explain_single_table_modification(thd, thd, &plan, select_lex);
-      DBUG_RETURN(err);
+      return err;
     }
     my_ok(thd, 0);
-    DBUG_RETURN(false);
+    return false;
   }
 
   qep_tab.set_table(table);
@@ -357,7 +358,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
       qep_tab.set_quick(qck);
     }
     if (thd->is_error())  // test_quick_select() has improper error propagation
-      DBUG_RETURN(true);
+      return true;
 
     if (no_rows) {
       if (lex->is_explain()) {
@@ -365,11 +366,11 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
                                0);
         bool err =
             explain_single_table_modification(thd, thd, &plan, select_lex);
-        DBUG_RETURN(err);
+        return err;
       }
 
       my_ok(thd, 0);
-      DBUG_RETURN(false);  // Nothing to delete
+      return false;  // Nothing to delete
     }
   }  // Ends scope for optimizer trace wrapper
 
@@ -388,12 +389,12 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
     if (!thd->lex->is_explain() && safe_update && !using_limit) {
       my_error(ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE, MYF(0),
                thd->get_stmt_da()->get_first_condition_message());
-      DBUG_RETURN(true);
+      return true;
     }
   }
 
   if (order) {
-    if (table->update_const_key_parts(conds)) DBUG_RETURN(true);
+    if (table->update_const_key_parts(conds)) return true;
     order = simple_remove_const(order, conds);
     ORDER_with_src order_src(order, ESC_ORDER_BY);
     usable_index =
@@ -421,38 +422,42 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
 
     if (lex->is_explain()) {
       bool err = explain_single_table_modification(thd, thd, &plan, select_lex);
-      DBUG_RETURN(err);
+      return err;
     }
 
     if (select_lex->active_options() & OPTION_QUICK)
       (void)table->file->ha_extra(HA_EXTRA_QUICK);
 
     unique_ptr_destroy_only<Filesort> fsort;
-    READ_RECORD info;
+    unique_ptr_destroy_only<RowIterator> iterator;
     ha_rows examined_rows = 0;
     if (usable_index == MAX_KEY || qep_tab.quick())
-      setup_read_record(&info, thd, NULL, &qep_tab, false,
-                        /*ignore_not_found_rows=*/false, &examined_rows);
+      iterator =
+          create_table_iterator(thd, NULL, &qep_tab, false,
+                                /*ignore_not_found_rows=*/false, &examined_rows,
+                                /*using_table_scan=*/nullptr);
     else
-      setup_read_record_idx(&info, thd, table, usable_index, reverse, &qep_tab);
+      iterator = create_table_iterator_idx(thd, table, usable_index, reverse,
+                                           &qep_tab);
 
     if (need_sort) {
       DBUG_ASSERT(usable_index == MAX_KEY);
 
-      unique_ptr_destroy_only<RowIterator> iterator = move(info.iterator);
-
       if (qep_tab.condition() != nullptr) {
-        iterator.reset(new (&info.sort_condition_holder) FilterIterator(
-            thd, move(iterator), qep_tab.condition()));
+        iterator = NewIterator<FilterIterator>(thd, move(iterator),
+                                               qep_tab.condition());
       }
 
-      fsort.reset(new (thd->mem_root) Filesort(&qep_tab, order, HA_POS_ERROR));
-      unique_ptr_destroy_only<RowIterator> sort(new (
-          &info.sort_holder) SortingIterator(thd, fsort.get(), move(iterator),
-                                             /*force_sort_position=*/true,
-                                             /*rows_examined=*/nullptr));
-      if (sort->Init()) DBUG_RETURN(true);
-      info.iterator = move(sort);
+      fsort.reset(new (thd->mem_root)
+                      Filesort(thd, &qep_tab, order, HA_POS_ERROR,
+                               /*force_stable_sort=*/false,
+                               /*remove_duplicates=*/false,
+                               /*force_sort_positions=*/true));
+      unique_ptr_destroy_only<RowIterator> sort =
+          NewIterator<SortingIterator>(thd, fsort.get(), move(iterator),
+                                       /*rows_examined=*/nullptr);
+      if (sort->Init()) return true;
+      iterator = move(sort);
       thd->inc_examined_row_count(examined_rows);
 
       /*
@@ -461,11 +466,11 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
       */
       qep_tab.set_condition(NULL);
     } else {
-      if (info.iterator->Init()) DBUG_RETURN(true);
+      if (iterator->Init()) return true;
     }
 
     if (select_lex->has_ft_funcs() && init_ftfuncs(thd, select_lex))
-      DBUG_RETURN(true); /* purecov: inspected */
+      return true; /* purecov: inspected */
 
     THD_STAGE_INFO(thd, stage_updating);
 
@@ -482,7 +487,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
       will_batch = !table->file->start_bulk_delete();
     }
     table->mark_columns_needed_for_delete(thd);
-    if (thd->is_error()) DBUG_RETURN(true);
+    if (thd->is_error()) return true;
 
     if ((table->file->ha_table_flags() & HA_READ_BEFORE_WRITE_REMOVAL) &&
         !using_limit && !has_delete_triggers && qep_tab.quick() &&
@@ -493,7 +498,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
 
     // The loop that reads rows and delete those that qualify
 
-    while (!(error = info->Read()) && !thd->killed) {
+    while (!(error = iterator->Read()) && !thd->killed) {
       DBUG_ASSERT(!thd->is_error());
       thd->inc_examined_row_count(1);
 
@@ -609,7 +614,7 @@ cleanup:
     my_ok(thd, deleted_rows);
     DBUG_PRINT("info", ("%ld records deleted", (long)deleted_rows));
   }
-  DBUG_RETURN(error > 0);
+  return error > 0;
 }
 
 /**
@@ -617,7 +622,7 @@ cleanup:
 */
 
 bool Sql_cmd_delete::prepare_inner(THD *thd) {
-  DBUG_ENTER("Sql_cmd_delete::prepare_inner");
+  DBUG_TRACE;
 
   Prepare_error_tracker tracker(thd);
 
@@ -640,9 +645,13 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
 
     Prepared_stmt_arena_holder ps_holder(thd);
     result = new (thd->mem_root) Query_result_delete();
-    if (result == NULL) DBUG_RETURN(true); /* purecov: inspected */
+    if (result == NULL) return true; /* purecov: inspected */
 
+    // The former is for the pre-iterator executor; the latter is for the
+    // iterator executor.
+    // TODO(sgunders): Get rid of this when we remove Query_result.
     select->set_query_result(result);
+    select->master_unit()->set_query_result(result);
 
     select->make_active_options(SELECT_NO_JOIN_CACHE | SELECT_NO_UNLOCK,
                                 OPTION_BUFFER_RESULT);
@@ -655,7 +664,7 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
   }
 
   if (select->setup_tables(thd, table_list, false))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
 
   ulong want_privilege_saved = thd->want_privilege;
   thd->want_privilege = SELECT_ACL;
@@ -663,11 +672,9 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
   thd->mark_used_columns = MARK_COLUMNS_READ;
 
   if (select->derived_table_count || select->table_func_count) {
-    if (select->resolve_placeholder_tables(thd, apply_semijoin))
-      DBUG_RETURN(true);
+    if (select->resolve_placeholder_tables(thd, apply_semijoin)) return true;
 
-    if (select->check_view_privileges(thd, DELETE_ACL, SELECT_ACL))
-      DBUG_RETURN(true);
+    if (select->check_view_privileges(thd, DELETE_ACL, SELECT_ACL)) return true;
   }
 
   /*
@@ -688,19 +695,19 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
     // Cannot delete from a non-updatable view or derived table.
     if (!table_ref->is_updatable()) {
       my_error(ER_NON_UPDATABLE_TABLE, MYF(0), table_ref->alias, "DELETE");
-      DBUG_RETURN(true);
+      return true;
     }
 
     // DELETE does not allow deleting from multi-table views
     if (table_ref->is_multiple_tables()) {
       my_error(ER_VIEW_DELETE_MERGE_VIEW, MYF(0), table_ref->view_db.str,
                table_ref->view_name.str);
-      DBUG_RETURN(true);
+      return true;
     }
 
     if (check_key_in_view(thd, table_ref, table_ref->updatable_base_table())) {
       my_error(ER_NON_UPDATABLE_TABLE, MYF(0), table_ref->alias, "DELETE");
-      DBUG_RETURN(true);
+      return true;
     }
 
     // A view must be merged, and thus cannot have a TABLE
@@ -710,7 +717,7 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
     TABLE_LIST *base_table = table_ref->updatable_base_table();
     if (base_table->table->file->ha_table_flags() & HA_DELETE_NOT_SUPPORTED) {
       my_error(ER_ILLEGAL_HA, MYF(0), base_table->table_name);
-      DBUG_RETURN(true);
+      return true;
     }
 
     for (TABLE_LIST *tr = base_table; tr != nullptr;
@@ -722,20 +729,20 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
   // Precompute and store the row types of NATURAL/USING joins.
   if (select->leaf_table_count >= 2 &&
       setup_natural_join_row_types(thd, select->join_list, &select->context))
-    DBUG_RETURN(true);
+    return true;
 
   // Enable the following code if allowing LIMIT with multi-table DELETE
   DBUG_ASSERT(sql_command_code() == SQLCOM_DELETE || select->select_limit == 0);
 
   lex->allow_sum_func = 0;
 
-  if (select->setup_conds(thd)) DBUG_RETURN(true);
+  if (select->setup_conds(thd)) return true;
 
   DBUG_ASSERT(select->having_cond() == NULL &&
               select->group_list.elements == 0 && select->offset_limit == NULL);
 
   if (select->master_unit()->prepare_limit(thd, select))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
 
   // check ORDER BY even if it can be ignored
   if (select->order_list.first) {
@@ -747,18 +754,17 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
     tables.alias = table_list->alias;
 
     DBUG_ASSERT(!select->group_list.elements);
-    if (select->setup_base_ref_items(thd))
-      DBUG_RETURN(true); /* purecov: inspected */
+    if (select->setup_base_ref_items(thd)) return true; /* purecov: inspected */
     if (setup_order(thd, select->base_ref_items, &tables, fields, all_fields,
                     select->order_list.first))
-      DBUG_RETURN(true);
+      return true;
   }
 
   thd->want_privilege = want_privilege_saved;
   thd->mark_used_columns = mark_used_columns_saved;
 
   if (select->has_ft_funcs() && setup_ftfuncs(thd, select))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
 
   /*
     Check tables to be deleted from for duplicate entries -
@@ -777,7 +783,7 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
                                          lex->query_tables, false);
     if (duplicate) {
       update_non_unique_table_error(table_ref, "DELETE", duplicate);
-      DBUG_RETURN(true);
+      return true;
     }
   }
 
@@ -785,21 +791,21 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
 
   if (select->query_result() &&
       select->query_result()->prepare(thd, select->fields_list, lex->unit))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
 
   opt_trace_print_expanded_query(thd, select, &trace_wrapper);
 
   if (select->has_sj_candidates() && select->flatten_subqueries(thd))
-    DBUG_RETURN(true);
+    return true;
 
   select->set_sj_candidates(NULL);
 
   if (select->apply_local_transforms(thd, true))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
 
   if (!multitable && select->is_empty_query()) set_empty_query();
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -821,7 +827,7 @@ extern "C" int refpos_order_cmp(const void *arg, const void *a, const void *b) {
 }
 
 bool Query_result_delete::prepare(THD *thd, List<Item> &, SELECT_LEX_UNIT *u) {
-  DBUG_ENTER("Query_result_delete::prepare");
+  DBUG_TRACE;
   unit = u;
 
   for (TABLE_LIST *tr = u->first_select()->leaf_tables; tr;
@@ -836,7 +842,7 @@ bool Query_result_delete::prepare(THD *thd, List<Item> &, SELECT_LEX_UNIT *u) {
   }
 
   THD_STAGE_INFO(thd, stage_deleting_from_main_table);
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -848,7 +854,7 @@ bool Query_result_delete::prepare(THD *thd, List<Item> &, SELECT_LEX_UNIT *u) {
 */
 
 bool Query_result_delete::optimize() {
-  DBUG_ENTER("Query_result_delete::optimize");
+  DBUG_TRACE;
 
   SELECT_LEX *const select = unit->first_select();
 
@@ -859,14 +865,14 @@ bool Query_result_delete::optimize() {
 
   if ((thd->variables.option_bits & OPTION_SAFE_UPDATES) &&
       error_if_full_join(join))
-    DBUG_RETURN(true);
+    return true;
 
   if (!(tempfiles =
             (Unique **)sql_calloc(sizeof(Unique *) * delete_table_count)))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
 
   if (!(tables = (TABLE **)sql_calloc(sizeof(TABLE *) * delete_table_count)))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
 
   bool delete_while_scanning = true;
   for (TABLE_LIST *tr = select->leaf_tables; tr; tr = tr->next_leaf) {
@@ -907,7 +913,7 @@ bool Query_result_delete::optimize() {
     if (thd->lex->is_ignore()) table->file->ha_extra(HA_EXTRA_IGNORE_DUP_KEY);
     table->prepare_for_position();
     table->mark_columns_needed_for_delete(thd);
-    if (thd->is_error()) DBUG_RETURN(true);
+    if (thd->is_error()) return true;
   }
   /*
     In some cases, rows may be deleted from the first table(s) in the join order
@@ -935,14 +941,13 @@ bool Query_result_delete::optimize() {
     if (!(*tempfile++ = new (thd->mem_root)
               Unique(refpos_order_cmp, (void *)table->file,
                      table->file->ref_length, thd->variables.sortbuff_size)))
-      DBUG_RETURN(true); /* purecov: inspected */
+      return true; /* purecov: inspected */
     *(table_ptr++) = table;
   }
-  DBUG_ASSERT(select == thd->lex->current_select());
 
-  if (select->has_ft_funcs() && init_ftfuncs(thd, select)) DBUG_RETURN(true);
+  if (select->has_ft_funcs() && init_ftfuncs(thd, select)) return true;
 
-  DBUG_RETURN(thd->is_fatal_error());
+  return thd->is_fatal_error();
 }
 
 void Query_result_delete::cleanup(THD *) {
@@ -958,11 +963,10 @@ void Query_result_delete::cleanup(THD *) {
 }
 
 bool Query_result_delete::send_data(THD *thd, List<Item> &) {
-  DBUG_ENTER("Query_result_delete::send_data");
+  DBUG_TRACE;
 
   JOIN *const join = unit->first_select()->join;
 
-  DBUG_ASSERT(thd->lex->current_select() == unit->first_select());
   int unique_counter = 0;
 
   for (uint i = 0; i < join->primary_tables; i++) {
@@ -994,7 +998,7 @@ bool Query_result_delete::send_data(THD *thd, List<Item> &) {
       if (table->triggers &&
           table->triggers->process_triggers(thd, TRG_EVENT_DELETE,
                                             TRG_ACTION_BEFORE, false))
-        DBUG_RETURN(true);
+        return true;
       table->set_deleted_row();
       if (map & non_transactional_table_map) non_transactional_deleted = true;
       if (!(error = table->file->ha_delete_row(table->record[0]))) {
@@ -1005,7 +1009,7 @@ bool Query_result_delete::send_data(THD *thd, List<Item> &) {
         if (table->triggers &&
             table->triggers->process_triggers(thd, TRG_EVENT_DELETE,
                                               TRG_ACTION_AFTER, false))
-          DBUG_RETURN(true);
+          return true;
       } else {
         myf error_flags = MYF(0);
         if (table->file->is_fatal_error(error)) error_flags |= ME_FATALERROR;
@@ -1015,7 +1019,7 @@ bool Query_result_delete::send_data(THD *thd, List<Item> &) {
           If IGNORE option is used errors caused by ha_delete_row will
           be downgraded to warnings and don't have to stop the iteration.
         */
-        if (thd->is_error()) DBUG_RETURN(true);
+        if (thd->is_error()) return true;
 
         /*
           If IGNORE keyword is used, then 'error' variable will have the error
@@ -1030,31 +1034,29 @@ bool Query_result_delete::send_data(THD *thd, List<Item> &) {
       if (error) {
         /* purecov: begin inspected */
         error = 1;
-        DBUG_RETURN(true);
+        return true;
         /* purecov: end */
       }
     }
   }
-  DBUG_RETURN(false);
+  return false;
 }
 
 void Query_result_delete::send_error(THD *, uint errcode, const char *err) {
-  DBUG_ENTER("Query_result_delete::send_error");
+  DBUG_TRACE;
 
   /* First send error what ever it is ... */
   my_message(errcode, err, MYF(0));
-
-  DBUG_VOID_RETURN;
 }
 
 void Query_result_delete::abort_result_set(THD *thd) {
-  DBUG_ENTER("Query_result_delete::abort_result_set");
+  DBUG_TRACE;
 
   /* the error was handled or nothing deleted and no side effects return */
   if (error_handled ||
       (!thd->get_transaction()->cannot_safely_rollback(Transaction_ctx::STMT) &&
        deleted_rows == 0))
-    DBUG_VOID_RETURN;
+    return;
 
   /*
     If rows from the first table only has been deleted and it is
@@ -1070,7 +1072,7 @@ void Query_result_delete::abort_result_set(THD *thd) {
     error = 1;
     send_eof(thd);
     DBUG_ASSERT(error_handled);
-    DBUG_VOID_RETURN;
+    return;
   }
 
   if (thd->get_transaction()->cannot_safely_rollback(Transaction_ctx::STMT)) {
@@ -1085,7 +1087,6 @@ void Query_result_delete::abort_result_set(THD *thd) {
                               false, false, errcode);
     }
   }
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -1096,29 +1097,28 @@ void Query_result_delete::abort_result_set(THD *thd) {
 */
 
 int Query_result_delete::do_deletes(THD *thd) {
-  DBUG_ENTER("Query_result_delete::do_deletes");
+  DBUG_TRACE;
   DBUG_ASSERT(!delete_completed);
 
-  DBUG_ASSERT(thd->lex->current_select() == unit->first_select());
   delete_completed = true;  // Mark operation as complete
-  if (found_rows == 0) DBUG_RETURN(0);
+  if (found_rows == 0) return 0;
 
   for (uint counter = 0; counter < delete_table_count; counter++) {
     TABLE *const table = tables[counter];
     if (table == NULL) break;
 
-    if (tempfiles[counter]->get(table)) DBUG_RETURN(1);
+    if (tempfiles[counter]->get(table)) return 1;
 
     int local_error = do_table_deletes(thd, table);
 
-    if (thd->killed && !local_error) DBUG_RETURN(1);
+    if (thd->killed && !local_error) return 1;
 
     if (local_error == -1)  // End of file
       local_error = 0;
 
-    if (local_error) DBUG_RETURN(local_error);
+    if (local_error) return local_error;
   }
-  DBUG_RETURN(0);
+  return 0;
 }
 
 /**
@@ -1137,18 +1137,17 @@ int Query_result_delete::do_deletes(THD *thd) {
 int Query_result_delete::do_table_deletes(THD *thd, TABLE *table) {
   myf error_flags = MYF(0); /**< Flag for fatal errors */
   int local_error = 0;
-  READ_RECORD info;
   ha_rows last_deleted = deleted_rows;
-  DBUG_ENTER("Query_result_delete::do_table_deletes");
+  DBUG_TRACE;
   /*
     Ignore any rows not found in reference tables as they may already have
     been deleted by foreign key handling
   */
-  if (init_read_record(&info, thd, table, NULL, false,
-                       /*ignore_not_found_rows=*/true))
-    DBUG_RETURN(1);
+  unique_ptr_destroy_only<RowIterator> iterator = init_table_iterator(
+      thd, table, nullptr, false, /*ignore_not_found_rows=*/true);
+  if (iterator == nullptr) return 1;
   bool will_batch = !table->file->start_bulk_delete();
-  while (!(local_error = info->Read()) && !thd->killed) {
+  while (!(local_error = iterator->Read()) && !thd->killed) {
     if (table->triggers &&
         table->triggers->process_triggers(thd, TRG_EVENT_DELETE,
                                           TRG_ACTION_BEFORE, false)) {
@@ -1201,7 +1200,7 @@ int Query_result_delete::do_table_deletes(THD *thd, TABLE *table) {
     thd->get_transaction()->mark_modified_non_trans_table(
         Transaction_ctx::STMT);
 
-  DBUG_RETURN(local_error);
+  return local_error;
 }
 
 /**
@@ -1244,10 +1243,10 @@ bool Query_result_delete::send_eof(THD *thd) {
   if (local_error != 0)
     error_handled = true;  // to force early leave from ::send_error()
 
-  if (!local_error) {
+  if (!local_error && !thd->is_error()) {
     ::my_ok(thd, deleted_rows);
   }
-  return 0;
+  return thd->is_error();
 }
 
 bool Sql_cmd_delete::accept(THD *thd, Select_lex_visitor *visitor) {
