@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_HINT_PARSER_EXPORT_HOME_PB2_BUILD_SB_3_34751636_1561574802_18_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED
-# define YY_HINT_PARSER_EXPORT_HOME_PB2_BUILD_SB_3_34751636_1561574802_18_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED
+#ifndef YY_HINT_PARSER_EXPORT_HOME3_PB2_BUILD_SB_4_36015059_1568969063_34_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED
+# define YY_HINT_PARSER_EXPORT_HOME3_PB2_BUILD_SB_4_36015059_1568969063_34_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -75,13 +75,15 @@ extern int HINT_PARSER_debug;
     SET_VAR_HINT = 285,
     SKIP_SCAN_HINT = 286,
     NO_SKIP_SCAN_HINT = 287,
-    HINT_ARG_NUMBER = 288,
-    HINT_ARG_IDENT = 289,
-    HINT_ARG_QB_NAME = 290,
-    HINT_ARG_TEXT = 291,
-    HINT_IDENT_OR_NUMBER_WITH_SCALE = 292,
-    HINT_CLOSE = 293,
-    HINT_ERROR = 294
+    HASH_JOIN_HINT = 288,
+    NO_HASH_JOIN_HINT = 289,
+    HINT_ARG_NUMBER = 290,
+    HINT_ARG_IDENT = 291,
+    HINT_ARG_QB_NAME = 292,
+    HINT_ARG_TEXT = 293,
+    HINT_IDENT_OR_NUMBER_WITH_SCALE = 294,
+    HINT_CLOSE = 295,
+    HINT_ERROR = 296
   };
 #endif
 /* Tokens.  */
@@ -115,13 +117,15 @@ extern int HINT_PARSER_debug;
 #define SET_VAR_HINT 285
 #define SKIP_SCAN_HINT 286
 #define NO_SKIP_SCAN_HINT 287
-#define HINT_ARG_NUMBER 288
-#define HINT_ARG_IDENT 289
-#define HINT_ARG_QB_NAME 290
-#define HINT_ARG_TEXT 291
-#define HINT_IDENT_OR_NUMBER_WITH_SCALE 292
-#define HINT_CLOSE 293
-#define HINT_ERROR 294
+#define HASH_JOIN_HINT 288
+#define NO_HASH_JOIN_HINT 289
+#define HINT_ARG_NUMBER 290
+#define HINT_ARG_IDENT 291
+#define HINT_ARG_QB_NAME 292
+#define HINT_ARG_TEXT 293
+#define HINT_IDENT_OR_NUMBER_WITH_SCALE 294
+#define HINT_CLOSE 295
+#define HINT_ERROR 296
 
 /* Value type.  */
 
@@ -129,4 +133,4 @@ extern int HINT_PARSER_debug;
 
 int HINT_PARSER_parse (class THD *thd, class Hint_scanner *scanner, class PT_hint_list **ret);
 
-#endif /* !YY_HINT_PARSER_EXPORT_HOME_PB2_BUILD_SB_3_34751636_1561574802_18_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED  */
+#endif /* !YY_HINT_PARSER_EXPORT_HOME3_PB2_BUILD_SB_4_36015059_1568969063_34_DIST_GPL_SQL_SQL_HINTS_YY_H_INCLUDED  */
