@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,14 +22,13 @@
 
 #include "sql/sql_initialize.h"
 
-#include <assert.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include "components/mysql_server/log_builtins_filter_imp.h"  // verbosity
 #include "m_ctype.h"
-
 #include "my_dir.h"
 #include "my_inttypes.h"
 #include "my_io.h"
@@ -38,17 +36,21 @@
 #include "my_rnd.h"
 #include "my_sys.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql_com.h"
 #include "mysqld_error.h"
 #include "scripts/sql_commands_help_data.h"
 #include "scripts/sql_commands_system_data.h"
 #include "scripts/sql_commands_system_tables.h"
 #include "scripts/sql_commands_system_users.h"
 #include "scripts/sys_schema/sql_commands.h"
+#include "sql/current_thd.h"
+#include "sql/log.h"
 #include "sql/mysqld.h"
-#include "sql/server_component/log_builtins_filter_imp.h"  // verbosity
 #include "sql/sql_bootstrap.h"
+#include "sql/sql_class.h"
+#include "sql/sql_error.h"
 
-static const char *initialization_cmds[] = {"USE mysql;\n", nullptr};
+static const char *initialization_cmds[] = {"USE mysql;\n", NULL};
 
 #define INSERT_USER_CMD \
   "CREATE USER root@localhost IDENTIFIED BY '%s' PASSWORD EXPIRE;\n"
@@ -63,15 +65,9 @@ bool opt_initialize_insecure = false;
 bool mysql_initialize_directory_freshly_created = false;
 
 static const char *initialization_data[] = {
-    "FLUSH PRIVILEGES",
-    insert_user_buffer,
+    "FLUSH PRIVILEGES", insert_user_buffer,
     "GRANT ALL PRIVILEGES ON *.* TO root@localhost WITH GRANT OPTION;\n",
-    "GRANT PROXY ON ''@'' TO 'root'@'localhost' WITH GRANT OPTION;\n",
-    "INSERT IGNORE INTO mysql.global_grants VALUES ('root', 'localhost', "
-    "'AUDIT_ABORT_EXEMPT', 'Y');\n",
-    "INSERT IGNORE INTO mysql.global_grants VALUES ('root', 'localhost', "
-    "'FIREWALL_EXEMPT', 'Y')",
-    nullptr};
+    "GRANT PROXY ON ''@'' TO 'root'@'localhost' WITH GRANT OPTION;\n", NULL};
 
 static const char **cmds[] = {initialization_cmds, mysql_system_tables,
                               initialization_data, mysql_system_data,
@@ -89,9 +85,18 @@ static const char *cmd_descs[] = {
     "Creating the sys schema",
     nullptr};
 
-bool generate_password(char *password, int size) {
+static bool generate_password(char *password, int size) {
+#define UPCHARS "QWERTYUIOPASDFGHJKLZXCVBNM"
+#define LOWCHARS "qwertyuiopasdfghjklzxcvbnm"
+#define NUMCHARS "1234567890"
+#define SYMCHARS ",.-+*;:_!#%&/()=?><"
 #define rnd_of(x) x[((int)(my_rnd_ssl(&failed) * 100)) % (sizeof(x) - 1)]
 
+  static const char g_allowed_pwd_chars[] = LOWCHARS SYMCHARS UPCHARS NUMCHARS;
+  static const char g_upper_case_chars[] = UPCHARS;
+  static const char g_lower_case_chars[] = LOWCHARS;
+  static const char g_numeric_chars[] = NUMCHARS;
+  static const char g_special_chars[] = SYMCHARS;
   bool failed = false;
   char *ptr = password;
   bool had_upper = false, had_lower = false, had_numeric = false,
@@ -177,16 +182,15 @@ bool Compiled_in_command_iterator::begin(void) {
 }
 
 int Compiled_in_command_iterator::next(std::string &query) {
-  while (cmds[m_cmds_ofs] != nullptr &&
-         cmds[m_cmds_ofs][m_cmd_ofs] == nullptr) {
+  while (cmds[m_cmds_ofs] != NULL && cmds[m_cmds_ofs][m_cmd_ofs] == NULL) {
     m_cmds_ofs++;
-    if (cmds[m_cmds_ofs] != nullptr)
+    if (cmds[m_cmds_ofs] != NULL)
       LogErr(INFORMATION_LEVEL, ER_SERVER_INIT_COMPILED_IN_COMMANDS,
              cmd_descs[m_cmds_ofs]);
     m_cmd_ofs = 0;
   }
 
-  if (cmds[m_cmds_ofs] == nullptr) {
+  if (cmds[m_cmds_ofs] == NULL) {
     return READ_BOOTSTRAP_EOF;
   }
 
@@ -202,7 +206,7 @@ void Compiled_in_command_iterator::report_error_details(
     so ::next() never returns errors.
     Hence, there should never be an error to print.
   */
-  assert(false);
+  DBUG_ASSERT(false);
   return;
 }
 
@@ -234,7 +238,7 @@ bool initialize_create_data_directory(const char *data_home) {
 #endif
       ;
 
-  if (nullptr != (dir = my_dir(data_home, MYF(MY_DONT_SORT)))) {
+  if (NULL != (dir = my_dir(data_home, MYF(MY_DONT_SORT)))) {
     bool no_files = true;
     char path[FN_REFLEN];
     File fd;
@@ -257,8 +261,8 @@ bool initialize_create_data_directory(const char *data_home) {
 
     LogErr(INFORMATION_LEVEL, ER_INIT_DATADIR_EXISTS_WONT_INITIALIZE);
 
-    if (nullptr == fn_format(path, "is_writable", data_home, "",
-                             MY_UNPACK_FILENAME | MY_SAFE_PATH)) {
+    if (NULL == fn_format(path, "is_writable", data_home, "",
+                          MY_UNPACK_FILENAME | MY_SAFE_PATH)) {
       LogErr(ERROR_LEVEL,
              ER_INIT_DATADIR_EXISTS_AND_PATH_TOO_LONG_WONT_INITIALIZE);
       return true; /* purecov: inspected */
