@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,6 +40,10 @@
 #include "sql/sql_lex_hash.h"
 #include "sql_chars.h"
 
+// This must be last, due to bison 2.3 on OsX
+#ifndef YYSTYPE_IS_DECLARED
+#define YYSTYPE_IS_DECLARED 1
+#endif  // YYSTYPE_IS_DECLARED
 #include "sql/sql_hints.yy.h"
 
 class PT_hint_list;
@@ -58,6 +61,7 @@ class Hint_scanner {
   THD *thd;
   const CHARSET_INFO *cs;
   const bool is_ansi_quotes;
+  const bool backslash_escapes;
   size_t lineno;
   const hint_lex_char_classes *char_classes;
 
@@ -105,20 +109,64 @@ class Hint_scanner {
   }
 
  protected:
-  int scan();
+  int scan() {
+    int whitespaces = 0;
+    for (;;) {
+      start_token();
+      switch (peek_class()) {
+        case HINT_CHR_NL:
+          skip_newline();
+          whitespaces++;
+          continue;
+        case HINT_CHR_SPACE:
+          skip_byte();
+          whitespaces++;
+          continue;
+        case HINT_CHR_DIGIT:
+          return scan_number_or_ident();
+        case HINT_CHR_IDENT:
+          return scan_ident_or_keyword();
+        case HINT_CHR_MB:
+          return scan_ident();
+        case HINT_CHR_QUOTE:
+          return scan_quoted<HINT_CHR_QUOTE>();
+        case HINT_CHR_BACKQUOTE:
+          return scan_quoted<HINT_CHR_BACKQUOTE>();
+        case HINT_CHR_DOUBLEQUOTE:
+          return scan_quoted<HINT_CHR_DOUBLEQUOTE>();
+        case HINT_CHR_ASTERISK:
+          if (peek_class2() == HINT_CHR_SLASH) {
+            ptr += 2;  // skip "*/"
+            input_buf_end = ptr;
+            return HINT_CLOSE;
+          } else
+            return get_byte();
+        case HINT_CHR_AT:
+          if (prev_token == '(' ||
+              (prev_token == HINT_ARG_IDENT && whitespaces == 0))
+            return scan_query_block_name();
+          else
+            return get_byte();
+        case HINT_CHR_EOF:
+          return 0;
+        default:
+          return get_byte();
+      }
+    }
+  }
 
   template <hint_lex_char_classes Quote>
   int scan_quoted() {
-    assert(Quote == HINT_CHR_BACKQUOTE || Quote == HINT_CHR_DOUBLEQUOTE ||
-           Quote == HINT_CHR_QUOTE);
-    assert(*ptr == '`' || *ptr == '"' || *ptr == '\'');
+    DBUG_ASSERT(Quote == HINT_CHR_BACKQUOTE || Quote == HINT_CHR_DOUBLEQUOTE ||
+                Quote == HINT_CHR_QUOTE);
+    DBUG_ASSERT(*ptr == '`' || *ptr == '"' || *ptr == '\'');
 
     const bool is_ident = (Quote == HINT_CHR_BACKQUOTE) ||
                           (is_ansi_quotes && Quote == HINT_CHR_DOUBLEQUOTE);
     const int ret = is_ident ? HINT_ARG_IDENT : HINT_ARG_TEXT;
 
-    skip_byte("\"'`");  // skip opening quote sign
-    adjust_token();     // reset yytext & yyleng
+    skip_byte();     // skip opening quote sign
+    adjust_token();  // reset yytext & yyleng
 
     size_t double_separators = 0;
 
@@ -135,14 +183,14 @@ class Hint_scanner {
           if (peek_class2() == HINT_CHR_SLASH)
             return HINT_ERROR;  // we don't support "*/" inside quoted
                                 // identifiers
-          skip_byte('*');
+          skip_byte();
           continue;
         case HINT_CHR_EOF:
           return HINT_ERROR;
         case Quote:
           if (peek_class2() == Quote) {
-            skip_byte("\"'`");
-            skip_byte("\"'`");
+            skip_byte();  // skip quote
+            skip_byte();  // skip quote
             double_separators++;
             continue;
           } else {
@@ -150,10 +198,8 @@ class Hint_scanner {
 
             ptr++;  // skip closing quote
 
-            if (thd->charset_is_system_charset && double_separators == 0) {
-              yytext = thd->strmake(yytext, yyleng);  // null-terminate it.
+            if (thd->charset_is_system_charset && double_separators == 0)
               return ret;
-            }
 
             LEX_STRING s;
             if (!thd->charset_is_system_charset) {
@@ -161,10 +207,10 @@ class Hint_scanner {
                                       thd->charset()))
                 return HINT_ERROR;  // OOM etc.
             } else {
-              assert(0 < double_separators && double_separators < yyleng);
+              DBUG_ASSERT(0 < double_separators && double_separators < yyleng);
               s.length = yyleng - double_separators;
-              s.str = static_cast<char *>(thd->alloc(s.length));
-              if (s.str == nullptr) return HINT_ERROR;  // OOM
+              s.str = (char *)thd->alloc(s.length);
+              if (s.str == NULL) return HINT_ERROR;  // OOM
             }
             if (double_separators > 0)
               compact<Quote>(&s, yytext, yyleng, double_separators);
@@ -197,8 +243,8 @@ class Hint_scanner {
     }
   }
 
-  int scan_multiplier_or_ident() {
-    assert(peek_class() == HINT_CHR_IDENT);
+  int scan_scale_or_ident() {
+    DBUG_ASSERT(peek_class() == HINT_CHR_IDENT);
     switch (peek_byte()) {
       case 'K':
       case 'M':
@@ -219,18 +265,16 @@ class Hint_scanner {
   }
 
   int scan_query_block_name() {
-    skip_byte('@');
+    DBUG_ASSERT(*ptr == '@');
+
+    skip_byte();  // skip '@'
     start_token();
 
     switch (peek_class()) {
       case HINT_CHR_IDENT:
       case HINT_CHR_DIGIT:
       case HINT_CHR_MB:
-        if (scan_ident() == HINT_ARG_IDENT) {
-          yytext = thd->strmake(yytext, yyleng);
-          return HINT_ARG_QB_NAME;
-        } else
-          return HINT_ERROR;
+        return scan_ident() == HINT_ARG_IDENT ? HINT_ARG_QB_NAME : HINT_ERROR;
       case HINT_CHR_BACKQUOTE:
         return scan_quoted<HINT_CHR_BACKQUOTE>() == HINT_ARG_IDENT
                    ? HINT_ARG_QB_NAME
@@ -265,7 +309,7 @@ class Hint_scanner {
               keyword string since symbol array is a global constant).
             */
             yytext = symbol->name;
-            assert(yyleng == symbol->length);
+            DBUG_ASSERT(yyleng == symbol->length);
 
             return symbol->tok;
           }
@@ -276,19 +320,14 @@ class Hint_scanner {
     }
   }
 
-  int scan_number_or_multiplier_or_ident() {
-    assert(peek_class() == HINT_CHR_DIGIT);
-    skip_byte();
-
+  int scan_number_or_ident() {
     for (;;) {
       switch (peek_class()) {
         case HINT_CHR_DIGIT:
           skip_byte();
           continue;
-        case HINT_CHR_DOT:
-          return scan_fraction_digits();
         case HINT_CHR_IDENT:
-          return scan_multiplier_or_ident();
+          return scan_scale_or_ident();
         case HINT_CHR_MB:
           return scan_ident();
         case HINT_CHR_EOF:
@@ -298,35 +337,13 @@ class Hint_scanner {
     }
   }
 
-  int scan_fraction_digits() {
-    skip_byte('.');
-
-    if (peek_class() == HINT_CHR_DIGIT)
-      skip_byte();
-    else
-      return HINT_ERROR;
-
-    for (;;) {
-      switch (peek_class()) {
-        case HINT_CHR_DIGIT:
-          skip_byte();
-          continue;
-        case HINT_CHR_IDENT:
-        case HINT_CHR_MB:
-          return HINT_ERROR;
-        default:
-          return HINT_ARG_FLOATING_POINT_NUMBER;
-      }
-    }
-  }
-
   bool eof() const {
-    assert(ptr <= input_buf_end);
+    DBUG_ASSERT(ptr <= input_buf_end);
     return ptr >= input_buf_end;
   }
 
   char peek_byte() const {
-    assert(!eof());
+    DBUG_ASSERT(!eof());
     return *ptr;
   }
 
@@ -335,19 +352,19 @@ class Hint_scanner {
   }
 
   hint_lex_char_classes peek_class2() const {
-    assert(ptr + 1 <= input_buf_end);
+    DBUG_ASSERT(ptr + 1 <= input_buf_end);
     return ptr + 1 >= input_buf_end ? HINT_CHR_EOF
                                     : char_classes[static_cast<uchar>(ptr[1])];
   }
 
   void skip_newline() {
-    assert(!eof() && peek_byte() == '\n');
+    DBUG_ASSERT(!eof() && peek_byte() == '\n');
     skip_byte();
     lineno++;
   }
 
   uchar get_byte() {
-    assert(!eof());
+    DBUG_ASSERT(!eof());
     char ret = *ptr;
     yyleng++;
     ptr++;
@@ -355,33 +372,11 @@ class Hint_scanner {
   }
 
   void skip_byte() {
-    assert(!eof());
+    DBUG_ASSERT(!eof());
     yyleng++;
     ptr++;
   }
 
-  /**
-    Skips the next byte. In the debug mode, abort if it's not found in @p byte.
-
-    @param byte         A byte to compare with the byte we skip.
-                        Unused in non-debug builds.
-  */
-  void skip_byte(char byte [[maybe_unused]]) {
-    assert(peek_byte() == byte);
-    skip_byte();
-  }
-
-  /**
-    Skips the next byte. In the debug mode, abort if it's not found in @p str.
-
-    @param str          A string of characters to compare with the next byte.
-                        Unused in non-debug builds.
-  */
-  void skip_byte(const char *str [[maybe_unused]]) {
-    assert(strchr(str, peek_byte()));
-    skip_byte();
-  }
-
   bool skip_mb() {
     size_t len = my_ismbchar(cs, ptr, input_buf_end);
     if (len == 0) {
@@ -406,22 +401,22 @@ class Hint_scanner {
 
   template <hint_lex_char_classes Separator>
   void compact(LEX_STRING *to, const char *from, size_t len, size_t doubles) {
-    assert(doubles > 0);
+    DBUG_ASSERT(doubles > 0);
 
     size_t d = doubles;
     char *t = to->str;
     for (const char *s = from, *end = from + len; s < end;) {
       switch (char_classes[(uchar)*s]) {
         case HINT_CHR_MB: {
-          size_t hint_len = my_ismbchar(cs, s, end);
-          assert(hint_len > 1);
-          memcpy(t, s, hint_len);
-          t += hint_len;
-          s += hint_len;
+          size_t len = my_ismbchar(cs, s, end);
+          DBUG_ASSERT(len > 1);
+          memcpy(t, s, len);
+          t += len;
+          s += len;
         }
           continue;
         case Separator:
-          assert(char_classes[(uchar)*s] == Separator);
+          DBUG_ASSERT(char_classes[(uchar)*s] == Separator);
           *t++ = *s++;
           s++;  // skip the 2nd separator
           d--;
@@ -432,14 +427,14 @@ class Hint_scanner {
           }
           continue;
         case HINT_CHR_EOF:
-          assert(0);
+          DBUG_ASSERT(0);
           to->length = 0;
           return;
         default:
           *t++ = *s++;
       }
     }
-    assert(0);
+    DBUG_ASSERT(0);
     to->length = 0;
     return;
   }
@@ -453,7 +448,7 @@ class Hint_scanner {
     @param token        A token number to add.
   */
   void add_digest(uint token) {
-    if (digest_state == nullptr) return;  // Digest buffer is full.
+    if (digest_state == NULL) return;  // Digest buffer is full.
 
     Lexer_yystype fake_yylvalue;
     /*
