@@ -1,18 +1,17 @@
 #ifndef INCLUDES_MYSQL_SQL_LIST_H
 #define INCLUDES_MYSQL_SQL_LIST_H
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,7 +50,7 @@ class SQL_I_List {
   /** A reference to the next element in the list. */
   T **next;
 
-  SQL_I_List() { clear(); }
+  SQL_I_List() { empty(); }
 
   SQL_I_List(const SQL_I_List &tmp)
       : elements(tmp.elements),
@@ -60,9 +59,9 @@ class SQL_I_List {
 
   SQL_I_List(SQL_I_List &&) = default;
 
-  inline void clear() {
+  inline void empty() {
     elements = 0;
-    first = nullptr;
+    first = NULL;
     next = &first;
   }
 
@@ -70,12 +69,12 @@ class SQL_I_List {
     elements++;
     (*next) = element;
     next = next_ptr;
-    *next = nullptr;
+    *next = NULL;
   }
 
   inline void save_and_clear(SQL_I_List<T> *save) {
     *save = *this;
-    clear();
+    empty();
   }
 
   inline void push_front(SQL_I_List<T> *save) {
@@ -121,7 +120,7 @@ struct list_node {
       : next(next_par), info(info_par) {}
   list_node() /* For end_of_list */
   {
-    info = nullptr;
+    info = 0;
     next = this;
   }
 };
@@ -139,12 +138,12 @@ class base_list {
     return elements == rhs.elements && first == rhs.first && last == rhs.last;
   }
 
-  inline void clear() {
+  inline void empty() {
     elements = 0;
     first = &end_of_list;
     last = &first;
   }
-  inline base_list() { clear(); }
+  inline base_list() { empty(); }
   /**
     This is a shallow copy constructor that implicitly passes the ownership
     from the source list to the new instance. The old instance is not
@@ -170,22 +169,20 @@ class base_list {
   */
   base_list(const base_list &rhs, MEM_ROOT *mem_root);
   inline bool push_back(void *info) {
-    *last = new (*THR_MALLOC) list_node(info, &end_of_list);
-    if (*last) {
+    if (((*last) = new (*THR_MALLOC) list_node(info, &end_of_list))) {
       last = &(*last)->next;
       elements++;
-      return false;
+      return 0;
     }
-    return true;
+    return 1;
   }
   inline bool push_back(void *info, MEM_ROOT *mem_root) {
-    *last = new (mem_root) list_node(info, &end_of_list);
-    if (*last) {
+    if (((*last) = new (mem_root) list_node(info, &end_of_list))) {
       last = &(*last)->next;
       elements++;
-      return false;
+      return 0;
     }
-    return true;
+    return 1;
   }
   inline bool push_front(void *info) {
     list_node *node = new (*THR_MALLOC) list_node(info, first);
@@ -193,9 +190,9 @@ class base_list {
       if (last == &first) last = &node->next;
       first = node;
       elements++;
-      return false;
+      return 0;
     }
-    return true;
+    return 1;
   }
   inline bool push_front(void *info, MEM_ROOT *mem_root) {
     list_node *node = new (mem_root) list_node(info, first);
@@ -225,7 +222,7 @@ class base_list {
     }
   }
   inline void *pop(void) {
-    if (first == &end_of_list) return nullptr;
+    if (first == &end_of_list) return 0;
     list_node *tmp = first;
     first = first->next;
     if (!--elements) last = &first;
@@ -264,9 +261,7 @@ class base_list {
   inline list_node *first_node() { return first; }
   inline void *head() { return first->info; }
   inline const void *head() const { return first->info; }
-  inline void **head_ref() {
-    return first != &end_of_list ? &first->info : nullptr;
-  }
+  inline void **head_ref() { return first != &end_of_list ? &first->info : 0; }
   inline bool is_empty() const { return first == &end_of_list; }
   inline list_node *last_ref() { return &end_of_list; }
   inline uint size() const { return elements; }
@@ -347,16 +342,15 @@ class base_list_iterator {
   }
 
  public:
-  base_list_iterator()
-      : list(nullptr), el(nullptr), prev(nullptr), current(nullptr) {}
+  base_list_iterator() : list(0), el(0), prev(0), current(0) {}
 
   base_list_iterator(base_list &list_par) { init(list_par); }
 
   inline void init(base_list &list_par) {
     list = &list_par;
     el = &list_par.first;
-    prev = nullptr;
-    current = nullptr;
+    prev = 0;
+    current = 0;
   }
 
   inline void *next(void) {
@@ -374,7 +368,7 @@ class base_list_iterator {
   inline void rewind(void) { el = &list->first; }
   inline void *replace(void *element) {  // Return old element
     void *tmp = current->info;
-    assert(current->info != nullptr);
+    DBUG_ASSERT(current->info != 0);
     current->info = element;
     return tmp;
   }
@@ -394,7 +388,7 @@ class base_list_iterator {
   {
     list->remove(prev);
     el = prev;
-    current = nullptr;  // Safeguard
+    current = 0;  // Safeguard
   }
   void after(void *element)  // Insert element after current
   {
@@ -414,7 +408,7 @@ class base_list_iterator {
     return &current->info;
   }
   inline bool is_last(void) { return el == list->last; }
-  inline bool is_before_first() const { return current == nullptr; }
+  inline bool is_before_first() const { return current == NULL; }
   bool prepend(void *a, MEM_ROOT *mem_root) {
     if (list->push_front(a, mem_root)) return true;
 
@@ -472,7 +466,7 @@ class List : public base_list {
       next = element->next;
       delete (T *)element->info;
     }
-    clear();
+    empty();
   }
 
   void destroy_elements(void) {
@@ -481,18 +475,18 @@ class List : public base_list {
       next = element->next;
       destroy((T *)element->info);
     }
-    clear();
+    empty();
   }
 
   T *operator[](uint index) const {
-    assert(index < elements);
+    DBUG_ASSERT(index < elements);
     list_node *current = first;
     for (uint i = 0; i < index; ++i) current = current->next;
     return static_cast<T *>(current->info);
   }
 
   void replace(uint index, T *new_value) {
-    assert(index < elements);
+    DBUG_ASSERT(index < elements);
     list_node *current = first;
     for (uint i = 0; i < index; ++i) current = current->next;
     current->info = new_value;
@@ -551,7 +545,7 @@ class List : public base_list {
     // If the list overlaps another list, last isn't actually
     // the last element, and if so, we'd give a different result from
     // List_iterator_fast.
-    assert((*last)->next == &end_of_list);
+    DBUG_ASSERT((*last)->next == &end_of_list);
 
     return iterator(*last);
   }
@@ -559,12 +553,12 @@ class List : public base_list {
   using const_iterator = List_STL_Iterator<const T>;
   const_iterator begin() const { return const_iterator(first); }
   const_iterator end() const {
-    assert((*last)->next == &end_of_list);
+    DBUG_ASSERT((*last)->next == &end_of_list);
     return const_iterator(*last);
   }
   const_iterator cbegin() const { return const_iterator(first); }
   const_iterator cend() const {
-    assert((*last)->next == &end_of_list);
+    DBUG_ASSERT((*last)->next == &end_of_list);
     return const_iterator(*last);
   }
 };
@@ -584,10 +578,7 @@ class List_iterator : public base_list_iterator {
   inline bool after(T *a, MEM_ROOT *mem_root) {
     return base_list_iterator::after(a, mem_root);
   }
-  inline T **ref(void) {
-    return const_cast<T **>(
-        (std::remove_const_t<T> **)base_list_iterator::ref());
-  }
+  inline T **ref(void) { return (T **)base_list_iterator::ref(); }
 };
 
 template <class T>
@@ -648,7 +639,7 @@ class List_STL_Iterator {
   T *operator->() const { return static_cast<T *>(m_current->info); }
 
   // DefaultConstructible (required for ForwardIterator).
-  List_STL_Iterator() = default;
+  List_STL_Iterator() {}
 
   // ForwardIterator.
   List_STL_Iterator operator++(int) {
@@ -679,14 +670,14 @@ class ilink {
   T **prev, *next;
 
  public:
-  ilink() : prev(nullptr), next(nullptr) {}
+  ilink() : prev(NULL), next(NULL) {}
 
   void unlink() {
     /* Extra tests because element doesn't have to be linked */
     if (prev) *prev = next;
     if (next) next->prev = prev;
-    prev = nullptr;
-    next = nullptr;
+    prev = NULL;
+    next = NULL;
   }
 
   friend class base_ilist<T>;
@@ -698,7 +689,7 @@ class ilink {
 class i_string : public ilink<i_string> {
  public:
   const char *ptr;
-  i_string() : ptr(nullptr) {}
+  i_string() : ptr(0) {}
   i_string(const char *s) : ptr(s) {}
 };
 
@@ -707,7 +698,7 @@ class i_string_pair : public ilink<i_string_pair> {
  public:
   const char *key;
   const char *val;
-  i_string_pair() : key(nullptr), val(nullptr) {}
+  i_string_pair() : key(0), val(0) {}
   i_string_pair(const char *key_arg, const char *val_arg)
       : key(key_arg), val(val_arg) {}
 };
@@ -725,11 +716,11 @@ class base_ilist {
 
  public:
   // The sentinel is not a T, but at least it is a POD
-  void clear() SUPPRESS_UBSAN {
+  void empty() SUPPRESS_UBSAN {
     first = static_cast<T *>(&sentinel);
     sentinel.prev = &first;
   }
-  base_ilist() { clear(); }
+  base_ilist() { empty(); }
 
   // The sentinel is not a T, but at least it is a POD
   bool is_empty() const SUPPRESS_UBSAN {
@@ -754,13 +745,13 @@ class base_ilist {
 
   // Unlink first element, and return it.
   T *get() {
-    if (is_empty()) return nullptr;
+    if (is_empty()) return NULL;
     T *first_link = first;
     first_link->unlink();
     return first_link;
   }
 
-  T *head() { return is_empty() ? nullptr : first; }
+  T *head() { return is_empty() ? NULL : first; }
 
   /**
     Moves list elements to new owner, and empties current owner (i.e. this).
@@ -770,10 +761,10 @@ class base_ilist {
   */
 
   void move_elements_to(base_ilist *new_owner) {
-    assert(new_owner->is_empty());
+    DBUG_ASSERT(new_owner->is_empty());
     new_owner->first = first;
     new_owner->sentinel = sentinel;
-    clear();
+    empty();
   }
 
   friend class base_ilist_iterator<T>;
@@ -795,13 +786,13 @@ class base_ilist_iterator {
 
  public:
   base_ilist_iterator(base_ilist<T> &list_par)
-      : list(&list_par), el(&list_par.first), current(nullptr) {}
+      : list(&list_par), el(&list_par.first), current(NULL) {}
 
   // The sentinel is not a T, but at least it is a POD
   T *next(void) SUPPRESS_UBSAN {
     /* This is coded to allow push_back() while iterating */
     current = *el;
-    if (current == static_cast<T *>(&list->sentinel)) return nullptr;
+    if (current == static_cast<T *>(&list->sentinel)) return NULL;
     el = &current->next;
     return current;
   }
@@ -810,7 +801,7 @@ class base_ilist_iterator {
 template <class T>
 class I_List : private base_ilist<T> {
  public:
-  using base_ilist<T>::clear;
+  using base_ilist<T>::empty;
   using base_ilist<T>::is_empty;
   using base_ilist<T>::get;
   using base_ilist<T>::push_front;
@@ -829,6 +820,13 @@ class I_List_iterator : public base_ilist_iterator<T> {
   inline T *operator++(int) { return base_ilist_iterator<T>::next(); }
 };
 
+void free_list(I_List<i_string_pair> *list);
 void free_list(I_List<i_string> *list);
 
+template <class T>
+List<T> *List_merge(T *head, List<T> *tail) {
+  tail->push_front(head);
+  return tail;
+}
+
 #endif  // INCLUDES_MYSQL_SQL_LIST_H
