@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,9 +39,9 @@
 #include "my_loglevel.h"
 #include "my_systime.h"
 #include "my_thread.h"  // my_thread_t
-#include "mysql/components/services/bits/mysql_cond_bits.h"
-#include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/components/services/mysql_cond_bits.h"
+#include "mysql/components/services/mysql_mutex_bits.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/psi/mysql_thread.h"
@@ -60,7 +59,7 @@ mysql_mutex_t LOCK_manager;
 mysql_cond_t COND_manager;
 
 extern "C" {
-static void *handle_manager(void *arg [[maybe_unused]]) {
+static void *handle_manager(void *arg MY_ATTRIBUTE((unused))) {
   int error = 0;
   struct timespec abstime;
   bool reset_flush_time = true;
@@ -69,7 +68,7 @@ static void *handle_manager(void *arg [[maybe_unused]]) {
     DBUG_TRACE;
 
     manager_thread = my_thread_self();
-    manager_thread_in_use = true;
+    manager_thread_in_use = 1;
 
     for (;;) {
       mysql_mutex_lock(&LOCK_manager);
@@ -96,10 +95,10 @@ static void *handle_manager(void *arg [[maybe_unused]]) {
         reset_flush_time = true;
       }
     }
-    manager_thread_in_use = false;
+    manager_thread_in_use = 0;
   }  // Can't use DBUG_RETURN after my_thread_end
   my_thread_end();
-  return (nullptr);
+  return (NULL);
 }
 }  // extern "C"
 
@@ -110,9 +109,8 @@ void start_handle_manager() {
   if (flush_time && flush_time != ~(ulong)0L) {
     my_thread_handle hThread;
     int error;
-    if ((error =
-             mysql_thread_create(key_thread_handle_manager, &hThread,
-                                 &connection_attrib, handle_manager, nullptr)))
+    if ((error = mysql_thread_create(key_thread_handle_manager, &hThread,
+                                     &connection_attrib, handle_manager, 0)))
       LogErr(WARNING_LEVEL, ER_CANT_CREATE_HANDLE_MGR_THREAD, error);
   }
 }
