@@ -1,16 +1,15 @@
-/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,9 +42,6 @@
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"            // check_access
 #include "sql/dd/cache/dictionary_client.h"  // dd::cache::Dictionary_client
-#include "sql/dd/properties.h"               // dd::Properties
-#include "sql/dd/sdi_api.h"                  // dd::sdi:drop_all
-#include "sql/dd/types/table.h"              // dd::Table
 #include "sql/dd_table_share.h"              // open_table_def
 #include "sql/debug_sync.h"                  // DEBUG_SYNC
 #include "sql/handler.h"
@@ -75,10 +71,10 @@ class Table;
 bool Sql_cmd_alter_table_exchange_partition::execute(THD *thd) {
   /* Moved from mysql_execute_command */
   LEX *lex = thd->lex;
-  /* first Query_block (have special meaning for many of non-SELECTcommands) */
-  Query_block *query_block = lex->query_block;
-  /* first table of first Query_block */
-  Table_ref *first_table = query_block->get_table_list();
+  /* first SELECT_LEX (have special meaning for many of non-SELECTcommands) */
+  SELECT_LEX *select_lex = lex->select_lex;
+  /* first table of first SELECT_LEX */
+  TABLE_LIST *first_table = select_lex->table_list.first;
   /*
     Code in mysql_alter_table() may modify its HA_CREATE_INFO argument,
     so we have to use a copy of this structure to make execution
@@ -96,21 +92,21 @@ bool Sql_cmd_alter_table_exchange_partition::execute(THD *thd) {
     return true;
 
   /* also check the table to be exchanged with the partition */
-  assert(alter_info.flags & Alter_info::ALTER_EXCHANGE_PARTITION);
+  DBUG_ASSERT(alter_info.flags & Alter_info::ALTER_EXCHANGE_PARTITION);
 
   if (check_access(thd, priv_needed, first_table->db,
                    &first_table->grant.privilege,
-                   &first_table->grant.m_internal, false, false) ||
+                   &first_table->grant.m_internal, 0, 0) ||
       check_access(thd, priv_needed, first_table->next_local->db,
                    &first_table->next_local->grant.privilege,
-                   &first_table->next_local->grant.m_internal, false, false))
+                   &first_table->next_local->grant.m_internal, 0, 0))
     return true;
 
   if (check_grant(thd, priv_needed, first_table, false, UINT_MAX, false))
     return true;
 
   /* Not allowed with EXCHANGE PARTITION */
-  assert(!create_info.data_file_name && !create_info.index_file_name);
+  DBUG_ASSERT(!create_info.data_file_name && !create_info.index_file_name);
 
   thd->enable_slow_log = opt_log_slow_admin_statements;
   return exchange_partition(thd, first_table, &alter_info);
@@ -198,7 +194,7 @@ static bool compare_table_with_partition(THD *thd, TABLE *table,
       return true;
     }
     // Should not happen, we know the table exists and can be opened.
-    assert(part_table_def != nullptr);
+    DBUG_ASSERT(part_table_def != nullptr);
   }
 
   bool metadata_equal = false;
@@ -245,7 +241,7 @@ static bool compare_table_with_partition(THD *thd, TABLE *table,
     ha_myisam compares pointers to verify that DATA/INDEX DIRECTORY is
     the same, so any table using data/index_file_name will fail.
   */
-  if (mysql_compare_tables(thd, table, &part_alter_info, &part_create_info,
+  if (mysql_compare_tables(table, &part_alter_info, &part_create_info,
                            &metadata_equal)) {
     my_error(ER_TABLES_DIFFERENT_METADATA, MYF(0));
     return true;
@@ -256,8 +252,8 @@ static bool compare_table_with_partition(THD *thd, TABLE *table,
     my_error(ER_TABLES_DIFFERENT_METADATA, MYF(0));
     return true;
   }
-  assert(table->s->db_create_options == part_table->s->db_create_options);
-  assert(table->s->db_options_in_use == part_table->s->db_options_in_use);
+  DBUG_ASSERT(table->s->db_create_options == part_table->s->db_create_options);
+  DBUG_ASSERT(table->s->db_options_in_use == part_table->s->db_options_in_use);
 
   if (table_create_info.avg_row_length != part_create_info.avg_row_length) {
     my_error(ER_PARTITION_EXCHANGE_DIFFERENT_OPTION, MYF(0), "AVG_ROW_LENGTH");
@@ -308,16 +304,21 @@ static bool compare_table_with_partition(THD *thd, TABLE *table,
   @note This is a DDL operation so triggers will not be used.
 */
 bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
-    THD *thd, Table_ref *table_list, Alter_info *alter_info) {
+    THD *thd, TABLE_LIST *table_list, Alter_info *alter_info) {
   TABLE *part_table, *swap_table;
-  Table_ref *swap_table_list;
+  TABLE_LIST *swap_table_list;
   partition_element *part_elem;
   String *partition_name;
+  char temp_name[FN_REFLEN + 1];
+  char part_file_name[FN_REFLEN + 1];
+  char swap_file_name[FN_REFLEN + 1];
+  char temp_file_name[FN_REFLEN + 1];
   uint swap_part_id;
+  size_t part_file_name_len;
   Alter_table_prelocking_strategy alter_prelocking_strategy;
   uint table_counter;
   DBUG_TRACE;
-  assert(alter_info->flags & Alter_info::ALTER_EXCHANGE_PARTITION);
+  DBUG_ASSERT(alter_info->flags & Alter_info::ALTER_EXCHANGE_PARTITION);
 
   /* Don't allow to exchange with log table */
   swap_table_list = table_list->next_local;
@@ -362,15 +363,29 @@ bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
 
   THD_STAGE_INFO(thd, stage_verifying_table);
 
+  /* Will append the partition name later in part_info->get_part_elem() */
+  part_file_name_len =
+      build_table_filename(part_file_name, sizeof(part_file_name),
+                           table_list->db, table_list->table_name, "", 0);
+  build_table_filename(swap_file_name, sizeof(swap_file_name),
+                       swap_table_list->db, swap_table_list->table_name, "", 0);
+  /* create a unique temp name #sqlx-nnnn_nnnn, x for eXchange */
+  snprintf(temp_name, sizeof(temp_name), "%sx-%lx_%x", tmp_file_prefix,
+           current_pid, thd->thread_id());
+  if (lower_case_table_names) my_casedn_str(files_charset_info, temp_name);
+  build_table_filename(temp_file_name, sizeof(temp_file_name),
+                       table_list->next_local->db, temp_name, "", FN_IS_TMP);
+
   if (!(part_elem = part_table->part_info->get_part_elem(
-            partition_name->c_ptr(), &swap_part_id))) {
+            partition_name->c_ptr(), part_file_name + part_file_name_len,
+            &swap_part_id))) {
     my_error(ER_UNKNOWN_PARTITION, MYF(0), partition_name->c_ptr(),
              part_table->alias);
     return true;
   }
 
   if (swap_part_id == NOT_A_PARTITION_ID) {
-    assert(part_table->part_info->is_sub_partitioned());
+    DBUG_ASSERT(part_table->part_info->is_sub_partitioned());
     my_error(ER_PARTITION_INSTEAD_OF_SUBPARTITION, MYF(0));
     return true;
   }
@@ -392,7 +407,7 @@ bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
   /* OK to exchange */
 
   /*
-    Get exclusive mdl lock on both tables, always the non partitioned table
+    Get exclusive mdl lock on both tables, alway the non partitioned table
     first. Remember the tickets for downgrading locks later.
   */
   auto downgrade_mdl_lambda = [thd](MDL_ticket *ticket) {
@@ -436,28 +451,20 @@ bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
     return true;
 
   /* Tables were successfully opened above. */
-  assert(part_table_def != nullptr && swap_table_def != nullptr);
-
-  if (part_table_def->options().exists("secondary_engine") ||
-      swap_table_def->options().exists("secondary_engine")) {
-    /* Exchange operation is not allowed for tables with secondary engine
-     * since it's not currently supported by change propagation
-     */
-    my_error(ER_SECONDARY_ENGINE_DDL, MYF(0));
-    return true;
-  }
+  DBUG_ASSERT(part_table_def != nullptr && swap_table_def != nullptr);
 
   DEBUG_SYNC(thd, "swap_partition_before_exchange");
 
-  int ha_error = part_handler->exchange_partition(swap_part_id, part_table_def,
-                                                  swap_table_def);
+  int ha_error = part_handler->exchange_partition(
+      part_file_name, swap_file_name, swap_part_id, part_table_def,
+      swap_table_def);
 
   if (ha_error) {
     handlerton *hton = part_table->file->ht;
     part_table->file->print_error(ha_error, MYF(0));
     // Close TABLE instances which marked as old earlier.
-    close_all_tables_for_name(thd, swap_table->s, false, nullptr);
-    close_all_tables_for_name(thd, part_table->s, false, nullptr);
+    close_all_tables_for_name(thd, swap_table->s, false, NULL);
+    close_all_tables_for_name(thd, part_table->s, false, NULL);
     /*
       Rollback all possible changes to data-dictionary and SE which
       Partition_handler::exchange_partitions() might have done before
@@ -476,44 +483,41 @@ bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
     (void)thd->locked_tables_list.reopen_tables(thd);
     return true;
   } else {
-    bool drop_sdi_res = dd::sdi::drop_all_for_table(thd, swap_table_def) ||
-                        dd::sdi::drop_all_for_table(thd, part_table_def);
-
     if (part_table->file->ht->flags & HTON_SUPPORTS_ATOMIC_DDL) {
       handlerton *hton = part_table->file->ht;
 
       // Close TABLE instances which marked as old earlier.
-      close_all_tables_for_name(thd, swap_table->s, false, nullptr);
-      close_all_tables_for_name(thd, part_table->s, false, nullptr);
+      close_all_tables_for_name(thd, swap_table->s, false, NULL);
+      close_all_tables_for_name(thd, part_table->s, false, NULL);
 
       /*
         Ensure that we call post-DDL hook and re-open tables even
         in case of error.
       */
-      auto rollback_post_ddl_reopen_lambda = [hton](THD *thd_arg) {
+      auto rollback_post_ddl_reopen_lambda = [hton](THD *thd) {
         /*
           Rollback all possible changes to data-dictionary and SE which
           Partition_handler::exchange_partitions() might have done before
           reporting an error. Do this before we downgrade metadata locks.
         */
-        (void)trans_rollback_stmt(thd_arg);
+        (void)trans_rollback_stmt(thd);
         /*
           Full rollback in case we have THD::transaction_rollback_request
           and to synchronize DD state in cache and on disk (as statement
           rollback doesn't clear DD cache of modified uncommitted objects).
         */
-        (void)trans_rollback(thd_arg);
+        (void)trans_rollback(thd);
         /*
           Call SE post DDL hook. This handles both rollback and commit cases.
         */
-        if (hton->post_ddl) hton->post_ddl(thd_arg);
-        (void)thd_arg->locked_tables_list.reopen_tables(thd_arg);
+        if (hton->post_ddl) hton->post_ddl(thd);
+        (void)thd->locked_tables_list.reopen_tables(thd);
       };
 
       std::unique_ptr<THD, decltype(rollback_post_ddl_reopen_lambda)>
           rollback_post_ddl_reopen_guard(thd, rollback_post_ddl_reopen_lambda);
 
-      if (drop_sdi_res || thd->dd_client()->update(part_table_def) ||
+      if (thd->dd_client()->update(part_table_def) ||
           thd->dd_client()->update(swap_table_def) ||
           write_bin_log(thd, true, thd->query().str, thd->query().length,
                         true)) {
@@ -527,12 +531,11 @@ bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
         tables. Ignore the fact that the statement might fail due to binlog
         write failure.
       */
-      close_all_tables_for_name(thd, swap_table->s, false, nullptr);
-      close_all_tables_for_name(thd, part_table->s, false, nullptr);
+      close_all_tables_for_name(thd, swap_table->s, false, NULL);
+      close_all_tables_for_name(thd, part_table->s, false, NULL);
       (void)thd->locked_tables_list.reopen_tables(thd);
 
-      if (drop_sdi_res ||
-          write_bin_log(thd, true, thd->query().str, thd->query().length))
+      if (write_bin_log(thd, true, thd->query().str, thd->query().length))
         return true;
     }
   }
@@ -545,7 +548,7 @@ bool Sql_cmd_alter_table_exchange_partition::exchange_partition(
 bool Sql_cmd_alter_table_analyze_partition::execute(THD *thd) {
   bool res;
   DBUG_TRACE;
-  assert(m_alter_info->flags & Alter_info::ALTER_ADMIN_PARTITION);
+  DBUG_ASSERT(m_alter_info->flags & Alter_info::ALTER_ADMIN_PARTITION);
 
   res = Sql_cmd_analyze_table::execute(thd);
 
@@ -555,7 +558,7 @@ bool Sql_cmd_alter_table_analyze_partition::execute(THD *thd) {
 bool Sql_cmd_alter_table_check_partition::execute(THD *thd) {
   bool res;
   DBUG_TRACE;
-  assert(m_alter_info->flags & Alter_info::ALTER_ADMIN_PARTITION);
+  DBUG_ASSERT(m_alter_info->flags & Alter_info::ALTER_ADMIN_PARTITION);
 
   res = Sql_cmd_check_table::execute(thd);
 
@@ -565,7 +568,7 @@ bool Sql_cmd_alter_table_check_partition::execute(THD *thd) {
 bool Sql_cmd_alter_table_optimize_partition::execute(THD *thd) {
   bool res;
   DBUG_TRACE;
-  assert(m_alter_info->flags & Alter_info::ALTER_ADMIN_PARTITION);
+  DBUG_ASSERT(m_alter_info->flags & Alter_info::ALTER_ADMIN_PARTITION);
 
   res = Sql_cmd_optimize_table::execute(thd);
 
@@ -575,7 +578,7 @@ bool Sql_cmd_alter_table_optimize_partition::execute(THD *thd) {
 bool Sql_cmd_alter_table_repair_partition::execute(THD *thd) {
   bool res;
   DBUG_TRACE;
-  assert(m_alter_info->flags & Alter_info::ALTER_ADMIN_PARTITION);
+  DBUG_ASSERT(m_alter_info->flags & Alter_info::ALTER_ADMIN_PARTITION);
 
   res = Sql_cmd_repair_table::execute(thd);
 
@@ -585,15 +588,15 @@ bool Sql_cmd_alter_table_repair_partition::execute(THD *thd) {
 bool Sql_cmd_alter_table_truncate_partition::execute(THD *thd) {
   int error;
   ulong timeout = thd->variables.lock_wait_timeout;
-  Table_ref *first_table = thd->lex->query_block->get_table_list();
+  TABLE_LIST *first_table = thd->lex->select_lex->table_list.first;
   uint table_counter;
   Partition_handler *part_handler = nullptr;
   handlerton *hton;
   DBUG_TRACE;
-  assert((m_alter_info->flags & (Alter_info::ALTER_ADMIN_PARTITION |
-                                 Alter_info::ALTER_TRUNCATE_PARTITION)) ==
-         (Alter_info::ALTER_ADMIN_PARTITION |
-          Alter_info::ALTER_TRUNCATE_PARTITION));
+  DBUG_ASSERT((m_alter_info->flags & (Alter_info::ALTER_ADMIN_PARTITION |
+                                      Alter_info::ALTER_TRUNCATE_PARTITION)) ==
+              (Alter_info::ALTER_ADMIN_PARTITION |
+               Alter_info::ALTER_TRUNCATE_PARTITION));
 
   /* Fix the lock types (not the same as ordinary ALTER TABLE). */
   first_table->set_lock({TL_WRITE, THR_DEFAULT});
@@ -655,23 +658,7 @@ bool Sql_cmd_alter_table_truncate_partition::execute(THD *thd) {
     return true;
 
   /* Table was successfully opened above. */
-  assert(table_def != nullptr);
-
-  Table_ddl_hton_notification_guard notification_guard{
-      thd, &first_table->mdl_request.key, ha_ddl_type::HA_TRUNCATE_DDL};
-
-  if (notification_guard.notify()) return true;
-
-  if (table_def->options().exists("secondary_engine")) {
-    LEX_CSTRING secondary_engine;
-    table_def->options().get("secondary_engine", &secondary_engine,
-                             thd->mem_root);
-
-    if (!ha_secondary_engine_supports_ddl(thd, secondary_engine)) {
-      my_error(ER_SECONDARY_ENGINE_DDL, MYF(0));
-      return true;
-    }
-  }
+  DBUG_ASSERT(table_def != nullptr);
 
   if (hton->partition_flags() & HA_TRUNCATE_PARTITION_PRECLOSE) {
     /*
@@ -706,8 +693,8 @@ bool Sql_cmd_alter_table_truncate_partition::execute(THD *thd) {
     init_tmp_table_share(thd, &share, first_table->db, 0,
                          first_table->table_name, path, nullptr);
 
-    auto free_share_lambda = [](TABLE_SHARE *share_arg) {
-      free_table_share(share_arg);
+    auto free_share_lambda = [](TABLE_SHARE *share) {
+      free_table_share(share);
     };
     std::unique_ptr<TABLE_SHARE, decltype(free_share_lambda)> free_share_guard(
         &share, free_share_lambda);
@@ -724,7 +711,7 @@ bool Sql_cmd_alter_table_truncate_partition::execute(THD *thd) {
                                     &table, true, nullptr);
 
       if (!error) {
-        auto closefrm_lambda = [](TABLE *t) { (void)closefrm(t, false); };
+        auto closefrm_lambda = [](TABLE *table) { (void)closefrm(table, 0); };
         std::unique_ptr<TABLE, decltype(closefrm_lambda)> closefrm_guard(
             &table, closefrm_lambda);
 
@@ -732,7 +719,7 @@ bool Sql_cmd_alter_table_truncate_partition::execute(THD *thd) {
 
         part_handler = table.file->get_partition_handler();
         // We succeeded obtaining Partition_hanlder earlier.
-        assert(part_handler != nullptr);
+        DBUG_ASSERT(part_handler != nullptr);
 
         if (!error && (error = part_handler->truncate_partition(table_def))) {
           table.file->print_error(error, MYF(0));
@@ -754,7 +741,7 @@ bool Sql_cmd_alter_table_truncate_partition::execute(THD *thd) {
       Since we about to update table definition in the data-dictionary below
       we need to remove its TABLE/TABLE_SHARE from TDC now.
     */
-    close_all_tables_for_name(thd, first_table->table->s, false, nullptr);
+    close_all_tables_for_name(thd, first_table->table->s, false, NULL);
   }
 
   if (hton->flags & HTON_SUPPORTS_ATOMIC_DDL) {
