@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,6 +34,7 @@
 #include "mysqld_error.h"               // ER_SPECIFIC_ACCESS_DENIED_ERROR
 #include "sql/auth/auth_acls.h"         // SHUTDOWN_ACL
 #include "sql/auth/sql_security_ctx.h"  // Security_context
+#include "sql/log.h"                    // Query_logger
 #include "sql/sql_class.h"              // THD
 #include "sql/sql_lex.h"                // LEX
 
@@ -71,7 +71,7 @@ bool Sql_cmd_restart_server::execute(THD *thd) {
   if (check_restart_server_admin_privilege(thd)) return true;
 
   // RESTART shall not be binlogged.
-  thd->lex->no_write_to_binlog = true;
+  thd->lex->no_write_to_binlog = 1;
 
   query_logger.general_log_print(thd, COM_QUERY, NullS);
 
