@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +25,7 @@
 
 #include <set>
 #include "my_sqlcommand.h"
-#include "sql/table.h"
+#include "table.h"
 
 /* Forward declarations */
 class THD;
@@ -42,16 +41,14 @@ enum class Consumer_type {
 };
 
 /**
-  An interface to wrap the parameters required by specific Rewriter.
+  An interface to wrap the paramters required by specific Rewriter.
   Parameters required by specific Rewriter must be added in the concrete
   implementation.
   Clients need to wrap the parameters in specific concrete object.
-  The Rewrite_params objects are not expected to change and are passed around as
-  const objects.
 */
 class Rewrite_params {
  protected:
-  virtual ~Rewrite_params() = default;
+  virtual ~Rewrite_params() {}
 };
 
 /**
@@ -70,15 +67,12 @@ class User_params : public Rewrite_params {
 */
 class Show_user_params : public Rewrite_params {
  public:
-  Show_user_params(bool hide_password_hash, bool print_identified_with_as_hex,
-                   String *param_metadata_str)
+  Show_user_params(bool hide_password_hash, bool print_identified_with_as_hex)
       : Rewrite_params(),
         hide_password_hash(hide_password_hash),
-        print_identified_with_as_hex_(print_identified_with_as_hex),
-        metadata_str(param_metadata_str) {}
+        print_identified_with_as_hex_(print_identified_with_as_hex) {}
   bool hide_password_hash;
   bool print_identified_with_as_hex_;
-  String *metadata_str;
 };
 
 /**
@@ -101,14 +95,13 @@ class Grant_params : public Rewrite_params {
   or clears it if no rewriting took place.
 */
 void mysql_rewrite_query(THD *thd, Consumer_type type = Consumer_type::TEXTLOG,
-                         const Rewrite_params *params = nullptr);
+                         Rewrite_params *params = nullptr);
 /**
   Provides the default interface to rewrite the ACL query.
-  If do_ps_instrument, it sets the thd->rewritten_query with
-  a rewritten query.
+  It sets the thd->rewritten_query with a rewritten query.
 */
-void mysql_rewrite_acl_query(THD *thd, String &rlb, Consumer_type type,
-                             const Rewrite_params *params = nullptr,
+void mysql_rewrite_acl_query(THD *thd, Consumer_type type,
+                             Rewrite_params *params = nullptr,
                              bool do_ps_instrument = true);
 
 /**
@@ -135,7 +128,7 @@ class I_rewriter {
   /* Return the current consumer type */
   Consumer_type consumer_type();
   /* Concrete classes must implement the logic to rewrite query here */
-  virtual bool rewrite(String &rlb) const = 0;
+  virtual bool rewrite() const = 0;
 
  protected:
   THD *const m_thd;
@@ -160,19 +153,14 @@ class Rewriter_user : public I_rewriter {
   virtual void append_auth_str(LEX_USER *lex, String *str) const;
   /* Append the authentication plugin name for the user */
   void append_plugin_name(const LEX_USER *user, String *str) const;
-  /* Append authentication plugin name from LEX_MFA for the user */
-  void append_mfa_plugin_name(const LEX_MFA *user, String *str) const;
-  /* Append the authentication string from LEX_MFA for the user */
-  void append_mfa_auth_str(const LEX_MFA *user, String *str) const;
-
   /*
     Rewrites some of the user specific properties which are common to
     concrete classes.
   */
-  bool rewrite(String &rlb) const override;
+  virtual bool rewrite() const;
   /*
     Abstract method to be implemented by the concrete classes.
-    The implementation methods should add the user authID, plugin info and
+    The implementation methos should add the user authID, plugin info and
     auth str
   */
   virtual void append_user_auth_info(LEX_USER *user, bool comma,
@@ -181,12 +169,6 @@ class Rewriter_user : public I_rewriter {
   virtual void rewrite_password_history(const LEX *lex, String *str) const = 0;
   /* Append the PASSWORD REUSE OPTIONS clause for users */
   virtual void rewrite_password_reuse(const LEX *lex, String *str) const = 0;
-  /* Append the ATTRIBUTE or COMMENT clause for user */
-  virtual void rewrite_user_application_user_metadata(const LEX *lex,
-                                                      String *str) const = 0;
-  /* Use LEX to reconstruct the ATTRIBUTE or COMMENT clauses */
-  void rewrite_in_memory_user_application_user_metadata(const LEX *user,
-                                                        String *str) const;
 
  private:
   /* Append the SSL OPTIONS clause for users */
@@ -199,8 +181,6 @@ class Rewriter_user : public I_rewriter {
   void rewrite_password_expired(const LEX *lex, String *str) const;
   /* Append the PASSWORD REQUIRE CURRENT clause for users */
   void rewrite_password_require_current(LEX *lex, String *str) const;
-  /* Append FAILED_LOGIN_ATTEMPTS/PASSWORD_LOCK_TIME */
-  void rewrite_account_lock_state(LEX *lex, String *str) const;
   /* Append the DEFAULT ROLE OPTIONS clause */
   void rewrite_default_roles(const LEX *lex, String *str) const;
 };
@@ -210,9 +190,7 @@ class Rewriter_create_user final : public Rewriter_user {
 
  public:
   Rewriter_create_user(THD *thd, Consumer_type type);
-  bool rewrite(String &rlb) const override;
-  void rewrite_user_application_user_metadata(const LEX *lex,
-                                              String *str) const override;
+  bool rewrite() const override;
 
  private:
   void append_user_auth_info(LEX_USER *user, bool comma,
@@ -226,9 +204,7 @@ class Rewriter_alter_user final : public Rewriter_user {
 
  public:
   Rewriter_alter_user(THD *thd, Consumer_type type = Consumer_type::TEXTLOG);
-  bool rewrite(String &rlb) const override;
-  void rewrite_user_application_user_metadata(const LEX *lex,
-                                              String *str) const override;
+  bool rewrite() const override;
 
  private:
   void append_user_auth_info(LEX_USER *user, bool comma,
@@ -242,28 +218,25 @@ class Rewriter_show_create_user final : public Rewriter_user {
 
  public:
   Rewriter_show_create_user(THD *thd, Consumer_type type,
-                            const Rewrite_params *params);
-  bool rewrite(String &rlb) const override;
-  void rewrite_user_application_user_metadata(const LEX *lex,
-                                              String *str) const override;
+                            Rewrite_params *params);
+  bool rewrite() const override;
 
  protected:
   /* Append the password hash to the output string */
-  void append_auth_str(LEX_USER *lex, String *str) const override;
+  virtual void append_auth_str(LEX_USER *lex, String *str) const override;
 
  private:
   void append_user_auth_info(LEX_USER *user, bool comma,
                              String *str) const override;
   void rewrite_password_history(const LEX *lex, String *str) const override;
   void rewrite_password_reuse(const LEX *lex, String *str) const override;
-  const Show_user_params *show_params_;
+  Show_user_params *show_params_;
 };
-
 /** Rewrites the SET statement. */
 class Rewriter_set : public I_rewriter {
  public:
   Rewriter_set(THD *thd, Consumer_type type);
-  bool rewrite(String &rlb) const override;
+  bool rewrite() const override;
 };
 /*
   Rewrites the SET PASSWORD statement
@@ -272,9 +245,8 @@ class Rewriter_set_password final : public Rewriter_set {
   using parent = Rewriter_set;
 
  public:
-  Rewriter_set_password(THD *thd, Consumer_type type,
-                        const Rewrite_params *params);
-  bool rewrite(String &rlb) const override;
+  Rewriter_set_password(THD *thd, Consumer_type type, Rewrite_params *params);
+  bool rewrite() const override;
 
  private:
   /* Name of the user whose password has to be changed */
@@ -284,26 +256,26 @@ class Rewriter_set_password final : public Rewriter_set {
 /** Rewrites the GRANT statement. */
 class Rewriter_grant final : public I_rewriter {
  public:
-  Rewriter_grant(THD *thd, Consumer_type type, const Rewrite_params *params);
-  bool rewrite(String &rlb) const override;
+  Rewriter_grant(THD *thd, Consumer_type type, Rewrite_params *params);
+  bool rewrite() const override;
 
  private:
   /* GRANT AS information */
-  const Grant_params *grant_params = nullptr;
+  Grant_params *grant_params = nullptr;
 };
 
-/** Rewrites the CHANGE REPLICATION SOURCE statement. */
-class Rewriter_change_replication_source final : public I_rewriter {
+/** Rewrites the CHANGE MASTER statement. */
+class Rewriter_change_master final : public I_rewriter {
  public:
-  Rewriter_change_replication_source(THD *thd, Consumer_type);
-  bool rewrite(String &rlb) const override;
+  Rewriter_change_master(THD *thd, Consumer_type);
+  bool rewrite() const override;
 };
 
-/** Rewrites the START REPLICA statement. */
-class Rewriter_replica_start final : public I_rewriter {
+/** Rewrites the START SLAVE statement. */
+class Rewriter_slave_start final : public I_rewriter {
  public:
-  Rewriter_replica_start(THD *thd, Consumer_type type);
-  bool rewrite(String &rlb) const override;
+  Rewriter_slave_start(THD *thd, Consumer_type type);
+  bool rewrite() const override;
 };
 /** Base class for SERVER OPTIONS related statement */
 class Rewriter_server_option : public I_rewriter {
@@ -320,7 +292,7 @@ class Rewriter_create_server final : public Rewriter_server_option {
 
  public:
   Rewriter_create_server(THD *thd, Consumer_type type);
-  bool rewrite(String &rlb) const override;
+  bool rewrite() const override;
 };
 /** Rewrites the ALTER SERVER statement. */
 class Rewriter_alter_server final : public Rewriter_server_option {
@@ -328,28 +300,13 @@ class Rewriter_alter_server final : public Rewriter_server_option {
 
  public:
   Rewriter_alter_server(THD *thd, Consumer_type type);
-  bool rewrite(String &rlb) const override;
+  bool rewrite() const override;
 };
 
 /** Rewrites the PREPARE statement.*/
 class Rewriter_prepare final : public I_rewriter {
  public:
   Rewriter_prepare(THD *thd, Consumer_type type);
-  bool rewrite(String &rlb) const override;
+  bool rewrite() const override;
 };
-
-/** Rewrites CLONE statement.*/
-class Rewriter_clone final : public I_rewriter {
- public:
-  Rewriter_clone(THD *thd, Consumer_type type);
-  bool rewrite(String &rlb) const override;
-};
-
-/** Rewrites the START GROUP_REPLICATION command.*/
-class Rewriter_start_group_replication final : public I_rewriter {
- public:
-  Rewriter_start_group_replication(THD *thd, Consumer_type type);
-  bool rewrite(String &rlb) const override;
-};
-
 #endif /* SQL_REWRITE_INCLUDED */
