@@ -1,16 +1,15 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,18 +22,17 @@
 
 #include "sql/sql_show_status.h"
 
+#include <stddef.h>
+
 #include "lex_string.h"
 #include "m_string.h"  // STRING_WITH_LEN
-#include "my_alloc.h"  // operator new
 #include "my_sqlcommand.h"
 #include "sql/item_cmpfunc.h"  // Item_func_like
 #include "sql/mem_root_array.h"
-#include "sql/parse_tree_items.h"      // PTI_simple_ident_ident
-#include "sql/parse_tree_node_base.h"  // Parse_context
-#include "sql/parse_tree_nodes.h"      // PT_select_item_list
-#include "sql/parser_yystype.h"        // Create_col_name_list
-#include "sql/sql_class.h"             // THD
-#include "sql/sql_lex.h"               // Query_options
+#include "sql/parse_tree_items.h"  // PTI_simple_ident_ident
+#include "sql/parse_tree_nodes.h"  // PT_select_item_list
+#include "sql/sql_class.h"         // THD
+#include "sql/sql_lex.h"           // Query_options
 #include "sql/strfunc.h"
 #include "sql_string.h"
 
@@ -84,10 +82,10 @@
               WHERE @<where_clause@>
   </code>
 */
-static Query_block *build_query(const POS &pos, THD *thd,
-                                enum_sql_command command,
-                                const LEX_CSTRING &table_name,
-                                const String *wild, Item *where_cond) {
+static SELECT_LEX *build_query(const POS &pos, THD *thd,
+                               enum_sql_command command,
+                               const LEX_CSTRING &table_name,
+                               const String *wild, Item *where_cond) {
   /*
     MAINTAINER:
     This code builds a parsed tree for a query.
@@ -102,6 +100,8 @@ static Query_block *build_query(const POS &pos, THD *thd,
   static const LEX_CSTRING as_value = {STRING_WITH_LEN("Value")};
   static const LEX_CSTRING pfs = {STRING_WITH_LEN("performance_schema")};
 
+  static const LEX_CSTRING star = {STRING_WITH_LEN("*")};
+
   static const Query_options options = {
       0 /* query_spec_options */
   };
@@ -109,29 +109,29 @@ static Query_block *build_query(const POS &pos, THD *thd,
   /* ... VARIABLE_NAME ... */
   PTI_simple_ident_ident *ident_name;
   ident_name = new (thd->mem_root) PTI_simple_ident_ident(pos, col_name);
-  if (ident_name == nullptr) return nullptr;
+  if (ident_name == NULL) return NULL;
 
   /* ... VARIABLE_NAME as Variable_name ... */
   PTI_expr_with_alias *expr_name;
   expr_name = new (thd->mem_root)
       PTI_expr_with_alias(pos, ident_name, pos.cpp, as_name);
-  if (expr_name == nullptr) return nullptr;
+  if (expr_name == NULL) return NULL;
 
   /* ... VARIABLE_VALUE ... */
   PTI_simple_ident_ident *ident_value;
   ident_value = new (thd->mem_root) PTI_simple_ident_ident(pos, col_value);
-  if (ident_value == nullptr) return nullptr;
+  if (ident_value == NULL) return NULL;
 
   /* ... VARIABLE_VALUE as Value ... */
   PTI_expr_with_alias *expr_value;
   expr_value = new (thd->mem_root)
       PTI_expr_with_alias(pos, ident_value, pos.cpp, as_value);
-  if (expr_value == nullptr) return nullptr;
+  if (expr_value == NULL) return NULL;
 
   /* ... VARIABLE_NAME as Variable_name, VARIABLE_VALUE as Value ... */
   PT_select_item_list *item_list;
   item_list = new (thd->mem_root) PT_select_item_list();
-  if (item_list == nullptr) return nullptr;
+  if (item_list == NULL) return NULL;
   item_list->push_back(expr_name);
   item_list->push_back(expr_value);
 
@@ -143,28 +143,28 @@ static Query_block *build_query(const POS &pos, THD *thd,
   /* ... performance_schema ... */
   LEX_CSTRING tmp_db_name;
   if (lex_string_strmake(thd->mem_root, &tmp_db_name, pfs.str, pfs.length))
-    return nullptr;
+    return NULL;
 
   /* ... <table_name> ... */
   LEX_CSTRING tmp_table_name;
   if (lex_string_strmake(thd->mem_root, &tmp_table_name, table_name.str,
                          table_name.length))
-    return nullptr;
+    return NULL;
 
   /* ... performance_schema.<table_name> ... */
   Table_ident *table_ident;
   table_ident = new (thd->mem_root) Table_ident(tmp_db_name, tmp_table_name);
-  if (table_ident == nullptr) return nullptr;
+  if (table_ident == NULL) return NULL;
 
   /* ... FROM performance_schema.<table_name> ... */
   PT_table_factor_table_ident *table_factor;
   table_factor = new (thd->mem_root)
-      PT_table_factor_table_ident(table_ident, nullptr, NULL_CSTR, nullptr);
-  if (table_factor == nullptr) return nullptr;
+      PT_table_factor_table_ident(table_ident, NULL, NULL_CSTR, NULL);
+  if (table_factor == NULL) return NULL;
 
   Mem_root_array_YY<PT_table_reference *> table_reference_list;
   table_reference_list.init(thd->mem_root);
-  if (table_reference_list.push_back(table_factor)) return nullptr;
+  if (table_reference_list.push_back(table_factor)) return NULL;
 
   /* Form subquery */
   /* SELECT VARIABLE_NAME as Variable_name, VARIABLE_VALUE as Value FROM
@@ -173,69 +173,75 @@ static Query_block *build_query(const POS &pos, THD *thd,
   query_specification =
       new (thd->mem_root) PT_query_specification(options, item_list,
                                                  table_reference_list,  // from
-                                                 nullptr);              // where
-  if (query_specification == nullptr) return nullptr;
+                                                 NULL);                 // where
+  if (query_specification == NULL) return NULL;
+
+  PT_query_expression_body_primary *query_expression_body_primary;
+  query_expression_body_primary =
+      new (thd->mem_root) PT_query_expression_body_primary(query_specification);
+  if (query_expression_body_primary == NULL) return NULL;
 
   PT_query_expression *query_expression;
   query_expression =
-      new (thd->mem_root) PT_query_expression(query_specification);
-  if (query_expression == nullptr) return nullptr;
+      new (thd->mem_root) PT_query_expression(query_expression_body_primary);
+  if (query_expression == NULL) return NULL;
 
   PT_subquery *sub_query;
   sub_query = new (thd->mem_root) PT_subquery(pos, query_expression);
-  if (sub_query == nullptr) return nullptr;
+  if (sub_query == NULL) return NULL;
 
   Create_col_name_list column_names;
   column_names.init(thd->mem_root);
   PT_derived_table *derived_table;
   derived_table = new (thd->mem_root)
       PT_derived_table(false, sub_query, table_name, &column_names);
-  if (derived_table == nullptr) return nullptr;
+  if (derived_table == NULL) return NULL;
 
   Mem_root_array_YY<PT_table_reference *> table_reference_list1;
   table_reference_list1.init(thd->mem_root);
-  if (table_reference_list1.push_back(derived_table)) return nullptr;
+  if (table_reference_list1.push_back(derived_table)) return NULL;
 
   /* SELECT * ... */
-  Item_asterisk *ident_star;
-  ident_star = new (thd->mem_root) Item_asterisk(pos, nullptr, nullptr);
-  if (ident_star == nullptr) return nullptr;
+  PTI_simple_ident_ident *ident_star;
+  ident_star = new (thd->mem_root) PTI_simple_ident_ident(pos, star);
+  if (ident_star == NULL) return NULL;
 
   PT_select_item_list *item_list1;
   item_list1 = new (thd->mem_root) PT_select_item_list();
-  if (item_list1 == nullptr) return nullptr;
+  if (item_list1 == NULL) return NULL;
   item_list1->push_back(ident_star);
 
   /* Process where clause */
-  Item *where_clause = nullptr;
+  Item *where_clause = NULL;
 
-  if (wild != nullptr) {
+  if (wild != NULL) {
     /* ... Variable_name ... */
     PTI_simple_ident_ident *ident_name_where;
     ident_name_where = new (thd->mem_root) PTI_simple_ident_ident(pos, as_name);
-    if (ident_name_where == nullptr) return nullptr;
+    if (ident_name_where == NULL) return NULL;
 
     /* ... <value> ... */
     LEX_STRING *lex_string;
     lex_string = static_cast<LEX_STRING *>(thd->alloc(sizeof(LEX_STRING)));
-    if (lex_string == nullptr) return nullptr;
+    if (lex_string == NULL) return NULL;
     lex_string->length = wild->length();
     lex_string->str = thd->strmake(wild->ptr(), wild->length());
-    if (lex_string->str == nullptr) return nullptr;
+    if (lex_string->str == NULL) return NULL;
 
     PTI_text_literal_text_string *wild_string;
     wild_string = new (thd->mem_root) PTI_text_literal_text_string(
         pos, false, *lex_string);  // TODO WL#6629 check is_7bit
-    if (wild_string == nullptr) return nullptr;
+    if (wild_string == NULL) return NULL;
 
     /* ... Variable_name LIKE <value> ... */
-    Item_func_like *func_like =
-        new (thd->mem_root) Item_func_like(pos, ident_name_where, wild_string);
-    if (func_like == nullptr) return nullptr;
+    Item_func_like *func_like;
+    func_like = new (thd->mem_root)
+        Item_func_like(pos, ident_name_where, wild_string, NULL);
+    if (func_like == NULL) return NULL;
 
     /* ... WHERE Variable_name LIKE <value> ... */
     where_clause = new (thd->mem_root) PTI_where(pos, func_like);
-    if (where_clause == nullptr) return nullptr;
+    if (where_clause == NULL) return NULL;
   } else {
     where_clause = where_cond;
   }
@@ -248,54 +254,58 @@ static Query_block *build_query(const POS &pos, THD *thd,
       new (thd->mem_root) PT_query_specification(options, item_list1,
                                                  table_reference_list1,  // from
                                                  where_clause);  // where
-  if (query_specification2 == nullptr) return nullptr;
+  if (query_specification2 == NULL) return NULL;
+
+  PT_query_expression_body_primary *query_expression_body_primary2;
+  query_expression_body_primary2 = new (thd->mem_root)
+      PT_query_expression_body_primary(query_specification2);
+  if (query_expression_body_primary2 == NULL) return NULL;
 
   PT_query_expression *query_expression2;
   query_expression2 =
-      new (thd->mem_root) PT_query_expression(query_specification2);
-  if (query_expression2 == nullptr) return nullptr;
+      new (thd->mem_root) PT_query_expression(query_expression_body_primary2);
+  if (query_expression2 == NULL) return NULL;
 
   LEX *lex = thd->lex;
-  Query_block *current_query_block = lex->current_query_block();
-  Parse_context pc(thd, current_query_block);
-  if (thd->is_error()) return nullptr;
+  SELECT_LEX *current_select = lex->current_select();
+  Parse_context pc(thd, current_select);
+  if (thd->is_error()) return NULL;
 
   lex->sql_command = SQLCOM_SELECT;
-  if (query_expression2->contextualize(&pc)) return nullptr;
-  if (pc.finalize_query_expression()) return nullptr;
+  if (query_expression2->contextualize(&pc)) return NULL;
+
   /* contextualize sets to COM_SELECT */
   lex->sql_command = command;
 
-  return current_query_block;
+  return current_select;
 }
 
-Query_block *build_show_session_status(const POS &pos, THD *thd,
-                                       const String *wild, Item *where_cond) {
+SELECT_LEX *build_show_session_status(const POS &pos, THD *thd,
+                                      const String *wild, Item *where_cond) {
   static const LEX_CSTRING table_name = {STRING_WITH_LEN("session_status")};
 
   return build_query(pos, thd, SQLCOM_SHOW_STATUS, table_name, wild,
                      where_cond);
 }
 
-Query_block *build_show_global_status(const POS &pos, THD *thd,
-                                      const String *wild, Item *where_cond) {
+SELECT_LEX *build_show_global_status(const POS &pos, THD *thd,
+                                     const String *wild, Item *where_cond) {
   static const LEX_CSTRING table_name = {STRING_WITH_LEN("global_status")};
 
   return build_query(pos, thd, SQLCOM_SHOW_STATUS, table_name, wild,
                      where_cond);
 }
 
-Query_block *build_show_session_variables(const POS &pos, THD *thd,
-                                          const String *wild,
-                                          Item *where_cond) {
+SELECT_LEX *build_show_session_variables(const POS &pos, THD *thd,
+                                         const String *wild, Item *where_cond) {
   static const LEX_CSTRING table_name = {STRING_WITH_LEN("session_variables")};
 
   return build_query(pos, thd, SQLCOM_SHOW_VARIABLES, table_name, wild,
                      where_cond);
 }
 
-Query_block *build_show_global_variables(const POS &pos, THD *thd,
-                                         const String *wild, Item *where_cond) {
+SELECT_LEX *build_show_global_variables(const POS &pos, THD *thd,
+                                        const String *wild, Item *where_cond) {
   static const LEX_CSTRING table_name = {STRING_WITH_LEN("global_variables")};
 
   return build_query(pos, thd, SQLCOM_SHOW_VARIABLES, table_name, wild,
