@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -90,11 +90,12 @@ const char *lock_descriptions[TL_WRITE_ONLY + 1] = {
 
 #ifndef DBUG_OFF
 
-void print_where(Item *cond, const char *info, enum_query_type query_type) {
+void print_where(const THD *thd, const Item *cond, const char *info,
+                 enum_query_type query_type) {
   char buff[256];
   String str(buff, sizeof(buff), system_charset_info);
   str.length(0);
-  if (cond) cond->print(current_thd, &str, query_type);
+  if (cond) cond->print(thd, &str, query_type);
   str.append('\0');
 
   DBUG_LOCK_FILE;
@@ -120,7 +121,7 @@ static void print_cached_tables(void) {
 
 void TEST_join(JOIN *join) {
   uint i, ref;
-  DBUG_ENTER("TEST_join");
+  DBUG_TRACE;
   DBUG_ASSERT(!join->join_tab);
   /*
     Assemble results of all the calls to full_name() first,
@@ -165,7 +166,6 @@ void TEST_join(JOIN *join) {
     }
   }
   DBUG_UNLOCK_FILE;
-  DBUG_VOID_RETURN;
 }
 
 #endif /* !DBUG_OFF */
@@ -311,8 +311,7 @@ static inline int dl_compare(const TABLE_LOCK_INFO *a,
   return 1;
 }
 
-class DL_commpare : public std::binary_function<const TABLE_LOCK_INFO &,
-                                                const TABLE_LOCK_INFO &, bool> {
+class DL_commpare {
  public:
   bool operator()(const TABLE_LOCK_INFO &a, const TABLE_LOCK_INFO &b) {
     return dl_compare(&a, &b) < 0;
@@ -589,7 +588,7 @@ void Dbug_table_list_dumper::dump_one_struct(TABLE_LIST *tbl) {
 
 int Dbug_table_list_dumper::dump_graph(SELECT_LEX *select_lex,
                                        TABLE_LIST *first_leaf) {
-  DBUG_ENTER("Dbug_table_list_dumper::dump_graph");
+  DBUG_TRACE;
   char filename[500];
   int no = 0;
   do {
@@ -604,7 +603,7 @@ int Dbug_table_list_dumper::dump_graph(SELECT_LEX *select_lex,
   /* Ok, found an unoccupied name, create the file */
   if (!(out = fopen(filename, "wt"))) {
     DBUG_PRINT("tree_dump", ("Failed to create output file"));
-    DBUG_RETURN(1);
+    return 1;
   }
 
   DBUG_PRINT("tree_dump", ("dumping tree to %s", filename));
@@ -646,7 +645,7 @@ int Dbug_table_list_dumper::dump_graph(SELECT_LEX *select_lex,
     //    fprintf(out, "%s", current_thd->query);
     fclose(out);
   }
-  DBUG_RETURN(0);
+  return 0;
 }
 
 #endif
