@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -368,12 +368,12 @@ void thd_set_ha_data(MYSQL_THD thd, const struct handlerton *hton,
                      const void *ha_data) {
   plugin_ref *lock = &thd->get_ha_data(hton->slot)->lock;
   if (ha_data && !*lock)
-    *lock = ha_lock_engine(NULL, (handlerton *)hton);
+    *lock = ha_lock_engine(NULL, hton);
   else if (!ha_data && *lock) {
     plugin_unlock(NULL, *lock);
     *lock = NULL;
   }
-  *thd_ha_data(thd, hton) = (void *)ha_data;
+  *thd_ha_data(thd, hton) = const_cast<void *>(ha_data);
 }
 
 long long thd_test_options(const MYSQL_THD thd, long long test_options) {
@@ -492,7 +492,8 @@ char *thd_security_context(MYSQL_THD thd, char *buffer, size_t length,
 }
 
 void thd_get_xid(const MYSQL_THD thd, MYSQL_XID *xid) {
-  *xid = *(MYSQL_XID *)thd->get_transaction()->xid_state()->get_xid();
+  *xid = *pointer_cast<const MYSQL_XID *>(
+      thd->get_transaction()->xid_state()->get_xid());
 }
 
 /**
@@ -542,7 +543,7 @@ int thd_allow_batch(MYSQL_THD thd) {
 }
 
 void thd_mark_transaction_to_rollback(MYSQL_THD thd, int all) {
-  DBUG_ENTER("thd_mark_transaction_to_rollback");
+  DBUG_TRACE;
   DBUG_ASSERT(thd);
   /*
     The parameter "all" has type int since the function is defined
@@ -552,7 +553,6 @@ void thd_mark_transaction_to_rollback(MYSQL_THD thd, int all) {
     specifically.
   */
   thd->mark_transaction_to_rollback((all != 0));
-  DBUG_VOID_RETURN;
 }
 
 //////////////////////////////////////////////////////////
@@ -641,13 +641,11 @@ void thd_wait_end(MYSQL_THD thd) {
    called.
 */
 void thd_report_row_lock_wait(THD *self, THD *wait_for) {
-  DBUG_ENTER("thd_report_row_lock_wait");
+  DBUG_TRACE;
 
   if (self != NULL && wait_for != NULL && is_mts_worker(self) &&
       is_mts_worker(wait_for))
     commit_order_manager_check_deadlock(self, wait_for);
-
-  DBUG_VOID_RETURN;
 }
 
 /**
@@ -655,7 +653,7 @@ void thd_report_row_lock_wait(THD *self, THD *wait_for) {
 */
 
 void remove_ssl_err_thread_state() {
-#if !defined(HAVE_WOLFSSL) && !defined(HAVE_OPENSSL11)
+#if !defined(HAVE_OPENSSL11)
   ERR_remove_thread_state(nullptr);
 #endif
 }
