@@ -63,7 +63,7 @@ struct mysql_mutex_t;
 
 void thd_init(THD *thd, char *stack_start, bool bound MY_ATTRIBUTE((unused)),
               PSI_thread_key psi_key MY_ATTRIBUTE((unused))) {
-  DBUG_ENTER("thd_init");
+  DBUG_TRACE;
   // TODO: Purge threads currently terminate too late for them to be added.
   // Note that P_S interprets all threads with thread_id != 0 as
   // foreground threads. And THDs need thread_id != 0 to be added
@@ -92,7 +92,6 @@ void thd_init(THD *thd, char *stack_start, bool bound MY_ATTRIBUTE((unused)),
   thd_set_thread_stack(thd, stack_start);
 
   thd->store_globals();
-  DBUG_VOID_RETURN;
 }
 
 THD *create_thd(bool enable_plugins, bool background_thread, bool bound,
