@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,9 +24,10 @@
 
 #include "sql/sql_udf.h"
 
+#include "my_config.h"
+
 #include <stdio.h>
 #include <string.h>
-
 #include <iterator>
 #include <memory>
 #include <new>
@@ -40,7 +40,6 @@
 #include "map_helpers.h"
 #include "my_alloc.h"
 #include "my_base.h"
-#include "my_config.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_io.h"
@@ -51,36 +50,33 @@
 #include "my_sys.h"
 #include "my_thread_local.h"
 #include "mysql/components/service_implementation.h"
-#include "mysql/components/services/bits/mysql_rwlock_bits.h"
-#include "mysql/components/services/bits/psi_bits.h"
-#include "mysql/components/services/bits/psi_memory_bits.h"
-#include "mysql/components/services/bits/psi_rwlock_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
+#include "mysql/components/services/mysql_rwlock_bits.h"
+#include "mysql/components/services/psi_memory_bits.h"
+#include "mysql/components/services/psi_rwlock_bits.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_rwlock.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"  // ER_*
-#include "sql/derror.h"    // ER_THD
 #include "sql/field.h"
 #include "sql/handler.h"
-#include "sql/iterators/row_iterator.h"
 #include "sql/mdl.h"
-#include "sql/mysqld.h"     // opt_allow_suspicious_udfs
+#include "sql/mysqld.h"   // opt_allow_suspicious_udfs
+#include "sql/records.h"  // unique_ptr_destroy_only<RowIterator>
+#include "sql/row_iterator.h"
 #include "sql/sql_base.h"   // close_mysql_tables
 #include "sql/sql_class.h"  // THD
 #include "sql/sql_const.h"
-#include "sql/sql_executor.h"            // unique_ptr_destroy_only<RowIterator>
-#include "sql/sql_parse.h"               // check_string_char_length
-#include "sql/sql_plugin.h"              // check_valid_path
-#include "sql/sql_system_table_check.h"  // System_table_intact
-#include "sql/sql_table.h"               // write_bin_log
-#include "sql/table.h"                   // Table_ref
+#include "sql/sql_parse.h"   // check_string_char_length
+#include "sql/sql_plugin.h"  // check_valid_path
+#include "sql/sql_table.h"   // write_bin_log
+#include "sql/table.h"       // TABLE_LIST
 #include "sql/thd_raii.h"
 #include "sql/thr_malloc.h"
 #include "sql/transaction.h"  // trans_*
 #include "thr_lock.h"
-#include "udf_registration_imp.h"
 
 #ifdef HAVE_DLFCN_H
 #include <dlfcn.h>
@@ -103,33 +99,16 @@
   \ref mem and \ref THR_LOCK_udf are always initialized, even in
   --skip-grant-tables mode.
 */
-static bool initialized = false;
+static bool initialized = 0;
 static MEM_ROOT mem;
 static collation_unordered_map<std::string, udf_func *> *udf_hash;
 static mysql_rwlock_t THR_LOCK_udf;
 
-static constexpr const size_t UDF_ALLOC_BLOCK_SIZE{1024};
-
 static udf_func *add_udf(LEX_STRING *name, Item_result ret, char *dl,
                          Item_udftype typ);
 static void udf_hash_delete(udf_func *udf);
 static void *find_udf_dl(const char *dl);
 
-// mysql.func table definition.
-static const int MYSQL_UDF_TABLE_FIELD_COUNT = 4;
-static const TABLE_FIELD_TYPE
-    mysql_udf_table_fields[MYSQL_UDF_TABLE_FIELD_COUNT] = {
-        {{STRING_WITH_LEN("name")},
-         {STRING_WITH_LEN("char(64)")},
-         {nullptr, 0}},
-        {{STRING_WITH_LEN("ret")}, {STRING_WITH_LEN("tinyint")}, {nullptr, 0}},
-        {{STRING_WITH_LEN("dl")}, {STRING_WITH_LEN("char(128)")}, {nullptr, 0}},
-        {{STRING_WITH_LEN("type")},
-         {STRING_WITH_LEN("enum('function','aggregate')")},
-         {STRING_WITH_LEN("utf8mb3")}}};
-static const TABLE_FIELD_DEF mysql_udf_table_def = {MYSQL_UDF_TABLE_FIELD_COUNT,
-                                                    mysql_udf_table_fields};
-
 static char *init_syms(udf_func *tmp, char *nm) {
   char *end;
 
@@ -160,7 +139,7 @@ static char *init_syms(udf_func *tmp, char *nm) {
     if (!opt_allow_suspicious_udfs) return nm;
     LogErr(WARNING_LEVEL, ER_FAILED_TO_FIND_DL_ENTRY, nm);
   }
-  return nullptr;
+  return 0;
 }
 
 static PSI_memory_key key_memory_udf_mem;
@@ -174,7 +153,7 @@ static PSI_rwlock_info all_udf_rwlocks[] = {{&key_rwlock_THR_LOCK_udf,
 
 static PSI_memory_info all_udf_memory[] = {{&key_memory_udf_mem, "udf_mem",
                                             PSI_FLAG_ONLY_GLOBAL_STAT, 0,
-                                            "Shared structure of UDFs."}};
+                                            PSI_DOCUMENT_ME}};
 
 static void init_udf_psi_keys(void) {
   const char *category = "sql";
@@ -191,7 +170,7 @@ static void init_udf_psi_keys(void) {
 /**
   Initialize the UDF global structures.
   This is done as a separate step so that the UDF registration
-  service can work when initializing plugins, which happens
+  service can work when initalizing plugins, which happens
   before reading the UDF table.
 */
 void udf_init_globals() {
@@ -203,7 +182,7 @@ void udf_init_globals() {
 #endif
 
   mysql_rwlock_init(key_rwlock_THR_LOCK_udf, &THR_LOCK_udf);
-  init_sql_alloc(key_memory_udf_mem, &mem, UDF_ALLOC_BLOCK_SIZE);
+  init_sql_alloc(key_memory_udf_mem, &mem, UDF_ALLOC_BLOCK_SIZE, 0);
 
   udf_hash = new collation_unordered_map<std::string, udf_func *>(
       system_charset_info, key_memory_udf_mem);
@@ -223,16 +202,16 @@ void udf_read_functions_table() {
   char db[] = "mysql"; /* A subject to casednstr, can't be constant */
 
   if (initialized) {
-    assert("wrong init order: reading UDFs from the table twice");
+    DBUG_ASSERT("wrong init order: reading UDFs from the table twice");
     return;
   }
 
-  initialized = true;
+  initialized = 1;
 
   THD *new_thd = new (std::nothrow) THD;
   if (new_thd == nullptr) {
     LogErr(ERROR_LEVEL, ER_UDF_CANT_ALLOC_FOR_STRUCTURES);
-    mem.Clear();
+    free_root(&mem, MYF(0));
     delete new_thd;
     return;
   }
@@ -243,7 +222,7 @@ void udf_read_functions_table() {
     new_thd->set_db(db_lex_cstr);
   }
 
-  Table_ref tables(db, "func", TL_READ, MDL_SHARED_READ_ONLY);
+  TABLE_LIST tables(db, "func", TL_READ, MDL_SHARED_READ_ONLY);
 
   if (open_trans_system_tables_for_read(new_thd, &tables)) {
     DBUG_PRINT("error", ("Can't open udf table"));
@@ -252,31 +231,16 @@ void udf_read_functions_table() {
   }
 
   table = tables.table;
-  iterator =
-      init_table_iterator(new_thd, table, /*ignore_not_found_rows=*/false,
-                          /*count_examined_rows=*/false);
+  iterator = init_table_iterator(new_thd, table, NULL, false,
+                                 /*ignore_not_found_rows=*/false);
   if (iterator == nullptr) goto end;
   while (!(error = iterator->Read())) {
     DBUG_PRINT("info", ("init udf record"));
     LEX_STRING name;
     name.str = get_field(&mem, table->field[0]);
-
-    // Check the name.str is NULL or not.
-    if (name.str == nullptr) {
-      sql_print_error("Invalid row in mysql.func table for column 'name'");
-      continue;
-    }
-
     name.length = strlen(name.str);
     char *dl_name = get_field(&mem, table->field[2]);
-
-    if (dl_name == nullptr) {
-      sql_print_error("Invalid row in mysql.func table for function '%.64s'",
-                      name.str);
-      continue;
-    }
-
-    bool new_dl = false;
+    bool new_dl = 0;
     Item_udftype udftype = UDFTYPE_FUNCTION;
     if (table->s->fields >= 4)  // New func table
       udftype = (Item_udftype)table->field[3]->val_int();
@@ -292,7 +256,7 @@ void udf_read_functions_table() {
     LEX_CSTRING name_cstr = {name.str, name.length};
     if (check_valid_path(dl_name, strlen(dl_name)) ||
         check_string_char_length(name_cstr, "", NAME_CHAR_LEN,
-                                 system_charset_info, true)) {
+                                 system_charset_info, 1)) {
       LogErr(ERROR_LEVEL, ER_UDF_INVALID_ROW_IN_FUNCTION_TABLE, name.str);
       continue;
     }
@@ -304,7 +268,7 @@ void udf_read_functions_table() {
     }
 
     void *dl = find_udf_dl(tmp->dl);
-    if (dl == nullptr) {
+    if (dl == NULL) {
       char dlpath[FN_REFLEN];
       strxnmov(dlpath, sizeof(dlpath) - 1, opt_plugin_dir, "/", tmp->dl, NullS);
       (void)unpack_filename(dlpath, dlpath);
@@ -316,10 +280,10 @@ void udf_read_functions_table() {
         // Print warning to log
         LogErr(ERROR_LEVEL, ER_FAILED_TO_OPEN_SHARED_LIBRARY, tmp->dl,
                error_number, errmsg);
-        udf_hash_delete(tmp);
+        // Keep the udf in the hash so that we can remove it later
         continue;
       }
-      new_dl = true;
+      new_dl = 1;
     }
     tmp->dlhandle = dl;
     {
@@ -333,18 +297,17 @@ void udf_read_functions_table() {
   }
   if (error > 0) LogErr(ERROR_LEVEL, ER_UNKNOWN_ERROR_NUMBER, my_errno());
   iterator.reset();
-  table->invalidate_dict();  // Force close to free memory
+  table->m_needs_reopen = true;  // Force close to free memory
 
 end:
-  if (new_thd->is_attachable_ro_transaction_active())
-    close_trans_system_tables(new_thd);
+  close_trans_system_tables(new_thd);
   delete new_thd;
 }
 
 /**
-   Deinitialize the UDF subsystem.
+   Deintialize the UDF subsystem.
 
-   This function closes the shared libraries.
+   This function closes the shared libaries.
 */
 void udf_unload_udfs() {
   DBUG_TRACE;
@@ -357,7 +320,7 @@ void udf_unload_udfs() {
         for (auto it2 = std::next(it1); it2 != udf_hash->end(); ++it2) {
           udf_func *tmp = it2->second;
           if (udf->dlhandle == tmp->dlhandle)
-            tmp->dlhandle = nullptr;  // Already closed
+            tmp->dlhandle = 0;  // Already closed
         }
         dlclose(udf->dlhandle);
       }
@@ -366,7 +329,7 @@ void udf_unload_udfs() {
 }
 
 /**
-   Deinitialize the UDF subsystem.
+   Deintialize the UDF subsystem.
 
    This function does the following:
    1. Free the UDF hash.
@@ -379,8 +342,8 @@ void udf_deinit_globals() {
     delete udf_hash;
     udf_hash = nullptr;
   }
-  mem.Clear();
-  initialized = false;
+  free_root(&mem, MYF(0));
+  initialized = 0;
 
   mysql_rwlock_destroy(&THR_LOCK_udf);
 }
@@ -402,7 +365,7 @@ static void udf_hash_delete(udf_func *udf) {
 
   const auto it = udf_hash->find(to_string(udf->name));
   if (it == udf_hash->end()) {
-    assert(false);
+    DBUG_ASSERT(false);
     return;
   }
 
@@ -412,7 +375,7 @@ static void udf_hash_delete(udf_func *udf) {
   } else {
     /*
       The functions is in use ; Rename the functions instead of removing it.
-      The functions will be automatically removed when the least threads
+      The functions will be automaticly removed when the least threads
       doesn't use it anymore
     */
     udf_hash->erase(it);
@@ -435,8 +398,11 @@ void free_udf(udf_func *udf) {
       while another thread still was using the udf
     */
     const auto it = udf_hash->find(to_string(udf->name));
-    if (it != udf_hash->end()) udf_hash->erase(it);
-
+    if (it == udf_hash->end()) {
+      DBUG_ASSERT(false);
+      return;
+    }
+    udf_hash->erase(it);
     using_udf_functions = !udf_hash->empty();
     if (udf->dlhandle && !find_udf_dl(udf->dl)) dlclose(udf->dlhandle);
   }
@@ -446,10 +412,10 @@ void free_udf(udf_func *udf) {
 /* This is only called if using_udf_functions != 0 */
 
 udf_func *find_udf(const char *name, size_t length, bool mark_used) {
-  udf_func *udf = nullptr;
+  udf_func *udf = 0;
   DBUG_TRACE;
 
-  if (!initialized) return nullptr;
+  if (!initialized) return NULL;
 
   /* TODO: This should be changed to reader locks someday! */
   if (mark_used)
@@ -471,17 +437,17 @@ udf_func *find_udf(const char *name, size_t length, bool mark_used) {
 static void *find_udf_dl(const char *dl) {
   DBUG_TRACE;
 
-  if (!dl) return nullptr;
+  if (!dl) return 0;
   /*
-    Because only the function name is hashed, we have to search through
+    Because only the function name is hashed, we have to search trough
     all rows to find the dl.
   */
   for (const auto &key_and_value : *udf_hash) {
     udf_func *udf = key_and_value.second;
-    if (udf->dl && !strcmp(dl, udf->dl) && udf->dlhandle != nullptr)
+    if (udf->dl && !strcmp(dl, udf->dl) && udf->dlhandle != NULL)
       return udf->dlhandle;
   }
-  return nullptr;
+  return 0;
 }
 
 /* Assume that name && dl is already allocated */
@@ -503,7 +469,7 @@ static udf_func *add_udf(LEX_STRING *name, Item_result ret, char *dl,
   mysql_rwlock_wrlock(&THR_LOCK_udf);
 
   udf_hash->emplace(to_string(tmp->name), tmp);
-  using_udf_functions = true;
+  using_udf_functions = 1;
 
   mysql_rwlock_unlock(&THR_LOCK_udf);
   return tmp;
@@ -530,7 +496,7 @@ static bool udf_end_transaction(THD *thd, bool rollback, udf_func *udf,
   bool rollback_transaction = thd->transaction_rollback_request || rollback;
   udf_func *u_f = nullptr;
 
-  assert(stmt_causes_implicit_commit(thd, CF_IMPLICIT_COMMIT_END));
+  DBUG_ASSERT(stmt_causes_implicit_commit(thd, CF_IMPLICIT_COMMIT_END));
 
   if (!rollback_transaction && insert_udf) {
     udf->name.str = strdup_root(&mem, udf->name.str);
@@ -553,7 +519,7 @@ static bool udf_end_transaction(THD *thd, bool rollback, udf_func *udf,
     CREATE/DROP UDF operations must acquire IX Backup Lock in order
     to be mutually exclusive with LOCK INSTANCE FOR BACKUP.
   */
-  assert(thd->mdl_context.owns_equal_or_stronger_lock(
+  DBUG_ASSERT(thd->mdl_context.owns_equal_or_stronger_lock(
       MDL_key::BACKUP_LOCK, "", "", MDL_INTENTION_EXCLUSIVE));
 
   /*
@@ -599,16 +565,15 @@ static bool udf_end_transaction(THD *thd, bool rollback, udf_func *udf,
 
   @param thd                 THD context.
   @param udf                 Pointer to UDF function.
-  @param if_not_exists       True if 'IF NOT EXISTS' clause was specified.
 
   @note Like implementations of other DDL/DML in MySQL, this function
   relies on the caller to close the thread tables. This is done in the
   end of dispatch_command().
 */
 
-bool mysql_create_function(THD *thd, udf_func *udf, bool if_not_exists) {
+bool mysql_create_function(THD *thd, udf_func *udf) {
   bool error = true;
-  void *dl = nullptr;
+  void *dl = 0;
   int new_dl = 0;
   TABLE *table;
 
@@ -624,7 +589,7 @@ bool mysql_create_function(THD *thd, udf_func *udf, bool if_not_exists) {
   }
 
   /* must not be dynamically registered */
-  assert(udf->dl);
+  DBUG_ASSERT(udf->dl);
 
   /*
     Ensure that the .dll doesn't have a path
@@ -637,7 +602,7 @@ bool mysql_create_function(THD *thd, udf_func *udf, bool if_not_exists) {
   }
   LEX_CSTRING udf_name_cstr = {udf->name.str, udf->name.length};
   if (check_string_char_length(udf_name_cstr, "", NAME_CHAR_LEN,
-                               system_charset_info, true)) {
+                               system_charset_info, 1)) {
     my_error(ER_TOO_LONG_IDENT, MYF(0), udf->name.str);
     return error;
   }
@@ -646,31 +611,11 @@ bool mysql_create_function(THD *thd, udf_func *udf, bool if_not_exists) {
     Acquire MDL SNRW for TL_WRITE type so that deadlock and
     timeout errors are avoided from the Storage Engine.
   */
-  Table_ref tables("mysql", "func", TL_WRITE, MDL_SHARED_NO_READ_WRITE);
+  TABLE_LIST tables("mysql", "func", TL_WRITE, MDL_SHARED_NO_READ_WRITE);
 
   if (open_and_lock_tables(thd, &tables, MYSQL_LOCK_IGNORE_TIMEOUT))
     return error;
   table = tables.table;
-
-  /*
-    System table mysql.func is supported by only InnoDB engine. Changing system
-    table's engine is not allowed. But to support logical upgrade creating
-    system table is allowed in MyISAM engine. CREATE FUNCTION operation is
-    *not* allowed in this case.
-  */
-  if ((table->file->ht->is_supported_system_table != nullptr) &&
-      !table->file->ht->is_supported_system_table(tables.db, tables.table_name,
-                                                  true)) {
-    my_error(ER_UNSUPPORTED_ENGINE, MYF(0),
-             ha_resolve_storage_engine_name(table->file->ht), tables.db,
-             tables.table_name);
-    return error;
-  }
-
-  // CREATE FUNCTION operation is *not* allowed if table structure is changed.
-  System_table_intact table_intact(thd);
-  if (table_intact.check(thd, table, &mysql_udf_table_def)) return error;
-
   /*
     Turn off row binlogging of this statement and use statement-based
     so that all supporting tables are updated for CREATE FUNCTION command.
@@ -679,19 +624,8 @@ bool mysql_create_function(THD *thd, udf_func *udf, bool if_not_exists) {
 
   mysql_rwlock_rdlock(&THR_LOCK_udf);
   if (udf_hash->count(to_string(udf->name)) != 0) {
+    my_error(ER_UDF_EXISTS, MYF(0), udf->name.str);
     mysql_rwlock_unlock(&THR_LOCK_udf);
-
-    // UDF with the same name already exists
-    if (if_not_exists) {
-      push_warning_printf(thd, Sql_condition::SL_NOTE, ER_UDF_EXISTS,
-                          ER_THD(thd, ER_UDF_EXISTS), udf->name.str);
-      error = (write_bin_log(thd, true, thd->query().str, thd->query().length,
-                             false) != 0);
-      if (error) error = udf_end_transaction(thd, error, nullptr, false);
-
-    } else {
-      my_error(ER_UDF_EXISTS, MYF(0), udf->name.str);
-    }
     return error;
   }
   dl = find_udf_dl(udf->dl);
@@ -783,30 +717,11 @@ bool mysql_drop_function(THD *thd, const LEX_STRING *udf_name) {
     return error;
   }
 
-  Table_ref tables("mysql", "func", TL_WRITE, MDL_SHARED_NO_READ_WRITE);
+  TABLE_LIST tables("mysql", "func", TL_WRITE, MDL_SHARED_NO_READ_WRITE);
 
   if (open_and_lock_tables(thd, &tables, MYSQL_LOCK_IGNORE_TIMEOUT))
     return error;
   table = tables.table;
-
-  /*
-    System table mysql.func is supported by only InnoDB engine. Changing system
-    table's engine is not allowed. But to support logical upgrade creating
-    system table is allowed in MyISAM engine. DROP and CREATE FUNCTION
-    operations are not allowed in this case.
-  */
-  if (!table->file->ht->is_supported_system_table(tables.db, tables.table_name,
-                                                  true)) {
-    my_error(ER_UNSUPPORTED_ENGINE, MYF(0),
-             ha_resolve_storage_engine_name(table->file->ht), tables.db,
-             tables.table_name);
-    return error;
-  }
-
-  // DROP FUNCTION operation is *not* allowed if table structure is changed.
-  System_table_intact table_intact(thd);
-  if (table_intact.check(thd, table, &mysql_udf_table_def)) return error;
-
   /*
     Turn off row binlogging of this statement and use statement-based
     so that all supporting tables are updated for DROP FUNCTION command.
@@ -832,8 +747,8 @@ bool mysql_drop_function(THD *thd, const LEX_STRING *udf_name) {
   table->use_all_columns();
   table->field[0]->store(udf->name.str, udf->name.length, &my_charset_bin);
   if (!table->file->ha_index_read_idx_map(table->record[0], 0,
-                                          table->field[0]->field_ptr(),
-                                          HA_WHOLE_KEY, HA_READ_KEY_EXACT)) {
+                                          table->field[0]->ptr, HA_WHOLE_KEY,
+                                          HA_READ_KEY_EXACT)) {
     int delete_err;
     if ((delete_err = table->file->ha_delete_row(table->record[0])))
       table->file->print_error(delete_err, MYF(0));
@@ -859,17 +774,19 @@ bool mysql_drop_function(THD *thd, const LEX_STRING *udf_name) {
   return error;
 }
 
+#include "sql/udf_registration_imp.h"
+
 bool mysql_udf_registration_imp::udf_register_inner(udf_func *ufunc) {
   mysql_rwlock_wrlock(&THR_LOCK_udf);
 
-  assert(ufunc->dl == nullptr);
-  assert(ufunc->dlhandle == nullptr);
+  DBUG_ASSERT(ufunc->dl == NULL);
+  DBUG_ASSERT(ufunc->dlhandle == NULL);
 
   auto res = udf_hash->emplace(to_string(ufunc->name), ufunc);
   if (!res.second)
     ufunc = nullptr;
   else
-    using_udf_functions = true;
+    using_udf_functions = 1;
 
   mysql_rwlock_unlock(&THR_LOCK_udf);
   return ufunc == nullptr;
@@ -883,7 +800,7 @@ udf_func *mysql_udf_registration_imp::alloc_udf(const char *name,
   udf_func *ufunc;
 
   ufunc = (udf_func *)mem.Alloc(sizeof(udf_func));
-  if (!ufunc) return nullptr;
+  if (!ufunc) return NULL;
   memset(ufunc, 0, sizeof(udf_func));
   ufunc->name.str = strdup_root(&mem, name);
   ufunc->name.length = strlen(name);
@@ -932,7 +849,7 @@ DEFINE_BOOL_METHOD(mysql_udf_registration_imp::udf_register_aggregate,
 
 DEFINE_BOOL_METHOD(mysql_udf_registration_imp::udf_unregister,
                    (const char *name, int *was_present)) {
-  udf_func *udf = nullptr;
+  udf_func *udf = NULL;
 
   if (was_present) *was_present = 0;
   mysql_rwlock_wrlock(&THR_LOCK_udf);
@@ -943,16 +860,15 @@ DEFINE_BOOL_METHOD(mysql_udf_registration_imp::udf_unregister,
     udf = it->second;
 
     if (!udf->dl && !udf->dlhandle &&  // Not registered via CREATE FUNCTION
-        (udf->usage_count == 1))       // Not used
+        !--udf->usage_count)           // Not used
     {
-      --udf->usage_count;
       udf_hash->erase(it);
       using_udf_functions = !udf_hash->empty();
     } else  // error
-      udf = nullptr;
+      udf = NULL;
   }
   mysql_rwlock_unlock(&THR_LOCK_udf);
-  return udf != nullptr ? false : true;
+  return udf != NULL ? false : true;
 }
 
 void udf_hash_rlock(void) { mysql_rwlock_rdlock(&THR_LOCK_udf); }
