@@ -46,7 +46,9 @@
 #include "my_sys.h"
 #include "mysql/udf_registration_types.h"
 #include "mysqld_error.h"
+#include "scope_guard.h"
 #include "sql/auth/auth_acls.h"
+#include "sql/basic_row_iterators.h"
 #include "sql/current_thd.h"
 #include "sql/debug_sync.h"     // DEBUG_SYNC
 #include "sql/error_handler.h"  // Strict_error_handler
@@ -58,9 +60,12 @@
 #include "sql/mem_root_array.h"
 #include "sql/opt_explain.h"  // explain_no_table
 #include "sql/opt_explain_format.h"
+#include "sql/opt_trace.h"
 #include "sql/opt_trace_context.h"
 #include "sql/parse_tree_node_base.h"
 #include "sql/parse_tree_nodes.h"  // PT_with_clause
+#include "sql/pfs_batch_mode.h"
+#include "sql/protocol.h"
 #include "sql/query_options.h"
 #include "sql/set_var.h"
 #include "sql/sql_base.h"  // fill_record
@@ -74,16 +79,21 @@
 #include "sql/sql_tmp_table.h"   // tmp tables
 #include "sql/table_function.h"  // Table_function
 #include "sql/thr_malloc.h"
+#include "sql/timing_iterator.h"
 #include "sql/window.h"  // Window
 #include "template_utils.h"
 
+using std::move;
+using std::vector;
+
 bool Query_result_union::prepare(THD *, List<Item> &, SELECT_LEX_UNIT *u) {
   unit = u;
   return false;
 }
 
 bool Query_result_union::send_data(THD *thd, List<Item> &values) {
-  if (fill_record(thd, table, table->visible_field_ptr(), values, NULL, NULL))
+  if (fill_record(thd, table, table->visible_field_ptr(), values, NULL, NULL,
+                  false))
     return true; /* purecov: inspected */
 
   if (!check_unique_constraint(table)) return false;
@@ -110,7 +120,8 @@ bool Query_result_union::send_eof(THD *) { return false; }
 bool Query_result_union::flush() { return false; }
 
 /**
-  Create a temporary table to store the result of Query_result_union.
+  Create a temporary table to store the result of a query expression
+  (used, among others, when materializing a UNION DISTINCT).
 
   @param thd_arg            thread handle
   @param column_types       a list of items used to define columns of the
@@ -150,22 +161,24 @@ bool Query_result_union::create_result_table(
       tmp_table_param.can_use_pk_for_unique = false;
     }
     if (unit->mixed_union_operators()) {
-      /*
-        Generally, UNIQUE key can be promoted to PK, saving the space
-        consumption of a hidden PK. However, if the query mixes UNION ALL and
-        UNION DISTINCT, the PK will be disabled at some point in execution,
-        which InnoDB doesn't support as it uses a clustered PK. Then, no PK:
-      */
-      tmp_table_param.can_use_pk_for_unique = false;
+      // If we have mixed UNION DISTINCT / UNION ALL, we can't use an unique
+      // index to deduplicate, as we need to be able to turn off deduplication
+      // checking when we get to the UNION ALL part. The handler supports
+      // turning off indexes (and the pre-iterator executor used this to
+      // implement mixed DISTINCT/ALL), but not selectively, and we might very
+      // well need the other indexes when querying against the table.
+      // (Also, it would be nice to be able to remove this functionality
+      // altogether from the handler.) Thus, we do it manually instead.
+      tmp_table_param.force_hash_field_for_unique = true;
     }
   }
 
   if (!(table = create_tmp_table(thd_arg, &tmp_table_param, *column_types, NULL,
                                  is_union_distinct, true, options, HA_POS_ERROR,
-                                 (char *)table_alias)))
+                                 table_alias)))
     return true;
   if (create_table) {
-    table->file->extra(HA_EXTRA_IGNORE_DUP_KEY);
+    table->file->ha_extra(HA_EXTRA_IGNORE_DUP_KEY);
     if (table->hash_field) table->file->ha_index_init(0, 0);
   }
   return false;
@@ -313,7 +326,7 @@ bool Query_result_union_direct::send_data(THD *thd, List<Item> &items) {
     return false;
   }
 
-  if (fill_record(thd, table, table->field, items, NULL, NULL))
+  if (fill_record(thd, table, table->field, items, NULL, NULL, false))
     return true; /* purecov: inspected */
 
   return result->send_data(thd, unit->item_list);
@@ -379,7 +392,7 @@ class Change_current_select {
 */
 
 bool SELECT_LEX_UNIT::prepare_fake_select_lex(THD *thd_arg) {
-  DBUG_ENTER("SELECT_LEX_UNIT::prepare_fake_select_lex");
+  DBUG_TRACE;
 
   DBUG_ASSERT(thd_arg->lex->current_select() == fake_select_lex);
 
@@ -423,18 +436,33 @@ bool SELECT_LEX_UNIT::prepare_fake_select_lex(THD *thd_arg) {
               fake_select_lex->where_cond() == NULL &&
               fake_select_lex->having_cond() == NULL);
 
-  if (is_recursive()) {
-    /*
-      The fake_select_lex's JOIN is going to read result_table_list
-      repeatedly, so this table has all the attributes of a recursive
-      reference:
-    */
-    result_table_list.set_recursive_reference();
+  if (fake_select_lex->prepare(thd_arg)) return true;
+
+  return false;
+}
+
+bool SELECT_LEX_UNIT::can_materialize_directly_into_result(THD *thd) const {
+  // There's no point in doing this if we're not already trying to materialize.
+  if (!is_union()) {
+    return false;
   }
 
-  if (fake_select_lex->prepare(thd_arg)) DBUG_RETURN(true);
+  // For now, we don't accept LIMIT or OFFSET; this restriction could probably
+  // be lifted fairly easily in the future.
+  if (global_parameters()->get_offset(thd) != 0 ||
+      global_parameters()->get_limit(thd) != HA_POS_ERROR) {
+    return false;
+  }
 
-  DBUG_RETURN(false);
+  // We can only do this in an all-iterator world. (This mirrors
+  // create_iterators().)
+  if (!all_query_blocks_use_iterator_executor()) {
+    return false;
+  }
+
+  // We can't materialize directly into the result if we have sorting.
+  // Otherwise, we're fine.
+  return global_parameters()->order_list.elements == 0;
 }
 
 /**
@@ -452,7 +480,7 @@ bool SELECT_LEX_UNIT::prepare_fake_select_lex(THD *thd_arg) {
 bool SELECT_LEX_UNIT::prepare(THD *thd, Query_result *sel_result,
                               ulonglong added_options,
                               ulonglong removed_options) {
-  DBUG_ENTER("SELECT_LEX_UNIT::prepare");
+  DBUG_TRACE;
 
   DBUG_ASSERT(!is_prepared());
   Change_current_select save_select(thd);
@@ -489,31 +517,32 @@ bool SELECT_LEX_UNIT::prepare(THD *thd, Query_result *sel_result,
         goto err; /* purecov: inspected */
       instantiate_tmp_table = true;
     }
+
+    if (fake_select_lex != NULL) {
+      /*
+        There exists a query block that consolidates the UNION result.
+        Prepare the active options for this query block. If these options
+        contain OPTION_BUFFER_RESULT, the query block will perform a buffering
+        operation, which means that an underlying query block does not need to
+        buffer its result, and the buffer option for the underlying query blocks
+        can be cleared.
+        For subqueries in form "a IN (SELECT .. UNION SELECT ..):
+        when optimizing the fake_select_lex that reads the results of the union
+        from a temporary table, do not mark the temp. table as constant because
+        the contents in it may vary from one subquery execution to another, by
+        adding OPTION_NO_CONST_TABLES.
+      */
+      fake_select_lex->make_active_options(
+          (added_options & (OPTION_FOUND_ROWS | OPTION_BUFFER_RESULT)) |
+              OPTION_NO_CONST_TABLES | SELECT_NO_UNLOCK,
+          0);
+      added_options &= ~OPTION_BUFFER_RESULT;
+    }
   } else {
     // Only one query block, and no "fake" object: No extra result needed:
     tmp_result = sel_result;
   }
 
-  if (fake_select_lex != NULL) {
-    /*
-      There exists a query block that consolidates the UNION result.
-      Prepare the active options for this query block. If these options
-      contain OPTION_BUFFER_RESULT, the query block will perform a buffering
-      operation, which means that an underlying query block does not need to
-      buffer its result, and the buffer option for the underlying query blocks
-      can be cleared.
-      For subqueries in form "a IN (SELECT .. UNION SELECT ..):
-      when optimizing the fake_select_lex that reads the results of the union
-      from a temporary table, do not mark the temp. table as constant because
-      the contents in it may vary from one subquery execution to another, by
-      adding OPTION_NO_CONST_TABLES.
-    */
-    fake_select_lex->make_active_options(
-        (added_options & (OPTION_FOUND_ROWS | OPTION_BUFFER_RESULT)) |
-            OPTION_NO_CONST_TABLES | SELECT_NO_UNLOCK,
-        0);
-    added_options &= ~OPTION_BUFFER_RESULT;
-  }
   first_select()->context.resolve_in_select_list = true;
 
   for (SELECT_LEX *sl = first_select(); sl; sl = sl->next_select()) {
@@ -659,8 +688,8 @@ bool SELECT_LEX_UNIT::prepare(THD *thd, Query_result *sel_result,
             instantiate_tmp_table))
       goto err;
     result_table_list = TABLE_LIST();
-    result_table_list.db = (char *)"";
-    result_table_list.table_name = result_table_list.alias = (char *)"union";
+    result_table_list.db = "";
+    result_table_list.table_name = result_table_list.alias = "union";
     result_table_list.table = table = union_result->table;
     table->pos_in_table_list = &result_table_list;
     result_table_list.select_lex =
@@ -689,35 +718,30 @@ bool SELECT_LEX_UNIT::prepare(THD *thd, Query_result *sel_result,
   // Query blocks are prepared, update the state
   set_prepared();
 
-  DBUG_RETURN(false);
+  return false;
 
 err:
   (void)cleanup(thd, false);
-  DBUG_RETURN(true);
+  return true;
 }
 
-/**
-  Optimize all query blocks of a query expression, including fake_select_lex
-
-  @param thd    thread handler
-
-  @returns false if optimization successful, true if error
-*/
-
-bool SELECT_LEX_UNIT::optimize(THD *thd) {
-  DBUG_ENTER("SELECT_LEX_UNIT::optimize");
+bool SELECT_LEX_UNIT::optimize(THD *thd, TABLE *materialize_destination) {
+  DBUG_TRACE;
 
   DBUG_ASSERT(is_prepared() && !is_optimized());
 
   Change_current_select save_select(thd);
 
+  ha_rows estimated_rowcount = 0;
+  double estimated_cost = 0.0;
+
   for (SELECT_LEX *sl = first_select(); sl; sl = sl->next_select()) {
     thd->lex->set_current_select(sl);
 
     // LIMIT is required for optimization
-    if (set_limit(thd, sl)) DBUG_RETURN(true); /* purecov: inspected */
+    if (set_limit(thd, sl)) return true; /* purecov: inspected */
 
-    if (sl->optimize(thd)) DBUG_RETURN(true);
+    if (sl->optimize(thd)) return true;
 
     /*
       Accumulate estimated number of rows.
@@ -725,17 +749,28 @@ bool SELECT_LEX_UNIT::optimize(THD *thd) {
          rows).
       2. If GROUP BY clause is optimized away because it was a constant then
          query produces at most one row.
-    */
-    if (query_result()) {
-      query_result()->estimated_rowcount +=
-          sl->is_implicitly_grouped() || sl->join->group_optimized_away
-              ? 1
-              : sl->join->best_rowcount;
-      query_result()->estimated_cost += sl->join->best_read;
+     */
+    estimated_rowcount +=
+        sl->is_implicitly_grouped() || sl->join->group_optimized_away
+            ? 1
+            : sl->join->best_rowcount;
+    estimated_cost += sl->join->best_read;
+
+    // TABLE_LIST::fetch_number_of_rows() expects to get the number of rows
+    // from all earlier query blocks from the query result, so we need to update
+    // it as we go. In particular, this is used when optimizing a recursive
+    // SELECT in a CTE, so that it knows how many rows the non-recursive query
+    // blocks will produce.
+    //
+    // TODO(sgunders): Communicate this in a different way when the query result
+    // goes away.
+    if (query_result() != nullptr) {
+      query_result()->estimated_rowcount = estimated_rowcount;
+      query_result()->estimated_cost = estimated_cost;
     }
   }
-  if ((uncacheable & UNCACHEABLE_DEPENDENT) && query_result() &&
-      query_result()->estimated_rowcount <= 1) {
+
+  if ((uncacheable & UNCACHEABLE_DEPENDENT) && estimated_rowcount <= 1) {
     /*
       This depends on outer references, so optimization cannot assume that all
       executions will always produce the same row. So, increase the counter to
@@ -743,14 +778,13 @@ bool SELECT_LEX_UNIT::optimize(THD *thd) {
       Not testing all bits of "uncacheable", as if derived table sets user
       vars (UNCACHEABLE_SIDEEFFECT) the logic above doesn't apply.
     */
-    query_result()->estimated_rowcount = PLACEHOLDER_TABLE_ROW_ESTIMATE;
+    estimated_rowcount = PLACEHOLDER_TABLE_ROW_ESTIMATE;
   }
 
   if (fake_select_lex) {
     thd->lex->set_current_select(fake_select_lex);
 
-    if (set_limit(thd, fake_select_lex))
-      DBUG_RETURN(true); /* purecov: inspected */
+    if (set_limit(thd, fake_select_lex)) return true; /* purecov: inspected */
 
     /*
       In EXPLAIN command, constant subqueries that do not use any
@@ -768,10 +802,240 @@ bool SELECT_LEX_UNIT::optimize(THD *thd) {
                 fake_select_lex->where_cond() == NULL &&
                 fake_select_lex->having_cond() == NULL);
 
-    if (fake_select_lex->optimize(thd)) DBUG_RETURN(true);
+    if (fake_select_lex->optimize(thd)) return true;
+  }
+
+  query_result()->estimated_rowcount = estimated_rowcount;
+  query_result()->estimated_cost = estimated_cost;
+
+  // If the caller has asked for materialization directly into a table of its
+  // own, and we can do so, do an unfinished materialization (see the comment
+  // on this function for more details).
+  if (thd->lex->m_sql_cmd != nullptr &&
+      thd->lex->m_sql_cmd->using_secondary_storage_engine()) {
+    // Not supported when using secondary storage engine.
+    create_iterators(thd);
+  } else if (estimated_rowcount <= 1) {
+    // Don't do it for const tables, as for those, optimize_derived() wants to
+    // run the query during optimization, and thus needs an iterator.
+    //
+    // Do note that JOIN::extract_func_dependent_tables() can want to read from
+    // the derived table during the optimization phase even if it has
+    // estimated_rowcount larger than one (e.g., because it understands it can
+    // get only one row due to a unique index), but will detect that the table
+    // has not been created, and treat the the lookup as non-const.
+    create_iterators(thd);
+  } else if (materialize_destination != nullptr &&
+             can_materialize_directly_into_result(thd)) {
+    m_query_blocks_to_materialize = setup_materialization(
+        thd, materialize_destination, /*union_distinct_only=*/false);
+  } else {
+    // Recursive CTEs expect to see the rows in the result table immediately
+    // after writing them.
+    DBUG_ASSERT(!is_recursive());
+    create_iterators(thd);
+  }
+
+  if (false) {
+    // This can be useful during debugging.
+    fprintf(stderr, "Query plan:\n%s\n",
+            PrintQueryPlan(0, m_root_iterator.get()).c_str());
   }
   set_optimized();  // All query blocks optimized, update the state
-  DBUG_RETURN(false);
+  return false;
+}
+
+Mem_root_array<MaterializeIterator::QueryBlock>
+SELECT_LEX_UNIT::setup_materialization(THD *thd, TABLE *dst_table,
+                                       bool union_distinct_only) {
+  Mem_root_array<MaterializeIterator::QueryBlock> query_blocks(thd->mem_root);
+
+  bool activate_deduplication = (union_distinct != nullptr);
+  for (SELECT_LEX *select = first_select(); select != nullptr;
+       select =
+           select->next_select()) {  // Termination condition at end of loop.
+    JOIN *join = select->join;
+    MaterializeIterator::QueryBlock query_block;
+    DBUG_ASSERT(join && join->is_optimized());
+    DBUG_ASSERT(join->root_iterator() != nullptr);
+    ConvertItemsToCopy(join->fields, dst_table->visible_field_ptr(),
+                       &join->tmp_table_param, join);
+
+    query_block.subquery_iterator = join->release_root_iterator();
+    query_block.select_number = select->select_number;
+    query_block.join = join;
+    if (mixed_union_operators() && !activate_deduplication) {
+      query_block.disable_deduplication_by_hash_field = true;
+    }
+    // See the class comment on AggregateIterator.
+    query_block.copy_fields_and_items =
+        !join->streaming_aggregation ||
+        join->tmp_table_param.precomputed_group_by;
+    query_block.temp_table_param = &join->tmp_table_param;
+    query_block.is_recursive_reference = select->recursive_reference;
+
+    if (query_block.is_recursive_reference) {
+      // Find the recursive reference to ourselves; there should be exactly one,
+      // as per the standard.
+      for (unsigned table_idx = 0; table_idx < join->tables; ++table_idx) {
+        QEP_TAB *qep_tab = &join->qep_tab[table_idx];
+        if (qep_tab->recursive_iterator != nullptr) {
+          DBUG_ASSERT(query_block.recursive_reader == nullptr);
+          query_block.recursive_reader = qep_tab->recursive_iterator;
+#ifndef DBUG_OFF
+          break;
+#endif
+        }
+      }
+      if (query_block.recursive_reader == nullptr) {
+        // The recursive reference was optimized away, e.g. due to an impossible
+        // WHERE condition, so we're not a recursive reference after all.
+        query_block.is_recursive_reference = false;
+      }
+    }
+
+    query_blocks.push_back(move(query_block));
+
+    if (select == union_distinct) {
+      // Last query block that is part of a UNION DISTINCT.
+      activate_deduplication = false;
+      if (union_distinct_only) {
+        // The rest will be done by appending.
+        break;
+      }
+    }
+  }
+  return query_blocks;
+}
+
+void SELECT_LEX_UNIT::create_iterators(THD *thd) {
+  if (is_simple()) {
+    JOIN *join = first_select()->join;
+    DBUG_ASSERT(join && join->is_optimized());
+    m_root_iterator = join->release_root_iterator();
+    return;
+  }
+
+  if (!all_query_blocks_use_iterator_executor()) {
+    return;
+  }
+
+  // Decide whether we can stream rows, ie., never actually put them into the
+  // temporary table. If we can, we materialize the UNION DISTINCT blocks first,
+  // and then stream the remaining UNION ALL blocks (if any) by means of
+  // AppendIterator.
+  //
+  // If we cannot stream (ie., everything has to go into the temporary table),
+  // our strategy for mixed UNION ALL/DISTINCT becomes a bit different;
+  // see MaterializeIterator for details.
+  bool streaming_allowed = true;
+  if (global_parameters()->order_list.size() != 0) {
+    // If we're sorting, we currently put it in a real table no matter what.
+    // This is a legacy decision, because we used to not know whether filesort
+    // would want to refer to rows in the table after the sort (sort by row ID).
+    // We could probably be more intelligent here now.
+    streaming_allowed = false;
+  } else if ((thd->lex->sql_command == SQLCOM_INSERT_SELECT ||
+              thd->lex->sql_command == SQLCOM_REPLACE_SELECT) &&
+             thd->lex->unit == this) {
+    // If we're doing an INSERT or REPLACE, and we're not outputting to
+    // a temporary table already (ie., we are the topmost unit), then we
+    // don't want to insert any records before we're done scanning. Otherwise,
+    // we would risk incorrect results and/or infinite loops, as we'd be seeing
+    // our own records as they get inserted.
+    //
+    // @todo Figure out if we can check for OPTION_BUFFER_RESULT instead;
+    //       see bug #23022426.
+    streaming_allowed = false;
+  }
+
+  TABLE *tmp_table = union_result->table;
+  tmp_table->alias = "<union temporary>";
+
+  ha_rows offset = global_parameters()->get_offset(thd);
+  ha_rows limit = global_parameters()->get_limit(thd);
+  if (limit + offset >= limit)
+    limit += offset;
+  else
+    limit = HA_POS_ERROR; /* purecov: inspected */
+  const bool calc_found_rows =
+      (first_select()->active_options() & OPTION_FOUND_ROWS);
+
+  vector<unique_ptr_destroy_only<RowIterator>> union_all_sub_iterators;
+
+  // If streaming is allowed, we can do all the parts that are UNION ALL by
+  // streaming; the rest have to go to the table.
+  //
+  // Handle the query blocks that we need to materialize. This may be
+  // UNION DISTINCT query blocks only, or all blocks.
+  if (union_distinct != nullptr || !streaming_allowed) {
+    Mem_root_array<MaterializeIterator::QueryBlock> query_blocks =
+        setup_materialization(thd, tmp_table, streaming_allowed);
+
+    unique_ptr_destroy_only<RowIterator> table_iterator;
+    if (fake_select_lex != nullptr) {
+      table_iterator = fake_select_lex->join->release_root_iterator();
+    } else {
+      table_iterator =
+          NewIterator<TableScanIterator>(thd, tmp_table, nullptr, nullptr);
+    }
+    bool push_limit_down =
+        global_parameters()->order_list.size() == 0 && !calc_found_rows;
+    union_all_sub_iterators.emplace_back(NewIterator<MaterializeIterator>(
+        thd, move(query_blocks), tmp_table, move(table_iterator),
+        /*cte=*/nullptr, /*unit=*/nullptr, /*join=*/nullptr,
+        /*ref_slice=*/-1,
+        /*rematerialize=*/true, push_limit_down ? limit : HA_POS_ERROR));
+  }
+
+  if (streaming_allowed) {
+    SELECT_LEX *first_union_all = (union_distinct == nullptr)
+                                      ? first_select()
+                                      : union_distinct->next_select();
+    for (SELECT_LEX *select = first_union_all; select != nullptr;
+         select = select->next_select()) {
+      JOIN *join = select->join;
+      DBUG_ASSERT(join && join->is_optimized());
+      ConvertItemsToCopy(join->fields, tmp_table->visible_field_ptr(),
+                         &join->tmp_table_param, join);
+      bool copy_fields_and_items = !join->streaming_aggregation ||
+                                   join->tmp_table_param.precomputed_group_by;
+      union_all_sub_iterators.emplace_back(NewIterator<StreamingIterator>(
+          thd, join->release_root_iterator(), &join->tmp_table_param, tmp_table,
+          copy_fields_and_items));
+    }
+  }
+
+  DBUG_ASSERT(!union_all_sub_iterators.empty());
+  if (union_all_sub_iterators.size() == 1) {
+    m_root_iterator = move(union_all_sub_iterators[0]);
+  } else {
+    // Just append all the UNION ALL sub-blocks.
+    DBUG_ASSERT(streaming_allowed);
+    m_root_iterator =
+        NewIterator<AppendIterator>(thd, move(union_all_sub_iterators));
+  }
+
+  // NOTE: If there's a fake_select_lex, its JOIN's iterator already handles
+  // LIMIT/OFFSET, so we don't do it again here.
+  if ((limit != HA_POS_ERROR || offset != 0) && fake_select_lex == nullptr) {
+    m_root_iterator = NewIterator<LimitOffsetIterator>(
+        thd, move(m_root_iterator), limit, offset, calc_found_rows,
+        &send_records);
+  }
+}
+
+bool SELECT_LEX_UNIT::all_query_blocks_use_iterator_executor() const {
+  // Note that select->join() can be nullptr at this point, if we're
+  // called before optimize(). Thus, we can give a false negative.
+  for (SELECT_LEX *select = first_select(); select != nullptr;
+       select = select->next_select()) {
+    if (select->join != nullptr && select->join->root_iterator() == nullptr) {
+      // No support yet.
+      return false;
+    }
+  }
+  return true;
 }
 
 /**
@@ -784,7 +1048,7 @@ bool SELECT_LEX_UNIT::optimize(THD *thd) {
 */
 
 bool SELECT_LEX_UNIT::explain(THD *explain_thd, const THD *query_thd) {
-  DBUG_ENTER("SELECT_LEX_UNIT::explain");
+  DBUG_TRACE;
 
 #ifndef DBUG_OFF
   SELECT_LEX *lex_select_save = query_thd->lex->current_select();
@@ -798,13 +1062,13 @@ bool SELECT_LEX_UNIT::explain(THD *explain_thd, const THD *query_thd) {
               outer_select()->join == nullptr ||
               outer_select()->join->zero_result_cause);
 
-  if (fmt->begin_context(CTX_UNION)) DBUG_RETURN(true);
+  if (fmt->begin_context(CTX_UNION)) return true;
 
   for (SELECT_LEX *sl = first_select(); sl; sl = sl->next_select()) {
-    if (fmt->begin_context(CTX_QUERY_SPEC)) DBUG_RETURN(true);
+    if (fmt->begin_context(CTX_QUERY_SPEC)) return true;
     if (explain_query_specification(explain_thd, query_thd, sl, CTX_JOIN) ||
         fmt->end_context(CTX_QUERY_SPEC))
-      DBUG_RETURN(true);
+      return true;
   }
 
   if (fake_select_lex != NULL) {
@@ -815,16 +1079,21 @@ bool SELECT_LEX_UNIT::explain(THD *explain_thd, const THD *query_thd) {
   if (!other)
     DBUG_ASSERT(current_thd->lex->current_select() == lex_select_save);
 
-  if (ret) DBUG_RETURN(true);
+  if (ret) return true;
   fmt->end_context(CTX_UNION);
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
-   Empties all correlated CTEs defined in the unit's WITH clause.
-*/
-bool SELECT_LEX_UNIT::clear_corr_ctes() {
+  Empties all correlated query blocks defined within the query expression;
+  that is, correlated CTEs defined in the expression's WITH clause, and
+  correlated derived tables.
+ */
+bool SELECT_LEX_UNIT::clear_correlated_query_blocks() {
+  for (SELECT_LEX *sl = first_select(); sl; sl = sl->next_select()) {
+    sl->join->clear_corr_derived_tmp_tables();
+  }
   if (!m_with_clause) return false;
   for (auto el : m_with_clause->m_list->elements()) {
     Common_table_expr &cte = el->m_postparse;
@@ -1125,6 +1394,143 @@ class Recursive_executor {
   }
 };
 
+bool SELECT_LEX_UNIT::ExecuteIteratorQuery(THD *thd) {
+  THD_STAGE_INFO(thd, stage_executing);
+  DEBUG_SYNC(thd, "before_join_exec");
+
+  Opt_trace_context *const trace = &thd->opt_trace;
+  Opt_trace_object trace_wrapper(trace);
+  Opt_trace_object trace_exec(trace, "join_execution");
+  if (is_simple()) {
+    trace_exec.add_select_number(first_select()->select_number);
+  }
+  Opt_trace_array trace_steps(trace, "steps");
+
+  if (is_executed()) {
+    if (clear_correlated_query_blocks()) return true;
+
+    // TODO(sgunders): Most of JOIN::reset() should be done in iterators.
+    for (SELECT_LEX *sl = first_select(); sl; sl = sl->next_select()) {
+      if (sl->join->is_executed()) {
+        thd->lex->set_current_select(sl);
+        sl->join->reset();
+      }
+      if (fake_select_lex != nullptr) {
+        thd->lex->set_current_select(fake_select_lex);
+        fake_select_lex->join->reset();
+      }
+    }
+  }
+
+  for (SELECT_LEX *select_lex = first_select(); select_lex;
+       select_lex = select_lex->next_select()) {
+    JOIN *join = select_lex->join;
+    select_lex->join->examined_rows = 0;
+    select_lex->join
+        ->set_executed();  // The dynamic range optimizer expects this.
+
+    // TODO(sgunders): Consider doing this in some iterator instead.
+    if (join->m_windows.elements > 0 && !join->m_windowing_steps) {
+      // Initialize state of window functions as end_write_wf() will be shortcut
+      for (Window &w : select_lex->join->m_windows) {
+        w.reset_all_wf_state();
+      }
+    }
+  }
+
+  List<Item> *fields = get_field_list();
+  Query_result *query_result = this->query_result();
+  DBUG_ASSERT(query_result != nullptr);
+
+  if (query_result->start_execution(thd)) return true;
+
+  if (query_result->send_result_set_metadata(
+          thd, *fields, Protocol::SEND_NUM_ROWS | Protocol::SEND_EOF)) {
+    return true;
+  }
+
+  set_executed();
+
+  if (item) {
+    item->reset_value_registration();
+
+    if (item->assigned()) {
+      item->assigned(false);  // Prepare for re-execution of this unit
+      item->reset();
+    }
+  }
+
+  // We need to accumulate in the first join's send_records as long as
+  // we support SQL_CALC_FOUND_ROWS, since LimitOffsetIterator will use it
+  // for reporting rows skipped by OFFSET or LIMIT. When we get rid of
+  // SQL_CALC_FOUND_ROWS, we can use a local variable here instead.
+  ha_rows *send_records_ptr;
+  if (fake_select_lex != nullptr) {
+    // UNION with LIMIT: found_rows() applies to the outermost block.
+    // LimitOffsetIterator will write skipped OFFSET rows into the
+    // fake_select_lex's send_records, so use that.
+    send_records_ptr = &fake_select_lex->join->send_records;
+  } else if (is_simple()) {
+    // Not an UNION: found_rows() applies to the join.
+    // LimitOffsetIterator will write skipped OFFSET rows into the JOIN's
+    // send_records, so use that.
+    send_records_ptr = &first_select()->join->send_records;
+  } else {
+    // UNION, but without a fake_select_lex (may or may not have a
+    // LIMIT): found_rows() applies to the outermost block. See
+    // SELECT_LEX_UNIT::send_records for more information.
+    send_records_ptr = &send_records;
+  }
+  *send_records_ptr = 0;
+
+  thd->get_stmt_da()->reset_current_row_for_condition();
+  if (m_root_iterator->Init()) {
+    return true;
+  }
+
+  {
+    PFSBatchMode pfs_batch_mode(m_root_iterator.get());
+    auto join_cleanup = create_scope_guard([this, thd] {
+      for (SELECT_LEX *sl = first_select(); sl; sl = sl->next_select()) {
+        JOIN *join = sl->join;
+        join->join_free();
+        thd->inc_examined_row_count(join->examined_rows);
+      }
+      if (fake_select_lex != nullptr) {
+        thd->inc_examined_row_count(fake_select_lex->join->examined_rows);
+      }
+    });
+
+    for (;;) {
+      int error = m_root_iterator->Read();
+      DBUG_EXECUTE_IF("bug13822652_1", thd->killed = THD::KILL_QUERY;);
+
+      if (error > 0 || thd->is_error())  // Fatal error
+        return true;
+      else if (error < 0)
+        break;
+      else if (thd->killed)  // Aborted by user
+      {
+        thd->send_kill_message();
+        return true;
+      }
+
+      ++*send_records_ptr;
+      if (query_result->send_data(thd, *fields)) {
+        return true;
+      }
+      thd->get_stmt_da()->inc_current_row_for_condition();
+    }
+
+    // NOTE: join_cleanup must be done before we send EOF, so that we get the
+    // row counts right.
+  }
+
+  thd->current_found_rows = *send_records_ptr;
+
+  return query_result->send_eof(thd);
+}
+
 /**
   Execute a query expression that may be a UNION and/or have an ordered result.
 
@@ -1134,10 +1540,31 @@ class Recursive_executor {
 */
 
 bool SELECT_LEX_UNIT::execute(THD *thd) {
-  DBUG_ENTER("SELECT_LEX_UNIT::exec");
+  DBUG_TRACE;
   DBUG_ASSERT(is_optimized());
 
-  if (is_executed() && !uncacheable) DBUG_RETURN(false);
+  if (is_executed() && !uncacheable) return false;
+
+  if (unfinished_materialization()) {
+    // We were asked to materialize directly into a parent query block's
+    // temporary table (which only works in the iterator executor),
+    // but we're called from the non-iterator executor. (A typical case is
+    // where the parent query ended up using BKA, which we don't know when
+    // setting up derived tables.) This means we'll need some last-ditch
+    // execution strategy; we solve this by putting the iterators back
+    // where we found them and create iterators for normal (non-direct)
+    // derived table materialization. These iterators will generate our
+    // output (possibly by use of MaterializeIterator), which goes into the
+    // derived table by means of Query_result_union.
+    for (MaterializeIterator::QueryBlock &query_block :
+         m_query_blocks_to_materialize) {
+      query_block.join->set_root_iterator(move(query_block.subquery_iterator));
+      query_block.temp_table_param->items_to_copy = nullptr;
+    }
+    m_query_blocks_to_materialize.init_empty_const();
+    create_iterators(thd);
+    DBUG_ASSERT(m_root_iterator != nullptr);
+  }
 
   /*
     Even if we return "true" the statement might continue
@@ -1146,8 +1573,12 @@ bool SELECT_LEX_UNIT::execute(THD *thd) {
   */
   Change_current_select save_select(thd);
 
+  if (m_root_iterator != nullptr) {
+    return ExecuteIteratorQuery(thd);
+  }
+
   if (is_executed()) {
-    if (clear_corr_ctes()) return true;
+    if (clear_correlated_query_blocks()) return true;
     for (SELECT_LEX *sl = first_select(); sl; sl = sl->next_select()) {
       if (sl->join->is_executed()) {
         thd->lex->set_current_select(sl);
@@ -1160,10 +1591,10 @@ bool SELECT_LEX_UNIT::execute(THD *thd) {
     }
     if (table && table->is_created())  // reset UNION tmp table
     {
-      if (union_result->reset()) DBUG_RETURN(true); /* purecov: inspected */
+      if (union_result->reset()) return true; /* purecov: inspected */
       table->file->info(HA_STATUS_VARIABLE);
       if (union_distinct && table->file->ha_enable_indexes(HA_KEY_SWITCH_ALL))
-        DBUG_RETURN(true);    /* purecov: inspected */
+        return true;          /* purecov: inspected */
       if (table->hash_field)  // Prepare for duplicate elimination
         table->file->ha_index_init(0, false);
     }
@@ -1187,12 +1618,12 @@ bool SELECT_LEX_UNIT::execute(THD *thd) {
     JOIN *join = first_select()->join;
     DBUG_ASSERT(join && join->is_optimized());
     join->exec();
-    DBUG_RETURN(join->error);
+    return join->error;
   }
 
   Recursive_executor recursive_executor(this, thd);
   if (recursive_executor.initialize(table))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
 
   bool status = false;  // Execution error status
 
@@ -1202,7 +1633,7 @@ bool SELECT_LEX_UNIT::execute(THD *thd) {
       thd->lex->set_current_select(sl);
 
       // Set limit and offset for each execution:
-      if (set_limit(thd, sl)) DBUG_RETURN(true); /* purecov: inspected */
+      if (set_limit(thd, sl)) return true; /* purecov: inspected */
 
       // Execute this query block
       sl->join->exec();
@@ -1212,30 +1643,29 @@ bool SELECT_LEX_UNIT::execute(THD *thd) {
         // This is UNION DISTINCT, so there should be a fake_select_lex
         DBUG_ASSERT(fake_select_lex != NULL);
         if (table->file->ha_disable_indexes(HA_KEY_SWITCH_ALL))
-          DBUG_RETURN(true); /* purecov: inspected */
+          return true; /* purecov: inspected */
         table->no_keyread = 1;
       }
 
-      if (status) DBUG_RETURN(true);
+      if (status) return true;
 
       if (union_result && union_result->flush())
-        DBUG_RETURN(true); /* purecov: inspected */
+        return true; /* purecov: inspected */
     }
 
     if (fake_select_lex != NULL) {
       thd->lex->set_current_select(fake_select_lex);
       if (table->hash_field)  // Prepare for access method of JOIN::exec
         table->file->ha_index_or_rnd_end();
-      if (set_limit(thd, fake_select_lex))
-        DBUG_RETURN(true); /* purecov: inspected */
+      if (set_limit(thd, fake_select_lex)) return true; /* purecov: inspected */
       JOIN *join = fake_select_lex->join;
       if (recursive_executor.prepare_for_scan())
-        DBUG_RETURN(true); /* purecov: inspected */
+        return true; /* purecov: inspected */
       join->exec();
       status = join->error != 0;
-      if (status) DBUG_RETURN(true);
+      if (status) return true;
       if (recursive_executor.save_scan_position())
-        DBUG_RETURN(true);    /* purecov: inspected */
+        return true;          /* purecov: inspected */
       if (table->hash_field)  // Prepare for duplicate elimination
         table->file->ha_index_init(0, false);
     }
@@ -1246,12 +1676,12 @@ bool SELECT_LEX_UNIT::execute(THD *thd) {
     int error = table->file->info(HA_STATUS_VARIABLE);
     if (error) {
       table->file->print_error(error, MYF(0)); /* purecov: inspected */
-      DBUG_RETURN(true);                       /* purecov: inspected */
+      return true;                             /* purecov: inspected */
     }
     thd->current_found_rows = (ulonglong)table->file->stats.records;
   }
 
-  DBUG_RETURN(status);
+  return status;
 }
 
 /**
@@ -1264,14 +1694,18 @@ bool SELECT_LEX_UNIT::execute(THD *thd) {
 */
 
 bool SELECT_LEX_UNIT::cleanup(THD *thd, bool full) {
-  DBUG_ENTER("SELECT_LEX_UNIT::cleanup");
+  DBUG_TRACE;
 
   DBUG_ASSERT(thd == current_thd);
 
-  if (cleaned >= (full ? UC_CLEAN : UC_PART_CLEAN)) DBUG_RETURN(false);
+  if (cleaned >= (full ? UC_CLEAN : UC_PART_CLEAN)) return false;
 
   cleaned = (full ? UC_CLEAN : UC_PART_CLEAN);
 
+  if (full) {
+    m_root_iterator.reset();
+  }
+
   bool error = false;
   for (SELECT_LEX *sl = first_select(); sl; sl = sl->next_select())
     error |= sl->cleanup(thd, full);
@@ -1298,7 +1732,7 @@ bool SELECT_LEX_UNIT::cleanup(THD *thd, bool full) {
     thus be preserved for the next execution, if this is a prepared statement.
   */
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 #ifndef DBUG_OFF
@@ -1323,6 +1757,7 @@ void SELECT_LEX_UNIT::assert_not_fully_clean() {
 
 void SELECT_LEX_UNIT::reinit_exec_mechanism() {
   prepared = optimized = executed = false;
+  m_root_iterator.reset();
 #ifndef DBUG_OFF
   if (is_union()) {
     List_iterator_fast<Item> it(item_list);
@@ -1399,7 +1834,13 @@ List<Item> *SELECT_LEX_UNIT::get_unit_column_types() {
 List<Item> *SELECT_LEX_UNIT::get_field_list() {
   DBUG_ASSERT(is_optimized());
 
-  return is_union() ? &types : first_select()->join->fields;
+  if (fake_select_lex != nullptr) {
+    return fake_select_lex->join->fields;
+  } else if (is_union()) {
+    return &item_list;
+  } else {
+    return first_select()->join->fields;
+  }
 }
 
 const Query_result *SELECT_LEX_UNIT::recursive_result(
@@ -1508,7 +1949,7 @@ static void destroy_materialized(THD *thd, TABLE_LIST *list) {
 */
 
 bool SELECT_LEX::cleanup(THD *thd, bool full) {
-  DBUG_ENTER("SELECT_LEX::cleanup()");
+  DBUG_TRACE;
 
   bool error = false;
   if (join) {
@@ -1534,7 +1975,7 @@ bool SELECT_LEX::cleanup(THD *thd, bool full) {
     while ((w = li++)) w->cleanup(thd);
   }
 
-  DBUG_RETURN(error);
+  return error;
 }
 
 void SELECT_LEX::cleanup_all_joins() {
