@@ -157,22 +157,22 @@ static void make_unique_view_field_name(Item *target, List<Item> &item_list,
 
 bool check_duplicate_names(const Create_col_name_list *column_names,
                            List<Item> &item_list, bool gen_unique_view_name) {
-  DBUG_ENTER("check_duplicate_names");
+  DBUG_TRACE;
   if (column_names) {
     const uint count = column_names->size();
     if (count != item_list.elements) {
       my_error(ER_VIEW_WRONG_LIST, MYF(0));
-      DBUG_RETURN(true);
+      return true;
     }
     for (uint i = 0; i < count; ++i)
       for (uint j = i + 1; j < count; ++j) {
         if (!my_strcasecmp(system_charset_info, (*column_names)[i].str,
                            (*column_names)[j].str)) {
           my_error(ER_DUP_FIELDNAME, MYF(0), (*column_names)[i].str);
-          DBUG_RETURN(true);
+          return true;
         }
       }
-    DBUG_RETURN(false);
+    return false;
   }
 
   Item *item;
@@ -197,11 +197,11 @@ bool check_duplicate_names(const Create_col_name_list *column_names,
       }
     }
   }
-  DBUG_RETURN(false);
+  return false;
 
 err:
   my_error(ER_DUP_FIELDNAME, MYF(0), item->item_name.ptr());
-  DBUG_RETURN(true);
+  return true;
 }
 
 /**
@@ -318,7 +318,7 @@ bool create_view_precheck(THD *thd, TABLE_LIST *tables, TABLE_LIST *view,
   /* first table in list is target VIEW name => cut off it */
   SELECT_LEX *const select_lex = lex->select_lex;
   bool res = true;
-  DBUG_ENTER("create_view_precheck");
+  DBUG_TRACE;
 
   /*
     Privilege check for view creation:
@@ -395,7 +395,7 @@ bool create_view_precheck(THD *thd, TABLE_LIST *tables, TABLE_LIST *view,
   res = false;
 
 err:
-  DBUG_RETURN(res || thd->is_error());
+  return res || thd->is_error();
 }
 
 /**
@@ -436,7 +436,7 @@ bool mysql_create_view(THD *thd, TABLE_LIST *views,
   SELECT_LEX_UNIT *const unit = lex->unit;
   bool res = false;
   bool exists = false;
-  DBUG_ENTER("mysql_create_view");
+  DBUG_TRACE;
   dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
 
   /* This is ensured in the parser. */
@@ -688,9 +688,9 @@ bool mysql_create_view(THD *thd, TABLE_LIST *views,
   // Binlog CREATE/ALTER/CREATE OR REPLACE event.
   if (mysql_bin_log.is_open()) {
     String buff;
-    const LEX_STRING command[3] = {{C_STRING_WITH_LEN("CREATE ")},
-                                   {C_STRING_WITH_LEN("ALTER ")},
-                                   {C_STRING_WITH_LEN("CREATE OR REPLACE ")}};
+    const LEX_CSTRING command[3] = {{STRING_WITH_LEN("CREATE ")},
+                                    {STRING_WITH_LEN("ALTER ")},
+                                    {STRING_WITH_LEN("CREATE OR REPLACE ")}};
 
     buff.append(command[static_cast<int>(thd->lex->create_view_mode)].str,
                 command[static_cast<int>(thd->lex->create_view_mode)].length);
@@ -728,7 +728,7 @@ bool mysql_create_view(THD *thd, TABLE_LIST *views,
 
   my_ok(thd);
   lex->link_first_table_back(view, link_to_local);
-  DBUG_RETURN(false);
+  return false;
 
 err_with_rollback:
   DBUG_EXECUTE_IF("simulate_create_view_failure",
@@ -747,7 +747,7 @@ err:
   lex->link_first_table_back(view, link_to_local);
   unit->cleanup(thd, true);
 
-  DBUG_RETURN(res || thd->is_error());
+  return res || thd->is_error();
 }
 
 /*
@@ -874,7 +874,7 @@ bool mysql_register_view(THD *thd, TABLE_LIST *view,
   char is_query_buff[4096];
   String is_query(is_query_buff, sizeof(is_query_buff), system_charset_info);
 
-  DBUG_ENTER("mysql_register_view");
+  DBUG_TRACE;
 
   /*
     A view can be merged if it is technically possible and if the user didn't
@@ -913,7 +913,7 @@ bool mysql_register_view(THD *thd, TABLE_LIST *view,
   if (lex_string_strmake(thd->mem_root, &view->select_stmt, view_query.ptr(),
                          view_query.length())) {
     my_error(ER_OUT_OF_RESOURCES, MYF(0));
-    DBUG_RETURN(true);
+    return true;
   }
 
   if (lex->create_view_algorithm == VIEW_ALGORITHM_MERGE && !can_be_merged) {
@@ -935,19 +935,18 @@ bool mysql_register_view(THD *thd, TABLE_LIST *view,
   /* check old definition */
   bool update_view = false;
   const dd::Abstract_table *at = nullptr;
-  if (thd->dd_client()->acquire(view->db, view->table_name, &at))
-    DBUG_RETURN(true);
+  if (thd->dd_client()->acquire(view->db, view->table_name, &at)) return true;
 
   if (at != nullptr) {
     if (mode == enum_view_create_mode::VIEW_CREATE_NEW) {
       my_error(ER_TABLE_EXISTS_ERROR, MYF(0), view->alias);
-      DBUG_RETURN(true);
+      return true;
     }
 
     if (at->type() != dd::enum_table_type::USER_VIEW &&
         at->type() != dd::enum_table_type::SYSTEM_VIEW) {
       my_error(ER_WRONG_OBJECT, MYF(0), view->db, view->table_name, "VIEW");
-      DBUG_RETURN(true);
+      return true;
     }
 
     update_view = true;
@@ -959,7 +958,7 @@ bool mysql_register_view(THD *thd, TABLE_LIST *view,
   } else {
     if (mode == enum_view_create_mode::VIEW_ALTER) {
       my_error(ER_NO_SUCH_TABLE, MYF(0), view->db, view->alias);
-      DBUG_RETURN(true);
+      return true;
     }
   }
 
@@ -991,17 +990,17 @@ bool mysql_register_view(THD *thd, TABLE_LIST *view,
   */
   if (is_invalid_string(LEX_CSTRING{is_query.ptr(), is_query.length()},
                         system_charset_info))
-    DBUG_RETURN(true);
+    return true;
 
   if (lex_string_strmake(thd->mem_root, &view->view_body_utf8, is_query.ptr(),
                          is_query.length())) {
     my_error(ER_OUT_OF_RESOURCES, MYF(0));
-    DBUG_RETURN(true);
+    return true;
   }
 
   if (view->with_check != VIEW_CHECK_NONE && !view->updatable_view) {
     my_error(ER_VIEW_NONUPD_CHECK, MYF(0), view->db, view->table_name);
-    DBUG_RETURN(true);
+    return true;
   }
 
   // It is either ALTER or CREATE OR REPLACE of an existing view.
@@ -1009,23 +1008,23 @@ bool mysql_register_view(THD *thd, TABLE_LIST *view,
     dd::View *new_view = nullptr;
     if (thd->dd_client()->acquire_for_modification(view->db, view->table_name,
                                                    &new_view))
-      DBUG_RETURN(true);
+      return true;
 
     DBUG_ASSERT(new_view != nullptr);
 
-    DBUG_RETURN(dd::update_view(thd, new_view, view));
+    return dd::update_view(thd, new_view, view);
   }
 
   // It is either CREATE or CREATE OR REPLACE of non-existent view.
   const dd::Schema *schema = nullptr;
-  if (thd->dd_client()->acquire(view->db, &schema)) DBUG_RETURN(true);
+  if (thd->dd_client()->acquire(view->db, &schema)) return true;
 
   if (schema == nullptr) {
     my_error(ER_BAD_DB_ERROR, MYF(0), view->db);
-    DBUG_RETURN(true);
+    return true;
   }
 
-  DBUG_RETURN(dd::create_view(thd, *schema, view));
+  return dd::create_view(thd, *schema, view);
 }
 
 /// RAII class to ease error handling in parse_view_definition()
@@ -1064,14 +1063,14 @@ class Make_view_tracker {
 */
 
 bool open_and_read_view(THD *thd, TABLE_SHARE *share, TABLE_LIST *view_ref) {
-  DBUG_ENTER("open_and_read_view");
+  DBUG_TRACE;
 
   TABLE_LIST *const top_view = view_ref->top_table();
 
   if (view_ref->required_type == dd::enum_table_type::BASE_TABLE) {
     my_error(ER_WRONG_OBJECT, MYF(0), share->db.str, share->table_name.str,
              "BASE TABLE");
-    DBUG_RETURN(true);
+    return true;
   }
 
   Prepared_stmt_arena_holder ps_arena_holder(thd);
@@ -1080,13 +1079,13 @@ bool open_and_read_view(THD *thd, TABLE_SHARE *share, TABLE_LIST *view_ref) {
     DBUG_PRINT("info",
                ("VIEW %s.%s is already processed on previous PS/SP execution",
                 view_ref->view_db.str, view_ref->view_name.str));
-    DBUG_RETURN(false);
+    return false;
   }
 
   if (view_ref->index_hints && view_ref->index_hints->elements) {
     my_error(ER_KEY_DOES_NOT_EXITS, MYF(0),
              view_ref->index_hints->head()->key_name.str, view_ref->table_name);
-    DBUG_RETURN(true);
+    return true;
   }
 
   // Check that view is not referenced recursively
@@ -1100,7 +1099,7 @@ bool open_and_read_view(THD *thd, TABLE_SHARE *share, TABLE_LIST *view_ref) {
                       view_ref->db) == 0) {
       my_error(ER_VIEW_RECURSIVE, MYF(0), top_view->view_db.str,
                top_view->view_name.str);
-      DBUG_RETURN(true);
+      return true;
     }
   }
 
@@ -1117,7 +1116,7 @@ bool open_and_read_view(THD *thd, TABLE_SHARE *share, TABLE_LIST *view_ref) {
 
   // Read view details from the view object.
   if (dd::read_view(view_ref, *share->view_object, thd->mem_root))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
 
   // Check old format view.
   if (!view_ref->definer.user.str) {
@@ -1136,7 +1135,7 @@ bool open_and_read_view(THD *thd, TABLE_SHARE *share, TABLE_LIST *view_ref) {
   */
   view_ref->view_creation_ctx = View_creation_ctx::create(thd, view_ref);
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -1181,7 +1180,7 @@ void merge_query_blocks(LEX *view_lex, LEX *parent_lex) {
 */
 
 bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
-  DBUG_ENTER("parse_view_definition");
+  DBUG_TRACE;
 
   TABLE_LIST *const top_view = view_ref->top_table();
 
@@ -1203,9 +1202,9 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
       prepare_security() below.
     */
     if (!view_ref->prelocking_placeholder && view_ref->prepare_security(thd))
-      DBUG_RETURN(true);
+      return true;
 
-    DBUG_RETURN(false);
+    return false;
   }
 
   // Save view's name, which will be wiped out by materialization
@@ -1231,7 +1230,7 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
   // A parsed view requires its own LEX object
   LEX *const old_lex = thd->lex;
   LEX *const view_lex = (LEX *)new (thd->mem_root) st_lex_local;
-  if (!view_lex) DBUG_RETURN(true);
+  if (!view_lex) return true;
 
   bool result = false;
   Make_view_tracker view_tracker(thd, view_ref, &result);
@@ -1244,7 +1243,7 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
   Parser_state parser_state;
   if ((result = parser_state.init(thd, view_ref->select_stmt.str,
                                   view_ref->select_stmt.length)))
-    DBUG_RETURN(true); /* purecov: inspected */
+    return true; /* purecov: inspected */
   /*
     Use view db name as thread default database, in order to ensure
     that the view is parsed and prepared correctly.
@@ -1308,7 +1307,7 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
   thd->reset_db(current_db_name_saved);
   mysql_mutex_unlock(&thd->LOCK_thd_data);
 
-  if (result) DBUG_RETURN(true); /* purecov: inspected */
+  if (result) return true; /* purecov: inspected */
 
   // sql_calc_found_rows is only relevant for outer-most query expression
   view_lex->select_lex->remove_base_options(OPTION_FOUND_ROWS);
@@ -1368,7 +1367,7 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
       if (!is_infoschema_db(view_ref->db) && view_ref->view_no_explain) {
         my_error(ER_VIEW_NO_EXPLAIN, MYF(0));
         result = true;
-        DBUG_RETURN(true);
+        return true;
       }
     } else if ((old_lex->sql_command == SQLCOM_SHOW_CREATE) &&
                !view_ref->belong_to_view) {
@@ -1378,14 +1377,14 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
           check_table_access(thd, SHOW_VIEW_ACL, view_ref, false, UINT_MAX,
                              false)) {
         result = true;
-        DBUG_RETURN(true);
+        return true;
       }
     }
   }
 
   if (!(view_ref->view_tables = new (thd->mem_root) List<TABLE_LIST>)) {
     result = true;
-    DBUG_RETURN(true);
+    return true;
   }
   /*
     Apply necessary updates to the tables underlying this view.
@@ -1527,7 +1526,7 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
     this view serves as simple placeholder and we should not continue
     further processing.
   */
-  if (view_ref->prelocking_placeholder) DBUG_RETURN(false);
+  if (view_ref->prelocking_placeholder) return false;
 
   // Move nondeterminism information to whole query.
   old_lex->safe_to_cache_query &= view_lex->safe_to_cache_query;
@@ -1545,9 +1544,9 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
       DBUG_ASSERT(thd->stmt_arena->mem_root);
       view_ref->view_sctx = new (thd->stmt_arena->mem_root)
           Security_context(thd->stmt_arena->mem_root);
-      if (view_ref->view_sctx == nullptr) DBUG_RETURN(true);
+      if (view_ref->view_sctx == nullptr) return true;
     } catch (...) {
-      DBUG_RETURN(true);
+      return true;
     }
     security_ctx = view_ref->view_sctx;
     DBUG_PRINT("info",
@@ -1610,7 +1609,7 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
 
   lex_end(view_lex);
 
-  DBUG_RETURN(result);
+  return result;
 }
 
 /**
@@ -1639,7 +1638,7 @@ bool parse_view_definition(THD *thd, TABLE_LIST *view_ref) {
 bool mysql_drop_view(THD *thd, TABLE_LIST *views) {
   bool some_views_deleted = false;
 
-  DBUG_ENTER("mysql_drop_view");
+  DBUG_TRACE;
 
   /*
     We can't allow dropping of unlocked view under LOCK TABLES since this
@@ -1648,11 +1647,11 @@ bool mysql_drop_view(THD *thd, TABLE_LIST *views) {
   */
   if (thd->locked_tables_mode) {
     my_error(ER_LOCK_OR_ACTIVE_TRANSACTION, MYF(0));
-    DBUG_RETURN(true);
+    return true;
   }
 
   if (lock_table_names(thd, views, 0, thd->variables.lock_wait_timeout, 0))
-    DBUG_RETURN(true);
+    return true;
 
   dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   Security_context *sctx = thd->security_context();
@@ -1669,8 +1668,7 @@ bool mysql_drop_view(THD *thd, TABLE_LIST *views) {
       different actions depending on the table type.
     */
     const dd::Abstract_table *at = nullptr;
-    if (thd->dd_client()->acquire(view->db, view->table_name, &at))
-      DBUG_RETURN(true);
+    if (thd->dd_client()->acquire(view->db, view->table_name, &at)) return true;
 
     if (at == nullptr) {
       String tbl_name(view->db, system_charset_info);
@@ -1686,12 +1684,12 @@ bool mysql_drop_view(THD *thd, TABLE_LIST *views) {
       }
     } else if (at->type() == dd::enum_table_type::BASE_TABLE) {
       my_error(ER_WRONG_OBJECT, MYF(0), view->db, view->table_name, "VIEW");
-      DBUG_RETURN(true);
+      return true;
     }
   }
   if (non_existant_views.length()) {
     my_error(ER_BAD_TABLE_ERROR, MYF(0), non_existant_views.c_ptr());
-    DBUG_RETURN(true);
+    return true;
   }
 
   // Then actually start dropping views.
@@ -1713,7 +1711,7 @@ bool mysql_drop_view(THD *thd, TABLE_LIST *views) {
       trans_rollback_stmt(thd);
       // Full rollback in case we have THD::transaction_rollback_request.
       trans_rollback(thd);
-      DBUG_RETURN(true);
+      return true;
     }
 
     if (at == nullptr) {
@@ -1731,8 +1729,7 @@ bool mysql_drop_view(THD *thd, TABLE_LIST *views) {
       only if latter also has same privilege.
     */
     Auth_id definer(vw->definer_user().c_str(), vw->definer_host().c_str());
-    if (sctx->can_operate_with(definer, consts::system_user, true))
-      DBUG_RETURN(true);
+    if (sctx->can_operate_with(definer, consts::system_user, true)) return true;
 
     Uncommitted_tables_guard uncommitted_tables(thd);
     /*
@@ -1756,7 +1753,7 @@ bool mysql_drop_view(THD *thd, TABLE_LIST *views) {
         rollback doesn't clear DD cache of modified uncommitted objects).
       */
       trans_rollback(thd);
-      DBUG_RETURN(true);
+      return true;
     }
 
     thd->add_to_binlog_accessed_dbs(view->db);
@@ -1777,13 +1774,13 @@ bool mysql_drop_view(THD *thd, TABLE_LIST *views) {
       rollback doesn't clear DD cache of modified uncommitted objects).
     */
     trans_rollback(thd);
-    DBUG_RETURN(true);
+    return true;
   }
 
-  if (trans_commit_stmt(thd) || trans_commit(thd)) DBUG_RETURN(true);
+  if (trans_commit_stmt(thd) || trans_commit(thd)) return true;
 
   my_ok(thd);
-  DBUG_RETURN(false);
+  return false;
 }
 
 /**
@@ -1805,7 +1802,7 @@ bool mysql_drop_view(THD *thd, TABLE_LIST *views) {
 
 bool check_key_in_view(THD *thd, TABLE_LIST *view,
                        const TABLE_LIST *table_ref) {
-  DBUG_ENTER("check_key_in_view");
+  DBUG_TRACE;
 
   /*
     we do not support updatable UNIONs in VIEW, so we can check just limit of
@@ -1815,7 +1812,7 @@ bool check_key_in_view(THD *thd, TABLE_LIST *view,
   if ((!view->is_view() && !view->belong_to_view) ||
       thd->lex->sql_command == SQLCOM_INSERT ||
       thd->lex->select_lex->select_limit == 0)
-    DBUG_RETURN(false); /* it is normal table or query without LIMIT */
+    return false; /* it is normal table or query without LIMIT */
 
   TABLE *const table = table_ref->table;
   view = view->top_table();
@@ -1839,7 +1836,7 @@ bool check_key_in_view(THD *thd, TABLE_LIST *view,
     thd->want_privilege = 0;
     for (Field_translator *fld = trans; fld < end_of_trans; fld++) {
       if (!fld->item->fixed && fld->item->fix_fields(thd, &fld->item))
-        DBUG_RETURN(true); /* purecov: inspected */
+        return true; /* purecov: inspected */
     }
     thd->mark_used_columns = save_mark_used_columns;
     thd->want_privilege = want_privilege_saved;
@@ -1859,8 +1856,8 @@ bool check_key_in_view(THD *thd, TABLE_LIST *view,
               field->field == key_part->field)
             break;
         }
-        if (k == end_of_trans) break;  // Key is not possible
-        if (++key_part == key_part_end) DBUG_RETURN(false);  // Found usable key
+        if (k == end_of_trans) break;                  // Key is not possible
+        if (++key_part == key_part_end) return false;  // Found usable key
       }
     }
   }
@@ -1888,14 +1885,14 @@ bool check_key_in_view(THD *thd, TABLE_LIST *view,
           /* update allowed, but issue warning */
           push_warning(thd, Sql_condition::SL_NOTE, ER_WARN_VIEW_WITHOUT_KEY,
                        ER_THD(thd, ER_WARN_VIEW_WITHOUT_KEY));
-          DBUG_RETURN(false);
+          return false;
         }
         /* prohibit update */
-        DBUG_RETURN(true);
+        return true;
       }
     }
   }
-  DBUG_RETURN(false);
+  return false;
 }
 
 /*
@@ -1914,19 +1911,19 @@ bool check_key_in_view(THD *thd, TABLE_LIST *view,
 bool insert_view_fields(List<Item> *list, TABLE_LIST *view) {
   Field_translator *trans_end;
   Field_translator *trans;
-  DBUG_ENTER("insert_view_fields");
+  DBUG_TRACE;
 
-  if (!(trans = view->field_translation)) DBUG_RETURN(false);
+  if (!(trans = view->field_translation)) return false;
   trans_end = view->field_translation_end;
 
   for (Field_translator *entry = trans; entry < trans_end; entry++) {
     Item_field *fld = entry->item->field_for_view_update();
     if (fld == NULL) {
       my_error(ER_NONUPDATEABLE_COLUMN, MYF(0), entry->name);
-      DBUG_RETURN(true);
+      return true;
     }
 
     list->push_back(fld);
   }
-  DBUG_RETURN(false);
+  return false;
 }
