@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MYSQL_EXPORT_HOME3_PB2_BUILD_SB_4_33646267_1555156040_89_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
-# define YY_MYSQL_EXPORT_HOME3_PB2_BUILD_SB_4_33646267_1555156040_89_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+#ifndef YY_MYSQL_EXPORT_HOME3_PB2_BUILD_SB_4_36015059_1568969063_34_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+# define YY_MYSQL_EXPORT_HOME3_PB2_BUILD_SB_4_36015059_1568969063_34_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -80,10 +80,10 @@ extern int MYSQLdebug;
     BINARY_SYM = 290,
     BINLOG_SYM = 291,
     BIN_NUM = 292,
-    BIT_AND = 293,
-    BIT_OR = 294,
+    BIT_AND_SYM = 293,
+    BIT_OR_SYM = 294,
     BIT_SYM = 295,
-    BIT_XOR = 296,
+    BIT_XOR_SYM = 296,
     BLOB_SYM = 297,
     BLOCK_SYM = 298,
     BOOLEAN_SYM = 299,
@@ -433,7 +433,7 @@ extern int MYSQLdebug;
     ORDER_SYM = 643,
     OR_OR_SYM = 644,
     OR_SYM = 645,
-    OUTER = 646,
+    OUTER_SYM = 646,
     OUTFILE = 647,
     OUT_SYM = 648,
     OWNER_SYM = 649,
@@ -763,19 +763,25 @@ extern int MYSQLdebug;
     ACTIVE_SYM = 973,
     INACTIVE_SYM = 974,
     LATERAL_SYM = 975,
-    OPTIONAL_SYM = 976,
-    SECONDARY_SYM = 977,
-    SECONDARY_ENGINE_SYM = 978,
-    SECONDARY_LOAD_SYM = 979,
-    SECONDARY_UNLOAD_SYM = 980,
-    RETAIN_SYM = 981,
-    OLD_SYM = 982,
-    ENFORCED_SYM = 983,
-    OJ_SYM = 984,
-    NETWORK_NAMESPACE_SYM = 985,
-    CONDITIONLESS_JOIN = 986,
-    SUBQUERY_AS_EXPR = 987,
-    EMPTY_FROM_CLAUSE = 988
+    ARRAY_SYM = 976,
+    MEMBER_SYM = 977,
+    OPTIONAL_SYM = 978,
+    SECONDARY_SYM = 979,
+    SECONDARY_ENGINE_SYM = 980,
+    SECONDARY_LOAD_SYM = 981,
+    SECONDARY_UNLOAD_SYM = 982,
+    RETAIN_SYM = 983,
+    OLD_SYM = 984,
+    ENFORCED_SYM = 985,
+    OJ_SYM = 986,
+    NETWORK_NAMESPACE_SYM = 987,
+    RANDOM_SYM = 988,
+    MASTER_COMPRESSION_ALGORITHM_SYM = 989,
+    MASTER_ZSTD_COMPRESSION_LEVEL_SYM = 990,
+    PRIVILEGE_CHECKS_USER_SYM = 991,
+    CONDITIONLESS_JOIN = 992,
+    SUBQUERY_AS_EXPR = 993,
+    EMPTY_FROM_CLAUSE = 994
   };
 #endif
 /* Tokens.  */
@@ -814,10 +820,10 @@ extern int MYSQLdebug;
 #define BINARY_SYM 290
 #define BINLOG_SYM 291
 #define BIN_NUM 292
-#define BIT_AND 293
-#define BIT_OR 294
+#define BIT_AND_SYM 293
+#define BIT_OR_SYM 294
 #define BIT_SYM 295
-#define BIT_XOR 296
+#define BIT_XOR_SYM 296
 #define BLOB_SYM 297
 #define BLOCK_SYM 298
 #define BOOLEAN_SYM 299
@@ -1167,7 +1173,7 @@ extern int MYSQLdebug;
 #define ORDER_SYM 643
 #define OR_OR_SYM 644
 #define OR_SYM 645
-#define OUTER 646
+#define OUTER_SYM 646
 #define OUTFILE 647
 #define OUT_SYM 648
 #define OWNER_SYM 649
@@ -1497,19 +1503,25 @@ extern int MYSQLdebug;
 #define ACTIVE_SYM 973
 #define INACTIVE_SYM 974
 #define LATERAL_SYM 975
-#define OPTIONAL_SYM 976
-#define SECONDARY_SYM 977
-#define SECONDARY_ENGINE_SYM 978
-#define SECONDARY_LOAD_SYM 979
-#define SECONDARY_UNLOAD_SYM 980
-#define RETAIN_SYM 981
-#define OLD_SYM 982
-#define ENFORCED_SYM 983
-#define OJ_SYM 984
-#define NETWORK_NAMESPACE_SYM 985
-#define CONDITIONLESS_JOIN 986
-#define SUBQUERY_AS_EXPR 987
-#define EMPTY_FROM_CLAUSE 988
+#define ARRAY_SYM 976
+#define MEMBER_SYM 977
+#define OPTIONAL_SYM 978
+#define SECONDARY_SYM 979
+#define SECONDARY_ENGINE_SYM 980
+#define SECONDARY_LOAD_SYM 981
+#define SECONDARY_UNLOAD_SYM 982
+#define RETAIN_SYM 983
+#define OLD_SYM 984
+#define ENFORCED_SYM 985
+#define OJ_SYM 986
+#define NETWORK_NAMESPACE_SYM 987
+#define RANDOM_SYM 988
+#define MASTER_COMPRESSION_ALGORITHM_SYM 989
+#define MASTER_ZSTD_COMPRESSION_LEVEL_SYM 990
+#define PRIVILEGE_CHECKS_USER_SYM 991
+#define CONDITIONLESS_JOIN 992
+#define SUBQUERY_AS_EXPR 993
+#define EMPTY_FROM_CLAUSE 994
 
 /* Value type.  */
 
@@ -1531,4 +1543,4 @@ struct YYLTYPE
 
 int MYSQLparse (class THD *YYTHD, class Parse_tree_root **parse_tree);
 
-#endif /* !YY_MYSQL_EXPORT_HOME3_PB2_BUILD_SB_4_33646267_1555156040_89_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
+#endif /* !YY_MYSQL_EXPORT_HOME3_PB2_BUILD_SB_4_36015059_1568969063_34_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
