@@ -1,16 +1,15 @@
-/* Copyright (c) 2003, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,8 +24,6 @@
 
 #include "sql/strfunc.h"
 
-#include <string.h>
-
 #include "m_ctype.h"  // my_charset_latin1
 #include "my_alloc.h"
 #include "my_dbug.h"
@@ -63,7 +60,7 @@ ulonglong find_set(const TYPELIB *lib, const char *str, size_t length,
   const CHARSET_INFO *strip = cs ? cs : &my_charset_latin1;
   const char *end = str + strip->cset->lengthsp(strip, str, length);
   ulonglong found = 0;
-  *err_pos = nullptr;  // No error yet
+  *err_pos = 0;  // No error yet
   *err_len = 0;
   if (str != end) {
     const char *start = str;
@@ -90,7 +87,7 @@ ulonglong find_set(const TYPELIB *lib, const char *str, size_t length,
       {
         *err_pos = start;
         *err_len = var_len;
-        *set_warning = true;
+        *set_warning = 1;
       } else if (find)  // avoid 1ULL << 4294967295
         found |= 1ULL << (find - 1);
 
@@ -200,8 +197,7 @@ uint check_word(TYPELIB *lib, const char *val, const char *end,
   /* Fiend end of word */
   for (ptr = val; ptr < end && my_isalpha(&my_charset_latin1, *ptr); ptr++)
     ;
-  if ((res = find_type(lib, val, (uint)(ptr - val), true)) > 0)
-    *end_of_word = ptr;
+  if ((res = find_type(lib, val, (uint)(ptr - val), 1)) > 0) *end_of_word = ptr;
   return res;
 }
 
@@ -227,6 +223,7 @@ uint check_word(TYPELIB *lib, const char *val, const char *end,
 size_t strconvert(const CHARSET_INFO *from_cs, const char *from,
                   CHARSET_INFO *to_cs, char *to, size_t to_length,
                   uint *errors) {
+  int cnvres;
   my_wc_t wc;
   char *to_start = to;
   uchar *to_end = (uchar *)to + to_length - 1;
@@ -234,14 +231,15 @@ size_t strconvert(const CHARSET_INFO *from_cs, const char *from,
   my_charset_conv_wc_mb wc_mb = to_cs->cset->wc_mb;
   uint error_count = 0;
 
-  while (true) {
+  while (1) {
     /*
-      Lookahead of max 10 bytes should suffice for all character sets.
+      Using 'from + 10' is safe:
+      - it is enough to scan a single character in any character set.
+      - if remaining string is shorter than 10, then mb_wc will return
+        with error because of unexpected '\0' character.
     */
-    const size_t max_char_len = strnlen(from, 10);
-    int cnvres = (*mb_wc)(from_cs, &wc, pointer_cast<const uchar *>(from),
-                          pointer_cast<const uchar *>(from) + max_char_len);
-    if (cnvres > 0) {
+    if ((cnvres = (*mb_wc)(from_cs, &wc, pointer_cast<const uchar *>(from),
+                           pointer_cast<const uchar *>(from) + 10)) > 0) {
       if (!wc) break;
       from += cnvres;
     } else if (cnvres == MY_CS_ILSEQ) {
