@@ -1,6 +1,6 @@
 #ifndef SYS_VARS_H_INCLUDED
 #define SYS_VARS_H_INCLUDED
-/* Copyright (c) 2002, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -101,8 +101,8 @@ class Sys_var_integer;
 #define VALID_RANGE(X, Y) X, Y
 #define DEFAULT(X) X
 #define BLOCK_SIZE(X) X
-#define GLOBAL_VAR(X)                                                   \
-  sys_var::GLOBAL, (((char *)&(X)) - (char *)&global_system_variables), \
+#define GLOBAL_VAR(X)                                                         \
+  sys_var::GLOBAL, (((const char *)&(X)) - (char *)&global_system_variables), \
       sizeof(X)
 #define SESSION_VAR(X)                             \
   sys_var::SESSION, offsetof(System_variables, X), \
@@ -279,8 +279,8 @@ bool global_update(THD *, set_var *var) {
 }
 bool check_update_type(Item_result type) { return type != INT_RESULT; }
 void session_save_default(THD *thd, set_var *var) {
-  var->save_result.ulonglong_value =
-      static_cast<ulonglong>(*(T *)global_value_ptr(thd, 0));
+  var->save_result.ulonglong_value = static_cast<ulonglong>(
+      *pointer_cast<const T *>(global_value_ptr(thd, nullptr)));
 }
 void global_save_default(THD *, set_var *var) {
   var->save_result.ulonglong_value = option.def_value;
@@ -412,11 +412,12 @@ class Sys_var_enum : public Sys_var_typelib {
   void saved_value_to_string(THD *, set_var *var, char *def_val) {
     longlong10_to_str((longlong)var->save_result.ulonglong_value, def_val, 10);
   }
-  uchar *session_value_ptr(THD *, THD *target_thd, LEX_STRING *) {
-    return (uchar *)typelib.type_names[session_var(target_thd, ulong)];
+  const uchar *session_value_ptr(THD *, THD *target_thd, LEX_STRING *) {
+    return pointer_cast<const uchar *>(
+        typelib.type_names[session_var(target_thd, ulong)]);
   }
-  uchar *global_value_ptr(THD *, LEX_STRING *) {
-    return (uchar *)typelib.type_names[global_var(ulong)];
+  const uchar *global_value_ptr(THD *, LEX_STRING *) {
+    return pointer_cast<const uchar *>(typelib.type_names[global_var(ulong)]);
   }
 };
 
@@ -454,8 +455,8 @@ class Sys_var_bool : public Sys_var_typelib {
     return false;
   }
   void session_save_default(THD *thd, set_var *var) {
-    var->save_result.ulonglong_value =
-        static_cast<ulonglong>(*(bool *)global_value_ptr(thd, 0));
+    var->save_result.ulonglong_value = static_cast<ulonglong>(
+        *pointer_cast<const bool *>(global_value_ptr(thd, nullptr)));
   }
   void global_save_default(THD *, set_var *var) {
     var->save_result.ulonglong_value = option.def_value;
@@ -596,7 +597,7 @@ class Sys_var_multi_enum : public sys_var {
     argument.
   */
   const char *fixup_command_line(const char *value_str) {
-    DBUG_ENTER("Sys_var_multi_enum::fixup_command_line");
+    DBUG_TRACE;
     char *end = NULL;
     long value;
 
@@ -618,46 +619,46 @@ class Sys_var_multi_enum : public sys_var {
       if (value >= 0 && (longlong)value < (longlong)value_count) goto end;
 
     // Not a valid value.
-    DBUG_RETURN(value_str);
+    return value_str;
 
   end:
     global_var(ulong) = value;
-    DBUG_RETURN(NULL);
+    return NULL;
   }
 
   bool do_check(THD *, set_var *var) {
-    DBUG_ENTER("Sys_var_multi_enum::do_check");
+    DBUG_TRACE;
     char buff[STRING_BUFFER_USUAL_SIZE];
     String str(buff, sizeof(buff), system_charset_info), *res;
     if (var->value->result_type() == STRING_RESULT) {
       res = var->value->val_str(&str);
-      if (!res) DBUG_RETURN(true);
+      if (!res) return true;
 
       /* Check if the value is a valid string. */
       size_t valid_len;
       bool len_error;
       if (validate_string(system_charset_info, res->ptr(), res->length(),
                           &valid_len, &len_error))
-        DBUG_RETURN(true);
+        return true;
 
       int value = find_value(res->ptr());
-      if (value == -1) DBUG_RETURN(true);
+      if (value == -1) return true;
       var->save_result.ulonglong_value = (uint)value;
     } else {
       longlong value = var->value->val_int();
       if (value < 0 || value >= (longlong)value_count)
-        DBUG_RETURN(true);
+        return true;
       else
         var->save_result.ulonglong_value = value;
     }
 
-    DBUG_RETURN(false);
+    return false;
   }
   bool check_update_type(Item_result type) {
     return type != INT_RESULT && type != STRING_RESULT;
   }
   bool session_update(THD *, set_var *) {
-    DBUG_ENTER("Sys_var_multi_enum::session_update");
+    DBUG_TRACE;
     DBUG_ASSERT(0);
     /*
     Currently not used: uncomment if this class is used as a base for
@@ -666,10 +667,10 @@ class Sys_var_multi_enum : public sys_var {
     session_var(thd, ulong)=
       static_cast<ulong>(var->save_result.ulonglong_value);
     */
-    DBUG_RETURN(false);
+    return false;
   }
   bool global_update(THD *, set_var *) {
-    DBUG_ENTER("Sys_var_multi_enum::global_update");
+    DBUG_TRACE;
     DBUG_ASSERT(0);
     /*
     Currently not used: uncomment if this some inheriting class does
@@ -679,10 +680,10 @@ class Sys_var_multi_enum : public sys_var {
       static_cast<ulong>(var->save_result.ulonglong_value);
     global_var(ulong)= val;
     */
-    DBUG_RETURN(false);
+    return false;
   }
   void session_save_default(THD *, set_var *) {
-    DBUG_ENTER("Sys_var_multi_enum::session_save_default");
+    DBUG_TRACE;
     DBUG_ASSERT(0);
     /*
     Currently not used: uncomment if this class is used as a base for
@@ -692,33 +693,33 @@ class Sys_var_multi_enum : public sys_var {
     DBUG_ASSERT(value != -1);
     var->save_result.ulonglong_value= value;
     */
-    DBUG_VOID_RETURN;
+    return;
   }
   void global_save_default(THD *, set_var *var) {
-    DBUG_ENTER("Sys_var_multi_enum::global_save_default");
+    DBUG_TRACE;
     int value = find_value((char *)option.def_value);
     DBUG_ASSERT(value != -1);
     var->save_result.ulonglong_value = value;
-    DBUG_VOID_RETURN;
+    return;
   }
   void saved_value_to_string(THD *, set_var *var, char *def_val) {
     longlong10_to_str((longlong)var->save_result.ulonglong_value, def_val, 10);
   }
 
-  uchar *session_value_ptr(THD *, THD *, LEX_STRING *) {
-    DBUG_ENTER("Sys_var_multi_enum::session_value_ptr");
+  const uchar *session_value_ptr(THD *, THD *, LEX_STRING *) {
+    DBUG_TRACE;
     DBUG_ASSERT(0);
     /*
     Currently not used: uncomment if this class is used as a base for
     a session variable.
 
-    DBUG_RETURN((uchar*)aliases[session_var(target_thd, ulong)].alias);
+    return (uchar*)aliases[session_var(target_thd, ulong)].alias;
     */
-    DBUG_RETURN(0);
+    return 0;
   }
-  uchar *global_value_ptr(THD *, LEX_STRING *) {
-    DBUG_ENTER("Sys_var_multi_enum::global_value_ptr");
-    DBUG_RETURN((uchar *)aliases[global_var(ulong)].alias);
+  const uchar *global_value_ptr(THD *, LEX_STRING *) {
+    DBUG_TRACE;
+    return pointer_cast<const uchar *>(aliases[global_var(ulong)].alias);
   }
 
  private:
@@ -849,12 +850,12 @@ class Sys_var_version : public Sys_var_charptr {
 
   ~Sys_var_version() {}
 
-  virtual uchar *global_value_ptr(THD *thd, LEX_STRING *base) {
-    uchar *value = Sys_var_charptr::global_value_ptr(thd, base);
+  virtual const uchar *global_value_ptr(THD *thd, LEX_STRING *base) {
+    const uchar *value = Sys_var_charptr::global_value_ptr(thd, base);
 
     DBUG_EXECUTE_IF("alter_server_version_str", {
       static const char *altered_value = "some-other-version";
-      uchar *altered_value_ptr = reinterpret_cast<uchar *>(&altered_value);
+      const uchar *altered_value_ptr = pointer_cast<uchar *>(&altered_value);
       value = altered_value_ptr;
     });
 
@@ -891,9 +892,9 @@ class Sys_var_proxy_user : public sys_var {
   bool check_update_type(Item_result) { return true; }
 
  protected:
-  virtual uchar *session_value_ptr(THD *, THD *target_thd, LEX_STRING *) {
+  virtual const uchar *session_value_ptr(THD *, THD *target_thd, LEX_STRING *) {
     const char *proxy_user = target_thd->security_context()->proxy_user().str;
-    return proxy_user[0] ? (uchar *)proxy_user : NULL;
+    return proxy_user[0] ? pointer_cast<const uchar *>(proxy_user) : nullptr;
   }
 };
 
@@ -904,9 +905,10 @@ class Sys_var_external_user : public Sys_var_proxy_user {
       : Sys_var_proxy_user(name_arg, comment_arg, is_os_charset_arg) {}
 
  protected:
-  virtual uchar *session_value_ptr(THD *, THD *target_thd, LEX_STRING *) {
+  virtual const uchar *session_value_ptr(THD *, THD *target_thd, LEX_STRING *) {
     LEX_CSTRING external_user = target_thd->security_context()->external_user();
-    return external_user.length ? (uchar *)external_user.str : NULL;
+    return external_user.length ? pointer_cast<const uchar *>(external_user.str)
+                                : nullptr;
   }
 };
 
@@ -1007,12 +1009,12 @@ class Sys_var_dbug : public sys_var {
     memcpy(def_val, var->save_result.string_value.str,
            var->save_result.string_value.length);
   }
-  uchar *session_value_ptr(THD *running_thd, THD *, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *running_thd, THD *, LEX_STRING *) {
     char buf[256];
     DBUG_EXPLAIN(buf, sizeof(buf));
     return (uchar *)running_thd->mem_strdup(buf);
   }
-  uchar *global_value_ptr(THD *thd, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *thd, LEX_STRING *) {
     char buf[256];
     DBUG_EXPLAIN_INITIAL(buf, sizeof(buf));
     return (uchar *)thd->mem_strdup(buf);
@@ -1064,26 +1066,25 @@ class Sys_var_keycache : public Sys_var_ulonglong {
   }
   bool global_update(THD *thd, set_var *var) {
     ulonglong new_value = var->save_result.ulonglong_value;
-    LEX_STRING *base_name = &var->base;
-    KEY_CACHE *key_cache;
 
-    if (base_name != NULL && base_name->str)
+    if (var->base.str)
       push_warning_printf(thd, Sql_condition::SL_WARNING,
                           ER_WARN_DEPRECATED_SYNTAX,
                           "%s.%s syntax "
                           "is deprecated and will be removed in a "
                           "future release",
-                          base_name->str, name.str);
+                          var->base.str, name.str);
 
+    LEX_CSTRING base_name = var->base;
     /* If no basename, assume it's for the key cache named 'default' */
-    if (!base_name->length) base_name = &default_key_cache_base;
+    if (!base_name.length) base_name = default_key_cache_base;
 
-    key_cache = get_key_cache(base_name);
+    KEY_CACHE *key_cache = get_key_cache(&base_name);
 
     if (!key_cache) {  // Key cache didn't exists */
       if (!new_value)  // Tried to delete cache
         return false;  // Ok, nothing to do
-      if (!(key_cache = create_key_cache(base_name->str, base_name->length)))
+      if (!(key_cache = create_key_cache(base_name.str, base_name.length)))
         return true;
     }
 
@@ -1096,7 +1097,7 @@ class Sys_var_keycache : public Sys_var_ulonglong {
 
     return keycache_update(thd, key_cache, offset, new_value);
   }
-  uchar *global_value_ptr(THD *thd, LEX_STRING *base) {
+  const uchar *global_value_ptr(THD *thd, LEX_STRING *base) {
     if (base != NULL && base->str)
       push_warning_printf(thd, Sql_condition::SL_WARNING,
                           ER_WARN_DEPRECATED_SYNTAX,
@@ -1105,7 +1106,8 @@ class Sys_var_keycache : public Sys_var_ulonglong {
                           "future release",
                           base->str, name.str);
 
-    KEY_CACHE *key_cache = get_key_cache(base);
+    LEX_CSTRING cstr = to_lex_cstring(*base);
+    KEY_CACHE *key_cache = get_key_cache(&cstr);
     if (!key_cache) key_cache = &zero_key_cache;
     return keycache_var_ptr(key_cache, offset);
   }
@@ -1194,7 +1196,7 @@ class Sys_var_test_flag : public Sys_var_bool {
                      NO_CMD_LINE, DEFAULT(false)) {
     test_flag_mask = mask;
   }
-  uchar *global_value_ptr(THD *, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *, LEX_STRING *) {
     test_flag_value = ((test_flags & test_flag_mask) > 0);
     return (uchar *)&test_flag_value;
   }
@@ -1222,11 +1224,11 @@ class Sys_var_max_user_conn : public Sys_var_uint {
       : Sys_var_uint(name_arg, comment, SESSION, off, size, getopt, min_val,
                      max_val, def_val, block_size, lock, binlog_status_arg,
                      on_check_func, on_update_func, substitute) {}
-  uchar *session_value_ptr(THD *running_thd, THD *target_thd,
-                           LEX_STRING *base) {
+  const uchar *session_value_ptr(THD *running_thd, THD *target_thd,
+                                 LEX_STRING *base) {
     const USER_CONN *uc = target_thd->get_user_connect();
     if (uc && uc->user_resources.user_conn)
-      return (uchar *)&(uc->user_resources.user_conn);
+      return pointer_cast<const uchar *>(&(uc->user_resources.user_conn));
     return global_value_ptr(running_thd, base);
   }
 };
@@ -1325,11 +1327,12 @@ class Sys_var_flagset : public Sys_var_typelib {
     strcpy(def_val, flagset_to_string(thd, 0, var->save_result.ulonglong_value,
                                       typelib.type_names));
   }
-  uchar *session_value_ptr(THD *running_thd, THD *target_thd, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *running_thd, THD *target_thd,
+                                 LEX_STRING *) {
     return (uchar *)flagset_to_string(
         running_thd, 0, session_var(target_thd, ulonglong), typelib.type_names);
   }
-  uchar *global_value_ptr(THD *thd, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *thd, LEX_STRING *) {
     return (uchar *)flagset_to_string(thd, 0, global_var(ulonglong),
                                       typelib.type_names);
   }
@@ -1371,7 +1374,7 @@ class Sys_var_set : public Sys_var_typelib {
       if (!(res = var->value->val_str(&str)))
         return true;
       else {
-        char *error;
+        const char *error;
         uint error_len;
         bool not_used;
 
@@ -1418,11 +1421,12 @@ class Sys_var_set : public Sys_var_typelib {
     strcpy(def_val, set_to_string(thd, 0, var->save_result.ulonglong_value,
                                   typelib.type_names));
   }
-  uchar *session_value_ptr(THD *running_thd, THD *target_thd, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *running_thd, THD *target_thd,
+                                 LEX_STRING *) {
     return (uchar *)set_to_string(
         running_thd, 0, session_var(target_thd, ulonglong), typelib.type_names);
   }
-  uchar *global_value_ptr(THD *thd, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *thd, LEX_STRING *) {
     return (uchar *)set_to_string(thd, 0, global_var(ulonglong),
                                   typelib.type_names);
   }
@@ -1448,7 +1452,7 @@ class Sys_var_plugin : public sys_var {
   Sys_var_plugin(
       const char *name_arg, const char *comment, int flag_args, ptrdiff_t off,
       size_t size MY_ATTRIBUTE((unused)), CMD_LINE getopt, int plugin_type_arg,
-      char **def_val, PolyLock *lock = 0,
+      const char **def_val, PolyLock *lock = 0,
       enum binlog_status_enum binlog_status_arg = VARIABLE_NOT_IN_BINLOG,
       on_check_function on_check_func = 0,
       on_update_function on_update_func = 0, const char *substitute = 0,
@@ -1469,14 +1473,13 @@ class Sys_var_plugin : public sys_var {
     /* NULLs can't be used as a default storage engine */
     if (!(res = var->value->val_str(&str))) return true;
 
-    const LEX_STRING pname = {const_cast<char *>(res->ptr()), res->length()};
+    LEX_CSTRING pname_cstr = res->lex_cstring();
     plugin_ref plugin;
 
     // special code for storage engines (e.g. to handle historical aliases)
     if (plugin_type == MYSQL_STORAGE_ENGINE_PLUGIN)
-      plugin = ha_resolve_by_name(thd, &pname, false);
+      plugin = ha_resolve_by_name(thd, &pname_cstr, false);
     else {
-      LEX_CSTRING pname_cstr = {pname.str, pname.length};
       plugin = my_plugin_lock_by_name(thd, pname_cstr, plugin_type);
     }
 
@@ -1511,7 +1514,7 @@ class Sys_var_plugin : public sys_var {
     var->save_result.plugin = my_plugin_lock(thd, &plugin);
   }
   void global_save_default(THD *thd, set_var *var) {
-    LEX_STRING pname;
+    LEX_CSTRING pname;
     char **default_value = reinterpret_cast<char **>(option.def_value);
     pname.str = *default_value;
     pname.length = strlen(pname.str);
@@ -1520,8 +1523,7 @@ class Sys_var_plugin : public sys_var {
     if (plugin_type == MYSQL_STORAGE_ENGINE_PLUGIN)
       plugin = ha_resolve_by_name(thd, &pname, false);
     else {
-      LEX_CSTRING pname_cstr = {pname.str, pname.length};
-      plugin = my_plugin_lock_by_name(thd, pname_cstr, plugin_type);
+      plugin = my_plugin_lock_by_name(thd, pname, plugin_type);
     }
     DBUG_ASSERT(plugin);
 
@@ -1532,13 +1534,14 @@ class Sys_var_plugin : public sys_var {
             plugin_name(var->save_result.plugin)->length);
   }
   bool check_update_type(Item_result type) { return type != STRING_RESULT; }
-  uchar *session_value_ptr(THD *running_thd, THD *target_thd, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *running_thd, THD *target_thd,
+                                 LEX_STRING *) {
     plugin_ref plugin = session_var(target_thd, plugin_ref);
     return (uchar *)(plugin ? running_thd->strmake(plugin_name(plugin)->str,
                                                    plugin_name(plugin)->length)
                             : 0);
   }
-  uchar *global_value_ptr(THD *thd, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *thd, LEX_STRING *) {
     plugin_ref plugin = global_var(plugin_ref);
     return (uchar *)(plugin ? thd->strmake(plugin_name(plugin)->str,
                                            plugin_name(plugin)->length)
@@ -1590,10 +1593,10 @@ class Sys_var_debug_sync : public sys_var {
   }
   void global_save_default(THD *, set_var *) { DBUG_ASSERT(false); }
   void saved_value_to_string(THD *, set_var *, char *) { DBUG_ASSERT(false); }
-  uchar *session_value_ptr(THD *running_thd, THD *, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *running_thd, THD *, LEX_STRING *) {
     return debug_sync_value_ptr(running_thd);
   }
-  uchar *global_value_ptr(THD *, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *, LEX_STRING *) {
     DBUG_ASSERT(false);
     return 0;
   }
@@ -1665,13 +1668,14 @@ class Sys_var_bit : public Sys_var_typelib {
   void saved_value_to_string(THD *, set_var *var, char *def_val) {
     longlong10_to_str((longlong)var->save_result.ulonglong_value, def_val, 10);
   }
-  uchar *session_value_ptr(THD *running_thd, THD *target_thd, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *running_thd, THD *target_thd,
+                                 LEX_STRING *) {
     running_thd->sys_var_tmp.bool_value = static_cast<bool>(
         reverse_semantics ^
         ((session_var(target_thd, ulonglong) & bitmask) != 0));
     return (uchar *)&running_thd->sys_var_tmp.bool_value;
   }
-  uchar *global_value_ptr(THD *thd, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *thd, LEX_STRING *) {
     thd->sys_var_tmp.bool_value = static_cast<bool>(
         reverse_semantics ^ ((global_var(ulonglong) & bitmask) != 0));
     return (uchar *)&thd->sys_var_tmp.bool_value;
@@ -1725,13 +1729,14 @@ class Sys_var_session_special : public Sys_var_ulonglong {
   void session_save_default(THD *, set_var *var) { var->value = 0; }
   void global_save_default(THD *, set_var *) { DBUG_ASSERT(false); }
   void saved_value_to_string(THD *, set_var *, char *) { DBUG_ASSERT(false); }
-  uchar *session_value_ptr(THD *running_thd, THD *target_thd, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *running_thd, THD *target_thd,
+                                 LEX_STRING *) {
     running_thd->sys_var_tmp.ulonglong_value = read_func(target_thd);
     return (uchar *)&running_thd->sys_var_tmp.ulonglong_value;
   }
-  uchar *global_value_ptr(THD *, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *, LEX_STRING *) {
     DBUG_ASSERT(false);
-    return 0;
+    return nullptr;
   }
 };
 
@@ -1770,13 +1775,14 @@ class Sys_var_session_special_double : public Sys_var_double {
   void session_save_default(THD *, set_var *var) { var->value = 0; }
   void global_save_default(THD *, set_var *) { DBUG_ASSERT(false); }
   void saved_value_to_string(THD *, set_var *, char *) { DBUG_ASSERT(false); }
-  uchar *session_value_ptr(THD *running_thd, THD *target_thd, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *running_thd, THD *target_thd,
+                                 LEX_STRING *) {
     running_thd->sys_var_tmp.double_value = read_func(target_thd);
     return (uchar *)&running_thd->sys_var_tmp.double_value;
   }
-  uchar *global_value_ptr(THD *, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *, LEX_STRING *) {
     DBUG_ASSERT(false);
-    return 0;
+    return nullptr;
   }
 };
 
@@ -1825,12 +1831,13 @@ class Sys_var_have : public sys_var {
   void session_save_default(THD *, set_var *) {}
   void global_save_default(THD *, set_var *) {}
   void saved_value_to_string(THD *, set_var *, char *) {}
-  uchar *session_value_ptr(THD *, THD *, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *, THD *, LEX_STRING *) {
     DBUG_ASSERT(false);
-    return 0;
+    return nullptr;
   }
-  uchar *global_value_ptr(THD *, LEX_STRING *) {
-    return (uchar *)show_comp_option_name[global_var(enum SHOW_COMP_OPTION)];
+  const uchar *global_value_ptr(THD *, LEX_STRING *) {
+    return pointer_cast<const uchar *>(
+        show_comp_option_name[global_var(enum SHOW_COMP_OPTION)]);
   }
   bool check_update_type(Item_result) { return false; }
 };
@@ -1861,8 +1868,8 @@ class Sys_var_have_func : public Sys_var_have {
                      READ_ONLY NON_PERSIST GLOBAL_VAR(dummy_), NO_CMD_LINE),
         func_(func) {}
 
-  uchar *global_value_ptr(THD *thd, LEX_STRING *) {
-    return (uchar *)show_comp_option_name[func_(thd)];
+  const uchar *global_value_ptr(THD *thd, LEX_STRING *) {
+    return pointer_cast<const uchar *>(show_comp_option_name[func_(thd)]);
   }
 
  protected:
@@ -1927,17 +1934,19 @@ class Sys_var_struct : public sys_var {
     var->save_result.ptr = *default_value;
   }
   void saved_value_to_string(THD *, set_var *var, char *def_val) {
-    const Struct_type *ptr = (Struct_type *)var->save_result.ptr;
-    if (ptr) strcpy(def_val, (char *)Name_getter(ptr).get_name());
+    const Struct_type *ptr =
+        static_cast<const Struct_type *>(var->save_result.ptr);
+    if (ptr)
+      strcpy(def_val, pointer_cast<const char *>(Name_getter(ptr).get_name()));
   }
   bool check_update_type(Item_result type) {
     return type != INT_RESULT && type != STRING_RESULT;
   }
-  uchar *session_value_ptr(THD *, THD *target_thd, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *, THD *target_thd, LEX_STRING *) {
     const Struct_type *ptr = session_var(target_thd, const Struct_type *);
     return ptr ? Name_getter(ptr).get_name() : nullptr;
   }
-  uchar *global_value_ptr(THD *, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *, LEX_STRING *) {
     const Struct_type *ptr = global_var(const Struct_type *);
     return ptr ? Name_getter(ptr).get_name() : nullptr;
   }
@@ -1968,6 +1977,7 @@ class Sys_var_tz : public sys_var {
                 parse_flag) {
     DBUG_ASSERT(getopt.id == -1);
     DBUG_ASSERT(size == sizeof(Time_zone *));
+    option.var_type = GET_STR;
   }
   bool do_check(THD *thd, set_var *var) {
     char buff[MAX_TIME_ZONE_NAME_LENGTH];
@@ -1998,9 +2008,9 @@ class Sys_var_tz : public sys_var {
     var->save_result.time_zone = *(Time_zone **)(intptr)option.def_value;
   }
   void saved_value_to_string(THD *, set_var *var, char *def_val) {
-    strcpy(def_val, (char *)var->save_result.time_zone->get_name()->ptr());
+    strcpy(def_val, var->save_result.time_zone->get_name()->ptr());
   }
-  uchar *session_value_ptr(THD *, THD *target_thd, LEX_STRING *) {
+  const uchar *session_value_ptr(THD *, THD *target_thd, LEX_STRING *) {
     /*
       This is an ugly fix for replication: we don't replicate properly queries
       invoking system variables' values to update tables; but
@@ -2009,11 +2019,13 @@ class Sys_var_tz : public sys_var {
       timezone). If it's the global value which was used we can't replicate
       (binlog code stores session value only).
     */
-    target_thd->time_zone_used = 1;
-    return (uchar *)(session_var(target_thd, Time_zone *)->get_name()->ptr());
+    target_thd->time_zone_used = true;
+    return pointer_cast<const uchar *>(
+        session_var(target_thd, Time_zone *)->get_name()->ptr());
   }
-  uchar *global_value_ptr(THD *, LEX_STRING *) {
-    return (uchar *)(global_var(Time_zone *)->get_name()->ptr());
+  const uchar *global_value_ptr(THD *, LEX_STRING *) {
+    return pointer_cast<const uchar *>(
+        global_var(Time_zone *)->get_name()->ptr());
   }
   bool check_update_type(Item_result type) { return type != STRING_RESULT; }
 };
@@ -2102,29 +2114,30 @@ class Sys_var_gtid_next : public sys_var {
     return true;
   }
   void session_save_default(THD *, set_var *var) {
-    DBUG_ENTER("Sys_var_gtid_next::session_save_default");
+    DBUG_TRACE;
     char *ptr = (char *)(intptr)option.def_value;
     var->save_result.string_value.str = ptr;
     var->save_result.string_value.length = ptr ? strlen(ptr) : 0;
-    DBUG_VOID_RETURN;
+    return;
   }
   void global_save_default(THD *, set_var *) { DBUG_ASSERT(false); }
   void saved_value_to_string(THD *, set_var *, char *) { DBUG_ASSERT(false); }
   bool do_check(THD *, set_var *) { return false; }
   bool check_update_type(Item_result type) { return type != STRING_RESULT; }
-  uchar *session_value_ptr(THD *running_thd, THD *target_thd, LEX_STRING *) {
-    DBUG_ENTER("Sys_var_gtid_next::session_value_ptr");
+  const uchar *session_value_ptr(THD *running_thd, THD *target_thd,
+                                 LEX_STRING *) {
+    DBUG_TRACE;
     char buf[Gtid_specification::MAX_TEXT_LENGTH + 1];
     global_sid_lock->rdlock();
     ((Gtid_specification *)session_var_ptr(target_thd))
         ->to_string(global_sid_map, buf);
     global_sid_lock->unlock();
     char *ret = running_thd->mem_strdup(buf);
-    DBUG_RETURN((uchar *)ret);
+    return (uchar *)ret;
   }
-  uchar *global_value_ptr(THD *, LEX_STRING *) {
+  const uchar *global_value_ptr(THD *, LEX_STRING *) {
     DBUG_ASSERT(false);
-    return NULL;
+    return nullptr;
   }
 };
 
@@ -2158,41 +2171,41 @@ class Sys_var_gtid_set : public sys_var {
     return true;
   }
   void session_save_default(THD *thd, set_var *var) {
-    DBUG_ENTER("Sys_var_gtid_set::session_save_default");
+    DBUG_TRACE;
     global_sid_lock->rdlock();
     char *ptr = (char *)(intptr)option.def_value;
     var->save_result.string_value.str = ptr;
     var->save_result.string_value.length = ptr ? strlen(ptr) : 0;
     global_sid_lock->unlock();
-    DBUG_VOID_RETURN;
+    return;
   }
   void global_save_default(THD *thd, set_var *var) { DBUG_ASSERT(false); }
   void saved_value_to_string(THD *, set_var *, char *) { DBUG_ASSERT(false); }
   bool do_check(THD *thd, set_var *var) {
-    DBUG_ENTER("Sys_var_gtid_set::do_check");
+    DBUG_TRACE;
     String str;
     String *res = var->value->val_str(&str);
     if (res == NULL) {
       var->save_result.string_value.str = NULL;
-      DBUG_RETURN(false);
+      return false;
     }
     DBUG_ASSERT(res->ptr() != NULL);
     var->save_result.string_value.str = thd->strmake(res->ptr(), res->length());
     if (var->save_result.string_value.str == NULL) {
       my_error(ER_OUT_OF_RESOURCES, MYF(0));  // thd->strmake failed
-      DBUG_RETURN(1);
+      return 1;
     }
     var->save_result.string_value.length = res->length();
     bool ret = !Gtid_set::is_valid(res->ptr());
-    DBUG_RETURN(ret);
+    return ret;
   }
   bool check_update_type(Item_result type) { return type != STRING_RESULT; }
   uchar *session_value_ptr(THD *running_thd, THD *target_thd,
                            LEX_STRING *base) {
-    DBUG_ENTER("Sys_var_gtid_set::session_value_ptr");
+    DBUG_TRACE;
     Gtid_set_or_null *gsn = (Gtid_set_or_null *)session_var_ptr(target_thd);
     Gtid_set *gs = gsn->get_gtid_set();
-    if (gs == NULL) DBUG_RETURN(NULL);
+    if (gs == NULL) return NULL;
     char *buf;
     global_sid_lock->rdlock();
     buf = (char *)running_thd->alloc(gs->get_string_length() + 1);
@@ -2201,7 +2214,7 @@ class Sys_var_gtid_set : public sys_var {
     else
       my_error(ER_OUT_OF_RESOURCES, MYF(0));  // thd->alloc failed
     global_sid_lock->unlock();
-    DBUG_RETURN((uchar *)buf);
+    return (uchar *)buf;
   }
   uchar *global_value_ptr(THD *thd, LEX_STRING *base) {
     DBUG_ASSERT(false);
@@ -2249,13 +2262,13 @@ class Sys_var_charptr_func : public sys_var {
     DBUG_ASSERT(false);
     return true;
   }
-  virtual uchar *session_value_ptr(THD *, THD *, LEX_STRING *) {
+  virtual const uchar *session_value_ptr(THD *, THD *, LEX_STRING *) {
     DBUG_ASSERT(false);
-    return NULL;
+    return nullptr;
   }
-  virtual uchar *global_value_ptr(THD *, LEX_STRING *) {
+  virtual const uchar *global_value_ptr(THD *, LEX_STRING *) {
     DBUG_ASSERT(false);
-    return NULL;
+    return nullptr;
   }
 };
 
@@ -2267,8 +2280,8 @@ class Sys_var_gtid_executed : Sys_var_charptr_func {
   Sys_var_gtid_executed(const char *name_arg, const char *comment_arg)
       : Sys_var_charptr_func(name_arg, comment_arg, GLOBAL) {}
 
-  uchar *global_value_ptr(THD *thd, LEX_STRING *) {
-    DBUG_ENTER("Sys_var_gtid_executed::global_value_ptr");
+  const uchar *global_value_ptr(THD *thd, LEX_STRING *) {
+    DBUG_TRACE;
     global_sid_lock->wrlock();
     const Gtid_set *gs = gtid_state->get_executed_gtids();
     char *buf = (char *)thd->alloc(gs->get_string_length() + 1);
@@ -2277,7 +2290,7 @@ class Sys_var_gtid_executed : Sys_var_charptr_func {
     else
       gs->to_string(buf);
     global_sid_lock->unlock();
-    DBUG_RETURN((uchar *)buf);
+    return (uchar *)buf;
   }
 };
 
@@ -2316,28 +2329,28 @@ class Sys_var_gtid_purged : public sys_var {
   }
 
   bool do_check(THD *thd, set_var *var) {
-    DBUG_ENTER("Sys_var_gtid_purged::do_check");
+    DBUG_TRACE;
     char buf[1024];
     String str(buf, sizeof(buf), system_charset_info);
     String *res = var->value->val_str(&str);
-    if (!res) DBUG_RETURN(true);
+    if (!res) return true;
     var->save_result.string_value.str =
         thd->strmake(res->c_ptr(), res->length());
     if (!var->save_result.string_value.str) {
       my_error(ER_OUT_OF_RESOURCES, MYF(0));  // thd->strmake failed
-      DBUG_RETURN(true);
+      return true;
     }
     var->save_result.string_value.length = res->length();
     bool ret =
         Gtid_set::is_valid(var->save_result.string_value.str) ? false : true;
     DBUG_PRINT("info", ("ret=%d", ret));
-    DBUG_RETURN(ret);
+    return ret;
   }
 
   bool check_update_type(Item_result type) { return type != STRING_RESULT; }
 
-  uchar *global_value_ptr(THD *thd, LEX_STRING *) {
-    DBUG_ENTER("Sys_var_gtid_purged::global_value_ptr");
+  const uchar *global_value_ptr(THD *thd, LEX_STRING *) {
+    DBUG_TRACE;
     const Gtid_set *gs;
     global_sid_lock->wrlock();
     if (opt_bin_log)
@@ -2357,12 +2370,12 @@ class Sys_var_gtid_purged : public sys_var {
     else
       gs->to_string(buf);
     global_sid_lock->unlock();
-    DBUG_RETURN((uchar *)buf);
+    return (uchar *)buf;
   }
 
-  uchar *session_value_ptr(THD *, THD *, LEX_STRING *) {
-    DBUG_ASSERT(0);
-    return NULL;
+  const uchar *session_value_ptr(THD *, THD *, LEX_STRING *) {
+    DBUG_ASSERT(false);
+    return nullptr;
   }
 };
 
@@ -2372,16 +2385,17 @@ class Sys_var_gtid_owned : Sys_var_charptr_func {
       : Sys_var_charptr_func(name_arg, comment_arg, SESSION) {}
 
  public:
-  uchar *session_value_ptr(THD *running_thd, THD *target_thd, LEX_STRING *) {
-    DBUG_ENTER("Sys_var_gtid_owned::session_value_ptr");
+  const uchar *session_value_ptr(THD *running_thd, THD *target_thd,
+                                 LEX_STRING *) {
+    DBUG_TRACE;
     char *buf = NULL;
     bool remote = (target_thd != running_thd);
 
     if (target_thd->owned_gtid.sidno == 0)
-      DBUG_RETURN((uchar *)running_thd->mem_strdup(""));
+      return (uchar *)running_thd->mem_strdup("");
     else if (target_thd->owned_gtid.sidno == THD::OWNED_SIDNO_ANONYMOUS) {
       DBUG_ASSERT(gtid_state->get_anonymous_ownership_count() > 0);
-      DBUG_RETURN((uchar *)running_thd->mem_strdup("ANONYMOUS"));
+      return (uchar *)running_thd->mem_strdup("ANONYMOUS");
     } else if (target_thd->owned_gtid.sidno == THD::OWNED_SIDNO_GTID_SET) {
 #ifdef HAVE_GTID_NEXT_LIST
       buf = (char *)running_thd->alloc(
@@ -2405,11 +2419,11 @@ class Sys_var_gtid_owned : Sys_var_charptr_func {
       } else
         my_error(ER_OUT_OF_RESOURCES, MYF(0));
     }
-    DBUG_RETURN((uchar *)buf);
+    return (uchar *)buf;
   }
 
-  uchar *global_value_ptr(THD *thd, LEX_STRING *) {
-    DBUG_ENTER("Sys_var_gtid_owned::global_value_ptr");
+  const uchar *global_value_ptr(THD *thd, LEX_STRING *) {
+    DBUG_TRACE;
     const Owned_gtids *owned_gtids = gtid_state->get_owned_gtids();
     global_sid_lock->wrlock();
     char *buf = (char *)thd->alloc(owned_gtids->get_max_string_length());
@@ -2418,7 +2432,7 @@ class Sys_var_gtid_owned : Sys_var_charptr_func {
     else
       my_error(ER_OUT_OF_RESOURCES, MYF(0));  // thd->alloc failed
     global_sid_lock->unlock();
-    DBUG_RETURN((uchar *)buf);
+    return (uchar *)buf;
   }
 };
 
