@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -61,8 +60,11 @@
   Helper: Tell tracker (if any) that transaction ended.
 */
 void trans_track_end_trx(THD *thd) {
-  TX_TRACKER_GET(tst);
-  tst->end_trx(thd);
+  if (thd->variables.session_track_transaction_info > TX_TRACK_NONE) {
+    ((Transaction_state_tracker *)thd->session_tracker.get_tracker(
+         TRANSACTION_INFO_TRACKER))
+        ->end_trx(thd);
+  }
 }
 
 /**
@@ -71,7 +73,10 @@ void trans_track_end_trx(THD *thd) {
 */
 void trans_reset_one_shot_chistics(THD *thd) {
   if (thd->variables.session_track_transaction_info > TX_TRACK_NONE) {
-    TX_TRACKER_GET(tst);
+    Transaction_state_tracker *tst =
+        (Transaction_state_tracker *)thd->session_tracker.get_tracker(
+            TRANSACTION_INFO_TRACKER);
+
     tst->set_read_flags(thd, TX_READ_INHERIT);
     tst->set_isol_level(thd, TX_ISOL_INHERIT);
   }
@@ -97,7 +102,7 @@ bool trans_check_state(THD *thd) {
     Always commit statement transaction before manipulating with
     the normal one.
   */
-  assert(thd->get_transaction()->is_empty(Transaction_ctx::STMT));
+  DBUG_ASSERT(thd->get_transaction()->is_empty(Transaction_ctx::STMT));
 
   if (unlikely(thd->in_sub_stmt)) {
     my_error(ER_COMMIT_NOT_ALLOWED_IN_SF_OR_TRG, MYF(0));
@@ -124,15 +129,19 @@ bool trans_check_state(THD *thd) {
 
 bool trans_begin(THD *thd, uint flags) {
   bool res = false;
+  Transaction_state_tracker *tst = NULL;
+
   DBUG_TRACE;
 
   if (trans_check_state(thd)) return true;
 
-  TX_TRACKER_GET(tst);
+  if (thd->variables.session_track_transaction_info > TX_TRACK_NONE)
+    tst = (Transaction_state_tracker *)thd->session_tracker.get_tracker(
+        TRANSACTION_INFO_TRACKER);
 
   thd->locked_tables_list.unlock_locked_tables(thd);
 
-  assert(!thd->locked_tables_mode);
+  DBUG_ASSERT(!thd->locked_tables_mode);
 
   if (thd->in_multi_stmt_transaction_mode() ||
       (thd->variables.option_bits & OPTION_TABLE_LOCK)) {
@@ -155,8 +164,8 @@ bool trans_begin(THD *thd, uint flags) {
   thd->mdl_context.release_transactional_locks();
 
   // The RO/RW options are mutually exclusive.
-  assert(!((flags & MYSQL_START_TRANS_OPT_READ_ONLY) &&
-           (flags & MYSQL_START_TRANS_OPT_READ_WRITE)));
+  DBUG_ASSERT(!((flags & MYSQL_START_TRANS_OPT_READ_ONLY) &&
+                (flags & MYSQL_START_TRANS_OPT_READ_WRITE)));
   if (flags & MYSQL_START_TRANS_OPT_READ_ONLY) {
     thd->tx_read_only = true;
     if (tst) tst->set_read_flags(thd, TX_READ_ONLY);
@@ -177,7 +186,7 @@ bool trans_begin(THD *thd, uint flags) {
   }
 
   DBUG_EXECUTE_IF("dbug_set_high_prio_trx", {
-    assert(thd->tx_priority == 0);
+    DBUG_ASSERT(thd->tx_priority == 0);
     thd->tx_priority = 1;
   });
 
@@ -204,9 +213,9 @@ bool trans_begin(THD *thd, uint flags) {
     BEGIN but do not touch any transactional tables.
   */
 #ifdef HAVE_PSI_TRANSACTION_INTERFACE
-  if (thd->m_transaction_psi == nullptr) {
+  if (thd->m_transaction_psi == NULL) {
     thd->m_transaction_psi =
-        MYSQL_START_TRANSACTION(&thd->m_transaction_state, nullptr, nullptr,
+        MYSQL_START_TRANSACTION(&thd->m_transaction_state, NULL, NULL,
                                 thd->tx_isolation, thd->tx_read_only, false);
     DEBUG_SYNC(thd, "after_set_transaction_psi_before_set_transaction_gtid");
     gtid_set_performance_schema_values(thd);
@@ -234,11 +243,6 @@ bool trans_commit(THD *thd, bool ignore_global_read_lock) {
   int res;
   DBUG_TRACE;
 
-  DBUG_EXECUTE_IF(
-      "crash_on_transactional_ddl_commit",
-      if (thd->m_transactional_ddl.inited() &&
-          thd->lex->sql_command == SQLCOM_COMMIT) { DBUG_SUICIDE(); });
-
   if (trans_check_state(thd)) return true;
 
   thd->server_status &=
@@ -268,7 +272,7 @@ bool trans_commit(THD *thd, bool ignore_global_read_lock) {
   thd->lex->start_transaction_opt = 0;
 
   /* The transaction should be marked as complete in P_S. */
-  assert(thd->m_transaction_psi == nullptr);
+  DBUG_ASSERT(thd->m_transaction_psi == NULL);
 
   thd->tx_priority = 0;
 
@@ -294,8 +298,6 @@ bool trans_commit(THD *thd, bool ignore_global_read_lock) {
 
   thd->locked_tables_list.adjust_renamed_tablespace_mdls(&thd->mdl_context);
 
-  thd->m_transactional_ddl.post_ddl();
-
   return res;
 }
 
@@ -325,9 +327,9 @@ bool trans_commit_implicit(THD *thd, bool ignore_global_read_lock) {
     by asserting that conditions that are checked in the former function are
     true.
   */
-  assert(thd->get_transaction()->is_empty(Transaction_ctx::STMT) &&
-         !thd->in_sub_stmt &&
-         !thd->get_transaction()->xid_state()->check_in_xa(false));
+  DBUG_ASSERT(thd->get_transaction()->is_empty(Transaction_ctx::STMT) &&
+              !thd->in_sub_stmt &&
+              !thd->get_transaction()->xid_state()->check_in_xa(false));
 
   if (thd->in_multi_stmt_transaction_mode() ||
       (thd->variables.option_bits & OPTION_TABLE_LOCK)) {
@@ -349,7 +351,7 @@ bool trans_commit_implicit(THD *thd, bool ignore_global_read_lock) {
   thd->get_transaction()->reset_unsafe_rollback_flags(Transaction_ctx::SESSION);
 
   /* The transaction should be marked as complete in P_S. */
-  assert(thd->m_transaction_psi == nullptr);
+  DBUG_ASSERT(thd->m_transaction_psi == NULL);
 
   /*
     Upon implicit commit, reset the current transaction
@@ -398,8 +400,6 @@ bool trans_rollback(THD *thd) {
 
   if (trans_check_state(thd)) return true;
 
-  thd->m_transactional_ddl.rollback();
-
   thd->server_status &=
       ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
   DBUG_PRINT("info", ("clearing SERVER_STATUS_IN_TRANS"));
@@ -409,7 +409,7 @@ bool trans_rollback(THD *thd) {
   thd->lex->start_transaction_opt = 0;
 
   /* The transaction should be marked as complete in P_S. */
-  assert(thd->m_transaction_psi == nullptr);
+  DBUG_ASSERT(thd->m_transaction_psi == NULL);
 
   thd->tx_priority = 0;
 
@@ -427,8 +427,6 @@ bool trans_rollback(THD *thd) {
 
   thd->locked_tables_list.discard_renamed_tablespace_mdls();
 
-  thd->m_transactional_ddl.post_ddl();
-
   return res;
 }
 
@@ -457,8 +455,8 @@ bool trans_rollback_implicit(THD *thd) {
     Don't perform rollback in the middle of sub-statement, wait till
     its end.
   */
-  assert(thd->get_transaction()->is_empty(Transaction_ctx::STMT) &&
-         !thd->in_sub_stmt);
+  DBUG_ASSERT(thd->get_transaction()->is_empty(Transaction_ctx::STMT) &&
+              !thd->in_sub_stmt);
 
   thd->server_status &=
       ~(SERVER_STATUS_IN_TRANS | SERVER_STATUS_IN_TRANS_READONLY);
@@ -468,9 +466,9 @@ bool trans_rollback_implicit(THD *thd) {
   thd->get_transaction()->reset_unsafe_rollback_flags(Transaction_ctx::SESSION);
 
   /* Rollback should clear transaction_rollback_request flag. */
-  assert(!thd->transaction_rollback_request);
+  DBUG_ASSERT(!thd->transaction_rollback_request);
   /* The transaction should be marked as complete in P_S. */
-  assert(thd->m_transaction_psi == nullptr);
+  DBUG_ASSERT(thd->m_transaction_psi == NULL);
 
   trans_track_end_trx(thd);
 
@@ -519,13 +517,13 @@ bool trans_commit_stmt(THD *thd, bool ignore_global_read_lock) {
     a savepoint for each nested statement, and release the
     savepoint when statement has succeeded.
   */
-  assert(!thd->in_sub_stmt);
+  DBUG_ASSERT(!thd->in_sub_stmt);
 
   /*
     Some code in MYSQL_BIN_LOG::commit and ha_commit_low() is not safe
     for attachable transactions.
   */
-  assert(!thd->is_attachable_ro_transaction_active());
+  DBUG_ASSERT(!thd->is_attachable_ro_transaction_active());
 
   thd->get_transaction()->merge_unsafe_rollback_flags();
 
@@ -540,8 +538,8 @@ bool trans_commit_stmt(THD *thd, bool ignore_global_read_lock) {
             thd))
       LogErr(WARNING_LEVEL, ER_TRX_GTID_COLLECT_REJECT);
   /* In autocommit=1 mode the transaction should be marked as complete in P_S */
-  assert(thd->in_active_multi_stmt_transaction() ||
-         thd->m_transaction_psi == nullptr);
+  DBUG_ASSERT(thd->in_active_multi_stmt_transaction() ||
+              thd->m_transaction_psi == NULL);
 
   thd->get_transaction()->reset(Transaction_ctx::STMT);
 
@@ -565,13 +563,13 @@ bool trans_rollback_stmt(THD *thd) {
     a savepoint for each nested statement, and release the
     savepoint when statement has succeeded.
   */
-  assert(!thd->in_sub_stmt);
+  DBUG_ASSERT(!thd->in_sub_stmt);
 
   /*
     Some code in MYSQL_BIN_LOG::rollback and ha_rollback_low() is not safe
     for attachable transactions.
   */
-  assert(!thd->is_attachable_ro_transaction_active());
+  DBUG_ASSERT(!thd->is_attachable_ro_transaction_active());
 
   thd->get_transaction()->merge_unsafe_rollback_flags();
 
@@ -613,10 +611,10 @@ bool trans_rollback_stmt(THD *thd) {
   if (is_atomic_ddl_commit_on_slave(thd)) thd->rli_slave->post_rollback();
 
   /* In autocommit=1 mode the transaction should be marked as complete in P_S */
-  assert(thd->in_active_multi_stmt_transaction() ||
-         thd->m_transaction_psi == nullptr ||
-         /* Todo: BUG#20488921 is in the way. */
-         DBUG_EVALUATE_IF("simulate_xa_commit_log_failure", true, false));
+  DBUG_ASSERT(thd->in_active_multi_stmt_transaction() ||
+              thd->m_transaction_psi == NULL ||
+              /* Todo: BUG#20488921 is in the way. */
+              DBUG_EVALUATE_IF("simulate_xa_commit_log_failure", true, false));
 
   thd->get_transaction()->reset(Transaction_ctx::STMT);
 
@@ -640,23 +638,23 @@ bool trans_commit_attachable(THD *thd) {
   int res = 0;
 
   /* This function only handles attachable transactions. */
-  assert(thd->is_attachable_ro_transaction_active());
+  DBUG_ASSERT(thd->is_attachable_ro_transaction_active());
 
   /*
     Since the attachable transaction is AUTOCOMMIT we only need to commit
     statement transaction.
   */
-  assert(!thd->get_transaction()->is_active(Transaction_ctx::SESSION));
+  DBUG_ASSERT(!thd->get_transaction()->is_active(Transaction_ctx::SESSION));
 
   /* Attachable transactions should not do anything unsafe. */
-  assert(
+  DBUG_ASSERT(
       !thd->get_transaction()->cannot_safely_rollback(Transaction_ctx::STMT));
 
   if (thd->get_transaction()->is_active(Transaction_ctx::STMT)) {
     res = ha_commit_attachable(thd);
   }
 
-  assert(thd->m_transaction_psi == nullptr);
+  DBUG_ASSERT(thd->m_transaction_psi == NULL);
 
   thd->get_transaction()->reset(Transaction_ctx::STMT);
 
@@ -704,12 +702,12 @@ bool trans_savepoint(THD *thd, LEX_STRING name) {
   {
     newsv = *sv;
     if (ha_release_savepoint(thd, *sv)) {
-      assert(thd->is_error() || thd->is_killed());
+      DBUG_ASSERT(thd->is_error() || thd->is_killed());
       return true;
     }
     *sv = (*sv)->prev;
   } else if ((newsv = (SAVEPOINT *)thd->get_transaction()->allocate_memory(
-                  savepoint_alloc_size)) == nullptr) {
+                  savepoint_alloc_size)) == NULL) {
     my_error(ER_OUT_OF_RESOURCES, MYF(0));
     return true;
   }
@@ -768,7 +766,7 @@ bool trans_rollback_to_savepoint(THD *thd, LEX_STRING name) {
   SAVEPOINT *sv = *find_savepoint(thd, name);
   DBUG_TRACE;
 
-  if (sv == nullptr) {
+  if (sv == NULL) {
     my_error(ER_SP_DOES_NOT_EXIST, MYF(0), "SAVEPOINT", name.str);
     return true;
   }
@@ -800,7 +798,7 @@ bool trans_rollback_to_savepoint(THD *thd, LEX_STRING name) {
       log accompanied with ROLLBACK TO SAVEPOINT statement. Since the real
       write happens at the end of transaction releasing MDL on tables
       mentioned in these events (i.e. acquired after savepoint and before
-      rollback of it) can break replication, as concurrent DROP TABLES
+      rollback ot it) can break replication, as concurrent DROP TABLES
       statements will be able to drop these tables before events will get
       into binary log,
   */
@@ -836,7 +834,7 @@ bool trans_release_savepoint(THD *thd, LEX_STRING name) {
   SAVEPOINT *sv = *find_savepoint(thd, name);
   DBUG_TRACE;
 
-  if (sv == nullptr) {
+  if (sv == NULL) {
     my_error(ER_SP_DOES_NOT_EXIST, MYF(0), "SAVEPOINT", name.str);
     return true;
   }
