@@ -1,16 +1,15 @@
-/* Copyright (c) 2001, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,8 +50,8 @@
 #include "my_dbug.h"
 #include "my_io.h"
 #include "my_tree.h"  // element_count
-#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/psi/mysql_file.h"
+#include "mysql/psi/psi_base.h"
 #include "mysql/service_mysql_alloc.h"
 #include "priority_queue.h"
 #include "sql/field.h"    // Field
@@ -138,7 +137,7 @@ static uint uniq_read_to_buffer(IO_CACHE *fromfile, Merge_chunk *merge_chunk,
 /**
   Merge buffers to one buffer.
 
-  @param thd            thread context
+  @param thd
   @param param          Sort parameter
   @param from_file      File with source data (Merge_chunks point to this file)
   @param to_file        File to write the sorted result data.
@@ -148,13 +147,15 @@ static uint uniq_read_to_buffer(IO_CACHE *fromfile, Merge_chunk *merge_chunk,
   @param chunk_array    Array of chunks to merge.
   @param flag           0 - write full record, 1 - write addon/ref
 
-  @retval 0      OK
-  @retval other  error
+  @returns
+    0      OK
+  @returns
+    other  error
 */
 static int merge_buffers(THD *thd, Uniq_param *param, IO_CACHE *from_file,
                          IO_CACHE *to_file, Sort_buffer sort_buffer,
                          Merge_chunk *last_chunk, Merge_chunk_array chunk_array,
-                         int flag [[maybe_unused]]) {
+                         int flag MY_ATTRIBUTE((unused))) {
   int error = 0;
   uint rec_length;
   ha_rows maxcount;
@@ -177,9 +178,9 @@ static int merge_buffers(THD *thd, Uniq_param *param, IO_CACHE *from_file,
   org_max_rows = max_rows = param->max_rows;
 
   /* The following will fire if there is not enough space in sort_buffer */
-  assert(maxcount != 0);
+  DBUG_ASSERT(maxcount != 0);
 
-  assert(param->unique_buff != nullptr);
+  DBUG_ASSERT(param->unique_buff != nullptr);
 
   cmp = param->compare;
   first_cmp_arg = &param->cmp_context;
@@ -192,8 +193,8 @@ static int merge_buffers(THD *thd, Uniq_param *param, IO_CACHE *from_file,
   Priority_queue<Merge_chunk *,
                  std::vector<Merge_chunk *, Malloc_allocator<Merge_chunk *>>,
                  decltype(greater)>
-      queue(greater,
-            Malloc_allocator<Merge_chunk *>(key_memory_Filesort_info_merge));
+  queue(greater,
+        Malloc_allocator<Merge_chunk *>(key_memory_Filesort_info_merge));
 
   if (queue.reserve(chunk_array.size())) return 1;
 
@@ -356,11 +357,11 @@ Unique::Unique(qsort2_cmp comp_func, void *comp_func_fixed_arg, uint size_arg,
     : file_ptrs(PSI_INSTRUMENT_ME),
       max_elements(0),
       max_in_memory_size(max_in_memory_size_arg),
-      record_pointers(nullptr),
+      record_pointers(NULL),
       size(size_arg),
       elements(0) {
   my_b_clear(&file);
-  init_tree(&tree,
+  init_tree(&tree, /* default_alloc_size */ 0,
             /* memory_limit */ 0, size, comp_func,
             /* with_delete */ false,
             /* free_element */ nullptr, comp_func_fixed_arg);
@@ -413,10 +414,10 @@ static inline double log2_n_fact(ulong n) {
 
   SYNOPSIS
     get_merge_buffers_cost()
-      total_buf_elems   Number of elements to merge in all
-      n_buffers         Number of different buffers the elements are sprea dover
-      elem_size         Size of element stored in buffer
-      cost_model        Cost model to use for constants
+      buff_elems  Array of #s of elements in buffers
+      elem_size   Size of element stored in buffer
+      first       Pointer to first merged element size
+      last        Pointer to last merged element size
 
   RETURN
     Cost of merge_buffers operation in disk seeks.
@@ -437,9 +438,16 @@ static inline double log2_n_fact(ulong n) {
       key_compare_cost(total_buf_elems * log2(n_buffers));
 */
 
-static double get_merge_buffers_cost(size_t total_buf_elems, size_t n_buffers,
-                                     uint elem_size,
+static double get_merge_buffers_cost(Unique::Imerge_cost_buf_type buff_elems,
+                                     uint elem_size, uint first, uint last,
                                      const Cost_model_table *cost_model) {
+  uint total_buf_elems = 0;
+  for (uint pbuf = first; pbuf <= last; pbuf++)
+    total_buf_elems += buff_elems[pbuf];
+  buff_elems[last] = total_buf_elems;
+
+  const size_t n_buffers = last - first + 1;
+
   const double io_ops =
       static_cast<double>(total_buf_elems * elem_size) / IO_SIZE;
   const double io_cost = cost_model->io_block_read_cost(io_ops);
@@ -458,15 +466,16 @@ static double get_merge_buffers_cost(size_t total_buf_elems, size_t n_buffers,
 
   SYNOPSIS
     get_merge_many_buffs_cost()
-      num_full_buffers       # of full buffers
-      elems_per_buffer       # of elements in each full buffers
-      additional_num_elems   # of elements in last buffer
-      elem_size              size of each buffer element
+      buffer        buffer space for temporary data, at least
+                    Unique::get_cost_calc_buff_size bytes
+      maxbuffer     # of full buffers
+      max_n_elems   # of elements in first maxbuffer buffers
+      last_n_elems  # of elements in last buffer
+      elem_size     size of buffer element
 
   NOTES
-    num_full_buffers+1 buffers are merged, where first "num_full_buffers"
-    buffers contain "elems_per_buffer" elements each and last buffer contains
-    "additional_num_elems" elements.
+    maxbuffer+1 buffers are merged, where first maxbuffer buffers contain
+    max_n_elems elements each and last buffer contains last_n_elems elements.
 
     The current implementation does a dumb simulation of merge_many_buffs
     function actions.
@@ -475,52 +484,43 @@ static double get_merge_buffers_cost(size_t total_buf_elems, size_t n_buffers,
     Cost of merge in disk seeks.
 */
 
-static double get_merge_many_buffs_cost(uint num_full_buffers,
-                                        uint elems_per_buffer,
-                                        uint additional_num_elems,
-                                        int elem_size,
+static double get_merge_many_buffs_cost(Unique::Imerge_cost_buf_type buffer,
+                                        uint maxbuffer, uint max_n_elems,
+                                        uint last_n_elems, int elem_size,
                                         const Cost_model_table *cost_model) {
-  if (num_full_buffers == 0 && additional_num_elems == 0) {
-    return 0.0;
-  }
+  uint i;
   double total_cost = 0.0;
+  Unique::Imerge_cost_buf_type buff_elems =
+      buffer; /* #s of elements in each of merged sequences */
 
-  // For uniformity, make sure additional_num_elems > 0, always.
-  if (additional_num_elems == 0) {
-    additional_num_elems = elems_per_buffer;
-    --num_full_buffers;
-  }
+  /*
+    Set initial state: first maxbuffer sequences contain max_n_elems elements
+    each, last sequence contains last_n_elems elements.
+  */
+  for (i = 0; i < maxbuffer; i++) buff_elems[i] = max_n_elems;
+  buff_elems[maxbuffer] = last_n_elems;
 
   /*
     Do it exactly as merge_many_buff function does, calling
     get_merge_buffers_cost to get cost of merge_buffers.
   */
-  while ((num_full_buffers + 1) > MERGEBUFF2) {
-    uint new_num_full_buffers = 0;
-    unsigned i;
-    for (i = 0; i < (num_full_buffers + 1) - MERGEBUFF * 3U / 2U;
-         i += MERGEBUFF) {
-      ++new_num_full_buffers;
+  if (maxbuffer >= MERGEBUFF2) {
+    while (maxbuffer >= MERGEBUFF2) {
+      uint lastbuff = 0;
+      for (i = 0; i <= maxbuffer - MERGEBUFF * 3U / 2U; i += MERGEBUFF) {
+        total_cost += get_merge_buffers_cost(buff_elems, elem_size, i,
+                                             i + MERGEBUFF - 1, cost_model);
+        lastbuff++;
+      }
+      total_cost += get_merge_buffers_cost(buff_elems, elem_size, i, maxbuffer,
+                                           cost_model);
+      maxbuffer = lastbuff;
     }
-    total_cost += new_num_full_buffers *
-                  get_merge_buffers_cost(elems_per_buffer * MERGEBUFF,
-                                         MERGEBUFF, elem_size, cost_model);
-
-    const uint leftover_full_buffers = num_full_buffers - i;
-    total_cost += get_merge_buffers_cost(
-        elems_per_buffer * leftover_full_buffers + additional_num_elems,
-        leftover_full_buffers + 1, elem_size, cost_model);
-    additional_num_elems += elems_per_buffer * leftover_full_buffers;
-
-    elems_per_buffer *= MERGEBUFF;
-    num_full_buffers = new_num_full_buffers;
   }
 
-  // Merge the final buffers (<= MERGEBUFF2) in one big merge.
-  total_cost += get_merge_buffers_cost(
-      elems_per_buffer * num_full_buffers + additional_num_elems,
-      num_full_buffers + 1, elem_size, cost_model);
-
+  /* Simulate final merge_buff call. */
+  total_cost +=
+      get_merge_buffers_cost(buff_elems, elem_size, 0, maxbuffer, cost_model);
   return total_cost;
 }
 
@@ -530,6 +530,8 @@ static double get_merge_many_buffs_cost(uint num_full_buffers,
 
   SYNOPSIS
     Unique::get_use_cost()
+      buffer    space for temporary data, use Unique::get_cost_calc_buff_size
+                to get # bytes needed.
       nkeys     #of elements in Unique
       key_size  size of each elements in bytes
       max_in_memory_size amount of memory Unique will be allowed to use
@@ -569,8 +571,8 @@ static double get_merge_many_buffs_cost(uint num_full_buffers,
       these will be random seeks.
 */
 
-double Unique::get_use_cost(uint nkeys, uint key_size,
-                            ulonglong max_in_memory_size,
+double Unique::get_use_cost(Imerge_cost_buf_type buffer, uint nkeys,
+                            uint key_size, ulonglong max_in_memory_size,
                             const Cost_model_table *cost_model) {
   ulong max_elements_in_tree;
   ulong last_tree_elems;
@@ -606,7 +608,7 @@ double Unique::get_use_cost(uint nkeys, uint key_size,
 
   /* Cost of merge */
   double merge_cost =
-      get_merge_many_buffs_cost(n_full_trees, max_elements_in_tree,
+      get_merge_many_buffs_cost(buffer, n_full_trees, max_elements_in_tree,
                                 last_tree_elems, key_size, cost_model);
   if (merge_cost < 0.0) return merge_cost;
 
@@ -655,7 +657,7 @@ void Unique::reset() {
   */
   if (elements) {
     file_ptrs.clear();
-    reinit_io_cache(&file, WRITE_CACHE, 0L, false, true);
+    reinit_io_cache(&file, WRITE_CACHE, 0L, 0, 1);
   }
   /*
     If table is used - finish index access and delete all records.
@@ -731,7 +733,7 @@ static bool merge_walk(uchar *merge_buffer, size_t merge_buffer_size,
                        IO_CACHE *file) {
   if (end <= begin ||
       merge_buffer_size < (ulong)(key_length * (end - begin + 1)))
-    return true;
+    return 1;
 
   Merge_chunk_compare_context compare_context = {compare, compare_arg};
   Priority_queue<Merge_chunk *,
@@ -739,7 +741,7 @@ static bool merge_walk(uchar *merge_buffer, size_t merge_buffer_size,
                  Merge_chunk_less>
       queue((Merge_chunk_less(compare_context)),
             (Malloc_allocator<Merge_chunk *>(key_memory_Unique_merge_buffer)));
-  if (queue.reserve(end - begin)) return true;
+  if (queue.reserve(end - begin)) return 1;
 
   /* we need space for one key when a piece of merge buffer is re-read */
   merge_buffer_size -= key_length;
@@ -768,7 +770,7 @@ static bool merge_walk(uchar *merge_buffer, size_t merge_buffer_size,
     top->set_max_keys(max_key_count_per_piece);
     bytes_read = uniq_read_to_buffer(file, top, &uniq_param);
     if (bytes_read == (uint)(-1)) goto end;
-    assert(bytes_read);
+    DBUG_ASSERT(bytes_read);
     queue.push(top);
   }
   top = queue.top();
@@ -860,10 +862,9 @@ bool Unique::walk(tree_walk_action action, void *walk_action_arg) {
     return tree_walk(&tree, action, walk_action_arg, left_root_right);
 
   /* flush current tree to the file to have some memory for merge buffer */
-  if (flush()) return true;
-  if (flush_io_cache(&file) ||
-      reinit_io_cache(&file, READ_CACHE, 0L, false, false))
-    return true;
+  if (flush()) return 1;
+  if (flush_io_cache(&file) || reinit_io_cache(&file, READ_CACHE, 0L, 0, 0))
+    return 1;
 
   /*
     Compute the size of the merge buffer used by merge_walk(). This buffer
@@ -876,7 +877,7 @@ bool Unique::walk(tree_walk_action action, void *walk_action_arg) {
 
   if (!(merge_buffer = (uchar *)my_malloc(key_memory_Unique_merge_buffer,
                                           merge_buffer_size, MYF(0))))
-    return true;
+    return 1;
   res = merge_walk(merge_buffer, merge_buffer_size, size, file_ptrs.begin(),
                    file_ptrs.end(), action, walk_action_arg, tree.compare,
                    tree.custom_arg, &file);
@@ -890,53 +891,52 @@ bool Unique::walk(tree_walk_action action, void *walk_action_arg) {
 */
 
 bool Unique::get(TABLE *table) {
-  THD *thd = current_thd;
   table->unique_result.found_records = elements + tree.elements_in_tree;
 
   if (my_b_tell(&file) == 0) {
     /* Whole tree is in memory;  Don't use disk if you don't need to */
-    assert(table->unique_result.sorted_result == nullptr);
+    DBUG_ASSERT(table->unique_result.sorted_result == NULL);
     table->unique_result.sorted_result.reset(
         (uchar *)my_malloc(key_memory_Filesort_info_record_pointers,
                            size * tree.elements_in_tree, MYF(0)));
     if ((record_pointers = table->unique_result.sorted_result.get())) {
       (void)tree_walk(&tree, unique_write_to_ptrs, this, left_root_right);
-      return false;
+      return 0;
     }
   }
   /* Not enough memory; Save the result to file && free memory used by tree */
-  if (flush()) return true;
+  if (flush()) return 1;
 
   IO_CACHE *outfile = table->unique_result.io_cache;
   Merge_chunk *file_ptr = file_ptrs.begin();
   size_t num_chunks = file_ptrs.size();
   uchar *sort_memory;
   my_off_t save_pos;
-  bool error = true;
+  bool error = 1;
 
   /* Open cached file if it isn't open */
-  assert(table->unique_result.io_cache == nullptr);
+  DBUG_ASSERT(table->unique_result.io_cache == NULL);
   outfile = table->unique_result.io_cache = (IO_CACHE *)my_malloc(
       key_memory_TABLE_sort_io_cache, sizeof(IO_CACHE), MYF(MY_ZEROFILL));
 
   if (!outfile || (!my_b_inited(outfile) &&
                    open_cached_file(outfile, mysql_tmpdir, TEMP_PREFIX,
                                     READ_RECORD_BUFFER, MYF(MY_WME))))
-    return true;
-  reinit_io_cache(outfile, WRITE_CACHE, 0L, false, false);
+    return 1;
+  reinit_io_cache(outfile, WRITE_CACHE, 0L, 0, 0);
 
   Uniq_param uniq_param;
   uniq_param.max_rows = elements;
   uniq_param.rec_length = size;
   uniq_param.max_keys_per_buffer =
       static_cast<uint>(max_in_memory_size / uniq_param.rec_length);
-  uniq_param.not_killable = true;
+  uniq_param.not_killable = 1;
 
   const size_t num_bytes =
       (uniq_param.max_keys_per_buffer + 1) * uniq_param.rec_length;
   if (!(sort_memory = (uchar *)my_malloc(key_memory_Unique_sort_buffer,
                                          num_bytes, MYF(0))))
-    return true;
+    return 1;
   uniq_param.unique_buff =
       sort_memory + (uniq_param.max_keys_per_buffer * uniq_param.rec_length);
 
@@ -945,31 +945,30 @@ bool Unique::get(TABLE *table) {
   uniq_param.cmp_context.key_compare_arg = tree.custom_arg;
 
   /* Merge the buffers to one file, removing duplicates */
-  if (merge_many_buff(thd, &uniq_param, Sort_buffer(sort_memory, num_bytes),
+  if (merge_many_buff(table->in_use, &uniq_param,
+                      Sort_buffer(sort_memory, num_bytes),
                       Merge_chunk_array(file_ptrs.begin(), file_ptrs.size()),
                       &num_chunks, &file))
     goto err;
-  if (flush_io_cache(&file) ||
-      reinit_io_cache(&file, READ_CACHE, 0L, false, false))
+  if (flush_io_cache(&file) || reinit_io_cache(&file, READ_CACHE, 0L, 0, 0))
     goto err;
-  if (merge_buffers(thd, &uniq_param, &file, outfile,
+  if (merge_buffers(table->in_use, &uniq_param, &file, outfile,
                     Sort_buffer(sort_memory, num_bytes), file_ptr,
                     Merge_chunk_array(file_ptr, num_chunks), 0))
     goto err;
-  error = false;
+  error = 0;
 err:
   my_free(sort_memory);
-  if (flush_io_cache(outfile)) error = true;
+  if (flush_io_cache(outfile)) error = 1;
 
   /* Setup io_cache for reading */
   save_pos = outfile->pos_in_file;
-  if (reinit_io_cache(outfile, READ_CACHE, 0L, false, false)) error = true;
+  if (reinit_io_cache(outfile, READ_CACHE, 0L, 0, 0)) error = 1;
   outfile->end_of_file = save_pos;
   return error;
 }
 
 bool Unique_on_insert::unique_add(void *ptr) {
-  THD *thd = current_thd;
   Field *key = *m_table->visible_field_ptr();
   if (key->store((const char *)ptr, m_size, &my_charset_bin) != TYPE_OK)
     return true; /* purecov: inspected */
@@ -978,8 +977,7 @@ bool Unique_on_insert::unique_add(void *ptr) {
   if (res) {
     bool dup = false;
     if (res == HA_ERR_FOUND_DUPP_KEY) return true;
-    if (create_ondisk_from_heap(thd, m_table, res, /*insert_last_record=*/true,
-                                /*ignore_last_dup=*/false, &dup) ||
+    if (create_ondisk_from_heap(m_table->in_use, m_table, res, false, &dup) ||
         dup)
       return true;
   }
@@ -1003,6 +1001,5 @@ bool Unique_on_insert::init() {
 
 void Unique_on_insert::cleanup() {
   reset(false);
-  close_tmp_table(m_table);
-  free_tmp_table(m_table);
+  free_tmp_table(m_table->in_use, m_table);
 }
