@@ -87,10 +87,10 @@ class Unique {
       true   the value was inserted
   */
   inline bool unique_add(void *ptr) {
-    DBUG_ENTER("unique_add");
+    DBUG_TRACE;
     DBUG_PRINT("info", ("tree %u - %lu", tree.elements_in_tree, max_elements));
-    if (tree.elements_in_tree > max_elements && flush()) DBUG_RETURN(1);
-    DBUG_RETURN(!tree_insert(&tree, ptr, 0, tree.custom_arg));
+    if (tree.elements_in_tree > max_elements && flush()) return 1;
+    return !tree_insert(&tree, ptr, 0, tree.custom_arg);
   }
 
   bool get(TABLE *table);
