@@ -480,7 +480,7 @@ bool Window::resolve_reference(THD *thd, Item_sum *wf, PT_window **m_window) {
 }
 
 void Window::check_partition_boundary() {
-  DBUG_ENTER("check_partition_boundary");
+  DBUG_TRACE;
   bool anything_changed = false;
 
   if (m_part_row_number == 0)  // first row in first partition
@@ -507,12 +507,10 @@ void Window::check_partition_boundary() {
   } else {
     m_part_row_number++;
   }
-
-  DBUG_VOID_RETURN;
 }
 
 void Window::reset_order_by_peer_set() {
-  DBUG_ENTER("reset_order_by_peer_set");
+  DBUG_TRACE;
 
   List_iterator<Cached_item> li(m_order_by_items);
   Cached_item *item;
@@ -524,12 +522,10 @@ void Window::reset_order_by_peer_set() {
     */
     (void)item->cmp();
   }
-
-  DBUG_VOID_RETURN;
 }
 
 bool Window::in_new_order_by_peer_set(bool compare_all_order_by_items) {
-  DBUG_ENTER("in_new_order_by_peer_set");
+  DBUG_TRACE;
   bool anything_changed = false;
 
   List_iterator<Cached_item> li(m_order_by_items);
@@ -540,7 +536,7 @@ bool Window::in_new_order_by_peer_set(bool compare_all_order_by_items) {
     if (!compare_all_order_by_items) break;
   }
 
-  DBUG_RETURN(anything_changed);
+  return anything_changed;
 }
 
 bool Window::before_or_after_frame(bool before) {
@@ -716,7 +712,7 @@ bool Window::resolve_window_ordering(THD *thd, Ref_item_array ref_item_array,
                                      TABLE_LIST *tables, List<Item> &fields,
                                      List<Item> &all_fields, ORDER *o,
                                      bool partition_order) {
-  DBUG_ENTER("resolve_window_ordering");
+  DBUG_TRACE;
   DBUG_ASSERT(o);
 
   const char *sav_where = thd->where;
@@ -728,12 +724,12 @@ bool Window::resolve_window_ordering(THD *thd, Ref_item_array ref_item_array,
     /* Order by position is not allowed for windows: legacy SQL 1992 only */
     if (oi->type() == Item::INT_ITEM && oi->basic_const_item()) {
       my_error(ER_WINDOW_ILLEGAL_ORDER_BY, MYF(0), printable_name());
-      DBUG_RETURN(true);
+      return true;
     }
 
     if (find_order_in_list(thd, ref_item_array, tables, order, fields,
                            all_fields, false, true))
-      DBUG_RETURN(true);
+      return true;
     oi = *order->item;
 
     if (order->used_alias) {
@@ -743,10 +739,10 @@ bool Window::resolve_window_ordering(THD *thd, Ref_item_array ref_item_array,
         argument of a window function, or any function.
       */
       my_error(ER_BAD_FIELD_ERROR, MYF(0), oi->item_name.ptr(), thd->where);
-      DBUG_RETURN(true);
+      return true;
     }
 
-    if (!oi->fixed && oi->fix_fields(thd, order->item)) DBUG_RETURN(true);
+    if (!oi->fixed && oi->fix_fields(thd, order->item)) return true;
     oi = *order->item;  // fix_fields() may have changed *order->item
 
     /*
@@ -756,7 +752,7 @@ bool Window::resolve_window_ordering(THD *thd, Ref_item_array ref_item_array,
     if (oi->has_wf()) {
       my_error(ER_WINDOW_NESTED_WINDOW_FUNC_USE_IN_WINDOW_SPEC, MYF(0),
                printable_name());
-      DBUG_RETURN(true);
+      return true;
     }
 
     /*
@@ -765,12 +761,12 @@ bool Window::resolve_window_ordering(THD *thd, Ref_item_array ref_item_array,
     */
     if (oi->has_aggregation() && oi->type() != Item::SUM_FUNC_ITEM) {
       oi->split_sum_func(thd, ref_item_array, all_fields);
-      if (thd->is_error()) DBUG_RETURN(true);
+      if (thd->is_error()) return true;
     }
   }
 
   thd->where = sav_where;
-  DBUG_RETURN(false);
+  return false;
 }
 
 bool Window::equal_sort(Window *w1, Window *w2) {
