@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2004, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -80,7 +79,7 @@
 
   At some point a recovery method for such a drastic case needs to be divised.
 
-  Locks are row level, and you will get a consistent read.
+  Locks are row level, and you will get a consistant read.
 
   For performance as far as table scans go it is quite fast. I don't have
   good numbers but locally it has out performed both Innodb and MyISAM. For
@@ -189,8 +188,8 @@ static PSI_memory_info all_archive_memory[] = {
 #endif /* HAVE_PSI_MEMORY_INTERFACE */
 
 static void init_archive_psi_keys(void) {
-  const char *category [[maybe_unused]] = "archive";
-  int count [[maybe_unused]];
+  const char *category MY_ATTRIBUTE((unused)) = "archive";
+  int count MY_ATTRIBUTE((unused));
 
 #ifdef HAVE_PSI_MUTEX_INTERFACE
   count = static_cast<int>(array_elements(all_archive_mutexes));
@@ -259,7 +258,7 @@ Archive_share::Archive_share() {
 }
 
 ha_archive::ha_archive(handlerton *hton, TABLE_SHARE *table_arg)
-    : handler(hton, table_arg), share(nullptr), bulk_insert(false) {
+    : handler(hton, table_arg), share(NULL), bulk_insert(0) {
   /* Set our original buffer from pre-allocated memory */
   buffer.set((char *)byte_buffer, IO_SIZE, system_charset_info);
 
@@ -358,7 +357,7 @@ unsigned int ha_archive::pack_row_v1(uchar *record) {
     Field_blob *field = down_cast<Field_blob *>(table->field[*blob]);
     const uint32 length = field->get_length();
     if (length) {
-      memcpy(pos, field->get_blob_data(), length);
+      memcpy(pos, field->get_ptr(), length);
       pos += length;
     }
   }
@@ -443,7 +442,7 @@ Archive_share *ha_archive::get_share(const char *table_name, int *rc) {
     if (!(azopen(&archive_tmp, tmp_share->data_file_name, O_RDONLY))) {
       delete tmp_share;
       *rc = my_errno() ? my_errno() : HA_ERR_CRASHED;
-      tmp_share = nullptr;
+      tmp_share = NULL;
       goto err;
     }
     stats.auto_increment_value = archive_tmp.auto_increment + 1;
@@ -459,7 +458,7 @@ Archive_share *ha_archive::get_share(const char *table_name, int *rc) {
 err:
   unlock_shared_ha_data();
 
-  assert(tmp_share || *rc);
+  DBUG_ASSERT(tmp_share || *rc);
 
   return tmp_share;
 }
@@ -469,7 +468,7 @@ int Archive_share::init_archive_writer() {
   /*
     It is expensive to open and close the data files and since you can't have
     a gzip file that can be both read and written we keep a writer open
-    that is shared among all open tables.
+    that is shared amoung all open tables.
   */
   if (!(azopen(&archive_write, data_file_name, O_RDWR))) {
     DBUG_PRINT("ha_archive", ("Could not open archive write file"));
@@ -499,7 +498,7 @@ int ha_archive::init_archive_reader() {
   /*
     It is expensive to open and close the data files and since you can't have
     a gzip file that can be both read and written we keep a writer open
-    that is shared among all open tables, but have one reader open for
+    that is shared amoung all open tables, but have one reader open for
     each handler instance.
   */
   if (!archive_reader_open) {
@@ -537,7 +536,7 @@ int ha_archive::open(const char *name, int, uint open_options,
       break;
     case HA_ERR_CRASHED_ON_USAGE:
       if (open_options & HA_OPEN_FOR_REPAIR) break;
-      [[fallthrough]];
+      /* fall through */
     default:
       return rc;
   }
@@ -547,7 +546,7 @@ int ha_archive::open(const char *name, int, uint open_options,
 
   if (!record_buffer) return HA_ERR_OUT_OF_MEM;
 
-  thr_lock_data_init(&share->lock, &lock, nullptr);
+  thr_lock_data_init(&share->lock, &lock, NULL);
 
   DBUG_PRINT("ha_archive", ("archive table was crashed %s",
                             rc == HA_ERR_CRASHED_ON_USAGE ? "yes" : "no"));
@@ -617,7 +616,7 @@ int ha_archive::create(const char *name, TABLE *table_arg,
     for (; key_part != key_part_end; key_part++) {
       Field *field = key_part->field;
 
-      if (!field->is_flag_set(AUTO_INCREMENT_FLAG)) {
+      if (!(field->flags & AUTO_INCREMENT_FLAG)) {
         error = -1;
         DBUG_PRINT("ha_archive", ("Index error in creating archive table"));
         goto error;
@@ -760,7 +759,8 @@ unsigned int ha_archive::pack_row(uchar *record, azio_stream *writer) {
   ptr = record_buffer->buffer + table->s->null_bytes + ARCHIVE_ROW_HEADER_SIZE;
 
   for (Field **field = table->field; *field; field++) {
-    if (!((*field)->is_null())) ptr = (*field)->pack(ptr);
+    if (!((*field)->is_null()))
+      ptr = (*field)->pack(ptr, record + (*field)->offset(record));
   }
 
   int4store(record_buffer->buffer,
@@ -783,7 +783,7 @@ unsigned int ha_archive::pack_row(uchar *record, azio_stream *writer) {
 */
 int ha_archive::write_row(uchar *buf) {
   int rc;
-  uchar *read_buf = nullptr;
+  uchar *read_buf = NULL;
   ulonglong temp_auto;
   uchar *record = table->record[0];
   DBUG_TRACE;
@@ -801,7 +801,7 @@ int ha_archive::write_row(uchar *buf) {
   if (table->next_number_field && record == table->record[0]) {
     KEY *mkey = &table->s->key_info[0];  // We only support one key right now
     update_auto_increment();
-    temp_auto = (table->next_number_field->is_unsigned() ||
+    temp_auto = (((Field_num *)table->next_number_field)->unsigned_flag ||
                          table->next_number_field->val_int() > 0
                      ? table->next_number_field->val_int()
                      : 0);
@@ -862,7 +862,7 @@ int ha_archive::index_read(uchar *buf, const uchar *key, uint key_len,
 int ha_archive::index_read_idx(uchar *buf, uint index, const uchar *key,
                                uint key_len, enum ha_rkey_function) {
   int rc;
-  bool found = false;
+  bool found = 0;
   KEY *mkey = &table->s->key_info[index];
   current_k_offset = mkey->key_part->offset;
   current_key = key;
@@ -876,7 +876,7 @@ int ha_archive::index_read_idx(uchar *buf, uint index, const uchar *key,
 
   while (!(get_row(&archive, buf))) {
     if (!memcmp(current_key, buf + current_k_offset, current_key_len)) {
-      found = true;
+      found = 1;
       break;
     }
   }
@@ -891,14 +891,14 @@ error:
 }
 
 int ha_archive::index_next(uchar *buf) {
-  bool found = false;
+  bool found = 0;
   int rc;
 
   DBUG_TRACE;
 
   while (!(get_row(&archive, buf))) {
     if (!memcmp(current_key, buf + current_k_offset, current_key_len)) {
-      found = true;
+      found = 1;
       break;
     }
   }
@@ -955,21 +955,21 @@ int ha_archive::get_row(azio_stream *file_to_read, uchar *buf) {
 bool ha_archive::fix_rec_buff(unsigned int length) {
   DBUG_TRACE;
   DBUG_PRINT("ha_archive", ("Fixing %u for %u", length, record_buffer->length));
-  assert(record_buffer->buffer);
+  DBUG_ASSERT(record_buffer->buffer);
 
   if (length > record_buffer->length) {
     uchar *newptr;
     if (!(newptr = (uchar *)my_realloc(az_key_memory_record_buffer,
                                        (uchar *)record_buffer->buffer, length,
                                        MYF(MY_ALLOW_ZERO_PTR))))
-      return true;
+      return 1;
     record_buffer->buffer = newptr;
     record_buffer->length = length;
   }
 
-  assert(length <= record_buffer->length);
+  DBUG_ASSERT(length <= record_buffer->length);
 
-  return false;
+  return 0;
 }
 
 int ha_archive::unpack_row(azio_stream *file_to_read, uchar *record) {
@@ -996,7 +996,7 @@ int ha_archive::unpack_row(azio_stream *file_to_read, uchar *record) {
   if (fix_rec_buff(row_len)) {
     return HA_ERR_OUT_OF_MEM;
   }
-  assert(row_len <= record_buffer->length);
+  DBUG_ASSERT(row_len <= record_buffer->length);
 
   read = azread(file_to_read, record_buffer->buffer, row_len, &error);
 
@@ -1021,8 +1021,7 @@ int ha_archive::unpack_row(azio_stream *file_to_read, uchar *record) {
   ptr += table->s->null_bytes;
   for (Field **field = table->field; *field; field++) {
     if (!((*field)->is_null_in_record(record))) {
-      ptr =
-          (*field)->unpack(record + (*field)->offset(table->record[0]), ptr, 0);
+      ptr = (*field)->unpack(record + (*field)->offset(table->record[0]), ptr);
     }
   }
   return 0;
@@ -1067,7 +1066,7 @@ int ha_archive::get_row_version2(azio_stream *file_to_read, uchar *buf) {
   for (ptr = table->s->blob_field, end = ptr + table->s->blob_fields;
        ptr != end; ptr++) {
     if (bitmap_is_set(read_set,
-                      (((Field_blob *)table->field[*ptr])->field_index())))
+                      (((Field_blob *)table->field[*ptr])->field_index)))
       total_blob_length += ((Field_blob *)table->field[*ptr])->get_length();
   }
 
@@ -1081,7 +1080,7 @@ int ha_archive::get_row_version2(azio_stream *file_to_read, uchar *buf) {
     size_t size = ((Field_blob *)table->field[*ptr])->get_length();
     if (size) {
       if (bitmap_is_set(read_set,
-                        ((Field_blob *)table->field[*ptr])->field_index())) {
+                        ((Field_blob *)table->field[*ptr])->field_index)) {
         read = azread(file_to_read, const_cast<char *>(last), size, &error);
 
         if (error) return HA_ERR_CRASHED_ON_USAGE;
@@ -1231,7 +1230,7 @@ int ha_archive::optimize(THD *, HA_CHECK_OPT *check_opt) {
 
   table->copy_blobs = true;
 
-  /* read rows up to the remembered rows */
+  /* read rows upto the remembered rows */
   for (ha_rows cur_count = count; cur_count; cur_count--) {
     if ((rc = get_row(&archive, table->record[0]))) break;
     real_write_row(table->record[0], &writer);
@@ -1351,7 +1350,7 @@ int ha_archive::info(uint flag) {
   mysql_mutex_lock(&share->mutex);
   if (share->dirty) {
     DBUG_PRINT("ha_archive", ("archive flushing out rows for scan"));
-    assert(share->archive_write_open);
+    DBUG_ASSERT(share->archive_write_open);
     azflush(&(share->archive_write), Z_SYNC_FLUSH);
     share->dirty = false;
   }
@@ -1411,7 +1410,7 @@ int ha_archive::info(uint flag) {
     @return != 0 Error
 */
 
-int ha_archive::extra(enum ha_extra_function operation [[maybe_unused]]) {
+int ha_archive::extra(enum ha_extra_function operation MY_ATTRIBUTE((unused))) {
   int ret = 0;
   DBUG_TRACE;
   /* On windows we need to close all files before rename/delete. */
@@ -1550,6 +1549,8 @@ bool ha_archive::check_and_repair(THD *thd) {
   HA_CHECK_OPT check_opt;
   DBUG_TRACE;
 
+  check_opt.init();
+
   return repair(thd, &check_opt);
 }
 
@@ -1559,14 +1560,14 @@ archive_record_buffer *ha_archive::create_record_buffer(unsigned int length) {
   if (!(r = (archive_record_buffer *)my_malloc(az_key_memory_record_buffer,
                                                sizeof(archive_record_buffer),
                                                MYF(MY_WME)))) {
-    return nullptr; /* purecov: inspected */
+    return NULL; /* purecov: inspected */
   }
   r->length = (int)length;
 
   if (!(r->buffer = (uchar *)my_malloc(az_key_memory_record_buffer, r->length,
                                        MYF(MY_WME)))) {
     my_free(r);
-    return nullptr; /* purecov: inspected */
+    return NULL; /* purecov: inspected */
   }
 
   return r;
@@ -1596,15 +1597,15 @@ mysql_declare_plugin(archive){
     MYSQL_STORAGE_ENGINE_PLUGIN,
     &archive_storage_engine,
     "ARCHIVE",
-    PLUGIN_AUTHOR_ORACLE,
+    "Brian Aker, MySQL AB",
     "Archive storage engine",
     PLUGIN_LICENSE_GPL,
     archive_db_init, /* Plugin Init */
-    nullptr,         /* Plugin check uninstall */
-    nullptr,         /* Plugin Deinit */
+    NULL,            /* Plugin check uninstall */
+    NULL,            /* Plugin Deinit */
     0x0300 /* 3.0 */,
-    nullptr, /* status variables                */
-    nullptr, /* system variables                */
-    nullptr, /* config options                  */
-    0,       /* flags                           */
+    NULL, /* status variables                */
+    NULL, /* system variables                */
+    NULL, /* config options                  */
+    0,    /* flags                           */
 } mysql_declare_plugin_end;
