@@ -1,16 +1,15 @@
-/* Copyright (c) 2004, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -125,14 +124,6 @@ static int example_init_func(void *p) {
   return 0;
 }
 
-static int example_deinit_func(void *p [[maybe_unused]]) {
-  DBUG_TRACE;
-
-  assert(p);
-
-  return 0;
-}
-
 /**
   @brief
   Example of simple lock controls. The "share" it creates is a
@@ -176,7 +167,7 @@ ha_example::ha_example(handlerton *hton, TABLE_SHARE *table_arg)
   This array is optional, so every SE need not implement it.
 */
 static st_handler_tablename ha_example_system_tables[] = {
-    {(const char *)nullptr, (const char *)nullptr}};
+    {(const char *)NULL, (const char *)NULL}};
 
 /**
   @brief Check if the given db.tablename is a system table for this SE.
@@ -186,8 +177,9 @@ static st_handler_tablename ha_example_system_tables[] = {
   @param is_sql_layer_system_table  if the supplied db.table_name is a SQL
                                     layer system table.
 
-  @retval true   Given db.table_name is supported system table.
-  @retval false  Given db.table_name is not a supported system table.
+  @return
+    @retval true   Given db.table_name is supported system table.
+    @retval false  Given db.table_name is not a supported system table.
 */
 static bool example_is_supported_system_table(const char *db,
                                               const char *table_name,
@@ -228,7 +220,7 @@ int ha_example::open(const char *, int, uint, const dd::Table *) {
   DBUG_TRACE;
 
   if (!(share = get_share())) return 1;
-  thr_lock_data_init(&share->lock, &lock, nullptr);
+  thr_lock_data_init(&share->lock, &lock, NULL);
 
   return 0;
 }
@@ -578,14 +570,14 @@ int ha_example::extra(enum ha_extra_function) {
   Item_sum_count_distinct::clear(), and Item_func_group_concat::clear().
   Called from sql_delete.cc by mysql_delete().
   Called from sql_select.cc by JOIN::reinit().
-  Called from sql_union.cc by st_query_block_query_expression::exec().
+  Called from sql_union.cc by st_select_lex_unit::exec().
 
   @see
   Item_func_group_concat::clear(), Item_sum_count_distinct::clear() and
   Item_func_group_concat::clear() in item_sum.cc;
   mysql_delete() in sql_delete.cc;
   JOIN::reinit() in sql_select.cc and
-  st_query_block_query_expression::exec() in sql_union.cc.
+  st_select_lex_unit::exec() in sql_union.cc.
 */
 int ha_example::delete_all_rows() {
   DBUG_TRACE;
@@ -721,11 +713,11 @@ ha_rows ha_example::records_in_range(uint, key_range *, key_range *) {
   return 10;  // low number to force index usage
 }
 
-static MYSQL_THDVAR_STR(last_create_thdvar, PLUGIN_VAR_MEMALLOC, nullptr,
-                        nullptr, nullptr, nullptr);
+static MYSQL_THDVAR_STR(last_create_thdvar, PLUGIN_VAR_MEMALLOC, NULL, NULL,
+                        NULL, NULL);
 
-static MYSQL_THDVAR_UINT(create_count_thdvar, 0, nullptr, nullptr, nullptr, 0,
-                         0, 1000, 0);
+static MYSQL_THDVAR_UINT(create_count_thdvar, 0, NULL, NULL, NULL, 0, 0, 1000,
+                         0);
 
 /**
   @brief
@@ -783,52 +775,52 @@ static longlong srv_signed_longlong_var = 0;
 const char *enum_var_names[] = {"e1", "e2", NullS};
 
 TYPELIB enum_var_typelib = {array_elements(enum_var_names) - 1,
-                            "enum_var_typelib", enum_var_names, nullptr};
+                            "enum_var_typelib", enum_var_names, NULL};
 
 static MYSQL_SYSVAR_ENUM(enum_var,                        // name
                          srv_enum_var,                    // varname
                          PLUGIN_VAR_RQCMDARG,             // opt
                          "Sample ENUM system variable.",  // comment
-                         nullptr,                         // check
-                         nullptr,                         // update
+                         NULL,                            // check
+                         NULL,                            // update
                          0,                               // def
                          &enum_var_typelib);              // typelib
 
 static MYSQL_SYSVAR_ULONG(ulong_var, srv_ulong_var, PLUGIN_VAR_RQCMDARG,
-                          "0..1000", nullptr, nullptr, 8, 0, 1000, 0);
+                          "0..1000", NULL, NULL, 8, 0, 1000, 0);
 
 static MYSQL_SYSVAR_DOUBLE(double_var, srv_double_var, PLUGIN_VAR_RQCMDARG,
-                           "0.500000..1000.500000", nullptr, nullptr, 8.5, 0.5,
+                           "0.500000..1000.500000", NULL, NULL, 8.5, 0.5,
                            1000.5,
                            0);  // reserved always 0
 
 static MYSQL_THDVAR_DOUBLE(double_thdvar, PLUGIN_VAR_RQCMDARG,
-                           "0.500000..1000.500000", nullptr, nullptr, 8.5, 0.5,
+                           "0.500000..1000.500000", NULL, NULL, 8.5, 0.5,
                            1000.5, 0);
 
 static MYSQL_SYSVAR_INT(signed_int_var, srv_signed_int_var, PLUGIN_VAR_RQCMDARG,
-                        "INT_MIN..INT_MAX", nullptr, nullptr, -10, INT_MIN,
-                        INT_MAX, 0);
+                        "INT_MIN..INT_MAX", NULL, NULL, -10, INT_MIN, INT_MAX,
+                        0);
 
 static MYSQL_THDVAR_INT(signed_int_thdvar, PLUGIN_VAR_RQCMDARG,
-                        "INT_MIN..INT_MAX", nullptr, nullptr, -10, INT_MIN,
-                        INT_MAX, 0);
+                        "INT_MIN..INT_MAX", NULL, NULL, -10, INT_MIN, INT_MAX,
+                        0);
 
 static MYSQL_SYSVAR_LONG(signed_long_var, srv_signed_long_var,
-                         PLUGIN_VAR_RQCMDARG, "LONG_MIN..LONG_MAX", nullptr,
-                         nullptr, -10, LONG_MIN, LONG_MAX, 0);
+                         PLUGIN_VAR_RQCMDARG, "LONG_MIN..LONG_MAX", NULL, NULL,
+                         -10, LONG_MIN, LONG_MAX, 0);
 
 static MYSQL_THDVAR_LONG(signed_long_thdvar, PLUGIN_VAR_RQCMDARG,
-                         "LONG_MIN..LONG_MAX", nullptr, nullptr, -10, LONG_MIN,
+                         "LONG_MIN..LONG_MAX", NULL, NULL, -10, LONG_MIN,
                          LONG_MAX, 0);
 
 static MYSQL_SYSVAR_LONGLONG(signed_longlong_var, srv_signed_longlong_var,
-                             PLUGIN_VAR_RQCMDARG, "LLONG_MIN..LLONG_MAX",
-                             nullptr, nullptr, -10, LLONG_MIN, LLONG_MAX, 0);
+                             PLUGIN_VAR_RQCMDARG, "LLONG_MIN..LLONG_MAX", NULL,
+                             NULL, -10, LLONG_MIN, LLONG_MAX, 0);
 
 static MYSQL_THDVAR_LONGLONG(signed_longlong_thdvar, PLUGIN_VAR_RQCMDARG,
-                             "LLONG_MIN..LLONG_MAX", nullptr, nullptr, -10,
-                             LLONG_MIN, LLONG_MAX, 0);
+                             "LLONG_MIN..LLONG_MAX", NULL, NULL, -10, LLONG_MIN,
+                             LLONG_MAX, 0);
 
 static SYS_VAR *example_system_variables[] = {
     MYSQL_SYSVAR(enum_var),
@@ -843,7 +835,7 @@ static SYS_VAR *example_system_variables[] = {
     MYSQL_SYSVAR(signed_long_thdvar),
     MYSQL_SYSVAR(signed_longlong_var),
     MYSQL_SYSVAR(signed_longlong_thdvar),
-    nullptr};
+    NULL};
 
 // this is an example of SHOW_FUNC
 static int show_func_example(MYSQL_THD, SHOW_VAR *var, char *buf) {
@@ -867,20 +859,19 @@ struct example_vars_t {
   ulong var6;
 };
 
-example_vars_t example_vars = {100, 20.01, "three hundred", true, false, 8250};
+example_vars_t example_vars = {100, 20.01, "three hundred", true, 0, 8250};
 
 static SHOW_VAR show_status_example[] = {
     {"var1", (char *)&example_vars.var1, SHOW_LONG, SHOW_SCOPE_GLOBAL},
     {"var2", (char *)&example_vars.var2, SHOW_DOUBLE, SHOW_SCOPE_GLOBAL},
-    {nullptr, nullptr, SHOW_UNDEF,
-     SHOW_SCOPE_UNDEF}  // null terminator required
+    {0, 0, SHOW_UNDEF, SHOW_SCOPE_UNDEF}  // null terminator required
 };
 
 static SHOW_VAR show_array_example[] = {
     {"array", (char *)show_status_example, SHOW_ARRAY, SHOW_SCOPE_GLOBAL},
     {"var3", (char *)&example_vars.var3, SHOW_CHAR, SHOW_SCOPE_GLOBAL},
     {"var4", (char *)&example_vars.var4, SHOW_BOOL, SHOW_SCOPE_GLOBAL},
-    {nullptr, nullptr, SHOW_UNDEF, SHOW_SCOPE_UNDEF}};
+    {0, 0, SHOW_UNDEF, SHOW_SCOPE_UNDEF}};
 
 static SHOW_VAR func_status[] = {
     {"example_func_example", (char *)show_func_example, SHOW_FUNC,
@@ -891,21 +882,21 @@ static SHOW_VAR func_status[] = {
      SHOW_SCOPE_GLOBAL},
     {"example_status", (char *)show_array_example, SHOW_ARRAY,
      SHOW_SCOPE_GLOBAL},
-    {nullptr, nullptr, SHOW_UNDEF, SHOW_SCOPE_UNDEF}};
+    {0, 0, SHOW_UNDEF, SHOW_SCOPE_UNDEF}};
 
 mysql_declare_plugin(example){
     MYSQL_STORAGE_ENGINE_PLUGIN,
     &example_storage_engine,
     "EXAMPLE",
-    PLUGIN_AUTHOR_ORACLE,
+    "Brian Aker, MySQL AB",
     "Example storage engine",
     PLUGIN_LICENSE_GPL,
-    example_init_func,   /* Plugin Init */
-    nullptr,             /* Plugin check uninstall */
-    example_deinit_func, /* Plugin Deinit */
+    example_init_func, /* Plugin Init */
+    NULL,              /* Plugin check uninstall */
+    NULL,              /* Plugin Deinit */
     0x0001 /* 0.1 */,
     func_status,              /* status variables */
     example_system_variables, /* system variables */
-    nullptr,                  /* config options */
+    NULL,                     /* config options */
     0,                        /* flags */
 } mysql_declare_plugin_end;
