@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,10 +24,10 @@
 #define MYSQL_SERVER 1
 #include "storage/heap/ha_heap.h"
 
-#include <assert.h>
 #include <errno.h>
 #include <limits.h>
 
+#include "my_dbug.h"
 #include "my_macros.h"
 #include "my_pointer_arithmetic.h"
 #include "my_psi_config.h"
@@ -77,9 +76,11 @@ static handler *heap_create_handler(handlerton *hton, TABLE_SHARE *table, bool,
 *****************************************************************************/
 
 ha_heap::ha_heap(handlerton *hton, TABLE_SHARE *table_arg)
-    : handler(hton, table_arg) {
-  ref_length = sizeof(HP_HEAP_POSITION);
-}
+    : handler(hton, table_arg),
+      file(0),
+      records_changed(0),
+      key_stat_version(0),
+      single_instance(0) {}
 
 /*
   Hash index statistics is updated (copied from HP_KEYDEF::hash_buckets to
@@ -107,7 +108,7 @@ int ha_heap::open(const char *name, int mode, uint test_if_locked,
     HP_CREATE_INFO create_info;
     bool created_new_share;
     int rc;
-    file = nullptr;
+    file = 0;
     if (heap_prepare_hp_create_info(table, single_instance, delete_on_close,
                                     &create_info))
       goto end;
@@ -129,6 +130,7 @@ int ha_heap::open(const char *name, int mode, uint test_if_locked,
     }
   }
 
+  ref_length = sizeof(HP_HEAP_POSITION);
   /*
     We cannot run update_key_stats() here because we do not have a
     lock on the table. The 'records' count might just be changed
@@ -142,6 +144,9 @@ end:
 
   const int ret = file ? 0 : 1;
 
+  DBUG_PRINT("heap_api", ("this=%p %s; return=%d", this,
+                          table_definition(name, table).c_str(), ret));
+
   return (ret);
 }
 
@@ -165,9 +170,9 @@ handler *ha_heap::clone(const char *, MEM_ROOT *mem_root) {
   handler *new_handler =
       get_new_handler(table->s, false, mem_root, table->s->db_type());
   if (new_handler && !new_handler->ha_open(table, file->s->name, table->db_stat,
-                                           HA_OPEN_IGNORE_IF_LOCKED, nullptr))
+                                           HA_OPEN_IGNORE_IF_LOCKED, NULL))
     return new_handler;
-  return nullptr; /* purecov: inspected */
+  return NULL; /* purecov: inspected */
 }
 
 const char *ha_heap::table_type() const { return "MEMORY"; }
@@ -218,6 +223,9 @@ int ha_heap::write_row(uchar *buf) {
     file->s->key_stat_version++;
   }
 
+  DBUG_PRINT("heap_api", ("this=%p row=(%s); return=%d", this,
+                          row_to_string(buf, table).c_str(), res));
+
   return res;
 }
 
@@ -254,16 +262,27 @@ int ha_heap::delete_row(const uchar *buf) {
 int ha_heap::index_read_map(uchar *buf, const uchar *key,
                             key_part_map keypart_map,
                             enum ha_rkey_function find_flag) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_key_count);
   int error = heap_rkey(file, buf, active_index, key, keypart_map, find_flag);
 
+#ifndef DBUG_OFF
+  const uint key_len = calculate_key_len(table, active_index, keypart_map);
+#endif /* DBUG_OFF */
+  DBUG_PRINT(
+      "heap_api",
+      ("this=%p cells=(%s) cells_len=%u find_flag=%s out=(%s); return=%d", this,
+       indexed_cells_to_string(key, key_len, table->key_info[active_index])
+           .c_str(),
+       key_len, ha_rkey_function_to_str(find_flag),
+       (error == 0 ? row_to_string(buf, table).c_str() : ""), error));
+
   return error;
 }
 
 int ha_heap::index_read_last_map(uchar *buf, const uchar *key,
                                  key_part_map keypart_map) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_key_count);
   int error =
       heap_rkey(file, buf, active_index, key, keypart_map, HA_READ_PREFIX_LAST);
@@ -279,28 +298,28 @@ int ha_heap::index_read_idx_map(uchar *buf, uint index, const uchar *key,
 }
 
 int ha_heap::index_next(uchar *buf) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_next_count);
   int error = heap_rnext(file, buf);
   return error;
 }
 
 int ha_heap::index_prev(uchar *buf) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_prev_count);
   int error = heap_rprev(file, buf);
   return error;
 }
 
 int ha_heap::index_first(uchar *buf) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_first_count);
   int error = heap_rfirst(file, buf, active_index);
   return error;
 }
 
 int ha_heap::index_last(uchar *buf) {
-  assert(inited == INDEX);
+  DBUG_ASSERT(inited == INDEX);
   ha_statistic_increment(&System_status_var::ha_read_last_count);
   int error = heap_rlast(file, buf, active_index);
   return error;
@@ -312,6 +331,9 @@ int ha_heap::rnd_next(uchar *buf) {
   ha_statistic_increment(&System_status_var::ha_read_rnd_next_count);
   int error = heap_scan(file, buf);
 
+  DBUG_PRINT("heap_api",
+             ("this=%p out=(%s); return=%d", this,
+              (error == 0 ? row_to_string(buf, table).c_str() : ""), error));
   return error;
 }
 
@@ -477,7 +499,7 @@ THR_LOCK_DATA **ha_heap::store_lock(THD *, THR_LOCK_DATA **to,
     as they don't have properly initialized THR_LOCK and THR_LOCK_DATA
     structures.
   */
-  assert(!single_instance);
+  DBUG_ASSERT(!single_instance);
   if (lock_type != TL_IGNORE && file->lock.type == TL_UNLOCK)
     file->lock.type = lock_type;
   *to++ = &file->lock;
@@ -495,7 +517,7 @@ int ha_heap::delete_table(const char *name, const dd::Table *) {
 }
 
 void ha_heap::drop_table(const char *) {
-  file->s->delete_on_close = true;
+  file->s->delete_on_close = 1;
   close();
 }
 
@@ -518,7 +540,7 @@ ha_rows ha_heap::records_in_range(uint inx, key_range *min_key,
   if (stats.records <= 1) return stats.records;
 
   /* Assert that info() did run. We need current statistics here. */
-  assert(key_stat_version == file->s->key_stat_version);
+  DBUG_ASSERT(key_stat_version == file->s->key_stat_version);
   const ha_rows rec_in_range = static_cast<ha_rows>(
       key->records_per_key(key->user_defined_key_parts - 1));
   return rec_in_range;
@@ -533,7 +555,7 @@ static int heap_prepare_hp_create_info(TABLE *table_arg, bool single_instance,
   HP_KEYDEF *keydef;
   HA_KEYSEG *seg;
   TABLE_SHARE *share = table_arg->s;
-  bool found_real_auto_increment = false;
+  bool found_real_auto_increment = 0;
 
   memset(hp_create_info, 0, sizeof(*hp_create_info));
 
@@ -564,7 +586,7 @@ static int heap_prepare_hp_create_info(TABLE *table_arg, bool single_instance,
         mem_per_row += sizeof(TREE_ELEMENT) + pos->key_length + sizeof(char *);
         break;
       default:
-        assert(0);  // cannot happen
+        DBUG_ASSERT(0);  // cannot happen
     }
 
     for (; key_part != key_part_end; key_part++, seg++) {
@@ -584,18 +606,18 @@ static int heap_prepare_hp_create_info(TABLE *table_arg, bool single_instance,
       seg->length = (uint)key_part->length;
       seg->flag = key_part->key_part_flag;
 
-      if (field->is_flag_set(ENUM_FLAG) || field->is_flag_set(SET_FLAG))
+      if (field->flags & (ENUM_FLAG | SET_FLAG))
         seg->charset = &my_charset_bin;
       else
         seg->charset = field->charset_for_protocol();
-      if (field->is_nullable()) {
+      if (field->real_maybe_null()) {
         seg->null_bit = field->null_bit;
         seg->null_pos = field->null_offset();
       } else {
         seg->null_bit = 0;
         seg->null_pos = 0;
       }
-      if (field->is_flag_set(AUTO_INCREMENT_FLAG) &&
+      if (field->flags & AUTO_INCREMENT_FLAG &&
           table_arg->found_next_number_field &&
           key == share->next_number_index) {
         /*
@@ -635,7 +657,7 @@ int ha_heap::create(const char *name, TABLE *table_arg,
   int error;
   bool created;
   HP_CREATE_INFO hp_create_info;
-  assert(!single_instance);
+  DBUG_ASSERT(!single_instance);
 
   error = heap_prepare_hp_create_info(table_arg, false, false, &hp_create_info);
   if (error == 0) {
@@ -644,9 +666,11 @@ int ha_heap::create(const char *name, TABLE *table_arg,
                                          : 0);
     error = heap_create(name, &hp_create_info, &internal_share, &created);
     my_free(hp_create_info.keydef);
-    assert(file == nullptr);
+    DBUG_ASSERT(file == 0);
   }
 
+  DBUG_PRINT("heap_api", ("this=%p %s; return=%d", this,
+                          table_definition(name, table_arg).c_str(), error));
   return (error);
 }
 
@@ -683,15 +707,15 @@ mysql_declare_plugin(heap){
     MYSQL_STORAGE_ENGINE_PLUGIN,
     &heap_storage_engine,
     "MEMORY",
-    PLUGIN_AUTHOR_ORACLE,
+    "MySQL AB",
     "Hash based, stored in memory, useful for temporary tables",
     PLUGIN_LICENSE_GPL,
     heap_init,
-    nullptr,
-    nullptr,
-    0x0100,  /* 1.0 */
-    nullptr, /* status variables                */
-    nullptr, /* system variables                */
-    nullptr, /* config options                  */
-    0,       /* flags                           */
+    NULL,
+    NULL,
+    0x0100, /* 1.0 */
+    NULL,   /* status variables                */
+    NULL,   /* system variables                */
+    NULL,   /* config options                  */
+    0,      /* flags                           */
 } mysql_declare_plugin_end;
