@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +31,7 @@
 int heap_update(HP_INFO *info, const uchar *old, const uchar *heap_new) {
   HP_KEYDEF *keydef, *end, *p_lastinx;
   uchar *pos;
-  bool auto_key_changed = false;
+  bool auto_key_changed = 0;
   HP_SHARE *share = info->s;
   DBUG_TRACE;
 
@@ -52,14 +51,14 @@ int heap_update(HP_INFO *info, const uchar *old, const uchar *heap_new) {
           (*keydef->write_key)(info, keydef, heap_new, pos))
         goto err;
       if (share->auto_key == (uint)(keydef - share->keydef + 1))
-        auto_key_changed = true;
+        auto_key_changed = 1;
     }
   }
 
   memcpy(pos, heap_new, (size_t)share->reclength);
   if (++(share->records) == share->blength) share->blength += share->blength;
 
-#if !defined(NDEBUG) && defined(EXTRA_HEAP_DEBUG)
+#if !defined(DBUG_OFF) && defined(EXTRA_HEAP_DEBUG)
   DBUG_EXECUTE("check_heap", heap_check_heap(info, 0););
 #endif
   if (auto_key_changed) heap_update_auto_increment(info, heap_new);
