@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+Copyright (c) 2014, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -44,6 +44,7 @@ long innobase_fill_factor;
 Note: we commit all mtrs on failure.
 @return error code. */
 dberr_t PageBulk::init() {
+  mtr_t *mtr;
   buf_block_t *new_block;
   page_t *new_page;
   page_zip_des_t *new_page_zip;
@@ -53,8 +54,7 @@ dberr_t PageBulk::init() {
 
   m_heap = mem_heap_create(1000);
 
-  auto mtr_alloc = mem_heap_alloc(m_heap, sizeof(mtr_t));
-  mtr_t *mtr = new (mtr_alloc) mtr_t();
+  mtr = static_cast<mtr_t *>(mem_heap_alloc(m_heap, sizeof(mtr_t)));
   mtr_start(mtr);
 
   if (!dict_index_is_online_ddl(m_index)) {
@@ -167,9 +167,10 @@ dberr_t PageBulk::init() {
 @param[in]  tuple     tuple to insert
 @param[in]  big_rec   external record
 @param[in]  rec_size  record size
+@param[in]  n_ext     number of externally stored columns
 @return error code */
 dberr_t PageBulk::insert(const dtuple_t *tuple, const big_rec_t *big_rec,
-                         ulint rec_size) {
+                         ulint rec_size, ulint n_ext) {
   ulint *offsets = nullptr;
 
   DBUG_EXECUTE_IF("BtrBulk_insert_inject_error", return DB_INTERRUPTED;);
@@ -177,7 +178,7 @@ dberr_t PageBulk::insert(const dtuple_t *tuple, const big_rec_t *big_rec,
   /* Convert tuple to record. */
   byte *rec_mem = static_cast<byte *>(mem_heap_alloc(m_heap, rec_size));
 
-  rec_t *rec = rec_convert_dtuple_to_rec(rec_mem, m_index, tuple);
+  rec_t *rec = rec_convert_dtuple_to_rec(rec_mem, m_index, tuple, n_ext);
   offsets = rec_get_offsets(rec, m_index, offsets, ULINT_UNDEFINED, &m_heap);
 
   /* Insert the record.*/
@@ -213,11 +214,9 @@ void PageBulk::insert(const rec_t *rec, ulint *offsets) {
     ulint *old_offsets =
         rec_get_offsets(old_rec, m_index, nullptr, ULINT_UNDEFINED, &m_heap);
 
-    ut_ad(cmp_rec_rec(rec, old_rec, offsets, old_offsets, m_index,
-                      page_is_spatial_non_leaf(old_rec, m_index)) > 0 ||
+    ut_ad(cmp_rec_rec(rec, old_rec, offsets, old_offsets, m_index) > 0 ||
           (m_index->is_multi_value() &&
-           cmp_rec_rec(rec, old_rec, offsets, old_offsets, m_index,
-                       page_is_spatial_non_leaf(old_rec, m_index)) >= 0));
+           cmp_rec_rec(rec, old_rec, offsets, old_offsets, m_index) >= 0));
   }
 
   m_total_data += rec_size;
@@ -280,8 +279,8 @@ void PageBulk::finish() {
   ulint n_rec_to_assign = m_rec_no - m_slotted_rec_no;
 
   /* Fill slots for non-supremum records if possible.
-   Slot for supremum record could store up to
-   PAGE_DIR_SLOT_MAX_N_OWNED-1 records. */
+   * Slot for supremum record could store up to
+   * PAGE_DIR_SLOT_MAX_N_OWNED-1 records. */
   while (n_rec_to_assign >= PAGE_DIR_SLOT_MAX_N_OWNED) {
     static constexpr ulint RECORDS_PER_SLOT =
         (PAGE_DIR_SLOT_MAX_N_OWNED + 1) / 2;
@@ -366,7 +365,7 @@ dtuple_t *PageBulk::getNodePtr() {
 }
 
 /** Split the page records between this and given bulk.
- @param new_page_bulk  The new bulk to store split records. */
+ * @param new_page_bulk  The new bulk to store split records. */
 void PageBulk::split(PageBulk &new_page_bulk) {
   auto split_point = getSplitRec();
 
@@ -412,7 +411,7 @@ PageBulk::SplitPoint PageBulk::getSplitRec() {
     ut_ad(page_rec_is_user_rec(rec));
   } else {
     /* rec is to be moved, and this is used as number of records
-     before split */
+     * before split */
     n_recs--;
   }
 
@@ -517,9 +516,9 @@ void PageBulk::setPrev(page_no_t prev_page_no) {
   btr_page_set_prev(m_page, nullptr, prev_page_no, m_mtr);
 }
 
-/** Check if required space is available in the page for the rec
-to be inserted.	We check fill factor & padding here.
-@param[in]	rec_size	required space
+/** Check if required space is available in the page for the rec to be inserted.
+We check fill factor & padding here.
+@param[in]	rec_size	required length
 @return true	if space is available */
 bool PageBulk::isSpaceAvailable(ulint rec_size) const {
   ulint slot_size = page_dir_calc_reserved_space(m_rec_no + 1) -
@@ -600,7 +599,6 @@ void PageBulk::release() {
 
   ut_ad(!dict_index_is_spatial(m_index));
 
-  ut_ad(m_block->page.buf_fix_count > 0);
   /* We fix the block because we will re-pin it soon. */
   buf_block_buf_fix_inc(m_block, __FILE__, __LINE__);
 
@@ -621,8 +619,6 @@ void PageBulk::latch() {
   mtr_set_log_mode(m_mtr, MTR_LOG_NO_REDO);
   mtr_set_flush_observer(m_mtr, m_flush_observer);
 
-  ut_ad(m_block->page.buf_fix_count > 0);
-
   /* TODO: need a simple and wait version of buf_page_optimistic_get. */
   auto ret =
       buf_page_optimistic_get(RW_X_LATCH, m_block, m_modify_clock,
@@ -639,23 +635,6 @@ void PageBulk::latch() {
   }
 
   buf_block_buf_fix_dec(m_block);
-  /*
-  The caller is going to use the m_block, so it needs to be buffer-fixed even
-  after the decrement above. This works like this:
-  release(){ //initially buf_fix_count == N > 0
-    buf_fix_count++ // N+1
-    mtr_commit(){
-      buf_fix_count-- // N
-    }
-  }//at the end buf_fix_count == N > 0
-  latch(){//initially buf_fix_count == M > 0
-    buf_page_get_gen/buf_page_optimistic_get internally(){
-      buf_fix_count++ // M+1
-    }
-    buf_fix_count-- // M
-  }//at the end buf_fix_count == M > 0
-  */
-  ut_ad(m_block->page.buf_fix_count > 0);
 
   ut_ad(m_cur_rec > m_page && m_cur_rec < m_heap_top);
 }
@@ -742,8 +721,7 @@ dberr_t BtrBulk::pageCommit(PageBulk *page_bulk, PageBulk *next_page_bulk,
   need to acquire a lock in that case. */
   ut_ad(!page_bulk->isIndexXLocked());
 
-  DBUG_EXECUTE_IF("innodb_bulk_load_sleep",
-                  std::this_thread::sleep_for(std::chrono::seconds(1)););
+  DBUG_EXECUTE_IF("innodb_bulk_load_sleep", os_thread_sleep(1000000););
 
   /* Compress page if it's a compressed table. */
   if (page_bulk->isTableCompressed() && !page_bulk->compress()) {
@@ -911,9 +889,10 @@ dberr_t BtrBulk::prepareSpace(PageBulk *&page_bulk, ulint level,
 @param[in]  big_rec     big record vector, could be nullptr if there is no
                         data to be stored externally.
 @param[in]  rec_size    record size
+@param[in]  n_ext       number of externally stored columns
 @return error code */
 dberr_t BtrBulk::insert(PageBulk *page_bulk, dtuple_t *tuple,
-                        big_rec_t *big_rec, ulint rec_size) {
+                        big_rec_t *big_rec, ulint rec_size, ulint n_ext) {
   dberr_t err = DB_SUCCESS;
 
   if (big_rec != nullptr) {
@@ -929,7 +908,7 @@ dberr_t BtrBulk::insert(PageBulk *page_bulk, dtuple_t *tuple,
     }
   }
 
-  err = page_bulk->insert(tuple, big_rec, rec_size);
+  err = page_bulk->insert(tuple, big_rec, rec_size, n_ext);
 
   if (big_rec != nullptr) {
     /* Restore latches */
@@ -985,18 +964,19 @@ dberr_t BtrBulk::insert(dtuple_t *tuple, ulint level) {
                          dtuple_get_info_bits(tuple) | REC_INFO_MIN_REC_FLAG);
   }
 
-  ulint rec_size = rec_get_converted_size(m_index, tuple);
+  ulint n_ext = 0;
+  ulint rec_size = rec_get_converted_size(m_index, tuple, n_ext);
   big_rec_t *big_rec = nullptr;
 
   if (page_bulk->needExt(tuple, rec_size)) {
     /* The record is so big that we have to store some fields
     externally on separate database pages */
-    big_rec = dtuple_convert_big_rec(m_index, nullptr, tuple);
+    big_rec = dtuple_convert_big_rec(m_index, 0, tuple, &n_ext);
     if (big_rec == nullptr) {
       return (DB_TOO_BIG_RECORD);
     }
 
-    rec_size = rec_get_converted_size(m_index, tuple);
+    rec_size = rec_get_converted_size(m_index, tuple, n_ext);
   }
 
   if (page_bulk->isTableCompressed() && page_zip_is_too_big(m_index, tuple)) {
@@ -1018,7 +998,7 @@ dberr_t BtrBulk::insert(dtuple_t *tuple, ulint level) {
     }
   });
 
-  err = insert(page_bulk, tuple, big_rec, rec_size);
+  err = insert(page_bulk, tuple, big_rec, rec_size, n_ext);
 
 func_exit:
   if (big_rec != nullptr) {
@@ -1106,7 +1086,7 @@ dberr_t BtrBulk::finish(dberr_t err) {
       btr_page_free_low(m_index, last_block, m_root_level, &mtr);
 
       /* Do not flush the last page. */
-      last_block->page.reset_flush_observer();
+      last_block->page.flush_observer = nullptr;
 
       mtr_commit(&mtr);
 
