@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2025, Oracle and/or its affiliates.
+Copyright (c) 1995, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -37,8 +36,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "buf0buf.h"
 #include "buf0types.h"
 #include "fil0fil.h"
-#include "fil0types.h"
-#include "log0log.h" /* log_get_lsn */
 #include "mach0data.h"
 #include "my_dbug.h"
 #include "page0size.h"
@@ -64,8 +61,8 @@ when it is written to a file and also checked for a match when reading from
 the file. When reading we allow both normal CRC32 and CRC-legacy-big-endian
 variants. Note that we must be careful to calculate the same value on 32-bit
 and 64-bit architectures.
-@param[in]      page                    buffer page (UNIV_PAGE_SIZE bytes)
-@param[in]      use_legacy_big_endian   if true then use big endian
+@param[in]	page			buffer page (UNIV_PAGE_SIZE bytes)
+@param[in]	use_legacy_big_endian	if true then use big endian
 byteorder when converting byte strings to integers
 @return checksum */
 uint32_t buf_calc_page_crc32(const byte *page,
@@ -95,8 +92,10 @@ uint32_t buf_calc_page_crc32(const byte *page,
  to a file. Note that we must be careful to calculate the same value on
  32-bit and 64-bit architectures.
  @return checksum */
-uint32_t buf_calc_page_new_checksum(const byte *page) /*!< in: buffer page */
+ulint buf_calc_page_new_checksum(const byte *page) /*!< in: buffer page */
 {
+  ulint checksum;
+
   /* Since the field FIL_PAGE_FILE_FLUSH_LSN, and in versions <= 4.1.x
   FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID, are written outside the buffer pool
   to the first pages of data files, we have to skip them in the page
@@ -104,11 +103,15 @@ uint32_t buf_calc_page_new_checksum(const byte *page) /*!< in: buffer page */
   We must also skip the field FIL_PAGE_SPACE_OR_CHKSUM where the
   checksum is stored, and also the last 8 bytes of page because
   there we store the old formula checksum. */
-  return ut::hash_binary_ib(page + FIL_PAGE_OFFSET,
-                            FIL_PAGE_FILE_FLUSH_LSN - FIL_PAGE_OFFSET) +
-         ut::hash_binary_ib(
-             page + FIL_PAGE_DATA,
-             UNIV_PAGE_SIZE - FIL_PAGE_DATA - FIL_PAGE_END_LSN_OLD_CHKSUM);
+
+  checksum =
+      ut_fold_binary(page + FIL_PAGE_OFFSET,
+                     FIL_PAGE_FILE_FLUSH_LSN - FIL_PAGE_OFFSET) +
+      ut_fold_binary(page + FIL_PAGE_DATA, UNIV_PAGE_SIZE - FIL_PAGE_DATA -
+                                               FIL_PAGE_END_LSN_OLD_CHKSUM);
+  checksum = checksum & 0xFFFFFFFFUL;
+
+  return (checksum);
 }
 
 /** In versions < 4.0.14 and < 4.1.1 there was a bug that the checksum only
@@ -118,9 +121,15 @@ uint32_t buf_calc_page_new_checksum(const byte *page) /*!< in: buffer page */
  FIL_PAGE_SPACE_OR_CHKSUM before calculating and storing this old checksum
  because this takes that field as an input!
  @return checksum */
-uint32_t buf_calc_page_old_checksum(const byte *page) /*!< in: buffer page */
+ulint buf_calc_page_old_checksum(const byte *page) /*!< in: buffer page */
 {
-  return ut::hash_binary_ib(page, FIL_PAGE_FILE_FLUSH_LSN);
+  ulint checksum;
+
+  checksum = ut_fold_binary(page, FIL_PAGE_FILE_FLUSH_LSN);
+
+  checksum = checksum & 0xFFFFFFFFUL;
+
+  return (checksum);
 }
 
 /** Return a printable string describing the checksum algorithm.
@@ -147,9 +156,9 @@ const char *buf_checksum_algorithm_name(
 }
 
 /** Do lsn checks on a page during innodb recovery.
-@param[in]      check_lsn       if recv_lsn_checks_on & check_lsn
+@param[in]	check_lsn	if recv_lsn_checks_on & check_lsn
                                 perform lsn check
-@param[in]      read_buf        buffer containing the page. */
+@param[in]	read_buf	buffer containing the page. */
 inline void buf_page_lsn_check(bool check_lsn, const byte *read_buf) {
 #if !defined(UNIV_HOTBACKUP) && !defined(UNIV_LIBRARY)
   if (check_lsn && recv_lsn_checks_on) {
@@ -183,20 +192,19 @@ inline void buf_page_lsn_check(bool check_lsn, const byte *read_buf) {
       ib::error(ER_IB_MSG_147)
 #endif /* UNIV_NO_ERR_MSGS */
           << "Your database may be corrupt or you may have copied the InnoDB"
-          << " tablespace but not the InnoDB redo log files. "
-          << FORCE_RECOVERY_MSG;
+          << " tablespace but not the InnoDB log files. " << FORCE_RECOVERY_MSG;
     }
   }
 #endif /* !UNIV_HOTBACKUP && !UNIV_LIBRARY */
 }
 
 /** Checks if the page is in innodb checksum format.
-@param[in]      checksum_field1 new checksum field
-@param[in]      checksum_field2 old checksum field
-@param[in]      algo            current checksum algorithm
+@param[in]	checksum_field1	new checksum field
+@param[in]	checksum_field2	old checksum field
+@param[in]	algo		current checksum algorithm
 @return true if the page is in innodb checksum format. */
 bool BlockReporter::is_checksum_valid_innodb(
-    uint32_t checksum_field1, uint32_t checksum_field2,
+    ulint checksum_field1, ulint checksum_field2,
     const srv_checksum_algorithm_t algo) const {
   /* There are 2 valid formulas for
   checksum_field2 (old checksum field) which algo=innodb could have
@@ -208,8 +216,8 @@ bool BlockReporter::is_checksum_valid_innodb(
   2. Newer InnoDB versions store the old formula checksum
   (buf_calc_page_old_checksum()). */
 
-  const auto old_checksum = buf_calc_page_old_checksum(m_read_buf);
-  const auto new_checksum = buf_calc_page_new_checksum(m_read_buf);
+  ulint old_checksum = buf_calc_page_old_checksum(m_read_buf);
+  ulint new_checksum = buf_calc_page_new_checksum(m_read_buf);
 
   print_innodb_checksum(old_checksum, new_checksum, checksum_field1,
                         checksum_field2, algo);
@@ -228,12 +236,12 @@ bool BlockReporter::is_checksum_valid_innodb(
 }
 
 /** Checks if the page is in none checksum format.
-@param[in]      checksum_field1 new checksum field
-@param[in]      checksum_field2 old checksum field
-@param[in]      algo            current checksum algorithm
+@param[in]	checksum_field1	new checksum field
+@param[in]	checksum_field2	old checksum field
+@param[in]	algo		current checksum algorithm
 @return true if the page is in none checksum format. */
 bool BlockReporter::is_checksum_valid_none(
-    uint32_t checksum_field1, uint32_t checksum_field2,
+    ulint checksum_field1, ulint checksum_field2,
     const srv_checksum_algorithm_t algo) const {
   print_strict_none(checksum_field1, checksum_field2, algo);
 
@@ -242,44 +250,32 @@ bool BlockReporter::is_checksum_valid_none(
 }
 
 /** Checks if the page is in crc32 checksum format.
-@param[in]      checksum_field1         new checksum field
-@param[in]      checksum_field2         old checksum field
-@param[in]      algo                    current checksum algorithm
-@param[in]      use_legacy_big_endian   big endian algorithm
+@param[in]	checksum_field1		new checksum field
+@param[in]	checksum_field2		old checksum field
+@param[in]	algo			current checksum algorithm
+@param[in]	use_legacy_big_endian	big endian algorithm
 @return true if the page is in crc32 checksum format. */
-bool BlockReporter::is_checksum_valid_crc32(uint32_t checksum_field1,
-                                            uint32_t checksum_field2,
+bool BlockReporter::is_checksum_valid_crc32(ulint checksum_field1,
+                                            ulint checksum_field2,
                                             const srv_checksum_algorithm_t algo,
                                             bool use_legacy_big_endian) const {
   if (checksum_field1 != checksum_field2) {
     return (false);
   }
 
-  const auto crc32 = buf_calc_page_crc32(m_read_buf, use_legacy_big_endian);
+  uint32_t crc32 = buf_calc_page_crc32(m_read_buf, use_legacy_big_endian);
 
   print_strict_crc32(checksum_field1, checksum_field2, crc32, algo);
 
   return (checksum_field1 == crc32);
 }
 
-bool BlockReporter::is_encrypted() const noexcept {
-  const auto page_type = mach_read_from_2(m_read_buf + FIL_PAGE_TYPE);
-
-  return page_type == FIL_PAGE_ENCRYPTED ||
-         page_type == FIL_PAGE_COMPRESSED_AND_ENCRYPTED ||
-         page_type == FIL_PAGE_ENCRYPTED_RTREE;
-}
-
 /** Checks if a page is corrupt.
-@retval true    if page is corrupt
-@retval false   if page is not corrupt */
+@retval	true	if page is corrupt
+@retval	false	if page is not corrupt */
 bool BlockReporter::is_corrupted() const {
-  /* This function should be used for unencrypted pages. During recovery it is
-  possible that this function is called for encrypted pages, when decryption
-  failed. So report it as corrupted. */
-  if (is_encrypted()) {
-    return true;
-  }
+  ulint checksum_field1;
+  ulint checksum_field2;
 
   if (!m_page_size.is_compressed() &&
       memcmp(
@@ -305,13 +301,14 @@ bool BlockReporter::is_corrupted() const {
     return (!verify_zip_checksum());
   }
 
-  const auto checksum_field1 =
-      mach_read_from_4(m_read_buf + FIL_PAGE_SPACE_OR_CHKSUM);
+  checksum_field1 = mach_read_from_4(m_read_buf + FIL_PAGE_SPACE_OR_CHKSUM);
 
-  const auto checksum_field2 = mach_read_from_4(
-      m_read_buf + m_page_size.logical() - FIL_PAGE_END_LSN_OLD_CHKSUM);
+  checksum_field2 = mach_read_from_4(m_read_buf + m_page_size.logical() -
+                                     FIL_PAGE_END_LSN_OLD_CHKSUM);
 
-  static_assert(FIL_PAGE_LSN % 8 == 0, "FIL_PAGE_LSN must be 64 bit aligned");
+#if FIL_PAGE_LSN % 8
+#error "FIL_PAGE_LSN must be 64 bit aligned"
+#endif
 
   /* declare empty pages non-corrupted */
   if (checksum_field1 == 0 && checksum_field2 == 0 &&
@@ -471,24 +468,46 @@ bool BlockReporter::is_corrupted() const {
       return (true);
 
     case SRV_CHECKSUM_ALGORITHM_NONE:
-      /* should have returned false earlier */
+      /* should have returned FALSE earlier */
       break;
       /* no default so the compiler will emit a warning if new enum
       is added and not handled here */
   }
 
   ut_error;
+  return (false);
 }
 
-uint32_t BlockReporter::calc_zip_checksum(srv_checksum_algorithm_t algo) const {
-  return (calc_zip_checksum(m_read_buf, m_page_size.physical(), algo));
+/** Calculate the compressed page checksum.
+@param[in]	algo			checksum algorithm to use
+@param[in]	use_legacy_big_endian	only used if algo is
+SRV_CHECKSUM_ALGORITHM_CRC32 or SRV_CHECKSUM_ALGORITHM_STRICT_CRC32 -
+if true then use big endian byteorder when converting byte strings to
+integers.
+@return page checksum */
+uint32_t BlockReporter::calc_zip_checksum(
+    srv_checksum_algorithm_t algo,
+    bool use_legacy_big_endian /* = false */) const {
+  return (calc_zip_checksum(m_read_buf, m_page_size.physical(), algo,
+                            use_legacy_big_endian));
 }
 
-uint32_t BlockReporter::calc_zip_checksum(const byte *read_buf,
-                                          ulint phys_page_size,
-                                          srv_checksum_algorithm_t algo) const {
+/** Calculate the compressed page checksum. This variant
+should be used when only the page_size_t is unknown and
+only physical page_size of compressed page is available
+@param[in]	read_buf		buffer holding the page
+@param[in]	phys_page_size		physical page size
+@param[in]	algo			checksum algorithm to use
+@param[in]	use_legacy_big_endian	only used if algo is
+SRV_CHECKSUM_ALGORITHM_CRC32 or SRV_CHECKSUM_ALGORITHM_STRICT_CRC32 -
+if true then use big endian byteorder when converting byte strings to
+integers.
+@return page checksum */
+uint32_t BlockReporter::calc_zip_checksum(
+    const byte *read_buf, ulint phys_page_size, srv_checksum_algorithm_t algo,
+    bool use_legacy_big_endian /* = false */) const {
   uLong adler;
-  uint32_t crc32;
+  ib_uint32_t crc32;
   const Bytef *s = read_buf;
   const ulint size = phys_page_size;
 
@@ -517,7 +536,7 @@ uint32_t BlockReporter::calc_zip_checksum(const byte *read_buf,
           adler32(adler, s + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID,
                   static_cast<uInt>(size) - FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID);
 
-      return ((uint32_t)adler);
+      return ((ib_uint32_t)adler);
     case SRV_CHECKSUM_ALGORITHM_NONE:
     case SRV_CHECKSUM_ALGORITHM_STRICT_NONE:
       return (BUF_NO_CHECKSUM_MAGIC);
@@ -528,11 +547,18 @@ uint32_t BlockReporter::calc_zip_checksum(const byte *read_buf,
   ut_error;
 }
 
+/** Verify a compressed page's checksum.
+@retval		true		if stored checksum is valid according
+to the value of srv_checksum_algorithm
+@retval		false		if stored schecksum is not valid according
+to the value of srv_checksum_algorithm */
 bool BlockReporter::verify_zip_checksum() const {
   const uint32_t stored = static_cast<uint32_t>(
       mach_read_from_4(m_read_buf + FIL_PAGE_SPACE_OR_CHKSUM));
 
-  static_assert(FIL_PAGE_LSN % 8 == 0, "FIL_PAGE_LSN must be 64 bit aligned");
+#if FIL_PAGE_LSN % 8
+#error "FIL_PAGE_LSN must be 64 bit aligned"
+#endif
 
   /* Check if page is empty */
   if (stored == 0 && mach_read_from_8(m_read_buf + FIL_PAGE_LSN) == 0) {
@@ -592,7 +618,7 @@ bool BlockReporter::verify_zip_checksum() const {
       matching legacy big endian checksum, we try to match it first.
       Otherwise we check innodb checksum first. */
       if (legacy_big_endian_checksum) {
-        if (stored == calc_zip_checksum(SRV_CHECKSUM_ALGORITHM_CRC32)) {
+        if (stored == calc_zip_checksum(SRV_CHECKSUM_ALGORITHM_CRC32, true)) {
           return (true);
         }
         legacy_checksum_checked = true;
@@ -609,7 +635,7 @@ bool BlockReporter::verify_zip_checksum() const {
 
       /* If legacy checksum is not checked, do it now. */
       if (!legacy_checksum_checked &&
-          stored == calc_zip_checksum(SRV_CHECKSUM_ALGORITHM_CRC32)) {
+          stored == calc_zip_checksum(SRV_CHECKSUM_ALGORITHM_CRC32, true)) {
         /* This page's checksum has been created by the
         legacy software CRC32 implementation on big endian
         CPUs which generates a different result than the
@@ -630,7 +656,8 @@ bool BlockReporter::verify_zip_checksum() const {
         return (true);
       }
 
-      if (stored == calc_zip_checksum(SRV_CHECKSUM_ALGORITHM_CRC32)) {
+      if (stored == calc_zip_checksum(SRV_CHECKSUM_ALGORITHM_CRC32) ||
+          stored == calc_zip_checksum(SRV_CHECKSUM_ALGORITHM_CRC32, true)) {
         if (curr_algo == SRV_CHECKSUM_ALGORITHM_STRICT_INNODB) {
           page_warn_strict_checksum(curr_algo, SRV_CHECKSUM_ALGORITHM_CRC32,
                                     page_id);
@@ -641,7 +668,8 @@ bool BlockReporter::verify_zip_checksum() const {
       break;
     case SRV_CHECKSUM_ALGORITHM_STRICT_NONE:
 
-      if (stored == calc_zip_checksum(SRV_CHECKSUM_ALGORITHM_CRC32)) {
+      if (stored == calc_zip_checksum(SRV_CHECKSUM_ALGORITHM_CRC32) ||
+          stored == calc_zip_checksum(SRV_CHECKSUM_ALGORITHM_CRC32, true)) {
         page_warn_strict_checksum(curr_algo, SRV_CHECKSUM_ALGORITHM_CRC32,
                                   page_id);
         return (true);
@@ -665,9 +693,9 @@ bool BlockReporter::verify_zip_checksum() const {
 
 /** Issue a warning when the checksum that is stored in the page is valid,
 but different than the global setting innodb_checksum_algorithm.
-@param[in]      curr_algo       current checksum algorithm
-@param[in]      page_checksum   page valid checksum
-@param[in]      page_id         page identifier */
+@param[in]	curr_algo	current checksum algorithm
+@param[in]	page_checksum	page valid checksum
+@param[in]	page_id		page identifier */
 void BlockReporter::page_warn_strict_checksum(
     srv_checksum_algorithm_t curr_algo, srv_checksum_algorithm_t page_checksum,
     const page_id_t &page_id) const {
@@ -705,33 +733,11 @@ void BlockReporter::page_warn_strict_checksum(
 }
 
 /** Print the given page_id_t object.
-@param[in,out]  out     the output stream
-@param[in]      page_id the page_id_t object to be printed
+@param[in,out]	out	the output stream
+@param[in]	page_id	the page_id_t object to be printed
 @return the output stream */
 std::ostream &operator<<(std::ostream &out, const page_id_t &page_id) {
   out << "[page id: space=" << page_id.m_space
       << ", page number=" << page_id.m_page_no << "]";
   return (out);
 }
-
-bool BlockReporter::is_lsn_valid(const byte *frame,
-                                 uint32_t page_size) noexcept {
-  const uint32_t lsn1 = mach_read_from_4(frame + FIL_PAGE_LSN + 4);
-  const uint32_t lsn2 = mach_read_from_4(frame + page_size - 4);
-
-#ifdef UNIV_DEBUG
-  if (lsn1 != lsn2) {
-    ut_ad(lsn1 == lsn2);
-  }
-#endif /* UNIV_DEBUG */
-
-  return lsn1 == lsn2;
-}
-
-space_id_t BlockReporter::space_id() const noexcept {
-  return mach_read_from_4(m_read_buf + FIL_PAGE_SPACE_ID);
-}
-
-page_no_t BlockReporter::page_no() const noexcept {
-  return mach_read_from_4(m_read_buf + FIL_PAGE_OFFSET);
-}
