@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+Copyright (c) 2018, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -31,7 +30,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
  *******************************************************/
 
 #include "clone0desc.h"
-#include "dict0dict.h"
 #include "mach0data.h"
 
 /** Maximum supported descriptor version. The version represents the current
@@ -104,7 +102,7 @@ static const uint CLONE_TASK_BLOCK_OFFSET = CLONE_TASK_CHUNK_OFFSET + 4;
 static const uint CLONE_TASK_META_LEN = CLONE_TASK_BLOCK_OFFSET + 4;
 
 /** Initialize header
-@param[in]      version descriptor version */
+@param[in]	version	descriptor version */
 void Clone_Desc_Task_Meta::init_header(uint version) {
   m_header.m_version = version;
 
@@ -359,7 +357,7 @@ void Chunk_Info::serialize(byte *desc_chunk, uint &len) {
   ut_ad(len_left >= 4);
   len_left -= 4;
 
-  ulint index [[maybe_unused]] = 0;
+  ulint index = 0;
 
   for (auto &key_value : m_incomplete_chunks) {
     ut_ad(index < chunk_map_size);
@@ -395,15 +393,15 @@ void Chnunk_Bitmap::deserialize(const byte *desc_chunk, uint &len_left) {
   desc_chunk += 4;
 
   if (len_left < 4) {
-    ut_d(ut_error);
-    ut_o(return);
+    ut_ad(false);
+    return;
   }
 
   len_left -= 4;
 
   if (bitmap_size > m_size) {
-    ut_d(ut_error);
-    ut_o(return);
+    ut_ad(false);
+    return;
   }
 
   for (ulint index = 0; index < bitmap_size; index++) {
@@ -412,8 +410,8 @@ void Chnunk_Bitmap::deserialize(const byte *desc_chunk, uint &len_left) {
     desc_chunk += 4;
 
     if (len_left < 4) {
-      ut_d(ut_error);
-      ut_o(return);
+      ut_ad(false);
+      return;
     }
 
     len_left -= 4;
@@ -428,8 +426,8 @@ void Chunk_Info::deserialize(const byte *desc_chunk, uint &len_left) {
   desc_chunk += 4;
 
   if (len_left < 4) {
-    ut_d(ut_error);
-    ut_o(return);
+    ut_ad(false);
+    return;
   }
 
   len_left -= 4;
@@ -437,10 +435,10 @@ void Chunk_Info::deserialize(const byte *desc_chunk, uint &len_left) {
   auto max_map_size = static_cast<uint32_t>(2 * CLONE_MAX_TASKS);
   /* Each task can have one incomplete chunk at most */
   if (chunk_map_size > max_map_size) {
+    ut_ad(false);
     ib::error(ER_IB_CLONE_RESTART)
         << "Clone too many incomplete chunks: " << chunk_map_size;
-    ut_d(ut_error);
-    ut_o(return);
+    return;
   }
 
   for (ulint index = 0; index < chunk_map_size; index++) {
@@ -449,8 +447,8 @@ void Chunk_Info::deserialize(const byte *desc_chunk, uint &len_left) {
     desc_chunk += 4;
 
     if (len_left < 4) {
-      ut_d(ut_error);
-      ut_o(return);
+      ut_ad(false);
+      return;
     }
     len_left -= 4;
 
@@ -458,8 +456,8 @@ void Chunk_Info::deserialize(const byte *desc_chunk, uint &len_left) {
     desc_chunk += 4;
 
     if (len_left < 4) {
-      ut_d(ut_error);
-      ut_o(return);
+      ut_ad(false);
+      return;
     }
     len_left -= 4;
 
@@ -471,7 +469,7 @@ void Chunk_Info::deserialize(const byte *desc_chunk, uint &len_left) {
   ut_ad(len_left == 0);
 }
 
-void Clone_Desc_Locator::init(uint64_t id, uint64_t snap_id,
+void Clone_Desc_Locator::init(ib_uint64_t id, ib_uint64_t snap_id,
                               Snapshot_State state, uint version, uint index) {
   m_header.m_version = version;
 
@@ -544,14 +542,14 @@ bool clone_validate_locator(const byte *desc_loc, uint desc_len) {
   Clone_Desc_Header header;
 
   if (!header.deserialize(desc_loc, desc_len)) {
-    ut_d(ut_error);
-    ut_o(return (false));
+    ut_ad(false);
+    return (false);
   }
   if (desc_len < CLONE_DESC_LOC_BASE_LEN ||
       header.m_length < CLONE_DESC_LOC_BASE_LEN || header.m_length > desc_len ||
       header.m_type != CLONE_DESC_LOCATOR) {
-    ut_d(ut_error);
-    ut_o(return (false));
+    ut_ad(false);
+    return (false);
   }
   return (true);
 }
@@ -564,8 +562,8 @@ void Clone_Desc_Locator::deserialize(const byte *desc_loc, uint desc_len,
 
   if (m_header.m_length < CLONE_DESC_LOC_BASE_LEN ||
       m_header.m_length > desc_len) {
-    ut_d(ut_error);
-    ut_o(return);
+    ut_ad(false);
+    return;
   }
 
   m_clone_id = mach_read_from_8(desc_loc + CLONE_LOC_CID_OFFSET);
@@ -589,16 +587,16 @@ void Clone_Desc_Locator::deserialize(const byte *desc_loc, uint desc_len,
 }
 
 /** Check a specific bit in flag.
-@param[in]      flag    bit flag
-@param[in]      bit     check bit
+@param[in]	flag	bit flag
+@param[in]	bit	check bit
 @return true, iff bit is set in flag. */
 inline bool DESC_CHECK_FLAG(ulint flag, ulint bit) {
   return ((flag & (1ULL << (bit - 1))) > 0);
 }
 
 /** Set a specific bit in flag.
-@param[in]      flag    bit flag
-@param[in]      bit     set bit */
+@param[in]	flag	bit flag
+@param[in]	bit	set bit */
 inline void DESC_SET_FLAG(ulint &flag, ulint bit) {
   flag |= static_cast<ulint>(1ULL << (bit - 1));
 }
@@ -626,12 +624,6 @@ static const uint CLONE_DESC_FILE_FLAG_ZLIB = 1;
 static const uint CLONE_DESC_FILE_FLAG_LZ4 = 2;
 /** Clone File Flag: Encryption type AES */
 static const uint CLONE_DESC_FILE_FLAG_AES = 3;
-/** Clone File Flag: File is renamed. */
-static const uint CLONE_DESC_FILE_FLAG_RENAMED = 4;
-/** Clone File Flag: File is deleted. */
-static const uint CLONE_DESC_FILE_FLAG_DELETED = 5;
-/** Clone File Flag: File metadata has encryption key. */
-static const uint CLONE_DESC_FILE_HAS_KEY = 6;
 
 /** File Metadata: Tablespace ID in 4 bytes */
 static const uint CLONE_FILE_SPACE_ID_OFFSET = CLONE_FILE_FLAGS_OFFSET + 2;
@@ -660,11 +652,6 @@ void Clone_Desc_File_MetaData::init_header(uint version) {
   m_header.m_length = CLONE_FILE_BASE_LEN;
   m_header.m_length += static_cast<uint>(m_file_meta.m_file_name_len);
 
-  if (m_file_meta.m_transfer_encryption_key) {
-    m_header.m_length += sizeof(m_file_meta.m_encryption_metadata.m_key);
-    m_header.m_length += sizeof(m_file_meta.m_encryption_metadata.m_iv);
-  }
-
   m_header.m_type = CLONE_DESC_FILE_METADATA;
 }
 
@@ -700,22 +687,9 @@ void Clone_Desc_File_MetaData::serialize(byte *&desc_file, uint &len,
     DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_FLAG_LZ4);
   }
   /* Set file encryption type */
-  if (m_file_meta.m_encryption_metadata.m_type == Encryption::AES) {
+  if (m_file_meta.m_encrypt_type == Encryption::AES) {
     DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_FLAG_AES);
   }
-  /* Set file renamed attribute */
-  if (m_file_meta.m_renamed) {
-    DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_FLAG_RENAMED);
-  }
-  /* Set file deleted attribute */
-  if (m_file_meta.m_deleted) {
-    DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_FLAG_DELETED);
-  }
-
-  if (m_file_meta.m_transfer_encryption_key) {
-    DESC_SET_FLAG(file_flags, CLONE_DESC_FILE_HAS_KEY);
-  }
-
   mach_write_to_2(desc_file + CLONE_FILE_FLAGS_OFFSET, file_flags);
 
   mach_write_to_4(desc_file + CLONE_FILE_SPACE_ID_OFFSET,
@@ -736,20 +710,6 @@ void Clone_Desc_File_MetaData::serialize(byte *&desc_file, uint &len,
            static_cast<const void *>(m_file_meta.m_file_name),
            m_file_meta.m_file_name_len);
   }
-
-  auto dest_key =
-      desc_file + CLONE_FILE_FNAME_OFFSET + m_file_meta.m_file_name_len;
-
-  /* Append Encryption key information if requested. */
-  if (m_file_meta.m_transfer_encryption_key) {
-    memcpy(dest_key, m_file_meta.m_encryption_metadata.m_key,
-           sizeof(m_file_meta.m_encryption_metadata.m_key));
-
-    dest_key += sizeof(m_file_meta.m_encryption_metadata.m_key);
-
-    memcpy(dest_key, m_file_meta.m_encryption_metadata.m_iv,
-           sizeof(m_file_meta.m_encryption_metadata.m_iv));
-  }
 }
 
 bool Clone_Desc_File_MetaData::deserialize(const byte *desc_file,
@@ -789,19 +749,11 @@ bool Clone_Desc_File_MetaData::deserialize(const byte *desc_file,
   }
 
   /* Get file encryption information */
-  m_file_meta.m_encryption_metadata.m_type = Encryption::NONE;
+  m_file_meta.m_encrypt_type = Encryption::NONE;
   if (DESC_CHECK_FLAG(file_flags, CLONE_DESC_FILE_FLAG_AES)) {
-    m_file_meta.m_encryption_metadata.m_type = Encryption::AES;
+    m_file_meta.m_encrypt_type = Encryption::AES;
   }
 
-  /* Get file renamed attribute */
-  m_file_meta.m_renamed =
-      DESC_CHECK_FLAG(file_flags, CLONE_DESC_FILE_FLAG_RENAMED);
-
-  /* Get file renamed attribute */
-  m_file_meta.m_deleted =
-      DESC_CHECK_FLAG(file_flags, CLONE_DESC_FILE_FLAG_DELETED);
-
   m_file_meta.m_space_id =
       mach_read_from_4(desc_file + CLONE_FILE_SPACE_ID_OFFSET);
   m_file_meta.m_file_index =
@@ -815,15 +767,14 @@ bool Clone_Desc_File_MetaData::deserialize(const byte *desc_file,
   m_file_meta.m_file_name_len =
       mach_read_from_4(desc_file + CLONE_FILE_FNAMEL_OFFSET);
 
-  m_file_meta.m_file_name_alloc_len = 0;
+  ut_ad(m_header.m_length ==
+        CLONE_FILE_FNAME_OFFSET + m_file_meta.m_file_name_len);
 
   /* Check if we have enough length. */
   if (desc_len < m_file_meta.m_file_name_len) {
     return (false);
   }
 
-  desc_len -= m_file_meta.m_file_name_len;
-
   if (m_file_meta.m_file_name_len == 0) {
     m_file_meta.m_file_name = nullptr;
   } else {
@@ -836,40 +787,6 @@ bool Clone_Desc_File_MetaData::deserialize(const byte *desc_file,
       return (false);
     }
   }
-
-  /* Check if encryption key is transferred. */
-  m_file_meta.m_transfer_encryption_key =
-      DESC_CHECK_FLAG(file_flags, CLONE_DESC_FILE_HAS_KEY);
-
-  /* Extract Encryption key information if transferred. */
-  if (m_file_meta.m_transfer_encryption_key) {
-    auto src_key =
-        desc_file + CLONE_FILE_FNAME_OFFSET + m_file_meta.m_file_name_len;
-
-    /* Check if we have enough length. */
-    if (desc_len < (sizeof(m_file_meta.m_encryption_metadata.m_key) +
-                    sizeof(m_file_meta.m_encryption_metadata.m_iv))) {
-      return false; /* purecov: inspected */
-    }
-
-    memcpy(m_file_meta.m_encryption_metadata.m_key, src_key,
-           sizeof(m_file_meta.m_encryption_metadata.m_key));
-
-    src_key += sizeof(m_file_meta.m_encryption_metadata.m_key);
-
-    memcpy(m_file_meta.m_encryption_metadata.m_iv, src_key,
-           sizeof(m_file_meta.m_encryption_metadata.m_iv));
-
-    ut_ad(m_header.m_length ==
-          CLONE_FILE_FNAME_OFFSET +
-              sizeof(m_file_meta.m_encryption_metadata.m_key) +
-              sizeof(m_file_meta.m_encryption_metadata.m_iv) +
-              m_file_meta.m_file_name_len);
-  } else {
-    ut_ad(m_header.m_length ==
-          CLONE_FILE_FNAME_OFFSET + m_file_meta.m_file_name_len);
-  }
-
   return (true);
 }
 
@@ -1068,28 +985,3 @@ bool Clone_Desc_Data::deserialize(const byte *desc_data, uint desc_len) {
 
   return (true);
 }
-
-void Clone_File_Meta::init() {
-  m_file_index = 0;
-  m_file_name = nullptr;
-  m_file_name_len = 0;
-  m_file_name_alloc_len = 0;
-
-  m_file_size = 0;
-  m_alloc_size = 0;
-
-  m_space_id = dict_sys_t::s_invalid_space_id;
-  m_fsp_flags = UINT32_UNDEFINED;
-
-  m_compress_type = Compression::NONE;
-  m_encryption_metadata = {};
-  m_punch_hole = false;
-  m_fsblk_size = 0;
-
-  m_transfer_encryption_key = false;
-
-  m_begin_chunk = 0;
-  m_end_chunk = 0;
-
-  reset_ddl();
-}
