@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -870,7 +870,8 @@ UNIV_INLINE
 void buf_page_set_old(buf_page_t *bpage, ibool old);
 
 /** Determine the time of first access of a block in the buffer pool.
- @return ut_time_ms() at the time of first access, 0 if not accessed */
+ @return ut_time_monotonic_ms() at the time of first access, 0 if not accessed
+ */
 UNIV_INLINE
 unsigned buf_page_is_accessed(const buf_page_t *bpage) /*!< in: control block */
     MY_ATTRIBUTE((warn_unused_result));
@@ -1109,6 +1110,11 @@ if needed.
 UNIV_INLINE
 ulint buf_pool_size_align(ulint size);
 
+/** Adjust the proposed chunk unit size so that it satisfies all invariants
+@param[in]      size    proposed size of buffer pool chunk unit in bytes
+@return adjusted size which meets invariants */
+ulonglong buf_pool_adjust_chunk_unit(ulonglong size);
+
 /** Calculate the checksum of a page from compressed table and update the
 page.
 @param[in,out]  page              page to update
@@ -1760,7 +1766,7 @@ struct buf_pool_t {
   ulint n_pend_unzip;          /*!< number of pending decompressions.
                                Accessed atomically. */
 
-  time_t last_printout_time;
+  ib_time_monotonic_t last_printout_time;
   /*!< when buf_print_io was last time
   called. Accesses not protected. */
   buf_buddy_stat_t buddy_stat[BUF_BUDDY_SIZES_MAX + 1];
