@@ -1,6 +1,6 @@
 /***********************************************************************
 
-Copyright (c) 1995, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2019, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2009, Percona Inc.
 
 Portions of this file contain modifications contributed and copyrighted
@@ -49,6 +49,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 #ifndef _WIN32
 #include <dirent.h>
 #include <sys/stat.h>
+#include <sys/statvfs.h>
 #include <time.h>
 #else
 #include <Strsafe.h>
@@ -59,6 +60,10 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 #include <functional>
 #include <stack>
 
+/** Prefix all files and directory created under data directory with special
+string so that it never conflicts with MySQL schema directory. */
+#define OS_FILE_PREFIX "#"
+
 /** File node of a tablespace or the log data space */
 struct fil_node_t;
 
@@ -195,6 +200,8 @@ static const ulint OS_BUFFERED_FILE = 102;
 
 static const ulint OS_CLONE_DATA_FILE = 103;
 static const ulint OS_CLONE_LOG_FILE = 104;
+
+static const ulint OS_REDO_LOG_ARCHIVE_FILE = 105;
 /* @} */
 
 /** Error codes from os_file_get_last_error @{ */
@@ -382,13 +389,14 @@ struct Encryption {
   static void get_master_key(ulint *master_key_id, byte **master_key);
 
   /** Fill the encryption information.
-  @param[in]	key		encryption key
-  @param[in]	iv		encryption iv
+  @param[in]		key		encryption key
+  @param[in]		iv		encryption iv
   @param[in,out]	encrypt_info	encryption information
-  @param[in]	is_boot		if it's for bootstrap
+  @param[in]		is_boot		if it's for bootstrap
+  @param[in]		encrypt_key	encrypt with master key
   @return true if success. */
   static bool fill_encryption_info(byte *key, byte *iv, byte *encrypt_info,
-                                   bool is_boot);
+                                   bool is_boot, bool encrypt_key);
 
   /** Get master key from encryption information
   @param[in]	encrypt_info	encryption information
@@ -406,9 +414,10 @@ struct Encryption {
   @param[in,out]	key		key
   @param[in,out]	iv		iv
   @param[in]		encryption_info	encryption info
+  @param[in]		decrypt_key	decrypt key using master key
   @return true if success */
-  static bool decode_encryption_info(byte *key, byte *iv,
-                                     byte *encryption_info);
+  static bool decode_encryption_info(byte *key, byte *iv, byte *encryption_info,
+                                     bool decrypt_key);
 
   /** Encrypt the redo log block.
   @param[in]	type		IORequest
@@ -1116,23 +1125,27 @@ The wrapper functions have the prefix of "innodb_". */
   pfs_os_aio_func(type, mode, name, file, buf, offset, n, read_only, message1, \
                   message2, __FILE__, __LINE__)
 
-#define os_file_read_pfs(type, file, buf, offset, n) \
-  pfs_os_file_read_func(type, file, buf, offset, n, __FILE__, __LINE__)
+#define os_file_read_pfs(type, file_name, file, buf, offset, n)          \
+  pfs_os_file_read_func(type, file_name, file, buf, offset, n, __FILE__, \
+                        __LINE__)
 
-#define os_file_read_first_page_pfs(type, file, buf, n) \
-  pfs_os_file_read_first_page_func(type, file, buf, n, __FILE__, __LINE__)
+#define os_file_read_first_page_pfs(type, file_name, file, buf, n)          \
+  pfs_os_file_read_first_page_func(type, file_name, file, buf, n, __FILE__, \
+                                   __LINE__)
 
 #define os_file_copy_pfs(src, src_offset, dest, dest_offset, size)          \
   pfs_os_file_copy_func(src, src_offset, dest, dest_offset, size, __FILE__, \
                         __LINE__)
 
-#define os_file_read_no_error_handling_pfs(type, file, buf, offset, n, o) \
-  pfs_os_file_read_no_error_handling_func(type, file, buf, offset, n, o,  \
-                                          __FILE__, __LINE__)
+#define os_file_read_no_error_handling_pfs(type, file_name, file, buf, offset, \
+                                           n, o)                               \
+  pfs_os_file_read_no_error_handling_func(type, file_name, file, buf, offset,  \
+                                          n, o, __FILE__, __LINE__)
 
-#define os_file_read_no_error_handling_int_fd(type, file, buf, offset, n, o) \
-  pfs_os_file_read_no_error_handling_int_fd_func(type, file, buf, offset, n, \
-                                                 o, __FILE__, __LINE__)
+#define os_file_read_no_error_handling_int_fd(type, file_name, file, buf, \
+                                              offset, n, o)               \
+  pfs_os_file_read_no_error_handling_int_fd_func(                         \
+      type, file_name, file, buf, offset, n, o, __FILE__, __LINE__)
 
 #define os_file_write_pfs(type, name, file, buf, offset, n) \
   pfs_os_file_write_func(type, name, file, buf, offset, n, __FILE__, __LINE__)
@@ -1241,6 +1254,7 @@ this function!
 This is the performance schema instrumented wrapper function for
 os_file_read() which requests a synchronous read operation.
 @param[in, out]	type		IO request context
+@param[in]  file_name file name
 @param[in]	file		Open file handle
 @param[out]	buf		buffer where to read
 @param[in]	offset		file offset where to read
@@ -1249,9 +1263,9 @@ os_file_read() which requests a synchronous read operation.
 @param[in]	src_line	line where the func invoked
 @return DB_SUCCESS if request was successful */
 UNIV_INLINE
-dberr_t pfs_os_file_read_func(IORequest &type, pfs_os_file_t file, void *buf,
-                              os_offset_t offset, ulint n, const char *src_file,
-                              uint src_line);
+dberr_t pfs_os_file_read_func(IORequest &type, const char *file_name,
+                              pfs_os_file_t file, void *buf, os_offset_t offset,
+                              ulint n, const char *src_file, uint src_line);
 
 /** NOTE! Please use the corresponding macro os_file_read_first_page(),
 not directly this function!
@@ -1259,6 +1273,7 @@ This is the performance schema instrumented wrapper function for
 os_file_read_first_page() which requests a synchronous read operation
 of page 0 of IBD file
 @param[in, out]	type		IO request context
+@param[in]  file_name file name
 @param[in]	file		Open file handle
 @param[out]	buf		buffer where to read
 @param[in]	n		number of bytes to read
@@ -1266,8 +1281,8 @@ of page 0 of IBD file
 @param[in]	src_line	line where the func invoked
 @return DB_SUCCESS if request was successful */
 UNIV_INLINE
-dberr_t pfs_os_file_read_first_page_func(IORequest &type, pfs_os_file_t file,
-                                         void *buf, ulint n,
+dberr_t pfs_os_file_read_first_page_func(IORequest &type, const char *file_name,
+                                         pfs_os_file_t file, void *buf, ulint n,
                                          const char *src_file, uint src_line);
 
 /** copy data from one file to another file. Data is read/written
@@ -1291,6 +1306,7 @@ This is the performance schema instrumented wrapper function for
 os_file_read_no_error_handling_func() which requests a synchronous
 read operation.
 @param[in, out]	type		IO request context
+@param[in]  file_name file name
 @param[in]	file		Open file handle
 @param[out]	buf		buffer where to read
 @param[in]	offset		file offset where to read
@@ -1300,11 +1316,9 @@ read operation.
 @param[in]	src_line	line where the func invoked
 @return DB_SUCCESS if request was successful */
 UNIV_INLINE
-dberr_t pfs_os_file_read_no_error_handling_func(IORequest &type,
-                                                pfs_os_file_t file, void *buf,
-                                                os_offset_t offset, ulint n,
-                                                ulint *o, const char *src_file,
-                                                uint src_line);
+dberr_t pfs_os_file_read_no_error_handling_func(
+    IORequest &type, const char *file_name, pfs_os_file_t file, void *buf,
+    os_offset_t offset, ulint n, ulint *o, const char *src_file, uint src_line);
 
 /** NOTE! Please use the corresponding macro
 os_file_read_no_error_handling_int_fd(), not directly this function!
@@ -1312,6 +1326,7 @@ This is the performance schema instrumented wrapper function for
 os_file_read_no_error_handling_int_fd_func() which requests a
 synchronous read operation on files with int type descriptors.
 @param[in, out] type            IO request context
+@param[in]      file_name       file name
 @param[in]      file            Open file handle
 @param[out]     buf             buffer where to read
 @param[in]      offset          file offset where to read
@@ -1323,8 +1338,9 @@ synchronous read operation on files with int type descriptors.
 
 UNIV_INLINE
 dberr_t pfs_os_file_read_no_error_handling_int_fd_func(
-    IORequest &type, int file, void *buf, os_offset_t offset, ulint n, ulint *o,
-    const char *src_file, ulint src_line);
+    IORequest &type, const char *file_name, int file, void *buf,
+    os_offset_t offset, ulint n, ulint *o, const char *src_file,
+    ulint src_line);
 
 /** NOTE! Please use the corresponding macro os_aio(), not directly this
 function!
@@ -1476,20 +1492,22 @@ to original un-instrumented file I/O APIs */
   os_aio_func(type, mode, name, file, buf, offset, n, read_only, message1,  \
               message2)
 
-#define os_file_read_pfs(type, file, buf, offset, n) \
-  os_file_read_func(type, file, buf, offset, n)
+#define os_file_read_pfs(type, file_name, file, buf, offset, n) \
+  os_file_read_func(type, file_name, file, buf, offset, n)
 
-#define os_file_read_first_page_pfs(type, file, buf, n) \
-  os_file_read_first_page_func(type, file, buf, n)
+#define os_file_read_first_page_pfs(type, file_name, file, buf, n) \
+  os_file_read_first_page_func(type, file_name, file, buf, n)
 
 #define os_file_copy_pfs(src, src_offset, dest, dest_offset, size) \
   os_file_copy_func(src, src_offset, dest, dest_offset, size)
 
-#define os_file_read_no_error_handling_pfs(type, file, buf, offset, n, o) \
-  os_file_read_no_error_handling_func(type, file, buf, offset, n, o)
+#define os_file_read_no_error_handling_pfs(type, file_name, file, buf, offset, \
+                                           n, o)                               \
+  os_file_read_no_error_handling_func(type, file_name, file, buf, offset, n, o)
 
-#define os_file_read_no_error_handling_int_fd(type, file, buf, offset, n, o) \
-  os_file_read_no_error_handling_func(type, file, buf, offset, n, o)
+#define os_file_read_no_error_handling_int_fd(type, file_name, file, buf, \
+                                              offset, n, o)               \
+  os_file_read_no_error_handling_func(type, file_name, file, buf, offset, n, o)
 
 #define os_file_write_pfs(type, name, file, buf, offset, n) \
   os_file_write_func(type, name, file, buf, offset, n)
@@ -1516,19 +1534,19 @@ to original un-instrumented file I/O APIs */
 #endif
 
 #ifdef UNIV_PFS_IO
-#define os_file_read(type, file, buf, offset, n) \
-  os_file_read_pfs(type, file, buf, offset, n)
+#define os_file_read(type, file_name, file, buf, offset, n) \
+  os_file_read_pfs(type, file_name, file, buf, offset, n)
 #else
-#define os_file_read(type, file, buf, offset, n) \
-  os_file_read_pfs(type, file.m_file, buf, offset, n)
+#define os_file_read(type, file_name, file, buf, offset, n) \
+  os_file_read_pfs(type, file_name, file.m_file, buf, offset, n)
 #endif
 
 #ifdef UNIV_PFS_IO
-#define os_file_read_first_page(type, file, buf, n) \
-  os_file_read_first_page_pfs(type, file, buf, n)
+#define os_file_read_first_page(type, file_name, file, buf, n) \
+  os_file_read_first_page_pfs(type, file_name, file, buf, n)
 #else
-#define os_file_read_first_page(type, file, buf, n) \
-  os_file_read_first_page_pfs(type, file.m_file, buf, n)
+#define os_file_read_first_page(type, file_name, file, buf, n) \
+  os_file_read_first_page_pfs(type, file_name, file.m_file, buf, n)
 #endif
 
 #ifdef UNIV_PFS_IO
@@ -1554,11 +1572,14 @@ to original un-instrumented file I/O APIs */
 #endif
 
 #ifdef UNIV_PFS_IO
-#define os_file_read_no_error_handling(type, file, buf, offset, n, o) \
-  os_file_read_no_error_handling_pfs(type, file, buf, offset, n, o)
+#define os_file_read_no_error_handling(type, file_name, file, buf, offset, n, \
+                                       o)                                     \
+  os_file_read_no_error_handling_pfs(type, file_name, file, buf, offset, n, o)
 #else
-#define os_file_read_no_error_handling(type, file, buf, offset, n, o) \
-  os_file_read_no_error_handling_pfs(type, file.m_file, buf, offset, n, o)
+#define os_file_read_no_error_handling(type, file_name, file, buf, offset, n, \
+                                       o)                                     \
+  os_file_read_no_error_handling_pfs(type, file_name, file.m_file, buf,       \
+                                     offset, n, o)
 #endif
 
 #ifdef UNIV_HOTBACKUP
@@ -1635,25 +1656,27 @@ ulint os_file_get_last_error(bool report_all_errors);
 function!
 Requests a synchronous read operation.
 @param[in]	type		IO request context
+@param[in]  file_name file name
 @param[in]	file		Open file handle
 @param[out]	buf		buffer where to read
 @param[in]	offset		file offset where to read
 @param[in]	n		number of bytes to read
 @return DB_SUCCESS if request was successful */
-dberr_t os_file_read_func(IORequest &type, os_file_t file, void *buf,
-                          os_offset_t offset, ulint n)
-    MY_ATTRIBUTE((warn_unused_result));
+dberr_t os_file_read_func(IORequest &type, const char *file_name,
+                          os_file_t file, void *buf, os_offset_t offset,
+                          ulint n) MY_ATTRIBUTE((warn_unused_result));
 
 /** NOTE! Use the corresponding macro os_file_read_first_page(),
 not directly this function!
 Requests a synchronous read operation of page 0 of IBD file
 @param[in]	type		IO request context
+@param[in]  file_name file name
 @param[in]	file		Open file handle
 @param[out]	buf		buffer where to read
 @param[in]	n		number of bytes to read
 @return DB_SUCCESS if request was successful */
-dberr_t os_file_read_first_page_func(IORequest &type, os_file_t file, void *buf,
-                                     ulint n)
+dberr_t os_file_read_first_page_func(IORequest &type, const char *file_name,
+                                     os_file_t file, void *buf, ulint n)
     MY_ATTRIBUTE((warn_unused_result));
 
 /** copy data from one file to another file. Data is read/written
@@ -1681,16 +1704,16 @@ not directly this function!
 Requests a synchronous positioned read operation. This function does not do
 any error handling. In case of error it returns FALSE.
 @param[in]	type		IO request context
+@param[in]  file_name file name
 @param[in]	file		Open file handle
 @param[out]	buf		buffer where to read
 @param[in]	offset		file offset where to read
 @param[in]	n		number of bytes to read
 @param[out]	o		number of bytes actually read
 @return DB_SUCCESS or error code */
-dberr_t os_file_read_no_error_handling_func(IORequest &type, os_file_t file,
-                                            void *buf, os_offset_t offset,
-                                            ulint n, ulint *o)
-    MY_ATTRIBUTE((warn_unused_result));
+dberr_t os_file_read_no_error_handling_func(
+    IORequest &type, const char *file_name, os_file_t file, void *buf,
+    os_offset_t offset, ulint n, ulint *o) MY_ATTRIBUTE((warn_unused_result));
 
 /** NOTE! Use the corresponding macro os_file_write(), not directly this
 function!
@@ -1837,6 +1860,12 @@ void os_aio_print_pending_io(FILE *file);
 
 #endif /* UNIV_DEBUG */
 
+/** Get available free space on disk
+@param[in]	path		pathname of a directory or file in disk
+@param[out]	free_space	free space available in bytes
+@return DB_SUCCESS if all OK */
+dberr_t os_get_free_space(const char *path, uint64_t &free_space);
+
 /** This function returns information about the specified file
 @param[in]	path		pathname of the file
 @param[in]	stat_info	information of a file in a directory
@@ -1875,6 +1904,10 @@ ulint os_file_original_page_size(const byte *buf);
 @param[in]	umask		The umask to use for file creation. */
 void os_file_set_umask(ulint umask);
 
+/** Get the file create umask
+@return the umask to use for file creation. */
+ulint os_file_get_umask();
+
 /** Free storage space associated with a section of the file.
 @param[in]	fh		Open file handle
 @param[in]	off		Starting offset (SEEK_SET)
@@ -1909,12 +1942,24 @@ dberr_t os_file_decompress_page(bool dblwr_recover, byte *src, byte *dst,
                                 ulint dst_len)
     MY_ATTRIBUTE((warn_unused_result));
 
+/** Compress a data page
+@param[in]	compression	Compression algorithm
+@param[in]	block_size	File system block size
+@param[in]	src		Source contents to compress
+@param[in]	src_len		Length in bytes of the source
+@param[out]	dst		Compressed page contents
+@param[out]	dst_len		Length in bytes of dst contents
+@return buffer data, dst_len will have the length of the data */
+byte *os_file_compress_page(Compression compression, ulint block_size,
+                            byte *src, ulint src_len, byte *dst,
+                            ulint *dst_len);
+
 /** Determine if O_DIRECT is supported.
 @retval	true	if O_DIRECT is supported.
 @retval	false	if O_DIRECT is not supported. */
 bool os_is_o_direct_supported() MY_ATTRIBUTE((warn_unused_result));
 
-/** Class to scan the directory heirarch using a depth first scan. */
+/** Class to scan the directory heirarchy using a depth first scan. */
 class Dir_Walker {
  public:
   using Path = std::string;
@@ -1925,9 +1970,9 @@ class Dir_Walker {
   static bool is_directory(const Path &path);
 
   /** Depth first traversal of the directory starting from basedir
-  @param[in]	basedir		Start scanning from this directory
-  @param[in]    recursive       True if scan should be recursive
-  @param[in]	f		Function to call for each entry */
+  @param[in]  basedir    Start scanning from this directory
+  @param[in]  recursive  `true` if scan should be recursive
+  @param[in]  f          Function to call for each entry */
   template <typename F>
   static void walk(const Path &basedir, bool recursive, F &&f) {
 #ifdef _WIN32
@@ -1958,9 +2003,9 @@ class Dir_Walker {
   using Function = std::function<void(const Path &, size_t)>;
 
   /** Depth first traversal of the directory starting from basedir
-  @param[in]	basedir		Start scanning from this directory
-  @param[in]    recursive       True if scan should be recursive
-  @param[in]	f		Function to call for each entry */
+  @param[in]      basedir    Start scanning from this directory
+  @param[in]      recursive  `true` if scan should be recursive
+  @param[in]      f          Callback for each entry found */
 #ifdef _WIN32
   static void walk_win32(const Path &basedir, bool recursive, Function &&f);
 #else
