@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2021, Oracle and/or its affiliates.
+Copyright (c) 1994, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -69,9 +69,9 @@ bool cmp_cols_are_equal(const dict_col_t *col1, const dict_col_t *col2,
 @retval 0 if data1 is equal to data2
 @retval negative if data1 is less than data2
 @retval positive if data1 is greater than data2 */
-[[nodiscard]] int cmp_data_data(ulint mtype, ulint prtype, bool is_asc,
-                                const byte *data1, ulint len1,
-                                const byte *data2, ulint len2);
+int cmp_data_data(ulint mtype, ulint prtype, bool is_asc, const byte *data1,
+                  ulint len1, const byte *data2, ulint len2)
+    MY_ATTRIBUTE((warn_unused_result));
 
 /** Compare two data fields.
 @param[in]	dfield1	data field; must have type field set
@@ -81,17 +81,19 @@ bool cmp_cols_are_equal(const dict_col_t *col1, const dict_col_t *col2,
 @retval 0 if dfield1 is equal to dfield2
 @retval negative if dfield1 is less than dfield2
 @retval positive if dfield1 is greater than dfield2 */
-[[nodiscard]] static inline int cmp_dfield_dfield(const dfield_t *dfield1,
-                                                  const dfield_t *dfield2,
-                                                  bool is_asc);
+UNIV_INLINE
+int cmp_dfield_dfield(const dfield_t *dfield1, const dfield_t *dfield2,
+                      bool is_asc) MY_ATTRIBUTE((warn_unused_result));
 
 /** Compare two data fields, the first one can be of any form of multi-value
 field, while the second one must be one field from multi-value index
 @param[in]	dfield1	multi-value data field;
 @param[in]	dfield2	data field; must have type field set
 @return 0 if dfield1 has dfield2 or they are equal if both NULL, otherwise 1 */
-[[nodiscard]] static inline int cmp_multi_value_dfield_dfield(
-    const dfield_t *dfield1, const dfield_t *dfield2);
+UNIV_INLINE
+int cmp_multi_value_dfield_dfield(const dfield_t *dfield1,
+                                  const dfield_t *dfield2)
+    MY_ATTRIBUTE((warn_unused_result));
 
 /** Compare a GIS data tuple to a physical record.
 @param[in] dtuple data tuple
@@ -144,9 +146,11 @@ field that is not matched
 @retval 0 if dtuple is equal to rec
 @retval negative if dtuple is less than rec
 @retval positive if dtuple is greater than rec */
-[[nodiscard]] int cmp_dtuple_rec_with_match_bytes(
-    const dtuple_t *dtuple, const rec_t *rec, const dict_index_t *index,
-    const ulint *offsets, ulint *matched_fields, ulint *matched_bytes);
+int cmp_dtuple_rec_with_match_bytes(const dtuple_t *dtuple, const rec_t *rec,
+                                    const dict_index_t *index,
+                                    const ulint *offsets, ulint *matched_fields,
+                                    ulint *matched_bytes)
+    MY_ATTRIBUTE((warn_unused_result));
 /** Compare a data tuple to a physical record.
 @see cmp_dtuple_rec_with_match
 @param[in]	dtuple	data tuple
@@ -157,87 +161,81 @@ field that is not matched
 @retval 0 if dtuple is equal to rec
 @retval negative if dtuple is less than rec
 @retval positive if dtuple is greater than rec */
-[[nodiscard]] int cmp_dtuple_rec(const dtuple_t *dtuple, const rec_t *rec,
-                                 const dict_index_t *index,
-                                 const ulint *offsets);
+int cmp_dtuple_rec(const dtuple_t *dtuple, const rec_t *rec,
+                   const dict_index_t *index, const ulint *offsets)
+    MY_ATTRIBUTE((warn_unused_result));
 /** Check if a dtuple is a prefix of a record.
 @param[in]	dtuple	data tuple
 @param[in]	rec	B-tree record
 @param[in]	index	B-tree index
 @param[in]	offsets	rec_get_offsets(rec)
 @return true if prefix */
-[[nodiscard]] bool cmp_dtuple_is_prefix_of_rec(const dtuple_t *dtuple,
-                                               const rec_t *rec,
-                                               const dict_index_t *index,
-                                               const ulint *offsets);
+bool cmp_dtuple_is_prefix_of_rec(const dtuple_t *dtuple, const rec_t *rec,
+                                 const dict_index_t *index,
+                                 const ulint *offsets)
+    MY_ATTRIBUTE((warn_unused_result));
 /** Compare two physical records that contain the same number of columns,
 none of which are stored externally.
 @retval positive if rec1 (including non-ordering columns) is greater than rec2
 @retval negative if rec1 (including non-ordering columns) is less than rec2
 @retval 0 if rec1 is a duplicate of rec2 */
-[[nodiscard]] int cmp_rec_rec_simple(
+int cmp_rec_rec_simple(
     const rec_t *rec1,         /*!< in: physical record */
     const rec_t *rec2,         /*!< in: physical record */
     const ulint *offsets1,     /*!< in: rec_get_offsets(rec1, ...) */
     const ulint *offsets2,     /*!< in: rec_get_offsets(rec2, ...) */
     const dict_index_t *index, /*!< in: data dictionary index */
-    struct TABLE *table);      /*!< in: MySQL table, for reporting
-                              duplicate key value if applicable,
-                              or NULL */
+    struct TABLE *table)       /*!< in: MySQL table, for reporting
+                               duplicate key value if applicable,
+                               or NULL */
+    MY_ATTRIBUTE((warn_unused_result));
 /** Compare two B-tree records.
 @param[in] rec1 B-tree record
 @param[in] rec2 B-tree record
 @param[in] offsets1 rec_get_offsets(rec1, index)
 @param[in] offsets2 rec_get_offsets(rec2, index)
 @param[in] index B-tree index
-@param[in] spatial_index_non_leaf true if record is in spatial non leaf page
 @param[in] nulls_unequal true if this is for index cardinality
 statistics estimation, and innodb_stats_method=nulls_unequal
 or innodb_stats_method=nulls_ignored
 @param[out] matched_fields number of completely matched fields
 within the first field not completely matched
-@param[in] cmp_btree_recs true if we're comparing two b-tree records
 @return the comparison result
 @retval 0 if rec1 is equal to rec2
 @retval negative if rec1 is less than rec2
 @retval positive if rec2 is greater than rec2 */
 int cmp_rec_rec_with_match(const rec_t *rec1, const rec_t *rec2,
                            const ulint *offsets1, const ulint *offsets2,
-                           const dict_index_t *index,
-                           bool spatial_index_non_leaf, bool nulls_unequal,
-                           ulint *matched_fields, bool cmp_btree_recs = true);
+                           const dict_index_t *index, bool nulls_unequal,
+                           ulint *matched_fields);
 
 /** Compare two B-tree records.
 Only the common first fields are compared, and externally stored field
 are treated as equal.
-@param[in]	rec1			B-tree record
-@param[in]	rec2			B-tree record
-@param[in]	offsets1		rec_get_offsets(rec1, index)
-@param[in]	offsets2		rec_get_offsets(rec2, index)
-@param[in]	index			B-tree index
-@param[in]	spatial_index_non_leaf  true if spatial index non leaf records
+@param[in]	rec1		B-tree record
+@param[in]	rec2		B-tree record
+@param[in]	offsets1	rec_get_offsets(rec1, index)
+@param[in]	offsets2	rec_get_offsets(rec2, index)
+@param[in]	index		B-tree index
 @param[out]	matched_fields	number of completely matched fields
-within the first field not completely matched
-@param[in]  cmp_btree_recs  true if the both the records are b-tree records
+                                within the first field not completely matched
 @return positive, 0, negative if rec1 is greater, equal, less, than rec2,
 respectively */
-static inline int cmp_rec_rec(const rec_t *rec1, const rec_t *rec2,
-                              const ulint *offsets1, const ulint *offsets2,
-                              const dict_index_t *index,
-                              bool spatial_index_non_leaf,
-                              ulint *matched_fields = nullptr,
-                              bool cmp_btree_recs = true);
+UNIV_INLINE
+int cmp_rec_rec(const rec_t *rec1, const rec_t *rec2, const ulint *offsets1,
+                const ulint *offsets2, const dict_index_t *index,
+                ulint *matched_fields = NULL);
 
-#ifndef UNIV_HOTBACKUP
 /** Compare two data fields.
 @param[in] dfield1 data field
 @param[in] dfield2 data field
 @return the comparison result of dfield1 and dfield2
 @retval true if dfield1 is equal to dfield2, or a prefix of dfield1
 @retval false otherwise */
-[[nodiscard]] static inline bool cmp_dfield_dfield_eq_prefix(
-    const dfield_t *dfield1, const dfield_t *dfield2);
-#endif /* !UNIV_HOTBACKUP */
+UNIV_INLINE
+bool cmp_dfield_dfield_eq_prefix(const dfield_t *dfield1,
+                                 const dfield_t *dfield2)
+    MY_ATTRIBUTE((warn_unused_result));
 
 #include "rem0cmp.ic"
 
