@@ -1,6 +1,6 @@
 /***********************************************************************
 
-Copyright (c) 2010, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2010, 2019, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify
@@ -139,6 +139,9 @@ enum monitor_id_t {
   /* Lock manager related counters */
   MONITOR_MODULE_LOCK,
   MONITOR_DEADLOCK,
+  MONITOR_DEADLOCK_FALSE_POSITIVES,
+  MONITOR_DEADLOCK_ROUNDS,
+  MONITOR_LOCK_THREADS_WAITING,
   MONITOR_TIMEOUT,
   MONITOR_LOCKREC_WAIT,
   MONITOR_TABLELOCK_WAIT,
@@ -753,7 +756,7 @@ monitor counter
   MONITOR_CHECK_DEFINED(value);                               \
   if (MONITOR_IS_ON(monitor)) {                               \
     uintmax_t old_time = (value);                             \
-    value = ut_time_us(NULL);                                 \
+    value = ut_time_monotonic_us();                           \
     MONITOR_VALUE(monitor) += (mon_type_t)(value - old_time); \
   }
 
