@@ -107,10 +107,14 @@ dberr_t srv_undo_tablespace_fixup(const char *space_name, const char *file_name,
 any tables (including data dictionary tables) can be accessed. */
 void srv_dict_recover_on_restart();
 
-/** Start up the remaining InnoDB service threads.
+/** Start up the InnoDB service threads which are independent of DDL recovery
 @param[in]	bootstrap	True if this is in bootstrap */
 void srv_start_threads(bool bootstrap);
 
+/** Start the remaining InnoDB service threads which must wait for
+complete DD recovery(post the DDL recovery) */
+void srv_start_threads_after_ddl_recovery();
+
 /** Shut down all InnoDB background tasks that may look up objects in
 the data dictionary. */
 void srv_pre_dd_shutdown();
@@ -143,9 +147,10 @@ ulint srv_path_copy(char *dest,             /*!< out: destination buffer */
 single-table tablespace.
 @param[in]	table		table object
 @param[out]	filename	filename
-@param[in]	max_len		filename max length */
+@param[in]	max_len		filename max length
+@param[in]	convert		convert to lower case */
 void srv_get_encryption_data_filename(dict_table_t *table, char *filename,
-                                      ulint max_len);
+                                      ulint max_len, bool convert = false);
 #endif /* !UNIV_HOTBACKUP */
 
 /** true if the server is being started */
