@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2025, Oracle and/or its affiliates.
+Copyright (c) 1994, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -34,50 +33,43 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #ifndef ut0dbg_h
 #define ut0dbg_h
 
-#include "my_compiler.h"
-
 /* Do not include univ.i because univ.i includes this. */
 
-#include <cstdio>
 #include <functional>
+#include "os0thread.h"
 
 /** Set a callback function to be called before exiting.
-@param[in]      callback        user callback function */
+@param[in]	callback	user callback function */
 void ut_set_assert_callback(std::function<void()> &callback);
 
-/** Report a failed assertion.
-@param[in] expr The failed assertion
-@param[in] file Source file containing the assertion
-@param[in] line Line number of the assertion */
-[[noreturn]] void ut_dbg_assertion_failed(const char *expr, const char *file,
-                                          uint64_t line);
+/** Report a failed assertion. */
+[[noreturn]] void ut_dbg_assertion_failed(
+    const char *expr, /*!< in: the failed assertion */
+    const char *file, /*!< in: source file containing the assertion */
+    ulint line);      /*!< in: line number of the assertion */
 
 /** Abort execution if EXPR does not evaluate to nonzero.
 @param EXPR assertion expression that should hold */
-#define ut_a(EXPR)                                        \
-  do {                                                    \
-    if (unlikely(false == (bool)(EXPR))) {                \
-      ut_dbg_assertion_failed(#EXPR, __FILE__, __LINE__); \
-    }                                                     \
+#define ut_a(EXPR)                                               \
+  do {                                                           \
+    if (UNIV_UNLIKELY(!(ulint)(EXPR))) {                         \
+      ut_dbg_assertion_failed(#EXPR, __FILE__, (ulint)__LINE__); \
+    }                                                            \
   } while (0)
 
 /** Abort execution. */
-#define ut_error ut_dbg_assertion_failed(0, __FILE__, __LINE__)
+#define ut_error ut_dbg_assertion_failed(0, __FILE__, (ulint)__LINE__)
 
 #ifdef UNIV_DEBUG
 /** Debug assertion. Does nothing unless UNIV_DEBUG is defined. */
 #define ut_ad(EXPR) ut_a(EXPR)
 /** Debug statement. Does nothing unless UNIV_DEBUG is defined. */
 #define ut_d(EXPR) EXPR
-/** Opposite of ut_d().  Does nothing if UNIV_DEBUG is defined. */
-#define ut_o(EXPR)
 #else
 /** Debug assertion. Does nothing unless UNIV_DEBUG is defined. */
 #define ut_ad(EXPR)
 /** Debug statement. Does nothing unless UNIV_DEBUG is defined. */
 #define ut_d(EXPR)
-/** Opposite of ut_d().  Does nothing if UNIV_DEBUG is defined. */
-#define ut_o(EXPR) EXPR
 #endif
 
 /** Debug crash point */
@@ -95,7 +87,6 @@ void ut_set_assert_callback(std::function<void()> &callback);
     snprintf(buf, sizeof buf, prefix "_%u", count);                    \
     DBUG_EXECUTE_IF(buf, log_buffer_flush_to_disk(); DBUG_SUICIDE();); \
   } while (0)
-
 #else
 #define DBUG_INJECT_CRASH(prefix, count)
 #define DBUG_INJECT_CRASH_WITH_LOG_FLUSH(prefix, count)
@@ -103,7 +94,7 @@ void ut_set_assert_callback(std::function<void()> &callback);
 
 /** Silence warnings about an unused variable by doing a null assignment.
 @param A the unused variable */
-#define UT_NOT_USED(A) std::ignore = A
+#define UT_NOT_USED(A) A = A
 
 #if defined(HAVE_SYS_TIME_H) && defined(HAVE_SYS_RESOURCE_H)
 
@@ -115,21 +106,21 @@ void ut_set_assert_callback(std::function<void()> &callback);
 
 /** A "chronometer" used to clock snippets of code.
 Example usage:
-        ut_chrono_t     ch("this loop");
+        ut_chrono_t	ch("this loop");
         for (;;) { ... }
         ch.show();
 would print the timings of the for() loop, prefixed with "this loop:" */
 class ut_chrono_t {
  public:
   /** Constructor.
-  @param[in]    name    chrono's name, used when showing the values */
+  @param[in]	name	chrono's name, used when showing the values */
   ut_chrono_t(const char *name) : m_name(name), m_show_from_destructor(true) {
     reset();
   }
 
   /** Resets the chrono (records the current time in it). */
   void reset() {
-    gettimeofday(&m_tv, nullptr);
+    gettimeofday(&m_tv, NULL);
 
     getrusage(RUSAGE_SELF, &m_ru);
   }
@@ -142,7 +133,7 @@ class ut_chrono_t {
 
     getrusage(RUSAGE_SELF, &ru_now);
 
-    gettimeofday(&tv_now, nullptr);
+    gettimeofday(&tv_now, NULL);
 
 #ifndef timersub
 #define timersub(a, b, r)                       \
