@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2016, 2020, Oracle and/or its affiliates.
+Copyright (c) 2016, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -132,24 +132,13 @@ void z_frag_entry_t::update(const z_frag_page_t &frag_page) {
   set_big_free_len(frag_page.get_big_free_len());
 }
 
-void z_frag_entry_t::free_frag_page(mtr_t *mtr, dict_index_t *index) {
-  page_no_t page_no = get_page_no();
-  if (page_no != FIL_NULL) {
-    page_id_t page_id = page_id_t(index->space_id(), page_no);
-    page_size_t page_size = index->get_page_size();
-    buf_block_t *block = buf_page_get(page_id, page_size, RW_X_LATCH, mtr);
-    btr_page_free_low(index, block, ULINT_UNDEFINED, mtr);
-    set_page_no(FIL_NULL);
-  }
-}
-
 /** Insert a single zlib stream.
 @param[in]	index	the index to which the LOB belongs.
 @param[in]	first	the first page of the compressed LOB.
 @param[in]	trxid	the id of the current transaction.
 @param[in]	blob	in memory copy of the LOB.
 @param[in]	len	the length of the LOB.
-@param[in]	mtr	the mini-transaction context.
+@param[in]	mtr	the mini transaction context.
 @param[in]	bulk	true if bulk operation, false otherwise.
 @param[out]	start_page_no	the first page into which zlib stream
                                 was written.
@@ -323,14 +312,14 @@ dberr_t z_insert_strm(dict_index_t *index, z_first_page_t &first,
 }
 
 /** Insert one chunk of input.  The maximum size of a chunk is Z_CHUNK_SIZE.
-@param[in]  index      Clustered index in which LOB is inserted.
-@param[in]  first      First page of the LOB.
-@param[in]  trx        Transaction doing the insertion.
+@param[in]  index      clustered index in which LOB is inserted.
+@param[in]  first      the first page of the LOB.
+@param[in]  trx        transaction doing the insertion.
 @param[in]  ref        LOB reference in the clust rec.
-@param[in]  blob       Uncompressed LOB to be inserted.
-@param[in]  len        Length of the blob.
-@param[out] out_entry  Newly inserted index entry. can be NULL.
-@param[in]  mtr        Mini-transaction
+@param[in]  blob       the uncompressed LOB to be inserted.
+@param[in]  len        length of the blob.
+@param[out] out_entry  the newly inserted index entry. can be NULL.
+@param[in]  mtr        the mini transaction
 @param[in]  bulk       true if it is bulk operation, false otherwise.
 @return DB_SUCCESS on success, error code on failure. */
 dberr_t z_insert_chunk(dict_index_t *index, z_first_page_t &first, trx_t *trx,
@@ -404,12 +393,11 @@ dberr_t z_insert_chunk(dict_index_t *index, z_first_page_t &first, trx_t *trx,
   return (DB_SUCCESS);
 }
 
-/** Insert a compressed large object (LOB) into the system.
-@param[in]	ctx	the B-tree context for this LOB operation.
-@param[in]	trx	transaction doing the insertion.
-@param[in,out]	ref	the LOB reference.
-@param[in]	field	the LOB field.
-@param[in]	field_j	the LOB field index in big rec vector.
+/** Insert a large object (LOB) into the system.
+@param[in]      ctx    the B-tree context for this LOB operation.
+@param[in]      trx    transaction doing the insertion.
+@param[in,out]  ref    the LOB reference.
+@param[in]      field  the LOB field.
 @return DB_SUCCESS on success, error code on failure.*/
 dberr_t z_insert(InsertContext *ctx, trx_t *trx, ref_t &ref,
                  big_rec_field_t *field, ulint field_j) {
@@ -428,7 +416,7 @@ dberr_t z_insert(InsertContext *ctx, trx_t *trx, ref_t &ref,
   ut_ad(remain > 0);
 
   if (ref.length() > 0) {
-    ref.set_length(len, nullptr);
+    ref.set_length(len, 0);
     if (!ctx->is_bulk()) {
       ctx->zblob_write_blobref(field->field_no, ctx->m_mtr);
     }
@@ -469,15 +457,11 @@ dberr_t z_insert(InsertContext *ctx, trx_t *trx, ref_t &ref,
 
   ulint nth_chunk = 0;
 
-  ut_o(const) ulint chunk_size = Z_CHUNK_SIZE;
-
-  DBUG_EXECUTE_IF("zlob_reduce_chunk_size", chunk_size = 20000;);
-
   while (remain > 0) {
     ut_ad(first.get_page_type() == FIL_PAGE_TYPE_ZLOB_FIRST);
 
     z_index_entry_t entry(mtr, index);
-    ulint size = (remain >= chunk_size) ? chunk_size : remain;
+    ulint size = (remain >= Z_CHUNK_SIZE) ? Z_CHUNK_SIZE : remain;
 
     err = z_insert_chunk(index, first, trx, ref, ptr, size, &entry, mtr,
                          ctx->is_bulk());
@@ -511,8 +495,8 @@ dberr_t z_insert(InsertContext *ctx, trx_t *trx, ref_t &ref,
   field_ref = ctx->get_field_ref(field->field_no);
   ref.set_ref(field_ref);
 
-  ref.update(space_id, first_page_no, 1, nullptr);
-  ref.set_length(len, nullptr);
+  ref.update(space_id, first_page_no, 1, 0);
+  ref.set_length(len, 0);
 
   ctx->make_nth_extern(field->field_no);
 
@@ -551,8 +535,12 @@ dberr_t z_print_info(const dict_index_t *index, const lob::ref_t &ref,
   return (DB_SUCCESS);
 }
 
-buf_block_t *z_frag_page_t::alloc(z_first_page_t &first, page_no_t hint,
-                                  bool bulk) {
+/** Allocate the fragment page.
+@param[in]	hint	hint page number for allocation.
+@param[in]	bulk	true if bulk operation (OPCODE_INSERT_BULK)
+                        false otherwise.
+@return the allocated buffer block. */
+buf_block_t *z_frag_page_t::alloc(page_no_t hint, bool bulk) {
   /* The m_block member could point to valid block.  Overwriting it is
   good enough. */
 
@@ -568,26 +556,7 @@ buf_block_t *z_frag_page_t::alloc(z_first_page_t &first, page_no_t hint,
   /* Set page type to FIL_PAGE_TYPE_ZLOB_FRAG. */
   set_page_type();
   set_version_0();
-
-  /* All allocated fragment pages are linked via the next page of the first page
-   * of LOB. */
-  page_no_t frag_page_no = first.get_frag_page_no();
-
-  if (frag_page_no == 0) {
-    /* If the frag_page_no is equal to 0, it means that this LOB was created
-     * before storing the fragment page list in the FIL_PAGE_PREV of the first
-     * page.  So don't change that. */
-  } else {
-    if (frag_page_no != FIL_NULL) {
-      /* Load the first fragment page and updates its prev page. */
-      z_frag_page_t tmp(m_mtr, m_index);
-      tmp.load_x(frag_page_no);
-      tmp.set_page_prev(get_page_no());
-    }
-    set_page_next(frag_page_no);
-    set_page_prev(FIL_NULL);
-    first.set_frag_page_no(get_page_no());
-  }
+  set_page_next(FIL_NULL);
 
   set_frag_entry_null();
 
@@ -766,45 +735,10 @@ z_frag_entry_t z_frag_page_t::get_frag_entry_s() {
   return (entry);
 }
 
-void z_frag_page_t::dealloc_with_entry(z_first_page_t &first,
-                                       mtr_t *alloc_mtr) {
+void z_frag_page_t::dealloc(z_first_page_t &first, mtr_t *alloc_mtr) {
   ut_ad(get_n_frags() == 0);
   z_frag_entry_t entry = get_frag_entry_x();
   entry.purge(first.frag_list(), first.free_frag_list());
-
-  page_no_t top_frag_page = first.get_frag_page_no();
-
-  if (top_frag_page == 0) {
-    /* If the first page contains 0 in FIL_PAGE_PREV, then this LOB does not use
-     * FIL_PAGE_PREV to point to the doubly-linked list of fragment pages.  In
-     * this case, don't touch FIL_PAGE_PREV. */
-  } else {
-    page_no_t next_frag_page = get_next_page_no();
-    page_no_t prev_frag_page = get_prev_page_no();
-
-    if (top_frag_page == get_page_no()) {
-      /* The fragment page pointed to by first LOB page is being deallocated. */
-      ut_ad(prev_frag_page == FIL_NULL);
-      first.set_frag_page_no(alloc_mtr, next_frag_page);
-    } else {
-      ut_ad(prev_frag_page != FIL_NULL);
-    }
-
-    /* The fragment pages are doubly linked via FIL_PAGE_NEXT and
-     * FIL_PAGE_PREV. Update the links before deallocating a fragment page. */
-    if (next_frag_page != FIL_NULL) {
-      z_frag_page_t zfp_next(alloc_mtr, m_index);
-      zfp_next.load_x(next_frag_page);
-      zfp_next.set_page_prev(prev_frag_page);
-    }
-
-    if (prev_frag_page != FIL_NULL) {
-      z_frag_page_t zfp_prev(alloc_mtr, m_index);
-      zfp_prev.load_x(prev_frag_page);
-      zfp_prev.set_page_next(next_frag_page);
-    }
-  }
-
   btr_page_free_low(m_index, m_block, ULINT_UNDEFINED, alloc_mtr);
   m_block = nullptr;
 }
@@ -929,11 +863,10 @@ void z_frag_page_t::dealloc_frag_id() {
 }
 
 /** Insert a large object (LOB) into the system.
-@param[in]	ctx	the B-tree context for this LOB operation.
-@param[in]	trx	transaction doing the insertion.
-@param[in,out]	ref	the LOB reference.
-@param[in]	field	the LOB field.
-@param[in]	field_j	the LOB field index in big rec vector.
+@param[in]      ctx    the B-tree context for this LOB operation.
+@param[in]      trx    transaction doing the insertion.
+@param[in,out]  ref    the LOB reference.
+@param[in]      field  the LOB field.
 @return DB_SUCCESS on success, error code on failure.*/
 dberr_t insert(InsertContext *ctx, trx_t *trx, ref_t &ref,
                big_rec_field_t *field, ulint field_j) {
