@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2021, Oracle and/or its affiliates.
+Copyright (c) 1994, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -40,6 +40,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "handler0alter.h"
 #include "rem0cmp.h"
 #include "srv0srv.h"
+
 namespace dd {
 class Spatial_reference_system;
 }
@@ -71,9 +72,9 @@ has more fields than the other. */
 @param[in] b_length length of b, in bytes (not UNIV_SQL_NULL)
 @return positive, 0, negative, if a is greater, equal, less than b,
 respectively */
-static inline int innobase_mysql_cmp(ulint prtype, const byte *a,
-                                     size_t a_length, const byte *b,
-                                     size_t b_length) {
+UNIV_INLINE
+int innobase_mysql_cmp(ulint prtype, const byte *a, size_t a_length,
+                       const byte *b, size_t b_length) {
 #ifdef UNIV_DEBUG
   switch (prtype & DATA_MYSQL_TYPE_MASK) {
     case MYSQL_TYPE_BIT:
@@ -109,8 +110,7 @@ static inline int innobase_mysql_cmp(ulint prtype, const byte *a,
     return (cs->coll->strnncollsp(cs, a, a_length, b, b_length));
   }
 
-  ib::fatal(UT_LOCATION_HERE, ER_IB_MSG_919)
-      << "Unable to find charset-collation " << cs_num;
+  ib::fatal(ER_IB_MSG_919) << "Unable to find charset-collation " << cs_num;
   return (0);
 }
 
@@ -303,7 +303,7 @@ static int cmp_gis_field(
     return (cmp_geometry_field(DATA_GEOMETRY, DATA_GIS_MBR, a, a_length, b,
                                b_length));
   } else {
-    return (rtree_key_cmp(mode, a, a_length, b, b_length, srs) ? 0 : 1);
+    return (rtree_key_cmp(mode, a, a_length, b, b_length, srs));
   }
 }
 
@@ -363,7 +363,7 @@ static int cmp_whole_field(ulint mtype, ulint prtype, bool is_asc,
                                     " using a character set collation!";
         ut_ad(0);
       }
-      [[fallthrough]];
+      /* fall through */
     case DATA_VARMYSQL:
     case DATA_MYSQL:
       cmp = innobase_mysql_cmp(prtype, a, a_length, b, b_length);
@@ -373,8 +373,7 @@ static int cmp_whole_field(ulint mtype, ulint prtype, bool is_asc,
     case DATA_GEOMETRY:
       return (cmp_geometry_field(mtype, prtype, a, a_length, b, b_length));
     default:
-      ib::fatal(UT_LOCATION_HERE, ER_IB_MSG_921)
-          << "Unknown data type number " << mtype;
+      ib::fatal(ER_IB_MSG_921) << "Unknown data type number " << mtype;
       cmp = 0;
   }
   if (!is_asc) {
@@ -420,7 +419,7 @@ inline int cmp_data(ulint mtype, ulint prtype, bool is_asc, const byte *data1,
         pad = 0x20;
         break;
       }
-      [[fallthrough]];
+      /* fall through */
     case DATA_INT:
     case DATA_SYS_CHILD:
     case DATA_SYS:
@@ -444,7 +443,7 @@ inline int cmp_data(ulint mtype, ulint prtype, bool is_asc, const byte *data1,
         pad = ULINT_UNDEFINED;
         break;
       }
-      [[fallthrough]];
+      /* fall through */
     default:
       return (cmp_whole_field(mtype, prtype, is_asc, data1, (unsigned)len1,
                               data2, (unsigned)len2));
@@ -692,10 +691,11 @@ int cmp_dtuple_rec_with_match_low(const dtuple_t *dtuple, const rec_t *rec,
 }
 
 /** Get the pad character code point for a type.
-@param[in]	type SQL data type
+@param[in]	type
 @return		pad character code point
 @retval		ULINT_UNDEFINED if no padding is specified */
-static inline ulint cmp_get_pad_char(const dtype_t *type) {
+UNIV_INLINE
+ulint cmp_get_pad_char(const dtype_t *type) {
   switch (type->mtype) {
     case DATA_FIXBINARY:
     case DATA_BINARY:
@@ -705,7 +705,7 @@ static inline ulint cmp_get_pad_char(const dtype_t *type) {
         VARBINARY or BINARY columns. */
         return (ULINT_UNDEFINED);
       }
-      [[fallthrough]];
+      /* Fall through */
     case DATA_CHAR:
     case DATA_VARCHAR:
     case DATA_MYSQL:
@@ -720,7 +720,7 @@ static inline ulint cmp_get_pad_char(const dtype_t *type) {
       if (!(type->prtype & DATA_BINARY_TYPE)) {
         return (0x20);
       }
-      [[fallthrough]];
+      /* Fall through */
     default:
       /* No padding specified */
       return (ULINT_UNDEFINED);
@@ -803,7 +803,7 @@ int cmp_dtuple_rec_with_match_bytes(const dtuple_t *dtuple, const rec_t *rec,
         if (type->prtype & DATA_BINARY_TYPE) {
           break;
         }
-        [[fallthrough]];
+        /* fall through */
       default:
         ut_ad(!(dfield_is_multi_value(dfield) &&
                 dtuple_f_len == UNIV_MULTI_VALUE_ARRAY_MARKER));
@@ -909,7 +909,7 @@ bool cmp_dtuple_is_prefix_of_rec(const dtuple_t *dtuple, const rec_t *rec,
 @retval positive if rec1 field is greater than rec2
 @retval negative if rec1 field is less than rec2
 @retval 0 if rec1 field equals to rec2 */
-[[nodiscard]] static int cmp_rec_rec_simple_field(
+static MY_ATTRIBUTE((warn_unused_result)) int cmp_rec_rec_simple_field(
     const rec_t *rec1, const rec_t *rec2, const ulint *offsets1,
     const ulint *offsets2, const dict_index_t *index, ulint n) {
   ulint rec1_f_len;
@@ -931,20 +931,22 @@ bool cmp_dtuple_is_prefix_of_rec(const dtuple_t *dtuple, const rec_t *rec,
 
 int cmp_rec_rec_simple(const rec_t *rec1, const rec_t *rec2,
                        const ulint *offsets1, const ulint *offsets2,
-                       const dict_index_t *index, TABLE *table) {
+                       const dict_index_t *index, struct TABLE *table) {
   ulint n;
-  bool null_eq{};
-  const auto n_uniq = dict_index_get_n_unique(index);
+  ulint n_uniq = dict_index_get_n_unique(index);
+  bool null_eq = false;
 
   ut_ad(rec_offs_n_fields(offsets1) >= n_uniq);
-  ut_ad(rec_offs_comp(offsets1) == rec_offs_comp(offsets2));
   ut_ad(rec_offs_n_fields(offsets2) == rec_offs_n_fields(offsets2));
 
-  for (n = 0; n < n_uniq; n++) {
-    auto r = cmp_rec_rec_simple_field(rec1, rec2, offsets1, offsets2, index, n);
+  ut_ad(rec_offs_comp(offsets1) == rec_offs_comp(offsets2));
 
-    if (r != 0) {
-      return r;
+  for (n = 0; n < n_uniq; n++) {
+    int cmp =
+        cmp_rec_rec_simple_field(rec1, rec2, offsets1, offsets2, index, n);
+
+    if (cmp) {
+      return (cmp);
     }
 
     /* If the fields are internally equal, they must both
@@ -964,30 +966,35 @@ int cmp_rec_rec_simple(const rec_t *rec1, const rec_t *rec2,
   if (!null_eq && table && dict_index_is_unique(index)) {
     /* Report erroneous row using new version of table. */
     innobase_rec_to_mysql(table, rec1, index, offsets1);
-    return 0;
+    return (0);
   }
 
-  /* Else, keep comparing so that we have the full internal order. */
+  /* Else, keep comparing so that we have the full internal
+  order. */
   for (; n < dict_index_get_n_fields(index); n++) {
-    auto r = cmp_rec_rec_simple_field(rec1, rec2, offsets1, offsets2, index, n);
+    int cmp =
+        cmp_rec_rec_simple_field(rec1, rec2, offsets1, offsets2, index, n);
 
-    if (r != 0) {
-      return r;
+    if (cmp) {
+      return (cmp);
     }
 
-    /* If the fields are equal, they must both be NULL or non-NULL. */
+    /* If the fields are internally equal, they must both
+    be NULL or non-NULL. */
     ut_ad(rec_offs_nth_sql_null(offsets1, n) ==
           rec_offs_nth_sql_null(offsets2, n));
   }
 
-  return 0;
+  /* This should never be reached. Internally, an index must
+  never contain duplicate entries. */
+  ut_ad(0);
+  return (0);
 }
 
 int cmp_rec_rec_with_match(const rec_t *rec1, const rec_t *rec2,
                            const ulint *offsets1, const ulint *offsets2,
-                           const dict_index_t *index,
-                           bool spatial_index_non_leaf, bool nulls_unequal,
-                           ulint *matched_fields, bool cmp_btree_recs) {
+                           const dict_index_t *index, bool nulls_unequal,
+                           ulint *matched_fields) {
   ut_ad(rec1 != nullptr);
   ut_ad(rec2 != nullptr);
   ut_ad(index != nullptr);
@@ -1001,25 +1008,20 @@ int cmp_rec_rec_with_match(const rec_t *rec1, const rec_t *rec2,
 
   *matched_fields = 0;
 
-  /* NOTE: This is an optimisation where we're comparing two B-tree records
-  during index validation. */
-  if (cmp_btree_recs) {
-    /* Test if rec is the predefined minimum record */
-    if (rec_get_info_bits(rec1, comp) & REC_INFO_MIN_REC_FLAG) {
-      ut_ad(!(rec_get_info_bits(rec2, comp) & REC_INFO_MIN_REC_FLAG));
-      return (-1);
-    } else if (rec_get_info_bits(rec2, comp) & REC_INFO_MIN_REC_FLAG) {
-      return (1);
-    }
+  /* Test if rec is the predefined minimum record */
+  if (rec_get_info_bits(rec1, comp) & REC_INFO_MIN_REC_FLAG) {
+    /* There should only be one such record. */
+    ut_ad(!(rec_get_info_bits(rec2, comp) & REC_INFO_MIN_REC_FLAG));
+    return (-1);
+  } else if (rec_get_info_bits(rec2, comp) & REC_INFO_MIN_REC_FLAG) {
+    return (1);
   }
 
   ulint i;
 
   for (i = 0; i < rec1_n_fields && i < rec2_n_fields; ++i) {
     /* If this is node-ptr records then avoid comparing node-ptr
-    field. Only key field needs to be compared. In case of a
-    spatial index we need to compare the node-ptr for a non-leaf
-    page */
+    field. Only key field needs to be compared. */
     if (i == dict_index_get_n_unique_in_tree(index)) {
       *matched_fields = i;
       return (0);
@@ -1034,15 +1036,6 @@ int cmp_rec_rec_with_match(const rec_t *rec1, const rec_t *rec2,
       mtype = DATA_BINARY;
       prtype = 0;
       is_asc = true;
-    } else if ((i == 1) && spatial_index_non_leaf) {
-      /* When the page is non-leaf spatial index page we should
-      not depend upon the dictionary information because the
-      page doesn't hold any primary key information.The spatial
-      non-leaf has only two fields MBR and page number to child
-      node.*/
-      mtype = DATA_SYS_CHILD;
-      prtype = 0;
-      is_asc = true;
     } else {
       auto col = index->get_col(i);
       const auto field = index->get_field(i);
@@ -1050,13 +1043,13 @@ int cmp_rec_rec_with_match(const rec_t *rec1, const rec_t *rec2,
       mtype = col->mtype;
       prtype = col->prtype;
       is_asc = field->is_ascending;
-    }
 
-    /* If the index is spatial index, we mark the
-    prtype of the first field as MBR field. */
-    if (i == 0 && dict_index_is_spatial(index)) {
-      ut_ad(DATA_GEOMETRY_MTYPE(mtype));
-      prtype |= DATA_GIS_MBR;
+      /* If the index is spatial index, we mark the
+      prtype of the first field as MBR field. */
+      if (i == 0 && dict_index_is_spatial(index)) {
+        ut_ad(DATA_GEOMETRY_MTYPE(mtype));
+        prtype |= DATA_GIS_MBR;
+      }
     }
 
     /* We should never encounter an externally stored field.
