@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2012, 2023, Oracle and/or its affiliates.
+Copyright (c) 2012, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -33,10 +33,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include <errno.h>
 #include <my_aes.h>
 
-#include "dd/cache/dictionary_client.h"
-#include "sql/dd/dictionary.h"
-#include "sql/dd/types/column_type_element.h"
-
 #include "dict0dd.h"
 #include "fsp0sysspace.h"
 #include "ha_prototypes.h"
@@ -50,36 +46,27 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 /** Write the meta data (index user fields) config file.
  @return DB_SUCCESS or error code. */
-[[nodiscard]] static dberr_t row_quiesce_write_index_fields(
-    const dict_index_t *index, /*!< in: write the meta data for
-                               this index */
-    FILE *file,                /*!< in: file to write to */
-    THD *thd)                  /*!< in/out: session */
+static MY_ATTRIBUTE((warn_unused_result)) dberr_t
+    row_quiesce_write_index_fields(
+        const dict_index_t *index, /*!< in: write the meta data for
+                                   this index */
+        FILE *file,                /*!< in: file to write to */
+        THD *thd)                  /*!< in/out: session */
 {
-  /* This row will store prefix_len, fixed_len,
-  and in IB_EXPORT_CFG_VERSION_V4, is_ascending */
-  byte row[sizeof(uint32_t) * 3];
-  size_t row_len = sizeof(row);
+  byte row[sizeof(ib_uint32_t) * 2];
 
   for (ulint i = 0; i < index->n_fields; ++i) {
     byte *ptr = row;
     const dict_field_t *field = &index->fields[i];
 
     mach_write_to_4(ptr, field->prefix_len);
-    ptr += sizeof(uint32_t);
+    ptr += sizeof(ib_uint32_t);
 
     mach_write_to_4(ptr, field->fixed_len);
-    ptr += sizeof(uint32_t);
-
-    /* In IB_EXPORT_CFG_VERSION_V4 we also write the is_ascending boolean. */
-    mach_write_to_4(ptr, field->is_ascending);
-
-    DBUG_EXECUTE_IF("ib_export_use_cfg_version_3",
-                    row_len = sizeof(uint32_t) * 2;);
 
     DBUG_EXECUTE_IF("ib_export_io_write_failure_9", close(fileno(file)););
 
-    if (fwrite(row, 1, row_len, file) != row_len) {
+    if (fwrite(row, 1, sizeof(row), file) != sizeof(row)) {
       ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
                   strerror(errno), "while writing index fields.");
 
@@ -87,7 +74,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
     }
 
     /* Include the NUL byte in the length. */
-    uint32_t len = static_cast<uint32_t>(strlen(field->name) + 1);
+    ib_uint32_t len = static_cast<ib_uint32_t>(strlen(field->name) + 1);
     ut_a(len > 1);
 
     mach_write_to_4(row, len);
@@ -106,19 +93,20 @@ this program; if not, write to the Free Software Foundation, Inc.,
   return (DB_SUCCESS);
 }
 /** Write the meta data config file index information
-@param[in]      index   write metadata for this index
-@param[in,out]  file    file to write to
-@param[in,out]  thd     session
+@param[in]	index	write metadata for this index
+@param[in,out]	file	file to write to
+@param[in,out]	thd	session
 @return DB_SUCCESS or error code. */
-[[nodiscard]] static dberr_t row_quiesce_write_one_index(
-    const dict_index_t *index, FILE *file, THD *thd) {
+static MY_ATTRIBUTE((warn_unused_result)) dberr_t
+    row_quiesce_write_one_index(const dict_index_t *index, FILE *file,
+                                THD *thd) {
   dberr_t err;
   byte *ptr;
   byte row[sizeof(space_index_t) + sizeof(uint32_t) * 8];
 
   ptr = row;
 
-  static_assert(sizeof(space_index_t) == 8);
+  ut_ad(sizeof(space_index_t) == 8);
   mach_write_to_8(ptr, index->id);
   ptr += sizeof(space_index_t);
 
@@ -177,7 +165,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 /** Write the meta data config file index information.
  @return DB_SUCCESS or error code. */
-[[nodiscard]] static MY_ATTRIBUTE((nonnull)) dberr_t
+static MY_ATTRIBUTE((nonnull, warn_unused_result)) dberr_t
     row_quiesce_write_indexes(const dict_table_t *table, /*!< in: write the meta
                                                          data for this table */
                               FILE *file, /*!< in: file to write to */
@@ -208,6 +196,8 @@ this program; if not, write to the Free Software Foundation, Inc.,
     return (DB_IO_ERROR);
   }
 
+  dberr_t err = DB_SUCCESS;
+
   /* Write SDI Index */
   if (has_sdi) {
     dict_mutex_enter_for_mysql();
@@ -216,163 +206,31 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
     dict_mutex_exit_for_mysql();
 
-    ut_ad(index != nullptr);
-    const auto err = row_quiesce_write_one_index(index, file, thd);
-    if (err != DB_SUCCESS) {
-      return err;
-    }
+    ut_ad(index != NULL);
+    err = row_quiesce_write_one_index(index, file, thd);
   }
 
   /* Write the table indexes meta data. */
-  for (const dict_index_t *index : table->indexes) {
-    const auto err = row_quiesce_write_one_index(index, file, thd);
-    if (err != DB_SUCCESS) {
-      return err;
-    }
+  for (const dict_index_t *index = UT_LIST_GET_FIRST(table->indexes);
+       index != 0 && err == DB_SUCCESS;
+       index = UT_LIST_GET_NEXT(indexes, index)) {
+    err = row_quiesce_write_one_index(index, file, thd);
   }
 
-  return DB_SUCCESS;
+  if (err != DB_SUCCESS) {
+    return (err);
+  }
+
+  return (err);
 }
 
 /** Write the metadata (table columns) config file. Serialise the contents
 of dict_col_t default value part if exists.
-@param[in]  col         column to which the default value belongs
-@param[in]  col_name    column name
-@param[in,out]  file        file to write to
-@param[in]  dict_table  InnoDB table cache
-@param[in]  thd         THD
+@param[in]	col	column to which the default value belongs
+@param[in]	file	file to write to
 @return DB_SUCCESS or DB_IO_ERROR. */
-static dberr_t row_quiesce_write_dropped_col_metadata(
-    const dict_col_t *col, const char *col_name, FILE *file,
-    const dict_table_t *dict_table, THD *thd) {
-  /* Total metadata to be written
-    1 byte for is NULLABLE
-    1 byte for is_unsigned
-    4 bytes for char_length
-    4 bytes for column type
-    4 bytes for numeric scale
-    8 bytes for collation id */
-  constexpr size_t METADATA_SIZE = 22;
-
-  byte row[METADATA_SIZE];
-
-  ut_ad(col->is_instant_dropped());
-
-  const dd::Table *dd_table = nullptr;
-  dd::cache::Dictionary_client *client = dd::get_dd_client(thd);
-  dd::cache::Dictionary_client::Auto_releaser releaser(client);
-
-  /* Find schema and table name */
-  std::string dict_name(dict_table->name.m_name);
-  std::string schema_name;
-  std::string table_name;
-  size_t table_begin = dict_name.find(dict_name::SCHEMA_SEPARATOR);
-  /* schema name must be part of table name */
-  ut_ad(table_begin != std::string::npos);
-
-  schema_name.assign(dict_name.substr(0, table_begin));
-  table_begin += dict_name::SCHEMA_SEPARATOR_LEN;
-
-  /* If partition table, skip partition name. */
-  size_t part_name_begin = dict_name.find(dict_name::PART_SEPARATOR);
-  if (part_name_begin != std::string::npos) {
-    table_name.assign(dict_name, table_begin, part_name_begin - table_begin);
-  } else {
-    table_name.assign(dict_name.substr(table_begin));
-  }
-
-  if (client->acquire(dd::String_type(schema_name.c_str()),
-                      dd::String_type(table_name.c_str()), &dd_table)) {
-    ut_ad(false);
-    return (DB_ERROR);
-  }
-
-  if (dd_table == nullptr) {
-    ut_ad(false);
-    return DB_ERROR;
-  }
-
-  const dd::Column *column = dd_find_column(dd_table, col_name);
-  ut_ad(column != nullptr);
-
-  byte *ptr = row;
-
-  /* 1 byte for is NULLABLE */
-  mach_write_to_1(ptr, column->is_nullable());
-  ptr++;
-
-  /* 1 byte for is_unsigned */
-  mach_write_to_1(ptr, column->is_unsigned());
-  ptr++;
-
-  /* 4 bytes for char_length() */
-  mach_write_to_4(ptr, column->char_length());
-  ptr += 4;
-
-  /* 4 bytes for column type */
-  mach_write_to_4(ptr, (uint32_t)column->type());
-  ptr += 4;
-
-  /* 4 bytes for numeric scale */
-  mach_write_to_4(ptr, column->numeric_scale());
-  ptr += 4;
-
-  /* 8 bytes for collation id */
-  mach_write_to_8(ptr, column->collation_id());
-  ptr += 8;
-
-  if (fwrite(row, 1, sizeof(row), file) != sizeof(row)) {
-    return (DB_IO_ERROR);
-  }
-
-  /* Write elements for enum column type.
-  [4]     bytes : number of elements
-  For each element
-    [4]     bytes : element name length (len+1)
-    [len+1] bytes : element name */
-  if (column->type() == dd::enum_column_types::ENUM ||
-      column->type() == dd::enum_column_types::SET) {
-    byte _row[sizeof(uint32_t)];
-
-    /* Write element count */
-    mach_write_to_4(_row, column->elements().size());
-    if (fwrite(_row, 1, sizeof(_row), file) != sizeof(_row)) {
-      ib_senderrf(
-          thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno, strerror(errno),
-          "while writing enum column element count for column %s.", col_name);
-
-      return (DB_IO_ERROR);
-    }
-
-    /* Write out the enum/set column element name as [len, byte array]. */
-    for (const auto *source_elem : column->elements()) {
-      const char *elem_name = source_elem->name().c_str();
-      uint32_t len = strlen(elem_name) + 1;
-      ut_a(len > 1);
-
-      mach_write_to_4(_row, len);
-
-      if (fwrite(_row, 1, sizeof(len), file) != sizeof(len) ||
-          fwrite(elem_name, 1, len, file) != len) {
-        ib_senderrf(
-            thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno, strerror(errno),
-            "while writing enum column element name for column %s.", col_name);
-
-        return (DB_IO_ERROR);
-      }
-    }
-  }
-
-  return (DB_SUCCESS);
-}
-
-/** Write the metadata (table columns) config file. Serialise the contents
-of dict_col_t default value part if exists.
-@param[in]      col     column to which the default value belongs
-@param[in]      file    file to write to
-@return DB_SUCCESS or DB_IO_ERROR. */
-[[nodiscard]] static dberr_t row_quiesce_write_default_value(
-    const dict_col_t *col, FILE *file) {
+static MY_ATTRIBUTE((warn_unused_result)) dberr_t
+    row_quiesce_write_default_value(const dict_col_t *col, FILE *file) {
   byte row[6];
 
   if (col->instant_default != nullptr) {
@@ -406,39 +264,39 @@ of dict_col_t default value part if exists.
 
 /** Write the meta data (table columns) config file. Serialise the contents of
  dict_col_t structure, along with the column name. All fields are serialized
- as uint32_t.
+ as ib_uint32_t.
  @return DB_SUCCESS or error code. */
-[[nodiscard]] static dberr_t row_quiesce_write_table(
-    const dict_table_t *table, /*!< in: write the meta
-                               data for this table */
-    FILE *file,                /*!< in: file to write to */
-    THD *thd)                  /*!< in/out: session */
+static MY_ATTRIBUTE((warn_unused_result)) dberr_t
+    row_quiesce_write_table(const dict_table_t *table, /*!< in: write the meta
+                                                       data for this table */
+                            FILE *file, /*!< in: file to write to */
+                            THD *thd)   /*!< in/out: session */
 {
   dict_col_t *col;
-  byte row[sizeof(uint32_t) * 7];
+  byte row[sizeof(ib_uint32_t) * 7];
 
   col = table->cols;
 
-  for (ulint i = 0; i < table->get_total_cols(); ++i, ++col) {
+  for (ulint i = 0; i < table->n_cols; ++i, ++col) {
     byte *ptr = row;
 
     mach_write_to_4(ptr, col->prtype);
-    ptr += sizeof(uint32_t);
+    ptr += sizeof(ib_uint32_t);
 
     mach_write_to_4(ptr, col->mtype);
-    ptr += sizeof(uint32_t);
+    ptr += sizeof(ib_uint32_t);
 
     mach_write_to_4(ptr, col->len);
-    ptr += sizeof(uint32_t);
+    ptr += sizeof(ib_uint32_t);
 
     mach_write_to_4(ptr, col->mbminmaxlen);
-    ptr += sizeof(uint32_t);
+    ptr += sizeof(ib_uint32_t);
 
     mach_write_to_4(ptr, col->ind);
-    ptr += sizeof(uint32_t);
+    ptr += sizeof(ib_uint32_t);
 
     mach_write_to_4(ptr, col->ord_part);
-    ptr += sizeof(uint32_t);
+    ptr += sizeof(ib_uint32_t);
 
     mach_write_to_4(ptr, col->max_prefix);
 
@@ -453,13 +311,13 @@ of dict_col_t default value part if exists.
 
     /* Write out the column name as [len, byte array]. The len
     includes the NUL byte. */
-    uint32_t len;
+    ib_uint32_t len;
     const char *col_name;
 
     col_name = table->get_col_name(dict_col_get_no(col));
 
     /* Include the NUL byte in the length. */
-    len = static_cast<uint32_t>(strlen(col_name) + 1);
+    len = static_cast<ib_uint32_t>(strlen(col_name) + 1);
     ut_a(len > 1);
 
     mach_write_to_4(row, len);
@@ -474,51 +332,6 @@ of dict_col_t default value part if exists.
       return (DB_IO_ERROR);
     }
 
-    /* Write column's INSTANT metadata */
-    uint32_t cfg_version = IB_EXPORT_CFG_VERSION_V7;
-    DBUG_EXECUTE_IF("ib_export_use_cfg_version_3",
-                    cfg_version = IB_EXPORT_CFG_VERSION_V3;);
-    DBUG_EXECUTE_IF("ib_export_use_cfg_version_99",
-                    cfg_version = IB_EXPORT_CFG_VERSION_V99;);
-    if (cfg_version >= IB_EXPORT_CFG_VERSION_V7) {
-      byte row[2 + sizeof(uint32_t)];
-      byte *ptr = row;
-
-      /* version added */
-      byte value =
-          col->is_instant_added() ? col->get_version_added() : UINT8_UNDEFINED;
-      mach_write_to_1(ptr, value);
-      ptr++;
-
-      /* version dropped */
-      value = col->is_instant_dropped() ? col->get_version_dropped()
-                                        : UINT8_UNDEFINED;
-      mach_write_to_1(ptr, value);
-      ptr++;
-
-      /* physical position */
-      mach_write_to_4(ptr, col->get_phy_pos());
-
-      if (fwrite(row, 1, sizeof(row), file) != sizeof(row)) {
-        ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
-                    strerror(errno),
-                    "while writing table column instant metadata.");
-        return (DB_IO_ERROR);
-      }
-
-      /* Write DD::Column specific info for dropped columns */
-      if (col->is_instant_dropped()) {
-        const char *col_name = table->get_col_name(dict_col_get_no(col));
-        if (row_quiesce_write_dropped_col_metadata(col, col_name, file, table,
-                                                   thd) != DB_SUCCESS) {
-          ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
-                      strerror(errno),
-                      "while writing dropped column metadata.");
-          return (DB_IO_ERROR);
-        }
-      }
-    }
-
     if (row_quiesce_write_default_value(col, file) != DB_SUCCESS) {
       ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
                   strerror(errno), "while writing table column default.");
@@ -530,21 +343,17 @@ of dict_col_t default value part if exists.
 }
 
 /** Write the meta data config file header.
-@param[in]      table   write the meta data for this table
-@param[in]      file    file to write to
-@param[in,out]  thd     session
-@return DB_SUCCESS or error code. */
-[[nodiscard]] static dberr_t row_quiesce_write_header(const dict_table_t *table,
-                                                      FILE *file, THD *thd) {
-  byte value[sizeof(uint32_t)];
+ @return DB_SUCCESS or error code. */
+static MY_ATTRIBUTE((warn_unused_result)) dberr_t
+    row_quiesce_write_header(const dict_table_t *table, /*!< in: write the meta
+                                                        data for this table */
+                             FILE *file, /*!< in: file to write to */
+                             THD *thd)   /*!< in/out: session */
+{
+  byte value[sizeof(ib_uint32_t)];
 
-  /* Write the current meta-data version number. */
-  uint32_t cfg_version = IB_EXPORT_CFG_VERSION_V7;
-  DBUG_EXECUTE_IF("ib_export_use_cfg_version_3",
-                  cfg_version = IB_EXPORT_CFG_VERSION_V3;);
-  DBUG_EXECUTE_IF("ib_export_use_cfg_version_99",
-                  cfg_version = IB_EXPORT_CFG_VERSION_V99;);
-  mach_write_to_4(value, cfg_version);
+  /* Write the meta-data version number. */
+  mach_write_to_4(value, IB_EXPORT_CFG_VERSION_V3);
 
   DBUG_EXECUTE_IF("ib_export_io_write_failure_4", close(fileno(file)););
 
@@ -556,10 +365,11 @@ of dict_col_t default value part if exists.
   }
 
   /* Write the server hostname. */
+  ib_uint32_t len;
   const char *hostname = server_get_hostname();
 
   /* Play it safe and check for NULL. */
-  if (hostname == nullptr) {
+  if (hostname == 0) {
     static const char NullHostname[] = "Hostname unknown";
 
     ib::warn(ER_IB_MSG_1013) << "Unable to determine server hostname.";
@@ -568,7 +378,7 @@ of dict_col_t default value part if exists.
   }
 
   /* The server hostname includes the NUL byte. */
-  uint32_t len = static_cast<uint32_t>(strlen(hostname) + 1);
+  len = static_cast<ib_uint32_t>(strlen(hostname) + 1);
   mach_write_to_4(value, len);
 
   DBUG_EXECUTE_IF("ib_export_io_write_failure_5", close(fileno(file)););
@@ -582,8 +392,8 @@ of dict_col_t default value part if exists.
   }
 
   /* The table name includes the NUL byte. */
-  ut_a(table->name.m_name != nullptr);
-  len = static_cast<uint32_t>(strlen(table->name.m_name) + 1);
+  ut_a(table->name.m_name != NULL);
+  len = static_cast<ib_uint32_t>(strlen(table->name.m_name) + 1);
 
   /* Write the table name. */
   mach_write_to_4(value, len);
@@ -598,14 +408,14 @@ of dict_col_t default value part if exists.
     return (DB_IO_ERROR);
   }
 
-  byte row[sizeof(uint32_t) * 3];
+  byte row[sizeof(ib_uint32_t) * 3];
 
   /* Write the next autoinc value. */
   mach_write_to_8(row, table->autoinc);
 
   DBUG_EXECUTE_IF("ib_export_io_write_failure_7", close(fileno(file)););
 
-  if (fwrite(row, 1, sizeof(uint64_t), file) != sizeof(uint64_t)) {
+  if (fwrite(row, 1, sizeof(ib_uint64_t), file) != sizeof(ib_uint64_t)) {
     ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
                 strerror(errno), "while writing table autoinc value.");
 
@@ -616,15 +426,14 @@ of dict_col_t default value part if exists.
 
   /* Write the system page size. */
   mach_write_to_4(ptr, UNIV_PAGE_SIZE);
-  ptr += sizeof(uint32_t);
+  ptr += sizeof(ib_uint32_t);
 
   /* Write the table->flags. */
   mach_write_to_4(ptr, table->flags);
-  ptr += sizeof(uint32_t);
+  ptr += sizeof(ib_uint32_t);
 
-  /* Write the number of columns in the table. In case of INSTANT, include
-  dropped columns as well. */
-  mach_write_to_4(ptr, table->get_total_cols());
+  /* Write the number of columns in the table. */
+  mach_write_to_4(ptr, table->n_cols);
 
   DBUG_EXECUTE_IF("ib_export_io_write_failure_8", close(fileno(file)););
 
@@ -635,50 +444,6 @@ of dict_col_t default value part if exists.
     return (DB_IO_ERROR);
   }
 
-  if (cfg_version >= IB_EXPORT_CFG_VERSION_V5) {
-    /* write number of nullable column before first instant column */
-    mach_write_to_4(value, table->first_index()->get_instant_nullable());
-
-    if (fwrite(&value, 1, sizeof(value), file) != sizeof(value)) {
-      ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
-                  strerror(errno), "while writing table meta-data.");
-
-      return (DB_IO_ERROR);
-    }
-  }
-
-  /* Write table instant metadata */
-  if (cfg_version >= IB_EXPORT_CFG_VERSION_V7) {
-    byte row[sizeof(uint32_t) * 5];
-    byte *ptr = row;
-
-    /* Write initial column count */
-    mach_write_to_4(ptr, table->initial_col_count);
-    ptr += sizeof(uint32_t);
-
-    /* Write current column count */
-    mach_write_to_4(ptr, table->current_col_count);
-    ptr += sizeof(uint32_t);
-
-    /* Write total column count */
-    mach_write_to_4(ptr, table->total_col_count);
-    ptr += sizeof(uint32_t);
-
-    /* Write number of instantly dropped columns */
-    mach_write_to_4(ptr, table->get_n_instant_drop_cols());
-    ptr += sizeof(uint32_t);
-
-    /* Write current row version */
-    mach_write_to_4(ptr, table->current_row_version);
-
-    if (fwrite(row, 1, sizeof(row), file) != sizeof(row)) {
-      ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
-                  strerror(errno), "while writing table meta-data.");
-
-      return (DB_IO_ERROR);
-    }
-  }
-
   /* Write the space flags */
   uint32_t space_flags = fil_space_get_flags(table->space);
   ut_ad(space_flags != UINT32_UNDEFINED);
@@ -691,42 +456,28 @@ of dict_col_t default value part if exists.
     return (DB_IO_ERROR);
   }
 
-  if (cfg_version >= IB_EXPORT_CFG_VERSION_V6) {
-    /* Write compression type info. */
-    uint8_t compression_type =
-        static_cast<uint8_t>(fil_get_compression(table->space));
-    mach_write_to_1(value, compression_type);
-
-    if (fwrite(&value, 1, sizeof(uint8_t), file) != sizeof(uint8_t)) {
-      ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
-                  strerror(errno), "while writing compression type info.");
-
-      return DB_IO_ERROR;
-    }
-  }
-
   return (DB_SUCCESS);
 }
 
 /** Write the table meta data after quiesce.
  @return DB_SUCCESS or error code */
-[[nodiscard]] static dberr_t row_quiesce_write_cfg(
-    dict_table_t *table, /*!< in: write the meta data for
-                                 this table */
-    THD *thd)            /*!< in/out: session */
+static MY_ATTRIBUTE((warn_unused_result)) dberr_t
+    row_quiesce_write_cfg(dict_table_t *table, /*!< in: write the meta data for
+                                                       this table */
+                          THD *thd)            /*!< in/out: session */
 {
   dberr_t err;
   char name[OS_FILE_MAX_PATH];
 
-  dd_get_meta_data_filename(table, nullptr, name, sizeof(name));
+  dd_get_meta_data_filename(table, NULL, name, sizeof(name));
 
   ib::info(ER_IB_MSG_1014) << "Writing table metadata to '" << name << "'";
 
   FILE *file = fopen(name, "w+b");
 
-  if (file == nullptr) {
-    ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_CANT_CREATE_FILE, name, errno,
-                strerror(errno));
+  if (file == NULL) {
+    ib_errf(thd, IB_LOG_LEVEL_WARN, ER_CANT_CREATE_FILE, name, errno,
+            strerror(errno));
 
     err = DB_IO_ERROR;
   } else {
@@ -763,22 +514,23 @@ of dict_col_t default value part if exists.
 }
 
 /** Write the transfer key to CFP file.
-@param[in]      encryption_metadata  the encryption key and iv to store in file
-@param[in]      file            file to write to
-@param[in]      thd             session
+@param[in]	table		write the data for this table
+@param[in]	file		file to write to
+@param[in]	thd		session
 @return DB_SUCCESS or error code. */
-[[nodiscard]] static MY_ATTRIBUTE((nonnull)) dberr_t
-    row_quiesce_write_transfer_key(
-        const Encryption_metadata &encryption_metadata, FILE *file, THD *thd) {
-  byte key_size[sizeof(uint32_t)];
-  byte row[Encryption::KEY_LEN * 3];
+static MY_ATTRIBUTE((nonnull, warn_unused_result)) dberr_t
+    row_quiesce_write_transfer_key(const dict_table_t *table, FILE *file,
+                                   THD *thd) {
+  byte key_size[sizeof(ib_uint32_t)];
+  byte row[ENCRYPTION_KEY_LEN * 3];
   byte *ptr = row;
   byte *transfer_key = ptr;
   lint elen;
-  ut_a(encryption_metadata.can_encrypt());
+
+  ut_ad(table->encryption_key != NULL && table->encryption_iv != NULL);
 
   /* Write the encryption key size. */
-  mach_write_to_4(key_size, Encryption::KEY_LEN);
+  mach_write_to_4(key_size, ENCRYPTION_KEY_LEN);
 
   if (fwrite(&key_size, 1, sizeof(key_size), file) != sizeof(key_size)) {
     ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
@@ -789,20 +541,20 @@ of dict_col_t default value part if exists.
 
   /* Generate and write the transfer key. */
   Encryption::random_value(transfer_key);
-  if (fwrite(transfer_key, 1, Encryption::KEY_LEN, file) !=
-      Encryption::KEY_LEN) {
+  if (fwrite(transfer_key, 1, ENCRYPTION_KEY_LEN, file) != ENCRYPTION_KEY_LEN) {
     ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
                 strerror(errno), "while writing transfer key.");
 
     return (DB_IO_ERROR);
   }
 
-  ptr += Encryption::KEY_LEN;
+  ptr += ENCRYPTION_KEY_LEN;
 
   /* Encrypt tablespace key. */
-  elen = my_aes_encrypt(encryption_metadata.m_key, Encryption::KEY_LEN, ptr,
-                        transfer_key, Encryption::KEY_LEN, my_aes_256_ecb,
-                        nullptr, false);
+  elen = my_aes_encrypt(
+      reinterpret_cast<unsigned char *>(table->encryption_key),
+      ENCRYPTION_KEY_LEN, ptr, reinterpret_cast<unsigned char *>(transfer_key),
+      ENCRYPTION_KEY_LEN, my_aes_256_ecb, NULL, false);
 
   if (elen == MY_AES_BAD_DATA) {
     ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
@@ -811,18 +563,19 @@ of dict_col_t default value part if exists.
   }
 
   /* Write encrypted tablespace key */
-  if (fwrite(ptr, 1, Encryption::KEY_LEN, file) != Encryption::KEY_LEN) {
+  if (fwrite(ptr, 1, ENCRYPTION_KEY_LEN, file) != ENCRYPTION_KEY_LEN) {
     ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
                 strerror(errno), "while writing encrypted tablespace key.");
 
     return (DB_IO_ERROR);
   }
-  ptr += Encryption::KEY_LEN;
+  ptr += ENCRYPTION_KEY_LEN;
 
   /* Encrypt tablespace iv. */
-  elen = my_aes_encrypt(encryption_metadata.m_iv, Encryption::KEY_LEN, ptr,
-                        transfer_key, Encryption::KEY_LEN, my_aes_256_ecb,
-                        nullptr, false);
+  elen = my_aes_encrypt(reinterpret_cast<unsigned char *>(table->encryption_iv),
+                        ENCRYPTION_KEY_LEN, ptr,
+                        reinterpret_cast<unsigned char *>(transfer_key),
+                        ENCRYPTION_KEY_LEN, my_aes_256_ecb, NULL, false);
 
   if (elen == MY_AES_BAD_DATA) {
     ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
@@ -831,7 +584,7 @@ of dict_col_t default value part if exists.
   }
 
   /* Write encrypted tablespace iv */
-  if (fwrite(ptr, 1, Encryption::KEY_LEN, file) != Encryption::KEY_LEN) {
+  if (fwrite(ptr, 1, ENCRYPTION_KEY_LEN, file) != ENCRYPTION_KEY_LEN) {
     ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_IO_WRITE_ERROR, errno,
                 strerror(errno), "while writing encrypted tablespace iv.");
 
@@ -842,10 +595,10 @@ of dict_col_t default value part if exists.
 }
 
 /** Write the encryption data after quiesce.
-@param[in]      table           write the data for this table
-@param[in]      thd             session
+@param[in]	table		write the data for this table
+@param[in]	thd		session
 @return DB_SUCCESS or error code */
-[[nodiscard]] static MY_ATTRIBUTE((nonnull)) dberr_t
+static MY_ATTRIBUTE((nonnull, warn_unused_result)) dberr_t
     row_quiesce_write_cfp(dict_table_t *table, THD *thd) {
   dberr_t err;
   char name[OS_FILE_MAX_PATH];
@@ -857,12 +610,27 @@ of dict_col_t default value part if exists.
 
   /* Get the encryption key and iv from space */
   /* For encrypted table, before we discard the tablespace,
-  we need to save the encryption information into table, otherwise,
+  we need save the encryption information into table, otherwise,
   this information will be lost in fil_discard_tablespace along
   with fil_space_free(). */
+  if (table->encryption_key == NULL) {
+    lint old_size = mem_heap_get_size(table->heap);
 
-  fil_space_t *space = fil_space_get(table->space);
-  ut_ad(space != nullptr && FSP_FLAGS_GET_ENCRYPTION(space->flags));
+    table->encryption_key =
+        static_cast<byte *>(mem_heap_alloc(table->heap, ENCRYPTION_KEY_LEN));
+
+    table->encryption_iv =
+        static_cast<byte *>(mem_heap_alloc(table->heap, ENCRYPTION_KEY_LEN));
+
+    lint new_size = mem_heap_get_size(table->heap);
+    dict_sys->size += new_size - old_size;
+
+    fil_space_t *space = fil_space_get(table->space);
+    ut_ad(space != NULL && FSP_FLAGS_GET_ENCRYPTION(space->flags));
+
+    memcpy(table->encryption_key, space->encryption_key, ENCRYPTION_KEY_LEN);
+    memcpy(table->encryption_iv, space->encryption_iv, ENCRYPTION_KEY_LEN);
+  }
 
   srv_get_encryption_data_filename(table, name, sizeof(name));
 
@@ -871,14 +639,13 @@ of dict_col_t default value part if exists.
 
   FILE *file = fopen(name, "w+b");
 
-  if (file == nullptr) {
-    ib_senderrf(thd, IB_LOG_LEVEL_WARN, ER_CANT_CREATE_FILE, name, errno,
-                strerror(errno));
+  if (file == NULL) {
+    ib_errf(thd, IB_LOG_LEVEL_WARN, ER_CANT_CREATE_FILE, name, errno,
+            strerror(errno));
 
     err = DB_IO_ERROR;
   } else {
-    err =
-        row_quiesce_write_transfer_key(space->m_encryption_metadata, file, thd);
+    err = row_quiesce_write_transfer_key(table, file, thd);
 
     if (fflush(file) != 0) {
       char msg[BUFSIZ];
@@ -902,6 +669,10 @@ of dict_col_t default value part if exists.
     }
   }
 
+  /* Clean the encryption information */
+  table->encryption_key = NULL;
+  table->encryption_iv = NULL;
+
   return (err);
 }
 
@@ -914,7 +685,8 @@ static bool row_quiesce_table_has_fts_index(
 
   dict_mutex_enter_for_mysql();
 
-  for (const dict_index_t *index : table->indexes) {
+  for (const dict_index_t *index = UT_LIST_GET_FIRST(table->indexes);
+       index != 0; index = UT_LIST_GET_NEXT(indexes, index)) {
     if (index->type & DICT_FTS) {
       exists = true;
       break;
@@ -926,17 +698,17 @@ static bool row_quiesce_table_has_fts_index(
   return (exists);
 }
 
-/** Quiesce the tablespace that the table resides in.
-@param[in] table Quiesce this table
-@param[in,out] trx Transaction/session */
-void row_quiesce_table_start(dict_table_t *table, trx_t *trx) {
-  ut_a(trx->mysql_thd != nullptr);
+/** Quiesce the tablespace that the table resides in. */
+void row_quiesce_table_start(dict_table_t *table, /*!< in: quiesce this table */
+                             trx_t *trx) /*!< in/out: transaction/session */
+{
+  ut_a(trx->mysql_thd != 0);
   ut_a(srv_n_purge_threads > 0);
   ut_ad(!srv_read_only_mode);
 
-  ut_a(trx->mysql_thd != nullptr);
+  ut_a(trx->mysql_thd != 0);
 
-  ut_ad(fil_space_get(table->space) != nullptr);
+  ut_ad(fil_space_get(table->space) != NULL);
 
   ib::info(ER_IB_MSG_1016) << "Sync to disk of " << table->name << " started.";
 
@@ -988,13 +760,14 @@ void row_quiesce_table_start(dict_table_t *table, trx_t *trx) {
   ut_a(err == DB_SUCCESS);
 }
 
-/** Cleanup after table quiesce.
-@param[in] table Quiesce this table
-@param[in,out] trx Transaction/session */
-void row_quiesce_table_complete(dict_table_t *table, trx_t *trx) {
+/** Cleanup after table quiesce. */
+void row_quiesce_table_complete(
+    dict_table_t *table, /*!< in: quiesce this table */
+    trx_t *trx)          /*!< in/out: transaction/session */
+{
   ulint count = 0;
 
-  ut_a(trx->mysql_thd != nullptr);
+  ut_a(trx->mysql_thd != 0);
 
   /* We need to wait for the operation to complete if the
   transaction has been killed. */
@@ -1006,7 +779,8 @@ void row_quiesce_table_complete(dict_table_t *table, trx_t *trx) {
           << "Waiting for quiesce of " << table->name << " to complete";
     }
 
-    std::this_thread::sleep_for(std::chrono::seconds(1));
+    /* Sleep for a second. */
+    os_thread_sleep(1000000);
 
     ++count;
   }
@@ -1016,9 +790,9 @@ void row_quiesce_table_complete(dict_table_t *table, trx_t *trx) {
   the user tries to drop the database (remove directory). */
   char cfg_name[OS_FILE_MAX_PATH];
 
-  dd_get_meta_data_filename(table, nullptr, cfg_name, sizeof(cfg_name));
+  dd_get_meta_data_filename(table, NULL, cfg_name, sizeof(cfg_name));
 
-  os_file_delete_if_exists(innodb_data_file_key, cfg_name, nullptr);
+  os_file_delete_if_exists(innodb_data_file_key, cfg_name, NULL);
 
   ib::info(ER_IB_MSG_1024) << "Deleting the meta-data file '" << cfg_name
                            << "'";
@@ -1028,7 +802,7 @@ void row_quiesce_table_complete(dict_table_t *table, trx_t *trx) {
 
     srv_get_encryption_data_filename(table, cfp_name, sizeof(cfp_name));
 
-    os_file_delete_if_exists(innodb_data_file_key, cfp_name, nullptr);
+    os_file_delete_if_exists(innodb_data_file_key, cfp_name, NULL);
 
     ib::info(ER_IB_MSG_1025)
         << "Deleting the meta-data file '" << cfp_name << "'";
@@ -1095,24 +869,10 @@ dberr_t row_quiesce_set_state(
                 " FTS auxiliary tables will not be flushed.");
   }
 
-  if (srv_thread_is_active(srv_threads.m_trx_recovery_rollback)) {
-    /* We should wait until rollback after recovery end,
-    to lock the table consistently. */
-    srv_threads.m_trx_recovery_rollback.wait();
-  }
-  if (trx_purge_state() != PURGE_STATE_DISABLED) {
-    /* We should stop purge to lock the table consistently. */
-    trx_purge_stop();
-  }
-
-  row_mysql_lock_data_dictionary(trx, UT_LOCATION_HERE);
+  row_mysql_lock_data_dictionary(trx);
 
   dict_table_x_lock_indexes(table);
 
-  if (trx_purge_state() != PURGE_STATE_DISABLED) {
-    trx_purge_run();
-  }
-
   switch (state) {
     case QUIESCE_START:
       break;
