@@ -55,6 +55,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "btr0sea.h"
 #include "buf0flu.h"
 #include "buf0lru.h"
+#include "clone0api.h"
 #include "dict0boot.h"
 #include "dict0load.h"
 #include "dict0stats_bg.h"
@@ -107,7 +108,7 @@ bool srv_upgrade_old_undo_found = false;
 #endif /* INNODB_DD_TABLE */
 
 /* The following is the maximum allowed duration of a lock wait. */
-ulint srv_fatal_semaphore_wait_threshold = 600;
+ulong srv_fatal_semaphore_wait_threshold = 600;
 
 /* How much data manipulation language (DML) statements need to be delayed,
 in microseconds, in order to reduce the lagging of the purge thread. */
@@ -380,9 +381,18 @@ ulint srv_buf_pool_size = ULINT_MAX;
 const ulint srv_buf_pool_min_size = 5 * 1024 * 1024;
 /** Default pool size in bytes */
 const ulint srv_buf_pool_def_size = 128 * 1024 * 1024;
+/** Maximum pool size in bytes */
+const longlong srv_buf_pool_max_size = LLONG_MAX;
 /** Requested buffer pool chunk size. Each buffer pool instance consists
 of one or more chunks. */
 ulonglong srv_buf_pool_chunk_unit;
+/** Minimum buffer pool chunk size. */
+const ulonglong srv_buf_pool_chunk_unit_min = (1024 * 1024);
+/** The buffer pool chunk size must be a multiple of this number. */
+const ulonglong srv_buf_pool_chunk_unit_blk_sz = (1024 * 1024);
+/** Maximum buffer pool chunk size. */
+const ulonglong srv_buf_pool_chunk_unit_max =
+    srv_buf_pool_max_size / MAX_BUFFER_POOLS;
 /** Requested number of buffer pool instances */
 ulong srv_buf_pool_instances;
 /** Default number of buffer pool instances */
@@ -404,6 +414,9 @@ ulong srv_buf_pool_dump_pct;
 /** Lock table size in bytes */
 ulint srv_lock_table_size = ULINT_MAX;
 
+const ulong srv_idle_flush_pct_default = 100;
+ulong srv_idle_flush_pct = srv_idle_flush_pct_default;
+
 /* This parameter is deprecated. Use srv_n_io_[read|write]_threads
 instead. */
 ulong srv_n_read_io_threads;
@@ -551,7 +564,7 @@ const char *srv_io_thread_op_info[SRV_MAX_N_IO_THREADS];
 const char *srv_io_thread_function[SRV_MAX_N_IO_THREADS];
 
 #ifndef UNIV_HOTBACKUP
-static time_t srv_last_monitor_time;
+static ib_time_monotonic_t srv_last_monitor_time;
 #endif /* !UNIV_HOTBACKUP */
 
 static ib_mutex_t srv_innodb_monitor_mutex;
@@ -1055,6 +1068,15 @@ static void srv_init(void) {
 
   mutex_create(LATCH_ID_SRV_INNODB_MONITOR, &srv_innodb_monitor_mutex);
 
+  ut_d(srv_threads.shutdown_cleanup_dbg = os_event_create(nullptr));
+
+  srv_threads.m_purge_coordinator = {};
+
+  srv_threads.m_purge_workers_n = srv_n_purge_threads;
+
+  srv_threads.m_purge_workers =
+      UT_NEW_ARRAY_NOKEY(IB_thread, srv_threads.m_purge_workers_n);
+
   if (!srv_read_only_mode) {
     /* Number of purge threads + master thread */
     n_sys_threads = srv_n_purge_threads + 1;
@@ -1062,6 +1084,13 @@ static void srv_init(void) {
     srv_sys_sz += n_sys_threads * sizeof(*srv_sys->sys_threads);
   }
 
+  srv_threads.m_page_cleaner_coordinator = {};
+
+  srv_threads.m_page_cleaner_workers_n = srv_n_page_cleaners;
+
+  srv_threads.m_page_cleaner_workers =
+      UT_NEW_ARRAY_NOKEY(IB_thread, srv_threads.m_page_cleaner_workers_n);
+
   srv_sys = static_cast<srv_sys_t *>(ut_zalloc_nokey(srv_sys_sz));
 
   srv_sys->n_sys_threads = n_sys_threads;
@@ -1155,6 +1184,26 @@ void srv_free(void) {
   ut_free(srv_sys);
 
   srv_sys = 0;
+
+  if (srv_threads.m_page_cleaner_workers != nullptr) {
+    for (size_t i = 0; i < srv_threads.m_page_cleaner_workers_n; ++i) {
+      srv_threads.m_page_cleaner_workers[i] = {};
+    }
+    ut_free(srv_threads.m_page_cleaner_workers);
+    srv_threads.m_page_cleaner_workers = nullptr;
+  }
+
+  if (srv_threads.m_purge_workers != nullptr) {
+    for (size_t i = 0; i < srv_threads.m_purge_workers_n; ++i) {
+      srv_threads.m_purge_workers[i] = {};
+    }
+    ut_free(srv_threads.m_purge_workers);
+    srv_threads.m_purge_workers = nullptr;
+  }
+
+  ut_d(os_event_destroy(srv_threads.shutdown_cleanup_dbg));
+
+  srv_threads = {};
 }
 
 /** Initializes the synchronization primitives, memory system, and the thread
@@ -1186,7 +1235,7 @@ void srv_boot(void) {
 static void srv_refresh_innodb_monitor_stats(void) {
   mutex_enter(&srv_innodb_monitor_mutex);
 
-  srv_last_monitor_time = time(NULL);
+  srv_last_monitor_time = ut_time_monotonic();
 
   os_aio_refresh_stats();
 
@@ -1217,22 +1266,20 @@ ibool srv_printf_innodb_monitor(
     ulint *trx_end)       /*!< out: file position of the end of
                           the list of active transactions */
 {
-  double time_elapsed;
-  time_t current_time;
   ulint n_reserved;
   ibool ret;
 
   mutex_enter(&srv_innodb_monitor_mutex);
 
-  current_time = time(NULL);
+  const auto current_time = ut_time_monotonic();
 
   /* We add 0.001 seconds to time_elapsed to prevent division
   by zero if two users happen to call SHOW ENGINE INNODB STATUS at the
   same time */
 
-  time_elapsed = difftime(current_time, srv_last_monitor_time) + 0.001;
+  const auto time_elapsed = current_time - srv_last_monitor_time + 0.001;
 
-  srv_last_monitor_time = time(NULL);
+  srv_last_monitor_time = ut_time_monotonic();
 
   fputs("\n=====================================\n", file);
 
@@ -1558,28 +1605,35 @@ void srv_export_innodb_status(void) {
 
 #ifdef UNIV_DEBUG
   rw_lock_s_lock(&purge_sys->latch);
-  trx_id_t up_limit_id;
   trx_id_t done_trx_no = purge_sys->done.trx_no;
 
-  up_limit_id = purge_sys->view_active ? purge_sys->view.up_limit_id() : 0;
+  /* Purge always deals with transaction end points represented by
+  transaction number. We are allowed to purge transactions with number
+  below the low limit. */
+  ReadView oldest_view;
+  trx_sys->mvcc->clone_oldest_view(&oldest_view);
+  trx_id_t low_limit_no = oldest_view.view_low_limit_no();
 
   rw_lock_s_unlock(&purge_sys->latch);
 
   mutex_enter(&trx_sys->mutex);
-  trx_id_t max_trx_id = trx_sys->rw_max_trx_id;
+  /* Maximum transaction number added to history list for purge. */
+  trx_id_t max_trx_no = trx_sys->rw_max_trx_no;
   mutex_exit(&trx_sys->mutex);
 
-  if (!done_trx_no || max_trx_id < done_trx_no - 1) {
+  if (done_trx_no == 0 || max_trx_no < done_trx_no) {
     export_vars.innodb_purge_trx_id_age = 0;
   } else {
-    export_vars.innodb_purge_trx_id_age = (ulint)(max_trx_id - done_trx_no + 1);
+    /* Add 1 as done_trx_no always points to the next transaction ID. */
+    export_vars.innodb_purge_trx_id_age = (ulint)(max_trx_no - done_trx_no + 1);
   }
 
-  if (!up_limit_id || max_trx_id < up_limit_id) {
+  if (low_limit_no == 0 || max_trx_no < low_limit_no) {
     export_vars.innodb_purge_view_trx_id_age = 0;
   } else {
+    /* Add 1 as low_limit_no always points to the next transaction ID. */
     export_vars.innodb_purge_view_trx_id_age =
-        (ulint)(max_trx_id - up_limit_id);
+        (ulint)(max_trx_no - low_limit_no + 1);
   }
 #endif /* UNIV_DEBUG */
 
@@ -1589,15 +1643,16 @@ void srv_export_innodb_status(void) {
 /** A thread which prints the info output by various InnoDB monitors. */
 void srv_monitor_thread() {
   int64_t sig_count;
-  double time_elapsed;
-  time_t current_time;
-  time_t last_monitor_time;
+  ib_time_monotonic_t current_time;
+  ib_time_monotonic_t time_elapsed;
   ulint mutex_skipped;
   ibool last_srv_print_monitor;
 
   ut_ad(!srv_read_only_mode);
 
-  srv_last_monitor_time = last_monitor_time = ut_time();
+  auto last_monitor_time = ut_time_monotonic();
+  srv_last_monitor_time = last_monitor_time;
+
   mutex_skipped = 0;
   last_srv_print_monitor = srv_print_innodb_monitor;
 loop:
@@ -1608,12 +1663,12 @@ loop:
 
   os_event_wait_time_low(srv_monitor_event, 5000000, sig_count);
 
-  current_time = ut_time();
+  current_time = ut_time_monotonic();
 
-  time_elapsed = difftime(current_time, last_monitor_time);
+  time_elapsed = current_time - last_monitor_time;
 
   if (time_elapsed > 15) {
-    last_monitor_time = ut_time();
+    last_monitor_time = ut_time_monotonic();
 
     if (srv_print_innodb_monitor) {
       /* Reset mutex_skipped counter everytime
@@ -1655,19 +1710,9 @@ loop:
     }
   }
 
-  if (srv_shutdown_state >= SRV_SHUTDOWN_CLEANUP) {
-    goto exit_func;
-  }
-
-  if (srv_print_innodb_monitor || srv_print_innodb_lock_monitor) {
+  if (srv_shutdown_state.load() == SRV_SHUTDOWN_NONE) {
     goto loop;
   }
-
-  goto loop;
-
-exit_func:
-  std::atomic_thread_fence(std::memory_order_seq_cst);
-  srv_threads.m_monitor_thread_active = false;
 }
 
 /** A thread which prints warnings about semaphore waits which have lasted
@@ -1702,7 +1747,7 @@ loop:
 
   old_lsn = new_lsn;
 
-  if (difftime(time(NULL), srv_last_monitor_time) > 60) {
+  if (ut_difftime(ut_time_monotonic(), srv_last_monitor_time) > 60) {
     /* We referesh InnoDB Monitor values so that averages are
     printed from at most 60 last seconds */
 
@@ -1740,80 +1785,23 @@ loop:
 
   os_event_wait_time_low(srv_error_event, 1000000, sig_count);
 
-  if (srv_shutdown_state < SRV_SHUTDOWN_CLEANUP) {
+  if (srv_shutdown_state.load() == SRV_SHUTDOWN_NONE) {
     goto loop;
   }
-
-  std::atomic_thread_fence(std::memory_order_seq_cst);
-  srv_threads.m_error_monitor_thread_active = false;
 }
 
 /** Increment the server activity count. */
 void srv_inc_activity_count(void) { srv_sys->activity_count.inc(); }
 
-/** Check whether any background thread (except the master thread) is active.
-Send the threads wakeup signal.
-
-NOTE: this check is part of the final shutdown, when the first phase of
-shutdown has already been completed.
-@see srv_pre_dd_shutdown()
-@return name of thread that is active
-@retval NULL if no thread is active */
-const char *srv_any_background_threads_are_active() {
-  const char *thread_active = NULL;
-
-  ut_ad(!srv_threads.m_ts_alter_encrypt_thread_active);
-  ut_ad(!srv_threads.m_dict_stats_thread_active);
-
-  if (srv_read_only_mode) {
-    if (srv_threads.m_buf_resize_thread_active) {
-      thread_active = "buf_resize_thread";
-    }
-    os_event_set(srv_buf_resize_event);
-    return (thread_active);
-  } else if (srv_threads.m_error_monitor_thread_active) {
-    thread_active = "srv_error_monitor_thread";
-  } else if (srv_threads.m_timeout_thread_active) {
-    thread_active = "srv_lock_timeout thread";
-  } else if (srv_threads.m_monitor_thread_active) {
-    thread_active = "srv_monitor_thread";
-  } else if (srv_threads.m_buf_dump_thread_active) {
-    thread_active = "buf_dump_thread";
-  } else if (srv_threads.m_buf_resize_thread_active) {
-    thread_active = "buf_resize_thread";
-  }
-
-  os_event_set(srv_error_event);
-  os_event_set(srv_monitor_event);
-  os_event_set(srv_buf_dump_event);
-  os_event_set(lock_sys->timeout_event);
-  os_event_set(srv_buf_resize_event);
-
-  return (thread_active);
-}
-
 /** Check whether the master thread is active.
 This is polled during the final phase of shutdown.
 The first phase of server shutdown must have already been executed
 (or the server must not have been fully started up).
 @see srv_pre_dd_shutdown()
-@see srv_any_background_threads_are_active()
 @retval true	if any thread is active
 @retval false	if no thread is active */
-bool srv_master_thread_active() {
-  if (srv_read_only_mode) {
-    return (false);
-  }
-
-  ut_a(!srv_threads.m_dict_stats_thread_active);
-  ut_a(!srv_threads.m_ts_alter_encrypt_thread_active);
-  srv_sys_mutex_enter();
-  ut_a(srv_sys->n_threads_active[SRV_WORKER] == 0);
-  ut_a(srv_sys->n_threads_active[SRV_PURGE] == 0);
-  bool active = srv_sys->n_threads_active[SRV_MASTER] != 0;
-  srv_sys_mutex_exit();
-
-  return (active);
+bool srv_master_thread_is_active() {
+  return (srv_thread_is_active(srv_threads.m_master));
 }
 
 /** Tells the InnoDB server that there has been activity in the database
@@ -1912,21 +1900,19 @@ static ulint srv_master_evict_from_table_cache(
 /** This function prints progress message every 60 seconds during server
  shutdown, for any activities that master thread is pending on. */
 static void srv_shutdown_print_master_pending(
-    ib_time_t *last_print_time, /*!< last time the function
-                                print the message */
-    ulint n_tables_to_drop,     /*!< number of tables to
-                                be dropped */
-    ulint n_bytes_merged)       /*!< number of change buffer
-                                just merged */
+    ib_time_monotonic_t *last_print_time, /*!< last time the function
+                                          print the message */
+    ulint n_tables_to_drop,               /*!< number of tables to
+                                          be dropped */
+    ulint n_bytes_merged)                 /*!< number of change buffer
+                                          just merged */
 {
-  ib_time_t current_time;
-  double time_elapsed;
+  const auto current_time = ut_time_monotonic();
 
-  current_time = ut_time();
-  time_elapsed = ut_difftime(current_time, *last_print_time);
+  const auto time_elapsed = current_time - *last_print_time;
 
   if (time_elapsed > 60) {
-    *last_print_time = ut_time();
+    *last_print_time = ut_time_monotonic();
 
     if (n_tables_to_drop) {
       ib::info(ER_IB_MSG_1048, ulonglong{n_tables_to_drop});
@@ -1952,7 +1938,7 @@ static void srv_master_do_disabled_loop(void) {
 
   while (srv_master_thread_disabled_debug) {
     os_event_set(srv_master_thread_disabled_event);
-    if (srv_shutdown_state != SRV_SHUTDOWN_NONE) {
+    if (srv_shutdown_state.load() != SRV_SHUTDOWN_NONE) {
       break;
     }
     os_thread_sleep(100000);
@@ -2208,8 +2194,8 @@ static void srv_update_cpu_usage() {
  server is active. The second category is of such tasks which are
  performed at some interval e.g.: purge, dict_LRU cleanup etc. */
 static void srv_master_do_active_tasks(void) {
-  ib_time_t cur_time = ut_time();
-  uintmax_t counter_time = ut_time_us(NULL);
+  const auto cur_time = ut_time_monotonic();
+  auto counter_time = ut_time_monotonic_us();
 
   /* First do the tasks that we are suppose to do at each
   invocation of this function. */
@@ -2228,13 +2214,13 @@ static void srv_master_do_active_tasks(void) {
 
   ut_d(srv_master_do_disabled_loop());
 
-  if (srv_shutdown_state > 0) {
+  if (srv_shutdown_state.load() != SRV_SHUTDOWN_NONE) {
     return;
   }
 
   /* Do an ibuf merge */
   srv_main_thread_op_info = "doing insert buffer merge";
-  counter_time = ut_time_us(NULL);
+  counter_time = ut_time_monotonic_us();
   ibuf_merge_in_background(false);
   MONITOR_INC_TIME_IN_MICRO_SECS(MONITOR_SRV_IBUF_MERGE_MICROSECOND,
                                  counter_time);
@@ -2242,7 +2228,7 @@ static void srv_master_do_active_tasks(void) {
   /* Now see if various tasks that are performed at defined
   intervals need to be performed. */
 
-  if (srv_shutdown_state > 0) {
+  if (srv_shutdown_state.load() != SRV_SHUTDOWN_NONE) {
     return;
   }
 
@@ -2276,7 +2262,7 @@ static void srv_master_do_idle_tasks(void) {
   /* ALTER TABLE in MySQL requires on Unix that the table handler
   can drop tables lazily after there no longer are SELECT
   queries to them. */
-  counter_time = ut_time_us(NULL);
+  counter_time = ut_time_monotonic_us();
   srv_main_thread_op_info = "doing background drop tables";
   row_drop_tables_for_mysql_in_background();
   MONITOR_INC_TIME_IN_MICRO_SECS(MONITOR_SRV_BACKGROUND_DROP_TABLE_MICROSECOND,
@@ -2284,18 +2270,18 @@ static void srv_master_do_idle_tasks(void) {
 
   ut_d(srv_master_do_disabled_loop());
 
-  if (srv_shutdown_state > 0) {
+  if (srv_shutdown_state.load() != SRV_SHUTDOWN_NONE) {
     return;
   }
 
   /* Do an ibuf merge */
-  counter_time = ut_time_us(NULL);
+  counter_time = ut_time_monotonic_us();
   srv_main_thread_op_info = "doing insert buffer merge";
   ibuf_merge_in_background(true);
   MONITOR_INC_TIME_IN_MICRO_SECS(MONITOR_SRV_IBUF_MERGE_MICROSECOND,
                                  counter_time);
 
-  if (srv_shutdown_state > 0) {
+  if (srv_shutdown_state.load() != SRV_SHUTDOWN_NONE) {
     return;
   }
 
@@ -2318,8 +2304,8 @@ static void srv_master_do_idle_tasks(void) {
  merge
  @return true if some work was done. false otherwise */
 static ibool srv_master_do_shutdown_tasks(
-    ib_time_t *last_print_time) /*!< last time the function
-                                print the message */
+    ib_time_monotonic_t *last_print_time) /*!< last time the function
+                                          print the message */
 {
   ulint n_bytes_merged = 0;
   ulint n_tables_to_drop = 0;
@@ -2328,7 +2314,7 @@ static ibool srv_master_do_shutdown_tasks(
 
   ++srv_main_shutdown_loops;
 
-  ut_a(srv_shutdown_state > 0);
+  ut_a(srv_shutdown_state.load() != SRV_SHUTDOWN_NONE);
 
   /* In very fast shutdown none of the following is necessary */
   if (srv_fast_shutdown == 2) {
@@ -2352,10 +2338,8 @@ static ibool srv_master_do_shutdown_tasks(
 
 func_exit:
   /* Print progress message every 60 seconds during shutdown */
-  if (srv_shutdown_state > 0) {
-    srv_shutdown_print_master_pending(last_print_time, n_tables_to_drop,
-                                      n_bytes_merged);
-  }
+  srv_shutdown_print_master_pending(last_print_time, n_tables_to_drop,
+                                    n_bytes_merged);
 
   return (n_bytes_merged || n_tables_to_drop);
 }
@@ -2363,7 +2347,7 @@ func_exit:
 void undo_rotate_default_master_key() {
   fil_space_t *space;
 
-  if (srv_shutdown_state != SRV_SHUTDOWN_NONE) {
+  if (srv_shutdown_state.load() != SRV_SHUTDOWN_NONE) {
     return;
   }
 
@@ -2445,8 +2429,54 @@ bool srv_enable_redo_encryption(bool is_boot) {
   return false;
 }
 
+/* Set encryption for UNDO tablespace with given space id. */
+bool set_undo_tablespace_encryption(space_id_t space_id, mtr_t *mtr,
+                                    bool is_boot) {
+  ut_ad(fsp_is_undo_tablespace(space_id));
+  fil_space_t *space = fil_space_get(space_id);
+
+  dberr_t err;
+  byte encrypt_info[ENCRYPTION_INFO_SIZE];
+  byte key[ENCRYPTION_KEY_LEN];
+  byte iv[ENCRYPTION_KEY_LEN];
+
+  Encryption::random_value(key);
+  Encryption::random_value(iv);
+
+  /* 0 fill encryption info */
+  memset(encrypt_info, 0, ENCRYPTION_INFO_SIZE);
+
+  /* Fill up encryption info to be set */
+  if (!Encryption::fill_encryption_info(key, iv, encrypt_info, is_boot, true)) {
+    ib::error(ER_IB_MSG_1052, space->name);
+    return true;
+  }
+
+  ulint new_flags = space->flags | FSP_FLAGS_MASK_ENCRYPTION;
+
+  /* Write encryption info on tablespace header page */
+  if (!fsp_header_write_encryption(space->id, new_flags, encrypt_info, true,
+                                   false, mtr)) {
+    ib::error(ER_IB_MSG_1053, space->name);
+    return true;
+  }
+
+  /* Update In-Mem encryption information for UNDO tablespace */
+  fsp_flags_set_encryption(space->flags);
+  err = fil_set_encryption(space->id, Encryption::AES, key, iv);
+  if (err != DB_SUCCESS) {
+    ib::error(ER_IB_MSG_1054, space->name, int{err}, ut_strerr(err));
+    return true;
+  }
+
+  return false;
+}
+
 /* Enable UNDO tablespace encryption */
 bool srv_enable_undo_encryption(bool is_boot) {
+  /* Make sure undo::ddl_mutex is owned. */
+  ut_ad(mutex_own(&(undo::ddl_mutex)));
+
   /* Traverse over all UNDO tablespaces and mark them encrypted. */
   undo::spaces->s_lock();
   for (auto undo_space : undo::spaces->m_spaces) {
@@ -2466,54 +2496,14 @@ bool srv_enable_undo_encryption(bool is_boot) {
 
     undo_space->rsegs()->s_lock();
 
-    ulint new_flags = space->flags | FSP_FLAGS_MASK_ENCRYPTION;
-
     /* Make sure that there is enough reusable space in the redo log files. */
     log_free_check();
 
-    dberr_t err;
     mtr_t mtr;
-    byte encrypt_info[ENCRYPTION_INFO_SIZE];
-    byte key[ENCRYPTION_KEY_LEN];
-    byte iv[ENCRYPTION_KEY_LEN];
-
-    Encryption::random_value(key);
-    Encryption::random_value(iv);
-
     mtr_start(&mtr);
     mtr_x_lock_space(space, &mtr);
 
-    /* 0 fill encryption info */
-    memset(encrypt_info, 0, ENCRYPTION_INFO_SIZE);
-
-    /* Fill up encryption info to be set */
-    if (!Encryption::fill_encryption_info(key, iv, encrypt_info, is_boot)) {
-      ib::error(ER_IB_MSG_1052, undo_space->space_name());
-
-      mtr_commit(&mtr);
-      undo_space->rsegs()->s_unlock();
-      undo::spaces->s_unlock();
-      return true;
-    }
-
-    /* Write encryption info on tablespace header page */
-    if (!fsp_header_write_encryption(space->id, new_flags, encrypt_info, true,
-                                     false, &mtr)) {
-      ib::error(ER_IB_MSG_1053, undo_space->space_name());
-
-      mtr_commit(&mtr);
-      undo_space->rsegs()->s_unlock();
-      undo::spaces->s_unlock();
-      return true;
-    }
-
-    /* Update In-Mem encryption information for UNDO tablespace */
-    fsp_flags_set_encryption(space->flags);
-    err = fil_set_encryption(space->id, Encryption::AES, key, iv);
-    if (err != DB_SUCCESS) {
-      ib::error(ER_IB_MSG_1054, undo_space->space_name(), int{err},
-                ut_strerr(err));
-
+    if (set_undo_tablespace_encryption(undo_space->id(), &mtr, is_boot)) {
       mtr_commit(&mtr);
       undo_space->rsegs()->s_unlock();
       undo::spaces->s_unlock();
@@ -2540,15 +2530,66 @@ static void srv_master_sleep(void) {
   srv_main_thread_op_info = "";
 }
 
+/** Check redo and undo log encryption and rotate default master key. */
+static void srv_sys_check_set_encryption() {
+  /* Rotate default master key for redo log encryption if it is set */
+  if (srv_redo_log_encrypt) {
+    fil_space_t *space = fil_space_get(dict_sys_t::s_log_space_first_id);
+    ut_a(space);
+
+    /* Encryption for redo tablesapce must already have been set. This is
+    safeguard to encrypt it if not done earlier. */
+    ut_ad(FSP_FLAGS_GET_ENCRYPTION(space->flags));
+
+    if (!FSP_FLAGS_GET_ENCRYPTION(space->flags)) {
+      ib::warn(ER_IB_MSG_1285, space->name, "srv_redo_log_encrypt");
+      srv_enable_redo_encryption(false);
+    }
+    redo_rotate_default_master_key();
+  }
+
+  if (!srv_undo_log_encrypt) {
+    return;
+  }
+
+  /* Rotate default master key for undo log encryption if it is set */
+  ut_ad(!undo::spaces->empty());
+
+  mutex_enter(&(undo::ddl_mutex));
+
+  bool encrypt_undo = false;
+  undo::spaces->s_lock();
+  for (auto &undo_ts : undo::spaces->m_spaces) {
+    fil_space_t *space = fil_space_get(undo_ts->id());
+    ut_ad(space != nullptr);
+
+    /* Encryption for undo tablesapce must already have been set. This is
+    safeguard to encrypt it if not done earlier. */
+    ut_ad(FSP_FLAGS_GET_ENCRYPTION(space->flags));
+    if (!FSP_FLAGS_GET_ENCRYPTION(space->flags)) {
+      ib::warn(ER_IB_MSG_1285, space->name, "srv_undo_log_encrypt");
+      /* No need to loop further as srv_enable_undo_encryption() would
+      loop through all UNDO tablespaces and encrypt. */
+      encrypt_undo = true;
+      break;
+    }
+  }
+  undo::spaces->s_unlock();
+
+  if (encrypt_undo) {
+    ut_d(bool ret =) srv_enable_undo_encryption(false);
+    ut_ad(!ret);
+  }
+  undo_rotate_default_master_key();
+  mutex_exit(&(undo::ddl_mutex));
+}
+
 /** The master thread controlling the server. */
 void srv_master_thread() {
-  DBUG_ENTER("srv_master_thread");
+  DBUG_TRACE;
 
   srv_slot_t *slot;
   ulint old_activity_count = srv_get_activity_count();
-  ib_time_t last_print_time;
-
-  my_thread_init();
 
   THD *thd = create_thd(false, true, true, 0);
 
@@ -2560,13 +2601,13 @@ void srv_master_thread() {
   slot = srv_reserve_slot(SRV_MASTER);
   ut_a(slot == srv_sys->sys_threads);
 
-  last_print_time = ut_time();
+  auto last_print_time = ut_time_monotonic();
 loop:
   if (srv_force_recovery >= SRV_FORCE_NO_BACKGROUND) {
     goto suspend_thread;
   }
 
-  while (srv_shutdown_state == SRV_SHUTDOWN_NONE) {
+  while (srv_shutdown_state.load() == SRV_SHUTDOWN_NONE) {
     srv_master_sleep();
 
     MONITOR_INC(MONITOR_MASTER_THREAD_SLEEP);
@@ -2585,40 +2626,27 @@ loop:
     }
 
     /* Make sure that early encryption processing of UNDO/REDO log is done. */
-    if (is_early_redo_undo_encryption_done()) {
-      /* Rotate default master key for redo log encryption if it is set */
-      if (srv_redo_log_encrypt) {
-        fil_space_t *space = fil_space_get(dict_sys_t::s_log_space_first_id);
-        ut_a(space);
-        ut_ad(FSP_FLAGS_GET_ENCRYPTION(space->flags));
-
-        if (!FSP_FLAGS_GET_ENCRYPTION(space->flags)) {
-          ib::warn(ER_IB_MSG_1285, space->name, "srv_redo_log_encrypt");
-          srv_enable_redo_encryption(false);
-        }
-        redo_rotate_default_master_key();
-      }
-
-      /* Rotate default master key for undo log encryption if it is set */
-      if (srv_undo_log_encrypt) {
-        ut_ad(!undo::spaces->empty());
-        for (auto &undo_ts : undo::spaces->m_spaces) {
-          fil_space_t *space = fil_space_get(undo_ts->id());
-          if (space) {
-            ut_ad(FSP_FLAGS_GET_ENCRYPTION(space->flags));
-
-            if (!FSP_FLAGS_GET_ENCRYPTION(space->flags)) {
-              ib::warn(ER_IB_MSG_1285, space->name, "srv_undo_log_encrypt");
-              srv_enable_undo_encryption(false);
-            }
-          }
-        }
-        undo_rotate_default_master_key();
-      }
+    if (!is_early_redo_undo_encryption_done()) {
+      continue;
     }
+
+    /* Let clone wait when redo/undo log encryption is set. If clone is already
+    in progress we skip the check and come back later. */
+    if (!clone_mark_wait()) {
+      continue;
+    }
+
+    /* Check encryption property for system tablespaces. */
+    srv_sys_check_set_encryption();
+
+    /* Allow any blocking clone to progress. */
+    clone_mark_free();
   }
 
-  while (srv_shutdown_state != SRV_SHUTDOWN_EXIT_THREADS &&
+  /* This is just for test scenarios. */
+  srv_thread_delay_cleanup_if_needed(true);
+
+  while (srv_shutdown_state.load() != SRV_SHUTDOWN_MASTER_STOP &&
          srv_master_do_shutdown_tasks(&last_print_time)) {
     /* Shouldn't loop here in case of very fast shutdown */
     ut_ad(srv_fast_shutdown < 2);
@@ -2637,16 +2665,12 @@ suspend_thread:
 
   os_event_wait(slot->event);
 
-  if (srv_shutdown_state != SRV_SHUTDOWN_EXIT_THREADS) {
+  if (srv_shutdown_state.load() != SRV_SHUTDOWN_MASTER_STOP) {
     goto loop;
   }
 
   srv_main_thread_op_info = "exiting";
   destroy_thd(thd);
-  my_thread_end();
-
-  std::atomic_thread_fence(std::memory_order_seq_cst);
-  srv_threads.m_master_thread_active = false;
 }
 
 /**
@@ -2655,18 +2679,21 @@ Check if purge should stop.
 static bool srv_purge_should_exit(
     ulint n_purged) /*!< in: pages purged in last batch */
 {
-  switch (srv_shutdown_state) {
+  switch (srv_shutdown_state.load()) {
     case SRV_SHUTDOWN_NONE:
       /* Normal operation. */
       break;
 
     case SRV_SHUTDOWN_CLEANUP:
-    case SRV_SHUTDOWN_EXIT_THREADS:
       /* Exit unless slow shutdown requested or all done. */
       return (srv_fast_shutdown != 0 || n_purged == 0);
 
+    case SRV_SHUTDOWN_EXIT_THREADS:
+      return (true);
+
     case SRV_SHUTDOWN_LAST_PHASE:
     case SRV_SHUTDOWN_FLUSH_PHASE:
+    case SRV_SHUTDOWN_MASTER_STOP:
       ut_error;
   }
 
@@ -2706,8 +2733,6 @@ static bool srv_task_execute(void) {
 void srv_worker_thread() {
   srv_slot_t *slot;
 
-  my_thread_init();
-
   ut_ad(!srv_read_only_mode);
   ut_a(srv_force_recovery < SRV_FORCE_NO_BACKGROUND);
 
@@ -2752,19 +2777,16 @@ void srv_worker_thread() {
 
   ut_a(!purge_sys->running);
   ut_a(purge_sys->state == PURGE_STATE_EXIT);
-  ut_a(srv_shutdown_state > SRV_SHUTDOWN_NONE);
+  ut_a(srv_shutdown_state.load() != SRV_SHUTDOWN_NONE);
 
   rw_lock_x_unlock(&purge_sys->latch);
 
   destroy_thd(thd);
-
-  my_thread_end();
 }
 
 /** Do the actual purge operation.
  @return length of history list before the last purge batch. */
 static ulint srv_do_purge(
-    ulint n_threads,       /*!< in: number of threads to use */
     ulint *n_total_purged) /*!< in/out: total pages purged */
 {
   ulint n_pages_purged;
@@ -2774,6 +2796,8 @@ static ulint srv_do_purge(
   static ulint rseg_history_len = 0;
   ulint old_activity_count = srv_get_activity_count();
 
+  const auto n_threads = srv_threads.m_purge_workers_n;
+
   ut_a(n_threads > 0);
   ut_ad(!srv_read_only_mode);
 
@@ -2892,15 +2916,16 @@ static void srv_purge_coordinator_suspend(
 
     rw_lock_x_lock(&purge_sys->latch);
 
-    stop = (srv_shutdown_state == SRV_SHUTDOWN_NONE &&
+    stop = (srv_shutdown_state.load() == SRV_SHUTDOWN_NONE &&
             purge_sys->state == PURGE_STATE_STOP);
 
     if (!stop) {
       bool check = true;
-      DBUG_EXECUTE_IF("skip_purge_check_shutdown",
-                      if (srv_shutdown_state != SRV_SHUTDOWN_NONE &&
-                          purge_sys->state == PURGE_STATE_STOP &&
-                          srv_fast_shutdown != 0) { check = false; };);
+      DBUG_EXECUTE_IF(
+          "skip_purge_check_shutdown",
+          if (srv_shutdown_state.load() != SRV_SHUTDOWN_NONE &&
+              purge_sys->state == PURGE_STATE_STOP &&
+              srv_fast_shutdown != 0) { check = false; };);
 
       if (check) {
         ut_a(purge_sys->n_stop == 0);
@@ -2953,8 +2978,6 @@ void srv_purge_coordinator_thread() {
 
   ulint n_total_purged = ULINT_UNDEFINED;
 
-  my_thread_init();
-
   ut_ad(!srv_read_only_mode);
   ut_a(srv_n_purge_threads >= 1);
   ut_a(trx_purge_state() == PURGE_STATE_INIT);
@@ -2975,7 +2998,7 @@ void srv_purge_coordinator_thread() {
     /* If there are no records to purge or the last
     purge didn't purge any records then wait for activity. */
 
-    if (srv_shutdown_state == SRV_SHUTDOWN_NONE &&
+    if (srv_shutdown_state.load() == SRV_SHUTDOWN_NONE &&
         (purge_sys->state == PURGE_STATE_STOP || n_total_purged == 0)) {
       srv_purge_coordinator_suspend(slot, rseg_history_len);
     }
@@ -2987,10 +3010,18 @@ void srv_purge_coordinator_thread() {
 
     n_total_purged = 0;
 
-    rseg_history_len = srv_do_purge(srv_n_purge_threads, &n_total_purged);
+    rseg_history_len = srv_do_purge(&n_total_purged);
 
   } while (!srv_purge_should_exit(n_total_purged));
 
+  /* This is just for test scenarios. Do not pass thd here,
+  because it would lead to wait on event then, and we would
+  never exit the srv_pre_dd_shutdown() which waits for this
+  thread to exit. That's because the signal for which we
+  would wait is signalled in srv_shutdown which happens
+  after the srv_pre_dd_shutdown is ended. */
+  srv_thread_delay_cleanup_if_needed(false);
+
   /* Ensure that we don't jump out of the loop unless the
   exit condition is satisfied. */
 
@@ -3044,9 +3075,11 @@ void srv_purge_coordinator_thread() {
     srv_release_threads(SRV_WORKER, srv_n_purge_threads - 1);
   }
 
-  destroy_thd(thd);
+  /* This is just for test scenarios. Do not pass thd here.
+  For explanation look at comment for similar usage above. */
+  srv_thread_delay_cleanup_if_needed(false);
 
-  my_thread_end();
+  destroy_thd(thd);
 }
 
 /** Enqueues a task to server task queue and releases a worker thread, if there
@@ -3086,8 +3119,9 @@ void srv_purge_wakeup(void) {
   if (srv_force_recovery < SRV_FORCE_NO_BACKGROUND) {
     srv_release_threads(SRV_PURGE, 1);
 
-    if (srv_n_purge_threads > 1) {
-      ulint n_workers = srv_n_purge_threads - 1;
+    if (srv_threads.m_purge_workers_n > 1) {
+      /* SRV_PURGE is not counted here. */
+      ulint n_workers = srv_threads.m_purge_workers_n - 1;
 
       srv_release_threads(SRV_WORKER, n_workers);
     }
@@ -3097,25 +3131,31 @@ void srv_purge_wakeup(void) {
 /** Check if the purge threads are active, both coordinator and worker threads
 @return true if any thread is active, false if no thread is active */
 bool srv_purge_threads_active() {
-  for (uint i = 0; i < srv_sys->n_sys_threads; ++i) {
-    srv_slot_t *slot;
+  if (srv_threads.m_purge_workers == nullptr) {
+#ifdef UNIV_DEBUG
+    ut_a(srv_read_only_mode);
+#endif /* UNIV_DEBUG */
+    ut_ad(!srv_thread_is_active(srv_threads.m_purge_coordinator));
+    return (false);
+  }
 
-    slot = &srv_sys->sys_threads[i];
+  ut_ad(!srv_read_only_mode);
 
-    /* The slots for purge could be never used due to no purge
-    threads having been created, so check the flag first. */
-    if (slot->in_use) {
-      srv_thread_type type = srv_slot_get_type(slot);
-
-      if (type == SRV_WORKER || type == SRV_PURGE) {
-        return (true);
-      }
+  for (size_t i = 0; i < srv_threads.m_purge_workers_n; ++i) {
+    if (srv_thread_is_active(srv_threads.m_purge_workers[i])) {
+      return (true);
     }
   }
 
+  ut_ad(!srv_thread_is_active(srv_threads.m_purge_coordinator));
+
   return (false);
 }
 
+bool srv_thread_is_active(const IB_thread &thread) {
+  return (thread_is_active(thread));
+}
+
 #endif /* !UNIV_HOTBACKUP */
 
 const char *srv_get_server_errmsgs(int errcode) {
