@@ -162,6 +162,8 @@ static void trx_init(trx_t *trx) {
 
   trx->lock.n_rec_locks.store(0);
 
+  trx->lock.blocking_trx.store(nullptr);
+
   trx->dict_operation = TRX_DICT_OP_NONE;
 
   trx->ddl_operation = false;
@@ -282,6 +284,7 @@ struct TrxFactory {
 
     ut_a(trx->lock.wait_lock == NULL);
     ut_a(trx->lock.wait_thr == NULL);
+    ut_a(trx->lock.blocking_trx.load() == nullptr);
 
     ut_a(!trx->has_search_latch);
 
@@ -347,6 +350,7 @@ struct TrxFactory {
 
     ut_a(trx->lock.wait_thr == NULL);
     ut_a(trx->lock.wait_lock == NULL);
+    ut_a(trx->lock.blocking_trx.load() == nullptr);
 
     ut_a(!trx->has_search_latch);
 
@@ -354,7 +358,7 @@ struct TrxFactory {
 
     ut_a(UT_LIST_GET_LEN(trx->lock.trx_locks) == 0);
 
-    ut_ad(trx->autoinc_locks == NULL);
+    ut_ad(trx->lock.autoinc_locks == NULL);
 
     ut_ad(trx->lock.table_locks.empty());
 
@@ -465,7 +469,7 @@ static trx_t *trx_create_low() {
   alloc = ib_heap_allocator_create(heap);
 
   /* Remember to free the vector explicitly in trx_free(). */
-  trx->autoinc_locks = ib_vector_create(alloc, sizeof(void **), 4);
+  trx->lock.autoinc_locks = ib_vector_create(alloc, sizeof(void **), 4);
 
   /* Should have been either just initialized or .clear()ed by
   trx_free(). */
@@ -483,11 +487,11 @@ static void trx_free(trx_t *&trx) {
   trx->mysql_thd = 0;
 
   // FIXME: We need to avoid this heap free/alloc for each commit.
-  if (trx->autoinc_locks != NULL) {
-    ut_ad(ib_vector_is_empty(trx->autoinc_locks));
+  if (trx->lock.autoinc_locks != NULL) {
+    ut_ad(ib_vector_is_empty(trx->lock.autoinc_locks));
     /* We allocated a dedicated heap for the vector. */
-    ib_vector_free(trx->autoinc_locks);
-    trx->autoinc_locks = NULL;
+    ib_vector_free(trx->lock.autoinc_locks);
+    trx->lock.autoinc_locks = NULL;
   }
 
   trx->mod_tables.clear();
@@ -1249,7 +1253,7 @@ static void trx_start_low(
 
   trx->no = TRX_ID_MAX;
 
-  ut_a(ib_vector_is_empty(trx->autoinc_locks));
+  ut_a(ib_vector_is_empty(trx->lock.autoinc_locks));
   ut_a(trx->lock.table_locks.empty());
 
   /* If this transaction came from trx_allocate_for_mysql(),
@@ -1815,7 +1819,7 @@ written */
     /* Note: We are asserting without holding the lock mutex. But
     that is OK because this transaction is not waiting and cannot
     be rolled back and no new locks can (or should not) be added
-    becuase it is flagged as a non-locking read-only transaction. */
+    because it is flagged as a non-locking read-only transaction. */
 
     ut_a(UT_LIST_GET_LEN(trx->lock.trx_locks) == 0);
 
@@ -2642,7 +2646,7 @@ static lsn_t trx_prepare_low(
   return 0;
 }
 
-bool trx_is_mysql_xa(trx_t *trx) {
+bool trx_is_mysql_xa(const trx_t *trx) {
   auto my_xid = trx->xid->get_my_xid();
   return (my_xid != 0);
 }
@@ -2686,6 +2690,7 @@ static void trx_prepare(trx_t *trx) /*!< in/out: transaction */
   }
   trx_sys_mutex_exit();
   /*--------------------------------------*/
+  DEBUG_SYNC_C("trx_prepare_has_changed_state");
 
   /* Reset after successfully adding GTID to in memory table. */
   trx->persists_gtid = false;
@@ -2748,6 +2753,8 @@ dberr_t trx_prepare_for_mysql(trx_t *trx) {
 
   TrxInInnoDB trx_in_innodb(trx, true);
 
+  DEBUG_SYNC_C("trx_prepare_for_mysql_has_entered_innodb");
+
   if (trx_in_innodb.is_aborted() && trx->killed_by != os_thread_get_curr_id()) {
     return (DB_FORCED_ABORT);
   }
