@@ -1,4 +1,4 @@
-/* Copyright (c) 2001, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -119,7 +119,7 @@ static int walk_and_match(void *v_word, uint32 count, void *v_aio) {
   while (!r && gweight) {
     if (keylen && ha_compare_text(aio->charset, info->lastkey + 1,
                                   info->lastkey_length - extra - 1, keybuff + 1,
-                                  keylen - 1, false))
+                                  keylen - 1, 0))
       break;
 
     if (subkeys < 0) {
@@ -202,7 +202,7 @@ static int walk_and_push(void *v_from, uint32, void *v_best) {
   QUEUE *best = static_cast<QUEUE *>(v_best);
   DBUG_TRACE;
   from->doc.weight += from->tmp_weight * from->word_ptr->weight;
-  best->elements = std::min(best->elements, uint(ft_query_expansion_limit - 1));
+  set_if_smaller(best->elements, ft_query_expansion_limit - 1);
   queue_insert(best, (uchar *)&from->doc);
   return 0;
 }
@@ -219,15 +219,15 @@ FT_INFO *ft_init_nlq_search(MI_INFO *info, uint keynr, uchar *query,
   TREE wtree;
   ALL_IN_ONE aio;
   FT_DOC *dptr;
-  st_ft_info_nlq *dlist = nullptr;
+  st_ft_info_nlq *dlist = NULL;
   my_off_t saved_lastpos = info->lastpos;
   struct st_mysql_ftparser *parser;
   MYSQL_FTPARSER_PARAM *ftparser_param;
   DBUG_TRACE;
 
   /* black magic ON */
-  if ((int)(keynr = _mi_check_index(info, keynr)) < 0) return nullptr;
-  if (_mi_readinfo(info, F_RDLCK, 1)) return nullptr;
+  if ((int)(keynr = _mi_check_index(info, keynr)) < 0) return NULL;
+  if (_mi_readinfo(info, F_RDLCK, 1)) return NULL;
   /* black magic OFF */
 
   aio.info = info;
@@ -239,8 +239,8 @@ FT_INFO *ft_init_nlq_search(MI_INFO *info, uint keynr, uchar *query,
 
   memset(&wtree, 0, sizeof(wtree));
 
-  init_tree(&aio.dtree, 0, sizeof(FT_SUPERDOC), &FT_SUPERDOC_cmp, false,
-            nullptr, nullptr);
+  init_tree(&aio.dtree, 0, 0, sizeof(FT_SUPERDOC), &FT_SUPERDOC_cmp, 0, NULL,
+            NULL);
 
   ft_parse_init(&wtree, aio.charset);
   ftparser_param->flags = 0;
@@ -252,8 +252,8 @@ FT_INFO *ft_init_nlq_search(MI_INFO *info, uint keynr, uchar *query,
 
   if (flags & FT_EXPAND && ft_query_expansion_limit) {
     QUEUE best;
-    init_queue(&best, key_memory_QUEUE, ft_query_expansion_limit, 0, false,
-               &FT_DOC_cmp, nullptr);
+    init_queue(&best, key_memory_QUEUE, ft_query_expansion_limit, 0, 0,
+               &FT_DOC_cmp, 0);
     tree_walk(&aio.dtree, &walk_and_push, &best, left_root_right);
     while (best.elements) {
       my_off_t docid = ((FT_DOC *)queue_remove(&best, 0))->dpos;
@@ -307,12 +307,8 @@ int ft_nlq_read_next(FT_INFO *handler_base, char *record) {
   st_ft_info_nlq *handler = (st_ft_info_nlq *)handler_base;
   MI_INFO *info = (MI_INFO *)handler->info;
 
-  // Move to the next document that has a non-zero score.
-  while (++handler->curdoc < handler->ndocs &&
-         ft_nlq_get_relevance(handler) == 0.0) {
-  }
-
-  if (handler->curdoc >= handler->ndocs) {
+  if (++handler->curdoc >= handler->ndocs) {
+    --handler->curdoc;
     return HA_ERR_END_OF_FILE;
   }
 
@@ -327,8 +323,8 @@ int ft_nlq_read_next(FT_INFO *handler_base, char *record) {
 }
 
 float ft_nlq_find_relevance(FT_INFO *handler_base,
-                            uchar *record [[maybe_unused]],
-                            uint length [[maybe_unused]]) {
+                            uchar *record MY_ATTRIBUTE((unused)),
+                            uint length MY_ATTRIBUTE((unused))) {
   st_ft_info_nlq *handler = (st_ft_info_nlq *)handler_base;
   int a, b, c;
   FT_DOC *docs = handler->doc;
