@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +21,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /*
-  Functions for read record caching with myisam
+  Functions for read record cacheing with myisam
   Used for reading dynamic/compressed records from datafile.
 
   Can fetch data directly from file (outside cache),
@@ -43,8 +42,6 @@
 
 #include <sys/types.h>
 
-#include <algorithm>
-
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_macros.h"
@@ -61,7 +58,7 @@ int _mi_read_cache(IO_CACHE *info, uchar *buff, my_off_t pos, uint length,
     read_length = length;
     if ((my_off_t)read_length > (my_off_t)(info->pos_in_file - pos))
       read_length = (uint)(info->pos_in_file - pos);
-    info->seek_not_done = true;
+    info->seek_not_done = 1;
     if (mysql_file_pread(info->file, buff, read_length, pos, MYF(MY_NABP)))
       return 1;
     if (!(length -= read_length)) return 0;
@@ -72,7 +69,7 @@ int _mi_read_cache(IO_CACHE *info, uchar *buff, my_off_t pos, uint length,
       (offset = (my_off_t)(pos - info->pos_in_file)) <
           (my_off_t)(info->read_end - info->request_pos)) {
     in_buff_pos = info->request_pos + (uint)offset;
-    in_buff_length = std::min<size_t>(length, (info->read_end - in_buff_pos));
+    in_buff_length = MY_MIN(length, (size_t)(info->read_end - in_buff_pos));
     memcpy(buff, info->request_pos + (uint)offset, (size_t)in_buff_length);
     if (!(length -= in_buff_length)) return 0;
     pos += in_buff_length;
@@ -84,13 +81,13 @@ int _mi_read_cache(IO_CACHE *info, uchar *buff, my_off_t pos, uint length,
         (info->pos_in_file + (uint)(info->read_end - info->request_pos))) {
       info->pos_in_file = pos; /* Force start here */
       info->read_pos = info->read_end = info->request_pos; /* Everything used */
-      info->seek_not_done = true;
+      info->seek_not_done = 1;
     } else
       info->read_pos = info->read_end; /* All block used */
     if (!(*info->read_function)(info, buff, length)) return 0;
     read_length = info->error;
   } else {
-    info->seek_not_done = true;
+    info->seek_not_done = 1;
     if ((read_length =
              mysql_file_pread(info->file, buff, length, pos, MYF(0))) == length)
       return 0;
