@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -60,7 +60,7 @@ void _mi_print_key(FILE *stream, HA_KEYSEG *keyseg, const uchar *key,
           (void)fprintf(stream, "%d", (uint)*key++);
           break;
         }
-        [[fallthrough]];
+        /* fall through */
       case HA_KEYTYPE_TEXT:
       case HA_KEYTYPE_NUM:
         if (keyseg->flag & HA_SPACE_PACK) {
@@ -117,14 +117,14 @@ void _mi_print_key(FILE *stream, HA_KEYSEG *keyseg, const uchar *key,
         break;
       case HA_KEYTYPE_LONGLONG: {
         char buff[21];
-        llstr(mi_sint8korr(key), buff);
+        longlong2str(mi_sint8korr(key), buff, -10);
         (void)fprintf(stream, "%s", buff);
         key = end;
         break;
       }
       case HA_KEYTYPE_ULONGLONG: {
         char buff[21];
-        ullstr(mi_sint8korr(key), buff);
+        longlong2str(mi_sint8korr(key), buff, 10);
         (void)fprintf(stream, "%s", buff);
         key = end;
         break;
