@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +23,6 @@
 /* Functions to handle keys */
 
 #include <sys/types.h>
-#include <algorithm>
 #include <cmath>
 
 #include "m_ctype.h"
@@ -39,7 +37,7 @@
   do {                                                              \
     if (length > char_length)                                       \
       char_length = my_charpos(cs, pos, pos + length, char_length); \
-    char_length = std::min(char_length, length);                    \
+    set_if_smaller(char_length, length);                            \
   } while (0)
 
 static int _mi_put_key_in_record(MI_INFO *info, uint keynr, bool unpack_blobs,
@@ -124,7 +122,7 @@ uint _mi_make_key(MI_INFO *info, uint keynr, uchar *key, const uchar *record,
       uint pack_length = (keyseg->bit_start == 1 ? 1 : 2);
       uint tmp_length = (pack_length == 1 ? (uint)*pos : uint2korr(pos));
       pos += pack_length; /* Skip VARCHAR length */
-      length = std::min(length, tmp_length);
+      set_if_smaller(length, tmp_length);
       FIX_LENGTH(cs, pos, length, char_length);
       store_key_length_inc(key, char_length);
       memcpy((uchar *)key, pos, (size_t)char_length);
@@ -133,7 +131,7 @@ uint _mi_make_key(MI_INFO *info, uint keynr, uchar *key, const uchar *record,
     } else if (keyseg->flag & HA_BLOB_PART) {
       uint tmp_length = _mi_calc_blob_length(keyseg->bit_start, pos);
       memcpy(&pos, pos + keyseg->bit_start, sizeof(char *));
-      length = std::min(length, tmp_length);
+      set_if_smaller(length, tmp_length);
       FIX_LENGTH(cs, pos, length, char_length);
       store_key_length_inc(key, char_length);
       if (char_length > 0) memcpy((uchar *)key, pos, (size_t)char_length);
@@ -141,7 +139,8 @@ uint _mi_make_key(MI_INFO *info, uint keynr, uchar *key, const uchar *record,
       continue;
     } else if (keyseg->flag & HA_SWAP_KEY) { /* Numerical column */
       if (type == HA_KEYTYPE_FLOAT) {
-        float nr = float4get(pos);
+        float nr;
+        float4get(&nr, pos);
         if (std::isnan(nr)) {
           /* Replace NAN with zero */
           memset(key, 0, length);
@@ -149,7 +148,8 @@ uint _mi_make_key(MI_INFO *info, uint keynr, uchar *key, const uchar *record,
           continue;
         }
       } else if (type == HA_KEYTYPE_DOUBLE) {
-        double nr = float8get(pos);
+        double nr;
+        float8get(&nr, pos);
         if (std::isnan(nr)) {
           memset(key, 0, length);
           key += length;
@@ -206,7 +206,7 @@ uint _mi_pack_key(MI_INFO *info, uint keynr, uchar *key, const uchar *old,
     keypart_map = (((key_part_map)1) << (2 * SPDIMS)) - 1;
 
   /* only key prefixes are supported */
-  assert(((keypart_map + 1) & keypart_map) == 0);
+  DBUG_ASSERT(((keypart_map + 1) & keypart_map) == 0);
 
   for (keyseg = info->s->keyinfo[keynr].seg; keyseg->type && keypart_map;
        old += keyseg->length, keyseg++) {
@@ -245,7 +245,7 @@ uint _mi_pack_key(MI_INFO *info, uint keynr, uchar *key, const uchar *old,
       /* Length of key-part used with mi_rkey() always 2 */
       uint tmp_length = uint2korr(pos);
       pos += 2;
-      length = std::min(length, tmp_length); /* Safety */
+      set_if_smaller(length, tmp_length); /* Safety */
       FIX_LENGTH(cs, pos, length, char_length);
       store_key_length_inc(key, char_length);
       old += 2; /* Skip length */
@@ -474,14 +474,16 @@ ulonglong retrieve_auto_increment(MI_INFO *info, const uchar *record) {
       break;
     case HA_KEYTYPE_FLOAT: /* This shouldn't be used */
     {
-      float f_1 = float4get(key);
+      float f_1;
+      float4get(&f_1, key);
       /* Ignore negative values */
       value = (f_1 < (float)0.0) ? 0 : (ulonglong)f_1;
       break;
     }
     case HA_KEYTYPE_DOUBLE: /* This shouldn't be used */
     {
-      double f_1 = float8get(key);
+      double f_1;
+      float8get(&f_1, key);
       /* Ignore negative values */
       value = (f_1 < 0.0) ? 0 : (ulonglong)f_1;
       break;
@@ -493,13 +495,13 @@ ulonglong retrieve_auto_increment(MI_INFO *info, const uchar *record) {
       value = uint8korr(key);
       break;
     default:
-      assert(0);
+      DBUG_ASSERT(0);
       value = 0; /* Error */
       break;
   }
 
   /*
-    The following code works because if s_value < 0 then value is 0
+    The following code works becasue if s_value < 0 then value is 0
     and if s_value == 0 then value will contain either s_value or the
     correct value.
   */
