@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -35,7 +35,6 @@
 
 #include "my_config.h"
 
-#include <algorithm>
 #include <memory>
 
 #include <errno.h>
@@ -85,7 +84,7 @@ MI_INFO *test_if_reopen(char *filename) {
     if (!strcmp(share->unique_file_name, filename) && share->last_version)
       return info;
   }
-  return nullptr;
+  return 0;
 }
 
 /******************************************************************************
@@ -113,7 +112,7 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
   ST_FILE_ID file_id = {0, 0};
   DBUG_TRACE;
 
-  m_info = nullptr;
+  m_info = NULL;
   kfile = -1;
   lock_error = 1;
   errpos = 0;
@@ -126,7 +125,7 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
     if (realpath_err || (*myisam_test_invalid_symlink)(name_buff) ||
         my_is_symlink(name_buff, &file_id)) {
       set_my_errno(HA_WRONG_CREATE_OPTION);
-      return nullptr;
+      return NULL;
     }
   }
 
@@ -294,6 +293,10 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
     max_key_file_length =
         mi_safe_mul(MI_MIN_KEY_BLOCK_LENGTH,
                     ((ulonglong)1 << (share->base.key_reflength * 8)) - 1);
+#if SIZEOF_OFF_T == 4
+    set_if_smaller(max_data_file_length, INT_MAX32);
+    set_if_smaller(max_key_file_length, INT_MAX32);
+#endif
     share->base.max_data_file_length = (my_off_t)max_data_file_length;
     share->base.max_key_file_length = (my_off_t)max_key_file_length;
 
@@ -333,7 +336,7 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
     my_stpcpy(share->index_file_name, index_name);
     my_stpcpy(share->data_file_name, data_name);
 
-    share->blocksize = std::min(ulong{IO_SIZE}, myisam_block_size);
+    share->blocksize = MY_MIN(IO_SIZE, myisam_block_size);
     {
       HA_KEYSEG *pos = share->keyparts;
       uint32 ftkey_nr = 1;
@@ -343,9 +346,8 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
         disk_pos_assert(disk_pos + share->keyinfo[i].keysegs * HA_KEYSEG_SIZE,
                         end_pos);
         if (share->keyinfo[i].key_alg == HA_KEY_ALG_RTREE)
-          share->have_rtree = true;
-        share->blocksize =
-            std::min(share->blocksize, uint(share->keyinfo[i].block_length));
+          share->have_rtree = 1;
+        set_if_smaller(share->blocksize, share->keyinfo[i].block_length);
         share->keyinfo[i].seg = pos;
         for (j = 0; j < share->keyinfo[i].keysegs; j++, pos++) {
           disk_pos = mi_keyseg_read(disk_pos, pos);
@@ -449,7 +451,7 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
     for (i = j = offset = 0; i < share->base.fields; i++) {
       disk_pos = mi_recinfo_read(disk_pos, &share->rec[i]);
       share->rec[i].pack_type = 0;
-      share->rec[i].huff_tree = nullptr;
+      share->rec[i].huff_tree = 0;
       share->rec[i].offset = offset;
       if (share->rec[i].type == (int)FIELD_BLOB) {
         share->blobs[j].pack_length =
@@ -486,7 +488,7 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
     share->base.margin_key_file_length =
         (share->base.max_key_file_length -
          (keys ? MI_INDEX_BLOCK_MARGIN * share->blocksize * keys : 0));
-    share->blocksize = std::min(ulong{IO_SIZE}, myisam_block_size);
+    share->blocksize = MY_MIN(IO_SIZE, myisam_block_size);
     share->data_file_type = STATIC_RECORD;
     if (share->options & HA_OPTION_COMPRESS_RECORD) {
       share->data_file_type = COMPRESSED_RECORD;
@@ -513,8 +515,8 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
             (HA_OPTION_READ_ONLY_DATA | HA_OPTION_TMP_TABLE |
              HA_OPTION_COMPRESS_RECORD | HA_OPTION_TEMP_COMPRESS_RECORD)) ||
            (open_flags & HA_OPEN_TMP_TABLE) || share->have_rtree)
-              ? false
-              : true;
+              ? 0
+              : 1;
       if (share->concurrent_insert) {
         share->lock.get_status = mi_get_status;
         share->lock.copy_status = mi_copy_status;
@@ -545,7 +547,7 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
     goto err;
   errpos = 6;
 
-  if (!share->have_rtree) info.rtree_recursion_state = nullptr;
+  if (!share->have_rtree) info.rtree_recursion_state = NULL;
 
   my_stpcpy(info.filename, name);
   memcpy(info.blobs, share->blobs, sizeof(MI_BLOB) * share->base.blobs);
@@ -568,11 +570,11 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
   info.last_loop = share->state.update_count;
   if (mode == O_RDONLY) share->options |= HA_OPTION_READ_ONLY_DATA;
   info.lock_type = F_UNLCK;
-  info.quick_mode = false;
-  info.bulk_insert = nullptr;
-  info.ft1_to_ft2 = nullptr;
+  info.quick_mode = 0;
+  info.bulk_insert = 0;
+  info.ft1_to_ft2 = 0;
   info.errkey = -1;
-  info.page_changed = true;
+  info.page_changed = 1;
   mysql_mutex_lock(&share->intern_lock);
   info.read_record = share->read_record;
   share->reopen++;
@@ -584,7 +586,7 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
   }
   if ((open_flags & HA_OPEN_TMP_TABLE) ||
       (share->options & HA_OPTION_TMP_TABLE)) {
-    share->temporary = share->delay_key_write = true;
+    share->temporary = share->delay_key_write = 1;
     share->write_flag = MYF(MY_NABP);
     share->w_locks++; /* We don't have to update status */
     share->tot_locks++;
@@ -593,7 +595,7 @@ MI_INFO *mi_open_share(const char *name, MYISAM_SHARE *old_share, int mode,
   if (((open_flags & HA_OPEN_DELAY_KEY_WRITE) ||
        (share->options & HA_OPTION_DELAY_KEY_WRITE)) &&
       myisam_delay_key_write)
-    share->delay_key_write = true;
+    share->delay_key_write = 1;
   info.state = &share->state.state; /* Change global values by default */
   mysql_mutex_unlock(&share->intern_lock);
 
@@ -629,30 +631,30 @@ err:
   switch (errpos) {
     case 6:
       my_free(m_info);
-      [[fallthrough]];
+      /* fall through */
     case 5:
       (void)mysql_file_close(info.dfile, MYF(0));
       if (old_share) break; /* Don't remove open table */
 
-      [[fallthrough]];
+      /* fall through */
     case 4:
       my_free(share);
-      [[fallthrough]];
+      /* fall through */
     case 3:
       if (!lock_error) (void)my_lock(kfile, F_UNLCK, MYF(MY_SEEK_NOT_DONE));
-      [[fallthrough]];
+      /* fall through */
     case 2:
-      [[fallthrough]];
+      /* fall through */
     case 1:
       (void)mysql_file_close(kfile, MYF(0));
-      [[fallthrough]];
+      /* fall through */
     case 0:
     default:
       break;
   }
   if (!internal_table) mysql_mutex_unlock(&THR_LOCK_myisam);
   set_my_errno(save_errno);
-  return nullptr;
+  return NULL;
 } /* mi_open_share */
 
 uchar *mi_alloc_rec_buff(MI_INFO *info, ulong length, uchar **buf) {
@@ -665,11 +667,10 @@ uchar *mi_alloc_rec_buff(MI_INFO *info, ulong length, uchar **buf) {
     /* to simplify initial init of info->rec_buf in mi_open and mi_extra */
     if (length == (ulong)-1) {
       if (info->s->options & HA_OPTION_COMPRESS_RECORD)
-        length =
-            std::max(info->s->base.pack_reclength, info->s->max_pack_length);
+        length = MY_MAX(info->s->base.pack_reclength, info->s->max_pack_length);
       else
         length = info->s->base.pack_reclength;
-      length = std::max<ulong>(length, info->s->base.max_key_length);
+      length = MY_MAX(length, info->s->base.max_key_length);
       /* Avoid unnecessary realloc */
       if (newptr && length == old_length) return newptr;
     }
@@ -703,7 +704,7 @@ void mi_setup_functions(MYISAM_SHARE *share) {
     share->read_record = _mi_read_pack_record;
     share->read_rnd = _mi_read_rnd_pack_record;
     if (!(share->options & HA_OPTION_TEMP_COMPRESS_RECORD))
-      share->calc_checksum = nullptr; /* No checksum */
+      share->calc_checksum = 0; /* No checksum */
     else if (share->options & HA_OPTION_PACK_RECORD)
       share->calc_checksum = mi_checksum;
     else
@@ -737,7 +738,7 @@ void mi_setup_functions(MYISAM_SHARE *share) {
   }
   share->file_read = mi_nommap_pread;
   share->file_write = mi_nommap_pwrite;
-  if (!(share->options & HA_OPTION_CHECKSUM)) share->calc_checksum = nullptr;
+  if (!(share->options & HA_OPTION_CHECKSUM)) share->calc_checksum = 0;
   return;
 }
 
@@ -761,7 +762,7 @@ static void setup_key_functions(MI_KEYDEF *keyinfo) {
         _mi_prefix_search() compares end-space against ASCII blank (' ').
         It cannot be used for character sets, that do not encode the
         blank character like ASCII does. UCS2 is an example. All
-        character sets with a fixed width > 1 or a minimum width > 1
+        character sets with a fixed width > 1 or a mimimum width > 1
         cannot represent blank like ASCII does. In these cases we have
         to use _mi_seq_search() for the search.
       */
@@ -872,7 +873,7 @@ uint mi_state_info_write(File file, MI_STATE_INFO *state, uint pWrite) {
 
   if (pWrite & 1)
     return mysql_file_pwrite(file, buff, (size_t)(ptr - buff), 0L,
-                             MYF(MY_NABP)) != 0;
+                             MYF(MY_NABP | MY_THREADSAFE)) != 0;
   return mysql_file_write(file, buff, (size_t)(ptr - buff), MYF(MY_NABP)) != 0;
 }
 
@@ -1154,7 +1155,7 @@ uchar *mi_keyseg_read(uchar *ptr, HA_KEYSEG *keyseg) {
   keyseg->null_pos = mi_uint4korr(ptr);
   ptr += 4;
   keyseg->bit_end = 0;
-  keyseg->charset = nullptr; /* Will be filled in later */
+  keyseg->charset = 0;  /* Will be filled in later */
   if (keyseg->null_bit) /* We adjust bit_pos if null_bit is last in the byte */
     keyseg->bit_pos =
         (uint16)(keyseg->null_pos + (keyseg->null_bit == (1 << 7)));
@@ -1227,7 +1228,7 @@ exist a dup()-like call that would give us two different file descriptors.
 *************************************************************************/
 
 int mi_open_datafile(MI_INFO *info, MYISAM_SHARE *share, const char *org_name,
-                     File file_to_dup [[maybe_unused]]) {
+                     File file_to_dup MY_ATTRIBUTE((unused))) {
   char *data_name = share->data_file_name;
   char real_data_name[FN_REFLEN];
   ST_FILE_ID file_id = {0, 0};
