@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -38,19 +38,19 @@
 */
 
 int mi_rsame_with_pos(MI_INFO *info, uchar *record, int inx, my_off_t filepos) {
-  DBUG_ENTER("mi_rsame_with_pos");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("index: %d  filepos: %ld", inx, (long)filepos));
 
   if (inx < -1 ||
       (inx >= 0 && !mi_is_key_active(info->s->state.key_map, inx))) {
     set_my_errno(HA_ERR_WRONG_INDEX);
-    DBUG_RETURN(HA_ERR_WRONG_INDEX);
+    return HA_ERR_WRONG_INDEX;
   }
 
   info->update &= (HA_STATE_CHANGED | HA_STATE_ROW_CHANGED);
   if ((*info->s->read_rnd)(info, record, filepos, 0)) {
     if (my_errno() == HA_ERR_RECORD_DELETED) set_my_errno(HA_ERR_KEY_NOT_FOUND);
-    DBUG_RETURN(my_errno());
+    return my_errno();
   }
   info->lastpos = filepos;
   info->lastinx = inx;
@@ -59,5 +59,5 @@ int mi_rsame_with_pos(MI_INFO *info, uchar *record, int inx, my_off_t filepos) {
         _mi_make_key(info, (uint)inx, info->lastkey, record, info->lastpos);
     info->update |= HA_STATE_KEY_CHANGED; /* Don't use indexposition */
   }
-  DBUG_RETURN(0);
+  return 0;
 } /* mi_rsame_pos */
