@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -60,17 +60,17 @@ int mi_write(MI_INFO *info, uchar *record) {
   my_off_t filepos;
   uchar *buff;
   bool lock_tree = share->concurrent_insert;
-  DBUG_ENTER("mi_write");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("isam: %d  data: %d", info->s->kfile, info->dfile));
 
   DBUG_EXECUTE_IF("myisam_pretend_crashed_table_on_usage",
                   mi_print_error(info->s, HA_ERR_CRASHED);
-                  set_my_errno(HA_ERR_CRASHED); DBUG_RETURN(HA_ERR_CRASHED););
+                  set_my_errno(HA_ERR_CRASHED); return HA_ERR_CRASHED;);
   if (share->options & HA_OPTION_READ_ONLY_DATA) {
     set_my_errno(EACCES);
-    DBUG_RETURN(EACCES);
+    return EACCES;
   }
-  if (_mi_readinfo(info, F_WRLCK, 1)) DBUG_RETURN(my_errno());
+  if (_mi_readinfo(info, F_WRLCK, 1)) return my_errno();
 
   filepos =
       ((share->state.dellink != HA_OFFSET_ERROR && !info->append_insert_at_end)
@@ -154,7 +154,7 @@ int mi_write(MI_INFO *info, uchar *record) {
   */
   if (share->is_log_table) mi_update_status((void *)info);
 
-  DBUG_RETURN(0);
+  return 0;
 
 err:
   save_errno = my_errno();
@@ -198,18 +198,18 @@ err2:
   myisam_log_record(MI_LOG_WRITE, info, record, filepos, my_errno());
   (void)_mi_writeinfo(info, WRITEINFO_UPDATE_KEYFILE);
   set_my_errno(save_errno);
-  DBUG_RETURN(save_errno);
+  return save_errno;
 } /* mi_write */
 
 /* Write one key to btree */
 
 int _mi_ck_write(MI_INFO *info, uint keynr, uchar *key, uint key_length) {
-  DBUG_ENTER("_mi_ck_write");
+  DBUG_TRACE;
 
   if (info->bulk_insert && is_tree_inited(&info->bulk_insert[keynr])) {
-    DBUG_RETURN(_mi_ck_write_tree(info, keynr, key, key_length));
+    return _mi_ck_write_tree(info, keynr, key, key_length);
   } else {
-    DBUG_RETURN(_mi_ck_write_btree(info, keynr, key, key_length));
+    return _mi_ck_write_btree(info, keynr, key, key_length);
   }
 } /* _mi_ck_write */
 
@@ -222,7 +222,7 @@ int _mi_ck_write_btree(MI_INFO *info, uint keynr, uchar *key, uint key_length) {
   uint comp_flag;
   MI_KEYDEF *keyinfo = info->s->keyinfo + keynr;
   my_off_t *root = &info->s->state.key_root[keynr];
-  DBUG_ENTER("_mi_ck_write_btree");
+  DBUG_TRACE;
 
   if (keyinfo->flag & HA_SORT_ALLOWS_SAME)
     comp_flag = SEARCH_BIGGER; /* Put after same key */
@@ -240,19 +240,19 @@ int _mi_ck_write_btree(MI_INFO *info, uint keynr, uchar *key, uint key_length) {
     my_free(info->ft1_to_ft2);
     info->ft1_to_ft2 = 0;
   }
-  DBUG_RETURN(error);
+  return error;
 } /* _mi_ck_write_btree */
 
 int _mi_ck_real_write_btree(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key,
                             uint key_length, my_off_t *root, uint comp_flag) {
   int error;
-  DBUG_ENTER("_mi_ck_real_write_btree");
+  DBUG_TRACE;
   /* key_length parameter is used only if comp_flag is SEARCH_FIND */
   if (*root == HA_OFFSET_ERROR ||
       (error = w_search(info, keyinfo, comp_flag, key, key_length, *root,
                         (uchar *)0, (uchar *)0, (my_off_t)0, 1)) > 0)
     error = _mi_enlarge_root(info, keyinfo, key, root);
-  DBUG_RETURN(error);
+  return error;
 } /* _mi_ck_real_write_btree */
 
 /* Make a new root with key as only pointer */
@@ -262,7 +262,7 @@ int _mi_enlarge_root(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key,
   uint t_length, nod_flag;
   MI_KEY_PARAM s_temp;
   MYISAM_SHARE *share = info->s;
-  DBUG_ENTER("_mi_enlarge_root");
+  DBUG_TRACE;
 
   nod_flag = (*root != HA_OFFSET_ERROR) ? share->base.key_reflength : 0;
   _mi_kpointer(info, info->buff + 2, *root); /* if nod */
@@ -273,8 +273,8 @@ int _mi_enlarge_root(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key,
   info->buff_used = info->page_changed = 1; /* info->buff is used */
   if ((*root = _mi_new(info, keyinfo, DFLT_INIT_HITS)) == HA_OFFSET_ERROR ||
       _mi_write_keypage(info, keyinfo, *root, DFLT_INIT_HITS, info->buff))
-    DBUG_RETURN(-1);
-  DBUG_RETURN(0);
+    return -1;
+  return 0;
 } /* _mi_enlarge_root */
 
 /*
@@ -294,13 +294,13 @@ static int w_search(MI_INFO *info, MI_KEYDEF *keyinfo, uint comp_flag,
   uchar keybuff[MI_MAX_KEY_BUFF];
   bool was_last_key;
   my_off_t next_page, dupp_key_pos;
-  DBUG_ENTER("w_search");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("page: %ld", (long)page));
 
   search_key_length = (comp_flag & SEARCH_FIND) ? key_length : USE_WHOLE_KEY;
   if (!(temp_buff = (uchar *)my_alloca((uint)keyinfo->block_length +
                                        MI_MAX_KEY_BUFF * 2)))
-    DBUG_RETURN(-1);
+    return -1;
   if (!_mi_fetch_keypage(info, keyinfo, page, DFLT_INIT_HITS, temp_buff, 0))
     goto err;
 
@@ -346,16 +346,16 @@ static int w_search(MI_INFO *info, MI_KEYDEF *keyinfo, uint comp_flag,
         if (!error)
           error =
               _mi_write_keypage(info, keyinfo, page, DFLT_INIT_HITS, temp_buff);
-        DBUG_RETURN(error);
+        return error;
       }
     } else /* not HA_FULLTEXT, normal HA_NOSAME key */
     {
       info->dupp_key_pos = dupp_key_pos;
       set_my_errno(HA_ERR_FOUND_DUPP_KEY);
-      DBUG_RETURN(-1);
+      return -1;
     }
   }
-  if (flag == MI_FOUND_WRONG_KEY) DBUG_RETURN(-1);
+  if (flag == MI_FOUND_WRONG_KEY) return -1;
   if (!was_last_key) insert_last = 0;
   next_page = _mi_kpos(nod_flag, keypos);
   if (next_page == HA_OFFSET_ERROR ||
@@ -366,10 +366,10 @@ static int w_search(MI_INFO *info, MI_KEYDEF *keyinfo, uint comp_flag,
     if (_mi_write_keypage(info, keyinfo, page, DFLT_INIT_HITS, temp_buff))
       goto err;
   }
-  DBUG_RETURN(error);
+  return error;
 err:
   DBUG_PRINT("exit", ("Error: %d", my_errno()));
-  DBUG_RETURN(-1);
+  return -1;
 } /* w_search */
 
 /*
@@ -404,7 +404,7 @@ int _mi_insert(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key, uchar *anc_buff,
   int t_length;
   uchar *endpos, *prev_key;
   MI_KEY_PARAM s_temp;
-  DBUG_ENTER("_mi_insert");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("key_pos: %p", key_pos));
   DBUG_EXECUTE("key",
                _mi_print_key(DBUG_FILE, keyinfo->seg, key, USE_WHOLE_KEY););
@@ -432,14 +432,14 @@ int _mi_insert(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key, uchar *anc_buff,
     if (t_length >= keyinfo->maxlength * 2 + MAX_POINTER_LENGTH) {
       mi_print_error(info->s, HA_ERR_CRASHED);
       set_my_errno(HA_ERR_CRASHED);
-      DBUG_RETURN(-1);
+      return -1;
     }
     memmove(key_pos + t_length, key_pos, (size_t)(endpos - key_pos));
   } else {
     if (-t_length >= keyinfo->maxlength * 2 + MAX_POINTER_LENGTH) {
       mi_print_error(info->s, HA_ERR_CRASHED);
       set_my_errno(HA_ERR_CRASHED);
-      DBUG_RETURN(-1);
+      return -1;
     }
     memmove(key_pos, key_pos - t_length, (uint)(endpos - key_pos) + t_length);
   }
@@ -457,12 +457,12 @@ int _mi_insert(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key, uchar *anc_buff,
         Let's consider converting.
         We'll compare 'key' and the first key at anc_buff
        */
-      uchar *a = key, *b = anc_buff + 2 + nod_flag;
-      uint alen, blen, ft2len = info->s->ft2_keyinfo.keylength;
+      const uchar *a = key, *b = anc_buff + 2 + nod_flag;
+      uint blen, ft2len = info->s->ft2_keyinfo.keylength;
       /* the very first key on the page is always unpacked */
       DBUG_ASSERT((*b & 128) == 0);
       blen = *b++;
-      get_key_length(alen, a);
+      uint alen = get_key_length(&a);
       DBUG_ASSERT(info->ft1_to_ft2 == 0);
       if (alen == blen &&
           ha_compare_text(keyinfo->seg->charset, a, alen, b, blen, 0) == 0) {
@@ -486,7 +486,7 @@ int _mi_insert(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key, uchar *anc_buff,
             if (insert_dynamic(info->ft1_to_ft2, b)) {
               mi_print_error(info->s, HA_ERR_OUT_OF_MEM);
               set_my_errno(HA_ERR_OUT_OF_MEM);
-              DBUG_RETURN(-1);
+              return -1;
             }
           }
 
@@ -496,16 +496,15 @@ int _mi_insert(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key, uchar *anc_buff,
         /* the rest will be done when we're back from recursion */
       }
     }
-    DBUG_RETURN(0); /* There is room on page */
+    return 0; /* There is room on page */
   }
   /* Page is full */
   if (nod_flag) insert_last = 0;
   if (!(keyinfo->flag & (HA_VAR_LENGTH_KEY | HA_BINARY_PACK_KEY)) &&
       father_buff && !insert_last)
-    DBUG_RETURN(_mi_balance_page(info, keyinfo, key, anc_buff, father_buff,
-                                 father_key_pos, father_page));
-  DBUG_RETURN(
-      _mi_split_page(info, keyinfo, key, anc_buff, key_buff, insert_last));
+    return _mi_balance_page(info, keyinfo, key, anc_buff, father_buff,
+                            father_key_pos, father_page);
+  return _mi_split_page(info, keyinfo, key, anc_buff, key_buff, insert_last);
 } /* _mi_insert */
 
 /* split a full page in two and assign emerging item to key */
@@ -516,7 +515,7 @@ int _mi_split_page(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key, uchar *buff,
   uchar *key_pos, *pos, *after_key = NULL;
   my_off_t new_pos;
   MI_KEY_PARAM s_temp;
-  DBUG_ENTER("mi_split_page");
+  DBUG_TRACE;
   DBUG_DUMP("buff", (uchar *)buff, mi_getint(buff));
 
   if (info->s->keyinfo + info->lastinx == keyinfo)
@@ -530,7 +529,7 @@ int _mi_split_page(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key, uchar *buff,
   else
     key_pos = _mi_find_half_pos(nod_flag, keyinfo, buff, key_buff, &key_length,
                                 &after_key);
-  if (!key_pos) DBUG_RETURN(-1);
+  if (!key_pos) return -1;
 
   length = (uint)(key_pos - buff);
   a_length = mi_getint(buff);
@@ -545,12 +544,11 @@ int _mi_split_page(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key, uchar *buff,
 
   /* Move middle item to key and pointer to new page */
   if ((new_pos = _mi_new(info, keyinfo, DFLT_INIT_HITS)) == HA_OFFSET_ERROR)
-    DBUG_RETURN(-1);
+    return -1;
   _mi_kpointer(info, _mi_move_key(keyinfo, key, key_buff), new_pos);
 
   /* Store new page */
-  if (!(*keyinfo->get_key)(keyinfo, nod_flag, &key_pos, key_buff))
-    DBUG_RETURN(-1);
+  if (!(*keyinfo->get_key)(keyinfo, nod_flag, &key_pos, key_buff)) return -1;
 
   t_length = (*keyinfo->pack_key)(keyinfo, nod_flag, (uchar *)0, (uchar *)0,
                                   (uchar *)0, key_buff, &s_temp);
@@ -561,9 +559,9 @@ int _mi_split_page(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key, uchar *buff,
   mi_putint(info->buff, length + t_length + key_ref_length, nod_flag);
 
   if (_mi_write_keypage(info, keyinfo, new_pos, DFLT_INIT_HITS, info->buff))
-    DBUG_RETURN(-1);
+    return -1;
   DBUG_DUMP("key", (uchar *)key, _mi_keylength(keyinfo, key));
-  DBUG_RETURN(2); /* Middle key up */
+  return 2; /* Middle key up */
 } /* _mi_split_page */
 
 /*
@@ -579,7 +577,7 @@ uchar *_mi_find_half_pos(uint nod_flag, MI_KEYDEF *keyinfo, uchar *page,
                          uchar **after_key) {
   uint keys, length, key_ref_length;
   uchar *end, *lastpos;
-  DBUG_ENTER("_mi_find_half_pos");
+  DBUG_TRACE;
 
   key_ref_length = 2 + nod_flag;
   length = mi_getint(page) - key_ref_length;
@@ -592,7 +590,7 @@ uchar *_mi_find_half_pos(uint nod_flag, MI_KEYDEF *keyinfo, uchar *page,
     end = page + keys * key_ref_length;
     *after_key = end + key_ref_length;
     memcpy(key, end, key_ref_length);
-    DBUG_RETURN(end);
+    return end;
   }
 
   end = page + length / 2 - key_ref_length; /* This is aprox. half */
@@ -600,12 +598,12 @@ uchar *_mi_find_half_pos(uint nod_flag, MI_KEYDEF *keyinfo, uchar *page,
   do {
     lastpos = page;
     if (!(length = (*keyinfo->get_key)(keyinfo, nod_flag, &page, key)))
-      DBUG_RETURN(0);
+      return 0;
   } while (page < end);
   *return_key_length = length;
   *after_key = page;
   DBUG_PRINT("exit", ("returns: %p  page: %p  half: %p", lastpos, page, end));
-  DBUG_RETURN(lastpos);
+  return lastpos;
 } /* _mi_find_half_pos */
 
 /*
@@ -619,7 +617,7 @@ static uchar *_mi_find_last_pos(MI_KEYDEF *keyinfo, uchar *page, uchar *key,
   uint keys, length, last_length = 0, key_ref_length;
   uchar *end, *lastpos, *prevpos = NULL;
   uchar key_buff[MI_MAX_KEY_BUFF];
-  DBUG_ENTER("_mi_find_last_pos");
+  DBUG_TRACE;
 
   key_ref_length = 2;
   length = mi_getint(page) - key_ref_length;
@@ -631,7 +629,7 @@ static uchar *_mi_find_last_pos(MI_KEYDEF *keyinfo, uchar *page, uchar *key,
     end = page + keys * length;
     *after_key = end + length;
     memcpy(key, end, length);
-    DBUG_RETURN(end);
+    return end;
   }
 
   end = page + length - key_ref_length;
@@ -646,13 +644,13 @@ static uchar *_mi_find_last_pos(MI_KEYDEF *keyinfo, uchar *page, uchar *key,
     if (!(length = (*keyinfo->get_key)(keyinfo, 0, &page, key_buff))) {
       mi_print_error(keyinfo->share, HA_ERR_CRASHED);
       set_my_errno(HA_ERR_CRASHED);
-      DBUG_RETURN(0);
+      return 0;
     }
   }
   *return_key_length = last_length;
   *after_key = lastpos;
   DBUG_PRINT("exit", ("returns: %p  page: %p  end: %p", prevpos, page, end));
-  DBUG_RETURN(prevpos);
+  return prevpos;
 } /* _mi_find_last_pos */
 
 /* Balance page with not packed keys with page on right/left */
@@ -668,7 +666,7 @@ static int _mi_balance_page(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key,
   uchar *pos, *buff, *extra_buff;
   my_off_t next_page, new_pos;
   uchar tmp_part_key[MI_MAX_KEY_BUFF];
-  DBUG_ENTER("_mi_balance_page");
+  DBUG_TRACE;
 
   k_length = keyinfo->keylength;
   father_length = mi_getint(father_buff);
@@ -737,7 +735,7 @@ static int _mi_balance_page(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key,
         _mi_write_keypage(info, keyinfo, father_page, DFLT_INIT_HITS,
                           father_buff))
       goto err;
-    DBUG_RETURN(0);
+    return 0;
   }
 
   /* curr_buff[] and buff[] are full, lets split and make new nod */
@@ -786,10 +784,10 @@ static int _mi_balance_page(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *key,
                         DFLT_INIT_HITS, extra_buff))
     goto err;
 
-  DBUG_RETURN(1); /* Middle key up */
+  return 1; /* Middle key up */
 
 err:
-  DBUG_RETURN(-1);
+  return -1;
 } /* _mi_balance_page */
 
 /**********************************************************************
@@ -803,7 +801,7 @@ typedef struct {
 
 int _mi_ck_write_tree(MI_INFO *info, uint keynr, uchar *key, uint key_length) {
   int error;
-  DBUG_ENTER("_mi_ck_write_tree");
+  DBUG_TRACE;
 
   error = tree_insert(&info->bulk_insert[keynr], key,
                       key_length + info->s->rec_reflength,
@@ -811,16 +809,16 @@ int _mi_ck_write_tree(MI_INFO *info, uint keynr, uchar *key, uint key_length) {
               ? 0
               : HA_ERR_OUT_OF_MEM;
 
-  DBUG_RETURN(error);
+  return error;
 } /* _mi_ck_write_tree */
 
 /* typeof(_mi_keys_compare)=qsort2_cmp */
 
 static int keys_compare(const void *a, const void *b, const void *c) {
   uint not_used[2];
-  bulk_insert_param *param = (bulk_insert_param *)a;
-  uchar *key1 = (uchar *)b;
-  uchar *key2 = (uchar *)c;
+  const bulk_insert_param *param = static_cast<const bulk_insert_param *>(a);
+  const uchar *key1 = static_cast<const uchar *>(b);
+  const uchar *key2 = static_cast<const uchar *>(c);
   return ha_key_cmp(param->info->s->keyinfo[param->keynr].seg, key1, key2,
                     USE_WHOLE_KEY, SEARCH_SAME, not_used);
 }
@@ -865,7 +863,7 @@ int mi_init_bulk_insert(MI_INFO *info, ulong cache_size, ha_rows rows) {
   bulk_insert_param *params;
   uint i, num_keys, total_keylength;
   ulonglong key_map;
-  DBUG_ENTER("_mi_init_bulk_insert");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("cache_size: %lu", cache_size));
 
   DBUG_ASSERT(!info->bulk_insert &&
@@ -882,7 +880,7 @@ int mi_init_bulk_insert(MI_INFO *info, ulong cache_size, ha_rows rows) {
   }
 
   if (num_keys == 0 || num_keys * MI_MIN_SIZE_BULK_INSERT_TREE > cache_size)
-    DBUG_RETURN(0);
+    return 0;
 
   if (rows && rows * total_keylength < cache_size)
     cache_size = (ulong)rows;
@@ -894,7 +892,7 @@ int mi_init_bulk_insert(MI_INFO *info, ulong cache_size, ha_rows rows) {
       (sizeof(TREE) * share->base.keys + sizeof(bulk_insert_param) * num_keys),
       MYF(0));
 
-  if (!info->bulk_insert) DBUG_RETURN(HA_ERR_OUT_OF_MEM);
+  if (!info->bulk_insert) return HA_ERR_OUT_OF_MEM;
 
   params = (bulk_insert_param *)(info->bulk_insert + share->base.keys);
   for (i = 0; i < share->base.keys; i++) {
@@ -909,7 +907,7 @@ int mi_init_bulk_insert(MI_INFO *info, ulong cache_size, ha_rows rows) {
       info->bulk_insert[i].root = 0;
   }
 
-  DBUG_RETURN(0);
+  return 0;
 }
 
 void mi_flush_bulk_insert(MI_INFO *info, uint inx) {
