@@ -804,7 +804,7 @@ static int myisamchk(MI_CHECK *param, char *filename) {
   char llbuff[22], llbuff2[22];
   bool state_updated = 0;
   MYISAM_SHARE *share;
-  DBUG_ENTER("myisamchk");
+  DBUG_TRACE;
 
   param->out_flag = error = param->warning_printed = param->error_printed =
       recreate = 0;
@@ -862,7 +862,7 @@ static int myisamchk(MI_CHECK *param, char *filename) {
                              my_errno(), filename);
         break;
     }
-    DBUG_RETURN(1);
+    return 1;
   }
   share = info->s;
   share->options &= ~HA_OPTION_READ_ONLY_DATA; /* We are modifing it */
@@ -904,9 +904,9 @@ static int myisamchk(MI_CHECK *param, char *filename) {
       if (mi_close(info)) {
         mi_check_print_error(param, "%d when closing MyISAM-table '%s'",
                              my_errno(), filename);
-        DBUG_RETURN(1);
+        return 1;
       }
-      DBUG_RETURN(0);
+      return 0;
     }
   }
   if ((param->testflag &
@@ -929,7 +929,7 @@ static int myisamchk(MI_CHECK *param, char *filename) {
       (void)fprintf(stderr,
                     "MyISAM-table '%s' is not fixed because of errors\n",
                     filename);
-      DBUG_RETURN(-1);
+      return -1;
     }
     recreate = 1;
     if (!(param->testflag & T_REP_ANY)) {
@@ -1126,7 +1126,7 @@ end2:
   if (mi_close(info)) {
     mi_check_print_error(param, "%d when closing MyISAM-table '%s'", my_errno(),
                          filename);
-    DBUG_RETURN(1);
+    return 1;
   }
   if (error == 0) {
     if (param->out_flag & O_NEW_DATA)
@@ -1161,7 +1161,7 @@ end2:
     (void)fprintf(stderr, "MyISAM-table '%s' is usable but should be fixed\n",
                   filename);
   (void)fflush(stderr);
-  DBUG_RETURN(error);
+  return error;
 } /* myisamchk */
 
 /* Write info about table */
@@ -1175,7 +1175,7 @@ static void descript(MI_CHECK *param, MI_INFO *info, char *name) {
   enum en_fieldtype type;
   MYISAM_SHARE *share = info->s;
   char llbuff[22], llbuff2[22];
-  DBUG_ENTER("describe");
+  DBUG_TRACE;
 
   printf("\nMyISAM file:         %s\n", name);
   fputs("Record format:       ", stdout);
@@ -1230,7 +1230,7 @@ static void descript(MI_CHECK *param, MI_INFO *info, char *name) {
   }
   printf("Data records:        %13s  Deleted blocks:     %13s\n",
          llstr(info->state->records, llbuff), llstr(info->state->del, llbuff2));
-  if (param->testflag & T_SILENT) DBUG_VOID_RETURN; /* This is enough */
+  if (param->testflag & T_SILENT) return; /* This is enough */
 
   if (param->testflag & T_VERBOSE) {
     printf("Datafile parts:      %13s  Deleted data:       %13s\n",
@@ -1379,7 +1379,6 @@ static void descript(MI_CHECK *param, MI_INFO *info, char *name) {
       start += share->rec[field].length;
     }
   }
-  DBUG_VOID_RETURN;
 } /* describe */
 
 /* Sort records according to one key */
@@ -1396,7 +1395,7 @@ static int mi_sort_records(MI_CHECK *param, MI_INFO *info, char *name,
   char llbuff[22], llbuff2[22];
   SORT_INFO sort_info;
   MI_SORT_PARAM sort_param;
-  DBUG_ENTER("sort_records");
+  DBUG_TRACE;
 
   memset(&sort_info, 0, sizeof(sort_info));
   memset(&sort_param, 0, sizeof(sort_param));
@@ -1412,18 +1411,18 @@ static int mi_sort_records(MI_CHECK *param, MI_INFO *info, char *name,
                            "Can't sort table '%s' on key %d;  No such key",
                            name, sort_key + 1);
     param->error_printed = 0;
-    DBUG_RETURN(0); /* Nothing to do */
+    return 0; /* Nothing to do */
   }
   if (keyinfo->flag & HA_FULLTEXT) {
     mi_check_print_warning(param, "Can't sort table '%s' on FULLTEXT key %d",
                            name, sort_key + 1);
     param->error_printed = 0;
-    DBUG_RETURN(0); /* Nothing to do */
+    return 0; /* Nothing to do */
   }
   if (share->data_file_type == COMPRESSED_RECORD) {
     mi_check_print_warning(param, "Can't sort read-only table '%s'", name);
     param->error_printed = 0;
-    DBUG_RETURN(0); /* Nothing to do */
+    return 0; /* Nothing to do */
   }
   if (!(param->testflag & T_SILENT)) {
     printf("- Sorting records for MyISAM-table '%s'\n", name);
@@ -1433,7 +1432,7 @@ static int mi_sort_records(MI_CHECK *param, MI_INFO *info, char *name,
              llstr(info->state->del, llbuff2));
   }
   if (share->state.key_root[sort_key] == HA_OFFSET_ERROR)
-    DBUG_RETURN(0); /* Nothing to do */
+    return 0; /* Nothing to do */
 
   init_key_cache(dflt_key_cache, opt_key_cache_block_size,
                  (size_t)param->use_buffers, 0, 0);
@@ -1531,7 +1530,7 @@ err:
   my_free(sort_info.buff);
   sort_info.buff = 0;
   share->state.sortkey = sort_key;
-  DBUG_RETURN(flush_blocks(param, share->key_cache, share->kfile) | got_error);
+  return flush_blocks(param, share->key_cache, share->kfile) | got_error;
 } /* sort_records */
 
 /* Sort records recursive using one index */
@@ -1546,7 +1545,7 @@ static int sort_record_index(MI_SORT_PARAM *sort_param, MI_INFO *info,
   char llbuff[22];
   SORT_INFO *sort_info = sort_param->sort_info;
   MI_CHECK *param = sort_info->param;
-  DBUG_ENTER("sort_record_index");
+  DBUG_TRACE;
 
   nod_flag = mi_test_if_nod(buff);
   temp_buff = 0;
@@ -1554,7 +1553,7 @@ static int sort_record_index(MI_SORT_PARAM *sort_param, MI_INFO *info,
   if (nod_flag) {
     if (!(temp_buff = (uchar *)my_alloca((uint)keyinfo->block_length))) {
       mi_check_print_error(param, "Not Enough memory");
-      DBUG_RETURN(-1);
+      return -1;
     }
   }
   used_length = mi_getint(buff);
@@ -1602,9 +1601,9 @@ static int sort_record_index(MI_SORT_PARAM *sort_param, MI_INFO *info,
     mi_check_print_error(param, "%d when updating keyblock", my_errno());
     goto err;
   }
-  DBUG_RETURN(0);
+  return 0;
 err:
-  DBUG_RETURN(1);
+  return 1;
 } /* sort_record_index */
 
 /*
@@ -1636,7 +1635,7 @@ void mi_check_print_info(MI_CHECK *param MY_ATTRIBUTE((unused)),
 
 void mi_check_print_warning(MI_CHECK *param, const char *fmt, ...) {
   va_list args;
-  DBUG_ENTER("mi_check_print_warning");
+  DBUG_TRACE;
 
   fflush(stdout);
   if (!param->warning_printed && !param->error_printed) {
@@ -1652,14 +1651,13 @@ void mi_check_print_warning(MI_CHECK *param, const char *fmt, ...) {
   (void)fputc('\n', stderr);
   fflush(stderr);
   va_end(args);
-  DBUG_VOID_RETURN;
 }
 
 /* VARARGS */
 
 void mi_check_print_error(MI_CHECK *param, const char *fmt, ...) {
   va_list args;
-  DBUG_ENTER("mi_check_print_error");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("format: %s", fmt));
 
   fflush(stdout);
@@ -1676,7 +1674,6 @@ void mi_check_print_error(MI_CHECK *param, const char *fmt, ...) {
   (void)fputc('\n', stderr);
   fflush(stderr);
   va_end(args);
-  DBUG_VOID_RETURN;
 }
 
 #include "storage/myisam/mi_extrafunc.h"
