@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -53,10 +52,10 @@ MYRG_INFO *myrg_open(const char *name, int mode, int handle_locking) {
   uint files = 0, i, dir_length, length, key_parts = 0, min_keys = 0;
   ulonglong file_offset = 0;
   char name_buff[FN_REFLEN * 2], buff[FN_REFLEN], *end;
-  MYRG_INFO *m_info = nullptr;
+  MYRG_INFO *m_info = 0;
   File fd;
   IO_CACHE file;
-  MI_INFO *isam = nullptr;
+  MI_INFO *isam = 0;
   uint found_merge_insert_method = 0;
   size_t name_buff_length;
   bool bad_children = false;
@@ -69,7 +68,7 @@ MYRG_INFO *myrg_open(const char *name, int mode, int handle_locking) {
                             O_RDONLY, MYF(0))) < 0)
     goto err;
   errpos = 1;
-  if (init_io_cache(&file, fd, 4 * IO_SIZE, READ_CACHE, 0, false,
+  if (init_io_cache(&file, fd, 4 * IO_SIZE, READ_CACHE, 0, 0,
                     MYF(MY_WME | MY_NABP)))
     goto err;
   errpos = 2;
@@ -115,7 +114,7 @@ MYRG_INFO *myrg_open(const char *name, int mode, int handle_locking) {
                                                 key_parts * sizeof(long),
                                             MYF(MY_WME | MY_ZEROFILL))))
         goto err;
-      assert(files);
+      DBUG_ASSERT(files);
       m_info->open_tables = (MYRG_TABLE *)(m_info + 1);
       m_info->rec_per_key_part = (ulong *)(m_info->open_tables + files);
       m_info->tables = files;
@@ -185,15 +184,15 @@ err:
     case 3:
       while (files) (void)mi_close(m_info->open_tables[--files].table);
       my_free(m_info);
-      [[fallthrough]];
+      /* Fall through */
     case 2:
       end_io_cache(&file);
-      [[fallthrough]];
+      /* Fall through */
     case 1:
       (void)mysql_file_close(fd, MYF(0));
   }
   set_my_errno(save_errno);
-  return nullptr;
+  return NULL;
 }
 
 /**
@@ -219,7 +218,7 @@ err:
 MYRG_INFO *myrg_parent_open(const char *parent_name,
                             int (*callback)(void *, const char *),
                             void *callback_param) {
-  MYRG_INFO *m_info = nullptr;
+  MYRG_INFO *m_info = NULL;
   int rc;
   int errpos;
   int save_errno;
@@ -245,7 +244,7 @@ MYRG_INFO *myrg_parent_open(const char *parent_name,
     goto err; /* purecov: inspected */
   errpos = 1;
 
-  if (init_io_cache(&file_cache, fd, 4 * IO_SIZE, READ_CACHE, 0, false,
+  if (init_io_cache(&file_cache, fd, 4 * IO_SIZE, READ_CACHE, 0, 0,
                     MYF(MY_WME | MY_NABP)))
     goto err; /* purecov: inspected */
   errpos = 2;
@@ -325,15 +324,15 @@ err:
   switch (errpos) {
     case 3:
       my_free(m_info);
-      [[fallthrough]];
+      /* Fall through */
     case 2:
       end_io_cache(&file_cache);
-      [[fallthrough]];
+      /* Fall through */
     case 1:
       (void)mysql_file_close(fd, MYF(0));
   }
   set_my_errno(save_errno);
-  return nullptr;
+  return NULL;
   /* purecov: end */
 }
 
@@ -410,7 +409,7 @@ int myrg_attach_children(MYRG_INFO *m_info, int handle_locking,
       key_parts = myisam->s->base.key_parts;
       if (*need_compat_check && m_info->rec_per_key_part) {
         my_free(m_info->rec_per_key_part);
-        m_info->rec_per_key_part = nullptr;
+        m_info->rec_per_key_part = NULL;
       }
       if (!m_info->rec_per_key_part) {
         if (!(m_info->rec_per_key_part =
@@ -472,7 +471,7 @@ err:
   switch (errpos) {
     case 1:
       my_free(m_info->rec_per_key_part);
-      m_info->rec_per_key_part = nullptr;
+      m_info->rec_per_key_part = NULL;
   }
   mysql_mutex_unlock(&m_info->mutex);
   set_my_errno(save_errno);
@@ -485,7 +484,7 @@ err:
   @param[in]    m_info          MERGE parent table structure
 
   @note Detach must not touch the children in any way.
-    They may have been closed at this point already.
+    They may have been closed at ths point already.
     All references to the children should be removed.
 
   @return status
