@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,7 +50,7 @@
 
 int myrg_rkey(MYRG_INFO *info, uchar *buf, int inx, const uchar *key,
               key_part_map keypart_map, enum ha_rkey_function search_flag) {
-  uchar *key_buff = nullptr;
+  uchar *key_buff = NULL;
   uint pack_key_length = 0;
   uint16 last_used_keyseg = 0;
   MYRG_TABLE *table;
@@ -65,7 +64,7 @@ int myrg_rkey(MYRG_INFO *info, uchar *buf, int inx, const uchar *key,
     mi = table->table;
 
     if (table == info->open_tables) {
-      err = mi_rkey(mi, nullptr, inx, key, keypart_map, search_flag);
+      err = mi_rkey(mi, 0, inx, key, keypart_map, search_flag);
       /* Get the saved packed key and packed key length. */
       key_buff = (uchar *)mi->lastkey + mi->s->base.max_key_length;
       pack_key_length = mi->pack_key_length;
@@ -73,7 +72,7 @@ int myrg_rkey(MYRG_INFO *info, uchar *buf, int inx, const uchar *key,
     } else {
       mi->once_flags |= USE_PACKED_KEYS;
       mi->last_used_keyseg = last_used_keyseg;
-      err = mi_rkey(mi, nullptr, inx, key_buff, pack_key_length, search_flag);
+      err = mi_rkey(mi, 0, inx, key_buff, pack_key_length, search_flag);
     }
     info->last_used_table = table + 1;
 
