@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,7 +37,7 @@ int myrg_rprev(MYRG_INFO *info, uchar *buf, int inx) {
   if (!info->current_table) return (HA_ERR_KEY_NOT_FOUND);
 
   /* at first, do rprev for the table found before */
-  if ((err = mi_rprev(info->current_table->table, nullptr, inx))) {
+  if ((err = mi_rprev(info->current_table->table, NULL, inx))) {
     if (err == HA_ERR_END_OF_FILE) {
       queue_remove(&(info->by_key), 0);
       if (!info->by_key.elements) return HA_ERR_END_OF_FILE;
