@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -67,7 +66,7 @@ int myrg_rrnd(MYRG_INFO *info, uchar *buf, ulonglong filepos) {
     for (;;) {
       isam_info->update &= HA_STATE_CHANGED;
       if ((error = (*isam_info->s->read_rnd)(isam_info, (uchar *)buf,
-                                             (my_off_t)filepos, true)) !=
+                                             (my_off_t)filepos, 1)) !=
           HA_ERR_END_OF_FILE)
         return error;
       if (info->current_table + 1 == info->end_table) return HA_ERR_END_OF_FILE;
@@ -88,7 +87,7 @@ int myrg_rrnd(MYRG_INFO *info, uchar *buf, ulonglong filepos) {
   isam_info->update &= HA_STATE_CHANGED;
   return (*isam_info->s->read_rnd)(
       isam_info, (uchar *)buf,
-      (my_off_t)(filepos - info->current_table->file_offset), false);
+      (my_off_t)(filepos - info->current_table->file_offset), 0);
 }
 
 /* Find which table to use according to file-pos */
