@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,22 +33,48 @@
 
 
 /**
- * No of 32 bits words needed to store a node bitmask
- *   containing all the nodes in the system
- *   Both NDB nodes and API, MGM... nodes
+ * No. of 32 bits words needed to store a node bitmask
+ * containing all the nodes in the system
+ *
+ * - Data nodes, API and MGMD nodes
  *
  * Note that this is used in a lot of signals
  */
 #define _NODE_BITMASK_SIZE 8
 
 /**
- * No of 32 bits words needed to store a node bitmask
- *   containing all the ndb nodes in the system
+ * No. of 32 bits words needed to store a node bitmask
+ * containing all the nodes in the system prior to 8.0.15
+ *
+ * The maximum number of total nodes is 255 prior to 8.0.15
+ *
+ * - Data nodes, API and MGMD nodes
+ * Note that this is used in a lot of signals
+ */
+#define _NODE_BITMASK_SIZE_255_NODES 8
+
+/**
+ * No. of 32 bits words needed to store a node bitmask
+ * containing all the data nodes in the system
  *
  * Note that this is used in a lot of signals
  */
-#define _NDB_NODE_BITMASK_SIZE 2
+#define _NDB_NODE_BITMASK_SIZE 5
 
+/**
+ * No of 32 bits words needed to store a node bitmask
+ * containing all the data nodes in the system prior to 8.0.15
+ *
+ * The maximum number of data nodes is 48 prior to 8.0.15
+ *
+ * Note that this is used in a lot of signals_NDB_NODE_BITMASK_SIZE
+ */
+#define _NDB_NODE_BITMASK_SIZE_48_NODES 2
+
+#define _NDB_NBM_DIFF_BYTES (_NDB_NODE_BITMASK_SIZE - _NDB_NODE_BITMASK_SIZE_48_NODES)\
+  * sizeof(Uint32)
+#define _NBM_DIFF_BYTES (_NODE_BITMASK_SIZE - _NODE_BITMASK_SIZE_255_NODES)\
+  * sizeof(Uint32)
 /**
  * No of 32 bits word needed to store B bits for N nodes
  */
@@ -60,6 +86,10 @@ typedef BitmaskPOD<(unsigned int)_NODE_BITMASK_SIZE> NodeBitmaskPOD;
 typedef Bitmask<(unsigned int)_NDB_NODE_BITMASK_SIZE> NdbNodeBitmask;
 typedef BitmaskPOD<(unsigned int)_NDB_NODE_BITMASK_SIZE> NdbNodeBitmaskPOD;
 
+typedef Bitmask<(unsigned int)_NODE_BITMASK_SIZE_255_NODES> NodeBitmask255;
+typedef Bitmask<(unsigned int)_NDB_NODE_BITMASK_SIZE_48_NODES> NdbNodeBitmask48;
+
+
 #define __NBM_SZ  ((MAX_NODES >> 5) + ((MAX_NODES & 31) != 0))
 #define __NNBM_SZ ((MAX_NDB_NODES >> 5) + ((MAX_NDB_NODES & 31) != 0))
 
