@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -51,10 +58,16 @@ struct AlterTableReq {
   a = Add attribute
   f = Add fragment(s)
   r = Reorg fragment(s)
-
+  c = Reorg commit flag
+  C = Reorg complete
+  u = Reorg Suma enable flag
+  U = Reorg Suma filter flag
+  F = Fragment count type flag
+  R = Changed Read Backup flag
+  m = Modified attribute
            1111111111222222222233
  01234567890123456789012345678901
- nfdrtsafr-----------------------
+ nfdrtsafrcCuUFRm----------------
 */
 #define NAME_SHIFT        (0)
 #define FRM_SHIFT         (1)
@@ -69,10 +82,25 @@ struct AlterTableReq {
 #define REORG_COMPLETE_SHIFT (10)
 #define REORG_SUMA_ENABLE (11)
 #define REORG_SUMA_FILTER (12)
+#define PARTITION_BALANCE_SHIFT (13)
+#define READ_BACKUP_SHIFT (14)
+#define MODIFY_ATTR_SHIFT (15)
 
  /**
    * Getters and setters
    */ 
+
+  /**
+   * These are that flags that can be set from the NDB API
+   * as part of an online alter table (inplace).
+   * We can change the name of a table,
+   * we can change the frm file of a table,
+   * we can change the read backup flag of a table,
+   * we can change the name of an attribute of a table and
+   * we can add attributes to a table and
+   * we can change the partition balance of a table,
+   * we can add fragments to the table.
+   */
   static Uint8 getNameFlag(const UintR & changeMask);
   static void setNameFlag(UintR &  changeMask, Uint32 nameFlg);
   static Uint8 getFrmFlag(const UintR & changeMask);
@@ -81,16 +109,37 @@ struct AlterTableReq {
   static void setFragDataFlag(UintR &  changeMask, Uint32 fragFlg);
   static Uint8 getRangeListFlag(const UintR & changeMask);
   static void setRangeListFlag(UintR &  changeMask, Uint32 rangeFlg);
+  static Uint8 getAddAttrFlag(const UintR & changeMask);
+  static void setAddAttrFlag(UintR &  changeMask, Uint32 tsFlg);
+  static Uint8 getModifyAttrFlag(const UintR & changeMask);
+  static void setModifyAttrFlag(UintR &  changeMask, Uint32 tsFlg);
+  static Uint8 getAddFragFlag(const UintR & changeMask);
+  static void setAddFragFlag(UintR &  changeMask, Uint32 tsFlg);
+  static void setReadBackupFlag(UintR & changeMask, Uint32 tsFlg);
+  static Uint8 getReadBackupFlag(const UintR & changeMask);
+
+  /**
+   * These flags are never used.
+   */
   static Uint8 getTsNameFlag(const UintR & changeMask);
   static void setTsNameFlag(UintR &  changeMask, Uint32 tsNameFlg);
   static Uint8 getTsFlag(const UintR & changeMask);
   static void setTsFlag(UintR &  changeMask, Uint32 tsFlg);
-  static Uint8 getAddAttrFlag(const UintR & changeMask);
-  static void setAddAttrFlag(UintR &  changeMask, Uint32 tsFlg);
-  static Uint8 getAddFragFlag(const UintR & changeMask);
-  static void setAddFragFlag(UintR &  changeMask, Uint32 tsFlg);
+
+  /**
+   * The getReorgFragFlag is set by DICT when the hashmap changes
+   * as part of reorganise of partitions. It should not be set
+   * by the NDB API, it is set by DICT.
+   */
   static Uint8 getReorgFragFlag(const UintR & changeMask);
   static void setReorgFragFlag(UintR &  changeMask, Uint32 tsFlg);
+
+  /**
+   * The flags below are al defined as part of DICT subops. This means
+   * that they should not be set by the NDB API. They are set in the
+   * subops handling in DICT as part of executing the ALTER_TABLE_REQ
+   * signal from the NDB API.
+   */
   static Uint8 getReorgCommitFlag(const UintR & changeMask);
   static void setReorgCommitFlag(UintR &  changeMask, Uint32 tsFlg);
   static Uint8 getReorgCompleteFlag(const UintR & changeMask);
@@ -99,8 +148,20 @@ struct AlterTableReq {
   static void setReorgSumaEnableFlag(UintR &  changeMask, Uint32 tsFlg);
   static Uint8 getReorgSumaFilterFlag(const UintR & changeMask);
   static void setReorgSumaFilterFlag(UintR &  changeMask, Uint32 tsFlg);
+  static void setPartitionBalanceFlag(UintR & changeMask, Uint32 tsFlg);
+  static Uint8 getPartitionBalanceFlag(const UintR & changeMask);
 
-  static bool getReorgSubOp(const UintR & changeMask){
+  static bool getSubOp(const UintR & changeMask)
+  {
+    return
+      getReorgCommitFlag(changeMask) ||
+      getReorgCompleteFlag(changeMask) ||
+      getReorgSumaEnableFlag(changeMask) ||
+      getReorgSumaFilterFlag(changeMask);
+  }
+
+  static bool getReorgSubOp(const UintR & changeMask)
+  {
     return
       getReorgCommitFlag(changeMask) ||
       getReorgCompleteFlag(changeMask) ||
@@ -193,6 +254,18 @@ AlterTableReq::setAddAttrFlag(UintR & changeMask, Uint32 addAttrFlg){
   changeMask |= (addAttrFlg << ADD_ATTR_SHIFT);
 }
 
+inline
+Uint8
+AlterTableReq::getModifyAttrFlag(const UintR & changeMask){
+  return (Uint8)((changeMask >> MODIFY_ATTR_SHIFT) & 1);
+}
+
+inline
+void
+AlterTableReq::setModifyAttrFlag(UintR & changeMask, Uint32 addAttrFlg){
+  changeMask |= (addAttrFlg << MODIFY_ATTR_SHIFT);
+}
+
 inline
 Uint8
 AlterTableReq::getAddFragFlag(const UintR & changeMask){
@@ -266,6 +339,30 @@ AlterTableReq::setReorgSumaFilterFlag(UintR & changeMask, Uint32 reorgAttrFlg){
   changeMask |= (reorgAttrFlg << REORG_SUMA_FILTER);
 }
 
+inline
+Uint8
+AlterTableReq::getPartitionBalanceFlag(const UintR & changeMask){
+  return (Uint8)((changeMask >> PARTITION_BALANCE_SHIFT) & 1);
+}
+
+inline
+void
+AlterTableReq::setPartitionBalanceFlag(UintR & changeMask, Uint32 fctFlag){
+  changeMask |= (fctFlag << PARTITION_BALANCE_SHIFT);
+}
+
+inline
+Uint8
+AlterTableReq::getReadBackupFlag(const UintR & changeMask){
+  return (Uint8)((changeMask >> READ_BACKUP_SHIFT) & 1);
+}
+
+inline
+void
+AlterTableReq::setReadBackupFlag(UintR & changeMask, Uint32 rbFlag){
+  changeMask |= (rbFlag << READ_BACKUP_SHIFT);
+}
+
 struct AlterTableConf {
   STATIC_CONST( SignalLength = 6 );
 
