@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -31,6 +31,13 @@
 #define JAM_FILE_ID 78
 
 
+/*
+ * DefineBackupReq
+ *
+ * Global signal, but only between data nodes of same version since mixed
+ * version backup is not allowed.
+ * No logic for mixed versions is needed.
+ */
 class DefineBackupReq {
   /**
    * Sender(s)
@@ -46,7 +53,7 @@ class DefineBackupReq {
 
   friend bool printDEFINE_BACKUP_REQ(FILE *, const Uint32 *, Uint32, Uint16);
 public:
-  STATIC_CONST( SignalLength = 11 + NdbNodeBitmask::Size);
+  STATIC_CONST( SignalLength_v1 = 11 + NdbNodeBitmask48::Size);
 
 private:
   /**
@@ -58,11 +65,12 @@ private:
   Uint32 clientRef;
   Uint32 clientData;
   Uint32 senderRef;
-  
+
   /**
    * Which node(s) is participating in the backup
+   * Note: Only to support versions < 8.0.18
    */
-  NdbNodeBitmask nodes;
+  NdbNodeBitmask48 nodes;
   
   /**
    * Generated random number
