@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004, 2017, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -83,7 +83,8 @@ class CntrStartConf {
   friend bool printCNTR_START_CONF(FILE*, const Uint32 *, Uint32, Uint16);
 
 public:
-  STATIC_CONST( SignalLength = 4 + 2 * NdbNodeBitmask::Size );
+  STATIC_CONST( SignalLength = 4);
+  STATIC_CONST( SignalLength_v1 = 4 + 2 * 2 );
   
 private:
   
@@ -91,14 +92,18 @@ private:
   Uint32 startGci;
   Uint32 masterNodeId;
   Uint32 noStartNodes;
-  Uint32 startedNodes[NdbNodeBitmask::Size];
-  Uint32 startingNodes[NdbNodeBitmask::Size];
+  Uint32 startedNodes_v1[NdbNodeBitmask48::Size];
+  Uint32 startingNodes_v1[NdbNodeBitmask48::Size];
 };
 
 struct CntrWaitRep
 {
   Uint32 nodeId;
   Uint32 waitPoint;
+
+  // Below words only used for Grant not for WaitFor.
+  // WaitFor ZWAITPOINT_4_2 also pass node bitmask in section.
+  // For old versions ZWAITPOINT_4_2 pass a two word bitmask in signal here.
   Uint32 request;
   Uint32 sp;
 
