@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -33,6 +40,8 @@ struct DiGetNodesConf {
    */
   friend class Dbtc;
   friend class Dbspj;
+  friend class Backup;
+  friend class Suma;
 
   /**
    * Sender(s)
@@ -56,17 +65,22 @@ class DiGetNodesReq {
    */
   friend class Dbtc;
   friend class Dbspj;
+  friend class Backup;
+  friend class Suma;
   /**
    * Receiver(s)
    */
   friend class Dbdih;
 public:
-  STATIC_CONST( SignalLength = 4 + (sizeof(void*) / sizeof(Uint32)) );
+  STATIC_CONST( SignalLength = 6 + (sizeof(void*) / sizeof(Uint32)) );
+  STATIC_CONST(MAX_DIGETNODESREQS = 16);
 private:
   Uint32 tableId;
   Uint32 hashValue;
   Uint32 distr_key_indicator;
-  Uint32 unused;
+  Uint32 scan_indicator;
+  Uint32 get_next_fragid_indicator;
+  Uint32 anyNode;
   union {
     void * jamBufferPtr;
     Uint32 jamBufferStorage[2];
