@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -61,6 +61,8 @@ class FsOpenReq {
 
   friend class Dbtup;
 
+  friend class Cmvmi;
+
   /**
    * For printing
    */
@@ -105,6 +107,8 @@ private:
   STATIC_CONST( OM_GZ             = 0x8000 );
   STATIC_CONST( OM_THREAD_POOL    = 0x10000 );
   STATIC_CONST( OM_WRITE_BUFFER   = 0x20000 );
+  STATIC_CONST( OM_READ_SIZE      = 0x40000 );
+  STATIC_CONST( OM_DIRECT_SYNC    = 0x80000 );
   
   enum Suffixes {
     S_DATA = 0,
@@ -153,11 +157,15 @@ private:
    */
   static Uint32 v2_getSequence(const Uint32 fileNumber[]);
   static Uint32 v2_getNodeId(const Uint32 fileNumber[]);
+  static Uint32 v2_getPartNum(const Uint32 fileNumber[]);
   static Uint32 v2_getCount(const Uint32 fileNumber[]);
+  static Uint32 v2_getTotalParts(const Uint32 fileNumber[]);
 
   static void v2_setSequence(Uint32 fileNumber[], Uint32 no);
   static void v2_setNodeId(Uint32 fileNumber[], Uint32 no);
+  static void v2_setPartNum(Uint32 fileNumber[], Uint32 no);
   static void v2_setCount(Uint32 fileNumber[], Uint32 no);
+  static void v2_setTotalParts(Uint32 fileNumber[], Uint32 no);
 
   /**
    * V4 - Specified filename
@@ -220,11 +228,10 @@ private:
  * 01234567890123456789012345678901
  *                 ssssssssvvvvvvvv
  *
- * -- v3 --
+ * -- v5 --
  * File number[0] = Table
  * File number[1] = LcpNo
- * File number[2] = 
- * File number[3] =
+ * File number[2] = Fragment Id
  *   v = version   24 - 31
  *   s = v1_suffix 16 - 23
  *
@@ -319,22 +326,46 @@ void FsOpenReq::v2_setSequence(Uint32 fileNumber[], Uint32 val){
 
 inline 
 Uint32 FsOpenReq::v2_getNodeId(const Uint32 fileNumber[]){
-  return fileNumber[1];
+  return (fileNumber[1] & 0x0000FFFF);
 }
 
 inline
 void FsOpenReq::v2_setNodeId(Uint32 fileNumber[], Uint32 val){
-  fileNumber[1] = val;
+  const Uint32 t = fileNumber[1];
+  fileNumber[1] = (t & 0xFFFF0000) | (((Uint32)val) & 0x0000FFFF);
 }
 
 inline 
+Uint32 FsOpenReq::v2_getPartNum(const Uint32 fileNumber[]){
+  return ((fileNumber[1] >> 16) & 0x0000FFFF);
+}
+
+inline
+void FsOpenReq::v2_setPartNum(Uint32 fileNumber[], Uint32 val){
+  Uint32 t = fileNumber[1] ;
+  fileNumber[1] = (t & 0x0000FFFF) | ((val << 16) & 0xFFFF0000);
+}
+
+inline
 Uint32 FsOpenReq::v2_getCount(const Uint32 fileNumber[]){
-  return fileNumber[2];
+  return (fileNumber[2] & 0x0000FFFF);
 }
 
 inline
 void FsOpenReq::v2_setCount(Uint32 fileNumber[], Uint32 val){
-  fileNumber[2] = val;
+  const Uint32 t = fileNumber[2];
+  fileNumber[2] = (t & 0xFFFF0000) | (((Uint32)val) & 0x0000FFFF);
+}
+
+inline
+Uint32 FsOpenReq::v2_getTotalParts(const Uint32 fileNumber[]){
+  return ((fileNumber[2] >> 16) & 0x0000FFFF);
+}
+
+inline
+void FsOpenReq::v2_setTotalParts(Uint32 fileNumber[], Uint32 val){
+  Uint32 t = fileNumber[2] ;
+  fileNumber[2] = (t & 0x0000FFFF) | ((val << 16) & 0xFFFF0000);
 }
 
 /****************/
