@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -56,11 +56,15 @@ private:
     DelayTCKEYCONF                         = 18,
     ZNF_CHECK_TRANSACTIONS                 = 19,
     ZSEND_FIRE_TRIG_REQ                    = 20,
-    ZSTART_FRAG_SCANS                      = 21
+    ZSTART_FRAG_SCANS                      = 21,
+    ZSEND_FRAG_SCANS                       = 22
 #ifdef ERROR_INSERT
-    ,ZDEBUG_DELAYED_ABORT                  = 22
-    ,ZDEBUG_DELAY_TCROLLBACKREP            = 23
+    ,ZDEBUG_DELAYED_ABORT                  = 23
+    ,ZDEBUG_DELAY_TCROLLBACKREP            = 24
 #endif
+    ,ZSCAN_FOR_READ_BACKUP                 = 25
+    ,ZTRANSIENT_POOL_STAT                  = 26
+    ,ZSHRINK_TRANSIENT_POOLS               = 27
   };
 };
 
