@@ -1,12 +1,16 @@
-//>>built
 define("dijit/_base/scroll", [
 	"dojo/window", // windowUtils.scrollIntoView
-	".."	// export symbol to dijit
+	"../main"	// export symbol to dijit
 ], function(windowUtils, dijit){
 	// module:
 	//		dijit/_base/scroll
-	// summary:
-	//		Back compatibility module, new code should use windowUtils directly instead of using this module.
+
+	/*=====
+	return {
+		// summary:
+		//		Back compatibility module, new code should use windowUtils directly instead of using this module.
+	};
+	=====*/
 
 	dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
 		// summary:
