@@ -1,13 +1,12 @@
 /*
-	Copyright (c) 2004-2011, The Dojo Foundation All Rights Reserved.
+	Copyright (c) 2004-2012, The Dojo Foundation All Rights Reserved.
 	Available via Academic Free License >= 2.1 OR the modified BSD license.
 	see: http://dojotoolkit.org/license for details
 */
 
 //>>built
 define("dojo/_base/NodeList",["./kernel","../query","./array","./html","../NodeList-dom"],function(_1,_2,_3){
-var _4=_2.NodeList;
-var _5=_4.prototype;
+var _4=_2.NodeList,_5=_4.prototype;
 _5.connect=_4._adaptAsForEach(function(){
 return _1.connect.apply(this,arguments);
 });
@@ -20,5 +19,5 @@ return this.connect(_7,a,b);
 };
 });
 _1.NodeList=_4;
-return _1.NodeList;
+return _4;
 });
