@@ -1,66 +1,79 @@
-//>>built
 define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"dojo/_base/declare", 
-		"../Element", "./_PlotEvents", "./common", "../axis2d/common", 
+		"./Base", "./_PlotEvents", "./common", "../axis2d/common",
 		"dojox/gfx", "dojox/gfx/matrix", "dojox/lang/functional", "dojox/lang/utils"],
-	function(lang, arr, declare, Element, PlotEvents, dc, da, g, m, df, du){
+	function(lang, arr, declare, Base, PlotEvents, dc, da, g, m, df, du){
 
 	/*=====
-	var Element = dojox.charting.Element;
-	var PlotEvents = dojox.charting.plot2d._PlotEvents;
-	dojo.declare("dojox.charting.plot2d.__PieCtorArgs", dojox.charting.plot2d.__DefaultCtorArgs, {
-		//	summary:
+	declare("dojox.charting.plot2d.__PieCtorArgs", dojox.charting.plot2d.__DefaultCtorArgs, {
+		// summary:
 		//		Specialized keyword arguments object for use in defining parameters on a Pie chart.
 	
-		//	labels: Boolean?
+		// labels: Boolean?
 		//		Whether or not to draw labels for each pie slice.  Default is true.
 		labels:			true,
 	
-		//	ticks: Boolean?
+		// ticks: Boolean?
 		//		Whether or not to draw ticks to labels within each slice. Default is false.
 		ticks:			false,
 	
-		//	fixed: Boolean?
+		// fixed: Boolean?
 		//		TODO
 		fixed:			true,
 	
-		//	precision: Number?
+		// precision: Number?
 		//		The precision at which to sum/add data values. Default is 1.
 		precision:		1,
 	
-		//	labelOffset: Number?
+		// labelOffset: Number?
 		//		The amount in pixels by which to offset labels.  Default is 20.
 		labelOffset:	20,
 	
-		//	labelStyle: String?
+		// labelStyle: String?
 		//		Options as to where to draw labels.  Values include "default", and "columns".	Default is "default".
 		labelStyle:		"default",	// default/columns
-	
-		//	htmlLabels: Boolean?
+		
+		// omitLabels: Boolean?
+		//		Whether labels of slices small to the point of not being visible are omitted.	Default false.
+		omitLabels: false,
+		
+		// htmlLabels: Boolean?
 		//		Whether or not to use HTML to render slice labels. Default is true.
 		htmlLabels:		true,
 	
-		//	radGrad: String?
+		// radGrad: String?
 		//		The type of radial gradient to use in rendering.  Default is "native".
 		radGrad:        "native",
 	
-		//	fanSize: Number?
+		// fanSize: Number?
 		//		The amount for a radial gradient.  Default is 5.
 		fanSize:		5,
 	
-		//	startAngle: Number?
+		// startAngle: Number?
 		//		Where to being rendering gradients in slices, in degrees.  Default is 0.
 		startAngle:     0,
 	
-		//	radius: Number?
+		// radius: Number?
 		//		The size of the radial gradient.  Default is 0.
-		radius:		0
+		radius:		0,
+
+		// shadow: dojox.gfx.Stroke?
+		//		An optional stroke to use to draw any shadows for a series on a plot.
+		shadow:		{},
+
+		// fill: dojox.gfx.Fill?
+		//		Any fill to be used for elements on the plot.
+		fill:		{},
+
+		// styleFunc: Function?
+		//		A function that returns a styling object for the a given data item.
+		styleFunc:	null
 	});
 	=====*/
 
 	var FUDGE_FACTOR = 0.2; // use to overlap fans
 
-	return declare("dojox.charting.plot2d.Pie", [Element, PlotEvents], {
-		//	summary:
+	return declare("dojox.charting.plot2d.Pie", [Base, PlotEvents], {
+		// summary:
 		//		The plot that represents a typical pie chart.
 		defaultParams: {
 			labels:			true,
@@ -76,76 +89,73 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 		},
 		optionalParams: {
 			radius:		0,
+			omitLabels: false,
 			// theme components
 			stroke:		{},
 			outline:	{},
 			shadow:		{},
 			fill:		{},
+			styleFunc:	null,
 			font:		"",
 			fontColor:	"",
 			labelWiring: {}
 		},
 
 		constructor: function(chart, kwArgs){
-			//	summary:
+			// summary:
 			//		Create a pie plot.
 			this.opt = lang.clone(this.defaultParams);
 			du.updateWithObject(this.opt, kwArgs);
 			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
+			this.axes = [];
 			this.run = null;
 			this.dyn = [];
 		},
 		clear: function(){
-			//	summary:
+			// summary:
 			//		Clear out all of the information tied to this plot.
-			//	returns: dojox.charting.plot2d.Pie
+			// returns: dojox/charting/plot2d/Pie
 			//		A reference to this plot for functional chaining.
-			this.dirty = true;
+			this.inherited(arguments);
 			this.dyn = [];
 			this.run = null;
-			return this;	//	dojox.charting.plot2d.Pie
+			return this;	//	dojox/charting/plot2d/Pie
 		},
 		setAxis: function(axis){
-			//	summary:
+			// summary:
 			//		Dummy method, since axes are irrelevant with a Pie chart.
-			//	returns: dojox.charting.plot2d.Pie
+			// returns: dojox/charting/plot2d/Pie
 			//		The reference to this plot for functional chaining.
-			return this;	//	dojox.charting.plot2d.Pie
+			return this;	//	dojox/charting/plot2d/Pie
 		},
 		addSeries: function(run){
-			//	summary:
+			// summary:
 			//		Add a series of data to this plot.
-			//	returns: dojox.charting.plot2d.Pie
+			// returns: dojox/charting/plot2d/Pie
 			//		The reference to this plot for functional chaining.
 			this.run = run;
-			return this;	//	dojox.charting.plot2d.Pie
+			return this;	//	dojox/charting/plot2d/Pie
 		},
 		getSeriesStats: function(){
-			//	summary:
+			// summary:
 			//		Returns default stats (irrelevant for this type of plot).
-			//	returns: Object
+			// returns: Object
 			//		{hmin, hmax, vmin, vmax} min/max in both directions.
-			return lang.delegate(dc.defaultStats);
-		},
-		initializeScalers: function(){
-			//	summary:
-			//		Does nothing (irrelevant for this type of plot).
-			return this;
+			return lang.delegate(dc.defaultStats); // Object
 		},
 		getRequiredColors: function(){
-			//	summary:
+			// summary:
 			//		Return the number of colors needed to draw this plot.
 			return this.run ? this.run.data.length : 0;
 		},
-
 		render: function(dim, offsets){
-			//	summary:
+			// summary:
 			//		Render the plot on the chart.
-			//	dim: Object
+			// dim: Object
 			//		An object of the form { width, height }.
-			//	offsets: Object
+			// offsets: Object
 			//		An object of the form { l, r, t, b }.
-			//	returns: dojox.charting.plot2d.Pie
+			// returns: dojox/charting/plot2d/Pie
 			//		A reference to this plot for functional chaining.
 			if(!this.dirty){ return this; }
 			this.resetEvents();
@@ -162,54 +172,15 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 			var rx = (dim.width  - offsets.l - offsets.r) / 2,
 				ry = (dim.height - offsets.t - offsets.b) / 2,
 				r  = Math.min(rx, ry),
-				taFont = "font" in this.opt ? this.opt.font : t.axis.font,
-				size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0,
-				taFontColor = "fontColor" in this.opt ? this.opt.fontColor : t.axis.fontColor,
+				labelFont = "font" in this.opt ? this.opt.font : t.series.font,
+				size,
 				startAngle = m._degToRad(this.opt.startAngle),
 				start = startAngle, step, filteredRun, slices, labels, shift, labelR,
 				run = this.run.data,
 				events = this.events();
-			if(typeof run[0] == "number"){
-				filteredRun = df.map(run, "x ? Math.max(x, 0) : 0");
-				if(df.every(filteredRun, "<= 0")){
-					return this;
-				}
-				slices = df.map(filteredRun, "/this", df.foldl(filteredRun, "+", 0));
-				if(this.opt.labels){
-					labels = arr.map(slices, function(x){
-						return x > 0 ? this._getLabel(x * 100) + "%" : "";
-					}, this);
-				}
-			}else{
-				filteredRun = df.map(run, "x ? Math.max(x.y, 0) : 0");
-				if(df.every(filteredRun, "<= 0")){
-					return this;
-				}
-				slices = df.map(filteredRun, "/this", df.foldl(filteredRun, "+", 0));
-				if(this.opt.labels){
-					labels = arr.map(slices, function(x, i){
-						if(x <= 0){ return ""; }
-						var v = run[i];
-						return "text" in v ? v.text : this._getLabel(x * 100) + "%";
-					}, this);
-				}
-			}
-			var themes = df.map(run, function(v, i){
-				if(v === null || typeof v == "number"){
-					return t.next("slice", [this.opt, this.run], true);
-				}
-				return t.next("slice", [this.opt, this.run, v], true);
-			}, this);
-			if(this.opt.labels){
-				shift = df.foldl1(df.map(labels, function(label, i){
-					var font = themes[i].series.font;
-					return g._base._getTextBox(label, {font: font}).w;
-				}, this), "Math.max(a, b)") / 2;
-				if(this.opt.labelOffset < 0){
-					r = Math.min(rx - 2 * shift, ry - size) + this.opt.labelOffset;
-				}
-				labelR = r - this.opt.labelOffset;
-			}
+
+			this.dyn = [];
+
 			if("radius" in this.opt){
 				r = this.opt.radius;
 				labelR = r - this.opt.labelOffset;
@@ -220,7 +191,73 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 					r:  r
 				};
 
-			this.dyn = [];
+			// draw shadow
+			if(this.opt.shadow || t.shadow){
+				var shadow = this.opt.shadow || t.shadow;
+				var scircle = lang.clone(circle);
+				scircle.cx += shadow.dx;
+				scircle.cy += shadow.dy;
+				s.createCircle(scircle).setFill(shadow.color).setStroke(shadow);
+			}
+
+			if(typeof run[0] == "number"){
+				filteredRun = df.map(run, "x ? Math.max(x, 0) : 0");
+				if(df.every(filteredRun, "<= 0")){
+					s.createCircle(circle).setStroke(t.series.stroke);
+					this.dyn = arr.map(filteredRun, function(){
+						return {  };
+					});
+					return this;
+				}else{
+					slices = df.map(filteredRun, "/this", df.foldl(filteredRun, "+", 0));
+				 	if(this.opt.labels){
+				 		labels = arr.map(slices, function(x){
+							return x > 0 ? this._getLabel(x * 100) + "%" : "";
+						}, this);
+					}
+				}
+			}else{
+				filteredRun = df.map(run, "x ? Math.max(x.y, 0) : 0");
+				if(df.every(filteredRun, "<= 0")){
+					s.createCircle(circle).setStroke(t.series.stroke);
+					this.dyn = arr.map(filteredRun, function(){
+						return {  };
+					});
+					return this;
+				}else{
+					slices = df.map(filteredRun, "/this", df.foldl(filteredRun, "+", 0));
+					if(this.opt.labels){
+						labels = arr.map(slices, function(x, i){
+							if(x < 0){ return ""; }
+							var v = run[i];
+							return "text" in v ? v.text : this._getLabel(x * 100) + "%";
+						}, this);
+					}
+				}
+			}
+			var themes = df.map(run, function(v, i){
+				var tMixin = [this.opt, this.run];
+				if(v !== null && typeof v != "number"){
+					tMixin.push(v);
+				}
+				if(this.opt.styleFunc){
+					tMixin.push(this.opt.styleFunc(v));
+				}
+				return t.next("slice", tMixin, true);
+			}, this);
+
+			if(this.opt.labels){
+				size = labelFont ? g.normalizedLength(g.splitFontString(labelFont).size) : 0;
+				shift = df.foldl1(df.map(labels, function(label, i){
+					var font = themes[i].series.font;
+					return g._base._getTextBox(label, {font: font}).w;
+				}, this), "Math.max(a, b)") / 2;
+				if(this.opt.labelOffset < 0){
+					r = Math.min(rx - 2 * shift, ry - size) + this.opt.labelOffset;
+				}
+				labelR = r - this.opt.labelOffset;
+			}
+
 			// draw slices
 			var eventSeries = new Array(slices.length);
 			arr.some(slices, function(slice, i){
@@ -232,7 +269,7 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 				  this.dyn.push({fill: null, stroke: null});
 				  return false;
 				}
-				var v = run[i], theme = themes[i], specialFill;
+				var v = run[i], theme = themes[i], specialFill, o;
 				if(slice >= 1){
 					// whole pie
 					specialFill = this._plotFill(theme.series.fill, dim, offsets);
@@ -246,7 +283,7 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 					this.dyn.push({fill: specialFill, stroke: theme.series.stroke});
 
 					if(events){
-						var o = {
+						o = {
 							element: "slice",
 							index:   i,
 							run:     this.run,
@@ -308,7 +345,7 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 						lineTo(circle.cx, circle.cy).
 						closePath().
 						setStroke(theme.series.stroke);
-					var specialFill = theme.series.fill;
+					specialFill = theme.series.fill;
 					if(specialFill && specialFill.type === "radial"){
 						specialFill = this._shapeFill(specialFill, {x: circle.cx - circle.r, y: circle.cy - circle.r, width: 2 * circle.r, height: 2 * circle.r});
 						if(this.opt.radGrad === "linear"){
@@ -323,7 +360,7 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 				this.dyn.push({fill: specialFill, stroke: theme.series.stroke});
 
 				if(events){
-					var o = {
+					o = {
 						element: "slice",
 						index:   i,
 						run:     this.run,
@@ -351,10 +388,10 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 							// degenerated slice
 							return false;	// continue
 						}
-						var theme = themes[i];
+						var theme = themes[i], elem;
 						if(slice >= 1){
 							// whole pie
-							var v = run[i], elem = da.createText[this.opt.htmlLabels && g.renderer != "vml" ? "html" : "gfx"](
+							elem = da.createText[this.opt.htmlLabels && g.renderer != "vml" ? "html" : "gfx"](
 									this.chart, s, circle.cx, circle.cy + size / 2, "middle", labels[i],
 									theme.series.font, theme.series.fontColor);
 							if(this.opt.htmlLabels){
@@ -363,15 +400,18 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 							return true;	// stop iteration
 						}
 						// calculate the geometry of the slice
-						var end = start + slice * 2 * Math.PI, v = run[i];
+						var end = start + slice * 2 * Math.PI;
 						if(i + 1 == slices.length){
 							end = startAngle + 2 * Math.PI;
 						}
+						if(this.opt.omitLabels && end-start < 0.001){
+							return false;	// continue
+						}
 						var	labelAngle = (start + end) / 2,
 							x = circle.cx + labelR * Math.cos(labelAngle),
 							y = circle.cy + labelR * Math.sin(labelAngle) + size / 2;
 						// draw the label
-						var elem = da.createText[this.opt.htmlLabels && g.renderer != "vml" ? "html" : "gfx"]
+						elem = da.createText[this.opt.htmlLabels && g.renderer != "vml" ? "html" : "gfx"]
 								(this.chart, s, x, y, "middle", labels[i], theme.series.font, theme.series.fontColor);
 						if(this.opt.htmlLabels){
 							this.htmlElements.push(elem);
@@ -381,6 +421,7 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 					}, this);
 				}else if(this.opt.labelStyle == "columns"){
 					start = startAngle;
+					var omitLabels = this.opt.omitLabels;
 					//calculate label angles
 					var labeledSlices = [];
 					arr.forEach(slices, function(slice, i){
@@ -394,31 +435,31 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 							left: Math.cos(labelAngle) < 0,
 							theme: themes[i],
 							index: i,
-							omit: end - start < 0.001
+							omit: omitLabels?end - start < 0.001:false
 						});
 						start = end;
 					});
 					//calculate label radius to each slice
-					var labelHeight = g._base._getTextBox("a",{font:taFont}).h;
+					var labelHeight = g._base._getTextBox("a",{ font: labelFont }).h;
 					this._getProperLabelRadius(labeledSlices, labelHeight, circle.r * 1.1);
 					//draw label and wiring
 					arr.forEach(labeledSlices, function(slice, i){
-						if (!slice.omit) {
+						if(!slice.omit){
 							var leftColumn = circle.cx - circle.r * 2,
 								rightColumn = circle.cx + circle.r * 2,
-								labelWidth = g._base._getTextBox(labels[i], {font: taFont}).w,
+								labelWidth = g._base._getTextBox(labels[i], {font: slice.theme.series.font}).w,
 								x = circle.cx + slice.labelR * Math.cos(slice.angle),
 								y = circle.cy + slice.labelR * Math.sin(slice.angle),
 								jointX = (slice.left) ? (leftColumn + labelWidth) : (rightColumn - labelWidth),
 								labelX = (slice.left) ? leftColumn : jointX;
-							var wiring = s.createPath().moveTo(circle.cx + circle.r * Math.cos(slice.angle), circle.cy + circle.r * Math.sin(slice.angle))
-							if (Math.abs(slice.labelR * Math.cos(slice.angle)) < circle.r * 2 - labelWidth) {
+							var wiring = s.createPath().moveTo(circle.cx + circle.r * Math.cos(slice.angle), circle.cy + circle.r * Math.sin(slice.angle));
+							if(Math.abs(slice.labelR * Math.cos(slice.angle)) < circle.r * 2 - labelWidth){
 								wiring.lineTo(x, y);
 							}
 							wiring.lineTo(jointX, y).setStroke(slice.theme.series.labelWiring);
 							var elem = da.createText[this.opt.htmlLabels && g.renderer != "vml" ? "html" : "gfx"](
 								this.chart, s, labelX, y, "left", labels[i], slice.theme.series.font, slice.theme.series.fontColor);
-							if (this.opt.htmlLabels) {
+							if(this.opt.htmlLabels){
 								this.htmlElements.push(elem);
 							}
 						}
@@ -430,39 +471,40 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 			this._eventSeries[this.run.name] = df.map(run, function(v){
 				return v <= 0 ? null : eventSeries[esi++];
 			});
-			return this;	//	dojox.charting.plot2d.Pie
+			return this;	//	dojox/charting/plot2d/Pie
 		},
 		
 		_getProperLabelRadius: function(slices, labelHeight, minRidius){
-			var leftCenterSlice = {},rightCenterSlice = {},leftMinSIN = 1, rightMinSIN = 1;
-			if (slices.length == 1) {
+			var leftCenterSlice, rightCenterSlice,
+				leftMinSIN = 1, rightMinSIN = 1;
+			if(slices.length == 1){
 				slices[0].labelR = minRidius;
 				return;
 			}
-			for(var i = 0;i<slices.length;i++){
+			for(var i = 0; i < slices.length; i++){
 				var tempSIN = Math.abs(Math.sin(slices[i].angle));
 				if(slices[i].left){
-					if(leftMinSIN > tempSIN){
+					if(leftMinSIN >= tempSIN){
 						leftMinSIN = tempSIN;
 						leftCenterSlice = slices[i];
 					}
 				}else{
-					if(rightMinSIN > tempSIN){
+					if(rightMinSIN >= tempSIN){
 						rightMinSIN = tempSIN;
 						rightCenterSlice = slices[i];
 					}
 				}
 			}
 			leftCenterSlice.labelR = rightCenterSlice.labelR = minRidius;
-			this._calculateLabelR(leftCenterSlice,slices,labelHeight);
-			this._calculateLabelR(rightCenterSlice,slices,labelHeight);
+			this._calculateLabelR(leftCenterSlice, slices, labelHeight);
+			this._calculateLabelR(rightCenterSlice, slices, labelHeight);
 		},
-		_calculateLabelR: function(firstSlice,slices,labelHeight){
+		_calculateLabelR: function(firstSlice, slices, labelHeight){
 			var i = firstSlice.index,length = slices.length,
-				currentLabelR = firstSlice.labelR;
+				currentLabelR = firstSlice.labelR, nextLabelR;
 			while(!(slices[i%length].left ^ slices[(i+1)%length].left)){
-				if (!slices[(i + 1) % length].omit) {
-					var nextLabelR = (Math.sin(slices[i % length].angle) * currentLabelR + ((slices[i % length].left) ? (-labelHeight) : labelHeight)) /
+				if(!slices[(i + 1) % length].omit){
+					nextLabelR = (Math.sin(slices[i % length].angle) * currentLabelR + ((slices[i % length].left) ? (-labelHeight) : labelHeight)) /
 					Math.sin(slices[(i + 1) % length].angle);
 					currentLabelR = (nextLabelR < firstSlice.labelR) ? firstSlice.labelR : nextLabelR;
 					slices[(i + 1) % length].labelR = currentLabelR;
@@ -472,8 +514,8 @@ define("dojox/charting/plot2d/Pie", ["dojo/_base/lang", "dojo/_base/array" ,"doj
 			i = firstSlice.index;
 			var j = (i == 0)?length-1 : i - 1;
 			while(!(slices[i].left ^ slices[j].left)){
-				if (!slices[j].omit) {
-					var nextLabelR = (Math.sin(slices[i].angle) * currentLabelR + ((slices[i].left) ? labelHeight : (-labelHeight))) /
+				if(!slices[j].omit){
+					nextLabelR = (Math.sin(slices[i].angle) * currentLabelR + ((slices[i].left) ? labelHeight : (-labelHeight))) /
 					Math.sin(slices[j].angle);
 					currentLabelR = (nextLabelR < firstSlice.labelR) ? firstSlice.labelR : nextLabelR;
 					slices[j].labelR = currentLabelR;
