@@ -1,25 +1,17 @@
-//>>built
 define("dojox/mobile/EdgeToEdgeDataList", [
 	"dojo/_base/declare",
 	"./EdgeToEdgeList",
 	"./_DataListMixin"
 ], function(declare, EdgeToEdgeList, DataListMixin){
 
-/*=====
-	var EdgeToEdgeList = dojox.mobile.EdgeToEdgeList;
-	var DataListMixin = dojox.mobile._DataListMixin;
-=====*/
-
 	// module:
 	//		dojox/mobile/EdgeToEdgeDataList
-	// summary:
-	//		An enhanced version of EdgeToEdgeList.
 
 	return declare("dojox.mobile.EdgeToEdgeDataList", [EdgeToEdgeList, DataListMixin],{
 		// summary:
-		//		An enhanced version of EdgeToEdgeList.
+		//		A dojo/data-enabled version of EdgeToEdgeList.
 		// description:
 		//		EdgeToEdgeDataList is an enhanced version of EdgeToEdgeList. It
-		//		can generate ListItems according to the given dojo.data store.
+		//		can generate ListItems according to the given dojo/data store.
 	});
 });
