@@ -1,23 +1,20 @@
 //>>built
 define("dojox/mobile/scrollable",["dojo/_base/kernel","dojo/_base/connect","dojo/_base/event","dojo/_base/lang","dojo/_base/window","dojo/dom-class","dojo/dom-construct","dojo/dom-style","./sniff"],function(_1,_2,_3,_4,_5,_6,_7,_8,_9){
 var dm=_4.getObject("dojox.mobile",true);
-var _a=function(_b,_c){
-this.fixedHeaderHeight=0;
-this.fixedFooterHeight=0;
-this.isLocalFooter=false;
-this.scrollBar=true;
-this.scrollDir="v";
-this.weight=0.6;
-this.fadeScrollBar=true;
-this.disableFlashScrollBar=false;
-this.threshold=4;
-this.constraint=true;
-this.touchNode=null;
-this.isNested=false;
-this.dirLock=false;
-this.height="";
-this.androidWorkaroud=true;
-this.init=function(_d){
+_9.add("translate3d",function(){
+if(_9("webkit")){
+var _a=_5.doc.createElement("div");
+_a.style.webkitTransform="translate3d(0px,1px,0px)";
+_5.doc.documentElement.appendChild(_a);
+var v=_5.doc.defaultView.getComputedStyle(_a,"")["-webkit-transform"];
+var _b=v&&v.indexOf("matrix")===0;
+_5.doc.documentElement.removeChild(_a);
+return _b;
+}
+});
+var _c=function(){
+};
+_4.extend(_c,{fixedHeaderHeight:0,fixedFooterHeight:0,isLocalFooter:false,scrollBar:true,scrollDir:"v",weight:0.6,fadeScrollBar:true,disableFlashScrollBar:false,threshold:4,constraint:true,touchNode:null,propagatable:true,dirLock:false,height:"",scrollType:0,init:function(_d){
 if(_d){
 for(var p in _d){
 if(_d.hasOwnProperty(p)){
@@ -30,28 +27,31 @@ this._v=(this.scrollDir.indexOf("v")!=-1);
 this._h=(this.scrollDir.indexOf("h")!=-1);
 this._f=(this.scrollDir=="f");
 this._ch=[];
-this._ch.push(_2.connect(this.touchNode,_9("touch")?"touchstart":"onmousedown",this,"onTouchStart"));
+this._ch.push(_2.connect(this.touchNode,_9("touch")?"ontouchstart":"onmousedown",this,"onTouchStart"));
 if(_9("webkit")){
+this._useTopLeft=this.scrollType?this.scrollType===2:_9("android")<3;
+if(!this._useTopLeft){
+this._useTransformTransition=this.scrollType?this.scrollType===3:_9("iphone")>=6;
+}
+if(!this._useTopLeft){
+if(this._useTransformTransition){
+this._ch.push(_2.connect(this.domNode,"webkitTransitionEnd",this,"onFlickAnimationEnd"));
+this._ch.push(_2.connect(this.domNode,"webkitTransitionStart",this,"onFlickAnimationStart"));
+}else{
 this._ch.push(_2.connect(this.domNode,"webkitAnimationEnd",this,"onFlickAnimationEnd"));
 this._ch.push(_2.connect(this.domNode,"webkitAnimationStart",this,"onFlickAnimationStart"));
-this._aw=this.androidWorkaroud&&_9("android")>=2.2&&_9("android")<3;
-if(this._aw){
-this._ch.push(_2.connect(_5.global,"onresize",this,"onScreenSizeChanged"));
-this._ch.push(_2.connect(_5.global,"onfocus",this,function(e){
-if(this.containerNode.style.webkitTransform){
-this.stopAnimation();
-this.toTopLeft();
-}
-}));
-this._sz=this.getScreenSize();
-}
 for(var i=0;i<3;i++){
 this.setKeyframes(null,null,i);
 }
 }
-if(_9("iphone")){
+if(_9("translate3d")){
 _8.set(this.containerNode,"webkitTransform","translate3d(0,0,0)");
 }
+}else{
+this._ch.push(_2.connect(this.domNode,"webkitTransitionEnd",this,"onFlickAnimationEnd"));
+this._ch.push(_2.connect(this.domNode,"webkitTransitionStart",this,"onFlickAnimationStart"));
+}
+}
 this._speed={x:0,y:0};
 this._appFooterHeight=0;
 if(this.isTopLevel()&&!this.noResize){
@@ -61,22 +61,22 @@ var _e=this;
 setTimeout(function(){
 _e.flashScrollBar();
 },600);
-};
-this.isTopLevel=function(){
+},isTopLevel:function(){
 return true;
-};
-this.cleanup=function(){
+},cleanup:function(){
 if(this._ch){
 for(var i=0;i<this._ch.length;i++){
 _2.disconnect(this._ch[i]);
 }
 this._ch=null;
 }
-};
-this.findDisp=function(_f){
+},findDisp:function(_f){
 if(!_f.parentNode){
 return null;
 }
+if(_f.nodeType===1&&_6.contains(_f,"mblSwapView")&&_f.style.display!=="none"){
+return _f;
+}
 var _10=_f.parentNode.childNodes;
 for(var i=0;i<_10.length;i++){
 var n=_10[i];
@@ -85,87 +85,9 @@ return n;
 }
 }
 return _f;
-};
-this.getScreenSize=function(){
+},getScreenSize:function(){
 return {h:_5.global.innerHeight||_5.doc.documentElement.clientHeight||_5.doc.documentElement.offsetHeight,w:_5.global.innerWidth||_5.doc.documentElement.clientWidth||_5.doc.documentElement.offsetWidth};
-};
-this.isKeyboardShown=function(e){
-if(!this._sz){
-return false;
-}
-var sz=this.getScreenSize();
-return (sz.w*sz.h)/(this._sz.w*this._sz.h)<0.8;
-};
-this.disableScroll=function(v){
-if(this.disableTouchScroll===v||this.domNode.style.display==="none"){
-return;
-}
-this.disableTouchScroll=v;
-this.scrollBar=!v;
-dm.disableHideAddressBar=dm.disableResizeAll=v;
-var of=v?"visible":"hidden";
-_8.set(this.domNode,"overflow",of);
-_8.set(_5.doc.documentElement,"overflow",of);
-_8.set(_5.body(),"overflow",of);
-var c=this.containerNode;
-if(v){
-if(!c.style.webkitTransform){
-this.stopAnimation();
-this.toTopLeft();
-}
-var mt=parseInt(c.style.marginTop)||0;
-var h=c.offsetHeight+mt+this.fixedFooterHeight-this._appFooterHeight;
-_8.set(this.domNode,"height",h+"px");
-this._cPos={x:parseInt(c.style.left)||0,y:parseInt(c.style.top)||0};
-_8.set(c,{top:"0px",left:"0px"});
-var a=_5.doc.activeElement;
-if(a){
-var at=0;
-for(var n=a;n.tagName!="BODY";n=n.offsetParent){
-at+=n.offsetTop;
-}
-var st=at+a.clientHeight+10-this.getScreenSize().h;
-if(st>0){
-_5.body().scrollTop=st;
-}
-}
-}else{
-if(this._cPos){
-_8.set(c,{top:this._cPos.y+"px",left:this._cPos.x+"px"});
-this._cPos=null;
-}
-var _11=this.domNode.getElementsByTagName("*");
-for(var i=0;i<_11.length;i++){
-_11[i].blur&&_11[i].blur();
-}
-dm.resizeAll&&dm.resizeAll();
-}
-};
-this.onScreenSizeChanged=function(e){
-var sz=this.getScreenSize();
-if(sz.w*sz.h>this._sz.w*this._sz.h){
-this._sz=sz;
-}
-this.disableScroll(this.isKeyboardShown());
-};
-this.toTransform=function(e){
-var c=this.containerNode;
-if(c.offsetTop===0&&c.offsetLeft===0||!c._webkitTransform){
-return;
-}
-_8.set(c,{webkitTransform:c._webkitTransform,top:"0px",left:"0px"});
-c._webkitTransform=null;
-};
-this.toTopLeft=function(){
-var c=this.containerNode;
-if(!c.style.webkitTransform){
-return;
-}
-c._webkitTransform=c.style.webkitTransform;
-var pos=this.getPos();
-_8.set(c,{webkitTransform:"",top:pos.y+"px",left:pos.x+"px"});
-};
-this.resize=function(e){
+},resize:function(e){
 this._appFooterHeight=(this.fixedFooterHeight&&!this.isLocalFooter)?this.fixedFooterHeight:0;
 if(this.isLocalHeader){
 this.containerNode.style.marginTop=this.fixedHeaderHeight+"px";
@@ -178,25 +100,25 @@ break;
 }
 top+=n.offsetTop;
 }
-var h,_12=this.getScreenSize().h,dh=_12-top-this._appFooterHeight;
+var h,_11=this.getScreenSize().h,dh=_11-top-this._appFooterHeight;
 if(this.height==="inherit"){
 if(this.domNode.offsetParent){
 h=this.domNode.offsetParent.offsetHeight+"px";
 }
 }else{
 if(this.height==="auto"){
-var _13=this.domNode.offsetParent;
-if(_13){
+var _12=this.domNode.offsetParent;
+if(_12){
 this.domNode.style.height="0px";
-var _14=_13.getBoundingClientRect(),_15=this.domNode.getBoundingClientRect(),_16=_14.bottom-this._appFooterHeight;
-if(_15.bottom>=_16){
-dh=_12-(_15.top-_14.top)-this._appFooterHeight;
+var _13=_12.getBoundingClientRect(),_14=this.domNode.getBoundingClientRect(),_15=_13.bottom-this._appFooterHeight;
+if(_14.bottom>=_15){
+dh=_11-(_14.top-_13.top)-this._appFooterHeight;
 }else{
-dh=_16-_15.bottom;
+dh=_15-_14.bottom;
 }
 }
-var _17=Math.max(this.domNode.scrollHeight,this.containerNode.scrollHeight);
-h=(_17?Math.min(_17,dh):dh)+"px";
+var _16=Math.max(this.domNode.scrollHeight,this.containerNode.scrollHeight);
+h=(_16?Math.min(_16,dh):dh)+"px";
 }else{
 if(this.height){
 h=this.height;
@@ -210,18 +132,20 @@ if(h.charAt(0)!=="-"&&h!=="default"){
 this.domNode.style.height=h;
 }
 this.onTouchEnd();
-};
-this.onFlickAnimationStart=function(e){
+},onFlickAnimationStart:function(e){
 _3.stop(e);
-};
-this.onFlickAnimationEnd=function(e){
+},onFlickAnimationEnd:function(e){
 var an=e&&e.animationName;
 if(an&&an.indexOf("scrollableViewScroll2")===-1){
 if(an.indexOf("scrollableViewScroll0")!==-1){
+if(this._scrollBarNodeV){
 _6.remove(this._scrollBarNodeV,"mblScrollableScrollTo0");
+}
 }else{
 if(an.indexOf("scrollableViewScroll1")!==-1){
+if(this._scrollBarNodeH){
 _6.remove(this._scrollBarNodeH,"mblScrollableScrollTo1");
+}
 }else{
 if(this._scrollBarNodeV){
 this._scrollBarNodeV.className="";
@@ -233,36 +157,43 @@ this._scrollBarNodeH.className="";
 }
 return;
 }
+if(this._useTransformTransition||this._useTopLeft){
+var n=e.target;
+if(n===this._scrollBarV||n===this._scrollBarH){
+var cls="mblScrollableScrollTo"+(n===this._scrollBarV?"0":"1");
+if(_6.contains(n,cls)){
+_6.remove(n,cls);
+}else{
+n.className="";
+}
+return;
+}
+}
 if(e&&e.srcElement){
 _3.stop(e);
 }
 this.stopAnimation();
 if(this._bounce){
-var _18=this;
-var _19=_18._bounce;
+var _17=this;
+var _18=_17._bounce;
 setTimeout(function(){
-_18.slideTo(_19,0.3,"ease-out");
+_17.slideTo(_18,0.3,"ease-out");
 },0);
-_18._bounce=undefined;
+_17._bounce=undefined;
 }else{
 this.hideScrollBar();
 this.removeCover();
-if(this._aw){
-this.toTopLeft();
 }
+},isFormElement:function(_19){
+if(_19&&_19.nodeType!==1){
+_19=_19.parentNode;
 }
-};
-this.isFormElement=function(_1a){
-if(_1a&&_1a.nodeType!==1){
-_1a=_1a.parentNode;
-}
-if(!_1a||_1a.nodeType!==1){
+if(!_19||_19.nodeType!==1){
 return false;
 }
-var t=_1a.tagName;
+var t=_19.tagName;
 return (t==="SELECT"||t==="INPUT"||t==="TEXTAREA"||t==="BUTTON");
-};
-this.onTouchStart=function(e){
+},onTouchStart:function(e){
 if(this.disableTouchScroll){
 return;
 }
@@ -271,8 +202,8 @@ return;
 }
 if(!this._conn){
 this._conn=[];
-this._conn.push(_2.connect(_5.doc,_9("touch")?"touchmove":"onmousemove",this,"onTouchMove"));
-this._conn.push(_2.connect(_5.doc,_9("touch")?"touchend":"onmouseup",this,"onTouchEnd"));
+this._conn.push(_2.connect(_5.doc,_9("touch")?"ontouchmove":"onmousemove",this,"onTouchMove"));
+this._conn.push(_2.connect(_5.doc,_9("touch")?"ontouchend":"onmouseup",this,"onTouchEnd"));
 }
 this._aborted=false;
 if(_6.contains(this.containerNode,"mblScrollableScrollTo2")){
@@ -285,9 +216,6 @@ if(this._scrollBarNodeH){
 this._scrollBarNodeH.className="";
 }
 }
-if(this._aw){
-this.toTransform(e);
-}
 this.touchStartX=e.touches?e.touches[0].pageX:e.clientX;
 this.touchStartY=e.touches?e.touches[0].pageY:e.clientY;
 this.startTime=(new Date()).getTime();
@@ -297,11 +225,10 @@ this._time=[0];
 this._posX=[this.touchStartX];
 this._posY=[this.touchStartY];
 this._locked=false;
-if(!this.isFormElement(e.target)&&!this.isNested){
-_3.stop(e);
+if(!this.isFormElement(e.target)){
+this.propagatable?e.preventDefault():_3.stop(e);
 }
-};
-this.onTouchMove=function(e){
+},onTouchMove:function(e){
 if(this._locked){
 return;
 }
@@ -320,35 +247,41 @@ this._locked=true;
 return;
 }
 }
-if(this._v&&Math.abs(dy)<this.threshold||(this._h||this._f)&&Math.abs(dx)<this.threshold){
+if(this._v&&this._h){
+if(dy<this.threshold&&dx<this.threshold){
 return;
 }
+}else{
+if(this._v&&dy<this.threshold||(this._h||this._f)&&dx<this.threshold){
+return;
+}
+}
 this.addCover();
 this.showScrollBar();
 }
-var _1b=this.weight;
+var _1a=this.weight;
 if(this._v&&this.constraint){
 if(to.y>0){
-to.y=Math.round(to.y*_1b);
+to.y=Math.round(to.y*_1a);
 }else{
 if(to.y<-dim.o.h){
 if(dim.c.h<dim.d.h){
-to.y=Math.round(to.y*_1b);
+to.y=Math.round(to.y*_1a);
 }else{
-to.y=-dim.o.h-Math.round((-dim.o.h-to.y)*_1b);
+to.y=-dim.o.h-Math.round((-dim.o.h-to.y)*_1a);
 }
 }
 }
 }
 if((this._h||this._f)&&this.constraint){
 if(to.x>0){
-to.x=Math.round(to.x*_1b);
+to.x=Math.round(to.x*_1a);
 }else{
 if(to.x<-dim.o.w){
 if(dim.c.w<dim.d.w){
-to.x=Math.round(to.x*_1b);
+to.x=Math.round(to.x*_1a);
 }else{
-to.x=-dim.o.w-Math.round((-dim.o.w-to.x)*_1b);
+to.x=-dim.o.w-Math.round((-dim.o.w-to.x)*_1a);
 }
 }
 }
@@ -382,12 +315,11 @@ this._posY.shift();
 this._time.push((new Date()).getTime()-this.startTime);
 this._posX.push(x);
 this._posY.push(y);
-};
-this.onTouchEnd=function(e){
+},onTouchEnd:function(e){
 if(this._locked){
 return;
 }
-var _1c=this._speed={x:0,y:0};
+var _1b=this._speed={x:0,y:0};
 var dim=this._dim;
 var pos=this.getPos();
 var to={};
@@ -400,40 +332,33 @@ _2.disconnect(this._conn[i]);
 }
 this._conn=null;
 var n=this._time.length;
-var _1d=false;
+var _1c=false;
 if(!this._aborted){
 if(n<=1){
-_1d=true;
+_1c=true;
 }else{
 if(n==2&&Math.abs(this._posY[1]-this._posY[0])<4&&_9("touch")){
-_1d=true;
+_1c=true;
 }
 }
 }
-var _1e=this.isFormElement(e.target);
-if(_1d&&!_1e){
+if(_1c){
 this.hideScrollBar();
 this.removeCover();
-if(_9("touch")){
-var _1f=e.target;
-if(_1f.nodeType!=1){
-_1f=_1f.parentNode;
+if(_9("touch")&&!this.isFormElement(e.target)&&!(_9("android")>=4.1)){
+var _1d=e.target;
+if(_1d.nodeType!=1){
+_1d=_1d.parentNode;
 }
 var ev=_5.doc.createEvent("MouseEvents");
 ev.initMouseEvent("click",true,true,_5.global,1,e.screenX,e.screenY,e.clientX,e.clientY);
 setTimeout(function(){
-_1f.dispatchEvent(ev);
+_1d.dispatchEvent(ev);
 },0);
 }
 return;
-}else{
-if(this._aw&&_1d&&_1e){
-this.hideScrollBar();
-this.toTopLeft();
-return;
 }
-}
-_1c=this._speed=this.getSpeed();
+_1b=this._speed=this.getSpeed();
 }else{
 if(pos.x==0&&pos.y==0){
 return;
@@ -441,12 +366,14 @@ return;
 dim=this.getDim();
 }
 if(this._v){
-to.y=pos.y+_1c.y;
+to.y=pos.y+_1b.y;
 }
 if(this._h||this._f){
-to.x=pos.x+_1c.x;
+to.x=pos.x+_1b.x;
+}
+if(this.adjustDestination(to,pos,dim)===false){
+return;
 }
-this.adjustDestination(to,pos);
 if(this.scrollDir=="v"&&dim.c.h<dim.d.h){
 this.slideTo({y:0},0.3,"ease-out");
 return;
@@ -461,27 +388,27 @@ return;
 }
 }
 }
-var _20,_21="ease-out";
-var _22={};
+var _1e,_1f="ease-out";
+var _20={};
 if(this._v&&this.constraint){
 if(to.y>0){
 if(pos.y>0){
-_20=0.3;
+_1e=0.3;
 to.y=0;
 }else{
 to.y=Math.min(to.y,20);
-_21="linear";
-_22.y=0;
+_1f="linear";
+_20.y=0;
 }
 }else{
-if(-_1c.y>dim.o.h-(-pos.y)){
+if(-_1b.y>dim.o.h-(-pos.y)){
 if(pos.y<-dim.o.h){
-_20=0.3;
+_1e=0.3;
 to.y=dim.c.h<=dim.d.h?0:-dim.o.h;
 }else{
 to.y=Math.max(to.y,-dim.o.h-20);
-_21="linear";
-_22.y=-dim.o.h;
+_1f="linear";
+_20.y=-dim.o.h;
 }
 }
 }
@@ -489,70 +416,86 @@ _22.y=-dim.o.h;
 if((this._h||this._f)&&this.constraint){
 if(to.x>0){
 if(pos.x>0){
-_20=0.3;
+_1e=0.3;
 to.x=0;
 }else{
 to.x=Math.min(to.x,20);
-_21="linear";
-_22.x=0;
+_1f="linear";
+_20.x=0;
 }
 }else{
-if(-_1c.x>dim.o.w-(-pos.x)){
+if(-_1b.x>dim.o.w-(-pos.x)){
 if(pos.x<-dim.o.w){
-_20=0.3;
+_1e=0.3;
 to.x=dim.c.w<=dim.d.w?0:-dim.o.w;
 }else{
 to.x=Math.max(to.x,-dim.o.w-20);
-_21="linear";
-_22.x=-dim.o.w;
+_1f="linear";
+_20.x=-dim.o.w;
 }
 }
 }
 }
-this._bounce=(_22.x!==undefined||_22.y!==undefined)?_22:undefined;
-if(_20===undefined){
-var _23,_24;
+this._bounce=(_20.x!==undefined||_20.y!==undefined)?_20:undefined;
+if(_1e===undefined){
+var _21,_22;
 if(this._v&&this._h){
-_24=Math.sqrt(_1c.x+_1c.x+_1c.y*_1c.y);
-_23=Math.sqrt(Math.pow(to.y-pos.y,2)+Math.pow(to.x-pos.x,2));
+_22=Math.sqrt(_1b.x*_1b.x+_1b.y*_1b.y);
+_21=Math.sqrt(Math.pow(to.y-pos.y,2)+Math.pow(to.x-pos.x,2));
 }else{
 if(this._v){
-_24=_1c.y;
-_23=to.y-pos.y;
+_22=_1b.y;
+_21=to.y-pos.y;
 }else{
 if(this._h){
-_24=_1c.x;
-_23=to.x-pos.x;
+_22=_1b.x;
+_21=to.x-pos.x;
 }
 }
 }
-if(_23===0&&!e){
+if(_21===0&&!e){
 return;
 }
-_20=_24!==0?Math.abs(_23/_24):0.01;
+_1e=_22!==0?Math.abs(_21/_22):0.01;
 }
-this.slideTo(to,_20,_21);
-};
-this.adjustDestination=function(to,pos){
-};
-this.abort=function(){
+this.slideTo(to,_1e,_1f);
+},adjustDestination:function(to,pos,dim){
+return true;
+},abort:function(){
 this.scrollTo(this.getPos());
 this.stopAnimation();
 this._aborted=true;
-};
-this.stopAnimation=function(){
+},stopAnimation:function(){
 _6.remove(this.containerNode,"mblScrollableScrollTo2");
-if(_9("android")){
-_8.set(this.containerNode,"webkitAnimationDuration","0s");
-}
 if(this._scrollBarV){
 this._scrollBarV.className="";
 }
 if(this._scrollBarH){
 this._scrollBarH.className="";
 }
-};
-this.getSpeed=function(){
+if(this._useTransformTransition||this._useTopLeft){
+this.containerNode.style.webkitTransition="";
+if(this._scrollBarV){
+this._scrollBarV.style.webkitTransition="";
+}
+if(this._scrollBarH){
+this._scrollBarH.style.webkitTransition="";
+}
+}
+},scrollIntoView:function(_23,_24,_25){
+if(!this._v){
+return;
+}
+var c=this.containerNode,h=this.getDim().d.h,top=0;
+for(var n=_23;n!==c;n=n.offsetParent){
+if(!n||n.tagName==="BODY"){
+return;
+}
+top+=n.offsetTop;
+}
+var y=_24?Math.max(h-c.offsetHeight,-top):Math.min(0,h-top-_23.offsetHeight);
+(_25&&typeof _25==="number")?this.slideTo({y:y},_25,"ease-out"):this.scrollTo({y:y});
+},getSpeed:function(){
 var x=0,y=0,n=this._time.length;
 if(n>=2&&(new Date()).getTime()-this.startTime-this._time[n-1]<500){
 var dy=this._posY[n-(n>3?2:1)]-this._posY[(n-6)>=0?n-6:0];
@@ -562,15 +505,26 @@ y=this.calcSpeed(dy,dt);
 x=this.calcSpeed(dx,dt);
 }
 return {x:x,y:y};
-};
-this.calcSpeed=function(d,t){
-return Math.round(d/t*100)*4;
-};
-this.scrollTo=function(to,_25,_26){
-var s=(_26||this.containerNode).style;
+},calcSpeed:function(_26,_27){
+return Math.round(_26/_27*100)*4;
+},scrollTo:function(to,_28,_29){
+var s=(_29||this.containerNode).style;
 if(_9("webkit")){
+if(!this._useTopLeft){
+if(this._useTransformTransition){
+s.webkitTransition="";
+}
 s.webkitTransform=this.makeTranslateStr(to);
 }else{
+s.webkitTransition="";
+if(this._v){
+s.top=to.y+"px";
+}
+if(this._h||this._f){
+s.left=to.x+"px";
+}
+}
+}else{
 if(this._v){
 s.top=to.y+"px";
 }
@@ -578,41 +532,41 @@ if(this._h||this._f){
 s.left=to.x+"px";
 }
 }
-if(!_25){
+if(!_28){
 this.scrollScrollBarTo(this.calcScrollBarPos(to));
 }
-};
-this.slideTo=function(to,_27,_28){
-this._runSlideAnimation(this.getPos(),to,_27,_28,this.containerNode,2);
-this.slideScrollBarTo(to,_27,_28);
-};
-this.makeTranslateStr=function(to){
+},slideTo:function(to,_2a,_2b){
+this._runSlideAnimation(this.getPos(),to,_2a,_2b,this.containerNode,2);
+this.slideScrollBarTo(to,_2a,_2b);
+},makeTranslateStr:function(to){
 var y=this._v&&typeof to.y=="number"?to.y+"px":"0px";
 var x=(this._h||this._f)&&typeof to.x=="number"?to.x+"px":"0px";
-return dm.hasTranslate3d?"translate3d("+x+","+y+",0px)":"translate("+x+","+y+")";
-};
-this.getPos=function(){
+return _9("translate3d")?"translate3d("+x+","+y+",0px)":"translate("+x+","+y+")";
+},getPos:function(){
 if(_9("webkit")){
-var m=_5.doc.defaultView.getComputedStyle(this.containerNode,"")["-webkit-transform"];
+var s=_5.doc.defaultView.getComputedStyle(this.containerNode,"");
+if(!this._useTopLeft){
+var m=s["-webkit-transform"];
 if(m&&m.indexOf("matrix")===0){
 var arr=m.split(/[,\s\)]+/);
 return {y:arr[5]-0,x:arr[4]-0};
 }
 return {x:0,y:0};
 }else{
+return {x:parseInt(s.left)||0,y:parseInt(s.top)||0};
+}
+}else{
 var y=parseInt(this.containerNode.style.top)||0;
 return {y:y,x:this.containerNode.offsetLeft};
 }
-};
-this.getDim=function(){
+},getDim:function(){
 var d={};
 d.c={h:this.containerNode.offsetHeight,w:this.containerNode.offsetWidth};
 d.v={h:this.domNode.offsetHeight+this._appFooterHeight,w:this.domNode.offsetWidth};
 d.d={h:d.v.h-this.fixedHeaderHeight-this.fixedFooterHeight,w:d.v.w};
 d.o={h:d.c.h-d.v.h+this.fixedHeaderHeight+this.fixedFooterHeight,w:d.c.w-d.v.w};
 return d;
-};
-this.showScrollBar=function(){
+},showScrollBar:function(){
 if(!this.scrollBar){
 return;
 }
@@ -626,54 +580,49 @@ return;
 if(this._v&&this._h&&dim.c.h<=dim.d.h&&dim.c.w<=dim.d.w){
 return;
 }
-var _29=function(_2a,dir){
-var bar=_2a["_scrollBarNode"+dir];
+var _2c=function(_2d,dir){
+var bar=_2d["_scrollBarNode"+dir];
 if(!bar){
-var _2b=_7.create("div",null,_2a.domNode);
-var _2c={position:"absolute",overflow:"hidden"};
+var _2e=_7.create("div",null,_2d.domNode);
+var _2f={position:"absolute",overflow:"hidden"};
 if(dir=="V"){
-_2c.right="2px";
-_2c.width="5px";
+_2f.right="2px";
+_2f.width="5px";
 }else{
-_2c.bottom=(_2a.isLocalFooter?_2a.fixedFooterHeight:0)+2+"px";
-_2c.height="5px";
+_2f.bottom=(_2d.isLocalFooter?_2d.fixedFooterHeight:0)+2+"px";
+_2f.height="5px";
 }
-_8.set(_2b,_2c);
-_2b.className="mblScrollBarWrapper";
-_2a["_scrollBarWrapper"+dir]=_2b;
-bar=_7.create("div",null,_2b);
+_8.set(_2e,_2f);
+_2e.className="mblScrollBarWrapper";
+_2d["_scrollBarWrapper"+dir]=_2e;
+bar=_7.create("div",null,_2e);
 _8.set(bar,{opacity:0.6,position:"absolute",backgroundColor:"#606060",fontSize:"1px",webkitBorderRadius:"2px",MozBorderRadius:"2px",webkitTransformOrigin:"0 0",zIndex:2147483647});
 _8.set(bar,dir=="V"?{width:"5px"}:{height:"5px"});
-_2a["_scrollBarNode"+dir]=bar;
+_2d["_scrollBarNode"+dir]=bar;
 }
 return bar;
 };
 if(this._v&&!this._scrollBarV){
-this._scrollBarV=_29(this,"V");
+this._scrollBarV=_2c(this,"V");
 }
 if(this._h&&!this._scrollBarH){
-this._scrollBarH=_29(this,"H");
+this._scrollBarH=_2c(this,"H");
 }
 this.resetScrollBar();
-};
-this.hideScrollBar=function(){
-var _2d;
+},hideScrollBar:function(){
 if(this.fadeScrollBar&&_9("webkit")){
 if(!dm._fadeRule){
-var _2e=_7.create("style",null,_5.doc.getElementsByTagName("head")[0]);
-_2e.textContent=".mblScrollableFadeScrollBar{"+"  -webkit-animation-duration: 1s;"+"  -webkit-animation-name: scrollableViewFadeScrollBar;}"+"@-webkit-keyframes scrollableViewFadeScrollBar{"+"  from { opacity: 0.6; }"+"  to { opacity: 0; }}";
-dm._fadeRule=_2e.sheet.cssRules[1];
+var _30=_7.create("style",null,_5.doc.getElementsByTagName("head")[0]);
+_30.textContent=".mblScrollableFadeScrollBar{"+"  -webkit-animation-duration: 1s;"+"  -webkit-animation-name: scrollableViewFadeScrollBar;}"+"@-webkit-keyframes scrollableViewFadeScrollBar{"+"  from { opacity: 0.6; }"+"  to { opacity: 0; }}";
+dm._fadeRule=_30.sheet.cssRules[1];
 }
-_2d=dm._fadeRule;
 }
 if(!this.scrollBar){
 return;
 }
-var f=function(bar,_2f){
+var f=function(bar,_31){
 _8.set(bar,{opacity:0,webkitAnimationDuration:""});
-if(_2f._aw){
-bar.style.webkitTransform="";
-}else{
+if(!(_31._useTopLeft&&_9("android"))){
 bar.className="mblScrollableFadeScrollBar";
 }
 };
@@ -685,17 +634,16 @@ if(this._scrollBarH){
 f(this._scrollBarH,this);
 this._scrollBarH=null;
 }
-};
-this.calcScrollBarPos=function(to){
+},calcScrollBarPos:function(to){
 var pos={};
 var dim=this._dim;
-var f=function(_30,_31,t,d,c){
-var y=Math.round((d-_31-8)/(d-c)*t);
-if(y<-_31+5){
-y=-_31+5;
+var f=function(_32,_33,t,d,c){
+var y=Math.round((d-_33-8)/(d-c)*t);
+if(y<-_33+5){
+y=-_33+5;
 }
-if(y>_30-5){
-y=_30-5;
+if(y>_32-5){
+y=_32-5;
 }
 return y;
 };
@@ -706,75 +654,113 @@ if(typeof to.x=="number"&&this._scrollBarH){
 pos.x=f(this._scrollBarWrapperH.offsetWidth,this._scrollBarH.offsetWidth,to.x,dim.d.w,dim.c.w);
 }
 return pos;
-};
-this.scrollScrollBarTo=function(to){
+},scrollScrollBarTo:function(to){
 if(!this.scrollBar){
 return;
 }
 if(this._v&&this._scrollBarV&&typeof to.y=="number"){
 if(_9("webkit")){
+if(!this._useTopLeft){
+if(this._useTransformTransition){
+this._scrollBarV.style.webkitTransition="";
+}
 this._scrollBarV.style.webkitTransform=this.makeTranslateStr({y:to.y});
 }else{
+_8.set(this._scrollBarV,{webkitTransition:"",top:to.y+"px"});
+}
+}else{
 this._scrollBarV.style.top=to.y+"px";
 }
 }
 if(this._h&&this._scrollBarH&&typeof to.x=="number"){
 if(_9("webkit")){
+if(!this._useTopLeft){
+if(this._useTransformTransition){
+this._scrollBarH.style.webkitTransition="";
+}
 this._scrollBarH.style.webkitTransform=this.makeTranslateStr({x:to.x});
 }else{
+_8.set(this._scrollBarH,{webkitTransition:"",left:to.x+"px"});
+}
+}else{
 this._scrollBarH.style.left=to.x+"px";
 }
 }
-};
-this.slideScrollBarTo=function(to,_32,_33){
+},slideScrollBarTo:function(to,_34,_35){
 if(!this.scrollBar){
 return;
 }
-var _34=this.calcScrollBarPos(this.getPos());
-var _35=this.calcScrollBarPos(to);
+var _36=this.calcScrollBarPos(this.getPos());
+var _37=this.calcScrollBarPos(to);
 if(this._v&&this._scrollBarV){
-this._runSlideAnimation({y:_34.y},{y:_35.y},_32,_33,this._scrollBarV,0);
+this._runSlideAnimation({y:_36.y},{y:_37.y},_34,_35,this._scrollBarV,0);
 }
 if(this._h&&this._scrollBarH){
-this._runSlideAnimation({x:_34.x},{x:_35.x},_32,_33,this._scrollBarH,1);
+this._runSlideAnimation({x:_36.x},{x:_37.x},_34,_35,this._scrollBarH,1);
 }
-};
-this._runSlideAnimation=function(_36,to,_37,_38,_39,idx){
+},_runSlideAnimation:function(_38,to,_39,_3a,_3b,idx){
 if(_9("webkit")){
-this.setKeyframes(_36,to,idx);
-_8.set(_39,{webkitAnimationDuration:_37+"s",webkitAnimationTimingFunction:_38});
-_6.add(_39,"mblScrollableScrollTo"+idx);
+if(!this._useTopLeft){
+if(this._useTransformTransition){
+if(to.x===undefined){
+to.x=_38.x;
+}
+if(to.y===undefined){
+to.y=_38.y;
+}
+if(to.x!==_38.x||to.y!==_38.y){
+_8.set(_3b,{webkitTransitionProperty:"-webkit-transform",webkitTransitionDuration:_39+"s",webkitTransitionTimingFunction:_3a});
+var t=this.makeTranslateStr(to);
+setTimeout(function(){
+_8.set(_3b,{webkitTransform:t});
+},0);
+_6.add(_3b,"mblScrollableScrollTo"+idx);
+}else{
+this.hideScrollBar();
+this.removeCover();
+}
+}else{
+this.setKeyframes(_38,to,idx);
+_8.set(_3b,{webkitAnimationDuration:_39+"s",webkitAnimationTimingFunction:_3a});
+_6.add(_3b,"mblScrollableScrollTo"+idx);
 if(idx==2){
-this.scrollTo(to,true,_39);
+this.scrollTo(to,true,_3b);
 }else{
 this.scrollScrollBarTo(to);
 }
+}
 }else{
-if(_b.fx&&_b.fx.easing&&_37){
-var s=_b.fx.slideTo({node:_39,duration:_37*1000,left:to.x,top:to.y,easing:(_38=="ease-out")?_b.fx.easing.quadOut:_b.fx.easing.linear}).play();
+_8.set(_3b,{webkitTransitionProperty:"top, left",webkitTransitionDuration:_39+"s",webkitTransitionTimingFunction:_3a});
+setTimeout(function(){
+_8.set(_3b,{top:(to.y||0)+"px",left:(to.x||0)+"px"});
+},0);
+_6.add(_3b,"mblScrollableScrollTo"+idx);
+}
+}else{
+if(_1.fx&&_1.fx.easing&&_39){
+var s=_1.fx.slideTo({node:_3b,duration:_39*1000,left:to.x,top:to.y,easing:(_3a=="ease-out")?_1.fx.easing.quadOut:_1.fx.easing.linear}).play();
 if(idx==2){
 _2.connect(s,"onEnd",this,"onFlickAnimationEnd");
 }
 }else{
 if(idx==2){
-this.scrollTo(to,false,_39);
+this.scrollTo(to,false,_3b);
 this.onFlickAnimationEnd();
 }else{
 this.scrollScrollBarTo(to);
 }
 }
 }
-};
-this.resetScrollBar=function(){
-var f=function(_3a,bar,d,c,hd,v){
+},resetScrollBar:function(){
+var f=function(_3c,bar,d,c,hd,v){
 if(!bar){
 return;
 }
-var _3b={};
-_3b[v?"top":"left"]=hd+4+"px";
+var _3d={};
+_3d[v?"top":"left"]=hd+4+"px";
 var t=(d-8)<=0?1:d-8;
-_3b[v?"height":"width"]=t+"px";
-_8.set(_3a,_3b);
+_3d[v?"height":"width"]=t+"px";
+_8.set(_3c,_3d);
 var l=Math.round(d*d/c);
 l=Math.min(Math.max(l-8,5),t);
 bar.style[v?"height":"width"]=l+"px";
@@ -784,8 +770,7 @@ var dim=this.getDim();
 f(this._scrollBarWrapperV,this._scrollBarV,dim.d.h,dim.c.h,this.fixedHeaderHeight,true);
 f(this._scrollBarWrapperH,this._scrollBarH,dim.d.w,dim.c.w,0);
 this.createMask();
-};
-this.createMask=function(){
+},createMask:function(){
 if(!_9("webkit")){
 return;
 }
@@ -814,8 +799,7 @@ ctx.fillStyle="rgb(0,0,0)";
 ctx.fillRect(2,0,w-4,5);
 this._scrollBarWrapperH.style.webkitMaskImage="-webkit-canvas(scrollBarMaskH)";
 }
-};
-this.flashScrollBar=function(){
+},flashScrollBar:function(){
 if(this.disableFlashScrollBar||!this.domNode){
 return;
 }
@@ -824,81 +808,69 @@ if(this._dim.d.h<=0){
 return;
 }
 this.showScrollBar();
-var _3c=this;
+var _3e=this;
 setTimeout(function(){
-_3c.hideScrollBar();
+_3e.hideScrollBar();
 },300);
-};
-this.addCover=function(){
+},addCover:function(){
 if(!_9("touch")&&!this.noCover){
-if(!this._cover){
-this._cover=_7.create("div",null,_5.doc.body);
-_8.set(this._cover,{backgroundColor:"#ffff00",opacity:0,position:"absolute",top:"0px",left:"0px",width:"100%",height:"100%",zIndex:2147483647});
-this._ch.push(_2.connect(this._cover,_9("touch")?"touchstart":"onmousedown",this,"onTouchEnd"));
+if(!dm._cover){
+dm._cover=_7.create("div",null,_5.doc.body);
+dm._cover.className="mblScrollableCover";
+_8.set(dm._cover,{backgroundColor:"#ffff00",opacity:0,position:"absolute",top:"0px",left:"0px",width:"100%",height:"100%",zIndex:2147483647});
+this._ch.push(_2.connect(dm._cover,_9("touch")?"ontouchstart":"onmousedown",this,"onTouchEnd"));
 }else{
-this._cover.style.display="";
+dm._cover.style.display="";
 }
-this.setSelectable(this._cover,false);
+this.setSelectable(dm._cover,false);
 this.setSelectable(this.domNode,false);
 }
-};
-this.removeCover=function(){
-if(!_9("touch")&&this._cover){
-this._cover.style.display="none";
-this.setSelectable(this._cover,true);
+},removeCover:function(){
+if(!_9("touch")&&dm._cover){
+dm._cover.style.display="none";
+this.setSelectable(dm._cover,true);
 this.setSelectable(this.domNode,true);
 }
-};
-this.setKeyframes=function(_3d,to,idx){
+},setKeyframes:function(_3f,to,idx){
 if(!dm._rule){
 dm._rule=[];
 }
 if(!dm._rule[idx]){
-var _3e=_7.create("style",null,_5.doc.getElementsByTagName("head")[0]);
-_3e.textContent=".mblScrollableScrollTo"+idx+"{-webkit-animation-name: scrollableViewScroll"+idx+";}"+"@-webkit-keyframes scrollableViewScroll"+idx+"{}";
-dm._rule[idx]=_3e.sheet.cssRules[1];
+var _40=_7.create("style",null,_5.doc.getElementsByTagName("head")[0]);
+_40.textContent=".mblScrollableScrollTo"+idx+"{-webkit-animation-name: scrollableViewScroll"+idx+";}"+"@-webkit-keyframes scrollableViewScroll"+idx+"{}";
+dm._rule[idx]=_40.sheet.cssRules[1];
 }
-var _3f=dm._rule[idx];
+var _41=dm._rule[idx];
+if(_41){
 if(_3f){
-if(_3d){
-_3f.deleteRule("from");
-_3f.insertRule("from { -webkit-transform: "+this.makeTranslateStr(_3d)+"; }");
+_41.deleteRule("from");
+_41.insertRule("from { -webkit-transform: "+this.makeTranslateStr(_3f)+"; }");
 }
 if(to){
 if(to.x===undefined){
-to.x=_3d.x;
+to.x=_3f.x;
 }
 if(to.y===undefined){
-to.y=_3d.y;
+to.y=_3f.y;
 }
-_3f.deleteRule("to");
-_3f.insertRule("to { -webkit-transform: "+this.makeTranslateStr(to)+"; }");
+_41.deleteRule("to");
+_41.insertRule("to { -webkit-transform: "+this.makeTranslateStr(to)+"; }");
 }
 }
-};
-this.setSelectable=function(_40,_41){
-_40.style.KhtmlUserSelect=_41?"auto":"none";
-_40.style.MozUserSelect=_41?"":"none";
-_40.onselectstart=_41?null:function(){
+},setSelectable:function(_42,_43){
+_42.style.KhtmlUserSelect=_43?"auto":"none";
+_42.style.MozUserSelect=_43?"":"none";
+_42.onselectstart=_43?null:function(){
 return false;
 };
 if(_9("ie")){
-_40.unselectable=_41?"":"on";
-var _42=_40.getElementsByTagName("*");
-for(var i=0;i<_42.length;i++){
-_42[i].unselectable=_41?"":"on";
+_42.unselectable=_43?"":"on";
+var _44=_42.getElementsByTagName("*");
+for(var i=0;i<_44.length;i++){
+_44[i].unselectable=_43?"":"on";
 }
 }
-};
-if(_9("webkit")){
-var _43=_5.doc.createElement("div");
-_43.style.webkitTransform="translate3d(0px,1px,0px)";
-_5.doc.documentElement.appendChild(_43);
-var v=_5.doc.defaultView.getComputedStyle(_43,"")["-webkit-transform"];
-dm.hasTranslate3d=v&&v.indexOf("matrix")===0;
-_5.doc.documentElement.removeChild(_43);
-}
-};
-dm.scrollable=_a;
-return _a;
+}});
+_4.setObject("dojox.mobile.scrollable",_c);
+return _c;
 });
