@@ -1,13 +1,16 @@
-//>>built
 define("dojox/mobile/transition", [
 	"dojo/_base/Deferred",
 	"dojo/_base/config"
 ], function(Deferred, config){
-	/* summary: this is the wrapper module which load
-	 * dojox/css3/transit conditionally. If mblCSS3Transition
-	 * is set to 'dojox/css3/transit', it will be loaded as
-	 * the module to conduct the view transition.
-	 */
+	/*=====
+	return {
+		// summary:
+		//		This is the wrapper module which loads
+		//		dojox/css3/transit conditionally. If mblCSS3Transition
+		//		is set to 'dojox/css3/transit', it will be loaded as
+		//		the module to conduct view transitions, otherwise this module returns null.
+	};
+	=====*/
 	if(config['mblCSS3Transition']){
 		//require dojox/css3/transit and resolve it as the result of transitDeferred.
 		var transitDeferred = new Deferred();
