@@ -1,6 +1,5 @@
 /*
- Copyright (c) 2011, Oracle and/or its affiliates. All rights
- reserved.
+ Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -20,8 +19,7 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- 02110-1301  USA
+ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
  */
 #ifndef NDBMEMCACHE_RECORD_H
 #define NDBMEMCACHE_RECORD_H
