@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2007, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2007, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -527,36 +527,28 @@ int delete_key(Ndb *myNdb)
   return 1;
 }
 
-
-int main(int argc, char**argv)
+void mysql_connect_and_create(const char *socket)
 {
-  if (argc != 3)
-  {
-    std::cout << "Arguments are <socket mysqld> <connect_string cluster>.\n";
-    exit(-1);
-  }
-  char *mysqld_sock  = argv[1];
-  const char *connectstring = argv[2];
-  ndb_init();
   MYSQL mysql;
+  bool ok;
 
-  /* Connect to mysql server and create table. */
-  {
-    if ( !mysql_init(&mysql) ) {
-      std::cout << "mysql_init failed.\n";
-      exit(-1);
-    }
-    if ( !mysql_real_connect(&mysql, "localhost", "root", "", "",
-                             0, mysqld_sock, 0) )
-      MYSQLERROR(mysql);
+  mysql_init(&mysql);
 
+  ok = mysql_real_connect(&mysql, "localhost", "root", "", "", 0, socket, 0);
+  if(ok) {
     mysql_query(&mysql, "CREATE DATABASE ndb_examples");
-    if (mysql_query(&mysql, "USE ndb_examples") != 0)
-      MYSQLERROR(mysql);
-
+    ok = ! mysql_select_db(&mysql, "ndb_examples");
+  }
+  if(ok) {
     create_table(mysql);
   }
+  mysql_close(&mysql);
 
+  if(! ok) MYSQLERROR(mysql);
+}
+
+void ndb_run_ndbrecord_blob_operations(const char * connectstring)
+{
   /* Connect to ndb cluster. */
 
   Ndb_cluster_connection cluster_connection(connectstring);
@@ -597,6 +589,23 @@ int main(int argc, char**argv)
 
   if(delete_key(&myNdb) > 0)
     std::cout << "delete_key: Success!" << std::endl;
+}
+
+int main(int argc, char**argv)
+{
+  if (argc != 3)
+  {
+    std::cout << "Arguments are <socket mysqld> <connect_string cluster>.\n";
+    exit(-1);
+  }
+  char *mysqld_sock  = argv[1];
+  const char *connectstring = argv[2];
+
+  mysql_connect_and_create(mysqld_sock);
+
+  ndb_init();
+  ndb_run_ndbrecord_blob_operations(connectstring);
+  ndb_end(0);
 
   return 0;
 }
