@@ -1,15 +1,21 @@
 /*
-   Copyright (C) 2003-2008 MySQL AB, 2009 Sun Microsystems, Inc.
-    All rights reserved. Use is subject to license terms.
+   Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -44,12 +50,6 @@ printCONTINUEB_DBDIH(FILE * output, const Uint32 * theData,
 	    theData[1], theData[2], theData[3], theData[4]);
     return true;
     break;
-#if 0
-  case DihContinueB::ZREAD_TAB_DESCRIPTION:
-    fprintf(output, " Read Table description: %d\n", theData[1]);
-    return true;
-    break;
-#endif
   case DihContinueB::ZCOPY_TABLE:
     fprintf(output, " Copy Table: %d\n", theData[1]);
     return true;
@@ -155,6 +155,14 @@ printCONTINUEB_DBDIH(FILE * output, const Uint32 * theData,
     fprintf(output, " To Start Copy Frag: TakeOverPtr: %d\n", theData[1]);
     return true;
     break;
+  case DihContinueB::ZTO_START_LOGGING:
+    fprintf(output, " To Start REDO logging: TakeOverPtr: %d\n", theData[1]);
+    return true;
+    break;
+  case DihContinueB::ZTO_START_FRAGMENTS:
+    fprintf(output, " To Start Fragments: TakeOverPtr: %d\n", theData[1]);
+    return true;
+    break;
   case DihContinueB::ZINVALIDATE_NODE_LCP:
     fprintf(output, " Invalide LCP: NodeId: %d TableId %d\n",
             theData[1], theData[2]);
@@ -195,6 +203,18 @@ printCONTINUEB_DBDIH(FILE * output, const Uint32 * theData,
     fprintf(output, " Lcp trylock: attempt %u\n",
             theData[1]);
     break;
+  case DihContinueB::ZWAIT_OLD_SCAN:
+    fprintf(output, " Wait old scan\n");
+    break;
+  case DihContinueB::ZDEQUEUE_LCP_REP:
+    fprintf(output, " Get Dequeue LCP report\n");
+    break;
+  case DihContinueB::ZGET_TABINFO:
+    fprintf(output, " Get Tabinfo\n");
+    break;
+  case DihContinueB::ZGET_TABINFO_SEND:
+    fprintf(output, " Get Tabinfo Send\n");
+    break;
   default:
     fprintf(output, " Default system error lab...\n");
     break;
