@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2014  Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2014, 2019 Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -33,12 +40,18 @@ printISOLATE_ORD(FILE * output, const Uint32 * theData, Uint32 len, Uint16 recei
            "??"),
           sig->delayMillis);
   
-  for (Uint32 i=0; i < NdbNodeBitmask::Size; i++)
+  if (len == sig->SignalLengthWithBitmask48)
   {
-    fprintf(output, " %x", sig->nodesToIsolate[i]);
+    for (Uint32 i=0; i < NdbNodeBitmask48::Size; i++)
+    {
+      fprintf(output, " %x", sig->nodesToIsolate[i]);
+    }
+    fprintf(output, "\n");
+  }
+  else
+  {
+    fprintf(output, " nodesToIsolate in signal section");
   }
-  fprintf(output, "\n");
-
   return true;
 }
 
