@@ -1,6 +1,5 @@
 /*
-   Copyright (C) 2004-2006 MySQL AB
-    Use is subject to license terms.
+   Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,17 +32,6 @@ printREAD_NODES_CONF(FILE * output, const Uint32 * theData,
   fprintf(output, " ndynamicId: %x\n", sig->ndynamicId);
   fprintf(output, " masterNodeId: %x\n", sig->masterNodeId);
 
-  char buf[32*NdbNodeBitmask::Size+1];
-  fprintf(output, " allNodes(defined): %s\n", 
-	  BitmaskImpl::getText(NdbNodeBitmask::Size, sig->allNodes, buf));
-  fprintf(output, " inactiveNodes: %s\n", 
-	  BitmaskImpl::getText(NdbNodeBitmask::Size, sig->inactiveNodes, buf));
-  fprintf(output, " clusterNodes: %s\n", 
-	  BitmaskImpl::getText(NdbNodeBitmask::Size, sig->clusterNodes, buf));
-  fprintf(output, " startedNodes: %s\n", 
-	  BitmaskImpl::getText(NdbNodeBitmask::Size, sig->startedNodes, buf));
-  fprintf(output, " startingNodes: %s\n", 
-	  BitmaskImpl::getText(NdbNodeBitmask::Size, sig->startingNodes, buf));
   return true;
 }
 
