@@ -1,18 +1,25 @@
-/* Copyright (c) 2003, 2005, 2006 MySQL AB
-   Use is subject to license terms
+/* Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #include <signaldata/SumaImpl.hpp>
 
@@ -26,6 +33,7 @@ printSUB_CREATE_REQ(FILE * output, const Uint32 * theData,
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " subscriptionType: %x\n", sig->subscriptionType);
   fprintf(output, " tableId: %x\n", sig->tableId);
+  fprintf(output, " schemaTransId: %x\n", sig->schemaTransId);
   return false;
 }
 
@@ -149,6 +157,10 @@ printSUB_SYNC_REQ(FILE * output, const Uint32 * theData,
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " syncPart: %x\n", sig->part);
+  fprintf(output, " requestInfo : %x\n", sig->requestInfo);
+  fprintf(output, " fragCount : %u\n", sig->fragCount);
+  fprintf(output, " fragId : %u\n", sig->fragId);
+  fprintf(output, " batchSize : %u\n", sig->batchSize); 
   return false;
 }
 
@@ -174,10 +186,16 @@ printSUB_TABLE_DATA(FILE * output, const Uint32 * theData,
 		    Uint32 len, Uint16 receiverBlockNo) {
   const SubTableData * const sig = (SubTableData *)theData;
   fprintf(output, " senderData: %x\n", sig->senderData);
-  fprintf(output, " gci: %x\n", sig->gci);
+  fprintf(output, " gci_hi: %x\n", sig->gci_hi);
+  fprintf(output, " gci_lo: %x\n", sig->gci_lo);
   fprintf(output, " tableId: %x\n", sig->tableId);
   fprintf(output, " operation: %x\n", 
 	  SubTableData::getOperation(sig->requestInfo));
+  if (len == SubTableData::SignalLengthWithTransId)
+  {
+    fprintf(output, " TransId : %x %x\n",
+            sig->transId1, sig->transId2);
+  }
   return false;
 }
 
@@ -212,7 +230,7 @@ bool
 printSUB_GCP_COMPLETE_REP(FILE * output, const Uint32 * theData, 
 			  Uint32 len, Uint16 receiverBlockNo) {
   const SubGcpCompleteRep * const sig = (SubGcpCompleteRep *)theData;
-  fprintf(output, " gci: %x\n", sig->gci);
+  fprintf(output, " gci_hi: %x gci_lo: %x\n", sig->gci_hi, sig->gci_lo);
   return false;
 }
 
