@@ -1,6 +1,4 @@
-/*
- Copyright (c) 2010, 2021, Oracle and/or its affiliates.
- All rights reserved. Use is subject to license terms.
+/* Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -23,17 +21,11 @@
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-/*
- *  CharsetMapImpl.cpp
- *
-*/
-
 #include "CharsetMapImpl.h"
 
-#include <string.h> // not using namespaces yet
+#include <string.h>
 
-#include "my_global.h"
-#include "mysql.h"
+#include "m_ctype.h"
 #include "my_sys.h"
 
 #define MYSQL_BINARY_CHARSET 63
@@ -96,10 +88,10 @@ void CharsetMapImpl::build_map()
     put("macce", "MacCentralEurope");
     
     /* Build the fixed map */
-    for(unsigned int i = 0 ; i < 255 ; i++) 
+    for(unsigned int i = 0 ; i < NDB_ARRAY_SIZE(mysql_charset_name) ; i++)
     {
         CHARSET_INFO *cs = get_charset(i, MYF(0));
-        register const char *mysql_name = 0;
+        const char *mysql_name = 0;
         const char *mapped_name = 0;
         
         if(cs) 
@@ -139,7 +131,7 @@ void CharsetMapImpl::build_map()
 
 const char * CharsetMapImpl::getName(int csnum)  
 {
-    if((csnum > 255) || (csnum < 0)) 
+    if((csnum >= (int)NDB_ARRAY_SIZE(mysql_charset_name)) || (csnum < 0))
     {
         return 0;
     }
