@@ -30,9 +30,12 @@
 #ifdef DBINFO_SCAN_TRACE
 #include <debugger/DebuggerNames.hpp>
 #endif
+#include <NdbGetRUsage.h>
+#include <EventLogger.hpp>
 
 #define JAM_FILE_ID 437
 
+extern EventLogger *g_eventLogger;
 
 LocalProxy::LocalProxy(BlockNumber blockNumber, Block_context& ctx) :
   SimulatedBlock(blockNumber, ctx)
@@ -126,6 +129,7 @@ LocalProxy::~LocalProxy()
 void
 LocalProxy::sendREQ(Signal* signal, SsSequential& ss)
 {
+  jam();
   ss.m_worker = 0;
   ndbrequire(ss.m_sendREQ != 0);
   SectionHandle handle(this);
@@ -137,6 +141,7 @@ LocalProxy::sendREQ(Signal* signal, SsSequential& ss)
 void
 LocalProxy::recvCONF(Signal* signal, SsSequential& ss)
 {
+  jam();
   ndbrequire(ss.m_sendCONF != 0);
   (this->*ss.m_sendCONF)(signal, ss.m_ssId);
 
@@ -153,28 +158,38 @@ LocalProxy::recvCONF(Signal* signal, SsSequential& ss)
 void
 LocalProxy::recvREF(Signal* signal, SsSequential& ss, Uint32 error)
 {
+  jam();
   ndbrequire(error != 0);
   if (ss.m_error == 0)
+  {
+    jam();
     ss.m_error = error;
+  }
   recvCONF(signal, ss);
 }
 
 void
 LocalProxy::skipReq(SsSequential& ss)
 {
+  jam();
 }
 
 void
 LocalProxy::skipConf(SsSequential& ss)
 {
+  jam();
 }
 
 void
 LocalProxy::saveSections(SsCommon& ss, SectionHandle & handle)
 {
+  jam();
   ss.m_sec_cnt = handle.m_cnt;
   for (Uint32 i = 0; i<ss.m_sec_cnt; i++)
+  {
+    jam();
     ss.m_sec_ptr[i] = handle.m_ptr[i].i;
+  }
   handle.clear();
 }
 
@@ -183,7 +198,10 @@ LocalProxy::restoreHandle(SectionHandle & handle, SsCommon& ss)
 {
   handle.m_cnt = ss.m_sec_cnt;
   for (Uint32 i = 0; i<ss.m_sec_cnt; i++)
+  {
+    jam();
     handle.m_ptr[i].i = ss.m_sec_ptr[i];
+  }
 
   getSections(handle.m_cnt, handle.m_ptr);
   ss.m_sec_cnt = 0;
@@ -204,6 +222,7 @@ LocalProxy::lastReply(const SsSequential& ss)
 void
 LocalProxy::sendREQ(Signal* signal, SsParallel& ss, bool skipLast)
 {
+  jam();
   ndbrequire(ss.m_sendREQ != 0);
 
   ss.m_workerMask.clear();
@@ -223,6 +242,7 @@ LocalProxy::sendREQ(Signal* signal, SsParallel& ss, bool skipLast)
 void
 LocalProxy::recvCONF(Signal* signal, SsParallel& ss)
 {
+  jam();
   ndbrequire(ss.m_sendCONF != 0);
 
   BlockReference ref = signal->getSendersBlockRef();
@@ -241,15 +261,20 @@ LocalProxy::recvCONF(Signal* signal, SsParallel& ss)
 void
 LocalProxy::recvREF(Signal* signal, SsParallel& ss, Uint32 error)
 {
+  jam();
   ndbrequire(error != 0);
   if (ss.m_error == 0)
+  {
+    jam();
     ss.m_error = error;
+  }
   recvCONF(signal, ss);
 }
 
 void
 LocalProxy::skipReq(SsParallel& ss)
 {
+  jam();
   ndbrequire(ss.m_workerMask.get(ss.m_worker));
   ss.m_workerMask.clear(ss.m_worker);
 }
@@ -258,6 +283,7 @@ LocalProxy::skipReq(SsParallel& ss)
 void
 LocalProxy::skipConf(SsParallel& ss)
 {
+  jam();
   ndbrequire(!ss.m_workerMask.get(ss.m_worker));
   ss.m_workerMask.set(ss.m_worker);
 }
@@ -286,8 +312,12 @@ void
 LocalProxy::setMask(SsParallel& ss)
 {
   Uint32 i;
+  jam();
   for (i = 0; i < c_workers; i++)
+  {
+    jam();
     ss.m_workerMask.set(i);
+  }
 }
 
 void
@@ -356,6 +386,7 @@ LocalProxy::forwardToAnyWorker(Signal* signal)
 void
 LocalProxy::execREAD_CONFIG_REQ(Signal* signal)
 {
+  jam();
   Ss_READ_CONFIG_REQ& ss = ssSeize<Ss_READ_CONFIG_REQ>(1);
 
   const ReadConfigReq* req = (const ReadConfigReq*)signal->getDataPtr();
@@ -367,12 +398,14 @@ LocalProxy::execREAD_CONFIG_REQ(Signal* signal)
 void
 LocalProxy::callREAD_CONFIG_REQ(Signal* signal)
 {
+  jam();
   backREAD_CONFIG_REQ(signal);
 }
 
 void
 LocalProxy::backREAD_CONFIG_REQ(Signal* signal)
 {
+  jam();
   Ss_READ_CONFIG_REQ& ss = ssFind<Ss_READ_CONFIG_REQ>(1);
 
   // run sequentially due to big mallocs and initializations
@@ -383,6 +416,7 @@ void
 LocalProxy::sendREAD_CONFIG_REQ(Signal* signal, Uint32 ssId,
                                 SectionHandle* handle)
 {
+  jam();
   Ss_READ_CONFIG_REQ& ss = ssFind<Ss_READ_CONFIG_REQ>(ssId);
 
   ReadConfigReq* req = (ReadConfigReq*)signal->getDataPtrSend();
@@ -396,19 +430,41 @@ LocalProxy::sendREAD_CONFIG_REQ(Signal* signal, Uint32 ssId,
 void
 LocalProxy::execREAD_CONFIG_CONF(Signal* signal)
 {
+  jam();
   const ReadConfigConf* conf = (const ReadConfigConf*)signal->getDataPtr();
   Uint32 ssId = conf->senderData;
   Ss_READ_CONFIG_REQ& ss = ssFind<Ss_READ_CONFIG_REQ>(ssId);
+
+#ifdef DEBUG_RSS
+  {
+    ndb_rusage ru;
+    if (Ndb_GetRUsage(&ru, true) != 0)
+    {
+      g_eventLogger->error("LocalProxy : Failed to get rusage");
+    }
+    else
+    {
+      g_eventLogger->info("LocalProxy (conf from worker %u/%u) : RSS : %llu kB",
+                          ss.m_worker,
+                          c_workers,
+                          ru.ru_rss);
+    }
+  }
+#endif
   recvCONF(signal, ss);
 }
 
 void
 LocalProxy::sendREAD_CONFIG_CONF(Signal* signal, Uint32 ssId)
 {
+  jam();
   Ss_READ_CONFIG_REQ& ss = ssFind<Ss_READ_CONFIG_REQ>(ssId);
 
   if (!lastReply(ss))
+  {
+    jam();
     return;
+  }
 
   SectionHandle handle(this);
   restoreHandle(handle, ss);
@@ -428,12 +484,14 @@ LocalProxy::sendREAD_CONFIG_CONF(Signal* signal, Uint32 ssId)
 void
 LocalProxy::execSTTOR(Signal* signal)
 {
+  jam();
   Ss_STTOR& ss = ssSeize<Ss_STTOR>(1);
 
   const Uint32 startphase  = signal->theData[1];
   const Uint32 typeOfStart = signal->theData[7];
 
-  if (startphase == 3) {
+  if (startphase == 3)
+  {
     jam();
     c_typeOfStart = typeOfStart;
   }
@@ -447,12 +505,14 @@ LocalProxy::execSTTOR(Signal* signal)
 void
 LocalProxy::callSTTOR(Signal* signal)
 {
+  jam();
   backSTTOR(signal);
 }
 
 void
 LocalProxy::backSTTOR(Signal* signal)
 {
+  jam();
   Ss_STTOR& ss = ssFind<Ss_STTOR>(1);
   sendREQ(signal, ss);
 }
@@ -460,6 +520,7 @@ LocalProxy::backSTTOR(Signal* signal)
 void
 LocalProxy::sendSTTOR(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
+  jam();
   Ss_STTOR& ss = ssFind<Ss_STTOR>(ssId);
 
   memcpy(signal->getDataPtrSend(), ss.m_reqdata, ss.m_reqlength << 2);
@@ -470,6 +531,7 @@ LocalProxy::sendSTTOR(Signal* signal, Uint32 ssId, SectionHandle* handle)
 void
 LocalProxy::execSTTORRY(Signal* signal)
 {
+  jam();
   Ss_STTOR& ss = ssFind<Ss_STTOR>(1);
   recvCONF(signal, ss);
 }
@@ -477,22 +539,31 @@ LocalProxy::execSTTORRY(Signal* signal)
 void
 LocalProxy::sendSTTORRY(Signal* signal, Uint32 ssId)
 {
+  jam();
   Ss_STTOR& ss = ssFind<Ss_STTOR>(ssId);
 
   const Uint32 conflength = signal->getLength();
   const Uint32* confdata = signal->getDataPtr();
 
   // the reply is identical from all
-  if (firstReply(ss)) {
+  if (firstReply(ss))
+  {
+    jam();
     ss.m_conflength = conflength;
     memcpy(ss.m_confdata, confdata, conflength << 2);
-  } else {
+  }
+  else
+  {
+    jam();
     ndbrequire(ss.m_conflength == conflength);
     ndbrequire(memcmp(ss.m_confdata, confdata, conflength << 2) == 0);
   }
 
   if (!lastReply(ss))
+  {
+    jam();
     return;
+  }
 
   memcpy(signal->getDataPtrSend(), ss.m_confdata, ss.m_conflength << 2);
   sendSignal(NDBCNTR_REF, GSN_STTORRY,
@@ -506,6 +577,7 @@ LocalProxy::sendSTTORRY(Signal* signal, Uint32 ssId)
 void
 LocalProxy::execNDB_STTOR(Signal* signal)
 {
+  jam();
   Ss_NDB_STTOR& ss = ssSeize<Ss_NDB_STTOR>(1);
 
   const NdbSttor* req = (const NdbSttor*)signal->getDataPtr();
@@ -517,12 +589,14 @@ LocalProxy::execNDB_STTOR(Signal* signal)
 void
 LocalProxy::callNDB_STTOR(Signal* signal)
 {
+  jam();
   backNDB_STTOR(signal);
 }
 
 void
 LocalProxy::backNDB_STTOR(Signal* signal)
 {
+  jam();
   Ss_NDB_STTOR& ss = ssFind<Ss_NDB_STTOR>(1);
   sendREQ(signal, ss);
 }
@@ -530,6 +604,7 @@ LocalProxy::backNDB_STTOR(Signal* signal)
 void
 LocalProxy::sendNDB_STTOR(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
+  jam();
   Ss_NDB_STTOR& ss = ssFind<Ss_NDB_STTOR>(ssId);
 
   NdbSttor* req = (NdbSttor*)signal->getDataPtrSend();
@@ -542,6 +617,7 @@ LocalProxy::sendNDB_STTOR(Signal* signal, Uint32 ssId, SectionHandle* handle)
 void
 LocalProxy::execNDB_STTORRY(Signal* signal)
 {
+  jam();
   Ss_NDB_STTOR& ss = ssFind<Ss_NDB_STTOR>(1);
 
   // the reply contains only senderRef
@@ -553,10 +629,14 @@ LocalProxy::execNDB_STTORRY(Signal* signal)
 void
 LocalProxy::sendNDB_STTORRY(Signal* signal, Uint32 ssId)
 {
+  jam();
   Ss_NDB_STTOR& ss = ssFind<Ss_NDB_STTOR>(ssId);
 
   if (!lastReply(ss))
+  {
+    jam();
     return;
+  }
 
   NdbSttorry* conf = (NdbSttorry*)signal->getDataPtrSend();
   conf->senderRef = reference();
@@ -571,6 +651,7 @@ LocalProxy::sendNDB_STTORRY(Signal* signal, Uint32 ssId)
 void
 LocalProxy::sendREAD_NODESREQ(Signal* signal)
 {
+  jam();
   signal->theData[0] = reference();
   sendSignal(NDBCNTR_REF, GSN_READ_NODESREQ, signal, 1, JBB);
 }
@@ -578,17 +659,28 @@ LocalProxy::sendREAD_NODESREQ(Signal* signal)
 void
 LocalProxy::execREAD_NODESCONF(Signal* signal)
 {
+  jam();
   Ss_READ_NODES_REQ& ss = c_ss_READ_NODESREQ;
 
   const ReadNodesConf* conf = (const ReadNodesConf*)signal->getDataPtr();
 
   c_masterNodeId = conf->masterNodeId;
 
+  /**
+   * READ_NODESCONF comes with a section containing a bitmap, since the
+   * the proxy block didn't have its own method to receive this, it isn't
+   * interested in the contents of this. So can simply release it.
+   */
+  SectionHandle handle(this, signal);
+  releaseSections(handle);
+
   switch (ss.m_gsn) {
   case GSN_STTOR:
+    jam();
     backSTTOR(signal);
     break;
   case GSN_NDB_STTOR:
+    jam();
     backNDB_STTOR(signal);
     break;
   default:
@@ -601,6 +693,7 @@ LocalProxy::execREAD_NODESCONF(Signal* signal)
 void
 LocalProxy::execREAD_NODESREF(Signal* signal)
 {
+  jam();
   Ss_READ_NODES_REQ& ss = c_ss_READ_NODESREQ;
   ndbrequire(ss.m_gsn != 0);
   ndbabort();
@@ -611,11 +704,30 @@ LocalProxy::execREAD_NODESREF(Signal* signal)
 void
 LocalProxy::execNODE_FAILREP(Signal* signal)
 {
+  jam();
   Ss_NODE_FAILREP& ss = ssFindSeize<Ss_NODE_FAILREP>(1, 0);
-  const NodeFailRep* req = (const NodeFailRep*)signal->getDataPtr();
-  ss.m_req = *req;
-  ndbrequire(signal->getLength() == NodeFailRep::SignalLength);
+  NodeFailRep* req = (NodeFailRep*)signal->getDataPtr();
+  ndbrequire(signal->getLength() == NodeFailRep::SignalLength ||
+             signal->getLength() == NodeFailRep::SignalLength_v1);
 
+  if(signal->getLength() == NodeFailRep::SignalLength)
+  {
+    ndbrequire(signal->getNoOfSections() == 1);
+    ndbrequire(getNodeInfo(refToNode(signal->getSendersBlockRef())).m_version);
+    SegmentedSectionPtr ptr;
+    SectionHandle handle(this, signal);
+    handle.getSection(ptr, 0);
+    memset(req->theNodes, 0 ,sizeof(req->theNodes));
+    copy(req->theNodes, ptr);
+    releaseSections(handle);
+  }
+  else
+  {
+    memset(req->theNodes + NdbNodeBitmask48::Size,
+           0,
+           _NDB_NBM_DIFF_BYTES);
+  }
+  ss.m_req = *req;
   NdbNodeBitmask mask;
   mask.assign(NdbNodeBitmask::Size, req->theNodes);
 
@@ -638,10 +750,17 @@ LocalProxy::execNODE_FAILREP(Signal* signal)
 void
 LocalProxy::sendNODE_FAILREP(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
+  jam();
   Ss_NODE_FAILREP& ss = ssFind<Ss_NODE_FAILREP>(ssId);
 
   NodeFailRep* req = (NodeFailRep*)signal->getDataPtrSend();
   *req = ss.m_req;
+  handle->clear();
+  LinearSectionPtr lsptr[3];
+  lsptr[0].p = req->theNodes;
+  lsptr[0].sz = NdbNodeBitmask::getPackedLengthInWords(req->theNodes);
+  ndbrequire(import(handle->m_ptr[0], lsptr[0].p, lsptr[0].sz));
+  handle->m_cnt = 1;
   sendSignalNoRelease(workerRef(ss.m_worker), GSN_NODE_FAILREP,
                       signal, NodeFailRep::SignalLength, JBB, handle);
 }
@@ -649,6 +768,7 @@ LocalProxy::sendNODE_FAILREP(Signal* signal, Uint32 ssId, SectionHandle* handle)
 void
 LocalProxy::execNF_COMPLETEREP(Signal* signal)
 {
+  jam();
   Ss_NODE_FAILREP& ss = ssFind<Ss_NODE_FAILREP>(1);
   ndbrequire(!ss.noReply(number()));
   ss.m_workerMask.set(ss.m_worker); // Avoid require in recvCONF
@@ -658,6 +778,7 @@ LocalProxy::execNF_COMPLETEREP(Signal* signal)
 void
 LocalProxy::sendNF_COMPLETEREP(Signal* signal, Uint32 ssId)
 {
+  jam();
   Ss_NODE_FAILREP& ss = ssFind<Ss_NODE_FAILREP>(ssId);
 
   const NFCompleteRep* conf = (const NFCompleteRep*)signal->getDataPtr();
@@ -714,6 +835,7 @@ LocalProxy::sendNF_COMPLETEREP(Signal* signal, Uint32 ssId)
 void
 LocalProxy::execINCL_NODEREQ(Signal* signal)
 {
+  jam();
   Ss_INCL_NODEREQ& ss = ssSeize<Ss_INCL_NODEREQ>(1);
 
   ss.m_reqlength = signal->getLength();
@@ -726,6 +848,7 @@ LocalProxy::execINCL_NODEREQ(Signal* signal)
 void
 LocalProxy::sendINCL_NODEREQ(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
+  jam();
   Ss_INCL_NODEREQ& ss = ssFind<Ss_INCL_NODEREQ>(ssId);
 
   Ss_INCL_NODEREQ::Req* req =
@@ -740,6 +863,7 @@ LocalProxy::sendINCL_NODEREQ(Signal* signal, Uint32 ssId, SectionHandle* handle)
 void
 LocalProxy::execINCL_NODECONF(Signal* signal)
 {
+  jam();
   Ss_INCL_NODEREQ& ss = ssFind<Ss_INCL_NODEREQ>(1);
   recvCONF(signal, ss);
 }
@@ -747,10 +871,14 @@ LocalProxy::execINCL_NODECONF(Signal* signal)
 void
 LocalProxy::sendINCL_NODECONF(Signal* signal, Uint32 ssId)
 {
+  jam();
   Ss_INCL_NODEREQ& ss = ssFind<Ss_INCL_NODEREQ>(ssId);
 
   if (!lastReply(ss))
+  {
+    jam();
     return;
+  }
 
   Ss_INCL_NODEREQ::Conf* conf =
     (Ss_INCL_NODEREQ::Conf*)signal->getDataPtrSend();
@@ -768,6 +896,7 @@ LocalProxy::sendINCL_NODECONF(Signal* signal, Uint32 ssId)
 void
 LocalProxy::execNODE_STATE_REP(Signal* signal)
 {
+  jam();
   Ss_NODE_STATE_REP& ss = ssSeize<Ss_NODE_STATE_REP>();
   sendREQ(signal, ss);
   SimulatedBlock::execNODE_STATE_REP(signal);
@@ -778,6 +907,7 @@ void
 LocalProxy::sendNODE_STATE_REP(Signal* signal, Uint32 ssId,
                                SectionHandle* handle)
 {
+  jam();
   Ss_NODE_STATE_REP& ss = ssFind<Ss_NODE_STATE_REP>(ssId);
 
   sendSignalNoRelease(workerRef(ss.m_worker), GSN_NODE_STATE_REP,
@@ -789,6 +919,7 @@ LocalProxy::sendNODE_STATE_REP(Signal* signal, Uint32 ssId,
 void
 LocalProxy::execCHANGE_NODE_STATE_REQ(Signal* signal)
 {
+  jam();
   Ss_CHANGE_NODE_STATE_REQ& ss = ssSeize<Ss_CHANGE_NODE_STATE_REQ>(1);
 
   ChangeNodeStateReq * req = (ChangeNodeStateReq*)signal->getDataPtrSend();
@@ -801,6 +932,7 @@ void
 LocalProxy::sendCHANGE_NODE_STATE_REQ(Signal* signal, Uint32 ssId,
                                       SectionHandle* handle)
 {
+  jam();
   Ss_CHANGE_NODE_STATE_REQ& ss = ssFind<Ss_CHANGE_NODE_STATE_REQ>(ssId);
 
   ChangeNodeStateReq * req = (ChangeNodeStateReq*)signal->getDataPtrSend();
@@ -813,6 +945,7 @@ LocalProxy::sendCHANGE_NODE_STATE_REQ(Signal* signal, Uint32 ssId,
 void
 LocalProxy::execCHANGE_NODE_STATE_CONF(Signal* signal)
 {
+  jam();
   Ss_CHANGE_NODE_STATE_REQ& ss = ssFind<Ss_CHANGE_NODE_STATE_REQ>(1);
 
   ChangeNodeStateConf * conf = (ChangeNodeStateConf*)signal->getDataPtrSend();
@@ -823,10 +956,14 @@ LocalProxy::execCHANGE_NODE_STATE_CONF(Signal* signal)
 void
 LocalProxy::sendCHANGE_NODE_STATE_CONF(Signal* signal, Uint32 ssId)
 {
+  jam();
   Ss_CHANGE_NODE_STATE_REQ& ss = ssFind<Ss_CHANGE_NODE_STATE_REQ>(ssId);
 
   if (!lastReply(ss))
+  {
+    jam();
     return;
+  }
 
   /**
    * SimulatedBlock::execCHANGE_NODE_STATE_REQ will reply
@@ -842,6 +979,7 @@ LocalProxy::sendCHANGE_NODE_STATE_CONF(Signal* signal, Uint32 ssId)
 void
 LocalProxy::execDUMP_STATE_ORD(Signal* signal)
 {
+  jam();
   Ss_DUMP_STATE_ORD& ss = ssSeize<Ss_DUMP_STATE_ORD>();
 
   ss.m_reqlength = signal->getLength();
@@ -854,6 +992,7 @@ void
 LocalProxy::sendDUMP_STATE_ORD(Signal* signal, Uint32 ssId,
                                SectionHandle* handle)
 {
+  jam();
   Ss_DUMP_STATE_ORD& ss = ssFind<Ss_DUMP_STATE_ORD>(ssId);
 
   memcpy(signal->getDataPtrSend(), ss.m_reqdata, ss.m_reqlength << 2);
@@ -866,16 +1005,19 @@ LocalProxy::sendDUMP_STATE_ORD(Signal* signal, Uint32 ssId,
 void
 LocalProxy::execNDB_TAMPER(Signal* signal)
 {
+  jam();
   Ss_NDB_TAMPER& ss = ssSeize<Ss_NDB_TAMPER>();
 
   const Uint32 siglen = signal->getLength();
   if (siglen == 1)
   {
+    jam();
     ss.m_errorInsert = signal->theData[0];
     ss.m_haveErrorInsertExtra = false;
   }
   else
   {
+    jam();
     ndbrequire(siglen == 2);
     ss.m_errorInsert = signal->theData[0];
     ss.m_haveErrorInsertExtra = true;
@@ -890,12 +1032,14 @@ LocalProxy::execNDB_TAMPER(Signal* signal)
 void
 LocalProxy::sendNDB_TAMPER(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
+  jam();
   Ss_NDB_TAMPER& ss = ssFind<Ss_NDB_TAMPER>(ssId);
 
   Uint32 siglen = 1;
   signal->theData[0] = ss.m_errorInsert;
   if (ss.m_haveErrorInsertExtra)
   {
+    jam();
     signal->theData[1] = ss.m_errorInsertExtra;
     siglen ++;
   }
@@ -908,6 +1052,7 @@ LocalProxy::sendNDB_TAMPER(Signal* signal, Uint32 ssId, SectionHandle* handle)
 void
 LocalProxy::execTIME_SIGNAL(Signal* signal)
 {
+  jam();
   Ss_TIME_SIGNAL& ss = ssSeize<Ss_TIME_SIGNAL>();
 
   sendREQ(signal, ss);
@@ -917,6 +1062,7 @@ LocalProxy::execTIME_SIGNAL(Signal* signal)
 void
 LocalProxy::sendTIME_SIGNAL(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
+  jam();
   Ss_TIME_SIGNAL& ss = ssFind<Ss_TIME_SIGNAL>(ssId);
   signal->theData[0] = 0;
   sendSignalNoRelease(workerRef(ss.m_worker), GSN_TIME_SIGNAL,
@@ -931,8 +1077,12 @@ LocalProxy::execCREATE_TRIG_IMPL_REQ(Signal* signal)
   if (!assembleFragments(signal))
     return;
 
+  jam();
   if (ssQueue<Ss_CREATE_TRIG_IMPL_REQ>(signal))
+  {
+    jam();
     return;
+  }
   const CreateTrigImplReq* req = (const CreateTrigImplReq*)signal->getDataPtr();
   Ss_CREATE_TRIG_IMPL_REQ& ss = ssSeize<Ss_CREATE_TRIG_IMPL_REQ>();
   ss.m_req = *req;
@@ -948,6 +1098,7 @@ void
 LocalProxy::sendCREATE_TRIG_IMPL_REQ(Signal* signal, Uint32 ssId,
                                      SectionHandle * handle)
 {
+  jam();
   Ss_CREATE_TRIG_IMPL_REQ& ss = ssFind<Ss_CREATE_TRIG_IMPL_REQ>(ssId);
 
   CreateTrigImplReq* req = (CreateTrigImplReq*)signal->getDataPtrSend();
@@ -962,6 +1113,7 @@ LocalProxy::sendCREATE_TRIG_IMPL_REQ(Signal* signal, Uint32 ssId,
 void
 LocalProxy::execCREATE_TRIG_IMPL_CONF(Signal* signal)
 {
+  jam();
   const CreateTrigImplConf* conf = (const CreateTrigImplConf*)signal->getDataPtr();
   Uint32 ssId = conf->senderData;
   Ss_CREATE_TRIG_IMPL_REQ& ss = ssFind<Ss_CREATE_TRIG_IMPL_REQ>(ssId);
@@ -971,6 +1123,7 @@ LocalProxy::execCREATE_TRIG_IMPL_CONF(Signal* signal)
 void
 LocalProxy::execCREATE_TRIG_IMPL_REF(Signal* signal)
 {
+  jam();
   const CreateTrigImplRef* ref = (const CreateTrigImplRef*)signal->getDataPtr();
   Uint32 ssId = ref->senderData;
   Ss_CREATE_TRIG_IMPL_REQ& ss = ssFind<Ss_CREATE_TRIG_IMPL_REQ>(ssId);
@@ -980,13 +1133,18 @@ LocalProxy::execCREATE_TRIG_IMPL_REF(Signal* signal)
 void
 LocalProxy::sendCREATE_TRIG_IMPL_CONF(Signal* signal, Uint32 ssId)
 {
+  jam();
   Ss_CREATE_TRIG_IMPL_REQ& ss = ssFind<Ss_CREATE_TRIG_IMPL_REQ>(ssId);
   BlockReference dictRef = ss.m_req.senderRef;
 
   if (!lastReply(ss))
+  {
+    jam();
     return;
+  }
 
-  if (ss.m_error == 0) {
+  if (ss.m_error == 0)
+  {
     jam();
     CreateTrigImplConf* conf = (CreateTrigImplConf*)signal->getDataPtrSend();
     conf->senderRef = reference();
@@ -996,7 +1154,10 @@ LocalProxy::sendCREATE_TRIG_IMPL_CONF(Signal* signal, Uint32 ssId)
     conf->triggerInfo = ss.m_req.triggerInfo;
     sendSignal(dictRef, GSN_CREATE_TRIG_IMPL_CONF,
                signal, CreateTrigImplConf::SignalLength, JBB);
-  } else {
+  }
+  else
+  {
+    jam();
     CreateTrigImplRef* ref = (CreateTrigImplRef*)signal->getDataPtrSend();
     ref->senderRef = reference();
     ref->senderData = ss.m_req.senderData;
@@ -1016,8 +1177,12 @@ LocalProxy::sendCREATE_TRIG_IMPL_CONF(Signal* signal, Uint32 ssId)
 void
 LocalProxy::execDROP_TRIG_IMPL_REQ(Signal* signal)
 {
+  jam();
   if (ssQueue<Ss_DROP_TRIG_IMPL_REQ>(signal))
+  {
+    jam();
     return;
+  }
   const DropTrigImplReq* req = (const DropTrigImplReq*)signal->getDataPtr();
   Ss_DROP_TRIG_IMPL_REQ& ss = ssSeize<Ss_DROP_TRIG_IMPL_REQ>();
   ss.m_req = *req;
@@ -1029,6 +1194,7 @@ void
 LocalProxy::sendDROP_TRIG_IMPL_REQ(Signal* signal, Uint32 ssId,
                                    SectionHandle * handle)
 {
+  jam();
   Ss_DROP_TRIG_IMPL_REQ& ss = ssFind<Ss_DROP_TRIG_IMPL_REQ>(ssId);
 
   DropTrigImplReq* req = (DropTrigImplReq*)signal->getDataPtrSend();
@@ -1042,6 +1208,7 @@ LocalProxy::sendDROP_TRIG_IMPL_REQ(Signal* signal, Uint32 ssId,
 void
 LocalProxy::execDROP_TRIG_IMPL_CONF(Signal* signal)
 {
+  jam();
   const DropTrigImplConf* conf = (const DropTrigImplConf*)signal->getDataPtr();
   Uint32 ssId = conf->senderData;
   Ss_DROP_TRIG_IMPL_REQ& ss = ssFind<Ss_DROP_TRIG_IMPL_REQ>(ssId);
@@ -1051,6 +1218,7 @@ LocalProxy::execDROP_TRIG_IMPL_CONF(Signal* signal)
 void
 LocalProxy::execDROP_TRIG_IMPL_REF(Signal* signal)
 {
+  jam();
   const DropTrigImplRef* ref = (const DropTrigImplRef*)signal->getDataPtr();
   Uint32 ssId = ref->senderData;
   Ss_DROP_TRIG_IMPL_REQ& ss = ssFind<Ss_DROP_TRIG_IMPL_REQ>(ssId);
@@ -1060,13 +1228,18 @@ LocalProxy::execDROP_TRIG_IMPL_REF(Signal* signal)
 void
 LocalProxy::sendDROP_TRIG_IMPL_CONF(Signal* signal, Uint32 ssId)
 {
+  jam();
   Ss_DROP_TRIG_IMPL_REQ& ss = ssFind<Ss_DROP_TRIG_IMPL_REQ>(ssId);
   BlockReference dictRef = ss.m_req.senderRef;
 
   if (!lastReply(ss))
+  {
+    jam();
     return;
+  }
 
-  if (ss.m_error == 0) {
+  if (ss.m_error == 0)
+  {
     jam();
     DropTrigImplConf* conf = (DropTrigImplConf*)signal->getDataPtrSend();
     conf->senderRef = reference();
@@ -1075,7 +1248,10 @@ LocalProxy::sendDROP_TRIG_IMPL_CONF(Signal* signal, Uint32 ssId)
     conf->triggerId = ss.m_req.triggerId;
     sendSignal(dictRef, GSN_DROP_TRIG_IMPL_CONF,
                signal, DropTrigImplConf::SignalLength, JBB);
-  } else {
+  }
+  else
+  {
+    jam();
     DropTrigImplRef* ref = (DropTrigImplRef*)signal->getDataPtrSend();
     ref->senderRef = reference();
     ref->senderData = ss.m_req.senderData;
@@ -1284,6 +1460,7 @@ LocalProxy::execDBINFO_SCANCONF(Signal* signal)
 void
 LocalProxy::execSYNC_REQ(Signal* signal)
 {
+  jam();
   Ss_SYNC_REQ& ss = ssSeize<Ss_SYNC_REQ>();
 
   ss.m_req = * CAST_CONSTPTR(SyncReq, signal->getDataPtr());
@@ -1295,6 +1472,7 @@ void
 LocalProxy::sendSYNC_REQ(Signal* signal, Uint32 ssId,
                          SectionHandle* handle)
 {
+  jam();
   Ss_SYNC_REQ& ss = ssFind<Ss_SYNC_REQ>(ssId);
 
   SyncReq * req = CAST_PTR(SyncReq, signal->getDataPtrSend());
@@ -1310,6 +1488,7 @@ LocalProxy::sendSYNC_REQ(Signal* signal, Uint32 ssId,
 void
 LocalProxy::execSYNC_REF(Signal* signal)
 {
+  jam();
   SyncRef ref = * CAST_CONSTPTR(SyncRef, signal->getDataPtr());
   Ss_SYNC_REQ& ss = ssFind<Ss_SYNC_REQ>(ref.senderData);
 
@@ -1319,6 +1498,7 @@ LocalProxy::execSYNC_REF(Signal* signal)
 void
 LocalProxy::execSYNC_CONF(Signal* signal)
 {
+  jam();
   SyncConf conf = * CAST_CONSTPTR(SyncConf, signal->getDataPtr());
   Ss_SYNC_REQ& ss = ssFind<Ss_SYNC_REQ>(conf.senderData);
 
@@ -1328,10 +1508,14 @@ LocalProxy::execSYNC_CONF(Signal* signal)
 void
 LocalProxy::sendSYNC_CONF(Signal* signal, Uint32 ssId)
 {
+  jam();
   Ss_SYNC_REQ& ss = ssFind<Ss_SYNC_REQ>(ssId);
 
   if (!lastReply(ss))
+  {
+    jam();
     return;
+  }
 
   /**
    * SimulatedBlock::execSYNC_REQ will reply
@@ -1367,6 +1551,7 @@ LocalProxy::sendSYNC_CONF(Signal* signal, Uint32 ssId)
 void
 LocalProxy::execSYNC_PATH_REQ(Signal* signal)
 {
+  jam();
   SyncPathReq* req = CAST_PTR(SyncPathReq, signal->getDataPtrSend());
   req->count *= c_workers;
 
@@ -1385,6 +1570,7 @@ LocalProxy::execSYNC_PATH_REQ(Signal* signal)
 void
 LocalProxy::execAPI_FAILREQ(Signal* signal)
 {
+  jam();
   Uint32 nodeId = signal->theData[0];
   Ss_API_FAILREQ& ss = ssSeize<Ss_API_FAILREQ>(nodeId);
 
@@ -1395,6 +1581,7 @@ LocalProxy::execAPI_FAILREQ(Signal* signal)
 void
 LocalProxy::sendAPI_FAILREQ(Signal* signal, Uint32 nodeId, SectionHandle*)
 {
+  jam();
   Ss_API_FAILREQ& ss = ssFind<Ss_API_FAILREQ>(nodeId);
 
   /*
@@ -1461,6 +1648,7 @@ LocalProxy::sendAPI_FAILREQ(Signal* signal, Uint32 nodeId, SectionHandle*)
 void
 LocalProxy::execAPI_FAILCONF(Signal* signal)
 {
+  jam();
   Uint32 nodeId = signal->theData[0];
   Ss_API_FAILREQ& ss = ssFind<Ss_API_FAILREQ>(nodeId);
   recvCONF(signal, ss);
@@ -1469,10 +1657,14 @@ LocalProxy::execAPI_FAILCONF(Signal* signal)
 void
 LocalProxy::sendAPI_FAILCONF(Signal* signal, Uint32 ssId)
 {
+  jam();
   Ss_API_FAILREQ& ss = ssFind<Ss_API_FAILREQ>(ssId);
 
   if (!lastReply(ss))
+  {
+    jam();
     return;
+  }
 
   signal->theData[0] = ssId;
   signal->theData[1] = reference();
