@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -42,7 +42,9 @@ void Dbdih::initData()
                  ZCREATE_REPLICA_FILE_SIZE);
 
   nodeGroupRecord = (NodeGroupRecord*)
-    allocRecord("NodeGroupRecord", sizeof(NodeGroupRecord), MAX_NDB_NODES);
+    allocRecord("NodeGroupRecord",
+                sizeof(NodeGroupRecord),
+                MAX_NDB_NODE_GROUPS);
 
   nodeRecord = (NodeRecord*)
     allocRecord("NodeRecord", sizeof(NodeRecord), MAX_NDB_NODES);
@@ -62,7 +64,7 @@ void Dbdih::initData()
     }
     while (c_masterActiveTakeOverList.first(ptr))
     {
-      releaseTakeOver(ptr, true);
+      releaseTakeOver(ptr, true, true);
     }
   }
   
@@ -83,20 +85,20 @@ void Dbdih::initData()
   c_sr_wait_to = false;
   c_2pass_inr = false;
   c_handled_master_take_over_copy_gci = 0;
-  
+  c_start_node_lcp_req_outstanding = false;
+
   c_lcpTabDefWritesControl.init(MAX_CONCURRENT_LCP_TAB_DEF_FLUSHES);
+  for (Uint32 i = 0; i < MAX_NDB_NODES; i++)
+  {
+    m_node_redo_alert_state[i] = RedoStateRep::NO_REDO_ALERT;
+  }
+  m_global_redo_alert_state = RedoStateRep::NO_REDO_ALERT;
+  m_master_lcp_req_lcp_already_completed = false;
 }//Dbdih::initData()
 
 void Dbdih::initRecords()
 {
   // Records with dynamic sizes
-  for (Uint32 i = 0; i < c_diverify_queue_cnt; i++)
-  {
-    c_diverify_queue[i].apiConnectRecord = (ApiConnectRecord*)
-      allocRecord("ApiConnectRecord",
-                  sizeof(ApiConnectRecord),
-                  capiConnectFileSize);
-  }
 
   connectRecord = (ConnectRecord*)allocRecord("ConnectRecord",
                                               sizeof(ConnectRecord), 
@@ -121,22 +123,16 @@ void Dbdih::initRecords()
                                               ctabFileSize);
 
   // Initialize BAT for interface to file system
-  NewVARIABLE* bat = allocateBat(22);
-  bat[1].WA = &pageRecord->word[0];
-  bat[1].nrr = cpageFileSize;
-  bat[1].ClusterSize = sizeof(PageRecord);
-  bat[1].bits.q = 11;
-  bat[1].bits.v = 5;
-  bat[20].WA = &sysfileData[0];
-  bat[20].nrr = 1;
-  bat[20].ClusterSize = sizeof(sysfileData);
-  bat[20].bits.q = 7;
-  bat[20].bits.v = 5;
-  bat[21].WA = &sysfileDataToFile[0];
-  bat[21].nrr = 1;
-  bat[21].ClusterSize = sizeof(sysfileDataToFile);
-  bat[21].bits.q = 7;
-  bat[21].bits.v = 5;
+  NewVARIABLE* bat = allocateBat(3);
+  bat[0].WA = &pageRecord->word[0];
+  bat[0].nrr = cpageFileSize;
+  bat[0].ClusterSize = sizeof(PageRecord);
+  bat[0].bits.q = 11;
+  bat[0].bits.v = 5;
+  bat[1].WA = &cdata[0];
+  bat[1].nrr = 1;
+  bat[2].WA = &sysfileDataToFile[0];
+  bat[2].nrr = 1;
 }//Dbdih::initRecords()
 
 Dbdih::Dbdih(Block_context& ctx):
@@ -158,6 +154,7 @@ Dbdih::Dbdih(Block_context& ctx):
   c_mainTakeOverPtr.p = 0;
   c_activeThreadTakeOverPtr.i = RNIL;
   c_activeThreadTakeOverPtr.p = 0;
+  m_max_node_id = Uint32(~0);
 
   /* Node Recovery Status Module signals */
   addRecSignal(GSN_ALLOC_NODEID_REP, &Dbdih::execALLOC_NODEID_REP);
@@ -188,8 +185,6 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_MASTER_GCPREQ, &Dbdih::execMASTER_GCPREQ);
   addRecSignal(GSN_MASTER_GCPREF, &Dbdih::execMASTER_GCPREF);
   addRecSignal(GSN_MASTER_GCPCONF, &Dbdih::execMASTER_GCPCONF);
-  addRecSignal(GSN_EMPTY_LCP_CONF, &Dbdih::execEMPTY_LCP_CONF);
-  addRecSignal(GSN_EMPTY_LCP_REP, &Dbdih::execEMPTY_LCP_REP);
 
   addRecSignal(GSN_MASTER_LCPREQ, &Dbdih::execMASTER_LCPREQ);
   addRecSignal(GSN_MASTER_LCPREF, &Dbdih::execMASTER_LCPREF);
@@ -239,6 +234,7 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_COPY_GCICONF, &Dbdih::execCOPY_GCICONF);
   addRecSignal(GSN_COPY_TABREQ, &Dbdih::execCOPY_TABREQ);
   addRecSignal(GSN_COPY_TABCONF, &Dbdih::execCOPY_TABCONF);
+  addRecSignal(GSN_CHECK_LCP_IDLE_ORD, &Dbdih::execCHECK_LCP_IDLE_ORD);
   addRecSignal(GSN_TCGETOPSIZECONF, &Dbdih::execTCGETOPSIZECONF);
   addRecSignal(GSN_TC_CLOPSIZECONF, &Dbdih::execTC_CLOPSIZECONF);
 
@@ -246,6 +242,7 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_LCP_FRAG_REP, &Dbdih::execLCP_FRAG_REP);
   addRecSignal(GSN_START_LCP_REQ, &Dbdih::execSTART_LCP_REQ);
   addRecSignal(GSN_START_LCP_CONF, &Dbdih::execSTART_LCP_CONF);
+  addRecSignal(GSN_START_NODE_LCP_CONF, &Dbdih::execSTART_NODE_LCP_CONF);
   
   addRecSignal(GSN_READ_CONFIG_REQ, &Dbdih::execREAD_CONFIG_REQ, true);
   addRecSignal(GSN_UNBLO_DICTCONF, &Dbdih::execUNBLO_DICTCONF);
@@ -258,7 +255,6 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_DIGETNODESREQ, &Dbdih::execDIGETNODESREQ);
   addRecSignal(GSN_STTOR, &Dbdih::execSTTOR);
   addRecSignal(GSN_DIH_SCAN_TAB_REQ, &Dbdih::execDIH_SCAN_TAB_REQ);
-  addRecSignal(GSN_DIH_SCAN_GET_NODES_REQ, &Dbdih::execDIH_SCAN_GET_NODES_REQ);
   addRecSignal(GSN_DIH_SCAN_TAB_COMPLETE_REP,
                &Dbdih::execDIH_SCAN_TAB_COMPLETE_REP);
   addRecSignal(GSN_GCP_TCFINISHED, &Dbdih::execGCP_TCFINISHED);
@@ -267,6 +263,8 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_DICTSTARTCONF, &Dbdih::execDICTSTARTCONF);
   addRecSignal(GSN_NDB_STARTREQ, &Dbdih::execNDB_STARTREQ);
   addRecSignal(GSN_GETGCIREQ, &Dbdih::execGETGCIREQ);
+  addRecSignal(GSN_GET_LATEST_GCI_REQ, &Dbdih::execGET_LATEST_GCI_REQ);
+  addRecSignal(GSN_SET_LATEST_LCP_ID, &Dbdih::execSET_LATEST_LCP_ID);
   addRecSignal(GSN_DIH_RESTARTREQ, &Dbdih::execDIH_RESTARTREQ);
   addRecSignal(GSN_START_RECCONF, &Dbdih::execSTART_RECCONF);
   addRecSignal(GSN_START_FRAGCONF, &Dbdih::execSTART_FRAGCONF);
@@ -321,6 +319,8 @@ Dbdih::Dbdih(Block_context& ctx):
   addRecSignal(GSN_WAIT_GCP_REF, &Dbdih::execWAIT_GCP_REF);
   addRecSignal(GSN_WAIT_GCP_CONF, &Dbdih::execWAIT_GCP_CONF);
 
+  addRecSignal(GSN_REDO_STATE_REP, &Dbdih::execREDO_STATE_REP);
+
   addRecSignal(GSN_PREP_DROP_TAB_REQ, &Dbdih::execPREP_DROP_TAB_REQ);
   addRecSignal(GSN_DROP_TAB_REQ, &Dbdih::execDROP_TAB_REQ);
 
@@ -369,8 +369,9 @@ Dbdih::Dbdih(Block_context& ctx):
   nodeGroupRecord = 0;
   nodeRecord = 0;
   c_nextNodeGroup = 0;
+  memset(c_next_replica_node, 0, sizeof(c_next_replica_node));
   c_fragments_per_node_ = 0;
-  bzero(c_node_groups, sizeof(c_node_groups));
+  memset(c_node_groups, 0, sizeof(c_node_groups));
   if (globalData.ndbMtTcThreads == 0)
   {
     c_diverify_queue_cnt = 1;
@@ -383,14 +384,6 @@ Dbdih::Dbdih(Block_context& ctx):
 
 Dbdih::~Dbdih()
 {
-  for (Uint32 i = 0; i<c_diverify_queue_cnt; i++)
-  {
-    deallocRecord((void **)&c_diverify_queue[i].apiConnectRecord,
-                  "ApiConnectRecord",
-                  sizeof(ApiConnectRecord),
-                  capiConnectFileSize);
-  }
-
   deallocRecord((void **)&connectRecord, "ConnectRecord",
                 sizeof(ConnectRecord), 
                 cconnectFileSize);
@@ -417,7 +410,7 @@ Dbdih::~Dbdih()
                 ZCREATE_REPLICA_FILE_SIZE);
   
   deallocRecord((void **)&nodeGroupRecord, "NodeGroupRecord", 
-                sizeof(NodeGroupRecord), MAX_NDB_NODES);
+                sizeof(NodeGroupRecord), MAX_NDB_NODE_GROUPS);
   
   deallocRecord((void **)&nodeRecord, "NodeRecord", 
                 sizeof(NodeRecord), MAX_NDB_NODES);
