@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,7 +25,6 @@
 #ifndef RECORDS_HPP
 #define RECORDS_HPP
 
-#include <NdbMain.h>
 #include <NdbOut.hpp>
 #include <ndb_types.h>
 
@@ -40,10 +39,8 @@
 #define ZCOMPLETED_GCI_TYPE 8
 #define ZINVALID_COMMIT_TYPE 9
 
-#define MAX_FILE_DESCRIPTORS 40
-
-#define PAGESIZE 8192
-#define NO_PAGES_IN_MBYTE 32
+#define REDOLOG_PAGESIZE 8192
+#define REDOLOG_PAGES_IN_MBYTE 32
 
 #define COMMITTRANSACTIONRECORDSIZE 9
 #define COMPLETEDGCIRECORDSIZE 2
