@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,7 +24,7 @@
 
 #define DBTUX_META_CPP
 #include "Dbtux.hpp"
-#include <my_sys.h>
+#include "my_sys.h"
 
 /*
  * Create index.
@@ -214,7 +214,7 @@ Dbtux::execTUX_ADD_ATTRREQ(Signal* signal)
       // compute min prefix
       const KeySpec& keySpec = indexPtr.p->m_keySpec;
       unsigned attrs = 0;
-      unsigned bytes = keySpec.get_nullmask_len(false);
+      unsigned bytes = 0;
       unsigned maxAttrs = indexPtr.p->m_numAttrs;
 #ifdef VM_TRACE
 #ifdef NDB_USE_GET_ENV
@@ -225,16 +225,21 @@ Dbtux::execTUX_ADD_ATTRREQ(Signal* signal)
       }
 #endif
 #endif
-      while (attrs < maxAttrs) {
+      while (attrs < maxAttrs)
+      {
+        /**
+         * Prefix is now saved as a normal Attrinfo data stream.
+         * This means that each column uses 4 bytes Attrinfo header
+         * the data is aligned on a word boundary.
+         */
         const KeyType& keyType = keySpec.get_type(attrs);
-        const unsigned newbytes = bytes + keyType.get_byte_size();
+        const unsigned word_size = (keyType.get_byte_size() + 3) / 4;
+        const unsigned newbytes = bytes + ((word_size + 1) * 4);
         if (newbytes > (MAX_TTREE_PREF_SIZE << 2))
           break;
         attrs++;
         bytes = newbytes;
       }
-      if (attrs == 0)
-        bytes = 0;
       indexPtr.p->m_prefAttrs = attrs;
       indexPtr.p->m_prefBytes = bytes;
       // fragment is defined
@@ -323,7 +328,7 @@ Dbtux::execTUXFRAGREQ(Signal* signal)
     fragPtr.p->m_tupTableFragPtrI = req->tupTableFragPtrI;
     fragPtr.p->m_accTableFragPtrI = req->accTableFragPtrI;
     // add the fragment to the index
-    Uint32 fragNo = indexPtr.p->m_numFrags;
+    const Uint32 fragNo = indexPtr.p->m_numFrags;
     indexPtr.p->m_fragId[indexPtr.p->m_numFrags] = req->fragId;
     indexPtr.p->m_fragPtrI[indexPtr.p->m_numFrags] = fragPtr.i;
     indexPtr.p->m_numFrags++;
@@ -664,8 +669,8 @@ Dbtux::execDROP_FRAG_REQ(Signal* signal)
 
   IndexPtr indexPtr;
   c_indexPool.getPtr(indexPtr, req->tableId);
-  Uint32 i = 0;
-  for (i = 0; i < indexPtr.p->m_numFrags; i++)
+
+  for (Uint32 i = 0; i < indexPtr.p->m_numFrags; i++)
   {
     jam();
     if (indexPtr.p->m_fragId[i] == req->fragId)
@@ -678,6 +683,9 @@ Dbtux::execDROP_FRAG_REQ(Signal* signal)
       for (i++; i < indexPtr.p->m_numFrags; i++)
       {
         jam();
+        /* Check array bounds to silence gcc 4.8.2 false warning bug */
+        ndbrequire(i < NDB_ARRAY_SIZE(indexPtr.p->m_fragPtrI) &&
+                   i < NDB_ARRAY_SIZE(indexPtr.p->m_fragId));
         indexPtr.p->m_fragPtrI[i-1] = indexPtr.p->m_fragPtrI[i];
         indexPtr.p->m_fragId[i-1] = indexPtr.p->m_fragId[i];
       }
