@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -23,6 +30,8 @@
 
 /*
  * Allocate index node in TUP.
+ *
+ * Can be called from MT-build of ordered indexes.
  */
 int
 Dbtux::allocNode(TuxCtx& ctx, NodeHandle& node)
@@ -32,24 +41,32 @@ Dbtux::allocNode(TuxCtx& ctx, NodeHandle& node)
     CLEAR_ERROR_INSERT_VALUE;
     return TuxMaintReq::NoMemError;
   }
-  Frag& frag = node.m_frag;
   Uint32 pageId = NullTupLoc.getPageId();
   Uint32 pageOffset = NullTupLoc.getPageOffset();
   Uint32* node32 = 0;
   int errorCode = c_tup->tuxAllocNode(ctx.jamBuffer,
-                                      frag.m_tupIndexFragPtrI,
-                                      pageId, pageOffset, node32);
-  thrjamEntry(ctx.jamBuffer);
-  if (errorCode == 0) {
-    thrjam(ctx.jamBuffer);
+                                      ctx.tupIndexFragPtr,
+                                      ctx.tupIndexTablePtr,
+                                      pageId,
+                                      pageOffset,
+                                      node32);
+  thrjamEntryDebug(ctx.jamBuffer);
+  if (likely(errorCode == 0))
+  {
+    thrjamDebug(ctx.jamBuffer);
     node.m_loc = TupLoc(pageId, pageOffset);
     node.m_node = reinterpret_cast<TreeNode*>(node32);
     ndbrequire(node.m_loc != NullTupLoc && node.m_node != 0);
-  } else {
+  }
+  else
+  {
     switch (errorCode) {
     case 827:
       errorCode = TuxMaintReq::NoMemError;
       break;
+    case 921:
+      errorCode = TuxMaintReq::NoTransMemError;
+      break;
     }
   }
   return errorCode;
@@ -61,12 +78,14 @@ Dbtux::allocNode(TuxCtx& ctx, NodeHandle& node)
 void
 Dbtux::freeNode(NodeHandle& node)
 {
-  Frag& frag = node.m_frag;
   Uint32 pageId = node.m_loc.getPageId();
   Uint32 pageOffset = node.m_loc.getPageOffset();
   Uint32* node32 = reinterpret_cast<Uint32*>(node.m_node);
-  c_tup->tuxFreeNode(frag.m_tupIndexFragPtrI,
-                     pageId, pageOffset, node32);
+  c_tup->tuxFreeNode(c_ctx.tupIndexFragPtr,
+                     c_ctx.tupIndexTablePtr,
+                     pageId,
+                     pageOffset,
+                     node32);
   jamEntry();
   // invalidate the handle
   node.m_loc = NullTupLoc;
@@ -75,16 +94,20 @@ Dbtux::freeNode(NodeHandle& node)
 
 /*
  * Set handle to point to existing node.
+ * Can be called from MT-build of ordered indexes.
  */
 void
-Dbtux::selectNode(NodeHandle& node, TupLoc loc)
+Dbtux::selectNode(TuxCtx& ctx, NodeHandle& node, TupLoc loc)
 {
-  Frag& frag = node.m_frag;
   ndbrequire(loc != NullTupLoc);
   Uint32 pageId = loc.getPageId();
   Uint32 pageOffset = loc.getPageOffset();
   Uint32* node32 = 0;
-  c_tup->tuxGetNode(frag.m_tupIndexFragPtrI, pageId, pageOffset, node32);
+  c_tup->tuxGetNode(ctx.attrDataOffset,
+                    ctx.tuxFixHeaderSize,
+                    pageId,
+                    pageOffset,
+                    node32);
   node.m_loc = loc;
   node.m_node = reinterpret_cast<TreeNode*>(node32);
   ndbrequire(node.m_loc != NullTupLoc && node.m_node != 0);
@@ -92,13 +115,15 @@ Dbtux::selectNode(NodeHandle& node, TupLoc loc)
 
 /*
  * Set handle to point to new node.  Uses a pre-allocated node.
+ *
+ * Can be called from MT-build of ordered indexes.
  */
 void
-Dbtux::insertNode(NodeHandle& node)
+Dbtux::insertNode(TuxCtx& ctx, NodeHandle& node)
 {
   Frag& frag = node.m_frag;
   // use up pre-allocated node
-  selectNode(node, frag.m_freeLoc);
+  selectNode(ctx, node, frag.m_freeLoc);
   frag.m_freeLoc = NullTupLoc;
   new (node.m_node) TreeNode();
 #ifdef VM_TRACE
@@ -144,7 +169,7 @@ Dbtux::freePreallocatedNode(Frag& frag)
   {
     jam();
     NodeHandle node(frag);
-    selectNode(node, frag.m_freeLoc);
+    selectNode(c_ctx, node, frag.m_freeLoc);
     freeNode(node);
     frag.m_freeLoc = NullTupLoc;
   }
@@ -152,6 +177,8 @@ Dbtux::freePreallocatedNode(Frag& frag)
 
 /*
  * Set prefix.  Copies the defined number of attributes.
+ *
+ * Can be called from MT-build of ordered indexes.
  */
 void
 Dbtux::setNodePref(TuxCtx & ctx, NodeHandle& node)
@@ -165,18 +192,14 @@ Dbtux::setNodePref(TuxCtx & ctx, NodeHandle& node)
    * then set_buf() could overwrite m_pageId1 in first entry, causing
    * random crash in TUP via readKeyAttrs().
    */
-  if (index.m_prefAttrs > 0) {
-    KeyData prefKey(index.m_keySpec, false, 0);
-    prefKey.set_buf(node.getPref(), index.m_prefBytes);
+  if (index.m_prefAttrs > 0)
+  {
     thrjam(ctx.jamBuffer);
-    readKeyAttrs(ctx, frag, node.getEnt(0), prefKey, index.m_prefAttrs);
-#ifdef VM_TRACE
-    if (debugFlags & DebugMaint) {
-      debugOut << "setNodePref: " << node;
-      debugOut << " " << prefKey.print(ctx.c_debugBuffer, DebugBufferBytes);
-      debugOut << endl;
-    }
-#endif
+    readKeyAttrs(ctx,
+                 frag,
+                 node.getEnt(0),
+                 index.m_prefAttrs,
+                 node.getPref());
   }
 }
 
@@ -192,9 +215,15 @@ Dbtux::setNodePref(TuxCtx & ctx, NodeHandle& node)
  *      0 1 2 3 4 5 6      0 1 2 3 4 5 6
  *
  * Add list of scans at the new entry.
+ *
+ * Can be called from MT-build of ordered indexes.
  */
 void
-Dbtux::nodePushUp(TuxCtx & ctx, NodeHandle& node, unsigned pos, const TreeEnt& ent, Uint32 scanList)
+Dbtux::nodePushUp(TuxCtx & ctx,
+                  NodeHandle& node,
+                  unsigned pos,
+                  const TreeEnt& ent,
+                  Uint32 scanList)
 {
   Frag& frag = node.m_frag;
   TreeHead& tree = frag.m_tree;
@@ -206,7 +235,7 @@ Dbtux::nodePushUp(TuxCtx & ctx, NodeHandle& node, unsigned pos, const TreeEnt& e
   // fix node
   TreeEnt* const entList = tree.getEntList(node.m_node);
   for (unsigned i = occup; i > pos; i--) {
-    thrjam(ctx.jamBuffer);
+    thrjamDebug(ctx.jamBuffer);
     entList[i] = entList[i - 1];
   }
   entList[pos] = ent;
@@ -219,6 +248,12 @@ Dbtux::nodePushUp(TuxCtx & ctx, NodeHandle& node, unsigned pos, const TreeEnt& e
     setNodePref(ctx, node);
 }
 
+/**
+ * Can be called from MT-build of ordered indexes.
+ * But should never enter here since there cannot be
+ * any active scans while we are rebuilding ordered
+ * index.
+ */
 void
 Dbtux::nodePushUpScans(NodeHandle& node, unsigned pos)
 {
@@ -323,9 +358,15 @@ Dbtux::nodePopDownScans(NodeHandle& node, unsigned pos)
  *      0 1 2 3 4 5 6      0 1 2 3 4 5 6
  *
  * Return list of scans at the removed position 0.
+ *
+ * Can be called from MT-build of ordered indexes.
  */
 void
-Dbtux::nodePushDown(TuxCtx& ctx, NodeHandle& node, unsigned pos, TreeEnt& ent, Uint32& scanList)
+Dbtux::nodePushDown(TuxCtx& ctx,
+                    NodeHandle& node,
+                    unsigned pos,
+                    TreeEnt& ent,
+                    Uint32& scanList)
 {
   Frag& frag = node.m_frag;
   TreeHead& tree = frag.m_tree;
@@ -342,7 +383,7 @@ Dbtux::nodePushDown(TuxCtx& ctx, NodeHandle& node, unsigned pos, TreeEnt& ent, U
   TreeEnt* const entList = tree.getEntList(node.m_node);
   TreeEnt oldMin = entList[0];
   for (unsigned i = 0; i < pos; i++) {
-    thrjam(ctx.jamBuffer);
+    thrjamDebug(ctx.jamBuffer);
     entList[i] = entList[i + 1];
   }
   entList[pos] = ent;
@@ -352,6 +393,11 @@ Dbtux::nodePushDown(TuxCtx& ctx, NodeHandle& node, unsigned pos, TreeEnt& ent, U
     setNodePref(ctx, node);
 }
 
+/**
+ * Can be called from MT-build of ordered indexes, but should
+ * never happen since no active scans can be around when
+ * building ordered indexes.
+ */
 void
 Dbtux::nodePushDownScans(NodeHandle& node, unsigned pos)
 {
@@ -470,6 +516,10 @@ Dbtux::nodeSlide(TuxCtx& ctx, NodeHandle& dstNode, NodeHandle& srcNode, unsigned
 
 /*
  * Add list of scans to node at given position.
+ *
+ * Can be called from MT-build of ordered indexes, but it
+ * should never happen since no active scans should be around
+ * when building ordered indexes.
  */
 void
 Dbtux::addScanList(NodeHandle& node, unsigned pos, Uint32 scanList)
@@ -499,6 +549,10 @@ Dbtux::addScanList(NodeHandle& node, unsigned pos, Uint32 scanList)
 /*
  * Remove list of scans from node at given position.  The return
  * location must point to existing list (in fact RNIL always).
+ *
+ * Can be called from MT-build of ordered indexes, but should
+ * never occur since no active scans can be around when
+ * building ordered indexes.
  */
 void
 Dbtux::removeScanList(NodeHandle& node, unsigned pos, Uint32& scanList)
@@ -524,7 +578,7 @@ Dbtux::removeScanList(NodeHandle& node, unsigned pos, Uint32& scanList)
       scanList = scanPtr.i;
       // unset position but leave direction alone
       scanPos.m_loc = NullTupLoc;
-      scanPos.m_pos = ZNIL;
+      scanPos.m_pos = Uint32(~0);
     }
     scanPtr.i = nextPtrI;
   } while (scanPtr.i != RNIL);
@@ -553,7 +607,9 @@ Dbtux::moveScanList(NodeHandle& node, unsigned pos)
         debugOut << "At pos=" << pos << " " << node << endl;
       }
 #endif
-      scanNext(scanPtr, true);
+      prepare_move_scan_ctx(scanPtr);
+      Frag& frag = *c_ctx.fragPtr.p;
+      scanNext(scanPtr, true, frag);
       ndbrequire(! (scanPos.m_loc == node.m_loc && scanPos.m_pos == pos));
     }
     scanPtr.i = nextPtrI;
@@ -567,48 +623,40 @@ Dbtux::moveScanList(NodeHandle& node, unsigned pos)
 void
 Dbtux::linkScan(NodeHandle& node, ScanOpPtr scanPtr)
 {
-#ifdef VM_TRACE
-  if (debugFlags & DebugScan) {
-    debugOut << "Link scan " << scanPtr.i << " " << *scanPtr.p << endl;
-    debugOut << "To node " << node << endl;
-  }
-#endif
-  ndbrequire(! islinkScan(node, scanPtr) && scanPtr.p->m_nodeScan == RNIL);
+  ndbassert(! islinkScan(node, scanPtr) && scanPtr.p->m_nodeScan == RNIL);
   scanPtr.p->m_nodeScan = node.getNodeScan();
   node.setNodeScan(scanPtr.i);
 }
 
 /*
  * Unlink a scan from the list under the node.
+ *
+ * Can be called from MT-build of ordered indexes, but should
+ * not since no active scans should be around when building
+ * ordered indexes.
  */
 void
 Dbtux::unlinkScan(NodeHandle& node, ScanOpPtr scanPtr)
 {
-#ifdef VM_TRACE
-  if (debugFlags & DebugScan) {
-    debugOut << "Unlink scan " << scanPtr.i << " " << *scanPtr.p << endl;
-    debugOut << "From node " << node << endl;
-  }
-#endif
   ScanOpPtr currPtr;
   currPtr.i = node.getNodeScan();
   ScanOpPtr prevPtr;
   prevPtr.i = RNIL;
   while (true) {
-    jam();
+    jamDebug();
     c_scanOpPool.getPtr(currPtr);
     Uint32 nextPtrI = currPtr.p->m_nodeScan;
     if (currPtr.i == scanPtr.i) {
-      jam();
+      jamDebug();
       if (prevPtr.i == RNIL) {
         node.setNodeScan(nextPtrI);
       } else {
-        jam();
+        jamDebug();
         prevPtr.p->m_nodeScan = nextPtrI;
       }
       scanPtr.p->m_nodeScan = RNIL;
       // check for duplicates
-      ndbrequire(! islinkScan(node, scanPtr));
+      ndbassert(!islinkScan(node, scanPtr));
       return;
     }
     prevPtr = currPtr;
@@ -625,10 +673,10 @@ Dbtux::islinkScan(NodeHandle& node, ScanOpPtr scanPtr)
   ScanOpPtr currPtr;
   currPtr.i = node.getNodeScan();
   while (currPtr.i != RNIL) {
-    jam();
+    jamDebug();
     c_scanOpPool.getPtr(currPtr);
     if (currPtr.i == scanPtr.i) {
-      jam();
+      jamDebug();
       return true;
     }
     currPtr.i = currPtr.p->m_nodeScan;
@@ -637,7 +685,19 @@ Dbtux::islinkScan(NodeHandle& node, ScanOpPtr scanPtr)
 }
 
 void
-Dbtux::NodeHandle::progError(int line, int cause, const char* file)
+Dbtux::NodeHandle::progError(int line, int cause, const char* file,
+                             const char* check)
 {
-  ErrorReporter::handleAssert("Dbtux::NodeHandle: assert failed", file, line);
+  char buf[500];
+  /*Add the check to the log message only if default value of ""
+    is over-written. */
+  if(native_strcasecmp(check,"") == 0)
+    BaseString::snprintf(buf, sizeof(buf),
+                         "Dbtux::NodeHandle: assert failed");
+  else
+    BaseString::snprintf(buf, sizeof(buf),
+                         "Dbtux::NodeHandle: assert %.400s failed",
+                         check);
+
+  ErrorReporter::handleAssert(buf, file, line);
 }
