@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -87,7 +94,7 @@ VoidFs::execSTTOR(Signal* signal)
     sendSignal(NDBCNTR_REF, GSN_STTORRY, signal, 4, JBB);
     return;
   }
-  ndbrequire(0);
+  ndbabort();
 }
 
 void
@@ -186,7 +193,8 @@ VoidFs::execFSREADREQ(Signal* signal)
   releaseSections(handle);
 
   signal->theData[0] = userPointer;
-  sendSignal(userRef, GSN_FSREADCONF, signal, 1, JBB);
+  signal->theData[1] = 0; /* Bytes read 0 */
+  sendSignal(userRef, GSN_FSREADCONF, signal, 2, JBB);
 #if 0
   FsRef * const fsRef = (FsRef *)&signal->theData[0];
   fsRef->userPointer = userPointer;
