@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2017, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -272,7 +272,7 @@ private:
   
   struct req lookup_extent(Uint32 page_no, const Datafile*) const;
   Uint32 calc_page_no_in_extent(Uint32 page_no, const struct req* val) const;
-  uint64 calculate_extent_pages_in_file(Uint64 extents,
+  Uint64 calculate_extent_pages_in_file(Uint64 extents,
                                         Uint32 extent_size,
                                         Uint64 data_pages,
                                         bool v2);
