@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <ndb_global.h>
 
@@ -46,6 +46,9 @@
 #endif
 
 #include <EventLogger.hpp>
+#include <OutputStream.hpp>
+#include <LogBuffer.hpp>
+#include <NdbGetRUsage.h>
 
 #define JAM_FILE_ID 484
 
@@ -54,7 +57,7 @@ extern EventLogger * g_eventLogger;
 static void
 systemInfo(const Configuration & config, const LogLevel & logLevel)
 {
-#ifdef NDB_WIN32
+#ifdef _WIN32
   int processors = 0;
   int speed;
   SYSTEM_INFO sinfo;
@@ -251,7 +254,7 @@ compute_acc_32kpages(const ndb_mgm_configuration_iterator * p)
  * overallocated at any size as long as there is still memory
  * remaining.
  *
- * RG_DISK_OPERATIONS:
+ * RG_TRANSACTION_MEMORY:
  * This is a resource that is either set to zero size but can be overallocated
  * without limit. If a log file group is allocated based on the config, then
  * the size of the UNDO log buffer is used to set the size of this resource.
@@ -259,6 +262,22 @@ compute_acc_32kpages(const ndb_mgm_configuration_iterator * p)
  * file group and there can only be one such group. It is using overallocating
  * if this happens through an SQL command.
  *
+ * RG_QUERY_MEMORY:
+ * Like transaction memory, query memory may be overallocated. Unlike
+ * transaction memory, query memory may not use the last free 10% of shared
+ * global memory.  This is controlled by setting the reserved memory to zero.
+ * This indicates to memory manager that the resource is low priority and
+ * should not be allowed to starve out other higher priority resource.
+ *
+ * Dbspj is the user of query memory serving join queries and "only" read data.
+ * A bad join query could easily consume a lot of memory.
+ *
+ * Dbtc, which uses transaction memory, on the other hand also serves writes,
+ * and typically memory consumption per request are more limited.
+ *
+ * In situations there there are small amount of free memory left one want to
+ * Dbtc to be prioritized over Dbspj.
+ *
  * Overallocating and total memory
  * -------------------------------
  * The total memory allocated by the global memory manager is the sum of the
@@ -372,7 +391,7 @@ init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
   }
 
   Uint32 sbpages = 0;
-  if (globalTransporterRegistry.get_using_default_send_buffer() == false)
+  if (globalData.isNdbMt)
   {
     /**
      * This path is normally always taken for ndbmtd as the transporter
@@ -449,7 +468,48 @@ init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
     ed.m_mem_manager->set_resource_limit(rl);
   }
 
-  Uint32 undopages = 0;
+  Uint32 transmem = 0;
+  Uint32 tcInstances = 1;
+  if (globalData.ndbMtTcThreads > 1)
+  {
+    tcInstances = globalData.ndbMtTcThreads;
+  }
+
+  Uint32 MaxNoOfConcurrentIndexOperations = 8192;
+  Uint32 MaxNoOfConcurrentOperations = 32768;
+  Uint32 MaxNoOfConcurrentScans = 256;
+  Uint32 MaxNoOfConcurrentTransactions = 4096;
+  Uint32 MaxNoOfFiredTriggers = 4000;
+  Uint32 MaxNoOfLocalScans = 0;
+  Uint32 TransactionBufferMemory = 1048576;
+
+  ndb_mgm_get_int_parameter(p, CFG_DB_NO_INDEX_OPS,
+                            &MaxNoOfConcurrentIndexOperations);
+  ndb_mgm_get_int_parameter(p, CFG_DB_NO_OPS, &MaxNoOfConcurrentOperations);
+  ndb_mgm_get_int_parameter(p, CFG_DB_NO_SCANS, &MaxNoOfConcurrentScans);
+  ndb_mgm_get_int_parameter(p, CFG_DB_NO_TRANSACTIONS, &MaxNoOfConcurrentTransactions);
+  ndb_mgm_get_int_parameter(p, CFG_DB_NO_TRIGGERS, &MaxNoOfFiredTriggers);
+  // Use CFG_TC_LOCAL_SCAN instead of CFG_DB_NO_LOCAL_SCANS since it is
+  // calculated if MaxNoOfLocalScans is not set.
+  ndb_mgm_get_int_parameter(p, CFG_TC_LOCAL_SCAN, &MaxNoOfLocalScans);
+  ndb_mgm_get_int_parameter(p, CFG_DB_TRANS_BUFFER_MEM, &TransactionBufferMemory);
+
+  const Uint32 TakeOverOperations = MaxNoOfConcurrentOperations;
+
+  Uint64 transmem_bytes =
+      globalEmulatorData.theSimBlockList->getTransactionMemoryNeed(
+        tcInstances,
+        p,
+        TakeOverOperations,
+        MaxNoOfConcurrentIndexOperations,
+        MaxNoOfConcurrentOperations,
+        MaxNoOfConcurrentScans,
+        MaxNoOfConcurrentTransactions,
+        MaxNoOfFiredTriggers,
+        MaxNoOfLocalScans,
+        TransactionBufferMemory);
+
+  transmem = transmem_bytes / 32768;
   {
     /**
      * Request extra undo buffer memory to be allocated when
@@ -477,31 +537,38 @@ init_global_memory_manager(EmulatorData &ed, Uint32 *watchCounter)
                                          "undo_buffer_size=",
                                          undo_buffer_size);
 
-        undopages = Uint32(undo_buffer_size / GLOBAL_PAGE_SIZE);
+        Uint32 undopages = Uint32(undo_buffer_size / GLOBAL_PAGE_SIZE);
         g_eventLogger->info("reserving %u extra pages for undo buffer memory",
                             undopages);
+        transmem += undopages;
         Resource_limit rl;
-        rl.m_min = undopages;
+        rl.m_min = transmem;
         rl.m_max = 0;
-        rl.m_resource_id = RG_DISK_OPERATIONS;
+        rl.m_resource_id = RG_TRANSACTION_MEMORY;
         ed.m_mem_manager->set_resource_limit(rl);
       }
     }
   }
 
-  Uint32 sum = shared_pages + tupmem + filepages + jbpages + sbpages +
-    pgman_pages + stpages + undopages;
-
-  if (sum)
   {
     Resource_limit rl;
+    /*
+     * Setting m_min = 0 makes QUERY_MEMORY a low priority resource group
+     * which can not use the last 10% of shared global page memory.
+     *
+     * For example TRANSACTION_MEMORY will have access to those last
+     * percent of global shared global page memory.
+     */
     rl.m_min = 0;
-    rl.m_max = sum;
-    rl.m_resource_id = 0;
+    rl.m_max = 0;
+    rl.m_resource_id = RG_QUERY_MEMORY;
     ed.m_mem_manager->set_resource_limit(rl);
   }
 
-  if (!ed.m_mem_manager->init(watchCounter))
+  Uint32 sum = shared_pages + tupmem + filepages + jbpages + sbpages +
+    pgman_pages + stpages + transmem;
+
+  if (!ed.m_mem_manager->init(watchCounter, sum))
   {
     struct ndb_mgm_param_info dm;
     struct ndb_mgm_param_info sga;
@@ -650,7 +717,7 @@ handler_error(int signum){
       my_thread_equal(thread_id, my_thread_self()))
   {
     // Shutdown thread received signal
-#ifndef NDB_WIN32
+#ifndef _WIN32
 	signal(signum, SIG_DFL);
     kill(getpid(), signum);
 #endif
@@ -768,13 +835,105 @@ DWORD WINAPI shutdown_thread(LPVOID)
 }
 #endif
 
+struct ThreadData
+{
+  FILE* f;
+  LogBuffer* logBuf;
+  bool stop;
+};
+
+/**
+ * This function/thread is responsible for getting
+ * bytes from the log buffer and writing them
+ * to the log file.
+ */
+
+void* async_log_func(void* args)
+{
+  ThreadData* data = (ThreadData*)args;
+  FILE* f = data->f;
+  LogBuffer* logBuf = data->logBuf;
+  const size_t get_bytes = 512;
+  char buf[get_bytes + 1];
+  size_t bytes;
+  int part_bytes = 0, bytes_printed = 0;
+
+  while(!data->stop)
+  {
+    part_bytes = 0;
+    bytes_printed = 0;
+
+    if((bytes = logBuf->get(buf, get_bytes)))
+    {
+      fwrite(buf, bytes, 1, f);
+      fflush(f);
+    }
+  }
+
+  while((bytes = logBuf->get(buf, get_bytes, 1)))// flush remaining logs
+  {
+    fwrite(buf, bytes, 1, f);
+    fflush(f);
+  }
+
+  // print lost count in the end, if any
+  size_t lost_count = logBuf->getLostCount();
+  if(lost_count)
+  {
+    fprintf(f, "\n*** %lu BYTES LOST ***\n", (unsigned long)lost_count);
+    fflush(f);
+  }
+
+  return NULL;
+}
+
+static void log_memusage(const char* where=NULL)
+{
+#ifdef DEBUG_RSS
+  const char* location = (where != NULL)?where : "Unknown";
+  ndb_rusage ru;
+  if (Ndb_GetRUsage(&ru, true) != 0)
+  {
+    g_eventLogger->error("Failed to get rusage");
+  }
+  else
+  {
+    g_eventLogger->info("ndbd.cpp %s : RSS : %llu kB", location, ru.ru_rss);
+  }
+#else
+  (void)where;
+#endif
+}
 
 void
 ndbd_run(bool foreground, int report_fd,
          const char* connect_str, int force_nodeid, const char* bind_address,
          bool no_start, bool initial, bool initialstart,
-         unsigned allocated_nodeid, int connect_retries, int connect_delay)
+         unsigned allocated_nodeid, int connect_retries, int connect_delay,
+         size_t logbuffer_size)
 {
+  log_memusage("ndbd_run");
+  LogBuffer* logBuf = new LogBuffer(logbuffer_size);
+  BufferedOutputStream* ndbouts_bufferedoutputstream = new BufferedOutputStream(logBuf);
+
+  // Make ndbout point to the BufferedOutputStream.
+  NdbOut_ReInit(ndbouts_bufferedoutputstream, ndbouts_bufferedoutputstream);
+
+  struct NdbThread* log_threadvar= NULL;
+  ThreadData thread_args=
+  {
+    stdout,
+    logBuf,
+    false
+  };
+
+  // Create log thread.
+  log_threadvar = NdbThread_Create(async_log_func,
+                       (void**)&thread_args,
+                       0,
+                       (char*)"async_log_thread",
+                       NDB_THREAD_PRIO_MEAN);
+
 #ifdef _WIN32
   {
     char shutdown_event_name[32];
@@ -841,8 +1000,12 @@ ndbd_run(bool foreground, int report_fd,
       "Normal start of data node using checkpoint and log info if existing");
   }
 
+  log_memusage("init1");
+
   globalEmulatorData.create();
 
+  log_memusage("Emulator init");
+
   Configuration* theConfig = globalEmulatorData.theConfiguration;
   if(!theConfig->init(no_start, initial, initialstart))
   {
@@ -850,6 +1013,8 @@ ndbd_run(bool foreground, int report_fd,
     ndbd_exit(-1);
   }
 
+  log_memusage("Config init");
+
   /**
     Read the configuration from the assigned management server (could be
     a set of management servers, normally when we arrive here we have
@@ -874,6 +1039,8 @@ ndbd_run(bool foreground, int report_fd,
     // Ignore error
   }
 
+  log_memusage("Config fetch");
+
   theConfig->setupConfiguration();
 
 
@@ -893,6 +1060,8 @@ ndbd_run(bool foreground, int report_fd,
   */
   NdbThread* pWatchdog = globalEmulatorData.theWatchDog->doStart();
 
+  log_memusage("Watchdog started");
+
   g_eventLogger->info("Memory Allocation for global memory pools Starting");
   {
     /*
@@ -909,6 +1078,8 @@ ndbd_run(bool foreground, int report_fd,
   }
   g_eventLogger->info("Memory Allocation for global memory pools Completed");
 
+  log_memusage("Global memory pools allocated");
+
   /**
     Initialise the data of the run-time environment, this prepares the
     data setup for the various threads that need to communicate using
@@ -917,6 +1088,10 @@ ndbd_run(bool foreground, int report_fd,
   */
   globalEmulatorData.theThreadConfig->init();
 
+  globalEmulatorData.theConfiguration->addThread(log_threadvar, NdbfsThread);
+
+  log_memusage("Thread config initialised");
+
 #ifdef VM_TRACE
   // Initialize signal logger before block constructors
   char *signal_log_name = NdbConfig_SignalLogFileName(globalData.ownId);
@@ -947,7 +1122,7 @@ ndbd_run(bool foreground, int report_fd,
     ndbout_c("Failed to open signal logging file '%s', errno: %d",
              signal_log_name, errno);
   }
-  NdbMem_Free(signal_log_name);
+  free(signal_log_name);
 #endif
 
   /** Create all the blocks used by the run-time environment. */
@@ -955,6 +1130,8 @@ ndbd_run(bool foreground, int report_fd,
   // Load blocks (both main and workers)
   globalEmulatorData.theSimBlockList->load(globalEmulatorData);
 
+  log_memusage("Load blocks completed");
+
   catchsigs(foreground);
 
   /**
@@ -1046,13 +1223,26 @@ ndbd_run(bool foreground, int report_fd,
   globalEmulatorData.theConfiguration->removeThread(pWatchdog);
   globalEmulatorData.theConfiguration->removeThread(pTrp);
   globalEmulatorData.theConfiguration->removeThread(pSockServ);
+
   NdbShutdown(0, NST_Normal);
 
+  /**
+   * Stopping the log thread is done at the very end since the
+   * data node logs should be available until complete shutdown.
+   */
+  void* dummy_return_status;
+  thread_args.stop = true;
+  logBuf->stop();
+  NdbThread_WaitFor(log_threadvar, &dummy_return_status);
+  globalEmulatorData.theConfiguration->removeThread(log_threadvar);
+  NdbThread_Destroy(&log_threadvar);
+  delete logBuf;
+  delete ndbouts_bufferedoutputstream;
   ndbd_exit(0);
 }
 
 
-extern "C" my_bool opt_core;
+extern "C" bool opt_core;
 
 // instantiated and updated in NdbcntrMain.cpp
 extern Uint32 g_currentStartPhase;
@@ -1140,7 +1330,7 @@ NdbShutdown(int error_code,
       }
     }
 
-#ifndef NDB_WIN32
+#ifndef _WIN32
     if (simulate_error_during_shutdown)
     {
       kill(getpid(), simulate_error_during_shutdown);
@@ -1162,7 +1352,7 @@ NdbShutdown(int error_code,
      *   cause with ndbmtd, there are locks and nasty stuff
      *   and we don't know which we are holding...
      */
-#if NOT_YET
+#ifdef NOT_YET
 
     /**
      * Stop all transporter connection attempts and accepts
