@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -179,7 +179,8 @@ private:
   ndb_mgm_configuration * m_ownConfig;
   const class ConfigValues* get_own_config_values();
   ndb_mgm_configuration * m_clusterConfig;
-  UtilBuffer m_clusterConfigPacked;
+  UtilBuffer m_clusterConfigPacked_v1;
+  UtilBuffer m_clusterConfigPacked_v2;
 
   ndb_mgm_configuration_iterator * m_clusterConfigIter;
   ndb_mgm_configuration_iterator * m_ownConfigIterator;
