@@ -578,6 +578,7 @@ Ndbd_mem_manager::ndb_log2(Uint32 input)
 
 Ndbd_mem_manager::Ndbd_mem_manager()
 : m_base_page(NULL),
+  m_dump_on_alloc_fail(false),
   m_mapped_pages_count(0),
   m_mapped_pages_new_count(0)
 {
@@ -649,6 +650,48 @@ Ndbd_mem_manager::get_resource_limit_nolock(Uint32 id, Resource_limit& rl) const
   return false;
 }
 
+Uint32
+Ndbd_mem_manager::get_allocated() const
+{
+  return m_resource_limits.get_allocated();
+}
+
+Uint32
+Ndbd_mem_manager::get_reserved() const
+{
+  return m_resource_limits.get_reserved();
+}
+
+Uint32
+Ndbd_mem_manager::get_shared() const
+{
+  return m_resource_limits.get_shared();
+}
+
+Uint32
+Ndbd_mem_manager::get_spare() const
+{
+  return m_resource_limits.get_spare();
+}
+
+Uint32
+Ndbd_mem_manager::get_in_use() const
+{
+  return m_resource_limits.get_in_use();
+}
+
+Uint32
+Ndbd_mem_manager::get_reserved_in_use() const
+{
+  return m_resource_limits.get_reserved_in_use();
+}
+
+Uint32
+Ndbd_mem_manager::get_shared_in_use() const
+{
+  return m_resource_limits.get_shared_in_use();
+}
+
 int
 cmp_chunk(const void * chunk_vptr_1, const void * chunk_vptr_2)
 {
@@ -1156,7 +1199,18 @@ Ndbd_mem_manager::alloc(AllocZone zone,
       return;
     }
     if (z == 0)
+    {
+      if (unlikely(m_dump_on_alloc_fail))
+      {
+        printf("%s: Page allocation failed: zone=%u pages=%u (at least %u)\n",
+               __func__,
+               zone,
+               save,
+               min);
+        dump();
+      }
       return;
+    }
     * pages = save;
   }
 }
@@ -1325,6 +1379,12 @@ Ndbd_mem_manager::dump() const
   mt_mem_manager_unlock();
 }
 
+void
+Ndbd_mem_manager::dump_on_alloc_fail(bool on)
+{
+  m_dump_on_alloc_fail = on;
+}
+
 void
 Ndbd_mem_manager::lock()
 {
