@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -26,12 +33,46 @@
 
 #define JAM_FILE_ID 282
 
+/* Jam buffer pointer. */
+struct EmulatedJamBuffer;
+extern thread_local EmulatedJamBuffer* NDB_THREAD_TLS_JAM;
+
+/* Thread self pointer. */
+struct thr_data;
+extern thread_local thr_data* NDB_THREAD_TLS_THREAD;
+
+#ifdef NDB_DEBUG_RES_OWNERSHIP
+
+/* (Debug only) Shared resource owner. */
+extern thread_local Uint32 NDB_THREAD_TLS_RES_OWNER;
+
+#endif
+
+/**
+ * To enable jamDebug and its siblings in a production simply
+ * remove the comment and get EXTRA_JAM defined.
+ * It is enabled in builds using ERROR_INSERT to simplify tracing
+ * of bugs from autotest.
+ *
+ * Similarly enable initialisation of global variables in a block
+ * thread before executing each asynchronous signal by enabling
+ * USE_INIT_GLOBAL_VARIABLES. This is also enabled in all builds
+ * using ERROR_INSERT to ensure that we quickly discover failures
+ * in using global variables.
+ */
+#if defined(ERROR_INSERT)
+#define EXTRA_JAM 1
+#define USE_INIT_GLOBAL_VARIABLES 1
+#endif
 
 #ifdef NO_EMULATED_JAM
 
 #define jam()
 #define jamLine(line)
 #define jamEntry()
+#define jamDebug()
+#define jamLineDebug(line)
+#define jamEntryDebug()
 #define jamEntryLine(line)
 #define jamBlock(block)
 #define jamBlockLine(block, line)
@@ -43,6 +84,10 @@
 #define thrjamEntryLine(buf, line)
 #define thrjam(buf)
 #define thrjamLine(buf, line)
+#define thrjamEntryDebug(buf)
+#define thrjamEntryLineDebug(buf, line)
+#define thrjamDebug(buf)
+#define thrjamLineDebug(buf, line)
 
 #else
 
@@ -61,8 +106,7 @@
     jamBuffer->theEmulatedJam[jamIndex++] = JamEvent((JAM_FILE_ID), (line)); \
     jamBuffer->theEmulatedJamIndex = jamIndex & JAM_MASK; \
     /* Occasionally check that the jam buffer belongs to this thread.*/ \
-    assert((jamIndex & 3) != 0 || \
-           jamBuffer == NdbThread_GetTlsKey(NDB_THREAD_TLS_JAM));       \
+    assert((jamIndex & 3) != 0 || jamBuffer == NDB_THREAD_TLS_JAM);       \
     /* Occasionally check that jamFileNames[JAM_FILE_ID] matches __FILE__.*/ \
     assert((jamIndex & 0xff) != 0 ||                     \
            JamEvent::verifyId((JAM_FILE_ID), __FILE__)); \
@@ -79,8 +123,7 @@
 #define jamEntry() jamEntryLine(__LINE__)
 
 #define jamNoBlockLine(line) \
-    thrjamLine((EmulatedJamBuffer *)NdbThread_GetTlsKey(NDB_THREAD_TLS_JAM), \
-               (line))
+    thrjamLine(NDB_THREAD_TLS_JAM, line)
 #define jamNoBlock() jamNoBlockLine(__LINE__)
 
 #define thrjamEntryLine(buf, line) thrjamEntryBlockLine(buf, number(), line)
@@ -88,6 +131,23 @@
 #define thrjam(buf) thrjamLine(buf, __LINE__)
 #define thrjamEntry(buf) thrjamEntryLine(buf, __LINE__)
 
+#if defined VM_TRACE || defined ERROR_INSERT || defined EXTRA_JAM
+#define jamDebug() jam()
+#define jamLineDebug(line) jamLine(line)
+#define jamEntryDebug() jamEntry()
+#define thrjamEntryDebug(buf) thrjamEntry(buf)
+#define thrjamEntryLineDebug(buf, line) thrJamEntryLine(guf, line)
+#define thrjamDebug(buf) thrjam(buf)
+#define thrjamLineDebug(buf, line) thrjamLine(buf, line)
+#else
+#define jamDebug()
+#define jamLineDebug(line)
+#define jamEntryDebug()
+#define thrjamEntryDebug(buf)
+#define thrjamEntryLineDebug(buf, line)
+#define thrjamDebug(buf)
+#define thrjamLineDebug(buf, line)
+#endif
 #endif
 
 #ifndef NDB_OPT
@@ -203,6 +263,7 @@
 #define NO_LCP
 #define NO_GCP
 #endif
+#define ZUNDEFINED_GCI_LIMIT 1
 
 /**
  * Ndb kernel blocks assertion handling
@@ -222,7 +283,7 @@
   if(likely(check)){ \
   } else {     \
     jamNoBlock(); \
-    progError(__LINE__, NDBD_EXIT_NDBASSERT, __FILE__); \
+    progError(__LINE__, NDBD_EXIT_NDBASSERT, __FILE__, #check); \
   }
 #else
 #define ndbassert(check) do { } while(0)
@@ -232,12 +293,18 @@
   if(likely(check)){ \
   } else {     \
     jamNoBlock(); \
-    progError(__LINE__, error, __FILE__); \
+    progError(__LINE__, error, __FILE__, #check); \
   }
 
 #define ndbrequire(check) \
   ndbrequireErr(check, NDBD_EXIT_NDBREQUIRE)
 
+#define ndbabort() \
+  do { \
+    jamNoBlock(); \
+    progError(__LINE__, NDBD_EXIT_PRGERR, __FILE__, ""); \
+  } while (false)
+
 #define CRASH_INSERTION(errorType) \
   if (!ERROR_INSERTED((errorType))) { \
   } else { \
@@ -260,8 +327,7 @@
 // Get the jam buffer for the current thread.
 inline EmulatedJamBuffer* getThrJamBuf()
 {
-  return reinterpret_cast<EmulatedJamBuffer*>
-    (NdbThread_GetTlsKey(NDB_THREAD_TLS_JAM));
+  return NDB_THREAD_TLS_JAM;
 }
 
 #undef JAM_FILE_ID
