@@ -1,21 +1,30 @@
 /*
-  Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
 
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#ifdef TEST_MGMCONFIG
+
 #include <ndb_global.h>
 #include "InitConfigFileParser.hpp"
 #include "ConfigInfo.hpp"
@@ -143,7 +152,7 @@ create_config(const char* first, ...)
 }
 
 // Global variable for my_getopt
-extern "C" const char* my_defaults_file;
+extern const char* my_defaults_file;
 
 static
 unsigned
@@ -283,7 +292,7 @@ diff_config(void)
 }
 
 
-void
+static void
 print_restart_info(void)
 {
   Vector<const char*> initial_node;
@@ -389,14 +398,14 @@ test_param_values(void)
   {
     ndbout_c("testing %s", t->param);
     {
-      const Config* c =
-        create_config("[ndbd]", "NoOfReplicas=1",
+ const Config* c =
+   create_config("[ndbd]", "NoOfReplicas=1",
                       t->param,
-                      "[ndb_mgmd]", "HostName=localhost",
-                      "[mysqld]", NULL);
+                 "[ndb_mgmd]", "HostName=localhost",
+                 "[mysqld]", NULL);
       if (t->result)
       {
-        CHECK(c);
+  CHECK(c);
         delete c;
       }
       else
@@ -430,6 +439,7 @@ test_param_values(void)
 static void
 test_hostname_mycnf(void)
 {
+  ndbout_c("test_hostname_mycnf");
   // Check the special rule for my.cnf that says
   // the two hostname specs must match
   {
@@ -476,9 +486,9 @@ TAPTEST(MgmConfig)
   checksum_config();
   test_param_values();
   test_hostname_mycnf();
-#if 0
-  print_restart_info();
-#endif
+  if (false)
+    print_restart_info();
   ndb_end(0);
   return 1; // OK
 }
+#endif
