@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -37,11 +37,14 @@
 #include <NdbTick.h>
 #include <ProcessInfo.hpp>
 #include <OwnProcessInfo.hpp>
+#include "ndb_internal.hpp"
 
 #include <signaldata/NodeFailRep.hpp>
 #include <signaldata/NFCompleteRep.hpp>
 #include <signaldata/ApiRegSignalData.hpp>
 #include <signaldata/AlterTable.hpp>
+#include "kernel/signaldata/DumpStateOrd.hpp"
+#include "kernel/signaldata/TestOrd.hpp"
 #include <signaldata/SumaImpl.hpp>
 #include <signaldata/ProcessInfoRep.hpp>
 
@@ -389,8 +392,9 @@ ClusterMgr::threadMain()
 
     NodeFailRep * nodeFailRep = CAST_PTR(NodeFailRep,
                                          nodeFail_signal.getDataPtrSend());
+    Uint32 theAllNodes[NodeBitmask::Size];
     nodeFailRep->noOfNodes = 0;
-    NodeBitmask::clear(nodeFailRep->theAllNodes);
+    NodeBitmask::clear(theAllNodes);
 
     for (int i = 1; i < MAX_NODES; i++)
     {
@@ -460,7 +464,7 @@ ClusterMgr::threadMain()
       if (cm_node.hbMissed == 4 && cm_node.hbFrequency > 0)
       {
         nodeFailRep->noOfNodes++;
-        NodeBitmask::set(nodeFailRep->theAllNodes, nodeId);
+        NodeBitmask::set(theAllNodes, nodeId);
       }
     }
     flush_send_buffers();
@@ -469,7 +473,11 @@ ClusterMgr::threadMain()
     if (nodeFailRep->noOfNodes)
     {
       lock();
-      raw_sendSignal(&nodeFail_signal, getOwnNodeId());
+      LinearSectionPtr lsptr[3];
+      lsptr[0].p = theAllNodes;
+      lsptr[0].sz = NodeBitmask::getPackedLengthInWords(theAllNodes);
+
+      raw_sendSignal(&nodeFail_signal, getOwnNodeId(), lsptr, 1);
       flush_send_buffers();
       unlock();
     }
@@ -503,6 +511,10 @@ ClusterMgr::trp_deliver_signal(const NdbApiSignal* sig,
     execAPI_REGREF(theData);
     break;
 
+  case GSN_DUMP_STATE_ORD:
+    execDUMP_STATE_ORD(sig, ptr);
+    break;
+
   case GSN_NODE_FAILREP:
     execNODE_FAILREP(sig, ptr);
     break;
@@ -936,6 +948,166 @@ ClusterMgr::execAPI_REGREF(const Uint32 * theData){
   }
 }
 
+void
+ClusterMgr::execDUMP_STATE_ORD(const NdbApiSignal* signal,
+                               const LinearSectionPtr ptr[])
+{
+  const Uint32* data = signal->getDataPtr();
+  const Uint32 length = signal->getLength();
+  if (length < 1)
+  {
+    return;
+  }
+  switch (data[0])
+  {
+  case DumpStateOrd::CmvmiDummySignal:
+  {
+    /* Log in event logger that signal sent by dump command
+     * CmvmiSendDummySignal is received.  Include information about
+     * signal size and its sections and which node sent it.
+     *
+     * Use rep node as reporting node, typically a data node.
+     */
+    const Uint32 rep_node_id = data[1];
+    const Uint32 node_id = data[2];
+    const Uint32 num_secs = signal->m_noOfSections;
+    char msg[24*4];
+    snprintf(msg,
+             sizeof(msg),
+             "Receiving CmvmiDummySignal"
+             " (size %u+%u+%u+%u+%u) from %u to %u.",
+             length,
+             num_secs,
+             (num_secs > 0) ? ptr[0].sz : 0,
+             (num_secs > 1) ? ptr[1].sz : 0,
+             (num_secs > 2) ? ptr[2].sz : 0,
+             node_id,
+             getOwnNodeId());
+    const Uint32 len = strlen(msg) + 1;
+    assert(len <= 24*4);
+    NdbApiSignal aSignal(numberToRef(API_CLUSTERMGR, getOwnNodeId()));
+    aSignal.theTrace                = TestOrd::TraceAPI;
+    aSignal.theReceiversBlockNumber = CMVMI;
+    aSignal.theVerId_signalNumber   = GSN_EVENT_REP;
+    aSignal.theLength               = ((len + 3) / 4) + 1;
+    Uint32* data = aSignal.getDataPtrSend();
+    data[0] = NDB_LE_InfoEvent;
+    memcpy(&data[1], msg, len);
+    safe_sendSignal(&aSignal, rep_node_id);
+    return;
+  }
+  case DumpStateOrd::CmvmiSendDummySignal:
+  {
+    /* Send a CmvmiDummySignal to specified node with specified size and
+     * sections.  This is used to verify that messages with certain
+     * signal sizes and sections can be sent and received.
+     *
+     * The sending is also logged in event logger.  This log entry should
+     * be matched with corresponding log when receiving the
+     * CmvmiDummySignal dump command.  See preceding dump command above.
+     *
+     * args: rep-node dest-node padding frag-size
+     *       #secs sec#1-len sec#2-len sec#3-len
+     */
+    if (length < 5)
+    {
+      // Not enough words to send a dummy signal
+      return;
+    }
+    const Uint32 rep_node_id = data[1];
+    const Uint32 node_id = data[2];
+    const Uint32 fill_word = data[3];
+    const Uint32 frag_size = data[4];
+    if (frag_size != 0)
+    {
+      // Fragmented signals are not supported yet.
+      return;
+    }
+    const Uint32 num_secs = (length > 5) ? data[5] : 0;
+    if (num_secs > 3)
+    {
+      return;
+    }
+    Uint32 tot_len = length;
+    LinearSectionPtr ptr[3];
+    Uint32 sec_max_len = 0;
+    for (Uint32 i = 0; i < num_secs; i++)
+    {
+      const Uint32 sec_len = data[6 + i];
+      if (sec_len > sec_max_len)
+      {
+        sec_max_len = sec_len;
+      }
+      ptr[i].sz = sec_len;
+      tot_len += sec_len;
+    }
+    Uint32* dummy_data = new Uint32[sec_max_len];
+    for (Uint32 i = 0; i < sec_max_len; i++)
+    {
+      dummy_data[i] = fill_word;
+    }
+    for (Uint32 i = 0; i < num_secs; i++)
+    {
+      ptr[i].p = dummy_data;
+    }
+    for (Uint32 i = num_secs; i < 3; i++)
+    {
+      ptr[i].sz = 0;
+      ptr[i].p = NULL;
+    }
+    NdbApiSignal dummy_signal(numberToRef(API_CLUSTERMGR, getOwnNodeId()));
+    Uint32* dummy_sigdata = dummy_signal.getDataPtrSend();
+    dummy_sigdata[0] = DumpStateOrd::CmvmiDummySignal;
+    for (Uint32 i = 1; i < length; i++)
+    {
+      dummy_sigdata[i] = data[i];
+    }
+    dummy_sigdata[2] = getOwnNodeId();
+    dummy_signal.theVerId_signalNumber = GSN_DUMP_STATE_ORD;
+    const trp_node & theNode = theNodes[node_id];
+    dummy_signal.theReceiversBlockNumber =
+      (theNode.m_info.m_type == NodeInfo::DB)
+      ? CMVMI
+      : API_CLUSTERMGR;
+    dummy_signal.theTrace  = 0;
+    dummy_signal.theLength = length;
+    dummy_signal.m_noOfSections = num_secs;
+    safe_sendSignal(&dummy_signal, node_id, ptr, num_secs);
+    delete[] dummy_data;
+
+    /* Send event log about the sending of CmvmiDummySignal.
+     * Use rep node as reporting node, typically a data node.
+     */
+    char msg[24 * sizeof(Uint32)];
+    snprintf(msg,
+             sizeof(msg),
+             "Sending CmvmiDummySignal"
+             " (size %u+%u+%u+%u+%u) from %u to %u.",
+             length,
+             num_secs,
+             ptr[0].sz,
+             ptr[1].sz,
+             ptr[2].sz,
+             getOwnNodeId(),
+             node_id);
+    const Uint32 len = strlen(msg) + 1;
+    assert(len <= 24*4);
+    NdbApiSignal aSignal(numberToRef(API_CLUSTERMGR, getOwnNodeId()));
+    aSignal.theTrace                = TestOrd::TraceAPI;
+    aSignal.theReceiversBlockNumber = CMVMI;
+    aSignal.theVerId_signalNumber   = GSN_EVENT_REP;
+    aSignal.theLength               = (Uint32)((len+3)/4)+1;
+    Uint32* data = aSignal.getDataPtrSend();
+    data[0] = NDB_LE_InfoEvent;
+    memcpy(&data[1], msg, len);
+    safe_sendSignal(&aSignal, rep_node_id);
+    return;
+  }
+  default:
+    return;
+  }
+}
+
 void
 ClusterMgr::execNF_COMPLETEREP(const NdbApiSignal* signal,
                                const LinearSectionPtr ptr[3])
@@ -1124,14 +1296,19 @@ ClusterMgr::reportDisconnected(NodeId nodeId)
     signal.theReceiversBlockNumber = API_CLUSTERMGR;
     signal.theTrace  = 0;
     signal.theLength = NodeFailRep::SignalLengthLong;
+    signal.m_noOfSections = 1;
 
     NodeFailRep * rep = CAST_PTR(NodeFailRep, signal.getDataPtrSend());
+    Uint32 theAllNodes[NodeBitmask::Size];
     rep->failNo = 0;
     rep->masterNodeId = 0;
     rep->noOfNodes = 1;
-    NodeBitmask::clear(rep->theAllNodes);
-    NodeBitmask::set(rep->theAllNodes, nodeId);
-    execNODE_FAILREP(&signal, 0);
+    NodeBitmask::clear(theAllNodes);
+    NodeBitmask::set(theAllNodes, nodeId);
+    LinearSectionPtr lsptr[3];
+    lsptr[0].p = theAllNodes;
+    lsptr[0].sz = NodeBitmask::getPackedLengthInWords(theAllNodes);
+    execNODE_FAILREP(&signal, lsptr);
   }
 }
 
@@ -1141,13 +1318,18 @@ ClusterMgr::execNODE_FAILREP(const NdbApiSignal* sig,
 {
   const NodeFailRep * rep = CAST_CONSTPTR(NodeFailRep, sig->getDataPtr());
   NodeBitmask mask;
-  if (sig->getLength() == NodeFailRep::SignalLengthLong)
+  if (sig->getLength() == NodeFailRep::SignalLengthLong_v1)
   {
     mask.assign(NodeBitmask::Size, rep->theAllNodes);
   }
+  else if (sig->getLength() == NodeFailRep::SignalLength_v1)
+  {
+    mask.assign(NdbNodeBitmask48::Size, rep->theNodes);
+  }
   else
   {
-    mask.assign(NdbNodeBitmask::Size, rep->theNodes);
+    assert(sig->m_noOfSections == 1);
+    mask.assign(ptr[0].sz, ptr[0].p);
   }
 
   NdbApiSignal signal(sig->theSendersBlockRef);
@@ -1155,12 +1337,14 @@ ClusterMgr::execNODE_FAILREP(const NdbApiSignal* sig,
   signal.theReceiversBlockNumber = API_CLUSTERMGR;
   signal.theTrace  = 0;
   signal.theLength = NodeFailRep::SignalLengthLong;
+  signal.m_noOfSections = 1;
 
   NodeFailRep * copy = CAST_PTR(NodeFailRep, signal.getDataPtrSend());
   copy->failNo = 0;
   copy->masterNodeId = 0;
   copy->noOfNodes = 0;
-  NodeBitmask::clear(copy->theAllNodes);
+  Uint32 theAllNodes[NodeBitmask::Size];
+  NodeBitmask::clear(theAllNodes);
 
   for (Uint32 i = mask.find_first(); i != NodeBitmask::NotFound;
        i = mask.find_next(i + 1))
@@ -1175,7 +1359,7 @@ ClusterMgr::execNODE_FAILREP(const NdbApiSignal* sig,
     if (node_failrep == false)
     {
       theNode.m_node_fail_rep = true;
-      NodeBitmask::set(copy->theAllNodes, i);
+      NodeBitmask::set(theAllNodes, i);
       copy->noOfNodes++;
     }
 
@@ -1188,7 +1372,10 @@ ClusterMgr::execNODE_FAILREP(const NdbApiSignal* sig,
   recalcMinDbVersion();
   if (copy->noOfNodes)
   {
-    theFacade.for_each(this, &signal, 0); // report GSN_NODE_FAILREP
+    LinearSectionPtr lsptr[3];
+    lsptr[0].p = theAllNodes;
+    lsptr[0].sz = NodeBitmask::getPackedLengthInWords(theAllNodes);
+    theFacade.for_each(this, &signal, lsptr); // report GSN_NODE_FAILREP
   }
 
   if (noOfAliveNodes == 0)
