@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -655,6 +662,7 @@ NdbOperation::setAnyValue(Uint32 any_value)
       return 0;
     }
   }
+  // Fall through - let setValue call set error
   default:
     return setValue(impl, (const char *)&any_value);
   }
@@ -941,9 +949,9 @@ int
 NdbOperation::insertATTRINFO( Uint32 aData )
 {
   NdbApiSignal* tSignal;
-  register Uint32 tAI_LenInCurrAI = theAI_LenInCurrAI;
-  register Uint32* tAttrPtr = theATTRINFOptr;
-  register Uint32 tTotCurrAILen = theTotalCurrAI_Len;
+  Uint32 tAI_LenInCurrAI = theAI_LenInCurrAI;
+  Uint32* tAttrPtr = theATTRINFOptr;
+  Uint32 tTotCurrAILen = theTotalCurrAI_Len;
 
   if (tAI_LenInCurrAI >= 25) {
     Ndb* tNdb = theNdb;
@@ -997,9 +1005,9 @@ NdbOperation::insertATTRINFOloop(register const Uint32* aDataPtr,
 				 register Uint32 aLength)
 {
   NdbApiSignal* tSignal;
-  register Uint32 tAI_LenInCurrAI = theAI_LenInCurrAI;
-  register Uint32 tTotCurrAILen = theTotalCurrAI_Len;
-  register Uint32* tAttrPtr = theATTRINFOptr;  
+  Uint32 tAI_LenInCurrAI = theAI_LenInCurrAI;
+  Uint32 tTotCurrAILen = theTotalCurrAI_Len;
+  Uint32* tAttrPtr = theATTRINFOptr;
   Ndb* tNdb = theNdb;
 
   while (aLength > 0) {
@@ -1025,7 +1033,7 @@ NdbOperation::insertATTRINFOloop(register const Uint32* aDataPtr,
       }//if
     }//if
     {
-      register Uint32 tData = *aDataPtr;
+      Uint32 tData = *aDataPtr;
       aDataPtr++;
       aLength--;
       tAI_LenInCurrAI++;
@@ -1110,6 +1118,33 @@ NdbOperation::prepareGetLockHandleNdbRecord()
   return 0;
 }
 
+int
+NdbOperation::setNoWait()
+{
+  if (theStatus == UseNdbRecord)
+  {
+    /**
+     * Method not allowed for NdbRecord, use OperationOptions or
+     * ScanOptions structure instead
+     */
+    setErrorCodeAbort(4515);
+    return -1;
+  }
+
+  if ((! ((theOperationType == ReadRequest) ||
+          (theOperationType == ReadExclusive))) ||
+      theDirtyIndicator)
+  {
+    /* Only allowed for locking reads */
+    setErrorCodeAbort(4108); /* Faulty operation type */
+    return -1;
+  }
+
+  m_flags |= OF_NOWAIT;
+
+  return 0;
+}
+
 /*
  * handleOperationOptions
  * static member for setting operation options
@@ -1414,5 +1449,18 @@ NdbOperation::handleOperationOptions (const OperationType type,
     op->m_flags |= OF_DISABLE_FK;
   }
 
+  if (opts->optionsPresent & OperationOptions::OO_NOWAIT)
+  {
+    if ((! ((type == ReadRequest) ||
+            (type == ReadExclusive))) ||
+        (op->theLockMode == LM_CommittedRead))
+    {
+      /* Only allowed for locking reads */
+      return 4108; /* Faulty operation type */
+    }
+
+    op->m_flags |= OF_NOWAIT;
+  }
+
   return 0;
 }
