@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -48,19 +48,23 @@ public:
   virtual void trp_deliver_signal(const NdbApiSignal *,
                                   const LinearSectionPtr ptr[3]) = 0;
   virtual void trp_wakeup()
-    {};
+    {}
 
-  Uint32 open(class TransporterFacade*, int blockNo = -1,
-              bool receive_thread = false);
+  Uint32 open(class TransporterFacade*, int blockNo = -1);
   void close();
 
-  void start_poll();
+  void prepare_poll();
   void do_poll(Uint32);
   void complete_poll();
   void wakeup();
 
+  // Called under m_mutex protection
+  void set_enabled_send(const NodeBitmask &nodes);
+  void enable_send(NodeId node);
+  void disable_send(NodeId node);
+  
   void flush_send_buffers();
-  int do_forceSend(int val = 1);
+  int do_forceSend(bool forceSend = true);
 
   int raw_sendSignal(const NdbApiSignal*, Uint32 nodeId);
   int raw_sendSignal(const NdbApiSignal*, Uint32 nodeId,
@@ -75,13 +79,13 @@ public:
   const trp_node & getNodeInfo(Uint32 i) const;
 
   virtual void recordWaitTimeNanos(Uint64 nanos)
-    {};
+    {}
 
   void lock();
   void unlock();
   /* Interface used by Multiple NDB waiter code */
   void lock_client();
-  bool check_if_locked(void);
+  bool check_if_locked(void) const;
 
   Uint32 getOwnNodeId() const;
 
@@ -93,6 +97,8 @@ public:
    * This variant does flush thread-local send-buffer
    */
   int safe_sendSignal(const NdbApiSignal*, Uint32 nodeId);
+  int safe_sendSignal(const NdbApiSignal*, Uint32 nodeId,
+                      const LinearSectionPtr ptr[3], Uint32 secs);
 
   /**
    * This sendSignal variant can be called on any trp_client
@@ -102,12 +108,19 @@ public:
    * This variant does not flush thread-local send-buffer
    */
   int safe_noflush_sendSignal(const NdbApiSignal*, Uint32 nodeId);
+  int safe_noflush_sendSignal(const NdbApiSignal*, Uint32 nodeId,
+                              const LinearSectionPtr ptr[3], Uint32 secs);
+
 private:
   /**
    * TransporterSendBufferHandle interface
    */
-  virtual Uint32 *getWritePtr(NodeId node, Uint32 lenBytes, Uint32 prio,
-                              Uint32 max_use);
+  virtual bool isSendEnabled(NodeId node) const;
+  virtual Uint32 *getWritePtr(NodeId node,
+                              Uint32 lenBytes,
+                              Uint32 prio,
+                              Uint32 max_use,
+                              SendStatus* error);
   virtual Uint32 updateWritePtr(NodeId node, Uint32 lenBytes, Uint32 prio);
   virtual void getSendBufferLevel(NodeId node, SB_LevelType &level);
   virtual bool forceSend(NodeId node);
@@ -117,25 +130,49 @@ private:
   TransporterFacade * m_facade;
 
   /**
-   * This is used for polling
+   * This is used for polling by the poll_owner:
+   *   A client is 'locked_for_poll' iff it is registered in the 
+   *   m_locked_clients[] array by the poll owner.
+   *   'm_locked_for_poll' also implies 'm_mutex' is locked
    */
   bool m_locked_for_poll;
+  bool m_is_receiver_thread;
 public:
+  bool is_receiver_thread()
+  {
+    return m_is_receiver_thread;
+  }
   NdbMutex* m_mutex; // thread local mutex...
   void set_locked_for_poll(bool val)
   {
     m_locked_for_poll = val;
   }
-  bool is_locked_for_poll()
+  bool is_locked_for_poll() const
   {
     return m_locked_for_poll;
   }
+
+  bool has_unflushed_sends() const
+  {
+    return m_send_nodes_cnt > 0;
+  }
 private:
   struct PollQueue
   {
     PollQueue();
-    void assert_destroy() const;
+    ~PollQueue();
 
+    /**
+     * PQ_WAITING - trp_client either waits in the poll-queue, or
+     *              owns the poll-right. In both cases it waits 
+     *              for a ::wakeup() in order to proceed.
+     * PQ_WOKEN -   trp_client got a ::wakeup() while PQ_WAITING.
+     *              Might be waiting in the poll-queue, needing to
+     *              be signaled, or trp_client is the poller, which
+     *              now can complete.
+     * PQ_IDLE -    This *PollQueue instance* is idle/unused.
+     *              (Thus, the client itself is likely active)
+     */
     enum { PQ_WOKEN, PQ_IDLE, PQ_WAITING } m_waiting;
     bool m_locked;
     bool m_poll_owner;
@@ -143,28 +180,27 @@ private:
     trp_client *m_prev;
     trp_client *m_next;
     NdbCondition * m_condition;
-
-    /**
-     * This is called by poll owner
-     *   before doing external poll
-     */
-    void start_poll(trp_client* self);
-
-    void lock_client(trp_client*);
-    bool check_if_locked(const trp_client*,
-                         const Uint32 start) const;
-    Uint32 m_locked_cnt;
-    Uint32 m_lock_array_size;
-    trp_client ** m_locked_clients;
   } m_poll;
 
+  NodeBitmask m_enabled_nodes_mask;
+
   /**
-   * This is used for sending
+   * This is used for sending.
+   * m_send_nodes_* are the nodes we have pending unflushed
+   * messages to
    */
+  NodeBitmask m_send_nodes_mask;
   Uint32 m_send_nodes_cnt;
   NodeId m_send_nodes_list[MAX_NODES];
-  NodeBitmask m_send_nodes_mask;
+
   TFBuffer* m_send_buffers;
+
+  /**
+   * The m_flushed_nodes_mask are the aggregated set of nodes
+   * we have flushed messages to, which are yet not known to
+   * to have been (force-)sent by the transporter.
+   */
+  NodeBitmask m_flushed_nodes_mask;
 };
 
 class PollGuard
@@ -174,11 +210,11 @@ public:
   ~PollGuard() { unlock_and_signal(); }
   int wait_n_unlock(int wait_time, Uint32 nodeId, Uint32 state,
                     bool forceSend= false);
-  int wait_for_input_in_loop(int wait_time, bool forceSend);
   void wait_for_input(int wait_time);
   int wait_scan(int wait_time, Uint32 nodeId, bool forceSend);
   void unlock_and_signal();
 private:
+  int wait_for_input_in_loop(int wait_time, bool forceSend);
   class trp_client* m_clnt;
   class NdbWaiter *m_waiter;
   bool  m_complete_poll_called;
@@ -188,9 +224,9 @@ private:
 
 inline
 bool
-trp_client::check_if_locked()
+trp_client::check_if_locked() const
 {
-  return m_facade->m_poll_owner->m_poll.check_if_locked(this, (Uint32)0);
+  return m_facade->check_if_locked(this, 0);
 }
 
 inline
@@ -199,7 +235,7 @@ trp_client::lock_client()
 {
   if (!check_if_locked())
   {
-    m_facade->m_poll_owner->m_poll.lock_client(this);
+    m_facade->lock_client(this);
   }
 }
 
@@ -216,7 +252,7 @@ inline
 void
 trp_client::unlock()
 {
-  assert(m_send_nodes_mask.isclear()); // Nothing unsent when unlocking...
+  assert(has_unflushed_sends() == false); //Nothing unsent when unlocking...
   assert(m_poll.m_locked == true);
   m_poll.m_locked = false;
   NdbMutex_Unlock(m_mutex);
@@ -274,43 +310,4 @@ trp_client::raw_sendFragmentedSignal(const NdbApiSignal * signal, Uint32 nodeId,
   return m_facade->sendFragmentedSignal(this, signal, nodeId, ptr, secs);
 }
 
-inline
-trp_client::PollQueue::PollQueue()
-{
-  m_waiting = PQ_IDLE;
-  m_locked = false;
-  m_poll_owner = false;
-  m_poll_queue = false;
-  m_next = 0;
-  m_prev = 0;
-  m_condition = NdbCondition_Create();
-  m_locked_cnt = 0;
-  m_lock_array_size = 0;
-  m_locked_clients = NULL;
-}
-
-inline
-void
-trp_client::PollQueue::assert_destroy() const
-{
-  if (m_waiting != PQ_IDLE ||
-      m_locked == true ||
-      m_poll_owner == true ||
-      m_poll_queue == true ||
-      m_next != 0 ||
-      m_prev != 0 ||
-      m_locked_cnt != 0)
-  {
-    ndbout << "ERR: ~trp_client: Deleting trp_clnt in use: waiting"
-	   << m_waiting
-	   << " locked  " << m_locked
-	   << " poll_owner " << m_poll_owner
-	   << " poll_queue " << m_poll_queue
-	   << " next " << m_next
-	   << " prev " << m_prev
-	   << " condition " << m_condition << endl;
-    require(false);
-  }
-}
-
 #endif
