@@ -1,5 +1,5 @@
 /*
- Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2010, 2017, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -906,6 +906,20 @@ struct NdbApiWrapper {
         obj.removeCachedIndex(p0, p1);
     }
 
+    static void
+    NdbDictionary__Dictionary__invalidateTable__0 // disambiguate overloaded function
+    ( NdbDictionary::Dictionary & obj, const char * p0 )
+    {
+        obj.invalidateTable(p0);
+    }
+
+    static void
+    NdbDictionary__Dictionary__invalidateIndex__1 // disambiguate overloaded function
+    ( NdbDictionary::Dictionary & obj, const char * p0, const char * p1 )
+    {
+        obj.invalidateIndex(p0, p1);
+    }
+
     static int
     NdbDictionary__Dictionary__createIndex__0 // disambiguate overloaded function
     ( NdbDictionary::Dictionary & obj, const NdbDictionary::Index & p0, bool p1 )
@@ -3513,6 +3527,13 @@ struct NdbApiWrapper {
         obj.set_name(p0);
     }
 
+    static void
+    Ndb_cluster_connection__set_service_uri
+    ( Ndb_cluster_connection & obj, const char * p0, const char * p1, int p2, const char * p3)
+    {
+        obj.set_service_uri(p0, p1, p2, p3);
+    }
+
     static int
     Ndb_cluster_connection__set_timeout
     ( Ndb_cluster_connection & obj, int p0 )
@@ -3548,6 +3569,34 @@ struct NdbApiWrapper {
         obj.unlock_ndb_objects();
     }
 
+    static int
+    Ndb_cluster_connection__set_recv_thread_activation_threshold
+    ( Ndb_cluster_connection & obj, int p0 )
+    {
+        return obj.set_recv_thread_activation_threshold(p0);
+    }
+
+    static int
+    Ndb_cluster_connection__get_recv_thread_activation_threshold
+    ( Ndb_cluster_connection & obj )
+    {
+        return obj.get_recv_thread_activation_threshold();
+    }
+
+    static int
+    Ndb_cluster_connection__set_recv_thread_cpu
+    ( Ndb_cluster_connection & obj, short p0 )
+    {
+        return obj.set_recv_thread_cpu(p0);
+    }
+
+    static int
+    Ndb_cluster_connection__unset_recv_thread_cpu
+    ( Ndb_cluster_connection & obj )
+    {
+        return obj.unset_recv_thread_cpu(0);
+    }
+
     static const Ndb *
     Ndb_cluster_connection__get_next_ndb_object
     ( Ndb_cluster_connection & obj, const Ndb * p0 )
@@ -3555,6 +3604,13 @@ struct NdbApiWrapper {
         return obj.get_next_ndb_object(p0);
     }
 
+    static const char *
+    Ndb_cluster_connection__get_system_name
+    ( const Ndb_cluster_connection & obj )
+    {
+        return obj.get_system_name();
+    }
+
 // ---------------------------------------------------------------------------
 
 };
