@@ -163,20 +163,27 @@ static
 int
 setupUDPartitioning(Ndb* ndb, NdbDictionary::Table& tab)
 {
-  /* Following should really be taken from running test system : */
-  const Uint32 numNodes= ndb->get_ndb_cluster_connection().no_db_nodes();
-  const Uint32 numReplicas= 2; // Assumption
-  const Uint32 guessNumNgs= numNodes/2;
-  const Uint32 numNgs= guessNumNgs?guessNumNgs : 1;
-  const Uint32 numFragsPerNode= 2 + (rand() % 3);
-  const Uint32 numPartitions= numReplicas * numNgs * numFragsPerNode;
+  NdbRestarter restarter;
+  Vector<int> node_groups;
+  int max_alive_replicas;
+  if (restarter.getNodeGroups(node_groups, &max_alive_replicas) == -1)
+  {
+    return -1;
+  }
+
+  const Uint32 numNgs = node_groups.size();
+
+  // Assume at least one node group had all replicas alive.
+  const Uint32 numReplicas = max_alive_replicas;
+  const Uint32 numFragsPerNode = 2 + (rand() % 3);
+  const Uint32 numPartitions = numReplicas * numNgs * numFragsPerNode;
 
   tab.setFragmentType(NdbDictionary::Table::UserDefined);
   tab.setFragmentCount(numPartitions);
   tab.setPartitionBalance(NdbDictionary::Object::PartitionBalance_Specific);
-  for (Uint32 i=0; i<numPartitions; i++)
+  for (Uint32 i = 0; i < numPartitions; i++)
   {
-    frag_ng_mappings[i]= i % numNgs;
+    frag_ng_mappings[i] = node_groups[i % numNgs];
   }
   tab.setFragmentData(frag_ng_mappings, numPartitions);
 
