@@ -1,18 +1,25 @@
-/* Copyright (C) 2008 MySQL AB
-   Use is subject to license terms
+/*
+  Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <NDBT.hpp>
 #include <NDBT_Test.hpp>
@@ -99,7 +106,8 @@ runTestSingleUserMode(NDBT_Context* ctx, NDBT_Step* step)
   Ndb* pNdb = GETNDB(step);
   NdbRestarter restarter;
   char tabName[255];
-  strncpy(tabName, ctx->getTab()->getName(), 255);
+  strncpy(tabName, ctx->getTab()->getName(), sizeof(tabName) - 1);
+  tabName[sizeof(tabName) - 1] = '\0';
   ndbout << "tabName="<<tabName<<endl;
 
   int i = 0;
@@ -116,6 +124,7 @@ runTestSingleUserMode(NDBT_Context* ctx, NDBT_Step* step)
     CHECK(restarter.restartOneDbNode(nodeId) != 0)
     CHECK(restarter.exitSingleUserMode() == 0);
     CHECK(restarter.waitClusterStarted(timeout) == 0);
+    CHK_NDB_READY(pNdb);
 
     // Test that the single user mode api can do everything
     CHECK(restarter.enterSingleUserMode(pNdb->getNodeId()) == 0);
@@ -132,6 +141,7 @@ runTestSingleUserMode(NDBT_Context* ctx, NDBT_Step* step)
     CHECK(utilTrans.clearTable(pNdb, records/2) == 0);
     CHECK(restarter.exitSingleUserMode() == 0);
     CHECK(restarter.waitClusterStarted(timeout) == 0);
+    CHK_NDB_READY(pNdb);
 
     // Test create index in single user mode
     CHECK(restarter.enterSingleUserMode(pNdb->getNodeId()) == 0);
@@ -146,6 +156,7 @@ runTestSingleUserMode(NDBT_Context* ctx, NDBT_Step* step)
     CHECK(drop_index_on_pk(pNdb, tabName) == 0);
     CHECK(restarter.exitSingleUserMode() == 0);
     CHECK(restarter.waitClusterStarted(timeout) == 0);
+    CHK_NDB_READY(pNdb);
 
     // Test recreate index in single user mode
     CHECK(create_index_on_pk(pNdb, tabName) == 0);
@@ -158,6 +169,7 @@ runTestSingleUserMode(NDBT_Context* ctx, NDBT_Step* step)
     CHECK(create_index_on_pk(pNdb, tabName) == 0);
     CHECK(restarter.exitSingleUserMode() == 0);
     CHECK(restarter.waitClusterStarted(timeout) == 0);
+    CHK_NDB_READY(pNdb);
     CHECK(drop_index_on_pk(pNdb, tabName) == 0);
 
     CHECK(utilTrans.clearTable(GETNDB(step),  records) == 0);
@@ -180,7 +192,7 @@ TESTCASE("SingleUserMode",
   INITIALIZER(runTestSingleUserMode);
   FINALIZER(runClearTable);
 }
-NDBT_TESTSUITE_END(testSingleUserMode);
+NDBT_TESTSUITE_END(testSingleUserMode)
 
 
 int main(int argc, const char** argv){
