@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -26,9 +33,14 @@
 #include <ndb_version.h>
 #include <random.h>
 #include <NdbMutex.h>
+#include <NdbEnv.h>
+#include <signaldata/DumpStateOrd.hpp>
 
 static Vector<BaseString> table_list;
 
+static Uint32 preVersion = 0;
+static Uint32 postVersion = 0;
+
 struct NodeInfo
 {
   int nodeId;
@@ -36,6 +48,23 @@ struct NodeInfo
   int nodeGroup;
 };
 
+static
+const char* getBaseDir()
+{
+  /* Atrt basedir exposed via env var MYSQL_HOME */
+  const char* envHome = NdbEnv_GetEnv("MYSQL_HOME", NULL, 0);
+
+  if (envHome == NULL)
+  {
+    ndbout_c("testUpgrade getBaseDir() MYSQL_HOME not set");
+    return "./";
+  }
+  else
+  {
+    return envHome;
+  }
+}
+
 int CMT_createTableHook(Ndb* ndb,
                         NdbDictionary::Table& table,
                         int when,
@@ -54,7 +83,11 @@ int CMT_createTableHook(Ndb* ndb,
                          num);
     table.setName(buf);
     if (fragCount > 0)
+    {
       table.setFragmentCount(fragCount);
+      table.setPartitionBalance(
+        NdbDictionary::Object::PartitionBalance_Specific);
+    }
     
     ndbout << "Creating " << buf 
            << " with fragment count " << fragCount 
@@ -397,7 +430,7 @@ uint getNodeCount(NodeSet set, uint numNodes)
     abort();
     return 0;
   }
-};
+}
 
 
 /**
@@ -435,27 +468,23 @@ int runUpgrade_NR1(NDBT_Context* ctx, NDBT_Step* step){
       return NDBT_FAILED;
     
     uint mgmdCount = mgmds.numRows();
+    uint mgmd_start_count = mgmdCount;
     uint restartCount = getNodeCount(mgmdNodeSet, mgmdCount);
-    
-    ndbout << "Restarting "
-             << restartCount << " of " << mgmdCount
-             << " mgmds" << endl;
       
-    while (mgmds.next() && restartCount --)
+    while (mgmds.next() && mgmdCount --)
     {
-      ndbout << "Restart mgmd " << mgmds.columnAsInt("node_id") << endl;
-      if (!atrt.changeVersion(mgmds.columnAsInt("id"), ""))
+      ndbout << "Restart mgmd" << mgmds.columnAsInt("node_id") << endl;
+      if (!atrt.stopProcess(mgmds.columnAsInt("id")) ||
+          !atrt.switchConfig(mgmds.columnAsInt("id"),"--initial"))
+        return NDBT_FAILED;
+    }
+    mgmds.reset();
+    while (mgmds.next() && mgmd_start_count --)
+    {
+      ndbout << "Restart mgmd" << mgmds.columnAsInt("node_id") << endl;
+      if (!atrt.startProcess(mgmds.columnAsInt("id")))
         return NDBT_FAILED;
-      
-      if (restarter.waitConnected())
-        return NDBT_FAILED;
-      ndbout << "Connected to mgmd"<< endl;
     }
-    
-    ndbout << "Waiting for started"<< endl;
-    if (restarter.waitClusterStarted())
-      return NDBT_FAILED;
-    ndbout << "Started"<< endl;
     
     // Restart ndbd(s)
     SqlResultSet ndbds;
@@ -635,6 +664,8 @@ runUpgrade_Half(NDBT_Context* ctx, NDBT_Step* step)
     if (restarter.waitClusterStarted())
       return NDBT_FAILED;
 
+    CHK_NDB_READY(GETNDB(step));
+
     if (event && createDropEvent(ctx, step))
     {
       return NDBT_FAILED;
@@ -690,6 +721,8 @@ runUpgrade_Half(NDBT_Context* ctx, NDBT_Step* step)
     if (restarter.waitClusterStarted())
       return NDBT_FAILED;
 
+    CHK_NDB_READY(GETNDB(step));
+
     if (event && createDropEvent(ctx, step))
     {
       return NDBT_FAILED;
@@ -766,6 +799,22 @@ int runUpgrade_NdbdFirst(NDBT_Context* ctx, NDBT_Step* step)
 */
 int runUpgrade_NotAllMGMD(NDBT_Context* ctx, NDBT_Step* step)
 {
+  NdbRestarter restarter;
+  int minMgmVer = 0;
+  int maxMgmVer = 0;
+  int myVer = NDB_VERSION;
+
+  if (restarter.getNodeTypeVersionRange(NDB_MGM_NODE_TYPE_MGM,
+                                        minMgmVer,
+                                        maxMgmVer) == -1)
+  {
+    g_err << "runUpgrade_NotAllMGMD: getNodeTypeVersionRange call failed" << endl;
+  }
+
+  g_err << "runUpgrade_NotAllMGMD: My version " << myVer
+        << " Min mgm version " << minMgmVer
+        << " Max mgm version " << maxMgmVer << endl;
+
   ctx->setProperty("MgmdNodeSet", (Uint32) NodeSet(NotAll));
   ctx->setProperty("NdbdNodeSet", (Uint32) NodeSet(None));
   int res = runUpgrade_Half(ctx, step);
@@ -1103,7 +1152,7 @@ runBug14702377(NDBT_Context* ctx, NDBT_Step* step)
       int msec = (int)(t2-t1);
       const NdbError& txerr = pTx->getNdbError();
       const NdbError& operr = pOp->getNdbError();
-      char info2[200];
+      char info2[300];
       sprintf(info2, "%s msec=%d ret=%d txerr=%d operr=%d",
                       info1, msec, ret, txerr.code, operr.code);
       g_info << "PK read T1 done: " << info2 << endl;
@@ -1310,7 +1359,9 @@ runPostUpgradeChecks(NDBT_Context* ctx, NDBT_Step* step)
       ndbout_c("waitClusterStarted() failed");
       return NDBT_FAILED;
     }
-    
+
+    CHK_NDB_READY(pNdb);
+
     if (pDict->getTable("I3") == 0)
     {
       ndbout_c("Table disappered");
@@ -1560,11 +1611,412 @@ runUpgrade_SR(NDBT_Context* ctx, NDBT_Step* step)
 }
 
 
+static
+int
+runStartBlockLcp(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbRestarter restarter;
+
+  restarter.setReconnect(true);
+
+  
+  while ((ctx->getProperty("HalfStartedDone", (Uint32)0) == 0) &&
+         !ctx->isTestStopped())
+  {
+    ndbout << "runStartBlockLcp: waiting for half nodes to be restarted..." << endl;
+    NdbSleep_MilliSleep(5000);
+  }
+
+  if (ctx->isTestStopped())
+  {
+    return NDBT_FAILED;
+  }
+
+  ndbout << "Half of the nodes restarted, beginning slow LCPs for remainder..." << endl;
+
+  /* Trigger LCPs which will be slow to complete, 
+   * testing more complex LCP takeover protocols
+   * especially when the last 'old' data node 
+   * (likely to be DIH Master) fails.
+   * */
+  do
+  {
+    int dumpCode[] = { 7099 };
+    while (restarter.dumpStateAllNodes(dumpCode, 1) != 0) {};
+
+    /* Stall fragment completions */
+    while (restarter.insertErrorInAllNodes(5073) != 0) {};
+
+    /* Allow restarts to continue... */
+    ctx->setProperty("HalfStartedHold", Uint32(0));
+
+    /**
+     * Only stall for 20s to avoid default LCP frag
+     * watchdog timeouts
+     */
+    NdbSleep_MilliSleep(20000);
+    
+    ndbout << "Unblocking LCP..." << endl;
+    while (restarter.insertErrorInAllNodes(0) != 0) 
+    {};
+
+    NdbSleep_MilliSleep(5000);
+    
+  } while (!ctx->isTestStopped());
+
+  return NDBT_OK;
+}
+
+static
+int
+runUpgradeAndFail(NDBT_Context* ctx, NDBT_Step* step)
+{
+  AtrtClient atrt;
+  SqlResultSet clusters;
+
+  if (!atrt.getClusters(clusters))
+    return NDBT_FAILED;
+
+  // Get the first cluster
+  clusters.next();
+
+  uint clusterId= clusters.columnAsInt("id");
+  SqlResultSet tmp_result;
+  if (!atrt.getConnectString(clusterId, tmp_result))
+    return NDBT_FAILED;
+
+  NdbRestarter restarter(tmp_result.column("connectstring"));
+  restarter.setReconnect(true); // Restarting mgmd
+  ndbout << "Cluster '" << clusters.column("name")
+              << "@" << tmp_result.column("connectstring") << "'" << endl;
+
+  if (restarter.waitClusterStarted())
+    return NDBT_FAILED;
+
+  // Restart ndb_mgmd(s)
+  SqlResultSet mgmds;
+  if (!atrt.getMgmds(clusterId, mgmds))
+    return NDBT_FAILED;
+
+  uint mgmdCount = mgmds.numRows();
+  uint restartCount = mgmdCount;
+
+  ndbout << "Restarting "
+      << restartCount << " of " << mgmdCount
+      << " mgmds" << endl;
+
+  while (mgmds.next() && restartCount --)
+  {
+    ndbout << "Restart mgmd " << mgmds.columnAsInt("node_id") << endl;
+    if (!atrt.changeVersion(mgmds.columnAsInt("id"), ""))
+      return NDBT_FAILED;
+
+    if (restarter.waitConnected())
+      return NDBT_FAILED;
+    ndbout << "Connected to mgmd"<< endl;
+  }
+
+  ndbout << "Waiting for started"<< endl;
+  if (restarter.waitClusterStarted())
+    return NDBT_FAILED;
+  ndbout << "Started"<< endl;
+
+  // Restart one ndbd
+  SqlResultSet ndbds;
+  if (!atrt.getNdbds(clusterId, ndbds))
+    return NDBT_FAILED;
+
+  //Get the node id of first node
+  ndbds.next();
+  int nodeId = ndbds.columnAsInt("node_id");
+  int processId = ndbds.columnAsInt("id");
+
+  ndbout << "Restart node " << nodeId << endl;
+  if (!atrt.changeVersion(processId, "--initial=0"))
+  {
+    g_err << "Unable to change version of data node" << endl;
+    return NDBT_FAILED;
+  }
+
+  if (restarter.waitNodesNoStart(&nodeId, 1))
+  {
+    g_err << "The newer version of the node never came up" << endl;
+    return NDBT_FAILED;
+  }
+
+  ndbout << "Node changed version and in NOSTART state" << endl;
+  ndbout << "Arranging for start failure to cause return to NOSTART" << endl;
+
+  /* We need the node to go to NO START after crash.  */
+  int restartDump[] = { DumpStateOrd::CmvmiSetRestartOnErrorInsert, 1 };
+  if (restarter.dumpStateOneNode(nodeId, restartDump, 2))
+    return NDBT_FAILED;
+
+  /* 1007 forces the node to crash instead of failing with
+   * NDBD_EXIT_UPGRADE_INITIAL_REQUIRED */
+  restarter.insertErrorInNode(nodeId, 1007);
+
+  int origConnectCount = restarter.getNodeConnectCount(nodeId);
+
+  ndbout << "Starting node" << endl;
+  /* Now start it, should fail */
+  restarter.startAll();
+
+  ndbout << "Waiting for node to return to NOSTART state" << endl;
+
+  int newConnectCount = origConnectCount;
+
+  while (newConnectCount == origConnectCount)
+  {
+    /* Wait for the node to go to no start */
+    if (restarter.waitNodesNoStart(&nodeId, 1))
+    {
+      g_err << "Node never crashed" << nodeId << endl;
+      return NDBT_FAILED;
+    }
+
+    newConnectCount = restarter.getNodeConnectCount(nodeId);
+  }
+
+  ndbout << "Node reconnected and returned to NOSTART state "
+         << "due to start failure"
+         << endl;
+
+  return NDBT_OK;
+}
+
+static
+int
+runShowVersion(NDBT_Context* ctx, NDBT_Step* step)
+{
+  const bool postUpgrade =
+    (ctx->getProperty("PostUpgrade", Uint32(0))) != 0;
+
+  ndbout_c("*******************************");
+  ndbout_c("%s version",
+           (postUpgrade?"Post":"Pre"));
+
+  ndbout_c("  Mysql version : %u.%u.%u%s",
+           MYSQL_VERSION_MAJOR,
+           MYSQL_VERSION_MINOR,
+           MYSQL_VERSION_PATCH,
+           MYSQL_VERSION_EXTRA);
+  ndbout_c("  Ndb version   : %u.%u.%u %s",
+           NDB_VERSION_MAJOR,
+           NDB_VERSION_MINOR,
+           NDB_VERSION_BUILD,
+           NDB_VERSION_STATUS);
+  ndbout_c("*******************************");
+
+  if (!postUpgrade)
+  {
+    ndbout_c("  Note that this test may fail if Post version does"
+             "  not implement ShowVersions, this is not a bug");
+  }
+  return NDBT_OK;
+}
+
+static const char* preVersionFileName = "preVersion.txt";
+static const char* postVersionFileName = "postVersion.txt";
+
+static BaseString
+getFileName(bool post)
+{
+  /* Path is 1 level *above* basedir, treated as 'run' directory */
+  BaseString nameBuff;
+  nameBuff.assfmt("%s/../%s",
+                  getBaseDir(),
+                  (post?
+                   postVersionFileName:
+                   preVersionFileName));
+  return nameBuff;
+}
+
+static int
+runRecordVersion(NDBT_Context* ctx, NDBT_Step* step)
+{
+  const bool postUpgrade =
+    (ctx->getProperty("PostUpgrade", Uint32(0))) != 0;
+  BaseString fileNameBuf = getFileName(postUpgrade);
+  const char* fileName = fileNameBuf.c_str();
+
+  ndbout_c("Writing version info to %s", fileName);
+
+  FILE* versionFile = fopen(fileName, "w");
+  if (!versionFile)
+  {
+    ndbout_c("Error opening file %s",
+             fileName);
+    return NDBT_FAILED;
+  }
+  fprintf(versionFile,
+          "%u.%u.%u\n",
+           NDB_VERSION_MAJOR,
+           NDB_VERSION_MINOR,
+           NDB_VERSION_BUILD);
+  fclose(versionFile);
+
+  return NDBT_OK;
+}
+
+static
+bool
+readVersionFile(const char* fileName,
+                Uint32& version)
+{
+  bool ok = true;
+  version = 0;
+
+  ndbout_c("Reading version info from %s", fileName);
+
+  FILE* versionFile = fopen(fileName, "r");
+  if (versionFile != NULL)
+  {
+    unsigned int major = 0;
+    unsigned int minor = 0;
+    unsigned int build = 0;
+    int res = fscanf(versionFile,
+                     "%u.%u.%u",
+                     &major,
+                     &minor,
+                     &build);
+    if (res == 3)
+    {
+      version = NDB_MAKE_VERSION(major, minor, build);
+    }
+    else
+    {
+      ndbout_c("Version file %s read failed res = %d",
+               fileName,
+               res);
+      ok = false;
+    }
+
+    fclose(versionFile);
+  }
+
+  return ok;
+}
+
+/**
+ * runReadVersions
+ *
+ * Read, print and store version info captured by previous test
+ * run including runRecordVersion on both versions
+ *
+ * This can allow version specific conditions to be included
+ * in tests.
+ */
+static int
+runReadVersions(NDBT_Context* ctx, NDBT_Step* step)
+{
+  preVersion = postVersion = 0;
+
+  const bool preVersionReadOk =
+    readVersionFile(getFileName(false).c_str(),
+                    preVersion);
+  const bool postVersionReadOk =
+    readVersionFile(getFileName(true).c_str(),
+                    postVersion);
+
+  ndbout_c("runReadVersions");
+  ndbout_c("  PreVersion %s  : %u.%u.%u",
+           (preVersionReadOk?"read":"not found"),
+           ndbGetMajor(preVersion),
+           ndbGetMinor(preVersion),
+           ndbGetBuild(preVersion));
+  ndbout_c("  PostVersion %s : %u.%u.%u",
+           (postVersionReadOk?"read":"not found"),
+           ndbGetMajor(postVersion),
+           ndbGetMinor(postVersion),
+           ndbGetBuild(postVersion));
+
+  return NDBT_OK;
+}
+
+/**
+ * runSkipIfCannotKeepFS
+ *
+ * Check whether we can perform an upgrade while keeping the
+ * FS with the versions being tested.
+ * If not, skip
+ */
+static int
+runSkipIfCannotKeepFS(NDBT_Context* ctx, NDBT_Step* step)
+{
+  if (preVersion == 0 || postVersion == 0)
+  {
+    ndbout_c("Missing version info to determine whether to skip, running.");
+    return NDBT_OK;
+  }
+
+  const Uint32 problemBoundary = NDB_MAKE_VERSION(7,6,3);  // NDBD_LOCAL_SYSFILE_VERSION
+
+  if (versionsSpanBoundary(preVersion, postVersion, problemBoundary))
+  {
+    ndbout_c("Cannot run with these versions as they do not support "
+                  "non initial upgrades.");
+    return NDBT_SKIPPED;
+  }
+  return NDBT_OK;
+}
+
+/**
+ * runSkipIfPostCanKeepFS
+ *
+ * Will skip unless the post version cannot handle a
+ * 'keepFS' upgrade.
+ */
+static int
+runSkipIfPostCanKeepFS(NDBT_Context* ctx, NDBT_Step* step)
+{
+  if (preVersion == 0 ||
+      postVersion == 0)
+  {
+    ndbout_c("Not running as specific version info missing");
+    return NDBT_SKIPPED;
+  }
+
+  const bool upgrade = (postVersion > preVersion);
+
+  if (!upgrade)
+  {
+    /* Only  support upgrade test, downgrade is not handled cleanly attm */
+    ndbout_c("Not running as not an upgrade");
+    return NDBT_SKIPPED;
+  }
+
+  const Uint32 problemBoundary = NDB_MAKE_VERSION(7,6,3); // NDBD_LOCAL_SYSFILE_VERSION
+
+  if (!versionsSpanBoundary(preVersion, postVersion, problemBoundary))
+  {
+    ndbout_c("Not running as versions involved are not relevant for this test");
+    return NDBT_SKIPPED;
+  }
+  return NDBT_OK;
+}
+
 
 NDBT_TESTSUITE(testUpgrade);
+TESTCASE("ShowVersions",
+         "Upgrade API, showing actual versions run")
+{
+  INITIALIZER(runCheckStarted);
+  STEP(runShowVersion);
+  STEP(runRecordVersion);
+  VERIFIER(startPostUpgradeChecks);
+}
+POSTUPGRADE("ShowVersions")
+{
+  TC_PROPERTY("PostUpgrade", Uint32(1));
+  INITIALIZER(runCheckStarted);
+  STEP(runShowVersion);
+  STEP(runRecordVersion);
+};
 TESTCASE("Upgrade_NR1",
 	 "Test that one node at a time can be upgraded"){
   INITIALIZER(runCheckStarted);
+  INITIALIZER(runReadVersions);
   INITIALIZER(runBug48416);
   STEP(runUpgrade_NR1);
   VERIFIER(startPostUpgradeChecks);
@@ -1601,6 +2053,8 @@ TESTCASE("Upgrade_FS",
 {
   TC_PROPERTY("KeepFS", 1);
   INITIALIZER(runCheckStarted);
+  INITIALIZER(runReadVersions);
+  INITIALIZER(runSkipIfCannotKeepFS);
   INITIALIZER(runCreateAllTables);
   INITIALIZER(runLoadAll);
   STEP(runUpgrade_Traffic);
@@ -1632,6 +2086,8 @@ TESTCASE("Upgrade_Traffic_FS",
   TC_PROPERTY("UseRangeScanT1", (Uint32)1);
   TC_PROPERTY("KeepFS", 1);
   INITIALIZER(runCheckStarted);
+  INITIALIZER(runReadVersions);
+  INITIALIZER(runSkipIfCannotKeepFS);
   INITIALIZER(runCreateAllTables);
   STEP(runUpgrade_Traffic);
   STEP(runBasic);
@@ -1661,6 +2117,8 @@ TESTCASE("Upgrade_Traffic_FS_one",
 {
   TC_PROPERTY("KeepFS", 1);
   INITIALIZER(runCheckStarted);
+  INITIALIZER(runReadVersions);
+  INITIALIZER(runSkipIfCannotKeepFS);
   INITIALIZER(runCreateOneTable);
   STEP(runUpgrade_Traffic);
   STEP(runBasic);
@@ -1763,6 +2221,8 @@ TESTCASE("Upgrade_SR_ManyTablesMaxFrag",
   TC_PROPERTY("SkipMgmds", Uint32(1)); /* For 7.0.14... */
   TC_PROPERTY("FragmentCount", ~Uint32(0));
   INITIALIZER(runCheckStarted);
+  INITIALIZER(runReadVersions);
+  INITIALIZER(runSkipIfCannotKeepFS);
   INITIALIZER(createManyTables);
   STEP(runUpgrade_SR);
   VERIFIER(startPostUpgradeChecks);
@@ -1773,8 +2233,37 @@ POSTUPGRADE("Upgrade_SR_ManyTablesMaxFrag")
   INITIALIZER(runPostUpgradeChecks);
   INITIALIZER(dropManyTables);
 }
-  
-NDBT_TESTSUITE_END(testUpgrade);
+TESTCASE("Upgrade_NR3_LCP_InProgress",
+         "Check that half-cluster upgrade with LCP in progress is ok")
+{
+  TC_PROPERTY("HalfStartedHold", Uint32(1)); /* Stop half way through */
+  INITIALIZER(runCheckStarted);
+  STEP(runStartBlockLcp);
+  STEP(runUpgrade_NR3);
+  /* No need for postUpgrade, and cannot rely on it existing for
+   * downgrades...
+   * Better solution needed for downgrades where postUpgrade is
+   * useful, e.g. RunIfPresentElseIgnore...
+   */
+  //VERIFIER(startPostUpgradeChecks);
+}
+//POSTUPGRADE("Upgrade_NR3_LCP_InProgress")
+//{
+//  INITIALIZER(runCheckStarted);
+//  INITIALIZER(runPostUpgradeChecks);
+//}
+TESTCASE("Upgrade_Newer_LCP_FS_Fail",
+         "Try upgrading a data node from any lower version to 7.6.4 and fail."
+         "7.6.4 has a newer LCP file system and requires a upgrade with initial."
+         "(Bug#27308632)")
+{
+  INITIALIZER(runCheckStarted);
+  INITIALIZER(runReadVersions);
+  INITIALIZER(runSkipIfPostCanKeepFS);
+  STEP(runUpgradeAndFail);
+  // No postupgradecheck required as the upgrade is expected to fail
+}
+NDBT_TESTSUITE_END(testUpgrade)
 
 int main(int argc, const char** argv){
   ndb_init();
