@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -126,7 +126,9 @@ UtilTransactions::clearTable(Ndb* pNdb,
       } while((check = pOp->nextResult(false)) == 0);
       
       if(check != -1){
-	check = pTrans->execute(Commit, AbortOnError);   
+	check = pTrans->execute(Commit, AbortOnError);
+        if (check != -1)
+          pTrans->getGCI(&m_latest_gci);
 	pTrans->restart();
       }
       
@@ -135,7 +137,10 @@ UtilTransactions::clearTable(Ndb* pNdb,
 	if(err.status == NdbError::TemporaryError){
 	  NDB_ERR(err);
 	  closeTransaction(pNdb);
-	  NdbSleep_MilliSleep(50);
+          if (err.code == 410 || err.code == 1501)
+	    NdbSleep_MilliSleep(2000);
+          else
+	    NdbSleep_MilliSleep(50);
 	  par = 1;
 	  goto restart;
 	}
@@ -281,6 +286,8 @@ UtilTransactions::copyTableData(Ndb* pNdb,
       } while((eof = pOp->nextResult(false)) == 0);
       
       check = pTrans->execute(Commit, AbortOnError);   
+      if (check != -1)
+        pTrans->getGCI(&m_latest_gci);
       pTrans->restart();
       if( check == -1 ) {
 	const NdbError err = pTrans->getNdbError();    
@@ -1124,7 +1131,7 @@ UtilTransactions::verifyOrderedIndex(Ndb* pNdb,
       return NDBT_FAILED;
     }
         
-    int eof;
+    int eof = 0;
     int rows = 0;
     while(check == 0 && (eof = pOp->nextResult()) == 0){
       rows++;
@@ -1303,6 +1310,7 @@ UtilTransactions::getOperation(NdbConnection* pTrans,
         abort();
       }
     }
+    return pTrans->getNdbOperation(tab.getName());
   case NdbOperation::InsertRequest:
   case NdbOperation::WriteRequest:
     return pTrans->getNdbOperation(tab.getName());
