@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,22 +28,25 @@
 #include <NdbApi.hpp>
 #include <NdbSleep.h>
 
+#include "my_alloc.h"
+
 void desc_AutoGrowSpecification(struct NdbDictionary::AutoGrowSpecification ags);
-int desc_logfilegroup(Ndb *myndb, char* name);
-int desc_undofile(Ndb_cluster_connection &con, Ndb *myndb, char* name);
-int desc_datafile(Ndb_cluster_connection &con, Ndb *myndb, char* name);
-int desc_tablespace(Ndb *myndb,char* name);
-int desc_table(Ndb *myndb,char* name);
-int desc_hashmap(Ndb_cluster_connection &con, Ndb *myndb, char* name);
+int desc_logfilegroup(Ndb *myndb, char const* name);
+int desc_undofile(Ndb_cluster_connection &con, Ndb *myndb, char const* name);
+int desc_datafile(Ndb_cluster_connection &con, Ndb *myndb, char const* name);
+int desc_tablespace(Ndb *myndb, char const* name);
+int desc_index(Ndb *myndb, char const* name);
+int desc_table(Ndb *myndb,char const* name);
+int desc_hashmap(Ndb_cluster_connection &con, Ndb *myndb, char const* name);
 
 static const char* _dbname = "TEST_DB";
+static const char* _tblname = NULL;
 static int _unqualified = 0;
 static int _partinfo = 0;
 static int _blobinfo = 0;
+static int _indexinfo = 0;
 static int _nodeinfo = 0;
 
-const char *load_default_groups[]= { "mysql_cluster",0 };
-
 static int _retries = 0;
 
 static struct my_option my_long_options[] =
@@ -54,50 +57,42 @@ static struct my_option my_long_options[] =
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
   { "unqualified", 'u', "Use unqualified table names",
     (uchar**) &_unqualified, (uchar**) &_unqualified, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 }, 
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "extra-partition-info", 'p', "Print more info per partition",
     (uchar**) &_partinfo, (uchar**) &_partinfo, 0,
-    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 }, 
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "retries", 'r', "Retry every second for # retries",
     (uchar**) &_retries, (uchar**) &_retries, 0,
-    GET_INT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 }, 
-  { "blob-info", 'b', "Show information for hidden blob tables (requires -p)",
+    GET_INT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
+  { "blob-info", 'b', "Show information for hidden blob tables",
     (uchar**) &_blobinfo, (uchar**) &_blobinfo, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "extra-node-info", 'n', "Print node info for partitions (requires -p)",
     (uchar**) &_nodeinfo, (uchar**) &_nodeinfo, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "index-info", 'i', "Show information for indexes",
+    (uchar**) &_indexinfo, (uchar**) &_indexinfo, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "table", 't', "Base table for index",
+    (uchar**) &_tblname, (uchar**) &_tblname, 0,
+    GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
-static void short_usage_sub(void)
-{
-  ndb_short_usage_sub(NULL);
-}
-
-static void usage()
-{
-  ndb_usage(short_usage_sub, load_default_groups, my_long_options);
-}
-
-static void print_part_info(Ndb* pNdb, NDBT_Table* pTab);
+static void print_part_info(Ndb* pNdb, NdbDictionary::Table const* pTab);
 
 int main(int argc, char** argv){
   NDB_INIT(argv[0]);
-
-  ndb_opt_set_usage_funcs(short_usage_sub, usage);
-  load_defaults("my",load_default_groups,&argc,&argv);
-  int ho_error;
+  Ndb_opts opts(argc, argv, my_long_options);
 #ifndef DBUG_OFF
   opt_debug= "d:t:O,/tmp/ndb_desc.trace";
 #endif
-  if ((ho_error=handle_options(&argc, &argv, my_long_options, 
-			       ndb_std_get_one_option)))
+  if (opts.handle_options())
     return NDBT_ProgramExit(NDBT_WRONGARGS);
 
   Ndb_cluster_connection con(opt_ndb_connectstring, opt_ndb_nodeid);
   con.set_name("ndb_desc");
-  if(con.connect(12, 5, 1) != 0)
+  if(con.connect(opt_connect_retries - 1, opt_connect_retry_delay, 1) != 0)
   {
     ndbout << "Unable to connect to management server." << endl;
     return NDBT_ProgramExit(NDBT_FAILED);
@@ -110,13 +105,15 @@ int main(int argc, char** argv){
 
   Ndb MyNdb(&con, _dbname);
   if(MyNdb.init() != 0){
-    ERR(MyNdb.getNdbError());
+    NDB_ERR(MyNdb.getNdbError());
     return NDBT_ProgramExit(NDBT_FAILED);
   }
 
   for(int i= 0; i<argc;i++)
   {
-    if(desc_table(&MyNdb,argv[i]))
+    if (desc_index(&MyNdb, argv[i]))
+      ;
+    else if(desc_table(&MyNdb, argv[i]))
       ;
     else if(desc_tablespace(&MyNdb,argv[i]))
       ;
@@ -143,10 +140,10 @@ void desc_AutoGrowSpecification(struct NdbDictionary::AutoGrowSpecification ags)
   ndbout << "AutoGrow.filename_pattern: " << ags.filename_pattern << endl;
 }
 
-int desc_logfilegroup(Ndb *myndb, char* name)
+int desc_logfilegroup(Ndb *myndb, char const* name)
 {
   NdbDictionary::Dictionary *dict= myndb->getDictionary();
-  assert(dict);
+  require(dict);
   NdbDictionary::LogfileGroup lfg= dict->getLogfileGroup(name);
   NdbError err= dict->getNdbError();
   if( (int) err.classification != (int) ndberror_cl_none)
@@ -165,10 +162,10 @@ int desc_logfilegroup(Ndb *myndb, char* name)
   return 1;
 }
 
-int desc_tablespace(Ndb *myndb, char* name)
+int desc_tablespace(Ndb *myndb, char const* name)
 {
   NdbDictionary::Dictionary *dict= myndb->getDictionary();
-  assert(dict);
+  require(dict);
   NdbDictionary::Tablespace ts= dict->getTablespace(name);
   NdbError err= dict->getNdbError();
   if ((int) err.classification != (int) ndberror_cl_none)
@@ -183,13 +180,13 @@ int desc_tablespace(Ndb *myndb, char* name)
   return 1;
 }
 
-int desc_undofile(Ndb_cluster_connection &con, Ndb *myndb, char* name)
+int desc_undofile(Ndb_cluster_connection &con, Ndb *myndb, char const* name)
 {
   unsigned id;
   NdbDictionary::Dictionary *dict= myndb->getDictionary();
   Ndb_cluster_connection_node_iter iter;
 
-  assert(dict);
+  require(dict);
 
   con.init_get_next_node(iter);
 
@@ -220,11 +217,11 @@ int desc_undofile(Ndb_cluster_connection &con, Ndb *myndb, char* name)
   return 1;
 }
 
-int desc_datafile(Ndb_cluster_connection &con, Ndb *myndb, char* name)
+int desc_datafile(Ndb_cluster_connection &con, Ndb *myndb, char const* name)
 {
   unsigned id;
   NdbDictionary::Dictionary *dict= myndb->getDictionary();
-  assert(dict);
+  require(dict);
   Ndb_cluster_connection_node_iter iter;
 
   con.init_get_next_node(iter);
@@ -255,65 +252,87 @@ int desc_datafile(Ndb_cluster_connection &con, Ndb *myndb, char* name)
   return 1;
 }
 
-int desc_table(Ndb *myndb, char* name)
+int desc_index(Ndb *myndb, char const* name)
 {
   NdbDictionary::Dictionary * dict= myndb->getDictionary();
-  NDBT_Table* pTab;
-  while ((pTab = (NDBT_Table*)dict->getTable(name)) == NULL && --_retries >= 0) NdbSleep_SecSleep(1);
+  NdbDictionary::Index const* pIndex;
+
+  /* need to know base table */
+  if (_tblname == NULL)
+    return 0;
+
+  while ((pIndex = dict->getIndex(name, _tblname)) == NULL && --_retries >= 0)
+    NdbSleep_SecSleep(1);
+  if (pIndex == NULL)
+    return 0;
+
+  ndbout << "-- " << _tblname << "/" << pIndex->getName() << " --" << endl;
+  dict->print(ndbout, *pIndex);
+  return 1;
+}
+
+int desc_table(Ndb *myndb, char const* name)
+{
+  NdbDictionary::Dictionary * dict= myndb->getDictionary();
+  NdbDictionary::Table const* pTab;
+  while ((pTab = dict->getTable(name)) == NULL && --_retries >= 0) NdbSleep_SecSleep(1);
   if (!pTab)
     return 0;
 
-  ndbout << (* pTab) << endl;
-
-  NdbDictionary::Dictionary::List list;
-  if (dict->listIndexes(list, name) != 0){
-    ndbout << name << ": " << dict->getNdbError() << endl;
-    return NDBT_ProgramExit(NDBT_FAILED);
-  }
-
-  ndbout << "-- Indexes -- " << endl;
-  ndbout << "PRIMARY KEY(";
-  unsigned j;
-  for (j= 0; (int)j < pTab->getNoOfPrimaryKeys(); j++)
-  {
-    const NdbDictionary::Column * col= pTab->getColumn(pTab->getPrimaryKey(j));
-    ndbout << col->getName();
-    if ((int)j < pTab->getNoOfPrimaryKeys()-1)
-      ndbout << ", ";
-  }
-  ndbout << ") - UniqueHashIndex" << endl;
-  for (j= 0; j < list.count; j++) {
-    NdbDictionary::Dictionary::List::Element& elt = list.elements[j];
-    const NdbDictionary::Index *pIdx = dict->getIndex(elt.name, name);
-    if (!pIdx){
-      ndbout << name << ": " << dict->getNdbError() << endl;
-      return NDBT_ProgramExit(NDBT_FAILED);
-    }
-
-    ndbout << (*pIdx) << endl;
-  }
-  ndbout << endl;
+  ndbout << "-- " << pTab->getName() << " --" << endl;
+  dict->print(ndbout, *pTab);
 
   if (_partinfo)
   {
     print_part_info(myndb, pTab);
     ndbout << endl;
-    if (_blobinfo)
+  }
+
+  if (_indexinfo)
+  {
+    NdbDictionary::Dictionary::List list;
+    if (dict->listIndexes(list, *pTab) != -1)
     {
-      int noOfAttributes = pTab->getNoOfColumns();
-      for (int i = 0; i < noOfAttributes; i++)
+      list.sortByName();
+      _tblname = name;
+      for (unsigned i = 0; i < list.count; i++)
       {
-        const NdbDictionary::Column* column = pTab->getColumn(i);
-        if ((column->getType() == NdbDictionary::Column::Blob) || 
-          (column->getType() == NdbDictionary::Column::Text))
-        {
-          print_part_info(myndb, (NDBT_Table*) column->getBlobTable());
-          ndbout << endl;
-        }
+        NdbDictionary::Dictionary::List::Element& elt = list.elements[i];
+        desc_index(myndb, elt.name);
+        ndbout << endl;
       }
     }
   }
-	
+
+  if (_blobinfo)
+  {
+    int noOfAttributes = pTab->getNoOfColumns();
+    for (int i = 0; i < noOfAttributes; i++)
+    {
+      const NdbDictionary::Column* column = pTab->getColumn(i);
+      if ((column->getType() == NdbDictionary::Column::Blob) ||
+        (column->getType() == NdbDictionary::Column::Text))
+      {
+        NDBT_Table * blobTable= (NDBT_Table*) column->getBlobTable();
+
+        if(blobTable) /* blob table present */
+        {
+          /* The check is added because TINYBLOB/TINYTEXT columns do not
+           * have blob tables , in which case the variable blobTable is NULL.
+           * print_part_info is therefore called only for non-NULL values of
+           * blobTable.  */
+          desc_table(myndb, blobTable->getName());
+        }
+        else if(column -> getPartSize() > 0) /* blob table not present and part size greater than 0 */
+        {
+          ndbout << "Error: Blob table for column \"" << column -> getName() << "\" is not present" << endl;
+        }
+
+        ndbout << endl;
+      }
+    }
+  }
+
   return 1;
 }
 
@@ -326,7 +345,7 @@ struct InfoInfo
 
 
 static 
-void print_part_info(Ndb* pNdb, NDBT_Table* pTab)
+void print_part_info(Ndb* pNdb, NdbDictionary::Table const* pTab)
 {
   InfoInfo g_part_info[] = {
     { "Partition", 0, NdbDictionary::Column::FRAGMENT },
@@ -369,7 +388,11 @@ void print_part_info(Ndb* pNdb, NDBT_Table* pTab)
     if (pOp == NULL)
       break;
     
-    int rs = pOp->readTuples(NdbOperation::LM_CommittedRead); 
+    int rs = pOp->readTuples(NdbOperation::LM_CommittedRead,
+                             0 /* scan_flags */,
+                             1 /* parallel */,
+                             0 /* batch */);
+
     if (rs != 0)
       break;
     
@@ -451,10 +474,10 @@ void print_part_info(Ndb* pNdb, NDBT_Table* pTab)
   pTrans->close();
 }
 
-int desc_hashmap(Ndb_cluster_connection &con, Ndb *myndb, char* name)
+int desc_hashmap(Ndb_cluster_connection &con, Ndb *myndb, char const* name)
 {
   NdbDictionary::Dictionary *dict= myndb->getDictionary();
-  assert(dict);
+  require(dict);
 
   NdbDictionary::HashMap hm;
   if (dict->getHashMap(hm, name) == 0)
