@@ -62,13 +62,6 @@ enum TableChangesMask
   TCM_ATTRIBUTE_DEMOTION = 0x4
 };
 
-inline
-bool
-isDrop6(Uint32 version)
-{
-  return (getMajor(version) == 5 && getMinor(version) == 2);
-}
-
 typedef NdbDictionary::Table NDBTAB;
 typedef NdbDictionary::Column NDBCOL;
 typedef  void* (*AttrConvertFunc)(const void *old_data, 
@@ -421,6 +414,7 @@ public:
   Uint32 getObjType(Uint32 i) const { return m_objects[i].m_objType; }
   void* getObjPtr(Uint32 i) const { return m_objects[i].m_objPtr; }
   
+  Uint32 getStartGCP() const;
   Uint32 getStopGCP() const;
   Uint32 getNdbVersion() const { return m_fileHeader.NdbVersion; }
 }; // RestoreMetaData
@@ -514,6 +508,13 @@ public:
 };
 
 class RestoreLogIterator : public BackupFile {
+  /* The BackupFile buffer need to be big enough for biggest log entry data,
+   * not including log entry header.
+   * No harm in require space for a few extra words to header too.
+   */
+  static_assert(BackupFile::BUFFER_SIZE >=
+                  BackupFormat::LogFile::LogEntry::MAX_SIZE,
+                "");
 private:
   const RestoreMetaData & m_metaData;
 
@@ -524,6 +525,10 @@ public:
   RestoreLogIterator(const RestoreMetaData &);
   virtual ~RestoreLogIterator() {}
 
+  bool isSnapshotstartBackup()
+  {
+    return m_is_undolog;
+  }
   const LogEntry * getNextLogEntry(int & res);
 };
 
