@@ -23,15 +23,16 @@
 */
 
 #include <Logger.hpp>
-#include <NDBT_ReturnCodes.h>
 #include <NdbOut.hpp>
 #include <NdbTCP.h>
+#include <NdbToolsProgramExitCodes.hpp>
 #include <OutputStream.hpp>
 #include <Properties.hpp>
 #include <Vector.hpp>
 #include <ndb_global.h>
 #include <ndb_limits.h>
 #include <ndb_opts.h>
+#include <ndb_version.h>
 
 #include "../src/ndbapi/NdbDictionaryImpl.hpp"
 #include "consumer_printer.hpp"
@@ -108,6 +109,8 @@ Vector<class RestoreOption *> g_include_exclude;
 static void save_include_exclude(int optid, char * argument);
 
 static inline void parse_rewrite_database(char * argument);
+static void exitHandler(int code);
+static void free_include_excludes_vector();
 
 /**
  * print and restore flags
@@ -486,7 +489,7 @@ get_one_option(int optid, const struct my_option *opt MY_ATTRIBUTE((unused)),
     if (ga_nodeId == 0)
     {
       err << "Error in --nodeid,-n setting, see --help";
-      exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+      exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
     }
     info.setLevel(254);
     info << "Nodeid = " << ga_nodeId << endl;
@@ -495,7 +498,7 @@ get_one_option(int optid, const struct my_option *opt MY_ATTRIBUTE((unused)),
     if (ga_backupId == 0)
     {
       err << "Error in --backupid,-b setting, see --help";
-      exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+      exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
     }
     info.setLevel(254);
     info << "Backup Id = " << ga_backupId << endl;
@@ -512,7 +515,7 @@ get_one_option(int optid, const struct my_option *opt MY_ATTRIBUTE((unused)),
     if (analyse_nodegroup_map(opt_nodegroup_map_str,
                               &opt_nodegroup_map[0]))
     {
-      exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+      exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
     }
     break;
   case OPT_INCLUDE_DATABASES:
@@ -567,7 +570,7 @@ processTableList(const char* str, Vector<BaseString> &lst)
     if (makeInternalTableName(tmp[i], internalName))
     {
       info << "`" << tmp[i] << "` is not a valid tablename!" << endl;
-      exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+      exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
     }
     lst.push_back(internalName);
   }
@@ -588,7 +591,7 @@ makeExternalTableName(const BaseString &internalName)
 }
 
 // Exclude the legacy list of six privilege tables from Cluster 7.x
-#include "sql/ndb_dist_priv_util.h"
+#include "storage/ndb/plugin/ndb_dist_priv_util.h"
 void
 exclude_privilege_tables()
 {
@@ -618,17 +621,17 @@ readArguments(Ndb_opts & opts, char*** pargv)
 
   if (opts.handle_options(get_one_option))
   {
-    exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+    exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
   }
   if (ga_nodeId == 0)
   {
     err << "Backup file node ID not specified, please provide --nodeid" << endl;
-    exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+    exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
   }
   if (ga_backupId == 0)
   {
     err << "Backup ID not specified, please provide --backupid" << endl;
-    exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+    exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
   }
 
 
@@ -659,7 +662,7 @@ o verify nodegroup mapping
   for (i = 0; i < 8 ; i++)
     printf("NG %u mapped to %u \n", i >> 1, map_ng[i]);
   }
-  exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+  exit(NdbToolsProgramExitCode::WRONG_ARGS);
 #endif
 
   for (;;)
@@ -1044,7 +1047,7 @@ static void parse_rewrite_database(char * argument)
 
   info << "argument `" << arg.c_str()
        << "` is not a pair 'a,b' of non-empty names." << endl;
-  exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+  exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
 }
 
 static void save_include_exclude(int optid, char * argument)
@@ -1064,7 +1067,7 @@ static void save_include_exclude(int optid, char * argument)
       if (makeInternalTableName(args[i], arg))
       {
         info << "`" << args[i] << "` is not a valid tablename!" << endl;
-        exit(NDBT_ProgramExit(NDBT_WRONGARGS));
+        exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
       }
       break;
     default:
@@ -1186,9 +1189,13 @@ checkDbAndTableName(const TableS* table)
     return false;
   }
 
-  // If new options are given, ignore the old format
-  if (opt_include_tables || g_exclude_tables.size() > 0 ||
-      opt_include_databases || opt_exclude_databases ) {
+  // If --exclude lists are given, check them
+  if ((g_exclude_tables.size() || opt_exclude_databases)
+       && ! checkDoRestore(table))
+    return false;
+
+  // If --include lists are given, ignore the old-style table list
+  if (opt_include_tables || opt_include_databases ) {
     return (checkDoRestore(table));
   }
   
@@ -1285,7 +1292,6 @@ free_include_excludes_vector()
 static void exitHandler(int code)
 {
   free_include_excludes_vector();
-  NDBT_ProgramExit(code);
   if (opt_core)
     abort();
   else
@@ -1303,15 +1309,15 @@ static void init_restore()
     if (g_cluster_connection == NULL)
     {
       err << "Failed to create cluster connection!!" << endl;
-      exitHandler(NDBT_FAILED);
+      exitHandler(NdbToolsProgramExitCode::FAILED);
     }
     g_cluster_connection->set_name(g_options.c_str());
-    if(g_cluster_connection->connect(opt_connect_retries - 1,
+    if (g_cluster_connection->connect(opt_connect_retries - 1,
             opt_connect_retry_delay, 1) != 0)
     {
       delete g_cluster_connection;
       g_cluster_connection = NULL;
-      exitHandler(NDBT_FAILED);
+      exitHandler(NdbToolsProgramExitCode::FAILED);
     }
   }
 }
@@ -1403,7 +1409,7 @@ int do_restore(RestoreThreadData *thrdata)
   if (!metaData.readHeader())
   {
     restoreLogger.log_error("Failed to read %s", metaData.getFilename());
-    return NDBT_FAILED;
+    return NdbToolsProgramExitCode::FAILED;
   }
 
   const BackupFormat::FileHeader & tmp = metaData.getFileHeader();
@@ -1417,7 +1423,7 @@ int do_restore(RestoreThreadData *thrdata)
   {
   restoreLogger.log_info("Backup version in files: %s ndb version: %s",
            ndbGetVersionString(version, 0,
-                               isDrop6(version) ? "-drop6" : 0,
+                               ndbd_drop6(version) ? "-drop6" : 0,
                                buf, sizeof(buf)),
            ndbGetVersionString(tmp.NdbVersion, tmp.MySQLVersion, 0,
                                 buf, sizeof(buf)));
@@ -1426,7 +1432,7 @@ int do_restore(RestoreThreadData *thrdata)
   {
     restoreLogger.log_info("Backup version in files: %s",
            ndbGetVersionString(version, 0,
-                               isDrop6(version) ? "-drop6" : 0,
+                               ndbd_drop6(version) ? "-drop6" : 0,
                                buf, sizeof(buf)));
   }
 
@@ -1441,13 +1447,13 @@ int do_restore(RestoreThreadData *thrdata)
         ,ndbGetVersionString(MAKE_VERSION(5,1,3), 0, 0, buf, sizeof(buf))
         ,ndbGetVersionString(MAKE_VERSION(5,1,9), 0, 0, new_buf, sizeof(new_buf))
        );
-    return NDBT_FAILED;
+    return NdbToolsProgramExitCode::FAILED;
   }
 
   if (version > NDB_VERSION)
   {
     restoreLogger.log_error("Restore program older than backup version. Not supported. Use new restore program");
-    return NDBT_FAILED;
+    return NdbToolsProgramExitCode::FAILED;
   }
 
   restoreLogger.log_debug("Load content");
@@ -1457,26 +1463,27 @@ int do_restore(RestoreThreadData *thrdata)
   int res  = metaData.loadContent();
 
   restoreLogger.log_info("Stop GCP of Backup: %u", metaData.getStopGCP());
+  restoreLogger.log_info("Start GCP of Backup: %u", metaData.getStartGCP());
   
   if (res == 0)
   {
     restoreLogger.log_error("Restore: Failed to load content");
-    return NDBT_FAILED;
+    return NdbToolsProgramExitCode::FAILED;
   }
   restoreLogger.log_debug("Get number of Tables");
   Logger::format_timestamp(time(NULL), timestamp, sizeof(timestamp));
   restoreLogger.log_info("%s [restore_metadata] Get number of Tables", timestamp);
-  if (metaData.getNoOfTables() == 0) 
+  if (metaData.getNoOfTables() == 0)
   {
     restoreLogger.log_error("The backup contains no tables");
-    return NDBT_FAILED;
+    return NdbToolsProgramExitCode::FAILED;
   }
 
-  if(_print_sql_log && _print_log)
+  if (_print_sql_log && _print_log)
   {
     restoreLogger.log_debug("Check to ensure that both print-sql-log and print-log options are not passed");
     restoreLogger.log_error("Both print-sql-log and print-log options passed. Exiting...");
-    return NDBT_FAILED;
+    return NdbToolsProgramExitCode::FAILED;
   }
 
   if (_print_sql_log)
@@ -1493,20 +1500,20 @@ int do_restore(RestoreThreadData *thrdata)
       BaseString tableName(table->getTableName());
       Vector<BaseString> tableNameParts;
       tableName.split(tableNameParts, "/");
-      if (tableNameParts[2].substr(0,8) == "NDB$BLOB")
+      if (tableNameParts[2].starts_with("NDB$BLOB"))
       {
         restoreLogger.log_error("Found column of type blob with print-sql-log option set. Exiting..." );
-        return NDBT_FAILED;
+        return NdbToolsProgramExitCode::FAILED;
       }
       /* Hidden PKs are stored with the name $PK */
       int noOfPK = table->m_dictTable->getNoOfPrimaryKeys();
-      for(int j = 0; j < noOfPK; j++)
+      for (int j = 0; j < noOfPK; j++)
       {
         const char* pkName = table->m_dictTable->getPrimaryKey(j);
-        if(strcmp(pkName,"$PK") == 0)
+        if (strcmp(pkName, "$PK") == 0)
         {
           restoreLogger.log_error("Found hidden primary key with print-sql-log option set. Exiting...");
-          return NDBT_FAILED;
+          return NdbToolsProgramExitCode::FAILED;
         }
       }
     }
@@ -1516,21 +1523,20 @@ int do_restore(RestoreThreadData *thrdata)
   Logger::format_timestamp(time(NULL), timestamp, sizeof(timestamp));
   restoreLogger.log_info("%s [restore_metadata] Validate Footer", timestamp);
 
-  if (!metaData.validateFooter()) 
+  if (!metaData.validateFooter())
   {
     restoreLogger.log_error("Restore: Failed to validate footer.");
-    return NDBT_FAILED;
+    return NdbToolsProgramExitCode::FAILED;
   }
   restoreLogger.log_debug("Init Backup objects");
   Uint32 i;
-  for(i= 0; i < g_consumers.size(); i++)
+  for (i = 0; i < g_consumers.size(); i++)
   {
     if (!g_consumers[i]->init(g_tableCompabilityMask))
     {
       restoreLogger.log_error("Failed to initialize consumers");
-      return NDBT_FAILED;
+      return NdbToolsProgramExitCode::FAILED;
     }
-
   }
 
   if(ga_exclude_missing_tables)
@@ -1543,7 +1549,7 @@ int do_restore(RestoreThreadData *thrdata)
   /* before syncing on m_barrier, check if any threads have already exited */
   if (ga_error_thread > 0)
   {
-    return NDBT_FAILED;
+    return NdbToolsProgramExitCode::FAILED;
   }
 
   if (!thrdata->m_restore_meta)
@@ -1570,21 +1576,22 @@ int do_restore(RestoreThreadData *thrdata)
     if (!thrdata->m_barrier->wait())
     {
       ga_error_thread = thrdata->m_part_id;
-      return NDBT_FAILED;
+      return NdbToolsProgramExitCode::FAILED;
     }
   }
   restoreLogger.log_debug("Restore objects (tablespaces, ..)");
   Logger::format_timestamp(time(NULL), timestamp, sizeof(timestamp));
   restoreLogger.log_info("%s [restore_metadata] Restore objects (tablespaces, ..)", timestamp);
-  for(i = 0; i<metaData.getNoOfObjects(); i++)
+  for (i = 0; i < metaData.getNoOfObjects(); i++)
   {
-    for(Uint32 j= 0; j < g_consumers.size(); j++)
+    for (Uint32 j = 0; j < g_consumers.size(); j++)
       if (!g_consumers[j]->object(metaData.getObjType(i),
-				  metaData.getObjPtr(i)))
+                                  metaData.getObjPtr(i)))
       {
-	restoreLogger.log_error("Restore: Failed to restore table: %s ... Exiting",
-                                metaData[i]->getTableName());
-	return NDBT_FAILED;
+        restoreLogger.log_error(
+          "Restore: Failed to restore table: %s ... Exiting",
+          metaData[i]->getTableName());
+        return NdbToolsProgramExitCode::FAILED;
       } 
     if (check_progress())
     {
@@ -1633,26 +1640,34 @@ int do_restore(RestoreThreadData *thrdata)
                       MYF(MY_WME));
         if (res == 0)
         {
-          return NDBT_FAILED;
+          return NdbToolsProgramExitCode::FAILED;
         }
         FileOutputStream *f= new FileOutputStream(res);
         table_output[i]= f;
       }
-      for(Uint32 j= 0; j < g_consumers.size(); j++)
-	if (!g_consumers[j]->table(* table))
-	{
-	  restoreLogger.log_error("Restore: Failed to restore table: `%s` ... Exiting ",
-                                table->getTableName());
-	  return NDBT_FAILED;
-	} 
-    } else {
-      for(Uint32 j= 0; j < g_consumers.size(); j++)
+      for (Uint32 j = 0; j < g_consumers.size(); j++)
+      {
+        if (!g_consumers[j]->table(* table))
+        {
+          restoreLogger.log_error(
+            "Restore: Failed to restore table: `%s` ... Exiting ",
+            table->getTableName());
+          return NdbToolsProgramExitCode::FAILED;
+        }
+      }
+    }
+    else
+    {
+      for (Uint32 j = 0; j < g_consumers.size(); j++)
+      {
         if (!g_consumers[j]->createSystable(* table))
         {
-	  restoreLogger.log_error("Restore: Failed to restore system table: `%s` ... Exiting",
-                                table->getTableName());
-          return NDBT_FAILED;
+          restoreLogger.log_error(
+            "Restore: Failed to restore system table: `%s` ... Exiting",
+            table->getTableName());
+          return NdbToolsProgramExitCode::FAILED;
         }
+      }
     }
     if (check_progress())
     {
@@ -1665,38 +1680,40 @@ int do_restore(RestoreThreadData *thrdata)
   restoreLogger.log_debug("Save foreign key info");
   Logger::format_timestamp(time(NULL), timestamp, sizeof(timestamp));
   restoreLogger.log_info("%s [restore_metadata] Save foreign key info", timestamp);
-  for(i = 0; i<metaData.getNoOfObjects(); i++)
+  for (i = 0; i < metaData.getNoOfObjects(); i++)
   {
-    for(Uint32 j= 0; j < g_consumers.size(); j++)
+    for (Uint32 j = 0; j < g_consumers.size(); j++)
+    {
       if (!g_consumers[j]->fk(metaData.getObjType(i),
-			      metaData.getObjPtr(i)))
+                              metaData.getObjPtr(i)))
       {
-        return NDBT_FAILED;
-      } 
+        return NdbToolsProgramExitCode::FAILED;
+      }
+    }
   }
 
   restoreLogger.log_debug("Close tables" );
-  for(i= 0; i < g_consumers.size(); i++)
+  for (i = 0; i < g_consumers.size(); i++)
   {
     if (!g_consumers[i]->endOfTables())
     {
-      restoreLogger.log_error("Restore: Failed while closing tables" );
-      return NDBT_FAILED;
+      restoreLogger.log_error("Restore: Failed while closing tables");
+      return NdbToolsProgramExitCode::FAILED;
     } 
     if (!ga_disable_indexes && !ga_rebuild_indexes)
     {
       if (!g_consumers[i]->endOfTablesFK())
       {
-        restoreLogger.log_error("Restore: Failed while closing tables FKs" );
-        return NDBT_FAILED;
-      } 
+        restoreLogger.log_error("Restore: Failed while closing tables FKs");
+        return NdbToolsProgramExitCode::FAILED;
+      }
     }
   }
 
   /* before syncing on m_barrier, check if any threads have already exited */
   if (ga_error_thread > 0)
   {
-    return NDBT_FAILED;
+    return NdbToolsProgramExitCode::FAILED;
   }
 
   if (thrdata->m_restore_meta)
@@ -1705,7 +1722,7 @@ int do_restore(RestoreThreadData *thrdata)
     if (!thrdata->m_barrier->wait())
     {
       ga_error_thread = thrdata->m_part_id;
-      return NDBT_FAILED;
+      return NdbToolsProgramExitCode::FAILED;
     }
   }
 
@@ -1727,18 +1744,18 @@ int do_restore(RestoreThreadData *thrdata)
             checkDbAndTableName(metaData[i]))
         {
           TableS & tableS = *metaData[i]; // not const
-          for(Uint32 j= 0; j < g_consumers.size(); j++)
+          for(Uint32 j = 0; j < g_consumers.size(); j++)
           {
             if (!g_consumers[j]->table_compatible_check(tableS))
             {
               restoreLogger.log_error("Restore: Failed to restore data, %s table structure incompatible with backup's ... Exiting ", tableS.getTableName());
-              return NDBT_FAILED;
+              return NdbToolsProgramExitCode::FAILED;
             } 
             if (tableS.m_staging &&
                 !g_consumers[j]->prepare_staging(tableS))
             {
               restoreLogger.log_error("Restore: Failed to restore data, %s failed to prepare staging table for data conversion ... Exiting", tableS.getTableName());
-              return NDBT_FAILED;
+              return NdbToolsProgramExitCode::FAILED;
             }
           } 
         }
@@ -1750,14 +1767,17 @@ int do_restore(RestoreThreadData *thrdata)
         {
           // blob table checks use data which is populated by table compatibility checks
           TableS & tableS = *metaData[i];
-          if(isBlobTable(&tableS))
+          if (isBlobTable(&tableS))
           {
-            for(Uint32 j= 0; j < g_consumers.size(); j++)
+            for (Uint32 j = 0; j < g_consumers.size(); j++)
             {
               if (!g_consumers[j]->check_blobs(tableS))
               {
-                 restoreLogger.log_error("Restore: Failed to restore data, %s table's blobs incompatible with backup's ... Exiting ", tableS.getTableName());;
-                  return NDBT_FAILED;
+                restoreLogger.log_error(
+                  "Restore: Failed to restore data, "
+                  "%s table's blobs incompatible with backup's ... Exiting ",
+                  tableS.getTableName());
+                return NdbToolsProgramExitCode::FAILED;
               }
             }
           }
@@ -1768,15 +1788,16 @@ int do_restore(RestoreThreadData *thrdata)
 
       if (!dataIter.validateBackupFile())
       {
-          restoreLogger.log_error("Unable to allocate memory for BackupFile constructor");
-          return NDBT_FAILED;
+        restoreLogger.log_error(
+          "Unable to allocate memory for BackupFile constructor");
+        return NdbToolsProgramExitCode::FAILED;
       }
 
 
       if (!dataIter.validateRestoreDataIterator())
       {
           restoreLogger.log_error("Unable to allocate memory for RestoreDataIterator constructor");
-          return NDBT_FAILED;
+          return NdbToolsProgramExitCode::FAILED;
       }
       
       Logger::format_timestamp(time(NULL), timestamp, sizeof(timestamp));
@@ -1785,8 +1806,9 @@ int do_restore(RestoreThreadData *thrdata)
       // Read data file header
       if (!dataIter.readHeader())
       {
-	restoreLogger.log_error("Failed to read header of data file. Exiting...");
-	return NDBT_FAILED;
+        restoreLogger.log_error(
+          "Failed to read header of data file. Exiting...");
+        return NdbToolsProgramExitCode::FAILED;
       }
       
       Logger::format_timestamp(time(NULL), timestamp, sizeof(timestamp));
@@ -1811,21 +1833,27 @@ int do_restore(RestoreThreadData *thrdata)
             report_progress("Data file progress: ", dataIter);
 	} // while (tuple != NULL);
 	
-	if (res < 0)
-	{
-	  restoreLogger.log_error(" Restore: An error occured while restoring data. Exiting...");
-	  return NDBT_FAILED;
-	}
-	if (!dataIter.validateFragmentFooter()) {
-	  restoreLogger.log_error("Restore: Error validating fragment footer. ... Exiting");
-	  return NDBT_FAILED;
-	}
-      } // while (dataIter.readFragmentHeader(res))
+        if (res < 0)
+        {
+          restoreLogger.log_error(
+            "Restore: An error occurred while restoring data. Exiting...");
+          return NdbToolsProgramExitCode::FAILED;
+        }
+        if (!dataIter.validateFragmentFooter())
+        {
+          restoreLogger.log_error(
+            "Restore: Error validating fragment footer. Exiting...");
+          return NdbToolsProgramExitCode::FAILED;
+        }
+      }  // while (dataIter.readFragmentHeader(res))
       
       if (res < 0)
       {
-	restoreLogger.log_error("Restore: An error occured while restoring data. Exiting... res= %u", res);
-	return NDBT_FAILED;
+        restoreLogger.log_error(
+          "Restore: An error occurred while restoring data."
+          "Exiting... res = %u",
+          res);
+        return NdbToolsProgramExitCode::FAILED;
       }
       
       
@@ -1850,8 +1878,9 @@ int do_restore(RestoreThreadData *thrdata)
 
       if (!logIter.readHeader())
       {
-	restoreLogger.log_error("Failed to read header of data file. Exiting...");
-	return NDBT_FAILED;
+        restoreLogger.log_error(
+          "Failed to read header of data file. Exiting...");
+        return NdbToolsProgramExitCode::FAILED;
       }
       
       const LogEntry * logEntry = 0;
@@ -1872,8 +1901,10 @@ int do_restore(RestoreThreadData *thrdata)
       }
       if (res < 0)
       {
-	restoreLogger.log_error("Restore: An restoring the data log. Exiting... res=%u", res);
-	return NDBT_FAILED;
+        restoreLogger.log_error(
+          "Restore: An restoring the data log. Exiting... res = %u",
+          res);
+        return NdbToolsProgramExitCode::FAILED;
       }
       logIter.validateFooter(); //not implemented
       for (i= 0; i < g_consumers.size(); i++)
@@ -1894,13 +1925,14 @@ int do_restore(RestoreThreadData *thrdata)
         const TableS* table = metaData[i];
         if (table->m_staging)
         {
-          for(Uint32 j= 0; j < g_consumers.size(); j++)
+          for (Uint32 j = 0; j < g_consumers.size(); j++)
           {
             if (!g_consumers[j]->finalize_staging(*table))
             {
-              restoreLogger.log_error("Restore: Failed staging data to table: %s. Exiting...",
-                            table->getTableName());
-              return NDBT_FAILED;
+              restoreLogger.log_error(
+                "Restore: Failed staging data to table: %s. Exiting...",
+                table->getTableName());
+              return NdbToolsProgramExitCode::FAILED;
             }
           }
         }
@@ -1915,13 +1947,19 @@ int do_restore(RestoreThreadData *thrdata)
         check_data_truncations(table);
         OutputStream *output = table_output[table->getLocalId()];
         if (!output)
+        {
           continue;
-        for(Uint32 j= 0; j < g_consumers.size(); j++)
+        }
+        for (Uint32 j = 0; j < g_consumers.size(); j++)
+        {
           if (!g_consumers[j]->finalize_table(*table))
           {
-            restoreLogger.log_error("Restore: Failed to finalize restore table: %s. Exiting... ",  metaData[i]->getTableName());
-            return NDBT_FAILED;
+            restoreLogger.log_error(
+              "Restore: Failed to finalize restore table: %s. Exiting...",
+              metaData[i]->getTableName());
+            return NdbToolsProgramExitCode::FAILED;
           }
+        }
       }
     }
   }
@@ -1929,26 +1967,35 @@ int do_restore(RestoreThreadData *thrdata)
   if (ga_error_thread > 0)
   {
     restoreLogger.log_error("Thread %u exits on error", thrdata->m_part_id);
-    return NDBT_FAILED; // thread 1 failed to restore metadata, exiting
+    // thread 1 failed to restore metadata, exiting
+    return NdbToolsProgramExitCode::FAILED;  
   }
 
   if (ga_restore_epoch)
   {
     Logger::format_timestamp(time(NULL), timestamp, sizeof(timestamp));
     restoreLogger.log_info("%s [restore_epoch] Restoring epoch", timestamp);
+    RestoreLogIterator logIter(metaData);
 
+    if (!logIter.readHeader())
+    {
+      err << "Failed to read snapshot info from log file. Exiting..." << endl;
+      return NdbToolsProgramExitCode::FAILED;
+    }
+    bool snapshotstart = logIter.isSnapshotstartBackup();
     for (i= 0; i < g_consumers.size(); i++)
-      if (!g_consumers[i]->update_apply_status(metaData))
+      if (!g_consumers[i]->update_apply_status(metaData, snapshotstart))
       {
         restoreLogger.log_error("Restore: Failed to restore epoch");
-        return NDBT_FAILED;
+        return NdbToolsProgramExitCode::FAILED;
       }
   }
 
   if (ga_error_thread > 0)
   {
     restoreLogger.log_error("Thread %u exits on error", thrdata->m_part_id);
-    return NDBT_FAILED; // thread 1 failed to restore metadata, exiting
+    // thread 1 failed to restore metadata, exiting
+    return NdbToolsProgramExitCode::FAILED;  
   }
 
   unsigned j;
@@ -1972,7 +2019,7 @@ int do_restore(RestoreThreadData *thrdata)
     if (!thrdata->m_barrier->wait())
     {
       ga_error_thread = thrdata->m_part_id;
-      return NDBT_FAILED;
+      return NdbToolsProgramExitCode::FAILED;
     }
 
     restoreLogger.log_debug("Rebuilding indexes");
@@ -1986,23 +2033,28 @@ int do_restore(RestoreThreadData *thrdata)
         continue;
       if (isBlobTable(table) || isIndex(table))
         continue;
-      for(Uint32 j= 0; j < g_consumers.size(); j++)
+      for (Uint32 j = 0; j < g_consumers.size(); j++)
       {
         if (!g_consumers[j]->rebuild_indexes(* table))
-          return NDBT_FAILED;
+        {
+          return NdbToolsProgramExitCode::FAILED;
+        }
       }
     }
-    for(Uint32 j= 0; j < g_consumers.size(); j++)
+    for (Uint32 j = 0; j < g_consumers.size(); j++)
     {
       if (!g_consumers[j]->endOfTablesFK())
-        return NDBT_FAILED;
+      {
+        return NdbToolsProgramExitCode::FAILED;
+      }
     }
   }
 
   if (ga_error_thread > 0)
   {
     restoreLogger.log_error("Thread %u exits on error", thrdata->m_part_id);
-    return NDBT_FAILED; // thread 1 failed to restore metadata, exiting
+    // thread 1 failed to restore metadata, exiting
+    return NdbToolsProgramExitCode::FAILED;
   }
 
   /* report to clusterlog if applicable */
@@ -2019,8 +2071,8 @@ int do_restore(RestoreThreadData *thrdata)
       table_output[i] = NULL;
     }
   }
-  return NDBT_OK;
-} // do_restore
+  return NdbToolsProgramExitCode::OK;
+}  // do_restore
 
 /* Detects the backup type (single part or multiple parts) by locating
  * the ctl file. It sets the backup format as BF_SINGLE/BF_MULTI_PART
@@ -2062,23 +2114,38 @@ int detect_backup_format()
       //      BACKUP-1-PART-1-OF-2 : not found, continue
       //      BACKUP-1-PART-1-OF-3 : not found, continue
       //      BACKUP-1-PART-1-OF-4 : FOUND, set ga_part_count and break
-      BaseString::snprintf(name, sz, "%s%sBACKUP-%d-PART-1-OF-%u%sBACKUP-%u.%d.ctl",
-              ga_backupPath, DIR_SEPARATOR, ga_backupId, ga_part_count,
-              DIR_SEPARATOR, ga_backupId, ga_nodeId);
-      if(my_stat(name, &buf, 0))
-        break; // part found, end of parts
-      if(ga_part_count == g_max_parts)
-        return NDBT_FAILED; // too many parts
+      BaseString::snprintf(name,
+                           sz,
+                           "%s%sBACKUP-%d-PART-1-OF-%u%sBACKUP-%u.%d.ctl",
+                           ga_backupPath,
+                           DIR_SEPARATOR,
+                           ga_backupId,
+                           ga_part_count,
+                           DIR_SEPARATOR,
+                           ga_backupId,
+                           ga_nodeId);
+      if (my_stat(name, &buf, 0))
+      {
+        info << "Found backup " << ga_backupId << " with " << ga_part_count
+             << " backup parts" << endl;
+        break;  // part found, end of parts
+      }
+      if (ga_part_count == g_max_parts)
+      {
+        err << "Failed to find backup " << ga_backupId << " in path "
+            << ga_backupPath << endl;
+        return NdbToolsProgramExitCode::FAILED;  // too many parts
+      }
     }
   }
-  return NDBT_OK;
-} // detect_backup_format
+  return NdbToolsProgramExitCode::OK;
+}  // detect_backup_format
 
 static void* start_restore_worker(void *data)
 {
   RestoreThreadData *rdata = (RestoreThreadData*)data;
   rdata->m_result = do_restore(rdata);
-  if (rdata->m_result == NDBT_FAILED)
+  if (rdata->m_result == NdbToolsProgramExitCode::FAILED)
   {
     info << "Thread " << rdata->m_part_id << " failed, exiting" << endl;
     ga_error_thread = rdata->m_part_id;
@@ -2096,7 +2163,7 @@ main(int argc, char** argv)
 
   if (!readArguments(opts, &argv))
   {
-    exitHandler(NDBT_FAILED);
+    exitHandler(NdbToolsProgramExitCode::FAILED);
   }
 
   g_options.appfmt(" -b %u", ga_backupId);
@@ -2136,8 +2203,10 @@ main(int argc, char** argv)
   // determine backup format: simple or multi-part, and count parts
   int result = detect_backup_format();
 
-  if (result != NDBT_OK)
+  if (result != NdbToolsProgramExitCode::OK)
+  {
     exitHandler(result);
+  }
 
   init_restore();
 
@@ -2183,7 +2252,7 @@ main(int argc, char** argv)
          break;
       }
 
-      if (do_restore(&thrdata) == NDBT_FAILED)
+      if (do_restore(&thrdata) == NdbToolsProgramExitCode::FAILED)
       {
         if (ga_backup_format == BF_SINGLE)
         {
@@ -2253,13 +2322,12 @@ main(int argc, char** argv)
   cleanup_restore();
 
   if (ga_error_thread > 0)
-    exitHandler(NDBT_FAILED);
+  {
+    exitHandler(NdbToolsProgramExitCode::FAILED);
+  }
 
-  if (opt_verbose)
-    return NDBT_ProgramExit(NDBT_OK);
-  else
-    return 0;
-} // main
+  return NdbToolsProgramExitCode::OK;
+}  // main
 
 template class Vector<BackupConsumer*>;
 template class Vector<OutputStream*>;
