@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2020, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -71,9 +71,9 @@ static const uchar *account_hash_get_key(const uchar *entry, size_t *length) {
   const PFS_account *account;
   const void *result;
   typed_entry = reinterpret_cast<const PFS_account *const *>(entry);
-  DBUG_ASSERT(typed_entry != nullptr);
+  DBUG_ASSERT(typed_entry != NULL);
   account = *typed_entry;
-  DBUG_ASSERT(account != nullptr);
+  DBUG_ASSERT(account != NULL);
   *length = account->m_key.m_key_length;
   result = account->m_key.m_hash_key;
   return reinterpret_cast<const uchar *>(result);
@@ -101,9 +101,9 @@ void cleanup_account_hash(void) {
 }
 
 static LF_PINS *get_account_hash_pins(PFS_thread *thread) {
-  if (unlikely(thread->m_account_hash_pins == nullptr)) {
+  if (unlikely(thread->m_account_hash_pins == NULL)) {
     if (!account_hash_inited) {
-      return nullptr;
+      return NULL;
     }
     thread->m_account_hash_pins = lf_hash_get_pins(&account_hash);
   }
@@ -136,9 +136,9 @@ PFS_account *find_or_create_account(PFS_thread *thread, const char *username,
                                     uint username_length, const char *hostname,
                                     uint hostname_length) {
   LF_PINS *pins = get_account_hash_pins(thread);
-  if (unlikely(pins == nullptr)) {
+  if (unlikely(pins == NULL)) {
     global_account_container.m_lost++;
-    return nullptr;
+    return NULL;
   }
 
   PFS_account_key key;
@@ -163,19 +163,19 @@ search:
   lf_hash_search_unpin(pins);
 
   pfs = global_account_container.allocate(&dirty_state);
-  if (pfs != nullptr) {
+  if (pfs != NULL) {
     pfs->m_key = key;
     if (username_length > 0) {
       pfs->m_username = &pfs->m_key.m_hash_key[0];
     } else {
-      pfs->m_username = nullptr;
+      pfs->m_username = NULL;
     }
     pfs->m_username_length = username_length;
 
     if (hostname_length > 0) {
       pfs->m_hostname = &pfs->m_key.m_hash_key[username_length + 1];
     } else {
-      pfs->m_hostname = nullptr;
+      pfs->m_hostname = NULL;
     }
     pfs->m_hostname_length = hostname_length;
 
@@ -203,11 +203,11 @@ search:
 
     if (pfs->m_user) {
       pfs->m_user->release();
-      pfs->m_user = nullptr;
+      pfs->m_user = NULL;
     }
     if (pfs->m_host) {
       pfs->m_host->release();
-      pfs->m_host = nullptr;
+      pfs->m_host = NULL;
     }
 
     global_account_container.deallocate(pfs);
@@ -215,16 +215,16 @@ search:
     if (res > 0) {
       if (++retry_count > retry_max) {
         global_account_container.m_lost++;
-        return nullptr;
+        return NULL;
       }
       goto search;
     }
 
     global_account_container.m_lost++;
-    return nullptr;
+    return NULL;
   }
 
-  return nullptr;
+  return NULL;
 }
 
 void PFS_account::aggregate(bool alive, PFS_user *safe_user,
@@ -240,11 +240,11 @@ void PFS_account::aggregate(bool alive, PFS_user *safe_user,
 }
 
 void PFS_account::aggregate_waits(PFS_user *safe_user, PFS_host *safe_host) {
-  if (read_instr_class_waits_stats() == nullptr) {
+  if (read_instr_class_waits_stats() == NULL) {
     return;
   }
 
-  if (likely(safe_user != nullptr && safe_host != nullptr)) {
+  if (likely(safe_user != NULL && safe_host != NULL)) {
     /*
       Aggregate EVENTS_WAITS_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_WAITS_SUMMARY_BY_USER_BY_EVENT_NAME
@@ -257,7 +257,7 @@ void PFS_account::aggregate_waits(PFS_user *safe_user, PFS_host *safe_host) {
     return;
   }
 
-  if (safe_user != nullptr) {
+  if (safe_user != NULL) {
     /*
       Aggregate EVENTS_WAITS_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_WAITS_SUMMARY_BY_USER_BY_EVENT_NAME
@@ -267,7 +267,7 @@ void PFS_account::aggregate_waits(PFS_user *safe_user, PFS_host *safe_host) {
     return;
   }
 
-  if (safe_host != nullptr) {
+  if (safe_host != NULL) {
     /*
       Aggregate EVENTS_WAITS_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_WAITS_SUMMARY_BY_HOST_BY_EVENT_NAME
@@ -283,11 +283,11 @@ void PFS_account::aggregate_waits(PFS_user *safe_user, PFS_host *safe_host) {
 }
 
 void PFS_account::aggregate_stages(PFS_user *safe_user, PFS_host *safe_host) {
-  if (read_instr_class_stages_stats() == nullptr) {
+  if (read_instr_class_stages_stats() == NULL) {
     return;
   }
 
-  if (likely(safe_user != nullptr && safe_host != nullptr)) {
+  if (likely(safe_user != NULL && safe_host != NULL)) {
     /*
       Aggregate EVENTS_STAGES_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_STAGES_SUMMARY_BY_USER_BY_EVENT_NAME
@@ -300,7 +300,7 @@ void PFS_account::aggregate_stages(PFS_user *safe_user, PFS_host *safe_host) {
     return;
   }
 
-  if (safe_user != nullptr) {
+  if (safe_user != NULL) {
     /*
       Aggregate EVENTS_STAGES_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_STAGES_SUMMARY_BY_USER_BY_EVENT_NAME
@@ -313,7 +313,7 @@ void PFS_account::aggregate_stages(PFS_user *safe_user, PFS_host *safe_host) {
     return;
   }
 
-  if (safe_host != nullptr) {
+  if (safe_host != NULL) {
     /*
       Aggregate EVENTS_STAGES_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_STAGES_SUMMARY_BY_HOST_BY_EVENT_NAME
@@ -334,11 +334,11 @@ void PFS_account::aggregate_stages(PFS_user *safe_user, PFS_host *safe_host) {
 
 void PFS_account::aggregate_statements(PFS_user *safe_user,
                                        PFS_host *safe_host) {
-  if (read_instr_class_statements_stats() == nullptr) {
+  if (read_instr_class_statements_stats() == NULL) {
     return;
   }
 
-  if (likely(safe_user != nullptr && safe_host != nullptr)) {
+  if (likely(safe_user != NULL && safe_host != NULL)) {
     /*
       Aggregate EVENTS_STATEMENTS_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_STATEMENTS_SUMMARY_BY_USER_BY_EVENT_NAME
@@ -351,7 +351,7 @@ void PFS_account::aggregate_statements(PFS_user *safe_user,
     return;
   }
 
-  if (safe_user != nullptr) {
+  if (safe_user != NULL) {
     /*
       Aggregate EVENTS_STATEMENTS_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_STATEMENTS_SUMMARY_BY_USER_BY_EVENT_NAME
@@ -364,7 +364,7 @@ void PFS_account::aggregate_statements(PFS_user *safe_user,
     return;
   }
 
-  if (safe_host != nullptr) {
+  if (safe_host != NULL) {
     /*
       Aggregate EVENTS_STATEMENTS_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_STATEMENTS_SUMMARY_BY_HOST_BY_EVENT_NAME
@@ -385,11 +385,11 @@ void PFS_account::aggregate_statements(PFS_user *safe_user,
 
 void PFS_account::aggregate_transactions(PFS_user *safe_user,
                                          PFS_host *safe_host) {
-  if (read_instr_class_transactions_stats() == nullptr) {
+  if (read_instr_class_transactions_stats() == NULL) {
     return;
   }
 
-  if (likely(safe_user != nullptr && safe_host != nullptr)) {
+  if (likely(safe_user != NULL && safe_host != NULL)) {
     /*
       Aggregate EVENTS_TRANSACTIONS_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_TRANSACTIONS_SUMMARY_BY_USER_BY_EVENT_NAME
@@ -403,7 +403,7 @@ void PFS_account::aggregate_transactions(PFS_user *safe_user,
     return;
   }
 
-  if (safe_user != nullptr) {
+  if (safe_user != NULL) {
     /*
       Aggregate EVENTS_TRANSACTIONS_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_TRANSACTIONS_SUMMARY_BY_USER_BY_EVENT_NAME
@@ -417,7 +417,7 @@ void PFS_account::aggregate_transactions(PFS_user *safe_user,
     return;
   }
 
-  if (safe_host != nullptr) {
+  if (safe_host != NULL) {
     /*
       Aggregate EVENTS_TRANSACTIONS_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       -  EVENTS_TRANSACTIONS_SUMMARY_BY_HOST_BY_EVENT_NAME
@@ -438,11 +438,11 @@ void PFS_account::aggregate_transactions(PFS_user *safe_user,
 }
 
 void PFS_account::aggregate_errors(PFS_user *safe_user, PFS_host *safe_host) {
-  if (read_instr_class_errors_stats() == nullptr) {
+  if (read_instr_class_errors_stats() == NULL) {
     return;
   }
 
-  if (likely(safe_user != nullptr && safe_host != nullptr)) {
+  if (likely(safe_user != NULL && safe_host != NULL)) {
     /*
       Aggregate EVENTS_ERRORS_SUMMARY_BY_ACCOUNT_BY_ERROR to:
       -  EVENTS_ERRORS_SUMMARY_BY_USER_BY_ERROR
@@ -455,7 +455,7 @@ void PFS_account::aggregate_errors(PFS_user *safe_user, PFS_host *safe_host) {
     return;
   }
 
-  if (safe_user != nullptr) {
+  if (safe_user != NULL) {
     /*
       Aggregate EVENTS_ERRORS_SUMMARY_BY_ACCOUNT_BY_ERROR to:
       -  EVENTS_ERRORS_SUMMARY_BY_USER_BY_ERROR
@@ -468,7 +468,7 @@ void PFS_account::aggregate_errors(PFS_user *safe_user, PFS_host *safe_host) {
     return;
   }
 
-  if (safe_host != nullptr) {
+  if (safe_host != NULL) {
     /*
       Aggregate EVENTS_ERRORS_SUMMARY_BY_ACCOUNT_BY_ERROR to:
       -  EVENTS_ERRORS_SUMMARY_BY_HOST_BY_ERROR
@@ -488,11 +488,11 @@ void PFS_account::aggregate_errors(PFS_user *safe_user, PFS_host *safe_host) {
 
 void PFS_account::aggregate_memory(bool alive, PFS_user *safe_user,
                                    PFS_host *safe_host) {
-  if (read_instr_class_memory_stats() == nullptr) {
+  if (read_instr_class_memory_stats() == NULL) {
     return;
   }
 
-  if (likely(safe_user != nullptr && safe_host != nullptr)) {
+  if (likely(safe_user != NULL && safe_host != NULL)) {
     /*
       Aggregate MEMORY_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       - MEMORY_SUMMARY_BY_USER_BY_EVENT_NAME
@@ -505,7 +505,7 @@ void PFS_account::aggregate_memory(bool alive, PFS_user *safe_user,
     return;
   }
 
-  if (safe_user != nullptr) {
+  if (safe_user != NULL) {
     /*
       Aggregate MEMORY_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       - MEMORY_SUMMARY_BY_USER_BY_EVENT_NAME
@@ -518,7 +518,7 @@ void PFS_account::aggregate_memory(bool alive, PFS_user *safe_user,
     return;
   }
 
-  if (safe_host != nullptr) {
+  if (safe_host != NULL) {
     /*
       Aggregate MEMORY_SUMMARY_BY_ACCOUNT_BY_EVENT_NAME to:
       - MEMORY_SUMMARY_BY_HOST_BY_EVENT_NAME
@@ -538,7 +538,7 @@ void PFS_account::aggregate_memory(bool alive, PFS_user *safe_user,
 }
 
 void PFS_account::aggregate_status(PFS_user *safe_user, PFS_host *safe_host) {
-  if (likely(safe_user != nullptr && safe_host != nullptr)) {
+  if (likely(safe_user != NULL && safe_host != NULL)) {
     /*
       Aggregate STATUS_BY_ACCOUNT to:
       - STATUS_BY_USER
@@ -550,7 +550,7 @@ void PFS_account::aggregate_status(PFS_user *safe_user, PFS_host *safe_host) {
     return;
   }
 
-  if (safe_user != nullptr) {
+  if (safe_user != NULL) {
     /*
       Aggregate STATUS_BY_ACCOUNT to:
       - STATUS_BY_USER
@@ -562,7 +562,7 @@ void PFS_account::aggregate_status(PFS_user *safe_user, PFS_host *safe_host) {
     return;
   }
 
-  if (safe_host != nullptr) {
+  if (safe_host != NULL) {
     /*
       Aggregate STATUS_BY_ACCOUNT to:
       - STATUS_BY_HOST
@@ -582,20 +582,20 @@ void PFS_account::aggregate_status(PFS_user *safe_user, PFS_host *safe_host) {
 }
 
 void PFS_account::aggregate_stats(PFS_user *safe_user, PFS_host *safe_host) {
-  if (likely(safe_user != nullptr && safe_host != nullptr)) {
+  if (likely(safe_user != NULL && safe_host != NULL)) {
     safe_user->m_disconnected_count += m_disconnected_count;
     safe_host->m_disconnected_count += m_disconnected_count;
     m_disconnected_count = 0;
     return;
   }
 
-  if (safe_user != nullptr) {
+  if (safe_user != NULL) {
     safe_user->m_disconnected_count += m_disconnected_count;
     m_disconnected_count = 0;
     return;
   }
 
-  if (safe_host != nullptr) {
+  if (safe_host != NULL) {
     safe_host->m_disconnected_count += m_disconnected_count;
     m_disconnected_count = 0;
     return;
@@ -615,60 +615,32 @@ void PFS_account::rebase_memory_stats() {
   }
 }
 
-void PFS_account::carry_memory_stat_alloc_delta(
-    PFS_memory_stat_alloc_delta *delta, uint index) {
+void PFS_account::carry_memory_stat_delta(PFS_memory_stat_delta *delta,
+                                          uint index) {
   PFS_memory_shared_stat *event_name_array;
   PFS_memory_shared_stat *stat;
-  PFS_memory_stat_alloc_delta delta_buffer;
-  PFS_memory_stat_alloc_delta *remaining_delta;
+  PFS_memory_stat_delta delta_buffer;
+  PFS_memory_stat_delta *remaining_delta;
 
   event_name_array = write_instr_class_memory_stats();
   stat = &event_name_array[index];
-  remaining_delta = stat->apply_alloc_delta(delta, &delta_buffer);
+  remaining_delta = stat->apply_delta(delta, &delta_buffer);
 
-  if (remaining_delta == nullptr) {
+  if (remaining_delta == NULL) {
     return;
   }
 
-  if (m_user != nullptr) {
-    m_user->carry_memory_stat_alloc_delta(remaining_delta, index);
+  if (m_user != NULL) {
+    m_user->carry_memory_stat_delta(remaining_delta, index);
     /* do not return, need to process m_host below */
   }
 
-  if (m_host != nullptr) {
-    m_host->carry_memory_stat_alloc_delta(remaining_delta, index);
+  if (m_host != NULL) {
+    m_host->carry_memory_stat_delta(remaining_delta, index);
     return;
   }
 
-  carry_global_memory_stat_alloc_delta(remaining_delta, index);
-}
-
-void PFS_account::carry_memory_stat_free_delta(
-    PFS_memory_stat_free_delta *delta, uint index) {
-  PFS_memory_shared_stat *event_name_array;
-  PFS_memory_shared_stat *stat;
-  PFS_memory_stat_free_delta delta_buffer;
-  PFS_memory_stat_free_delta *remaining_delta;
-
-  event_name_array = write_instr_class_memory_stats();
-  stat = &event_name_array[index];
-  remaining_delta = stat->apply_free_delta(delta, &delta_buffer);
-
-  if (remaining_delta == nullptr) {
-    return;
-  }
-
-  if (m_user != nullptr) {
-    m_user->carry_memory_stat_free_delta(remaining_delta, index);
-    /* do not return, need to process m_host below */
-  }
-
-  if (m_host != nullptr) {
-    m_host->carry_memory_stat_free_delta(remaining_delta, index);
-    return;
-  }
-
-  carry_global_memory_stat_free_delta(remaining_delta, index);
+  carry_global_memory_stat_delta(remaining_delta, index);
 }
 
 PFS_account *sanitize_account(PFS_account *unsafe) {
@@ -677,7 +649,7 @@ PFS_account *sanitize_account(PFS_account *unsafe) {
 
 static void purge_account(PFS_thread *thread, PFS_account *account) {
   LF_PINS *pins = get_account_hash_pins(thread);
-  if (unlikely(pins == nullptr)) {
+  if (unlikely(pins == NULL)) {
     return;
   }
 
@@ -691,13 +663,13 @@ static void purge_account(PFS_thread *thread, PFS_account *account) {
       lf_hash_delete(&account_hash, pins, account->m_key.m_hash_key,
                      account->m_key.m_key_length);
       account->aggregate(false, account->m_user, account->m_host);
-      if (account->m_user != nullptr) {
+      if (account->m_user != NULL) {
         account->m_user->release();
-        account->m_user = nullptr;
+        account->m_user = NULL;
       }
-      if (account->m_host != nullptr) {
+      if (account->m_host != NULL) {
         account->m_host->release();
-        account->m_host = nullptr;
+        account->m_host = NULL;
       }
       global_account_container.deallocate(account);
     }
@@ -710,7 +682,7 @@ class Proc_purge_account : public PFS_buffer_processor<PFS_account> {
  public:
   Proc_purge_account(PFS_thread *thread) : m_thread(thread) {}
 
-  void operator()(PFS_account *pfs) override {
+  virtual void operator()(PFS_account *pfs) {
     PFS_user *user = sanitize_user(pfs->m_user);
     PFS_host *host = sanitize_host(pfs->m_host);
     pfs->aggregate(true, user, host);
@@ -727,7 +699,7 @@ class Proc_purge_account : public PFS_buffer_processor<PFS_account> {
 /** Purge non connected accounts, reset stats of connected account. */
 void purge_all_account(void) {
   PFS_thread *thread = PFS_thread::get_current_thread();
-  if (unlikely(thread == nullptr)) {
+  if (unlikely(thread == NULL)) {
     return;
   }
 
@@ -740,7 +712,7 @@ class Proc_update_accounts_derived_flags
  public:
   Proc_update_accounts_derived_flags(PFS_thread *thread) : m_thread(thread) {}
 
-  void operator()(PFS_account *pfs) override {
+  virtual void operator()(PFS_account *pfs) {
     if (pfs->m_username_length > 0 && pfs->m_hostname_length > 0) {
       lookup_setup_actor(m_thread, pfs->m_username, pfs->m_username_length,
                          pfs->m_hostname, pfs->m_hostname_length,
