@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -40,8 +40,6 @@
 #include "my_inttypes.h"
 #include "sql/thr_malloc.h"
 
-#include <mysql/components/services/mysql_server_telemetry_traces_service.h>
-
 /**
   @file storage/perfschema/pfs_global.h
   Miscellaneous global dependencies (declarations).
@@ -102,18 +100,6 @@ struct PFS_cacheline_atomic_size_t {
   PFS_cacheline_atomic_size_t() : m_size_t(0) {}
 };
 
-/**
-  An atomic<T> variable, guaranteed to be alone in a CPU cache line.
-  This is for performance, for variables accessed very frequently.
-*/
-template <class T>
-struct PFS_cacheline_atomic_ptr {
-  std::atomic<T> m_ptr;
-  char m_full_cache_line[PFS_CACHE_LINE_SIZE - sizeof(std::atomic<T>)];
-
-  PFS_cacheline_atomic_ptr() : m_ptr(nullptr) {}
-};
-
 struct PFS_builtin_memory_class;
 
 /** Memory allocation for the performance schema. */
