@@ -1,16 +1,15 @@
-/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -47,7 +46,6 @@
 #include "mysql/psi/psi_system.h"
 #include "mysql/psi/psi_table.h"
 #include "mysql/psi/psi_thread.h"
-#include "mysql/psi/psi_tls_channel.h"
 #include "mysql/psi/psi_transaction.h"
 
 #ifdef HAVE_PSI_INTERFACE
@@ -56,13 +54,13 @@
 #define PFS_AUTOSIZE_VALUE (-1)
 
 #ifndef PFS_MAX_MUTEX_CLASS
-#define PFS_MAX_MUTEX_CLASS 350
+#define PFS_MAX_MUTEX_CLASS 300
 #endif
 #ifndef PFS_MAX_RWLOCK_CLASS
 #define PFS_MAX_RWLOCK_CLASS 60
 #endif
 #ifndef PFS_MAX_COND_CLASS
-#define PFS_MAX_COND_CLASS 150
+#define PFS_MAX_COND_CLASS 100
 #endif
 #ifndef PFS_MAX_THREAD_CLASS
 #define PFS_MAX_THREAD_CLASS 100
@@ -86,13 +84,8 @@
 #define PFS_MAX_MEMORY_CLASS 450
 #endif
 
-#ifndef PFS_MAX_GLOBAL_SERVER_ERRORS
-#define PFS_MAX_GLOBAL_SERVER_ERRORS \
-  (1 + pfs_session_error_stat_count + pfs_global_error_stat_count)
-#endif
-
-#ifndef PFS_MAX_SESSION_SERVER_ERRORS
-#define PFS_MAX_SESSION_SERVER_ERRORS (1 + pfs_session_error_stat_count)
+#ifndef PFS_MAX_SERVER_ERRORS
+#define PFS_MAX_SERVER_ERRORS ((total_error_count - obsolete_error_count) + 1)
 #endif
 
 /** Sizing hints, from the server configuration. */
@@ -100,9 +93,9 @@ struct PFS_sizing_hints {
   /** Value of @c Sys_table_def_size */
   ulong m_table_definition_cache;
   /** Value of @c Sys_table_cache_size */
-  ulong m_table_open_cache;
+  long m_table_open_cache;
   /** Value of @c Sys_max_connections */
-  ulong m_max_connections;
+  long m_max_connections;
   /** Value of @c Sys_open_files_limit */
   long m_open_files_limit;
   /** Value of @c Sys_max_prepared_stmt_count */
@@ -117,7 +110,6 @@ struct PFS_global_param {
   bool m_consumer_events_stages_current_enabled;
   bool m_consumer_events_stages_history_enabled;
   bool m_consumer_events_stages_history_long_enabled;
-  bool m_consumer_events_statements_cpu_enabled;
   bool m_consumer_events_statements_current_enabled;
   bool m_consumer_events_statements_history_enabled;
   bool m_consumer_events_statements_history_long_enabled;
@@ -131,9 +123,6 @@ struct PFS_global_param {
   bool m_consumer_thread_instrumentation_enabled;
   bool m_consumer_statement_digest_enabled;
 
-  /** True if SHOW PROCESSLIST is enabeld in the performance schema. */
-  bool m_processlist_enabled;
-
   /** Default instrument configuration option. */
   char *m_pfs_instrument;
 
@@ -328,9 +317,8 @@ void pre_initialize_performance_schema();
   @param [out] error_bootstrap Error instrumentation service bootstrap
   @param [out] data_lock_bootstrap Data Lock instrumentation service bootstrap
   @param [out] system_bootstrap System instrumentation service bootstrap
-  @param [out] tls_channel_bootstrap TLS channel instrumentation service
-  bootstrap
-  @retval 0 success
+  @returns
+    @retval 0 success
 */
 int initialize_performance_schema(
     PFS_global_param *param, PSI_thread_bootstrap **thread_bootstrap,
@@ -345,8 +333,7 @@ int initialize_performance_schema(
     PSI_memory_bootstrap **memory_bootstrap,
     PSI_error_bootstrap **error_bootstrap,
     PSI_data_lock_bootstrap **data_lock_bootstrap,
-    PSI_system_bootstrap **system_bootstrap,
-    PSI_tls_channel_bootstrap **tls_channel_bootstrap);
+    PSI_system_bootstrap **system_bootstrap);
 
 void pfs_automated_sizing(PFS_global_param *param);
 
