@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -42,7 +41,7 @@ class PFS_index_esmh_global : public PFS_engine_index {
   PFS_index_esmh_global()
       : PFS_engine_index(&m_key_1), m_key_1("BUCKET_NUMBER") {}
 
-  ~PFS_index_esmh_global() override = default;
+  ~PFS_index_esmh_global() {}
 
   bool match_bucket(ulong bucket_index);
 
@@ -93,22 +92,22 @@ class table_esmh_global : public PFS_engine_table {
   static int delete_all_rows();
   static ha_rows get_row_count();
 
-  void reset_position() override;
+  virtual void reset_position(void);
 
-  int rnd_next() override;
-  int rnd_pos(const void *pos) override;
+  virtual int rnd_next();
+  virtual int rnd_pos(const void *pos);
 
-  int index_init(uint idx, bool sorted) override;
-  int index_next() override;
+  virtual int index_init(uint idx, bool sorted);
+  virtual int index_next();
 
  protected:
-  int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
-                      bool read_all) override;
+  virtual int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
+                              bool read_all);
 
   table_esmh_global();
 
  public:
-  ~table_esmh_global() override = default;
+  ~table_esmh_global() {}
 
  protected:
   void materialize();
