@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -55,7 +55,7 @@ class PFS_index_ews_by_account_by_event_name : public PFS_engine_index {
         m_key_2("HOST"),
         m_key_3("EVENT_NAME") {}
 
-  ~PFS_index_ews_by_account_by_event_name() override = default;
+  ~PFS_index_ews_by_account_by_event_name() {}
 
   virtual bool match(PFS_account *pfs);
   virtual bool match_view(uint view);
@@ -120,22 +120,22 @@ class table_ews_by_account_by_event_name : public PFS_engine_table {
   static int delete_all_rows();
   static ha_rows get_row_count();
 
-  void reset_position(void) override;
+  virtual void reset_position(void);
 
-  int rnd_next() override;
-  int rnd_pos(const void *pos) override;
+  virtual int rnd_next();
+  virtual int rnd_pos(const void *pos);
 
-  int index_init(uint idx, bool sorted) override;
-  int index_next() override;
+  virtual int index_init(uint idx, bool sorted);
+  virtual int index_next();
 
  protected:
-  int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
-                      bool read_all) override;
+  virtual int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
+                              bool read_all);
 
   table_ews_by_account_by_event_name();
 
  public:
-  ~table_ews_by_account_by_event_name() override = default;
+  ~table_ews_by_account_by_event_name() {}
 
  protected:
   int make_row(PFS_account *account, PFS_instr_class *klass);
