@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2013, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,7 +34,6 @@
 
 #include "my_base.h"
 #include "sql/rpl_info.h" /*CHANNEL_NAME_LENGTH*/
-#include "sql/rpl_rli.h"  /*enum_require_table_primary_key*/
 #include "storage/perfschema/pfs_engine_table.h"
 #include "storage/perfschema/table_helper.h"
 
@@ -45,11 +43,6 @@ class Plugin_table;
 struct TABLE;
 struct THR_LOCK;
 
-#ifndef ENUM_RPL_YES_NO
-#define ENUM_RPL_YES_NO
-enum enum_rpl_yes_no { PS_RPL_YES = 1, PS_RPL_NO };
-#endif
-
 /**
   @addtogroup performance_schema_tables
   @{
@@ -61,13 +54,7 @@ struct st_row_applier_config {
   uint channel_name_length;
   time_t desired_delay;
   bool desired_delay_is_set;
-  std::string privilege_checks_user;
-  enum_rpl_yes_no requires_row_format;
-  Relay_log_info::enum_require_table_primary_key
-      require_table_primary_key_check;
-  Assign_gtids_to_anonymous_transactions_info::enum_type
-      assign_gtids_to_anonymous_transactions_type;
-  std::string assign_gtids_to_anonymous_transactions_value;
+  std::string privilege_checks_user{""};
 };
 
 class PFS_index_rpl_applier_config : public PFS_engine_index {
@@ -75,7 +62,7 @@ class PFS_index_rpl_applier_config : public PFS_engine_index {
   PFS_index_rpl_applier_config()
       : PFS_engine_index(&m_key), m_key("CHANNEL_NAME") {}
 
-  ~PFS_index_rpl_applier_config() override = default;
+  ~PFS_index_rpl_applier_config() {}
 
   virtual bool match(Master_info *mi);
 
@@ -111,26 +98,26 @@ class table_replication_applier_configuration : public PFS_engine_table {
     @param read_all         true if all columns are read.
   */
 
-  int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
-                      bool read_all) override;
+  virtual int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
+                              bool read_all);
 
   table_replication_applier_configuration();
 
  public:
-  ~table_replication_applier_configuration() override;
+  ~table_replication_applier_configuration();
 
   /** Table share. */
   static PFS_engine_table_share m_share;
   static PFS_engine_table *create(PFS_engine_table_share *);
   static ha_rows get_row_count();
 
-  void reset_position() override;
+  virtual void reset_position(void);
 
-  int rnd_next() override;
-  int rnd_pos(const void *pos) override;
+  virtual int rnd_next();
+  virtual int rnd_pos(const void *pos);
 
-  int index_init(uint idx, bool sorted) override;
-  int index_next() override;
+  virtual int index_init(uint idx, bool sorted);
+  virtual int index_next();
 
  private:
   PFS_index_rpl_applier_config *m_opened_index;
