@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -48,7 +48,8 @@
 */
 static void set_channel_name(void *const context, const char &value,
                              size_t length) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   const size_t max = CHANNEL_NAME_LENGTH;
   length = std::min(length, max);
 
@@ -57,7 +58,8 @@ static void set_channel_name(void *const context, const char &value,
 }
 
 static void set_view_id(void *const context, const char &value, size_t length) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   const size_t max = HOSTNAME_LENGTH;
   length = std::min(length, max);
 
@@ -67,7 +69,8 @@ static void set_view_id(void *const context, const char &value, size_t length) {
 
 static void set_member_id(void *const context, const char &value,
                           size_t length) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   const size_t max = UUID_LENGTH;
   length = std::min(length, max);
 
@@ -77,9 +80,10 @@ static void set_member_id(void *const context, const char &value,
 
 static void set_transactions_committed(void *const context, const char &value,
                                        size_t length) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
 
-  if (row->trx_committed != nullptr) {
+  if (row->trx_committed != NULL) {
     my_free(row->trx_committed);
   }
 
@@ -91,7 +95,8 @@ static void set_transactions_committed(void *const context, const char &value,
 static void set_last_conflict_free_transaction(void *const context,
                                                const char &value,
                                                size_t length) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   const size_t max = Gtid::MAX_TEXT_LENGTH + 1;
   length = std::min(length, max);
 
@@ -101,49 +106,57 @@ static void set_last_conflict_free_transaction(void *const context,
 
 static void set_transactions_in_queue(void *const context,
                                       unsigned long long int value) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   row->trx_in_queue = value;
 }
 
 static void set_transactions_certified(void *const context,
                                        unsigned long long int value) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   row->trx_checked = value;
 }
 
 static void set_transactions_conflicts_detected(void *const context,
                                                 unsigned long long int value) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   row->trx_conflicts = value;
 }
 
 static void set_transactions_rows_in_validation(void *const context,
                                                 unsigned long long int value) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   row->trx_rows_validating = value;
 }
 
 static void set_transactions_remote_applier_queue(
     void *const context, unsigned long long int value) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   row->trx_remote_applier_queue = value;
 }
 
 static void set_transactions_remote_applied(void *const context,
                                             unsigned long long int value) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   row->trx_remote_applied = value;
 }
 
 static void set_transactions_local_proposed(void *const context,
                                             unsigned long long int value) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   row->trx_local_proposed = value;
 }
 
 static void set_transactions_local_rollback(void *const context,
                                             unsigned long long int value) {
-  auto *row = static_cast<struct st_row_group_member_stats *>(context);
+  struct st_row_group_member_stats *row =
+      static_cast<struct st_row_group_member_stats *>(context);
   row->trx_local_rollback = value;
 }
 
@@ -176,8 +189,8 @@ Plugin_table table_replication_group_member_stats::m_table_def(
 PFS_engine_table_share table_replication_group_member_stats::m_share = {
     &pfs_readonly_acl,
     &table_replication_group_member_stats::create,
-    nullptr, /* write_row */
-    nullptr, /* delete_all_rows */
+    NULL, /* write_row */
+    NULL, /* delete_all_rows */
     table_replication_group_member_stats::get_row_count,
     sizeof(pos_t), /* ref length */
     &m_table_lock,
@@ -195,17 +208,17 @@ PFS_engine_table *table_replication_group_member_stats::create(
 
 table_replication_group_member_stats::table_replication_group_member_stats()
     : PFS_engine_table(&m_share, &m_pos), m_pos(0), m_next_pos(0) {
-  m_row.trx_committed = nullptr;
+  m_row.trx_committed = NULL;
 }
 
 table_replication_group_member_stats::~table_replication_group_member_stats() {
-  if (m_row.trx_committed != nullptr) {
+  if (m_row.trx_committed != NULL) {
     my_free(m_row.trx_committed);
-    m_row.trx_committed = nullptr;
+    m_row.trx_committed = NULL;
   }
 }
 
-void table_replication_group_member_stats::reset_position() {
+void table_replication_group_member_stats::reset_position(void) {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
@@ -214,7 +227,7 @@ ha_rows table_replication_group_member_stats::get_row_count() {
   return get_group_replication_members_number_info();
 }
 
-int table_replication_group_member_stats::rnd_next() {
+int table_replication_group_member_stats::rnd_next(void) {
   if (!is_group_replication_plugin_loaded()) {
     return HA_ERR_END_OF_FILE;
   }
@@ -230,15 +243,16 @@ int table_replication_group_member_stats::rnd_next() {
 }
 
 int table_replication_group_member_stats::rnd_pos(const void *pos) {
-  if (!is_group_replication_plugin_loaded() || get_row_count() == 0) {
+  if (!is_group_replication_plugin_loaded()) {
     return HA_ERR_END_OF_FILE;
   }
 
-  if (m_pos.m_index >= get_row_count()) {
-    return HA_ERR_RECORD_DELETED;
+  if (get_row_count() == 0) {
+    return HA_ERR_END_OF_FILE;
   }
 
   set_position(pos);
+  DBUG_ASSERT(m_pos.m_index < get_row_count());
   return make_row(m_pos.m_index);
 }
 
@@ -291,21 +305,20 @@ int table_replication_group_member_stats::read_row_values(TABLE *table,
                                                           bool read_all) {
   Field *f;
 
-  assert(table->s->null_bytes == 0);
+  DBUG_ASSERT(table->s->null_bytes == 0);
   buf[0] = 0;
 
   for (; (f = *fields); fields++) {
-    if (read_all || bitmap_is_set(table->read_set, f->field_index())) {
-      switch (f->field_index()) {
+    if (read_all || bitmap_is_set(table->read_set, f->field_index)) {
+      switch (f->field_index) {
         case 0: /** channel_name */
-          set_field_char_utf8mb4(f, m_row.channel_name,
-                                 m_row.channel_name_length);
+          set_field_char_utf8(f, m_row.channel_name, m_row.channel_name_length);
           break;
         case 1: /** view id */
-          set_field_char_utf8mb4(f, m_row.view_id, m_row.view_id_length);
+          set_field_char_utf8(f, m_row.view_id, m_row.view_id_length);
           break;
         case 2: /** member_id */
-          set_field_char_utf8mb4(f, m_row.member_id, m_row.member_id_length);
+          set_field_char_utf8(f, m_row.member_id, m_row.member_id_length);
           break;
         case 3: /** transaction_in_queue */
           set_field_ulonglong(f, m_row.trx_in_queue);
@@ -340,7 +353,7 @@ int table_replication_group_member_stats::read_row_values(TABLE *table,
           break;
 
         default:
-          assert(false);
+          DBUG_ASSERT(false);
       }
     }
   }
