@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -82,18 +82,18 @@ class table_variables_info : public PFS_engine_table {
   static PFS_engine_table *create(PFS_engine_table_share *);
   static ha_rows get_row_count();
 
-  int rnd_init(bool scan) override;
-  int rnd_next() override;
-  int rnd_pos(const void *pos) override;
-  void reset_position() override;
+  virtual int rnd_init(bool scan);
+  virtual int rnd_next();
+  virtual int rnd_pos(const void *pos);
+  virtual void reset_position(void);
 
  protected:
-  int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
-                      bool read_all) override;
+  virtual int read_row_values(TABLE *table, unsigned char *buf, Field **fields,
+                              bool read_all);
   table_variables_info();
 
  public:
-  ~table_variables_info() override = default;
+  ~table_variables_info() {}
 
  protected:
   int make_row(const System_variable *system_var);
@@ -114,4 +114,5 @@ class table_variables_info : public PFS_engine_table {
   pos_t m_next_pos;
 };
 
+/** @} */
 #endif
