@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2018, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -27,10 +26,10 @@ TempTable Storage. */
 #ifndef TEMPTABLE_STORAGE_H
 #define TEMPTABLE_STORAGE_H
 
-#include <assert.h>
 #include <cstddef>
 #include <utility>
 
+#include "my_dbug.h"
 #include "storage/temptable/include/temptable/allocator.h"
 #include "storage/temptable/include/temptable/constants.h"
 
@@ -183,9 +182,6 @@ class Storage {
   /** Delete all elements in the storage. After this `size()` will be zero. */
   void clear();
 
-  /** A simple getter. */
-  size_t number_of_elements_per_page() const;
-
  private:
   /** Align elements to this number of bytes. */
   static constexpr size_t ALIGN_TO = alignof(void *);
@@ -355,7 +351,7 @@ inline Storage::Element *Storage::Iterator::operator*() const {
 }
 
 inline Storage::Iterator &Storage::Iterator::operator=(const Element *element) {
-  assert(m_storage != nullptr || element == nullptr);
+  DBUG_ASSERT(m_storage != nullptr || element == nullptr);
   m_element = const_cast<Element *>(element);
   return *this;
 }
@@ -369,8 +365,8 @@ inline bool Storage::Iterator::operator!=(const Iterator &rhs) const {
 }
 
 inline Storage::Iterator &Storage::Iterator::operator++() {
-  assert(m_storage != nullptr);
-  assert(*this != m_storage->end());
+  DBUG_ASSERT(m_storage != nullptr);
+  DBUG_ASSERT(*this != m_storage->end());
 
   do {
     if (m_storage->element_last_on_page(m_element)) {
@@ -390,8 +386,8 @@ inline Storage::Iterator &Storage::Iterator::operator++() {
 }
 
 inline Storage::Iterator &Storage::Iterator::operator--() {
-  assert(m_storage != nullptr);
-  assert(*this != m_storage->begin());
+  DBUG_ASSERT(m_storage != nullptr);
+  DBUG_ASSERT(*this != m_storage->begin());
 
   /* Since *this != m_storage->begin() there is at least one non-deleted element
    * preceding our position (ie the one pointed to by begin()). */
@@ -402,9 +398,9 @@ inline Storage::Iterator &Storage::Iterator::operator--() {
     } else if (m_storage->element_first_on_page(m_element)) {
       /* Go to the last element on the previous page. */
       Page *prev_page = *m_storage->element_prev_page_ptr(m_element);
-      assert(prev_page != nullptr);
+      DBUG_ASSERT(prev_page != nullptr);
       m_element = m_storage->last_possible_element_on_page(prev_page);
-      assert(m_storage->element_last_on_page(m_element));
+      DBUG_ASSERT(m_storage->element_last_on_page(m_element));
     } else {
       m_element = m_storage->prev_element(m_element);
     }
@@ -429,9 +425,9 @@ inline Storage::Storage(Storage &&other)
 }
 
 inline Storage &Storage::operator=(Storage &&rhs) {
-  assert(m_first_page == nullptr);
-  assert(m_last_page == nullptr);
-  assert(m_last_element == nullptr);
+  DBUG_ASSERT(m_first_page == nullptr);
+  DBUG_ASSERT(m_last_page == nullptr);
+  DBUG_ASSERT(m_last_element == nullptr);
 
   m_allocator = rhs.m_allocator;
   rhs.m_allocator = nullptr;
@@ -467,12 +463,12 @@ inline Storage::Iterator Storage::begin() const {
     return end();
   }
 
-  assert(m_first_page != nullptr);
+  DBUG_ASSERT(m_first_page != nullptr);
 
   Iterator it(this, first_possible_element_on_page(m_first_page));
 
   for (;;) {
-    assert(it != end());
+    DBUG_ASSERT(it != end());
     if (!element_deleted(*it)) {
       break;
     }
@@ -487,14 +483,14 @@ inline Storage::Iterator Storage::end() const {
 }
 
 inline void Storage::element_size(size_t element_size) {
-  assert(m_number_of_elements == 0);
+  DBUG_ASSERT(m_number_of_elements == 0);
 
   m_element_size = element_size;
 
   const size_t element_size_plus_meta = element_size + META_BYTES_PER_ELEMENT;
 
   /* Confirm that ALIGN_TO is a power of 2 (or zero). */
-  assert(ALIGN_TO == 0 || (ALIGN_TO & (ALIGN_TO - 1)) == 0);
+  DBUG_ASSERT(ALIGN_TO == 0 || (ALIGN_TO & (ALIGN_TO - 1)) == 0);
 
   /* The next multiple of ALIGN_TO from element_size_plus_meta. */
   m_bytes_used_per_element =
@@ -513,7 +509,7 @@ inline Storage::Element *Storage::back() { return m_last_element; }
 inline Storage::Element *Storage::allocate_back() {
   if (m_last_page == nullptr) {
     /* The storage is empty, create the first page. */
-    assert(m_first_page == nullptr);
+    DBUG_ASSERT(m_first_page == nullptr);
 
     m_first_page = m_allocator->allocate(page_size());
 
@@ -524,9 +520,9 @@ inline Storage::Element *Storage::allocate_back() {
 
     m_last_element = first_possible_element_on_page(m_first_page);
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     *element_meta(m_last_element) = 0;
-#endif /* NDEBUG */
+#endif /* DBUG_OFF */
 
     element_first_on_page(true, m_last_element);
   } else if (m_last_element == last_possible_element_on_page(m_last_page)) {
@@ -540,9 +536,9 @@ inline Storage::Element *Storage::allocate_back() {
     m_last_page = new_page;
     m_last_element = first_possible_element_on_page(new_page);
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     *element_meta(m_last_element) = 0;
-#endif /* NDEBUG */
+#endif /* DBUG_OFF */
 
     element_first_on_page(true, m_last_element);
   } else {
@@ -550,9 +546,9 @@ inline Storage::Element *Storage::allocate_back() {
 
     m_last_element = next_element(m_last_element);
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     *element_meta(m_last_element) = 0;
-#endif /* NDEBUG */
+#endif /* DBUG_OFF */
 
     element_first_on_page(false, m_last_element);
   }
@@ -567,7 +563,7 @@ inline Storage::Element *Storage::allocate_back() {
 }
 
 inline void Storage::deallocate_back() {
-  assert(m_number_of_elements > 0);
+  DBUG_ASSERT(m_number_of_elements > 0);
 
   --m_number_of_elements;
 
@@ -575,7 +571,7 @@ inline void Storage::deallocate_back() {
     if (m_last_element != first_possible_element_on_page(m_last_page)) {
       m_last_element = prev_element(m_last_element);
     } else if (m_first_page == m_last_page) {
-      assert(m_number_of_elements == 0);
+      DBUG_ASSERT(m_number_of_elements == 0);
       m_allocator->deallocate(static_cast<uint8_t *>(m_first_page),
                               page_size());
       m_first_page = nullptr;
@@ -606,7 +602,7 @@ inline Storage::Iterator Storage::erase(const Iterator &position) {
   if (*position == m_last_element) {
     deallocate_back();
   } else {
-    assert(m_number_of_elements > 0);
+    DBUG_ASSERT(m_number_of_elements > 0);
     --m_number_of_elements;
 
     element_deleted(true, *position);
@@ -617,12 +613,12 @@ inline Storage::Iterator Storage::erase(const Iterator &position) {
 
 inline void Storage::clear() {
   if (m_first_page == nullptr) {
-    assert(m_number_of_elements == 0);
+    DBUG_ASSERT(m_number_of_elements == 0);
     return;
   }
 
   if (m_first_page == m_last_page) {
-    assert(m_number_of_elements <= m_number_of_elements_per_page);
+    DBUG_ASSERT(m_number_of_elements <= m_number_of_elements_per_page);
   } else {
     Page *p = m_first_page;
     do {
@@ -640,13 +636,9 @@ inline void Storage::clear() {
   m_number_of_elements = 0;
 }
 
-inline size_t Storage::number_of_elements_per_page() const {
-  return m_number_of_elements_per_page;
-}
-
 inline size_t Storage::page_size() const {
-  assert(m_bytes_used_per_element > 0);
-  assert(m_number_of_elements_per_page > 0);
+  DBUG_ASSERT(m_bytes_used_per_element > 0);
+  DBUG_ASSERT(m_number_of_elements_per_page > 0);
 
   return m_bytes_used_per_element * m_number_of_elements_per_page +
          META_BYTES_PER_PAGE;
@@ -694,49 +686,49 @@ inline void Storage::element_deleted(bool deleted, Element *element) {
 }
 
 inline Storage::Page **Storage::element_prev_page_ptr(Element *element) const {
-  assert(element_first_on_page(element));
+  DBUG_ASSERT(element_first_on_page(element));
   return reinterpret_cast<Page **>(static_cast<uint8_t *>(element) -
                                    sizeof(Page *));
 }
 
 inline Storage::Page **Storage::element_next_page_ptr(Element *element) const {
-  assert(element_last_on_page(element));
+  DBUG_ASSERT(element_last_on_page(element));
   return reinterpret_cast<Page **>(static_cast<uint8_t *>(element) +
                                    m_bytes_used_per_element);
 }
 
 inline Storage::Element *Storage::prev_element(Element *element) const {
-  assert(element != nullptr);
-  assert(!element_first_on_page(element));
+  DBUG_ASSERT(element != nullptr);
+  DBUG_ASSERT(!element_first_on_page(element));
   return static_cast<uint8_t *>(element) - m_bytes_used_per_element;
 }
 
 inline Storage::Element *Storage::next_element(Element *element) const {
-  assert(element != nullptr);
-  assert(!element_last_on_page(element));
+  DBUG_ASSERT(element != nullptr);
+  DBUG_ASSERT(!element_last_on_page(element));
   return static_cast<uint8_t *>(element) + m_bytes_used_per_element;
 }
 
 inline Storage::Element *Storage::first_possible_element_on_page(
     Page *page) const {
-  assert(page != nullptr);
+  DBUG_ASSERT(page != nullptr);
   return static_cast<uint8_t *>(page) + sizeof(Page *);
 }
 
 inline Storage::Element *Storage::last_possible_element_on_page(
     Page *page) const {
-  assert(page != nullptr);
+  DBUG_ASSERT(page != nullptr);
   return static_cast<uint8_t *>(page) + sizeof(Page *) +
          m_bytes_used_per_element * (m_number_of_elements_per_page - 1);
 }
 
 inline Storage::Page **Storage::page_prev_page_ptr(Page *page) const {
-  assert(page != nullptr);
+  DBUG_ASSERT(page != nullptr);
   return reinterpret_cast<Page **>(page);
 }
 
 inline Storage::Page **Storage::page_next_page_ptr(Page *page) const {
-  assert(page != nullptr);
+  DBUG_ASSERT(page != nullptr);
   /* The sizeof(Page*) bytes just after the last element (and its meta byte(s)
    * and padding). */
   return reinterpret_cast<Page **>(
