@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -48,7 +47,7 @@
 */
 
 /*
-   This file is basically tis620 character sets with some extra functions
+   This file is basicly tis620 character sets with some extra functions
    for tis-620 handling
 */
 
@@ -62,8 +61,6 @@
 #include <string.h>
 #include <sys/types.h>
 
-#include <algorithm>
-
 #include "m_ctype.h"
 #include "m_string.h"
 #include "my_compiler.h"
@@ -583,7 +580,7 @@ static size_t thai2sortable(uchar *tstr, size_t len) {
   string
 
   NOTE:
-    We can't cut strings at end \0 as this would break comparison with
+    We can't cut strings at end \0 as this would break comparision with
     LIKE characters, where the min range is stored as end \0
 
   Arg: 2 Strings and it compare length
@@ -591,7 +588,7 @@ static size_t thai2sortable(uchar *tstr, size_t len) {
 */
 
 extern "C" {
-static int my_strnncoll_tis620(const CHARSET_INFO *cs [[maybe_unused]],
+static int my_strnncoll_tis620(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
                                const uchar *s1, size_t len1, const uchar *s2,
                                size_t len2, bool s2_is_prefix) {
   uchar buf[80];
@@ -615,10 +612,10 @@ static int my_strnncoll_tis620(const CHARSET_INFO *cs [[maybe_unused]],
   return i;
 }
 
-static int my_strnncollsp_tis620(const CHARSET_INFO *cs [[maybe_unused]],
+static int my_strnncollsp_tis620(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
                                  const uchar *a0, size_t a_length,
                                  const uchar *b0, size_t b_length) {
-  uchar buf[80], *end, *a, *b, *alloced = nullptr;
+  uchar buf[80], *end, *a, *b, *alloced = NULL;
   size_t length;
   int res = 0;
 
@@ -634,7 +631,7 @@ static int my_strnncollsp_tis620(const CHARSET_INFO *cs [[maybe_unused]],
   a_length = thai2sortable(a, a_length);
   b_length = thai2sortable(b, b_length);
 
-  end = a + (length = std::min(a_length, b_length));
+  end = a + (length = MY_MIN(a_length, b_length));
   while (a < end) {
     if (*a++ != *b++) {
       res = ((int)a[-1] - (int)b[-1]);
@@ -669,17 +666,17 @@ ret:
 }
 
 /*
-  strnxfrm replacement, convert Thai string to sortable string
+  strnxfrm replacment, convert Thai string to sortable string
 
   Arg: Destination buffer, source string, dest length and source length
-  Ret: Converted string size
+  Ret: Conveted string size
 */
 
 static size_t my_strnxfrm_tis620(const CHARSET_INFO *cs, uchar *dst,
                                  size_t dstlen, uint nweights, const uchar *src,
                                  size_t srclen, uint flags) {
   size_t dstlen0 = dstlen;
-  size_t min_len = std::min(dstlen, srclen);
+  size_t min_len = MY_MIN(dstlen, srclen);
   size_t len = 0;
 
   /*
@@ -693,8 +690,8 @@ static size_t my_strnxfrm_tis620(const CHARSET_INFO *cs, uchar *dst,
   }
 
   len = thai2sortable(dst, len);
-  dstlen = std::min(dstlen, size_t(nweights));
-  len = std::min(len, size_t(dstlen));
+  set_if_smaller(dstlen, nweights);
+  set_if_smaller(len, dstlen);
   len = my_strxfrm_pad(cs, dst, dst + len, dst + dstlen, (uint)(dstlen - len),
                        flags);
   if ((flags & MY_STRXFRM_PAD_TO_MAXLEN) && len < dstlen0) {
@@ -827,50 +824,40 @@ static const uchar plFF[256] = {
     0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
     0x0000, 0x00FF, 0x0000, 0x0000};
 static const uchar *uni_to_cs[256] = {
-    pl00,    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, pl0E,    nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
-    nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, plFF};
+    pl00, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, pl0E, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+    NULL, NULL, NULL, plFF};
 
 extern "C" {
-static int my_mb_wc_tis620(const CHARSET_INFO *cs [[maybe_unused]], my_wc_t *wc,
-                           const uchar *str, const uchar *end) {
+static int my_mb_wc_tis620(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
+                           my_wc_t *wc, const uchar *str, const uchar *end) {
   if (str >= end) return MY_CS_TOOSMALL;
 
   *wc = cs_to_uni[*str];
   return (!wc[0] && str[0]) ? -1 : 1;
 }
 
-static int my_wc_mb_tis620(const CHARSET_INFO *cs [[maybe_unused]], my_wc_t wc,
-                           uchar *str, uchar *end) {
+static int my_wc_mb_tis620(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
+                           my_wc_t wc, uchar *str, uchar *end) {
   const uchar *pl;
 
   if (str >= end) return MY_CS_TOOSMALL;
@@ -896,8 +883,8 @@ static MY_COLLATION_HANDLER my_collation_ci_handler = {
     my_propagate_simple};
 
 static MY_CHARSET_HANDLER my_charset_handler = {
-    nullptr,           /* init */
-    nullptr,           /* ismbchar  */
+    NULL,              /* init */
+    NULL,              /* ismbchar  */
     my_mbcharlen_8bit, /* mbcharlen */
     my_numchars_8bit,
     my_charpos_8bit,
@@ -930,20 +917,20 @@ CHARSET_INFO my_charset_tis620_thai_ci = {
     0,                                               /* number    */
     MY_CS_COMPILED | MY_CS_PRIMARY | MY_CS_STRNXFRM, /* state     */
     "tis620",                                        /* cs name    */
-    "tis620_thai_ci",                                /* m_coll_name */
-    "TIS620 Thai",                                   /* comment   */
-    nullptr,                                         /* tailoring */
-    nullptr,                                         /* coll_param */
+    "tis620_thai_ci",                                /* name      */
+    "",                                              /* comment   */
+    NULL,                                            /* tailoring */
+    NULL,                                            /* coll_param */
     ctype_tis620,
     to_lower_tis620,
     to_upper_tis620,
     sort_order_tis620,
-    nullptr,             /* uca          */
-    nullptr,             /* tab_to_uni   */
-    nullptr,             /* tab_from_uni */
+    NULL,                /* uca          */
+    NULL,                /* tab_to_uni   */
+    NULL,                /* tab_from_uni */
     &my_unicase_default, /* caseinfo     */
-    nullptr,             /* state_map    */
-    nullptr,             /* ident_map    */
+    NULL,                /* state_map    */
+    NULL,                /* ident_map    */
     4,                   /* strxfrm_multiply */
     1,                   /* caseup_multiply  */
     1,                   /* casedn_multiply  */
@@ -953,7 +940,7 @@ CHARSET_INFO my_charset_tis620_thai_ci = {
     0,                   /* min_sort_char */
     255,                 /* max_sort_char */
     ' ',                 /* pad char      */
-    false,               /* escape_with_backslash_is_dangerous */
+    0,                   /* escape_with_backslash_is_dangerous */
     1,                   /* levels_for_compare */
     &my_charset_handler,
     &my_collation_ci_handler,
@@ -965,20 +952,20 @@ CHARSET_INFO my_charset_tis620_bin = {
     0,                              /* number    */
     MY_CS_COMPILED | MY_CS_BINSORT, /* state     */
     "tis620",                       /* cs name    */
-    "tis620_bin",                   /* m_coll_name */
-    "TIS620 Thai",                  /* comment   */
-    nullptr,                        /* tailoring */
-    nullptr,                        /* coll_param */
+    "tis620_bin",                   /* name      */
+    "",                             /* comment   */
+    NULL,                           /* tailoring */
+    NULL,                           /* coll_param */
     ctype_tis620,
     to_lower_tis620,
     to_upper_tis620,
-    nullptr,             /* sort_order   */
-    nullptr,             /* uca          */
-    nullptr,             /* tab_to_uni   */
-    nullptr,             /* tab_from_uni */
+    NULL,                /* sort_order   */
+    NULL,                /* uca          */
+    NULL,                /* tab_to_uni   */
+    NULL,                /* tab_from_uni */
     &my_unicase_default, /* caseinfo     */
-    nullptr,             /* state_map    */
-    nullptr,             /* ident_map    */
+    NULL,                /* state_map    */
+    NULL,                /* ident_map    */
     1,                   /* strxfrm_multiply */
     1,                   /* caseup_multiply  */
     1,                   /* casedn_multiply  */
@@ -988,7 +975,7 @@ CHARSET_INFO my_charset_tis620_bin = {
     0,                   /* min_sort_char */
     255,                 /* max_sort_char */
     ' ',                 /* pad char      */
-    false,               /* escape_with_backslash_is_dangerous */
+    0,                   /* escape_with_backslash_is_dangerous */
     1,                   /* levels_for_compare */
     &my_charset_handler,
     &my_collation_8bit_bin_handler,
