@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -26,18 +25,14 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 
-#include <algorithm>
-
 #include "m_ctype.h"
-#include "m_string.h"
 #include "my_byteorder.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_loglevel.h"
 #include "my_macros.h"
@@ -62,7 +57,7 @@
 
 */
 
-int (*my_string_stack_guard)(int) = nullptr;
+int (*my_string_stack_guard)(int) = NULL;
 
 static char *mstr(char *str, const char *src, size_t l1, size_t l2) {
   l1 = l1 < l2 ? l1 : l2;
@@ -270,14 +265,14 @@ static struct my_cs_file_section_st sec[] = {
     {_CS_RESET_LAST_VARIABLE,
      "charsets/charset/collation/rules/reset/last_variable"},
 
-    {0, nullptr}};
+    {0, NULL}};
 
 static struct my_cs_file_section_st *cs_file_sec(const char *attr, size_t len) {
   struct my_cs_file_section_st *s;
   for (s = sec; s->str; s++) {
     if (!strncmp(attr, s->str, len) && s->str[len] == 0) return s;
   }
-  return nullptr;
+  return NULL;
 }
 
 #define MY_CS_CSDESCR_SIZE 64
@@ -312,7 +307,7 @@ static void my_charset_file_reset_collation(MY_CHARSET_FILE *i) {
 static void my_charset_file_init(MY_CHARSET_FILE *i) {
   my_charset_file_reset_charset(i);
   my_charset_file_reset_collation(i);
-  i->tailoring = nullptr;
+  i->tailoring = NULL;
   i->tailoring_alloced_length = 0;
 }
 
@@ -342,7 +337,7 @@ static int fill_uchar(uchar *a, uint size, const char *str, size_t len) {
     for (; (s < e) && !strchr(" \t\r\n", s[0]); s++)
       ;
     if (s == b || i > size) break;
-    a[i] = (uchar)strtoul(b, nullptr, 16);
+    a[i] = (uchar)strtoul(b, NULL, 16);
   }
   return 0;
 }
@@ -358,7 +353,7 @@ static int fill_uint16(uint16 *a, uint size, const char *str, size_t len) {
     for (; (s < e) && !strchr(" \t\r\n", s[0]); s++)
       ;
     if (s == b || i > size) break;
-    a[i] = (uint16)strtol(b, nullptr, 16);
+    a[i] = (uint16)strtol(b, NULL, 16);
   }
   return 0;
 }
@@ -391,7 +386,7 @@ static int tailoring_append2(MY_XML_PARSER *st, const char *fmt, size_t len1,
 }
 
 static size_t scan_one_character(const char *s, const char *e, my_wc_t *wc) {
-  CHARSET_INFO *cs = &my_charset_utf8mb3_general_ci;
+  CHARSET_INFO *cs = &my_charset_utf8_general_ci;
   if (s >= e) return 0;
 
   /* Escape sequence: \uXXXX */
@@ -421,7 +416,7 @@ static int tailoring_append_abbreviation(MY_XML_PARSER *st, const char *fmt,
   my_wc_t wc;
 
   for (; (clen = scan_one_character(attr, attrend, &wc)) > 0; attr += clen) {
-    assert(attr < attrend);
+    DBUG_ASSERT(attr < attrend);
     if (tailoring_append(st, fmt, clen, attr) != MY_XML_OK) return MY_XML_ERROR;
   }
   return MY_XML_OK;
@@ -447,7 +442,7 @@ static int cs_enter(MY_XML_PARSER *st, const char *attr, size_t len) {
       break;
 
     case _CS_RESET:
-      return tailoring_append(st, " &", 0, nullptr);
+      return tailoring_append(st, " &", 0, NULL);
 
     default:
       break;
@@ -464,56 +459,57 @@ static int cs_leave(MY_XML_PARSER *st, const char *attr, size_t len) {
   switch (state) {
     case _CS_COLLATION:
       if (i->tailoring_length) i->cs.tailoring = i->tailoring;
-      rc = i->loader->add_collation(&i->cs);
+      rc = i->loader->add_collation ? i->loader->add_collation(&i->cs)
+                                    : MY_XML_OK;
       break;
 
     /* Rules: Logical Reset Positions */
     case _CS_RESET_FIRST_NON_IGNORABLE:
-      rc = tailoring_append(st, "[first non-ignorable]", 0, nullptr);
+      rc = tailoring_append(st, "[first non-ignorable]", 0, NULL);
       break;
 
     case _CS_RESET_LAST_NON_IGNORABLE:
-      rc = tailoring_append(st, "[last non-ignorable]", 0, nullptr);
+      rc = tailoring_append(st, "[last non-ignorable]", 0, NULL);
       break;
 
     case _CS_RESET_FIRST_PRIMARY_IGNORABLE:
-      rc = tailoring_append(st, "[first primary ignorable]", 0, nullptr);
+      rc = tailoring_append(st, "[first primary ignorable]", 0, NULL);
       break;
 
     case _CS_RESET_LAST_PRIMARY_IGNORABLE:
-      rc = tailoring_append(st, "[last primary ignorable]", 0, nullptr);
+      rc = tailoring_append(st, "[last primary ignorable]", 0, NULL);
       break;
 
     case _CS_RESET_FIRST_SECONDARY_IGNORABLE:
-      rc = tailoring_append(st, "[first secondary ignorable]", 0, nullptr);
+      rc = tailoring_append(st, "[first secondary ignorable]", 0, NULL);
       break;
 
     case _CS_RESET_LAST_SECONDARY_IGNORABLE:
-      rc = tailoring_append(st, "[last secondary ignorable]", 0, nullptr);
+      rc = tailoring_append(st, "[last secondary ignorable]", 0, NULL);
       break;
 
     case _CS_RESET_FIRST_TERTIARY_IGNORABLE:
-      rc = tailoring_append(st, "[first tertiary ignorable]", 0, nullptr);
+      rc = tailoring_append(st, "[first tertiary ignorable]", 0, NULL);
       break;
 
     case _CS_RESET_LAST_TERTIARY_IGNORABLE:
-      rc = tailoring_append(st, "[last tertiary ignorable]", 0, nullptr);
+      rc = tailoring_append(st, "[last tertiary ignorable]", 0, NULL);
       break;
 
     case _CS_RESET_FIRST_TRAILING:
-      rc = tailoring_append(st, "[first trailing]", 0, nullptr);
+      rc = tailoring_append(st, "[first trailing]", 0, NULL);
       break;
 
     case _CS_RESET_LAST_TRAILING:
-      rc = tailoring_append(st, "[last trailing]", 0, nullptr);
+      rc = tailoring_append(st, "[last trailing]", 0, NULL);
       break;
 
     case _CS_RESET_FIRST_VARIABLE:
-      rc = tailoring_append(st, "[first variable]", 0, nullptr);
+      rc = tailoring_append(st, "[first variable]", 0, NULL);
       break;
 
     case _CS_RESET_LAST_VARIABLE:
-      rc = tailoring_append(st, "[last variable]", 0, nullptr);
+      rc = tailoring_append(st, "[last variable]", 0, NULL);
       break;
 
     default:
@@ -545,25 +541,19 @@ static int cs_value(MY_XML_PARSER *st, const char *attr, size_t len) {
     case _CS_ORDER:
       break;
     case _CS_ID:
-      i->cs.number = strtol(attr, (char **)nullptr, 10);
+      i->cs.number = strtol(attr, (char **)NULL, 10);
       break;
     case _CS_BINARY_ID:
-      i->cs.binary_number = strtol(attr, (char **)nullptr, 10);
+      i->cs.binary_number = strtol(attr, (char **)NULL, 10);
       break;
     case _CS_PRIMARY_ID:
-      i->cs.primary_number = strtol(attr, (char **)nullptr, 10);
+      i->cs.primary_number = strtol(attr, (char **)NULL, 10);
       break;
     case _CS_COLNAME:
-      i->cs.m_coll_name = mstr(i->name, attr, len, MY_CS_NAME_SIZE - 1);
+      i->cs.name = mstr(i->name, attr, len, MY_CS_NAME_SIZE - 1);
       break;
     case _CS_CSNAME:
-      // Replace "utf8" with "utf8mb3" for external character sets.
-      if (0 == strncmp(attr, "utf8", len))
-        i->cs.csname =
-            mstr(i->csname, STRING_WITH_LEN("utf8mb3"), MY_CS_NAME_SIZE - 1);
-      else
-        i->cs.csname = mstr(i->csname, attr, len, MY_CS_NAME_SIZE - 1);
-      assert(0 != strcmp(i->cs.csname, "utf8"));
+      i->cs.csname = mstr(i->csname, attr, len, MY_CS_NAME_SIZE - 1);
       break;
     case _CS_CSDESCRIPT:
       i->cs.comment = mstr(i->comment, attr, len, MY_CS_CSDESCR_SIZE - 1);
@@ -799,6 +789,38 @@ uint my_charset_repertoire(const CHARSET_INFO *cs) {
                                      : MY_REPERTOIRE_UNICODE30;
 }
 
+/*
+  Detect whether a character set is ASCII compatible.
+
+  Returns true for:
+
+  - all 8bit character sets whose Unicode mapping of 0x7B is '{'
+    (ignores swe7 which maps 0x7B to "LATIN LETTER A WITH DIAERESIS")
+
+  - all multi-byte character sets having mbminlen == 1
+    (ignores ucs2 whose mbminlen is 2)
+
+  TODO:
+
+  When merging to 5.2, this function should be changed
+  to check a new flag MY_CS_NONASCII,
+
+     return (cs->flag & MY_CS_NONASCII) ? 0 : 1;
+
+  This flag was previously added into 5.2 under terms
+  of WL#3759 "Optimize identifier conversion in client-server protocol"
+  especially to mark character sets not compatible with ASCII.
+
+  We won't backport this flag to 5.0 or 5.1.
+  This function is Ok for 5.0 and 5.1, because we're not going
+  to introduce new tricky character sets between 5.0 and 5.2.
+*/
+bool my_charset_is_ascii_based(const CHARSET_INFO *cs) {
+  return (cs->mbmaxlen == 1 && cs->tab_to_uni &&
+          cs->tab_to_uni[static_cast<int>('{')] == '{') ||
+         (cs->mbminlen == 1 && cs->mbmaxlen > 1);
+}
+
 /*
   Detect if a character set is 8bit,
   and it is pure ascii, i.e. doesn't have
@@ -808,11 +830,11 @@ uint my_charset_repertoire(const CHARSET_INFO *cs) {
 */
 bool my_charset_is_8bit_pure_ascii(const CHARSET_INFO *cs) {
   size_t code;
-  if (!cs->tab_to_uni) return false;
+  if (!cs->tab_to_uni) return 0;
   for (code = 0; code < 256; code++) {
-    if (cs->tab_to_uni[code] > 0x7F) return false;
+    if (cs->tab_to_uni[code] > 0x7F) return 0;
   }
-  return true;
+  return 1;
 }
 
 /*
@@ -822,11 +844,11 @@ bool my_charset_is_8bit_pure_ascii(const CHARSET_INFO *cs) {
 */
 bool my_charset_is_ascii_compatible(const CHARSET_INFO *cs) {
   uint i;
-  if (!cs->tab_to_uni) return true;
+  if (!cs->tab_to_uni) return 1;
   for (i = 0; i < 128; i++) {
-    if (cs->tab_to_uni[i] != i) return false;
+    if (cs->tab_to_uni[i] != i) return 0;
   }
-  return true;
+  return 1;
 }
 
 /**
@@ -857,7 +879,7 @@ static size_t my_convert_internal(char *to, size_t to_length,
   my_charset_conv_wc_mb wc_mb = to_cs->cset->wc_mb;
   uint error_count = 0;
 
-  while (true) {
+  while (1) {
     if ((cnvres = (*mb_wc)(from_cs, &wc, pointer_cast<const uchar *>(from),
                            from_end)) > 0)
       from += cnvres;
@@ -918,7 +940,7 @@ size_t my_convert(char *to, size_t to_length, const CHARSET_INFO *to_cs,
     return my_convert_internal(to, to_length, to_cs, from, from_length, from_cs,
                                errors);
 
-  length = length2 = std::min(to_length, from_length);
+  length = length2 = MY_MIN(to_length, from_length);
 
 #if defined(__i386__) || defined(_WIN32) || defined(__x86_64__)
   /*
@@ -949,8 +971,8 @@ size_t my_convert(char *to, size_t to_length, const CHARSET_INFO *to_cs,
     }
   }
 
-  assert(false);  // Should never get to here
-  return 0;       // Make compiler happy
+  DBUG_ASSERT(false);  // Should never get to here
+  return 0;            // Make compiler happy
 }
 
 /**
@@ -970,7 +992,7 @@ uint my_mbcharlen_ptr(const CHARSET_INFO *cs, const char *s, const char *e) {
     len = my_mbcharlen_2(cs, (uchar)*s, (uchar) * (s + 1));
     /* It could be either a valid multi-byte GB18030 code, or invalid
     gb18030 code if return value is 0 */
-    assert(len == 0 || len == 2 || len == 4);
+    DBUG_ASSERT(len == 0 || len == 2 || len == 4);
   }
 
   return len;
