@@ -1,4 +1,4 @@
-/* Copyright (c) 2012, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,7 +24,6 @@
 
 #include <gtest/gtest.h>
 #include <algorithm>
-#include <random>
 #include <vector>
 
 #include "my_byteorder.h"
@@ -52,9 +51,7 @@ class AlignmentTest : public ::testing::Test {
     for (int ix = 0; ix < num_records; ++ix) {
       aligned_data[ix] = ix / 10;
     }
-    std::random_device rng;
-    std::mt19937 urng(rng());
-    std::shuffle(aligned_data, aligned_data + num_records, urng);
+    std::random_shuffle(aligned_data, aligned_data + num_records);
     memcpy(unaligned_data + 1, aligned_data, num_records * sizeof(int));
   }
 
@@ -63,7 +60,7 @@ class AlignmentTest : public ::testing::Test {
     delete[] unaligned_data;
   }
 
-  void SetUp() override {
+  virtual void SetUp() {
     aligned_keys = new uchar *[num_records];
     unaligned_keys = new uchar *[num_records];
     for (int ix = 0; ix < num_records; ++ix) {
@@ -73,7 +70,7 @@ class AlignmentTest : public ::testing::Test {
     }
   }
 
-  void TearDown() override {
+  virtual void TearDown() {
     delete[] aligned_keys;
     delete[] unaligned_keys;
   }
