@@ -1,16 +1,15 @@
-/* Copyright (c) 2019, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,20 +24,15 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <algorithm>
 #include <cstdlib>
-#include <map>
 #include <set>
 #include <string>
 
 #include <gtest/gtest.h>
 
-#include "lex_string.h"
-#include "m_string.h"
-#include "my_inttypes.h"
 #include "sql/auth/auth_common.h"
 #include "sql/auth/auth_utility.h"
 #include "sql/auth/sql_auth_cache.h"
 #include "sql/auth/sql_authentication.h"
-#include "sql/sql_class.h"
 #include "unittest/gunit/test_utils.h"
 
 namespace decoy_user_unittest {
@@ -48,8 +42,8 @@ using std::rand;
 
 class Decoy_user_Test : public ::testing::Test {
  protected:
-  void SetUp() override { initializer.SetUp(); }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void SetUp() { initializer.SetUp(); }
+  virtual void TearDown() { initializer.TearDown(); }
 
   THD *get_thd() { return initializer.thd(); }
 
