@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,18 +23,18 @@
 #ifndef FAKE_TABLE_H
 #define FAKE_TABLE_H
 
-#include <assert.h>
 #include <string.h>
 #include <sys/types.h>
 #include <ostream>
 #include <string>
 #include <vector>
 
+#include "gmock/gmock-generated-nice-strict.h"
 #include "gtest/gtest.h"
 #include "lex_string.h"
 #include "my_alloc.h"
 #include "my_bitmap.h"
-
+#include "my_dbug.h"
 #include "my_inttypes.h"
 #include "mysql_com.h"
 #include "sql/current_thd.h"
@@ -61,13 +60,12 @@ using ::testing::NiceMock;
 static const uint MAX_TABLE_COLUMNS = sizeof(int) * 8;
 
 /*
-  A fake class for setting up Table_ref object, required for table id
-  mgmt.
+  A fake class for setting up TABLE_LIST object, required for table id mgmt.
 */
-class Fake_Table_ref : public Table_ref {
+class Fake_TABLE_LIST : public TABLE_LIST {
  public:
-  Fake_Table_ref() = default;
-  ~Fake_Table_ref() = default;
+  Fake_TABLE_LIST() {}
+  ~Fake_TABLE_LIST() {}
 };
 
 /*
@@ -93,11 +91,10 @@ class Fake_TABLE_SHARE : public TABLE_SHARE {
     path.str = const_cast<char *>(fakepath);
     path.length = strlen(path.str);
 
-    EXPECT_EQ(0, bitmap_init(&all_set, &all_set_buf, fields));
-    bitmap_set_above(&all_set, 0, true);
+    EXPECT_EQ(0, bitmap_init(&all_set, &all_set_buf, fields, false));
+    bitmap_set_above(&all_set, 0, 1);
   }
-  ~Fake_TABLE_SHARE() = default;
-  void set_secondary_engine(bool enable) { m_secondary_engine = enable; }
+  ~Fake_TABLE_SHARE() {}
 };
 
 /*
@@ -111,9 +108,7 @@ class Fake_TABLE : public TABLE {
   // make room for up to 8 keyparts per index
   KEY_PART_INFO m_key_part_infos[max_keys][8];
 
-  uchar m_record[MAX_FIELD_WIDTH * MAX_TABLE_COLUMNS];
-  uchar m_null_flags[MAX_TABLE_COLUMNS + 7 / 8];
-
+  Fake_TABLE_LIST table_list;
   Fake_TABLE_SHARE table_share;
   // Storage space for the handler's handlerton
   Fake_handlerton fake_handlerton;
@@ -121,8 +116,7 @@ class Fake_TABLE : public TABLE {
   uint32 write_set_buf;
   MY_BITMAP read_set_struct;
   uint32 read_set_buf;
-  uint32 tmp_set_buf;
-  Field *m_field_array[MAX_TABLE_COLUMNS]{};
+  Field *m_field_array[MAX_TABLE_COLUMNS];
   Mock_field_long *m_mock_field_array[MAX_TABLE_COLUMNS];
 
   // Counter for creating unique index id's. See create_index().
@@ -131,37 +125,33 @@ class Fake_TABLE : public TABLE {
   // Counter for creating unique table id's. See initialize().
   static int highest_table_id;
 
+ public:
+  SELECT_LEX select_lex{nullptr, nullptr};
+
  private:
   void initialize() {
     TABLE *as_table = static_cast<TABLE *>(this);
     new (as_table) TABLE();
     s = &table_share;
     in_use = current_thd;
-    null_row = false;
+    null_row = '\0';
     read_set = &read_set_struct;
     write_set = &write_set_struct;
-    next_number_field = nullptr;  // No autoinc column
-    pos_in_table_list = new (*THR_MALLOC) Fake_Table_ref();
-    pos_in_table_list->table = this;
-    pos_in_table_list->query_block =
-        new (&mem_root) Query_block(&mem_root, nullptr, nullptr);
-    EXPECT_EQ(0, bitmap_init(write_set, &write_set_buf, s->fields));
-    EXPECT_EQ(0, bitmap_init(read_set, &read_set_buf, s->fields));
-    EXPECT_EQ(0, bitmap_init(&tmp_set, &tmp_set_buf, s->fields));
-    read_set_internal = *read_set;
+    next_number_field = NULL;  // No autoinc column
+    pos_in_table_list = &table_list;
+    pos_in_table_list->select_lex = &select_lex;
+    table_list.table = this;
+    EXPECT_EQ(0, bitmap_init(write_set, &write_set_buf, s->fields, false));
+    EXPECT_EQ(0, bitmap_init(read_set, &read_set_buf, s->fields, false));
 
     const_table = false;
-    pos_in_table_list->set_tableno(highest_table_id);
+    table_list.set_tableno(highest_table_id);
     highest_table_id = (highest_table_id + 1) % MAX_TABLES;
     key_info = &m_keys[0];
-    record[0] = &m_record[0];
-    memset(record[0], 0, sizeof(m_record));
-    null_flags = m_null_flags;
-    memset(null_flags, 0, sizeof(m_null_flags));
-    s->null_bytes = sizeof(m_null_flags);
     for (int i = 0; i < max_keys; i++)
       key_info[i].key_part = m_key_part_infos[i];
-    highest_index_id = 0;
+    // We choose non-zero to avoid it working by coincidence.
+    highest_index_id = 3;
 
     set_handler(&mock_handler);
     mock_handler.change_table_ptr(this, &table_share);
@@ -174,9 +164,11 @@ class Fake_TABLE : public TABLE {
     Mock_field_long columns, and this is their pack_length.
   */
   static const int DEFAULT_PACK_LENGTH = Mock_field_long::PACK_LENGTH;
-  NiceMock<Mock_HANDLER> mock_handler{&fake_handlerton, &table_share};
+  NiceMock<Mock_HANDLER> mock_handler;
 
-  explicit Fake_TABLE(List<Field> fields) : table_share(fields.elements) {
+  Fake_TABLE(List<Field> fields)
+      : table_share(fields.elements),
+        mock_handler(static_cast<handlerton *>(NULL), &table_share) {
     initialize();
     List_iterator<Field> it(fields);
     int nbr_fields = 0;
@@ -184,18 +176,24 @@ class Fake_TABLE : public TABLE {
       add(cur_field, nbr_fields++);
   }
 
-  explicit Fake_TABLE(Field *column) : table_share(1) {
+  Fake_TABLE(Field *column)
+      : table_share(1),
+        mock_handler(static_cast<handlerton *>(NULL), &table_share) {
     initialize();
     add(column, 0);
   }
 
-  Fake_TABLE(Field *column1, Field *column2) : table_share(2) {
+  Fake_TABLE(Field *column1, Field *column2)
+      : table_share(2),
+        mock_handler(static_cast<handlerton *>(NULL), &table_share) {
     initialize();
     add(column1, 0);
     add(column2, 1);
   }
 
-  Fake_TABLE(Field *column1, Field *column2, Field *column3) : table_share(3) {
+  Fake_TABLE(Field *column1, Field *column2, Field *column3)
+      : table_share(3),
+        mock_handler(static_cast<handlerton *>(NULL), &table_share) {
     initialize();
     add(column1, 0);
     add(column2, 1);
@@ -212,13 +210,12 @@ class Fake_TABLE : public TABLE {
   Fake_TABLE(int column_count, bool cols_nullable)
       : table_share(column_count),
         mock_handler(&fake_handlerton, &table_share) {
-    assert(static_cast<size_t>(column_count) <= sizeof(int) * 8);
+    DBUG_ASSERT(static_cast<size_t>(column_count) <= sizeof(int) * 8);
     initialize();
     for (int i = 0; i < column_count; ++i) {
       std::stringstream str;
       str << "field_" << (i + 1);
-      add(new (*THR_MALLOC)
-              Mock_field_long(str.str().c_str(), cols_nullable, false),
+      add(new (*THR_MALLOC) Mock_field_long(str.str().c_str(), cols_nullable),
           i);
     }
   }
@@ -226,24 +223,20 @@ class Fake_TABLE : public TABLE {
   /**
     Creates a fake TABLE and stores the values in their corresponding Fields.
 
-    @param column_values The column values to be stored.
+    @param column_value The column values to be stored.
     @param are_nullable Whether the columns are nullable.
   */
   Fake_TABLE(std::initializer_list<int> column_values, bool are_nullable = true)
       : table_share(column_values.size()),
-        mock_handler(static_cast<handlerton *>(nullptr), &table_share) {
+        mock_handler(static_cast<handlerton *>(NULL), &table_share) {
     field = m_field_array;
-    initialize();
     for (size_t i = 0; i < column_values.size(); ++i) {
       std::stringstream s;
       s << "field_" << i + 1;
-      field[i] =
-          new (*THR_MALLOC) Mock_field_long(s.str(), are_nullable, false);
+      field[i] = new (*THR_MALLOC) Mock_field_long(s.str(), are_nullable);
       field[i]->table = this;
-      const ptrdiff_t field_offset = i * MAX_FIELD_WIDTH;
-      field[i]->set_field_ptr(record[0] + field_offset + 1);
-      if (are_nullable) field[i]->set_null_ptr(record[0] + field_offset, 1);
     }
+    initialize();
     int i = 0;
     for (auto column_value : column_values) {
       auto item = new Item_int(column_value);
@@ -259,58 +252,27 @@ class Fake_TABLE : public TABLE {
   }
 
   // Defines an index over (column1, column2) and generates a unique id.
-  int create_index(Field *column1, Field *column2, bool unique = false) {
-    return create_index(column1, column2, unique ? ulong{HA_NOSAME} : 0);
-  }
-
-  int create_index(Field *column1, Field *column2, ulong key_flags) {
-    column1->set_flag(PART_KEY_FLAG);
+  int create_index(Field *column1, Field *column2) {
+    column1->flags |= PART_KEY_FLAG;
+    column2->flags |= PART_KEY_FLAG;
     int index_id = highest_index_id++;
     column1->key_start.set_bit(index_id);
     keys_in_use_for_query.set_bit(index_id);
-    column1->part_of_key.set_bit(index_id);
-
-    if (column2 != nullptr) {
-      column2->set_flag(PART_KEY_FLAG);
-      column2->part_of_key.set_bit(index_id);
-    }
-
-    KEY *key = &m_keys[index_id];
-    key->table = this;
-    key->flags = key->actual_flags = key_flags;
-    key->actual_key_parts = key->user_defined_key_parts = 1;
-    key->key_part[0].init_from_field(column1);
-    ++s->key_parts;
-    if (column2 != nullptr) {
-      key->actual_key_parts = key->user_defined_key_parts = 2;
-      key->key_part[1].init_from_field(column2);
-      ++s->key_parts;
-    }
-    key->name = "unittest_index";
-
-    ++s->keys;
-    s->visible_indexes.set_bit(index_id);
-    s->keys_in_use.set_bit(index_id);
     return index_id;
   }
 
   void set_handler(handler *h) { file = h; }
   TABLE_SHARE *get_share() { return &table_share; }
 
-  static void reset_highest_table_id() { highest_table_id = 0; }
-
  private:
   void add(Field *new_field, int pos) {
     field[pos] = new_field;
     new_field->table = this;
+    new_field->orig_table = this;
     static const char *table_name = "Fake";
     new_field->table_name = &table_name;
-    new_field->set_field_index(pos);
+    new_field->field_index = pos;
     bitmap_set_bit(read_set, pos);
-    const ptrdiff_t field_offset = pos * MAX_FIELD_WIDTH;
-    new_field->set_field_ptr(record[0] + field_offset);
-    if (new_field->get_null_ptr() != nullptr)
-      new_field->set_null_ptr(null_flags + pos / 8, 1 << (pos % 8));
   }
 };
 
