@@ -1,16 +1,15 @@
-/* Copyright (c) 2013, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,6 +20,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
+// First include (the generated) my_config.h, to get correct platform defines.
 #include "my_config.h"
 
 #include <gmock/gmock.h>
@@ -46,13 +46,13 @@ const char escape[] = "\\";
 
 class ItemLikeTest : public ::testing::Test {
  protected:
-  void SetUp() override {
+  virtual void SetUp() {
     initializer.SetUp();
     it_haystack =
         new Item_string(STRING_WITH_LEN(haystack), &my_charset_latin1);
     it_escape = new Item_string(STRING_WITH_LEN(escape), &my_charset_latin1);
   }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void TearDown() { initializer.TearDown(); }
 
   THD *thd() { return initializer.thd(); }
 
@@ -67,18 +67,36 @@ TEST_F(ItemLikeTest, TestOne) {
   Item_string *it_needle =
       new Item_string(STRING_WITH_LEN(needle), &my_charset_latin1);
   Item_func_like *item_BM =
-      new Item_func_like(it_haystack, it_needle, it_escape);
-  EXPECT_FALSE(item_BM->fix_fields(thd(), nullptr));
+      new Item_func_like(it_haystack, it_needle, it_escape, false);
+  EXPECT_FALSE(item_BM->fix_fields(thd(), NULL));
   EXPECT_EQ(1, item_BM->val_int());
 }
 
+// Increase number when doing performance comparisons.
+// Run with ./unittest/gunit/item_like-t --disable-tap-output
+//   to get timings from googletest.
+const int num_executions = 1;  // 100000;
+
+TEST_F(ItemLikeTest, PerfBasic) {
+  const char needle[] = "%CAAAACCACTATGAGATATCATCTCACACCAGTTA%";
+  Item_string *it_needle =
+      new Item_string(STRING_WITH_LEN(needle), &my_charset_latin1);
+
+  Item_func_like *item_BM =
+      new Item_func_like(it_haystack, it_needle, it_escape, false);
+  EXPECT_FALSE(item_BM->fix_fields(thd(), NULL));
+  for (int ix = 0; ix < num_executions; ++ix) {
+    EXPECT_EQ(1, item_BM->val_int());
+  }
+}
+
 class ItemLikeTestP : public ::testing::TestWithParam<const char *> {
  protected:
-  void SetUp() override {
+  virtual void SetUp() {
     initializer.SetUp();
     m_needle = GetParam();
   }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void TearDown() { initializer.TearDown(); }
 
   THD *thd() { return initializer.thd(); }
 
@@ -88,11 +106,11 @@ class ItemLikeTestP : public ::testing::TestWithParam<const char *> {
 
 class ItemFalseLikeTestP : public ::testing::TestWithParam<const char *> {
  protected:
-  void SetUp() override {
+  virtual void SetUp() {
     initializer.SetUp();
     m_needle = GetParam();
   }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void TearDown() { initializer.TearDown(); }
 
   THD *thd() { return initializer.thd(); }
 
@@ -173,8 +191,8 @@ static const char *needles[] = {"%CAAAACCACTATGAGATATCATCTCACACCAGTTA%",
                                 "%ACAGGTGCTGGAGAGGATGCGGAGAAATAGGAACA%",
                                 "%CAGGTGCTGGAGAGGATGCGGAGAAATAGGAACAC%"};
 
-INSTANTIATE_TEST_SUITE_P(MoreNeedles, ItemLikeTestP,
-                         ::testing::ValuesIn(needles));
+INSTANTIATE_TEST_CASE_P(MoreNeedles, ItemLikeTestP,
+                        ::testing::ValuesIn(needles));
 
 TEST_P(ItemLikeTestP, MoreNeedlesTest) {
   Item_string *it_haystack =
@@ -185,8 +203,8 @@ TEST_P(ItemLikeTestP, MoreNeedlesTest) {
       new Item_string(STRING_WITH_LEN(escape), &my_charset_latin1);
 
   Item_func_like *item_BM =
-      new Item_func_like(it_haystack, it_needle, it_escape);
-  EXPECT_FALSE(item_BM->fix_fields(thd(), nullptr));
+      new Item_func_like(it_haystack, it_needle, it_escape, false);
+  EXPECT_FALSE(item_BM->fix_fields(thd(), NULL));
   EXPECT_EQ(1, item_BM->val_int());
 }
 
@@ -260,8 +278,8 @@ static const char *falseNeedles[] = {
     "%ACAAGGATAAAGAGGCGTAGGAGAGGTCGTGGACA%",
     "%CACAAGGATAAAGAGGCGTAGGAGAGGTCGTGGAC%"};
 
-INSTANTIATE_TEST_SUITE_P(FalseNeedlesTest, ItemFalseLikeTestP,
-                         ::testing::ValuesIn(falseNeedles));
+INSTANTIATE_TEST_CASE_P(FalseNeedlesTest, ItemFalseLikeTestP,
+                        ::testing::ValuesIn(falseNeedles));
 
 TEST_P(ItemFalseLikeTestP, FalseNeedlesTest) {
   Item_string *it_haystack =
@@ -272,8 +290,8 @@ TEST_P(ItemFalseLikeTestP, FalseNeedlesTest) {
       new Item_string(STRING_WITH_LEN(escape), &my_charset_latin1);
 
   Item_func_like *item_BM =
-      new Item_func_like(it_haystack, it_needle, it_escape);
-  EXPECT_FALSE(item_BM->fix_fields(thd(), nullptr));
+      new Item_func_like(it_haystack, it_needle, it_escape, false);
+  EXPECT_FALSE(item_BM->fix_fields(thd(), NULL));
   EXPECT_EQ(0, item_BM->val_int());
 }
 
