@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,12 +22,10 @@
 
 #include <gtest/gtest.h>
 #include <sys/types.h>
-#include <random>
 #include <vector>
 
 #include "my_inttypes.h"
 #include "sql/merge_sort.h"
-#include "sql/sql_planner.h"
 #include "sql/sql_select.h"
 #include "unittest/gunit/test_utils.h"
 
@@ -38,15 +35,11 @@ using my_testing::Mock_error_handler;
 using my_testing::Server_initializer;
 
 class JTSortTest : public ::testing::Test {
- public:
-  JTSortTest() : urng(std::random_device().operator()()) {}
-
  protected:
-  void SetUp() override { initializer.SetUp(); }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void SetUp() { initializer.SetUp(); }
+  virtual void TearDown() { initializer.TearDown(); }
 
   Server_initializer initializer;
-  std::mt19937 urng;
 };
 
 class MOCK_JOIN_TAB : public JOIN_TAB {
@@ -60,7 +53,7 @@ class MOCK_JOIN_TAB : public JOIN_TAB {
   }
 
   TABLE m_table;
-  Table_ref m_table_list;
+  TABLE_LIST m_table_list;
   QEP_shared m_shared;
 };
 
@@ -99,13 +92,13 @@ TEST_F(JTSortTest, SortFoundRecordsTest) {
     arr[i] = new (*THR_MALLOC) MOCK_JOIN_TAB(i, 0);
 
   // MERGE SORT
-  std::shuffle(arr, arr + 50, urng);
+  std::random_shuffle(arr, arr + 50);
   merge_sort(arr, arr + num_tables, Join_tab_compare_default());
   for (int i = 1; i < num_tables; i++)
     EXPECT_TRUE(arr[i]->found_records > arr[i - 1]->found_records);
 
   // INSERT SORT
-  std::shuffle(arr, arr + 50, urng);
+  std::random_shuffle(arr, arr + 50);
   insert_sort(arr, arr + num_tables, Join_tab_compare_default());
   for (int i = 1; i < num_tables; i++)
     EXPECT_TRUE(arr[i]->found_records > arr[i - 1]->found_records);
@@ -129,7 +122,7 @@ TEST_F(JTSortTest, SortDependsTest) {
   }
 
   // MERGE SORT
-  std::shuffle(arr, arr + num_tables, urng);
+  std::random_shuffle(arr, arr + num_tables);
   merge_sort(arr, arr + num_tables, Join_tab_compare_default());
   for (int i = 1; i < num_tables; i++)
     EXPECT_TRUE(arr[i]->found_records < arr[i - 1]->found_records)
@@ -137,7 +130,7 @@ TEST_F(JTSortTest, SortDependsTest) {
         << "i-1: " << *(arr[i - 1]);
 
   // INSERT SORT
-  std::shuffle(arr, arr + num_tables, urng);
+  std::random_shuffle(arr, arr + num_tables);
   insert_sort(arr, arr + num_tables, Join_tab_compare_default());
   for (int i = 1; i < num_tables; i++)
     EXPECT_TRUE(arr[i]->found_records < arr[i - 1]->found_records);
@@ -162,13 +155,13 @@ TEST_F(JTSortTest, SortKeyDependsTest) {
   }
 
   // MERGE SORT
-  std::shuffle(arr, arr + num_tables, urng);
+  std::random_shuffle(arr, arr + num_tables);
   merge_sort(arr, arr + num_tables, Join_tab_compare_default());
   for (int i = 1; i < num_tables; i++)
     EXPECT_TRUE(arr[i]->found_records < arr[i - 1]->found_records);
 
   // INSERT SORT
-  std::shuffle(arr, arr + num_tables, urng);
+  std::random_shuffle(arr, arr + num_tables);
   insert_sort(arr, arr + num_tables, Join_tab_compare_default());
   for (int i = 1; i < num_tables; i++)
     EXPECT_TRUE(arr[i]->found_records < arr[i - 1]->found_records);
@@ -204,12 +197,12 @@ TEST_F(JTSortTest, SortIntTest) {
   EXPECT_TRUE(arr_ptr.size() == ints_to_sort);
 
   // MERGE SORT
-  std::shuffle(&arr_ptr.front(), &arr_ptr.back() + 1, urng);
+  std::random_shuffle(&arr_ptr.front(), &arr_ptr.back() + 1);
   merge_sort(&arr_ptr.front(), &arr_ptr.back() + 1, Int_compare_ptr());
   for (uint i = 0; i < arr_ptr.size(); i++) EXPECT_TRUE(*arr_ptr[i] == (int)i);
 
   // INSERT SORT
-  std::shuffle(&arr_ptr.front(), &arr_ptr.back() + 1, urng);
+  std::random_shuffle(&arr_ptr.front(), &arr_ptr.back() + 1);
   insert_sort(&arr_ptr.front(), &arr_ptr.back() + 1, Int_compare_ptr());
   for (uint i = 0; i < arr_ptr.size(); i++) EXPECT_TRUE(*arr_ptr[i] == (int)i);
 }
@@ -238,13 +231,13 @@ TEST_F(JTSortTest, SortInt2Test) {
   EXPECT_TRUE(arr_ptr.size() == ints_to_sort);
 
   // MERGE SORT
-  std::shuffle(&arr_ptr.front(), &arr_ptr.back() + 1, urng);
+  std::random_shuffle(&arr_ptr.front(), &arr_ptr.back() + 1);
   merge_sort(&arr_ptr.front(), &arr_ptr.back() + 1, Int_compare_ptr());
   for (uint i = 1; i < arr_ptr.size(); i++)
     EXPECT_TRUE(*arr_ptr[i - 1] < *arr_ptr[i]);
 
   // INSERT SORT
-  std::shuffle(&arr_ptr.front(), &arr_ptr.back() + 1, urng);
+  std::random_shuffle(&arr_ptr.front(), &arr_ptr.back() + 1);
   insert_sort(&arr_ptr.front(), &arr_ptr.back() + 1, Int_compare_ptr());
   for (uint i = 1; i < arr_ptr.size(); i++)
     EXPECT_TRUE(*arr_ptr[i - 1] < *arr_ptr[i]);
