@@ -1,16 +1,15 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,11 +34,11 @@ namespace gcs_xcom_packet_unittest {
 
 class GcsPacketTest : public GcsBaseTest {
  protected:
-  GcsPacketTest() = default;
+  GcsPacketTest() {}
 
-  void SetUp() override { lz4_stage = new Gcs_message_stage_lz4(true, 1024); }
+  virtual void SetUp() { lz4_stage = new Gcs_message_stage_lz4(true, 1024); }
 
-  void TearDown() override { delete lz4_stage; }
+  virtual void TearDown() { delete lz4_stage; }
 
   Gcs_message_stage_lz4 *lz4_stage;
 
