@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,18 +31,18 @@ namespace gcs_xcom_groupmanagement_unittest {
 class mock_gcs_xcom_proxy : public Gcs_xcom_proxy_base {
  public:
   mock_gcs_xcom_proxy() {
+    ON_CALL(*this, xcom_exit(_)).WillByDefault(Return(1));
     ON_CALL(*this, xcom_client_boot(_, _)).WillByDefault(Return(1));
     ON_CALL(*this, xcom_client_add_node(_, _, _)).WillByDefault(Return(1));
     ON_CALL(*this, xcom_client_send_data(_, _)).WillByDefault(Return(10));
     ON_CALL(*this, new_node_address_uuid(_, _, _))
-        .WillByDefault(Invoke(::new_node_address_uuid));
+        .WillByDefault(WithArgs<0, 1, 2>(Invoke(::new_node_address_uuid)));
     ON_CALL(*this, delete_node_address(_, _))
-        .WillByDefault(Invoke(::delete_node_address));
+        .WillByDefault(WithArgs<0, 1>(Invoke(::delete_node_address)));
   }
 
   MOCK_METHOD3(new_node_address_uuid,
-               node_address *(unsigned int n, char const *names[],
-                              blob uuids[]));
+               node_address *(unsigned int n, char *names[], blob uuid[]));
   MOCK_METHOD2(delete_node_address, void(unsigned int n, node_address *na));
   MOCK_METHOD3(xcom_client_add_node, bool(connection_descriptor *fd,
                                           node_list *nl, uint32_t group_id));
@@ -54,13 +53,6 @@ class mock_gcs_xcom_proxy : public Gcs_xcom_proxy_base {
                bool(uint32_t group_id, xcom_event_horizon &event_horizon));
   MOCK_METHOD2(xcom_client_set_event_horizon,
                bool(uint32_t group_id, xcom_event_horizon event_horizon));
-  MOCK_METHOD2(xcom_client_set_max_leaders,
-               bool(uint32_t group_id, node_no max_leaders));
-  MOCK_METHOD4(xcom_client_set_leaders,
-               bool(uint32_t group_id, u_int n, char const *names[],
-                    node_no max_nr_leaders));
-  MOCK_METHOD2(xcom_client_get_leaders,
-               bool(uint32_t gid, leader_info_data &leaders));
   MOCK_METHOD4(xcom_client_get_synode_app_data,
                bool(connection_descriptor *con, uint32_t group_id_hash,
                     synode_no_array &synodes, synode_app_data_array &reply));
@@ -72,7 +64,7 @@ class mock_gcs_xcom_proxy : public Gcs_xcom_proxy_base {
   MOCK_METHOD2(xcom_client_send_data,
                bool(unsigned long long size, char *data));
   MOCK_METHOD1(xcom_init, void(xcom_port listen_port));
-  MOCK_METHOD0(xcom_exit, void());
+  MOCK_METHOD1(xcom_exit, bool(bool xcom_input_open));
   MOCK_METHOD0(xcom_set_cleanup, void());
   MOCK_METHOD1(xcom_get_ssl_mode, int(const char *mode));
   MOCK_METHOD1(xcom_set_ssl_mode, int(int mode));
@@ -111,47 +103,22 @@ class mock_gcs_xcom_proxy : public Gcs_xcom_proxy_base {
   /* Mocking fails compilation on Windows. It attempts to copy the std::future
    * which is non-copyable. */
   Gcs_xcom_input_queue::future_reply xcom_input_try_push_and_get_reply(
-      app_data_ptr) override {
+      app_data_ptr) {
     return std::future<std::unique_ptr<Gcs_xcom_input_queue::Reply>>();
   }
   MOCK_METHOD0(xcom_input_try_pop, xcom_input_request_ptr());
 };
 
-class mock_gcs_xcom_view_change_control_interface
-    : public Gcs_xcom_view_change_control_interface {
- public:
-  MOCK_METHOD0(start_view_exchange, void());
-  MOCK_METHOD0(end_view_exchange, void());
-  MOCK_METHOD0(wait_for_view_change_end, void());
-  MOCK_METHOD0(is_view_changing, bool());
-  MOCK_METHOD0(start_leave, bool());
-  MOCK_METHOD0(end_leave, void());
-  MOCK_METHOD0(is_leaving, bool());
-  MOCK_METHOD0(start_join, bool());
-  MOCK_METHOD0(end_join, void());
-  MOCK_METHOD0(is_joining, bool());
-
-  MOCK_METHOD1(set_current_view, void(Gcs_view *));
-  MOCK_METHOD0(get_current_view, Gcs_view *());
-  MOCK_METHOD0(belongs_to_group, bool());
-  MOCK_METHOD1(set_belongs_to_group, void(bool));
-  MOCK_METHOD1(set_unsafe_current_view, void(Gcs_view *));
-  MOCK_METHOD0(get_unsafe_current_view, Gcs_view *());
-
-  MOCK_METHOD0(finalize, void());
-  MOCK_METHOD0(is_finalized, bool());
-};
-
 class XcomGroupManagementTest : public GcsBaseTest {
  protected:
-  XcomGroupManagementTest() = default;
+  XcomGroupManagementTest() {}
 
-  void SetUp() override {
+  virtual void SetUp() {
     group_id = new Gcs_group_identifier("only_group");
-    xcom_group_mgmt_if = new Gcs_xcom_group_management(&proxy, *group_id, &vce);
+    xcom_group_mgmt_if = new Gcs_xcom_group_management(&proxy, *group_id);
   }
 
-  void TearDown() override {
+  virtual void TearDown() {
     delete xcom_group_mgmt_if;
     delete group_id;
   }
@@ -159,7 +126,6 @@ class XcomGroupManagementTest : public GcsBaseTest {
   Gcs_group_identifier *group_id;
 
   mock_gcs_xcom_proxy proxy;
-  mock_gcs_xcom_view_change_control_interface vce;
   Gcs_xcom_group_management *xcom_group_mgmt_if;
 };
 
@@ -205,12 +171,10 @@ MATCHER_P(node_list_pointer_matcher, other_nl, "Derreference pointer") {
 TEST_F(XcomGroupManagementTest, TestListContent) {
   Gcs_xcom_node_information node_1("127.0.0.1:12345");
   Gcs_xcom_node_information node_2("127.0.0.1:12346");
-  Gcs_xcom_node_information node_3("127.0.0.1:12347");
 
   Gcs_xcom_nodes nodes;
   nodes.add_node(node_1);
   nodes.add_node(node_2);
-  nodes.add_node(node_3);
 
   node_list nl;
   nl.node_list_len = 2;
@@ -227,8 +191,8 @@ TEST_F(XcomGroupManagementTest, TestListContent) {
       reinterpret_cast<uchar **>(&blobs[1].data.data_val),
       &blobs[1].data.data_len);
 
-  nl.node_list_val =
-      ::new_node_address_uuid(nl.node_list_len, node_addrs, blobs);
+  nl.node_list_val = ::new_node_address_uuid(
+      nl.node_list_len, const_cast<char **>(node_addrs), blobs);
 
   EXPECT_CALL(proxy, xcom_client_force_config(node_list_pointer_matcher(nl), _))
       .Times(1)
@@ -252,76 +216,4 @@ TEST_F(XcomGroupManagementTest, TestListContent) {
   free(blobs[1].data.data_val);
 }
 
-TEST_F(XcomGroupManagementTest, DisallowForcingSameMembership) {
-  Gcs_xcom_node_information node_1("127.0.0.1:12345");
-  Gcs_xcom_node_information node_2("127.0.0.1:12346");
-
-  Gcs_xcom_nodes nodes;
-  nodes.add_node(node_1);
-  nodes.add_node(node_2);
-
-  node_list nl;
-  nl.node_list_len = 2;
-  const char *node_addrs[] = {node_1.get_member_id().get_member_id().c_str(),
-                              node_2.get_member_id().get_member_id().c_str()};
-  blob blobs[] = {{{0, static_cast<char *>(malloc(
-                           node_1.get_member_uuid().actual_value.size()))}},
-                  {{0, static_cast<char *>(malloc(
-                           node_2.get_member_uuid().actual_value.size()))}}};
-  node_1.get_member_uuid().encode(
-      reinterpret_cast<uchar **>(&blobs[0].data.data_val),
-      &blobs[0].data.data_len);
-  node_2.get_member_uuid().encode(
-      reinterpret_cast<uchar **>(&blobs[1].data.data_val),
-      &blobs[1].data.data_len);
-
-  nl.node_list_val =
-      ::new_node_address_uuid(nl.node_list_len, node_addrs, blobs);
-
-  EXPECT_CALL(proxy, xcom_client_force_config(node_list_pointer_matcher(nl), _))
-      .Times(0);
-
-  xcom_group_mgmt_if->set_xcom_nodes(nodes);
-
-  Gcs_interface_parameters forced_group_1;
-  forced_group_1.add_parameter("peer_nodes", "127.0.0.1:12346,127.0.0.1:12345");
-  enum_gcs_error result_1 =
-      xcom_group_mgmt_if->modify_configuration(forced_group_1);
-  ASSERT_EQ(GCS_NOK, result_1);
-
-  Gcs_interface_parameters forced_group_2;
-  forced_group_2.add_parameter("peer_nodes", "127.0.0.1:12345,127.0.0.1:12346");
-  enum_gcs_error result_2 =
-      xcom_group_mgmt_if->modify_configuration(forced_group_2);
-  ASSERT_EQ(GCS_NOK, result_2);
-
-  ASSERT_EQ((unsigned)2, nl.node_list_len);
-  ASSERT_STREQ("127.0.0.1:12345", nl.node_list_val[0].address);
-  ASSERT_STREQ("127.0.0.1:12346", nl.node_list_val[1].address);
-
-  ::delete_node_address(nl.node_list_len, nl.node_list_val);
-
-  free(blobs[0].data.data_val);
-  free(blobs[1].data.data_val);
-}
-
-TEST_F(XcomGroupManagementTest, getWriteConcurrencyGroupLeaving) {
-  EXPECT_CALL(vce, is_leaving()).Times(1).WillOnce(Return(true));
-
-  uint32_t out_value;
-  enum_gcs_error result = xcom_group_mgmt_if->get_write_concurrency(out_value);
-
-  ASSERT_EQ(GCS_NOK, result);
-}
-
-TEST_F(XcomGroupManagementTest, getWriteConcurrencyNoGroup) {
-  EXPECT_CALL(vce, is_leaving()).Times(1).WillOnce(Return(false));
-  EXPECT_CALL(proxy, xcom_is_exit()).Times(1).WillOnce(Return(true));
-
-  uint32_t out_value;
-  enum_gcs_error result = xcom_group_mgmt_if->get_write_concurrency(out_value);
-
-  ASSERT_EQ(GCS_NOK, result);
-}
-
 }  // namespace gcs_xcom_groupmanagement_unittest
