@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -42,9 +41,9 @@ namespace gcs_parameters_unittest {
 
 class GcsNodeAddressTest : public GcsBaseTest {
  protected:
-  GcsNodeAddressTest() = default;
+  GcsNodeAddressTest() {}
 
-  static void SetUpTestCase() {}
+  static void SetUpTestCase() { My_xp_util::init_time(); }
 };
 
 TEST_F(GcsNodeAddressTest, TestNodeAddress) {
@@ -258,16 +257,16 @@ TEST_F(GcsNodeAddressTest, TestNodeAddressV6) {
 
 class GcsUUIDTest : public GcsBaseTest {
  protected:
-  GcsUUIDTest() = default;
+  GcsUUIDTest() {}
 
-  static void SetUpTestCase() {}
+  static void SetUpTestCase() { My_xp_util::init_time(); }
 };
 
 TEST_F(GcsUUIDTest, TestGcsUUID) {
   /*
     Check that the UUIDs are different as expected.
   */
-  uchar *buffer = nullptr;
+  uchar *buffer = NULL;
   u_int size = 0;
   Gcs_xcom_uuid uuid_1 = Gcs_xcom_uuid::create_uuid();
 
@@ -294,7 +293,7 @@ TEST_F(GcsUUIDTest, TestGcsUUID) {
   */
   My_xp_util::sleep_seconds(1);
   Gcs_xcom_uuid uuid_2 = Gcs_xcom_uuid::create_uuid();
-  ASSERT_FALSE(uuid_2.decode(nullptr, size));
+  ASSERT_FALSE(uuid_2.decode(static_cast<uchar *>(NULL), size));
 
   /*
     Check that if we decode the buffer objects will have the same value.
@@ -308,9 +307,9 @@ TEST_F(GcsUUIDTest, TestGcsUUID) {
 
 class GcsNodeInformationTest : public GcsBaseTest {
  protected:
-  GcsNodeInformationTest() = default;
+  GcsNodeInformationTest() {}
 
-  static void SetUpTestCase() {}
+  static void SetUpTestCase() { My_xp_util::init_time(); }
 };
 
 TEST_F(GcsNodeInformationTest, TestGcsNodeInformation) {
@@ -395,9 +394,9 @@ TEST_F(GcsNodeInformationTest, TestGcsNodeInformation) {
 
 class GcsNodesTest : public GcsBaseTest {
  protected:
-  GcsNodesTest() = default;
+  GcsNodesTest() {}
 
-  static void SetUpTestCase() {}
+  static void SetUpTestCase() { My_xp_util::init_time(); }
 };
 
 TEST_F(GcsNodesTest, TestGcsNodesBasicProperties) {
@@ -412,7 +411,7 @@ TEST_F(GcsNodesTest, TestGcsNodesBasicProperties) {
   Gcs_xcom_node_information node_3("localhost:1033");
   node_3.set_node_no(3);
 
-  const Gcs_xcom_node_information *ret = nullptr;
+  const Gcs_xcom_node_information *ret = NULL;
 
   /*
     Initially the node's number is void and check whether the
@@ -456,7 +455,7 @@ TEST_F(GcsNodesTest, TestGcsNodesBasicProperties) {
   ASSERT_EQ(ret->get_member_id(), node_2.get_member_id());
 
   ret = nodes.get_node(node_3.get_member_id());
-  ASSERT_TRUE(ret == nullptr);
+  ASSERT_TRUE(ret == NULL);
 
   /*
     Trying to get a node using get_node(const std::string &member_id).
@@ -468,7 +467,7 @@ TEST_F(GcsNodesTest, TestGcsNodesBasicProperties) {
   ASSERT_EQ(ret->get_member_id(), node_2.get_member_id());
 
   ret = nodes.get_node(node_3.get_member_id().get_member_id());
-  ASSERT_TRUE(ret == nullptr);
+  ASSERT_TRUE(ret == NULL);
 
   /*
     Trying to get a node using get_node(unsigned int node_no).
@@ -480,7 +479,7 @@ TEST_F(GcsNodesTest, TestGcsNodesBasicProperties) {
   ASSERT_EQ(ret->get_member_id(), node_2.get_member_id());
 
   ret = nodes.get_node(node_3.get_member_id().get_member_id());
-  ASSERT_TRUE(ret == nullptr);
+  ASSERT_TRUE(ret == NULL);
 
   /*
     Trying to get a node using get_node(const Gcs_xcom_uuid &uuid).
@@ -505,9 +504,9 @@ TEST_F(GcsNodesTest, TestGcsNodesCopyingNodes) {
   node_2.set_node_no(2);
   nodes.add_node(node_2);
 
-  const Gcs_xcom_node_information *ret = nullptr;
-  Gcs_xcom_node_information *ret_1 = nullptr;
-  Gcs_xcom_node_information *ret_2 = nullptr;
+  const Gcs_xcom_node_information *ret = NULL;
+  Gcs_xcom_node_information *ret_1 = NULL;
+  Gcs_xcom_node_information *ret_2 = NULL;
 
   /*
     Copying nodes through the add_nodes method which cleans up
@@ -582,15 +581,15 @@ TEST_F(GcsNodesTest, TestGcsNodesEncoding) {
     addrs[n] -> Pointer to the member identifier as string.
     uuids[n] -> data.data_len and data.data_val.
   */
-  const Gcs_xcom_node_information *ret_1 = nullptr;
-  const Gcs_xcom_node_information *ret_2 = nullptr;
+  const Gcs_xcom_node_information *ret_1 = NULL;
+  const Gcs_xcom_node_information *ret_2 = NULL;
 
   Gcs_xcom_uuid uuid_1;
   Gcs_xcom_uuid uuid_2;
 
   unsigned int length = 0;
-  char const **addrs = nullptr;
-  blob *uuids = nullptr;
+  char **addrs = NULL;
+  blob *uuids = NULL;
 
   nodes.encode(&length, &addrs, &uuids);
 
@@ -618,7 +617,7 @@ TEST_F(GcsNodesTest, TestGcsNodesEncoding) {
 }
 
 TEST_F(GcsNodesTest, TestGcsNodesConstructor) {
-  const Gcs_xcom_node_information *ret = nullptr;
+  const Gcs_xcom_node_information *ret = NULL;
 
   Gcs_xcom_uuid uuid_1 = Gcs_xcom_uuid::create_uuid();
   blob blob_1 = {{0, static_cast<char *>(malloc(uuid_1.actual_value.size()))}};
@@ -630,14 +629,9 @@ TEST_F(GcsNodesTest, TestGcsNodesConstructor) {
   uuid_2.encode(reinterpret_cast<uchar **>(&blob_2.data.data_val),
                 &blob_2.data.data_len);
 
-  node_address node_addrs[2] = {{const_cast<char *>("127.0.0.1:12345"),
-                                 blob_1,
-                                 {x_1_0, x_1_2},
-                                 P_PROP | P_ACC | P_LEARN},
-                                {const_cast<char *>("127.0.0.1:12343"),
-                                 blob_2,
-                                 {x_1_0, x_1_2},
-                                 P_PROP | P_ACC | P_LEARN}};
+  node_address node_addrs[2] = {
+      {const_cast<char *>("127.0.0.1:12345"), blob_1, {x_1_0, x_1_2}},
+      {const_cast<char *>("127.0.0.1:12343"), blob_2, {x_1_0, x_1_2}}};
 
   site_def *site_config = new_site_def();
   init_site_def(2, node_addrs, site_config);
