@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,7 +25,6 @@
 #include <sys/types.h>
 
 #include "my_inttypes.h"
-#include "my_sys.h"  // ErrorHandlerFunctionPointer
 #include "mysqld_error.h"
 #include "sql/locking_service.h"
 #include "sql/mdl.h"
@@ -54,7 +53,7 @@ const char lock_name4[] = "lock4";
 
 class LockingServiceTest : public ::testing::Test {
  protected:
-  LockingServiceTest() = default;
+  LockingServiceTest() {}
 
   static void SetUpTestCase() {
     m_old_error_handler_hook = error_handler_hook;
@@ -69,7 +68,7 @@ class LockingServiceTest : public ::testing::Test {
     table_def_free();
   }
 
-  void SetUp() override {
+  virtual void SetUp() {
     mdl_init();
     m_initializer.SetUp();
     m_thd = m_initializer.thd();
@@ -79,7 +78,7 @@ class LockingServiceTest : public ::testing::Test {
     m_thd->system_thread = SYSTEM_THREAD_SLAVE_IO;
   }
 
-  void TearDown() override {
+  virtual void TearDown() {
     m_initializer.TearDown();
     mdl_destroy();
   }
@@ -87,10 +86,10 @@ class LockingServiceTest : public ::testing::Test {
   Server_initializer m_initializer;
   THD *m_thd;
 
-  static ErrorHandlerFunctionPointer m_old_error_handler_hook;
+  static void (*m_old_error_handler_hook)(uint, const char *, myf);
 };
 
-ErrorHandlerFunctionPointer LockingServiceTest::m_old_error_handler_hook;
+void (*LockingServiceTest::m_old_error_handler_hook)(uint, const char *, myf);
 
 /**
   Test acquire and release of several read or write locks.
@@ -182,7 +181,7 @@ TEST_F(LockingServiceTest, CaseSensitive) {
 TEST_F(LockingServiceTest, ValidNames) {
   const char *ok_name[] = {"test"};
   {
-    const char *null = nullptr;
+    const char *null = NULL;
     const char *names1[] = {null};
     Mock_error_handler error_handler(m_thd, ER_LOCKING_SERVICE_WRONG_NAME);
     EXPECT_TRUE(acquire_locking_service_locks(m_thd, namespace1, names1, 1,
@@ -273,7 +272,7 @@ class LockServiceThread : public Thread {
         m_lock_grabbed(grabbed_arg),
         m_lock_release(release_arg) {}
 
-  void run() override {
+  virtual void run() {
     Server_initializer m_initializer;
     m_initializer.SetUp();
     THD *m_thd = m_initializer.thd();
@@ -437,9 +436,9 @@ TEST_F(LockingServiceTest, Namespaces) {
 */
 class LockServiceDisconnectThread : public Thread {
  public:
-  LockServiceDisconnectThread() = default;
+  LockServiceDisconnectThread() {}
 
-  void run() override {
+  virtual void run() {
     Server_initializer m_initializer;
     m_initializer.SetUp();
     THD *m_thd = m_initializer.thd();
@@ -474,7 +473,7 @@ class LockServiceDeadlockThread : public Thread {
   LockServiceDeadlockThread(Notification *grabbed1_arg, Notification *wait_arg)
       : m_lock_grabbed1(grabbed1_arg), m_wait(wait_arg) {}
 
-  void run() override {
+  virtual void run() {
     Server_initializer m_initializer;
     m_initializer.SetUp();
     THD *m_thd = m_initializer.thd();
