@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,23 +23,16 @@
 #ifndef MOCK_FIELD_DATETIME_H
 #define MOCK_FIELD_DATETIME_H
 
-#include <memory>
-
 #include "sql/field.h"
-#include "unittest/gunit/fake_table.h"
 
 class Mock_field_datetime : public Field_datetime {
-  std::unique_ptr<Fake_TABLE> fake_table;
-
   void initialize() {
-    fake_table.reset(new Fake_TABLE(this));
-    table = fake_table.get();
-    ptr = table->record[0];
-    // Make it possible to write into this field
-    bitmap_set_bit(table->write_set, 0);
+    ptr = buffer;
+    memset(buffer, 0, PACK_LENGTH);
   }
 
  public:
+  uchar buffer[PACK_LENGTH];
   Mock_field_datetime() : Field_datetime("") { initialize(); }
 };
 
