@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2006, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2006, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -114,8 +113,8 @@ bool test_compare_operators(MY_BITMAP *map, uint bitsize) {
   MY_BITMAP *map2 = &map2_obj, *map3 = &map3_obj;
   my_bitmap_map map2buf[MAX_TESTED_BITMAP_SIZE];
   my_bitmap_map map3buf[MAX_TESTED_BITMAP_SIZE];
-  bitmap_init(&map2_obj, map2buf, bitsize);
-  bitmap_init(&map3_obj, map3buf, bitsize);
+  bitmap_init(&map2_obj, map2buf, bitsize, false);
+  bitmap_init(&map3_obj, map3buf, bitsize, false);
   bitmap_clear_all(map2);
   bitmap_clear_all(map3);
   for (i = 0; i < no_loops; i++) {
@@ -354,7 +353,7 @@ bool test_compare(MY_BITMAP *map, uint bitsize) {
   my_bitmap_map map2buf[MAX_TESTED_BITMAP_SIZE];
   uint i, test_bit;
   uint no_loops = bitsize > 128 ? 128 : bitsize;
-  bitmap_init(&map2, map2buf, bitsize);
+  bitmap_init(&map2, map2buf, bitsize, false);
 
   /* Test all 4 possible combinations of set/unset bits. */
   for (i = 0; i < no_loops; i++) {
@@ -398,51 +397,46 @@ bool test_intersect(MY_BITMAP *map, uint bitsize) {
   uint bitsize2 = 1 + get_rand_bit(MAX_TESTED_BITMAP_SIZE - 1);
   MY_BITMAP map2;
   my_bitmap_map *map2buf = new my_bitmap_map[bitsize2];
-  bitmap_init(&map2, map2buf, bitsize2);
+  uint i, test_bit1, test_bit2, test_bit3;
+  bitmap_init(&map2, map2buf, bitsize2, false);
 
-  uint test_bit1 = get_rand_bit(bitsize);
-  uint test_bit2 = get_rand_bit(bitsize2);
+  test_bit1 = get_rand_bit(bitsize);
+  test_bit2 = get_rand_bit(bitsize);
+  bitmap_set_bit(map, test_bit1);
+  bitmap_set_bit(map, test_bit2);
+  test_bit3 = get_rand_bit(bitsize2);
+  bitmap_set_bit(&map2, test_bit3);
+  if (test_bit2 < bitsize2) bitmap_set_bit(&map2, test_bit2);
 
-  if (test_bit2 < bitsize) {
-    // test_bit2 can be set in map, so the intersection is not empty iff
-    // test_bit1 == test_bit2
-    bitmap_set_bit(map, test_bit1);
-    bitmap_set_bit(&map2, test_bit2);
-    bitmap_intersect(map, &map2);
-    if (test_bit1 == test_bit2) {
-      if (!bitmap_is_set(map, test_bit1)) goto error;
-      bitmap_clear_bit(map, test_bit2);
-    } else {
-      if (bitmap_is_set(map, test_bit1)) goto error;
-    }
-  } else {
-    // test_bit2 cannot be set in map, so the intersection should be empty
-    bitmap_set_bit(map, test_bit1);
-    bitmap_set_bit(&map2, test_bit2);
-    bitmap_intersect(map, &map2);
+  bitmap_intersect(map, &map2);
+  if (test_bit2 < bitsize2) {
+    if (!bitmap_is_set(map, test_bit2)) goto error;
+    bitmap_clear_bit(map, test_bit2);
+  }
+  if (test_bit1 == test_bit3) {
+    if (!bitmap_is_set(map, test_bit1)) goto error;
+    bitmap_clear_bit(map, test_bit1);
   }
-
   if (!bitmap_is_clear_all(map)) goto error;
 
   bitmap_set_all(map);
   bitmap_set_all(&map2);
-  for (uint i = 0; i < bitsize2; i++) bitmap_clear_bit(&map2, i);
+  for (i = 0; i < bitsize2; i++) bitmap_clear_bit(&map2, i);
   bitmap_intersect(map, &map2);
   if (!bitmap_is_clear_all(map)) goto error;
   delete[] map2buf;
   return false;
 error:
-  delete[] map2buf;
   ADD_FAILURE() << "intersect error  bit1=" << test_bit1
-                << ",bit2=" << test_bit2;
+                << ",bit2=" << test_bit2 << ",bit3=" << test_bit3;
   return true;
 }
 
 class BitMapTest : public ::testing::TestWithParam<uint> {
  protected:
-  void SetUp() override {
+  virtual void SetUp() {
     bitsize = GetParam();
-    ASSERT_FALSE(bitmap_init(&map, buf, bitsize));
+    ASSERT_FALSE(bitmap_init(&map, buf, bitsize, false));
     bitmap_clear_all(&map);
   }
 
@@ -463,7 +457,7 @@ const uint test_values[] = {
     2 * 32U - 1, 2 * 32U,     2 * 32U + 1, 3 * 32U - 1, 3 * 32U,
     3 * 32U + 1, 4 * 32U - 1, 4 * 32U,     4 * 32U + 1, MAX_TESTED_BITMAP_SIZE};
 
-INSTANTIATE_TEST_SUITE_P(Foo, BitMapTest, ::testing::ValuesIn(test_values));
+INSTANTIATE_TEST_CASE_P(Foo, BitMapTest, ::testing::ValuesIn(test_values));
 
 TEST_P(BitMapTest, TestSetGetClearBit) {
   EXPECT_FALSE(test_set_get_clear_bit(&map, bitsize)) << "bitsize=" << bitsize;
@@ -515,7 +509,7 @@ bool bitmap_set_prefix_t() {
   MY_BITMAP map;
   my_bitmap_map buf[2]; /* 64-bit buffer */
   uint32 _max = ~((uint32)0);
-  bitmap_init(&map, buf, 32);
+  bitmap_init(&map, buf, 32, false);
 
   // set all bits in the 2nd half of the buf
   buf[1] = _max;
