@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,7 +35,7 @@ namespace mysys_my_symlink {
 #if !defined(_WIN32)
 TEST(Mysys, MysysMySymlink) {
   char filename[FN_REFLEN];
-  int fd = create_temp_file(filename, nullptr, "gunit_mysys_symlink",
+  int fd = create_temp_file(filename, NULL, "gunit_mysys_symlink",
                             O_CREAT | O_WRONLY, KEEP_FILE, MYF(MY_WME));
   EXPECT_GT(fd, 0);
 
