@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -43,7 +43,7 @@ class CostModelTest : public ::testing::Test {
     // Add a storage engine to the hton2plugin array.
     // This is needed for the cost model to add cost constants
     // for the storage engine
-    LEX_STRING engine_name = {C_STRING_WITH_LEN("InnoDB")};
+    LEX_CSTRING engine_name = {STRING_WITH_LEN("InnoDB")};
 
     insert_hton2plugin(0, new st_plugin_int())->name = engine_name;
     initializer.SetUp();
