@@ -1,18 +1,17 @@
 #ifndef PARSERTEST_INCLUDED
 #define PARSERTEST_INCLUDED
-/* Copyright (c) 2012, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,10 +37,10 @@ using my_testing::Server_initializer;
 
 /*
   Parses a query and returns a parse tree. In our parser this is
-  called a Query_block.
+  called a SELECT_LEX.
 */
-static Query_block *parse(const Server_initializer *initializer,
-                          const char *query, int expected_error_code) {
+static SELECT_LEX *parse(const Server_initializer *initializer,
+                         const char *query, int expected_error_code) {
   Parser_state state;
 
   size_t length = strlen(query);
@@ -71,7 +70,7 @@ static Query_block *parse(const Server_initializer *initializer,
   mysql_reset_thd_for_next_command(initializer->thd());
   parse_sql(initializer->thd(), &state, nullptr);
 
-  return initializer->thd()->lex->current_query_block();
+  return initializer->thd()->lex->current_select();
 }
 
 /*
@@ -79,8 +78,8 @@ static Query_block *parse(const Server_initializer *initializer,
 */
 class ParserTest : public ::testing::Test {
  protected:
-  void SetUp() override { initializer.SetUp(); }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void SetUp() { initializer.SetUp(); }
+  virtual void TearDown() { initializer.TearDown(); }
 
   THD *thd() const { return initializer.thd(); }
 
@@ -88,11 +87,11 @@ class ParserTest : public ::testing::Test {
 
   void assert_eq(int x, int y) const { ASSERT_EQ(x, y); }
 
-  Query_block *parse(const char *query, int expected_error_code) const {
+  SELECT_LEX *parse(const char *query, int expected_error_code) const {
     return ::parse(&initializer, query, expected_error_code);
   }
 
-  Query_block *parse(const char *query) const { return parse(query, 0); }
+  SELECT_LEX *parse(const char *query) const { return parse(query, 0); }
 };
 
 #endif  // PARSERTEST_INCLUDED
