@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is designed to work with certain software (including
+This program is also distributed with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,8 +35,8 @@ using namespace sha2_password;
 using my_testing::Server_initializer;
 
 class SHA256_digestTest : public ::testing::Test {
-  void SetUp() override { initializer.SetUp(); }
-  void TearDown() override { initializer.TearDown(); }
+  virtual void SetUp() { initializer.SetUp(); }
+  virtual void TearDown() { initializer.TearDown(); }
 
   THD *thd() { return initializer.thd(); }
 
@@ -353,8 +352,7 @@ TEST_F(SHA256_digestTest, SHA2_password_cache) {
 }
 
 TEST_F(SHA256_digestTest, Caching_sha2_password_Serialize_Deserialize) {
-  Caching_sha2_password caching_sha2_password(nullptr,
-                                              DEFAULT_STORED_DIGEST_ROUNDS);
+  Caching_sha2_password caching_sha2_password(0);
 
   Digest_info digest_type;
 
@@ -384,30 +382,9 @@ TEST_F(SHA256_digestTest, Caching_sha2_password_Serialize_Deserialize) {
   ASSERT_TRUE(sha256_valid_auth_string1 ==
               sha256_valid_auth_string1_serialized);
 
-  /* Valid string with largest iterations count */
-  std::string sha256_valid_auth_string_biggest(
-      "$A$FFF$ABCDEFGHIJKLMNOPQRSTabcdefgh01234567ijklmnop89012345ABCDEFGH678");
-  std::string sha256_valid_auth_string_biggest_serialized;
-
-  ASSERT_TRUE(caching_sha2_password.deserialize(
-                  sha256_valid_auth_string_biggest, digest_type, salt,
-                  sha256_digest, iterations) == false);
-  ASSERT_TRUE(digest_type == Digest_info::SHA256_DIGEST);
-  ASSERT_TRUE(memcmp(salt.c_str(), "ABCDEFGHIJKLMNOPQRST", SALT_LENGTH) == 0);
-  ASSERT_TRUE(memcmp(sha256_digest.c_str(),
-                     "abcdefgh01234567ijklmnop89012345ABCDEFGH678",
-                     STORED_SHA256_DIGEST_LENGTH) == 0);
-  ASSERT_TRUE(iterations == MAX_ITERATIONS);
-
-  ASSERT_TRUE(caching_sha2_password.serialize(
-                  sha256_valid_auth_string_biggest_serialized, digest_type,
-                  salt, sha256_digest, iterations) == false);
-  ASSERT_TRUE(sha256_valid_auth_string_biggest ==
-              sha256_valid_auth_string_biggest_serialized);
-
   /* Valid string with special characters */
   std::string sha256_valid_auth_string2(
-      "$A$20A$01234567ABCDEFGH!@#$abcdefgh01234567ijklm890!@#$%^&*;[-=,+]']+*");
+      "$A$005$01234567ABCDEFGH!@#$abcdefgh01234567ijklm890!@#$%^&*;[-=,+]']+*");
   std::string sha256_valid_auth_string2_serialized;
 
   ASSERT_TRUE(caching_sha2_password.deserialize(
@@ -418,7 +395,7 @@ TEST_F(SHA256_digestTest, Caching_sha2_password_Serialize_Deserialize) {
   ASSERT_TRUE(memcmp(sha256_digest.c_str(),
                      "abcdefgh01234567ijklm890!@#$%^&*;[-=,+]']+*",
                      STORED_SHA256_DIGEST_LENGTH) == 0);
-  ASSERT_TRUE(iterations == 0x20A * 1000);
+  ASSERT_TRUE(iterations == 5000);
 
   ASSERT_TRUE(caching_sha2_password.serialize(
                   sha256_valid_auth_string2_serialized, digest_type, salt,
@@ -498,10 +475,9 @@ TEST_F(SHA256_digestTest, Caching_sha2_password_Serialize_Deserialize) {
 }
 
 TEST_F(SHA256_digestTest, Caching_sha2_password_generate_fast_digest) {
-  Caching_sha2_password caching_sha2_password(nullptr,
-                                              DEFAULT_STORED_DIGEST_ROUNDS);
-  Caching_sha2_password caching_sha2_password_4(
-      nullptr, DEFAULT_STORED_DIGEST_ROUNDS, 4);
+  Caching_sha2_password caching_sha2_password(0);
+  Caching_sha2_password caching_sha2_password_4(0, DEFAULT_STORED_DIGEST_ROUNDS,
+                                                4);
 
   std::string plaintext("HahaH0hO1234#$@#%");
   sha2_cache_entry digest;
@@ -557,14 +533,13 @@ TEST_F(SHA256_digestTest, Caching_sha2_password_generate_sha2_multi_hash) {
   memset(stage2, 0, CRYPT_MAX_PASSWORD_SIZE + 1);
   my_crypt_genhash(
       stage2, CRYPT_MAX_PASSWORD_SIZE, plaintext_buffer_arthur.c_str(),
-      plaintext_buffer_arthur.length(), salt_buffer_arthur.c_str(), nullptr);
+      plaintext_buffer_arthur.length(), salt_buffer_arthur.c_str(), 0);
 
   std::string digest_string1(stage2);
   digest_string1 = digest_string1.substr(
       digest_string1.find('$', 3 + CRYPT_SALT_LENGTH) + 1, std::string::npos);
 
-  Caching_sha2_password caching_sha2_password(nullptr,
-                                              DEFAULT_STORED_DIGEST_ROUNDS);
+  Caching_sha2_password caching_sha2_password(0);
   std::string digest;
   ASSERT_TRUE(caching_sha2_password.generate_sha2_multi_hash(
                   plaintext_buffer_arthur, salt_buffer_arthur, &digest, 5000) ==
@@ -579,7 +554,7 @@ TEST_F(SHA256_digestTest, Caching_sha2_password_generate_sha2_multi_hash) {
   memset(stage2, 0, CRYPT_MAX_PASSWORD_SIZE + 1);
   my_crypt_genhash(
       stage2, CRYPT_MAX_PASSWORD_SIZE, plaintext_buffer_arthur.c_str(),
-      plaintext_buffer_arthur.length(), salt_buffer_arthur.c_str(), nullptr);
+      plaintext_buffer_arthur.length(), salt_buffer_arthur.c_str(), 0);
   digest_string1.assign(stage2);
   digest_string1 = digest_string1.substr(
       digest_string1.find('$', 3 + CRYPT_SALT_LENGTH) + 1, std::string::npos);
@@ -592,8 +567,7 @@ TEST_F(SHA256_digestTest, Caching_sha2_password_generate_sha2_multi_hash) {
 
 TEST_F(SHA256_digestTest,
        Caching_sha2_password_authenticate_fast_authenticate) {
-  Caching_sha2_password caching_sha2_password(nullptr,
-                                              DEFAULT_STORED_DIGEST_ROUNDS);
+  Caching_sha2_password caching_sha2_password(0);
   std::string serialized_string[MAX_PASSWORDS];
   Digest_info digest_type = Digest_info::SHA256_DIGEST;
   std::string digest;
@@ -739,8 +713,7 @@ TEST_F(SHA256_digestTest,
 }
 
 TEST_F(SHA256_digestTest, Caching_sha2_password_authenticate_sanity) {
-  Caching_sha2_password caching_sha2_password(nullptr,
-                                              DEFAULT_STORED_DIGEST_ROUNDS);
+  Caching_sha2_password caching_sha2_password(0);
   std::string serialized_string[MAX_PASSWORDS];
   std::string plaintext;
 
