@@ -1,16 +1,15 @@
-/* Copyright (c) 2012, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,7 +57,7 @@ class TableCacheBasicTest : public ::testing::Test {
  protected:
   static const uint MAX_THREADS = 3;
 
-  void SetUp() override {
+  virtual void SetUp() {
     Global_THD_manager *thd_manager = Global_THD_manager::get_instance();
     thd_manager->set_unit_test();
     // Reset thread ID counter for each test.
@@ -69,7 +68,7 @@ class TableCacheBasicTest : public ::testing::Test {
 
     ::testing::FLAGS_gtest_death_test_style = "threadsafe";
   }
-  void TearDown() override {
+  virtual void TearDown() {
     for (uint i = 0; i < MAX_THREADS; ++i) initializer[i].TearDown();
   }
 
@@ -86,7 +85,7 @@ class TableCacheBasicTest : public ::testing::Test {
 class TableCacheSingleCacheTest : public TableCacheBasicTest {
  protected:
   virtual uint CachesNumber() { return 1; }
-  void SetUp() override {
+  virtual void SetUp() {
     TableCacheBasicTest::SetUp();
 
     /*
@@ -99,7 +98,7 @@ class TableCacheSingleCacheTest : public TableCacheBasicTest {
     table_cache_size_per_instance = 100;
     ASSERT_FALSE(table_def_init());
   }
-  void TearDown() override {
+  virtual void TearDown() {
     table_def_free();
     TableCacheBasicTest::TearDown();
   }
@@ -112,7 +111,7 @@ class TableCacheSingleCacheTest : public TableCacheBasicTest {
 
 class TableCacheDoubleCacheTest : public TableCacheSingleCacheTest {
  protected:
-  uint CachesNumber() override { return 2; }
+  virtual uint CachesNumber() { return 2; }
 };
 
 /**
@@ -128,9 +127,7 @@ class Mock_share : public TABLE_SHARE {
   Mock_share(const char *key)
       :  // Assertion in some of Table_cache methods check that the
          // version of the share is up-to-date, so make sure it's set.
-        TABLE_SHARE(refresh_version, false),
-        // MEM_ROOT is used for constructing ha_example() instances.
-        m_mem_root(PSI_NOT_INSTRUMENTED, 1024) {
+        TABLE_SHARE(refresh_version, false) {
     /*
       Both table_cache_key and cache_element array are used by
       Table_cache code.
@@ -139,11 +136,13 @@ class Mock_share : public TABLE_SHARE {
     table_cache_key.length = strlen(key);
     memset(cache_element_arr, 0, sizeof(cache_element_arr));
     cache_element = cache_element_arr;
+    // MEM_ROOT is used for constructing ha_example() instances.
+    init_alloc_root(PSI_NOT_INSTRUMENTED, &m_mem_root, 1024, 0);
     // Ensure that share is never destroyed.
     increment_ref_count();
   }
 
-  ~Mock_share() { m_mem_root.Clear(); }
+  ~Mock_share() { free_root(&m_mem_root, MYF(0)); }
 
   TABLE *create_table(THD *thd) {
     TABLE *result =
@@ -288,14 +287,14 @@ TEST_F(TableCacheSingleCacheTest, CacheAddAndRemove) {
   TABLE_SHARE *share_2;
   table_2 = table_cache->get_table(thd, share_1.table_cache_key.str,
                                    share_1.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_2 == nullptr);
+  EXPECT_TRUE(table_2 == NULL);
   EXPECT_TRUE(share_2 == &share_1);
 
   // Table_cache_iterator should be able to find only one TABLE instance
   // in all caches. And this instance should be table_1.
   Table_cache_iterator it(&share_1);
   EXPECT_TRUE(it++ == table_1);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
 
   // We must be able to release TABLE into table cache and reuse it after
   // this.
@@ -312,11 +311,11 @@ TEST_F(TableCacheSingleCacheTest, CacheAddAndRemove) {
 
   table_2 = table_cache->get_table(thd, share_1.table_cache_key.str,
                                    share_1.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_2 == nullptr);
-  EXPECT_TRUE(share_2 == nullptr);
+  EXPECT_TRUE(table_2 == NULL);
+  EXPECT_TRUE(share_2 == NULL);
 
   it.rewind();
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
 
   // Also it should be possible to remove unused TABLE from the cache
   // Add TABLE instance and mark it as unused
@@ -330,8 +329,8 @@ TEST_F(TableCacheSingleCacheTest, CacheAddAndRemove) {
 
   table_2 = table_cache->get_table(thd, share_1.table_cache_key.str,
                                    share_1.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_2 == nullptr);
-  EXPECT_TRUE(share_2 == nullptr);
+  EXPECT_TRUE(table_2 == NULL);
+  EXPECT_TRUE(share_2 == NULL);
 
   table_cache->unlock();
 
@@ -422,8 +421,8 @@ TEST_F(TableCacheSingleCacheTest, CacheGetAndRelease) {
   // There should be no TABLE in cache, nor information about share.
   table_1 = table_cache->get_table(thd, share_1.table_cache_key.str,
                                    share_1.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_1 == nullptr);
-  EXPECT_TRUE(share_2 == nullptr);
+  EXPECT_TRUE(table_1 == NULL);
+  EXPECT_TRUE(share_2 == NULL);
 
   table_1 = share_1.create_table(thd);
   add_used_table(table_cache, thd, table_1);
@@ -432,15 +431,15 @@ TEST_F(TableCacheSingleCacheTest, CacheGetAndRelease) {
   // information about the share.
   table_2 = table_cache->get_table(thd, share_1.table_cache_key.str,
                                    share_1.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_2 == nullptr);
+  EXPECT_TRUE(table_2 == NULL);
   EXPECT_TRUE(share_2 == &share_1);
 
   // There should be even no information about the share for which
   // TABLE was not added to cache.
   table_2 = table_cache->get_table(thd, share_0.table_cache_key.str,
                                    share_0.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_2 == nullptr);
-  EXPECT_TRUE(share_2 == nullptr);
+  EXPECT_TRUE(table_2 == NULL);
+  EXPECT_TRUE(share_2 == NULL);
 
   table_2 = share_1.create_table(thd);
   add_used_table(table_cache, thd, table_2);
@@ -449,7 +448,7 @@ TEST_F(TableCacheSingleCacheTest, CacheGetAndRelease) {
   // be information about the share.
   table_3 = table_cache->get_table(thd, share_1.table_cache_key.str,
                                    share_1.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_3 == nullptr);
+  EXPECT_TRUE(table_3 == NULL);
   EXPECT_TRUE(share_2 == &share_1);
 
   table_cache->release_table(thd, table_1);
@@ -464,7 +463,7 @@ TEST_F(TableCacheSingleCacheTest, CacheGetAndRelease) {
   // But only once!
   table_3 = table_cache->get_table(thd, share_1.table_cache_key.str,
                                    share_1.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_3 == nullptr);
+  EXPECT_TRUE(table_3 == NULL);
   EXPECT_TRUE(share_2 == &share_1);
 
   // After releasing of both TABLE objects it should be possible to
@@ -475,20 +474,20 @@ TEST_F(TableCacheSingleCacheTest, CacheGetAndRelease) {
 
   table_3 = table_cache->get_table(thd, share_0.table_cache_key.str,
                                    share_0.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_3 == nullptr);
-  EXPECT_TRUE(share_2 == nullptr);
+  EXPECT_TRUE(table_3 == NULL);
+  EXPECT_TRUE(share_2 == NULL);
 
   table_3 = table_cache->get_table(thd, share_1.table_cache_key.str,
                                    share_1.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_3 != nullptr);
+  EXPECT_TRUE(table_3 != NULL);
   EXPECT_TRUE(share_2 == &share_1);
   table_3 = table_cache->get_table(thd, share_1.table_cache_key.str,
                                    share_1.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_3 != nullptr);
+  EXPECT_TRUE(table_3 != NULL);
   EXPECT_TRUE(share_2 == &share_1);
   table_3 = table_cache->get_table(thd, share_1.table_cache_key.str,
                                    share_1.table_cache_key.length, &share_2);
-  EXPECT_TRUE(table_3 == nullptr);
+  EXPECT_TRUE(table_3 == NULL);
   EXPECT_TRUE(share_2 == &share_1);
 
   // Clean-up
@@ -750,7 +749,7 @@ TEST_F(TableCacheDoubleCacheDeathTest, ManagerFreeTable) {
   // There should be assert failure since we are trying
   // to free all tables for share_1, while some tables
   // are in use.
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   EXPECT_DEATH_IF_SUPPORTED(
       table_cache_manager.free_table(thd_1, TDC_RT_REMOVE_ALL, &share_1),
       ".*Assertion.*is_empty.*");
@@ -788,7 +787,7 @@ TEST_F(TableCacheDoubleCacheDeathTest, ManagerFreeTable) {
   // There should be assert failure since we are trying
   // to free all not own TABLEs for share_1, while thd_2
   // has a TABLE object for it in used
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   EXPECT_DEATH_IF_SUPPORTED(
       table_cache_manager.free_table(thd_1, TDC_RT_REMOVE_NOT_OWN, &share_1),
       ".*Assertion.*0.*");
@@ -857,9 +856,9 @@ TEST_F(TableCacheDoubleCacheTest, Iterator) {
   // There is no TABLE objects for share_1 so the below iterator
   // should not find anything.
   Table_cache_iterator it(&share_1);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
   // Attempt to iterate behind the end should not give anything.
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
 
   Table_cache *table_cache_1 = table_cache_manager.get_cache(thd_1);
   Table_cache *table_cache_2 = table_cache_manager.get_cache(thd_2);
@@ -876,47 +875,47 @@ TEST_F(TableCacheDoubleCacheTest, Iterator) {
   TABLE *table_r1 = it++;
   EXPECT_TRUE(table_r1 == table_4);
   // But only it.
-  EXPECT_TRUE(it++ == nullptr);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
+  EXPECT_TRUE(it++ == NULL);
 
   add_used_table(table_cache_1, thd_1, table_1);
 
   // Now we should see two tables:
   it.rewind();
   table_r1 = it++;
-  EXPECT_TRUE(table_r1 != nullptr);
+  EXPECT_TRUE(table_r1 != NULL);
   TABLE *table_r2 = it++;
-  EXPECT_TRUE(table_r2 != nullptr);
+  EXPECT_TRUE(table_r2 != NULL);
   EXPECT_TRUE(table_r1 != table_r2);
-  EXPECT_TRUE(it++ == nullptr);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
+  EXPECT_TRUE(it++ == NULL);
 
   add_used_table(table_cache_1, thd_1, table_2);
 
   // And now three !
   it.rewind();
   table_r1 = it++;
-  EXPECT_TRUE(table_r1 != nullptr);
+  EXPECT_TRUE(table_r1 != NULL);
   table_r2 = it++;
-  EXPECT_TRUE(table_r2 != nullptr);
+  EXPECT_TRUE(table_r2 != NULL);
   TABLE *table_r3 = it++;
-  EXPECT_TRUE(table_r3 != nullptr);
+  EXPECT_TRUE(table_r3 != NULL);
   EXPECT_TRUE(table_r1 != table_r2 && table_r1 != table_r3 &&
               table_r2 != table_r3);
-  EXPECT_TRUE(it++ == nullptr);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
+  EXPECT_TRUE(it++ == NULL);
 
   table_cache_1->release_table(thd_1, table_1);
 
   // We should be seeing only used TABLE objects, so two tables now
   it.rewind();
   table_r1 = it++;
-  EXPECT_TRUE(table_r1 != nullptr);
+  EXPECT_TRUE(table_r1 != NULL);
   table_r2 = it++;
-  EXPECT_TRUE(table_r2 != nullptr);
+  EXPECT_TRUE(table_r2 != NULL);
   EXPECT_TRUE(table_r1 != table_r2);
-  EXPECT_TRUE(it++ == nullptr);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
+  EXPECT_TRUE(it++ == NULL);
 
   add_used_table(table_cache_1, thd_1, table_3);
   add_used_table(table_cache_2, thd_2, table_5);
@@ -924,12 +923,12 @@ TEST_F(TableCacheDoubleCacheTest, Iterator) {
   // We also should not be seeing TABLE objects for share_2
   it.rewind();
   table_r1 = it++;
-  EXPECT_TRUE(table_r1 != nullptr);
+  EXPECT_TRUE(table_r1 != NULL);
   table_r2 = it++;
-  EXPECT_TRUE(table_r2 != nullptr);
+  EXPECT_TRUE(table_r2 != NULL);
   EXPECT_TRUE(table_r1 != table_r2);
-  EXPECT_TRUE(it++ == nullptr);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
+  EXPECT_TRUE(it++ == NULL);
 
   table_cache_1->remove_table(table_2);
 
@@ -937,30 +936,30 @@ TEST_F(TableCacheDoubleCacheTest, Iterator) {
   it.rewind();
   table_r1 = it++;
   EXPECT_TRUE(table_r1 == table_4);
-  EXPECT_TRUE(it++ == nullptr);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
+  EXPECT_TRUE(it++ == NULL);
 
   table_cache_1->remove_table(table_4);
 
   // And now no used TABLE objects for share_1 at all
   it.rewind();
-  EXPECT_TRUE(it++ == nullptr);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
+  EXPECT_TRUE(it++ == NULL);
 
   table_cache_1->remove_table(table_1);
 
   // Still the same
   it.rewind();
-  EXPECT_TRUE(it++ == nullptr);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
+  EXPECT_TRUE(it++ == NULL);
 
   table_cache_1->remove_table(table_3);
   table_cache_2->remove_table(table_5);
 
   // Cache is empty so iterator should not show any TABLE objects.
   it.rewind();
-  EXPECT_TRUE(it++ == nullptr);
-  EXPECT_TRUE(it++ == nullptr);
+  EXPECT_TRUE(it++ == NULL);
+  EXPECT_TRUE(it++ == NULL);
 
   table_cache_manager.unlock_all_and_tdc();
 
