@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,22 +42,23 @@ class Mock_field_varstring : public Field_varstring {
       : Field_varstring(nullptr,                          // ptr_arg
                         calc_len_arg(char_len),           // len_arg
                         calc_length_bytes_arg(char_len),  // length_bytes_arg
-                        is_nullable ? buffer : nullptr,   // null_ptr_arg
+                        is_nullable ? &null_byte : NULL,  // null_ptr_arg
                         is_nullable ? 1 : 0,              // null_bit_arg
                         Field::NONE,                      // auto_flags_arg
                         name,                             // field_name_arg
                         share,                            // share
                         &my_charset_utf8mb4_general_ci)   // cs
   {
-    ptr = buffer + 1;
-    std::memset(buffer, 0, MAX_FIELD_VARCHARLENGTH + 1);
+    buffer.assign(pack_length(), 0);
+    ptr = buffer.data();
 
     static const char *table_name_buf = "table_name";
     table_name = &table_name_buf;
   }
 
  private:
-  uchar buffer[MAX_FIELD_VARCHARLENGTH + 1];
+  std::vector<uchar> buffer;
+  uchar null_byte = '\0';
 
   static uint calc_byte_len(uint char_len) {
     return char_len * my_charset_utf8mb4_general_ci.mbmaxlen;
