@@ -1,17 +1,16 @@
 /*
- * Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
+ * This program is also distributed with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
+ * separately licensed software that they have included with MySQL.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,12 +22,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
-#include <cstdint>
-
-#include "errmsg.h"         // NOLINT(build/include_subdir)
-#include "my_config.h"      // NOLINT(build/include_subdir)
-#include "my_macros.h"      // NOLINT(build/include_subdir)
-#include "mysql_version.h"  // NOLINT(build/include_subdir)
+#include "errmsg.h"
+#include "my_config.h"
+#include "my_inttypes.h"
+#include "my_macros.h"
+#include "mysql_version.h"
 
 #include "plugin/x/generated/mysqlx_error.h"
 #include "plugin/x/generated/mysqlx_version.h"
@@ -44,7 +42,7 @@ const char *expected_pass = "user_pass";
 const char *expected_host = "host";
 const char *expected_socket_file = "socket_file";
 const char *expected_schema = "schema";
-const uint16_t expected_port = 1290;
+const uint16 expected_port = 1290;
 const int expected_error_code = 10;
 const int expected_error_code_success = 0;
 
@@ -172,7 +170,6 @@ TEST_F(Xcl_session_impl_tests_connect, connection_tcp_already_connected) {
 TEST_F(Xcl_session_impl_tests_connect, connect_nullptrs) {
   m_sut->set_mysql_option(xcl::XSession::Mysqlx_option::Authentication_method,
                           "MYSQL41");
-
   EXPECT_CALL(m_mock_connection_state, is_ssl_activated())
       .WillRepeatedly(Return(false));
   EXPECT_CALL(m_mock_connection_state, is_ssl_configured())
@@ -181,14 +178,12 @@ TEST_F(Xcl_session_impl_tests_connect, connect_nullptrs) {
       .WillOnce(Return(Connection_type::Tcp));
   EXPECT_CALL(*m_mock_protocol, add_send_message_handler(_, _, _));
   EXPECT_CALL(*m_mock_protocol, remove_send_message_handler(0));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering());
   EXPECT_CALL(m_mock_connection, set_read_timeout(_))
       .WillRepeatedly(Return(XError{}));
   EXPECT_CALL(m_mock_connection, set_write_timeout(_))
       .WillRepeatedly(Return(XError{}));
   EXPECT_CALL(*m_mock_protocol, execute_authenticate("", "", "", "MYSQL41"))
       .WillOnce(Return(XError{}));
-  EXPECT_CALL(*m_mock_protocol, use_compression(Compression_algorithm::k_none));
 
   EXPECT_CALL(*m_mock_protocol, add_notice_handler(_, Handler_position::Begin,
                                                    Handler_priority_low))
@@ -207,7 +202,6 @@ TEST_F(Xcl_session_impl_tests_connect, connect_nullptrs) {
 TEST_F(Xcl_session_impl_tests_connect, connect_localhost_nullptrs) {
   m_sut->set_mysql_option(xcl::XSession::Mysqlx_option::Authentication_method,
                           "MYSQL41");
-
   EXPECT_CALL(m_mock_connection_state, is_ssl_activated())
       .WillRepeatedly(Return(false));
   EXPECT_CALL(m_mock_connection_state, is_ssl_configured())
@@ -216,14 +210,12 @@ TEST_F(Xcl_session_impl_tests_connect, connect_localhost_nullptrs) {
       .WillOnce(Return(Connection_type::Unix_socket));
   EXPECT_CALL(*m_mock_protocol, add_send_message_handler(_, _, _));
   EXPECT_CALL(*m_mock_protocol, remove_send_message_handler(0));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering());
   EXPECT_CALL(m_mock_connection, set_read_timeout(_))
       .WillRepeatedly(Return(XError{}));
   EXPECT_CALL(m_mock_connection, set_write_timeout(_))
       .WillRepeatedly(Return(XError{}));
   EXPECT_CALL(*m_mock_protocol, execute_authenticate("", "", "", "MYSQL41"))
       .WillOnce(Return(XError{}));
-  EXPECT_CALL(*m_mock_protocol, use_compression(Compression_algorithm::k_none));
 
   EXPECT_CALL(*m_mock_protocol, add_notice_handler(_, Handler_position::Begin,
                                                    Handler_priority_low))
@@ -356,8 +348,6 @@ TEST_P(Xcl_session_impl_tests_challenge_response_connect_param,
               execute_authenticate(expected_user, expected_pass,
                                    expected_schema, this->GetParam().m_auth))
       .WillOnce(Return(XError{}));
-  EXPECT_CALL(*m_mock_protocol, use_compression(Compression_algorithm::k_none));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   auto error = (this->*GetParam().m_open)(expected_error_code_success);
 
@@ -381,8 +371,6 @@ TEST_P(Xcl_session_impl_tests_challenge_response_connect_param,
               execute_authenticate(expected_user, expected_pass,
                                    expected_schema, this->GetParam().m_auth))
       .WillOnce(Return(XError{}));
-  EXPECT_CALL(*m_mock_protocol, use_compression(Compression_algorithm::k_none));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   m_sut->set_capability(XSession::Capability_can_handle_expired_password, true);
   auto error = (this->*GetParam().m_open)(expected_error_code_success);
@@ -403,7 +391,6 @@ TEST_P(Xcl_session_impl_tests_challenge_response_connect_param,
   EXPECT_CALL(*m_mock_protocol,
               execute_set_capability(Cmp_msg(m_cap_expired.get())))
       .WillOnce(Return(XError{expected_error_code, ""}));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   m_sut->set_capability(XSession::Capability_can_handle_expired_password, true);
   auto error = (this->*GetParam().m_open)(expected_error_code_success);
@@ -425,8 +412,6 @@ TEST_P(Xcl_session_impl_tests_challenge_response_connect_param,
               execute_authenticate(expected_user, expected_pass,
                                    expected_schema, this->GetParam().m_auth))
       .WillOnce(Return(XError{expected_error_code, ""}));
-  EXPECT_CALL(*m_mock_protocol, use_compression(Compression_algorithm::k_none));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   auto error = (this->*GetParam().m_open)(expected_error_code_success);
 
@@ -446,7 +431,6 @@ TEST_P(Xcl_session_impl_tests_challenge_response_connect_param,
   EXPECT_CALL(*m_mock_protocol,
               execute_set_capability(Cmp_msg(m_cap_connect_attrs.get())))
       .WillOnce(Return(XError{expected_error_code}));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   m_sut->set_capability(XSession::Capability_session_connect_attrs,
                         m_sut->get_connect_attrs());
@@ -469,8 +453,6 @@ TEST_P(Xcl_session_impl_tests_plain_connect_param,
               execute_authenticate(expected_user, expected_pass,
                                    expected_schema, this->GetParam().m_auth))
       .WillOnce(Return(XError{}));
-  EXPECT_CALL(*m_mock_protocol, use_compression(Compression_algorithm::k_none));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   auto error = (this->*GetParam().m_open)(expected_error_code_success);
 
@@ -490,7 +472,6 @@ TEST_P(Xcl_session_impl_tests_plain_connect_param,
   EXPECT_CALL(*m_mock_protocol,
               execute_set_capability(Cmp_msg(m_cap_set_tls.get())))
       .WillOnce(Return(XError{expected_error_code, ""}));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   auto error = (this->*GetParam().m_open)(expected_error_code_success);
 
@@ -512,7 +493,6 @@ TEST_P(Xcl_session_impl_tests_plain_connect_param,
       .WillOnce(Return(XError{}));
   EXPECT_CALL(m_mock_connection, activate_tls())
       .WillOnce(Return(XError{expected_error_code, ""}));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   auto error = (this->*GetParam().m_open)(expected_error_code_success);
 
@@ -537,8 +517,6 @@ TEST_P(Xcl_session_impl_tests_plain_connect_param, connect_plain_tls) {
               execute_authenticate(expected_user, expected_pass,
                                    expected_schema, this->GetParam().m_auth))
       .WillOnce(Return(XError{}));
-  EXPECT_CALL(*m_mock_protocol, use_compression(Compression_algorithm::k_none));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   auto error = (this->*GetParam().m_open)(expected_error_code_success);
 
@@ -558,7 +536,6 @@ TEST_P(Xcl_session_impl_tests_plain_connect_param,
   EXPECT_CALL(*m_mock_protocol,
               execute_set_capability(Cmp_msg(m_cap_expired.get())))
       .WillOnce(Return(XError{expected_error_code}));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   m_sut->set_capability(XSession::Capability_can_handle_expired_password, true);
   auto error = (this->*GetParam().m_open)(expected_error_code_success);
@@ -579,7 +556,6 @@ TEST_P(Xcl_session_impl_tests_plain_connect_param,
   EXPECT_CALL(*m_mock_protocol,
               execute_set_capability(Cmp_msg(m_cap_connect_attrs.get())))
       .WillOnce(Return(XError{expected_error_code}));
-  EXPECT_CALL(*m_mock_protocol, reset_buffering()).Times(testing::AtLeast(0));
 
   m_sut->set_capability(XSession::Capability_session_connect_attrs,
                         m_sut->get_connect_attrs());
@@ -619,7 +595,7 @@ auto param_pack_builder_reauthenticate = [](const std::string &method) {
       Open_close_methods::start_connected, method};
 };
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     Instantiation_connection_method, Xcl_session_impl_tests_connect_param,
     Values(param_pack_builder_connect("PLAIN"),
            param_pack_builder_connect_to_localhost("PLAIN"),
@@ -631,13 +607,13 @@ INSTANTIATE_TEST_SUITE_P(
            param_pack_builder_connect_to_localhost("SHA256_MEMORY"),
            param_pack_builder_reauthenticate("SHA256_MEMORY")));
 
-INSTANTIATE_TEST_SUITE_P(
-    Instantiation_connection_method, Xcl_session_impl_tests_plain_connect_param,
-    Values(param_pack_builder_connect("PLAIN"),
-           param_pack_builder_connect_to_localhost("PLAIN"),
-           param_pack_builder_reauthenticate("PLAIN")));
+INSTANTIATE_TEST_CASE_P(Instantiation_connection_method,
+                        Xcl_session_impl_tests_plain_connect_param,
+                        Values(param_pack_builder_connect("PLAIN"),
+                               param_pack_builder_connect_to_localhost("PLAIN"),
+                               param_pack_builder_reauthenticate("PLAIN")));
 
-INSTANTIATE_TEST_SUITE_P(
+INSTANTIATE_TEST_CASE_P(
     Instantiation_connection_method,
     Xcl_session_impl_tests_challenge_response_connect_param,
     Values(param_pack_builder_connect("MYSQL41"),
