@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -30,10 +30,7 @@
 #include "plugin/x/src/admin_cmd_index.h"
 #include "plugin/x/src/xpl_error.h"
 #include "unittest/gunit/xplugin/xpl/assert_error_code.h"
-#include "unittest/gunit/xplugin/xpl/mock/client.h"
-#include "unittest/gunit/xplugin/xpl/mock/protocol_encoder.h"
 #include "unittest/gunit/xplugin/xpl/mock/session.h"
-#include "unittest/gunit/xplugin/xpl/mock/sql_session.h"
 #include "unittest/gunit/xplugin/xpl/mysqlx_pb_wrapper.h"
 #include "unittest/gunit/xplugin/xpl/one_row_resultset.h"
 
@@ -48,7 +45,7 @@ using ::testing::StrictMock;
 
 class Admin_command_index_stub : public Admin_command_index {
  public:
-  explicit Admin_command_index_stub(iface::Session *session)
+  explicit Admin_command_index_stub(ngs::Session_interface *session)
       : Admin_command_index(session) {}
   using Admin_command_index::is_table_support_virtual_columns;
 };
@@ -57,7 +54,7 @@ class Admin_command_index_test : public ::testing::Test {
  public:
   using Sql = ngs::PFS_string;
 
-  void SetUp() override {
+  void SetUp() {
     command.reset(new Admin_command_index_stub(&session));
 
     EXPECT_CALL(session, data_context())
@@ -71,16 +68,17 @@ class Admin_command_index_test : public ::testing::Test {
     args.reset(new Admin_command_arguments_object(list));
   }
 
-  StrictMock<mock::Sql_session> data_context;
-  StrictMock<mock::Client> client;
-  StrictMock<mock::Protocol_encoder> encoder;
-  StrictMock<mock::Session> session;
+  StrictMock<ngs::test::Mock_sql_data_context> data_context;
+  StrictMock<xpl::test::Mock_client> client;
+  StrictMock<ngs::test::Mock_protocol_encoder> encoder;
+  StrictMock<ngs::test::Mock_session> session;
   std::unique_ptr<Admin_command_index_stub> command;
   Admin_command_arguments_object::List list;
   std::unique_ptr<Admin_command_arguments_object> args;
 };
 
 namespace {
+const char *const NAMESPACE = "mysqlx";
 #define ALPHA "alpha"
 #define BETA "beta"
 #define GAMMA "gamma"
@@ -134,32 +132,36 @@ const One_row_resultset TABLE_WITH_MYISAM_ENGINE{BETA,
 
 TEST_F(Admin_command_index_test, drop_empty_schema) {
   set_arguments(Any::Object{{"schema", ""}, COLLECTION, INDEX_NAME});
-  ASSERT_ERROR_CODE(ER_X_BAD_SCHEMA, command->drop(args.get()));
+  ASSERT_ERROR_CODE(ER_X_BAD_SCHEMA, command->drop(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, drop_missing_schema) {
   set_arguments(Any::Object{COLLECTION, INDEX_NAME});
-  ASSERT_ERROR_CODE(ER_X_CMD_NUM_ARGUMENTS, command->drop(args.get()));
+  ASSERT_ERROR_CODE(ER_X_CMD_NUM_ARGUMENTS,
+                    command->drop(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, drop_empty_collection) {
   set_arguments(Any::Object{SCHEMA, {"collection", ""}, INDEX_NAME});
-  ASSERT_ERROR_CODE(ER_X_BAD_TABLE, command->drop(args.get()));
+  ASSERT_ERROR_CODE(ER_X_BAD_TABLE, command->drop(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, drop_missing_collection) {
   set_arguments(Any::Object{SCHEMA, INDEX_NAME});
-  ASSERT_ERROR_CODE(ER_X_CMD_NUM_ARGUMENTS, command->drop(args.get()));
+  ASSERT_ERROR_CODE(ER_X_CMD_NUM_ARGUMENTS,
+                    command->drop(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, drop_empty_index_name) {
   set_arguments(Any::Object{SCHEMA, COLLECTION, {"name", ""}});
-  ASSERT_ERROR_CODE(ER_X_MISSING_ARGUMENT, command->drop(args.get()));
+  ASSERT_ERROR_CODE(ER_X_MISSING_ARGUMENT,
+                    command->drop(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, drop_missing_index_name) {
   set_arguments(Any::Object{SCHEMA, COLLECTION});
-  ASSERT_ERROR_CODE(ER_X_CMD_NUM_ARGUMENTS, command->drop(args.get()));
+  ASSERT_ERROR_CODE(ER_X_CMD_NUM_ARGUMENTS,
+                    command->drop(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, drop_no_schema) {
@@ -172,7 +174,7 @@ TEST_F(Admin_command_index_test, drop_no_schema) {
       .WillOnce(Return(ngs::Error_code(ER_BAD_DB_ERROR, "bad db error")));
 
   set_arguments(Any::Object{SCHEMA, COLLECTION, INDEX_NAME});
-  ASSERT_ERROR_CODE(ER_X_BAD_TABLE, command->drop(args.get()));
+  ASSERT_ERROR_CODE(ER_X_BAD_TABLE, command->drop(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, drop_no_collection) {
@@ -185,7 +187,7 @@ TEST_F(Admin_command_index_test, drop_no_collection) {
       .WillOnce(Return(ngs::Error_code(ER_NO_SUCH_TABLE, "no such table")));
 
   set_arguments(Any::Object{SCHEMA, COLLECTION, INDEX_NAME});
-  ASSERT_ERROR_CODE(ER_X_BAD_TABLE, command->drop(args.get()));
+  ASSERT_ERROR_CODE(ER_X_BAD_TABLE, command->drop(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, drop_no_virtual_column_no_index) {
@@ -198,7 +200,7 @@ TEST_F(Admin_command_index_test, drop_no_virtual_column_no_index) {
       .WillOnce(Return(ngs::Error_code(ER_NO_SUCH_INDEX, "no such index")));
 
   set_arguments(Any::Object{SCHEMA, COLLECTION, INDEX_NAME});
-  ASSERT_ERROR_CODE(ER_NO_SUCH_INDEX, command->drop(args.get()));
+  ASSERT_ERROR_CODE(ER_NO_SUCH_INDEX, command->drop(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, drop_index_with_column) {
@@ -216,7 +218,7 @@ TEST_F(Admin_command_index_test, drop_index_with_column) {
       .WillOnce(Return(ngs::Success()));
 
   set_arguments(Any::Object{SCHEMA, COLLECTION, INDEX_NAME});
-  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->drop(args.get()));
+  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->drop(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test,
@@ -277,7 +279,7 @@ TEST_F(Admin_command_index_test, create_invalid_schema) {
                             INDEX_NAME,
                             UNIQUE,
                             {"constraint", DECIMAL_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_BAD_SCHEMA, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_BAD_SCHEMA, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_invalid_collection) {
@@ -286,19 +288,21 @@ TEST_F(Admin_command_index_test, create_invalid_collection) {
                             INDEX_NAME,
                             UNIQUE,
                             {"constraint", DECIMAL_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_BAD_TABLE, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_BAD_TABLE, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_invalid_index_name) {
   set_arguments(Any::Object{
       SCHEMA, COLLECTION, {"name", ""}, UNIQUE, {"constraint", DECIMAL_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_CMD_ARGUMENT_VALUE, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_CMD_ARGUMENT_VALUE,
+                    command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_missing_unique) {
   set_arguments(Any::Object{
       SCHEMA, COLLECTION, {"name", ""}, {"constraint", DECIMAL_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_CMD_NUM_ARGUMENTS, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_CMD_NUM_ARGUMENTS,
+                    command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_check_virtual_support_no_collection) {
@@ -307,7 +311,7 @@ TEST_F(Admin_command_index_test, create_check_virtual_support_no_collection) {
 
   set_arguments(Any::Object{
       SCHEMA, COLLECTION, INDEX_NAME, UNIQUE, {"constraint", DECIMAL_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_BAD_TABLE, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_BAD_TABLE, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_check_virtual_support_goes_wrong) {
@@ -316,7 +320,7 @@ TEST_F(Admin_command_index_test, create_check_virtual_support_goes_wrong) {
 
   set_arguments(Any::Object{
       SCHEMA, COLLECTION, INDEX_NAME, UNIQUE, {"constraint", DECIMAL_FIELD}});
-  ASSERT_ERROR_CODE(ER_INTERNAL_ERROR, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_INTERNAL_ERROR, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_bad_constraint) {
@@ -326,7 +330,8 @@ TEST_F(Admin_command_index_test, create_bad_constraint) {
 
   set_arguments(Any::Object{
       SCHEMA, COLLECTION, INDEX_NAME, UNIQUE, {"constraint", Any::Object{}}});
-  ASSERT_ERROR_CODE(ER_X_CMD_NUM_ARGUMENTS, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_CMD_NUM_ARGUMENTS,
+                    command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_regular_index_with_virtual_column) {
@@ -351,7 +356,7 @@ TEST_F(Admin_command_index_test, create_regular_index_with_virtual_column) {
                             INDEX_NAME,
                             NOT_UNIQUE,
                             {"constraint", DECIMAL_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_regular_index_with_stored_column) {
@@ -376,7 +381,7 @@ TEST_F(Admin_command_index_test, create_regular_index_with_stored_column) {
                             INDEX_NAME,
                             NOT_UNIQUE,
                             {"constraint", DECIMAL_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_regular_index_without_column) {
@@ -400,7 +405,7 @@ TEST_F(Admin_command_index_test, create_regular_index_without_column) {
                             INDEX_NAME,
                             NOT_UNIQUE,
                             {"constraint", DECIMAL_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_regular_index_with_two_virtual_column) {
@@ -434,7 +439,7 @@ TEST_F(Admin_command_index_test, create_regular_index_with_two_virtual_column) {
                   NOT_UNIQUE,
                   INDEX_TYPE_PLAIN,
                   {"constraint", Any::Array{DECIMAL_FIELD, TEXT_FIELD}}});
-  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_spatial_index) {
@@ -463,7 +468,7 @@ TEST_F(Admin_command_index_test, create_spatial_index) {
                             NOT_UNIQUE,
                             INDEX_TYPE_SPATIAL,
                             {"constraint", GEOJSON_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_unique_spatial_index) {
@@ -473,7 +478,8 @@ TEST_F(Admin_command_index_test, create_unique_spatial_index) {
                             UNIQUE,
                             INDEX_TYPE_SPATIAL,
                             {"constraint", GEOJSON_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_CMD_ARGUMENT_VALUE, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_CMD_ARGUMENT_VALUE,
+                    command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_unable_to_create) {
@@ -498,7 +504,7 @@ TEST_F(Admin_command_index_test, create_unable_to_create) {
                             INDEX_NAME,
                             NOT_UNIQUE,
                             {"constraint", DECIMAL_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_ARTIFICIAL1, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_ARTIFICIAL1, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_bd_null_error_required_field_missing) {
@@ -524,7 +530,7 @@ TEST_F(Admin_command_index_test, create_bd_null_error_required_field_missing) {
                             NOT_UNIQUE,
                             {"constraint", DECIMAL_FIELD}});
   ASSERT_ERROR_CODE(ER_X_DOC_REQUIRED_FIELD_MISSING,
-                    command->create(args.get()));
+                    command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_bd_null_error) {
@@ -546,7 +552,7 @@ TEST_F(Admin_command_index_test, create_bd_null_error) {
 
   set_arguments(Any::Object{
       SCHEMA, COLLECTION, INDEX_NAME, NOT_UNIQUE, {"constraint", TEXT_FIELD}});
-  ASSERT_ERROR_CODE(ER_BAD_NULL_ERROR, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_BAD_NULL_ERROR, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_unable_to_craete_spatial_index) {
@@ -577,7 +583,7 @@ TEST_F(Admin_command_index_test, create_unable_to_craete_spatial_index) {
                             INDEX_TYPE_SPATIAL,
                             {"constraint", GEOJSON_FIELD}});
   ASSERT_ERROR_CODE(ER_X_DOC_REQUIRED_FIELD_MISSING,
-                    command->create(args.get()));
+                    command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_fulltext_index) {
@@ -604,7 +610,7 @@ TEST_F(Admin_command_index_test, create_fulltext_index) {
                             NOT_UNIQUE,
                             INDEX_TYPE_FULLTEXT,
                             {"constraint", FULLTEXT_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_fulltext_index_with_parser) {
@@ -634,7 +640,7 @@ TEST_F(Admin_command_index_test, create_fulltext_index_with_parser) {
                             INDEX_TYPE_FULLTEXT,
                             {"with_parser", "ngram"},
                             {"constraint", FULLTEXT_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_SUCCESS, command->create(NAMESPACE, args.get()));
 }
 
 TEST_F(Admin_command_index_test, create_unique_fulltext_index) {
@@ -644,7 +650,8 @@ TEST_F(Admin_command_index_test, create_unique_fulltext_index) {
                             UNIQUE,
                             INDEX_TYPE_FULLTEXT,
                             {"constraint", FULLTEXT_FIELD}});
-  ASSERT_ERROR_CODE(ER_X_CMD_ARGUMENT_VALUE, command->create(args.get()));
+  ASSERT_ERROR_CODE(ER_X_CMD_ARGUMENT_VALUE,
+                    command->create(NAMESPACE, args.get()));
 }
 
 }  // namespace test
