@@ -101,7 +101,8 @@ class Listener_tcp_testsuite : public Test {
   }
 
   void expect_create_socket(addrinfo &ai, const std::string &interface,
-                            const int family, const int result = SOCKET_OK) {
+                            const int family,
+                            const int64_t result = SOCKET_OK) {
     make_sut(interface, PORT, PORT_TIMEOUT);
 
     EXPECT_CALL(*m_mock_system,
@@ -263,7 +264,7 @@ TEST_P(Listener_tcp_retry_testsuite,
   EXPECT_CALL(*m_mock_system, get_socket_errno())
       .Times(n)
       .WillRepeatedly(Return(SOCKET_EADDRINUSE));
-  EXPECT_CALL(*m_mock_system, sleep(Gt(0))).Times(n);
+  EXPECT_CALL(*m_mock_system, sleep(Gt(0u))).Times(n);
 
   EXPECT_CALL(*m_mock_system, freeaddrinfo(&ai));
 
