@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -84,11 +84,6 @@ Scalar::Scalar(const double value) {
   m_base.set_v_double(value);
 }
 
-Scalar::Scalar(const char *value, unsigned type) {
-  m_base.set_type(Mysqlx::Datatypes::Scalar_Type_V_OCTETS);
-  m_base.mutable_v_octets()->CopyFrom(Scalar::Octets(value, type));
-}
-
 Scalar::Scalar(const Scalar::Octets &value) {
   m_base.set_type(Mysqlx::Datatypes::Scalar_Type_V_OCTETS);
   m_base.mutable_v_octets()->CopyFrom(value);
@@ -99,15 +94,20 @@ Scalar::Scalar(const Scalar::String &value) {
   m_base.mutable_v_string()->CopyFrom(value);
 }
 
+Scalar::Scalar(const char *value) {
+  m_base.set_type(Mysqlx::Datatypes::Scalar_Type_V_STRING);
+  *m_base.mutable_v_string()->mutable_value() = value;
+}
+
 Scalar::Scalar(Scalar::Null) {
   m_base.set_type(Mysqlx::Datatypes::Scalar_Type_V_NULL);
 }
 
 Scalar::String::String(const std::string &value) { m_base.set_value(value); }
 
-Scalar::Octets::Octets(const std::string &value, const unsigned type) {
+Scalar::Octets::Octets(const std::string &value, const Content_type type) {
   m_base.set_value(value);
-  m_base.set_content_type(type);
+  m_base.set_content_type(static_cast<uint32_t>(type));
 }
 
 Any::Any(const Scalar &scalar) {
