@@ -1,16 +1,15 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is designed to work with certain software (including
+ This program is also distributed with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have either included with
- the program or referenced in the documentation.
+ separately licensed software that they have included with MySQL.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -69,13 +68,6 @@ class One_row_resultset : public xpl::Collect_resultset {
     callbacks.set_resultset(resultset);
     callbacks.set_field_types(types);
   }
-
-  void set_server_status(const uint32_t bit) {
-    auto &callbacks = get_callbacks();
-    auto status_info = callbacks.get_info();
-    status_info.server_status |= bit;
-    callbacks.set_status_info(status_info);
-  }
 };
 
 ACTION_P(SetUpResultset, init_data) {
