@@ -41,6 +41,7 @@
 #include <new>
 #include <set>
 #include <utility>
+#include <vector>
 
 #include "m_ctype.h"
 #include "m_string.h"
@@ -57,7 +58,6 @@
 #include "welcome_copyright_notice.h"
 
 #define MAX_ERROR_NAME_LENGTH 64
-#define MAX_ROWS 6000
 #define HEADER_LENGTH 32 /* Length of header in errmsg.sys */
 #define ERRMSG_VERSION 3 /* Version number of errmsg.sys */
 #define DEFAULT_CHARSET_DIR "../share/charsets"
@@ -68,6 +68,7 @@
 static const char *OUTFILE = "errmsg.sys";
 static const char *HEADERFILE = "mysqld_error.h";
 static const char *NAMEFILE = "mysqld_ername.h";
+static const char *MSGFILE = "mysqld_errmsg.h";
 static const char *TXTFILE = "../share/errmsg-utf8.txt";
 static const char *DATADIRECTORY = "../share/";
 #ifndef DBUG_OFF
@@ -83,7 +84,7 @@ static const char *default_dbug_option = "d:t:O,/tmp/comp_err.trace";
 */
 uchar file_head[] = {254, 254, ERRMSG_VERSION, 1};
 /* Store positions to each error message row to store in errmsg.sys header */
-uint file_pos[MAX_ROWS];
+std::vector<uint> file_pos;
 
 const char *empty_string = ""; /* For empty states */
 /*
@@ -158,6 +159,8 @@ static struct my_option my_long_options[] = {
      GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"name_file", 'N', "mysqld_ername.h file ", &NAMEFILE, &NAMEFILE, 0,
      GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+    {"errmsg_file", 'N', "mysqld_errmsg.h file ", &MSGFILE, &MSGFILE, 0,
+     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
 
 static struct languages *parse_charset_string(char *str);
@@ -193,20 +196,17 @@ int main(int argc, char *argv[]) {
     uint row_count;
     struct errors *error_head;
     struct languages *lang_head;
-    DBUG_ENTER("main");
+    {
+      DBUG_TRACE;
+
+      charsets_dir = DEFAULT_CHARSET_DIR;
+      my_umask_dir = 0777;
+      if (get_options(&argc, &argv)) return 1;
+      if (!(row_count = parse_input_file(TXTFILE, &error_head, &lang_head))) {
+        fprintf(stderr, "Failed to parse input file %s\n", TXTFILE);
+        return 1;
+      }
 
-    charsets_dir = DEFAULT_CHARSET_DIR;
-    my_umask_dir = 0777;
-    if (get_options(&argc, &argv)) DBUG_RETURN(1);
-    if (!(row_count = parse_input_file(TXTFILE, &error_head, &lang_head))) {
-      fprintf(stderr, "Failed to parse input file %s\n", TXTFILE);
-      DBUG_RETURN(1);
-    }
-    if (row_count > MAX_ROWS) {
-      fprintf(stderr, "Found too many error messages. ");
-      fprintf(stderr, "Increase MAX_ROWS in utilities/comp_err.cc.\n");
-      DBUG_RETURN(1);
-    }
 #if MYSQL_VERSION_ID >= 50100 && MYSQL_VERSION_ID < 50500
 /* Number of error messages in 5.1 - do not change this number! */
 #define MYSQL_OLD_GA_ERROR_MESSAGE_COUNT 641
@@ -215,29 +215,29 @@ int main(int argc, char *argv[]) {
 #define MYSQL_OLD_GA_ERROR_MESSAGE_COUNT 728
 #endif
 #ifdef MYSQL_OLD_GA_ERROR_MESSAGE_COUNT
-    if (row_count != MYSQL_OLD_GA_ERROR_MESSAGE_COUNT) {
-      fprintf(stderr, "Can only add new error messages to latest GA. ");
-      fprintf(stderr, "Use ER_UNKNOWN_ERROR instead.\n");
-      fprintf(stderr, "Expected %u messages, found %u.\n",
-              MYSQL_OLD_GA_ERROR_MESSAGE_COUNT, row_count);
-      DBUG_RETURN(1);
-    }
+      if (row_count != MYSQL_OLD_GA_ERROR_MESSAGE_COUNT) {
+        fprintf(stderr, "Can only add new error messages to latest GA. ");
+        fprintf(stderr, "Use ER_UNKNOWN_ERROR instead.\n");
+        fprintf(stderr, "Expected %u messages, found %u.\n",
+                MYSQL_OLD_GA_ERROR_MESSAGE_COUNT, row_count);
+        return 1;
+      }
 #endif
-    if (lang_head == NULL || error_head == NULL) {
-      fprintf(stderr, "Failed to parse input file %s\n", TXTFILE);
-      DBUG_RETURN(1);
-    }
+      if (lang_head == NULL || error_head == NULL) {
+        fprintf(stderr, "Failed to parse input file %s\n", TXTFILE);
+        return 1;
+      }
 
-    if (create_header_files(error_head)) {
-      fprintf(stderr, "Failed to create header files\n");
-      DBUG_RETURN(1);
-    }
-    if (create_sys_files(lang_head, error_head, row_count)) {
-      fprintf(stderr, "Failed to create sys files\n");
-      DBUG_RETURN(1);
-    }
-    clean_up(lang_head, error_head);
-    DBUG_LEAVE; /* Can't use dbug after my_end() */
+      if (create_header_files(error_head)) {
+        fprintf(stderr, "Failed to create header files\n");
+        return 1;
+      }
+      if (create_sys_files(lang_head, error_head, row_count)) {
+        fprintf(stderr, "Failed to create sys files\n");
+        return 1;
+      }
+      clean_up(lang_head, error_head);
+    } /* Can't use dbug after my_end() */
     my_end(info_flag ? MY_CHECK_ERROR | MY_GIVE_INFO : 0);
     return 0;
   }
@@ -272,18 +272,24 @@ static void print_escaped_string(FILE *f, const char *str) {
 
 static int create_header_files(struct errors *error_head) {
   FILE *er_definef, *er_namef;
+  FILE *er_errmsg;
   struct errors *tmp_error;
   struct message *er_msg;
   const char *er_text;
 
-  DBUG_ENTER("create_header_files");
+  DBUG_TRACE;
 
   if (!(er_definef = my_fopen(HEADERFILE, O_WRONLY, MYF(MY_WME)))) {
-    DBUG_RETURN(1);
+    return 1;
   }
   if (!(er_namef = my_fopen(NAMEFILE, O_WRONLY, MYF(MY_WME)))) {
     my_fclose(er_definef, MYF(0));
-    DBUG_RETURN(1);
+    return 1;
+  }
+  if (!(er_errmsg = my_fopen(MSGFILE, O_WRONLY, MYF(MY_WME)))) {
+    my_fclose(er_definef, MYF(0));
+    my_fclose(er_namef, MYF(0));
+    return 1;
   }
 
   fprintf(er_definef, ORACLE_GPL_COPYRIGHT_NOTICE("2000"));
@@ -294,6 +300,9 @@ static int create_header_files(struct errors *error_head) {
   fprintf(er_definef, "#ifndef MYSQLD_ERROR_INCLUDED\n");
   fprintf(er_definef, "#define MYSQLD_ERROR_INCLUDED\n\n");
 
+  fprintf(er_errmsg, "#ifndef MYSQLD_ERRORMSG_INCLUDED\n");
+  fprintf(er_errmsg, "#define MYSQLD_ERRORMSG_INCLUDED\n\n");
+
   /*
     Find out how many sections of error messages we have, what the first
     number in each section is and the number of messages in each section.
@@ -337,7 +346,7 @@ static int create_header_files(struct errors *error_head) {
   for (tmp_error = error_head; tmp_error; tmp_error = tmp_error->next_error) {
     if (strlen(tmp_error->er_name) > MAX_ERROR_NAME_LENGTH) {
       fprintf(stderr, "Error name [%s] too long.\n", tmp_error->er_name);
-      DBUG_RETURN(1);
+      return 1;
     }
 
     /*
@@ -368,15 +377,26 @@ static int create_header_files(struct errors *error_head) {
       fprintf(er_namef, "\",\"HY000\", \"\""); /* General Error */
 
     fprintf(er_namef, ", %d },\n", temp_error_index);
+
+    /*
+       generating mysqld_errmsg.h
+    */
+    if (!isObsolete(tmp_error->er_name)) {
+      fprintf(er_errmsg, "#define %s_MSG \"", tmp_error->er_name);
+      print_escaped_string(er_errmsg, er_text);
+      fprintf(er_errmsg, "\"\n");
+    }
   }
 
   fprintf(er_definef, "static const int obsolete_error_count = %d;\n\n",
           obsolete_error_count);
   /* finishing off with mysqld_error.h */
   fprintf(er_definef, "#endif\n");
+  fprintf(er_errmsg, "#endif\n");
   my_fclose(er_definef, MYF(0));
   my_fclose(er_namef, MYF(0));
-  DBUG_RETURN(0);
+  my_fclose(er_errmsg, MYF(0));
+  return 0;
 }
 
 static int create_sys_files(struct languages *lang_head,
@@ -391,7 +411,7 @@ static int create_sys_files(struct languages *lang_head,
   struct errors *tmp_error;
 
   MY_STAT stat_info;
-  DBUG_ENTER("create_sys_files");
+  DBUG_TRACE;
 
   /*
      going over all languages and assembling corresponding error messages
@@ -401,7 +421,7 @@ static int create_sys_files(struct languages *lang_head,
     if (!(csnum = get_charset_number(tmp_lang->charset, MY_CS_PRIMARY))) {
       fprintf(stderr, "Unknown charset '%s' in '%s'\n", tmp_lang->charset,
               TXTFILE);
-      DBUG_RETURN(1);
+      return 1;
     }
 
     outfile_end =
@@ -409,14 +429,14 @@ static int create_sys_files(struct languages *lang_head,
     if (!my_stat(outfile, &stat_info, MYF(0))) {
       if (my_mkdir(outfile, 0777, MYF(0)) < 0) {
         fprintf(stderr, "Can't create output directory for %s\n", outfile);
-        DBUG_RETURN(1);
+        return 1;
       }
     }
 
     strxmov(outfile_end, FN_ROOTDIR, OUTFILE, NullS);
 
     if (!(to = my_fopen(outfile, O_WRONLY | MY_FOPEN_BINARY, MYF(MY_WME))))
-      DBUG_RETURN(1);
+      return 1;
 
     /* 4 is for 4 bytes to store row position / error message */
     start_pos = (long)(HEADER_LENGTH + row_count * 4);
@@ -454,23 +474,23 @@ static int create_sys_files(struct languages *lang_head,
       goto err;
 
     for (i = 0; i < row_count; i++) {
-      int4store(head, file_pos[i]);
+      int4store(head, file_pos.at(i));
       if (my_fwrite(to, (uchar *)head, 4, MYF(MY_WME | MY_FNABP))) goto err;
     }
     my_fclose(to, MYF(0));
   }
-  DBUG_RETURN(0);
+  return 0;
 
 err:
   my_fclose(to, MYF(0));
-  DBUG_RETURN(1);
+  return 1;
 }
 
 static void clean_up(struct languages *lang_head, struct errors *error_head) {
   struct languages *tmp_lang, *next_language;
   struct errors *tmp_error, *next_error;
 
-  my_free((void *)default_language);
+  my_free(const_cast<char *>(default_language));
 
   for (tmp_lang = lang_head; tmp_lang; tmp_lang = next_language) {
     next_language = tmp_lang->next_lang;
@@ -490,9 +510,11 @@ static void clean_up(struct languages *lang_head, struct errors *error_head) {
       my_free(tmp->text);
     }
 
-    if (tmp_error->sql_code1[0]) my_free((void *)tmp_error->sql_code1);
-    if (tmp_error->sql_code2[0]) my_free((void *)tmp_error->sql_code2);
-    my_free((void *)tmp_error->er_name);
+    if (tmp_error->sql_code1[0])
+      my_free(const_cast<char *>(tmp_error->sql_code1));
+    if (tmp_error->sql_code2[0])
+      my_free(const_cast<char *>(tmp_error->sql_code2));
+    my_free(const_cast<char *>(tmp_error->er_name));
     tmp_error->~errors();
     my_free(tmp_error);
   }
@@ -506,24 +528,24 @@ static int parse_input_file(const char *file_name, struct errors **top_error,
   struct message current_message;
   int rcount = 0; /* Number of error codes in current section. */
   int ecount = 0; /* Number of error codes in total. */
-  DBUG_ENTER("parse_input_file");
+  DBUG_TRACE;
 
   *top_error = 0;
   *top_lang = 0;
-  if (!(file = my_fopen(file_name, O_RDONLY, MYF(MY_WME)))) DBUG_RETURN(0);
+  if (!(file = my_fopen(file_name, O_RDONLY, MYF(MY_WME)))) return 0;
 
   while ((str = fgets(buff, sizeof(buff), file))) {
     if (is_prefix(str, "language")) {
       if (!(*top_lang = parse_charset_string(str))) {
         fprintf(stderr, "Failed to parse the charset string!\n");
-        DBUG_RETURN(0);
+        return 0;
       }
       continue;
     }
     if (is_prefix(str, "start-error-number")) {
       if (!(er_offset = parse_error_offset(str))) {
         fprintf(stderr, "Failed to parse the error offset string!\n");
-        DBUG_RETURN(0);
+        return 0;
       }
       rcount = 0; /* Reset count if a fixed number is set. */
       continue;
@@ -531,7 +553,7 @@ static int parse_input_file(const char *file_name, struct errors **top_error,
     if (is_prefix(str, "reserved-error-section")) {
       if (parse_reserved_error_section(str)) {
         fprintf(stderr, "Failed to parse the reserved error section string.\n");
-        DBUG_RETURN(0);
+        return 0;
       }
       continue;
     }
@@ -540,7 +562,7 @@ static int parse_input_file(const char *file_name, struct errors **top_error,
         DBUG_PRINT("info", ("default_slang: %s", default_language));
         fprintf(stderr,
                 "Failed to parse the default language line. Aborting\n");
-        DBUG_RETURN(0);
+        return 0;
       }
       continue;
     }
@@ -549,28 +571,28 @@ static int parse_input_file(const char *file_name, struct errors **top_error,
       /* New error message in another language for previous error */
       if (!current_error) {
         fprintf(stderr, "Error in the input file format\n");
-        DBUG_RETURN(0);
+        return 0;
       }
       if (!parse_message_string(&current_message, str)) {
         fprintf(stderr, "Failed to parse message string for error '%s'",
                 current_error->er_name);
-        DBUG_RETURN(0);
+        return 0;
       }
       if (find_message(current_error, current_message.lang_short_name, true)) {
         fprintf(stderr,
                 "Duplicate message string for error '%s'"
                 " in language '%s'\n",
                 current_error->er_name, current_message.lang_short_name);
-        DBUG_RETURN(0);
+        return 0;
       }
       if (check_message_format(current_error, current_message.text)) {
         fprintf(stderr,
                 "Wrong formatspecifier of error message string"
                 " for error '%s' in language '%s'\n",
                 current_error->er_name, current_message.lang_short_name);
-        DBUG_RETURN(0);
+        return 0;
       }
-      if (current_error->msg.push_back(current_message)) DBUG_RETURN(0);
+      if (current_error->msg.push_back(current_message)) return 0;
       continue;
     }
     if (is_prefix(str, ER_PREFIX) || is_prefix(str, WARN_PREFIX) ||
@@ -578,7 +600,7 @@ static int parse_input_file(const char *file_name, struct errors **top_error,
         is_prefix(str, OBSOLETE_WARN_PREFIX)) {
       if (!(current_error = parse_error_string(str, rcount))) {
         fprintf(stderr, "Failed to parse the error name string\n");
-        DBUG_RETURN(0);
+        return 0;
       }
       rcount++;
       ecount++; /* Count number of unique errors */
@@ -591,12 +613,12 @@ static int parse_input_file(const char *file_name, struct errors **top_error,
     if (*str == '#' || *str == '\n') continue; /* skip comment or empty lines */
 
     fprintf(stderr, "Wrong input file format. Stop!\nLine: %s\n", str);
-    DBUG_RETURN(0);
+    return 0;
   }
   *tail_error = 0; /* Mark end of list */
 
   my_fclose(file, MYF(0));
-  DBUG_RETURN(ecount);
+  return ecount;
 }
 
 static uint parse_error_offset(char *str) {
@@ -605,16 +627,15 @@ static uint parse_error_offset(char *str) {
   int error;
   uint ioffset;
 
-  DBUG_ENTER("parse_error_offset");
+  DBUG_TRACE;
   /* skipping the "start-error-number" keyword and spaces after it */
   str = find_end_of_word(str);
   str = skip_delimiters(str);
 
-  if (!*str)
-    DBUG_RETURN(0); /* Unexpected EOL: No error number after the keyword */
+  if (!*str) return 0; /* Unexpected EOL: No error number after the keyword */
 
   /* reading the error offset */
-  if (!(soffset = get_word(&str))) DBUG_RETURN(0); /* OOM: Fatal error */
+  if (!(soffset = get_word(&str))) return 0; /* OOM: Fatal error */
   DBUG_PRINT("info", ("default_error_offset: %s", soffset));
 
   /* skipping space(s) and/or tabs after the error offset */
@@ -623,7 +644,7 @@ static uint parse_error_offset(char *str) {
   if (*str) {
     /* The line does not end with the error offset -> error! */
     fprintf(stderr, "The error offset line does not end with an error offset");
-    DBUG_RETURN(0);
+    return 0;
   }
   DBUG_PRINT("info", ("str: %s", str));
 
@@ -635,12 +656,12 @@ static uint parse_error_offset(char *str) {
               "start-error-number %u overlaps with the reserved section (%u - "
               "%u).\n",
               ioffset, section.first, section.second);
-      DBUG_RETURN(0);
+      return 0;
     }
   }
 
   my_free(soffset);
-  DBUG_RETURN(ioffset);
+  return ioffset;
 }
 
 /*
@@ -655,28 +676,28 @@ static bool parse_reserved_error_section(char *str) {
   char *offset;
   int error;
 
-  DBUG_ENTER("parse_reserved_error_section");
+  DBUG_TRACE;
 
   /* skipping the "reserved-error-section" keyword and spaces after it */
   str = find_end_of_word(str);
   str = skip_delimiters(str);
 
   if (!*str)
-    DBUG_RETURN(true); /* Unexpected EOL: No error number after the keyword */
+    return true; /* Unexpected EOL: No error number after the keyword */
 
   /* reading the section start number */
-  if (!(offset = get_word(&str))) DBUG_RETURN(true); /* OOM: Fatal error */
+  if (!(offset = get_word(&str))) return true; /* OOM: Fatal error */
   uint sec_start = static_cast<uint>(my_strtoll10(offset, nullptr, &error));
   my_free(offset);
   DBUG_PRINT("info", ("reserved_range_start: %u", sec_start));
 
   /* skipping space(s) and/or tabs after the section start number */
   str = skip_delimiters(str);
-  if (!*str) DBUG_RETURN(true); /* Unexpected EOL: No section end number */
+  if (!*str) return true; /* Unexpected EOL: No section end number */
   DBUG_PRINT("info", ("str: %s", str));
 
   /* reading the section end number */
-  if (!(offset = get_word(&str))) DBUG_RETURN(true); /* OOM: Fatal error */
+  if (!(offset = get_word(&str))) return true; /* OOM: Fatal error */
   uint sec_end = static_cast<uint>(my_strtoll10(offset, nullptr, &error));
   my_free(offset);
   DBUG_PRINT("info", ("reserved_range_end: %u", sec_end));
@@ -686,14 +707,14 @@ static bool parse_reserved_error_section(char *str) {
   DBUG_PRINT("info", ("str: %s", str));
   if (*str) {
     fprintf(stderr, "The line does not end with an error number.\n");
-    DBUG_RETURN(true);
+    return true;
   }
 
   if (sec_start >= sec_end) {
     fprintf(stderr,
             "Section start %u should be smaller than the Section end %u.\n",
             sec_start, sec_end);
-    DBUG_RETURN(true);
+    return true;
   }
 
   /* Check if section overlaps with existing reserved sections */
@@ -703,14 +724,14 @@ static bool parse_reserved_error_section(char *str) {
           stderr,
           "Section (%u - %u) overlaps with the reserved section (%u - %u).\n",
           sec_start, sec_end, section.first, section.second);
-      DBUG_RETURN(true);
+      return true;
     }
   }
 
   auto ret = reserved_sections.insert(err_range(sec_start, sec_end));
-  if (!ret.second) DBUG_RETURN(true);
+  if (!ret.second) return true;
 
-  DBUG_RETURN(false);
+  return false;
 }
 
 /* Parsing of the default language line. e.g. "default-language eng" */
@@ -718,7 +739,7 @@ static bool parse_reserved_error_section(char *str) {
 static char *parse_default_language(char *str) {
   char *slang;
 
-  DBUG_ENTER("parse_default_language");
+  DBUG_TRACE;
   /* skipping the "default-language" keyword */
   str = find_end_of_word(str);
   /* skipping space(s) and/or tabs after the keyword */
@@ -726,11 +747,11 @@ static char *parse_default_language(char *str) {
   if (!*str) {
     fprintf(stderr,
             "Unexpected EOL: No short language name after the keyword\n");
-    DBUG_RETURN(0);
+    return 0;
   }
 
   /* reading the short language tag */
-  if (!(slang = get_word(&str))) DBUG_RETURN(0); /* OOM: Fatal error */
+  if (!(slang = get_word(&str))) return 0; /* OOM: Fatal error */
   DBUG_PRINT("info", ("default_slang: %s", slang));
 
   str = skip_delimiters(str);
@@ -739,10 +760,10 @@ static char *parse_default_language(char *str) {
     fprintf(stderr,
             "The default language line does not end with short language "
             "name\n");
-    DBUG_RETURN(0);
+    return 0;
   }
   DBUG_PRINT("info", ("str: %s", str));
-  DBUG_RETURN(slang);
+  return slang;
 }
 
 /*
@@ -760,19 +781,19 @@ static char *parse_default_language(char *str) {
 static struct message *find_message(struct errors *err, const char *lang,
                                     bool no_default) {
   struct message *tmp, *return_val = 0;
-  DBUG_ENTER("find_message");
+  DBUG_TRACE;
 
   size_t count = (err->msg).size();
   for (size_t i = 0; i < count; i++) {
     tmp = &err->msg[i];
 
-    if (!strcmp(tmp->lang_short_name, lang)) DBUG_RETURN(tmp);
+    if (!strcmp(tmp->lang_short_name, lang)) return tmp;
     if (!strcmp(tmp->lang_short_name, default_language)) {
       DBUG_ASSERT(tmp->text[0] != 0);
       return_val = tmp;
     }
   }
-  DBUG_RETURN(no_default ? NULL : return_val);
+  return no_default ? NULL : return_val;
 }
 
 /*
@@ -856,11 +877,10 @@ static ha_checksum checksum_format_specifier(const char *msg) {
 */
 static int check_message_format(struct errors *err, const char *mess) {
   struct message *first;
-  DBUG_ENTER("check_message_format");
+  DBUG_TRACE;
 
   /*  Get first message(if any) */
-  if ((err->msg).empty())
-    DBUG_RETURN(0); /* No previous message to compare against */
+  if ((err->msg).empty()) return 0; /* No previous message to compare against */
 
   first = err->msg.begin();
   DBUG_ASSERT(first != NULL);
@@ -868,9 +888,9 @@ static int check_message_format(struct errors *err, const char *mess) {
   if (checksum_format_specifier(first->text) !=
       checksum_format_specifier(mess)) {
     /* Check sum of format specifiers failed, they should be equal */
-    DBUG_RETURN(1);
+    return 1;
   }
-  DBUG_RETURN(0);
+  return 0;
 }
 
 /*
@@ -879,12 +899,12 @@ static int check_message_format(struct errors *err, const char *mess) {
 */
 
 static char *skip_delimiters(char *str) {
-  DBUG_ENTER("skip_delimiters");
+  DBUG_TRACE;
   for (; *str == ' ' || *str == ',' || *str == '\t' || *str == '\r' ||
          *str == '\n' || *str == '=';
        str++)
     ;
-  DBUG_RETURN(str);
+  return str;
 }
 
 /*
@@ -892,23 +912,23 @@ static char *skip_delimiters(char *str) {
 */
 
 static char *find_end_of_word(char *str) {
-  DBUG_ENTER("find_end_of_word");
+  DBUG_TRACE;
   for (; *str != ' ' && *str != '\t' && *str != '\n' && *str != '\r' && *str &&
          *str != ',' && *str != ';' && *str != '=';
        str++)
     ;
-  DBUG_RETURN(str);
+  return str;
 }
 
 /* Read the word starting from *str */
 
 static char *get_word(char **str) {
   char *start = *str;
-  DBUG_ENTER("get_word");
+  DBUG_TRACE;
 
   *str = find_end_of_word(start);
-  DBUG_RETURN(my_strndup(PSI_NOT_INSTRUMENTED, start, (uint)(*str - start),
-                         MYF(MY_WME | MY_FAE)));
+  return my_strndup(PSI_NOT_INSTRUMENTED, start, (uint)(*str - start),
+                    MYF(MY_WME | MY_FAE));
 }
 
 /*
@@ -920,7 +940,7 @@ static struct message *parse_message_string(struct message *new_message,
                                             char *str) {
   char *start;
 
-  DBUG_ENTER("parse_message_string");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("str: %s", str));
 
   /*skip space(s) and/or tabs in the beginning */
@@ -929,7 +949,7 @@ static struct message *parse_message_string(struct message *new_message,
   if (!*str) {
     /* It was not a message line, but an empty line. */
     DBUG_PRINT("info", ("str: %s", str));
-    DBUG_RETURN(0);
+    return 0;
   }
 
   /* reading the short lang */
@@ -938,7 +958,7 @@ static struct message *parse_message_string(struct message *new_message,
   if (!(new_message->lang_short_name =
             my_strndup(PSI_NOT_INSTRUMENTED, start, (uint)(str - start),
                        MYF(MY_WME | MY_FAE))))
-    DBUG_RETURN(0); /* Fatal error */
+    return 0; /* Fatal error */
   DBUG_PRINT("info", ("msg_slang: %s", new_message->lang_short_name));
 
   /*skip space(s) and/or tabs after the lang */
@@ -947,7 +967,7 @@ static struct message *parse_message_string(struct message *new_message,
   if (*str != '"') {
     fprintf(stderr, "Unexpected EOL");
     DBUG_PRINT("info", ("str: %s", str));
-    DBUG_RETURN(0);
+    return 0;
   }
 
   /* reading the text */
@@ -957,10 +977,10 @@ static struct message *parse_message_string(struct message *new_message,
   if (!(new_message->text =
             my_strndup(PSI_NOT_INSTRUMENTED, start, (uint)(str - start),
                        MYF(MY_WME | MY_FAE))))
-    DBUG_RETURN(0); /* Fatal error */
+    return 0; /* Fatal error */
   DBUG_PRINT("info", ("msg_text: %s", new_message->text));
 
-  DBUG_RETURN(new_message);
+  return new_message;
 }
 
 /*
@@ -970,7 +990,7 @@ static struct message *parse_message_string(struct message *new_message,
 
 static struct errors *parse_error_string(char *str, int er_count) {
   struct errors *new_error;
-  DBUG_ENTER("parse_error_string");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("str: %s", str));
 
   /* create a new element */
@@ -981,8 +1001,7 @@ static struct errors *parse_error_string(char *str, int er_count) {
   /* getting the error name */
   str = skip_delimiters(str);
 
-  if (!(new_error->er_name = get_word(&str)))
-    DBUG_RETURN(0); /* OOM: Fatal error */
+  if (!(new_error->er_name = get_word(&str))) return 0; /* OOM: Fatal error */
   DBUG_PRINT("info", ("er_name: %s", new_error->er_name));
 
   str = skip_delimiters(str);
@@ -997,7 +1016,7 @@ static struct errors *parse_error_string(char *str, int er_count) {
           stderr,
           "er_name %s overlaps with the reserved error section (%u - %u).\n",
           new_error->er_name, section.first, section.second);
-      DBUG_RETURN(0);
+      return 0;
     }
   }
 
@@ -1011,13 +1030,12 @@ static struct errors *parse_error_string(char *str, int er_count) {
     new_error->sql_code1 = empty_string;
     new_error->sql_code2 = empty_string;
     DBUG_PRINT("info", ("str: %s", str));
-    DBUG_RETURN(new_error);
+    return new_error;
   }
 
   /* getting the sql_code 1 */
 
-  if (!(new_error->sql_code1 = get_word(&str)))
-    DBUG_RETURN(0); /* OOM: Fatal error */
+  if (!(new_error->sql_code1 = get_word(&str))) return 0; /* OOM: Fatal error */
   DBUG_PRINT("info", ("sql_code1: %s", new_error->sql_code1));
 
   str = skip_delimiters(str);
@@ -1026,22 +1044,21 @@ static struct errors *parse_error_string(char *str, int er_count) {
   if (!*str) {
     new_error->sql_code2 = empty_string;
     DBUG_PRINT("info", ("str: %s", str));
-    DBUG_RETURN(new_error);
+    return new_error;
   }
 
   /* getting the sql_code 2 */
-  if (!(new_error->sql_code2 = get_word(&str)))
-    DBUG_RETURN(0); /* OOM: Fatal error */
+  if (!(new_error->sql_code2 = get_word(&str))) return 0; /* OOM: Fatal error */
   DBUG_PRINT("info", ("sql_code2: %s", new_error->sql_code2));
 
   str = skip_delimiters(str);
   if (*str) {
     fprintf(stderr, "The error line did not end with sql/odbc code: '%s'\n",
             str);
-    DBUG_RETURN(0);
+    return 0;
   }
 
-  DBUG_RETURN(new_error);
+  return new_error;
 }
 
 /*
@@ -1051,7 +1068,7 @@ static struct errors *parse_error_string(char *str, int er_count) {
 
 static struct languages *parse_charset_string(char *str) {
   struct languages *head = 0, *new_lang;
-  DBUG_ENTER("parse_charset_string");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("str: %s", str));
 
   /* skip over keyword */
@@ -1059,11 +1076,11 @@ static struct languages *parse_charset_string(char *str) {
   if (!*str) {
     /* unexpected EOL */
     DBUG_PRINT("info", ("str: %s", str));
-    DBUG_RETURN(0);
+    return 0;
   }
 
   str = skip_delimiters(str);
-  if (!(*str != ';' && *str)) DBUG_RETURN(0);
+  if (!(*str != ';' && *str)) return 0;
 
   do {
     /*creating new element of the linked list */
@@ -1075,21 +1092,21 @@ static struct languages *parse_charset_string(char *str) {
     /* get the full language name */
 
     if (!(new_lang->lang_long_name = get_word(&str)))
-      DBUG_RETURN(0); /* OOM: Fatal error */
+      return 0; /* OOM: Fatal error */
 
     DBUG_PRINT("info", ("long_name: %s", new_lang->lang_long_name));
 
     /* getting the short name for language */
     str = skip_delimiters(str);
-    if (!*str) DBUG_RETURN(0); /* Error: No space or tab */
+    if (!*str) return 0; /* Error: No space or tab */
 
     if (!(new_lang->lang_short_name = get_word(&str)))
-      DBUG_RETURN(0); /* OOM: Fatal error */
+      return 0; /* OOM: Fatal error */
     DBUG_PRINT("info", ("short_name: %s", new_lang->lang_short_name));
 
     /* getting the charset name */
     str = skip_delimiters(str);
-    if (!(new_lang->charset = get_word(&str))) DBUG_RETURN(0); /* Fatal error */
+    if (!(new_lang->charset = get_word(&str))) return 0; /* Fatal error */
     DBUG_PRINT("info", ("charset: %s", new_lang->charset));
 
     /* skipping space, tab or "," */
@@ -1097,7 +1114,7 @@ static struct languages *parse_charset_string(char *str) {
   } while (*str != ';' && *str);
 
   DBUG_PRINT("info", ("long name: %s", new_lang->lang_long_name));
-  DBUG_RETURN(head);
+  return head;
 }
 
 /* Read options */
@@ -1105,7 +1122,7 @@ static struct languages *parse_charset_string(char *str) {
 static bool get_one_option(int optid,
                            const struct my_option *opt MY_ATTRIBUTE((unused)),
                            char *argument MY_ATTRIBUTE((unused))) {
-  DBUG_ENTER("get_one_option");
+  DBUG_TRACE;
   switch (optid) {
     case 'V':
       print_version();
@@ -1117,11 +1134,11 @@ static bool get_one_option(int optid,
       DBUG_PUSH(argument ? argument : default_dbug_option);
       break;
   }
-  DBUG_RETURN(0);
+  return 0;
 }
 
 static void usage(void) {
-  DBUG_ENTER("usage");
+  DBUG_TRACE;
   print_version();
   printf(
       "This software comes with ABSOLUTELY NO WARRANTY. "
@@ -1131,16 +1148,15 @@ static void usage(void) {
       "Usage:\n");
   my_print_help(my_long_options);
   my_print_variables(my_long_options);
-  DBUG_VOID_RETURN;
 }
 
 static int get_options(int *argc, char ***argv) {
   int ho_error;
-  DBUG_ENTER("get_options");
+  DBUG_TRACE;
 
   if ((ho_error = handle_options(argc, argv, my_long_options, get_one_option)))
-    DBUG_RETURN(ho_error);
-  DBUG_RETURN(0);
+    return ho_error;
+  return 0;
 }
 
 /*
@@ -1152,7 +1168,7 @@ static char *parse_text_line(char *pos) {
   int i, nr;
   char *row = pos;
   size_t len;
-  DBUG_ENTER("parse_text_line");
+  DBUG_TRACE;
 
   len = strlen(pos);
   while (*pos) {
@@ -1182,19 +1198,20 @@ static char *parse_text_line(char *pos) {
   }
   while (pos > row + 1 && *pos != '"') pos--;
   *pos = 0;
-  DBUG_RETURN(pos);
+  return pos;
 }
 
 /* Copy rows from memory to file and remember position */
 
 static int copy_rows(FILE *to, char *row, int row_nr, long start_pos) {
-  DBUG_ENTER("copy_rows");
+  DBUG_TRACE;
 
-  file_pos[row_nr] = (int)(ftell(to) - start_pos);
+  file_pos.insert(file_pos.begin() + row_nr,
+                  static_cast<uint>(ftell(to) - start_pos));
   if (fputs(row, to) == EOF || fputc('\0', to) == EOF) {
     fprintf(stderr, "Can't write to outputfile\n");
-    DBUG_RETURN(1);
+    return 1;
   }
 
-  DBUG_RETURN(0);
+  return 0;
 }
