@@ -1,18 +1,17 @@
 /*****************************************************************************
 
-Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+Copyright (c) 2016, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is designed to work with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have either included with
-the program or referenced in the documentation.
+This program is also distributed with certain software (including but not
+limited to OpenSSL) that is licensed under separate terms, as designated in a
+particular file or component or in included license documentation. The authors
+of MySQL hereby grant you an additional permission to link the program and
+your derivative works with the separately licensed software that they have
+included with MySQL.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -186,67 +185,57 @@ static const char *checksum_algorithms[] = {"crc32", "innodb", "none", NullS};
 
 /** Used to define an enumerate type of the "checksum algorithm". */
 static TYPELIB checksum_algorithms_typelib = {
-    array_elements(checksum_algorithms) - 1, "", checksum_algorithms, nullptr};
+    array_elements(checksum_algorithms) - 1, "", checksum_algorithms, NULL};
 
 /* Command line argument for ibd2sdi tool. */
 static struct my_option ibd2sdi_options[] = {
-    {"help", 'h', "Display this help and exit.", nullptr, nullptr, nullptr,
-     GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
-    {"version", 'v', "Display version information and exit.", nullptr, nullptr,
-     nullptr, GET_NO_ARG, NO_ARG, 0, 0, 0, nullptr, 0, nullptr},
-#ifndef NDEBUG
+    {"help", 'h', "Display this help and exit.", 0, 0, 0, GET_NO_ARG, NO_ARG, 0,
+     0, 0, 0, 0, 0},
+    {"version", 'v', "Display version information and exit.", 0, 0, 0,
+     GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
+#ifndef DBUG_OFF
     {"debug", '#', "Output debug log. See " REFMAN "dbug-package.html",
-     &opts.dbug_setting, &opts.dbug_setting, nullptr, GET_STR, OPT_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
-#endif /* !NDEBUG */
+     &opts.dbug_setting, &opts.dbug_setting, 0, GET_STR, OPT_ARG, 0, 0, 0, 0, 0,
+     0},
+#endif /* !DBUG_OFF */
     {"dump-file", 'd',
      "Dump the tablespace SDI into the file passed by user."
      " Without the filename, it will default to stdout",
-     &opts.dump_filename, &opts.dump_filename, nullptr, GET_STR, REQUIRED_ARG,
-     0, 0, 0, nullptr, 0, nullptr},
+     &opts.dump_filename, &opts.dump_filename, 0, GET_STR, REQUIRED_ARG, 0, 0,
+     0, 0, 0, 0},
     {"skip-data", 's',
      "Skip retrieving data from SDI records. Retrieve only"
      " id and type.",
-     &opts.skip_data, &opts.skip_data, nullptr, GET_BOOL, NO_ARG, 0, 0, 0,
-     nullptr, 0, nullptr},
+     &opts.skip_data, &opts.skip_data, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"id", 'i', "Retrieve the SDI record matching the id passed by user.",
-     &opts.sdi_rec_id, &opts.sdi_rec_id, nullptr, GET_ULL, REQUIRED_ARG, 0, 0,
-     ULLONG_MAX, nullptr, 1, nullptr},
+     &opts.sdi_rec_id, &opts.sdi_rec_id, 0, GET_ULL, REQUIRED_ARG, 0, 0,
+     ULLONG_MAX, 0, 1, 0},
     {"type", 't', "Retrieve the SDI records matching the type passed by user.",
-     &opts.sdi_rec_type, &opts.sdi_rec_type, nullptr, GET_ULL, REQUIRED_ARG, 0,
-     0, ULLONG_MAX, nullptr, 1, nullptr},
+     &opts.sdi_rec_type, &opts.sdi_rec_type, 0, GET_ULL, REQUIRED_ARG, 0, 0,
+     ULLONG_MAX, 0, 1, 0},
     {"strict-check", 'c',
      "Specify the strict checksum algorithm by the user."
      " Allowed values are innodb, crc32, none.",
      &opts.strict_check, &opts.strict_check, &checksum_algorithms_typelib,
-     GET_ENUM, REQUIRED_ARG, 0, 0, 0, nullptr, 0, nullptr},
+     GET_ENUM, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
     {"no-check", 'n', "Ignore the checksum verification.", &opts.no_checksum,
-     &opts.no_checksum, nullptr, GET_BOOL, NO_ARG, 0, 0, 0, nullptr, 0,
-     nullptr},
+     &opts.no_checksum, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"pretty", 'p',
      "Pretty format the SDI output."
      "If false, SDI would be not human readable but it will be of less size",
-     &opts.pretty, &opts.pretty, nullptr, GET_BOOL, OPT_ARG, 1, 0, 0, nullptr,
-     0, nullptr},
+     &opts.pretty, &opts.pretty, 0, GET_BOOL, OPT_ARG, 1, 0, 0, 0, 0, 0},
 
-    {nullptr, 0, nullptr, nullptr, nullptr, nullptr, GET_NO_ARG, NO_ARG, 0, 0,
-     0, nullptr, 0, nullptr}};
-
-/** A dummy implementation.  Actual implementation available in fil0fil.cc */
-std::ostream &Fil_page_header::print(std::ostream &out) const noexcept {
-  return out;
-}
+    {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}};
 
 /** Report a failed assertion.
 @param[in]	expr	the failed assertion if not NULL
 @param[in]	file	source file containing the assertion
 @param[in]	line	line number of the assertion */
-[[noreturn]] void ut_dbg_assertion_failed(const char *expr, const char *file,
-                                          uint64_t line) {
-  fprintf(stderr, "ibd2sdi: Assertion failure in file %s line " UINT64PF "\n",
+void ut_dbg_assertion_failed(const char *expr, const char *file, ulint line) {
+  fprintf(stderr, "ibd2sdi: Assertion failure in file %s line " ULINTPF "\n",
           file, line);
 
-  if (expr != nullptr) {
+  if (expr != NULL) {
     fprintf(stderr, "ibd2sdi: Failing assertion: %s\n", expr);
   }
 
@@ -275,7 +264,7 @@ static void try_delete_temporary_filename(const char *temp_filename) {
 @return File pointer of the created file */
 static FILE *create_tmp_file(char *temp_file_buf, const char *dir,
                              const char *prefix_pattern) {
-  FILE *file = nullptr;
+  FILE *file = NULL;
   File fd = create_temp_file(temp_file_buf, dir, prefix_pattern,
                              O_CREAT | O_RDWR, KEEP_FILE, MYF(0));
 
@@ -283,9 +272,9 @@ static FILE *create_tmp_file(char *temp_file_buf, const char *dir,
     file = my_fdopen(fd, temp_file_buf, O_RDWR, MYF(0));
   }
 
-  DBUG_EXECUTE_IF("ib_tmp_file_fail", file = nullptr; errno = EACCES;);
+  DBUG_EXECUTE_IF("ib_tmp_file_fail", file = NULL; errno = EACCES;);
 
-  if (file == nullptr) {
+  if (file == NULL) {
     ib::error() << "Unable to create temporary file. err: " << strerror(errno);
 
     if (fd >= 0) {
@@ -299,11 +288,11 @@ static FILE *create_tmp_file(char *temp_file_buf, const char *dir,
 
 /** Print the ibd2sdi tool usage. */
 static void usage() {
-#ifdef NDEBUG
+#ifdef DBUG_OFF
   print_version();
 #else
   print_version_debug();
-#endif /* NDEBUG */
+#endif /* DBUG_OFF */
   puts(ORACLE_WELCOME_COPYRIGHT_NOTICE("2015"));
   printf(
       "Usage: %s [-v] [-c <strict-check>] [-d <dump file name>] [-n]"
@@ -315,21 +304,20 @@ static void usage() {
 }
 
 /** Parse the options passed to tool. */
-extern "C" bool ibd2sdi_get_one_option(int optid,
-                                       const struct my_option *opt
-                                       [[maybe_unused]],
-                                       char *argument [[maybe_unused]]) {
+extern "C" bool ibd2sdi_get_one_option(
+    int optid, const struct my_option *opt MY_ATTRIBUTE((unused)),
+    char *argument MY_ATTRIBUTE((unused))) {
   switch (optid) {
-#ifndef NDEBUG
+#ifndef DBUG_OFF
     case '#':
       opts.dbug_setting =
           argument ? argument
                    : IF_WIN("d:O,ibd2sdi.trace", "d:o,/tmp/ibd2sdi.trace");
       DBUG_PUSH(opts.dbug_setting);
       break;
-#endif /* !NDEBUG */
+#endif /* !DBUG_OFF */
     case 'v':
-#ifdef NDEBUG
+#ifdef DBUG_OFF
       print_version();
 #else
       print_version_debug();
@@ -393,7 +381,7 @@ static bool get_options(int *argc, char ***argv) {
 /** Error logging classes. */
 namespace ib {
 
-logger::~logger() = default;
+logger::~logger() {}
 
 info::~info() {
   std::cerr << "[INFO] ibd2sdi: " << m_oss.str() << "." << std::endl;
@@ -407,25 +395,15 @@ error::~error() {
   std::cerr << "[ERROR] ibd2sdi: " << m_oss.str() << "." << std::endl;
 }
 
-/*
-MSVS complains: Warning C4722: destructor never returns, potential memory leak.
-But, the whole point of using ib::fatal temporary object is to cause an abort.
-*/
-MY_COMPILER_DIAGNOSTIC_PUSH()
-MY_COMPILER_MSVC_DIAGNOSTIC_IGNORE(4722)
-
 fatal::~fatal() {
   std::cerr << "[FATAL] ibd2sdi: " << m_oss.str() << "." << std::endl;
   ut_error;
 }
 
-// Restore the MSVS checks for Warning C4722, silenced for ib::fatal::~fatal().
-MY_COMPILER_DIAGNOSTIC_POP()
-
-/* TODO: Improve Object creation & destruction on NDEBUG */
+/* TODO: Improve Object creation & destruction on DBUG_OFF */
 class dbug : public logger {
  public:
-  ~dbug() override { DBUG_PRINT("ibd2sdi", ("%s", m_oss.str().c_str())); }
+  ~dbug() { DBUG_PRINT("ibd2sdi", ("%s", m_oss.str().c_str())); }
 };
 }  // namespace ib
 
@@ -527,7 +505,14 @@ class ib_tablespace {
 
   /** Copy Constructor.
   @param[in]	copy	another object of ib_tablespace */
-  ib_tablespace(const ib_tablespace &copy) = default;
+  ib_tablespace(const ib_tablespace &copy)
+      : m_space_id(copy.m_space_id),
+        m_page_size(copy.m_page_size),
+        m_file_vec(copy.m_file_vec),
+        m_page_num_recs(copy.m_page_num_recs),
+        m_max_recs_per_page(copy.m_max_recs_per_page),
+        m_sdi_root(copy.m_sdi_root),
+        m_tot_pages(copy.m_tot_pages) {}
 
   /** Add Datafile to vector of datafiles. Also
   resize of vector of pages.
@@ -542,7 +527,7 @@ class ib_tablespace {
   }
 
   /** Add the SDI root page numbers to tablespace.
-  @param[in]	root	root page number of SDI */
+  @param[in]	copy	root page number of SDI */
   inline void add_sdi(page_no_t root) { m_sdi_root = root; }
 
   /** @return SDI root page number */
@@ -643,7 +628,7 @@ class ib_tablespace {
 
  private:
   /** Return the SDI Root page number stored in a page.
-  @param[in]	buf		Page read from buffer
+  @param[in]	byte		Page read from buffer
   @return SDI root page number */
   inline page_no_t get_sdi_root_page_num(byte *buf);
 
@@ -731,7 +716,8 @@ static size_t fetch_page(ib_tablespace *ts, page_no_t page_num,
   }
 
   if (page_size.is_compressed() && fil_page_get_type(buf) == FIL_PAGE_SDI) {
-    byte *uncomp_buf = static_cast<byte *>(ut::malloc(2 * page_size.logical()));
+    byte *uncomp_buf =
+        static_cast<byte *>(ut_malloc_nokey(2 * page_size.logical()));
 
     byte *uncomp_page =
         static_cast<byte *>(ut_align(uncomp_buf, page_size.logical()));
@@ -765,7 +751,7 @@ static size_t fetch_page(ib_tablespace *ts, page_no_t page_num,
       memcpy(buf, uncomp_page, page_size.logical());
     }
 
-    ut::free(uncomp_buf);
+    ut_free(uncomp_buf);
   }
 
   return n_bytes;
@@ -775,9 +761,10 @@ class tablespace_creator {
  public:
   /** Constructor
   @param[in]	num_files	number of ibd files
-  @param[in]	ibd_files	array of ibd file names */
+  @param[in]	ibd_files	array of ibd file names
+  @param[in]	map		tablespaces map */
   tablespace_creator(uint32_t num_files, char **ibd_files)
-      : m_num_files(num_files), m_ibd_files(ibd_files), m_tablespace(nullptr) {}
+      : m_num_files(num_files), m_ibd_files(ibd_files), m_tablespace(NULL) {}
 
   /* Destructor */
   ~tablespace_creator() { delete m_tablespace; }
@@ -857,7 +844,7 @@ bool tablespace_creator::create() {
     MY_STAT stat_info;
 
     /* stat the file to get size and page count. */
-    if (my_stat(filename, &stat_info, MYF(0)) == nullptr) {
+    if (my_stat(filename, &stat_info, MYF(0)) == NULL) {
       ib::error() << "Unable to get file stats " << filename;
       ib::error() << "File doesn't exist";
       return true;
@@ -933,12 +920,13 @@ bool tablespace_creator::create() {
                        " root page numbers couldn't"
                        " be determined";
         return true;
+      } else {
+        m_tablespace->add_sdi(root);
       }
-      m_tablespace->add_sdi(root);
 
     } else {
       /* We found next file of system tablespace. */
-      ut_ad(m_tablespace != nullptr);
+      ut_ad(m_tablespace != NULL);
 
       if (space_id != m_tablespace->get_space_id()) {
         ib::error() << "Multiple tablespaces passed."
@@ -1141,11 +1129,12 @@ bool tablespace_creator::determine_page_size(File file_in,
 
   if (min_corruption_ratio == 1.0) {
     ib::error() << "Page size couldn't be determined";
-    return false;
+    return (false);
+  } else {
+    ib::info() << "Page size determined is : " << final_page_size;
+    page_size.copy_from(final_page_size);
+    return (true);
   }
-  ib::info() << "Page size determined is : " << final_page_size;
-  page_size.copy_from(final_page_size);
-  return true;
 }
 
 /** Verify checksum of the page and return corruption ratio
@@ -1250,7 +1239,7 @@ inline bool ib_tablespace::check_sdi(page_no_t &root) {
 @param[in]	buf		Page of tablespace
 @return SDI root page number */
 inline page_no_t ib_tablespace::get_sdi_root_page_num(byte *buf) {
-  ut_ad(buf != nullptr);
+  ut_ad(buf != NULL);
   ulint sdi_offset = fsp_header_get_sdi_offset(m_page_size);
   uint32_t version = mach_read_from_4(buf + sdi_offset);
 
@@ -1279,7 +1268,7 @@ class ibd2sdi {
         m_is_specific_rec(false),
         m_specific_id(UINT64_MAX),
         m_specific_type(UINT64_MAX),
-        m_tablespace_creator(nullptr) {}
+        m_tablespace_creator(NULL) {}
 
   /** Destructor. Close all open files. */
   ~ibd2sdi() { delete m_tablespace_creator; }
@@ -1316,7 +1305,7 @@ class ibd2sdi {
   @return false on success, true on failure */
   bool process_sdi_from_copy(ib_tablespace *ts);
 
-  /** Iterate over record from a single SDI copy. There is no comparison
+  /** Iterate over record from a single SDI copy. There is no comparision
   involved with the records in other copy
   @param[in]	ts		tablespace structure
   @param[in]	root_page_num	SDI root page number
@@ -1458,7 +1447,7 @@ bool ibd2sdi::process_sdi_from_copy(ib_tablespace *ts) {
   return (dump_all_recs_in_leaf_level(ts, ts->get_sdi_root(), m_out_stream));
 }
 
-/** Iterate over record from a single SDI copy. There is no comparison
+/** Iterate over record from a single SDI copy. There is no comparision
 involved with the records in other copy
 @param[in]	ts		tablespace structure
 @param[in]	root_page_num	SDI root page number
@@ -1494,13 +1483,13 @@ bool ibd2sdi::dump_all_recs_in_leaf_level(ib_tablespace *ts,
   byte *current_rec = get_first_user_rec(ts, page_size.logical(), buf);
   uint64_t sdi_id;
   uint64_t sdi_type;
-  byte *sdi_data = nullptr;
+  byte *sdi_data = NULL;
   uint64_t sdi_data_len = 0;
   bool corrupt = false;
 
   /* Check and Dump records */
   fprintf(out_stream, "%s", "[\"ibd2sdi\"\n");
-  while (current_rec != nullptr &&
+  while (current_rec != NULL &&
          get_rec_type(current_rec) != REC_STATUS_SUPREMUM &&
          !explicit_sdi_rec_found && !corrupt) {
     dberr_t err = parse_fields_in_rec(ts, current_rec, &sdi_type, &sdi_id,
@@ -1511,7 +1500,7 @@ bool ibd2sdi::dump_all_recs_in_leaf_level(ib_tablespace *ts,
                                                      sdi_data_len, out_stream);
 
       free(sdi_data);
-      sdi_data = nullptr;
+      sdi_data = NULL;
     }
 
     if (explicit_sdi_rec_found) {
@@ -1540,8 +1529,9 @@ uint64_t ibd2sdi::read_page_and_return_level(ib_tablespace *ts,
   if (fetch_page(ts, page_num, buf_len, buf) == IB_ERROR) {
     ib::error() << "Couldn't read page " << page_num;
     return UINT64_MAX;
+  } else {
+    ib::dbug() << "Read page number: " << page_num;
   }
-  ib::dbug() << "Read page number: " << page_num;
 
   ulint page_type = fil_page_get_type(buf);
 
@@ -1638,6 +1628,8 @@ uint64_t ibd2sdi::copy_compressed_blob(ib_tablespace *ts,
   DBUG_TRACE;
 
   byte page_buf[UNIV_PAGE_SIZE_MAX];
+  uint64_t calc_length = 0;
+  uint64_t part_len;
   page_no_t page_num = first_blob_page_num;
   z_stream d_stream;
   int err;
@@ -1651,7 +1643,7 @@ uint64_t ibd2sdi::copy_compressed_blob(ib_tablespace *ts,
 
   /* Zlib inflate needs 32KB for the default window size, plus
   a few KB for small objects */
-  mem_heap_t *heap = mem_heap_create(40000, UT_LOCATION_HERE);
+  mem_heap_t *heap = mem_heap_create(40000);
   page_zip_set_alloc(&d_stream, heap);
 
   ut_ad(page_size.is_compressed());
@@ -1674,12 +1666,16 @@ uint64_t ibd2sdi::copy_compressed_blob(ib_tablespace *ts,
       break;
     }
 
+    part_len =
+        mach_read_from_4(page_buf + FIL_PAGE_DATA + lob::LOB_HDR_PART_LEN);
+
     page_no_t next_page_num = mach_read_from_4(page_buf + FIL_PAGE_NEXT);
     space_id_t space_id =
         mach_read_from_4(page_buf + FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID);
 
     d_stream.next_in = page_buf + FIL_PAGE_DATA;
     d_stream.avail_in = static_cast<uInt>(page_size.physical() - FIL_PAGE_DATA);
+    calc_length += part_len;
     err = inflate(&d_stream, Z_NO_FLUSH);
     switch (err) {
       case Z_OK:
@@ -1691,7 +1687,7 @@ uint64_t ibd2sdi::copy_compressed_blob(ib_tablespace *ts,
         if (next_page_num == FIL_NULL) {
           goto func_exit;
         }
-        [[fallthrough]];
+      /* fall through */
       default:
       inflate_error : {
         page_id_t page_id(space_id, page_num);
@@ -1786,8 +1782,9 @@ err_t ibd2sdi::reach_to_leftmost_leaf_level(ib_tablespace *ts, uint32_t buf_len,
     if (rec_type != REC_STATUS_INFIMUM) {
       ib::error() << "INFIMUM not found on index page " << cur_page_num;
       break;
+    } else {
+      ib::dbug() << "INFIMUM found";
     }
-    ib::dbug() << "INFIMUM found";
 
     ulint next_rec_off_t =
         mach_read_from_2(buf + PAGE_NEW_INFIMUM - REC_OFF_NEXT);
@@ -1819,9 +1816,10 @@ err_t ibd2sdi::reach_to_leftmost_leaf_level(ib_tablespace *ts, uint32_t buf_len,
     ib::error() << "Leftmost leaf level page not found"
                 << " or invalid";
     return FALIURE;
+  } else {
+    ib::dbug() << "Reached leaf level";
+    return SUCCESS;
   }
-  ib::dbug() << "Reached leaf level";
-  return SUCCESS;
 }
 
 /** Extract SDI record fields
@@ -1954,24 +1952,24 @@ byte ibd2sdi::get_rec_type(byte *rec) {
 
 /** Return the location of the next record
 @param[in]	ts		tablespace structure
-@param[in]	current_rec_arg	current sdi record in a page
+@param[in]	current_rec	current sdi record in a page
 @param[in]	buf_len		buffer length
 @param[in]	buf		page containing the current rec
 @param[out]	corrupt		true if corruption detected, else false
 @return location of the next record, else NULL if there are no
 user records */
-byte *ibd2sdi::get_next_rec(ib_tablespace *ts, byte *current_rec_arg,
+byte *ibd2sdi::get_next_rec(ib_tablespace *ts, byte *current_rec,
                             uint32_t buf_len, byte *buf, bool *corrupt) {
   DBUG_TRACE;
 
   page_no_t page_num = mach_read_from_4(buf + FIL_PAGE_OFFSET);
   bool is_comp = page_is_comp(buf);
-  ulint next_rec_offset = rec_get_next_offs(current_rec_arg, is_comp);
+  ulint next_rec_offset = rec_get_next_offs(current_rec, is_comp);
 
   if (next_rec_offset == 0) {
     ib::error() << "Record Corruption detected. Aborting";
     *corrupt = true;
-    return nullptr;
+    return NULL;
   }
 
   byte *next_rec = buf + next_rec_offset;
@@ -2005,15 +2003,16 @@ byte *ibd2sdi::get_next_rec(ib_tablespace *ts, byte *current_rec_arg,
 
     if (next_page_num == FIL_NULL) {
       *corrupt = false;
-      return nullptr;
+      return NULL;
     }
 
     if (fetch_page(ts, next_page_num, buf_len, buf) == IB_ERROR) {
       ib::error() << "Couldn't read next page " << next_page_num;
       *corrupt = true;
-      return nullptr;
+      return NULL;
+    } else {
+      ib::dbug() << "Read page number: " << next_page_num;
     }
-    ib::dbug() << "Read page number: " << next_page_num;
 
     ulint page_type = fil_page_get_type(buf);
 
@@ -2021,18 +2020,18 @@ byte *ibd2sdi::get_next_rec(ib_tablespace *ts, byte *current_rec_arg,
       ib::error() << "Unexpected page type: " << page_type
                   << ". Expected page type: " << FIL_PAGE_SDI;
       *corrupt = true;
-      return nullptr;
+      return NULL;
     }
 
     if (ts->inc_num_of_recs_on_page(next_page_num)) {
       *corrupt = true;
-      return nullptr;
+      return NULL;
     }
     next_rec = get_first_user_rec(ts, buf_len, buf);
   } else {
     if (ts->inc_num_of_recs_on_page(page_num)) {
       *corrupt = true;
-      return nullptr;
+      return NULL;
     }
   }
 
@@ -2056,7 +2055,7 @@ void ibd2sdi::dump_sdi_rec(uint64_t sdi_type, uint64_t sdi_id, byte *sdi_data,
   fprintf(out_stream, "\t\"id\": " UINT64PF, sdi_id);
 
   if (!m_skip_data) {
-    ut_ad(sdi_data != nullptr);
+    ut_ad(sdi_data != NULL);
     fprintf(out_stream, ",\n");
     fprintf(out_stream, "\t\"object\":\n");
     fprintf(out_stream, "\t\t");
@@ -2079,11 +2078,7 @@ void ibd2sdi::dump_sdi_rec(uint64_t sdi_type, uint64_t sdi_id, byte *sdi_data,
       d.Accept(writer);
       fprintf(out_stream, "%s", _b.GetString());
     } else {
-      if (fwrite(sdi_data, 1, static_cast<size_t>(sdi_data_len - 1),
-                 out_stream) != static_cast<size_t>(sdi_data_len - 1)) {
-        std::cerr << "File write error: " << ferror(out_stream) << std::endl;
-        exit(1);
-      }
+      fwrite(sdi_data, 1, static_cast<size_t>(sdi_data_len - 1), out_stream);
     }
   }
 
@@ -2108,7 +2103,7 @@ byte *ibd2sdi::get_first_user_rec(ib_tablespace *ts, uint32_t buf_len,
 
   if (next_rec_off_t > buf_len) {
     ut_ad(0);
-    return (nullptr);
+    return (NULL);
   }
 
   if (memcmp(buf + PAGE_NEW_INFIMUM, "infimum", strlen("infimum")) != 0) {
@@ -2131,7 +2126,7 @@ byte *ibd2sdi::get_first_user_rec(ib_tablespace *ts, uint32_t buf_len,
     current_rec =
         get_next_rec(ts, current_rec, page_size.logical(), buf, &corrupt);
     if (corrupt) {
-      return nullptr;
+      return NULL;
     }
   }
 
@@ -2182,7 +2177,7 @@ bool ibd2sdi::process_files() {
 @return false on success, true on failure */
 bool ibd2sdi::dump() {
   bool ret = true;
-  ut_a(m_tablespace_creator != nullptr);
+  ut_a(m_tablespace_creator != 0);
   ib_tablespace *ts = m_tablespace_creator->get_tablespace();
   ret = process_sdi_from_copy(ts);
   return (ret);
@@ -2264,7 +2259,7 @@ int main(int argc, char **argv) {
 
     dump_file = create_tmp_file(tmp_filename_buf, dir_buf, "ib_sdi");
 
-    if (dump_file == nullptr) {
+    if (dump_file == NULL) {
       ib::error() << "Invalid Dumpfile passed";
       return 1;
     }
@@ -2290,7 +2285,7 @@ int main(int argc, char **argv) {
 
   if (!ret && opts.is_dump_file) {
     /* Rename file can fail if the source and destination
-    are across partitions. */
+    are across parititions. */
     if (my_rename(tmp_filename_buf, opts.dump_filename, MYF(0)) == -1) {
       if (my_copy(tmp_filename_buf, opts.dump_filename, MYF(0)) != 0) {
         ib::error() << "Copy failed: from: " << tmp_filename_buf
@@ -2301,8 +2296,9 @@ int main(int argc, char **argv) {
                     << " temporary file " << tmp_filename_buf
                     << " and delete it manually";
         return 1;
+      } else {
+        try_delete_temporary_filename(tmp_filename_buf);
       }
-      try_delete_temporary_filename(tmp_filename_buf);
     }
   } else if (opts.is_dump_file) {
     try_delete_temporary_filename(tmp_filename_buf);
