@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -30,7 +29,7 @@
   Note that we can't have assertion on file descriptors;  The reason for
   this is that during mysql shutdown, another thread can close a file
   we are working on.  In this case we should just return read errors from
-  the file descriptor.
+  the file descriptior.
 */
 
 #include <errno.h>
@@ -42,14 +41,6 @@
 #include "mysql/psi/mysql_socket.h"
 #include "vio/vio_priv.h"
 
-/*
-  BIO_set_callback_ex was added in openSSL 1.1.1
-  For older openSSL, use the deprecated BIO_set_callback.
-*/
-#if OPENSSL_VERSION_NUMBER >= 0x10101000L
-#define HAVE_BIO_SET_CALLBACK_EX
-#endif
-
 /* clang-format off */
 /**
   @page page_protocol_basic_tls TLS
@@ -59,7 +50,7 @@
   after the data is compressed right before the data is written to
   the network layer.
 
-  The TLS support is announced in
+  The TLS suppport is announced in
   @ref page_protocol_connection_phase_packets_protocol_handshake sent by the
   server via ::CLIENT_SSL and is enabled if the client returns the same
   capability.
@@ -94,7 +85,7 @@
   00 00 00 00                                           ....
   ~~~~~~~
 
-  Then the rest of the communication is switched to TLS:
+  Then the rest of the communication is swithed to TLS:
   ~~~~~~~
   16 03 01 00 5e 01 00 00    5a 03 01 4c a3 49 2e 7a    ....^...Z..L.I.z
   b5 06 75 68 5c 30 36 73    f1 82 79 70 58 4c 64 bb    ..uh\06s..ypXLd.
@@ -119,7 +110,9 @@
 */
 /* clang-format on */
 
-#ifndef NDEBUG
+#ifdef HAVE_OPENSSL
+
+#ifndef DBUG_OFF
 
 static void report_errors(SSL *ssl) {
   unsigned long l;
@@ -130,11 +123,7 @@ static void report_errors(SSL *ssl) {
 
   DBUG_TRACE;
 
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-  while ((l = ERR_get_error_all(&file, &line, nullptr, &data, &flags))) {
-#else  /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
   while ((l = ERR_get_error_line_data(&file, &line, &data, &flags))) {
-#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
     DBUG_PRINT("error", ("OpenSSL: %s:%s:%d:%s\n", ERR_error_string(l, buf),
                          file, line, (flags & ERR_TXT_STRING) ? data : ""));
   }
@@ -244,7 +233,7 @@ static bool ssl_should_retry(Vio *vio, int ret, enum enum_vio_io_event *event,
       /* first save the top ERR error */
       err_error = ERR_get_error();
       /* now report all remaining errors on and/or clear the error stack */
-#ifndef NDEBUG /* Debug build */
+#ifndef DBUG_OFF /* Debug build */
       /* Note: the OpenSSL error queue gets cleared in report_errors(). */
       report_errors(ssl);
 #else /* Release build */
@@ -267,7 +256,7 @@ size_t vio_ssl_read(Vio *vio, uchar *buf, size_t size) {
 
   DBUG_TRACE;
 
-  while (true) {
+  while (1) {
     enum enum_vio_io_event event;
 
     /*
@@ -275,11 +264,11 @@ size_t vio_ssl_read(Vio *vio, uchar *buf, size_t size) {
       SSL_read() returns an error from the error queue, when SSL_read() failed
       because it would block.
     */
-    assert(ERR_peek_error() == 0);
+    DBUG_ASSERT(ERR_peek_error() == 0);
 
     ret = SSL_read(ssl, buf, (int)size);
 
-    if (ret > 0) break;
+    if (ret >= 0) break;
 
     /* Process the SSL I/O error. */
     if (!ssl_should_retry(vio, ret, &event, &ssl_errno_not_used)) break;
@@ -309,7 +298,7 @@ size_t vio_ssl_write(Vio *vio, const uchar *buf, size_t size) {
 
   DBUG_TRACE;
 
-  while (true) {
+  while (1) {
     enum enum_vio_io_event event;
 
     /*
@@ -317,11 +306,11 @@ size_t vio_ssl_write(Vio *vio, const uchar *buf, size_t size) {
       SSL_write() returns an error from the error queue, when SSL_write() failed
       because it would block.
     */
-    assert(ERR_peek_error() == 0);
+    DBUG_ASSERT(ERR_peek_error() == 0);
 
     ret = SSL_write(ssl, buf, (int)size);
 
-    if (ret > 0) break;
+    if (ret >= 0) break;
 
     /* Process the SSL I/O error. */
     if (!ssl_should_retry(vio, ret, &event, &ssl_errno_not_used)) break;
@@ -355,7 +344,7 @@ int vio_ssl_shutdown(Vio *vio) {
     alert on socket shutdown to avoid truncation attacks. However, this can
     cause problems since we often hold a lock during shutdown and this IO can
     take an unbounded amount of time to complete. Since our packets are self
-    describing with length, we aren't vulnerable to these attacks. Therefore,
+    describing with length, we aren't vunerable to these attacks. Therefore,
     we just shutdown by closing the socket (quiet shutdown).
     */
     SSL_set_quiet_shutdown(ssl, 1);
@@ -374,12 +363,6 @@ int vio_ssl_shutdown(Vio *vio) {
       default: /* Shutdown failed */
         DBUG_PRINT("vio_error",
                    ("SSL_shutdown() failed, error: %d", SSL_get_error(ssl, r)));
-#ifndef NDEBUG /* Debug build */
-        /* Note: the OpenSSL error queue gets cleared in report_errors(). */
-        report_errors(ssl);
-#else /* Release build */
-        ERR_clear_error();
-#endif
         break;
     }
   }
@@ -394,7 +377,7 @@ void vio_ssl_delete(Vio *vio) {
 
   if (vio->ssl_arg) {
     SSL_free((SSL *)vio->ssl_arg);
-    vio->ssl_arg = nullptr;
+    vio->ssl_arg = 0;
   }
 
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
@@ -428,7 +411,7 @@ static size_t ssl_handshake_loop(Vio *vio, SSL *ssl, ssl_handshake_func_t func,
   vio->ssl_arg = ssl;
 
   /* Initiate the SSL handshake. */
-  while (true) {
+  while (1) {
     enum enum_vio_io_event event;
 
     /*
@@ -436,7 +419,7 @@ static size_t ssl_handshake_loop(Vio *vio, SSL *ssl, ssl_handshake_func_t func,
       SSL-handshake-function returns an error from the error queue, when the
       function failed because it would block.
     */
-    assert(ERR_peek_error() == 0);
+    DBUG_ASSERT(ERR_peek_error() == 0);
 
     int handshake_ret;
     handshake_ret = func(ssl);
@@ -449,11 +432,6 @@ static size_t ssl_handshake_loop(Vio *vio, SSL *ssl, ssl_handshake_func_t func,
     /* Process the SSL I/O error. */
     if (!ssl_should_retry(vio, handshake_ret, &event, ssl_errno_holder)) break;
 
-    DBUG_EXECUTE_IF("bug32372038", {
-      DBUG_SET("+d,bug32372038_ssl_started");
-      return VIO_SOCKET_WANT_READ;
-    };);
-
     if (!vio->is_blocking_flag) {
       switch (event) {
         case VIO_IO_EVENT_READ:
@@ -469,176 +447,19 @@ static size_t ssl_handshake_loop(Vio *vio, SSL *ssl, ssl_handshake_func_t func,
     if (vio_socket_io_wait(vio, event)) break;
   }
 
-  vio->ssl_arg = nullptr;
+  vio->ssl_arg = NULL;
 
   return ret;
 }
 
-#ifdef HAVE_PSI_SOCKET_INTERFACE
-long pfs_ssl_bio_callback_ex(BIO *b, int oper, const char * /* argp */,
-                             size_t len, int /* argi */, long /* argl */,
-                             int ret, size_t *processed) {
-  Vio *vio;
-  static const char *method_name = "open_ssl::bio::socket";
-
-  /*
-    Note:
-    Normally, typical instrumentation for the performance schema would
-    - define a local PSI_socket_locker_state variable
-    - define a local PSI_socket_locker pointer
-    - call PSI_SOCKET_CALL(start_socket_wait)
-    - perform the operation
-    - call PSI_SOCKET_CALL(end_socket_wait)
-
-    Now, because this is done in a function callback attached to SSL itself,
-    invocation of the start and end event are done in different invocations,
-    so some state needs to be preserved between:
-    - BIO_CB_READ and BIO_CB_READ|BIO_CB_RETURN,
-    - BIO_CB_WRITE and BIO_CB_WRITE|BIO_CB_RETURN
-
-    This state is preserved in attributes of vio:
-    - m_psi_read_state and m_psi_read_locker for BIO_CB_READ
-    - m_psi_write_state and m_psi_write_locker for BIO_CB_WRITE
-
-    Raw socket operations are not supposed to be re-entrant,
-    so we assert here that:
-    - there is not a current read operation when performing a read
-    - there is not a current write operation when performing a write.
-    for extra safety.
-  */
-  switch (oper) {
-    case BIO_CB_READ:
-      vio = reinterpret_cast<Vio *>(BIO_get_callback_arg(b));
-      assert(vio->m_psi_read_locker == nullptr);
-      if (vio->mysql_socket.m_psi != nullptr) {
-        vio->m_psi_read_locker = PSI_SOCKET_CALL(start_socket_wait)(
-            &vio->m_psi_read_state, vio->mysql_socket.m_psi, PSI_SOCKET_RECV,
-            len, method_name, oper);
-      }
-      break;
-    case BIO_CB_READ | BIO_CB_RETURN:
-      vio = reinterpret_cast<Vio *>(BIO_get_callback_arg(b));
-      if (vio->m_psi_read_locker != nullptr) {
-        PSI_SOCKET_CALL(end_socket_wait)(vio->m_psi_read_locker, *processed);
-        vio->m_psi_read_locker = nullptr;
-      }
-      break;
-    case BIO_CB_WRITE:
-      vio = reinterpret_cast<Vio *>(BIO_get_callback_arg(b));
-      assert(vio->m_psi_write_locker == nullptr);
-      if (vio->mysql_socket.m_psi != nullptr) {
-        vio->m_psi_write_locker = PSI_SOCKET_CALL(start_socket_wait)(
-            &vio->m_psi_write_state, vio->mysql_socket.m_psi, PSI_SOCKET_SEND,
-            len, method_name, oper);
-      }
-      break;
-    case BIO_CB_WRITE | BIO_CB_RETURN:
-      vio = reinterpret_cast<Vio *>(BIO_get_callback_arg(b));
-      if (vio->m_psi_write_locker != nullptr) {
-        PSI_SOCKET_CALL(end_socket_wait)(vio->m_psi_write_locker, *processed);
-        vio->m_psi_write_locker = nullptr;
-      }
-      break;
-    case BIO_CB_CTRL:
-    case BIO_CB_CTRL | BIO_CB_RETURN:
-    case BIO_CB_FREE:
-    case BIO_CB_FREE | BIO_CB_RETURN:
-      break;
-    case BIO_CB_PUTS:
-    case BIO_CB_PUTS | BIO_CB_RETURN:
-    case BIO_CB_GETS:
-    case BIO_CB_GETS | BIO_CB_RETURN:
-    default:
-      assert(false);
-  }
-
-  return ret;
-}
-#endif /* HAVE_PSI_SOCKET_INTERFACE */
-
-#ifdef HAVE_PSI_SOCKET_INTERFACE
-#ifndef HAVE_BIO_SET_CALLBACK_EX
-/**
-  Forward openSSL old style callback to openSSL 1.1.1 style callback.
-*/
-long pfs_ssl_bio_callback(BIO *b, int oper, const char *argp, int argi,
-                          long argl, long ret) {
-  size_t len = argi;
-  /*
-    For pre events:
-    - irrelevant (not used in pfs_ssl_bio_callback_ex)
-    For post (BIO_CB_RETURN) events,
-    the number of bytes:
-    - actually read, per the return value of recv()
-    - actually written, per the return value of send()
-  */
-  size_t processed = (ret >= 0) ? ret : 0;
-
-  return pfs_ssl_bio_callback_ex(b, oper, argp, len, argi, argl, ret,
-                                 &processed);
-}
-#endif /* HAVE_BIO_SET_CALLBACK_EX */
-#endif /* HAVE_PSI_SOCKET_INTERFACE */
-
-#ifdef HAVE_PSI_SOCKET_INTERFACE
-static void pfs_ssl_setup_instrumentation(Vio *vio, const SSL *ssl) {
-  BIO *rbio = SSL_get_rbio(ssl);
-  assert(rbio != nullptr);
-  assert(BIO_method_type(rbio) == BIO_TYPE_SOCKET);
-
-  BIO *wbio = SSL_get_wbio(ssl);
-  assert(wbio != nullptr);
-  assert(BIO_method_type(wbio) == BIO_TYPE_SOCKET);
-
-  char *cb_arg = reinterpret_cast<char *>(vio);
-  assert(cb_arg != nullptr);
-
-  BIO_set_callback_arg(rbio, cb_arg);
-
-#ifdef HAVE_BIO_SET_CALLBACK_EX
-  BIO_set_callback_ex(rbio, pfs_ssl_bio_callback_ex);
-#else
-  BIO_set_callback(rbio, pfs_ssl_bio_callback);
-#endif
-
-  if (rbio != wbio) {
-    BIO_set_callback_arg(wbio, cb_arg);
-
-#ifdef HAVE_BIO_SET_CALLBACK_EX
-    BIO_set_callback_ex(wbio, pfs_ssl_bio_callback_ex);
-#else
-    BIO_set_callback(wbio, pfs_ssl_bio_callback);
-#endif
-  }
-}
-#endif /* HAVE_PSI_SOCKET_INTERFACE */
-
-#ifndef NDEBUG
-static void print_ssl_session_id(SSL_SESSION *sess, const char *action) {
-  unsigned int length;
-  const unsigned char *id = SSL_SESSION_get_id(sess, &length);
-  char tohex[1024], *w = &tohex[0];
-
-  if (id && length) {
-    for (unsigned inx = 0; inx < length && w < &tohex[1021]; inx++) {
-      sprintf(w, "%2.2X", id[inx]);
-      w += 2;
-    }
-    *w = 0;
-  } else
-    strcpy(tohex, "<empty>");
-  DBUG_PRINT("info", ("%sSSL session ID [%s]", action, tohex));
-}
-#endif  // !NDEBUG
-
 static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
-                  SSL_SESSION *ssl_session, ssl_handshake_func_t func,
-                  unsigned long *ssl_errno_holder, SSL **sslptr) {
+                  ssl_handshake_func_t func, unsigned long *ssl_errno_holder,
+                  SSL **sslptr) {
   SSL *ssl = nullptr;
   my_socket sd = mysql_socket_getfd(vio->mysql_socket);
 
   /* Declared here to make compiler happy */
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
   int j, n;
 #endif
 
@@ -656,21 +477,6 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
       return 1;
     }
 
-    if (ssl_session != nullptr) {
-#ifndef NDEBUG
-      print_ssl_session_id(ssl_session, "Setting for reuse ");
-#endif
-      if (!SSL_set_session(ssl, ssl_session)) {
-#ifndef NDEBUG
-        DBUG_PRINT("error", ("SSL_set_session failed"));
-        report_errors(ssl);
-#endif
-        ERR_clear_error();
-      } else {
-        DBUG_PRINT("info", ("reused existing session"));
-      }
-    }
-
     DBUG_PRINT("info", ("ssl: %p timeout: %ld", ssl, timeout));
     SSL_clear(ssl);
     SSL_SESSION_set_timeout(SSL_get_session(ssl), timeout);
@@ -681,9 +487,9 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
     sk_SSL_COMP_zero(SSL_COMP_get_compression_methods());
 #endif
 
-#if !defined(NDEBUG)
+#if !defined(DBUG_OFF)
     {
-      STACK_OF(SSL_COMP) *ssl_comp_methods = nullptr;
+      STACK_OF(SSL_COMP) *ssl_comp_methods = NULL;
       ssl_comp_methods = SSL_COMP_get_compression_methods();
       n = sk_SSL_COMP_num(ssl_comp_methods);
       DBUG_PRINT("info", ("Available compression methods:\n"));
@@ -703,15 +509,9 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
 #endif
 
     *sslptr = ssl;
-
-#ifdef HAVE_PSI_SOCKET_INTERFACE
-    pfs_ssl_setup_instrumentation(vio, ssl);
-#endif /* HAVE_PSI_SOCKET_INTERFACE */
-
   } else {
     ssl = *sslptr;
   }
-  ERR_clear_error();
 
   size_t loop_ret;
   if ((loop_ret = ssl_handshake_loop(vio, ssl, func, ssl_errno_holder))) {
@@ -724,9 +524,6 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
     *sslptr = nullptr;
     return (int)VIO_SOCKET_ERROR;
   }
-#ifndef NDEBUG
-  print_ssl_session_id(SSL_get_session(ssl), "Connected ");
-#endif
 
   /*
     Connection succeeded. Install new function handlers,
@@ -738,7 +535,7 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
     *sslptr = nullptr;
   }
 
-#ifndef NDEBUG
+#ifndef DBUG_OFF
   {
     /* Print some info about the peer */
     X509 *cert;
@@ -770,20 +567,19 @@ static int ssl_do(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
 int sslaccept(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
               unsigned long *ssl_errno_holder) {
   DBUG_TRACE;
-  int ret =
-      ssl_do(ptr, vio, timeout, nullptr, SSL_accept, ssl_errno_holder, nullptr);
+  int ret = ssl_do(ptr, vio, timeout, SSL_accept, ssl_errno_holder, nullptr);
   return ret;
 }
 
 int sslconnect(struct st_VioSSLFd *ptr, Vio *vio, long timeout,
-               SSL_SESSION *session, unsigned long *ssl_errno_holder,
-               SSL **ssl) {
+               unsigned long *ssl_errno_holder, SSL **ssl) {
   DBUG_TRACE;
-  int ret =
-      ssl_do(ptr, vio, timeout, session, SSL_connect, ssl_errno_holder, ssl);
+  int ret = ssl_do(ptr, vio, timeout, SSL_connect, ssl_errno_holder, ssl);
   return ret;
 }
 
 bool vio_ssl_has_data(Vio *vio) {
   return SSL_pending(static_cast<SSL *>(vio->ssl_arg)) > 0 ? true : false;
 }
+
+#endif /* HAVE_OPENSSL */
