@@ -29,7 +29,7 @@
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_loglevel.h"
-#if !defined(HAVE_WOLFSSL) && !defined(HAVE_PSI_INTERFACE)
+#if !defined(HAVE_PSI_INTERFACE)
 #include "mysql/psi/mysql_rwlock.h"
 #endif
 #include "mysql/service_mysql_alloc.h"
@@ -158,18 +158,15 @@ static void report_errors() {
   const char *data;
   int line, flags;
 
-  DBUG_ENTER("report_errors");
+  DBUG_TRACE;
 
-  // Note: WolfSSL returns failures to read data as negative int values
-  while (static_cast<int>(
-             l = ERR_get_error_line_data(&file, &line, &data, &flags)) > 0) {
+  while ((l = ERR_get_error_line_data(&file, &line, &data, &flags)) > 0) {
 #ifndef DBUG_OFF /* Avoid warning */
     char buf[200];
     DBUG_PRINT("error", ("OpenSSL: %s:%s:%d:%s\n", ERR_error_string(l, buf),
                          file, line, (flags & ERR_TXT_STRING) ? data : ""));
 #endif
   }
-  DBUG_VOID_RETURN;
 }
 
 static const char *ssl_error_string[] = {
@@ -192,7 +189,7 @@ const char *sslGetErrString(enum enum_ssl_init_error e) {
 static int vio_set_cert_stuff(SSL_CTX *ctx, const char *cert_file,
                               const char *key_file,
                               enum enum_ssl_init_error *error) {
-  DBUG_ENTER("vio_set_cert_stuff");
+  DBUG_TRACE;
   DBUG_PRINT("enter", ("ctx: %p  cert_file: %s  key_file: %s", ctx, cert_file,
                        key_file));
 
@@ -208,7 +205,7 @@ static int vio_set_cert_stuff(SSL_CTX *ctx, const char *cert_file,
     DBUG_EXECUTE("error", ERR_print_errors_fp(DBUG_FILE););
     my_message_local(ERROR_LEVEL, EE_SSL_ERROR_FROM_FILE,
                      sslGetErrString(*error), cert_file);
-    DBUG_RETURN(1);
+    return 1;
   }
 
   if (key_file &&
@@ -219,7 +216,7 @@ static int vio_set_cert_stuff(SSL_CTX *ctx, const char *cert_file,
     DBUG_EXECUTE("error", ERR_print_errors_fp(DBUG_FILE););
     my_message_local(ERROR_LEVEL, EE_SSL_ERROR_FROM_FILE,
                      sslGetErrString(*error), key_file);
-    DBUG_RETURN(1);
+    return 1;
   }
 
   /*
@@ -231,20 +228,18 @@ static int vio_set_cert_stuff(SSL_CTX *ctx, const char *cert_file,
     DBUG_PRINT("error", ("%s", sslGetErrString(*error)));
     DBUG_EXECUTE("error", ERR_print_errors_fp(DBUG_FILE););
     my_message_local(ERROR_LEVEL, EE_SSL_ERROR, sslGetErrString(*error));
-    DBUG_RETURN(1);
+    return 1;
   }
 
-  DBUG_RETURN(0);
+  return 0;
 }
 
-#ifndef HAVE_WOLFSSL
-
 /*
   OpenSSL 1.1 supports native platform threads,
   so we don't need the following callback functions.
 */
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-  /* OpenSSL specific */
+/* OpenSSL specific */
 
 #ifdef HAVE_PSI_INTERFACE
 static PSI_rwlock_key key_rwlock_openssl;
@@ -410,8 +405,6 @@ void vio_ssl_end() {
   }
 }
 
-#endif  // OpenSSL specific
-
 void ssl_start() {
   if (!ssl_initialized) {
     ssl_initialized = true;
@@ -420,16 +413,13 @@ void ssl_start() {
     OpenSSL_add_all_algorithms();
     SSL_load_error_strings();
 
-#ifndef HAVE_WOLFSSL
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
     init_ssl_locks();
     init_lock_callback_functions();
 #endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
-#endif
   }
 }
 
-#ifndef HAVE_WOLFSSL
 /**
   Set fips mode in openssl library,
   When we set fips mode ON/STRICT, it will perform following operations:
@@ -461,6 +451,14 @@ int set_fips_mode(const uint fips_mode, char err_string[OPENSSL_ERROR_LENGTH]) {
     goto EXIT;
   }
   if (!(rc = FIPS_mode_set(fips_mode))) {
+    /*
+      If OS doesn't have FIPS enabled openssl library and user sets FIPS mode
+      ON, It fails with proper error. But in the same time it doesn't allow to
+      perform any cryptographic operation. Now if FIPS mode set fails with
+      error, setting old working FIPS mode value in the OpenSSL library. It will
+      allow successful cryptographic operation and will not abort the server.
+    */
+    FIPS_mode_set(fips_mode_old);
     err_library = ERR_get_error();
     ERR_error_string_n(err_library, err_string, OPENSSL_ERROR_LENGTH - 1);
     err_string[OPENSSL_ERROR_LENGTH - 1] = '\0';
@@ -475,7 +473,6 @@ EXIT:
   @returns openssl current fips mode
 */
 uint get_fips_mode() { return FIPS_mode(); }
-#endif
 
 long process_tls_version(const char *tls_version) {
   const char *separator = ",";
@@ -525,26 +522,6 @@ long process_tls_version(const char *tls_version) {
     return tls_ctx_flag;
 }
 
-#ifdef HAVE_WOLFSSL
-static int wolfssl_recv(WOLFSSL *ssl, char *buf, int sz, void *vio) {
-  size_t ret;
-
-  (void)ssl;
-  ret = vio_read(static_cast<Vio *>(vio), (uchar *)buf, sz);
-
-  /* check if connection was closed */
-  if (ret == 0) {
-    return WOLFSSL_CBIO_ERR_CONN_CLOSE;
-  }
-  return static_cast<int>(ret);
-}
-
-static int wolfssl_send(WOLFSSL *ssl, char *buf, int sz, void *vio) {
-  (void)ssl;
-  return (int)vio_write(static_cast<Vio *>(vio), (unsigned char *)buf, sz);
-}
-#endif /* HAVE_WOLFSSL */
-
 /************************ VioSSLFd **********************************/
 static struct st_VioSSLFd *new_VioSSLFd(
     const char *key_file, const char *cert_file, const char *ca_file,
@@ -557,7 +534,7 @@ static struct st_VioSSLFd *new_VioSSLFd(
   long ssl_ctx_options = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
   int ret_set_cipherlist = 0;
   char cipher_list[SSL_CIPHER_LIST_SIZE] = {0};
-  DBUG_ENTER("new_VioSSLFd");
+  DBUG_TRACE;
   DBUG_PRINT(
       "enter",
       ("key_file: '%s'  cert_file: '%s'  ca_file: '%s'  ca_path: '%s'  "
@@ -571,7 +548,7 @@ static struct st_VioSSLFd *new_VioSSLFd(
     *error = SSL_TLS_VERSION_INVALID;
     DBUG_PRINT("error", ("TLS version invalid : %s", sslGetErrString(*error)));
     report_errors();
-    DBUG_RETURN(0);
+    return 0;
   }
 
   ssl_ctx_options = (ssl_ctx_options | ssl_ctx_flags) &
@@ -580,10 +557,10 @@ static struct st_VioSSLFd *new_VioSSLFd(
 #ifdef HAVE_TLSv13
                      | SSL_OP_NO_TLSv1_3
 #endif /* HAVE_TLSv13 */
-                    );
+                     | SSL_OP_NO_TICKET);
   if (!(ssl_fd = ((struct st_VioSSLFd *)my_malloc(
             key_memory_vio_ssl_fd, sizeof(struct st_VioSSLFd), MYF(0)))))
-    DBUG_RETURN(0);
+    return 0;
 
   if (!(ssl_fd->ssl_context = SSL_CTX_new(is_client ?
 #ifdef HAVE_TLSv13
@@ -598,7 +575,7 @@ static struct st_VioSSLFd *new_VioSSLFd(
     DBUG_PRINT("error", ("%s", sslGetErrString(*error)));
     report_errors();
     my_free(ssl_fd);
-    DBUG_RETURN(0);
+    return 0;
   }
 
 #ifdef HAVE_TLSv13
@@ -663,10 +640,6 @@ static struct st_VioSSLFd *new_VioSSLFd(
   }
 
   if (crl_file || crl_path) {
-#ifdef HAVE_WOLFSSL
-    DBUG_PRINT("warning", ("wolfSSL MYSQL doesn't support CRL"));
-    DBUG_ASSERT(0);
-#else
     X509_STORE *store = SSL_CTX_get_cert_store(ssl_fd->ssl_context);
     /* Load crls from the trusted ca */
     if (X509_STORE_load_locations(store, crl_file, crl_path) == 0 ||
@@ -676,7 +649,6 @@ static struct st_VioSSLFd *new_VioSSLFd(
       *error = SSL_INITERR_BAD_PATHS;
       goto error;
     }
-#endif
   }
 
   if (vio_set_cert_stuff(ssl_fd->ssl_context, cert_file, key_file, error)) {
@@ -701,22 +673,16 @@ static struct st_VioSSLFd *new_VioSSLFd(
 
   SSL_CTX_set_options(ssl_fd->ssl_context, ssl_ctx_options);
 
-  /* set IO functions used by wolfSSL */
-#ifdef HAVE_WOLFSSL
-  wolfSSL_SetIORecv(ssl_fd->ssl_context, wolfssl_recv);
-  wolfSSL_SetIOSend(ssl_fd->ssl_context, wolfssl_send);
-#endif
-
   DBUG_PRINT("exit", ("OK 1"));
 
-  DBUG_RETURN(ssl_fd);
+  return ssl_fd;
 
 error:
   DBUG_PRINT("error", ("%s", sslGetErrString(*error)));
   report_errors();
   SSL_CTX_free(ssl_fd->ssl_context);
   my_free(ssl_fd);
-  DBUG_RETURN(0);
+  return 0;
 }
 
 /************************ VioSSLConnectorFd
